//Title:        Access Road
//Version:      0.7.0
//Copyright:    2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.MySQL.gBase;

import ARoad0.gBase.*;
import ARoad0.gBaseInterface.*;
import ARoad0.AcsAddon.Accbee.MySQL.gBaseInterface.*;

import java.util.Map;
import java.util.TreeMap;
import java.util.Iterator;
import java.util.regex.Pattern;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;

import ARoad0.ErrB.CreateError;
import ARoad0.ErrB.BaseError;
import ARoad0.ErrB.UpDateError;
import ARoad0.gDMak.ActionNewEligibleParty;

import java.io.ObjectStreamField;

/**
 * Currently unused. This class is responsible for modeling a MySQL Role. <P>
 * @version 0.7.0
 */
public class RoleMySQLImpl extends GroupIDMySQLImpl implements RoleMySQL {
  // private variables
  private static final long   serialVersionUID__ = 0;
  // protected variables

  /**
   * A reusable empty instance for initialization,
   * to avoid the use of 'new' for temporary values.
   * Caution: never change the state of the returned instance.
   */
  public static final RoleMySQLImpl EMPTY_INSTANCE = new RoleMySQLImpl();

  /**
   * empty variable for the JDK 2 serialization
   */
  public final static ObjectStreamField[] serialPersistentFields = new ObjectStreamField[0];

 /**
  * The basic constructor
  */
  public RoleMySQLImpl() {


                          // for testing purpose
                     //System.out.println("Call to RoleMySQLImpl constructor");
  }


   /**
    * Constructor used in the beamer by the editor for the ACS EP list, from 'New'.
    * Calls the other constructor.
    * @param _name ImmutableName of the groupID
    * @param _acs ACS of the groupID
    * @param _epType is the eligible party type, and may be null if the ACS allows it.
    * @param _isAdmin true only if the instance has an administrative role in the ACS,
    *    for the technical administration or the users.
    * @exception CreateError if _name ends with '_tree>', or if
    *    ACSImpl.newEorEligibleParty() throws it - the instance must then be deleted !
    */
  public RoleMySQLImpl(ImmutableName _name, ACS _acs, String _epType, Boolean _isAdmin)
   throws CreateError {

      this(_name, _acs, _epType, _isAdmin, true);

  } // constructor


   /**
   * Used for the creation of a RoleIDMySQL instance.
   * @param _name ImmutableName of the eligible party to create
   * @param _acs Main ACS of the eligible party to create
   * @param _epType is the eligible party type, and may be null if the ACS allows it.
   * @param _isAdmin true only if the instance has an administrative role in the ACS,
   *    for the technical administration or the users.
   * @param _toRegisterInManager false if this method must not register
   *  the instance in its ACS.
   * @exception CreateError if ACSImpl.newEorEligibleParty() throws it, or if
   * the pattern-name policy is not fulfilled
   * - the instance must then be deleted
   */
  public RoleMySQLImpl(ImmutableName _name, ACS _acs, String _epType, boolean _isAdmin,
    boolean _toRegisterInManager) throws CreateError {

      super(_name, _acs, _epType, _isAdmin, _toRegisterInManager);

      if (type_.indexOf("<byAcsCreator> standard") != -1 ) {
            return;
      } // if

      TreeMap                   m_EPs = new TreeMap(_acs.getEorM_OwnEligibleParties());
      TreeMap                   m_patternGrps = m_EPs;
      String                    grpType = null;
      ImmutableEligibleParty    ep = null;
      ImmutableName             acs_name = null;

      if (_name.lastComponent().equals("%")) {
          try {
              finalizeForBase();
          }
          catch (UpDateError u) {
              throw new CreateError(" This is a wrong name, "+
                  "since '%' is forbidden; this group is replaced by '|AllSimpleHosts|'."+
                  "Error on finalizing: "+u.toString());
          }
          throw new CreateError(" The name '%' is forbidden. "+
                  "This group is replaced by '|AllSimpleHosts|'.");
      } // if

      // a group must not have '@' in its name
      if (name_.lastComponent().indexOf("@") != -1) {
           try {
              finalizeForBase();
          }
          catch (UpDateError u) {
              throw new CreateError("From RoleMySQLImpl constructor. "+
                  "Error on finalizing: "+u.toString());
          }

          throw new CreateError("  The name of a group cannot contain the character '@' ");
      } // if (name_.lastComponent().indexOf("@")

      try {
          if (name_.lastComponent().indexOf(":") != -1 &&
                  getNameFromHostComponent(name_.lastComponent()).equals(
                    getEorACS().getParentName())) {
              finalizeForBase();
              throw new CreateError(" This is the name of the MySQL ACS parent, "+
                      "and it is modelled by the group 'localhost'.");
          }
      }
      catch (BaseError b) { // from getNameFromHostComponent()
              throw new CreateError(" This is the name of the MySQL ACS parent; "+
                  "it is replaced by the group 'localhost'."+
                  "Error on the host name: "+b.toString());
      }
      catch (UpDateError u) { // from finalizeForBase()
              throw new CreateError(" This is a wrong name, "+
                  "since it is the ACS parent name; it is replaced by the group 'localhost'."+
                  "Error on finalizing: "+u.toString());
      }

      // the simple host, if not 'localhost', has to be an open ACS
      // the name last component of a new host has to have the form 'xxx:yy:zzz:tttt',
      // and it is converted to the ACS name ' xxx:: yy:: zzz:: tttt::  '.
      // add it to the 'AllSimpleHosts' group
      if (type_.equals("host") || type_.equals("<immutable> host")) {
        try {
            acs_name = getNameFromHostComponent(name_.lastComponent());
        }
        catch (BaseError b) {
              try {
                  finalizeForBase();
              }
              catch (UpDateError u) {
                  throw new CreateError(" This is a wrong name, "+
                 "since the host name is not a valid name. "+
                          " Error on finalizing: "+u.toString());
              }
              throw new CreateError(" The host name is not a valid name "+
                      "with the form 'xxx:yy:zzz:tttt': "+b.toString());
        }
        if (!BaseManagerImpl.getInstance().isOpenACS(acs_name)) {
              try {
                  finalizeForBase();
              }
              catch (UpDateError u) {
                  throw new CreateError(" This is a wrong name, "+
                 "since the host name is not the name of an open ACS."+
                          " Error on finalizing: "+u.toString());
              }
              throw new CreateError(" The host name is not the name of an open ACS. ");
        } // if

        ((GroupID)m_EPs.get(((Name)_acs.getName()).extended("<G>").
                      extended("|AllSimpleHosts|"))).addEorMember(this);
      } // if (type_.equals("host")
      else if (type_.equals("host <byAcsCreator>")) {
        ((GroupID)m_EPs.get(((Name)_acs.getName()).extended("<G>").
                      extended("|AllSimpleHosts|"))).addEorMember(this);
      } // else

      // if this group is a pattern-name group, add to it all its current members,
      // if its name complies with the rules, and add it to the correct standard group
      if (type_.indexOf("pattern-name_hosts group") != -1 ) {

          // a pattern-name group must be mutable at its creation
          if (type_.indexOf("<immutable>") != -1) {
               try {
                  finalizeForBase();
              }
              catch (UpDateError u) {
                  throw new CreateError("From RoleMySQLImpl constructor. "+
                      "Error on finalizing: "+u.toString());
              }

              throw new CreateError("  A pattern-name group cannot be immutable at its creation ");
          } // if (type_.equals("pattern-name_hosts group")

          if (type_.indexOf("pattern-name_accounts group") != -1) {
              ((GroupID)m_EPs.get(((Name)_acs.getName()).extended("<G>").
                      extended("|pattern-name_accounts|"))).addEorMember(this);
          }
          else {
              ((GroupID)m_EPs.get(((Name)_acs.getName()).extended("<G>").
                      extended("|pattern-name_hosts|"))).addEorMember(this);
          }

          // necessary for updating the members of this new group in the explorer
          ActionNewEligibleParty.handleNewGroup(this);

          try {

              /*
                if (type_.indexOf("pattern-name_accounts group") != -1) {

                    for (Object me : m_EPs.values()) {
                        // set 'pattern-name_accounts group' first
                      if (me instanceof ImmutableGroupID && me != this &&
                              ((ImmutableGroupID)me).getEPType().indexOf(
                              "pattern-name_accounts group") != -1 &&
                              isMemberOfMySQLPatternNameGroup((GroupIDMember)me, this))
                          addEorMemberToPatternNameGroup((GroupIDMember)me);
                    } // for me

                    for (Object me : m_EPs.values()) {
                        // set 'accounts'
                      if (me instanceof ImmutableUserID &&
                              isMemberOfMySQLPatternNameGroup((GroupIDMember)me, this) &&
                              !containsAsIndirectMember((GroupIDMember)me))
                          addEorMemberToPatternNameGroup((GroupIDMember)me);
                    } // for me

                } // if (type_.indexOf("pattern-name_accounts group") != -1)
               */

                if (type_.indexOf("pattern-name_hosts group") != -1) {
                    for (Object me : m_EPs.values()) {
                      if (me instanceof ImmutableGroupID && me != this &&
                              isMemberOfMySQLPatternNameGroup((GroupIDMember)me, this))
                          addEorMemberToPatternNameGroup((GroupIDMember)me);
                    } // for me
                } // else if (type_.indexOf("pattern-name_hosts group") != -1)

                setComment("In the name of a pattern-name group, '%' replaces "+
                        "any group of characters, and '_' replaces one character.");
          }
          catch (BaseError b) {
               try {
                  finalizeForBase();
              }
              catch (UpDateError u) {
                  throw new CreateError("From RoleMySQLImpl constructor. "+
                      "Error on finalizing: "+u.toString());
              }

              throw new CreateError("  From the constructor: "+b.toString());
          }
      } // if (type_.equals("pattern-name_accounts group")
      else {
          if (_name.lastComponent().indexOf("%") != -1 ||
                _name.lastComponent().indexOf("_") != -1  ) {
              try {
                  finalizeForBase();
              }
              catch (UpDateError u) {
                  throw new CreateError(" This is a wrong name, "+
                      "since it contains '%' or '_' while it is not a pattern-name group. "+
                      "Error on finalizing: "+u.toString());
              }
              throw new CreateError(" The name contains the character"+
                      " '%' or '_' while it is not a pattern-name group.");
          }
      } // else

      // add this group to the matched non-standard pattern-name groups
      for (Iterator ite = m_patternGrps.values().iterator() ; ite.hasNext() ; ) {
        ep = (ImmutableEligibleParty) ite.next();

        if (!(ep instanceof ImmutableGroupID) || ep == this) {
            ite.remove();
            continue;
        }

        grpType = ((ImmutableGroupID)ep).getEPType();
        if (grpType.indexOf("standard") != -1) {
            ite.remove();
            continue;
        }
        if (grpType.indexOf("pattern-name_hosts group") == -1 ) {
                        ite.remove();
        }
      } // for ite

      try {
        for (Object grp : m_patternGrps.values()) {
          if (isMemberOfMySQLPatternNameGroup(this, (GroupID)grp))
              ((RoleMySQLImpl)grp).addEorMemberToPatternNameGroup(this);
        } // for obj
      }
      catch (BaseError b) {
          try {
              finalizeForBase();
          }
          catch (UpDateError u) {
              throw new CreateError("  From the constructor. "+
                  "Error on finalizing: "+u.toString());
          }

          throw new CreateError(b.toString());
      } // catch

      m_EPs = null;
      m_patternGrps = null;
      grpType = null;
      ep = null;
      acs_name = null;
  } // constructor

  //
  // To implements the GroupIDImpl methods
  //

  /**
   * @return the static serialVersionUID__ of this class
   */
  public long getSerialVersionUID()	{
          return serialVersionUID__;
  }

  /**
   * Gets the specific events to listen from an ACSObject, because these events
   * should change the rights of an ACSObject in an ACS Addon.
   * These events are listened to update the views containing these ACSObjects.
   * This method does not cover the parent properties of a Resource or a VirtualFolder
   * (see ARoad0.gBase.AcsAddon.getAddonParentPropertiesToListen()).
   * To define the events, the map key may have the following values: <P>
   * - 'THIS' to say that the single object to listen is the ACSObject, <P>
   * - 'ACS.method_name' to say that the list of objects to listen is returned
   * by the method 'method_name' in the Addon ACS class. <P>
   * For each map key, the values have the following format: <P>
   * 'AAA.method_name IF BBB' <P>
   * where: <P>
   * - AAA: THIS (mandatory), <P>
   * - method_name: 'get' method name of the ACSObject that is associated,
   * in the relevant BeanInfo class, to the event to listen (mandatory) <P>
   * - IF BBB: where BBB is TRUE, or FALSE, or NULL, or NOT-NULL, to say that
   * the event must be listened if and only if the returned value of the method_name is
   * respectively a boolean TRUE, or FALSE, or an object NULL, or NOT-NULL (option). <P>
   * When the map value starts with 'ACS', the ACS method to call has the ACSObject has
   * one argument. Otherwise, the ACS or the ACSObject method has no argument.
   * Called by ARoad0.UtilityImpl.removeAddonParentPropertyChangeListener().
   * @return map to define the specific ACS Addon events to listen. Is always null.
   */
  public Map<String, String> getAddonInnerPropertiesToListen() {
    //Map<String, String> m_result = new HashMap();
    //m_result.put(
      //      "<THIS>",
        //    "<THIS>.getL_otherRights");
    return null;
  } // getAddonInnerPropertiesToListen()

  /**
  * Adds a GroupIDMember as a member of this GroupID.
  * A host in MySQL is simulated by a group with a specific type.
  * There is exactly one simple host per account, because the sorting order of the AclEntries
  * deliver at most one matched ACL per level of parent. This means that two simple hosts
  * per account would allow two possible matched ACLs without any constraint on the
  * sorting of theses ACLs.
  * An account may be member of a host only if the account exists in the ACS the host represents,
  * and 'anonymous' is the single exception. Nonetheless, it is possible to have two
  * 'jerry' account in two hosts. If they are used by the same person or program, only
  * one MySQL account has to be created to simulate the case. If the 'jerry' accounts
  * are for different persons or programs, this method allows to put,
  * as member of one host,  a MySQL account like 'jerry<2>' or 'jerry<Z>'.
  * Checks the non-conditional ACLs from _gm to a database, and resets it
  * if _gm is an UserID, with DirectoryMySQL.resetInheritedAclEntryRightsOfChildren(),
  * since the super method resets only the conditional ACLs.
  * UserIDMySQLImpl.setValueOf() calls this method at the creation of an UserIDMySQL.
  * Overrides and calls the super method.
  * @param _gm new member of the GroupID. No action if null, empty.
  * @exception CreateError if _gm is a GroupID which has this instance has
  *   direct or indirect member, if _gm is already a member, or if the _gm type
  *   is not authorized by the ACS, or if the type does not allow the operation,
  *   if this group is immutable, or if it is mutable and not the new member.
  */
  public void addEorMember(GroupIDMember _gm) throws CreateError {

    if (type_.indexOf("<byAcsCreator> standard") != -1 ) {
        super.addEorMember(_gm);
        return;
    } // if

    if (type_.indexOf("pattern-name_accounts group") != -1 ||
         (type_.indexOf("pattern-name_hosts group") != -1  &&
         _gm.getType().indexOf("pattern-name_accounts group") == -1 )) {
      throw new CreateError(" It is possible to change by hand the members of "+
              "a pattern-name group only for a hosts group having an accounts group as member.");
    } // if

    if (_gm.getType().indexOf("pattern-name_accounts group") != -1) {
        // then type_ is allowed for 'pattern-name_hosts group', 'host',
        // 'host <byAcsCreator>', 'all-hosts group <byAcsCreator>'
        Set<ImmutableGroupIDMember> l_members = BaseUtilityImpl.getAllIndirectGroupMembers(
                                                    (GroupIDMySQL)_gm);
        boolean                     deny = false;
        ImmutableGroupIDMember      mem = null;

        for (ImmutableGroupIDMember member : l_members) {
            if (!containsAsMember((GroupIDMember)member) &&
                    !containsAsIndirectMember((GroupIDMember)member)) {
                deny = true;
                mem = member;
                break;
            }
        } // for member

        l_members = null;
        if (deny) {
            throw new CreateError(" To add directly as member, a pattern-name_accounts group A "+
              "to a group H, all the (indirect) members of A, "+_gm.getNickName()+
              ", must be (indirect) member of H, and this condition is false at least for "+
              mem.getNickName());
        }
        else {
            super.addEorMember(_gm);
            return;
        }
    }
    else if (type_.indexOf("host") == -1 || type_.indexOf("all-hosts group") != -1) {
        super.addEorMember(_gm);
        return;
    }
    else if (!_gm.getType().equals("account")) {
        super.addEorMember(_gm);
        return;
    }
    // process the adding ao an account in a host
    if (((UserIDMySQLImpl)_gm).getEorFirstHost() != null)
        throw new CreateError("This account cannot have a second simple host.");

    // the MySQL account has to exist as UserID in the host as ACS
    boolean                   forLocalHost = false;
    ImmutableName             acs_name = null;
    ImmutableACS              acs = null;
    ImmutableName             account_name = null;
    UserID                    account = null;
    String                    lastCompo = name_.lastComponent();
    String                    gmNickName = null;
    EligibleParty             ep = null;
    String                    epNickName = null;

    if (type_.equals("host <byAcsCreator>"))
        forLocalHost = true;

    if (!forLocalHost) {

        try {
            acs_name = RoleMySQLImpl.getNameFromHostComponent(lastCompo);
        }
        catch (BaseError b) {
            // this group has to be correct
            throw new InternalError(" This host name is not a valid name "+b.toString());
        }
        // the host has to be an open ACS
        if (!BaseManagerImpl.getInstance().isOpenACS(acs_name)) {
            throw new CreateError(" "+acs_name.lastComponent()+
                    " is not associated to an open ACS. ");
        } // if

        acs = (ImmutableACS) BaseManagerImpl.getInstance().getEorM_OpenACS().get(acs_name);

    } // if (!lastCompo.equals("localhost"))
    else {
        // the ACS parent is the true local host
        acs_name = aCS_.getParentName();
        acs = (ImmutableACS) BaseManagerImpl.getInstance().getEorM_OpenACS().
                get(acs_name);
    } // else

    gmNickName = _gm.getName().lastComponent();
    if (Pattern.matches(".*<.>", gmNickName))
        gmNickName = gmNickName.substring(0, gmNickName.length() - 3);

    // no adding if the associated account in 'acs' already exists
    for (Object ob : aCS_.getEorM_EligibleParties().values()) {
        if (!(ob instanceof UserID)) continue;
        ep = (EligibleParty) ob;

        epNickName = ep.getName().lastComponent();
        if (Pattern.matches(".*<.>", epNickName))
            epNickName = epNickName.substring(0, epNickName.length() - 3);

        if (gmNickName.equals(epNickName) && ((UserIDMySQLImpl)ep).getEorFirstHost() == this) {
            gmNickName = null;
            break;
        }

    } // for ob

    if (gmNickName == null)
        throw new CreateError(" The MySQL account "+ep.getName().lastComponent()+
                "is already associated to the same UserID, in the ACS of this group.");

                            // for testing and tracking
                            //System.out.println("in RoleMySQLImpl.addEorMember(), for "+
                              //      getName()+", gmNickName: "+gmNickName);

    account_name = ((Name)acs_name.clone()).extended("<U>").extended(gmNickName );

    account = (UserID) acs.getEorM_EligibleParties().get(account_name);

    // the MySQL account has to be an existing account, modulo '<X>' at the end
    // 'anonymous' is the only exception
    if (account == null && !account_name.lastComponent().equals("anonymous")) {
        throw new CreateError(" "+account_name.lastComponent()+
                " is not the name of an account in the open ACS '"+
                acs_name.lastComponent()+"' this group represents");
    } // if (account == null)

    super.addEorMember(_gm);

    acs_name = null;
    acs = null;
    account_name = null;
    account = null;
    lastCompo = null;
    ep = null;
    epNickName = null;

    // the super method resets only the conditional ACLs.
    // checks up the non-conditional ACLs from _gm to a database,
    // and resets it, if necessary, with DirectoryMySQL.resetInheritedAclEntryRightsOfChildren().
    // If _gm is an UserID, its non-conditional ACLs may produce different user@host rights
    // while they are combined with the non-conditional ACL for this group.
    // The occurence of a non-conditional ACL for this group is not checked up as a condition.
    // If _gm is a GroupID, its non-conditional ACLs cannot produce
    // different user@host rights, after its removing from this group.
    ImmutableResource     aclTarget = null;
    ImmutableAclEntry[]   l_acl = aCS_.getEorL_AclEntries();
    for (int i = 0; i < l_acl.length; i++ ) {

      aclTarget = l_acl[i].getEorResource();

      if (aclTarget.getType().indexOf("database") == -1 ||
              !(l_acl[i].getEorEP() instanceof UserID) ||
                l_acl[i].isConditionalAclEntry())
            continue;

      // reset also the own ACL rights of aclTarget
      ((DirectoryMySQL)aclTarget).resetInheritedAclEntryRightsOfChildren(
              (AclEntry)l_acl[i]);

    } // for i

    aclTarget = null;
    l_acl = null;
  } // addEorMember()

  /**
  * Removes a groupID member, and updates the member through the removeGroupToGM() method,
  * if the group is not immutable. Noop if the argument is an account having this group
  * as first group. No removing for '|AllAccounts|' or a standard group.
  * Checks up the non-conditional ACLs from _gm to a database, and resets it
  * if _gm is an UserID, with DirectoryMySQL.resetInheritedAclEntryRightsOfChildren(),
  * since the super method resets only the conditional ACLs.
  * Called by the GUI.
  * Never called by the finalizeForBase() method of the members, which use
  * removeEorMemberWithoutControl().
  * Overrides the super method and calls it at the end.
  * @param _gm member of the GroupID. No action if unknown, null, empty.
  * @exception CreateError if the argument is an account having this group as first group, or if
  * the type does not allow the operation, of if this group is standard group, a pattern-name
  * group, the AllAccounts or the AllSimpleGroups group
  */
  public void removeEorMember(GroupIDMember _gm) throws CreateError {

      if (type_.indexOf("<byAcsCreator> standard") != -1 ) {
          throw new CreateError(" It is not possible to change directly the members of "+
                  "a standard group.");
      } // if

      if (type_.indexOf("pattern-name_accounts group") != -1 ||
             type_.indexOf("pattern-name_hosts group") != -1 ) {
          try {
              // false if _gm is 'pattern-name_accounts group' and
              // this group is a 'pattern-name_hosts group'
              if (isMemberOfMySQLPatternNameGroup(_gm, this))
                  throw new CreateError(" It is not possible to change directly the members of "+
                  "a pattern-name group.");
          }
          catch (BaseError b) {
              throw new CreateError(" From removeEorMember(), "+b.toString());
          }
      } // if (type_.indexOf("pattern-name_accounts group") != -1

      if (type_.equals("all-hosts group <byAcsCreator>") ) {
          throw new CreateError(" It is not possible to change directly the members of "+
                  "an 'All' group.");
      } // if

      if (isSimpleHost()) {
          if (_gm instanceof UserID && ((UserIDMySQLImpl)_gm).getEorFirstHost() == this)
              throw new CreateError("This account cannot be directly removed "+
                      "from this host, because this group is its first host. ");
      } // if (type_

      super.removeEorMember(_gm);

      ImmutableResource     aclTarget = null;
      ImmutableAclEntry[]   l_acl = aCS_.getEorL_AclEntries();

      // the super method resets only the conditional ACLs.
      // checks up the non-conditional ACLs from _gm to a database,
      // and resets it, if necessary, with DirectoryMySQL.resetInheritedAclEntryRightsOfChildren().
      // If _gm is an UserID, its non-conditional ACLs may produce different user@host rights
      // while they are combined with the non-conditional ACL for this group.
      // The occurence of a non-conditional ACL for this group is not checked up as a condition.
      // If _gm is a GroupID, its non-conditional ACLs cannot produce
      // different user@host rights, after its removing from this group.
      for (int i = 0; i < l_acl.length; i++ ) {

          aclTarget = l_acl[i].getEorResource();

          if (aclTarget.getType().indexOf("database") == -1 ||
                  !(l_acl[i].getEorEP() instanceof UserID) ||
                    l_acl[i].isConditionalAclEntry())
                continue;

          // reset also the own ACL rights of aclTarget
          ((DirectoryMySQL)aclTarget).resetInheritedAclEntryRightsOfChildren(
                  (AclEntry)l_acl[i]);

      } // for i

      aclTarget = null;
      l_acl = null;
  } // removeEorMember()

   /**
   * Adds a group for which this GroupIDMember is a member,
   * and sorts the groups in an ordered array that follows the third sorting order
   * for the MySQL AclEntries, from the least-specific source to the more-specific ones.
   * There are first, for the users, the source types 'pattern-name_accounts group',
   * and then, but this order is unuseful, 'host byAcsCreator',
   * 'host', 'pattern-name_hosts group', 'all-hosts group byAcsCreator'.
   * The removing by GroupIDMemberImpl.removeGroupToGM() maintains the sorting.
   * Calls BaseUtilityMySQLImpl.isAfterInTheGroupIDMemberOrder().
   * Overrides the GroupIDMemberImpl method and does not call it.
   * Fires changes on 'AssociatedGroups' like the overridden method. Used only by GroupIDImpl.
   * @param _grp is the groupID to add. No action if it is null, or with null name.
   */
  public synchronized void addGroupToGM(GroupID _grp) {
    // variables
    boolean recorded = false;
    GroupIDImpl[] newArray;
    GroupIDImpl[] l_oldGroups;
    String typeOne = UtilityImpl.getFirstInstance().
                                    getObjectTypeWithoutImmutable(_grp.getEPType());

    if (_grp == null) return;
    // test if _grp is not known
    for (int i = 0 ; i < l_Groups_.length ; i++) {
        if (_grp == l_Groups_[i] ) {
          recorded = true; //_grp is known
          break;
        }
    } // for i
    if (recorded) return;

    // saves the old Group list, since l_Groups_ is a bound property
    l_oldGroups = new GroupIDImpl[l_Groups_.length];
    System.arraycopy(l_Groups_, 0, l_oldGroups, 0, l_Groups_.length);

                            // for testing and tracking
                            //System.out.println("in RoleMySQLImpl.addGroupToGM(), for "+getName()+
                              //      ", l_Groups_ before the adding:");
                              //for (int i = 0 ; i < l_Groups_.length ; i++) {
                                //  if (l_Groups_[i] != null)
                                  //    System.out.println("-- at "+i+": "+l_Groups_[i].getName());
                                  //else
                                    //  System.out.println("-- at "+i+": NULL");
                              //} // for

        // adds at the right order
    newArray = new GroupIDImpl[l_Groups_.length + 1];
    if (l_Groups_.length != 0) {
        for (int i = 0 ; i < l_Groups_.length ; i++) {
            if (!recorded &&
                    BaseUtilityMySQLImpl.isAfterInTheGroupIDMemberOrder(
                        typeOne,
                        UtilityImpl.getFirstInstance().
                            getObjectTypeWithoutImmutable(l_Groups_[i].getEPType()),
                        _grp.getName().lastComponent(),
                        l_Groups_[i].getName().lastComponent())) {
                newArray[i] = l_Groups_[i];
                continue;
            } // if

            if (!recorded) {
                newArray[i] = (GroupIDImpl)_grp;
                recorded = true;
            } // if

            newArray[i + 1] = l_Groups_[i];
        } // for

        if (!recorded)
            newArray[l_Groups_.length] = (GroupIDImpl)_grp;
    }
    else {
        newArray[0] = (GroupIDImpl)_grp;
    } // else
    l_Groups_ = newArray;

                            // for testing and tracking
                            //System.out.println("in RoleMySQLImpl.addGroupToGM(), for "+getName()+
                              //      ", l_Groups_ after the adding:");
                              //for (int i = 0 ; i < l_Groups_.length ; i++) {
                                //  if (l_Groups_[i] != null)
                                  //    System.out.println("-- at "+i+": "+l_Groups_[i].getName());
                                  //else
                                    //  System.out.println("-- at "+i+": NULL");
                              //} // for

    try {
        firePropertyChange("AssociatedGroups", l_oldGroups, getEorL_GroupID());
    } catch (Exception e) {
        ARoad0.Gui1.DialogBox.showAlarmMessage(
          "There is an exception after the data update, from outside the database."+
          "You should save the base and stop the program, then run it once again.");
        System.out.println("From RoleMySQLImpl.addGroupToGM(), firePropertChange() throws:");
        e.printStackTrace();
    }

    newArray = null;
    l_oldGroups = null;
  } // addGroupToGM()

  /**
  * Removes the groupID member of a host, without any control like in the basic same method.
  * Called by UserIDMySQLImpl.finalizeForBase().
  * Simply calls the super method.
  * @param _gm member of the GroupID. No action if unknown, null, empty.
  * @exception CreateError if the type does not allow the operation
  */
  public void removeEorMemberWithoutControl(GroupIDMember _gm) throws CreateError {
      super.removeEorMember(_gm);
  } // removeEorMemberWithoutControl()

   /**
   * This method, if this instance is a host group, finalizes the accounts that
   * are members having this group as first host.
   * This method removes this instance in its groups directly.
   * The superclass cannot be used, since it is done through a call to
   * RoleMySQLImpl.removeEorMember(), which may fire an exception.
   * A pattern-name group does not finalize its members when it is finalized.
   * Overrides the super method and calls it at the end.
   * An unused object has to be deleted to inform the other objects, and this method
   * HAS TO BE USED before deleting the GroupID.
   * Note: when this method is called by the EligiblePartyImpl constructor, the proper
   * variables of this class are not still initialized.
   * @exception UpDateError if RoleMySQLImpl.removeEorMemberWithoutControl() throws it
   */
   public synchronized void finalizeForBase() throws UpDateError {

              // for testing purpose
              //System.out.println("call of RoleMySQLImpl.finalizeForBase() for "+getName());

    // noop if it is the second call to this method
    if (folderMember_ == null ) {
          //super.finalizeForBase();
          return;
    }

    if (isSimpleHost() && l_Members_ != null) {
      ImmutableGroupIDMember[] l_oldMembers;

      l_oldMembers = new GroupIDMemberImpl[l_Members_.length];
      System.arraycopy(l_Members_, 0, l_oldMembers, 0, l_Members_.length);

      for (int  i = 0 ; i < l_oldMembers.length ; i++) {
          // calls back removeEorMemberWithoutControl()
          if (l_oldMembers[i] != null && ((UserIDMySQLImpl)l_oldMembers[i]).host_ == this)
                l_oldMembers[i].finalizeForBase();
      } // for i

      l_oldMembers = null;
    } // if (isSimpleHost()

    // set l_Groups_ to an empty array
    if (l_Groups_ != null) {
        List<RoleMySQLImpl> l_grps = new ArrayList(Arrays.asList(l_Groups_));
        try {

            for (RoleMySQLImpl grp : l_grps) {
                // calls back removeGroupToGM() and updates l_Groups_.
                grp.removeEorMemberWithoutControl(this);
            } // for i

        }
        catch (CreateError c) {
            throw new UpDateError(" RoleMySQLImpl.finalizeForBase(): "+c.toString());
        }
    } // if (l_Groups_ != null)

    super.finalizeForBase();
  } // finalizeForBase()

  /**
   * Detects if it is a conditional AclEntry from this group ACS,
   * for which a change in the groupidmember membership relation to this group is able to
   * change the membership chain from the AclEntry source to at least one condition group
   * of the AclEntry. Overriddes the super method to return true for a host that is
   * the source of a non-conditional ACL. In that case, the mixed ACLs implying
   * this ACL have to be updated.
   * Overrides the super method and calls it for the generic cases.
   * Called by add/removeEorMember().
   * @param _acl from the _member ACS. Never null.
   * @param _member new or previous direct member of this group. Never null.
   * @return true if _acl has to be reset through AclEntryImpl.resetRightsAsInheritedAclEntry().
   * Return false in all other cases, including an ACS which does not manage the conditional ACL,
   * or a non-conditional _acl, or no identity or membership relation from the _acl source
   * to _member, or no membership relation from this group to any _acl condition group.
   */
   public boolean isConditionalACLtoResetAfterMembershipChangeInThisGroup(
            ImmutableAclEntry _acl, ImmutableGroupIDMember _member) {

        if (type_.indexOf("host") != -1 && this == _acl.getEorEP())
            return true;
        else
            return super.isConditionalACLtoResetAfterMembershipChangeInThisGroup(_acl, _member);
   } // isConditionalACLtoResetAfterMembershipChangeInThisGroup()

  /**
  * This method provides a copy of the instance
  * with another name.
  * The copy order is 3 times the source order.
  * Does not copy the GroupID members, nor the own resources,
  * the current actors, the own actors.
  * To copy all these links, you have to copy first the GroupID,
  * then to copy the GroupIDMembers, the Resources and the Actors
  * to copy the links.
  * Call copyOwnVariables().
  * This method is overridden by the subclasses.
  * @param _name copy name. Usually creates as an extension of _parent name.
  * @param _acs main ACS of the copy to create. May not be the ACS of this instance.
  * @param _parent is not used, wathever the value.
  * @param _toRegisterInManager false if this method must not register
  *  the instance in the ACS.
  * @return a new instance of RoleMySQLImpl
  * @exception CreateError of the constructor with the same parameters,
  *   or if _parent ACS is not _acs, or addAclEntry() exceptions.
  **/
  public Object copy(ImmutableName _name, ACS _acs, Directory _parent,
    Boolean _toRegisterInManager)
    throws CreateError {

      if (_acs == null || _name == null ) {
          throw new CreateError("in RoleMySQLImpl.copy(),"+
          " _name or _acs is null.");
      }

      RoleMySQLImpl result = new RoleMySQLImpl(
          _name, _acs, getEPType(),
          _toRegisterInManager.booleanValue());

      result = copyOwnVariables(result, false);

             // for testing and tracking
             //System.out.println("in RoleMySQLImpl.copy(), the source: "+this+" and the result: "+result );


      return result;
  } // copy(,,,)


  /**
  * This method provides a copy of the instance
  * as copy(), but without copying the EligibleParty internal aclEntries.
  * The copy order is 3 times the source order.
  * Does not copy the GroupID members, nor the own resources,
  * the current actors, the own actors.
  * To copy all these links, you have to copy first the GroupID,
  * then to copy the GroupIDMembers, the Resources and the Actors
  * to copy the links.
  * Calls copyOwnVariables().
  * Called by ACSImpl.copy().
  * This method is overridden by the subclasses.
  * @param _name copy name. Usually creates as an extension of _parent name.
  * @param _acs main ACS of the copy to create. May not be the ACS of this instance.
  * @param _parent is not used, wathever the value.
  * @param _toRegisterInManager false if this method must not register
  *  the instance in the ACS.
  * @return a new instance of RoleMySQLImpl
  * @exception CreateError of the constructor with the same parameters,
  *   or if _parent ACS is not _acs.
  **/
  public Object copyWithoutInternalAcl(ImmutableName _name, ACS _acs, Directory _parent,
    Boolean _toRegisterInManager) throws CreateError {

      if (_acs == null || _name == null ) {
          throw new CreateError("in RoleMySQLImpl.copy(),"+
          " _name or _acs is null.");
      }

      RoleMySQLImpl result = new RoleMySQLImpl(
          _name, _acs, getEPType(),
          _toRegisterInManager.booleanValue());

      // copies the external aclEntries if they exist
      result = copyOwnVariables(result, true); // no internal aclEntry copy

      // if (result.getEorL_AclConnectedResources().isEmpty()) return result;
      //else return deleteAclEntries(result);
      return result;
  } // copy(,,,)

  /**
  * This method provides a copy of the instance variables.
  * Copies the own actors and the result as groupIDMember
  * (done by GroupIDMemberImpl.copyOwnVariables()).
  * Does not copy the groupIDMembers (done by the copy() caller).
  * Does not copy the own resources, nor
  * the current actors,
  * but the resource and actor copies do the job.
  * The copy order is 3 times the source order.
  * Called by copy() and by the copyOwnVariables() methods
  * of the subclasses.
  * @param _result to update with this instance variables copies
  * @param _withoutInternalAcl true if the internal aclEntries
  *   - that is with a resource having the same ACS - must not be copied
  * @return updated _result
  * @exception CreateError
  **/
  protected RoleMySQLImpl copyOwnVariables(RoleMySQLImpl _result,
    boolean _withoutInternalAcl) throws CreateError {

      return (RoleMySQLImpl)super.copyOwnVariables(_result, _withoutInternalAcl);
  } // copyOwnVariables(,,,)

} // RoleMySQLImpl

