//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.MySQL.gBaseBeanInfo;

import ARoad0.gBaseBeanInfo.*;
import ARoad0.AcsAddon.Accbee.MySQL.gBase.*;
import ARoad0.AcsAddon.Accbee.MySQL.gBaseInterface.*;
import ARoad0.ErrW.ProcessError;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

/**
 * This class is responsible for describing the ACS class for MySQL.
 * Changes the property descriptor for 'OneEPVirtualFolder', since VirtualFolderMySQLImpl
 * has a new argument in its constructor.
 * @version 0.7.0
 */
public class ACSMySQLImplBeanInfo extends ACSImplBeanInfo {

  public ACSMySQLImplBeanInfo() {}

  /**
   * Gets the description of the bean.
   * Some standard values are set:
   * - 'hidden' is false since the bean is not hidden to the users
   * - 'expert' is false since the bean is not reserved to experts
   * <P>
   * Specific values are set in the descriptor. They are get with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of ACSUbuntuImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(ACSMySQLImpl.class );
    result.setDisplayName("ACS");

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of ACSUbuntuImpl
    result.setValue("withTabbedPanes", Boolean.TRUE );

    // these strings signal the nodes to use in an ACS tree
    // to display this ACS, derived from the ACSTree.M_ACS_TREE_UPPER_NODES
    // in the Gui2 package.
    result.setValue("nodesInACSTree", new String[]
      {"RESOURCES", "EPVFOLDERS", "USERS", "GROUPS", "ACLENTRIES",
       "ACSRIGHTS"});

    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    result.setHidden(false);
    result.setExpert(false);

    return result;
  } // getBeanDescriptor()

  /**
   * Changes the property descriptor for 'OneEPVirtualFolder', since VirtualFolderMySQLImpl
   * has a new argument in its constructor.
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;

    try {

      // ------------------- property "OneEPVirtualFolder" --------------------
      PropertyDescriptor pd_sixtyEight = new PropertyDescriptor(
        "OneEPVirtualFolder", ACSMySQLImpl.class, "getEorL_EPVirtualFolders", null);
      pd_sixtyEight.setDisplayName("Right-User VFolders");
      pd_sixtyEight.setShortDescription("The virtual folders of Right User Resources");
      pd_sixtyEight.setBound(true);
      //pd_sixtyEight.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_sixtyEight.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle.
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_sixtyEight.setValue("valueTypeInGroup", new Object[] {VirtualFolderMySQL.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixtyEight.setValue("tabbedPaneName", "  Right Users  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_nine.setValue("panelGroupName", "  List  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when is is not applicable.
          //pd_sixtyEight.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the CollectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer.
      pd_sixtyEight.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_sixtyEight.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_sixtyEight.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_sixtyEight.setValue("withDeleteButton", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke
      // to let the user defines the new restricted key, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      // The array is equal to 'new Object[1]' when the value 'withKnownArgumentsForNew' is null.
          //pd_sixtyEight.setValue("choiceMethodForNew", new Object[] {
            //    VirtualFolderImpl.EMPTY_INSTANCE, "valueOf"} );
      // 'withKnownArgumentsForNew' allows to define the constructor/add/copy arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the fixed argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor in the beamer,
      // and call the read method to initialize a l_fixedValue from this object.
      // Third case for the fixed argument: the string 'thisname' is used
      // to point to the value returned from getName() by the object in the beamer.
      // Fourth case, the string 'true' or 'false' is used
      // to define a boolean value.
      // Fifth case, if none of the previous cases matches: takes directly
      // the fixed argument if it is a String.
      // Used by SimplePropertyEditor.getFixedValuesForNewElement(), which
      // may use a subset of the returned arguments if the class of the object to create
      // waits a smaller set of arguments.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (for instance, children of Directory).
      // Caution: the known arguments have to be Objects.
      pd_sixtyEight.setValue("withKnownArgumentsForNew", new Object[] {
            null, "this", null, null, "false", null} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_sixtyEight.setValue("addMethod", "addEorChild");
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_sixtyEight.setValue("removeMethod", "finalizeForUser");

      //
      // ------------End of ACSUbuntuImpl properties definition --------------------
      //

      // start from the parent class bean info
      l_result = super.getPropertyDescriptors();
      
      for (int i = 0; i < l_result.length; i++) {
        if (l_result[i].getName().equals("OneEPVirtualFolder")) {
              l_result[i] = pd_sixtyEight;
        }
      } // for i


      return l_result;
    } catch (IntrospectionException e) {
      new ProcessError("In ACSImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In ACSImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()
} // ACSMySQLImplBeanInfo
