//Title:        Access Road
//Version:      0.7.0
//Copyright:    2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.MySQL.gBaseBeanInfo;

import ARoad0.gBaseBeanInfo.*;
import ARoad0.gBaseInterface.*;
import ARoad0.AcsAddon.Accbee.MySQL.gBaseInterface.*;
import ARoad0.AcsAddon.Accbee.MySQL.gBase.*;

import ARoad0.gBaseInterface.AclEntry;
import ARoad0.gBaseInterface.StringRight;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

/**
 * This class is responsible for describing the ActorMySQLImpl class.
 * It is used for removing the 'IsRightProxy' ActorImpl property, the Resource 
 * 'M_ConstrainedChildACS' property and the Actor 'CurrentGroupID' property.
 * It adds the ResourceImpl property 'InheritedTargetAclEntries'.
 * Copy all the changes done in ResourceMySQLImplBeanInfo,
 * since it is not its superclass, but not the removing of the Alias tab.
 * Caution: a change may has to be copied in EligiblePartyImplMySQLBeanInfo,
 * if it exists.
 * @version 0.7.0
 */
public class ActorMySQLImplBeanInfo extends ActorImplBeanInfo {

  public ActorMySQLImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor.
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(ActorMySQL.class );
    result.setDisplayName("Actor");

    result = getCommonBeanDescriptor(result);

    return result;
  } // getBeanDescriptor()

  /**
   * Get the common description of the bean.
   * Reuse super.getCommonBeanDescriptor().
   * @param _result is the BeanDescriptor to configure
   * @return the bean descriptor
   */
  protected BeanDescriptor getCommonBeanDescriptor(BeanDescriptor _result) {

    super.getCommonBeanDescriptor(_result);
    
    // ----------------------- constructor parameters ---------------------------
    
    // 'constrArgumentClasses' is the list of argument classes
    // for the class constructor
    _result.setValue("constrArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class, Directory.class, UserID.class, String.class,
      GroupIDMySQL.class} );

    // 'constrArgumentPropertyNames' is the list of property names
    // for the class constructor.
    _result.setValue("constrArgumentPropertyNames", new String[]{
      "Name", "ACS", "Parent Node", "CurrentUserID", "Resource Type",
      "CurrentGroupID"} );

    // 'constrArgumentFromSelect' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "constrArgumentPropertyNames", or, as here,
    // in the order of the constructor method in ResourceImpl
    _result.setValue("constrArgumentFromSelect", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.TRUE, Boolean.TRUE, Boolean.FALSE,
      Boolean.TRUE} );

    // 'constrArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another constructor argument,
    // what is this another argument which provides the first part of the name
    // if it is not null.
    _result.setValue("constrArgumentConcatenedNameSource", new Integer[]{
      new Integer(2), null, null, null, null, null} );

    // 'constrNullArgumentPossible' indicates if an argument may be null
    // in the constructor
    _result.setValue("constrNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE, Boolean.TRUE, Boolean.FALSE,
      Boolean.TRUE} );

    // 'constrOptionalArgument' indicates if an argument must not be used in the
    // call to the constructor when it is null.
    _result.setValue("constrOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE,
      Boolean.FALSE} );

    // 'constrRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names. Is optional.
    // If it is true, the array of restricted values for an argument is searched:
    // - first, in the ACS.getM_staticSetOfItems() map,
    // and then "constrArgumentPropertyNames" must contain the property descriptor name.
    // The name words may be separated by spaces in "constrArgumentPropertyNames",
    // although there is no separation in the ACS.getM_staticSetOfItems() map.
    // - second, in the value "staticSetOfItems" of the property in this BeanInfo.
    _result.setValue("constrRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.TRUE,
      Boolean.FALSE} );

    // 'constrNonVisibleArgument' indicates if an argument is masqued
    // in the constructor window and sets to null in the constructor.
    // True when the ACS does not manage the property, and then, 
    // this value gets the name of the ACS method which returns a boolean 'false'
    // when the argument has to be null and to be not displayed. If
    // the returned value is 'true', the argument is displayed as usual.
    // For each non null value:
    // "constrNullArgumentPossible" has to be Boolean.TRUE,
    // "constrOptionalArgument" has to be Boolean.FALSE.
    _result.setValue("constrNonVisibleArgument", new String[]{
      null, null, "getManageResourceTrees", "getManageAccounts", null, 
      null} );
    
    // ----------------------- copy parameters ---------------------------

      
    return _result;
  } // getCommonBeanDescriptor()

  /**
   * Removes the 'IsRightProxy' property from the ActorImpl, since this property
   * is immutable and dangerous in the MySQL simulation, and the Resource
   * 'M_ConstrainedChildACS' property.
   * Add two property descriptors for the ResourceImpl property 'InheritedTargetAclEntries'
   * and use the same event for the method getM_PrimaryTargetACLRights().
   * Change the property descriptor of 'ParentNode' in '  ACL Inheritance  '.
   * Change the property descriptor of 'CurrentUserID' without setEorCurrentUserID().
   * The ActorMySQLImpl class maintains an empty 'AclEntry' property for the 'view_delegate'
   * and the 'trigger' actors, while the 'stored_function' and 'stored_procedure' actors
   * allows direct ACL.
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;

    try {
      // ------------------- property "InheritedTargetAclEntries" ------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "InheritedTargetAclEntries", ActorMySQLImpl.class,
        "getEorImmutableL_TargetInheritedAclEntries", null);
      pd_one.setDisplayName(""); // Sorted inherited ACLs
      pd_one.setShortDescription("The inherited AclEntries "+
              "are sorted with 4 sortings which take account of the source type "+
              "and the target type.");
      pd_one.setValue("isCollection", Boolean.TRUE );
      pd_one.setValue("valueTypeInGroup", new Object[] {AclEntry.class} );
      pd_one.setValue("tabbedPaneName", "  ACL Inheritance  " );
      pd_one.setValue("panelGroupName",
              "    Sorted list - matched ACL at each parent level: "+
              "user@host or accounts-for-hosts or... ");
      pd_one.setBound(true);
      pd_one.setValue("withViewButton", Boolean.TRUE );
      //pd_one.setValue("withoutListSorting", Boolean.TRUE );

      // ------------------- property "InheritedTargetAclEntries" with the same event ----
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "InheritedTargetAclEntries", ActorMySQLImpl.class,
        "getM_TargetInheritedACLRights", null);
      pd_two.setDisplayName("Primary Inherited ACLs");
      pd_two.setShortDescription("The primary inherited rights "+
              "allow to show the right statements at each parent level");
      pd_two.setBound(true);
      pd_two.setValue("isRightMap", Boolean.TRUE );
      pd_two.setValue("valueTypeInGroup", new Object[] {StringRight.class} );
      pd_two.setValue("tabbedPaneName", "  ACL Inheritance  " );
      pd_two.setValue("panelGroupName",
              "   Unsorted primary rights at each parent level -"+
              " The level is the key ending number  ");
      pd_two.setValue("KeysUpdateInMap", Boolean.FALSE);
      pd_two.setValue("withPreviousButton", Boolean.FALSE );
      pd_two.setValue("withViewButton", Boolean.TRUE );
      pd_two.setValue("withNewButton", Boolean.FALSE );
      pd_two.setValue("withSelectButton", Boolean.FALSE );
      pd_two.setValue("withRemoveButton", Boolean.FALSE );
      pd_two.setValue("withDeleteButton", Boolean.FALSE );
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_two.setValue("limitedSizeForPresentation", new Integer(10));

      // ------------------- property "ParentNode" in "  ACL Inheritance  " -------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "ParentNode", ActorMySQLImpl.class, "getEorParent", null);
      pd_three.setDisplayName("Parent");
      pd_three.setShortDescription("The node which is the parent in "+
        "the tree structure");
      pd_three.setValue("tabbedPaneName", "  ACL Inheritance  " );
      pd_three.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "IsRightProxy" without setIsRightProxy --------
      PropertyDescriptor pd_eleven = new PropertyDescriptor(
        "IsRightProxy", ActorMySQLImpl.class, "getIsRightProxy", null);
      pd_eleven.setDisplayName("Is a Right Proxy");
      pd_eleven.setShortDescription("False only if it is proved that "+
              "this Actor NEVER gives to its accessors, some rights on the targets it may access. ");
      pd_eleven.setValue("tabbedPaneName", "  As Actor  " );
      pd_eleven.setValue("panelGroupName", "  A. Main values  " );
      pd_eleven.setBound(true);

      // ----------- property "CurrentUserID" without setEorCurrentUserID() ----
      PropertyDescriptor pd_five = new PropertyDescriptor(
        "CurrentUserID", ActorMySQLImpl.class, "getEorCurrentUserID", null);
      pd_five.setDisplayName("DEFINER User");
      pd_five.setShortDescription("In the DEFINER mode, this account is not null,"+
              " and it gives its rights to this Actor. The INVOKER mode is the second mode.");
      pd_five.setBound(false);
      pd_five.setValue("tabbedPaneName", "  As Actor  " );
      pd_five.setValue("panelGroupName", "  B. AG context as Actor - null in the MySQL INVOKER mode  " );
      pd_five.setValue("withViewButton", Boolean.TRUE );
      pd_five.setValue("withSelectButton", Boolean.FALSE );
      pd_five.setValue("withClearButton", Boolean.FALSE );
      
      // ------------------- property "CurrentGroupID" without setEorCurrentGroupID()
      PropertyDescriptor pd_six = new PropertyDescriptor(
        "CurrentGroupID", ActorMySQLImpl.class, "getEorCurrentGroupID", null);
      pd_six.setDisplayName("DEFINER Host");
      pd_six.setShortDescription("In the DEFINER mode, this actor "+
              "takes the rights of (DEFINER User@DEFINER Host) as rights user.");
      pd_six.setBound(false);
      pd_six.setValue("tabbedPaneName", "  As Actor  " );
      pd_six.setValue("panelGroupName", "  B. AG context as Actor - null in the MySQL INVOKER mode  " );
      pd_six.setValue("withViewButton", Boolean.TRUE );
      pd_six.setValue("withSelectButton", Boolean.FALSE );
      pd_six.setValue("withClearButton", Boolean.FALSE );

      //
      // ------------End of ActorMySQLImpl properties definition --------------------
      //

      // start from the parent class bean info
      l_pdFromParent = super.getPropertyDescriptors();
      l_result = new PropertyDescriptor[l_pdFromParent.length - 2];
      boolean foundTwo = false;
      int decrement = 0;
      int b = 0;

                    //System.out.println("ActorMySQLImplBeanInfo.getPropertyDescriptors()"+
                      //      " starting iteration on l_pdFromParent"+
                        //    ", l_pdFromParent.length: "+l_pdFromParent.length);
                    //for (int i = 0; i < l_pdFromParent.length; i++) {
                      //  System.out.println("------ "+l_pdFromParent[i].getName());
                    //} // for i

      // removes 'M_ConstrainedChildACS', 'ParentNode', 'AclAcsControllers', 'PriAcsControllers'
      // changes 'IsRightProxy', 'CurrentUserID', 'ParentNode'
      for (int i = 0; i < l_pdFromParent.length; i++) {

                    //System.out.println("---- on "+l_pdFromParent[i].getName());
                    //System.out.println("---- i: "+i+", decrement: "+decrement+", b: "+b);
                    //sum = i - decrement + b;
                    //System.out.println("---- i - decrement + b: "+sum);

          if (l_pdFromParent[i].getName().equals("IsRightProxy")) {
              l_result[i - decrement + b] = pd_eleven;
          }
          else if (l_pdFromParent[i].getName().equals("ParentNode")) {
              decrement++;
          }
          else if (l_pdFromParent[i].getName().equals("M_ConstrainedChildACS")) {
              decrement++;
          }
          else if (l_pdFromParent[i].getName().equals("AclAcsControllers")) {
              decrement++;
          }
          else if (l_pdFromParent[i].getName().equals("PriAcsControllers")) {
              decrement++;
          }
          else if (l_pdFromParent[i].getName().equals("SoftAlias")) {
              decrement++;
          }
          else if (l_pdFromParent[i].getName().equals("CurrentGroupID")) {
              l_result[i - decrement + b] = pd_six;
          }
          else if (l_pdFromParent[i].getName().equals("CurrentUserID")) {
              l_result[i - decrement + b] = pd_five;
          }
          else if (l_pdFromParent[i].getName().equals("EPsThroughAclEntry")) {
              foundTwo = true;
              l_result[i - decrement + b] = l_pdFromParent[i];
          }
          else if (l_pdFromParent[i].getName().equals("AclRights")) {
              l_pdFromParent[i].setValue("panelGroupName",
                    "    Sorted list for users 'account' (as ACL source)"+
                    " or 'member' (of the ACL source)  ");
              l_pdFromParent[i].setValue("withoutListSorting", Boolean.TRUE );
              l_result[i - decrement + b] = l_pdFromParent[i];
          }
          else if (foundTwo) {
              l_result[i - decrement] = pd_three; // 'ParentNode' just after 'EPsThroughAclEntry'
              l_result[i - decrement + 1] = pd_one;
              l_result[i - decrement + 2] = pd_two;
              foundTwo = false;
              b = 3;
              l_result[i - decrement + b] = l_pdFromParent[i];
          }
          else {
              l_result[i - decrement + b] = l_pdFromParent[i];
          }
      } // for i

                    //System.out.println("ActorMySQLImplBeanInfo.getPropertyDescriptors()"+
                      //      "final decrement: "+decrement+", final b: "+b);
                    //System.out.println("and property descriptors: ");
                    //for (int i = 0; i < l_result.length; i++) {
                      //  if (l_result[i] != null)
                        //    System.out.println("------ "+l_result[i].getName());
                        //else
                          //  System.out.println("------ null ");
                    //} // for i

      l_pdFromParent = null;
      pd_one = null;
      pd_five = null;
      pd_eleven = null;
      pd_three = null;
      pd_two = null;
      pd_six = null;

      return l_result;
    } catch (IntrospectionException e) {
      throw new InternalError("In ActorMySQLImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch
  } // getPropertyDescriptors()

} // ActorMySQLImplBeanInfo
