//Title:        Access Road
//Version:      0.7.0
//Copyright:    2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.MySQL.gBaseBeanInfo;

import ARoad0.gBaseBeanInfo.*;
import ARoad0.AcsAddon.Accbee.MySQL.gBaseInterface.*;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

/**
 * This class is responsible for describing the BasesSetMySQLImpl class.
 * It adds a property descriptor for the BasesSetMySQL property 'ConnectedDatabases',
 * and uses the event 'InheritedTargetAclEntries' for the ResourceImpl
 * method getM_PrimaryTargetACLRights().
 * Change a panel group name to the property 'AclRights' to describe the MySQL ACLs.
 * Remove the Basic tab 'Alias' and the Resource 'M_ConstrainedChildACS' property.
 * Change the property descriptor of 'ParentNode' in '  ACL Inheritance  '.
 */
public class BasesSetMySQLImplBeanInfo extends ResourceImplBeanInfo {

  public BasesSetMySQLImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor. There is no method to change
   * the class of a BeanDescriptor, so, to reuse the BeanInfo of the mother class,
   * the values are copied in a new BeanDescriptor.
   * Copy all the changes done in ResourceMySQLImplBeanInfo,
   * since it is not its superclass.
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(BasesSetMySQL.class );
    result.setDisplayName("BasesSet");

    result = getCommonBeanDescriptor(result);

    return result;
  } // getBeanDescriptor()

  /**
   * Adds a property descriptor for the BasesSetMySQLImpl the properties
   * 'Parent Node' (for the constructor only, not the beamer) and 'ConnectedDatabases'
   * to replace the ResourceMySQL property 'InheritedTargetAclEntries'.
   * Calls ResourceMySQLImplBeanInfo.getResourceMySQLPropertyDescriptors() with
   * the super method as argument.
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_result;
    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_interm;

    try {
      // ------------------- property "ParentNode" in "  AG Inheritance  " as in Resource
        // so it is not displayed in the beamer -------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "ParentNode", ResourceMySQL.class, "getEorParent", null);
      pd_three.setDisplayName("Parent");
      pd_three.setShortDescription("The node which is the parent in "+
        "the tree structure");
      pd_three.setValue("tabbedPaneName", "  AG Inheritance  " );
      pd_three.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "ConnectedDatabases" ------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "ConnectedDatabases", BasesSetMySQL.class,
        "getEorConnectedDatabases", null);
      pd_one.setDisplayName("Connected databases");
      pd_one.setShortDescription("The databases "+
              "having a name which matches this bases set name ");
      pd_one.setValue("isCollection", Boolean.TRUE );
      pd_one.setValue("valueTypeInGroup", new Object[] {DirectoryMySQL.class} );
      pd_one.setValue("tabbedPaneName", "  General  " );
      pd_one.setValue("panelGroupName", "  This bases set AclEntries are copied "+
              "onto these databases  ");
      pd_one.setBound(true);
      pd_one.setValue("withViewButton", Boolean.TRUE );

      // ------- End of properties description ---------------------------------

      // start from ResourceMySQLImplBeanInfo
      // caution: this code works because l_result is not greater than l_pdFromParent
      l_pdFromParent = ResourceMySQLImplBeanInfo.getResourceMySQLPropertyDescriptors(
              super.getPropertyDescriptors() );
      l_interm = new PropertyDescriptor[l_pdFromParent.length - 1];
      int       decrement = 0;

                    // for testing and tracking
                    //System.out.println("BasesSetMySQLImplBeanInfo.getPropertyDescriptors():");
                    //System.out.println("---- l_pdFromParent.length: "+l_pdFromParent.length);

      // remove the "  ACL Inheritance  " tab containing 3 properties to remove
      // 'InheritedTargetAclEntries', the displayed property 'PrimaryTargetACLRights'
      // and 'ParentNode'

      for (int i = 0; i < l_pdFromParent.length; i++) {
          // 3 'ACL Inheritance' tab properties to remove, but
          //  property "ParentNode" puts in "  AG Inheritance  "
          if (l_pdFromParent[i].getValue("tabbedPaneName").equals("  ACL Inheritance  ")) {
              decrement++;
          }
          else {
              l_interm[i - decrement] = l_pdFromParent[i];
          }
      } // for i

      if (decrement != 3)
          throw new InternalError("In BasesSetMySQLImplBeanInfo.getPropertyDescriptors(), "+
                  "the property removing is wrong.");

      l_result = new PropertyDescriptor[l_pdFromParent.length - decrement + 1];
      System.arraycopy(l_interm, 0, l_result, 0, l_pdFromParent.length - decrement);

      l_result[l_result.length - 2] = pd_three;
      l_result[l_result.length - 1] = pd_one;

                    //System.out.println("BasesSetMySQLImplBeanInfo.getPropertyDescriptors():");
                    //System.out.println("and property descriptors: ");
                    //for (int i = 0; i < l_result.length; i++) {
                      //  System.out.println("------ "+l_result[i].getName());
                    //} // for i

      pd_three = null;
      pd_one = null;
      l_pdFromParent = null;
      l_interm = null;
    } catch (IntrospectionException e) {
      throw new InternalError("In BasesSetMySQLImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

    return l_result;
  } // getPropertyDescriptors()

} // BasesSetMySQLImplBeanInfo