//Title:        Access Road
//Version:      0.7.0
//Copyright:    2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.MySQL.gBaseBeanInfo;

import ARoad0.gBaseBeanInfo.*;
import ARoad0.AcsAddon.Accbee.MySQL.gBaseInterface.*;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;

/**
 * This class is responsible for describing the Linux MySQL group.
 * It remove the pane 'Alias' from the Basic BeanInfo, the 'AG' pane from
 * the GroupID BeanInfo among the property descriptors, and the EligibleParty
 * 'M_ConstrainedChildACS' property.
 * @version 0.7.0
 */
public class GroupIDMySQLImplBeanInfo extends GroupIDImplBeanInfo {

  public GroupIDMySQLImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of ResourceImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
      // used by BaseUtilityImpl.getSuperInterfacesFromBeanInfo()
    BeanDescriptor result = new BeanDescriptor(GroupIDMySQL.class );
    result.setDisplayName("GroupID");
    result = getCommonBeanDescriptor(result);

    return result;
  } // getBeanDescriptor()

  /**
   * Removes the pane 'Alias' from the Basic BeanInfo, the 'AG' pane from
   * the GroupID BeanInfo, and the EligibleParty 'M_ConstrainedChildACS' property.
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

      PropertyDescriptor[] l_interm;
      PropertyDescriptor[] l_pdFromParent;
      PropertyDescriptor[] l_result;

      // ------- End of properties description ---------------------------------

      // start from the parent class bean info
      // caution: this code works because l_result is not greater than l_pdFromParent
      l_pdFromParent = super.getPropertyDescriptors();
      l_interm = new PropertyDescriptor[l_pdFromParent.length];
      int       decrement = 0;

      for (int i = 0; i < l_pdFromParent.length; i++) {
          // several Alias tab properties to remove
          if (l_pdFromParent[i].getValue("tabbedPaneName").equals("  Alias  ")) {
              decrement++;
          }
          // several AG tab properties to remove
          else if (l_pdFromParent[i].getValue("tabbedPaneName").equals("  AG  ")) {
              decrement++;
          }
          else if (l_pdFromParent[i].getName().equals("M_ConstrainedChildACS")) {
              decrement++;
          }
          else {
              l_interm[i - decrement] = l_pdFromParent[i];
          }
      } // for i

      l_result = new PropertyDescriptor[l_pdFromParent.length - decrement];
      System.arraycopy(l_interm, 0, l_result, 0, l_pdFromParent.length - decrement);

      l_interm = null;
      l_pdFromParent = null;
      return l_result;
  } // getPropertyDescriptors()


} // GroupIDMySQLImplBeanInfo