//Title:        Access Road
//Version:      0.7.0
//Copyright:    2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.MySQL.gBaseBeanInfo;

import ARoad0.gBaseBeanInfo.*;
import ARoad0.AcsAddon.Accbee.MySQL.gBase.*;
import ARoad0.AcsAddon.Accbee.MySQL.gBaseInterface.*;
import ARoad0.gBaseInterface.AclEntry;
import ARoad0.gBaseInterface.StringRight;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

/**
 * This class is responsible for describing the MySQL resource.
 * It adds a property descriptor for the ResourceImpl property 'InheritedTargetAclEntries',
 * and uses the same event for the method getM_PrimaryTargetACLRights().
 * Add a panel group name to the property 'AclRights' to describe the MySQL ACLs.
 * Remove the Basic tab 'Alias' and the Resource 'M_ConstrainedChildACS' property.
 * Change the property descriptor of 'ParentNode' in '  ACL Inheritance  '.
 * @version 0.7.0
 */
public class ResourceMySQLImplBeanInfo extends ResourceImplBeanInfo {

  public ResourceMySQLImplBeanInfo() {}

  /**
   * Gets the description of the bean.
   * The properties to handle in the GUI are set in the descriptor.
   * They are get through BeanDescriptor.getValue().
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
      // used by BaseUtilityImpl.getSuperInterfacesFromBeanInfo()
    BeanDescriptor result = new BeanDescriptor(ResourceMySQLImpl.class );
    result.setDisplayName("Resource");

    return getCommonBeanDescriptor(result);
  } // getBeanDescriptor()

  /**
   * Adds a property descriptor for the ResourceImpl property 'InheritedTargetAclEntries'.
   * Removes the Basic tab 'Alias' and the Resource 'M_ConstrainedChildACS' property.
   * Calls getResourceMySQLPropertyDescriptors(). Called by BasesSetMySQLImpl.
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {
      // start from the parent class bean info
      return getResourceMySQLPropertyDescriptors( super.getPropertyDescriptors() );

  } // getPropertyDescriptors()

  /**
   * Adds a property descriptor for the ResourceImpl property 'InheritedTargetAclEntries',
   * and uses the same event for the method getM_PrimaryTargetACLRights().
   * Add a panel group name to the property 'AclRights' to describe the MySQL ACLs.
   * Remove the Basic tab 'Alias' and the Resource 'M_ConstrainedChildACS' property.
   * Change the property descriptor of 'ParentNode' in '  ACL Inheritance  '.
   * Called by getPropertyDescriptors() and DirectoryMySQLImplBeanInfo.
   * This is why the PropertyDescriptors set Resource
   * or ResourceMySQL as class of the property.
   * Caution: all the changes in this method have to be copied in ActorMySQLImplBeanInfo.
   * @param _l_pdFromParent array from ResourceImplBeanInfo or DirectoryImplBeanInfo
   * @return array of property descriptors
   */
  protected static PropertyDescriptor[] getResourceMySQLPropertyDescriptors(
          PropertyDescriptor[] _l_pdFromParent) {

    PropertyDescriptor[] l_result;
    PropertyDescriptor[] l_interm;

    try {
      // ------------------- property "ParentNode" in "  ACL Inheritance  " -------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "ParentNode", ResourceMySQL.class, "getEorParent", null);
      pd_three.setDisplayName("Parent");
      pd_three.setShortDescription("The node which is the parent in "+
        "the tree structure");
      pd_three.setValue("tabbedPaneName", "  ACL Inheritance  " );
      pd_three.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "InheritedTargetAclEntries" ------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "InheritedTargetAclEntries", ResourceMySQL.class,
        "getEorImmutableL_TargetInheritedAclEntries", null);
      pd_one.setDisplayName(""); // Sorted inherited ACLs
      pd_one.setShortDescription("The inherited AclEntries "+
              "are sorted with 4 sortings which take account of the source type "+
              "and the target type.");
      pd_one.setValue("isCollection", Boolean.TRUE );
      pd_one.setValue("valueTypeInGroup", new Object[] {AclEntry.class} );
      pd_one.setValue("tabbedPaneName", "  ACL Inheritance  " );
      pd_one.setValue("panelGroupName",
              "    Sorted list - matched ACL at each parent level: user@host or accounts-for-hosts or...  ");
      pd_one.setBound(true);
      pd_one.setValue("withViewButton", Boolean.TRUE );
      //pd_one.setValue("withoutListSorting", Boolean.TRUE );

      // ------------------- displayed property "PrimaryTargetACLRights" with the event 'InheritedTargetAclEntries' ----
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "InheritedTargetAclEntries", ResourceMySQL.class,
        "getM_TargetInheritedACLRights", null);
      pd_two.setDisplayName("Primary Inherited ACLs");
      pd_two.setShortDescription("The primary inherited rights "+
              "allow to show the rights statements for each parent level");
      pd_two.setBound(true);
      pd_two.setValue("isRightMap", Boolean.TRUE );
      pd_two.setValue("valueTypeInGroup", new Object[] {StringRight.class} );
      pd_two.setValue("tabbedPaneName", "  ACL Inheritance  " );
      pd_two.setValue("panelGroupName",
              "   Unsorted primary rights at each parent level -"+
              " The level is the key ending number  ");
      pd_two.setValue("KeysUpdateInMap", Boolean.FALSE);
      pd_two.setValue("withPreviousButton", Boolean.FALSE );
      pd_two.setValue("withViewButton", Boolean.TRUE );
      pd_two.setValue("withNewButton", Boolean.FALSE );
      pd_two.setValue("withSelectButton", Boolean.FALSE );
      pd_two.setValue("withRemoveButton", Boolean.FALSE );
      pd_two.setValue("withDeleteButton", Boolean.FALSE );
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_two.setValue("limitedSizeForPresentation", new Integer(10));

      // ------- End of properties description ---------------------------------

      // start from the parent class bean info
      // caution: this code works because l_result is not greater than l_pdFromParent
      l_interm = new PropertyDescriptor[_l_pdFromParent.length - 1];
      boolean   foundTwo = false;
      int       decrement = 0;
      int       b = 0;

      for (int i = 0; i < _l_pdFromParent.length; i++) {
          // several Alias tab properties to remove
          if (_l_pdFromParent[i].getValue("tabbedPaneName").equals("  Alias  ")) {
              decrement++;
          }
          else if (_l_pdFromParent[i].getName().equals("ParentNode")) {
              decrement++;
          }
          else if (_l_pdFromParent[i].getName().equals("M_ConstrainedChildACS")) {
              decrement++;
          }
          else if (_l_pdFromParent[i].getName().equals("AclAcsControllers")) {
              decrement++;
          }
          else if (_l_pdFromParent[i].getName().equals("PriAcsControllers")) {
              decrement++;
          }
          else if (_l_pdFromParent[i].getName().equals("EPsThroughAclEntry")) {
              foundTwo = true;
              l_interm[i - decrement + b] = _l_pdFromParent[i];
          }
          else if (_l_pdFromParent[i].getName().equals("AclRights")) {
              _l_pdFromParent[i].setValue("panelGroupName",
                    "    Sorted list for users 'account' (as ACL source)"+
                    " or 'member' (of the ACL source)  ");
              _l_pdFromParent[i].setValue("withoutListSorting", Boolean.TRUE );
              l_interm[i - decrement + b] = _l_pdFromParent[i];
          }
          else if (foundTwo) {
              l_interm[i - decrement] = pd_three; // 'ParentNode' just after 'EPsThroughAclEntry'
              l_interm[i - decrement + 1] = pd_one;
              l_interm[i - decrement + 2] = pd_two;
              foundTwo = false;
              b = 3;
              l_interm[i - decrement + b] = _l_pdFromParent[i];
          }
          else {
              l_interm[i - decrement + b] = _l_pdFromParent[i];
          }
      } // for i

      l_result = new PropertyDescriptor[_l_pdFromParent.length - decrement + b];
      System.arraycopy(l_interm, 0, l_result, 0, _l_pdFromParent.length - decrement + b);


                    //System.out.println("ResourceMySQLImplBeanInfo.getResourceMySQLPropertyDescriptors()"+
                      //      "decrement: "+decrement+", b: "+b);
                    //System.out.println("and property descriptors: ");
                    //for (int i = 0; i < l_result.length; i++) {
                      //  System.out.println("------ "+l_result[i].getName());
                    //} // for i


      l_interm = null;
      pd_one = null;
      pd_three = null;
      pd_two = null;
      
      return l_result;
    } catch (IntrospectionException e) {
      throw new InternalError("In ResourceMySQLImplBeanInfo.getResourceMySQLPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getResourceMySQLPropertyDescriptors()


} // ResourceMySQLImplBeanInfo