//Title:        Access Road
//Version:      0.7.1
//Copyright:    2011-2012 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.MySQL.gBaseBeanInfo;

import ARoad0.gBaseBeanInfo.*;
import ARoad0.AcsAddon.Accbee.MySQL.gBaseInterface.UserIDMySQL;
import ARoad0.AcsAddon.Accbee.MySQL.gBase.GroupIDMySQLImpl;
import ARoad0.AcsAddon.Accbee.MySQL.gBase.UserIDMySQLImpl;

import ARoad0.gBaseInterface.ACS;
import ARoad0.gBaseInterface.ImmutableName;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the UserIDMySQLImpl class.
 * The constructor adds the host as argument for a new MySQL account,
 * and as property.
 * It removes the pane 'Alias' from the Basic BeanInfo among the property descriptors,
 * the UserID property 'Order' and the EligibleParty 'M_ConstrainedChildACS' property.
 * @version 0.7.1
 */
public class UserIDMySQLImplBeanInfo extends UserIDImplBeanInfo {

  public UserIDMySQLImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * The constructor adds the host as argument for a new MySQL account.
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(UserIDMySQL.class );
    result.setDisplayName("UserID");

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value)
    result.setValue("withTabbedPanes", Boolean.TRUE );

    // 'withDeleteMethod' is true to display the 'delete' button in the beamer
    result.setValue("withDeleteMethod", Boolean.TRUE );
    
    // ----------------------- constructor parameters ---------------------------
    
    // 'constrArgumentClasses' is the list of argument classes
    // for the class constructor
    result.setValue("constrArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class, String.class, Boolean.class, GroupIDMySQLImpl.class} );
    
    // 'constrArgumentPropertyNames' is the list of property names
    // for the class constructor, even when the new instance
    // is created through a constructor, and not through another class
    result.setValue("constrArgumentPropertyNames", new String[]{
      "Name", "ACS", "EP Type", "Is Admin", "CurrentGroupID"} );

    // 'constrArgumentFromSelect' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "constrArgumentPropertyNames", or, as here,
    // in the order of the constructor method in ResourceImpl
    result.setValue("constrArgumentFromSelect", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.FALSE, Boolean.FALSE, Boolean.TRUE} );

    // 'constrArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another constructor argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    result.setValue("constrArgumentConcatenedNameSource", new Integer[]{
      new Integer(1), null, null, null, null} );

    // 'constrNullArgumentPossible' indicates if an argument may be null in the constructor
    result.setValue("constrNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'constrOptionalArgument' indicates if an argument must not be used in the
    // call to the constructor when it is null.
    result.setValue("constrOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'constrRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names. Is optional.
    // If it is true, the array of restricted values for an argument is searched
    // through BaseUtilityImpl.getRestrictedValues():
    // - first, in 3 ACS maps which define the constraints from the ACS,
    // and then "constrArgumentPropertyNames" must contain the property descriptor name.
    // The name words may be separated by spaces in "constrArgumentPropertyNames",
    // although there is no separation in the ACS.getM_staticSetOfItems() map.
    // - second, in the value "staticSetOfItems" of the property in this BeanInfo.
    result.setValue("constrRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE, Boolean.FALSE, Boolean.FALSE} );
      
    // ----------------------- copy parameters ---------------------------

    // 'copyArgumentClasses' is the list of argument classes
    // for the copy method, excepted for _toRegister at the end,
    //  which is another argument set to true.
    result.setValue("copyArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class, GroupIDMySQLImpl.class} );
      
    // 'copyArgumentPropertyNames' is the list of property names
    // for the copy method,
    // excepted for _toRegister which is set to true
    // following the copy() pattern,
    // and excepted for 'Parent Node' which is set to ""
    // to not display it in the user frame creation.
    result.setValue("copyArgumentPropertyNames", new String[]{
      "Name", "ACS", "", "CurrentGroupID"} );

    // 'copyArgumentFromExplorer' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "copyArgumentPropertyNames"
    result.setValue("copyArgumentFromExplorer", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.TRUE, Boolean.TRUE} );

    // 'copyArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    result.setValue("copyArgumentConcatenedNameSource", new Integer[]{
      new Integer(2), null, null, null} );

    // 'copyNullArgumentPossible' indicates if an argument may be null
    //  in the method
    result.setValue("copyNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE, Boolean.FALSE} );

    // 'copyOptionalArgument' indicates if an argument must not be used
    //  in the call to the method when it is null.
    result.setValue("copyOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'copyRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names.
    //  Is optional.
    // If it is true, the array of restricted values for an argument
    // is in the value "staticSetOfItems" of the property in this BeanInfo.
    result.setValue("copyRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    result.setHidden(false);
    result.setExpert(true);

    return result;
  } // getBeanDescriptor()

  /**
   * Adds the properties 'HostGroup' and 'SoftAlias'.
   * Removes the pane 'Alias' from the Basic BeanInfo, the UserID property 'Order',
   * and the EligibleParty 'M_ConstrainedChildACS' property.
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

                                    // for testing and tracking
                                    //System.out.println("call to "+
                                    //"UserIDMySQLImplBeanInfo.getPropertyDescriptors() : ");

    PropertyDescriptor[] l_interm;
    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;

    try {
      // ------------------- property "CurrentGroupID" --------------------
      // is not 'HostGroup' to allow an association to create an Actor 'connection'
      // when an UserID is created
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "CurrentGroupID", UserIDMySQLImpl.class, "getEorFirstHost", null);
      pd_three.setDisplayName("First host");
      pd_three.setShortDescription("The first host of the user may be completed by other hosts, "+
              "through the adding of this user as simple member.");
      pd_three.setBound(true);
      pd_three.setValue("tabbedPaneName", "  Members  " );
      //pd_three.setValue("panelGroupName", "  Main attributes  " );
      pd_three.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "SoftAlias" --------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "SoftAlias", UserIDMySQLImpl.class, "getL_SoftAlias", null);
      pd_four.setValue("isCollection", Boolean.TRUE );

      pd_four.setDisplayName("Proxy accounts");
      pd_four.setShortDescription("Proxy accounts are those for which "+
              "this is the proxied account.");
      pd_four.setBound(true);
      pd_four.setValue("tabbedPaneName", "  Members  " );
      pd_four.setValue("withViewButton", Boolean.TRUE );

      // ------- End of properties description ---------------------------------

      // start from the parent class bean info
      l_pdFromParent = super.getPropertyDescriptors();
      l_interm = new PropertyDescriptor[l_pdFromParent.length];
      int       decrement = 0;

      for (int i = 0; i < l_pdFromParent.length; i++) {
          // several Alias tab properties to remove
          if (l_pdFromParent[i].getValue("tabbedPaneName").equals("  Alias  ")) {
              decrement++;
          }
          else if (l_pdFromParent[i].getName().equals("M_ConstrainedChildACS")) {
              decrement++;
          }
          else if (l_pdFromParent[i].getName().equals("Order")) {
              decrement++;
          }
          else {
              l_interm[i - decrement] = l_pdFromParent[i];
          }
      } // for i

      l_result = new PropertyDescriptor[l_pdFromParent.length + 2 - decrement];
      System.arraycopy(l_interm, 0, l_result, 0, l_pdFromParent.length - decrement);

      l_result[l_result.length - 2] = pd_three;
      l_result[l_result.length - 1] = pd_four;

      l_interm = null;
      l_pdFromParent = null;
      return l_result;
    } catch (IntrospectionException e) {
      new ProcessError("In UserIDMemberImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In UserIDMemberImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch
  } // getPropertyDescriptors()

} // UserIDMySQLImplBeanInfo
