//Title:        Access Road
//Version:      0.7.0
//Copyright:    2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.MySQL.gBaseBeanInfo;

import ARoad0.gBaseInterface.*;
import ARoad0.gBaseBeanInfo.*;
import ARoad0.AcsAddon.Accbee.MySQL.gBase.*;
import ARoad0.AcsAddon.Accbee.MySQL.gBaseInterface.*;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

/**
 * This class is responsible for describing the VirtualFolderMySQLImpl class.
 * Adds the 'Definer' and 'Definer Host' properties, and changes the VirtualFolder property
 * 'Children' to change the number of arguments in the valueOf() method
 * to build up a child. Overrides the getCommonBeanDescriptor() method.
 * @version 0.7.0
 */
public class VirtualFolderMySQLImplBeanInfo extends VirtualFolderImplBeanInfo {

  public VirtualFolderMySQLImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor.
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(VirtualFolderMySQL.class );
    result.setDisplayName("VirtualFolder");

    result = getCommonBeanDescriptor(result);

    return result;
  } // getBeanDescriptor()

  /**
   * Get the common description of the bean. Overrides the super method.
   * @param _result is the BeanDescriptor to configure
   * @return the bean descriptor
   */
  protected BeanDescriptor getCommonBeanDescriptor(BeanDescriptor _result) {

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of ResourceImpl
    _result.setValue("withTabbedPanes", Boolean.TRUE );

    // 'withDeleteMethod' is true to display the 'delete' button in the beamer
    _result.setValue("withDeleteMethod", Boolean.TRUE );

    // 'commonBeanClass' is the class which has the bean info defining getCommonBeanDescriptor(),
    // and then which is the class or the superclass of the bean
    _result.setValue("commonBeanClass", VirtualFolderMySQLImpl.class );
    
    // ----------------------- constructor parameters ---------------------------

    // 'constrArgumentClasses' is the list of argument classes
    // for the class constructor
    _result.setValue("constrArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class, VirtualFolderMySQL.class, 
      String.class, UserID.class, GroupIDMySQL.class} );
    
    // 'constrArgumentPropertyNames' is the list of property names
    // for the class constructor
    _result.setValue("constrArgumentPropertyNames", new String[]{
      "Name", "ACS", "Parent Node", 
      "Virtual Folder Type", "Definer", "Definer Host" } );

    // 'constrArgumentFromSelect' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "constrArgumentPropertyNames", or, as here,
    // in the order of the constructor method
    _result.setValue("constrArgumentFromSelect", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE, 
      Boolean.FALSE, Boolean.TRUE, Boolean.TRUE} );

    // 'constrArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another constructor argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    _result.setValue("constrArgumentConcatenedNameSource", new Integer[]{
      new Integer(2), null, null,
      null, null, null} );

    // 'constrNullArgumentPossible' indicates if an argument may be null in the constructor
    _result.setValue("constrNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE, 
      Boolean.FALSE, Boolean.TRUE, Boolean.TRUE} );

    // 'constrOptionalArgument' indicates if an argument must not be used in the
    // call to the constructor when it is null.
    _result.setValue("constrOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, 
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'constrRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names. Is optional.
    // If it is true, the array of restricted values for an argument is found
    // through BaseUtilityImpl.getRestrictedValues():
    // - first, in 3 ACS maps which define the constraints from the ACS,
    // and then "constrArgumentPropertyNames" must contain the property descriptor name.
    // The name words may be separated by spaces in "constrArgumentPropertyNames",
    // although there is no separation in the ACS map.
    // - second, in the value "staticSetOfItems" of the property in this BeanInfo.
    _result.setValue("constrRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, 
      Boolean.TRUE, Boolean.FALSE, Boolean.FALSE} );
    
    // 'constrNonVisibleArgument' indicates if an argument is masqued
    // in the constructor window and sets to null in the constructor.
    // True when the ACS does not manage the property, and then, 
    // this value gets the name of the ACS method which returns a boolean 'false'
    // when the argument has to be null and to be not displayed. If
    // the returned value is 'true', the argument is displayed as usual.
    // For each non null value:
    // "constrNullArgumentPossible" has to be Boolean.TRUE,
    // "constrOptionalArgument" has to be Boolean.FALSE.
    _result.setValue("constrNonVisibleArgument", new String[]{
      null, null, "getManageVirtualFolderTrees",
      null, null, null} );


    // ----------------------- copy parameters ---------------------------

    // 'copyArgumentClasses' is the list of argument classes
    // for the copy method, excepted for _toRegister at the end,
    //  which is another argument set to true.
    _result.setValue("copyArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class, VirtualFolder.class} );

    // 'copyArgumentPropertyNames' is the list of property names
    // for the copy method, excepted for _toRegister which is set to true
    // following the copy() pattern.
    _result.setValue("copyArgumentPropertyNames", new String[]{
      "Name", "ACS", "Parent Node"} );

    // 'copyArgumentFromExplorer' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "copyArgumentPropertyNames"
    _result.setValue("copyArgumentFromExplorer", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.TRUE} );

    // 'copyArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    _result.setValue("copyArgumentConcatenedNameSource", new Integer[]{
      new Integer(2), null, null} );

    // 'copyNullArgumentPossible' indicates if an argument may be null
    //  in the method
    _result.setValue("copyNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE} );

    // 'copyOptionalArgument' indicates if an argument must not be used
    //  in the call to the method when it is null.
    _result.setValue("copyOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'copyRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names.
    //  Is optional.
    // If it is true, the array of restricted values for an argument is searched:
    // - first, in the value "staticSetOfItems" of the property in this BeanInfo,
    // - second, in the ACS.getM_staticSetOfItems() map
    // and then "constrArgumentPropertyNames" must contain the property descriptor name
    // (but the name words may be separated by spaces in "constrArgumentPropertyNames").
    _result.setValue("copyRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    _result.setHidden(false);
    _result.setExpert(true);

    return _result;
  } // getCommonBeanDescriptor()

  /**
   * Adds the 'Definer' property, and changes the VirtualFolder property
   * 'Children' to change the number of arguments in the valueOf() method
   * to build up a child.
   * Removes the property 'SoftAlias'.
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;
    PropertyDescriptor[] l_interm;

    try {
  
      // ------------------- property "Definer" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "Definer", VirtualFolderMySQLImpl.class, "getEorDefinerForDelegate", null);
      pd_one.setDisplayName("Alias Definer User");
      pd_one.setShortDescription("This running account of the view delegate "+
              "is the 'MySQL Definer' under which the view has rights "+
              "on the database resources");
      pd_one.setValue("tabbedPaneName", "  Alias  " );
      pd_one.setBound(false);
  
      // ------------------- property "Definer Host" ------------------------
      PropertyDescriptor pd_ten = new PropertyDescriptor(
        "DefinerHost", VirtualFolderMySQLImpl.class, "getEorDefinerHostForDelegate", null);
      pd_ten.setDisplayName("Alias Definer Host");
      pd_ten.setShortDescription("This running host of the view delegate "+
              "is the MySQL simple host under which the view has rights on the database resources");
      pd_ten.setValue("tabbedPaneName", "  Alias  " );
      pd_ten.setBound(false);

      // ------------------- property "Children" --------------------
      PropertyDescriptor pd_nine = new PropertyDescriptor(
        "Children", VirtualFolderMySQLImpl.class, "getEorL_Children", null);
      pd_nine.setDisplayName("Children");
      pd_nine.setShortDescription("The direct children of the virtual folder.");
      pd_nine.setBound(true);
      pd_nine.setValue("isCollection", Boolean.TRUE );
      pd_nine.setValue("valueTypeInGroup", new Object[] {VirtualFolderMySQL.class} );
      pd_nine.setValue("tabbedPaneName", "  As Node  " );
      pd_nine.setValue("withViewButton", Boolean.TRUE );
      pd_nine.setValue("withNewButton", Boolean.TRUE );
      pd_nine.setValue("withSelectButton", Boolean.FALSE );
      pd_nine.setValue("withDeleteButton", Boolean.TRUE );
      pd_nine.setValue("withKnownArgumentsForNew", new Object[] {
          null, "ACS", "this", null, "false", null, null} );
      pd_nine.setValue("removeMethod", "finalizeForUser");
      pd_nine.setValue("limitedSizeForPresentation", new Integer(8));

      // ------- End of properties description ---------------------------------

      l_pdFromParent = super.getPropertyDescriptors();
      l_interm = new PropertyDescriptor[l_pdFromParent.length];
      int       decrement = 0;

      for (int i = 0; i < l_pdFromParent.length; i++) {
          if (l_pdFromParent[i].getName().equals("Children")) {
              l_interm[i - decrement] = pd_nine;
          }
          else if (l_pdFromParent[i].getName().equals("SoftAlias")) {
              decrement++;
          }
          else {
              l_interm[i - decrement] = l_pdFromParent[i];
          }
      } // for i

      l_result = new PropertyDescriptor[l_pdFromParent.length + 1 - decrement];
      System.arraycopy(l_interm, 0, l_result, 0, l_pdFromParent.length - decrement);

      l_result[l_result.length - 2] = pd_one;
      l_result[l_result.length - 1] = pd_ten;

      l_pdFromParent = null;
      l_interm = null;
      return l_result;
    } catch (IntrospectionException e) {
      throw new InternalError("In VirtualFolderMySQLImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

} // VirtualFolderMySQLImplBeanInfo