//Title:        Access Road
//Version:      0.7.0
//Copyright:    2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.MySQL.gBaseInterface;

import java.util.List;

import ARoad0.gBaseInterface.*;

/**
 * This interface is responsible for modeling a set of databases
 * having a wildcard in its name, like 'BASE O%'. It has to be a Directory
 * to be called by the AclEntry when there is a rights changing, but it is not
 * a ResourceMySQL with primary rights.
 * The direct AclEntries of this class are copied into the inherited AclEntries list
 * of each database that the databases set owns. For instance, the databases set
 * 'BASE O%' copies its ACL into the lists of the two databases 'BASE One' and 'BASE OFF'
 * The type of a BasesSetMySQL is 'pattern-name_bases set'.
 * Most of the behavior of a BasesSetMySQL is like ResourceMySQL,
 * but it is not a ResourceMySQL with primary rights, nor a Directory with children.
 * It has to be a Directory to be called by the AclEntry when there is a rights changing,
 * through the method resetInheritedAclEntryRightsOfChildren().
 * @version 0.7.0
 */
public interface BasesSetMySQL extends Directory, ResourceMySQL {

  /**
   * Get the databases having a name matching the name of this BasesSetMySQL.
   * @return the connected databases that have the copies of all the AclEntries
   * of this bases set
   */
  public List<DirectoryMySQL> getEorConnectedDatabases();

} // BasesSetMySQL
