//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls


/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation, purely the version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details (gpl.html at the root of Access Road directory).
This license is applicable for 20 years since the publication of the program.
The applicable laws are the French ones, and the relevant court is at Nanterre
(92), France.
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation, Inc.,
675 Mass Ave, Cambridge, MA 02139, USA.
*/

package ARoad0.AcsAddon.Accbee.MySQL.gBaseInterface;

import ARoad0.gBaseInterface.*;

/**
* This interface is a VirtualFolder for simulating a MySQL view.
* The DEFINER mode is the default one for the MySQL views. It is needed
* for having a view, to create a VirtualFolder with an UserID as parameter of the constructor.
* This UserID becomes the Account of the AG context of the view delegate.
* There is also a GroupID as host of the account.
* This feature implies the creation of a VirtualFolderMySQLImpl class in the AcsAddon
* gBase package, as subclass of VirtualFolderImpl in the generic gBase package.
* The view definition is “frozen” at creation time in MySQL, but it is not useful
* to extend this constraint to the view simulation in Access Road.
* As actor, the view delegate in the resources tree runs under the UserID/GroupID
* passed on to the VirtualFolder constructor of the view. This is also the responsibility
* of the VirtualFolderMySQLImpl constructor. <P>
* See the <a href="{@docRoot}/copyright.html">Copyright</a>.
* @version 0.7.0
*/
public interface VirtualFolderMySQL extends VirtualFolder, ACSObjectAddon {

  /**
   * In the MySQL DEFINER mode, the view delegate runs under an account.
   * @return the account under which the view delegate runs. Null when there is no delegate.
   */
  public ImmutableUserID getEorDefinerForDelegate();
  
  /**
   * In the MySQL DEFINER mode, the view delegate runs under an account and a current host.
   * @return the host under which the view delegate runs. Null when there is no delegate.
   */
  public GroupIDMySQL getEorDefinerHostForDelegate();
} // VirtualFolderMySQL
