//Title:        Access Road
//Version:      0.7.2
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.Ubuntu.gBase;

import ARoad0.gBaseInterface.*;
import ARoad0.gBase.*;

import java.util.Collections;

import ARoad0.ErrB.BaseError;

/**
 * This class is responsible for creating a Linux Ubuntu as an access control system (ACS).
 * Almost all the work is done by the super class, and this class has just to use
 * the right methods of the superclass.
 * Called by gDMak.ActionNewACSyst and Ubuntu.gWork.AcsFactoryUbuntu.
 * This class uses for its GUI two BeanInfo, Ubuntu.gBaseBeanInfo.ACSFactoryImplUbuntuBeanInfo 
 * and ARoad0.gBaseBeanInfo.ACSPreFactoryImplBeanInfo.
 * ACSPreFactoryImplBeanInfo ensures the creation of the ACSFactory instance and
 * the ACS rights list. It is a javabean without specific bound properties.
 * @see ARoad0.AcsAddon.Accbee.Ubuntu.gBaseBeanInfo.ACSUbuntuImplBeanInfo
 */
public class ACSFactoryUbuntuImpl extends ACSFactoryImpl {
  private static final int    PRIME = 1000003;

  /**
   * A reusable empty instance for initialization,
   * to avoid the use of 'new' for temporary values.
   * Caution: never change the state of the returned instance.
   */
  static {
      EMPTY_INSTANCE = new ACSFactoryUbuntuImpl();
  }


 /**
  * Default constructor. 
  * Calls ACSFActoryImpl.addAcsVocabulary() to define the AcsAddon classes for Ubuntu.
  * Sets the version of this ACS addon.
  */
    public ACSFactoryUbuntuImpl() {

       super();
       try {
            // for an AcsAddon ACS,
            // the ACS vocabulary must be set before the call to 
            // AssociateUtilityImpl.controlAssociations()
            // that factory.addAssociationsToNewObject(), addXXXroots() perform.


                // no change on ACSFactoryUtilityImpl.ACSADDON_EDITOR
                // and ACSADDON_PACKAGE_EDITOR

           addAcsVocabulary(
                ACSFactoryUtilityImpl.ACSADDON_NAME, new String[] {
                    "Ubuntu"});
           addAcsVocabulary(
                ACSFactoryUtilityImpl.ACSFACTORY_CLASS, new String[] {
                    "ARoad0.AcsAddon.Accbee.Ubuntu.gBase.ACSFactoryUbuntuImpl"});
           addAcsVocabulary(
                ACSFactoryUtilityImpl.ACS_CLASS, new String[] {
                    "ARoad0.AcsAddon.Accbee.Ubuntu.gBase.ACSUbuntuImpl"});
           addAcsVocabulary(
                ACSFactoryUtilityImpl.RESOURCE_CLASS, new String[] {
                    "ARoad0.AcsAddon.Accbee.Ubuntu.gBase.ResourceUbuntuImpl"});
           addAcsVocabulary(
                ACSFactoryUtilityImpl.ACTOR_CLASS, new String[] {
                    "ARoad0.AcsAddon.Accbee.Ubuntu.gBase.ActorUbuntuImpl"});
           addAcsVocabulary(
                ACSFactoryUtilityImpl.DIRECTORY_CLASS, new String[] {
                    "ARoad0.AcsAddon.Accbee.Ubuntu.gBase.DirectoryUbuntuImpl"});
           addAcsVocabulary(
                ACSFactoryUtilityImpl.USER_CLASS, new String[] {
                    "ARoad0.AcsAddon.Accbee.Ubuntu.gBase.UserIDUbuntuImpl"});
           addAcsVocabulary(
                ACSFactoryUtilityImpl.ACSADDON_EXTENSION_CLASSES, new String[] {
                    "Executable/is/Actor"});
           addAcsVocabulary(
                ACSFactoryUtilityImpl.ACSADDON_SPECIALIZED_RIGHT_DOMAINS, new String[] {
                    "OtherRights.GpORightsSet"});

           addAcsVocabulary(
                ACSFactoryUtilityImpl.ACSADDON_VERSION, new String[] {"0.7.2"});
       }
       catch (BaseError b) {
           throw new InternalError(b.toString());
       }

    } // constructor

  /**
   * Called by Ubuntu.gWork.AcsFactoryUbuntu.
   * Calls addAcsVocabulary() to define the AcsAddon classes for Ubuntu, and also
   * to set what are the non-generic rights in the
   * ACSObjects on which the ACS constraints of the specialized rights
   * are applicable, and in that case, what domain is applicable. For this AcsAddon,
   * the domain 'GpORightsSet' is set for the specialized rights of the ResourceUbuntu
   * property 'OtherRights', and this is declared by the value 'OtherRights.GpORightsSet'.
   * Sets the version of this ACS addon.
   * @param _iSname the name of the IS containing this new ACS. May be null, and
   * ISFactoryUtilityImpl.DEFAULT_IS_NAME is then set as IS name and it is created
   * BaseManagerImpl.
   * @param _name the name of the new ACS. Never null.
   * @param _ACStype the ACS type. Never null.
   * @param _parentName the parent of this new ACS. No use if _iSname is null.
   * Otherwise, it has to contain the IS name.
   * @exception BaseError if _type is null, _parentName does not start with _iSname.
   */
  public ACSFactoryUbuntuImpl(ImmutableName _iSname, ImmutableName _name,
          String _ACStype, ImmutableName _parentName)
          throws BaseError {

       super(_iSname, _name, _ACStype, _parentName);

        // for an AcsAddon ACS,
        // the ACS vocabulary must be set before the call to
        // AssociateUtilityImpl.controlAssociations()
        // that factory.addAssociationsToNewObject(), addXXXroots() perform.

       // no changes on ACSFactoryUtilityImpl.ACSADDON_EDITOR, ACSADDON_PACKAGE_EDITOR

       addAcsVocabulary(
            ACSFactoryUtilityImpl.ACSADDON_NAME, new String[] {
                "Ubuntu"});
       addAcsVocabulary(
            ACSFactoryUtilityImpl.ACSFACTORY_CLASS, new String[] {
                "ARoad0.AcsAddon.Accbee.Ubuntu.gBase.ACSFactoryUbuntuImpl"});
       addAcsVocabulary(
            ACSFactoryUtilityImpl.ACS_CLASS, new String[] {
                "ARoad0.AcsAddon.Accbee.Ubuntu.gBase.ACSUbuntuImpl"});
       addAcsVocabulary(
            ACSFactoryUtilityImpl.RESOURCE_CLASS, new String[] {
                "ARoad0.AcsAddon.Accbee.Ubuntu.gBase.ResourceUbuntuImpl"});
       addAcsVocabulary(
            ACSFactoryUtilityImpl.ACTOR_CLASS, new String[] {
                "ARoad0.AcsAddon.Accbee.Ubuntu.gBase.ActorUbuntuImpl"});
       addAcsVocabulary(
            ACSFactoryUtilityImpl.DIRECTORY_CLASS, new String[] {
                "ARoad0.AcsAddon.Accbee.Ubuntu.gBase.DirectoryUbuntuImpl"});
       addAcsVocabulary(
            ACSFactoryUtilityImpl.USER_CLASS, new String[] {
                "ARoad0.AcsAddon.Accbee.Ubuntu.gBase.UserIDUbuntuImpl"});

       // Executable is an extension class
       addAcsVocabulary(
            ACSFactoryUtilityImpl.ACSADDON_EXTENSION_CLASSES, new String[] {
                "Executable/is/Actor"});

       // AcsFactoryUbuntu.createACS() defines the specialized rights
       // 'Target.GpORightsSet.Interface.Directory' for directories.
       // BaseUtilityImpl.getL_AcsRestrictedRights() uses this value
       // to set for CollectionPropertyEditor the allowed AGO Other rights,
       // and also CAgoUserUbuntuImpl.setL_otherRights().
       // See initializeConstraintsBeforeCreation() which defines
       // the associated standard rights.
       addAcsVocabulary(
            ACSFactoryUtilityImpl.ACSADDON_SPECIALIZED_RIGHT_DOMAINS, new String[] {
                "OtherRights.GpORightsSet"});

       addAcsVocabulary(
            ACSFactoryUtilityImpl.ACSADDON_VERSION, new String[] {"0.7.2"});
  } // constructor
  
  /**
   * Initializes the ACS constraints maps before the call to ACSUbuntuImpl.valueOf().
   * Adds to ACSFactoryImpl.l_ImmutableRules_ 6 specific rules for the Linux/Unix ACS: <P>
   * - AGO inheritance 1: new child/node inherits denying mask rights from the account <P>
   * - AGO right inheritance 1: child/node inherits rights from the direct and indirect parents <P>
   * - AGO right inheritance 2: at creation, child may inherit some current rights from its direct parent node <P>
   * - AGO right inheritance 2: at creation, node may inherit some current rights from its direct parent node <P>
   * - resource 1: new child/node inherits the current G, if activated, and G is not the account primary group, <P>
   * - AGO right 3: at administration, only node owner has the deleting and renaming rights, if activated <P>
   * The last rule is added as key in the ACS structure.
   * This method adds to the standard rights the key 'Resource.OtherRights'
   * for the ResourceUbuntu property, and with the values used for the Group standard rights.
   * This AcsAddon method may also add to ACSImpl.m_Structure_ some new keys.
   * An ACSFactoryUbuntuImpl for a new ACS derived from an Ubuntu ACS
   * has already all these values.
   * Called by createACS(). Overrides and calls first the super method.
   */
  public synchronized void initializeConstraintsBeforeCreation() {
      
      super.initializeConstraintsBeforeCreation();

      if (l_ImmutableRules_.contains("AGO right 1: AG rights delete O rights"+
              "(no exception, no user update)"))
          return;

      // an ACSFactoryUbuntuImpl for a new ACS derived from an Ubuntu ACS
      // has all these values in l_ImmutableRules_
      l_ImmutableRules_.add("AGO right 1: AG rights delete O rights (no exception, no user update)");
      l_ImmutableRules_.add("AGO right inheritance 1: new child/node inherits "+
              "denying mask rights from the account");
      l_ImmutableRules_.add("AGO right inheritance 1: child/node inherits rights "+
              "from the direct and indirect parents");
      l_ImmutableRules_.add("AGO right inheritance 2: at creation, child may inherit "+
              "some current rights from its direct parent node");
      l_ImmutableRules_.add("AGO right inheritance 2: at creation, node may inherit "+
              "some current rights from its direct parent node");

      l_ImmutableRules_.add("resource 1: new child/node inherits the current G, "+
              "if activated, and G is not the account primary group");
      // for the StickyBit, but no use in DirectoryUbuntuImpl
      l_ImmutableRules_.add("AGO right 3: at administration, only node owner "+
              "has the deleting and renaming rights, if activated");

      Collections.sort(l_ImmutableRules_);

      // informal description
      l_ImmutableRules_.add("    ");
      l_ImmutableRules_.add("    END OF THE FORMAL ACCESS ROAD RULES.");
      l_ImmutableRules_.add("    ");
      l_ImmutableRules_.add("    ------- THE SPECIFIC RULES FOR LINUX UBUNTU ------------");
      l_ImmutableRules_.add("    ");
      l_ImmutableRules_.add("    HOW RIGHTS ARE MANAGED:");
      l_ImmutableRules_.add("    ");
      l_ImmutableRules_.add("    For the files, Ubuntu combines the information about the Account/Group");
      l_ImmutableRules_.add("    context of the right user, and the AGO rights of the target and its parents.");
      l_ImmutableRules_.add("    In Access Road, the effective access rights on an Ubuntu resource");
      l_ImmutableRules_.add("    are selected by a two-steps process based on AG rights completed by");
      l_ImmutableRules_.add("    the specific AGO Other rights of Linux Ubuntu.");
      l_ImmutableRules_.add("    At the first step of the rights analysis, the AG inherited rights");
      l_ImmutableRules_.add("    on the target are considered. The Account/Group context");
      l_ImmutableRules_.add("    of the right user is used to set if the AGO rights of each target parent");
      l_ImmutableRules_.add("    allow an access to the target. If not, there is no rights at all.");
      l_ImmutableRules_.add("    At the second step of the rights analysis, the Account rights of the target");
      l_ImmutableRules_.add("    are applied first, if the target Account is in the Account/Group context");
      l_ImmutableRules_.add("    of the right user. Otherwise, the Group rights of the target are tried,");
      l_ImmutableRules_.add("    and if the Group does not match, the 'AGO Other' rights are always applied.");
      l_ImmutableRules_.add("    Indeed, this sequence (Account, then Group, then Other) is also relevant");
      l_ImmutableRules_.add("    in the first step, for the AG inherited rights.");
      l_ImmutableRules_.add("    ");
      l_ImmutableRules_.add("    ");
      l_ImmutableRules_.add("    SOME PITFALLS IN THE Ubuntu MANAGEMENT OF RIGHTS:");
      l_ImmutableRules_.add("    ");
      l_ImmutableRules_.add("    In Ubuntu, the nature of the inherited rights is different to");
      l_ImmutableRules_.add("    the 'classical' direct rights on the target, the so-called 'rwx'.");
      l_ImmutableRules_.add("    For instance, a pitfall is that 'r', as an inherited right from the parent,");
      l_ImmutableRules_.add("    does not mean 'read', as when it is a direct right, but it means 'goto' and");
      l_ImmutableRules_.add("    'view'. This is why Access Road defines the right 'rdir' for a directory.");
      l_ImmutableRules_.add("    Inherited right or not, 'rdir' has these varied meanings to replace 'r'.");
      l_ImmutableRules_.add("    A second pitfall, in the AGO inherited rights, is the Account/Group context.");
      l_ImmutableRules_.add("    For each parent, the Account/Group of the parent is compared to");
      l_ImmutableRules_.add("    the context for the target, and this enforces if the Account, the Group");
      l_ImmutableRules_.add("    or the Other rights are applicable for this parent. Because the contexts");
      l_ImmutableRules_.add("    may be identical or very diferent, this may be confusing.");
      l_ImmutableRules_.add("    ");
      l_ImmutableRules_.add("    ");
      l_ImmutableRules_.add("    THESE ACCESS PATHS ARE DELETED BY THE Ubuntu ADDON");
      l_ImmutableRules_.add("    AFTER THEIR CREATION BY THE GENERIC FUNCTIONS:");
      l_ImmutableRules_.add("    ");
      l_ImmutableRules_.add("    - path containing an AGO Other right for a couple (source, target),");
      l_ImmutableRules_.add("       while an another path, for the same couple, has");
      l_ImmutableRules_.add("       an 'AGO Owner or Contain' relation.");
      l_ImmutableRules_.add("    - path containing an AGO Other right for a couple (source, target),");      
      l_ImmutableRules_.add("       while an another path contains a triplet (source, node, target)");
      l_ImmutableRules_.add("       with the two relations 'Run Under' and 'AGO Owner or Contain'."); 
      l_ImmutableRules_.add("    - path containing an AGO Other right for a couple (source, target),");      
      l_ImmutableRules_.add("       while an another path contains a triplet (source, node, target)");
      l_ImmutableRules_.add("       with the two relations 'Is Member' and 'AGO Contain'.");      
      l_ImmutableRules_.add("       ");

      // BaseUtilityImpl.getL_AcsRestrictedRights() uses this value
      // to set for CollectionPropertyEditor the allowed AGO Other rights,
      // and also CAgoUserUbuntuImpl.setL_otherRights().
      // See the constructor which defines the associated specialized rights.
      m_StandardRights_.put(
                "Resource.OtherRights",
                l_GORightsSet_);

  } // initializeConstraintsBeforeCreation()


  /**
   * Creates the ACSUbuntuImpl with its structural arguments.
   * Overrides and copies the ACSFactoryImpl method, with simply, as difference,
   * the call to the ACSUbuntuImpl constructor.
   * @see ARoad0.gBase.ACSFactoryImpl#createACS
   * @exception BaseError if the integrity rules are not followed (with the list of
   *   all the user errors, including up to 2500 words when all the rules
   *   are unmatched), if the ACS is already known by BaseManagerImpl.
   * @return the new configured ACSUbuntuImpl
   */
  public synchronized ACSImpl createACS() throws BaseError {

      String message = checkACSbeforeCreation();
      if (message != null) throw new BaseError(message);

                        // for testing and tracking
                        //System.out.println("ACSFactoryImpl createACS(): call 1");

      initializeConstraintsBeforeCreation();

                        // for testing and tracking
                        //System.out.println("ACSFactoryImpl createACS(): call 2");

      // computes the policy checksum
      int checksum = m_Structure_.values().hashCode();
      int inter = 1;
      for (String[] l_ri : m_TypesAndExplorer_.values()) {
          for (int i = 0; i < l_ri.length; i++) {
              inter = PRIME*inter/10000 + l_ri[i].hashCode();
          }
      } // for l_ri
      checksum = PRIME*checksum + inter;
      inter = 1;
      for (String[] l_ass : m_AssociationsForNewObjects_.values()) {
          for (int i = 0; i < l_ass.length; i++) {
              inter = PRIME*inter/10000 + l_ass[i].hashCode();
          }
      } // for l_ass
      checksum = PRIME*checksum + inter;
      inter = 1;
      // does not use StringRight.hashcode() because it integrates the ACS name
      for (StringRight[] l_ri : m_StandardRights_.values()) {
          for (int i = 0; i < l_ri.length; i++) {
              inter = PRIME*inter/10000 + l_ri[i].getRight().hashCode();
          }
      } // for l_ri
      checksum = PRIME*checksum + inter;
      inter = 1;
      // does not use StringRight.hashcode() because it integrates the ACS name
      for (StringRight[] l_ri : m_SpecializedRights_.values()) {
          for (int i = 0; i < l_ri.length; i++) {
              inter = PRIME*inter/10000 + l_ri[i].getRight().hashCode();
          }
      } // for l_ri
      checksum = PRIME*checksum + inter;
      inter = 1;
      if (m_SubAndGroupPolicy_ != null) {
          for (String[] l_na : m_SubAndGroupPolicy_.values()) {
              for (int i = 0; i < l_na.length; i++) {
                  inter = PRIME*inter/10000 + l_na[i].hashCode();
              } // for i
          } // for l_na
      } // if
      checksum = PRIME*checksum + inter;

      if (type_.indexOf(" | ") != -1)
          type_ = type_.substring(0, type_.indexOf(" | ")) + " | " + Integer.toString(checksum);
      else
          type_ = type_ + " | " + Integer.toString(checksum);

      // creates the ACS without registering in BaseManagerImpl
      ACSUbuntuImpl acs = (ACSUbuntuImpl) ACSUbuntuImpl.valueOf(
                      iSname_, parentName_, name_,
                      type_, m_Structure_, m_TypesAndExplorer_,
                      AssociateUtilityImpl.getM_FilteredAssociations(
                        m_AssociationsForNewObjects_, true),
                      m_StandardRights_, m_SpecializedRights_, l_Rights_,
                      l_ImmutableRules_, m_CommentsOnProperties_,
                      m_AcsVocabulary_);

      message = null;

      // registering in BaseManagerImpl
      return createACSforGenericBase(acs);
  } // createACS()

} // ACSFactoryUbuntuImpl