//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.Ubuntu.gBase;

import ARoad0.gBase.*;
import ARoad0.gBaseInterface.*;
import ARoad0.AcsAddon.Accbee.Ubuntu.gBaseInterface.*;

import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

/**
* This class is a utility class for the Ubuntu ACS, with only static methods.
* It provides synchronized methods to copy, get a BaseObject or a BeanInfo,
* and mainly handle rights and right operations.
* The calls to this class are reserved to the classes of this package, even if
* its methods are public since they override the super methods.
* Caution: this class is not defined in ACSFactoryUtilityImpl.M_GENERIC_ACS_VOCABULARY,
* then it is not possible to get it outside this package through
* ARoad0.gBaseInterface.ACSRun.getM_AcsVocabulary().
* This class has no subclass in this package.
* @version 0.7.0
*/
public class BaseUtilityUbuntuImpl extends BaseUtilityImpl {

  static {
      instance__ = new BaseUtilityUbuntuImpl();
  }

  private static Set<StringRight> l_rightsForLinuxStickybit__ = null;
  private static Set<StringRight> l_denyRightsForLinuxStickybit__ = null;

    // Constructors

  /**
   * Public constructor
  */
  public BaseUtilityUbuntuImpl() {
  }

    /**
    * Only method to obtain the unique instance of BaseUtilityUbuntuImpl.
    * @return a BaseUtilityUbuntuImpl instance
    */
  public static BaseUtilityImpl getInstance() {
    return instance__;
  } // getInstance()

  /**
  * Analyze the directory rights to say if the AGO inherited rights which
  * are derived from the directory rights allow some access to the child.
  * Return true if the Directory AGO rights contain an equivalent to the metaright
  * 'execute_for_directory', or if the method of the superclass returns true.
  * Overrides the superclass method.
  * Called by the Resource constructor, Directory.setL_ownerRights() and setL_groupRights().
  * @param _l_rights rights of a Directory
  * @return true if the rights allow an access to the Directory children
  **/
  public static synchronized boolean withAcrossToDirectoryAGRights(StringRight[] _l_rights) {
      if (_l_rights == null || _l_rights.length == 0) return false;
      
      boolean result = BaseUtilityImpl.withAcrossToDirectoryAGRights(_l_rights);

      if (result) return result;

      StringRight right;
      StringRight xdir = BaseManagerImpl.getInstance().getMetaRight("execute_for_directory");

      // test rights
      for (int k = 0; k < _l_rights.length; k++) {
        if (_l_rights[k] == null) continue;
        right = (StringRight)_l_rights[k].getEorMetaRight();
        if (right == null) continue;
        if (right.equals(xdir)) {
                result = true;
                break;
        }
      } // for ite

              // for testing and tracking
              //if (_l_rights != null && _l_rights[0].getRight().equals("xdir_Node"))
                //  System.out.println("in BaseUtilityUbuntuImpl.withAcrossToDirectoryAGORights(), result for 'xdir_Node' at 0: ");
                //  System.out.println("----------------------- "+result);

      right = null;
      xdir = null;
      return result;
  } // withAcrossToDirectoryAGORights()

  /**
   * This method allows to implement the 'sticky bit' management for Directories.
  * Called by ResourceImplUbuntu.setL_groupRights() and CAgoUserImplUbuntu.setL_otherRights().
  * @param _acs to process. May be null.
  * @return the set of the 6 positive deleting and renaming metarights, or their
  *    equivallnts as acsrights if they exist in the argument. Never null.
  **/
   public static synchronized Set<StringRight> getDeleteRenameRights(ImmutableACS _acs) {
        if (l_rightsForLinuxStickybit__ == null) {
           l_rightsForLinuxStickybit__ = new HashSet(INITIAL_CAPACITY); // capacity == 10
           l_rightsForLinuxStickybit__.add(BaseManagerImpl.getInstance().
                   getMetaRight("delete"));
           l_rightsForLinuxStickybit__.add(BaseManagerImpl.getInstance().
                   getMetaRight("delete_child"));
           l_rightsForLinuxStickybit__.add(BaseManagerImpl.getInstance().
                   getMetaRight("delete_item"));
           l_rightsForLinuxStickybit__.add(BaseManagerImpl.getInstance().
                   getMetaRight("delete_tree"));
           l_rightsForLinuxStickybit__.add(BaseManagerImpl.getInstance().
                   getMetaRight("rename"));
           l_rightsForLinuxStickybit__.add(BaseManagerImpl.getInstance().
                   getMetaRight("rename_child"));
        }

        if (_acs == null) return l_rightsForLinuxStickybit__;

        Set<StringRight> l_result = new HashSet(INITIAL_CAPACITY); // capacity == 10
        StringRight[]    l_rights = _acs.getL_Rights();
        for (int i = 0; i < l_rights.length; i++) {
            if (l_rightsForLinuxStickybit__.contains(l_rights[i].getEorMetaRight()))
                    l_result.add(l_rights[i]);
        } // for i

        l_rights = null;
        return l_result;
   } // getDeleteRenameRights()
  
  /**
   * This method allows to implement the 'sticky bit' management for Directories.
  * Called by ResourceImplUbuntu.setL_inheritedGroupRights() and
  * CAgoUserImplUbuntu.setL_inheritedOtherRights() constructor.
  * @param _acs to process. May be null.
  * @return the set of the 6 negative deleting and renaming metarights, or their
  *    equivalents as acsrights if they exist in the not-null _acs. Never null. 
  **/
   public static synchronized Set<StringRight> getDenyDeleteRenameRights(ImmutableACS _acs) {
        if (l_denyRightsForLinuxStickybit__ == null) {
           l_denyRightsForLinuxStickybit__ = new HashSet(INITIAL_CAPACITY); // capacity == 10
           l_denyRightsForLinuxStickybit__.add(BaseManagerImpl.getInstance().
                   getMetaRight("deny_delete"));
           l_denyRightsForLinuxStickybit__.add(BaseManagerImpl.getInstance().
                   getMetaRight("deny_delete_child"));
           l_denyRightsForLinuxStickybit__.add(BaseManagerImpl.getInstance().
                   getMetaRight("deny_delete_item"));
           l_denyRightsForLinuxStickybit__.add(BaseManagerImpl.getInstance().
                   getMetaRight("deny_delete_tree"));
           l_denyRightsForLinuxStickybit__.add(BaseManagerImpl.getInstance().
                   getMetaRight("deny_rename"));
           l_denyRightsForLinuxStickybit__.add(BaseManagerImpl.getInstance().
                   getMetaRight("deny_rename_child"));
        }
        
        if (_acs == null) return l_denyRightsForLinuxStickybit__;
        
        Set<StringRight> l_result = new HashSet(INITIAL_CAPACITY); // capacity == 10
        StringRight[]    l_rights = _acs.getL_Rights();
        for (int i = 0; i < l_rights.length; i++) {
            if (l_denyRightsForLinuxStickybit__.contains(l_rights[i].getEorMetaRight()))
                    l_result.add(l_rights[i]);
        } // for i
        
                          // for testing and tracking
                          //System.out.println("BaseUtilityUbuntuImpl.getDenyDeleteRenameRights() ");
                          //System.out.println("--- result are:");
                          //for (Iterator ite = l_result.iterator(); ite.hasNext(); ) {
                                  // for testing and tracking
                            //      System.out.println(" ------- "+ite.next());
                          //} // for
        
        l_rights = null;
        return l_result;
   } // getDenyDeleteRenameRights()
   
  /**
   * This method allows to implement the 'umask' management for Resources.
  * Subtracts the umask denying AGO rights from a set. The NX ACS is detected
  * when its type contains ACSImpl.LINUX or ACSImpl.UNIX.
  * If _l_Rights is empty, return an empty set.
  * Calls UserIDUbuntu.processUMask() and super.subtractNegativeNotRemovingInOneACS().
  * If the _res userID is null, returns the subtraction of the negative rights in
  * _l_Rights, if any, to the positive rights in the same _l_Rights.
  * @param _l_Rights positive or negative rights to process. Not null and no null item.
  *     May be empty. 
  * @param _acs uses all the rights in the two right sets. May be null.
  * @param _res the Resource on which the returned rights are applied. Never null.
  * @param _maskType is for AGO rights 'owner', 'group' or 'other'. Never null.
  * @return the set of positive rights. May be empty, not null. 
  **/
  public static synchronized Set<StringRight> subtractUMask(
           Set<StringRight> _l_Rights, ImmutableACS _acs, ImmutableResource _res,
           String _maskType) {
       
    Set<StringRight>    l_posRights = new HashSet(INITIAL_CAPACITY / 2);
    Set<StringRight>    l_negRights = new HashSet(INITIAL_CAPACITY / 2); 
    StringRight         right = null; 
    boolean             isNxDirectory = false;
    boolean             isDirectory = false;
    String              mask = null;
 
                          // for testing and tracking purpose
                     //if (_maskType.equals("other") //&& _res.getName().lastComponent().equals("f_jerry")
                     //) {
                       //  System.out.println("BaseUtilityImpl.subtractUMask(), for "+
                         //        "_res: "+_res.getName()+", _l_Rights argument: ");
                         //for (Iterator ite = _l_Rights.iterator(); ite.hasNext(); ) {
                           //  System.out.println("----------- "+ite.next());
                         //} // for ite
                     //} // if
    
    // filter the positive and negative rights
    if (!_l_Rights.isEmpty()) {
        for (Iterator ite = _l_Rights.iterator(); ite.hasNext(); ) {
            right = (StringRight)ite.next();
            if (right.isGrantingRight()) l_posRights.add(right);
            else l_negRights.add(right);
        }
    } // if
    else {
        // return an empty set
        l_negRights = null;
        return l_posRights;
    }
    
    // initialize the Directory booleans for Linux and Unix ACS
    if (_res instanceof ImmutableDirectory) {
        if (_acs.getACSType().indexOf(ACSImpl.LINUX) != -1
            ) isNxDirectory = true;
        else isDirectory = true;
    } // if

    // process the umask of the userID
    if (_res.getEorUserID() != null) {

        if (_maskType.equals("owner")) {
            mask = ((UserIDUbuntu)_res.getEorUserID()).getUMask().substring(0,
                    ((UserIDUbuntu)_res.getEorUserID()).getUMask().indexOf(" "));
        }
        else if (_maskType.equals("group")) {
            mask = ((UserIDUbuntu)_res.getEorUserID()).getUMask().substring(
                    ((UserIDUbuntu)_res.getEorUserID()).getUMask().indexOf(" ") + 1);
            mask = mask.substring(0, ((UserIDUbuntu)_res.getEorUserID()).getUMask().indexOf(" "));

                              // for testing and tracking purpose
                             //if (_maskType.equals("group") //&& _res.getName().lastComponent().equals("f_jerry")
                               //  ) {
                                     //System.out.println("BaseUtilityUbuntuImpl.subtractUMask(), "+
                                       // "mask for 'group': "+mask);
                             //} // if

        }
        else if (_maskType.equals("other")) {
            mask = ((UserIDUbuntu)_res.getEorUserID()).getUMask().substring(
                    ((UserIDUbuntu)_res.getEorUserID()).getUMask().indexOf(" ") + 1);
            mask = mask.substring(mask.indexOf(" ") + 1);

                              // for testing and tracking purpose
                         //System.out.println("in BaseUtilityUbuntuImpl.subtractUMask(), "+
                          //"mask for 'other': "+mask);

        } // else        
        
        l_negRights.addAll(
                ((UserIDUbuntu)_res.getEorUserID()).processUMask(mask, isDirectory, isNxDirectory));

                          // for testing and tracking purpose
                     //if (_maskType.equals("group") //&& _res.getName().lastComponent().equals("f_jerry")
                       //  ) {
                             //System.out.println("BaseUtilityImpl.subtractUMask(), "+
                              //"l_negRights(): ");
                             //for (Iterator ite = l_negRights.iterator(); ite.hasNext(); ) {
                                 //System.out.println("---- "+ite.next());
                             //} // for ite
                             //System.out.println("and l_posRights(): ");
                             //for (Iterator ite = l_posRights.iterator(); ite.hasNext(); ) {
                                 //System.out.println("---- "+ite.next());
                             //} // for ite
                     //} // if (_maskType.equals("group")
                      
    } // if
    
    // subtracts the negative rights to the positive ones
    l_posRights = UtilityImpl.getFirstInstance().subtractNegativeNotRemovingInOneACS(
           l_posRights, l_negRights, _acs, _res instanceof ImmutableDirectory);

                          // for testing and tracking purpose
                     //if (_maskType.equals("group") && _res.getName().lastComponent().equals("f_jerry")) {
                         //System.out.println("BaseUtilityImpl.subtractUMask(), "+
                          //"result: ");
                         //for (Iterator ite = l_posRights.iterator(); ite.hasNext(); ) {
                            // System.out.println("----------- "+ite.next());
                         //} // for ite
                         //System.out.println("");
                     //} // if
    
    
    l_negRights = null; 
    right = null; 
    mask = null;  
    return l_posRights;
  } // subtractUMask()
   
  /**
   * Gets the child inherited rights from the AGO directory rights, for the children.
   * Calls directly the superclass method.
   * Called by CAgoUserUbuntuImpl.getL_inheritedGroup/OtherRights().
   * @param _l_parentRights acsrights or metarights of a directory. Never null.
   * @param _acs of the Resource calling this method. Never null.
   * @return the child rights, without the rights in _l_parentRights which are not
   *     directory rights. May be an empty set but not null.
   */
   protected static synchronized Set<StringRight> getChildRightsSetFromParentAGORights(
            StringRight[] _l_parentRights, ImmutableACS _acs) {
        
      return BaseUtilityImpl.getChildRightsSetFromParentAGRights(_l_parentRights, _acs);
   } // getChildRightsSetFromParentAGORights()

} // BaseUtilityUbuntuImpl
