//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.Ubuntu.gBase;

import ARoad0.gBaseInterface.*;
import ARoad0.AcsAddon.Accbee.Ubuntu.gBaseInterface.*;
//import ARoad0.gBase.ACSFactoryUtilityImpl;

import ARoad0.ErrB.CreateError;
import ARoad0.ErrB.UpDateError;

import java.util.Map;
//import java.util.HashMap;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import java.io.ObjectStreamField;

/**
 * This class is responsible for modeling a Linux Ubuntu executable,
 * which may have Linux/Unix setUID and setGID capabilities. To be more general,
 * the changing capability is modelized, with EnforceGID and EnforceUID.
 * As every actor, it implements EligibleParty to have rights
 * on resources through aclEntries.
 * It uses an embedded instance of CAgoUserImplUbuntu to handle the AGO
 * 'Other' rights and sticky bit in a Linux/Unix operating system.
 * This class is declared in the ACS vocabulory by the key
 * ACSFactoryUtilityImpl.ACSADDON_EXTENSION_CLASSES.
 * <P>
 * It is a javabean with the following bound properties:
 * 'EnforceUID', 'EnforceGID'.
 * The listeners are proper to this instance, outside gBase,
 * transient (not serialized in this instance backup).
 * They are called in any order. They receive only a copy of the new value,
 * to protect the property. All the exceptions from the listeners are catched,
 * and a dialog box is displayed to inform the user.
 * All variables are protected. This class has no subclass in this package.
 * @version 0.7.0
 */
public class ExecutableUbuntuImpl extends ActorUbuntuImpl implements ExecutableUbuntu {
  // private variables
  private static final long   serialVersionUID__ = 0;
  // protected variables
  protected boolean             EnforceGID_ = false;
  protected boolean             EnforceUID_ = false;
  protected String              exeType_ = new String();

  /**
   * A reusable empty instance for initialization,
   * to avoid the use of 'new' for temporary values.
   * Caution: never change the state of the returned instance.
   */
  public static final ExecutableUbuntuImpl EMPTY_INSTANCE = new ExecutableUbuntuImpl();

  /** variable for the JDK 2 serialization
  * @serial ("EnforceGID_",Boolean.TYPE),("EnforceUID_",Boolean.TYPE),
  *   ("exeType_",String.class), ("agoHandler_",CAgoUserImplUbuntu.class)
  */
  public final static ObjectStreamField[] serialPersistentFields =
  {
        new ObjectStreamField("EnforceGID_",Boolean.TYPE),
        new ObjectStreamField("EnforceUID_",Boolean.TYPE),
        new ObjectStreamField("exeType_",String.class)
  };

  // Constructors
  public ExecutableUbuntuImpl() {
  }

  /**
  * Used by the beamer editor for the ACS resources list, from the 'New' button,
  * and by the gWork.AcsFactoryUbuntu. Calls the super method.
  * @param _name of the actor as a resource
  * @param _acs main ACS of the actor.
  * @param _parent of the actor
  * @param _owner owns the resource; may be null.
  * @param _resourceType is also the EP type, and may be null if the ACS allows it.
  * @param _bridgeRef another-ACS Actor for which this instance is a bridge;
  *   may be null.
  * @exception CreateError see super(_name, _acs, _parent, _actor_name, _bridgeRef) exception.
  */
  public ExecutableUbuntuImpl(ImmutableName _name, ACS _acs, Directory _parent, UserID _owner,
    String _resourceType, ImmutableActor _bridgeRef) throws CreateError {

      super(_name, _acs, _parent, _owner, _resourceType, _bridgeRef);

  } // ExecutableUbuntuImpl()

  //
  // ------------- Methods of the implemented interfaces ------------------
  //

  /**
   * UNUSED.
  * This method provides a copy of the instance
  * with another name. Each internal variable that is an external
  * object reference have a shallow copy (no deep copy).
  * Each aclEntry is copied, but not the list of ViewInBase
  * that contains this instance.
  * Caution: do not copy the user and the group if _result
  * does not belong to aCS_ and if there are no user or group
  * in the _result ACS which have similar names - that is that
  * are ended by the same Strings in this instance user or group names.
  * Call copyOwnVariables().
  * This method is overridden by the subclasses.
  * @param _name copy name. Usually creates as an extension of _parent name.
  * @param _acs main ACS of the copy to create. May not be the ACS of this instance.
  * @param _parent DirectoryImpl which belongs to _acs and will have the copy.
     May be null.
  * @param _toRegisterInManager false if this method must not register
  *  the instance in the ACS.
  * @return a new instance of ExecutableUbuntuImpl
  * @exception CreateError of the constructor with the same parameters,
  *   or if _parent ACS is not _acs, or addAclEntry() exceptions.
  **/
  public Object copy(ImmutableName _name, ACS _acs, Directory _parent,
    Boolean _toRegisterInManager) throws CreateError {

      if (_acs == null || _name == null ||
        (_parent != null && _parent.getEorACS() != _acs)) {
          throw new CreateError("in ExecutableImplUbuntu.copy(),"+
          " _name or _acs is null, or _aliasRef is not an Actor,"+
          " or _parent ACS is not _acs.");
      }

      ExecutableUbuntuImpl result = null;
      //result = new ExecutableImpl(
        //  _name, _acs, _parent, null, getResourceType(), getEPType(), null,
          //_toRegisterInManager.booleanValue());

      result = copyOwnVariables(result, false);

      return result;
  } // copy(,,,)


  /**
   * UNUSED.
  * This method provides a copy of the instance
  * as copy() but without copying internal aclEntries.
  * Each internal variable that is an external
  * object reference have a shallow copy (no deep copy).
  * Each aclEntry is copied, but not the list of ViewInBase
  * that contains this instance.
  * Caution: does not copy the user and the group if _result
  * does not belong to aCS_ and if there are no user or group
  * in the _result ACS which have similar names - that is that
  * are ended by the same Strings in this instance user or group names.
  * Calls copyOwnVariables().
  * This method calls the super method and it is overridden by the subclasses.
  * @param _name copy name. Usually creates as an extension of _parent name.
  * @param _acs main ACS of the copy to create. May not be the ACS of this instance.
  * @param _parent DirectoryImpl which belongs to _acs and will have the copy.
     May be null.
  * @param _toRegisterInManager false if this method must not register
  *  the instance in the ACS.
  * @return a new instance of ResourceImpl
  * @exception CreateError of the constructor with the same parameters,
  *   or if _parent ACS is not _acs, or addAclEntry() exceptions.
  **/
  public Object copyWithoutInternalAcl(ImmutableName _name, ACS _acs, Directory _parent,
    Boolean _toRegisterInManager) throws CreateError {

      if (_acs == null || _name == null ||
        (_parent != null && _parent.getEorACS() != _acs)) {
          throw new CreateError("in ExecutableImplUbuntu.copy(),"+
          " _name or _acs is null,"+
          " or _parent ACS is not _acs.");
      }

      ExecutableUbuntuImpl result = null;
      //result = new ExecutableImpl(
        //  _name, _acs, _parent, null, getResourceType(), getEPType(), null,
          //_toRegisterInManager.booleanValue());

      // copies the external aclEntries if they exist
      result = copyOwnVariables(result, true); // no internal aclEntry copy

      //if (result.getEorL_AclConnectedResources().isEmpty()) return result;
      //else return EligiblePartyImpl.deleteAclEntries(result);
      return result;
  } // copyWithoutAcl(,,,)

  /**
   * Fire an event 'ExeType'.
   * Noop if the resource type is 'byAcsCreator' or 'immutable'.
   * @param _p type of executable.
   *  Different to type_ from EP and resourceType from Resource.
   */
  public synchronized void setExeType(String _p){
    if (type_.indexOf("<byAcsCreator>") != -1 ||
            type_ .indexOf("<immutable>") != -1) return;

    String oldValue = exeType_;
    exeType_ = _p;

                // for testing purpose
                //System.out.println("call to ExecutableUbuntuImpl.setExeType() "+
                  //", oldValue: "+oldValue+", exeType_: "+exeType_);

    try {
        firePropertyChange("ExeType", oldValue, _p);
    } catch (Exception e) {
            ARoad0.Gui1.DialogBox.showAlarmMessage(
              "There is an exception after the data update, from outside the database."+
              "You should save the base and stop the program, then run it once again.");
            System.out.println("From ExecutableImplUbuntu.setExeType(), firePropertChange() throws:");
            e.printStackTrace();
    }
  } // setExeType()

  /**
   * @return String: type of executable.
   *  Different to type_ from EP and resourceType from Resource.
   */
  public final String getExeType(){
      return exeType_;
  }

  /**
   * Noop if the resource type contains 'byAcsCreator' or 'immutable'.
   * Calls super.setEorCurrentUserIDForUser() to set the Resource UserID as current account,
   * (even if null), or to set the current account to null if _b is false.
   * Fires an event 'EnforceUID'.
   * @param _b true if the userID is always its userID whatever the
   * caller userID (this is the setUID bit on *NX systems).
   * @exception UpDateError if the EnforceUID property is immutable.
   */
  public synchronized void setEnforceUID(boolean _b) throws UpDateError {
    if (type_.indexOf("<byAcsCreator>") != -1 ||
            type_ .indexOf("<immutable>") != -1)
          throw new UpDateError(
            "The resource type of "+getName().lastComponent()+
            " sets it as immutable for its EnforceUID property.");

    boolean oldValue = EnforceUID_;
    EnforceUID_ = _b;

   if (_b) super.setEorCurrentUserIDForUser(userID_);
   else super.setEorCurrentUserIDForUser(null);

    try {
        firePropertyChange("EnforceUID", new Boolean(oldValue), new Boolean(_b));
    } catch (Exception e) {
            ARoad0.Gui1.DialogBox.showAlarmMessage(
              "There is an exception after the data update, from outside the database."+
              "You should save the base and stop the program, then run it once again.");
            System.out.println("From ExecutableImplUbuntusetEnforceUID(), firePropertChange() throws:");
            e.printStackTrace();
    }
  } // setEnforceUID()

  /**
  * @return boolean true if the userID is always its userID whatever the
  * caller userID. This is the setUID bit on Linux/Unix systems.
  */
  public final boolean getEnforceUID(){
          return EnforceUID_;
  }

  /**
   * Noop if the resource type contains 'byAcsCreator' or 'immutable'.
   * Calls super.setEorCurrentGroupIDForUser() to set the Resource groupID as current group
   * (even if null), or to set the current group to null if _b is false.
   * Fires an event 'EnforceGID'.
   * @param _b true if the userID is always its userID whatever the
   * caller userID. This is the setUID bit on Linux/Unix systems.
   * @exception UpDateError if the EnforceGID property is immutable.
   */
  public synchronized void setEnforceGID(boolean _b) throws UpDateError {
    if (type_.indexOf("<byAcsCreator>") != -1 ||
            type_ .indexOf("<immutable>") != -1)
          throw new UpDateError(
            "The resource type of "+getName().lastComponent()+
            " sets it as immutable for its EnforceGID property.");

    boolean oldValue = EnforceGID_;
    EnforceGID_ = _b;

    if (_b) super.setEorCurrentGroupIDForUser(groupID_);
    else super.setEorCurrentGroupIDForUser(null);

    try {
        firePropertyChange("EnforceGID", new Boolean(oldValue), new Boolean(_b));
    } catch (Exception e) {
            ARoad0.Gui1.DialogBox.showAlarmMessage(
              "There is an exception after the data update, from outside the database."+
              "You should save the base and stop the program, then run it once again.");
            System.out.println("From ExecutableImplUbuntusetEnforceGID(), firePropertChange() throws:");
            e.printStackTrace();
    }
  } // setEnforceGID()

  /**
     * @return boolean true if the groupID is always its groupID whatever the
     * caller groupID (this is the setGID bit on *NX systems).
   */
  public final boolean getEnforceGID() {
      return EnforceGID_;
  }

  /**
   * Checks getEnforceUID() then calls the super method. When getEnforcedUID() is true,
   * the current account is always its Resource account, so this method is not operating.
   * To change the current account when enforceUID is true, sets enforceUID
   * to false to nullify the current account, then updates the current account.
   * Calls the super method which this method overrides.
  * @param _u the current userID of the actor. May be null.
  * @exception UpDateError if getEnforceUID() is true.
  */
  public synchronized void setEorCurrentUserIDForUser(ImmutableUserID _u)
          throws UpDateError {

    if (getEnforceUID())
          throw new UpDateError(
            " As a right user, the current account of "+getName().lastComponent()+
            " is always its Resource account, since it is enforced as Executable.");

    super.setEorCurrentUserIDForUser(_u);
  } // setEorCurrentUserIDForUser()


  /**
   * Check getEnforceGID() then call the super method. When getEnforceGID() is true,
   * the current group is always its Resource group, so this method is not operating.
   * To change the current group when enforceGID is true, set enforceGID
   * to false to nullify the current group, then update the current group.
  * @param _g the current groupID of the actor. May be null.
  * @exception UpDateError if getEnforceGID() is true.
  */
  public synchronized void setEorCurrentGroupIDForUser(ImmutableGroupID _g) throws UpDateError {

    if (getEnforceGID())
          throw new UpDateError(
            "As a rights user, the current group of "+getName().lastComponent()+
            " is always its Resource group, since it is enforced as Executable.");

    super.setEorCurrentGroupIDForUser(_g);
  } // setEorCurrentGroupIDForUser()

  //
  // --------------- Overridden public methods ---------------------
  //

  /**
  * Adds a listener to the bean.
  * Used by Gui1.CommonPropertyEditor for the properties:
  * 'EnforceUID', 'EnforceGID', 'ExeType'.
  * Overrides the super method and calls it.
  * @param _propertyName name of the property
  * @param _l PropertyChangeListener to add. Caution: the listener must NOT
  *   have specific hashCode() and equals() methods, to work with HashSet
  *   in the removing operations.
  */
  public void addPropertyChangeListener(String _propertyName,
    PropertyChangeListener _l) {

      if (changeSupport_ == null)
          changeSupport_ = new PropertyChangeSupport(this);

      if (_propertyName.equals("EnforceUID") ||
              _propertyName.equals("EnforceGID") ||
              _propertyName.equals("ExeType") )
          changeSupport_.addPropertyChangeListener(_propertyName, _l);
      else super.addPropertyChangeListener(_propertyName, _l);

  } // addPropertyChangeListener(String _propertyName, _l)

  /**
  * Removes a listener to the bean.
  * Used by Gui1.CommonPropertyEditor for the properties:
  * 'EnforceUID', 'EnforceGID', 'ExeType'.
  * Note: using this method does not remove a listener which has been
  * registered through addPropertyChangeListener(PropertyChangeListener _l).
  * Overrides the super method and calls it.
  * @param _propertyName name of the property
  * @param _l PropertyChangeListener to remove
  */
  public void removePropertyChangeListener(String _propertyName,
    PropertyChangeListener _l){

      // no action if the transiant variable is null
      if (changeSupport_ == null) return;

                // for testing and tracking
                //if (_propertyName.equals("AclRights"))
                  //  System.out.println("ExecutableUbuntuImpl.removePropertyChangeListener(), call from "+
                    //_l.getClass()+" to "+name_+" for the property "+_propertyName);

      if (_propertyName.equals("EnforceUID") ||
              _propertyName.equals("EnforceGID") ||
              _propertyName.equals("ExeType") )
          changeSupport_.removePropertyChangeListener(_propertyName, _l);
      else super.removePropertyChangeListener(_propertyName, _l);

                //if (_propertyName.equals("AclRights")) {
                  //  System.out.println("AclRights listeners after the removing of "+_l+": ");
                    //PropertyChangeListener[] l_list = changeSupport_.getPropertyChangeListeners("AclRights");
                    //for (int i = 0; i < l_list.length; i++) {
                      //  System.out.println("-------"+l_list[i]);
                    //}
                    //l_list = null;
                //} // if

  } // removePropertyChangeListener()

  /**
   * @return the static serialVersionUID__ of this class
   */
  public long getSerialVersionUID()	{
          return serialVersionUID__;
  }

  /**
   * Clones without register the new object in the ACSs.
   * Caution: don't use any modifying method that may altere the integrity
   * of the base.
   * @exception CloneNotSupportedException
   * @return Object clone of the instance without registration in the base.
   */
  public synchronized Object clone() {
    ExecutableUbuntuImpl clon = (ExecutableUbuntuImpl) super.clone();
    clon.exeType_ = exeType_;
    clon.EnforceGID_ = EnforceGID_;
    clon.EnforceUID_ = EnforceUID_;

    return clon;
  }

  /**
   * @param _obj to compare
   * @return boolean: true if _obj is an instance of ExecutableImplImplUbuntu (or a derived class)
   *  equals to this instance
   */
  public synchronized boolean equals(Object _obj)
  {
      if (this == _obj)
          return true;
      if (_obj == null)
          return false;
      if (_obj.getClass() != getClass())
          return false;

      if (!super.equals(_obj)) {

                // for testing and tracking
                //System.out.println("In ExecutableUbuntuImpl.equals(), "+
                //  "  ");

          return false;
      }

      ExecutableUbuntuImpl obj = (ExecutableUbuntuImpl) _obj;

      if (exeType_ == null || obj.getExeType() == null) {
          if (exeType_ != null | obj.getExeType() != null)
            return false;
      } // if
      else {
        if (!exeType_.equals( obj.getExeType() ))
          return false;
      } //else

      if(EnforceGID_ != obj.getEnforceGID())
          return false;
      if(EnforceUID_ != obj.getEnforceUID())
          return false;

      return true;
  } // equals

    /**
     * @return int the hashcode value for this object
     */
    public int hashCode() {
    int result = super.hashCode();

    result = PRIME*result + (EnforceGID_ ? 1:0);
    result = PRIME*result + (EnforceUID_ ? 1:0);
    result = PRIME*result + exeType_.hashCode();

    return result;
  } // hashCode()

  /* Over the extended interfaces and implemented classes */

  /**
   * @return the name last component
   */

  public String toString() {
    StringBuilder sb = new StringBuilder();
    sb.append(getName().lastComponent().toString() );
/*
    sb.append("EXECUTABLEIMPL :");
    sb.append(super.toString());
    sb.append(" ||| END OF ACTORIMPL");

    sb.append(" ||| exeType_ = ");
    sb.append(exeType_);
    sb.append("; EnforceGID_ = ");
    sb.append(EnforceGID_);
    sb.append("; EnforceUID_ = ");
    sb.append(EnforceUID_);

    return " [[[" + sb + " ]]]";
*/
    return " " + sb;
  } // toString()

  /**
   * Gets the specific events to listen from an ACSObject, because these events 
   * should change the view rights of an ACSObject in an ACS Addon. This method is
   * not ridden by the beamer. The events are
   * 'OtherRights', 'EnforceUID', 'EnforceGID' without conditions.
   * Then, these events are listened to update the views containing these ACSObjects.
   * This method does not cover the parent properties of a Resource or a VirtualFolder
   * (see ARoad0.gBase.AcsAddon.getAddonParentPropertiesToListen()).
   * To define the events, the map key may have the following values: <P>
   * - 'THIS' to say that the single object to listen is the ACSObject, <P>
   * - 'ACS.method_name' to say that the list of objects to listen is returned
   * by the method 'method_name' in the Addon ACS class. <P>
   * For each map key, the values have the following format: <P>
   * 'AAA.method_name IF BBB' <P>
   * where: <P>
   * - AAA: THIS (mandatory), <P>
   * - method_name: 'get' method name of the ACSObject that is associated,
   * in the relevant BeanInfo class, to the event to listen (mandatory) <P>
   * - IF BBB: where BBB is TRUE, or FALSE, or NULL, or NOT-NULL, to say that
   * the event must be listened if and only if the returned value of the method_name is
   * respectively a boolean TRUE, or FALSE, or an object NULL, or NOT-NULL (option). <P>
   * When the map value starts with 'ACS', the ACS method to call has the ACSObject has
   * one argument. Otherwise, the ACS or the ACSObject method has no argument.
   * Called by ARoad0.gBase.EPRViewInBaseImpl, NoThanViewInBaseImpl
   * and ARoad0.gWork.RightsMediatorImpl, through
   * ARoad0.gBase.UtilityImpl.changeAddonInnerPropertyChangeListener().
   * Calls the super method first.
   * @return map to define the specific ACS Addon events to listen. May be null. 
   */
  public Map<String, String> getAddonInnerPropertiesToListen() {
    Map<String, String> m_result = super.getAddonInnerPropertiesToListen();
    m_result.put(
            "<THIS>",
            "<THIS>.getEnforceUID");
    m_result.put(
            "<THIS>",
            "<THIS>.getEnforceGID");
    return m_result;
  } // getAddonInnerPropertiesToListen()

  //
  // ---------------- protected methods --------------
  //


  /**
  * This method provides a copy of the instance variables.
  * Called by copy() and by the copyOwnVariables() methods
  * of the subclasses.
  * @param _result to update with this instance variables copies
  * @param _withoutInternalAcl true if the internal aclEntries
  *   - that is with a resource having the same ACS - must not be copied
  * @return updated _result
  * @exception CreateError
  **/
  protected ExecutableUbuntuImpl copyOwnVariables(ExecutableUbuntuImpl _result,
    boolean _withoutInternalAcl) throws CreateError {

      if (_result == null) {
          throw new CreateError("in ExecutableImplUbuntu.copyOwnVariables(),"+
          " _result is null.");
      }

      _result = (ExecutableUbuntuImpl)super.copyOwnVariables(_result, _withoutInternalAcl);

      _result.exeType_ = exeType_;
      _result.EnforceGID_ = EnforceGID_;
      _result.EnforceUID_ = EnforceUID_;

      return _result;
  } // copyOwnVariables(,,,)


} // ExecutableUbuntuImpl

