//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.Ubuntu.gBase;

import ARoad0.gBase.*;
import ARoad0.gBaseInterface.*;
import ARoad0.gBase.UserIDImpl;
import ARoad0.AcsAddon.Accbee.Ubuntu.gBaseInterface.*;

import ARoad0.ErrB.CreateError;
import ARoad0.ErrB.UpDateError;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import java.util.HashSet;
import java.util.Set;
import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;

import java.io.ObjectStreamField;


/**
* This class is responsible for modeling an account in an ACS Ubuntu,
* as a kind of eligible party.
* As a Linux/Unix account, it has an umask value.
* The order is a 16-bit user identifier (UID).
* It is a javabean with the following bound properties:,
* 'UMask'.
* The listeners are proper to this instance, outside gBase,
* transient (not serialized in this instance backup).
* They are called in any order. They receive only a copy of the new value,
* to protect the property. All the exceptions from the listeners are catched,
* and a dialog box is displayed to inform the user.<P>
* All variables are private to secure the white box inheritances from this class.
* Most of the variables are protected. This class has no subclass in this package.
* @version 0.7.0
*/
public class UserIDUbuntuImpl extends UserIDImpl implements UserIDUbuntu {
  // protected variables
  private static final long serialVersionUID__ = 0;

  protected String          uMask_ = "--- --- ---";

  /**
   * A reusable empty instance for initialization,
   * to avoid the use of 'new' for default values.
   * Caution: never change the state of the returned instance.
   */
  public static final UserIDUbuntuImpl EMPTY_INSTANCE = new UserIDUbuntuImpl();

  /** variable for the JDK 2 serialization
  * @serial ("uMask_",Integer.class)
  */
  public final static ObjectStreamField[] serialPersistentFields =
  {
      new ObjectStreamField("uMask_",Integer.TYPE)
  };

 /**
  * simple constructor.
  */
  public UserIDUbuntuImpl() {
  }

   /**
   * Constructor used in the beamer by the editor for the ACS EP list, from the 'New' button.
   * @param _name ImmutableName of the userID as a kind of eligible party
   * @param _acs ACSUbuntuImpl owning this userID
   * @param _epType is the EligibleParty type, and may be null if the ACS allows it.
   * @param _isAdmin true only if the instance has an administrative role in the ACS,
   *    for the technical administration or the users administration.
   * @exception CreateError if ACSUbuntuImpl.newEorEligibleParty() throws it - the instance
   * must then be deleted !
   */
  public UserIDUbuntuImpl(ImmutableName _name, ACS _acs, String _epType, Boolean _isAdmin)
          throws CreateError  {
      super(_name, _acs, _epType, _isAdmin);

          // for testing purpose
          //System.out.println("call to UserIDUbuntuImpl constructor with 2 arguments, for "+_name);
  }

   /**
   * Used for the creation of an userID.
   * @param _name ImmutableName of the userID to create
   * @param _acs ACSUbuntuImpl owning this userID
   * @param _epType is the EligibleParty type, and may be null if the ACS allows it.
   * @param _isAdmin true only if the instance has an administrative role in the ACS,
   *    for the technical administration or the users administration.
   * @param _toRegisterInManager false if this method must not register
   *  the instance in its ACS.
   * @exception CreateError if ACSUbuntuImpl.newEorEligibleParty() throws it -
   *  the instance must then be deleted !
   */
  public UserIDUbuntuImpl(ImmutableName _name, ACS _acs, String _epType, boolean _isAdmin,
    boolean _toRegisterInManager) throws CreateError {

      super(_name, _acs, _epType, _toRegisterInManager);

      setComment("From the editor "+
              ((String[])_acs.getM_AcsVocabulary().get(ACSFactoryUtilityImpl.ACSADDON_EDITOR))[0]+
              ", in the AcsAddon "+
              ((String[])_acs.getM_AcsVocabulary().get(ACSFactoryUtilityImpl.ACSADDON_NAME))[0]+
              " - ");
      
          // for testing purpose
          //System.out.println("call to UserIDUbuntuImpl constructor with 3 arguments, for "+_name);

  }

  /**
  * This method provides a copy of the instance with another name.
  * Do not copy the own resources or the current actors.
  * The copy order is 3 times the source order.
  * _aliasRef may belong to another ACS than this instance ACS.
  * Caution: do not copy the user and the group if _result
  * does not belong to aCS_ and if there are no user or group
  * in the _result ACS which have similar names - that is that
  * are ended by the same Strings in this instance user or group names.
  * Call copyOwnVariables().
  * This method is overridden by the subclasses.
  * @param _name copy name. Usually creates as an extension of _parent name.
  * @param _acs main ACS of the copy to create. May not be the ACS of this instance.
  * @param _parent is not used, wathever the value.
  * @param _toRegisterInManager false if this method must not register
  *  the instance in the ACS.
  * @return a new instance of UserIDUbuntuImpl
  * @exception CreateError of the constructor with the same parameters,
  *   or if _parent ACS is not _acs, or addAclEntry() exceptions.
  **/
  public Object copy(ImmutableName _name, ACS _acs, Directory _parent,
    Boolean _toRegisterInManager)
    throws CreateError {

      if (_acs == null || _name == null  ) {
          throw new CreateError("in UserIDUbuntuImpl.copy(),"+
          " _name or _acs is null.");
      }

      UserIDUbuntuImpl result = new UserIDUbuntuImpl(
          _name, _acs, getEPType(),
          _toRegisterInManager.booleanValue());

      result = copyOwnVariables(result, false);

      return result;
  } // copy(,,,)


  /**
  * This method provides a copy of the instance
  * as copy() but without copying the internal aclEntries.
  * Does not copy the own resources or the current actors.
  * The copy order is 3 times the source order.
  * Caution: does not copy the user and the group if _result
  * does not belong to aCS_ and if there are no user or group
  * in the _result ACS which have similar names - that is that
  * are ended by the same Strings in this instance user or group names.
  * Calls copyOwnVariables().
  * @param _name copy name. Usually creates as an extension of _parent name.
  * @param _acs main ACS of the copy to create. May not be the ACS of this instance.
  * @param _parent is not used, wathever the value.
  * @param _toRegisterInManager false if this method must not register
  *  the instance in the ACS.
  * @return a new instance of UserIDUbuntuImpl
  * @exception CreateError of the constructor with the same parameters,
  *   or if _parent ACS is not _acs, or addAclEntry() exceptions.
  **/
  public Object copyWithoutInternalAcl(ImmutableName _name, ACS _acs, Directory _parent,
    Boolean _toRegisterInManager) throws CreateError {

      if (_acs == null || _name == null ) {
          throw new CreateError("in UserIDUbuntuImpl.copy(),"+
          " _name or _acs is null.");
      }

      UserIDUbuntuImpl result = new UserIDUbuntuImpl(
          _name, _acs, getEPType(),
          _toRegisterInManager.booleanValue());

      // copies the external aclEntries if they exist
      result = copyOwnVariables(result, true); // no internal aclEntry copy

      //if (result.getEorL_AclConnectedResources().isEmpty()) return result;
      //else return deleteAclEntries(result);
      return result;
  } // copy(,,,)

  /**
   * The allowed format is 3 blocks for respectively Account, Group and Other rights.
   * _umask is the target value extended by SimplePropertyEditor with " ".
   * No control of the management by the ACS of accounts, groups and other.
   * Noop and error mesage to the user if the ep type starts with 'byAcsCreator' or 'immutable'
   * after the creation.
   * @param _umask from '--- --- --- ' to 'rwx rwx rwx ', following Linux/Unix
   *           concept (from 0 to 777), or 3 used acsrights/metarights separated by one space,
   *           or 3 blocks separated by one space and inferior to 4,
   *           containing at most one of the characters 'wxr'.
   * @exception UpDateError for all the wrong formats or if the umask is immutable
   */
  public synchronized void setUMask(String _umask) throws UpDateError {

                // for testing purpose
                //System.out.println("call to UserIDUbuntuImpl.setUMask(), "+
                  //"oldValue: ||"+uMask_+"|| _umask: ||"+_umask+"||");

    if (type_.startsWith("<byAcsCreator>") || type_.startsWith("<immutable> ") ||
            type_.endsWith("<immutable> "))
          throw new UpDateError(
            "This object has been created immutable, or the umask value"+
            " is immutable for "+getName().lastComponent());

    if (_umask == null || _umask.trim().length() < 5)
        throw new UpDateError("The simplest umask format, if not for Linux/Unix, is '- - -'");

      // variables
    String  oldValue = uMask_;
    boolean error = false;
    boolean error2 = false;
    boolean nx = false;
    String  partialMask = null;
    String  blockMask = null;
    Set<StringRight> l_rights = new HashSet(INITIAL_CAPACITY);

    if (aCS_.getACSType().indexOf(ACSImpl.LINUX) != -1 ) {
        error = !isNxUMask(_umask);
        nx = true;
    }
    if (error)
        throw new UpDateError("The umask format for Linux/Unix is '___ ___ ___'"+
                ", where the allowed ranges are from '---' to 'rwx' "+
                "for  Account, Group, and Other respectively, in the 3 '___'.");

    if (!nx) {
        blockMask = _umask.trim().substring(0, _umask.trim().indexOf(" ") );
        // owner rights control
        if (!isMaskWithWXR(blockMask) && processNoWXRuMask(blockMask, l_rights).isEmpty()) {
            error2 = true;

                        // for testing and tracking
                        System.out.println("From UserIDUbuntuImpl.setUMask(), "+
                                "error2 on 'owner' with "+blockMask);

        }
        else {
            l_rights.clear();
            // partialMask is the group plus the other blocks
            partialMask = _umask.trim().substring(_umask.trim().indexOf(" ") + 1);
            if (partialMask.indexOf(" ") != -1) // else error = true hereafter
                blockMask = partialMask.substring(0, partialMask.indexOf(" ") );
            // test that the _umask first space is not more than ' ', and that the second space exists
            if (partialMask.indexOf(" ") == 0 || partialMask.indexOf(" ") == -1)
                error = true;
            // group rights control
            else if (!isMaskWithWXR(blockMask) && processNoWXRuMask(blockMask, l_rights).isEmpty()) {
                error2 = true;

                        // for testing and tracking
                        System.out.println("From UserIDUbuntuImpl.setUMask(), "+
                                "error2 on 'group' with "+blockMask);

            } else {
                l_rights.clear();
                partialMask = partialMask.substring(partialMask.indexOf(" ") + 1);
                blockMask = partialMask;
                // test that the second space is not more than ' ', and that there is no third space
                if (partialMask.indexOf(" ") != -1) error = true;
                // other rights control
                else if (!isMaskWithWXR(blockMask) && processNoWXRuMask(blockMask, l_rights).isEmpty()) {
                    error2 = true;

                        // for testing and tracking
                        System.out.println("From UserIDUbuntuImpl.setUMask(), "+
                                "error2 on 'other' with "+blockMask);

                }
            } // else
        } // else
    }
    if (error)
        throw new UpDateError("The umask format is 'right(s) right(s) right(s)'" +
                " where there is exactly one space between 2 right blocks, "+
                "and 'right(s)' may be a used right in the ACS, or a combinaison of the 4 'wxr-', or "+
                "any no-meaning sequence like '---' for instance.");
    if (error2)
        throw new UpDateError("The umask format is 'right(s) right(s) right(s)'+" +
                " and one of the 'right(s)' is not a used right, nor a combinaison of 'wxr' rights.");

    uMask_ = _umask.substring(0, _umask.length() - 1); // without the last space at the end

    try {
        firePropertyChange("UMask", oldValue, _umask);
    } catch (Exception e) {
        ARoad0.Gui1.DialogBox.showAlarmMessage(
          "There is an exception after the data update, from outside the database."+
          "You should save the base and stop the program, then run it once again.");
        System.out.println("From UserIDUbuntuImpl.setUMask(), firePropertChange() throws:");
        e.printStackTrace();
    }
    partialMask = null;
    blockMask = null;
    l_rights = null;
  } // setUMask()

  /**
   * This umask defines some rights to subtract to the 'full control' right,
   * at the creation of a Resource that belongs to this UserID.
   * The default value is '--- --- ---' for respectively Account, Group and Other rights.
   * @return the allowed format of 3 blocks of 3 letters among the values '-', 'r', 'w', 'x'.
   */
  public String getUMask() {
    if (uMask_ != null)
        return uMask_;
    else
        return "";
  }

  //
  // -------- Overridden public methods ------------------
  //

  /**
  * Adds a listener to the bean.
  * Used by Gui1.CommonPropertyEditor for the properties: 'UMask'.
  * Overrides the super method and calls it.
  * @param _propertyName name of the property
  * @param _l PropertyChangeListener to add. Caution: the listener must NOT
  *   have specific hashCode() and equals() methods, to work with HashSet
  *   in the removing operations.
  */
  public void addPropertyChangeListener(String _propertyName,
    PropertyChangeListener _l) {

      if (changeSupport_ == null)
          changeSupport_ = new PropertyChangeSupport(this);

      if (_propertyName.equals("UMask") )
          changeSupport_.addPropertyChangeListener(_propertyName, _l);
      else super.addPropertyChangeListener(_propertyName, _l);

  } // addPropertyChangeListener(String _propertyName, _l)

  /**
  * Removes a listener to the bean.
  * Used by Gui1.CommonPropertyEditor for the properties: 'UMask'.
  * Note: using this method does not remove a listener which has been
  * registered through addPropertyChangeListener(PropertyChangeListener _l).
  * Overrides the super method and calls it.
  * @param _propertyName name of the property
  * @param _l PropertyChangeListener to remove
  */
  public void removePropertyChangeListener(String _propertyName,
    PropertyChangeListener _l){

      // no action if the transiant variable is null
      if (changeSupport_ == null) return;

                // for testing and tracking
                //if (_propertyName.equals("AclRights"))
                  //  System.out.println("GrouIDMemberImpl.removePropertyChangeListener(), call from "+
                    //_l.getClass()+" to "+name_+" for the property "+_propertyName);

      if (_propertyName.equals("UMask") )
          changeSupport_.removePropertyChangeListener(_propertyName, _l);
      else super.removePropertyChangeListener(_propertyName, _l);

                //if (_propertyName.equals("AclRights")) {
                  //  System.out.println("AclRights listeners after the removing of "+_l+": ");
                    //PropertyChangeListener[] l_list = changeSupport_.getPropertyChangeListeners("AclRights");
                    //for (int i = 0; i < l_list.length; i++) {
                      //  System.out.println("-------"+l_list[i]);
                    //}
                    //l_list = null;
                //} // if

  } // removePropertyChangeListener()

  /**
   * @return the static serialVersionUID__ of this UserIDUbuntu class
   */
  public long getSerialVersionUID()	{
          return serialVersionUID__;
  }

  //
  // ------- protected methods ---------------------
  //

  /**
   * Gets the denying rights for a resource and its owner/group/other AGO rights.
   * Searching first in the acsrights, and if no matched right is found, uses the
   * correct metaright ('deny_read' for 'r', for instance).
   * Calls processNoWXRuMask() if the _mask lenght is superior to 3, or if it does not
   * contain the characters 'w', 'x' and 'r'. Returns an empty set if _mask
   * is '-', '--' or '---'.
   * @param _mask part of this userID umask which describes the requirement for owner
   *     or for group or other
   * @param _isDirectory true if the calling resource is a directory, not a Linux/Unix one.
   * Unused in this method, but it is kept for future uses.
   * @param _isNxDirectory true if the calling resource is a Linux/Unix directory.
   *          _isDirectory has to be false.
   * @return the denying rights set by the umask for A, G and O rights.
   *     May be empty but not null.
   */
  public Set<StringRight> processUMask(String _mask, boolean _isDirectory,
          boolean _isNxDirectory) {

    if (_isDirectory)// && _isNxDirectory)
        throw new InternalError("UserIDUbuntuImpl.processUMask(), "+
            "the _isDirectory argument is true.");

    Set<StringRight> l_result = new HashSet(INITIAL_CAPACITY);

    if (_mask.equals("-") || _mask.equals("--") || _mask.equals("---"))
        return l_result;

    if (!isMaskWithWXR(_mask)) return processNoWXRuMask(_mask, l_result);

    // variables
    StringRight W = null;
    StringRight X = null;
    StringRight R = null;
    //StringRight W2 = null;
    //StringRight X2 = null;
    //StringRight R2 = null;
    StringRight[] l_rights = null;

    l_rights = aCS_.getL_Rights();

    // w, x and r are indepêndant in the metarights network
    // R, W, X initialization
    for (int i = 0; i < l_rights.length; i++) {
        if (l_rights[i].getEorMetaRight().getRight().equals("deny_write") ) {
            if (!_isDirectory && !_isNxDirectory) W = l_rights[i];
            continue;
        }
        else if (l_rights[i].getEorMetaRight().getRight().equals(
                "deny_write_for_directory") ) {
            if (_isDirectory ) W = l_rights[i];
            continue;
        }
        else if (l_rights[i].getEorMetaRight().getRight().equals(
                "deny_write_for_nxdirectory") ) {
            if (_isNxDirectory) W = l_rights[i];
            continue;
        }
        else if (l_rights[i].getEorMetaRight().getRight().equals(
                "deny_execute") ) {
            if (!_isDirectory && !_isNxDirectory) X = l_rights[i];
            continue;
        }
        else if (l_rights[i].getEorMetaRight().getRight().equals(
                "deny_execute_for_directory") ) {
            if (_isNxDirectory ) X = l_rights[i];
            continue;
        }
        //else if (l_rights[i].getEorMetaRight().getRight().equals(
          //      "deny_execute_for_nxdirectory") ) {
            //if (_isNxDirectory) X = l_rights[i];
            //continue;
        //}
        else if (l_rights[i].getEorMetaRight().getRight().equals(
                "deny_read") ) {
            if (!_isDirectory && !_isNxDirectory) R = l_rights[i];
            continue;
        }
        else if (l_rights[i].getEorMetaRight().getRight().equals(
                "deny_read_for_directory") ) {
            if (_isDirectory || _isNxDirectory) R = l_rights[i];
            continue;
        }
    } // for i

    if (_mask.indexOf("w") != -1) {
        if (W == null ) { // not find a good ACS right
            if (!_isDirectory && !_isNxDirectory) l_result.add(
                    BaseManagerImpl.getInstance().getMetaRight("deny_write") );
            else if (_isDirectory ) l_result.add(
                    BaseManagerImpl.getInstance().getMetaRight("deny_write_for_directory") );
            else l_result.add(
                    BaseManagerImpl.getInstance().getMetaRight("deny_write_for_nxdirectory") );
        } else {
            l_result.add(W);
        }
    } // if
    if (_mask.indexOf("x") != -1) {
        if (X == null ) { // not find a good ACS right
            if (!_isDirectory && !_isNxDirectory) l_result.add(
                    BaseManagerImpl.getInstance().getMetaRight("deny_execute") );
            else if (_isDirectory ) l_result.add(
                    BaseManagerImpl.getInstance().getMetaRight("deny_execute_for_directory") );
        } else {
            l_result.add(X);
        }
    } // if
    if (_mask.indexOf("r") != -1) {
        if (R == null ) { // not find a good ACS right
            if (!_isDirectory && !_isNxDirectory) l_result.add(
                    BaseManagerImpl.getInstance().getMetaRight("deny_read") );
            else l_result.add(
                    BaseManagerImpl.getInstance().getMetaRight("deny_read_for_directory") );
        } else {
            l_result.add(R);
        }
    } // if

                // for testing and tracking
                //if ( _isNxDirectory) {
                    //System.out.println("UserIDUbuntuImpl.processUMask(), for umask "+_mask+
                      //      ", rights in result:");
                    //for (Object obj : l_result ) {
                        //for testing and tracking
                      //  System.out.println("----- "+obj);
                    //}
                //} // if

    W = null;
    X = null;
    R = null;
    l_rights = null;
    return l_result;
  } // processUMask()

    /**
    * This method sets the properties to null and unregisters the instance, but there is no
    * guaranee that the JVM calls this method before the end of the application.
    * @exception java.lang.Throwable
    * @exception  Throwable
    */
  protected void finalize() throws Throwable {
    try {
      finalizeForBase();
    } catch (UpDateError u) {
      super.finalize();
      throw new InternalError("in UserIDUbuntuImpl.finalize(): "+u.toString());
    }

    super.finalize();
  }

   /**
   * This method HAS TO BE USED before deleting the userID.
   * Note: when this method is called by the EligiblePartyImpl constructor, the proper
   * variables of this class are not still initialized.
   * @exception UpDateError if the userID is unknown of the main ACS
   * (if it is a clone, typically), or if a linked immutable object fires an exception
   */
   public synchronized void finalizeForBase() throws UpDateError {

              // for testing purpose
              //System.out.println("call 1 to UserIDUbuntuImpl.finalizeForBase() for "+getName());

    // noop if it is the second call to this method
    if (folderMember_ == null) {
      return;
    }

              // for testing purpose
              //System.out.println("call 2 to UserIDUbuntuImpl.finalizeForBase() for "+getName());

    uMask_ = null;
    super.finalizeForBase();
  } // finalizeForBase()

  /**
  * Does not call finalizeForBase(). Called only by EPRViewInBaseImpl.
   */
  protected synchronized void finalizeFromDeserialization() {

                //System.out.println("In UserIDUbuntuImpl, call of " + name_ +
                    //".finalizeFromDeserialization()");


    // noop if it is the second call to this method
    if (uMask_ == null) {
      return;
    }

    super.finalizeFromDeserialization();
  } // finalizeFromDeserialization()

  /**
  * Clone without register the new object in the ACSs.
  * Caution: don't use any modifying method that may altere the integrity
  * of the base.
  * @return Object is not registred in the ACSs. The umask is cloned.
  */
  public Object clone() {
      UserIDUbuntuImpl clon = (UserIDUbuntuImpl) super.clone();
      clon.uMask_ = uMask_; // clone() by JVM since it is String

      return clon;
  }

  /**
  * @param _obj Object to compare
  * @return true if _obj is an instance of UserIDUbuntuImpl (or a derived class)
  * equals to this instance (with the same class)
  */
  public boolean equals(Object _obj) {
      if (this == _obj)
          return true;
      if (_obj == null)
          return false;
      if (_obj.getClass() != getClass())
          return false;

      if (!super.equals(_obj))
          return false;

      UserIDUbuntu obj = (UserIDUbuntu) _obj;

      if (uMask_ != obj.getUMask() ) return false;

      return true;
  } // equals()

   //
   // -------------- Out of the extended interfaces -------------------------
   //


  /**
  * @return String: the variables of the instance : name (last component).
  *  May be null.
   */
  public String toString() {
    return getName().lastComponent();
  } // toString()

  //
  // ----------- PROTECTED METHODS -------------------------------------
  //

  /**
  * This method provides a copy of the instance variables.
  * Called by copy() and by the copyOwnVariables() methods
  * of the subclasses.
  * @param _result to update with this instance variables copies
  * @param _withoutInternalAcl true if the internal aclEntries
  *   - that is with a resource having the same ACS - must not be copied
  * @return updated _result
  * @exception CreateError
  **/
  protected UserIDUbuntuImpl copyOwnVariables(UserIDUbuntuImpl _result,
    boolean _withoutInternalAcl) throws CreateError {

      if (_result == null) {
          throw new CreateError("in UserIDUbuntuImpl.copyOwnVariables(),"+
          " _result is null.");
      }

      _result = (UserIDUbuntuImpl)super.copyOwnVariables(_result, _withoutInternalAcl);

      _result.uMask_ = uMask_;

      return _result;
  } // copyOwnVariables(,,,)


  /**
   * NO USE.
   * Gets one denying right for a resource and its owner/group/other AGO rights,
   * when the Linux/Unix 'wxr' format is not used. The kernel right of the
   * denying right has to be 'deny_' extended by _mask.
   * Called by processUMask() and setUMask() when isMaskWithWXR() return false.
   * @param _mask part of this userID umask which describes the requirement for owner,
   *     or for group, or for other
   * @param _l_result set to fill with the denying rights
   * @return the denying right set by the umask for A or G or O AGO rights.
   *     May be empty but not null.
   */
  protected Set<StringRight> processNoWXRuMask(String _mask, Set<StringRight> _l_result) {

    // variables
    StringRight right = aCS_.getOneRight("deny_"+_mask);
    if (right != null) _l_result.add(right);

    right = null;
    return _l_result;
  } // processNoWXRuMask()

  /**
   * Check if _umask is '--- --- --- ' to 'rwx rwx rwx ', following Linux/Unix
   * format (from 0 to 777).
   * Called by setUMask() when the ACS type is LINUX. For another ACS type, 'wxr-'
   * may be used in any order in umask, with the same use, and the umask is not tested there.
   * @param _umask
   * @return true if the umask complies to the Linux/Unix format
   */
  protected boolean isNxUMask(String _umask) {

    // variables
    boolean error = false;
    List l_charR = new ArrayList(INITIAL_CAPACITY);
    l_charR.add("-");
    l_charR.add("r");
    List l_charW = new ArrayList(INITIAL_CAPACITY);
    l_charW.add("-");
    l_charW.add("w");
    List l_charX = new ArrayList(INITIAL_CAPACITY);
    l_charX.add("-");
    l_charX.add("x");

    if (_umask.length() != 12) {
        error = true;
                // for testing purpose
                //System.out.println("call to UserIDUbuntuImpl.isNxUMask(), "+
                  //"error on the length");
    }
    else if (!_umask.substring(3, 4).equals(" ") ||
            !_umask.substring(7, 8).equals(" ") ) {
                    error = true;
                // for testing purpose
                //System.out.println("call to UserIDUbuntuImpl.isNxUMask(), "+
                  //"error on the 2 spaces between the 3 blocks");
    }
    else if (!l_charR.contains(_umask.substring(0, 1)) || !l_charW.contains(_umask.substring(1, 2)) ||
            !l_charX.contains(_umask.substring(2, 3)) ) {
        error = true;
                // for testing purpose
                //System.out.println("call to UserIDUbuntuImpl.isNxUMask(), "+
                  //"error on the owner block");
    }
    else if (!l_charR.contains(_umask.substring(4, 5)) || !l_charW.contains(_umask.substring(5, 6)) ||
            !l_charX.contains(_umask.substring(6, 7)) ) error = true;
    else if (!l_charR.contains(_umask.substring(8, 9)) || !l_charW.contains(_umask.substring(9, 10)) ||
            !l_charX.contains(_umask.substring(10, 11)) ) error = true;

    l_charR = null;
    l_charX = null;
    l_charW = null;

    return !error;
  } // isNxUMask()

  /**
   * Called by processUMask().
   * @param _mask is a block in an umask, used for defined the denying requirement
   *           for an owner, or group, or other rights in AGO rights
   * @return true if the mask contains only the characters '-', 'w', 'x' or 'r' in a size
   *           inferior to 4, in any order; several instances of a character is possible
   */
  protected boolean isMaskWithWXR(String _mask) {
    boolean result = true;

    if (_mask == null || _mask.length() == 0 || _mask.length() > 3) {
        return false;
    }
    List<String> l_char = Arrays.asList("w", "x", "r", "-");

    for (int i = 0; i < _mask.length(); i++) {
        if (!l_char.contains( _mask.substring(i, i+1) )) {
                result = false;
                break;
        }
    } // for i

    return result;
  } // isMaskWithWXR()


} // UserIDUbuntuImpl


