//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.Ubuntu.gBaseBeanInfo;

import ARoad0.gBaseBeanInfo.*;
import ARoad0.AcsAddon.Accbee.Ubuntu.gBase.*;
import ARoad0.AcsAddon.Accbee.Ubuntu.gBaseInterface.*;
import ARoad0.ErrW.ProcessError;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

/**
 * This class is responsible for describing the ACS class for Linux Ubuntu.
 * Adds the property descriptor for 'OneActorFromOtherInCurrentContext'.
 * @version 0.7.0
 */
public class ACSUbuntuImplBeanInfo extends ACSImplBeanInfo {

  public ACSUbuntuImplBeanInfo() {}

  /**
   * Gets the description of the bean.
   * Some standard values are set:
   * - 'hidden' is false since the bean is not hidden to the users
   * - 'expert' is false since the bean is not reserved to experts
   * <P>
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of ACSUbuntuImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(ACSUbuntuImpl.class );
    result.setDisplayName("ACS");

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of ACSUbuntuImpl
    result.setValue("withTabbedPanes", Boolean.TRUE );

    // these strings signal the nodes to use in an ACS tree
    // to display this ACS, derived from the ACSTree.M_ACS_TREE_UPPER_NODES
    // in the Gui2 package.
    result.setValue("nodesInACSTree", new String[]
      {"RESOURCES", "USERS", "GROUPS", "ACLENTRIES",
       "EEPARTIES", "ECRESOURCES", "ACSRIGHTS"});

    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    result.setHidden(false);
    result.setExpert(false);

    return result;
  } // getBeanDescriptor()


  /**
   * Adds the property descriptor for 'OneActorFromOtherInCurrentContext'.
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;

    try {
      // ------------------ property "OneActorFromOtherInCurrentContext" ---------------
      PropertyDescriptor pd_fourtyFour = new PropertyDescriptor(
        "OneActorFromOtherInCurrentContext", ACSUbuntuImpl.class,
        "getEorL_FromOtherInCurrentContextActors", null);
      pd_fourtyFour.setDisplayName("NX Other-Executed");
      pd_fourtyFour.setShortDescription("Programs with AGO 'execute' right for 'other' category,"+
              " and with a current account and/or group.");
      pd_fourtyFour.setBound(true);
      //pd_ten.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_fourtyFour.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_fourtyFour.setValue("valueTypeInGroup", new Object[] {ActorUbuntu.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyFour.setValue("tabbedPaneName", "  Proxies  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtyFour.setValue("panelGroupName", "  Right-proxy programs with AGO 'execute' "+
              "right for 'other' and some current AG  ");
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer.
      pd_fourtyFour.setValue("withViewButton", Boolean.TRUE );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_fourtyFour.setValue("limitedSizeForPresentation", new Integer(16));
      // 'horizontalTextPositionToLeft' is the boolean which is set if
      // the collectionPropertyEditor has to display the text from the left
      // in the beamer. The Right position is the default value.
      //pd_fourtyFour.setValue("horizontalTextPositionToLeft", Boolean.TRUE );

      //
      // ------------End of ACSUbuntuImpl properties definition --------------------
      //

      // start from the parent class bean info
      l_pdFromParent = super.getPropertyDescriptors();
      l_result = new PropertyDescriptor[l_pdFromParent.length + 1];
      System.arraycopy(l_pdFromParent, 0, l_result, 0, l_pdFromParent.length);
      l_result[l_result.length - 1] = pd_fourtyFour;

      l_pdFromParent = null;
      pd_fourtyFour = null;
      return l_result;
    } catch (IntrospectionException e) {
      new ProcessError("In ACSImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In ACSImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()
} // ACSUbuntuImplBeanInfo