//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.Ubuntu.gBaseBeanInfo;

import ARoad0.gBaseBeanInfo.*;
import ARoad0.AcsAddon.Accbee.Ubuntu.gBase.*;
import ARoad0.AcsAddon.Accbee.Ubuntu.gBaseInterface.*;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the DirectoryUbuntuImpl class.
 * It is used for displaying and editing the properties of DirectoryUbuntuImpl,
 * to create it and to copy it.
 */
public class DirectoryUbuntuImplBeanInfo extends DirectoryImplBeanInfo {

  public DirectoryUbuntuImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor. There is no method to change
   * the class of a BeanDescriptor, so, to reuse the BeanInfo of the mother class,
   * the values are copied in a new BeanDescriptor.
   * They are get with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of ResourceUbuntu
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(DirectoryUbuntu.class );
    result.setDisplayName("Directory");

    result = getCommonBeanDescriptor(result);

    return result;
  } // getBeanDescriptor()

  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue().
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;

    try {
      // ------------------- property "OtherRights" --------------------
      PropertyDescriptor pd_nine = new PropertyDescriptor(
        "OtherRights", DirectoryUbuntuImpl.class, "getL_otherRights", "setL_otherRights");
      pd_nine.setDisplayName("'other' as NX rights");
      pd_nine.setShortDescription("The rights on this object of 'other' accounts and groups,"+
        " in Linux/Unix systems");
      pd_nine.setBound(true);
        //pd_nine.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and the default value is SimplePropertyEditor.
      pd_nine.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_nine.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_nine.setValue("tabbedPaneName", "  AG  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      // caution: "panelGroupName" is superior to "tabbedPaneName" in the BeamerManager analysis.
      pd_nine.setValue("panelGroupName", "  Basic AG rights  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when is is not applicable.
        //pd_nine.setValue("BaseBeanManager", );
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the String values of the previous property for this current property,
      // following the order of the property descriptors returned by getPropertyDescriptors().
      pd_nine.setValue("withPreviousButton", Boolean.TRUE );
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_nine.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_nine.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_nine.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_nine.setValue("withRemoveButton", Boolean.TRUE );
      // 'limitedSizeForPresentation' is the maximal size of the visible list.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_nine.setValue("limitedSizeForPresentation", new Integer(8));
      // 'isRestrictedArgument' is true if it is a constrained property that is not
      // in the constructor (else, see 'constrRestrictedArgument' in the BeanDescriptor).
      // The restricted values are get from 'staticSetOfItems', for Strings only,
      // from 'choiceMethodForNew', or through a search in the ACS by the PropertyEditor
      // like here.
      pd_nine.setValue("isRestrictedArgument", Boolean.TRUE );

      // ------------------ property "FromTheParentOther" ---------------
      PropertyDescriptor pd_twentyone = new PropertyDescriptor(
        "FromTheParentOther", DirectoryUbuntuImpl.class, "getAGOAccessFromTheParentOther", null);
      pd_twentyone.setDisplayName("Parent 'other' access");
      pd_twentyone.setShortDescription("true if this target may be accessed "+
              "through its parent 'other' AG rights.");
      pd_twentyone.setBound(true);
      //pd_twentyone.setConstrained(false);
      //pd_twentyone.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twentyone.setValue("tabbedPaneName", "  AG Inheritance  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_twentyone.setValue("panelGroupName", "  AG access through the parent  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_twentyone.setValue("BaseBeanManager", null);

      // ------------------- property "InheritedOtherRights" --------------------
      PropertyDescriptor pd_twentyfour = new PropertyDescriptor(
        "InheritedOtherRights", DirectoryUbuntuImpl.class, "getL_inheritedOtherRights", null);
      pd_twentyfour.setDisplayName("Rights of parent O");
      pd_twentyfour.setShortDescription("The inherited 'Other' rights of the parent on"+
        " this child");
      pd_twentyfour.setBound(true);
        //pd_twentyfour.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and the default value is SimplePropertyEditor.
      pd_twentyfour.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_twentyfour.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twentyfour.setValue("tabbedPaneName", "  AG Inheritance  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      // caution: "panelGroupName" is superior to "tabbedPaneName" in the BeamerManager analysis.
      pd_twentyfour.setValue("panelGroupName", "  AG from the parent A&G point of view -"+
              " Caution if 'has parent A and G' is false  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_twentyfour.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_twentyfour.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_twentyfour.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_twentyfour.setValue("withRemoveButton", Boolean.FALSE );
      // 'limitedSizeForPresentation' is the maximal size of the visible list.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_twentyfour.setValue("limitedSizeForPresentation", new Integer(5));

      // ------------------- property "StickyBit" --------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "StickyBit", DirectoryUbuntuImpl.class, "getStickyBit", "setStickyBit");
      pd_three.setDisplayName("Deleting-renaming by");
      pd_three.setShortDescription("If true, following *NX stickybit, children"+
        " may be deleted or renamed only by their owner.");
      pd_three.setBound(true);
      //pd_three.setConstrained(false);
      // 'tags' is the name (String value) of the couple of strings,
      // with a size limited to 12 caracters,
      // to be displayed when the property is true or false, in this order.
      // Cannot be empty if it exists, and the default value for the property
      // editor is a checkbox when 'tags' does not exist, like in "EnforceChildGID".
      pd_three.setValue("tags", new String[] {"owner", "all"});
      //pd_three.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  As Directory  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_three.setValue("panelGroupName", "  Child management  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_three.setValue("BaseBeanManager", null);

      //
      // ------------End of DirectoryUbuntuImpl properties definition --------------------
      //

      // start from the parent class bean info
      l_pdFromParent = super.getPropertyDescriptors();
      l_result = new PropertyDescriptor[l_pdFromParent.length + 4];
      System.arraycopy(l_pdFromParent, 0, l_result, 0, l_pdFromParent.length);
      l_result[l_result.length - 4] = pd_nine;
      l_result[l_result.length - 3] = pd_twentyone;
      l_result[l_result.length - 2] = pd_twentyfour;
      l_result[l_result.length - 1] = pd_three;

      l_pdFromParent = null;
      return l_result;
    } catch (IntrospectionException e) {
      new ProcessError("In DirectoryImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In DirectoryImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

} // DirectoryUbuntuImplBeanInfo