//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.Ubuntu.gBaseBeanInfo;

import ARoad0.AcsAddon.Accbee.Ubuntu.gBase.*;
import ARoad0.AcsAddon.Accbee.Ubuntu.gBaseInterface.*;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the Linux Ubuntu executable.
 * It is used for displaying and editing the properties of ExecutableUbuntuImpl,
 * to create it and to copy it.
 * This is the reference class for the BeanInfo descriptions.
 * @version 0.7.0
 */
public class ExecutableUbuntuImplBeanInfo extends ActorUbuntuImplBeanInfo {

  public ExecutableUbuntuImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of ResourceImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
      // used by BaseUtilityImpl.getSuperInterfacesFromBeanInfo()
    BeanDescriptor result = new BeanDescriptor(ExecutableUbuntu.class );
    result.setDisplayName("Executable");
    result = getCommonBeanDescriptor(result);

    return result;
  } // getBeanDescriptor()

  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue().
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;

    try {
      // ------------------- property "EnforceUID" --------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "EnforceUID", ExecutableUbuntuImpl.class, "getEnforceUID", "setEnforceUID");
      pd_one.setDisplayName("setUID value");
      pd_one.setShortDescription("true to set the current account, as rights user, always "+
        "to its own Resource account for the executed process.");
      pd_one.setBound(true);
      //pd_one.setConstrained(false);
      // 'tags' is the name (String value) of the couple of strings,
      // with a size limited to 12 caracters,
      // to be displayed when the property is true or false, in this order.
      // Cannot be empty if it exists, and the default value for the property
      // editor is ("yes", "no") when 'tags' does not exist.
      pd_one.setValue("tags", new String[] {" true", " false"});
      //pd_one.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  As Executable  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_one.setValue("panelGroupName", "  User/Group changes  " );

      // ------------------- property "EnforceGID" --------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "EnforceGID", ExecutableUbuntuImpl.class, "getEnforceGID", "setEnforceGID");
      pd_two.setDisplayName("SetGID value");
      pd_two.setShortDescription("true to set the current group, as rights user, always "+
        "to its own Resource group for the executed process.");
      pd_two.setBound(true);
      //pd_two.setConstrained(false);
      // 'tags' is the name (String value) of the couple of strings,
      // with a size limited to 12 caracters,
      // to be displayed when the property is true or false, in this order.
      // Cannot be empty if it exists, and the default value for the property
      // editor is ("yes", "no") when 'tags' does not exist.
      pd_two.setValue("tags", new String[] {"true", "false"});
      //pd_two.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  As Executable  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_two.setValue("panelGroupName", "  User/Group changes  " );

      // ------------------- property "ExeType" --------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "ExeType", ExecutableUbuntuImpl.class, "getExeType", "setExeType");
      pd_three.setDisplayName("Executable type");
      pd_three.setShortDescription("This type is set by the program.");
      pd_three.setBound(true);
      //pd_three.setConstrained(false);
      //pd_three.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  As Executable  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_three.setValue("panelGroupName", "  Executable Controls  " );

      //
      // ------------End of ExecutableImpl properties definition --------------------
      //

      // start from the parent class bean info
      l_pdFromParent = super.getPropertyDescriptors();
      l_result = new PropertyDescriptor[l_pdFromParent.length + 3];
      System.arraycopy(l_pdFromParent, 0, l_result, 0, l_pdFromParent.length);
      l_result[l_result.length - 3] = pd_one;
      l_result[l_result.length - 2] = pd_two;
      l_result[l_result.length - 1] = pd_three;

      l_pdFromParent = null;
      return l_result;
    } catch (IntrospectionException e) {
      new ProcessError("In ExecutableImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In ExecutableImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

} // ExecutableImplBeanInfo