//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.Ubuntu.gBaseInterface;

import ARoad0.ErrB.UpDateError;


/**
 * This interface is responsible for modeling a Linux/Unix executable,
 * which may have setUID and setGID capabilities. To be more general,
 * the changing capability is modelized, with EnforceGID and EnforceUID.
 * As every actor, it implements EligibleParty to have rights
 * on Resources through AclEntries and through its current UserID and GroupID.
 * @version 0.7.0
 */
public interface ExecutableUbuntu extends ActorUbuntu {
    /**
     * @return type of executable.
     *  Different to type_ from EP and resourceType from Resource.
     */
    public  String getExeType ();

    /**
     * This type is not the type from EligibleParty nor from Resource.
     * @param _p type of executable.
     */
    public  void setExeType (String _p);

    /**
     * @return boolean true if the userID is always its userID whatever the
     * caller userID (this is the setUID bit on *NX systems).
     */
    public  boolean getEnforceUID ();

    /**
     * @return boolean true if the groupID is always its groupID whatever the
     * caller groupID (this is the setGID bit on *NX systems).
     */
    public  boolean getEnforceGID ();

    /**
     * @param _b boolean true if the userID may be changed.
     * @exception UpDateError if the enforceUID property is immutable.
     */
    public  void setEnforceUID (boolean _b) throws UpDateError;

    /**
     * @param _b boolean true if the groupID may be changed.
     * @exception UpDateError if the enforceGID property is immutable.
     */
    public void setEnforceGID (boolean _b) throws UpDateError;
} // ExecutableUbuntu
