//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.Ubuntu.gBaseInterface;

import ARoad0.gBaseInterface.*;
import ARoad0.ErrB.UpDateError;
import java.util.Set;

/**
* This interface is the immutable version of UserID for Ubuntu.
* As a Linux/Unix account, it has an umask value.
*/
public interface UserIDUbuntu extends UserID {
  /**
   * This umask defines some rights to subtract to the 'full control' right,
   * at the creation of a Resource that belongs to this UserID.
   * The default value is '--- --- ---' for respectively Account, Group and Other rights.
   * @return the allowed format of 3 blocks of 3 letters among the values '-', 'r', 'w', 'x'.
   */
  public String getUMask();

  /**
   * The allowed format is 3 blocks for respectively Account, Group and Other rights.
   * _umask is the target value extended by SimplePropertyEditor with " ".
   * No control of the management by the ACS of accounts, groups and other.
   * Noop and error mesage to the user if the ep type is '<byAcsCreator>'
   * or contains '<immutable>'.
   * @param _umask from '--- --- ---' to 'wxr wxr wxr', following Linux/Unix 
   *           concept (from 0 to 777), or 3 used acsrights/metarights separated by one space,
   *           or 3 blocks separated by one space and inferior to 4,
   *           containing at most one of the characters 'wxr'.
    * @exception UpDateError for all the wrong formats or if the umask is immutable
   */
   public void setUMask(String _umask) throws UpDateError;

  /**
   * Get the denying rights for a resource and its AGO rights.
   * @param _mask part of this userID umask which describes the requirement for owner,
   *     or group or other
   * @param _isDirectory true if the calling resource is a directory
   * @param _isNxDirectory true if the calling resource is a Linux/Unix directory
   * @return the denying rights set by the umask for 'A' or 'G' or 'O' rights.
   *             May be empty but not null.
   */
  public Set<StringRight> processUMask(String _mask, boolean _isDirectory,
          boolean _isNxDirectory);

} // UserIDUbuntu


