//Title:        Access Road
//Version:      0.7.1
//Copyright:    2010-2012 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.AcsAddon.Accbee.Ubuntu.gWork;

import ARoad0.gBaseInterface.*;
import ARoad0.gBase.*;

import ARoad0.AcsAddon.Accbee.Ubuntu.gBase.*;
import ARoad0.AcsAddon.Accbee.Ubuntu.gBaseInterface.*;
import ARoad0.Pattern.AcsFactoryForAcsAddon;

import ARoad0.ErrB.BaseError;
import ARoad0.ErrB.CreateError;
import ARoad0.ErrB.UpDateError;

/**
* This class is an utility class for the creation of the Linux Ubuntu ACS
* for simulating a Linux Ubuntu 8.04 Desktop.
* It uses the packages ARoad0.AcsAddon.Accbee.Ubuntu.xxx to create the ACS in the base.
* It appears the need to create 9 new gBase classes, and also NodeRightsUbuntuImpl and
* LinkRightsUbuntuImpl in gWork. <P>
* To be comply with the AcsAddon architectural pattern, this class offers the method
* create() called by ActionNewACSyst.createACS(), and the method configure()
* called by ActionNewACSyst.fillWithAcsObjects().
* @version 0.7.1
 */
public class AcsFactoryUbuntu implements AcsFactoryForAcsAddon {
      /* the only instance of the class, following singleton pattern */
  private static AcsFactoryUbuntu  instance__ = new AcsFactoryUbuntu();
      /**
       * the descriptor of the AcsAddon to set in the menu for creating a new ACS.
       * It has to contain a '(' character.
       * ActionNewACSyst uses it to define the ACS type, as the beginning of the text
       * up to the first '(' character.
       */
  public static final String PUBLIC_ACS_ADDON_NAME = "Linux Ubuntu 8.04 (by Accbee, France, EU)";

     // Constructors

  /**
   * with this public constructor for the subclasses.
  */
  public AcsFactoryUbuntu() {
  }

    /**
    * Only method to obtain the unique instance.
    * Called by reflection from ActionNewACSyst.createACS().
    * @return an AcsFactoryUbuntu instance
    */
  public static AcsFactoryForAcsAddon getInstance() {
    return instance__;
  } // getInstance()

  /**
   * Gets the descriptor of the AcsAddon, to set in the menu for creating a new ACS.
   * This method has to return a static variable, because it is called after a call
   * to the empty constructor.
   * Called by ActionNewACSyst.actionPerformed().
   */
  public String getPublicAcsAddonName() {
      return PUBLIC_ACS_ADDON_NAME;
  }

  /**
   * Creates a specific Linux Ubuntu ACS.
   * configure() is called after this method by ActionNewACSyst.fillWithAcsObjects().
   * This method makes most of the work, but the ACS vocabulary is set by
   * ACSFactoryUbuntuImpl directly.
   * Called by ActionNewACSyst.createACS().
   * @param _iSname the IS of the new ACS. May be null.
   * @param _name of the ACS to create or open.
   * No test of null value nor empty.
   * @param _type must be the beginning of PUBLIC_ACS_ADDON_NAME
   * @param _parent is the parent node of the new ACS in the IS. Never null.
   * @exception BaseError if the ACS is known, the ACS is open,
   *   the core ACS is null when it is mandatory, or
   *   ClassNotFoundException, IOException with
   *   invalidClassException if ACSImplUbuntu HAS CHANGED,
   *   or if the type or the specified structure are not correct.
   * @return the new ACS
   */
  public ACSImpl create(ImmutableName _iSname,
          ImmutableName _name, String _type, ImmutableName _parent) throws BaseError {

        if (!PUBLIC_ACS_ADDON_NAME.startsWith(_type))
            throw new BaseError("The ACS type "+_type+"is not a Linux Ubuntu 8.04 from "+
                    PUBLIC_ACS_ADDON_NAME);

        ACSImpl acs = null;
        ACSFactoryUbuntuImpl factory = null;
        StringRightImpl mdo = null;
        StringRightImpl ctrl = null;
        StringRightImpl dctrl = null;
        StringRightImpl x_dir = null;
        StringRightImpl dx_dir = null;
        StringRightImpl w_dir = null;
        StringRightImpl dw_dir = null;
        StringRightImpl r_dir = null;
        StringRightImpl dr_dir = null;
        StringRightImpl w = null;
        StringRightImpl x = null;
        StringRightImpl r = null;
        StringRightImpl drc = null;
        StringRightImpl vw = null;
        StringRightImpl dvw = null;
        StringRightImpl vdrc = null;
        StringRightImpl dvdrc = null;
        StringRightImpl ddrc = null;
        StringRightImpl doi = null;
        StringRightImpl doifc = null;
        StringRightImpl doifas = null;
        StringRightImpl doifcas = null;
        StringRightImpl ddoi = null;

        String[] l_asso = null;

            // creates the ACS
        try {
            factory = new ACSFactoryUbuntuImpl(_iSname, _name, _type, _parent);

            // creates the acsrights -------------------------------------------
            mdo = new StringRightImpl("|mdown|",
                    BaseManagerImpl.getInstance().getMetaRight("|modify_owner|"),
                    990, null, null, factory );
            ctrl = new StringRightImpl("ctrl",
                    BaseManagerImpl.getInstance().getMetaRight("full_control"),
                    970, null, null, factory );
            w = new StringRightImpl("w",
                    BaseManagerImpl.getInstance().getMetaRight("write"),
                    700, new StringRightImpl[] {ctrl}, null, factory );
            x = new StringRightImpl("x",
                    BaseManagerImpl.getInstance().getMetaRight("execute"),
                    400, new StringRightImpl[] {ctrl}, null, factory );
            r = new StringRightImpl("r",
                    BaseManagerImpl.getInstance().getMetaRight("read"),
                    100, new StringRightImpl[] {ctrl}, null, factory );

            dctrl = new StringRightImpl("deny_ctrl",
                    BaseManagerImpl.getInstance().getMetaRight("deny_full_control"),
                    970, null, null, factory );
            new StringRightImpl("deny_w",
                    BaseManagerImpl.getInstance().getMetaRight("deny_write"),
                    700, new StringRightImpl[] {dctrl}, null, factory );
            new StringRightImpl("deny_x",
                    BaseManagerImpl.getInstance().getMetaRight("deny_execute"),
                    400, new StringRightImpl[] {dctrl}, null, factory );
            new StringRightImpl("deny_r",
                    BaseManagerImpl.getInstance().getMetaRight("deny_read"),
                    100, new StringRightImpl[] {dctrl}, null, factory );

            r_dir = new StringRightImpl("rdir",
                    BaseManagerImpl.getInstance().getMetaRight("read_for_directory"),
                    600, new StringRightImpl[] {ctrl}, null, factory );
            vw = new StringRightImpl("vw",
                    BaseManagerImpl.getInstance().getMetaRight("view"),
                    50, new StringRightImpl[] {r_dir, r}, null, factory );

            dr_dir = new StringRightImpl("deny_rdir",
                    BaseManagerImpl.getInstance().getMetaRight("deny_read_for_directory"),
                    600, new StringRightImpl[] {dctrl}, null, factory );
            dvw = new StringRightImpl("deny_vw",
                    BaseManagerImpl.getInstance().getMetaRight("deny_view"),
                    50, new StringRightImpl[] {dr_dir}, null, factory );

            x_dir = new StringRightImpl("xdir",
                    BaseManagerImpl.getInstance().getMetaRight("execute_for_directory"),
                    600, new StringRightImpl[] {ctrl}, null, factory );
            vdrc =   new StringRightImpl("vwdlrncp",
                    BaseManagerImpl.getInstance().getMetaRight("viewDeleteRenameCopy"),
                    580, new StringRightImpl[] {x_dir}, null, factory );
            drc =   new StringRightImpl("dlrncp",
                    BaseManagerImpl.getInstance().getMetaRight("deleteRenameCopy"),
                    550, new StringRightImpl[] {vdrc}, null, factory );
            new StringRightImpl("cp",
                    BaseManagerImpl.getInstance().getMetaRight("copy"),
                    500, new StringRightImpl[] {drc}, null, factory );
            new StringRightImpl("rn",
                    BaseManagerImpl.getInstance().getMetaRight("rename"),
                    500, new StringRightImpl[] {drc}, null, factory );
            new StringRightImpl("gt",
                    BaseManagerImpl.getInstance().getMetaRight("goto"),
                    20, new StringRightImpl[] {x_dir, vw}, null, factory );

            dx_dir = new StringRightImpl("deny_xdir",
                    BaseManagerImpl.getInstance().getMetaRight("deny_execute_for_directory"),
                    600, new StringRightImpl[] {dctrl}, null, factory );
            dvdrc =   new StringRightImpl("deny_vwdlrncp",
                    BaseManagerImpl.getInstance().getMetaRight("deny_viewDeleteRenameCopy"),
                    580, new StringRightImpl[] {dx_dir}, null, factory );
            ddrc =   new StringRightImpl("deny_dlrncp",
                    BaseManagerImpl.getInstance().getMetaRight("deny_deleteRenameCopy"),
                    550, new StringRightImpl[] {dvdrc}, null, factory );
            new StringRightImpl("deny_cp",
                    BaseManagerImpl.getInstance().getMetaRight("deny_copy"),
                    500, new StringRightImpl[] {ddrc}, null, factory );
            new StringRightImpl("deny_rn",
                    BaseManagerImpl.getInstance().getMetaRight("deny_rename"),
                    500, new StringRightImpl[] {ddrc}, null, factory );
            new StringRightImpl("deny_gt",
                    BaseManagerImpl.getInstance().getMetaRight("deny_all"),
                    20, new StringRightImpl[] {dx_dir, dvw}, null, factory );

            w_dir = new StringRightImpl("wdir",
                    BaseManagerImpl.getInstance().getMetaRight("write_for_nxdirectory"),
                    600, new StringRightImpl[] {ctrl}, null, factory );
            new StringRightImpl("ce",
                    BaseManagerImpl.getInstance().getMetaRight("create"),
                    500, new StringRightImpl[] {w_dir}, null, factory );
            new StringRightImpl("dl",
                    BaseManagerImpl.getInstance().getMetaRight("delete"),
                    500, new StringRightImpl[] {drc, w_dir}, null, factory );

            dw_dir = new StringRightImpl("deny_wdir",
                    BaseManagerImpl.getInstance().getMetaRight("deny_write_for_nxdirectory"),
                    600, new StringRightImpl[] {dctrl}, null, factory );
            new StringRightImpl("deny_ce",
                    BaseManagerImpl.getInstance().getMetaRight("deny_create"),
                    500, new StringRightImpl[] {dw_dir}, null, factory );
            new StringRightImpl("deny_dl",
                    BaseManagerImpl.getInstance().getMetaRight("deny_delete"),
                    500, new StringRightImpl[] {ddrc, dw_dir}, null, factory );

            // for authorizations - all have the key = 100, the same for 'read'
            doifcas = new StringRightImpl("authorize<IF><Console><IF><ActiveSession>",
                    BaseManagerImpl.getInstance().getMetaRight("do_action_one"),
                    100, null, null, factory );
            doifc = new StringRightImpl("authorize<IF><Console>",
                    BaseManagerImpl.getInstance().getMetaRight("do_action_one"),
                    100, null, null, factory );
            doifas = new StringRightImpl("authorize<IF><ActiveSession>",
                    BaseManagerImpl.getInstance().getMetaRight("do_action_one"),
                    100, null, null, factory );
            doi = new StringRightImpl("authorize",
                    BaseManagerImpl.getInstance().getMetaRight("do_action_one"),
                    100, null, null, factory );
            // 'deny_authorize' will delete all the 'authorize' rights
            ddoi = new StringRightImpl("deny_authorize",
                    BaseManagerImpl.getInstance().getMetaRight("deny_do_action_one"),
                    100, null, null, factory );

            factory.addCommentsOnProperties("All the rights",
                    new String[] {"There are 6 types of Linux",
                                    "rights:",
                                    "- current file rights (r, w, x)",
                                    "  and their opposites",
                                    "  which are used for instance",
                                    "  in the umask operations",
                                    "- file rights for directories",
                                    "  and their opposites",
                                    "- the full control (ctrl)",
                                    "  on a file or directory",
                                    "- sticky bit right to set that",
                                    "  children of a directory",
                                    "  may be deleted or renamed",
                                    "  only by their owner",
                                    "- 4 authorize rights for",
                                    "  the authorization subACS,",
                                    "  and one common opposite",
                                    "- |mdown| is the only",
                                    "  administrative right",
                                    "  for changing the owner,",
                                    "  without opposite,",
                                    " ",
                                    "The directory rights have",
                                    "the file right names (r, w, x)",
                                    "in Unix and Linux, but",
                                    "their meanings are different!",
                                    "That is why the directory",
                                    "rights (rdir, wdir, xdir) are",
                                    "set in the simulation."
                                    });


            // creates the types ---------------------------------------------
            factory.setL_EPTypes(new String[] {
                "<byAcsCreator>", "<byAcsCreator><immutable>",
                "account", "<immutable> account",
                "group", "<immutable> group",
                "personal account", "<immutable> personal account",
                "personal group", "<immutable> personal group",
                // for Actor
                "link", "<immutable> link",
                "socket", "<immutable> socket",
                "executable",  "<immutable> executable",
                "script shell", "<immutable> script shell",
                 // for Actor in network system
                //"<byAcsCreator> network",
                // for the capability system
                "<byAcsCreator> capability",
                // authorization system
                "<byAcsCreator> authorization",
                "<byAcsCreator> authorization for active session",
                "<byAcsCreator> authorization for admin active session"});

            factory.setL_ResourceTypes(new String[] {
                "<byAcsCreator>",  "<byAcsCreator><immutable> ",
                "file", "<immutable> file",
                "directory", "<immutable> directory",
                "shared memory", "<immutable> shared memory",
                "fifo", "<immutable> fifo",
                // for Actor
                "link", "<immutable> link",
                "socket", "<immutable> socket",
                "executable",  "<immutable> executable",
                "script shell", "<immutable> script shell",
                 // network system
                //"<byAcsCreator> network",
                 // authorization system
                "<byAcsCreator> authorization"});

            // The Actors are "link", "socket", "executable", "script shell",
                 // "<byAcsCreator> authorization"

            // creates the structure --------------------------------------------
            factory.setManageResources(true);
            factory.setManageResourceTrees(true);
            factory.setManageAccounts(true);
            factory.setManageGroups(true);
            factory.setManageAGORights(true);
            factory.setManageActors(true);
            factory.setManageInternalAcls(true);
            factory.setManageExternalAcls(true); // new in 0.7.1, for targets in the file system
            factory.setManageBridges(true);
            factory.setManageSecGroupsForActor(true);
            factory.setManageGroupTrees(false);
            factory.setManageAGORightsInheritance(true);
            factory.setManageRightDefinedConditionalAcls(true);

            // creates the SubAcsNodes and creates the relevant types and policy
            // for the key Explorer.SubAcsNode ----------------------------------
            // (nothing for Explorer.AcsGroupTree)
            factory.addSubAndGroupPolicy("capability",
                    new String[] {"<GroupSubACS>"});
            factory.addSubAndGroupPolicy("authorization",
                    new String[] {"<GroupSubACS>", "<ResourceSubACS>"});

            // specialized types ------------------------------------------------
            // no 'Explorer.NoAcsNode' key to forbid some nodes in the explorer
            // CreationByBeamer.NoType to forbid creation of these types by the user
            factory.addTypesAndExplorer("CreationByBeamer.NoType",
                    new String[] {
                        "ResourceType.<byAcsCreator>",
                        "ResourceType.<byAcsCreator><immutable> ",
                        "ResourceType.directory",
                        "ResourceType.<immutable> directory",
                        "ResourceType.<byAcsCreator> authorization",
                        "ResourceType.socket",
                        "ResourceType.executable",
                        "ResourceType.script shell",
                        "ResourceType.<immutable> socket",
                        "ResourceType.<immutable> executable",
                        "ResourceType.<immutable> script shell",

                        // for Directory, only 'directory' is allowed
                        "DirectoryType.<byAcsCreator>",
                        "DirectoryType.<byAcsCreator><immutable> ",
                        "DirectoryType.<byAcsCreator> authorization",
                        "DirectoryType.file",
                        "DirectoryType.link",
                        "DirectoryType.shared memory",
                        "DirectoryType.socket",
                        "DirectoryType.fifo",
                        "DirectoryType.executable",
                        "DirectoryType.script shell",
                        "DirectoryType.<immutable> file",
                        "DirectoryType.<immutable> link",
                        "DirectoryType.<immutable> shared memory",
                        "DirectoryType.<immutable> socket",
                        "DirectoryType.<immutable> fifo",
                        "DirectoryType.<immutable> executable",
                        "DirectoryType.<immutable> script shell",

                        // for Actor
                        "ActorType.<byAcsCreator>",
                        "ActorType.<byAcsCreator><immutable> ",
                        "ActorType.<byAcsCreator> authorization",
                        "ActorType.file",
                        "ActorType.directory",
                        "ActorType.shared memory",
                        "ActorType.fifo",
                        "ActorType.<immutable> file",
                        "ActorType.<immutable> directory",
                        "ActorType.<immutable> shared memory",
                        "ActorType.<immutable> fifo",

                        // for UserID, only 'account', 'personal account'
                        "UserIDType.<byAcsCreator>",
                        "UserIDType.<byAcsCreator><immutable> ",
                        "UserIDType.<byAcsCreator> capability",
                        "UserIDType.<byAcsCreator> authorization",
                        "UserIDType.<byAcsCreator> authorization for active session",
                        "UserIDType.<byAcsCreator> authorization for admin active session",
                        "UserIDType.group",
                        "UserIDType.personal group",
                        "UserIDType.link",
                        "UserIDType.executable",
                        "UserIDType.script shell",
                        "UserIDType.<immutable> group",
                        "UserIDType.<immutable> personal group",
                        "UserIDType.<immutable> link",
                        "UserIDType.<immutable> executable",
                        "UserIDType.<immutable> script shell",
                        
                        // for GroupID, only 'group', 'personal group'
                        "GroupIDType.<byAcsCreator>",
                        "GroupIDType.<byAcsCreator><immutable> ",
                        "GroupIDType.<byAcsCreator> capability",
                        "GroupIDType.<byAcsCreator> authorization",
                        "GroupIDType.<byAcsCreator> authorization for active session",
                        "GroupIDType.<byAcsCreator> authorization for admin active session",
                        "GroupIDType.account",
                        "GroupIDType.personal account",
                        "GroupIDType.link",
                        "GroupIDType.executable",
                        "GroupIDType.script shell",
                        "GroupIDType.<immutable> account",
                        "GroupIDType.<immutable> personal account",
                        "GroupIDType.<immutable> link",
                        "GroupIDType.<immutable> executable",
                        "GroupIDType.<immutable> script shell",
            });

            factory.addCommentsOnProperties("All the ACS object types",
                    new String[] {"To understand this complex",
                                    "property, read the",
                                    "documentation.",
                                    "For instance,",
                                    "'CreationByBeamer.NoType'",
                                    "forbids the creation of some",
                                    "objects:",
                                    "- some types which do not",
                                    " match with the object class,",
                                    "- all the '<byAcsCreator>'",
                                    " types are reserved to ",
                                    " the program."
                                    });

            // GroupID.NoPrimaryGroup,
            // GroupID.MemberFromAdmin, GroupID.NoMoreThanOneMember for the group policy
            factory.addTypesAndExplorer("GroupID.NoPrimaryGroup",
                    new String[] {
                        "<byAcsCreator> capability",
                        "<byAcsCreator> authorization",
                        "<byAcsCreator> authorization for active session",
                        "<byAcsCreator> authorization for admin active session"
            });
            factory.addTypesAndExplorer("GroupID.MemberFromAdmin",
                    new String[] {
                        "<byAcsCreator> authorization for admin active session"
            });
            factory.addTypesAndExplorer("GroupID.NoMoreThanOneMember",
                    new String[] {
                        "<byAcsCreator> authorization for active session",
                        "<byAcsCreator> authorization for admin active session"
            });
            factory.addTypesAndExplorer("GroupID.ConditionalAclGroup",
                    new String[] {
                // authorization system groups
                "<byAcsCreator> authorization",
                "<byAcsCreator> authorization for active session",
                "<byAcsCreator> authorization for admin active session"
            });

            // on Nodes, Resources and Actors
            factory.addTypesAndExplorer("Directory.TypesOfChildFor."+
                    "NULL",
                    new String[] {
                        "<byAcsCreator>",
            });
            factory.addTypesAndExplorer("Directory.TypesOfChildFor."+
                    "<byAcsCreator>",
                    new String[] {
                        "<byAcsCreator>",
                        "<byAcsCreator> authorization",
                        //"<byAcsCreator> network",
                        "directory"
            });
            factory.addTypesAndExplorer("Directory.TypesOfChildFor."+
                    "directory",
                    new String[] {
                        "<byAcsCreator>",
                        "<byAcsCreator> authorization",
                        "directory",
                        "file", "executable",
                        "script shell", "link",
                        "shared memory",
                        "socket",
                        "fifo"
            });
            factory.addTypesAndExplorer("Resource.OneRightACL",
                    new String[] {
                        "<byAcsCreator> authorization"
            });

            // 'Actor.IsNotBridgeTarget'
            factory.addTypesAndExplorer("Actor.IsNotBridgeTarget",
                    new String[] {
                "link",
                //"socket",
                "executable",
                "script shell",
                "<byAcsCreator> authorization"
            });

            // 'Actor.IsNotBridgeSource'
            factory.addTypesAndExplorer("Actor.IsNotBridgeSource",
                    new String[] {
                "link",
                //"socket",
                "executable",
                "script shell",
                "<byAcsCreator> authorization"
            });

            // no 'Resource.NoConditionalACL': ACL on 'authorization' are conditional ACL
            factory.addTypesAndExplorer("Resource.NoConditionalACL",
                    new String[] {
                "file",
                "socket",
                "executable",
                "directory"
            });

            // creates the rights sets ---------------------------------------------
            factory.setL_AccountRightsSet(new StringRight[] {
                mdo, ctrl, w, x, r});
            factory.setL_GroupRightsSet(new StringRight[] {
                ctrl, w, x, r});
            //factory.setL_AclRightsSet(new StringRight[] {
              //  ctrl, w, x, r});
            factory.setL_bridgeRightsSet(new StringRight[] {x});
            factory.addSpecializedRights(
                    "Target.AccRightsSet.Interface.Directory",
                    new StringRight[] {mdo, ctrl, w_dir, x_dir, r_dir});
            // also for AGO Other Resource.OtherRights
            factory.addSpecializedRights(
                    "Target.GpORightsSet.Interface.Directory",
                    new StringRight[] {ctrl, w_dir, x_dir, r_dir});
            // for external ACL targets with only the 'x' right
            factory.addSpecializedRights(
                    "Target.AclRightsSet.Type.file",
                    new StringRight[] {x});
            factory.addSpecializedRights(
                    "Target.AclRightsSet.Type.socket",
                    new StringRight[] {x});
            factory.addSpecializedRights(
                    "Target.AclRightsSet.Type.executable",
                    new StringRight[] {x});
            factory.addSpecializedRights(
                    "Target.AclRightsSet.Type.directory",
                    new StringRight[] {x});
            // unused, but allow an external ACL target from an 'authorization' source
            factory.addSpecializedRights(
                    "Target.AclRightsSet.Type.<byAcsCreator> authorization",
                    new StringRight[] {doi, doifc, doifas, doifcas, ddoi});
            // for ACL sources
            factory.addSpecializedRights(
                    "Source.AclRightsSet.Type.<byAcsCreator> authorization",
                    new StringRight[] {doi});
            factory.addSpecializedRights(
                    "Source.AclRightsSet.Type.<byAcsCreator> authorization for active session",
                    new StringRight[] {doi});
            factory.addSpecializedRights(
                    "Source.AclRightsSet.Type.<byAcsCreator> authorization for admin active session",
                    new StringRight[] {doi});
            factory.addSpecializedRights(
                    "Source.AclRightsSet.Type.personal account",
                    new StringRight[] {doi, doifc, doifas, doifcas, ddoi});
            factory.addSpecializedRights(
                    "Source.AclRightsSet.Type.<byAcsCreator> capability",
                    new StringRight[0]);
            factory.addSpecializedRights(
                    "Source.AclRightsSet.Type.group",
                    new StringRight[0]);
            factory.addSpecializedRights(
                    "Source.AclRightsSet.Type.personal group",
                    new StringRight[0]);
            factory.addSpecializedRights(
                    "Source.AclRightsSet.Type.<byAcsCreator>",
                    new StringRight[0]);

            factory.addCommentsOnProperties("Specialized rights",
                    new String[] {"To understand this complex",
                                    "property, read the",
                                    "documentation.",
                                    "For instance, ",
                                    "'Target.AccRightsSet.",
                                    "Interface.Directory'",
                                    "is a key which",
                                    "defines the set of rights",
                                    "from a UserID to a Directory.",
                                    "It overrides the UserID",
                                    "standard rights which are",
                                    "defined by the key",
                                    "'Resource.OwnerRights'."
                                    });

            // associations ------------------------------------------------------
            // associations to any account
                // account in '<AllUserAccounts>'
                    // note: registration is deleted when the account is deleted
            // For the classes, the only types are 'Resource', 'Directory', 'Actor',
            // 'UserID', 'GroupID' and 'VirtualFolder' in a key as '<Class>.<Value>.UserID',
            // even in an AcsAddon.
            factory.addAssociationsToNewObject(
                    "<Class>.<Value>.UserID",
                    new String[] {
                        "<CalledMethod>.<Object>."+
                        ((Name)factory.getName()).extended("<G>").extended(
                                "<AllUserAccounts>").toString()+
                                "+<Class>+GroupID+<Value>+addEorMember",
                        "GroupMember.<Master>"});

            // associations to a new personal account: the master
                // personal group creation
            factory.addAssociationsToNewObject(
                    "UserID.EPType.<ValueContains>.personal account",
                    new String[] {
                        "<AssociateClass>.GroupID",
                        "Name.<Value>.<MasterNameLastComponent>",
                        "ACS.<Value>.<MasterValue>",
                        "EP Type.<Value>.personal group",
                        "Is Admin.<Value>.false"});
            l_asso = ((String[]) factory.getM_AssociationsOnNewObject().
                    get("UserIDUbuntu.EPType.<ValueContains>.personal account"));
            if ( l_asso != null && l_asso[l_asso.length - 1].contains("Error") )
                throw new BaseError("AcsFactory.createACS(): error for "+
                        "<AssociateClass>.GroupID "+l_asso[l_asso.length - 1]);


                        // for testing and tracking
                        //System.out.println("AcsFactoryUbuntu.create(), "+
                          //      "l_asso:");
                        //for (int i = 0; i < l_asso.length; i++) {
                          //  System.out.println("-------- "+l_asso[i]);
                        //} // for i

                // personal group as group of the master
            factory.addAssociationsToNewObject(
                    "UserID.EPType.<ValueContains>.personal account",
                    new String[] {
                        "<CalledMethod>.<Action>.1.addEorMember",
                        "GroupMember.<Master>"});
            l_asso = ((String[]) factory.getM_AssociationsOnNewObject().
                    get("UserID.EPType.<ValueContains>.personal account"));


                        // for testing and tracking
                        //System.out.println("AcsFactoryUbuntu.create(), "+
                          //      "l_asso:");
                        //for (int i = 0; i < l_asso.length; i++) {
                          //  System.out.println("-------- "+l_asso[i]);
                        //} // for i

            // immutable personal group
            factory.addAssociationsToNewObject(
                    "UserID.EPType.<ValueContains>.personal account",
                    new String[] {
                        "<CalledMethod>.<Action>.1.setEPType",
                        "EP Type.<Value>.<immutable> personal group"});
                // personal account in 'users'
            factory.addAssociationsToNewObject(
                    "UserID.EPType.<ValueContains>.personal account",
                    new String[] {
                        "<CalledMethod>.<Object>."+
                        ((Name)factory.getName()).extended("<G>").extended("users").toString()+
                        "+<Class>+GroupID+<Value>+addEorMember",
                        "GroupMember.<Master>"});
                // master UMask before the Directory creation
            factory.addAssociationsToNewObject(
                    "UserID.EPType.<ValueContains>.personal account",
                    new String[] {
                        "<CalledMethod>.<Master>.setUMask",
                        "UMask.<Value>.--- -w- rwx "});
            if ( l_asso != null && l_asso[l_asso.length - 1].contains("Error") )
                throw new BaseError("AcsFactory.createACS(): error for "+
                        "<AssociateClass>.GroupID "+l_asso[l_asso.length - 1]);
                // personal directory creation
            factory.addAssociationsToNewObject(
                    "UserID.EPType.<ValueContains>.personal account",
                    new String[] {
                        "<AssociateClass>.Directory",
                        "Name.<Value>.<MasterNameLastComponent>",
                        "ACS.<Value>.<MasterValue>",
                        "Parent Node.<Value>.<ACSName>+ <files_tree>:: home::  ",
                        "UserIDUbuntu.<Master>",
                        "Resource Type.<Value>.<immutable> directory"});

            // new directory comment
            factory.addAssociationsToNewObject(
                    "UserID.EPType.<ValueContains>.personal account",
                    new String[] {
                        "<CalledMethod>.<Action>.6.setComment",
                        "Comment.<Value>.This is the personal directory of the same-name account"});

                            // for testing and tracking
                            //l_asso = ((String[]) factory.getM_FilteredAssociationsOnNewObject(false).
                              //      get("UserIDUbuntu.EPType.<ValueContains>.personal account"));
                            //System.out.println("AcsFactoryUbuntu.create(), "+
                              //      "l_asso:");
                            //for (int i = 0; i < l_asso.length; i++) {
                              //  System.out.println("-------- "+l_asso[i]);
                            //} // for i

            // new delegate comment
            //factory.addAssociationsToNewObject(
              //      "UserID.EPType.<ValueContains>.personal account",
                //    new String[] {
                  //      "<CalledMethod>.<Action>.8.setComment",
                    //    "Comment.<Value>.This delegate allows a session from "+
                      //          "the network under the same-name account"});

            // master primary group
            factory.addAssociationsToNewObject(
                    "UserID.EPType.<ValueContains>.personal account",
                    new String[] {
                        "<CalledMethod>.<Master>.setEorPrimGroup",
                        "Prim Group.<ArgumentValue>.1"});

            // immutable master
            //factory.addAssociationsToNewObject(
              //      "UserID.EPType.<ValueContains>.personal account",
                //    new String[] {
                  //      "<CalledMethod>.<Master>.setEPType",
                    //    "EP Type.<Value>.<immutable> personal account"});

            factory.addCommentsOnProperties("Actions for association to a new object",
                    new String[] {"To understand this complex",
                                    "property, read the documentation.",
                                    "For instance,",
                                    "'UserID.EPType.<ValueContains>.",
                                    "personal account' is a key which",
                                    "sets a script of 8 actions when",
                                    "a new UserID is dedicated to",
                                    "a person. For each new personal",
                                    "account, the action 1 creates",
                                    "a personal group, the action 6",
                                    "creates a personal directory.",
                                    "The other actions set some",
                                    "associated properties, like",
                                    "the personal account",
                                    "membership to its personal",
                                    "group."
                                    });

            // the image is under Accbee.Ubuntu.gBase.
            // caution: use the AcsAddon package editor and the AcsAddon name
            // in the ACS vocabulary.
            factory.setIcon("linux.jpg");

            // final check and creation
            acs = factory.createACS();

        } catch (CreateError c) {

                // for testing and tracking
                c.printStackTrace();
                throw new BaseError("From AcsFactory.createImpl(), "+
                        "exception: "+c);
        } catch (UpDateError u) {

                // for testing and tracking
                u.printStackTrace();
                throw new BaseError("From AcsFactory.createImpl(), "+
                        "exception: "+u);
        }


                    // for testing and tracking
                    //System.out.println("in AcsFactoryUbuntu.create(), "+
                      //"acs at the end is open: "+
                      //BaseManagerImpl.getInstance().isOpenACS(acs.getFullName()));

        factory = null;
        mdo = null;
        ctrl = null;
        dctrl = null;
        x_dir = null;
        dx_dir = null;
        w_dir = null;
        dw_dir = null;
        r_dir = null;
        dr_dir = null;
        w = null;
        x = null;
        r = null;
        drc = null;
        vw = null;
        dvw = null;
        vdrc = null;
        dvdrc = null;
        ddrc = null;
        doi = null;
        doifc = null;
        doifas = null;
        doifcas = null;
        ddoi = null;

        return acs;
  } // create()

  /**
  * Creates the components in a Linux Ubuntu ACS.
  * Creates a resources tree with directories, leaves and executables.
  * Called by ActionNewACSyst.fillWithAcsObjects().
  * @param _result to construct
  * @return ACS with the initial and standard trees of resources,
  * eligible parties.
  */
  public ACSImpl configure(ACSImpl _result) {

            // for testing and tracking
            //System.out.println("call to AcsFactoryUbuntu.configure()");

    if (!_result.getACSType().startsWith(
            PUBLIC_ACS_ADDON_NAME.substring(0, PUBLIC_ACS_ADDON_NAME.indexOf(" ("))) ) {

                                // for testing and tracking
                                System.out.println("AcsFactoryUbuntu.configure() with error on "+
                                        "_result.getACSType(): "+_result.getACSType());

      throw new InternalError("AcsFactoryUbuntu.configure() is not called for a Linux Ubuntu 8.04 ");
    }

    //  creates non-actor EligibleParties
    ImmutableName   naACS = _result.getName();
    GroupID         grpWheel = null;
    Name            naRoot = ((Name)naACS.clone()).extended("root");
    UserIDUbuntu    root = null;
    //Name naGrpOne = ((Name)naACS.clone()).extended("tom");
    //GroupIDImpl grpOne = null;
    //Name naGrpTwo = ((Name)naACS.clone()).extended("jerry");
    //GroupIDImpl grpTwo = null;
    GroupID         grpThree = null;
    GroupID         src = null;
    /*
    Name naGrpFour = ((Name)naACS.clone()).extended("GROUP_FOUR");
    GroupIDImpl grpFour = null;
     * */
    Name            naUseOne = ((Name)naACS.clone()).extended("tom");
    UserIDUbuntu    useOne = null;
    Name            naUseTwo = ((Name)naACS.clone()).extended("jerry");
    UserIDUbuntu    useTwo = null;
    GroupID         mail = null;

    try {
      // <AllUserAccounts> in the authorization system,
      // contains all the new userids, by the mean of an associate command
      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("<AllUserAccounts>"),
              _result, "<byAcsCreator> authorization", false);
      grpThree.setComment("Contains all the personal and applicative accounts of Linux");

        // wrong EP type to allow the updating
      grpWheel = new GroupIDImpl(naRoot, _result, null, true);
      root = new UserIDUbuntuImpl(naRoot, _result, null, true);
      grpWheel.setOrder(0);
      root.setOrder(0);
      root.setEPType("<byAcsCreator>");
      grpWheel.setEPType("<byAcsCreator>");

      // users group
      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("users"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(100);

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("deamon"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(1);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("deamon"), _result, null, false);
      GroupIDMember[] l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(1);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("bin"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(2);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("bin"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(2);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("sys"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(3);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("sys"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(3);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("sync"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(65534);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("sync"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(4);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("games"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(60);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("games"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(5);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("tty"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(5);
      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("disk"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(6);

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("lp"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(7);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("lp"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(7);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      mail = new GroupIDImpl(((Name)naACS.clone()).extended("mail"),
              _result, "<byAcsCreator>", false);
      mail.setOrder(8);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("mail"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      mail.setEorL_Members(l_ep_one);
      useOne.setOrder(8);
      useOne.setEorPrimGroup(mail);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("news"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(9);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("news"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(9);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("uucp"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(10);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("uucp"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(10);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("man"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(12);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("man"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(6);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("proxy"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(13);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("proxy"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(13);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("sudo"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(27);

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("backup"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(34);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("backup"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(34);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("operator"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(37);

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("Mailing List Manager"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(38);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("list"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(38);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("ircd"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(39);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("irc"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(39);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      src = new GroupIDImpl(((Name)naACS.clone()).extended("src"),
              _result, "<byAcsCreator>", false);
      src.setOrder(40);

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("dhcp"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(101);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("dhcp"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(100);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("PolicyKit"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(121);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("polkituser"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(109);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("Hardware abstract layer"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(122);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("haldeamon"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(110);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("ntp"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(123);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("ntp"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(111);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("libuuid"),
              _result, "<byAcsCreator>", false);
      grpThree.setOrder(126);
      useOne = new UserIDUbuntuImpl(((Name)naACS.clone()).extended("libuuid"), _result, null, false);
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      useOne.setOrder(113);
      useOne.setEorPrimGroup(grpThree);
      useOne.setEPType("<byAcsCreator>");

      GroupIDMember[] l_ep_root = {root};
      grpWheel.setEorL_Members(l_ep_root);
      root.setEPType("<12345678901234567890>");
      root.setEorPrimGroup(grpWheel);
      root.setUMask("--- -w- -w- ");
      root.setEPType("<byAcsCreator>");

                // for testing and tracking
                //System.out.println("in AcsFactoryUbuntu.create(), "+
                  //"grpWheel ="+grpWheel+"\n"+"root ="+root+"\n"+"useOne ="+useOne);


      // creates a resources tree with directories, leaves
      // and executables (then actors)
      ResourceUbuntu      res;
      Name          name;
      StringRight[] l_right = new StringRight[] {
                                    _result.getOneRight("|mdown|"),
                                    _result.getOneRight("ctrl")
                                };
      StringRight[] l_right2 = new StringRight[] {
                                    _result.getOneRight("ctrl")
                                };
      StringRight[] l_right3 = new StringRight[] {
                                    _result.getOneRight("rdir"),
                                    _result.getOneRight("xdir")
                                };
      StringRight[] l_right4 = new StringRight[] {
                                    _result.getOneRight("x")
                                };
      StringRight[] l_right5 = new StringRight[] {
                                    _result.getOneRight("rdir")
                                };
      StringRight[] l_right6 = new StringRight[] {
                                    _result.getOneRight("r")
                                };
      StringRight[] l_right7 = new StringRight[0];

      // creates '<files_tree>', the root of the files tree
      Name naC = ((Name)_result.getName()).extended("<files_tree>");
      DirectoryUbuntuImpl tree = new DirectoryUbuntuImpl(naC,
                                    _result, null, root, "<byAcsCreator>");
      tree.setResourceType("<12345678901234567890>");
      tree.setL_otherRights(l_right3);
      tree.setResourceType("<byAcsCreator>");

      name = ((Name) naC.clone()).extended("vmlinuz");
      res = new ResourceUbuntuImpl(name, _result, tree, root, "<byAcsCreator>");
      res.setResourceType("<12345678901234567890>");
      res.setL_groupRights(l_right6);
      res.setL_otherRights(l_right6);
      res.setResourceType("<byAcsCreator>");

      Name bin = ((Name) naC.clone()).extended("bin");
      DirectoryUbuntuImpl directory =
              new DirectoryUbuntuImpl(bin, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_groupRights(l_right3);
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

      // bin children
      Actor a = new ActorUbuntuImpl(
              ((Name)bin.clone()).extended("<typical_nx_command>"),
              _result, directory, root, "<byAcsCreator>",
              null);
      a.setResourceType("<12345678901234567890>");
      a.setIsRightProxy(false);
      a.setResourceType("<byAcsCreator>");

      Name boot = ((Name) naC.clone()).extended("boot");
      directory = new DirectoryUbuntuImpl(boot, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_ownerRights(l_right);
      directory.setL_groupRights(l_right3);
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");
      res = new ResourceUbuntuImpl(
              ((Name) boot.clone()).extended("<typical_boot_file>"),
              _result, directory, root, "<byAcsCreator>");

      Name cdrom = ((Name) naC.clone()).extended("cdrom");
      directory = new DirectoryUbuntuImpl(cdrom, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

      Name dev = ((Name) naC.clone()).extended("dev");
      directory = new DirectoryUbuntuImpl(dev, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

      Name etc = ((Name) naC.clone()).extended("etc");
      directory = new DirectoryUbuntuImpl(etc, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

                // for testing and tracking
                //System.out.println("in AcsFactoryUbuntu.create(), "+
                  //"directory ="+directory);


      res = new ResourceUbuntuImpl(
              ((Name) etc.clone()).extended("<typical_conf_file>"),
              _result, directory, root, "<byAcsCreator>");

      res = new DirectoryUbuntuImpl(
              ((Name) etc.clone()).extended("<typical_application_directory>"),
              _result, directory, root, "<byAcsCreator>");

      Name home = ((Name) naC.clone()).extended("home");
      directory = new DirectoryUbuntuImpl(home, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

      Name initrd = ((Name) naC.clone()).extended("initrd.img");
      res = new ResourceUbuntuImpl(initrd, _result, tree, root, "<byAcsCreator>");
      res.setResourceType("<12345678901234567890>");
      res.setL_groupRights(l_right6);
      res.setL_otherRights(l_right6);
      res.setResourceType("<byAcsCreator>");

      Name lib = ((Name) naC.clone()).extended("lib");
      directory = new DirectoryUbuntuImpl(lib, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

                // no-root rights with root owner
              Name lost = ((Name) naC.clone()).extended("lost+found");
              directory = new DirectoryUbuntuImpl(lost, _result, tree, root, "<byAcsCreator>");
              directory.setResourceType("<12345678901234567890>");
              directory.setL_ownerRights(l_right);
              directory.setResourceType("<byAcsCreator>");

      Name media = ((Name) naC.clone()).extended("media");
      directory = new DirectoryUbuntuImpl(media, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

      Name mnt = ((Name) naC.clone()).extended("mnt");
      directory = new DirectoryUbuntuImpl(mnt, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

      Name opt = ((Name) naC.clone()).extended("opt");
      directory = new DirectoryUbuntuImpl(opt, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

                // no-root rights with root owner
              Name proc = ((Name) naC.clone()).extended("proc");
              directory = new DirectoryUbuntuImpl(proc, _result, tree, root, "<byAcsCreator>");
              directory.setResourceType("<12345678901234567890>");
              directory.setL_ownerRights(new StringRight[] {
                                            _result.getOneRight("|mdown|"),
                                            _result.getOneRight("rdir"),
                                            _result.getOneRight("xdir")
                                        });
              directory.setL_groupRights(l_right3);
              directory.setL_otherRights(l_right3);
              directory.setResourceType("<byAcsCreator>");

      Name nroot = ((Name) naC.clone()).extended("root");
      directory = new DirectoryUbuntuImpl(nroot, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_groupRights(l_right7); // no rights
      directory.setResourceType("<byAcsCreator>");

      Name sbin = ((Name) naC.clone()).extended("sbin");
      directory = new DirectoryUbuntuImpl(sbin, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

      a = new ActorUbuntuImpl(
              ((Name)sbin.clone()).extended("<typical_nx_command>"),
              _result, directory, root, "<byAcsCreator>",
              null);
      a.setResourceType("<12345678901234567890>");
      a.setIsRightProxy(false);
      a.setResourceType("<byAcsCreator>");

      Name srv = ((Name) naC.clone()).extended("srv");
      directory = new DirectoryUbuntuImpl(srv, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

      Name sys = ((Name) naC.clone()).extended("sys");
      directory = new DirectoryUbuntuImpl(sys, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

                // no-root rights with root owner
              Name tmp = ((Name) naC.clone()).extended("tmp");
              directory = new DirectoryUbuntuImpl(tmp, _result, tree, root, "<byAcsCreator>");
              directory.setResourceType("<12345678901234567890>");
              directory.setL_ownerRights(l_right);
              directory.setL_groupRights(l_right);
              directory.setL_otherRights(l_right);
              directory.setResourceType("<byAcsCreator>");
              directory.setStickyBit(true);

      Name usr = ((Name) naC.clone()).extended("usr");
      directory = new DirectoryUbuntuImpl(usr, _result, tree, root, "<byAcsCreator>");
      directory.setResourceType("<12345678901234567890>");
      directory.setL_otherRights(l_right3);
      directory.setResourceType("<byAcsCreator>");

      // usr children
      DirectoryUbuntuImpl usrbin = new DirectoryUbuntuImpl(
              ((Name) usr.clone()).extended("bin"),
              _result, directory, root, "<byAcsCreator>");
      usrbin.setResourceType("<12345678901234567890>");
      usrbin.setL_otherRights(l_right3);
      usrbin.setResourceType("<byAcsCreator>");

      DirectoryUbuntuImpl usrlib = new DirectoryUbuntuImpl(
              ((Name) usr.clone()).extended("lib"),
              _result, directory, root, "<byAcsCreator>");
      usrlib.setResourceType("<12345678901234567890>");
      usrlib.setL_otherRights(l_right3);
      usrlib.setResourceType("<byAcsCreator>");

      DirectoryUbuntuImpl usrlibtad = new DirectoryUbuntuImpl(
              ((Name) usrlib.getName().clone()).extended("<typical_application_directory>"),
              _result, usrlib, root, "<byAcsCreator>");
      usrlibtad.setResourceType("<12345678901234567890>");
      usrlibtad.setL_otherRights(l_right3);
      usrlibtad.setResourceType("<byAcsCreator>");

      // directory is 'usr'
      DirectoryUbuntuImpl usrlocal = new DirectoryUbuntuImpl(
              ((Name) usr.clone()).extended("local"),
              _result, directory, root, "<byAcsCreator>");
      usrlocal.setResourceType("<12345678901234567890>");
      usrlocal.setL_otherRights(l_right3);
      usrlocal.setResourceType("<byAcsCreator>");

      DirectoryUbuntuImpl usrsbin = new DirectoryUbuntuImpl(
              ((Name) usr.clone()).extended("sbin"),
              _result, directory, root, "<byAcsCreator>");
      usrsbin.setResourceType("<12345678901234567890>");
      usrsbin.setL_otherRights(l_right3);
      usrsbin.setResourceType("<byAcsCreator>");

      DirectoryUbuntuImpl usrshare = new DirectoryUbuntuImpl(
              ((Name) usr.clone()).extended("share"),
              _result, directory, root, "<byAcsCreator>");
      usrshare.setResourceType("<12345678901234567890>");
      usrshare.setL_otherRights(l_right3);
      usrshare.setResourceType("<byAcsCreator>");

              // no-root rights with root owner
              DirectoryUbuntuImpl usrsrc = new DirectoryUbuntuImpl(
                      ((Name) usr.clone()).extended("src"),
                      _result, directory, root, "<byAcsCreator>");
              usrsrc.setResourceType("<12345678901234567890>");
              usrsrc.setEorGroupID(src);
              usrsrc.setEnforceChildGID(true);
              usrsrc.setL_groupRights(l_right2);
              usrsrc.setL_otherRights(l_right3);
              usrsrc.setResourceType("<byAcsCreator>");

      // typical usr/bin and usr/sbin children
      a = new ActorUbuntuImpl(
              ((Name)usrbin.getName()).extended("<typical_nx_command>"),
              _result, usrbin, root, "<byAcsCreator>",
              null);
      a.setResourceType("<12345678901234567890>");
      a.setIsRightProxy(false);
      a.setResourceType("<byAcsCreator>");
      a = new ActorUbuntuImpl(
              ((Name)usrsbin.getName()).extended("<typical_nx_command>"),
              _result, usrsbin, root, "<byAcsCreator>",
              null);
      a.setResourceType("<12345678901234567890>");
      a.setIsRightProxy(false);
      a.setResourceType("<byAcsCreator>");

      // var directory and its children
      DirectoryUbuntuImpl var = new DirectoryUbuntuImpl(
              ((Name) naC.clone()).extended("var"),
              _result, tree, root, "<byAcsCreator>");
      var.setResourceType("<12345678901234567890>");
      var.setL_otherRights(l_right3);
      var.setResourceType("<byAcsCreator>");

      DirectoryUbuntuImpl varlog = new DirectoryUbuntuImpl(
              ((Name) var.getName().clone()).extended("log"),
              _result, var, root, "<byAcsCreator>");
      varlog.setResourceType("<12345678901234567890>");
      varlog.setL_otherRights(l_right3);
      varlog.setResourceType("<byAcsCreator>");

      varlog = new DirectoryUbuntuImpl(
              ((Name) var.getName().clone()).extended("lib"),
              _result, var, root, "<byAcsCreator>");
      varlog.setResourceType("<12345678901234567890>");
      varlog.setL_otherRights(l_right3);
      varlog.setResourceType("<byAcsCreator>");

      varlog = new DirectoryUbuntuImpl(
              ((Name) var.getName().clone()).extended("run"),
              _result, var, root, "<byAcsCreator>");
      varlog.setResourceType("<12345678901234567890>");
      varlog.setL_otherRights(l_right3);
      varlog.setResourceType("<byAcsCreator>");

      varlog = new DirectoryUbuntuImpl(
              ((Name) var.getName().clone()).extended("backups"),
              _result, var, root, "<byAcsCreator>");
      varlog.setResourceType("<12345678901234567890>");
      varlog.setL_otherRights(l_right3);
      varlog.setResourceType("<byAcsCreator>");

      varlog = new DirectoryUbuntuImpl(
              ((Name) var.getName().clone()).extended("cache"),
              _result, var, root, "<byAcsCreator>");
      varlog.setResourceType("<12345678901234567890>");
      varlog.setL_otherRights(l_right3);
      varlog.setResourceType("<byAcsCreator>");

      varlog = new DirectoryUbuntuImpl(
              ((Name) var.getName().clone()).extended("games"),
              _result, var, root, "<byAcsCreator>");
      varlog.setResourceType("<12345678901234567890>");
      varlog.setL_otherRights(l_right3);
      varlog.setResourceType("<byAcsCreator>");

      varlog = new DirectoryUbuntuImpl(
              ((Name) var.getName().clone()).extended("opt"),
              _result, var, root, "<byAcsCreator>");
      varlog.setResourceType("<12345678901234567890>");
      varlog.setL_otherRights(l_right3);
      varlog.setResourceType("<byAcsCreator>");

      varlog = new DirectoryUbuntuImpl(
              ((Name) var.getName().clone()).extended("spool"),
              _result, var, root, "<byAcsCreator>");
      varlog.setResourceType("<12345678901234567890>");
      varlog.setL_otherRights(l_right3);
      varlog.setResourceType("<byAcsCreator>");

        // enforce GID on 'mail' group
      varlog = new DirectoryUbuntuImpl(
              ((Name) var.getName().clone()).extended("mail"),
              _result, var, root, "<byAcsCreator>");
      varlog.setResourceType("<12345678901234567890>");
      varlog.setEorGroupID(mail);
      varlog.setEnforceChildGID(true);
      varlog.setL_groupRights(l_right2);
      varlog.setL_otherRights(l_right3);
      varlog.setResourceType("<byAcsCreator>");

              // for testing and tracking
              //System.out.println("in AcsFactoryUbuntu.create(), "+
                //"create under '/' the children: ");
              //for (int i = 1 ; i < c.getEorL_Children().length ; i++) {
                //System.out.println("child "+i+": "+((Resource)c.getEorL_Children()[i]).toString());
              //}

      // tom and jerry accounts
      useOne = new UserIDUbuntuImpl(naUseOne, _result, "personal account", true);
      useTwo = new UserIDUbuntuImpl(naUseTwo, _result, "personal account", false);
      useOne.setOrder(501);
      useTwo.setOrder(502);

      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("adm"),
              _result, "<byAcsCreator>", true);
      grpThree.setOrder(4);
      // 'tom' is an administrator
      l_ep_one = new GroupIDMember[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("admin"),
              _result, "<byAcsCreator>", true);
      grpThree.setOrder(103);
      // 'tom' is an administrator
      grpThree.setEorL_Members(l_ep_one);

      // creates associates of useOne and useTwo
      try {
            AssociateUtilityImpl.addAssociatesAfterCreation((EligibleParty)useOne);
            AssociateUtilityImpl.addAssociatesAfterCreation((EligibleParty)useTwo);
      } catch (BaseError b) {
            throw new InternalError("After the creation of a personal account"+
                    ", an association action throws an error: "+
                    b.toString());
      } // catch

      // LINUX CAP_DAC_OVERRIDE and CAP_DAC_READ_SEARCH capability groups
      // with 'tom' as member
      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("<CAP_DAC_OVERRIDE>"),
              _result, "<byAcsCreator> capability", true);
      l_ep_one = new UserIDUbuntu[] {useOne};
      grpThree.setEorL_Members(l_ep_one);
      grpThree.setComment("This capability grants a full_control permission by default,"+
                "but grants execute permission only when at least one of the resource's three "+
                "execute permission bits is set");
      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("<CAP_DAC_READ_SEARCH>"),
              _result, "<byAcsCreator> capability", true);
      grpThree.setEorL_Members(l_ep_one);
      grpThree.setComment("This capability grants read and search permission on directories, "+
                "and read permission on ordinary files");

      // add objects under the 'jerry' home directory, which is an associate
      directory = (DirectoryUbuntuImpl)_result.getEorM_OwnResources().get(
              ((Name) home.clone()).extended("jerry") );
      directory.setResourceType("directory");
      directory.setL_otherRights(l_right3); // rdir, xdir in other rights
      directory.setResourceType("<immutable> directory");

                      // for testing and tracking
                      //System.out.println("in AcsFactoryUbuntu.configure(), "+
                        //"set the AGO other rights of the directory 'jerry': ");
                      //StringRight[] l_r = directory.getL_otherRights();
                      //for (int i = 1 ; i < l_r.length ; i++) {
                        //System.out.println("right "+i+": "+l_r[i]);
                      //}


      new ResourceUbuntuImpl(
              ((Name) home.clone()).extended("jerry").extended("f_jerry"),
              _result, directory, useTwo, "file");
      directory = new DirectoryUbuntuImpl(
              ((Name) home.clone()).extended("jerry").extended("d_jerry"),
              _result, directory, useTwo, "directory");
      directory.setL_otherRights(l_right5); // rdir in other rights
      ExecutableUbuntuImpl eje = new ExecutableUbuntuImpl(
              ((Name) home.clone()).extended("jerry").extended("d_jerry").extended("e_jerry"),
              _result, (Directory)directory, useTwo, "executable", null);
      eje.setL_otherRights(l_right4); // x in other rights


      // AUTHORIZATION system
      // get <authorization_subacs> directory, created through ACSFactoryImpl.createACS()
      Directory cau = (Directory)_result.getEorM_OwnResources().get(
              ((Name) naACS.clone()).extended("<authorization_subacs>") );
      var = new DirectoryUbuntuImpl(
              ((Name) cau.getName()).extended("org"),
              _result, cau, null, "<byAcsCreator>");
      varlog = new DirectoryUbuntuImpl(
              ((Name) cau.getName()).extended("org").extended("gnome"),
              _result, var, null, "<byAcsCreator>");
      new DirectoryUbuntuImpl(
              ((Name) cau.getName()).extended("org").extended("freedesktop"),
              _result, var, null, "<byAcsCreator>");
          // the node types are not switched to '<byAcsCreator> authorization'
          // to forbid ACLs
      //var.setResourceType("<12345678901234567890>");
      //var.setResourceType("<byAcsCreator> authorization");

      // org.gnome processing
      var = new DirectoryUbuntuImpl(
              ((Name) varlog.getName()).extended("clockapplet"),
              _result, varlog, null, "<byAcsCreator>");
      //varlog.setResourceType("<12345678901234567890>");
      //varlog.setResourceType("<byAcsCreator> authorization");
      directory = new DirectoryUbuntuImpl(
              ((Name) var.getName()).extended("mechanism"),
              _result, var, null, "<byAcsCreator>");
      //var.setResourceType("<12345678901234567890>");
      //var.setResourceType("<byAcsCreator> authorization");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("set time zone"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("set time"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("configure hw clock"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      //directory.setResourceType("<12345678901234567890>");
      //directory.setResourceType("<byAcsCreator> authorization");

      // org.freedestop processing
      var = (DirectoryUbuntuImpl)_result.getEorM_OwnResources().get(
              ((Name) cau.getName()).extended("org").extended("freedesktop") );
      directory = new DirectoryUbuntuImpl(
              ((Name) var.getName()).extended("systemtoolsbackends"),
              _result, var, null, "<byAcsCreator>");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("set"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      varlog = new DirectoryUbuntuImpl(
              ((Name) directory.getName()).extended("self"),
              _result, directory, null, "<byAcsCreator>");
      //directory.setResourceType("<12345678901234567890>");
      //directory.setResourceType("<byAcsCreator> authorization");
      res = new ResourceUbuntuImpl(
        ((Name) varlog.getName()).extended("set"),
        _result, varlog, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      //varlog.setResourceType("<12345678901234567890>");
      //varlog.setResourceType("<byAcsCreator> authorization");

      directory = new DirectoryUbuntuImpl(
        ((Name) var.getName()).extended("policykit"),
        _result, var, null, "<byAcsCreator>");
      varlog = new DirectoryUbuntuImpl(
        ((Name) var.getName()).extended("hal"),
        _result, var, null, "<byAcsCreator>");
      //var.setResourceType("<12345678901234567890>");
      //var.setResourceType("<byAcsCreator> authorization");

      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("revoke authorizations from other users"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("read authorizations from other users"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("modify defaults for implicit authorizations"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("grant authorizations to other users"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      //directory.setResourceType("<12345678901234567890>");
      //directory.setResourceType("<byAcsCreator> authorization");

      // org.freedesktop.hal.wol
      directory = new DirectoryUbuntuImpl(
        ((Name) varlog.getName()).extended("wol"),
        _result, varlog, null, "<byAcsCreator>");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("If Wake on LAN is suppported"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("If Wake on LAN is enabled"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Enable or disable Wake on LAN"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      //directory.setResourceType("<12345678901234567890>");
      //directory.setResourceType("<byAcsCreator> authorization");

      directory = new DirectoryUbuntuImpl(
        ((Name) varlog.getName()).extended("storage"),
        _result, varlog, null, "<byAcsCreator>");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Unmount file systems mounted by other users"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Mount file systems from removable drives"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Mount file systems from internal drives"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Eject removable media"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Set up decryption for encrypted removable storage devices"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Set up decryption for encrypted fixed storage devices"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      //directory.setResourceType("<12345678901234567890>");
      //directory.setResourceType("<byAcsCreator> authorization");

      directory = new DirectoryUbuntuImpl(
        ((Name) varlog.getName()).extended("power-management"),
        _result, varlog, null, "<byAcsCreator>");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Suspend the system"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Shut down the system when multiple users are logged in"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Shut down the system"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Configure to system to prefer power savings"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Reboot the system when multiple users are logged in"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Reboot the system"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Detect ambient light using light sensor"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Set laptop panel brightness"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Set keyboard backlight"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Hibernate the system"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Configure CPU frequence scaling"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      //directory.setResourceType("<12345678901234567890>");
      //directory.setResourceType("<byAcsCreator> authorization");

      new ResourceUbuntuImpl(
        ((Name) varlog.getName()).extended("Acquire locks on devices"),
        _result, varlog, null, "<byAcsCreator> authorization");

      // org.freedesktop.hal.killswich
      directory = new DirectoryUbuntuImpl(
        ((Name) varlog.getName()).extended("killswitch"),
        _result, varlog, null, "<byAcsCreator>");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Turn WWAN radio On/Off"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Turn WLAN radio On/Off"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Turn Bluetooth radio On/Off"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      //directory.setResourceType("<12345678901234567890>");
      //directory.setResourceType("<byAcsCreator> authorization");

      directory = new DirectoryUbuntuImpl(
        ((Name) varlog.getName()).extended("dockstation"),
        _result, varlog, null, "<byAcsCreator>");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Undock a docking station"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      //directory.setResourceType("<12345678901234567890>");
      //directory.setResourceType("<byAcsCreator> authorization");

      directory = new DirectoryUbuntuImpl(
        ((Name) varlog.getName()).extended("device-access"),
        _result, varlog, null, "<byAcsCreator>");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Directly access video capture devices"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Directly access the uinput control device"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Directly access sound devices"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Directly access scanners"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Directly access PDA devices"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Directly access Firewire IIDC devices"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Directly access Firewire AVC devices"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Directly access DVB devices"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Directly access optical drives"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Directly access digital cameras"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      res = new ResourceUbuntuImpl(
        ((Name) directory.getName()).extended("Directly access audio players"),
        _result, directory, null, "<byAcsCreator> authorization");
      res.setComment("This is an authorization which may be granted, through "+
              "an ACL, to authorization groups or to a single user");
      //directory.setResourceType("<12345678901234567890>");
      //directory.setResourceType("<byAcsCreator> authorization");
      // end of the resources for Authorization

      // GroupIDs for AUTHORIZATION system
      // <Console> in the authorization system
      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("<Console>"),
              _result, "<byAcsCreator> authorization", false);
      grpThree.setComment("Put in this group, the accounts which are associated to a console");

      // <ActiveSession> in the authorization system
      // with a type in the 'GroupID.NoMoreThanOneMember' values
      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("<ActiveSession>"),
              _result, "<byAcsCreator> authorization for active session", false);
      grpThree.setComment("Put in this group, the account which is associated to an active session");

      // <AdminActiveSession> in the authorization system,
      // with a type in the 'GroupID.MemberFromAdmin' and 'GroupID.NoMoreThanOneMember' values
      grpThree = new GroupIDImpl(((Name)naACS.clone()).extended("<AdminActiveSession>"),
              _result, "<byAcsCreator> authorization for admin active session", true);
      grpThree.setComment("Put in this group, the account which is associated to "+
              "an active session for an administrator");

        // after the Actor creation, to fill the no-proxy actors list
      BaseManagerImpl.getInstance().restoreACSInternalLinks(_result);

        naACS = null;
        grpWheel = null;
        naRoot = null;
        root = null;
        grpThree = null;
        mail = null;
        src = null;
        naUseOne = null;
        useOne = null;
        naUseTwo = null;
        useTwo = null;

        res = null;
        name = null;
        l_right = null;
        l_right2 = null;
        l_right3 = null;
        l_right4 = null;
        l_right5 = null;
        l_right6 = null;
        l_right7 = null;
        tree = null;
        naC = null;
        cau = null;
        bin = null;
        directory = null;
        a = null;
        boot = null;
        cdrom = null;
        dev = null;
        etc = null;
        home = null;
        eje = null;
        initrd = null;
        lib = null;
        lost = null;
        media = null;
        mnt = null;
        opt = null;
        proc = null;
        nroot = null;
        sbin = null;
        srv = null;
        sys = null;
        tmp = null;
        usr = null;
        usrbin = null;
        usrlib = null;
        usrlibtad = null;
        usrlocal = null;
        usrsbin = null;
        usrshare = null;
        usrsrc = null;
        var = null;
        varlog = null;

    } // try
    catch (CreateError e) {
        e.printStackTrace();
        throw new InternalError(e.toString()+
              "BaseManagerImpl.createImplACS() failed: "+e);
    }
    catch (UpDateError u) {
        u.printStackTrace();
        throw new InternalError(u.toString()+
                "BaseManagerImpl.createImplACS() failed: "+u);
    }

    /*
     * TEST OF THE PATHS BETWEEN the 6 nodes:
     * <G>::  sys, <G>::  jerry, <U>::  jerry, home::  jerry, f_jerry, and e_jerry.
     * It covers the paths:
     * - <G>::  sys ->  e_jerry.
     *      group/other/actor  with AGO inheritance for gt/vw, weak link
     *      Main result: x Other direct right
     * - <G>::  jerry -> jerry:
     *      group/contain/directory with AGO inheritance for directory
     *      Main result: weak link, rdir/xdir AGO direct rights, and the home direct parent
     *      delivers vw/gt/dlrncp inherited rights
     * - <G>::  jerry -> e_jerry:
     *      group/contain/actor with AGO inheritance for actor
     *      Main result: weak link, r/x AGO direct rights, and the e_jerry direct parent
     *      delivers vw/gt/dlrncp inherited rights
     * - <G>::  jerry -> f_jerry:
     *      group/contain/resource with AGO inheritance for resource
     *      Main result: like the previous path
     * - <U>::  jerry -> jerry:
     *      account/own/directory with AGO inheritance for directory
     *      Main result: ctrl AGO direct rights, and the home direct parent
     *      delivers vw/gt/dlrncp inherited rights through AGO 'other' rights
     * - <U>::  jerry -> e_jerry:
     *      account/own/actor with AGO inheritance for actor
     *      Main result: ctrl AGO direct rights, and the jerry direct parent
     *      delivers vw/gt/dl/dlrncp/ce inherited rights through AGO 'account' rights
     * - <U>::  jerry -> f_jerry:
     *      account/contain/resource with AGO inheritance for resource
     *      Main result: like the previous path
     * - e_jerry -> <U>::  jerry:
     *      actor/run under/account
     *      Main result: 'runs under' and no rights
     * - e_jerry -> <G>::  jerry:
     *      actor/run under/group
     *      Main result: 'runs under' and no rights
     * - e_jerry -> <G>::  sys:
     *      actor/soft run under/group
     *      Main result: 'may run under' and no rights
     * - <U>::  jerry -> <G>::  jerry:
     *      account/member/group
     *      Main result: 'member' and no rights
     *
     * The view should generate the following SeeWhy text:
     *
FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <G>::  jerry
 TO  IO:: AA:: ubu:: <files_tree>:: home::  jerry.
   PATH number 1:
            Link number 1:
                  Contains (<WEAK> AGO Other rights)  --  <inherited AG rights>  -- All Rights: xdir/rdir/gt/dlrncp /vw

FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <G>::  jerry
 TO  IO:: AA:: ubu:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry.
   PATH number 1:
            Link number 1:
                  Contains (<WEAK>)  --  <inherited AG rights>  -- All Rights: gt/dlrncp/vw/r/x

FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <G>::  jerry
 TO  IO:: AA:: ubu:: <files_tree>:: home:: jerry::  f_jerry.
   PATH number 1:
            Link number 1:
                  Contains (<WEAK>)  --  <inherited AG rights>  -- All Rights: gt/dlrncp/vw/r/x

FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <G>::  sys
 TO  IO:: AA:: ubu:: <files_tree>:: home::  jerry.
   PATH number 1:
            Link number 1:
                   <inherited AG rights> AGO Other rights <WEAK>  -- All Rights: xdir/gt/rdir/dlrncp /vw

FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <G>::  sys
 TO  IO:: AA:: ubu:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry.
   PATH number 1:
            Link number 1:
                   <inherited AG rights> AGO Other rights <WEAK>  -- All Rights: gt/vw/x

FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <G>::  sys
 TO  IO:: AA:: ubu:: <files_tree>:: home:: jerry::  f_jerry.
   PATH number 1:
            Link number 1:
                   <inherited AG rights> AGO Other rights <WEAK>  -- All Rights: gt/dlrncp/vw

FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <U>::  jerry
 TO  IO:: AA:: ubu:: <G>::  jerry.
   PATH number 1:
            Link number 1:
                  Is Member  --  (no right)

FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <U>::  jerry
 TO  IO:: AA:: ubu:: <files_tree>:: home::  jerry.
   PATH number 1:
            Link number 1:
                  Owns (AGO Other rights)  --  <inherited AG rights>  -- All Rights: gt/ctrl/dlrncp/vw

FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <U>::  jerry
 TO  IO:: AA:: ubu:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry.
   PATH number 1:
            Link number 1:
                  Owns  --  <inherited AG rights>  -- All Rights: gt/dl/ce/ctrl/dlrncp /vw

FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <U>::  jerry
 TO  IO:: AA:: ubu:: <files_tree>:: home:: jerry::  f_jerry.
   PATH number 1:
            Link number 1:
                  Owns  --  <inherited AG rights>  -- All Rights: gt/dl/ce/ctrl/dlrncp /vw

FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
 TO  IO:: AA:: ubu:: <G>::  jerry.
   PATH number 1:
            Link number 1:
                  Runs Under  --  (no right)

FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
 TO  IO:: AA:: ubu:: <G>::  sys.
   PATH number 1:
            Link number 1:
                  May Run Under  --  (no right)

FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: AA:: ubu:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
 TO  IO:: AA:: ubu:: <U>::  jerry.
   PATH number 1:
            Link number 1:
                  Runs Under  --  (no right)
     */

    /*
     * TEST OF THE PATHS BETWEEN <U>::  tom and f_jerry.
     * It founds 18 paths for:
     * - <U>::  tom -> f_jerry:
     *      account/member/group/contain/resource through 2 Capability groups
     *      Main result: the 2 Capability groups delivers ctrl and r rights, but
     *          does not have AGO Other rights on 'e_jerry'
     *      account/member/group/AGO Other x/actor/run_under/account or group/
     *          owner or AGO Other rights/resource for 5 groups
     *          to the common actor 'e_jerry'
     *      Main result: LinkRightsUbuntuImpl.removeSomeOtherRightsInAllLinksAGORights()
     *          removes links in the fourth case, where the links end with
     *          Actor/run_under/GroupIDMember/AGO Other/Resource
     *      Main result: LinkRightsUbuntuImpl.removeSomeOtherRightsInLastLinkAGORights()
     *          removes several links for the second case, where the links end with
     *          Actor/run_under/GroupIDMember/AGO Other/Resource
     *
     * The view should generate this SeeWhy text:
     *
FROM A RIGHT USER TO AN ACCESS TARGET
 FROM  IO:: one:: tubun:: <U>::  tom
 TO  IO:: one:: tubun:: <files_tree>:: home:: jerry::  f_jerry.
-- PATH NUMBER 1:
            Link number 1:
                   <inherited AG rights> AGO Other rights  -- All Rights: dlrncp/gt/vw
-- PATH NUMBER 2:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <CAP_DAC_OVERRIDE>
            Link number 2:
                  Contains (Capability subACS)  -- All Rights: ctrl
-- PATH NUMBER 3:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <CAP_DAC_READ_SEARCH>
            Link number 2:
                  Contains (Capability subACS)  -- All Rights: r
-- PATH NUMBER 4:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  tom
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                   <inherited AG rights> AGO Other rights  -- All Rights: dlrncp/gt/vw
-- PATH NUMBER 5:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  adm
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                   <inherited AG rights> AGO Other rights  -- All Rights: dlrncp/gt/vw
-- PATH NUMBER 6:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <AllUserAccounts>
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                   <inherited AG rights> AGO Other rights  -- All Rights: dlrncp/gt/vw
-- PATH NUMBER 7:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  users
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                   <inherited AG rights> AGO Other rights  -- All Rights: dlrncp/gt/vw
-- PATH NUMBER 8:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  admin
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                   <inherited AG rights> AGO Other rights  -- All Rights: dlrncp/gt/vw
-- PATH NUMBER 9:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  tom
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                  Is Member  -- Runs Under
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <CAP_DAC_READ_SEARCH>
            Link number 4:
                  Contains (Capability subACS)  -- All Rights: r
-- PATH NUMBER 10:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  admin
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                  Is Member  -- Runs Under
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <CAP_DAC_READ_SEARCH>
            Link number 4:
                  Contains (Capability subACS)  -- All Rights: r
-- PATH NUMBER 11:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  adm
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                  Is Member  -- Runs Under
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <CAP_DAC_READ_SEARCH>
            Link number 4:
                  Contains (Capability subACS)  -- All Rights: r
-- PATH NUMBER 12:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  users
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                  Is Member  -- Runs Under
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <CAP_DAC_READ_SEARCH>
            Link number 4:
                  Contains (Capability subACS)  -- All Rights: r
-- PATH NUMBER 13:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <AllUserAccounts>
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                  Is Member  -- Runs Under
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <CAP_DAC_READ_SEARCH>
            Link number 4:
                  Contains (Capability subACS)  -- All Rights: r
-- PATH NUMBER 14:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <AllUserAccounts>
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                  Is Member  -- Runs Under
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <CAP_DAC_OVERRIDE>
            Link number 4:
                  Contains (Capability subACS)  -- All Rights: ctrl
-- PATH NUMBER 15:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  tom
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                  Is Member  -- Runs Under
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <CAP_DAC_OVERRIDE>
            Link number 4:
                  Contains (Capability subACS)  -- All Rights: ctrl
-- PATH NUMBER 16:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  admin
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                  Is Member  -- Runs Under
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <CAP_DAC_OVERRIDE>
            Link number 4:
                  Contains (Capability subACS)  -- All Rights: ctrl
-- PATH NUMBER 17:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  adm
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                  Is Member  -- Runs Under
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <CAP_DAC_OVERRIDE>
            Link number 4:
                  Contains (Capability subACS)  -- All Rights: ctrl
-- PATH NUMBER 18:
            Link number 1:
                  Is Member
            Intermediate node:
                   IO:: one:: tubun:: <G>::  users
            Link number 2:
                   <inherited AG rights> AGO Other rights <WEAK> <Sets Group>
            Intermediate node:
                   IO:: one:: tubun:: <files_tree>:: home:: jerry:: d_jerry::  e_jerry
            Link number 3:
                  Is Member  -- Runs Under
            Intermediate node:
                   IO:: one:: tubun:: <G>::  <CAP_DAC_OVERRIDE>
            Link number 4:
                  Contains (Capability subACS)  -- All Rights: ctrl
     */

    return _result;
  } // configure()

} // AcsFactoryUbuntu
