//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui1;

import java.beans.*;
import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.Map;
import java.util.List;
import java.util.ArrayList;
import java.util.HashSet;
//import java.util.Set;

//import java.lang.reflect.InvocationTargetException;

import ARoad0.ErrW.ProcessError;
import ARoad0.ErrW.SmallError;
import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBaseInterface.ImmutableName;

/**
 * This class is responsible for displaying, by the mean of panels,
 * a modifiable property map of a gBase JavaBean, 
 * for which the type is a Map<String, BaseObject[]> or a Map<String, ImmutableName[]>.
 * The keys may be very structured strings.
 * It completes CollectionPropertyEditor and StringMapPropertyEditor.
 * CollectionPropertyEditor handles more complex types
 * but handles maps only as collections of values. StringMapPropertyEditor
 * is for all-String maps.
 * The updating is never done through a write method or a 'choiceMethodForNew' in
 * the property descriptor, but through 'add/remove' methods defined in the property
 * descriptor.
 * This class is a type of merging of CollectionPropertyEditor, for the buttons handling,
 * and its super class StringMapPropertyEditor, for the map handling.
 * To set the ImmutableNames as items in the map values, the property descriptor
 * uses the parameter 'selectedTypeInGroup' which defines the type of the BaseObject
 * to select in the explorer for adding a new item in a map value.
 * <P>
 * This class is specified by the BeanInfo classes for the property descriptors
 * of ResourceImpl and ACSFactoryImpl. It is called by Gui1.BeamerManager.addPropertyEditor().
 * @version 0.7.0
 */
public class BaseObjectMapPropertyEditor extends StringMapPropertyEditor {
 
  
  //-----------------------------------------------------------------------

  /**
   * Constructor FOR USE ONLY through the valueOf() method,
   * EMPTY_INSTANCE and the subclasses.
   */
  BaseObjectMapPropertyEditor() {
    displayName_ = "";
  }

  /**
   * Factory method to use when a BaseObjectMapPropertyEditor is delegating to us.
   * It constructs the read method for the property, from the BeanInfo information.
   * The property must be a map of Strings as keys, and a BaseObject array or
   * an ImmutableNames array as values.
   * Overrides the StringMapPropertyEditor method and calls it.
   * Called by BeamerManager.addPropertyEditor() and by RightMapPropertyEditor.
   * @param _source owning the property.
   * @param _propertyDescriptor is the descriptor of the property
   * @exception NullPointerException if _source or _displayName is null
   * @exception ProcessError if BeanInfo and PropertyDecriptor calls fail
   * @return a BaseObjectMapPropertyEditor instance
   */
  public static CommonPropertyEditor valueOf(BaseObject _source,
    PropertyDescriptor _propertyDescriptor)
    throws NullPointerException, ProcessError {

      BaseObjectMapPropertyEditor result =
        (BaseObjectMapPropertyEditor) StringMapPropertyEditor.valueOf(
          _source, _propertyDescriptor);

            // for testing purpose
             //System.out.println("BaseObjectMapPropertyEditor.valueOf() "+
               //      "after StringMapPropertyEditor.valueOf(), for: "+
                 //       result.getDisplayName());     

      if (result.propertyDescriptor_.getValue("selectedTypeInGroup") != null) {

          result.selectedTypeInGroup_ = (Class<?>)((Object[])result.propertyDescriptor_.
                  getValue("selectedTypeInGroup"))[0];
          if (!BaseObject.class.isAssignableFrom(result.selectedTypeInGroup_))
              throw new ProcessError("The 'selectedTypeInGroup' type in the BeanInfo"+
                      " is not a BaseObject");

          result.propertyTypeInGroup_ = ImmutableName.class;

      } // if

      return result;
  } // valueOf()

  // ----------------------- PUBLIC METHODS ---------------------------------

  /**
   * Manages the events fired by the buttons 'New' and 'Delete' for the map keys,
   * and 'See', 'Previous', 'Select', 'Remove' for the map values,
   * in the JPanel displayed by getCustomEditor().
   * Calls DialogBox.showExplorerSelectionDialog() and
   * addRestrictedBaseObjectValueElement().
   * Overrides the super method, and calls its methods for the buttons 'New', 'Delete',
   * 'Remove'. Called by RightMapPropertyEditor.
   * @param _evt is the event fired by a button
   */
  public void actionPerformed(ActionEvent _evt) {

                      // for testing purpose
                 //System.out.println("BaseObjectMapPropertyEditor.actionPerformed(), "+
                  //"_evt from: "+((JButton)_evt.getSource()).getText());
      
    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
      
    String buttonText = ((JButton)_evt.getSource()).getText();

    if (!_evt.getSource().getClass().equals(JButton.class)) {
      throw new InternalError("actionPerformed activated "+
        "in BaseObjectMapPropertyEditor from a component which is not a button");
    }

    // for testing purpose
    // System.out.println("actionPerformed received in BaseObjectMapPropertyEditor from "+
    //  "the button "+((JButton)_evt.getSource()).getText());

    // only for a new key, not a new value
    if ( buttonText.equals( "  New  ")) {
      addKeyElement(); 
    } // if (((JButton)
    // only for a key, not a new value
    else if ( buttonText.equals("Delete")) {
      super.actionPerformed(_evt);
    }
    // select only for a value, not for a key
    else if ( buttonText.equals("Select")) {
      // user message if no selection in the keys JList
      if ( ((JList)boundComponent_).getSelectedValues().length != 1) {
        DialogBox.showErrorMessage("Not one selected key.");
      }
      // selection through the explorer - used by RightMapPropertyEditor
      else if (limitedValuesMethod_ == null) {
          HashSet l_type = new HashSet(INITIAL_CAPACITY);
          l_type.add(selectedTypeInGroup_);

          // DialogBox.showExplorerSelectionDialog() calls handleSelectionDialog()
          try {
            DialogBox.getInstance().showExplorerSelectionDialog(this, null, l_type, null, false);
          } catch (SmallError s) {
            throw new InternalError("In BaseObjectMapPropertyEditor.actionPerformed()"+
              ", DialogBox.showExplorerSelectionDialog() throws: "+s.toString());
          } // catch
          l_type = null;
      } // if (limitedValuesMethod_ == null)
      else {
          addRestrictedValueElements((String)
                  ((JList)boundComponent_).getSelectedValues()[0]);
      } // else if (limitedValuesMethod_ == null)
    } // else if 'Select'

    else if ( buttonText.equals("Remove" )) {
      super.actionPerformed(_evt);
    } // else if ( buttonText == "Remove"

    else if ( buttonText.equals(" See ")) {
      viewElement(); // the BaseObject is shown in the beamer
    } 
    // updates the current key with the previous key values in the same map
    else if ( buttonText.equals("Prev.")) { 
        if ( ((JList)boundComponent_).getSelectedValues().length == 0) {
          DialogBox.showErrorMessage("No selection in the keys list.");
          return;
        } // if
        int selectedKeyIndex = ((JList)boundComponent_).getSelectedIndex() ;
        if (selectedKeyIndex < 1) {
            DialogBox.showErrorMessage("The current key has no previous key in this map.");
            return;
        } // if
        Map map = (Map)getValue();
        // clear the values for selectedKeyIndex
        BaseObject[] l_obj = (BaseObject[]) map.get(keysModel_.get(selectedKeyIndex));
        // updates source_ then this instance
        if (l_obj != null) removeValueElements(l_obj, false);
        // sets with the previous values
        //map = (Map)getValue();
        l_obj = (BaseObject[]) map.get(keysModel_.get(selectedKeyIndex - 1));
        if (l_obj != null) addValueElements(l_obj);
        
        map = null;
        l_obj = null;
    } // else if (((JButton) == "Prev."
    
    buttonText = null;
    return;
  } // actionPerformed()

  /**
  * Called by DialogBox.showExplorerSelectionDialog() for actionPerformed(),
  * when a value has to be selected in the explorer through the button 'Select'.
  * Since the value type is ImmutableName, this method operates the
  * the conversion. Used by the subclass RightMapPropertyEditor.
  * Call StringMapPropertyEditor.addValueElements().
  * @param _validatedObjectsSetMap with one element, where
  *   - the key is selectedTypeInGroup_,
  *   - the value is a List of one selected object.
  * May be null or empty if the user cancels the operation.
  */
  public void handleSelectionDialog(Map _validatedObjectsSetMap) {
    // stop if the argument value is null or empty
    if (_validatedObjectsSetMap == null || _validatedObjectsSetMap.isEmpty()) {
        return;
    } // if      
      
    List    list = (List)_validatedObjectsSetMap.get(selectedTypeInGroup_);
    
    if (list == null) return;

    if (selectedTypeInGroup_ != propertyTypeInGroup_) {
        List<ImmutableName> l_new = new ArrayList(INITIAL_CAPACITY);
        // replace the BaseObjects by their names
        for (Object obj : list) {
            l_new.add(((BaseObject)obj).getName());
        } // for
        list = l_new;
        l_new = null;
    } // if

    addValueElements(list.toArray());

    list = null;
  } // handleSelectionDialog()

  /**
   * Get the values of variables for the instance.
   *
   * @return source, propertyName and listeners (if they have toString() methods).
   */
  public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("  BASE_OBJECT_MAP_PROPERTY_EDITOR.TOSTRING():");

        sb.append(" source_ = ");
        sb.append(source_);

        sb.append("; display name = ");
        sb.append(displayName_);

        sb.append("; listeners list = ");
        sb.append(listeners_);

        return " [" + sb + "] ";
  } // toString()

  /**
   * Finalizes the instance; the call is reserved to BeamerManager.
   */
  public void finalizeForProcess() {
    super.finalizeForProcess();
  } // finalizeForProcess()

  // -------------------- PROTECTED METHODS -----------------------------------


  // -------------------- PRIVATE METHODS ----------------------------------- 

} // BaseObjectMapPropertyEditor


