//Title:        Access Road
//Version:      0.7.1
//Copyright:    Copyright 2002, 2004, 2007-2012 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui1;


//import java.beans.*;
//import javax.swing.*;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.Component;
import javax.swing.DefaultListModel;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JScrollPane;
import javax.swing.border.LineBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.DefaultListCellRenderer;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Array;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.SortedMap;
import java.util.HashSet;
import java.util.TreeSet;
import java.util.ArrayList;
import java.util.SortedSet;
import java.util.List;
import java.util.Map;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyDescriptor;

import ARoad0.gWork.ACS_Facade;

import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBaseInterface.ACSObject;
import ARoad0.gBaseInterface.ACS;
import ARoad0.gBaseInterface.ImmutableAclEntry;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.FinalizedObjectForUser;
import ARoad0.gBaseInterface.DetailledName;
import ARoad0.gBaseInterface.PreviousDetailledName;

import ARoad0.gBase.EligiblePartyImpl;
import ARoad0.gBase.BaseUtilityImpl;
import ARoad0.gBase.NameImpl;
import ARoad0.gBase.UtilityImpl;

import ARoad0.ErrW.SmallError;
import ARoad0.ErrW.ProcessError;
import ARoad0.ErrB.UpDateError;
import ARoad0.ErrB.CreateError;
import ARoad0.ErrB.BaseError;
import ARoad0.Gui2.TreeManager;
import ARoad0.Pattern.BeanInfoPattern;


/**
 * This class is responsible for displaying and editing, by the mean of panels,
 * one property of a gBase BaseObject for which the type is an array, a map
 * or a collection. <P>
 * It completes SimplePropertyEditor, which handles simple types,
 * RightMapPropertyEditor for StringRight maps and StringRightPropertyEditor
 * for unmodifiable String maps. It is not overridden.
 * valueOf() is the only factory method to build up an instance.
 * It is possible to get a reusable empty instance.
 * <P>
 * getCustomEditor() is used to display the property editor.
 * The updating is done through a setValue() or an adding method.
 * setAsText() throws an exception, and getAsText() returns a null value.
 * This class handles the reading method, and for the updatings, it tries
 * the 'ChoiceMethodForNew', first, 'add' and 'remove' methods, second,
 * then the set method for the property.
 * This class is returned by the XXXBeanInfo classes in gBase,
 * in their property descriptors.
 * It is used by Gui1.BeamerManager.addPropertyEditor().
 * The possible property types are String[], BaseObject[], ImmutableName[],
 * StringRight[], java.util.Collection and java.util.Map,
 * where the values of the Map or the Collection are BaseObjects, DetailledNames,
 * Strings or Names. There are functionnal restrictions when the type is a Collection:<P>
 * - StringRight is a forbidden type for the values,<P>
 * and there are also functionnal restrictions with Map:<P>
 * - only BaseObject is allowed as type for the values,<P>
 * - the map is not displayed; only the values are,<P>
 * - the 'add/select' and 'remove/delete' buttons are not workable.<P>
 * When the type is String, it cannot use the remove method, only the write one.
 * When the type is a subclass of BaseObject without being a DetailledName,
 * the displayed list contains the names of the BaseObjects.
 * This class extends the SimplePropertyEditor class which handles the types
 * Boolean, String, ARoad0.gBase.ImmutableName and ARoad0.gBase.BaseObject.
 * This class listens actions in two cases:<P>
 * - from a JButton in the JComponent when the user enters data,<P>
 * - from gBase objects (typically the source) when the program updates the property,
 * through a PropertyChangeEvent about all the property or only ONE element.<P>
 * To allow the user updating to make changes in the data base, this class instance
 * edits the base object, then these changes are fired to the class instance
 * to update the JComponent in the GUI. If the new value is an array or a collection,
 * the GUI shows always the true data in the base. Otherwise, the new value is
 * an array or a collection item, and the GUI is updated whitout any call to the base. <P>
 * After an user click on a button in the JComponent, the sequence of calls is:<P>
 * - actionPerformed() from the JButton,<P>
 * - addElements(), addOneStringElement(), addRestrictedStringElements() or
 *    removeElements(), which calls to the data base object,<P>
 * - propertyChange() from the object,<P>
 * - updating of the JList in the JComponent.<P>
 * After an InvocationTargetException, this class does not restore the true value in the editor.
 * As a PropertyEditor, it registers PropertyChangeListeners but this is
 * not recommanded so far - use rather registration on gBase PropertyChangeListeners.
 * @see ARoad0.Gui1.SimplePropertyEditor
 * @see ARoad0.Gui1.StringMapPropertyEditor
 * @version 0.7.1
 */
public class CollectionPropertyEditor extends SimplePropertyEditor
  implements AddRemoveBaseObjectListener {

  /**
   * The listModel_ of ImmutableNames for BaseObjects not being DetailledName,
   * or Strings otherwise, in the JList which displays the collection
   */
  DefaultListModel  listModel_;

  /**
   * True if the property is an array, false if it is a collection or a map
   */
  protected boolean propertyIsArray_;

  /**
   * When the property type is a Collection or a Map,
   * propertyTypeInGroup_ is set following the 'valueTypeInGroup' value in the BeanInfo.
   * Otherwise, it is set with the type of the elements in the array.
   * It does not replace propertyType_ which gives the true type of the property,
   * that is array, List, Set or Map.
   */
  protected Class propertyTypeInGroup_;

  /**
   * the scrollpane of the list
   */
  protected JScrollPane  scrollPane_;  
  
  /**
   * The add method if it exists, for the property.
   * If it is non null, the writeMethod in 'super' may be still non null.
   */
  protected Method addMethod_;

  /**
   * The remove method if it exists, for the property
   * If it is non null, the writeMethod in 'super' may be still non null.
   */
  protected Method removeMethod_;

  /**
   * The maximal size which is allowed for this collection. No use in BeanInfo so far.
   */
  protected int maxSize_ = -1;

  /**
   * A reusable empty instance for initialization,
   * to avoid the use of 'new' for temporary values.
   * Caution: never change the state of the returned instance.
   */
  public static final CollectionPropertyEditor EMPTY_INSTANCE =
    new CollectionPropertyEditor();

  //-----------------------------------------------------------------------

  /**
   * Constructor for using only through the super.valueOf() method
   * and EMPTY_INSTANCE.
   */
  CollectionPropertyEditor() {
  }

  /**
   * Factory method to use when a CollectionPropertyEditor is delegating to us.
   * It constructs the read, write, add and remove methods for the property,
   * if they exist, from the BeanInfo informations.
   * The property must be an array, a collection or a map.
   * Overridden method. Calls the super class valueOf method.
   * The remove method is selected only if there is only one method
   * with the right name, and without argument.
   * Called by BeamerManager.addPropertyEditor() and initializeDeleteButton().
   * Calls BeamerManager.getElementaryPropertyType().
   * @param _source owning the property.
   * @param _propertyDescriptor is the descriptor of the property
   * @exception NullPointerException if _source or _displayName is null
   * @exception ProcessError if BeanInfo and PropertyDecriptor calls fail
   * @return a CollectionPropertyEditor instance
   */
  public static CommonPropertyEditor valueOf(BaseObject _source,
    PropertyDescriptor _propertyDescriptor)
    throws NullPointerException, ProcessError {

      CollectionPropertyEditor result =
        (CollectionPropertyEditor) SimplePropertyEditor.valueOf(
          _source, _propertyDescriptor);

      String      addArg;
      Method[]    l_methods;

      result.propertyIsArray_ = _propertyDescriptor.getPropertyType().isArray();
      result.propertyTypeInGroup_ = BeamerManager.getElementaryPropertyType(_propertyDescriptor);      
      
      try {
            // for testing purpose
            //System.out.println("in CollectionPropertyEditor.valueOf(), "+
            // " the property type name is: "+result.propertyTypeInGroup_.getName()+", and "+
            // "the property descriptor is:"+
            // result.propertyDescriptor_.getShortDescription());

        // if there is an add method
        if (result.propertyDescriptor_.getValue("addMethod") != null) {
          addArg = (String) result.propertyDescriptor_.getValue("addMethod");

              // for testing and tracking purpose
               //System.out.println("in CollectionPropertyEditor.valueOf(), "+
               // " the first argument is: "+l_addArg[0]+
               // ", the source class is:"+result.source_.getClass());

          if (!addArg.equals(result.propertyDescriptor_.getName())) {

            //l_addClasses = getClassesForMethod(l_addArg);
            l_methods = result.source_.getClass().getMethods();
            for (int k = 0; k < l_methods.length; k++){
              //System.out.println("in CollectionPropertyEditor.valueOf(),"+
              //  " source methods: "+l_methods[k].getName());
              if (l_methods[k].getName().equals(addArg)) {
                result.addMethod_ = l_methods[k];

                    // for testing and tracking purpose
                    //System.out.println("in CollectionPropertyEditor.valueOf(),"+
                    //  " the add method for the property "+
                    //  result.propertyTypeInGroup_.getName()+" is: "+result.addMethod_
                    //  +", and the parameter types are: ");
                    //for (int m = 0; m < l_methods[k].getParameterTypes().length; m++){
                    //  System.out.println(
                    //    "For m="+m+": "+l_methods[k].getParameterTypes()[m].getName());
                    //} // for m

                break;
              } // if (l_methods[k]
            } // for k
          } // if (!addArg.equals(result.propertyDescriptor_.getName()))

          // else, l_addArg[2] equals the property name,
          // and this implies to search the add method in the BeanInfo
          // where there is also a property with the same name.
          // For instance, AclEntries is an ACS property which is updated
          // through a ResourceImpl method
          else {
                  // for testing purpose
                   //System.out.println("in CollectionPropertyEditor.valueOf(), "+
                    //" for addMethod, case where addArg equals the property name");
          }
            
          if (result.addMethod_ == null)
                throw new ProcessError("From CollectionPropertyEditor.valueOf(), "+
                        "Add method is not found");
        } // if "addMethod"

        // if there is a remove method
        if (result.propertyDescriptor_.getValue("removeMethod") != null) {
            // there is only one element into the array "removeMethod"
            addArg = (String) result.propertyDescriptor_.getValue("removeMethod");
            if (addArg.equals("finalizeForUser")) {
                // Privilege, as DetailledName, in a collection or an array, uses 'finalizeForUser',
                // but this is not true for AclEntry
              if (FinalizedObjectForUser.class.isAssignableFrom(result.propertyTypeInGroup_)) {
                l_methods = result.propertyTypeInGroup_.getMethods();
              }
              else {
                  String typeName = UtilityImpl.getFirstInstance().getClassNameFromArrayOrInterface(
                        UtilityImpl.getFirstInstance().getACSVocabularyFromArrayOrInterface(
                            result.propertyTypeInGroup_.getName()),
                        result.propertyTypeInGroup_.getName());
                  // if it is a generic interface like BaseObject, Node
                  if (typeName == null) {

                                    // for testing and tracking
                                    System.out.println("call to CollectionPropertyEditor.valueOf():");
                                    System.out.println("--------- with a generic interface type: "+
                                            result.propertyTypeInGroup_.getName());

                      typeName = result.propertyTypeInGroup_.getName(); //.substring(
                          //result.propertyTypeInGroup_.getName().indexOf("gBaseInterface.") + 15,
                          //result.propertyTypeInGroup_.getName().length() );
                  } // if
                  
                  l_methods =  Class.forName(typeName).getMethods();
                  typeName = null;
              } // else

            } // if  (l_addArg[0]
            else { // if not "finalizeForUser"
              l_methods = result.source_.getClass().getMethods();
            } // else

                  // BUG from Java:
                  //l_addClasses = getClassesForMethod(l_addArg);
                  //result.removeMethod_ = result.source_.getClass().getMethod(
                  //  l_addArg[0], l_addClasses);

            for (int k = 0; k < l_methods.length; k++){
              if (l_methods[k].getName().equals(addArg)) {
                result.removeMethod_ = l_methods[k];

                    // for testing and tracking
                    //System.out.println("in CollectionPropertyEditor.valueOf(),"+
                    //  " the remove method is: "+
                    //  result.removeMethod_+", and the parameter types are: ");
                    //for (int m = 0; m < l_methods[k].getParameterTypes().length; m++){
                    //  System.out.println(
                    //    "For m="+m+": "+l_methods[k].getParameterTypes()[m].getName());
                    //} // for m

                break;
              } // if (l_methods[k]
            } // for k
            
            if (result.removeMethod_ == null)
                throw new ProcessError("Remove method is not found");
        } // if ("removeMethod")!= null

        // if there is a maximum for the number of values
        if (result.propertyDescriptor_.getValue("MaxSize") != null) {
              result.maxSize_ = ((Integer)result.propertyDescriptor_.
                getValue("MaxSize")).intValue();

                   // for testing purpose
                   //System.out.println("in CollectionPropertyEditor.valueOf(), "+
                    //" for the property "+result.propertyDescriptor_.getName()+
                    //", there is a maximum size: "+result.maxSize_);
        } // if
        
      } catch (SecurityException s) {
        throw new InternalError("in CollectionPropertyEditor.valueOf: "+s.toString());
      } catch (Exception l) {
        //l.printStackTrace();
        new ProcessError("from CollectionPropertyEditor.valueOf() for "+result.displayName_+
                ", there is a general exception: "+l.toString());
      }

      result.listModel_ = new DefaultListModel();

            // for testing purpose
            // System.out.println("end of CollectionPropertyEditor.valueOf() for: "+
            //  result.getDisplayName());

      addArg = null;
      l_methods = null;
      return result;
  } // valueOf()

  //
  // ----------------------- PUBLIC METHODS ---------------------------------
  //

  /**
   * Gets the true values of the property, from the gBase instance,
   * through the property descriptor in the relevant BeanInfo class.
   * Caution: does not return ImmutableNames for BaseObjects. Returns
   * also the true objects when propertyTypeInGroup_ is a DetailledName.
   * Creates a ProcessError without thrown it, if there is an exception
   * fired by java.lang.reflect.Method.invoke().
   * Overriddes the super method.
   * @return the value of the property in an ArrayList (also true if an array),
   * a HashMap, a TreeSet, a TreeMap. Is an empty HashSet if there is no read method.
   */
  public Object getValue() {
	  // initialization
    Object result = null;
    Object[] returnedArray = null;

    try {
      if (readMethod_ == null ||
                readMethod_.invoke(source_, (Object[])null) == null) 
            return new HashSet(INITIAL_CAPACITY);

        // if the property is an array
      else if (propertyIsArray_) {
          result = new ArrayList(INITIAL_CAPACITY);
          returnedArray = (Object[]) readMethod_.invoke(source_, (Object[])null);
          for (int i = 0 ; i < returnedArray.length ; i++) {
            ((List)result).add(returnedArray[i]);
          }
                  // for testing and tracking
                  //System.out.println("in CollectionPropertyEditor.getValue(),"+
                  //  " the value is: "+value.toString());
      }
      else if (Collection.class.isAssignableFrom(propertyType_)) {
        if (SortedSet.class.isAssignableFrom(propertyType_))
          result = new TreeSet( (TreeSet)readMethod_.invoke(source_, (Object[])null) );
        else result = new ArrayList( (Collection)readMethod_.invoke(source_, (Object[])null) );
      }
      else if (Map.class.isAssignableFrom(propertyType_)) {
        if (SortedMap.class.isAssignableFrom(propertyType_))
          result = new TreeMap( (SortedMap)readMethod_.invoke(source_, (Object[])null) );
        else result = new HashMap( (Map)readMethod_.invoke(source_, (Object[])null) );
      }

    } catch (Exception l) {
      new ProcessError("from CollectionPropertyEditor.getValue(): "+l.toString());
    }

    returnedArray = null;
    return result;
  } // getValue()

  /**
   * Gets the property value as a string suitable for presentation
   * to a human to edit. Return null.
   * Overridden method.
   * @return null
   */
  public String getAsText() {
    return ("null");
  } // getAsText()

  /**
   * Sets the property value by parsing a given String. Raise
   * java.lang.IllegalArgumentException, because this kind of property
   * can't be expressed as text.
   * Overridden method.
   * @param _text  The string to be parsed.
   */
  public void setAsText(String _text) throws java.lang.IllegalArgumentException {
    throw new java.lang.IllegalArgumentException("setAsText(String) is not implemented"+
      " in ARoda0.Gui1.CollectionPropertyEditor");
  } // setAsText(String _text)

  /**
   * Set the object in the base through the write method.
   * No action if the write method is unknown.
   * No update of the JComponent,
   * since the object fires an event to propertyChange() when it is changed.
   * No operation if the value Class is not a java.util.Collection, when the
   * property type is an array or a Collection,
   * or if the value Class is not a Map, when Map is the property type,
   * or if there is no write method, or a value that has not the right type,
   * or if the invocation to the write method fires an exception - then,
   * a ProcessError is created but not thrown, since the method signature
   * does not allow it.
   * Called by handleSelectionUpdate() and, for the button "Prev.", by actionPerformed().
   * Overriddes the super method.
   * @param _value The java.util.Collection or Map to be edited.
   *     This object is not modified by the PropertyEditor.
   */
  public void setValue(Object _value) {

    if (!setValueControl(_value)) return;

    Object value;
    Object[] l_argument = null;

    try {
      if (propertyIsArray_) { // if the property is an array
        if (((Collection)_value).size() != 0) {
          // uses propertyTypeInGroup_ to create an array of the right class
          Object[] l_propertyTypeArray = null;
          if (!DetailledName.class.isAssignableFrom(propertyTypeInGroup_)) {
              l_propertyTypeArray = (Object[])
                    Array.newInstance(propertyTypeInGroup_, ((Collection)_value).size());
          }
          else {
                throw new InternalError("CollectionPropertyEditor.setValue() is called "+
                        "for a DetailledName array, while any 'write' method is forbidden for it.");
          }

          // for testing purpose
           //System.out.println("in CollectionPropertyEditor.setValue(), "+
            //"the property type of l_propertyTypeArray is: "+l_propertyTypeArray.getClass());

          ((Collection)_value).toArray( l_propertyTypeArray );

          if (propertyDescriptor_.getValue("callerIndexInWriteArgument") != null) { // no use
            if (propertyDescriptor_.getValue("callerIndexInWriteArgument").equals(
              new Integer(1)) )
                l_argument = new Object[] {l_propertyTypeArray, this};

            else if (propertyDescriptor_.getValue("callerIndexInWriteArgument").equals(
              new Integer(0)) )
                l_argument = new Object[] {this, l_propertyTypeArray};
          }
          else l_argument = new Object[] {l_propertyTypeArray};

          // l_propertyTypeArray is an argument for having _value in the right format.
          // The argument is in an array.
          writeMethod_.invoke(source_, l_argument);

        } else {
          // _value is empty, so updates with an empty array
          Object[] l_propertyTypeArray = (Object[]) Array.newInstance(propertyTypeInGroup_, 0); // old dimension: 1

          if (propertyDescriptor_.getValue("callerIndexInWriteArgument") != null) { // no use
            if (propertyDescriptor_.getValue("callerIndexInWriteArgument").equals(
              new Integer(1)) )
                l_argument = new Object[] {l_propertyTypeArray, this};

            else if (propertyDescriptor_.getValue("callerIndexInWriteArgument").equals(
              new Integer(0)) )
                l_argument = new Object[] {this, l_propertyTypeArray};
          }
          else l_argument = new Object[] {  l_propertyTypeArray };

          writeMethod_.invoke(source_, l_argument);

        } //else
      } // if (propertyIsArray_)
      // the property is a collection or a map
      else {
        if ( Collection.class.isAssignableFrom(propertyType_)) {
          // construct the writeMethod_ argument - which is alone
          // with the right type (Collection, Set, List, SortedSet,...)
          value = (Collection) propertyType_.newInstance();
          if (propertyDescriptor_.getValue("callerIndexInWriteArgument") == null) {
            if (((Collection)value).addAll((Collection)_value))
              writeMethod_.invoke(source_, new Object[] {value});
            else throw new ProcessError("in CollectionPropertyEditor.setValue(), "+
              "the addAll method is not usable for the collection.");
          } // if (propertyDescriptor_
          else {
            if (propertyDescriptor_.getValue("callerIndexInWriteArgument").equals(
              new Integer(1)) ) {
                if (((Collection)value).addAll((Collection)_value))
                  writeMethod_.invoke(source_, new Object[] {value, this});
                else throw new ProcessError("in CollectionPropertyEditor.setValue(), "+
                  "the addAll method is not usable for the collection.");
            }
            else if (propertyDescriptor_.getValue("callerIndexInWriteArgument").equals(
              new Integer(0)) ) {
                if (((Collection)value).addAll((Collection)_value))
                  writeMethod_.invoke(source_, new Object[] {this, value});
                else throw new ProcessError("in CollectionPropertyEditor.setValue(), "+
                  "the addAll method is not ok for the collection.");
            }
          } // else
        } // if ( Collection.isAssignableFrom(propertyType_))
        else if (Map.class.isAssignableFrom(propertyType_)) {
          // construct the writeMethod_ argument - that is alone -
          // with the right type (Map, SortedMap,...)
          value = (Map) propertyType_.newInstance();
          ((Map)value).putAll((Map)_value);
          if (propertyDescriptor_.getValue("callerIndexInWriteArgument") == null)
            writeMethod_.invoke(source_, new Object[] {value});
          else {
            if (propertyDescriptor_.getValue("callerIndexInWriteArgument").equals(
              new Integer(1)) ) {
                writeMethod_.invoke(source_, new Object[] {value, this});
            }
            else if (propertyDescriptor_.getValue("callerIndexInWriteArgument").equals(
              new Integer(0)) ) {
                writeMethod_.invoke(source_, new Object[] {this, value});
            }
          } // else
        } // if (Map.isAssignableFrom(propertyType_))
      } // if (!propertyIsArray_)

    } catch (InvocationTargetException i) {
      i.printStackTrace();

      // shows the message to the user
      int lastIndex = ((InvocationTargetException)i).getTargetException().toString().lastIndexOf(": ");
      if (lastIndex != 0)
        DialogBox.showErrorMessage(
          ((InvocationTargetException)i).getTargetException().toString().substring(lastIndex+1));
      else DialogBox.showErrorMessage(
        ((InvocationTargetException)i).getTargetException().toString()); 
      
      // CreateError is for the user errors
      //if (!(i.getTargetException() instanceof CreateError)) {
        //  System.out.println("in CollectionPropertyEditor.setValue(), "+
          //  "the exception is an InvocationTargetException: ");
          //i.getCause().printStackTrace();
      //}
      
      // not restore the true value in the editor
      //propertyChange(new PropertyChangeEvent(source_, propertyDescriptor_.getName(), null, null));
      
      return;
    } catch (Exception e) {
        if (e instanceof CreateError || e instanceof UpDateError || e instanceof BaseError) {
          DialogBox.showErrorMessage(e.getCause().getMessage());
          //System.out.println("in CollectionPropertyEditor.setValue(), exception: ");
          e.getCause().printStackTrace();
          return;
        }

        new ProcessError("in CollectionPropertyEditor.setValue(), exception: "+e.toString());
        e.printStackTrace();
        return;
    } // catch

    value = null;
    l_argument = null;
  } // setValue(Object value)

  // ----------------------------------------------------------------------

  /**
   * Overridden method.
   * @return null.
   */
  public String[] getTags() {
    return null;
  } // getTags()

  /**
   * Makes a full custom JPanel that edits the property value.
   * A null or empty collection produces an empty panel. 
   * The possible property types are Object[], java.util.Collection, java.util.Map.
   * Calls getCollectionEditor().
   * @return a JPanel that will allow an user to directly edit the current property value.
   */
  public java.awt.Component getCustomEditor() {

                          // for testing and tracking         
                    //System.out.println("in CollectionPropertyEditor.getCustomEditor(),"+
                      //  " source_ is: "+source_+"; readMethod_ is: "+readMethod_);

    
    try {
      Object obj = readMethod_.invoke(source_, (Object[])null);

      if (obj == null && writeMethod_ == null && addMethod_ == null && removeMethod_ == null) {

                  // for testing purpose
                  //System.out.println("in CollectionPropertyEditor.getCustomEditor') "+
                    //"return an empty panel, since there is a null read method ");
          
            initializeListModel(); 
      } // if
      else if (obj instanceof Collection && ((Collection)obj).isEmpty() &&
              writeMethod_ == null && addMethod_ == null && removeMethod_ == null) {

                  // for testing purpose
                  //System.out.println("in CollectionPropertyEditor.getCustomEditor') "+
                    //"return an empty panel, since there is an empty read method ");

            initializeListModel(); 
      } // else if

      obj = null;
    } catch (InvocationTargetException j) {
      j.printStackTrace();
      new ProcessError("from CollectionPropertyEditor.getCustomEditor(): "+
        j.getCause());
      return null;
    } catch (Exception l) {
      l.printStackTrace();
      new ProcessError("from CollectionPropertyEditor.getCustomEditor(): "+
        l.toString());
      return null;
    }

    return getCollectionEditor();
  } // getCustomEditor()

  /**
   * Get the prefered size for the rectangle into which the value is displayed,
   * in the component returned by getCustomEditor().
   * Overridden method.
   * @return the prefered size.
   */
  public Dimension getPreferedSizeForEditing() {
    return new Dimension(50, 20);
  } // getPreferedSizeForEditing()
  
  /**
   * Manages the events fired by the buttons 'See', 'New', 'Select', 'Remove', 'Delete', 
   * 'Previous' in the JPanel displayed in the beamer. These buttons are returned
   * by getCustomEditor(). The bound component fires no event.
   * The 'Previous' button adds the values of the previous list in the beamer, if any,
   * to the current values.
   * Overridden method.
   */
  public void actionPerformed(ActionEvent _evt) {
 
    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
      
    String buttonText = ((JButton)_evt.getSource()).getText();

    if (!_evt.getSource().getClass().equals(JButton.class)) {
      throw new InternalError("actionPerformed received "+
        "in CollectionPropertyEditor from a component which is not a button");
    }

    // for testing purpose
    // System.out.println("actionPerformed received in CollectionPropertyEditor from "+
    //  "the button "+((JButton)_evt.getSource()).getText());

    if ( buttonText.equals( "  New  ")) {
      addElement(); 
    } // if (((JButton)

    else if ( buttonText.equals("Select")) {
        
        // if restricted argument, display a list of possible objects.
        // These are not Strings (processed through 'New' button), so
        // SimplePropertyEditor.getRestrictedValues() is not used there.
      if ( isRestrictedArgument_ && source_ instanceof ACSObject) {

          ACS acs = (ACS) ACS_Facade.getOpenACS().get(((ACSObject)source_).getAcsName());
          Object[] l_ToBeSelectedValues = null;
          if (source_ instanceof ImmutableAclEntry && 
                  propertyDescriptor_.getName().equals("AclRights")) // for get Resource.AclRights, etc
              l_ToBeSelectedValues = BaseUtilityImpl.getRestrictedValues(
                                                sourceBeanInfo_,
                                                propertyDescriptor_.getName(),
                                                acs,
                                                ((ImmutableAclEntry)source_).getEorResource(),
                                                ((ImmutableAclEntry)source_).getEorEP().getEPType());
          else l_ToBeSelectedValues = BaseUtilityImpl.getRestrictedValues(
                                                sourceBeanInfo_,
                                                propertyDescriptor_.getName(),
                                                acs,
                                                source_,
                                                null);
             
          if (l_ToBeSelectedValues == null) {
              DialogBox.showOkMessage("For this property and this object, the ACS sets no value."+
                      " For more information, you should look at the specialized rights of the ACS.");
              acs = null;
              l_ToBeSelectedValues = null;
              return;
          }
         
          // remove the current values
          List                    l_currentValues = null;
          List                    l_values = new ArrayList(INITIAL_CAPACITY);
          List                    l_selectedValues = new ArrayList(
                                            Arrays.asList(l_ToBeSelectedValues)); // temporary use

                  // for testing purpose
                  //System.out.println("CollectionPropertyEditor.actionPerformed(): "+
                    //"after a click on 'Select', allowed values: "+l_selectedValues);

          // get the true objects from gBase
          if (getValue() instanceof List) l_currentValues = (List) getValue();
          else if (getValue() instanceof Map)
            l_currentValues = new ArrayList(((Map)getValue()).values());
          else {
            new ProcessError("in CollectionPropertyEditor.actionPerformed(), "+
                "getValue() does not return a list or a map.");
          }           

          l_selectedValues.removeAll(l_currentValues);
          if (l_selectedValues.isEmpty()) {
              acs = null;
              l_ToBeSelectedValues = null;
              l_currentValues = null;
              l_selectedValues = null;
              DialogBox.showOkMessage("For this property and this object, all the "+
                      "allowed values are already selected."+
                      " For more information, you should look at the specialized rights of the ACS.");
              return;
          } // if

                  // for testing purpose
                  //System.out.println("CollectionPropertyEditor.actionPerformed(): "+
                    //"after a click on 'Select', values to select: "+l_selectedValues);
          
          l_ToBeSelectedValues = l_selectedValues.toArray();
          
          // request to the user to select
          JPanel                  panel = DialogBox.showSelectionsInOneDialog(
                                            displayName_, l_ToBeSelectedValues);
          java.awt.Component[]    l_boxes = panel.getComponents();
        
          // get all the selected Objects
          for (int i = 0 ; i < l_boxes.length ; i++) {
            if ( ((JCheckBox)l_boxes[i]).isSelected() )
                l_values.add( l_ToBeSelectedValues[i] );
          } // for i 
                  
          // update the property
          handleSelectionUpdate(l_values);
          
          acs = null;
          l_ToBeSelectedValues = null;
          l_selectedValues = null;
          l_values = null;
          l_boxes = null;        
          panel = null;
          
      } else {  // if !isRestrictedArgument_
          HashSet l_type = new HashSet(INITIAL_CAPACITY);
          if (source_ instanceof ACSObject)
              l_type.add( BeamerManager.getACSAllowedType(
                      (ACSObject)source_, propertyTypeInGroup_));
          else l_type.add(propertyTypeInGroup_);


                          // for testing purpose
                          //System.out.println("CollectionPropertyEditor.actionPerformed(), "+
                            //"after a click on 'Select', the allowed types are : ");
                          //for (Object obj : l_type) {
                            //  System.out.println("---- "+obj);
                          //}
          
          // DialogBox.showExplorerSelectionDialog() calls handleSelectionDialog()
          try {
                // addMethod is generally for only one selection
                if (addMethod_ != null)
                    DialogBox.getInstance().showExplorerSelectionDialog(
                        this, null, l_type, null, true);
                else DialogBox.getInstance().showExplorerSelectionDialog(
                        this, null, l_type, null, false);
          } catch (SmallError s) {
            throw new InternalError("In CollectionPropertyEditor.actionPerformed()"+
              ", DialogBox.showExplorerSelectionDialog() throws: "+s.toString());
          } // catch

          l_type = null;
      } // else
    } // else if (((JButton)

    else if ( buttonText.equals("Delete")) {

                  // for testing purpose
                  //System.out.println("actionPerformed() in CollectionPropertyEditor:"+
                   // "selection: "+((JList)boundComponent_).getSelectedValues().length);


      // user message if no selection in the beamer JList
      if ( ((JList)boundComponent_).getSelectedValues().length == 0) {
        DialogBox.showErrorMessage("No selection for deleting.");
      }
      else removeElements( ((JList)boundComponent_).getSelectedValues(), "delete" );
    }

    else if ( buttonText.equals("Remove" )) {
      // user message if no selection in the beamer JList
      if ( ((JList)boundComponent_).getSelectedValues().length == 0) {
        DialogBox.showErrorMessage("No selection for removing.");
      }
      else removeElements( ((JList)boundComponent_).getSelectedValues(), "remove" );
    } // else if ( buttonText == "Remove"

    else if ( buttonText.equals(" See ")) {
      viewElement();
    } // else if (((JButton) == "See"

    else if ( buttonText.equals("Prev.")) {
      // init
      BeanInfoPattern beanInfo = UtilityImpl.getFirstInstance().selectBeanInfo(source_);
      PropertyDescriptor[] l_descriptors = beanInfo.getPropertyDescriptors();
      int previous = -1;
      Object[] l_previousValues = null;
      List result = (List)getValue();

            // for testing and tracking
            //System.out.println("in CollectionPropertyEditor.actionPerformed(), "+
              //          "for 'Previous', displayName_ is: "+displayName_);
      
      for (int i = 0; i < l_descriptors.length; i++) {
            // for testing and tracking
            //System.out.println("in CollectionPropertyEditor.actionPerformed(), "+
              //          "for 'Previous', for i ="+i+", l_descriptors[i].getDisplayName(): "+
                //        l_descriptors[i].getDisplayName());
          
          if (displayName_.equals(l_descriptors[i].getDisplayName()) && i > 1) {
              previous = i - 1;
              break;
          }
      } // for i
      if (previous != -1 && 
        l_descriptors[previous].getValue( "isCollection" ) == Boolean.TRUE &&
        l_descriptors[previous].getPropertyType().getComponentType() == 
            l_descriptors[previous + 1].getPropertyType().getComponentType()) {

            try {
              l_previousValues = (Object[]) l_descriptors[previous].
                getReadMethod().invoke(source_, (Object[])null );
            } catch (InvocationTargetException i) {
                  System.out.println("in CollectionPropertyEditor.actionPerformed(), "+
                        "for 'Previous', exception: "+
                        i.getCause().toString());
            } catch (Exception e) {
                  throw new InternalError(e.toString() );
            } // catch

                    // for testing and tracking
                    //System.out.println("in CollectionPropertyEditor.actionPerformed(), "+
                    //          "for 'Previous', the new values are: ");
            for (int i = 0 ; i < l_previousValues.length ; i++) {
                result.add(l_previousValues[i]);
                    //    System.out.println("    for i ="+i+", "+l_previousValues[i]);
            } // for i
            setValue(result);
            
      } // if
      beanInfo = null;
      l_descriptors = null;
      l_previousValues = null;
      result = null;
    } // else if (((JButton) == "Prev."
    
    buttonText = null;
  } // actionPerformed()

  /**
   * Manages the events fired by the base object source_ displayed in the beamer,
   * if it is a bound property and when the property value has changed.
   * Handles changes on all the collection, map and array, as a total change or
   * only for one item if the value in _evt is a single item.
   * This is the single method that updates the value this property editor displays.
   * When the property is a map, the map values are displayed, and not the map keys.
   * The main behavior is that the old and new property values in the argument
   * may be ridden to update the values. This is not like StringRightMapEditor.
   * A DetailledName is displayed through its detailled name, and the PreviousDetailledName
   * interface is also tested to find the item to remove.
   * Synchronized on listModel_. By default, the displayed list is sorted using
   * the default sorting of the list. The PropertyDescriptor value 'withoutListSorting'
   * is used to forbid the sorting.
   * Overrides the super method.
   * @param _evt with a new value which may be the new property value
   *  (collection or array), and then the old value is not used, OR,
   *  when the change is about only one element in the collection, map or the array,
   *  it may be only ONE value to add or to remove in the collection or the array.
   */
  public void propertyChange(PropertyChangeEvent _evt) {

    if (_evt.getSource() != source_ ||
      !_evt.getPropertyName().equals(propertyDescriptor_.getName()) ||
      boundComponent_ == null) return;

                        // for testing purpose
                        //System.out.println("in CollectionPropertyEditor, "+
                          //"PropertyChangeEvent received for "+
                          //_evt.getPropertyName()+" from the BaseObject "+
                          //((BaseObject)_evt.getSource()).getName());
                        
    synchronized (listModel_) {
      if (propertyIsArray_) {
        if (_evt.getNewValue() == null) {

                    // for testing purpose
                    //System.out.println("in CollectionPropertyEditor."+
                    //  "PropertyChangeEvent(), there is an element to remove in the list.");

            if (_evt.getOldValue().getClass().isArray()) listModel_.clear();
            else {
              if (DetailledName.class.isAssignableFrom(propertyTypeInGroup_))
                listModel_.removeElement(((DetailledName)_evt.getOldValue()).getDetailledName());
              else if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_) &&
                ((BaseObject)_evt.getOldValue()).getName() != null)
                  listModel_.removeElement(((BaseObject)_evt.getOldValue()).getName());
              else if (propertyTypeInGroup_ == String.class)
                listModel_.removeElement(_evt.getOldValue());
              else if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_))
                listModel_.removeElement(_evt.getOldValue());
            } // else

        }
        // if (_evt.getNewValue() != null
        else if (_evt.getNewValue().getClass().isArray()) {
           Object[] l_newValue = (Object[])_evt.getNewValue();
           
                    // for testing purpose
                    //System.out.println("in CollectionPropertyEditor."+
                      //"PropertyChangeEvent(), l_newValue: ");
                    //for (int i = 0; i < l_newValue.length; i++) {
                      //  System.out.println("-------- "+l_newValue[i]);
                    //} // for i
            
          listModel_.clear();

          if (DetailledName.class.isAssignableFrom(propertyTypeInGroup_)) {
              if (l_newValue.length != 0 &&
                    (propertyDescriptor_.getValue("withoutListSorting") == null ||
                    !(Boolean)propertyDescriptor_.getValue("withoutListSorting")))
                Arrays.sort(l_newValue);

              for (int i = 0 ; i < l_newValue.length ; i++) {
                if (l_newValue[i] != null)
                    listModel_.addElement( ((DetailledName)l_newValue[i]).getDetailledName() );
              } // for i
          } // if
          else if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_)) {
            if (l_newValue.length != 0 && 
                    (propertyDescriptor_.getValue("withoutListSorting") == null ||
                    !(Boolean)propertyDescriptor_.getValue("withoutListSorting")))
                Arrays.sort(l_newValue);

            for (int i = 0 ; i < l_newValue.length ; i++) {
              if (l_newValue[i] != null && ((BaseObject)l_newValue[i]).getName() != null) {
                listModel_.addElement( ((BaseObject)l_newValue[i]).getName());
 
                    // for testing purpose
                    //System.out.println("in CollectionPropertyEditor."+
                      //"PropertyChangeEvent(), new element in the list: "+
                      //((BaseObject)l_newValue[i]).getName());               
                
                
              } // if
            } // for i
          } // if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_))
          else if (propertyTypeInGroup_ == String.class) {
              if (l_newValue.length != 0 &&
                    (propertyDescriptor_.getValue("withoutListSorting") == null ||
                    !(Boolean)propertyDescriptor_.getValue("withoutListSorting")))
                Arrays.sort(l_newValue);

              for (int i = 0 ; i < l_newValue.length ; i++) {
                if (l_newValue[i] != null) listModel_.addElement(l_newValue[i]);
              } // for i
          } // if
          else if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_)) {
              if (l_newValue.length != 0 &&
                    (propertyDescriptor_.getValue("withoutListSorting") == null ||
                    !(Boolean)propertyDescriptor_.getValue("withoutListSorting")))
                Arrays.sort(l_newValue);

              for (int i = 0 ; i < l_newValue.length ; i++) {
                if (l_newValue[i] != null) listModel_.addElement(l_newValue[i]);
              } // for i
          } // if
        } // else the new value is not an array but one value to add to the current array
        else {

          if (DetailledName.class.isAssignableFrom(propertyTypeInGroup_)) {
                listModel_.addElement( ((DetailledName)_evt.getNewValue()).getDetailledName());
          } // if (detailledName.class.isAssignableFrom(propertyTypeInGroup_))
          else if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_)) {
                listModel_.addElement( ((BaseObject)_evt.getNewValue()).getName());
          } // if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_))
          else if (propertyTypeInGroup_ == String.class) {
                listModel_.addElement( (String)_evt.getNewValue() );
          } // if
          else if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_)) {
                listModel_.addElement( (ImmutableName)_evt.getNewValue() );
          } // if
          
        } // else the new value is one value to add to the current array
      } // if (propertyIsArray_)
      else {

        if (_evt.getNewValue() == null) {

            if (Collection.class.isAssignableFrom(_evt.getOldValue().getClass())
              || Map.class.isAssignableFrom(_evt.getOldValue().getClass()))
                listModel_.clear();
            else if (DetailledName.class.isAssignableFrom(propertyTypeInGroup_)) {

                  boolean success = false;

                  // the previous detailled name may be registered or not.
                  // No registered from a finilizing privilege in a source or a target.
                  if (_evt.getOldValue() instanceof PreviousDetailledName) {
                      success = listModel_.removeElement(((PreviousDetailledName)_evt.getOldValue()).
                              getPreviousDetailledName());
                      if (!success)
                          success = listModel_.removeElement(((DetailledName)_evt.getOldValue()).
                              getDetailledName());
                  }
                  else {
                      success = listModel_.removeElement(((DetailledName)_evt.getOldValue()).
                              getDetailledName());
                  }

                  if (!success && _evt.getOldValue() instanceof PreviousDetailledName)
                      throw new InternalError("From CollectionPropertyEditor."+
                        "PropertyChangeEvent(), no removing of the privilege "+
                        _evt.getOldValue()+" where the previousDetailledName is "+
                        ((PreviousDetailledName)_evt.getOldValue()).getPreviousDetailledName()+
                        " and the DetailledName is "+
                        ((PreviousDetailledName)_evt.getOldValue()).getDetailledName());

            }
            else if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_))
              listModel_.removeElement(((BaseObject)_evt.getOldValue()).getName());
            else if (propertyTypeInGroup_ == String.class) 
              listModel_.removeElement(_evt.getOldValue());
            else if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_))
              listModel_.removeElement(_evt.getOldValue());

        } // if (_evt.getNewValue() == null) {
        else {
          if (_evt.getNewValue() instanceof Collection) {
              listModel_.clear();
              for (Iterator ite = ((Collection)_evt.getNewValue()).iterator() ; ite.hasNext() ; ) {

                if (DetailledName.class.isAssignableFrom(propertyTypeInGroup_))
                  listModel_.addElement(((DetailledName)ite.next()).getDetailledName());
                else if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_))
                  listModel_.addElement(((BaseObject)ite.next()).getName());
                else if (propertyTypeInGroup_ == String.class)
                  listModel_.addElement(ite.next());
                else if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_))
                  listModel_.addElement(ite.next());

              } // for
          }
          else if (_evt.getNewValue() instanceof Map) {
              listModel_.clear();
              for (Iterator ite = ((Map)_evt.getNewValue()).values().iterator() ; ite.hasNext() ; ) {

                if (DetailledName.class.isAssignableFrom(propertyTypeInGroup_))
                    listModel_.addElement(((DetailledName)ite.next()).getDetailledName());
                else if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_))
                    listModel_.addElement(((BaseObject)ite.next()).getName());
                else if (propertyTypeInGroup_ == String.class)
                  listModel_.addElement(ite.next());
                else if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_))
                  listModel_.addElement(ite.next());

              } // for

                    // for testing purpose
                    //System.out.println("in CollectionPropertyEditor, "+
                    //  "PropertyChangeEvent received for a map. The size is: "+
                    //  ((Map)_evt.getNewValue()).values().size());

          } // else if

          // the event contains only one element to add
          else if (DetailledName.class.isAssignableFrom(propertyTypeInGroup_) &&
            ((DetailledName)_evt.getNewValue()).getDetailledName() != null)
              listModel_.addElement(((DetailledName)_evt.getNewValue()).getDetailledName());
          else if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_) &&
            ((BaseObject)_evt.getNewValue()).getName() != null)
              listModel_.addElement(((BaseObject)_evt.getNewValue()).getName());
          else if (String.class.isAssignableFrom(propertyTypeInGroup_))
            listModel_.addElement(_evt.getNewValue());
          else if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_))
            listModel_.addElement(_evt.getNewValue());
        } // else

      } // else (!propertyIsArray_)
    } // synchro

    // reset the panel dimension, like into getCollectionEditor()
    JPanel  buttonPanel = setVerticalButtonsInEditor(getButtonsDisplaying(), false, false);
    boolean displayTheName = true;
    if ("".equals(getDisplayName())) displayTheName = false;

    if (buttonPanel != null) {
        if (displayTheName) {
            setListPanelDimension(listModel_.getSize(), scrollPane_, true, true, true);
        }
        else {
            setListPanelDimension(listModel_.getSize(), scrollPane_, false, false, true);
        }
    }
    else {
        if (displayTheName) {
            setListPanelDimension(listModel_.getSize(), scrollPane_,
                    true, false, true);
        } else { // used for instance by the ACS 'ImmutableRules' property
            setListPanelDimension(listModel_.getSize(), scrollPane_,
                    true, false, false);
        } // else
    } // else

    buttonPanel = null;
  } // propertyChange()

  /**
  * Called by BaseObjectHandler for addElement(),
  * to add a BaseObject by the mean of the calling to a specific method
  * on the right instance in the data base.
  * On the contrary of handleNewBaseObject(), the object is not already created
  * when this method is called, and this method makes the job with a call to
  * the addMethod_. The adding produces a backing call to propertyChange().
  * @param _l_obj is the BaseObject instance arguments to use
  *   when the 'add' method is invoked
  * @exception ProcessError if an InvocationTargetException is thrown
  */
  public void addOneBaseObject(Object[] _l_obj) throws ProcessError {

                  // for testing purpose
             //System.out.println("in CollectionPropertyEditor.addOneBaseObject(), "+
              //"the arguments are: ");
              //for (int i = 0; i < _l_obj.length; i++) {
                //System.out.println("      for "+i+": "+_l_obj[i].toString());
              //}
             //System.out.println("and the add method is: "+addMethod_);
             
    int currentSize = 0;
    if (listModel_ != null) currentSize = listModel_.size();
    if (maxSize_ > 0 && maxSize_ < currentSize + 1) {
      DialogBox.showErrorMessage("No operation. "+
        maxSize_+" is the maximum size for this collection.");
      return;
    }

    // variables
    Object result = null;

    try {
      result = (Object) addMethod_.invoke(source_, _l_obj );
    } catch (InvocationTargetException i) {
          if (i.getCause().getMessage() != null)
              DialogBox.showErrorMessage(i.getCause().getMessage());
          System.out.println("in CollectionPropertyEditor.addOneBaseObject(), exception: ");
          i.getCause().printStackTrace();
          result = null;
          return;
    } catch (Exception e) {
        if (e instanceof CreateError || e instanceof UpDateError || e instanceof BaseError) {
          DialogBox.showErrorMessage(e.getCause().getMessage());
          System.out.println("in CollectionPropertyEditor.addOneBaseObject(), exception: ");
          e.getCause().printStackTrace();
          result = null;
          return;
        }

        System.out.println("in CollectionPropertyEditor.addOneBaseObject(), exception: "+e);
        throw new ProcessError(e.toString() );
    } // catch

    // changes the selection in the explorer
    // but result may be null if it is an object selection through 'Select'
    if (result == null) return;
    TreeManager.selectNodeInExplorer((BaseObject)result);
    result = null;
  } // addOneBaseObject()

  /**
  * NO USE.
  * Called by BaseObjectHandler for actionPerformed(),
  * to remove a BaseObject in calling a specific method.
  * <P>
  * @param _l_obj is the BaseObject instance arguments to use
  *   when the 'remove' method is invoked. May be null.
  * @exception ProcessError if an InvocationTargetException is thrown
  */
  public void removeOneBaseObject(Object[] _l_obj) throws ProcessError {

    try {
      removeMethod_.invoke(source_, _l_obj );
    } catch (Exception i) {
      i.printStackTrace();
      new ProcessError("in CollectionPropertyEditor.removeOneBaseObject(), "+
        "Exception ");
    }
  } // handleRemoveBaseObject()

    //----------------------------------------------------------------------

  /**
   * Get the values of variables for the instance.
   *
   * @return source, propertyName and listeners (if they have toString() methods).
   */
  public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("  CollectionPropertyEditor.TOSTRING():");

        sb.append(" source_ = ");
        sb.append(getSource());

        sb.append("; display name = ");
        sb.append(getDisplayName());

        //sb.append("; listeners list = ");
        //sb.append(listeners_);

        return " [" + sb + "] ";
  } // toString()

  /**
  * Called by DialogBox.showExplorerSelectionDialog() for actionPerformed(),
  * when the property is a Collection, a Map or an array,
  * and the property type is a BaseObject
  * which has to be selected in the explorer through the button 'Select'.
  * Call handleSelectionUpdate().
  * Caution: it is an overridden method.
  * @param _validatedObjectsSetMap with one element, where
  *   - the key is propertyTypeInGroup_,
  *   - the value is a List of one selected object.
  * May be null or empty if the user cancels the operation.
  */
  public void handleSelectionDialog(Map _validatedObjectsSetMap) {
    // stop if the argument value is null or empty
    if (_validatedObjectsSetMap == null || _validatedObjectsSetMap.isEmpty()) {
        return;
    } // if      
      
    List    list = null;
    // check, as in actionPerformed(), the allowed types
    if (source_ instanceof ACSObject)
      list = (List)_validatedObjectsSetMap.get(
               BeamerManager.getACSAllowedType((ACSObject)source_, propertyTypeInGroup_));
    else list = (List)_validatedObjectsSetMap.get(propertyTypeInGroup_);
    
    if (list == null) return;

    handleSelectionUpdate(list);

    list = null;
  } // handleSelectionDialog()
  
  /**
  * Called by BaseObjectHandler for addElement(),
  * when there is a new BaseObject instance which has been created by BaseObjectHandler,
  * and which has to be added to the collection property.
  * No operation since the new object creation fires an exception.
  * No control of maxSize_ and value, since it is not an object selection
  * in the explorer.
  * See addOneBaseObject() when this class creates the new object,
  * and BaseObjectHandler is just the arguments producer.
  * Overriddes the super method.
  * Caution: the name of _obj may has been changed by gBase at its creation.
  * @param _obj is the BaseObject instance to add to this collection
  */
  public void handleNewBaseObject(Object _obj) {
    //List result = (List) getValue();
    //result.add(_obj);
    //setValue( result );
  } // handleNewBaseObject()

  // -------------------- PROTECTED METHODS -----------------------------------

  /**
   * Finalize the instance; the call is reserved to BeamerManager.
   * Overridden method.
   */
  public void finalizeForProcess() {
    propertyType_ = null;
    addMethod_ = null;
    removeMethod_ = null;
    listModel_ = null;
    scrollPane_ = null;
    
    super.finalizeForProcess();
  } // finalizeForProcess()

  //
  // -------------------- PRIVATE METHODS -----------------------------------
  //

  /**
   * Called by setValue() to control the parameter and the global variables.
   * @param _value is the parameter of the method setValue()
   * @return true if all is ok
   */
  private boolean setValueControl(Object _value) {

    // Initial tests
    if (writeMethod_ == null) return false;
    boolean error = false;
    Object object;
    if (!(_value instanceof Collection)) {
      new ProcessError("From CollectionPropertyEditor.setValue(),"+
        " no operation: the parameter is not a Collection.");
      error = true;
    }

    // test if propertyType_ is a map
    else if (Map.class.isAssignableFrom(propertyType_)) {
      if (!(_value instanceof Map)) {
        new ProcessError("From CollectionPropertyEditor.setValue(),"+
          " no operation: the parameter is not a Map.");
        error = true;
      } // if (!(_value instanceof Map))
      // control each value
      if (propertyTypeInGroup_ == String.class) {
        for (Iterator ite = ((Map)_value).values().iterator() ; ite.hasNext() ; ) {
          object = ite.next();
          if (object != null && !(object instanceof String)) {
            new ProcessError("From CollectionPropertyEditor.setValue(),"+
              " no operation: a value in the Map is not a String.");
            error = true;
            break;
          } // if (!(_value instanceof String))
        } // for
      } // if (propertyTypeInGroup_ == String.class
      else if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_)) {
        for (Iterator ite = ((Map)_value).values().iterator() ; ite.hasNext() ; ) {
          object = ite.next();
          if (object != null && !(object instanceof BaseObject)) {
            new ProcessError("From CollectionPropertyEditor.setValue(),"+
              " no operation: a value in the Map is not a BaseObject.");
            error = true;
            break;
          } // if (!(_value instanceof String))
        } // for
      } // if (propertyTypeInGroup_ == BaseObject
    } // else if (Map.class.

    // test if propertyType_ is a collection
    else if (Collection.class.isAssignableFrom(propertyType_)) {
      // control each value
      if (propertyTypeInGroup_ == String.class) {
        for (Iterator ite = ((Collection)_value).iterator() ; ite.hasNext() ; ) {
          object = ite.next();
          if (object != null && !(object instanceof String)) {
            new ProcessError("From CollectionPropertyEditor.setValue(),"+
              " no operation: a value in the Collection is not a String.");
            error = true;
            break;
          } // if (!(_value instanceof String))
        } // for
      } // if (propertyTypeInGroup_ == String.class
      else if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_)) {
        for (Iterator ite = ((Collection)_value).iterator() ; ite.hasNext() ; ) {
          object = ite.next();
          if (object != null && !(object instanceof BaseObject)) {
            new ProcessError("From CollectionPropertyEditor.setValue(),"+
              " no operation: a value in the Collection is not a BaseObject.");
            error = true;
            break;
          } // if (!(_value instanceof String))
        } // for
      } // if (propertyTypeInGroup_ == BaseObject
    } // else if (Collection.class.

    // else propertyType_ is an array
    else if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_)) {
      for (Iterator ite = ((Collection)_value).iterator() ; ite.hasNext() ; ) {
        object = ite.next();
        if (object != null && !(object instanceof BaseObject)) {
          new ProcessError("From CollectionPropertyEditor.setValue(),"+
            " no operation: a value of the array is not a BaseObject: "+object.getClass());
          error = true;
          break;
        } // if (!(_value instanceof String))
      } // for ite
    } // if (propertyTypeInGroup_ == BaseObject
    else if (propertyTypeInGroup_ == String.class) {
      for (Iterator ite = ((Collection)_value).iterator() ; ite.hasNext() ; ) {
        object = ite.next();
        if (object != null && !(object instanceof String)) {
          new ProcessError("From CollectionPropertyEditor.setValue(),"+
            " no operation: a value of the array is not a String.");
          error = true;
          break;
        } // if (!(_value instanceof String))
      } // for

          // for testing purpose
          //System.out.println("in CollectionPropertyEditor.setValue() for a string, "+
          //  "the values are: ");
          //for (Iterator ite = ((Collection)_value).iterator() ; ite.hasNext() ; ) {
          //  System.out.println("      :"+ite.next());
          //} // for ite

    } // if (propertyTypeInGroup_ == String.class
    else if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_)) {
      for (Iterator ite = ((Collection)_value).iterator() ; ite.hasNext() ; ) {
        object = ite.next();
        if (object != null && !(object instanceof ImmutableName)) {
          new ProcessError("From CollectionPropertyEditor.setValue(),"+
            " no operation: a value of the array is not an ImmutableName.");
          error = true;
          break;
        } // if (!(_value instanceof String))
      } // for
    } // if (propertyTypeInGroup_ == ImmutableName

    return !error;
  } // setValueControl()

  /**
   * Called by actionPerformed() when the user clicks on the button 'New'.
   * For a String or a BaseObject to add (not a DetailledName), calls addOneStringOrNameElements(),
   * or a dialog box displays a special frame to edit all the arguments
   * of one new BaseObject. <P>
   * Otherwise, the key 'choiceMethodForNew' is used to try to call a specific
   * adding method from the BeanInfo class for the property, typically to create
   * a Resource or an Eligible Party through the call of an gDMak.Action class. <P>
   * If such a method does not exist, the key 'addMethod' is tested
   * in the BeanInfo class for the property, to try to get the adding method. <P>
   * If none of these tests gets any result, the BaseObject constructor is called. <P>
   * In the two last cases (with an 'addMethod' or a constructor method),
   * the BaseObjectHandler class is used to let the user enters the properties of the new BaseObject,
   * and then to call the relevant method or contructor.
   * Defined arguments may be set in the BeanInfo class for the property. <P>
   * Stops all the actions on the beamer buttons until the ending of the method.
   * No action and error message if there is no open ACS which manages AclEntries,
   * for the creation of such an object. <P>
   * Uses SimplePropertyEditor.getKnownValuesForNewElement() to get the defined
   * arguments to use.
   */
  private void addElement() {
      
                          // for testing and tracking purpose
                     //System.out.println("in CollectionPropertyEditor.addElement(), "+
                       //     "the property type is "+propertyTypeInGroup_);

    
    if ( !BaseObject.class.isAssignableFrom(propertyTypeInGroup_) &&
            !DetailledName.class.isAssignableFrom(propertyTypeInGroup_)) {
        // to stop the other dialogs, as BaseObjectHandler.getBaseObjectEditor() done it.
        // this is the only case where BaseObjectHandler is not used in addElement().
        DialogBox.setHasHandlerDialog(true);
        addOneStringOrNameElements();
        DialogBox.setHasHandlerDialog(false);
        return;
    } // if

    if ( ImmutableAclEntry.class.isAssignableFrom(propertyTypeInGroup_) ) { 
        Map map = ACS_Facade.getOpenACS();
        boolean aclAreManaged = false;
        ACS acs = null;
        for (Iterator ite = map.values().iterator(); ite.hasNext();) {
            acs = (ACS)ite.next();
            if ( acs.getManageInternalAcls() || acs.getManageExternalAcls() ) {
                aclAreManaged = true;
                break;
            } 
        } // for
        map = null;
        acs = null;
        if (!aclAreManaged) {
            DialogBox.showErrorMessage("There is no open ACS able to manage AclEntries.");
            return;
        }
    } // if ( ImmutableAclEntry
    
    // initialization of defined values (not constrained values)
    Object[] l_definedValue = getKnownValuesForNewElement(); // SimplePropertyEditor method
    Object[] l_choice;
    Method[] l_methods;
    
    if (propertyDescriptor_.getValue("choiceMethodForNew") != null) {
      l_choice = (Object[]) propertyDescriptor_.getValue("choiceMethodForNew");
      // example of value for l_choice:
      //      pd_one.setValue("choiceMethodForNew", new Object[] {
      //        ActionNewResource.getInstance(), "createResource"} );
      l_methods = l_choice[0].getClass().getMethods();
      for (int k = 0; k < l_methods.length; k++){
          
                    //System.out.println("in CollectionPropertyEditor.addElements(),"+
                      //" source methods: "+l_methods[k].getName());
          
        try {
          if (l_methods[k].getName().equals(l_choice[1])) {

                    // for testing and tracking
                    //System.out.println("in CollectionPropertyEditor.addElement(), "+l_choice[0]+
                      //" object, having the "+l_choice[0].getClass()+
                      //", is called on the method "+l_choice[1]+
                      //" and the arguments:");
                    //for (int i = 0; i < l_definedValue.length; i++) {
                      //  System.out.println("-- the defined value at "+i+" is "+l_definedValue[i]);
                    //}
              
            // note: in the case of an UserID creation, the number of arguments is too large,
            // since it is sized to match to the Actor arguments.
            // The arguments reduction is done by the called method, in ActionNewEligibleParty.
            l_methods[k].invoke(l_choice[0], new Object[] {l_definedValue});
            break;
          } // if (l_methods[k]
        } catch (Exception e) {
           throw new InternalError("In CollectionPropertyEditor.addElements(), "+
            "for 'choiceMethodForNew':"+e.toString());
        } // catch
      } // for k

    } // if "choiceMethodForNew"
    else if (addMethod_ != null) {
        // calls back the addOneBaseObject() method
      BaseObjectHandler.valueOf(this, propertyDescriptor_, addMethod_).
        getBaseObjectEditor(l_definedValue);
    } // else
    else {
        // uses the constructor method, and calls back handleNewBaseObject() method
      BaseObjectHandler.valueOf(this, propertyTypeInGroup_).getBaseObjectEditor(l_definedValue);
    } // else

    l_definedValue = null;
    l_choice = null;
    l_methods = null;
  } // addElement()

  /**
   * Called by addElement() when the user clicks on the button 'New'
   * for adding one String or a Name value for a BaseObject.
   * A dialog box allows to enter the value, and a selection is displayed
   * when the allowed values are limited.
   * No action if it is a null value, a known value,
   * or with a size superior to 200.
   * Manages the restricted values, if any, in calling
   * super.getRestrictedValues() then addRestrictedStringElements().
   * Calls DialogBox.showSingleEditor().
   */
  private void addOneStringOrNameElements() {
    String[]        l_ACSlimitedValues = null;
    
    if (isRestrictedArgument_ && source_ instanceof ACSObject) {
        // call the super method
        l_ACSlimitedValues = getRestrictedValues( 
            (ACS)ACS_Facade.getOpenACS().get( ((ACSObject)source_).getAcsName()) );
    }   


                          // for testing and tracking purpose
                     //System.out.println("in CollectionPropertyEditor.addOneStringOrNameElements(), "+
                       //     "the property class is "+propertyDescriptor_.getClass().getName()+
                       //     " and the property is "+propertyDescriptor_.getName());
                     //if (m_ACSlimitedValues != null) {
                       //  System.out.println("in CollectionPropertyEditor.addOneStringOrNameElements(), "+
                       //     "the ACS limited values keys are: ");
                       // int i = 0;
                       // for (Iterator ite = m_ACSlimitedValues.keySet().iterator(); ite.hasNext();) {
                         //   System.out.println(i+": "+ite.next());
                         //   i++;
                        //}   // for ite 
                     //}
                     //if (l_ACSlimitedValues != null) {
                       //  System.out.println("in CollectionPropertyEditor.addOneStringOrNameElements(), "+
                       //     "the ACS limited values for "+source_.getClass()+" are: ");
                       // for (int i = 0; i < l_ACSlimitedValues.length; i++) {
                       //     System.out.println(i+": "+l_ACSlimitedValues[i]);
                       // }   // for i 
                     //}
    
    
    if (l_ACSlimitedValues != null) {
        // displays a window for editing the restricted string
        addRestrictedStringElements(l_ACSlimitedValues);
        return;
    } // if (propertyTypeInGroup_ == String.class

    // initialization
    String elementName;
    if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_) )
      elementName = "Add "+displayName_;
    else elementName = "Add "+displayName_;
    
    String userText = DialogBox.showSingleEditor(elementName);
    if (userText == null || userText.length() > 200) {
        elementName = null;
        return;
    }

          // uses an intermediate array to initialize l_newValues
    // since listModel_ is not a java.util.Collection
    Object[]    l_valuesArray = new Object[listModel_.getSize()];
    listModel_.copyInto(l_valuesArray);
    HashSet     l_newValues = new HashSet(INITIAL_CAPACITY);

    // initializes l_newValues with non-null values
    for (int i = 0 ; i < l_valuesArray.length ; i++) {
      if (l_valuesArray[i] != null) l_newValues.add(l_valuesArray[i] );
    } // for i

    // updates the property in the gBase instance
    // if it is unknown
    if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_) ) {
        ImmutableName name = new NameImpl(userText.trim());
        if (!l_newValues.contains(name)) l_newValues.add( name );
        name= null;
    } else {
        if (!l_newValues.contains(userText.trim())) l_newValues.add( userText.trim() );
    }
    setValue(l_newValues);


    userText = null;
    l_newValues = null;
    l_valuesArray = null;
    elementName = null;
  } // addOneStringOrNameElements()

  /**
   * This method displays to the user a list of allowed values
   * which are not still selected.
   * Called by addOneStringOrNameElement()
   * for a String value where a restricted set of values is defined
   * in the source ACS or in the BeanInfo class.
   * To select the allowed values, the order is:
   * - applies the source ACS restrictions to the proposed values, if any,
   * - if not, applies then the BeanInfo restrictions to the proposed values,
   * - removes the restricted values which are already selected.
   * A dialog box allows to select only one value. A message informs the user
   * when an entered value is not allowed.
   * @param _l_ACSlimitedValues is the array of the allowed values. Never null.
   */
  private void addRestrictedStringElements(String[] _l_ACSlimitedValues) {
    
    if (_l_ACSlimitedValues == null) {
        // for testing and tracking        
        throw new InternalError("in CollectionPropertyEditor.addRestrictedStringElements(), "+
          "the argument is null.");       
    } 
    
    // initialization
    String[]        l_restrictedValues = _l_ACSlimitedValues;
    boolean[]       l_currentValuesIndex = null;
    short           selectionCount = 0;
    String[]        l_ToBeSelectedValues;
    Component[]     l_boxes;
    JPanel          panel;
    
    // removes in the displayed list, all the values which are already selected
    // and returns if all of them are selected
    l_currentValuesIndex = new boolean[l_restrictedValues.length];
    for (int i = 0; i < l_restrictedValues.length; i++) {
        if (listModel_.contains(l_restrictedValues[i])) {
          l_currentValuesIndex[i] = true;
          selectionCount++;
        }
    }   // for i
    if (l_restrictedValues.length == selectionCount) {
      DialogBox.showErrorMessage(
          "All the allowed values are already selected." );
      l_restrictedValues = null;
      l_currentValuesIndex = null;
      return;
    }
    l_ToBeSelectedValues = new String[l_restrictedValues.length - selectionCount];
    selectionCount = 0;
    for (int i = 0; i < l_restrictedValues.length; i++) {
      if (!l_currentValuesIndex[i]) {
        l_ToBeSelectedValues[selectionCount] = " "+l_restrictedValues[i];
        selectionCount++;
      }
    } // for

    panel = DialogBox.showSelectionsInOneDialog(displayName_, l_ToBeSelectedValues);

    // uses an intermediate array to initialize l_newValues
    // since listModel_ is not a java.util.Collection
    Object[]    l_valuesArray = new Object[listModel_.getSize()];
    listModel_.copyInto(l_valuesArray);
    HashSet     l_newValues = new HashSet(INITIAL_CAPACITY);

    // initializes l_newValues
    for (int i = 0 ; i < l_valuesArray.length ; i++) {
      l_newValues.add(l_valuesArray[i] );
    } // for i

    // gets the selected Strings
    l_boxes = panel.getComponents();
    for (int i = 0 ; i < l_boxes.length ; i++) {
      if ( ((JCheckBox)l_boxes[i]).isSelected() )
        l_newValues.add( ((JCheckBox)l_boxes[i]).getActionCommand().substring(1) );
    } // for i

    // updates the selection in the gBase instance
    setValue(l_newValues);

    l_newValues = null;
    l_valuesArray = null;
    l_restrictedValues = null;
    l_currentValuesIndex = null;
    selectionCount = 0;
    l_ToBeSelectedValues = null;
    l_boxes = null;
    panel = null;
  } // addRestrictedStringElements()

  /**
   * This method calls the remove/delete method of the property.
   * Called by actionPerformed() when the user clicks on the button 'Remove'
   * or 'Delete'. Removes all the elements the user has selected in the list.
   * Called directly by BeamerManager.initializeDeleteButton().
   * There are two cases: <P>
   * 1 - Uses the removing method if it exists. In this case, in the property descriptor,
   * 'removeMethod' defines the removing method for the property.
   * If 'removeMethod' is finalizeForUser() and _type is 'delete', it is used there to delete.
   * Else, the removing method is a specific one which is returned by the property descriptor. <P>
   * 2 - otherwise, uses the writing method to update the collection
   * with no direct use of finalizeForUser() on each element. Then, it is more general. <P>
   * Caution: for Strings, the removing method cannot be used.
   * @param _l_objectsToRemove is the list of Strings or ImmutableNames
   *  selected in the JList to be removed or deleted.
   * @param _type 'remove' for removing, or 'delete' for deleting
   */
  private void removeElements(Object[] _l_objectsToRemove, String _type) {

    // test if no selection
    if ( _l_objectsToRemove == null) {
      throw new InternalError("In CollectionPropertyEditor.removeElements()"+
        ", the argument is null.");
    }

    if (!DialogBox.showConfirmDialog(
          " Do you want to "+_type+" the selected element(s)?", null))
        return;

    // initialization
    BaseObject object = EligiblePartyImpl.EMPTY_INSTANCE;

                                      // for testing and tracking purpose
                                 //System.out.println(
                                   // "in CollectionPropertyEditor.removeElements(), "+
                                   //"the object to remove are ");
                                   //for (int i = 0; i < _l_objectsToRemove.length; i++) {
                                     //  System.out.println("--- "+_l_objectsToRemove[i].getClass()+
                                       //     ": "+_l_objectsToRemove[i].toString());
                                   //} // for i

    try {
      if (removeMethod_ != null) {

                                      // for testing and tracking purpose
                                 //System.out.println(
                                   // "in CollectionPropertyEditor.removeElements(), "+
                                    //"the object to remove with removeMethod_");

        if (removeMethod_.getName().equals("finalizeForUser") &&
          _type.equals("delete")) {
              for (int i = 0; i < _l_objectsToRemove.length; i++) {

                if (_l_objectsToRemove[i] == null) continue;
                if (_l_objectsToRemove[i] instanceof ImmutableName) {
                  object = BeamerManager.getBaseObject 
                          ((ImmutableName)_l_objectsToRemove[i], propertyTypeInGroup_);
                  // ACS_Facade.getEorBaseObject throws a BaseError if object is empty

                                  // for testing and tracking purpose
                             //System.out.println("in CollectionPropertyEditor.removeElements(), "+
                              //"the object to remove is "+_l_objectsToRemove[i]);

                  ((FinalizedObjectForUser)object).finalizeForUser();
                } // else if
                else if (_l_objectsToRemove[i] instanceof String &&
                        DetailledName.class.isAssignableFrom(propertyTypeInGroup_)) {

                    // throws BaseError
                     object = ACS_Facade.getEorBaseObject(
                                getNameFromDetailledName((String)_l_objectsToRemove[i]));


                                  // for testing and tracking purpose
                             //System.out.println("in CollectionPropertyEditor.removeElements(), "+
                              //"the object to remove is "+object);

                    ((FinalizedObjectForUser)object).finalizeForUser();
                } // else if

              } // for i
        } // if equals("finalizeForUser"))
        else {
          // it is a specific method
          for (int i = 0; i < _l_objectsToRemove.length; i++) {

            if (_l_objectsToRemove[i] == null) continue;
            
            if (_l_objectsToRemove[i] instanceof ImmutableName) {
              if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_)) {
                // ACS_Facade.getEorBaseObject throws a BaseError if object is empty
                // and InternalError if the StringRight is not found
                object = BeamerManager.getBaseObject 
                          ((ImmutableName)_l_objectsToRemove[i], propertyTypeInGroup_);
                removeWithSpecificMethod(object);
              } // if
            } // if

            else if (_l_objectsToRemove[i] instanceof String) {
              if (DetailledName.class.isAssignableFrom(propertyTypeInGroup_)) {
                // throws BaseError
                 object = ACS_Facade.getEorBaseObject(
                            getNameFromDetailledName((String)_l_objectsToRemove[i]));
                removeWithSpecificMethod(object);
              } // if
            } // else if

          } // for i
        } // else if !equals("finalizeForUser"))

      } // if (removeMethod_ != null)
      else if (writeMethod_ != null) {
        ArrayList     l_newValues = (ArrayList)getValue();
        //int           index = -1;

                                      // for testing and tracking purpose
                                 //System.out.println(
                                   // "in CollectionPropertyEditor.removeElements(), "+
                                    //"the object to remove with writeMethod_");

        for (int i = 0; i < _l_objectsToRemove.length; i++) {

          if (_l_objectsToRemove[i] == null) continue;

          if (_l_objectsToRemove[i] instanceof ImmutableName) {
            if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_)) {
                object = BeamerManager.getBaseObject 
                          ((ImmutableName)_l_objectsToRemove[i], propertyTypeInGroup_);
                
                                          // for testing and tracking purpose
                                     //System.out.println("in CollectionPropertyEditor.removeElements(), before the removing of "+
                                       //      object+" for property type "+propertyTypeInGroup_+", the value classes are");
                                     //for (Iterator ite = l_newValues.iterator(); ite.hasNext(); ) {
                                       //  System.out.println("---------- "+ite.next().getClass());
                                     //}
                
                
                // ACS_Facade.getEorBaseObject throws a BaseError if object is empty
                boolean remove = l_newValues.remove(object);

                          // for testing and tracking purpose
                     //System.out.println("in CollectionPropertyEditor.removeElements(), "+
                      //"the object to remove is removed "+remove+" for property type "+propertyTypeInGroup_);
            
            }
            else if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_)) {
              l_newValues.remove(_l_objectsToRemove[i]);
            } // else
          } // if (_l_objectsToRemove[i] instanceof ImmutableName) {
          else if (propertyTypeInGroup_ == String.class) {
            l_newValues.remove(_l_objectsToRemove[i]);
          }

        } // for i

        // update the property in the gBase instance
        // without direct finalizing of each deprecated element (done in gBase if necessary)
        setValue(l_newValues);

        l_newValues = null;
      }  // with writeMethod_

    } 
    catch (UpDateError e) {
                // for testing purpose
      DialogBox.showErrorMessage(e.toString() );
      e.printStackTrace();
      throw new InternalError("in CollectionPropertyEditor.removeElements(), "+
            "for the property "+propertyDescriptor_.getName()+", the exception is: "+e);
    } // catch
    catch (BaseError s) {
                // for testing purpose
      DialogBox.showErrorMessage(s.toString() );
      s.printStackTrace();
      throw new InternalError("in CollectionPropertyEditor.removeElements(), "+
            "for the property "+propertyDescriptor_.getName()+", the exception is: "+s);
    } // catch
    catch (InstantiationException i) {
                // for testing purpose
      DialogBox.showErrorMessage(i.toString() );
      i.printStackTrace();
      throw new InternalError("in CollectionPropertyEditor.removeElements(), "+
            "for the property "+propertyDescriptor_.getName()+", the exception is: "+i.getCause());
    } // catch
    catch (InvocationTargetException v) {
                // for testing purpose
      DialogBox.showErrorMessage(v.getCause().toString() );
      v.printStackTrace();
      throw new InternalError("in CollectionPropertyEditor.removeElements(), "+
            "for the property "+propertyDescriptor_.getName()+", the exception is: "+
            v.getCause());
    } // catch
    catch (IllegalAccessException l) {
                // for testing purpose
      l.printStackTrace();
      throw new InternalError("in CollectionPropertyEditor.removeElements(), "+
            "for the property "+propertyDescriptor_.getName()+", the exception is: "+l);
    } // catch
    catch (ClassNotFoundException e) {
                // for testing purpose
      e.printStackTrace();
      throw new InternalError("in CollectionPropertyEditor.removeElements(), "+
            "for the property "+propertyDescriptor_.getName()+", the exception is: "+e);
    } // catch

    object = null;
  } // removeElements()

  /**
   * Removes a BaseObject when writeMethod_ and finalizeForUser() do not exist.
   * Called by removeElements() and by BeamerManager.initializeDeleteButton().
   * In the property descriptor, "removeMethod" defines:
   *    the 'remove' method for the property,
   *    and "removeArgumentsFromName", if it is non-null, defines the class and
   *    the method to call to get the right removing arguments from the object name.
   *    If "removeArgumentsFromName" is null, the remove method is invoked
   *    with the object to remove as single argument.
   *    For example, for removing an AclEntry, there is:
   *      - propertyDescriptor.setValue("removeMethod", "removeAclEntry");
   *      - propertyDescriptor.setValue("removeArgumentsFromName", new Object[] {"AclEntryImpl",
   *        "getKeyReferencesFromName", new Integer(0), new Integer(1), 
   *        new Integer(3), new Integer(4)} );
   *    The first returned array defines the name of the remove method.
   *    It allows this instance to call the remove method of the property.
   * Following the KeyPropertiesInName pattern, in the property descriptor,
   * 'removeArgumentsFromName' defines :
   *    how to find the right arguments for
   *    the 'remove' method of the property, when the name of the object
   *    to remove is known, and finalizeForUser() does not exist.
   *    The returned array defines:
   *      - the name of the class to call, at the index 0
   *      - the name of the static method to call, at the index 1, with
   *        the name of the object to remove as argument, and with
   *        an Object[] array as returned value
   *      - in the Object[] array returned as value by the static method,
   *        the sequence of index where are the n arguments to use
   *        in the remove method, in the correct order, if they do exist.
   *        Otherwise, the argument is null.
   *    These elements allow this method to call the remove method of the property.
   * @param _objectToRemove is the object to remove which is associated to
   *  a specific remove method. No operation if this object has not the right
   *  property type.
   * @exception thrown by Method.invoke(): ClassNotFoundException,
   *  IllegalAccessException, InvocationTargetException, InstantiationException
   */
  void removeWithSpecificMethod(BaseObject _objectToRemove) throws
    ClassNotFoundException, IllegalAccessException, InvocationTargetException,
    InstantiationException {

    if (!propertyTypeInGroup_.isInstance(_objectToRemove)) return;

    // initialization
    Object[]  l_removeArguments = (Object[]) propertyDescriptor_.
                getValue("removeArgumentsFromName");

    if (l_removeArguments == null) {

                      // for testing purpose
                    //System.out.println("CollectionPropertyEditor.removeWithSpecificMethod()"+
                      //      "for a simple removing, the argument is "+_objectToRemove.getName());
        
        // remove the object in the base
      removeMethod_.invoke(source_, new Object[] {_objectToRemove});
      _objectToRemove = null;
      return;
    }

    Class     descriptorClass = Class.forName("ARoad0.gBase."+(String)
                l_removeArguments[0]);
    Method    propertiesMethod = String.class.getMethods()[0]; // no-sens init
    Method[]  l_methods = descriptorClass.getMethods();
    Object[]  l_properties;
    Object[]  l_arguments;

    for (int j = 0; j < l_methods.length; j++){
      // search in descriptorClass the method to get the arguments
      if (l_methods[j].getName().equals((String)l_removeArguments[1])) {
        propertiesMethod = l_methods[j];

                      // for testing purpose
                 //System.out.println("in CollectionPropertyEditor.removeWithSpecificMethod(), "+
                  //"the method to get the arguments is "+propertiesMethod.toString());

      } // if
    } // for

              // for testing purpose
         //System.out.println("in CollectionPropertyEditor.removeWithSpecificMethod(), "+
          //"the object to remove class is "+_objectToRemove.getClass().toString());

    // get the remove arguments from the object name
    l_properties = (Object[]) propertiesMethod.invoke(
      descriptorClass.newInstance(), new Object[] {_objectToRemove.getName()});

              // for testing purpose
         //System.out.println("in CollectionPropertyEditor.removeWithSpecificMethod(), "+
         // "the number of arguments is "+l_removeArguments.length+" - 2");

    l_arguments = new Object[l_removeArguments.length - 2];
    // the order of the arguments, in l_properties, is get from the end of l_removeArguments.
    // propertiesMethod may return a l_properties array smaller than (l_removeArguments.length - 2),
    // and the last item(s) in l_arguments are let null.
    for (int j = 0; j < l_arguments.length; j++) {
      if (((Integer)l_removeArguments[j+2]).intValue() < l_properties.length)
          l_arguments[j] = l_properties[((Integer)l_removeArguments[j+2]).intValue()];

                      // for testing and tracking
                 //System.out.println("in CollectionPropertyEditor.removeWithSpecificMethod(), "+
                  //"the remove argument ["+j+"] is "+l_arguments[j]);
                 //if (l_arguments[j] != null)
                   //  System.out.println(" with the type "+l_arguments[j].getClass().toString());

    } // for j

                      // for testing and tracking
                 //System.out.println("in CollectionPropertyEditor.removeWithSpecificMethod(), "+
                  //"the remove method is: "+removeMethod_+", and the source is: "+source_.getName());

    // remove the object in the base
    removeMethod_.invoke(source_, l_arguments);

    l_removeArguments = null;
    descriptorClass = null;
    propertiesMethod = null;
    l_methods = null;
    l_properties = null;
    l_arguments = null;
  } // removeWithSpecificMethod()

  /**
   * Called by actionPerformed() when the user clicks on the button 'See'.
   * This method selects a BaseObject in the explorer, and then it is displayed
   * in the beamer. This method cannot handle a Collection of StringRights,
   * but it works with an array of StringRights.
   * Calls super.viewOneElement().
   */
  private void viewElement() {

    // initialization
    JList       jlist = (JList) boundComponent_;
    Object[]    l_selectedArray = jlist.getSelectedValues();
    Object      valueInList;

    if (jlist.getModel().getSize() == 0) {
      DialogBox.showErrorMessage("The list is empty.");
      return;
    } // if

    if (jlist.getModel().getSize() == 1) 
        valueInList = jlist.getModel().getElementAt(0);
    else if (l_selectedArray.length != 1) {
      DialogBox.showErrorMessage("Select in the list exactly one element to view.");
      return;
    }
    else {
        valueInList = l_selectedArray[0];
    }

    // get the selected object in the beamer.
    // finalizes this instance, if in the beamer.
    if (( ImmutableName.class.isAssignableFrom(propertyTypeInGroup_) || 
            BaseObject.class.isAssignableFrom(propertyTypeInGroup_) ||
            DetailledName.class.isAssignableFrom(propertyTypeInGroup_)) &&
         valueInList != null)
            viewOneElement( valueInList );

    jlist = null;
    l_selectedArray = null;
    valueInList = null;
  } // viewElement()

  /**
   * This is the main method of this class, which
   * gets the array, collection or map editor with the property name
   * and the relevant buttons.
   * The possible property types are Object[], java.util.Collection or Map.
   * Each call creates a new independant editor that has this instance
   * as an action listener for updating the property.
   * Calls getButtonsDisplaying(), initializeListModel().
   * @return the panel
   */
  private JPanel getCollectionEditor() {
    // initialization
    JPanel                      result = new JPanel();
    JLabel                      nameLabel = new JLabel(getDisplayName());
    JList                       jlist;
    JPanel                      listPanel = new JPanel();
    // an array of 9 booleans: withPreviousButton at 0, withViewButton,
    // withSelectButton, withNewButton, withDeleteButton, withRemoveButton,
    // withClearButton at 6 (the others are not for this editor)
    boolean[]                   l_buttonDisplay = getButtonsDisplaying();
    boolean                     displayTheName = true;
    if ("".equals(getDisplayName())) displayTheName = false;

    initializeListModel();

          // for testing purpose
          //System.out.println("in CollectionPropertyEditor.getCollectionEditor(),"+
            //" the list size is: "+listModel_.getSize());
          //if (!listModel_.isEmpty()) System.out.println("the first element is:"+
            //listModel_.get(0)+".");

    if (displayTheName) {
        nameLabel.setMaximumSize(NAME_DIMENSION);
        nameLabel.setMinimumSize(NAME_DIMENSION);
        nameLabel.setPreferredSize(NAME_DIMENSION);
        nameLabel.setFont(BeamerManager.plainFont);
        nameLabel.setToolTipText(propertyDescriptor_.getShortDescription());
    }

    jlist = new JList(listModel_);
    jlist.setBackground(InternalFrame.NON_EDITABLECOLOR);

    if (propertyDescriptor_.getValue("horizontalTextPositionToLeft") != null &&
            (Boolean)propertyDescriptor_.getValue("horizontalTextPositionToLeft")) {
        ((DefaultListCellRenderer)jlist.getCellRenderer()).
            setHorizontalAlignment(JLabel.LEFT);
        ((DefaultListCellRenderer)jlist.getCellRenderer()).
            setHorizontalTextPosition(JLabel.LEFT);
    } else {
        ((DefaultListCellRenderer)jlist.getCellRenderer()).
            setHorizontalAlignment(JLabel.RIGHT);
        ((DefaultListCellRenderer)jlist.getCellRenderer()).
            setHorizontalTextPosition(JLabel.RIGHT);
    }
    
    jlist.setFont(BeamerManager.plainFont); 

    // uses a variable scrollPane_, with test of the jlist size
    scrollPane_ = new JScrollPane(jlist);
        //scrollPane_.getViewport().add(jlist);
    
                  // for testing purpose 
                  //System.out.println("in CollectionPropertyEditor.getCollectionEditor(),"+
                    //" the font cell renderer in list is a: "+
                    //((DefaultListCellRenderer)jlist.getCellRenderer()).getFont() );

    // editable through two buttons, and listen user actions,
    // only if there is a set method
    JPanel buttonPanel = setVerticalButtonsInEditor(l_buttonDisplay, false, false);
    
                  // for testing purpose 
                  //System.out.println("CollectionPropertyEditor.getCollectionEditor() for "+
                    //      propertyDescriptor_.getName()+
                      //    ", displayTheName: "+displayTheName);
                  //if (buttonPanel == null) System.out.println("buttonPanel is null");
                  //else System.out.println("buttonPanel is not null");
    
    if (buttonPanel != null) {
        if (displayTheName) {
            setListPanelDimension(listModel_.getSize(), scrollPane_, true, true, true);
        }
        else {
            setListPanelDimension(listModel_.getSize(), scrollPane_, false, false, true);
        }
    }
    else {
        if (displayTheName) { 
            setListPanelDimension(listModel_.getSize(), scrollPane_,
                    true, false, true);
            nameLabel.setMaximumSize(BOOLEAN_NAME_DIMENSION);
            nameLabel.setMinimumSize(BOOLEAN_NAME_DIMENSION);
            nameLabel.setPreferredSize(BOOLEAN_NAME_DIMENSION);
        } else { // used for instance by the ACS 'ImmutableRules' property
            setListPanelDimension(listModel_.getSize(), scrollPane_,
                    true, false, false);
        } // else
    } // else

    scrollPane_.getHorizontalScrollBar().setValue(
      scrollPane_.getHorizontalScrollBar().getMaximum() );  // does not work  
    listPanel.add(scrollPane_);
    listPanel.setBackground(Color.white);
    //listPanel_2.setBackground(Color.white);
    listPanel.setBorder(new LineBorder(InternalFrame.FORECOLOR));
    
    if (displayTheName) result.add(nameLabel);
    result.add(listPanel);
    result.setBackground(Color.white);
    result.setBorder(new EtchedBorder());

    if (buttonPanel != null) result.add(buttonPanel);

    boundComponent_ = jlist;

              // for testing purpose
         //System.out.println("in CollectionPropertyEditor.getCollectionEditor(), "+
         // "the displayed list for "+propertyDescriptor_.getName()+" is:");
         // for (int i = 0; i < listModel_.getSize(); i++) {
         //   System.out.println("for "+i+": "+listModel_.getElementAt(i));
         // }

    nameLabel = null;
    jlist = null;
    listPanel = null;
    buttonPanel = null;

    return result;
  } // getCollectionEditor()

  /**
   * Called by getCollectionEditor() and getCustomEditor() to initialize the list model.
   * Does not control the size. Processes, in the order, the arrays, the collections and the maps
   * having as items some BaseObjects, DetailledNames, Strings or ImmutableNames.
   * If the read method returns null, sets an empty list.
   */
  private void initializeListModel() {

    // variables
    Object[] returnedArray; 
              
                          // for testing purpose
                          //System.out.println("CollectionPropertyEditor.initializeListModel() start for "+
                            //      propertyDescriptor_.getName());

    // Can't use getValue() since DefaultListModel has not an addAll(Set) method.
    listModel_.clear();
    try {
        // if the read method returns null, sets an empty list
        if (readMethod_.invoke(source_, (Object[])null) == null) return;

                          // for testing and tracking
                //if (propertyDescriptor_.getName().equals("ImpliedOpenViews")) {           
                    //System.out.println("in CollectionPropertyEditor.inializeListModel(),"+
                        //" for the property 'ImpliedOpenViews', readMethod_ is: "+
                        //readMethod_);
                //} // if
          
        if (propertyIsArray_) {
          returnedArray = (Object[]) readMethod_.invoke(source_, (Object[])null);

          // for AclEntries and Privileges, puts the detailled name in the listModel_
          if (DetailledName.class.isAssignableFrom(propertyTypeInGroup_)) {

                          // for testing and tracking
                     //if (getDisplayName().equals("'user' rights: "))
                     // System.out.println("in CollectionPropertyEditor.inializeListModel(),"+
                     //   " for the source: "+getSource().hashCode()+", the owner rights are: ");

            for (int i = 0 ; i < returnedArray.length ; i++) {
              if (returnedArray[i] != null)
                  listModel_.addElement(
                      ((DetailledName)returnedArray[i]).getDetailledName());

                      //if (getDisplayName().equals("'user' rights: "))
                      //System.out.println("-----"+returnedArray[i]);

            }

          } // if (propertyTypeInGroup_ == String.class
          // for BaseObjects, puts their names in the listModel_
          else if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_)) {
              
                          // for testing purpose
                          //if ( propertyDescriptor_.getName().equals("ConditionSourceGroups") ) {
                            //  System.out.println("in CollectionPropertyEditor.initializeListModel(), "+
                              //    "the returned value as BaseObject array is detected for 'Work if member of'");  
                          //} // if
              
            for (int i = 0 ; i < returnedArray.length ; i++) {
              if (returnedArray[i] != null && 
                ((BaseObject)returnedArray[i]).getName() != null) {
                  listModel_.addElement(((BaseObject)returnedArray[i]).getName());

                          // for testing purpose
                          //if ( propertyDescriptor_.getName().equals("InheritedGroupRights") ) {
                            //  System.out.println("----------- "+
                              //        ((BaseObject)returnedArray[i]).getName());  
                          //} // if
                  
              } // if
            } // for i
          } // if (propertyTypeInGroup_ == BaseObject
          // for simple Strings, puts them in the listModel_
          else if (propertyTypeInGroup_ == String.class) {
              
                          // for testing and tracking
                     //if (getDisplayName().equals("'user' rights: "))
                     // System.out.println("in CollectionPropertyEditor.inializeListModel(),"+
                     //   " for the source: "+getSource().hashCode()+", the owner rights are: ");

            for (int i = 0 ; i < returnedArray.length ; i++) {
              if (returnedArray[i] != null)
                  listModel_.addElement(returnedArray[i]);

                      //if (getDisplayName().equals("'user' rights: "))
                      //System.out.println("-----"+returnedArray[i]);

            }

          } // if (propertyTypeInGroup_ == String.class
          // for ImmutableNames, put them in the listModel_
          else if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_)) {
            for (int i = 0 ; i < returnedArray.length ; i++) {
              if (returnedArray[i] != null) listModel_.addElement(returnedArray[i]);
            }
          } // if (propertyTypeInGroup_ == String.class
          else throw new Exception("in CollectionPropertyeditor.initializeListModel()"+
            " the object to edit is not a string, a name nor a BaseObject.");
        }
        else { // if (!propertyIsArray_)
          // for AclEntries and Privileges, puts them in the listModel_
          if (DetailledName.class.isAssignableFrom(propertyTypeInGroup_)) {
            for (Iterator ite = ((List) getValue()).iterator() ; ite.hasNext() ; ) {
              listModel_.addElement(((DetailledName) ite.next()).getDetailledName());
            }
          } // if (propertyTypeInGroup_ == String.class
          // for BaseObjects, puts their names in the listModel_
          else if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_)) {
            if (Collection.class.isAssignableFrom(propertyType_)) {

                          // for testing and tracking
                     //if (propertyDescriptor_.getName().equals("OneResourceVirtualFolder"))
                      //System.out.println("in CollectionPropertyEditor.inializeListModel(),"+
                        //" the returned value as BaseObject Collection is detected "+
                        //"for 'OneResourceVirtualFolder'");
                
              for (Iterator ite = ((Collection) getValue()).iterator() ; ite.hasNext() ; ) {
                listModel_.addElement(((BaseObject)ite.next()).getName());
              }
            }
            else if (Map.class.isAssignableFrom(propertyType_)) {

                          // for testing and tracking
                     //if (propertyDescriptor_.getName().equals("ConditionSourceGroups"))
                      //System.out.println("in CollectionPropertyEditor.inializeListModel(),"+
                        //" the returned value as BaseObject Map is detected for 'ConditionSourceGroups'");                                
                
              for (Iterator ite = ((Map) getValue()).values().iterator() ; ite.hasNext() ; ) {
                listModel_.addElement(((BaseObject)ite.next()).getName());
              }
            }
          } // if (propertyTypeInGroup_ == BaseObject
          // for Strings, puts them in the listModel_
          else if (String.class.isAssignableFrom(propertyTypeInGroup_)) {
            for (Iterator ite = ((List) getValue()).iterator() ; ite.hasNext() ; ) {
              listModel_.addElement((String) ite.next());
            }
          } // if (propertyTypeInGroup_ == String.class
          // for ImmutableNames, puts them in the listModel_
          else if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_)) {
            if (Collection.class.isAssignableFrom(propertyType_)) {
              for (Iterator ite = ((Collection) getValue()).iterator() ; ite.hasNext() ; ) {
                listModel_.addElement(ite.next());
              }
            }
            else if (Map.class.isAssignableFrom(propertyType_)) {
              for (Iterator ite = ((Map) getValue()).values().iterator() ; ite.hasNext() ; ) {
                listModel_.addElement(ite.next());
              }
            }
          } // if (propertyTypeInGroup_ == BaseObject
        } // else if (propertyIsArray_)

    } catch (Exception l) {
      new ProcessError("from CollectionPropertyEditor.initializeListModel(): "+
              l.toString());
    }

    returnedArray = null;
  } // initializeListModel(


  /**
  * Called by handleSelectionDialog() for actionPerformed() and directly by
  * actionPerformed() for restricted values, when the property is a Collection,
  * a Map or an array, and the property type is a BaseObject
  * which has to be selected in the explorer through the button 'Select'.
  * Uses the 'add' method or the write method, in this order.
  * Controls the collection size and if the new value is already known.
  * Caution: it is an overridden method.
  * @param _list contains the new selected objects
  */
  private void handleSelectionUpdate(List _list) {
    // stop if the argument value is null or empty
    if (_list == null || _list.isEmpty()) {
        return;
    } // if      

    int currentSize = 0;
    if (listModel_ != null) currentSize = listModel_.size();
    if (maxSize_ > 0 && maxSize_ < currentSize + _list.size()) {
      DialogBox.showErrorMessage("No operation since "+
        maxSize_+" is the maximum size for this collection.");
      return;
    }
    
    List    l_newValues = new ArrayList(INITIAL_CAPACITY);

            // for testing purpose
            //System.out.println("in CollectionPropertyEditor.handleSelectionUpdate(), \n"+
              //      "for the source "+source_.getName()+" with listModel_: "+listModel_+
                //    "- new selection:");
            //for (Iterator itetest = _list.iterator(); itetest.hasNext(); ) {
              //  System.out.println("      "+itetest.next());
            //} // for

    // if there is an 'add' method, use it
    if (addMethod_ != null && (
      listModel_ != null && !listModel_.contains(_list.get(0)))  ) {
          try {
            addMethod_.invoke(source_, _list.toArray() );
          } catch (InvocationTargetException i) {
              //i.printStackTrace();
              if (i.getCause().toString() == null)
                  throw new InternalError("in CollectionPropertyEditor.handleSelectionUpdate(), "+
                    "the exception is an InvocationTargetException whith a null cause.");
              
              // show the message to the user
              int lastIndex = i.getCause().toString().lastIndexOf(": ");
              if (lastIndex != 0)
                DialogBox.showErrorMessage(i.getCause().toString().substring(lastIndex+1));
              else
                  DialogBox.showErrorMessage(i.getCause().toString());

              if (i.getTargetException().getClass() != CreateError.class) {
                  System.out.println("in CollectionPropertyEditor.handleSelectionUpdate(), error: "+
                          i.getTargetException().getClass());
                  i.getCause().printStackTrace();
              } // if
          } catch (Exception e) {
            new ProcessError("in CollectionPropertyEditor.handleSelectionUpdate(), "+
              "Exception:");
            e.printStackTrace();
          }

          l_newValues = null;
          return;
    } // if

    if (getValue() instanceof List) l_newValues = (List) getValue();
    else if (getValue() instanceof Map)
      l_newValues = new ArrayList(((Map)getValue()).values());
    else {
      new ProcessError("in CollectionPropertyEditor.handleSelectionUpdate(), "+
        "the write method has to be used, but getValue() does not return a list or a map.");
    }

    // from the selection, get the gBase instances
    for (int i = 0; i < _list.size(); i++) {
/*
      if (BaseObject.class.isAssignableFrom(propertyTypeInGroup_))
        l_newValues.add( ((BaseObject)list.get(i)).getName() );
      else if (propertyTypeInGroup_ == String.class)
        l_newValues.add( list.get(i) );
      else if (ImmutableName.class.isAssignableFrom(propertyTypeInGroup_))
*/
      if (!l_newValues.contains(_list.get(i))) l_newValues.add( _list.get(i) );
    } // for
    // update the selection in the data base
    setValue(l_newValues);

    l_newValues = null;
  } // handleSelectionUpdate()
  
} // CollectionPropertyEditor
