//Title:        Access Road
//Version:      0.6.0
//Copyright:    Copyright 2002, 2004, 2007, 2008, 2009 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui1;

import java.awt.Dimension;
import java.beans.*;
import java.lang.reflect.Method;

/**
 * This interface is responsible for displaying and editing the
 * properties of gBase classes, by the mean of panels.
 * It is designed to be implemented by a javax.bean.PropertyEditor,
 * like SimplePropertyEditor or CollectionPropertyEditor.
 * Each implementation must have a single static constructor valueOf(BaseObject _source,
 * PropertyDescriptor _propertyDescriptor). SimplePropertyEditor is the first
 * implementation.
 * @see ARoad0.Gui1.SimplePropertyEditor
 * @version 0.6.0
 */
public interface CommonPropertyEditor extends PropertyEditor {

  /**
   * Height dimension for all properties
   */
  public static final int HEIGHT_DIMENSION = 30;

  /**
   * Width dimension for current name
   */
  public static final int NAME_WIDTH_DIMENSION = 120;

  /**
   * Width dimension for current name of a boolean property
   */
  public static final int NAME_WIDTH_DIMENSION_2 = 150;

  /**
   * Width dimension for name in map
   */
  public static final int MAP_NAME_WIDTH_DIMENSION = 65;
  
  /**
   * dimension of colonms for displaying a map
   */
  public static final Dimension
    COLOMN_DIMENSION = new Dimension(200, HEIGHT_DIMENSION); 
  
  /**
   * Name dimension for most of the properties
   */
  public static final Dimension
    NAME_DIMENSION = new Dimension(NAME_WIDTH_DIMENSION, HEIGHT_DIMENSION); 
  
  /**
   * Name dimension for the boolean properties and a collection without buttons
   */
  public static final Dimension
    BOOLEAN_NAME_DIMENSION = new Dimension(NAME_WIDTH_DIMENSION_2, HEIGHT_DIMENSION); // 15

  /**
   * Name dimension for map properties
   */
  public static final Dimension
    NAME_MAP_DIMENSION = new Dimension(MAP_NAME_WIDTH_DIMENSION, HEIGHT_DIMENSION); // 150
  
  /**
   * Button dimension for SimplePropertyEditor
   */
  public static final Dimension
    BUTTON_DIMENSION = new Dimension(40, HEIGHT_DIMENSION); // 50

  /**
   * Button dimension for SimplePropertyEditor and CollectionPropertyEditor
   */
  public static final Dimension
    BUTTON_DIMENSION_2 = new Dimension(50, HEIGHT_DIMENSION);

  /**
   * Button dimension for SimplePropertyEditor
   */
  public static final Dimension
    BUTTON_DIMENSION_3 = new Dimension(50, HEIGHT_DIMENSION/5);
  
  /**
   * Width dimension for the larger label in all properties
   */
  public static final int NO_BUTTON_WIDTH_DIMENSION = 280; // 310

  /**
   * Field dimension for one boolean box without ComboBox
   */
  public static final Dimension BOOLEAN_FIELD_DIMENSION = new Dimension(
    40, HEIGHT_DIMENSION); 

  /**
   * Field dimension for one boolean box with ComboBox
   */
  public static final Dimension COMBOBOX_FIELD_DIMENSION = new Dimension(
    70, HEIGHT_DIMENSION);

  /**
   * Field dimension of the empty field for a boolean or integer box.
   * Used by BeamerManager.addPropertyEditor().
   */
  public static final Dimension EMPTY_FIELD_DIMENSION = new Dimension(
    NO_BUTTON_WIDTH_DIMENSION - 85, HEIGHT_DIMENSION); 

  /**
   * Field dimension for an integer box
   */
  public static final Dimension INT_FIELD_DIMENSION = new Dimension(
    70, HEIGHT_DIMENSION); // 100

  /**
   * Field dimension of the empty field for an integer box: INT_FIELD_DIMENSION
   * width + NAME_WIDTH_DIMENSION + 15, for the width
   */
  public static final Dimension INT_EMPTY_FIELD_DIMENSION = new Dimension(
    NO_BUTTON_WIDTH_DIMENSION - 75, HEIGHT_DIMENSION); //234

  /**
   * Field dimension for properties without View, New and Select buttons
   */
  public static final Dimension
    NO_BUTTON_FIELD_DIMENSION = new Dimension(
      NO_BUTTON_WIDTH_DIMENSION, HEIGHT_DIMENSION);

  /**
   * Field dimension for properties with one button
   */
  public static final Dimension
    ONE_BUTTON_FIELD_DIMENSION = new Dimension(
      NO_BUTTON_WIDTH_DIMENSION - 45, HEIGHT_DIMENSION);

  /**
   * Field dimension for properties with two buttons
   */
  public static final Dimension
    TWO_BUTTONS_FIELD_DIMENSION = new Dimension(
        NO_BUTTON_WIDTH_DIMENSION - 90, HEIGHT_DIMENSION);

  /**
   * Field dimension for properties with three buttons
   */
  public static final Dimension
    THREE_BUTTONS_FIELD_DIMENSION = new Dimension(
        NO_BUTTON_WIDTH_DIMENSION - 135, HEIGHT_DIMENSION); //175
  
  /**
   * Width dimension for one JList scrollPane
   */
  public static final int LIST_WIDTH_DIMENSION = 240;

  /**
   * Height dimension for one JList scrollPane where 8 items are visible
   */
  public static final int LIST_HEIGHT_DIMENSION = 160;

    /**
     * A CommonPropertyEditor makes available a full custom Component
     * that edits its property value.  Override the PropertyEditor method.
     * @see java.beans.PropertyEditor#getCustomEditor
     * @return a JPanel that will allow the user to directly
     *      edit the current property value.
     */
    java.awt.Component getCustomEditor();

  /**
   * Gets the value of the display property name (not the true name, given by
   * the property descriptor).
   * @return the display name of the property.
   */
  public String getDisplayName();

  /**
   * Gets the value of the property type, that is the Class of the returned
   * object in getValue().
   * @return The type of the property.
   */
  public Class getPropertyType();

  /**
   * Gets the descriptor of the property.
   * @return The descriptor of the property.
   */
  public PropertyDescriptor getPropertyDescriptor();

  /**
   * Gets the method on the source for updates.
   * @return The method on the source for updates.
   */
  public Method getWriteMethod();

  /**
   * Get the prefered size for the rectangle into which the value is displayed,
   * in the component returned by PropertyEditor.getCustomEditor().
   * @return prefered size.
   */
  public Dimension getPreferedSizeForEditing();

  /**
   * Finalize the instance; the call is reserved to BeamerManager.
   */
  public void finalizeForProcess();


} // CommonPropertyEditor
