//Title:        Access Road
//Version:      0.7.1
//Copyright:    Copyright 2002, 2004, 2007-2012 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/


package ARoad0.Gui1;

import javax.swing.*;
import javax.swing.plaf.metal.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.net.*;

import ARoad0.Pattern.*;
import ARoad0.gDMak.*;
import ARoad0.Gui2.ViewTree;
import ARoad0.gWork.ACS_Facade;
import ARoad0.gWork.ViewInBase_Facade;
import ARoad0.gBase.BaseManagerImpl;
import ARoad0.gBase.NameImpl;

import ARoad0.gBaseInterface.Name;
import ARoad0.gBaseInterface.ViewInBase;
import ARoad0.gBaseInterface.EPRViewInBase;
import ARoad0.gBaseInterface.ImmutableNameISTree;

import ARoad0.ErrB.BaseError;
import ARoad0.ErrW.DialogError;

/**
* This class creates the Access Road desktop, and contains the main method.
* It displays the welcome message. It adds and resizes the internal frames.
* It uses other classes for managing the frame moving. Each Action class
* is in the gDMak package; it is referenced through one single instance
* for the event handling from the program main menu.
* See the <a href="{@docRoot}/copyright.html">Copyright</a>.
* @see ARoad0.Gui1.InternalFrame
* @see ARoad0.Gui1.IconPolice
* @see ARoad0.Gui1.InsideDesktop
* @version 0.7.1
*/
public class Desktop extends JFrame implements Singleton {

  // environment
    /**
     * after the init, CURRENT_DIR is the current directory with '/' as separator
     * whatever the operating system
     */
  public static String                CURRENT_DIR = System.getProperty("user.dir");
  public static final String          USER_DIR = System.getProperty("user.home");
  /*
   * URL_SEPAR is '/' even for Windows platforms since it is used only to
   * define universal paths for URI 
   */
  public static final String          FILES_SEPAR = System.getProperty("file.separator");
  public static final String          URL_SEPAR = "/";
  public static final String          LINE_SEPAR = System.getProperty("line.separator");
 
  /**
   * Initial file path, only for the development context ON LINUX
   */ 
  public static URI                   CURRENT_DIR_URI = null; 
  /**
   * AR_URI is the URI of the 'ARoad0/' directory, whatever the operating system
   */
  public static URI                   AR_URI = null;
  /**
   * AR_DIR is the path to ARoad0 directory in String, with the operating system file separator.
   * Used by ActionBasicTutorial, ActionHelpConcepts, BaseManagerImpl.updateAndSaveAcsAddonDescriptors()
   */
  public static String                AR_DIR = CURRENT_DIR+"build"+
                                        FILES_SEPAR+"classes"+FILES_SEPAR+"ARoad0";
  
  static {
      try {
          char files = FILES_SEPAR.charAt(0);
          char url = URL_SEPAR.charAt(0);
          if (FILES_SEPAR.length() != 1)
              throw new InternalError("Desktop initialization fails about FILES_SEPAR ="+
                      FILES_SEPAR);
          
          if (files != url)
              CURRENT_DIR = CURRENT_DIR.replace(files, url);
          
          CURRENT_DIR_URI = new URI(
              "file:///"+CURRENT_DIR+URL_SEPAR
              );
      }
      catch (URISyntaxException u) {
                    // for testing and tracking purpose
                  System.out.println("in Desktop static initialization: "+u.toString());
                  u.printStackTrace();
      } // catch
      
      AR_URI = CURRENT_DIR_URI.resolve("build"+URL_SEPAR+"classes"+URL_SEPAR+"ARoad0"+URL_SEPAR);

        //try {
                    // for testing and tracking purpose
                  //System.out.println("in Desktop static initialization: "+AR_URI.toURL());
        //} catch (MalformedURLException m) {
          //  throw new InternalError("AR_URI wrong initialization: "+m);
        //}
                  
  } // static
  
  public static boolean               developContext__ = true;

  protected static Desktop      instance__ ;
  protected static String       title__;
  protected static JDesktopPane desk__;

  JMenuBar                      menuBar = new JMenuBar();
  JMenu                         fileMenuE = new JMenu("File"); //MenuE for english menus
  JMenu                         editMenuE = new JMenu("Edit");
  JMenu                         windowMenuE = new JMenu("Window");
  JMenu                         goMenuE = new JMenu("Go");
  JMenu                         toolsMenuE = new JMenu("Tools");
  JMenu                         helpMenuE = new JMenu("Help");

  //JMenu                         exportImageMenuE = new JMenu("Export");

  JMenu                         newMenuE = new JMenu("New");
  JMenu                         openMenuE = new JMenu("Open");

  JToolBar                      toolBar1;

  /* for File menu */
  public ActionNewView actionNewView;
  public ActionNewACSyst actionNewACSyst;
  public ActionOpenACSyst actionOpenACSyst;
  public ActionOpenGraphicView actionOpenUserV;
  public ActionSave actionSave;
  public ActionSaveAll actionSaveAll;
  public ActionClose actionClose;
  public ActionRemove actionRemove;
  public ActionExit actionExit;
  public ActionImportACS actionImportACS;
  public ActionExportPNG actionExportPNG;
  public ActionCopy actionCopy;

  /* for Window menu */
  public ActionOpenMainFrame actionOpenMainFrame;
  public ActionOpenBeamer actionOpenBeamer;
  public InternalFrameTiler actionTile;
  public ActionOpenAllFullViews actionOpenAllFullViews;
  public ActionOpenAllNoThanViews actionOpenAllNoThanViews;
  //public ActionOpenArchitecture actionOpenArchitecture;
  public ActionOpenSketcher actionOpenSketcher;
  public ActionOpenGlobalStructure actionOpenGlobalStructure;

  /* for Help menu */
  public ActionAboutARoad actionAboutARoad;
  public ActionBasicTutorial actionBasicTutorial;
  public ActionHelpConcepts actionHelpConcepts;
  public ActionAcsAddonGuides actionAcsAddonGuides;

  // mnemonic for menus
  char   f = 'F';
  char   e = 'E';
  char   l = 'l';
  char   w = 'W';
  char   g = 'G';
  char   t = 'T';
  char   h = 'H';
  char   n = 'N';
  char   o = 'O';
  char   r = 'R';
  char   x = 'X';
  char   c = 'C';
  char   I = 'I';
  char   a = 'A';
  
  // shortcuts for file, window and edit menus - 'F' is reserved to the search function
  char[] fileShortcuts = { 'O', 'O', 'A', 'S', 'E' }; // O unused
  char[] editShortcuts = { 'Z', 'Y', 'X', 'C', 'V' };
  char[] windowShortcuts = { 'I', 'U', 'M', 'N', 'K', 'B', 'T' };
  char[] helpShortcuts = {'H', 'G', 'A'};

  /* for comboxes in toolbar */
  JComboBox comboI, comboU;// comboA,
  
  /**
   * Font size
   */
  public static final int             FONT_SIZE = 10;
  /**
   * Italic font
   */
  public static final Font            ITALIC_FONT = new Font(
    "Serif", Font.ITALIC, FONT_SIZE);
  /**
   * Plain font
   */
  public static final Font            PLAIN_FONT = new Font(
    "Serif", Font.PLAIN, FONT_SIZE);
  /**
   * Bold font with FONT_SIZE + 1 as size
   */
  public static final Font            BOLD_FONT = new Font(
    "Serif", Font.BOLD, FONT_SIZE + 1);  

  static {
      if (CURRENT_DIR.length() < 60) {
          title__ = new String("Access Road 0.7.1... at "+CURRENT_DIR);
      }
      else {
          String partialDir = CURRENT_DIR.substring(CURRENT_DIR.length() - 60);
          int slash = partialDir.indexOf("/");
          if (slash != -1) {
              title__ = new String("Access Road 0.7.1... at ~"+partialDir.substring(slash));
          }
          else {
              title__ = new String("Access Road 0.7.1... at ~"+partialDir);
          }
          partialDir = null;
      } // else
  } // static

  /**
  * Protected constructor following the singleton pattern.
  */
  protected Desktop() throws Exception {

      // build the frame
    try {
      init();
    }
    catch (Exception e) {
      throw new InternalError("in the Gui1.Desktop constructor,"+
          "the opening of the main frame doesn't work - "+ e.toString() );
    }

  } //constructor()

  /**
  * The only method to get a desktop object.
  * (singleton class, so the constructor is protected)
  */
  public static synchronized Desktop getInstance(){
    return instance__;
  } // getInstance()

  /**
  * The main window is initialized there, then setVisible(true) is called, so
  * this is the only method for the GUI which can be called outside the Event Dispatch Thread.
  * @exception Exception
  */
  private void init() throws Exception {

                                // for testing and tracking purpose
                              //System.out.println("starting of Desktop.init()");

    JMenuItem item;

    setTitle(title__);

    setBackground(InternalFrame.BACKCOLOR);
    setForeground(InternalFrame.FORECOLOR);

    // shortcuts in the main menu
    fileMenuE.setMnemonic(f);
    editMenuE.setMnemonic(e);
    windowMenuE.setMnemonic(w);
//    goMenuE.setMnemonic(g);
//    toolsMenuE.setMnemonic(t);
    helpMenuE.setMnemonic(h);

    // Groups in the fileMenu the JMenuItems
    // (and a JMenu for New and ExportImage),
    // with sometimes keyboard accelerators from fileShortcuts
    for (int i = 0; i < 10; i++) {
      switch (i) {
        case 0:
          item = fileMenuE.insert(newMenuE, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setMnemonic(n);
          break;
        case 1:
          item = fileMenuE.insert(openMenuE, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setMnemonic(o);
          break;
        case 2:
          actionClose = ActionClose.getInstance();
          item = fileMenuE.insert(actionClose, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setMnemonic(l);
          break;
        case 3:
          actionRemove = ActionRemove.getInstance();
          item = fileMenuE.insert(actionRemove, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setMnemonic(r);
          break;
        case 4:
          actionSave = ActionSave.getInstance();
          item = fileMenuE.insert(actionSave, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(fileShortcuts[2],
            java.awt.Event.CTRL_MASK, false));
          break;
         /*
        case 2:
          actionSaveAs = ActionSaveAs.getInstance();
          item = fileMenuE.insert(actionSaveAs, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          break;
          */
        case 5:
          actionSaveAll = ActionSaveAll.getInstance();
          item = fileMenuE.insert(actionSaveAll, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(fileShortcuts[3],
            java.awt.Event.CTRL_MASK, false));
          break;
        case 6:
          actionCopy = ActionCopy.getInstance();
          item = fileMenuE.insert(actionCopy, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setMnemonic(c);
          break;
        case 7:
          actionImportACS = ActionImportACS.getInstance();
          item = fileMenuE.insert(actionImportACS, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setMnemonic(I);
          break;
        case 8:
          //item = fileMenuE.insert(exportImageMenuE, i);

          actionExportPNG = ActionExportPNG.getInstance();
          item = fileMenuE.insert(actionExportPNG, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(fileShortcuts[4],
            java.awt.Event.CTRL_MASK, false));
          break;
/*        case 7: ActionExportPNG
          actionPrint = ARoad0.gDMak.ActionPrint.getInstance();
          item = fileMenuE.insert(actionPrint, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          break;
        case 8:
          actionPrintSetUp = ARoad0.gDMak.ActionPrintSetUp.getInstance();
          item = fileMenuE.insert(actionPrintSetUp, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          break;
*/
        case 9:
          actionExit = ARoad0.gDMak.ActionExit.getInstance();
          item = fileMenuE.insert(actionExit, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setMnemonic(x);
          break;
      } // switch
    } // for

    // Groups in File/newMenuE the JMenuItems
    for (int i=2; i < 4; i++) {
      switch (i) {
/*        case 0:
          actionNewInfSyst = ARoad0.gDMak.ActionNewInfSyst.getInstance();
          item = newMenuE.insert(actionNewInfSyst, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          break;
          case 0:
          actionNewResource = ARoad0.gDMak.ActionNewResource.getInstance();
          item = newMenuE.insert(actionNewResource, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          break;
          case 1:
          actionNewEligibleParty = ARoad0.gDMak.ActionNewEligibleParty.getInstance();
          item = newMenuE.insert(actionNewEligibleParty, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          break; */
          case 2:
          actionNewACSyst = ARoad0.gDMak.ActionNewACSyst.getInstance();
          item = newMenuE.insert(actionNewACSyst, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          break;
          case 3:
          actionNewView = ARoad0.gDMak.ActionNewView.getInstance();
          item = newMenuE.insert(actionNewView, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          break;
      } // switch
    } // for
    newMenuE.setBackground(InternalFrame.BACKCOLOR_2);

    // Groups in File/openMenuE the JMenuItems
    for (int i=0; i < 2; i++) {
      switch (i) {
        case 0:
          actionOpenACSyst = ARoad0.gDMak.ActionOpenACSyst.getInstance();
          item = openMenuE.insert(actionOpenACSyst, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          break;
        case 1:
          actionOpenUserV = ARoad0.gDMak.ActionOpenGraphicView.getInstance();
          item = openMenuE.insert(actionOpenUserV, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          break;
      } // switch
    } // for
    openMenuE.setBackground(InternalFrame.BACKCOLOR_2);

    /* Groups in File/exportImageMenuE the JMenuItems */
    //for (int i=0; i < 1; i++) {
      //switch (i) {
        //case 0:
          //actionExportPNG = ActionExportPNG.getInstance();
          //item = exportImageMenuE.insert(actionExportPNG, i);
          //item.setBackground(InternalFrame.BACKCOLOR_2);
          //break;
        
        //case 1:
          //actionExportRTF = ActionExportRTF.getInstance();
          //item = exportImageMenuE.insert(actionExportRTF, i);
          //item.setBackground(InternalFrame.BACKCOLOR_2);
          //break;
        
      //} // switch
    //} // for

    /* Groups in the editMenu the JMenuItems,
     * with sometimes keyboard accelerators from editShortcuts
     */
    /*
    for (int i=0; i < 1; i++) {
      switch (i) {
        case 0:
          actionUndo = ActionUndo.getInstance();
          item = editMenuE.insert(actionUndo, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(editShortcuts[i],
            java.awt.Event.CTRL_MASK, false));
          break;
        case 1:
          actionRedo = ActionRedo.getInstance();
          item = editMenuE.insert(actionRedo, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(editShortcuts[i],
            java.awt.Event.CTRL_MASK, false));
          break;
        case 2:
          actionCut = ActionCut.getInstance();
          item = editMenuE.insert(actionCut, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(editShortcuts[i],
            java.awt.Event.CTRL_MASK, false));
          break;
         * 
         * 
       case 0:
          actionCopy = ActionCopy.getInstance();
          item = editMenuE.insert(actionCopy, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(editShortcuts[i],
            java.awt.Event.CTRL_MASK, false));
          break;
         * 
         * 
       case 4:
          actionPaste = ActionPaste.getInstance();
          item = editMenuE.insert(actionPaste, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(editShortcuts[i],
            java.awt.Event.CTRL_MASK, false));
          break;
        case 5:
          actionDelete = ActionDelete.getInstance();
          item = editMenuE.insert(actionDelete, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          break;
        case 6:
          actionSelectAll = ActionSelectAll.getInstance();
          item = editMenuE.insert(actionSelectAll, i);
          item.setBackground(InternalFrame.BACKCOLOR_2);
          break;

      } // switch
    } // for
     * */

    // Puts in the windowMenu the JMenuItems
    // with sometimes keyboard accelerators from windowShortcuts 
    for (int i=0; i < 8; i++) {
      switch (i) {
        case 0:
          actionOpenGlobalStructure = ActionOpenGlobalStructure.getInstance();
          item = windowMenuE.insert(actionOpenGlobalStructure, i );
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(windowShortcuts[i],
            java.awt.Event.CTRL_MASK, false));
          break;
        case 1:
          actionOpenMainFrame = ActionOpenMainFrame.getInstance();
          item = windowMenuE.insert(actionOpenMainFrame, i );
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(windowShortcuts[i],
            java.awt.Event.CTRL_MASK, false));
          break;
        case 2:
          actionOpenAllFullViews = ActionOpenAllFullViews.getInstance();
          item = windowMenuE.insert(actionOpenAllFullViews, i );
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(windowShortcuts[i],
            java.awt.Event.CTRL_MASK, false));
          break;
        case 3:
          actionOpenAllNoThanViews = ActionOpenAllNoThanViews.getInstance();
          item = windowMenuE.insert(actionOpenAllNoThanViews, i );
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(windowShortcuts[i],
            java.awt.Event.CTRL_MASK, false));
          break;
        case 4:
          actionOpenSketcher = ActionOpenSketcher.getInstance();
          item = windowMenuE.insert(actionOpenSketcher, i );
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(windowShortcuts[i],
            java.awt.Event.CTRL_MASK, false));
          break;
        //case 5:
          //actionOpenArchitecture = ActionOpenArchitecture.getInstance();
          //item = windowMenuE.insert(actionOpenArchitecture, i );
          //item.setBackground(InternalFrame.BACKCOLOR_2);
          //item.setAccelerator(KeyStroke.getKeyStroke(windowShortcuts[i],
            //java.awt.Event.CTRL_MASK, false));
          //break;
        case 5:
          actionOpenBeamer = ActionOpenBeamer.getInstance();
          item = windowMenuE.insert(actionOpenBeamer, i );
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(windowShortcuts[i],
            java.awt.Event.CTRL_MASK, false));
          break;
        case 6:
          actionTile = InternalFrameTiler.getInstance();
          item = windowMenuE.insert(actionTile, i );
          item.setBackground(InternalFrame.BACKCOLOR_2);
          item.setAccelerator(KeyStroke.getKeyStroke(windowShortcuts[i],
            java.awt.Event.CTRL_MASK, false));
          break;
      } // switch
    } // for

    // Put in the helpMenu the JMenuItems
    // with sometimes keyboard accelerators from helpShortcuts
    for (int i=0; i < 4; i++) {
      switch (i) {
        case 0:
            actionHelpConcepts = ARoad0.gDMak.ActionHelpConcepts.getInstance();
            item = helpMenuE.insert(actionHelpConcepts, 0);
            item.setBackground(InternalFrame.BACKCOLOR_2);
            item.setAccelerator(KeyStroke.getKeyStroke(helpShortcuts[0],
              java.awt.Event.CTRL_MASK, false));
            break;
        case 1:
            actionBasicTutorial = ARoad0.gDMak.ActionBasicTutorial.getInstance();
            item = helpMenuE.insert(actionBasicTutorial, 1);
            item.setBackground(InternalFrame.BACKCOLOR_2);
            item.setAccelerator(KeyStroke.getKeyStroke(helpShortcuts[1],
              java.awt.Event.CTRL_MASK, false));
            break;
        case 2:
            actionAcsAddonGuides = ARoad0.gDMak.ActionAcsAddonGuides.getInstance();
            item = helpMenuE.insert(actionAcsAddonGuides, 2);
            item.setBackground(InternalFrame.BACKCOLOR_2);
            item.setAccelerator(KeyStroke.getKeyStroke(helpShortcuts[2],
              java.awt.Event.CTRL_MASK, false));
            break;
        case 3:
            actionAboutARoad = ARoad0.gDMak.ActionAboutARoad.getInstance();
            item = helpMenuE.insert(actionAboutARoad, 3);
            item.setBackground(InternalFrame.BACKCOLOR_2);
            break;
      } // switch
    } // for i

    // Creates Help menu
    //actionHelpDoc = ARoad0.gDMak.ActionHelpDoc.getInstance();
    //item = helpMenuE.insert(actionHelpDoc, 2);
    //item.setBackground(InternalFrame.BACKCOLOR_2);
    //item.setAccelerator(KeyStroke.getKeyStroke(helpShortcuts[0],
      //java.awt.Event.CTRL_MASK, false));


    
    // Create the menuBar and the toolBar
    menuBar.add(fileMenuE);
//    menuBar.add(editMenuE);
    menuBar.add(windowMenuE);
//    menuBar.add(goMenuE);
//    menuBar.add(toolsMenuE);
    menuBar.add(helpMenuE);
    setJMenuBar(menuBar);

    //
    // set the toolBar's buttons with tooltips
    //
    toolBar1 = new JToolBar();

/*    buttonB = toolBar1.add(ActionSaveAs);
    buttonB.setActionCommand((String)ActionSaveAs.getValue(Action.NAME));
    buttonB.setToolTipText((String)ActionSaveAs.getValue(Action.NAME));
    buttonB.setText("");
    buttonB = toolBar1.add(actionPaste);
    buttonB.setActionCommand((String)actionPaste.getValue(Action.NAME));
    buttonB.setToolTipText((String)actionPaste.getValue(Action.NAME));
    buttonB.setText("");
*/
    /* set the toolBar's comboBoxes */
/*    toolBar1.addSeparator();
    String[] listCombo1 = { "Info. System 1", "Info. System 2" }; // to change
    comboI = new JComboBox(listCombo1);
    comboI.setMaximumSize(new Dimension(150, 40));
    comboI.setAlignmentY(0);
    comboI.setBackground(InternalFrame.BACKCOLOR);
    actionOpenInfSyst = ActionOpenInfSyst.getInstance();
    comboI.addActionListener(actionOpenInfSyst);
    comboI.setActionCommand((String)actionOpenInfSyst.getValue(Action.NAME));
    toolBar1.add(comboI);

  toolBar1.addSeparator();
    String[] listCombo2 = { "default_ACS resources", "default_ACS accounts" }; // to change
    comboA = new JComboBox(listCombo2);
    comboA.setMaximumSize(new Dimension(150, 40));
    comboA.setAlignmentY(0);
    comboA.setBackground(InternalFrame.BACKCOLOR);
    // no [actionOpenACSyst = ActionOpenACSyst.getInstance();]
    //  because it's done for menuFile/Open
    comboA.addActionListener(actionOpenACSyst);
    comboA.setActionCommand((String)actionOpenACSyst.getValue(Action.NAME));
    toolBar1.add(comboA);

    toolBar1.addSeparator();
    String[] listCombo3 = { "User View 1", "User View 2" }; // to change
    comboU = new JComboBox(listCombo3);
    comboU.setMaximumSize(new Dimension(150, 40));
    comboU.setAlignmentY(0);
    comboU.setBackground(InternalFrame.BACKCOLOR);
    actionOpenUserV = ActionOpenGraphicView.getInstance();
    comboU.addActionListener(actionOpenUserV);
    comboU.setActionCommand((String)actionOpenUserV.getValue(Action.NAME));
    toolBar1.add(comboU);
*/
    getContentPane().add(toolBar1, BorderLayout.NORTH);

    //
    // color
    //
    menuBar.setBackground(InternalFrame.BACKCOLOR_2);
    menuBar.setForeground(InternalFrame.FORECOLOR);
    toolBar1.setBackground(InternalFrame.BACKCOLOR_2);
    toolBar1.setForeground(InternalFrame.FORECOLOR);
    fileMenuE.setBackground(InternalFrame.BACKCOLOR_2);
//    editMenuE.setBackground(InternalFrame.BACKCOLOR_2);
    windowMenuE.setBackground(InternalFrame.BACKCOLOR_2);
//    goMenuE.setBackground(InternalFrame.BACKCOLOR_2);
//    toolsMenuE.setBackground(InternalFrame.BACKCOLOR_2);
    helpMenuE.setBackground(InternalFrame.BACKCOLOR_2);

    // Creates a desktop and set it as the content pane
    desk__ = new JDesktopPane();
    getContentPane().add(desk__, BorderLayout.CENTER);

    // Installs the custom desktop manager and the handling of closing
    desk__.setDesktopManager(InsideDesktop.getInstance());
    addWindowListener(new WindowAdapter(){
      public void windowClosing(WindowEvent e) {
        ActionExit.getInstance().actionPerformed(
            new ActionEvent(this,0,"main window") );
      }
    } );

    // color
    desk__.setBackground(InternalFrame.BACKCOLOR);
    desk__.setForeground(InternalFrame.FORECOLOR);

    // Size the window
    setSize(1200, 800);
    setVisible(true);


                                // for testing and tracking purpose
                              //System.out.println("end of Desktop.init()");
  } // init()

  /**
   * Adds a JInternalFrame in the main window.
   * @param _f JInternalFrame
   * @param _i Integer
   */
  public static synchronized void addFrame(JInternalFrame _f, Integer _i) {
    desk__.add(_f, _i);
    desk__.validate(); //useful? see javadoc on JDesktop.add()
  }

  /**
   * Get the open windows.
   * Called by GraphicViewBaseListenerImpl.changeFrameView(), updateFrameViewRights().
   * @return JInternalFrame[] array of the open windows
   */
  public static synchronized JInternalFrame[] getCurrentFrames() {
    return desk__.getAllFrames();
  }
  
  /**
   * Get the desktop pane of the main window.
   * @return JDesktopPane
   */
  public static JDesktopPane getDesktopPane() {
    return desk__;
  }

  /**
  * Resizes a frame
  * @param _f JInternalFrame
  * @param _x int
  * @param _y int
  * @param _w int width
  * @param _h int height of the frame
  */
  public static synchronized void resizeIFrame(JInternalFrame _f, int _x, int _y, int _w, int _h) {
    InsideDesktop.getInstance().setBoundsForFrame(_f, _x, _y, _w, _h);
  }

  /**
  * Gets the size of the main window
  * @return Dimension
  */
  public synchronized static Dimension getDSize() {
    return desk__.getSize();
  } // getDSize() of the desktop

  /**
 * Restores the positions of the last open frames, from the GUI configuration map.
 * Called by the method main.
 * @see ARoad0.gDMak.ActionSaveAll
 */
  public static void restoreLastGUI() {

    // sets the desktop size and the open objects from the configuration file
    Map m_GUIconfig = null;

    try {
        m_GUIconfig = BaseManagerImpl.getInstance().getGUI();
    } catch (BaseError b) {
        throw new InternalError(" From the restoring of the last GUI - "+b);
    }
    if (m_GUIconfig == null) return;
    
    // sets the desktop size
    //if (m_GUIconfig.get( new NameImpl("<Desktop>") ) != null) {
      //  instance__.setSize((Dimension) m_GUIconfig.get( new NameImpl("<Desktop>") ));
    //}

    if (m_GUIconfig.get( new NameImpl("<Last GUI>") ) == null) {
        m_GUIconfig = null;
        return;
    }

    // initialization
    Map m_LastConfig = null;
    Map views_map = null;
    Name name = null;
    Object obj = null;
    ViewInBase view = null;
    JInternalFrame frame = null;
    ImmutableNameISTree nametree = null;
    
    m_LastConfig = (Map)m_GUIconfig.get( new NameImpl("<Last GUI>") );

    try {
         // opens the acs and the explorer, and the beamer too
        for (Iterator ite = m_LastConfig.keySet().iterator(); ite.hasNext(); ) {
            obj = ite.next();
            if (obj instanceof String) { // in m_LastConfig, all String keys are from Names
                name = NameImpl.getNameFromString((String)obj);
                if (ACS_Facade.knowsACS(name) && !ACS_Facade.isOpenACS(name)) {
                    // opens first all the parent ACS
                    ActionOpenACSyst.getInstance().openACS(new Object[]{name});
                } // if (ACS_Facade.knowsACS(name)
            } // if String
        } // for

                                // for testing and tracking
                                //System.out.println("In Desktop.restoreLastGUI(), "+
                                  //      "after the opening of ACS, list of the opened ACS:");
                                //for (Iterator ite = ACS_Facade.getOpenACS().entrySet().iterator(); ite.hasNext(); ) {
                                  //  Map.Entry entry = (Map.Entry) ite.next();
                                  //  System.out.println("----------- "+entry.getKey());
                                //} // for ite

        // opens the acs main frames if any, the IS structure and the sketcher
        for (Iterator ite = m_LastConfig.keySet().iterator(); ite.hasNext(); ) {
            obj = ite.next();
            if (obj instanceof Name ) {
                if (ACS_Facade.knowsACS((Name)obj)) {
                    frame = ActionOpenMainFrame.openACSmainFrame((Name)obj, m_GUIconfig);
                } else if ( ((Name)obj).lastComponent().equals("<Sketcher>") ) {
                    // open with an empty acs
                    ActionOpenSketcher.openSketcher(new Object[] {"acs", null, null});
                } else if ( ((Name)obj).equals(ActionOpenGlobalStructure.
                        LOCATION_STRUCTURE_NAME) ) {
                    ActionOpenGlobalStructure.getInstance().openFrame(m_GUIconfig);
                } // else
            } // if (obj instanceof Name )
        } // for

        // opens the views and put them in the explorer
        // but does not open their main frames or tabs.
        // opens the Architecture view and its frame.
        for (Iterator ite = m_LastConfig.keySet().iterator(); ite.hasNext(); ) {
            obj = ite.next();
            if (obj instanceof String) { // in m_LastConfig, all String keys are from Names
                name = NameImpl.getNameFromString((String)obj);

                if (ViewInBase_Facade.knowsView(name)) {
                    if (!name.equals(ActionOpenArchitecture.ARCHI_VIEW_NAME)) {
                        if (!ViewInBase_Facade.isNoThanViewName(name))
                            view = ViewInBase_Facade.openOrCreateEPRView(
                                name, null, null);
                        else
                            view = ViewInBase_Facade.openOrCreateNoThanView(
                                name, null);
                        
                        ViewTree.createViewExplorer(view);
                    } else {

                                // for testing and tracking purpose
                              //System.out.println("in Desktop.restoreLastGUI(), "+
                                //"Architecture view is detected " );
                        
                        // opens the 'Architecture' view and frame - unused
                        if (!ACS_Facade.getOpenACS().isEmpty()) 
                          ActionOpenArchitecture.openArchitecture(
                            ACS_Facade.getOpenACS(), m_GUIconfig);
                    } // else
                } // if
            } // if String
        } // for 
        
        // opens the AllFullViews frame
        if (m_LastConfig.containsKey(new NameImpl("<AllFullViews>") )) {
            views_map = ViewInBase_Facade.getOpenFullViews();
            // open the frame and use m_LastConfig for restoring the dispositions
            if (!views_map.isEmpty()) {
                ActionOpenAllFullViews.getInstance().openAllFullViews(
                        views_map,
                        m_GUIconfig);
            } // if
        } else { 
            // opens the full view main frames and 'See why' frames
            for (Iterator ite = m_LastConfig.keySet().iterator(); ite.hasNext(); ) {
                obj = ite.next();

                if (obj instanceof Name && ViewInBase_Facade.knowsView((Name)obj) &&
                        !ViewInBase_Facade.isNoThanViewName((Name)obj) &&
                        !((Name)obj).equals(ActionOpenArchitecture.ARCHI_VIEW_NAME)) {
                    // restore the graphic view, main frame bounds, 'See why' frame
                    // if they exists in m_LastConfig
                    ActionOpenGraphicView.openViewInGui(
                            (EPRViewInBase)ViewInBase_Facade.getAllStandardOpenViews().get((Name)obj),
                            false, m_GUIconfig);
                } // if    

            } // for                     
        } // else
            
        // opens the AllNoThanViews frame
        if (m_LastConfig.containsKey(new NameImpl("<AllNoThanViews>") )) {
            views_map = ViewInBase_Facade.getOpenNoThanViews();
            // open the frame and use m_LastConfig for restoring the dispositions
            if (!views_map.isEmpty()) {
                ActionOpenAllNoThanViews.getInstance().openAllNoThanViews(
                        views_map,
                        m_GUIconfig);
            } // if
        } else { 
            // opens the NoThan view main frames and 'See why' frames
            for (Iterator ite = m_LastConfig.keySet().iterator(); ite.hasNext(); ) {
                obj = ite.next();

                if (obj instanceof Name && ViewInBase_Facade.knowsView((Name)obj) &&
                        ViewInBase_Facade.isNoThanViewName((Name)obj) &&
                        !((Name)obj).equals(ActionOpenArchitecture.ARCHI_VIEW_NAME)) {
                    // restore the graphic view, main frame bounds, 'See why' frame
                    // if they exists in m_LastConfig
                    ActionOpenGraphicView.openViewInGui(
                            (EPRViewInBase)ViewInBase_Facade.getAllStandardOpenViews().get((Name)obj),
                            false, m_GUIconfig);
                } // if    

            } // for                     
        } // else

    } catch (DialogError d) {
        d.printStackTrace();
        throw new InternalError("in Desktop.restoreLastGUI():"+d);         
    } catch (BaseError b) {
        b.printStackTrace();
        throw new InternalError("in Desktop.restoreLastGUI():"+b);        
    }
  } // restoreLastGUI()
  
  /**
 * Main method of the application. Test the java version and the state
 * development/production of the environment. Select the look and feel Metal.
 * Set the background image and restore the last GUI configuration, in a sequence
 * of calls to the Event Dispatch Thread which provides a smooth starting of the GUI.
 * @param args is null
 */
  public static void main(String args[]) throws Exception {

                                // for testing and tracking purpose
                              //System.out.println("starting of Desktop.main()");

    String vers = System.getProperty("java.version");
    if (vers.compareTo("1.6") < 0) {
      System.out.println("Sorry, Access Road must be run on a " +
                             "1.6 or higher version Java Virtual Machine.");
    }  // if

    // variables
    JEditorPane     pane = null;
    ImageIcon       icon = null;
    URI             uri = null;
    
    // Force AccessRoad to come up in the Metal L&F
    try {
      UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
      // If you want the System L&F instead, comment out the above line and
      // uncomment the following:
      //UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (Exception e) {
      System.out.println("Error loading LookAndFeel: " + e);
    } // catch

    // set foreGround and backGround JInternalFrame's color in something special
    MetalLookAndFeel.setCurrentTheme(new MexicTheme());

    // initialize the right file path, since in the development context, there is
    // a 'build' directory in the file path which is not there in the user context.
    // developContext__ is true at the init.
    try {
      pane = new JEditorPane();
      pane.setPage(CURRENT_DIR_URI.resolve("build"+URL_SEPAR+
              "classes"+URL_SEPAR+"ARoad0").toURL());
    } catch (java.io.FileNotFoundException m){
      // there is no 'build' folder under the current folder if
        // the program runs for an user, so AR_URI has to be reset
      AR_URI = CURRENT_DIR_URI.resolve("classes"+URL_SEPAR+"ARoad0"+URL_SEPAR);
      AR_DIR = System.getProperty("user.dir")+FILES_SEPAR+"classes"+FILES_SEPAR+"ARoad0";

                    // for testing and tracking purpose
                  //System.out.println("in Desktop.main(), ARoad0 folder path: "+
                    //      AR_URI.toString()+" and AR_DIR: "+AR_DIR.toString());

      //DialogBox.showAlarmMessage(
        // uri.toURL()+URL_SEPAR+"build"+URL_SEPAR+"classes"+URL_SEPAR+"ARoad0"+
         //" AND THE RESULT OF resolve(): "+
         //uri.resolve("build"+URL_SEPAR+"classes"+URL_SEPAR+"ARoad0").toURL());
      
      developContext__ = false;
    } // catch

     // create the main window, calls init()
    SwingUtilities.invokeAndWait(new Runnable(){
            public void run(){
                    try {
                        instance__ = new Desktop();
                    } catch (Exception e) {
                        System.out.println("Error in creating Desktop: " + e);
                    }
            }
    });

    try {
      // test the FileNotFoundException 
      pane = new JEditorPane();
      pane.setPage(AR_URI.resolve("Gui1"+URL_SEPAR+"images").toURL());
    } catch (java.io.FileNotFoundException m){
      // there is no 'ARoad0' folder under the current folder,
        // so it is an error in the program installation
      DialogBox.showAlarmMessage(
         "The Access Road program has not been properly installed."+
         LINE_SEPAR+LINE_SEPAR+
         "The folder 'images' cannot be found under the folder "+AR_URI.toURL()+
         "Gui1"+". Check your configuration and read the installation guide.");
      return;
    } // catch

    // add a background image and an icon
    try {
        icon = new ImageIcon(AR_URI.resolve("Gui1"+URL_SEPAR+"images"+URL_SEPAR+
            "background_desktop.jpg").toURL());
    } catch (MalformedURLException m) {
        throw new InternalError("Desktop.main() with exception "+m);
    }
    final JLabel la = new JLabel(icon);
    la.setBounds(0, 0, icon.getIconWidth(), icon.getIconHeight());

    uri = AR_URI.resolve("Gui1"+URL_SEPAR+"images"+URL_SEPAR+"acs.gif");
    final ImageIcon icon2 = new ImageIcon(uri.toURL());    
    
    SwingUtilities.invokeAndWait(new Runnable(){
            public void run(){

                desk__.add(la);

                instance__.setIconImage(icon2.getImage());

            } // run
    }); // invokeAndWait()
    
    SwingUtilities.invokeAndWait(new Runnable(){
            public void run(){
                
                Map m_GUIconfig = null;

                try {
                    m_GUIconfig = BaseManagerImpl.getInstance().getGUI();
                } catch (BaseError b) {
                    throw new InternalError("in Desktop.main(), error: "+b);
                }
                if (m_GUIconfig == null) return;

                // set the desktop size
                if (m_GUIconfig.get( new NameImpl("<Desktop>") ) != null) {
                    instance__.setSize((Dimension) m_GUIconfig.get( new NameImpl("<Desktop>") ));
                }

            } // run
    }); // invokeAndWait()
    
    SwingUtilities.invokeAndWait(new Runnable(){
            public void run(){
                // set the open objects from the configuration file
                restoreLastGUI();

            } // run
    }); // invokeAndWait()

    SwingUtilities.invokeAndWait(new Runnable(){
            public void run(){

                // display an information message about the project and the license
                DialogBox.showOkMessage(
                  "Welcome to Access Road."+
                  LINE_SEPAR+LINE_SEPAR+"Access Road is a professional software "+LINE_SEPAR+
                  "  to simulate access controls and roles in IT systems,"+LINE_SEPAR+
                  "    for checking their compliance with the security policy,"+LINE_SEPAR+
                  "      to communicate the results to the right people,"+LINE_SEPAR+
                  "        to learn access controls and the best practices."+LINE_SEPAR+LINE_SEPAR+
                  "Access Road is a free software with a GNU Affero General Public License version 3."+
                  " You are welcome to redistribute it and/or modify it under certain conditions,"+
                  " but WITH ABSOLUTELY NO WARRANTY, including warranty about the rights simulation."+
                  LINE_SEPAR+"See the license agpl.txt under the ARoad0 source directory."+
                  LINE_SEPAR+LINE_SEPAR+"Contact: patrick.thazard /AT/ accessroad /point/ info"+
                  LINE_SEPAR+LINE_SEPAR+"Copyright 1999-2012 ACCBEE for "+
                  "all the code and all the documentation");
            } // run
    }); // invokeAndWait()

    pane = null;
    icon = null;
    uri = null;

  } // main(String)

}//Desktop()

