//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui1;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.beans.*;
import java.util.Set;
import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;
import java.net.MalformedURLException;
import javax.swing.border.BevelBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
//import javax.swing.JTextPane;
//import javax.swing.text.Document;
//import javax.swing.text.SimpleAttributeSet;
//import javax.swing.text.BadLocationException;
//import javax.swing.text.StyleConstants;

import ARoad0.Pattern.*;
import ARoad0.ErrW.*;
import ARoad0.Gui2.TreeManager;
import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBase.NameImpl;
import ARoad0.gBase.UtilityImpl;
import ARoad0.gBaseInterface.Name;
import ARoad0.gBaseInterface.ImmutableName;

/**
 * This class is responsible to built up the short dialog boxes
 * in the desktop. According to the Model-View-Controller pattern, it is a View
 * used by BeamerManager, CommonPropertyEditor and its subclasses, BadseObjectHandler,
 * and numerous Action classes. The main methods are: <P>
 * - showListDialog() to display a list where the user clicks on its choices <P>
 * - showSimpleSelectionDialog() does the same job with check boxes <P>
 * - showSingleEditor() to display a field where the user enters one text then returns <P>
 * - showExplorerSelection Dialog() is a powerfull method to select objects in the explorer <P>
 * - showBackedJComponents() to display complex panels managed by the caller <P>
 * - showBackedJLists() to display a set of lists for an user choice <P>
 * - showImmutableStringMap() to display a String map without action on it <P>
 * - several methods to display a message <P>
 * @version 0.7.0
 * @since 0.5.0
 */
public class DialogBox implements Singleton, TreeSelectionListener {

  // for List
  private static final int  INITIAL_CAPACITY = 5; // the default value: 101

  // the only instance of the class, following singleton pattern
  private static DialogBox      instance__ = new DialogBox();

  // variables for the dialog
  protected static JDesktopPane desk__ ;

  // variables for showExplorerSelectionDialog() and the beamer
  protected static boolean      waitClickInExplorer__ = false;
  protected static boolean      hasHandlerDialog__ = false;
      /**
       * updated by showExplorerSelectionDialog() and used by valueChanged()
       * map of Class (as key, instance of BaseObject) and JList (as value).
       * each JList is associated to a DefaultListModel.
       */
  protected static final Map    selectedObjectsSetMap__ = new HashMap(INITIAL_CAPACITY);
  protected static int          selectedObjectsCount__ = 0;
  protected static boolean      onlyOneSelectionAllowed__ = false;

        /**
         * handled by showImmutableStringMap() and the static nested class
         * KeyListSelectionListener.
         */
  protected static JList        keyList__ = null;
  protected static DefaultListModel        valuesModel__ = null;
  protected static Map<String, List<String>> m_values__ = null;

        /**
         * nested static class for showImmutableStringMap()
         */
  static class KeyListSelectionListener implements ListSelectionListener {
      public void valueChanged(ListSelectionEvent e) {
        int index = keyList__.getSelectedIndex();
        valuesModel__.clear();
        for (String value : m_values__.get(keyList__.getModel().getElementAt(index))){
            valuesModel__.addElement(value);
        }
      } // valueChanged()
  } // static class KeyListSelectionListener

  /**
  * only one private constructor
  */
  public DialogBox() {
  }

  /**
  * Only method to get an instance.
  * @return DialogBox
  */
  public static synchronized DialogBox getInstance(){
    return instance__;
  } // getInstance()

/**
  * Displays an OK information message following the Access Road standard.
  * @param _msg message to display
  */
  public static synchronized void showOkMessage(String _msg) {

    // variables for the dialog box
    JDialog     dialog;
    JOptionPane pane;
    JPanel      panel = new JPanel();
    JTextArea   textArea;
    JScrollPane scrollPane;

                    // for testing and tracking
                    //System.out.println("DialogBox.showOkMessage(), _msg.length(): "+
                      //      _msg.length());

    // initialization
    desk__ = Desktop.getDesktopPane(); // can't be initialized before,
                                                     // when Desktop invokes this class
    panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS) );
    panel.setBackground(InternalFrame.BACKCOLOR_2);
    panel.setForeground(InternalFrame.FORECOLOR);

    if (_msg.length() < 80) textArea = new JTextArea(_msg, 2, 35);
    else if (_msg.length() > 500) textArea = new JTextArea(_msg, 22, 35);
    else if (_msg.length() > 160) textArea = new JTextArea(_msg, 18, 35);
    else textArea = new JTextArea(_msg, 5, 35);
    textArea.setLineWrap(true);
    textArea.setWrapStyleWord(true);
    textArea.setAlignmentX(textArea.CENTER_ALIGNMENT);
    textArea.setBackground(InternalFrame.BACKCOLOR_2);
    textArea.setForeground(InternalFrame.FORECOLOR_BLACK);
    textArea.setEditable(false);

    scrollPane = new JScrollPane(textArea);
    panel.add( scrollPane );

      // creates the dialog box
    pane = new JOptionPane(
      panel,
      JOptionPane.INFORMATION_MESSAGE,
      JOptionPane.DEFAULT_OPTION);

      // color
    pane.setBackground(InternalFrame.BACKCOLOR_2);
    pane.setForeground(InternalFrame.FORECOLOR);
    pane.setFont(BeamerManager.plainButtonFont);

    dialog = pane.createDialog(desk__, "Information");
    dialog.setDefaultCloseOperation(dialog.DISPOSE_ON_CLOSE);
    dialog.pack();
    dialog.setLocationRelativeTo(desk__);
    dialog.setVisible(true);

    dialog = null;
    pane = null;
    panel = null;
    textArea = null;
    scrollPane = null;
  } // showOkMessage()

/**
 * NO uSE.
  * Displays an OK information message following the Access Road standard,
  * in a non-modal window which does not forbid the display of error messages
  * in other windows.
  * @param _msg message to display
  */
  /*
  public static synchronized void showNonModalOkMessage(String _msg){
    // variables for the dialog box
    JTextPane        text = new JTextPane();
    //String           lineSepar = System.getProperty("line.separator");
    Document         doc = text.getDocument();

    // initialize the text font
    text.setFont(new Font("Serif", Font.PLAIN, 12));

    SimpleAttributeSet bold = new SimpleAttributeSet();
    StyleConstants.setBold(bold, true);

    try {
        doc.insertString(0, _msg, bold);
    } catch (BadLocationException b) {
        throw new InternalError("In DialogBox.showNonModalOkMessage(), exception: "
            +b.toString());
    }

    JPanel                    panel ;
    ImageIcon                 image = null;

    // create a view frame closable, iconifiable
    JInternalFrame frame = new JInternalFrame("Information", true, true, true ,true);

    // put the scrollPane in a panel
    panel = new JPanel(true);
    panel.setLayout(new BorderLayout());
    panel.add("Center", text);

    //frame.addVetoableChangeListener(IconPolice.getInstance());
    frame.getContentPane().add(panel);

    frame.setBounds(
      InternalFrameModel.getExplorer().getWidth(),
      0,
      590,
      400);

    // set a frame icon
    try {
      image = new ImageIcon(Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
                        Desktop.URL_SEPAR+"road.jpg").toURL() );
    } catch (MalformedURLException m){
      System.out.println("in InternalFrame.showTextFrame(), exception"+
        " for opening the image icon. Exception: "+m.toString() );
    }

    frame.setFrameIcon(image);

    // add a listener for activating actions
    //frame = InternalFrameUtilities.addActivationFrameListener(frame);

    // put the frame in the desktop
    Desktop.addFrame(frame, new Integer(1));
    frame.setVisible(true);

    frame.setBounds(
      InternalFrameModel.getExplorer().getWidth(),
      0,
      200,
      200);

                    // for testing and tracking
                    System.out.println("end of DialogBox.showNonModalOkMessage(), "+
                            "frame is visible: "+frame.isVisible()+
                            "; frame is closed: "+frame.isClosed()+
                            "; frame bounds: "+frame.getBounds());
                    System.out.println("explorer bounds "+
                            InternalFrameModel.getExplorer().getBounds());

    frame = null;
    text = null;
    doc = null;
    bold = null;
    return;
  } // showNonModalOkMessage
   */

/**
  * Displays a request text to the user for entering a short answer.
  * The request must have a lenght lesser than 300 characters.
  * Called by ActionNewACSyst.selectACSType(),
  * CollectionPropertyEditor.addOneStringOrNameElements(),
  * StringMapPropertyEditor.addKeyElement(), ActionImportACS.
  * Calls showOkCancelDialog().
  * @param _request explains the data to enter. Displayed to the user without adding.
  * Returns null if _request is null or superior to 300.
  * @return the text the user has entered, or null.
  */
  public static synchronized String showSingleEditor(String _request) {
    if (_request == null || _request.length() > 300) return null;
    // initialization
    String                    result = null;
    JOptionPane               pane;
    JPanel                    panel = new JPanel();

    // fired from the main menu, so
    // displays a dialog box for the ACS name input

      // initialization
    JTextField field = new JTextField();
    field.setActionCommand("field");
    //if (_request.length() > 10) { // does no work
      //  panel.setMinimumSize(dimension);
      //  panel.setMaximumSize(dimension);
    //}
    //field.setMaximumSize(dimension);

    JLabel label = new JLabel(" "); // to do a space
    label.setAlignmentX(label.CENTER_ALIGNMENT);
    panel.add( label );
    if (_request.length() > 30)
        label = new JLabel("Enter a value, "+
                "then return                                                                  ");
    else if (_request.length() > 15)
        label = new JLabel("Enter a value, then return                                    ");
    else
        label = new JLabel("Enter a value, then return");
    label.setAlignmentX(label.CENTER_ALIGNMENT);
    panel.add( label );
    field.setAlignmentX(field.CENTER_ALIGNMENT);
    panel.add(field);
    label = new JLabel(" "); // to do a space
    label.setAlignmentX(label.CENTER_ALIGNMENT);
    panel.add( label );
    panel.setFont(BeamerManager.plainButtonFont);
    
    //if (_request.length() > 22)
      //  panel.setMinimumSize(dimension2);

    // shows the dialog box
    JDialog dialog = showOkCancelDialog(_request, panel);

    // here, dialog waits an enter from the user,
    // then the next instructions are processed.
      // if the user enters a "return" in an editable field,
      // it is also processed here as OK_OPTION.

    pane = (JOptionPane)dialog.getContentPane().getComponent(0);
    // no action if CLOSED, and close the dialog box
    if ( pane.getValue() == null) {
        dialog.dispose();
        dialog = null;
        pane = null;
        panel = null;
        return null;
    }
    // else, it is an Integer
    int value = ((Integer)pane.getValue()).intValue();
 
    result = field.getText();
    dialog.dispose();
    dialog = null;
    pane = null;
    panel = null;
    
      // no action if CANCEL, and close the dialog box
    if (value == JOptionPane.CANCEL_OPTION) return null;
      // if OK, action and close the dialog box
         //if (value == JOptionPane.OK_OPTION) {        
            //  return result;
         //}
    return result;
  } // showSingleEditor()

/**
  * Displays an OK error message following the Access Road standard.
  * The message may have a length superior to 100.
  * @param _msg String message
  */
  public static synchronized void showErrorMessage(String _msg){

    // variables for the dialog box
    JDialog     dialog;
    JOptionPane pane;
    JPanel      panel = new JPanel();
    JTextArea   textArea;

    // initialization
    desk__ = Desktop.getDesktopPane(); // can't be initialized before,
                                                     // when Desktop invokes this class
    panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS) );
    panel.setBackground(InternalFrame.BACKCOLOR_2);
    panel.setForeground(InternalFrame.FORECOLOR);

    textArea = new JTextArea(_msg, 4, 30);
    //textArea.setPreferredSize(new Dimension(100,80));
    if (_msg.length() > 100) textArea = new JTextArea(_msg, 8, 30);
    textArea.setLineWrap(true);
    textArea.setWrapStyleWord(true);
    textArea.setAlignmentX(textArea.CENTER_ALIGNMENT);
    textArea.setBackground(InternalFrame.BACKCOLOR_2);
    textArea.setForeground(InternalFrame.FORECOLOR_BLACK);
    textArea.setEditable(false);
    panel.add( textArea );

      // create the dialog box.
    pane = new JOptionPane(
      panel,
      JOptionPane.ERROR_MESSAGE,
      JOptionPane.OK_CANCEL_OPTION); //DEFAULT_OPTION);

          // color
    pane.setBackground(InternalFrame.BACKCOLOR_2);
    pane.setForeground(InternalFrame.FORECOLOR);
    pane.setFont(BeamerManager.plainButtonFont);

    dialog = pane.createDialog(desk__, "Error");
    dialog.setDefaultCloseOperation(dialog.DISPOSE_ON_CLOSE);
    dialog.pack();
    dialog.setLocationRelativeTo(desk__);
    dialog.setVisible(true);

    //dialog = null;
    //pane = null;
    //panel = null;
    //textArea = null;
  } // showErrorMessage()

/**
  * Displays an OK alarm message following the Access Road standard.
  * @param _msg String message
  */
  public static synchronized void showAlarmMessage(String _msg){

    // variables for the dialog box
    JDialog     dialog;
    JOptionPane pane;
    JPanel      panel = new JPanel();
    JTextArea   textArea = new JTextArea(_msg);

    // initialization
    desk__ = Desktop.getDesktopPane(); // can't be initialized before,
                                                     // when Desktop invokes this class
    panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS) );
    panel.setBackground(InternalFrame.BACKCOLOR_2);
    panel.setForeground(InternalFrame.FORECOLOR);

    if (_msg.length() > 100 && _msg.length() <= 200)
      textArea = new JTextArea(_msg, 5, 30);
    else if (_msg.length() > 200 && _msg.length() <= 300)
      textArea = new JTextArea(_msg, 10, 30);
    else if (_msg.length() > 300)
      textArea = new JTextArea(_msg, 20, 30);
    textArea.setLineWrap(true);
    textArea.setWrapStyleWord(true);
    textArea.setAlignmentX(textArea.CENTER_ALIGNMENT);
    textArea.setBackground(InternalFrame.BACKCOLOR_2);
    textArea.setForeground(InternalFrame.FORECOLOR_BLACK);
    textArea.setEditable(false);
    panel.add( textArea );

      // create the dialog box.
    pane = new JOptionPane(
      panel,
      JOptionPane.WARNING_MESSAGE,
      JOptionPane.OK_CANCEL_OPTION); //DEFAULT_OPTION);

          // color
    pane.setBackground(InternalFrame.BACKCOLOR_2);
    pane.setForeground(InternalFrame.FORECOLOR);
    pane.setFont(BeamerManager.plainButtonFont);

    dialog = pane.createDialog(desk__, "Huge Alarm!");
    dialog.setDefaultCloseOperation(dialog.DISPOSE_ON_CLOSE);
    dialog.pack();
    dialog.setLocationRelativeTo(desk__);
    dialog.setVisible(true);

    //dialog = null;
    //pane = null;
    //panel = null;
    //textArea = null;
  } // showAlarmMessage()

  /**
  * Displays an OK_CANCEL dialog box with BoxLayout.Y_AXIS,
  * following the Access Road standard, in a modal window.
  * See also showConfirmDialog(), which is simpler to use.
  * If the user enters a "return" in an editable field of _panel,
  * it is also processed as OK_OPTION in the JDialog JOptionPane.
  * To display long texts, use one JLabel per line.
  * Called by showOneSelectionOneDialog() and showImmutableStringMap().
  * @param _title displayed to the user without adding
  * @param _panel JPanel to be displayed
  * @return JDialog displayed, with a JOptionPane as unique component
  *  in its contentPane.
  */
  public static synchronized JDialog showOkCancelDialog(
          String _title, JPanel _panel) {

    // variables for the dialog box
    JDialog           dialog;
    JOptionPane       pane;

        // can't be initialized before
    desk__ = Desktop.getDesktopPane();

    _panel.setLayout(new BoxLayout(_panel,BoxLayout.Y_AXIS) );

      // creates the dialog box.
    pane = new JOptionPane(
      _panel,
      JOptionPane.QUESTION_MESSAGE,
      JOptionPane.DEFAULT_OPTION);

          // color
    pane.setBackground(InternalFrame.BACKCOLOR_2);
    pane.setForeground(InternalFrame.FORECOLOR);
    _panel.setBackground(InternalFrame.BACKCOLOR_2);
    _panel.setForeground(InternalFrame.FORECOLOR);
    pane.setFont(BeamerManager.plainButtonFont);
    
    //pane.transferFocusUpCycle();
    //_panel.requestFocusInWindow();

    dialog = pane.createDialog(desk__, _title);
    dialog.setDefaultCloseOperation(dialog.DISPOSE_ON_CLOSE);
    dialog.pack();
    dialog.setLocationRelativeTo(desk__);
    
    dialog.setVisible(true);


                    // for testing and tracking
                    //System.out.println("DialogBox.showOkCancelDialog(), dialog: "+
                      //      dialog);

    //dialog = null;
    //pane = null;
    return dialog;
  } // showOkCancelDialog()

  /**
   * UNUSED because the JOptionPane.getValue() management is not working.
  * Display an OK_CANCEL dialog box with BoxLayout.Y_AXIS in an internal frame,
  * following the Access Road standard. Using an internal frame rather than
  * a JDialog to let the user works on other frames during this frame opening.
  * If the user enters a "return" in an editable field of _panel,
  * it is also processed as OK_OPTION in the JInternalFrame JOptionPane.
  * The frame cannot be closed directly by the user.
  * @param _title of the dialog box
  * @param _panel to be displayed
  * @return JInternalFrame displayed, with a JOptionPane as unique component
  *  in its contentPane.
  */
  /*
  public static synchronized JInternalFrame showOkCancelInternalFrame(
          String _title, JPanel _panel){

    // variables for the dialog box
    JInternalFrame    frame;
    JOptionPane       pane;

        // can't be initialized before
    desk__ = Desktop.getDesktopPane();

    _panel.setLayout(new BoxLayout(_panel,BoxLayout.Y_AXIS) );

      // create the dialog box.
    pane = new JOptionPane(
      _panel,
      JOptionPane.QUESTION_MESSAGE,
      JOptionPane.OK_CANCEL_OPTION);

          // color
    pane.setBackground(InternalFrame.BACKCOLOR_2);
    pane.setForeground(InternalFrame.FORECOLOR);
    _panel.setBackground(InternalFrame.BACKCOLOR_2);
    _panel.setForeground(InternalFrame.FORECOLOR);
    
    //pane.transferFocusUpCycle();
    //_panel.requestFocusInWindow();

    frame = pane.createInternalFrame(desk__, _title);
    frame.setDefaultCloseOperation(frame.DO_NOTHING_ON_CLOSE);
    
    frame.setVisible(true);

    return frame;
  } // showOkCancelInternalFrame()
  */

  /**
  * Displays an YES_NO_CANCEL dialog box with BoxLayout.Y_AXIS,
  * following the Access Road standard.
  * See also showConfirmDialog(), which is simpler to use.
  * If the user enters a "return" in an editable field of _panel,
  * it is also processed as OK_OPTION in the JDialog JOptionPane.
  * To display long texts, use one JLabel per line.
  * @param _title of the dialog box
  * @param _panel JPanel to be displayed
  * @return  displayed JDialog with a JOptionPane as unique component
  *  in its contentPane.
  */
  public static synchronized JDialog showYesNoCancelDialog(
        String _title, JPanel _panel){
    // variables for the dialog box
    JDialog           dialog;
    JOptionPane       pane;

        // can't be initialized before
    desk__ = Desktop.getDesktopPane();

    _panel.setLayout(new BoxLayout(_panel,BoxLayout.Y_AXIS) );
    _panel.setMaximumSize(new Dimension(100,400)); // for 100 only

      // create the dialog box.
    pane = new JOptionPane(
      _panel,
      JOptionPane.QUESTION_MESSAGE,
      JOptionPane.OK_CANCEL_OPTION); //YES_NO_CANCEL_OPTION);

      // color
    pane.setBackground(InternalFrame.BACKCOLOR_2);
    pane.setForeground(InternalFrame.FORECOLOR);
    _panel.setBackground(InternalFrame.BACKCOLOR_2);
    _panel.setForeground(InternalFrame.FORECOLOR);
    pane.setFont(BeamerManager.plainButtonFont);

    dialog = pane.createDialog(desk__, _title);
    dialog.setDefaultCloseOperation(dialog.DISPOSE_ON_CLOSE);
    dialog.pack();
    dialog.setLocationRelativeTo(desk__);
    dialog.setVisible(true);

    //pane = null;
    return dialog;
  } // showYesNoCancelDialog()

  /**
  * Displays an OK_CANCEL dialog box with a list and BoxLayout.Y_AXIS,
  * following the Access Road standard.
  * Called by ActionOpenACSyst, ActionOpenGraphicView and ActionRemove.
  * @param _title String for the dialog box
  * @param _list JList to be displayed
  * @return JDialog displayed, with a JOptionPane as unique component
  *  in its contentPane.
  */
  public static synchronized JDialog showListDialog(String _title, JList _list) {

    // variables for the dialog box
    JDialog           dialog;
    JOptionPane       pane;
    JScrollPane       scrollPane;
    JPanel            panel1 = new JPanel();
    JPanel            panel2 = new JPanel();
    JLabel            label1;
    JLabel            label2;

        // can't be initialized before
    desk__ = Desktop.getDesktopPane();

    label1 = new JLabel("Use mouse to select, or double-Tab and arrows.");
    label1.setAlignmentX(label1.CENTER_ALIGNMENT);
    label2 = new JLabel("For non-continuous items, use Ctrl and mouse.");
    label2.setAlignmentX(label2.CENTER_ALIGNMENT);

      // put the list in a scrollpane
    scrollPane = new JScrollPane(_list);
    scrollPane.setSize(_list.getSize());
    panel2.setLayout(new BoxLayout(panel2,BoxLayout.Y_AXIS) );
    panel2.add(scrollPane);

    panel1.setLayout(new BoxLayout(panel1,BoxLayout.Y_AXIS) );
    panel1.add( label1 );
    panel1.add( label2 );
    panel1.add( panel2 );

      // create the dialog box.
    pane = new JOptionPane(
        panel1,
        JOptionPane.QUESTION_MESSAGE,
        JOptionPane.OK_CANCEL_OPTION);

          // color
    pane.setBackground(InternalFrame.BACKCOLOR_2);
    pane.setForeground(InternalFrame.FORECOLOR);
    panel1.setBackground(InternalFrame.BACKCOLOR_2);
    panel1.setForeground(InternalFrame.FORECOLOR);
    _list.setBackground(InternalFrame.BACKCOLOR_2);
    //_list.setForeground(InternalFrame.FORECOLOR);
    scrollPane.setBackground(InternalFrame.BACKCOLOR_2);
    scrollPane.setForeground(InternalFrame.FORECOLOR);
    label1.setBackground(InternalFrame.BACKCOLOR_2);
    //label1.setForeground(InternalFrame.FORECOLOR);
    label2.setBackground(InternalFrame.BACKCOLOR_2);
    //label2.setForeground(InternalFrame.FORECOLOR);
    pane.setFont(BeamerManager.plainButtonFont);

          // display
    dialog = pane.createDialog(desk__, _title);
    dialog.setDefaultCloseOperation(dialog.DISPOSE_ON_CLOSE);
    dialog.pack();
    dialog.setLocationRelativeTo(desk__);
    dialog.setVisible(true);

    //pane = null;
    //scrollPane = null;
    panel1 = null;
    panel2 = null;
    label1 = null;
    label2 = null;
    return dialog;
  } // showListDialog()

  /**
  * Displays a powerful dialog for selecting from the Explorer
  * some instances of n specified BaseObject classes (n from 1 to 4, inclusively)
  * in a gBase package. <P>
  * The objects of each type are displayed in a dedicated list in the dialog panel.
  * The number of possible user-selected objects is limited to 15.
  * The number of selected object in each BaseObject classes may be limited to one.
  * For selection from a list already known, use showListDialog(). <P>
  * This method sends always a return value through the call to
  * _caller.handleSelectionDialog(Map).
  * Each JList is associated to a DefaultListModel.
  * If _selectedObjectsSetMap is null or empty, this method displays
  * initial empty jlists.
  * If the user clicks on the 'OK' button when one JList is empty,
  * or if it clicks on a value with a forbidden type,
  * or if it is the second selection when only one is expected,
  * an error message is displayed.
  * If the user clicks on the 'Cancel' button, the method calls the caller with
  * a 'null' argument.
  * Called by ActionNewView, SimplePropertyEditor, CollectionPropertyEditor,
  * BaseObjectHandler, ActionNewACS. Use valueChanged().
  * @param _caller which calls this method. Its handleSelectionDialog() method 
  * is called by this method when the user valids the selection inputs.
  * @param _message is the title after completed by ' selection'. 
   *    Its length has to be inferior to 15. May be null.
  * @param _objectsTypes is the set of the n required classes
  * implementing the BaseObject interface in a gBase package
  * (n between 1 and 4, inclusively).
  * @param _selectedObjectsSetMap is the map where:
  * - the key is an _objectsTypes item
  * - the value is a JList of objects that are instances of the key.
  *   May be null or empty.
  * @param _oneSelection is true if only one selected object is
  *   allowed for all BaseObject classes
  * @exception SmallError if a parameter is not a BaseObject instance,
  *   or if types are not matched, or if there are more than 4 _objectTypes.
  */
  public void showExplorerSelectionDialog(SelectionDialogListener _caller,
    String _message, Set _objectsTypes, Map _selectedObjectsSetMap, boolean _oneSelection)
    throws SmallError {

                    //System.out.println("call to DialogBox.showExplorerSelectionDialog() with object types:");
                    //for (Iterator ite = _objectsTypes.iterator() ; ite.hasNext() ; ) {
                      //        System.out.println("--------- : "+ ite.next().toString());
                    //}
      
      
    Class type = null; 
    
    // exception
    if (_objectsTypes.size() > 4) throw new SmallError("in DialogBox."+
        "showExplorerSelectionDialog(), the number of types is greater than 4.");
    for (Iterator ite = _objectsTypes.iterator() ; ite.hasNext() ; ) {
      type = (Class)ite.next();
      try {
        if (!(Class.forName("ARoad0.gBaseInterface.BaseObject").isAssignableFrom(type) ))
          throw new SmallError("in DialogBox."+
          "showExplorerSelectionDialog(), the type "+type+" is not a BaseObject.");
      } catch (ClassNotFoundException c) {
        throw new SmallError("in DialogBox."+
          "showExplorerSelectionDialog(), ARoad0.gBaseInterface.BaseObject doesn't exist!");
      }
      try {
        if (_selectedObjectsSetMap != null && !_selectedObjectsSetMap.isEmpty()) {
            if (!_selectedObjectsSetMap.containsKey(type) ||
              !_selectedObjectsSetMap.get(type).getClass().
              equals(Class.forName("javax.swing.JList") ) ) throw new SmallError(
                "in DialogBox.showExplorerSelectionDialog(),"+
                " a key is not in the list,"+
                "or a value in the Map argument is not a JList instance.");
        } // if
      } catch (ClassNotFoundException c) {
        throw new InternalError("From DialogBox.showExplorerSelectionDialog(): "+c);
      }
    } // for

    // variables for the dialog box
    onlyOneSelectionAllowed__ = _oneSelection;
    final SelectionDialogListener   caller = _caller;
    JList                           jlist = null;
        // can't be initialized before
    desk__ = Desktop.getDesktopPane();
    final Map result = new HashMap(INITIAL_CAPACITY);

    final JDialog     dialog;
    JScrollPane       scrollPane = new JScrollPane();;
    JPanel            panelOne = new JPanel();
    JPanel            panelType = new JPanel();
    JPanel            panelTwo = new JPanel();
    JPanel            panelThree = new JPanel();
    JPanel            panelFour = new JPanel();
    JLabel            labelTypeOne;
    JLabel            labelTypeThree = new JLabel(" ");

    Dimension         dimSet = new Dimension(70,50);
    String            typeName;

    JLabel labelOne;
    JLabel labelFour = null;
    JLabel labelTwo = null;
    JLabel labelTwoBis = null;
    JLabel labelThree = null;
    JLabel labelFive = null;
    JLabel labelSix = null;

    if (!_oneSelection) {
      labelOne = new JLabel("  In Explorer, click on object(s)"+
        " with the right type(s).  ");
      labelOne.setAlignmentX(labelOne.CENTER_ALIGNMENT);
      labelFour = new JLabel(" ");
      labelFour.setAlignmentX(labelFour.CENTER_ALIGNMENT);
      labelTwo = new JLabel("To remove selected objects,"+
        " select them");
      labelTwo.setAlignmentX(labelOne.CENTER_ALIGNMENT);
      labelTwoBis = new JLabel("in list(s) and click on 'Remove in list(s)'.");
      labelTwoBis.setAlignmentX(labelOne.CENTER_ALIGNMENT);
      labelThree = new JLabel("Ctrl + mouse clicks");
      labelThree.setAlignmentX(labelTwo.CENTER_ALIGNMENT);
      labelFive = new JLabel("to select non-continuous items.");
      labelFive.setAlignmentX(labelTwo.CENTER_ALIGNMENT);
      labelSix = new JLabel(" ");
      labelSix.setAlignmentX(labelTwo.CENTER_ALIGNMENT);
    } else {
      labelOne = new JLabel("  In Explorer, click on one object"+
        " with the type:  ");
      labelOne.setAlignmentX(labelOne.CENTER_ALIGNMENT);
    }

    labelTypeThree.setAlignmentX(labelTypeThree.CENTER_ALIGNMENT);

    synchronized (selectedObjectsSetMap__) {
        if (_selectedObjectsSetMap == null || _selectedObjectsSetMap.isEmpty() ) {
          // if the map of jlists parameter is empty,
          // create selectedObjectsSetMap__,
          // the map of the initially-empty selected objects jlists,
          // where object type is the key
          selectedObjectsSetMap__.clear();
          for (Iterator ite = _objectsTypes.iterator() ; ite.hasNext() ; ) {
            type = (Class)ite.next();
            selectedObjectsSetMap__.
                put(type, new JList( new DefaultListModel() ) );
          } // for
        } // if
        else {
          // the map of jlists parameter is NOT empty, then
          // update with it selectedObjectsSetMap__,
          // the map of the selected objects jlists,
          // where object type is the key
          selectedObjectsSetMap__.clear();
          for (Iterator ite = _objectsTypes.iterator() ; ite.hasNext() ; ) {
            type = (Class)ite.next();
            selectedObjectsSetMap__.
                put(type, _selectedObjectsSetMap.get(type) );
          } // for
        }

        for (Iterator ite = _objectsTypes.iterator() ; ite.hasNext() ; ) {
            // puts each selected-objects set (one for a BaseObject type)
            // in a scrollpane, in panelType and in panelTwo
          type = (Class) ite.next();

            // displays the interface name without the package name,
            // and converts to the AcsAddon name if necessary
          typeName = UtilityImpl.getFirstInstance().getClassNameFromArrayOrInterface(
                  UtilityImpl.getFirstInstance().
                    getACSVocabularyFromArrayOrInterface(type.getName()),
                  type.getName());
          if (typeName != null) {
              typeName = typeName.substring(
                  typeName.indexOf("gBase.") + 6,
                  typeName.length() );
              if (typeName.indexOf("Impl") != -1)
                  typeName = typeName.substring(
                                0,
                                typeName.indexOf("Impl"));
          }
          // it is a generic interface like BaseObject or Node
          else {

                            // for testing and tracking
                            //System.out.println("call to DialogBox.showExplorerSelectionDialog():");
                            //System.out.println("--------- with a generic interface type: "+ type.getName());

              typeName = type.getName().substring(
                  type.getName().indexOf("gBaseInterface.") + 15,
                  type.getName().length() );
          } // else

          labelTypeOne = new JLabel(typeName);
          labelTypeOne.setAlignmentX(labelTypeOne.CENTER_ALIGNMENT);

          jlist = (JList)selectedObjectsSetMap__.get(type);
          jlist.setBackground(InternalFrame.BACKCOLOR_2);
          scrollPane = new JScrollPane(jlist);
          scrollPane.setSize(dimSet);
          scrollPane.setBackground(InternalFrame.BACKCOLOR_2);
          scrollPane.setForeground(InternalFrame.FORECOLOR);
          panelType.setLayout(new BoxLayout(panelType,BoxLayout.Y_AXIS) );
          if (!_oneSelection) panelType.add(labelTypeThree);
          panelType.add(labelTypeOne);
          if (!_oneSelection) panelType.add(scrollPane);
          panelType.setBackground(InternalFrame.BACKCOLOR_2);

          panelTwo.add(panelType);
        } // for ite
        
        // there, if _oneSelection, 'type' is the single type to select

    } // synchro

    panelOne.setLayout(new BoxLayout(panelOne,BoxLayout.Y_AXIS) );
    if (!_oneSelection) panelOne.add( labelFour );
    panelOne.add( labelOne );
    if (!_oneSelection) panelOne.add( labelTwo );
    if (!_oneSelection) {
      panelOne.add( labelTwoBis );
      panelOne.add( labelThree );
      panelOne.add( labelFive );
    }

    panelOne.add( panelTwo );

    panelOne.setBackground(InternalFrame.BACKCOLOR_2);
    panelOne.setForeground(InternalFrame.FORECOLOR);
    panelOne.setFont(BeamerManager.plainButtonFont);
    
    panelTwo.setBackground(InternalFrame.BACKCOLOR_2);
    panelTwo.setForeground(InternalFrame.FORECOLOR);
    panelThree.setBackground(InternalFrame.BACKCOLOR_2);
    panelThree.setForeground(InternalFrame.FORECOLOR);
    panelFour.setBackground(InternalFrame.BACKCOLOR_2);
    panelFour.setForeground(InternalFrame.FORECOLOR);
    labelOne.setBackground(InternalFrame.BACKCOLOR_2);
    if (!_oneSelection) labelTwo.setBackground(InternalFrame.BACKCOLOR_2);


    // creates the window
    if (_message != null) 
        dialog = new JDialog(Desktop.getInstance(), 
                _message+" selection ", false);
    else dialog = new JDialog(Desktop.getInstance(), 
                " Selection ", false);
    
    // create 3 buttons 'Remove', 'OK' and 'Cancel'
    JButton removeButton = null;
    JButton okButton = null;
    if (!_oneSelection) {
      removeButton = new JButton("Remove in list(s)");
      okButton = new JButton("OK");
      panelThree.add( removeButton);
      panelFour.add( okButton);
    }

    JButton cancelButton = new JButton("Cancel");
    panelFour.add( cancelButton);

    if (!_oneSelection) panelOne.add(panelThree);
    panelOne.add(panelFour);

    // creates buttons action listeners
    if (!_oneSelection) removeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
          synchronized (selectedObjectsSetMap__) {
            Class type;
            JList jlist2;
            Object[] objArray;
            DefaultListModel listModel2;

            // search the selected items in each JList
            // and remove it
            for (Iterator ite = selectedObjectsSetMap__.keySet().iterator()
              ; ite.hasNext() ; ) {
              type = (Class) ite.next();
              jlist2 = (JList) selectedObjectsSetMap__.get(type);
              listModel2 = (DefaultListModel) jlist2.getModel();

              if (jlist2.getSelectedValue() != null) {
                objArray = jlist2.getSelectedValues();
                for (int i = 0 ; i < objArray.length ; i++) {
                  listModel2.removeElementAt(
                    listModel2.indexOf(objArray[i] ));
                  synchronized (new Integer(selectedObjectsCount__))
                      {selectedObjectsCount__--;}
                } // for i
              } // if

            } // for ite
            listModel2 = null;
            jlist2 = null;
            objArray = null;
            type = null;

          } // synchro
          return;
      } // actionPerformed()
    }); // removeButton.addActionListener

    if (!_oneSelection) okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
          synchronized (selectedObjectsSetMap__) {
            int typeCount = 0;
            Class type;
            DefaultListModel listModel2;

            // construct the set of empty Lists
            List[] listArray = new ArrayList[selectedObjectsSetMap__.keySet().size()];
            for (int i = 0 ; i < listArray.length ; i++) {
              listArray[i] = new ArrayList();
            }
            // constructs a copy of selectedObjectsSetMap__
            // but with List rather than JList
            for (Iterator ite = selectedObjectsSetMap__.keySet().iterator()
              ; ite.hasNext() ; ) {
              type = (Class) ite.next();
              listModel2 = (DefaultListModel)((JList)selectedObjectsSetMap__
                .get(type)).getModel();
              //if (listModel2.isEmpty() ) {
                //showErrorMessage(
                  //  "Select at least one object for each list.");
                //return;
              //}
              for (int j = 0 ; j < listModel2.getSize() ; j++) {
                listArray[typeCount].add( listModel2.elementAt(j) );
              } // for j
              typeCount++;
            } // for ite

            listModel2 = null;
            typeCount = 0;
            for (Iterator ite = selectedObjectsSetMap__.keySet().iterator()
              ; ite.hasNext() ; ) {
                result.put( ite.next(), listArray[typeCount] );
                typeCount++;
            } // for ite

            listArray = null;
            listModel2 = null;
            type = null;
          } // synchro

                    // for testing and tracking
                    //System.out.println("in DialogBox.showExplorerSelectionDialog(), caller ");

          dialog.dispose();
          waitClickInExplorer__ = false;
          selectedObjectsCount__ = 0;

          // return to the caller of this method
          caller.handleSelectionDialog(result);
          return;
        } // actionPerformed()
    }); // okButton.addActionListener

    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
          dialog.dispose();

          waitClickInExplorer__ = false;
          selectedObjectsCount__ = 0;
          // unusefull
          //onlyOneSelectionAllowed__ = false;

          // return to the caller of this method
          caller.handleSelectionDialog(null);
          return;
        } // actionPerformed()
    }); // cancelButton.addActionListener

    // handles dialog closing
    dialog.setDefaultCloseOperation(dialog.DO_NOTHING_ON_CLOSE);
        // inner class WindowAdapter for closing
    dialog.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e){
          if (dialog != null) dialog.dispose();

          waitClickInExplorer__ = false;
          selectedObjectsCount__ = 0;
          onlyOneSelectionAllowed__ = false;

          // return to the caller of this method
          caller.handleSelectionDialog(null);
          return;
      } // windowClosed()
    } ); // dialog.addWindowListener()

    // if one selection and one list, 'type' is the right class.
    // adds a list listener to return without clicking on the ok button;
    // valueChanged() updates the list and calls intervalAdded()
    final Class singleType = type; // the type to select if _oneSelection
    
            // for testing purpose
            //System.out.println("in DialogBox.showExplorerSelectionDialog(), "+
              //      "when one selection is true: "+_oneSelection+
              //      " the type to select is: "+type+
              //      " and the size of the keys in the set map is:"+
              //      selectedObjectsSetMap__.keySet().size());    
    
    if (selectedObjectsSetMap__.keySet().size() == 1 && _oneSelection) {
      ( (DefaultListModel)((JList)
        selectedObjectsSetMap__.get(type)).getModel() )
        .addListDataListener(new JComboBox() { //JComboBox is the only matched class here
          public void intervalAdded(ListDataEvent evt) {

                    // for testing purpose
                    //System.out.println("in DialogBox.showExplorerSelectionDialog() "+
                      //" with only one selection," +
                      //"the data listener says the upper index of the first list is: "+
                      //evt.getIndex1() );                
              
              
              if (evt.getType() != ListDataEvent.INTERVAL_ADDED) return;

              List list = new ArrayList();
              JList jlist = (JList)selectedObjectsSetMap__.get(singleType);

              list.add(
                jlist.getModel().getElementAt(0)
              );
              result.clear();
              result.put(singleType, list);

              dialog.dispose();

              waitClickInExplorer__ = false;
              selectedObjectsCount__ = 0;
              onlyOneSelectionAllowed__ = false;
              
              // returns to the caller of this method
              caller.handleSelectionDialog(result);
              list = null;
              jlist = null;
              return;
            } // intervalAdded()
          } //JComboBox
      ); // .addListDataListener()
    } // if (selectedObjectsSetMap__.keySet().size() == 1

        // displays the window
        // manages waitClickInExplorer__ and clear selection in the explorer
    dialog.getContentPane().add(panelOne);
    dialog.pack();
    // if the explorer is expanded, dialog is visible at the right of the explorer,
    // otherwise it is at the center of the desk
    if (InternalFrameModel.getExplorer().getBounds().getWidth() >
       Desktop.getDSize().getWidth() - 300) dialog.setLocationRelativeTo(desk__);
    else dialog.setLocation(
       (int)InternalFrameModel.getExplorer().getBounds().getWidth(), 0);
    waitClickInExplorer__ = true;
    InternalFrameModel.getTreeInExplorer().clearSelection();
    dialog.setVisible(true);

    jlist = null;
    scrollPane = null;
    panelOne = null;
    panelType = null;
    panelTwo = null;
    panelThree = null;
    panelFour = null;
    labelTypeOne = null;
    labelTypeThree = null;
    dimSet = null;
    labelOne = null;
    labelFour = null;
    labelTwo = null;
    labelTwoBis = null;
    labelThree = null;
    labelFive = null;
    labelSix = null;    
    
    type = null;
  } // showExplorerSelectionDialog()


  /**
  * Displays a question to the user in a modal window.
  * Does not use '\n' as line separation.
  * @param _question displayed, with a lenght limited to 200. If null or empty,
  *   return false.
  * @param _comment to the question, , with a lenght limited to 200. May be null.
  * @return boolean true only if the user answers yes to the question.
  */
  public static synchronized boolean showConfirmDialog(String _question, String _comment) {

    if (_question == null || _question.length() == 0) return false;

    // variables
    JLabel        label_one;
    JLabel        label_two = null;
    JLabel        label_three = null;
    JPanel        panel = new JPanel();
    JDialog       dialog;
    JOptionPane   pane;
    ButtonGroup   buttonGroup = new ButtonGroup();
    JRadioButton  rbutton;
    char          n = 'N';
    char          y = 'Y';

    panel.setFont(BeamerManager.plainButtonFont);
    
    // builds up the panel
    if (_question.length() < 50) {
      label_one = new JLabel(_question);
    } else if (_question.length() < 100) {

                          // for testing purpose
                          //System.out.println("DialogBox.showConfirmDialog(), "+
                            //"_question: "+_question); 

      if (_question.indexOf(" ", 30) != -1) {
          label_one = new JLabel(_question.substring(0, _question.indexOf(" ", 30) ));
          label_two = new JLabel(_question.substring(
            _question.indexOf(" ", 30) ));
      }
      else if (_question.indexOf(".", 30) != -1) {
          label_one = new JLabel(_question.substring(0, _question.indexOf(".", 30) ));
          label_two = new JLabel(_question.substring(
            _question.indexOf(".", 30) ));
      }
      else {
          label_one = new JLabel(_question.substring(0, 30 ));
          label_two = new JLabel(_question.substring(30 ));
      }

      label_two.setAlignmentX(label_two.LEFT_ALIGNMENT);
    } else if (_question.length() < 200) {

                          // for testing purpose
                          System.out.println("DialogBox.showConfirmDialog(), "+
                            "_question beyond 100 characters: "+_question);

      if (_question.indexOf(" ", 50) != -1) {
          label_one = new JLabel(_question.substring(0, _question.indexOf(" ", 50) ));

          if (_question.indexOf(" ", 120) != -1 && _question.indexOf(" ", 120) != _question.indexOf(" ", 50)) {
              label_two = new JLabel(_question.substring(
                _question.indexOf(" ", 50), _question.indexOf(" ", 120) ));
              label_three = new JLabel(_question.substring(
                      _question.indexOf(" ", 120)));
          }
          else {
              label_two = new JLabel(_question.substring(
                _question.indexOf(" ", 50), 120 ));
              label_three = new JLabel(_question.substring(120 ));
          }
      } // if (_question.indexOf(" ", 50) != -1)
      else if (_question.indexOf(".", 50) != -1) {
          label_one = new JLabel(_question.substring(0, _question.indexOf(".", 50) ));

          if (_question.indexOf(".", 120) != -1 && _question.indexOf(".", 120) != _question.indexOf(".", 50)) {
              label_two = new JLabel(_question.substring(
                _question.indexOf(".", 50), _question.indexOf(".", 120) ));
              label_three = new JLabel(_question.substring(
                      _question.indexOf(".", 120)));
          }
          else {
              label_two = new JLabel(_question.substring(
                _question.indexOf(".", 50), 120 ));
              label_three = new JLabel(_question.substring(120 ));
          }
      } // if (_question.indexOf(" ", 50) != -1)
      else {
          label_one = new JLabel(_question.substring(0, 50 ));
          label_two = new JLabel(_question.substring(
            _question.indexOf(" ", 50), 120 ));
          label_three = new JLabel(_question.substring(120 ));
      } // else

      label_two.setAlignmentX(label_two.LEFT_ALIGNMENT);
      label_three.setAlignmentX(label_two.LEFT_ALIGNMENT);
    } else {
      throw new InternalError(
              "In DialogBox.showConfirmDialog(), the question is too long.");
    }
    label_one.setAlignmentX(label_one.LEFT_ALIGNMENT);
    panel.add( label_one );
    if (_question.length() >= 50) panel.add( label_two );
    if (_question.length() >= 100) panel.add( label_three );

    // sets the comment label(s)
    if (_comment != null) {
      if (_comment.length() < 100) {
        label_one = new JLabel(_comment);
        label_one.setAlignmentX(label_one.LEFT_ALIGNMENT);
      } else if (_comment.length() < 200) {
        if ( _comment.indexOf(" ", 90) != -1) {
            label_one = new JLabel(_comment.substring(0, _comment.indexOf(" ", 90) ));
            label_two = new JLabel(_comment.substring(
              _comment.indexOf(" ", 90), _comment.length() ));
        }
        else {
            label_one = new JLabel(_comment);
        }
        label_two.setAlignmentX(label_two.LEFT_ALIGNMENT);
        label_one.setAlignmentX(label_one.LEFT_ALIGNMENT);
      } else {
        throw new InternalError("In DialogBox.showConfirmDialog(), the comment is too long.");
      }
      panel.add( label_one );
      if (_question.length() >= 100) panel.add( label_two );
    } // if _comment

    panel.add(new JLabel(" "));
    label_one = new JLabel("Please select one button:");
    label_one.setAlignmentX(label_one.LEFT_ALIGNMENT);
    panel.add( label_one );

    rbutton = new JRadioButton("No");
    panel.add(rbutton);
    rbutton.setActionCommand("No");
    rbutton.setMnemonic(n);
    rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
    rbutton.setBackground(InternalFrame.BACKCOLOR_2);
    rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
    buttonGroup.add(rbutton);

    rbutton = new JRadioButton("Yes");
    panel.add(rbutton);
    rbutton.setActionCommand("Yes");
    rbutton.setMnemonic(y);
    rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
    rbutton.setBackground(InternalFrame.BACKCOLOR_2);
    rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
    buttonGroup.add(rbutton);

    // shows the dialog box
    dialog = showOkCancelDialog("Question", panel);

    // here, dialog waits a user imput,
    // then the next instructions are processed.
    pane = (JOptionPane)dialog.getContentPane().getComponent(0);
    // no action if CLOSED, and close the dialog box
    if ( pane.getValue() == null) {
      return false;
    }
    // else, it is an Integer
    int value = ((Integer)pane.getValue()).intValue();

      // no close if CANCEL, and close the dialog box
    if (value == JOptionPane.CANCEL_OPTION) return false;
      // else, it is OK, continue

    // no close if no radio button is selected
    if (buttonGroup.getSelection() == null) {
      return false;
    }

    if (buttonGroup.getSelection().getActionCommand().equals("No") ) {
      return false;
    }
    if (buttonGroup.getSelection().getActionCommand().equals("Yes") ) {
        return true;
    }

    label_one = null;
    label_two = null;
    label_three = null;
    panel = null;
    dialog = null;
    pane = null;
    buttonGroup = null;
    rbutton = null;
    return false;
  } // showConfirmDialog()

  /**
  * Displays one list for selecting several items.
  * For selecting one item, see showOneSelectionDialog().
  * For selecting several items in several lists, see showBackedJLists().
  * The list size is limited to 32 items. The minimal size is 1.
  * The first item type is detected to display the list, and the type 
  * must be String, ImmutableName or BaseObject. For a BaseObject, getName().toString()
  * is used to display the item, and it is the same for ImmutableName.
  * result.getComponents() is an array of JCheckBox which are selected or not.
  * Called by showSimpleSelectionDialog() and CollectionPropertyEditor.
  * @param _displayName of the element to select. May be null.
  * @param _l_values is the String or BaseObject list of
  *   the non-null values to display. Not null, and size has to be from 1 to 22.
  * @return JPanel where are the selected JCheckBox(s).
  *   It is an empty JPanel if _l_values is not correct.
  */
  public static synchronized JPanel showSelectionsInOneDialog(
    String _displayName, Object[] _l_values) {

    // check _l_values
    boolean ok = true;
    if (_l_values == null || _l_values.length == 0 || _l_values.length > 32 ||
            _l_values[0] == null) {

                          // for testing purpose
                          System.out.println("DialogBox.showSelectionsInOneDialog(), "+
                            "_l_values is not correct - _l_values.length: "+_l_values.length);  
        
                     return new JPanel();
    }
    if (_l_values[0].getClass() != String.class && !(_l_values[0] instanceof ImmutableName) &&
      !(_l_values[0] instanceof BaseObject)) {

                          // for testing purpose
                          System.out.println("DialogBox.showSelectionsInOneDialog(), "+
                            "_l_values[0] is not correct - _l_values[0].getClass(): "+_l_values[0].getClass()); 
        
        return new JPanel();
    }

    // initialization
    JPanel          panel = new JPanel();
    JLabel          elementName = new JLabel("Select "+_displayName);
    JLabel          comment = new JLabel("   (Space to select; Tab to skip)");
    JPanel          checkBoxPanel = new JPanel();
    //JRadioButton    rbutton;
    //ButtonGroup     group = new ButtonGroup();
    JCheckBox       rbutton;
    Name            name = NameImpl.EMPTY_INSTANCE;
    int             widthSize = 75;
    JOptionPane     pane;

    // builds up the dialog box
    checkBoxPanel.setLayout(new BoxLayout(checkBoxPanel, BoxLayout.Y_AXIS) );
    checkBoxPanel.setBackground(InternalFrame.BACKCOLOR_2);
    checkBoxPanel.setForeground(InternalFrame.FORECOLOR);

    if (_l_values[0] instanceof String) {
      for (int i = 0; i < _l_values.length ; i++) {

        rbutton = new JCheckBox((String)_l_values[i]);
        checkBoxPanel.add(rbutton);
        rbutton.setActionCommand((String)_l_values[i]);
        rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
        rbutton.setBackground(InternalFrame.BACKCOLOR_2);
        rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
        widthSize = java.lang.Math.max(widthSize,
          ((String)_l_values[i]).length()+ 25);

      } // for
    } // if (_l_values[0] instanceof String)
    else if (_l_values[0] instanceof ImmutableName) {
      for (int i = 0; i < _l_values.length ; i++) {

        rbutton = new JCheckBox(((ImmutableName)_l_values[i]).toString());
        checkBoxPanel.add(rbutton);
        rbutton.setActionCommand(((ImmutableName)_l_values[i]).toString());
        rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
        rbutton.setBackground(InternalFrame.BACKCOLOR_2);
        rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
        widthSize = java.lang.Math.max(widthSize,
          ((ImmutableName)_l_values[i]).toString().length()+ 25);

      } // for
    } // if (_l_values[0] instanceof ImmutableName)
    else {
      for (int i = 0; i < _l_values.length ; i++) {

        name = (NameImpl) ((BaseObject)_l_values[i]).getName();
        rbutton = new JCheckBox( name.toString() );
        checkBoxPanel.add(rbutton);
        rbutton.setActionCommand( name.toString() );
        rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
        rbutton.setBackground(InternalFrame.BACKCOLOR_2);
        rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
        //group.add(rbutton);
        widthSize = java.lang.Math.max(widthSize, name.toString().length()+ 25);

      } // for
    } // else

    checkBoxPanel.setBorder(new BevelBorder(BevelBorder.LOWERED));
    panel.add(elementName);
    panel.add(new JPanel());
    panel.add(comment);
    panel.add(checkBoxPanel);
    panel.setSize(200, widthSize);
    panel.setFont(BeamerManager.plainButtonFont);

                          // for testing purpose
                          //System.out.println("DialogBox.showSelectionsInOneDialog(), "+
                            //"call to showOkCancelDialog() - _l_values.length: "+_l_values.length);

    // shows the dialog box
    JDialog dialog = showOkCancelDialog("Selection", panel);

    // here, dialog waits an enter from the user,
    // then the next instructions are processed.
    pane = (JOptionPane)dialog.getContentPane().getComponent(0);

    // no action if CLOSED, and closes the dialog box
    if ( pane.getValue() == null) return new JPanel();
    // else, it is an Integer
    int value = ((Integer)pane.getValue()).intValue();

    // no action if CANCEL, and closes the dialog box
    if (value != JOptionPane.OK_OPTION) return new JPanel();
    // if OK, action and close the dialog box

    panel = null;
    dialog = null;
    rbutton = null;
    elementName = null;
    //group = null;
    comment = null;
    name = null;
    pane = null;
    return checkBoxPanel;
  } // showSelectionsInOneDialog()

  /**
  * Displays one list for selecting several items.
  * For selecting one item, see showOneSelectionOneDialog().
  * The list size is limited to 22 items. The minimal size is 1.
  * All the items must have the same type: String, ImmutableName or BaseObject.
  * If it is BaseObject, getName().toString() is used to display the item
  * and it is in the returned set. It is the same for ImmutableName values to select.
  * The order of the items in the argument is kept in the returned list.
  * Wrap showSelectionDialog().
  * Called by BaseObjectHandler.actionPerformed() and 
  * StringMapPropertyEditor.addRestrictedValueElements().
  * @param _displayName of the element to select. May be null.
  * @param _l_values is the String, ImmutableName or BaseObject list of
  *   the allowed non-null values to display.
  * @return the selected Strings, or the String form of the selected values, or an empty set.
  */
  public static synchronized List<String> showSimpleSelectionsInOneDialog(
    String _displayName, Object[] _l_values) {
      
        ArrayList<String>       l_result = new ArrayList<String>(INITIAL_CAPACITY);
        JPanel                  panel = showSelectionsInOneDialog(_displayName, _l_values);
        java.awt.Component[]    l_boxes = panel.getComponents();
        
        // gets all the selected Strings.
        // l_boxes empty if the user has deleted the dialog in showSelectionsInOneDialog()
        for (int j = 0 ; j < l_boxes.length ; j++) {
          if ( ((JCheckBox)l_boxes[j]).isSelected() )
            l_result.add( ((JCheckBox)l_boxes[j]).getActionCommand() );
        } // for j 
        
        l_boxes = null;        
        panel = null;
        return l_result;
  } // showSimpleSelectionsInOneDialog()
  
  /**
   * Displays a JInternalFrame with several JComponents to be edited by the user,
   * where the components and the frame are closed AFTER the reading of the properties
   * in the components by the controller.
   * The components editing by user is done through compoent listeners outside
   * this method, so component states are not changed by this method.
   * The single listener is generally the method caller.
   * An exemple is when _panel contains a JTextField and a JButton 'Select',
   * where the button updates the text after an user selection in the explorer.
   * This method stops the object selection in the explorer.
   * The method calls _controller.handleCreationDialog() when the option pane is updated,
   * and it closes the internal frame before. It is a partially reentrant method
   * since _controller may recalls it from handleCreationDialog().
   * The frame cannot be closed directly by the user: only OK/CANCEL buttons are active.<P>
   * Called by BaseObjectHandler.getBaseObjectEditor() for entering the
   * arguments of a new object or a copy, by showBackedJLists() and
   * by showOneSelectionOneDialog() for RightMapPropertyEditor.addKeyElement().
   * The displayed OK and Cancel buttons have the names '      OK     ' and '   Cancel   '
   * in the parent panel of the argument _panel. BaseObjectHandler.actionPerformed()
   * may terminate the dialog through a direct call to one of these buttons.
   * @param _controller handles the operations when the user ends this dialog.
   *    May be an indirect caller like when this.showOneSelectionOneDialog() 
   *    is the direct caller. Must be non-null.
   * @param _request is the question to the user. Must be non-null.
   * @param _panel is a non-null group of JComponents which includes listener links
   *    for updating the values displayed in the panels, following the first user
   *    choices. The listener is generally _controller.
   */
  public static synchronized void showBackedJComponents(
          CreationDialogListener _controller, String _request, JPanel _panel) {

    if (_controller == null ) return;
    if (_request == null || _request.length() == 0) return;
    if (_panel == null ) return;  
    
            // for testing purpose
            //System.out.println("Call to DialogBox.showBackedJPanels() ");  
               
    desk__ = Desktop.getDesktopPane();
            
    // stop the object selection in the explorer
    hasHandlerDialog__ = true;

    // variables
    JButton                   okButton = new JButton("      OK     ");
    JButton                   cancelButton = new JButton("   Cancel   ");
    //JButton                   separatorButton = new JButton("    ");
    JComponent                pane;
    JPanel                    panel = new JPanel();
    JPanel                    panel2 = new JPanel();
    Dimension                 buttonDim = new Dimension(90,150); // noop
    
    final CreationDialogListener caller = _controller;

    // create a frame resizable, non closable, non maximisable, non iconifiable
    final JInternalFrame frame = new JInternalFrame(_request, true, false, false ,false);   

            // add the "OK" button
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
            // return to the default state where a selection in the 
            // explorer updates the beamer
            hasHandlerDialog__ = false;

            frame.setVisible(false);
            try {
                frame.setClosed(true);
            } catch (PropertyVetoException p) {
                System.out.println("DialogBox.showBackedJPanels():"+
                " the frame closing fires a PropertyVetoException: "+p);
            }
            
            // for testing and tracking
           //JInternalFrame[] allframes = ARoad0.Gui1.Desktop.getCurrentFrames();
           //System.out.println("DialogBox.showBackedJPanels(), after the frame closing"+
             //      " through 'OK', open frames are:");
           //for (int i = 0; i < allframes.length; i++) {
             // System.out.println("------------ "+allframes[i].getTitle());
           //} // for             

            caller.handleCreationDialog(true);
            return;
      } // actionPerformed()
    }); // whyButton.addActionListener

    okButton.setBackground(Color.white);
    okButton.setBorder(new BevelBorder(
      BevelBorder.RAISED, InternalFrame.FORECOLOR, InternalFrame.FORECOLOR));
    okButton.setToolTipText("Valid the data");
    okButton.setMaximumSize(buttonDim);
    okButton.setMinimumSize(buttonDim);
   
    panel.add(okButton);    

            // add the "Cancel" button if there are hidden nodes
    cancelButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
            // return to the default state where a selection in the 
            // explorer updates the beamer
            hasHandlerDialog__ = false;

            frame.setVisible(false);
            try {
                frame.setClosed(true);
            } catch (PropertyVetoException p) {
                System.out.println("DialogBox.showBackedJPanels():"+
                " the frame closing fires a PropertyVetoException: "+p);
            }            
            caller.handleCreationDialog(false);
            return;
        } // actionPerformed()
    }); // extendedButton.addActionListener
   
    cancelButton.setBackground(Color.white);
    cancelButton.setBorder(new BevelBorder(
        BevelBorder.RAISED, InternalFrame.FORECOLOR, InternalFrame.FORECOLOR));
    cancelButton.setToolTipText("Cancel the operation");
    cancelButton.setMaximumSize(buttonDim);
    cancelButton.setMinimumSize(buttonDim);
    
    //separatorButton.setBackground(InternalFrame.BACKCOLOR);
    //separatorButton.setBorder(new BevelBorder(
      // BevelBorder.RAISED, InternalFrame.BACKCOLOR, InternalFrame.BACKCOLOR));
    
    //panel.add(separatorButton);
    panel.add(cancelButton);
    //panel.setBackground(InternalFrame.BACKCOLOR);
    //panel2.setBackground(InternalFrame.BACKCOLOR);
    
    // add to the frame
    pane = (JComponent) frame.getContentPane();
    pane.setLayout(new BorderLayout());

    pane.add(panel, BorderLayout.SOUTH);
    pane.add(panel2, BorderLayout.WEST);
    pane.add(_panel, BorderLayout.CENTER);

    frame.setFont(BeamerManager.plainButtonFont);
    frame.setBounds(
          Desktop.getDSize().width/2 - 200,
          Desktop.getDSize().height/2 - 150,
          _panel.getPreferredSize().width +50, 
          _panel.getPreferredSize().height+80);
            
    // no direct closing by the user
    frame.setDefaultCloseOperation(
        JInternalFrame.DO_NOTHING_ON_CLOSE);

    try {
      frame.setFrameIcon(new ImageIcon(
              Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
              Desktop.URL_SEPAR+"properties.gif").toURL()
              ) );
    } catch (MalformedURLException m){
      System.out.println("in DialogBox.showBackedJPanelse(), exception"+
        " on view icon: "+m.toString() );
    }              
            
    // put the frame in the desktop above all the other frames,
    // including those produced by pane.createInternalFrame() 
    // in showOkCancelInternalFrame()
    Desktop.addFrame(frame, new Integer(1000));
    frame.setVisible(true);     
    
    okButton = null;
    cancelButton = null;
    //separatorButton = null;
    //buttonDim = null;
    pane = null;
    panel = null;      
      
    return;
  } // showBackedJComponents()   
  
  /**
  * Displays one list for selecting one item.
  * For selecting several items, see showSimpleSelectionDialog().
  * For selecting several items in several lists, see showBackedJLists().
  * All the items must have the same type: String, ImmutableName or a subclass of BaseObject.
  * If it is BaseObject, getName().toString() is used to display the item.
  * @param _displayName of the element to select. May be null.
  * @param _l_values is the String, ImmutableName or BaseObject array of
  *   the non-null values to display.
  * @exception SmallError if the arguments are not correct, or if the number is
  *     below 2 or above 30.
  * @return index of the selected JRadioButton in _l_values, or
  *   '-1' if there is no user choice.
  */
  public static synchronized int showOneSelectionOneDialog(
    String _displayName, Object[] _l_values) throws SmallError {

    // check _l_values
    boolean ok = true;
    if (_l_values == null || _l_values.length > 30 || _l_values.length < 2 ||
       _l_values[0] == null)
          throw new SmallError("In DialogBox.showOneSelectionOneDialog(),"+
            " the second argument is null or its number of items is below 2 or above 30.");
    if (_l_values[0].getClass() != String.class &&
            !ImmutableName.class.isAssignableFrom( _l_values[0].getClass() ) &&
            !(_l_values[0] instanceof BaseObject)) {
          throw new SmallError("In DialogBox.showOneSelectionOneDialog(),"+
            " _l_values[0] contains a wrong type.");
    } // if
    
            // for testing purpose
            //System.out.println("Call to DialogBox.showOneSelectionOneDialog() ");


    // initialization
    JLabel          elementName = new JLabel("Select one "+_displayName);
    JLabel          comment = new JLabel("   (Use 'Tab' to skip/'Space' to select)");
    JPanel          checkBoxPanel = new JPanel();
    checkBoxPanel.setLayout(new BoxLayout(checkBoxPanel, BoxLayout.Y_AXIS) );
    checkBoxPanel.setBackground(InternalFrame.BACKCOLOR_2);
    checkBoxPanel.setForeground(InternalFrame.FORECOLOR);
    checkBoxPanel.setBorder(new EmptyBorder(2, 2, 2, 2));
    JCheckBox       rbutton;
    ButtonGroup     buttonGroup = new ButtonGroup();
    JDialog         dialog;
    ImmutableName   name = null;
    int             widthSize = 75;
    JOptionPane     pane;
    int             result = -1;

    checkBoxPanel.add(elementName);
    checkBoxPanel.add(comment);

    // builds up the dialog box
    if (_l_values[0].getClass() == String.class) {
      for (int i = 0; i < _l_values.length ; i++) {

        rbutton = new JCheckBox((String)_l_values[i]);
        checkBoxPanel.add(rbutton);
        rbutton.setActionCommand((String)_l_values[i]);
        rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
        rbutton.setBackground(InternalFrame.BACKCOLOR_2);
        rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
        buttonGroup.add(rbutton);
        widthSize = java.lang.Math.max(widthSize, ((String)_l_values[i]).length()+ 25);

      } // for
    } // if
    else {
      for (int i = 0; i < _l_values.length ; i++) {

        if ( ImmutableName.class.isAssignableFrom( _l_values[0].getClass() ) )
            name = (ImmutableName)_l_values[i];
        else
            name = (NameImpl) ((BaseObject)_l_values[i]).getName();
        
        rbutton = new JCheckBox( name.toString() );
        checkBoxPanel.add(rbutton);
        rbutton.setActionCommand( name.toString() );
        rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
        rbutton.setBackground(InternalFrame.BACKCOLOR_2);
        rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
        buttonGroup.add(rbutton);
        widthSize = java.lang.Math.max(widthSize, name.toString().length()+ 25);

      } // for
    } // else

    checkBoxPanel.setSize(200, widthSize);
    checkBoxPanel.setMaximumSize(new Dimension(200, widthSize*4));
    checkBoxPanel.setFont(BeamerManager.plainButtonFont);

    // shows the dialog box
    dialog = showOkCancelDialog("Selection", checkBoxPanel);

    // here, dialog waits an enter from the user,
    // then the next instructions are processed.
    pane = (JOptionPane)dialog.getContentPane().getComponent(0);

    // no action if CLOSED, and closes the dialog box
    if ( pane.getValue() == null) {
        dialog = null;
        rbutton = null;
        elementName = null;
        buttonGroup = null;
        comment = null;
        name = null;
        pane = null;
        checkBoxPanel = null;
        return result;
    }
    // else, it is an Integer
    int value = ((Integer)pane.getValue()).intValue();

      // no action if CANCEL, and close the dialog box
    if (value == JOptionPane.CANCEL_OPTION) {
        dialog = null;
        rbutton = null;
        elementName = null;
        buttonGroup = null;
        comment = null;
        name = null;
        pane = null;
        checkBoxPanel = null;
        return result;
    }
    // else it is OK, action and close the dialog box

    // closes if no radio button is selected
    if (buttonGroup.getSelection() == null) {
        dialog = null;
        rbutton = null;
        elementName = null;
        buttonGroup = null;
        comment = null;
        name = null;
        pane = null;
        checkBoxPanel = null;
        return result;
    }

    if (_l_values[0].getClass() == String.class) {
      for (int i = 0 ; i < _l_values.length ; i++) {

          if (buttonGroup.getSelection().getActionCommand().equals(
            _l_values[i]) ) {
              result = i;
              break;
          } // if

      } // for
    } else { // it is a BaseObject or an immutableName
      for (int i = 0 ; i < _l_values.length ; i++) {

            if ( ImmutableName.class.isAssignableFrom( _l_values[0].getClass() ) ) {
              if (buttonGroup.getSelection().getActionCommand().equals(
                ((ImmutableName)_l_values[i]).toString()) ) {
                  result = i;
                  break;
              } // if (buttonGroup
            }
            else {
              if (buttonGroup.getSelection().getActionCommand().equals(
                ((BaseObject)_l_values[i]).getName().toString()) ) {
                  result = i;
                  break;
              } // if (buttonGroup
            } // else

      } // for
    } // else
    
    dialog = null;
    rbutton = null;
    elementName = null;
    buttonGroup = null;
    comment = null;
    name = null;
    pane = null;
    checkBoxPanel = null;

    return result;
  } // showOneSelectionOneDialog()
 
  /**
    * Displays several String/BaseObject lists and a text field
    * for selecting one item or more in each.
    * Uses a JInternalFrame which shows the components,
    * where the components and the frame are closed after the reading of the properties
    * in the components by _controller.
    * The components editing by user is done through component listeners outside
    * this method, so component states are not changed by this method.
    * The single listener is generally the method caller.
    * This method calls _controller.handleCreationDialog() after the closing
    * of the internal frame. It is a partially reentrant method
    * since _controller may recalls it from handleCreationDialog(). <P>
    * For selecting one or several items in the same list, see showSimpleSelectionDialog()
    * and showSelectionsInOneDialog().
    * The lists size is from 1 (inclusive) to 16 (inclusive).
    * One JTextField may be displayed before or after the JLists.
    * All the items must have the same type: String or BaseObject.
    * If it is BaseObject, getName().toString() is used to display the item.
    * Called by RightMapPropertyEditor, ActionMapPropertyEditor,
    * StringTwoKeysMapProppertyEditor.AddKeyElement() and ActionNewACSyst.
    * Calls showBackedJComponents(). 
    * @param _controller handles the operations when the user ends this dialog.
    *    Never null.
    * @param _request text to display at the bottom of the frame. The size
    *     is limited to 50. Never null nor empty.
    * @param _l_lists array of String or BaseObject lists for the non-null values to display.
    *     Never null nor empty, and the maximal lenght is 4.
    * @param _l_titles titles for each list and test. Never null. 
    *     The titles order is the _l_lists order.
    *     Must have the same lenght than _l_lists, in the same order 
    *    if _field is null. If not, the lenght is the _l_lists size plus one,
    *    and the field title is always put at the end of _l_titles.
    * @param _verticalDisplay true for presenting the lists vertically; false for
    *    an horizontal presentation
    * @param _field a text field. If null, no JTextField is added. May be empty.
    * @param _textBeforeLists if _field is not null, true for having it before the JLists
    *    in the presentation; if false, _field is put at the end.
    * @exception SmallError if the arguments are not correct.
  */
  public static synchronized void showBackedJLists(
    CreationDialogListener _controller, String _request, JList[] _l_lists,
    String[] _l_titles, boolean _verticalDisplay,
    JTextField _field, boolean _textBeforeLists) throws SmallError {

    // basic check on arguments
    boolean ok = true;
    boolean ok2 = true;
    int j = 0;
    if (_l_lists == null || _l_lists.length > 4 || _l_lists.length == 0 ||
       _l_titles == null ||
       _controller == null || _request == null || _request.length() == 0 || 
       _request.length() > 50)
          throw new SmallError("In DialogBox.showBackedJLists(),"+
            " the controller argument, the request or the lists/title array is null,"+
            " empty or too large.");
    if (_l_titles.length != _l_lists.length && _field == null)
          throw new SmallError("In DialogBox.showBackedJLists(),"+
            " the list array and the title array have not the same size.");
    else if ((_l_titles.length != _l_lists.length+1) && _field != null)
          throw new SmallError("In DialogBox.showBackedJLists(),"+
            " the title array have not the size of the list array plus one.");
    
    // initialization
    JPanel            panel = new JPanel();
    Box               box;
    JPanel            panel2 = null;
    JLabel            label1;
    ListModel         list = null;
    Class             clas = null;
    JScrollPane       scrollPane = null;
    int               maxLength = -1;
 
   // check _l_lists types    
    for (int i = 0; i < _l_lists.length; i++) {
        if (_l_lists[i] == null) {
            ok = false;
            break;
        }
        list = _l_lists[i].getModel();
        clas = list.getElementAt(0).getClass();
        if (!String.class.isAssignableFrom(clas) && !BaseObject.class.isAssignableFrom(clas)) {
            ok2 = false;
            break;
        }
        for (j = 0; j < list.getSize(); j++ ) {
            if (!clas.isAssignableFrom(list.getElementAt(j).getClass())) {
                ok2 = false;
                break;
            } // if (list
        } // for j
    } // for i
    if (!ok) throw new SmallError("in DialogBox.showBackedJLists(),"+
      " _l_lists contains a null value.");
    if (!ok2) throw new SmallError("in DialogBox.showBackedJLists(),"+
      " in _l_lists, the list "+j+" does not contain only Strings or BaseObjects,"+
      " or it does not contain only one type among these two types.");

    panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS) );
    panel.setBackground(InternalFrame.BACKCOLOR_2);
    panel.setForeground(InternalFrame.FORECOLOR); 
    panel.setFont(BeamerManager.plainButtonFont);
    
    label1 = null;
    if (_field != null)
        label1 = new JLabel("Select an item in each list. Enter a text in the text field.");
    else
        label1 = new JLabel("Select an item in each list.");
    label1.setAlignmentX(label1.CENTER_ALIGNMENT);
    label1.setBackground(InternalFrame.BACKCOLOR_2);
    //label1.setForeground(InternalFrame.FORECOLOR);
    
    panel.add( label1 );

    if (_verticalDisplay) box = Box.createVerticalBox();
    else box = Box.createHorizontalBox();

    if (_field != null && _textBeforeLists) {
        panel2 = new JPanel();
        //panel2.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS) );
        panel2.setBackground(InternalFrame.BACKCOLOR_2);
        panel2.setForeground(InternalFrame.FORECOLOR_BLACK); 
 
        label1 = new JLabel(_l_titles[_l_titles.length - 1]);
        label1.setAlignmentX(label1.CENTER_ALIGNMENT);
        label1.setBackground(InternalFrame.BACKCOLOR_2); 
        
        panel2.add(label1);
        panel2.add(_field);

        box.add(panel2);        
    } // if
    
    for (int i = 0; i < _l_lists.length; i++) {
        panel2 = new JPanel();
        //panel2.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS) ); // bug: BoxLayout can't be shared
        panel2.setBackground(InternalFrame.BACKCOLOR_2);
        panel2.setForeground(InternalFrame.FORECOLOR); 
 
        label1 = new JLabel(_l_titles[i]);
        label1.setAlignmentX(label1.CENTER_ALIGNMENT);
        label1.setBackground(InternalFrame.BACKCOLOR_2); 
        
        panel2.add(label1);
        if (_l_lists[i].getModel().getSize() < 7) panel2.add(_l_lists[i]);
        else {
            scrollPane = new JScrollPane(_l_lists[i]);
            for (j = 0; j < _l_lists[i].getModel().getSize(); j++) {
                if (_l_lists[i].getModel().getElementAt(j).toString().length() > maxLength)
                    maxLength = _l_lists[i].getModel().getElementAt(j).toString().length();
            } // for j
            if (maxLength < 13 && _l_lists[i].getModel().getSize() < 12) {
                scrollPane.setMaximumSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*2/5, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION/2));
                scrollPane.setMinimumSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*2/5, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION/2));
                scrollPane.setPreferredSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*2/5, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION/2)); 
            }
            else if (maxLength < 26 && _l_lists[i].getModel().getSize() < 12) {
                scrollPane.setMaximumSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*4/5, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION/2));
                scrollPane.setMinimumSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*4/5, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION/2));
                scrollPane.setPreferredSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*4/5, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION/2));                 
            }
            else if (_l_lists[i].getModel().getSize() < 12) {
                scrollPane.setMaximumSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*3/2, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION/2));
                scrollPane.setMinimumSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*3/2, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION/2));
                scrollPane.setPreferredSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*3/2, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION/2));                 
            }
            else if (maxLength < 13) {
                scrollPane.setMaximumSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*2/5, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION));
                scrollPane.setMinimumSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*2/5, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION));
                scrollPane.setPreferredSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*2/5, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION)); 
            }
            else if (maxLength < 26) {
                scrollPane.setMaximumSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*4/5, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION));
                scrollPane.setMinimumSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*4/5, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION));
                scrollPane.setPreferredSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*4/5, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION));                 
            }
            else {
                scrollPane.setMaximumSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*3/2, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION));
                scrollPane.setMinimumSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*3/2, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION));
                scrollPane.setPreferredSize(
                  new Dimension(
                    CommonPropertyEditor.LIST_WIDTH_DIMENSION*3/2, 
                    CommonPropertyEditor.LIST_HEIGHT_DIMENSION));                 
            }
            panel2.add(scrollPane);
            maxLength = -1;
        } // else

        box.add(panel2);
    } // i

    if (_field != null && !_textBeforeLists) {
        panel2 = new JPanel();
        //panel2.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS) );
        panel2.setBackground(InternalFrame.BACKCOLOR_2);
        panel2.setForeground(InternalFrame.FORECOLOR_BLACK); 
 
        label1 = new JLabel(_l_titles[_l_titles.length - 1]);
        label1.setAlignmentX(label1.CENTER_ALIGNMENT);
        label1.setBackground(InternalFrame.BACKCOLOR_2); 
        
        panel2.add(label1);
        panel2.add(_field);

        box.add(panel2);        
    } // if
    
    panel.add( box );

    list = null;
    clas = null;
    box = null;
    panel2 = null;
    label1 = null;

    showBackedJComponents(_controller, _request, panel);
    
    panel = null;
  } // showBackedJLists()  

  /**
  * Display a confirm dialog for closing and saving operations on a base object.
  * @param _name of the base object
  * @return boolean[] with 2 items: first (0) is true if closing is
  * a user request, second is true if closing and saving are user requests.
  */
  public static boolean[] showClosingAndSavingDialog(ImmutableName _name) {
    // variables
    JLabel        label;
    JPanel        panel = new JPanel();
    JDialog       dialog;
    JOptionPane   pane;
    ButtonGroup   group = new ButtonGroup();
    JRadioButton  rbutton;
    char          n = 'N';
    char          y = 'Y';
    char          e = 'E';

          // for testing and tracking
      //System.out.println("call to ActionUtilities.showClosingAndSavingDialog()");

    // build up the panel
    label = new JLabel("Do you want to CLOSE "+_name+"?");
    label.setAlignmentX(label.LEFT_ALIGNMENT);
    panel.add( label );

    label = new JLabel("Select one button:");
    label.setAlignmentX(label.LEFT_ALIGNMENT);
    panel.add( label );

    panel.add(rbutton = new JRadioButton("No") );
    rbutton.setActionCommand("No");
    rbutton.setMnemonic(n);
    rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
    rbutton.setBackground(InternalFrame.BACKCOLOR_8);
    rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
    group.add(rbutton);
    panel.add(rbutton = new JRadioButton("Yes, with saving") );
    rbutton.setActionCommand("Yes, with saving");
    rbutton.setMnemonic(y);
    rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
    rbutton.setBackground(InternalFrame.BACKCOLOR_8);
    rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
    group.add(rbutton);
    panel.add(rbutton = new JRadioButton("Yes, without saving") );
    rbutton.setActionCommand("Yes, without saving");
    rbutton.setMnemonic(e);
    rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
    rbutton.setBackground(InternalFrame.BACKCOLOR_8);
    rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
    group.add(rbutton);

    // show the dialog box
    dialog = DialogBox.showOkCancelDialog("Question", panel);

    // here, dialog waits a user imput,
    // then the next instructions are processed.
    pane = (JOptionPane)dialog.getContentPane().getComponent(0);
    // no action if CLOSED, and close the dialog box
    if ( pane.getValue() == null) {
      DialogBox.showOkMessage("The entity is not closed.");
      return new boolean[] {false, false};
    }
    // else, it is an Integer
    int value = ((Integer)pane.getValue()).intValue();

      // no close if CANCEL, and close the dialog box
    if (value == JOptionPane.CANCEL_OPTION) return new boolean[] {false, false};
      // else, it is OK, continue

    // no close if no radio button is selected
    if (group.getSelection() == null) {
      DialogBox.showOkMessage("No selection."+
          " The entity is not closed.");
      return new boolean[] {false, false};
    }

    if (group.getSelection().getActionCommand().equals("No") ) {
      return new boolean[] {false, false};
    }
    if (group.getSelection().getActionCommand().equals("Yes, with saving") ) {
        return new boolean[] {true, true};
    }
    if (group.getSelection().getActionCommand().equals("Yes, without saving") ) {
        return new boolean[] {true, false};
    }

    label = null;
    panel = null;
    dialog = null;
    pane = null;
    group = null;
    rbutton = null;
    return new boolean[] {false, false};
  } // showClosingAndSavingDialog()

  /**
   * Show a map of Strings to read, without action on it.
   * Derived from StringMapPropertyEditor.getCustomEditor().
   * Uses the static properties m_values__, keyList__ and valueList__, and
   * the static class DialogBox.KeyListSelectionListener.
   * Called by ActionImportACS. Calls showOkCancelDialog().
   * @param _title title of the map
   * @param _m_values map of Strings to display to the user
   */
  public static synchronized void showImmutableStringMap(String _title,
          Map<String, List<String>> _m_values) {

    JPanel result = new JPanel();

    // initialization
    valuesModel__ = new DefaultListModel();
    JList           valueList = new JList(valuesModel__);
    JPanel          keyListPanel = new JPanel();
    JPanel          valueListPanel = new JPanel();
    JPanel          mapPanel = new JPanel();
    JScrollPane     keyScrollPane = null;
    JScrollPane     valueScrollPane = null;
    int             width = CommonPropertyEditor.LIST_WIDTH_DIMENSION*2;
    int             height = CommonPropertyEditor.LIST_HEIGHT_DIMENSION;

    // these static properties are used by DialogBox.KeyListSelectionListener    
    m_values__ = _m_values;
    keyList__ = new JList(_m_values.keySet().toArray());

          // for testing purpose
          //System.out.println("DialogBox.showImmutableStringMap(),"+
            //" the keys list size is: "+keysModel_.getSize());
          //if (!keysModel_.isEmpty()) System.out.println("the first key is:"+
            //keysModel_.get(0)+".");
          //System.out.println("DialogBox.showImmutableStringMap(),"+
            //" the values list size is: "+valuesModel_.getSize());
          //if (!valuesModel_.isEmpty()) System.out.println("the first value is:"+
            //valuesModel_.get(0)+".");


    // valuesModel_ contains the values for the first key
    if (_m_values.keySet().size() != 0) {
        keyList__.setSelectedIndex(0);
        for (String value : _m_values.entrySet().iterator().next().getValue()){
            valuesModel__.addElement(value);
        }
    } // if
    keyList__.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    // handling of the selection events in the nested static class
    // DialogBox.KeyListSelectionListener
    keyList__.addListSelectionListener(new KeyListSelectionListener());
    
    keyList__.setBackground(InternalFrame.NON_EDITABLECOLOR);
    ((DefaultListCellRenderer)keyList__.getCellRenderer()).
      setHorizontalAlignment(JLabel.RIGHT);
    ((DefaultListCellRenderer)keyList__.getCellRenderer()).
      setHorizontalTextPosition(JLabel.RIGHT);
    keyList__.setFont(BeamerManager.plainFont);
    keyScrollPane = new JScrollPane(keyList__);
    
    valueList = new JList(valuesModel__);
    valueList.setBackground(InternalFrame.NON_EDITABLECOLOR);

    //if (propertyDescriptor_.getValue("horizontalTextPositionToLeft") != null &&
      //      (Boolean)propertyDescriptor_.getValue("horizontalTextPositionToLeft")) {
        //((DefaultListCellRenderer)valueList.getCellRenderer()).
         //   setHorizontalAlignment(JLabel.LEFT);
        //((DefaultListCellRenderer)valueList.getCellRenderer()).
          //  setHorizontalTextPosition(JLabel.LEFT);
    //} else {
        ((DefaultListCellRenderer)valueList.getCellRenderer()).
            setHorizontalAlignment(JLabel.RIGHT);
        ((DefaultListCellRenderer)valueList.getCellRenderer()).
            setHorizontalTextPosition(JLabel.RIGHT);
    //}
    
    valueList.setFont(BeamerManager.plainFont);
    // valueList has not this instance as ListSelectionListener
    valueScrollPane = new JScrollPane(valueList);
    
    keyScrollPane.getHorizontalScrollBar().setValue(
      keyScrollPane.getHorizontalScrollBar().getMaximum() );  // does not work well
    keyListPanel.add(keyScrollPane);
    keyListPanel.setBackground(Color.WHITE);
    keyListPanel.setBorder(new LineBorder(InternalFrame.FORECOLOR));
    keyScrollPane.setMaximumSize(
              new Dimension(width, height));
    keyScrollPane.setMinimumSize(
              new Dimension(width, height));
    keyScrollPane.setPreferredSize(
              new Dimension(width, height));

          // for testing purpose
          //System.out.println("DialogBox.showImmutableStringMap(),"+
            //" the font cell renderer in list is a: "+
            //((DefaultListCellRenderer)keyList.getCellRenderer()).getFont() );

    
    valueScrollPane.getHorizontalScrollBar().setValue(
      valueScrollPane.getHorizontalScrollBar().getMaximum() ); 
    valueListPanel.add(valueScrollPane);
    valueListPanel.setBackground(Color.WHITE);
    valueListPanel.setBorder(new LineBorder(InternalFrame.FORECOLOR));
    valueScrollPane.setMaximumSize(
              new Dimension(width, height));
    valueScrollPane.setMinimumSize(
              new Dimension(width, height));
    valueScrollPane.setPreferredSize(
              new Dimension(width, height));
    
    // builds up the result
    mapPanel.add(keyListPanel);
    mapPanel.add(valueListPanel);
    
    result.add(Box.createVerticalBox());
    result.setBackground(Color.white);
    result.setBorder(new EtchedBorder());
    result.add(mapPanel);

    showOkCancelDialog(_title, result);

    mapPanel = null;
    valueList = null;
    valueListPanel = null;
    keyListPanel = null;
    valueScrollPane = null;
    keyScrollPane = null;
    result = null;
    // static properties    
    valuesModel__ = null;
    m_values__ = null;
    keyList__ = null;
  } // showImmutableStringMap()
  
  /**
  * For implementing the user selection of items in the Explorer,
  * implement a continous TreeSelectionListener which is used only when
  * the showExplorerSelectionDialog() method is called. <P>
  * The user selection is put in selectedObjectsSetMap__,
  * in the JList where the key is an interface implemented by
  * the selection class.
  * This method is called by the JTree listener in Explorer,
  * when the user clicks on a node or a leaf.
  * The number of user selections is limited to 20.
  * When the selection class implements several interfaces
  * which are selectedObjectsSetMap__ keys, the selection
  * is put in every associated JList.
  * This method is independant to the beamer listener that listens to
  * the user selection in Explorer
  * (see ActionOpenBeamer.valueChanged() for instance).
  * Calls TreeManager.getTrueUserObjectInTreeExplorer().
  * @param _evt from Explorer JTree
  */
  public void valueChanged(TreeSelectionEvent _evt) {

            // for testing purpose
            //System.out.println("call to DialogBox.valueChanged() - "+
              //      "selectedObjectsCount__:"+selectedObjectsCount__);      
      
      // if showExplorerSelectionDialog() doesn't wait an event, return
    if (!waitClickInExplorer__) return;
      // if more than 20 selections, or more than one selection
    synchronized (new Integer(selectedObjectsCount__)) {
      if (selectedObjectsCount__ > 20 ||
        (selectedObjectsCount__ > 0 && onlyOneSelectionAllowed__)) {
          showErrorMessage("Too many selections.");
          return;
      } // if
    } // synchro
      // if no last path component
    if (_evt.getNewLeadSelectionPath() == null) return;

    // initialization
    DefaultMutableTreeNode node = (DefaultMutableTreeNode)_evt.
            getNewLeadSelectionPath().getLastPathComponent();
    Object userSelection = TreeManager.getTrueUserObjectInTreeExplorer(node);
    DefaultListModel listModel = null;
    boolean typeFound = false;

                                // for testing purpose
                                //System.out.println("in DialogBox.valueChanged() "+
                                  //"the selected object class is: "+userSelection.getClass() );

    synchronized (selectedObjectsSetMap__) {
        for (Iterator ite = selectedObjectsSetMap__.keySet().iterator()
          ; ite.hasNext() ; ) {
          Class type = (Class) ite.next();

                                // for testing purpose
                                //System.out.println("in DialogBox.valueChanged() "+
                                  //"one of the searched class is: "+type );

          // select the right jlist;
          // an actor is put in the eligible party and in the resource jlists
          if ( type.isAssignableFrom(userSelection.getClass() ) ) {
            typeFound = true;
            listModel = (DefaultListModel)((JList)
                selectedObjectsSetMap__.get(type)).getModel();
            if (!listModel.contains(userSelection) ) {
              synchronized (new Integer(selectedObjectsCount__))
                {selectedObjectsCount__++;}
              listModel.addElement(userSelection);

                                // for testing purpose
                                //System.out.println("in DialogBox.valueChanged() "+
                                  //"the selected object is added to the list for: "+type );
            
            } // if (!listModel.contains(
          } // if userSelection
        } //for ite
    } // synchro
                                
    if (!typeFound)
        showErrorMessage("The type of this selection is not applicable.");

    // unselect the selected node in the explorer,
    // to allow the user to see it through one clic in the explorer
    InternalFrameModel.getTreeInExplorer().clearSelection();

    node = null;
    listModel = null;
    userSelection = null;
  } // valueChanged()

  /**
  * Called by BaseObjectHandler and CollectionPropertyEditor.actionPerformed()
  * to indicate the start and the end of an user dialog.
  * See onGoingObjectSelectionOrUserDialog().
  * @param _b is the new value to set
  */
  public static void setHasHandlerDialog(boolean _b){
    hasHandlerDialog__ = _b;
  } // hasBeamerDialog()

  /**.
  * Called by ActionOpenBeamer.valueChanged(), BaseObjectHandler,
  * CollectionPropertyEditor.actionPerformed().
  * @return true if showExplorerSelectionDialog() is activated to select an object
  *   in the explorer, or if a beamer dialog is displayed (for creating
  *   a new BaseObject, or any other reason)
  */
  public static boolean onGoingObjectSelectionOrUserDialog(){
    return (waitClickInExplorer__ || hasHandlerDialog__);
  } // onGoingObjectSelectionOrUserDialog()


}  // DialogBox