//Title:        Access Road
//Version:      0.6.0
//Copyright:    Copyright 2002, 2004, 2007, 2008, 2009 EURL ACCBEE
//Author:       Patrick Thazard ( )
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui1;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.beans.*;
import ARoad0.Pattern.*;
import ARoad0.gDMak.*;

/**
 * A DesktopManager that keeps its frames inside the desktop when they are
 * resized or dragged, and that keeps the explorer frame location
 * at the upper-left corner of the desktop.
 * @version 0.5.1
 */
public class InsideDesktop extends DefaultDesktopManager implements Singleton {

  /**
   * Tag internal frames that are being resized using a client
   * property with the name RESIZING.  Used in setBoundsForFrame().
   */
  protected static final String RESIZING = "RESIZING";

  /**
   * Tag internal frames that are being moved using a client
   * property with the name DRAGGING.
   */
  protected static final String DRAGGING = "DRAGGING";

  /**
   * Initial location of the frame before the dragging
   */
  protected static Point initialLocation__ = new Point();

  /* the only instance of the class, following singleton pattern */
  protected static final InsideDesktop instance__ = new InsideDesktop();

  /**
   * With this private constructor, the compiler won't generate
   * a default public constructor.
   */
  private InsideDesktop() {
    super();
  }
  /**
   * Only mean to get an instance.
   * @return InsideDesktop
   */
  public static InsideDesktop getInstance(){
    return instance__;
  } // getInstance()

  /**
   * @param _f JComponent
   */
  public void beginDraggingFrame(JComponent _f) {
    _f.putClientProperty(DRAGGING, Boolean.TRUE);
    initialLocation__ = _f.getLocation();
  } // beginDraggingFrame(JComponent)

  /**
   * Keeps the explorer frame location at the upper-left corner of the desktop,
   * and the other frames in the desktop.
   * @param _f JComponent
   */
  public void endDraggingFrame(JComponent _f) {
    // only deal with internal frames
    if (_f instanceof JInternalFrame == false) {
      super.endDraggingFrame(_f);
      return;
    } // if

    if (((JInternalFrame)_f).getTitle() != null &&
            ((JInternalFrame)_f).getTitle().equals("Explorer")) {
      _f.setLocation(new Point(0,0));
    } // if
    else {
      JDesktopPane desk = ((JInternalFrame)_f).getDesktopPane();
      Dimension d = desk.getSize();
      // reset the location to the initial one
      // if the frame is dragged outside the desktop
      if (_f.getX() < 0 ||
          _f.getX()+_f.getWidth() > d.width ||
          _f.getY() < 0 ||
          _f.getY()+_f.getHeight() > d.height)
              _f.setLocation(initialLocation__);
      desk = null;
      d = null;
    } // else

    super.endDraggingFrame(_f);
    _f.putClientProperty(DRAGGING, Boolean.FALSE);
  }

  /**
   * @param _f JComponent
   * @param dir int
   */
  public void beginResizingFrame(JComponent _f, int dir) {
    _f.putClientProperty(RESIZING, Boolean.TRUE);
  } // beginResizingFrame(JComponent, int)

  /**
   * @param _f JComponent
   */
  public void endResizingFrame(JComponent _f) {
    _f.putClientProperty(RESIZING, Boolean.FALSE);
  } // endResizingFrame(JComponent)

  /**
   * This is called any time a frame is moved or resized.
   * This implementation keeps the frame
   * from leaving the desktop when it is resized.
   * @param _f JComponent
   * @param _x int left border
   * @param _y int top border
   * @param _w int width
   * @param _h int height
   */
  public void setBoundsForFrame(JComponent _f, int _x, int _y, int _w,
   int _h) {
    // only deal with internal frames
    if (!(_f instanceof JInternalFrame)) {
      super.setBoundsForFrame(_f, _x, _y, _w, _h);
      return;
    } // if

    JInternalFrame frame = (JInternalFrame)_f;

      // Figure out if we are being resized (otherwise it's just a move)
    boolean resizing = false;
    Object r = frame.getClientProperty(RESIZING);
    if (r != null && r instanceof Boolean) {
      resizing = ((Boolean)r).booleanValue();
    } // if

    JDesktopPane desk = frame.getDesktopPane();
    Dimension d = desk.getSize();

    // for testing and tracking
    //System.out.println("in InsideDesktop, resizing: "+resizing);

      // Adjust to keep the frame on the desktop
    if (_x < 0) {              // too far left?
      if (resizing)
          _w += _x;               // don't get wider!
      // work if resized but not if dragged, as it should be...
      _x=0;                    // flush against the left side
    } // if
    else {
      if (_x+_w>d.width) {      // too far right?
         if (resizing)
           _w = d.width-_x;       // don't get wider!
         else
           _x = d.width-_w;       // flush against the right side
      } // if
    } // else

    if (_y < 0) {              // too high?
      if (resizing)
          _h += _y;               // don't get taller!
      _y=0;                    // flush against the top
    } // if
    else {
      if (_y+_h > d.height) {   // too low?
          if (resizing)
            _h = d.height - _y;   // don't get taller!
          else
            _y = d.height-_h;     // flush against the bottom
      } // if
    } // else

      // Set the correct position and size for the internal frame
    super.setBoundsForFrame(_f, _x, _y, _w, _h);

    frame = null;
    desk = null;
    d = null;
    return;
  } // setBoundsForFrame(JComponent, int, int, int, int)

} // class InsideDesktop
