//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui1;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.Rectangle;
import java.beans.*;
import java.net.MalformedURLException;
import java.net.URI;
import javax.swing.border.BevelBorder;

import java.util.Set;
import java.util.HashSet;
import java.util.Map;

import ARoad0.Pattern.*;
import ARoad0.Gui2.*;
import ARoad0.ErrW.*;
import ARoad0.ErrB.BaseError;
import ARoad0.gDMak.*;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBase.BaseManagerImpl;
import ARoad0.gBase.NameImpl;
import ARoad0.gBaseInterface.ViewInBase;
import ARoad0.gBase.EPRViewInBaseImpl;

/**
 * This class is responsible to built up and manage the internal frames
 * in the desktop. It manages both the explorer frame,
 * the ACS frames, the view frames, the diagram frames, the text frames,
 * the 'All views' frames and the Structure frame.
 * An internalFrame may have a null title if the number of frames is more than
 * 20. It works with the InternalFrameModel class.
 * It implements the singleton pattern.
 * @version 0.7.0
 */
public class InternalFrame implements Singleton, MVC_View {

  /**
   * the only instance of this class, following the singleton pattern
   */
  private static InternalFrame      instance__ = new InternalFrame();
  /**
   * the instance, following the MVC pattern, which manages the internal frames data
   */
  private static InternalFrameModel model__ = InternalFrameModel.getInstance();

  // color 
  // ocre pale rose
  public static final Color      BACKCOLOR = new Color(244, 231, 186);
  // bleu-vert emeraude tres vif
  public static final Color      FORECOLOR = new Color(91, 240, 249);
  public static final Color      FORECOLOR_BLACK = new Color(0, 0, 0);
  // jaune
  public static final Color      BACKCOLOR_2 = new Color(244, 250, 186);
  // gris bleu
  public static final Color      BACKCOLOR_3 = new Color(200, 220, 220);
  // jaune pale
  public static final Color      NON_EDITABLECOLOR = new Color(244, 250, 200);
  // ocre pale
  public static final Color      BACKCOLOR_8 = new Color(244, 250, 186);  
  
  public static final Color      SOFTGREY = new Color(225, 225, 225);
  // marron chair
  public static final Color      BACKCOLOR_4 = new Color(243, 243, 186);
  // gris moyen-clair un peu vert
  public static final Color      BACKCOLOR_5 = new Color(186, 206, 206);
  // gris moyen un peu violet
  public static final Color      BACKCOLOR_6 = new Color(186, 186, 206);
  // bleu vert moyen in GraphicContainer
  public static final Color      FORECOLOR_2 = new Color(33, 180, 189); 
  // vert foncé in GraphicText
  public static final Color      FORECOLOR_3 = new Color(0, 120, 50);
  // rouge foncé in GraphicText
  public static final Color      FORECOLOR_4 = new Color(200, 30, 30); 
  
  // variables for the frames
  private static final int       INITIAL_CAPACITY = 10; //the default value:101
  
  // variables for the frames
  private static IconPolice      iconPolice__ = IconPolice.getInstance();

  /**
  * only one private constructor
  */
  public InternalFrame() {
  }

  /**
  * Only method to get an instance.
  * @return the instance
  */
  public static synchronized InternalFrame getInstance(){
    return instance__;
  } // getInstance()

  /**
  * Creates and displays a frame including an ACS tree,
  * with a frame listener which fires events to ActionCloseACS,
  * and register _object as a base object.
  * The frame name is the last 4 components, if possible.
  * Caution: it is so theorically possible to have 2 ACS frames having the same name,
  * if the IS is complex with 2 ACS names being identical at their ends.
  * Does not read the GUI configuration to apply the previous disposition.
  * Error, with message to user, if the number of frames
  * is superior to 20.
  * @param _name of the object shown, with 2 components at least.
  *   Equal to the name of _object in the base.
  * @param _tree to display
  * @param _object is associated to the tree in the base. May be null.
  * @exception ProcessError if _object is null.
  * @return the JInternalFrame displayed in the desktop or a new
  *   JInternalFrame() (not displayed) if there are more than 20 open frames
  */
  public static synchronized JInternalFrame showACSTreeFrame(ImmutableName _name,
      JTree _tree, BaseObject _object) throws ProcessError {
    // variables
    JInternalFrame            frame;

    // creates an ACS frame closable, iconifiable
    // The frame name is the last 4 components, if possible
    if (_name.noOfComponents() > 3) 
        frame = new JInternalFrame("ACS  "+ _name.subname(
                _name.noOfComponents() - 4, _name.noOfComponents()).toString(),
            true, true, true ,true);
    else if (_name.noOfComponents() > 2) 
        frame = new JInternalFrame("ACS  "+ _name.subname(
                _name.noOfComponents() - 3, _name.noOfComponents()).toString(),
            true, true, true ,true);
    else if (_name.noOfComponents() > 1)
        frame = new JInternalFrame("ACS  "+ _name.subname(
                _name.noOfComponents() - 2, _name.noOfComponents()).toString(),
            true, true, true ,true);
    else if (!_name.isEmpty())
        frame = new JInternalFrame("ACS  "+ _name.toString(),
            true, true, true ,true);
    else return null;

    _tree.setSize(new Dimension(900, 500)); // new: not PreferedSize()
    _tree.getSelectionModel().setSelectionMode(
        TreeSelectionModel.SINGLE_TREE_SELECTION);

    frame.getContentPane().setLayout(new BorderLayout());

    try {
      frame = prepareFrame(frame, _tree);
    } catch (DialogError d){
      // more than 20 open frames
      throw new ProcessError("The limit of 20 open frames is get.");
    }
    // sets the location below the beamer, excepted if it has been moved down
    frame = setFrameDisposition(frame);
    try {
      frame.setFrameIcon(new ImageIcon(
              Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
              Desktop.URL_SEPAR+"acs.gif").toURL()
        ) );
    } catch (MalformedURLException m){
      throw new InternalError("InternalFrame.showACSTreeFrame(): "+m.toString() );
    }

    // adds a frame listener when the frame is closed or activated
    // future: when an ACS frame is not always the main ACS frame
    frame = InternalFrameUtilities.addACSFrameListener(frame);

    // adds to m_BaseObjectsInMainFrames__, m_MainFramesOfBaseObjects__ and
    // m_JComponentOfBaseObjects__
    if (_object != null) {
      model__.addMainFrameBaseObject(_object, frame, _tree);
    }
    
    // puts the frame in the desktop
    Desktop.addFrame(frame, new Integer(1));
    frame.setVisible(true);

    return frame;
  } // showACSTreeFrame()

  /**
  * Creates and displays a frame including a graphical view,
  * with a frame listener which fires events to ActionCloseView,
  * and registers _object as a base object. Displays the NoThan view indicator
  * in the frame title if necessary.
  * Adds the 'See why' button.
  * If there are hidden nodes in the view, adds the 'New Extended view' button.
  * Error, with message to user, if the number of frames
  * is superior to 20. Error when the extended view is opened if the number of
  * view items is superior to 46.
  * Calls InternalFrameUtilities.getViewFrameTitleFromViewName().
  * @param _gview to display
  * @param _object is the view associated to the frame in the base.
  * Its name must have at least two components.
  * @exception ProcessError if _object is null.
  * @return the JInternalFrame displayed in the desktop. May be empty if
  *   there is more than 20 open frames in the desktop.
  */
  public static synchronized JInternalFrame showViewFrame(GraphicView _gview,
    BaseObject _object) throws ProcessError {
    // variables
    final GraphicView         gview = _gview;
    final BaseObject          object = _object;
    JInternalFrame            frame;
    JButton                   whyButton = new JButton("See why");
    JButton                   extendedButton = new JButton("Extended view");
    JButton                   separatorButton = new JButton("    ");
    JComponent                pane;
    JPanel                    panel = new JPanel();
    String                    title = InternalFrameUtilities.
                                        getViewFrameTitleFromViewName(_object.getName());

                  // for testing and tracking
                  //System.out.println("in InternalFrame.showViewFrame(), "+
                    //      "call to NameImpl.component() for "+_object.getName()+
                      //    " and the index "+_object.getName().noOfComponents()+" - 2");  
    
    // create a view frame closable, iconifiable
    frame = new JInternalFrame(title, true, true, true ,true);
    
    // add the "See why" button
    whyButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
          // test if the view justification frame does already exist
          JInternalFrame iframe;
          boolean open = false;
          for (int i = 0; i < Desktop.getCurrentFrames().length; i++) {
            iframe = Desktop.getCurrentFrames()[i];
            if (iframe.getTitle().equals(object.getName().toString()+ " 'See why' text")) 
                open = true;
          } // for
          if (open) {
              DialogBox.showErrorMessage("The 'See why' window is already opened.");
              iframe = null;
              return;
          }
          
          showTextFrame(gview.setWhyText(),
                  object.getName().toString()+ " 'See why' text",
                  Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
                        Desktop.URL_SEPAR+"view.gif"));
          iframe = null;
          return;
      } // actionPerformed()
    }); // whyButton.addActionListener

    whyButton.setBackground(Color.white);
    whyButton.setBorder(new BevelBorder(
      BevelBorder.RAISED, Color.white, FORECOLOR));
    whyButton.setToolTipText("Describe the access paths that are found in this view");
    panel.add(whyButton);

    // add the "Extended view" button if there are hidden nodes
    if ( !_gview.getHiddenNodesInLinks().isEmpty() ) {
      
      extendedButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
            if (gview.getAllNodesInLinks().size() > 46) {
              DialogBox.showErrorMessage(
              "Sorry, the number of nodes in this view cannot be superior to 46.");
              return;
            }
            /*
            if (gview.getHiddenNodesInLinks().isEmpty()) {
              DialogBox.getInstance().showErrorMessage(
              "This is an extended view, since there is no hidden node.");
              return;
            }
            */

            Set l_BaseObject = new HashSet( gview.getAllNodesInLinks() );

            ActionNewView.getInstance().createExtentedEPRView(
              object.getName(), l_BaseObject);
            return;
        } // actionPerformed()
      }); // extendedButton.addActionListener

      extendedButton.setBackground(Color.white);
      extendedButton.setBorder(new BevelBorder(
        BevelBorder.RAISED, Color.white, FORECOLOR));
      extendedButton.setToolTipText("New view including all the hidden objects"+
        " in the current access paths");
      separatorButton.setBackground(Color.white);
      separatorButton.setBorder(new BevelBorder(
        BevelBorder.RAISED, Color.white, Color.white));
      panel.add(separatorButton);
      panel.add(extendedButton);
    } // extended button

    panel.setBackground(Color.white);
    pane = (JComponent) frame.getContentPane();
    pane.setLayout(new BorderLayout());
    pane.add(panel, BorderLayout.NORTH);

    // add a scrollPane to the gview
    try {
      frame = prepareFrame(frame, _gview);
    } catch (DialogError d){
        // more than 20 open frames
        whyButton = null;
        extendedButton = null;
        separatorButton = null;
        pane = null;
        panel = null;  
        frame = null;        
        throw new ProcessError("The limit of 20 open frames is get.");
    }
    // set the location below the beamer, except if it has been moved down
    frame = setFrameDisposition(frame);
    // if the view has less than 4 nodes, set the height from 400 to 200
    if (_gview.getEorDataModel().getEorM_BaseObjects().size() < 4)
        frame.setBounds(
           (int)frame.getLocation().getX(),
           (int)frame.getLocation().getY()+200,
           frame.getWidth(),
           200);

    try {
      frame.setFrameIcon(new ImageIcon(
              Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
                        Desktop.URL_SEPAR+"view.gif").toURL()
              ) );
    } catch (MalformedURLException m){
      System.out.println("in InternalFrame.showViewFrame(), exception"+
        " on view icon: "+m.toString() );
    }

    // add frame listener
    frame = InternalFrameUtilities.addViewFrameListener(frame);

    // add to m_BaseObjectsInMainFrames__, m_MainFramesOfBaseObjects__,
    // m_JComponentOfBaseObjects__
    if (_object != null) {
      model__.addMainFrameBaseObject(_object, frame, _gview);
    }


                  // for testing and tracking
                  //System.out.println("in InternalFrame.showViewFrame(), "+
                    //      "create a visible JInternalFrame.");      
    
    // put the frame in the desktop
    Desktop.addFrame(frame, new Integer(1));
    frame.setVisible(true);

    whyButton = null;
    extendedButton = null;
    separatorButton = null;
    pane = null;
    panel = null;  
    
    return frame;
  } // showViewFrame()


  /**
  * Creates and displays the beamer panel for a base object. Called by
  * openBeamer(). Gets the beamer frame previous disposition, from the
  * GUI configuration file.
  * No update of m_BaseObjectsInMainFrames__, because it does not include the
  * beamer.
  * Error, with message to user, if the number of frames is superior to 20.
  * @param _pane to display, which is updated by the caller of this
  * method.
  * May be null. It is not used so far.
  */
  static synchronized void showBeamerFrame(JComponent _pane) {
    // variables
    JInternalFrame   beamer;
      
    // create a frame closable, iconifiable
    beamer = new JInternalFrame("Beamer ", true, true, true ,true);

    beamer.getContentPane().setLayout(new BorderLayout());
    //beamer.setFont(Desktop.getInstance().getGraphics().getFont().deriveFont(8.0f));

    // adds the scrollPane and the panel
    try {
      beamer = prepareFrameWithoutScrolling(beamer, _pane);
    } catch (DialogError d){
      // more than 20 open frames
      DialogBox.showErrorMessage("The limit of 20 open frames is get.");
      return;
    }
    if (Desktop.getDSize().getHeight() > 550 && Desktop.getDSize().getWidth() > 700)
        beamer.setBounds(
          (int) Desktop.getDSize().getWidth() - 463,
          //(int)model__.getExplorer().getWidth() + 5,
          0,
          463,
          730); // 620
    else beamer.setBounds(
          (int) Desktop.getDSize().getWidth() / 2,
          //(int)model__.getExplorer().getWidth() + 5,
          0,
          (int) Desktop.getDSize().getWidth() / 2,
          (int) Desktop.getDSize().getHeight() / 2);
    try {
      beamer.setFrameIcon(new ImageIcon(
              Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
                        Desktop.URL_SEPAR+"treeView.gif").toURL()
              ) );
    } catch (MalformedURLException m){
      System.out.println("in InternalFrame.showBeamerFrame(), exception"+
        " on view icon: "+m.toString() );
      new ProcessError("in InternalFrame.showBeamerFrame(), exception"+
        " on view icon: "+m.toString());
    }

    // update the beamer variables
    model__.setBeamer(beamer, _pane, true);

    // add a listener for activating actions
    beamer = InternalFrameUtilities.addActivationFrameListener(beamer);
    // ... and for closing actions, through an inner class InternalFrameAdapter
    beamer.addInternalFrameListener(new InternalFrameAdapter() {

      public void internalFrameClosing(InternalFrameEvent _e){
        model__.setBeamer(new JInternalFrame(), null, false);
        BeamerManager.getInstance().propertyChange(
          new PropertyChangeEvent(this, "beamerClosing", null, null));
      }

    }); // new InternalFrameAdapter()

    // set the frame disposition from the GUI configuration
    Map m_GUIconfig = null;
    Rectangle bounds = null;
    try {
        m_GUIconfig = BaseManagerImpl.getInstance().getGUI();
    } catch (BaseError b) {
        throw new InternalError("in InternalFrame.showBeamerFrame(), error: "+b);
    }

    if (m_GUIconfig != null &&
            m_GUIconfig.get(new NameImpl("<Last GUI>")) != null) {
       bounds = (Rectangle)((Map)m_GUIconfig.get(new NameImpl("<Last GUI>"))).
               get( new NameImpl("<Beamer>") );
    }
    if (m_GUIconfig != null && bounds == null &&
            m_GUIconfig.get(new NameImpl("<Previous GUI>")) != null) {
       bounds = (Rectangle)((Map)m_GUIconfig.get(new NameImpl("<Previous GUI>"))).
               get( new NameImpl("<Beamer>") );
    }
    if (bounds != null) beamer.setBounds(bounds);
    m_GUIconfig = null;
    bounds = null;
    
    // put the frame in the desktop
    Desktop.addFrame(beamer, new Integer(1));
    beamer.setVisible(true);

    return;
  } // showBeamerFrame()

  /**
  * Creates and displays a frame including a tabbed pane of all graphical views
  * for the open full views, with a frame listener which fires events to ActionClose.
  * Error, with message to user, if the number of frames is superior to 20.
  * @param _panel of a tabbed pane to display
  * @exception ProcessError if _object is null.
  * @return the JInternalFrame displayed in the desktop. May be empty.
  */
  public static synchronized JInternalFrame showAllFullViewsFrame(JPanel _panel)
          throws ProcessError {
    // variables
    JInternalFrame            result;

    // creates a frame closable, iconifiable.
    result = new JInternalFrame("Full Views ", true, true, true ,true);


                      // for testing and tracking purpose
                    //System.out.println("call of InternalFrame.showAllFullViewsFrame(),"+
                      //" for the panel:"+_panel.getName());   
    
    // adds a scrollPane to the gview
    try {
      result = prepareFrameWithoutScrolling(result, _panel);
    } catch (DialogError d){
        // more than 20 open frames       
        throw new ProcessError("The limit of 20 open frames is get.");
    }
    // set the location at the left side of the explorer 
    result.setBounds(
        model__.getExplorer().getWidth(),
        0,
        590,
        790);

    try {
      result.setFrameIcon(new ImageIcon(
              Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
                        Desktop.URL_SEPAR+"view.gif").toURL()
              ) );
    } catch (MalformedURLException m){
      System.out.println("in InternalFrame.showAllFullViewsFrame(), exception"+
        " on view icon: "+m.toString() );
    }

    // adds frame listener
    // '<AllFullViews>' is the name of the frame for ActionClose and InternalFrameModel    
    result = InternalFrameUtilities.addAllFullViewsFrameListener(result);

    model__.setAllFullViewsFrame(result);   
    
    // puts the frame in the desktop
    Desktop.addFrame(result, new Integer(1));
    result.setVisible(true); 

                      // for testing and tracking purpose
                    //System.out.println("call of InternalFrame.showAllFullViewsFrame(),"+
                      //" for the internal frame: "+result);    
    
    return result;
  } // showAllFullViewsFrame()  

  /**
  * Creates and displays a frame including a tabbed pane of all graphical views
  * for the open NoThan views, with a frame listener which fires events to ActionClose.
  * Error, with message to user, if the number of frames is superior to 20.
  * @param _panel of a tabbed pane to display
  * @exception ProcessError if _object is null.
  * @return the JInternalFrame displayed in the desktop. May be empty.
  */
  public static synchronized JInternalFrame showAllNoThanViewsFrame(JPanel _panel)
          throws ProcessError {
    // variables
    JInternalFrame            result;

    // create a frame closable, iconifiable.
    result = new JInternalFrame("NoMore-NoLess Views ", true, true, true ,true);


                      // for testing and tracking purpose
                    //System.out.println("call of InternalFrame.showAllNoThanViewsFrame(),"+
                      //" for the panel:"+_panel.getName());
    
    // adds a scrollPane to the gview
    try {
      result = prepareFrameWithoutScrolling(result, _panel);
    } catch (DialogError d){
        // more than 20 open frames       
        throw new ProcessError("The limit of 20 open frames is get.");
    }
    // sets the location at the left side of the explorer
    result.setBounds(
        model__.getExplorer().getWidth(),
        0,
        590,
        790);

    try {
      result.setFrameIcon(new ImageIcon(
              Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
                        Desktop.URL_SEPAR+"view.gif").toURL()
              ) );
    } catch (MalformedURLException m){
      System.out.println("in InternalFrame.showAllNoThanViewsFrame(), exception"+
        " on view icon: "+m.toString() );
    }

    // add frame listener
    // '<AllNoThanViews>' is the name of the frame for ActionClose and InternalFrameModel    
    result = InternalFrameUtilities.addAllNoThanViewsFrameListener(result);

    model__.setAllNoThanViewsFrame(result);   
    
    // puts the frame in the desktop
    Desktop.addFrame(result, new Integer(1));
    result.setVisible(true); 

                      // for testing and tracking purpose
                    //System.out.println("end of InternalFrame.showAllNoThanViewsFrame(),"+
                      //" for the internal frame: "+result);
    
    return result;
  } // showAllNoThanViewsFrame()  
  
  /**
  * Creates and displays a frame including an unique view of the ACS architecture.
  * Error, with message to user, if the number of frames is superior to 20.
  * @param _gview to display
  * @param _view is the view associated to the architecture in the base.
  * Its name has only one component.
  * @exception ProcessError if _object is null.
  * @return the JInternalFrame displayed in the desktop. May be empty.
  */
  public static synchronized JInternalFrame showArchitectureFrame(GraphicView _gview,
    ViewInBase _view) throws ProcessError {
    // variables
    JInternalFrame            result;

    // creates a frame closable, iconifiable.
    result = new JInternalFrame("Architecture ", true, true, true ,true);


                      // for testing and tracking purpose
                    //System.out.println("call of InternalFrame.showAllViewsFrame(),"+
                      //" for the panel:"+_panel.getName());   
    

    // adds a scrollPane to the gview
    try {
      result = prepareFrame(result, _gview);
    } catch (DialogError d){
        // more than 20 open frames      
        throw new ProcessError("The limit of 20 open frames is get.");
    }
    // sets the location below the beamer, except if it has been moved down
    result = setFrameDisposition(result);
    // if the view has less than 4 nodes, set the height from 400 to 200
    if (_gview.getEorDataModel().getEorM_BaseObjects().size() < 4)
        result.setBounds(
           (int)result.getLocation().getX(),
           (int)result.getLocation().getY()+200,
           result.getWidth(),
           200);

    try {
      result.setFrameIcon(new ImageIcon(
              Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
                        Desktop.URL_SEPAR+"acs.gif").toURL()
              ) );
    } catch (MalformedURLException m){
      System.out.println("in InternalFrame.showArchitectureFrame(), exception"+
        " on view icon: "+m.toString() );
    } // catch

    // adds frame listener
    result = InternalFrameUtilities.addViewFrameListener(result);

    // adds to m_BaseObjectsInMainFrames__, m_MainFramesOfBaseObjects__,
    // m_JComponentOfBaseObjects__. The key is ActionOpenArchitecture.ARCHI_VIEW_NAME.
    model__.addMainFrameBaseObject(_view, result, _gview);    
    
    // put the frame in the desktop
    Desktop.addFrame(result, new Integer(1));
    result.setVisible(true); 

                      // for testing and tracking purpose
                    //System.out.println("call of InternalFrame.showAllViewsFrame(),"+
                      //" for the internal frame: "+result);    
    
    return result;
  } // showArchitectureFrame()  

  /**
  * Creates and displays the 'IS Structure' frame including a beamer-like panel
  * and the IS trees, with a frame listener which fires events to ActionClose.
  * Error, with message to user, if the number of frames is superior to 20.
  * Called by ActionopenGlobalStructure.openFrame().
  * @param _panel to display, including a tabbed pane and a tree
  * @param _tree of the global structure, set at the bottom of _panel
  * @exception ProcessError if _object is null.
  * @return the JInternalFrame displayed in the desktop. May be empty.
  */
  public static synchronized JInternalFrame showStructureFrame(JPanel _panel, JTree _tree)
          throws ProcessError {
    // variables
    JInternalFrame            result;

    // creates a frame closable, iconifiable.
    result = new JInternalFrame("IS structure ", true, true, true ,true);


                      // for testing and tracking purpose
                    //System.out.println("call of InternalFrame.showStructureFrame(),"+
                      //" for the panel:"+_panel.getName());   
    
    // adds a scrollPane to the gview
    try {
      result = prepareFrameWithoutScrolling(result, _panel);
    } catch (DialogError d){
        // more than 20 open frames       
        throw new ProcessError("The limit of 20 open frames is get.");
    }
    // sets the location at the left side of the explorer
    result.setBounds(
        model__.getExplorer().getWidth(),
        0,
        460,
        790);

    try {
      result.setFrameIcon(new ImageIcon(
              Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
                        Desktop.URL_SEPAR+"treeView.gif").toURL()
              ) );
    } catch (MalformedURLException m){
      System.out.println("in InternalFrame.showStructureFrame(), exception"+
        " on view icon: "+m.toString() );
    }

    // adds frame listener
    // '<Structure>' is the name of the frame for ActionClose and InternalFrameModel
    result = InternalFrameUtilities.addStructureFrameListener(result);

    model__.setStructureFrame(result, _tree);
    
    // puts the frame in the desktop
    Desktop.addFrame(result, new Integer(1));
    result.setVisible(true); 

                      // for testing and tracking purpose
                    //System.out.println("call of InternalFrame.showStructureFrame(),"+
                      //" for the internal frame: "+result);    
    
    return result;
  } // showStructureFrame()

  /**
  * Create and display the panel for a dialog without buttons, but just the ability
  * to close the frame. Called by ActionNewACSyst.
  * Do not check the number of frames into the desktop.
  * @param _message displayed at the top of the frame
  * @param _pane to display, which is updated by the caller of this
  *     method. May be null.
  * @param _width of the frame
  * @param _height of the frame 
  * @return the frame
  */
  public static synchronized JInternalFrame showEmptyFrame(
          String _message, JComponent _pane, int _width, int _height) {
    // variables
    JInternalFrame   frame;
      
    // creates a frame closable, iconifiable
    frame = new JInternalFrame(_message, false, true, false , false);

    frame.getContentPane().setLayout(new BorderLayout());
    if (_pane != null)
        frame.getContentPane().add(_pane, BorderLayout.CENTER);

    if (Desktop.getDSize().getHeight() > 550 && Desktop.getDSize().getWidth() > 700)
        frame.setBounds(
          (int) Desktop.getDSize().getWidth() / 2 - _width / 2,
          (int) Desktop.getDSize().getHeight() / 7,
           _width,
          _height);
    else frame.setBounds(
          (int) Desktop.getDSize().getWidth() / 2,
          (int) Desktop.getDSize().getHeight() / 2,
          (int) Desktop.getDSize().getWidth() / 2,
          (int) Desktop.getDSize().getHeight() / 2);
    
    try {
      frame.setFrameIcon(new ImageIcon(
              Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
                        Desktop.URL_SEPAR+"treeView.gif").toURL()
              ) );
    } catch (MalformedURLException m){
      System.out.println("in InternalFrame.showEmptyFrame(), exception"+
        " on view icon: "+m.toString() );
      new ProcessError("in InternalFrame.showEmptyFrame(), exception"+
        " on view icon: "+m.toString());
    }
    
    frame.setFont(BeamerManager.boldUpFont);
    
    // puts the frame in the desktop
    Desktop.addFrame(frame, new Integer(1));
    frame.setVisible(true);

      //System.out.println("in InternalFrame.showEmptyFrame(), frame"+
        //" at the end: "+frame );    
    
    return frame;
  } // showEmptyFrame()


  /**
  * Displays a frame with an information message.
  * Puts the frame in the desktop, but does not set it visible.
  * Called by ActionOpenAllFull/NoThanViews, ActionOpenGraphicView.
  * @param _frame to display.
  * @param _width of the frame
  * @param _height of the frame
  * @return the frame
  */
  public static synchronized JInternalFrame prepareInformationFrame(
          JInternalFrame _frame, int _width, int _height) {

    _frame.getContentPane().setLayout(new BorderLayout());

    if (Desktop.getDSize().getHeight() > 550 && Desktop.getDSize().getWidth() > 700)
        _frame.setBounds(
          (int) Desktop.getDSize().getWidth() / 2 - _width / 2,
          (int) Desktop.getDSize().getHeight() / 7,
           _width,
          _height);
    else _frame.setBounds(
          (int) Desktop.getDSize().getWidth() / 2,
          (int) Desktop.getDSize().getHeight() / 2,
          (int) Desktop.getDSize().getWidth() / 2,
          (int) Desktop.getDSize().getHeight() / 2);

    try {
      _frame.setFrameIcon(new ImageIcon(
              Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
                        Desktop.URL_SEPAR+"treeView.gif").toURL()
              ) );
    } catch (MalformedURLException m){
      System.out.println("in InternalFrame.prepareInformationFrame(), exception"+
        " on view icon: "+m.toString() );
      new ProcessError("in InternalFrame.prepareInformationFrame(), exception"+
        " on view icon: "+m.toString());
    }

    _frame.setFont(BeamerManager.boldUpFont);

    // puts the frame in the desktop
    Desktop.addFrame(_frame, new Integer(1));

      //System.out.println("in InternalFrame.prepareInformationFrame(), frame"+
        //" at the end: "+frame );

    return _frame;
  } // prepareInformationFrame()

  /**
  * Updates the beamer panel for a base object. Called by openBeamer().
  * No update of m_BaseObjectsInMainFrames__, because it does not include the
  * beamer.
  * @param _pane to display, which is updated by the caller of this method.
  */
  static synchronized void updateBeamerObject(JComponent _pane) {

      // without finalization of old components in the beamer,
      // since it has been done in BeamerManager
      model__.getBeamer().getContentPane().removeAll();
      
      // adds the scrollPane and the panel
      try {
        prepareFrameWithoutScrolling(model__.getBeamer(), _pane);
      } catch (DialogError d){
        DialogBox.showErrorMessage("There are too many open frames. "+
          "You should close one to allow the beamer updating.");
        return;
      }

      model__.setBeamer(model__.getBeamer(), _pane, true);

      // displays the changes in the frame
      model__.getBeamer().updateUI();

      return;
  } // updateBeamerObject()

  /**
  * Opens the Beamer if it is closed, or updates the beamer if it is open.
  * Called by ActionOpenBeamer.
  * @param _pane to display, which is updated by the caller of this method.
  * May be null.
  */
  public static synchronized void openBeamer(JComponent _pane) {

    if (!model__.withBeamer()) {
              // displays the beamer in a frame
      showBeamerFrame(_pane);
    } // if
    else {
      updateBeamerObject(_pane);
    } // else
    return;
  } // openBeamer()

  /**
  * Opens the Explorer if it is closed.
  * Throw an InternalError if there is a null ACS in the base.
  */
  public static synchronized void openExplorer() {
    JTree tree = null;
    if (!model__.withExplorer()) {
      try {
        tree = TreeManager.createExplorer();
      } catch (BaseError b) {
        throw new InternalError(b.toString() );
      }
      // displays the explorer in a frame
      showExplorerFrame(tree);
    } // if
    tree = null;
    return;
  } // openExplorer()

  /**
  * Opens the sketcher if it is closed, or no operation.
  * The frame title contains 'Sketcher'.
  * Called by ActionOpenSketcher. Gets the previous sketcher frame disposition,
  * from the GUI configuration file. Calls ViewModel.removeViewRightsMediator()
  * at the frame closing, to finalize the sketch view mediator.
  * No updating of m_BaseObjectsInMainFrames__, because it does not include the
  * sketcher.
  * Error, with message to user, if the number of frames is superior to 20.
  * @param _pane to display, which is a non-null GraphicSketchView.
  */
  public static synchronized void openSketcher(GraphicSketchView _pane) {

    if (model__.withSketcher()) return;
    // variables
    final GraphicSketchView gview = _pane;
    JInternalFrame   sketcher;
      
    // creates a frame closable, iconifiable
    sketcher = new JInternalFrame("Sketcher (simplest links; no updating)",
            true, true, true ,true);

    sketcher.getContentPane().setLayout(new BorderLayout());
    //beamer.setFont(Desktop.getInstance().getGraphics().getFont().deriveFont(8.0f));

    // adds the scrollPane and the panel
    try {
      sketcher = prepareFrame(sketcher, _pane);
    } catch (DialogError d){
      // more than 20 open frames
      sketcher = null;
      return;
    }
    // sets the default position and the icon
    if (Desktop.getDSize().getHeight() > 550 && Desktop.getDSize().getWidth() > 700)
        sketcher.setBounds(
          (int) Desktop.getDSize().getWidth() - 470,
          (int) Desktop.getDSize().getHeight() / 2,
          470,
          300);
    else sketcher.setBounds(
          (int) Desktop.getDSize().getWidth() / 2,
          (int) Desktop.getDSize().getHeight() / 2,
          (int) Desktop.getDSize().getWidth() / 2,
          (int) Desktop.getDSize().getHeight() / 2);
    try {
      sketcher.setFrameIcon(new ImageIcon(
              Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
                        Desktop.URL_SEPAR+"treeView.gif").toURL()
              ) );
    } catch (MalformedURLException m){
      System.out.println("in InternalFrame.openSketcher(), exception"+
        " on view icon: "+m.toString() );
      new ProcessError("in InternalFrame.openSketcher(), exception"+
        " on view icon: "+m.toString());
    }

    // updates the sketcher variables
    model__.setSketcher(sketcher, _pane, true);

    // adds a listener for activating actions
    sketcher = InternalFrameUtilities.addActivationFrameListener(sketcher);
    // ... and for closing actions, through an inner class InternalFrameAdapter
    sketcher.addInternalFrameListener(new InternalFrameAdapter() {

      public void internalFrameClosing(InternalFrameEvent _e){
        model__.setSketcher(new JInternalFrame(), null, false);

        try {
          ViewModel.removeViewRightsMediator(gview.getEorDataModel());
        }
        catch (SmallError s) {
          System.out.println("Call to InternalFrame inner class on the closing action: "+
                  s.toString());
        }
        BaseManagerImpl.getInstance().closeSketchView(
                (EPRViewInBaseImpl)gview.getEorDataModel() );
        gview.finalizeForGui();
      } // internalFrameClosing()

    }); // new InternalFrameAdapter()

    // sets the frame disposition from the GUI configuration
    Map m_GUIconfig = null;
    Rectangle bounds = null;
    try {
        m_GUIconfig = BaseManagerImpl.getInstance().getGUI();
    } catch (BaseError b) {
        throw new InternalError("in InternalFrame.openSketcher(), error: "+b);
    }
    if (m_GUIconfig != null &&
            m_GUIconfig.get(new NameImpl("<Last GUI>")) != null) {
       bounds = (Rectangle)((Map)m_GUIconfig.get(new NameImpl("<Last GUI>"))).
               get( new NameImpl("<Sketcher>") );
    } 
    if (m_GUIconfig != null && bounds == null &&
            m_GUIconfig.get(new NameImpl("<Previous GUI>")) != null) {
       bounds = (Rectangle)((Map)m_GUIconfig.get(new NameImpl("<Previous GUI>"))).
               get( new NameImpl("<Sketcher>") );
    }
    if (bounds != null) sketcher.setBounds(bounds);
    m_GUIconfig = null;
    bounds = null;
    
    // puts the frame in the desktop
    Desktop.addFrame(sketcher, new Integer(1));
    sketcher.setVisible(true);

    return;
  } // openSketcher() 

  /**
  * Creates and displays a frame including a JTextPane.
  * No updating of m_BaseObjectsInMainFrames__, because it does not include the text frames.
  * Called by showViewFrame() and ActionOpenGraphicView.restoreMainAndSeeWhyFrames()
  * in the event dispatch thread.
  * @param _text to display, possibly with image(s)
  * @param _title of the frame
  * @param _uriIcon URI of the image to use as frame icon
  * @return JInternalFrame displayed in the desktop
  */
  public static JInternalFrame showTextFrame(JTextPane _text,
    String _title, URI _uriIcon) {
    // variables
    JInternalFrame            frame;
    JPanel                    panel ;
    JScrollPane               scrollPane;
    ImageIcon                 image = null;

    // exceptions: refuse frame creation
    // if there is more than 20 frames in the desktop
    JInternalFrame[] frames = Desktop.getCurrentFrames();
    int nb = frames.length;
    if (nb > 20)  {
      DialogBox.showErrorMessage(
          "There are more than 20 open internal windows in the Access Road desktop."+
            " Sorry, the opening of "+_text+" is aborted.");
      throw new InternalError();
    }

    // creates a view frame closable, iconifiable
    frame = new JInternalFrame(_title, true, true, true ,true);

    // put _text in a scrollpane
    scrollPane = new JScrollPane(_text);
    //scrollPane.setForeground(FORECOLOR);
    //scrollPane.setBackground(BACKCOLOR);

    // puts the scrollPane in a panel
    panel = new JPanel(true);
    panel.setLayout(new BorderLayout());
    panel.add("Center", scrollPane);

    frame.addVetoableChangeListener(iconPolice__);
    frame.getContentPane().add(panel);

    frame.setBounds(
      model__.getExplorer().getWidth(),
      0,
      590,
      400);

    // sets a frame icon
    try {
      image = new ImageIcon(_uriIcon.toURL() );
    } catch (MalformedURLException m){
      System.out.println("in InternalFrame.showTextFrame(), exception"+
        " for opening the image icon. Exception: "+m.toString() );
    }

    frame.setFrameIcon(image);

    // adds a listener for activating actions
    frame = InternalFrameUtilities.addActivationFrameListener(frame);

    // puts the frame in the desktop
    Desktop.addFrame(frame, new Integer(1));
    frame.setVisible(true);

                    // for testing and tracking
                    //System.out.println("call to InternalFrame.showTextFrame() "+
                      //      "within the EDT :"+
                        //    SwingUtilities.isEventDispatchThread());
                    //System.out.println("end of InternalFrame.showTextFrame(), frame is visible:"+frame);

    panel = null;
    scrollPane = null;
    image = null;

    return frame;
  } // showTextFrame()
 

  /**
  * Creates and displays a frame including a text in html.
  * No updating of m_BaseObjectsInMainFrames__, because it does not include the
  * text frames.
  * Called by GraphicViewBaseListenerImpl.changeFrameView(), updateFrameViewRights()
  * in the Event Dispatching Thread, since this is NOT a thread-safe method.
  * @param _text to display, possibly with image(s)
  * @param _frame to update
  * @return _frame
  */
  public synchronized static JInternalFrame updateTextFrame(JTextPane _text,
    JInternalFrame _frame) {
    // variables
    JInternalFrame            frame;
    JPanel                    panel ;
    JScrollPane               scrollPane;
    ImageIcon                 image = null;

    // this is NOT a thread-safe method
    _frame.getContentPane().removeAll();

    // puts _text in a scrollpane
    scrollPane = new JScrollPane(_text);

    // puts the scrollPane in a panel
    panel = new JPanel(true);
    panel.setLayout(new BorderLayout());
    panel.add("Center", scrollPane);

    _frame.getContentPane().add(panel);
    
    // this is NOT a thread-safe method
    _frame.updateUI();

    panel = null;
    scrollPane = null;
    image = null;

    return _frame;
  } // updateTextFrame()    
  
  //
  // ----------------------------- private methods ------------------------
  //

  /**
  * Creates a frame including a JComponent at the center, without any scroll pane,
  * and with the standard icon police.
  * Error and message to the user, if the number of frames is greater than 20.
  * @param _frame JInternalFrame to process.
  * @param _component JComponent to display
  * @exception DialogError if there is more than 20 open frames.
  * @return JInternalFrame to display in the desktop
  */
  private static JInternalFrame prepareFrameWithoutScrolling(JInternalFrame _frame,
      JComponent _component) throws DialogError {

    // exceptions: refuses frame creation
    // if there is more than 20 frames in the desktop
    JInternalFrame[] frames = Desktop.getInstance().getCurrentFrames();
    int nb = frames.length;
    if (nb > 20)  {
      DialogBox.getInstance().showErrorMessage(
          "There are more than 20 open frames in the desktop."+
            " Please, close one frame before the opening of a new frame."+
            " Caution: the other operations associated to the current"+
            " frame opening will be performed.");
      throw new DialogError();
    }

    _frame.setFont(BeamerManager.boldUpFont);
    
    _frame.addVetoableChangeListener(iconPolice__);
    _frame.getContentPane().add(_component, BorderLayout.CENTER);

    return _frame;
  } // prepareFrameWithoutScrolling()

  /**
  * Creates a frame including a JComponent at the center, with a scroll pane,
  * and with the standard icon police. Set the font frame.
  * Error and message to the user, if the number of frames is greater than 20.
  * @param _frame JInternalFrame to process.
  * @param _component JComponent to display
  * @exception DialogError if there is more than 20 open frames.
  * @return JInternalFrame to display in the desktop
  */
  private static JInternalFrame prepareFrame(JInternalFrame _frame,
      JComponent _component) throws DialogError {
    // variables
    JPanel                    panel ;
    JScrollPane               scrollPane;
    
    _frame.setFont(BeamerManager.boldUpFont);

    // exceptions: refuse frame creation
    // if there is more than 20 frames in the desktop
    JInternalFrame[] frames = Desktop.getCurrentFrames();
    int nb = frames.length;
    if (nb > 20)  {
      DialogBox.showErrorMessage(
          "There are more than 20 open internal windows in the Access Road desktop."+
            " Sorry, the opening of "+_frame.getTitle()+" is stopped.");
      throw new DialogError();
    }

    // puts the JComponent in a scrollpane
    scrollPane = new JScrollPane();
    scrollPane.setPreferredSize(_component.getPreferredSize()); 
    scrollPane.setMaximumSize(_component.getPreferredSize());
    scrollPane.setMinimumSize(_component.getPreferredSize());    
    scrollPane.getViewport().add(_component);

    // puts the scrollPane in a panel
    panel = new JPanel(true);
    panel.setLayout(new BorderLayout());
    panel.add("Center", scrollPane);

    _frame.addVetoableChangeListener(iconPolice__);
    _frame.getContentPane().add(panel, BorderLayout.CENTER);

    return _frame;
  } // prepareFrame()

  /**
  * Creates and displays an explorer frame including a tree,
  * at the top left of the main window.
  * ActionOpenBeamer and ActionOpenSketcher are listeners of all selection changes.
  * @param _tree to display
  * @return the Explorer displayed in the desktop
  */
  private static JInternalFrame showExplorerFrame(JTree _tree){
    // variables
    JInternalFrame            explorer;
    JPanel                    panel ;
    JScrollPane               scrollPane;

    _tree.setSize(900, 500);
    _tree.getSelectionModel().setSelectionMode(
        TreeSelectionModel.SINGLE_TREE_SELECTION);
    _tree.setFont(BeamerManager.plainFont);

    // puts the tree in a scrollpane
    scrollPane = new JScrollPane();
    //scrollPane.setPreferredSize(new Dimension(500, 500));
    scrollPane.getViewport().add(_tree);

    // puts the scrollPane in a panel
    panel = new JPanel(true);
    panel.setLayout(new BorderLayout());
    panel.add("Center", scrollPane);

    // creates a frame non closable, non iconifiable, and add the panel
    explorer = new JInternalFrame("Explorer", true, false, true ,false);
    explorer.addVetoableChangeListener(iconPolice__);
    explorer.getContentPane().add(panel);
        // moves and resizes from the top-left corner
    explorer.setBounds(0, 0, 380, 600);
    explorer.setFont(BeamerManager.plainFont);
    try {
      explorer.setFrameIcon(new ImageIcon(
              Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"images"+
                        Desktop.URL_SEPAR+"treeView.gif").toURL()
              ) );
    } catch (MalformedURLException m){
      System.out.println(m.toString() );
    }

    // adds a listener for activating actions
    explorer = InternalFrameUtilities.addActivationFrameListener(explorer);
    // inner class for node selection events on the explorer for the beamer
    // and the sketcher
    _tree.addTreeSelectionListener(new TreeSelectionListener() {
      public void valueChanged(TreeSelectionEvent _e){
        ActionOpenBeamer.getInstance().valueChanged(_e);
        ActionOpenSketcher.getInstance().valueChanged(_e);
        return;
      } // valueChanged()
    } ); // _tree.addTreeSelectionListener

    // updates model__
    model__.setExplorer(explorer, _tree, true);

    // sets the frame disposition from the GUI configuration
    Map m_GUIconfig = null;
    Rectangle bounds = null;
    try {
        m_GUIconfig = BaseManagerImpl.getInstance().getGUI();
    } catch (BaseError b) {
        throw new InternalError("in InternalFrame.showExplorerFrame(), error: "+b);
    }

    if (m_GUIconfig != null &&
            m_GUIconfig.get(new NameImpl("<Last GUI>")) != null) {
       bounds = (Rectangle)((Map)m_GUIconfig.get(new NameImpl("<Last GUI>"))).
               get( new NameImpl("<Explorer>") );
    }
    if (m_GUIconfig != null && bounds == null &&
            m_GUIconfig.get(new NameImpl("<Previous GUI>")) != null) {
       bounds = (Rectangle)((Map)m_GUIconfig.get(new NameImpl("<Previous GUI>"))).
               get( new NameImpl("<Explorer>") );
    }
    if (bounds != null) explorer.setBounds(bounds);
    m_GUIconfig = null;
    bounds = null;  
    
    explorer.setFont(BeamerManager.boldUpFont);
    
    // puts the frame in the desktop
    Desktop.addFrame(explorer, new Integer(1));
    explorer.setVisible(true);

    return explorer;
  } // showExplorerFrame()

  /**
  * Sets the frame to the down right corner of the desktop, in a 400x400 square.
  * @param _frame JInternalFrame to handle
  * @return _frame
  */
  private static JInternalFrame setFrameDisposition(JInternalFrame _frame) {

    Dimension d = Desktop.getDSize();
    _frame.setBounds(
      (int) d.getWidth() - 400,
      (int) d.getHeight() - 400,
      400,
      400);

    return _frame;

  /* // to set the frame at the right of the explorer, under the beamer
    if (model__.withBeamer() &&
      model__.getBeamer().getLocation().getY() == 0.0)
        _frame.setBounds(
        model__.getExplorer().getWidth(),
        model__.getBeamer().getHeight(),
        400,
        400);
    else _frame.setBounds(
      model__.getExplorer().getWidth(),
      0,
      400,
      400);
*/
  } // setFrameDisposition()


} // InternalFrame