//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui1;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.beans.*;

/**
 * An action that tiles all internal frames when requested.
 * The previous name of this class is ActionTile.
 * @version 0.7.0
 * @since 0.4.0
 */
public class InternalFrameTiler extends AbstractAction {

  /* the only instance of the class, following singleton pattern */
  protected static InternalFrameTiler instance_ = new InternalFrameTiler();

  private static int                beamerWidth = 490;
  private static int                beamerHeight = 500;
  private static int                iconsHeight = 25;

  /**
  * Single constructor.
  */
  private InternalFrameTiler() {
    super("Tiled Windows");
  }

  /**
   * Only mean to get an instance.
   * @return instance
   */
  public static synchronized InternalFrameTiler getInstance(){
    return instance_;
  } // getInstance()

  /**
   * @param ev ActionEvent
   */
  public void actionPerformed(ActionEvent ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
      
    // How many frames
    JInternalFrame[] allframes = ARoad0.Gui1.Desktop.getCurrentFrames();
    int count = allframes.length;
    if (count == 0) return;

    // Define some initial values for size and location of the main window
    Dimension size = Desktop.getDSize();
    boolean withNoniconifiedBeamer = false;
    JInternalFrame[] newframes;

    // remove Explorer and Beamer frames to allframes
    // and process them
    JInternalFrame explorer = InternalFrameModel.getExplorer();
    JInternalFrame beamer = InternalFrameModel.getBeamer();
    if (InternalFrameModel.withBeamer() &&  !beamer.isClosed() && !beamer.isIcon()) {
      withNoniconifiedBeamer = true;
      // create newframes with allframes items except explorer and beamer
      newframes = new JInternalFrame[allframes.length - 2];
    } else {
      // create newframes with allframes items except explorer
      newframes = new JInternalFrame[allframes.length - 1];
    }

    if (!withNoniconifiedBeamer) {
      // create newframes with allframes items except explorer
      int index = 0;
      for (int i = 0 ; i < allframes.length ; i++){
        if (allframes[i] == explorer ){
          index = 1;
          continue;
        }
        newframes[i - index] = allframes[i];
      } // for i
      // set explorer as the first item in allframes,
      // and the others after it
      allframes[0] = explorer;
      System.arraycopy(newframes, 0, allframes, 1, newframes.length);
    }
    else {
      // create newframes with allframes items without explorer and beamer
      int index = 0;
      for (int i = 0 ; i < allframes.length ; i++){
        if (allframes[i] == explorer || allframes[i] == beamer){
          index++;
          continue;
        }
        newframes[i - index] = allframes[i];
      } // for i
      // do not set explorer and beamer in allframes, only the others
      allframes = newframes;
    } // else

    // Determine the necessary grid size.
    // with beamer, explorer and beamer are tiled differently
    // with a width equal to beamerWidth
    if (withNoniconifiedBeamer) {

      if (size.getHeight() > beamerHeight + iconsHeight) {
        Desktop.resizeIFrame(explorer,
          0,
          0,
          beamerWidth,
          (int) size.getHeight() - beamerHeight - iconsHeight);
        Desktop.resizeIFrame(beamer,
          0,
          (int) size.getHeight() - beamerHeight - iconsHeight,
          beamerWidth,
          beamerHeight);
      } else {
        Desktop.resizeIFrame(explorer,
          0,
          0,
          beamerWidth,
          (int) size.getHeight() - iconsHeight);
        Desktop.resizeIFrame(beamer,
          0,
          (int) size.getHeight() - iconsHeight,
          beamerWidth,
          beamerHeight);
      }

      // define the new parameters for tiling the other frames
      count = count - 2;
      size = new Dimension(
        ((int)size.getWidth()) - beamerWidth, ((int)size.getHeight()) - iconsHeight);
    }

    if (count == 0) return;

    int sqrt = (int)Math.sqrt(count);
    int rows = sqrt;
    int cols = sqrt;
    // size up rows before cols
    if (rows*cols < count) {
      rows++;
      if (rows*cols < count) {
      cols++;
      } // if
    } // if

    int w = size.width/cols;
    int h = size.height/rows;
    int x;
    if (!withNoniconifiedBeamer) x = 0;
    else x = beamerWidth;
    int y = 0;
    JInternalFrame f;

    // Iterate over the frames,
    // deiconifying any iconified frames,
    // then relocating and resizing each
    for (int i=0; i<rows ; i++) {
      for (int j=0; j<cols && ((i*cols)+j<count); j++) {

        f = allframes[(i*cols)+j];

        if (!f.isClosed() && f.isIcon() ) {
          try {
            f.setIcon(false);
          } // try
          catch (PropertyVetoException ex) {
            throw new InternalError("from ActionTile");
          } // catch
        } // if (!f.isClosed()

        Desktop.resizeIFrame(f, x, y, w, h);
        x += w;
      } // for j
      // start the next col
      y += h;
      if (!withNoniconifiedBeamer) x = 0;
      else x = beamerWidth;
    } // for i

  } // actionPerformed(ActionEvent)

} // class InternalFrameTiler
