//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui1;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.beans.PropertyVetoException;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import ARoad0.Pattern.*;
import ARoad0.Gui2.*;
import ARoad0.gDMak.*;
import ARoad0.ErrW.*;
import ARoad0.ErrB.BaseError;
import ARoad0.gBaseInterface.Name;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBase.NameImpl;
import ARoad0.gWork.ViewInBase_Facade;

/**
 * This class is responsible to deliver utilities to internal frames
 * in the desktop. It detects the type of the frame, and adds the frame
 * listeners. It is used by InternalFrameView, InternalFrameModel.
 * @version 0.7.0
 */
public class InternalFrameUtilities implements StaticClass {

  /**
   * the only instance of this class, following the singleton pattern
   */
  private static InternalFrameUtilities   instance__ = new InternalFrameUtilities();
  /**
   * the only instance, following the MVC pattern, which manages
   * the internal frames data
   */
  private static InternalFrameModel       model__ = InternalFrameModel.getInstance();

  // variables for the frames
  private static final int                INITIAL_CAPACITY = 10;//the default value:101

  // empty JPanel for other classes
  private static final JPanel             EMPTY_PANEL = new JPanel();

  /**
  * only one private constructor
  */
  private InternalFrameUtilities() {
  }

  /**
  * Only method to get an instance.
  * @return InternalFrameUtilities
  */
/*  public static synchronized InternalFrameUtilities getInstance(){
    return instance__;
  } // getInstance()
*/

  /**
  * Extract the name from the title of an ACS main frame.
  * @param _title String of the internal frame
  * @exception DialogError if the selected frame has not the standard
  *   format of ACS frame titles
  * @return Name of the ACS displayed in the frame
  */
  public static Name extractACSName(String _title) throws DialogError {
    // variables
    Name name;
    int index = _title.indexOf(":: ");

    if (index == -1 || _title.indexOf("ACS ") == -1) {
      // display a warning message if it is not the title of an ACS
      //DialogBox.getInstance().showErrorMessage(
      //  "The selected frame doesn't display an ACS.");

        // for testing and tracking
        //System.out.println("in InternalFrameUtilities.extractACSName(),"+
        //  " it is not the title of an ACS:"+ _title);

      throw new DialogError("InternalFrameUtilities.extractACSName(): "+
              "no ':: ' or no 'ACS' in the frame title");
    }
    // extract the components, as strings
        // _title starts with "ACS  "...
    try {
        name = NameImpl.getNameFromString(_title.substring(5));
    }
    catch (BaseError b) {
        throw new DialogError("InternalFrameUtilities.extractACSName(): "+b.toString());
    }
    
    return name;
  } // extractACSName()

  /**
   * Gets the main frame title for an EPRViewInBase or a NoThanViewInBase.
   * For a NoThanViewInBase where the name has 3 components, the title contains
   * the component 'ViewInBase_Facade.NOTHAN_VIEW_INDICATOR ' just after the
   * value ViewInBase_Facade.VIEW_FRAME_INDICATOR.
   * Called by InternalFrame.showViewFrame().
   * @param _name of a ViewInBase
   * @return the title. Null if _name is null or has only one component.
   */
  public static String getViewFrameTitleFromViewName(ImmutableName _name) {
       if (_name == null)
           return null;
       else if (_name.noOfComponents() == 2) {
           return _name.lastComponent() +
            ViewInBase_Facade.VIEW_FRAME_INDICATOR +
            _name.component(0);
       }
       else if (_name.noOfComponents() == 3) {
           return _name.lastComponent() +
            ViewInBase_Facade.VIEW_FRAME_INDICATOR +
            ViewInBase_Facade.NOTHAN_VIEW_INDICATOR+" "+
            _name.component(0);
       }
       else return null;
  } // getViewFrameTitleFromViewName()

  /**
  * Extracts the name from the title of a view main frame. Detects the presence
  * of the NoThan view indicator. Works also for the Architecture view.
  * This is the inverse method for getViewFrameTitleFromViewName().
  * @param _title of the internal frame
  * @exception DialogError if the selected frame has not the standard
  *   format of view frame titles
  * @return Name of the view displayed in the frame
  */
  public static Name extractViewNameFromFrameTitle(String _title) throws DialogError {
    // variables
    String groupString;
    String viewString;
    Name name = null;

    if (_title.equals("Architecture ")) return ActionOpenArchitecture.ARCHI_VIEW_NAME;

    int index = _title.indexOf(ViewInBase_Facade.VIEW_FRAME_INDICATOR );

    if (index == -1) {
      // display a warning message if it is not the title of a view
      //DialogBox.getInstance().showErrorMessage(
      //  "The selected frame doesn't display a view.");

      throw new DialogError("The selected view frame has not the right format");
    }
    
    groupString = _title.substring(
            index+ViewInBase_Facade.VIEW_FRAME_INDICATOR.length(), 
            _title.length());

    viewString = _title.substring(0, index);

            //for testing and tracking
            //System.out.println("in InternalFrameUtilities.extractViewNameFromFrameTitle(), temporary view name is: "+
              //groupString+":: "+ viewString+":: ");

    
    
    if (groupString.startsWith( ViewInBase_Facade.NOTHAN_VIEW_INDICATOR+" " )) {
        name = new NameImpl(groupString.substring(
                ViewInBase_Facade.NOTHAN_VIEW_INDICATOR.length() + 1));
        name.extended(ViewInBase_Facade.NOTHAN_VIEW_INDICATOR);
        name.extended(viewString);
    }
    else {
        name = new NameImpl(groupString);
        name.extended(viewString);
    }

    groupString = null;
    viewString = null;
    return name;
  } // extractViewNameFromFrameTitle()

  /**
  * Extracts the name from the title of a view 'See why' frame, and detect the presence
  * of the NoThan view indicator. Does not process the Architecture view.
  * @param _title String of the internal frame
  * @exception DialogError if the selected frame has not the standard
  *   format of view 'See why' frame titles
  * @return Name of the view displayed in the frame
  */
  public static Name extractViewNameFromSeeWhy(String _title) throws DialogError {
    Name result= null;
    try {
        result = NameImpl.getNameFromString(_title.substring(0, _title.length() - 15));
    } catch (BaseError b) {
        throw new DialogError("InternalFrameUtilities.extractViewNameFromSeeWhy(): "+
                b.toString());
    }
    return result;
  } // extractViewNameFromSeeWhy()
  

  /**
  * Extract the name in the beamer title.
  * @param _beamerTitle of the internal frame
  * @exception DialogError if the selected frame has not the standard
  *   format of the beamer
  * @return Name of the beamer: title
  */
  public static Name extractBeamerName(String _beamerTitle)
    throws DialogError {
    // variables
    String groupString = _beamerTitle.trim();
    int length = groupString.length();
    int index = groupString.lastIndexOf(" ");

    if (index == -1 || groupString.indexOf("Beamer") == -1) {
      // display a warning message if it is not the title of a view
      //DialogBox.getInstance().showErrorMessage(
      //  "The selected frame doesn't display a view.");

      // return an empty name
      throw new DialogError();
    }
    // extract the last component, as strings
    groupString = groupString.substring(index, length);

    //for testing and tracking
    //System.out.println("in InternalFrameUtilities.extractBeamerName(), "+
     //"title is: "+_beamerTitle+", name is: "+
      //groupString+", with index: "+index);

    return new NameImpl(groupString);
  } // extractBeamerName()

  /**
  * Test if a frame is a ACS frame, registered or not. Test the title format.
  * @param _frame JInternalFrame to process
  * @return boolean true if it is an ACS frame
  */
  public static boolean isACSFrame(JInternalFrame _frame) {
      if (_frame == null || _frame.getTitle() == null) return false;
      if (_frame.getTitle().indexOf(" :: ") == -1 ||
        _frame.getTitle().indexOf("ACS ") != 0) return false;
      else return true;
  } // isACSFrame()

  /**
  * Test if a frame is a view frame, registered or not.
  * Test 'in the View group' the title format.
  * Used by ActionUtilities and ActionOpenAllViews
  * @param _frame to process
  * @return boolean true if it is an view frame
  */
  public static boolean isViewFrame(JInternalFrame _frame) {
      if (_frame == null || _frame.getTitle() == null) return false;
      if (_frame.getTitle().indexOf(ViewInBase_Facade.VIEW_FRAME_INDICATOR) == -1 ) 
          return false;
      else return true;
  } // isViewFrame()

  /**
  * Test if a frame is the 'Full views' frame.
  * Test the title format.
  * Used by ActionUtilities and ActionOpenAllViews
  * @param _frame to process
  * @return boolean true if it is the AllViews frame
  */
  public static boolean isAllFullViewsFrame(JInternalFrame _frame) {
      if (_frame == null || _frame.getTitle() == null) return false;
      if (_frame.getTitle().indexOf("Full Views") == -1 ) return false;
      else return true;
  } // isAllFullViewsFrame()

  /**
  * Test if a frame is the 'NoMore-NoLess views' frame.
  * Test the title format.
  * Used by ActionUtilities and ActionOpenAllViews
  * @param _frame to process
  * @return boolean true if it is the AllViews frame
  */
  public static boolean isAllNoThanViewsFrame(JInternalFrame _frame) {
      if (_frame == null || _frame.getTitle() == null) return false;
      if (_frame.getTitle().indexOf("NoMore-NoLess Views") == -1 ) return false;
      else return true;
  } // isAllNoThanViewsFrame()
  
  /**
  * Test if a frame is the beamer frame.
  * Test the title format.
  * @param _frame to process
  * @return boolean true if it is the beamer frame
  */
  public static boolean isBeamerFrame(JInternalFrame _frame) {
      if (_frame == null || _frame.getTitle() == null) return false;
      if (_frame.getTitle().indexOf("Beamer") == -1) return false;
      else return true;
  } // isBeamerFrame()
  
  /**
  * Test if a frame is the sketcher frame.
  * Test the title format.
  * @param _frame to process
  * @return boolean true if it is the sketcher frame
  */
  public static boolean isSketcherFrame(JInternalFrame _frame) {
      if (_frame == null || _frame.getTitle() == null) return false;
      if (_frame.getTitle().indexOf("Sketcher") == -1) return false;
      else return true;
  } // isBeamerFrame()
  
  /**
  * Test if a frame is the IS structure frame.
  * Test the title format.
  * @param _frame to process
  * @return boolean true if it is the IS structure frame
  */
  public static boolean isISstructureFrame(JInternalFrame _frame) {
      if (_frame == null || _frame.getTitle() == null) return false;
      if (_frame.getTitle().indexOf("IS Structure") == -1) return false;
      else return true;
  } // isStructureFrame()
  
  /**
  * Allow to avoid the use of 'new JPanel()' for the initialization of a variable.
  * @return an empty panel for initialization
  */
  public static JPanel getEmptyPanel() {
      return EMPTY_PANEL;
  } // getEmptyPanel()


  /**
  * Add a frame listener to the frame when it is activated and de-activated.
  * Used for explorer, beamer, sketcher and See why frames.
  * An inner class InternalFrameAdapter is used.
  * @param _frame JInternalFrame to handle
  * @return JInternalFrame with a listener
  */
  static JInternalFrame addActivationFrameListener(
    JInternalFrame _frame) {
    // variables
      // _frame has to be final to be used in an inner class
    final JInternalFrame frame = _frame;

    // inner class InternalFrameAdapter for activating
    frame.addInternalFrameListener(new InternalFrameAdapter() {

      public void internalFrameActivated(InternalFrameEvent _e){
        model__.setActiveFrame(_e.getInternalFrame() );

                  // for testing and tracking
                  //System.out.println("in InternalFrameUtilities.addActivationFrameListener()."+
                    //"new active frame: "+_e.getInternalFrame().getTitle());
        
      } // internalFrameActivated()

      public void internalFrameDeactivated(InternalFrameEvent _e){
        model__.setActiveFrame(null);
      } // internalFrameDeactivated()

    } ); // frame.addInternalFrameListener()

    return frame;
  } // addActivationFrameListener()

  /**
  * Adds a frame listener to an ACS main internal frame when it is closing
  * and when it is activated. An inner class InternalFrameAdapter is used.
  * If ActionClose has been already called, the closing listener cleans up
  * the frame.
  * The closing listener updates the variables m_BaseObjectsInMainFrames__,
  * m_MainFramesOfBaseObjects__, m_JComponentsOfBaseObjects__ in InternalFrameModel.
  * The activated listener updates the variable activeFrame__.
  * @param _frame JInternalFrame to handle
  * @return JInternalFrame with a closing listener
  */
  static JInternalFrame addACSFrameListener(
    JInternalFrame _frame) {
    // variables
      // _frame has to be final to be used in an inner class
    final JInternalFrame frame = _frame;

    // inner class InternalFrameAdapter for closing and for activating
    frame.addInternalFrameListener(new InternalFrameAdapter() {

      public void internalFrameActivated(InternalFrameEvent _e){
        model__.setActiveFrame(_e.getInternalFrame());
      } // internalFrameActivated()

      public void internalFrameDeactivated(InternalFrameEvent _e){
        model__.setActiveFrame(null);
      } // internalFrameDeactivated()

      public void internalFrameClosing(InternalFrameEvent _e){
          // variables
          DefaultTreeModel        model;
          JScrollPane             scrollPane;
          DefaultMutableTreeNode  root;
          String                  title;
          Name                    name;
          DefaultMutableTreeNode[] nodeArray;

              // for testing and tracking
              //System.out.println("in InternalFrameUtilities.addACSFrameListener()."+
              //  "InternalFrameAdapter,"+
              //  " call to internalFrameClosing().");

          // extract the name only if it is an ACS
          title = _e.getInternalFrame().getTitle();
          try {
            name = extractACSName(title);
          } catch (DialogError d) {
            // it is not an ACS

              // for testing and tracking
              System.out.println("in InternalFrameUtilities.internalFrameClosing(), "+
                      "from extractACSName() for "+title+",exception "+d);
              
            return;
          } // catch

          // removes the ACSTreeBaseListener associated to the tree
          // in the main frame
          // (not in the explorer, which is removed by ActionClose.close())
          try {

            TreeManager.removeACSMainTreeBaseListener(name);

          } catch (SmallError s) {
            System.out.println("in InternalFrameUtilities.addACSFrameListener()"+
              ".InternalFrameAdapter, TreeManager.getInstance()."+
              "removeACSMainTreeBaseListener() fires: "+
              s.toString() );
          } // try

          // removes the ACS name
          model__.removeMainFrameBaseObject(name);

          // fires an event to ActionClose if it is not implied,
          // to request to the user if he wants a closing.
              // (If ActionClose is implied, the frame closing has been
              // done by ActionCloseUtilities.closeBaseObjectMainFrame()
              // on a request from ActionClose)
          if (!model__.getOnGoingClosingFromActionClass().contains(name)) {

              // for testing and tracking
              //System.out.println("in InternalFrameUtilities.addACSFrameListener()."+
              //  "InternalFrameAdapter, fire ActionEvent to ActionClose.");

            // no more ACS closing when the ACS frame is closed
            //ActionClose.getInstance().actionPerformed(new ActionEvent(
            //  frame,0,title  ));

          } // else, clears the list
          else model__.removeOnGoingClosingFromActionClass(name);

          // clears the tree to avoid to let a big unused non-viewable tree
          // connected to objects base
          scrollPane = (JScrollPane)((JPanel)_e.getInternalFrame().getContentPane().
            getComponent(0)).getComponent(0);
          model = (DefaultTreeModel)((JTree)((JViewport)scrollPane.
            getComponent(0) ).getView()).getModel();
          root = (DefaultMutableTreeNode) model.getRoot();
            // copies the nodes under root
          nodeArray = new DefaultMutableTreeNode[root.getChildCount()];
          for (int i = 0 ; i < root.getChildCount() ; i++) {
            nodeArray[i] = (DefaultMutableTreeNode) model.getChild(root, i);
          }
            // removes the nodes
          for (int i = 0 ; i < nodeArray.length ; i++) {
            model.removeNodeFromParent(nodeArray[i]);
          }

          title = null;
          name = null;
          nodeArray = null;
          scrollPane = null;
          model = null;
          root = null;
      } // internalFrameClosing()

    } ); // frame.addInternalFrameListener()

    return frame;
  } // addACSFrameListener()

  /**
  * Adds a frame listener to a view or architecture main internal frame when it is closing
  * and when it is activated. An inner class InternalFrameAdapter is used.
  * The activation listener updates the variable activeFrame__.
  * The closing listener updates the variables m_BaseObjectsInMainFrames__,
  * m_MainFramesOfBaseObjects__, m_JComponentsOfBaseObjects__ in InternalFrameModel,
  * and fires an event to ActionClose if it is not already implied.
  * @param _frame JInternalFrame to handle
  * @return JInternalFrame with a closing listener
  */
  static JInternalFrame addViewFrameListener(JInternalFrame _frame) {
    // variables
      // _frame has to be final to be used in an inner class
    final JInternalFrame frame = _frame;

    // inner class InternalFrameAdapter for closing and for activating
    frame.addInternalFrameListener(new InternalFrameAdapter() {

      public void internalFrameActivated(InternalFrameEvent _e){
        model__.setActiveFrame(_e.getInternalFrame() );
      } // internalFrameActivated()

      public void internalFrameDeactivated(InternalFrameEvent _e){
        model__.setActiveFrame(null);
      } // internalFrameDeactivated()

      public void internalFrameClosing(InternalFrameEvent _e){
          // variables
          String                  title;
          Name                    name;
          JInternalFrame          seeWhyFrame;

              // for testing and tracking
              //System.out.println("in InternalFrameUtilities.addViewFrameListener().InternalFrameAdapter,"+
              //  " call to internalFrameClosing().");

          // extract the name only if it is a view
          title = _e.getInternalFrame().getTitle();

          // a wrong frame opened after the limit of 20 frames in the desktop
          // may have a null title
          if (title == null)
              return;

          // if it is the architecture frame
          if (title.equals("Architecture ")) {
              name = ActionOpenArchitecture.ARCHI_VIEW_NAME;
          } else {
              // else, it is a view frame
              try {
                name = extractViewNameFromFrameTitle(title);
              } catch (DialogError d) {
                // it is not a view
                throw new InternalError(
                  "error in InternalFrameUtilities.addViewFrameListener()");
              } // catch
          }

                      // for testing and tracking
                      //System.out.println("in InternalFrameUtilities.addViewFrameListener(): "+
                        //"title: "+title+", name: "+name);

          // fire an event to ActionClose if it is not implied,
          // to request to the user if he wants a ViewInBase closing.
              // If ActionClose is implied, closeMainFrameInGUI() has called
              // InternalFrameModel.addOnGoingClosingFromActionClass()
          if (!model__.getOnGoingClosingFromActionClass().contains(name)) {

                      // for testing and tracking
                      //System.out.println("in InternalFrameUtilities.addViewFrameListener()."+
                        //"InternalFrameAdapter, fire ActionEvent to ActionClose for "+
                        //"title: "+title+", name: "+name);
              
              // request to the user for closing only the view frame or the view
            ActionClose.getInstance().actionPerformed(new ActionEvent(
              frame,0,title  ));
            name = null;
            return;
          } // else, clear the list
          else model__.removeOnGoingClosingFromActionClass(name);
          
          // remove the view main frame
          model__.removeMainFrameBaseObject(name);

          // remove the 'See why' frame if it is open
          for (int i = 0; i < Desktop.getCurrentFrames().length; i++) {
            seeWhyFrame = Desktop.getCurrentFrames()[i];

                      // for testing and tracking
                      //if (seeWhyFrame.getTitle() == null)
                        //  System.out.println("in InternalFrameUtilities.addViewFrameListener(): "+
                          //  "null-title seeWhyFrame: "+seeWhyFrame);

            if (seeWhyFrame.getTitle() != null &&
                 seeWhyFrame.getTitle().equals(name.toString()+ " 'See why' text") ) {
                  try {
                    seeWhyFrame.setClosed(true);
                  } catch (PropertyVetoException p) {
                    System.out.println("in InternalFrameUtilities.addViewFrameListener(),"+
                      " a 'See why' view frame closing fires "+
                      "a PropertyVetoException: "+p.toString());
                  }
                  break;
            } // if
          } // for i

          title = null;
          name = null;
          seeWhyFrame = null;
      } // internalFrameClosing()

    } ); // frame.addInternalFrameListener()

    return frame;
  } // addViewFrameListener()

  /**
  * Add a frame listener to a full view main internal frame when it is closing
  * and when it is activated. An inner class InternalFrameAdapter is used.
  * The activation listener updates the variable activeFrame__.
  * The closing listener updates m_JComponentsOfBaseObjects__ in InternalFrameModel,
  * and calls ActionClose.closeAllFullViewsFrame() if it is not still implied.
  * @param _frame JInternalFrame to handle
  * @return JInternalFrame with a closing listener
  */
  static JInternalFrame addAllFullViewsFrameListener(
    JInternalFrame _frame) {
    // variables
      // _frame has to be final to be used in an inner class
    final JInternalFrame frame = _frame;

    // inner class InternalFrameAdapter for closing and for activating
    frame.addInternalFrameListener(new InternalFrameAdapter() {

      public void internalFrameActivated(InternalFrameEvent _e){
        model__.setActiveFrame(_e.getInternalFrame() );
      } // internalFrameActivated()

      public void internalFrameDeactivated(InternalFrameEvent _e){
        model__.setActiveFrame(null);
      } // internalFrameDeactivated()

      public void internalFrameClosing(InternalFrameEvent _e){
          // variables
          Name name = new NameImpl("<AllFullViews>");

              // for testing and tracking
              //System.out.println("in InternalFrameUtilities.
              //    addAllFullViewsFrameListener().InternalFrameAdapter,"+
              //  " call to internalFrameClosing().");
          
          // fire an event to ActionClose if it is not implied,
          // to manage the views closing.
              // If ActionClose is implied, the frame closing has been
              // done by ActionClose and model__ has been updated
          if (!model__.getOnGoingClosingFromActionClass().contains(name)) {

                      // for testing and tracking
                      //System.out.println("in InternalFrameUtilities.addAllFullViewFrameListener()."+
                        //"InternalFrameAdapter, call to ActionClose.closeAllFullViewsFrame().");

            ActionClose.getInstance().closeAllFullViewsFrame();
            return;
          } // else, clear the list
          else model__.removeOnGoingClosingFromActionClass(name);

          model__.setAllFullViewsFrame(null);
          name = null;
      } // internalFrameClosing()

    } ); // frame.addInternalFrameListener()

    return frame;
  } // addAllFullViewsFrameListener()


  /**
  * Adds a frame listener to a full view main internal frame when it is closing
  * and when it is activated. An inner class InternalFrameAdapter is used.
  * The activation listener updates the variable activeFrame__.
  * The closing listener updates m_JComponentsOfBaseObjects__ in InternalFrameModel,
  * and calls ActionClose.closeStructureFrame() if it is not still implied.
  * '<Structure>' is the name of the frame in the internal frame model.
  * @param _frame JInternalFrame to handle
  * @return JInternalFrame with a closing listener
  */
  static JInternalFrame addStructureFrameListener(
    JInternalFrame _frame) {
    // variables
      // _frame has to be final to be used in an inner class
    final JInternalFrame frame = _frame;

    // inner class InternalFrameAdapter for closing and for activating
    frame.addInternalFrameListener(new InternalFrameAdapter() {

      public void internalFrameActivated(InternalFrameEvent _e){
        model__.setActiveFrame(_e.getInternalFrame() );
      } // internalFrameActivated()

      public void internalFrameDeactivated(InternalFrameEvent _e){
        model__.setActiveFrame(null);
      } // internalFrameDeactivated()

      public void internalFrameClosing(InternalFrameEvent _e){
          // variables
          Name name = new NameImpl("<Structure>");

              // for testing and tracking
              //System.out.println("in InternalFrameUtilities.
              //    addStructureListener().InternalFrameAdapter,"+
              //  " call to internalFrameClosing().");

          // fires an event to ActionClose if it is not implied,
          // to manage the closing.
              // If ActionClose is implied, the frame closing has been
              // done by ActionClose and model__ has been updated
          if (!model__.getOnGoingClosingFromActionClass().contains(name)) {

                      // for testing and tracking
                      //System.out.println("in InternalFrameUtilities.addStructureListener()."+
                        //"InternalFrameAdapter, fire ActionEvent to ActionClose.");

            ActionClose.getInstance().closeStructureFrame();
            return;
          } // else, clear the list
          else model__.removeOnGoingClosingFromActionClass(name);

          model__.setStructureFrame(null, new JTree());
          name = null;
      } // internalFrameClosing()

    } ); // frame.addInternalFrameListener()

    return frame;
  } // addStructureFrameListener()

  /**
  * Add a frame listener to a NoThan view main internal frame when it is closing
  * and when it is activated. An inner class InternalFrameAdapter is used.
  * The activation listener updates the variable activeFrame__.
  * The closing listener updates m_JComponentsOfBaseObjects__ in InternalFrameModel,
  * and calls ActionClose.closeAllViewsFrame() if it is not still implied.
  * @param _frame JInternalFrame to handle
  * @return JInternalFrame with a closing listener
  */
  static JInternalFrame addAllNoThanViewsFrameListener(
    JInternalFrame _frame) {
    // variables
      // _frame has to be final to be used in an inner class
    final JInternalFrame frame = _frame;

    // inner class InternalFrameAdapter for closing and for activating
    frame.addInternalFrameListener(new InternalFrameAdapter() {

      public void internalFrameActivated(InternalFrameEvent _e){
        model__.setActiveFrame(_e.getInternalFrame() );
      } // internalFrameActivated()

      public void internalFrameDeactivated(InternalFrameEvent _e){
        model__.setActiveFrame(null);
      } // internalFrameDeactivated()

      public void internalFrameClosing(InternalFrameEvent _e){
          // variables
          Name name = new NameImpl("<AllNoThanViews>");

              // for testing and tracking
              //System.out.println("in InternalFrameUtilities.
              //    addAllNoThanViewsFrameListener().InternalFrameAdapter,"+
              //  " call to internalFrameClosing().");
          
          // fire an event to ActionClose if it is not implied,
          // to manage the views closing.
              // If ActionClose is implied, the frame closing has been
              // done by ActionClose and model__ has been updated
          if (!model__.getOnGoingClosingFromActionClass().contains(name)) {

                      // for testing and tracking
                      //System.out.println("in InternalFrameUtilities.addAllNoThanViewFrameListener()."+
                        //"InternalFrameAdapter, fire ActionEvent to ActionClose.");

            ActionClose.getInstance().closeAllNoThanViewsFrame();
            return;
          } // else, clear the list
          else model__.removeOnGoingClosingFromActionClass(name);

          model__.setAllNoThanViewsFrame(null);
          name = null;
      } // internalFrameClosing()

    } ); // frame.addInternalFrameListener()

    return frame;
  } // addAllNoThanViewsFrameListener()

  /**
  * Puts in the argument _GUIconfig the current location of a frame in the GUI,
  * as a Rectangle.
  * Does not save the nodes location in the views.
  * Called by ActionSave and ActionSaveAll.
  * @param _frame is the frame for which the disposition in the GUI is to save.
  *  May be null. 
  * @param _m_GUIconfig is the map of the objects in the GUI to save; object name
  *   is the key. Never null.
  * @exception ProcessError if the _frame is not null and not found
  * @return an updated _m_GUIconfig in its map put for the key 'new NameImpl("<Last GUI>")'
  */
  static public Map saveFrameDispositionInGUI(JInternalFrame _frame, Map _m_GUIconfig)
          throws ProcessError {

      if (_frame == null) return _m_GUIconfig;

      Map   m_Config = (Map) _m_GUIconfig.get(new NameImpl("<Last GUI>"));
      if (m_Config == null) {
          m_Config = new HashMap(10);
          _m_GUIconfig.put(new NameImpl("<Last GUI>"), m_Config);
      }
      boolean found = false;

      if (_frame == InternalFrameModel.getExplorer()) {
          m_Config.put(new NameImpl("<Explorer>"), _frame.getBounds()); // the value type is Rectangle
          found = true;
      } else if (_frame == InternalFrameModel.getBeamer()) {
          m_Config.put(new NameImpl("<Beamer>"), _frame.getBounds());
          found = true;
      } else if (_frame == InternalFrameModel.getAllFullViewsFrame()) {

                      // for testing and tracking
                      //System.out.println("in InternalFrameUtilities.saveFrameDispositionInGUI()."+
                        //" detection of <AllFullViews>.");
          
          m_Config.put(new NameImpl("<AllFullViews>"), _frame.getBounds());
          found = true;
      } else if (_frame == InternalFrameModel.getAllNoThanViewsFrame()) {

                      // for testing and tracking
                      //System.out.println("in InternalFrameUtilities.saveFrameDispositionInGUI()."+
                        //" detection of <AllNoThanViews>.");
                      
          m_Config.put(new NameImpl("<AllNoThanViews>"), _frame.getBounds());
          found = true;
      } else if (_frame == InternalFrameModel.getSketcher()) {
          m_Config.put(new NameImpl("<Sketcher>"), _frame.getBounds());
          found = true;
      } else if (_frame == InternalFrameModel.getStructureFrame()) {
          m_Config.put(ActionOpenGlobalStructure.LOCATION_STRUCTURE_NAME, _frame.getBounds());
          found = true;
      } else if (_frame.getTitle() != null &&
              _frame.getTitle().indexOf("'See why' text") != -1) {
          
          Name name = null;
          try {
              name = extractViewNameFromSeeWhy( _frame.getTitle());
          } catch (DialogError b) {
              throw new InternalError("in InternalFrameUtilities.saveFrameDispositionInGUI(), "+b);
          } // catch
          
          if (ViewInBase_Facade.isOpenView(name)) {
              m_Config.put(
                      (new NameImpl("<See why for>")).extended(name) , 
                      _frame.getBounds());
              found = true;              
          } // if
          name = null;
              
      } else { // include the 'Architecture' frame
          for (Iterator ite = InternalFrameModel.getMainFramesOfBaseObjects().
             entrySet().iterator(); ite.hasNext();) {
                Map.Entry entry = (Map.Entry) ite.next();

                      // for testing and tracking
                      //System.out.println("in InternalFrameUtilities.saveFrameDispositionInGUI() "+
                        //"InternalFrameModel.getMainFramesOfBaseObjects() entry "+entry.getKey());
                
                if (_frame == entry.getValue()) {
                    m_Config.put(entry.getKey(), _frame.getBounds()); // the value type is Rectangle
                    found = true;
                    break;
                } // if
          } // for
      } // else

      m_Config = null;

      if (_frame.getTitle() == null)
          return _m_GUIconfig;

      if (!found) throw new ProcessError("in InternalFrameUtilities.saveFrameDispositionInGUI(), "+
              "the frame "+_frame.getTitle()+" is not found.");

      return _m_GUIconfig;
  } // saveFrameDispositionInGUI()  
  
} // InternalFrameUtilities