//Title:        Access Road
//Version:      0.7.0
//Copyright:    2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui1;

import java.beans.*;
import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.border.LineBorder;
import java.lang.reflect.InvocationTargetException;
import java.awt.Color;
import java.util.Map;
import java.util.Iterator;
import java.util.List;

import ARoad0.ErrW.ProcessError;
import ARoad0.ErrW.SmallError;
import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBaseInterface.StringRight;
import ARoad0.gBase.ACSFactoryUtilityImpl;

/**
 * This class is responsible for displaying, by the mean of panels,
 * a modifiable property map of a gBase JavaBean, 
 * for which the type is a Map<String, StringRight[]>. <P>
 * The map updating, if any, is never done through a write method or a 'choiceMethodForNew'
 * in the property descriptor, but always through 'add/remove' methods defined in the property
 * descriptor.<P>
 * The keys updating may be forbidden or constrained, and if this updating is
 * allowed, the deleting of a key is always through the call of the removing method 
 * with a null value for each key to delete.
 * The values updating may be constrained by a specific method which declares
 * the allowed values. The Value buttons are mandatory, at least for 'See'. <P>
 * The keys are sometimes very structured strings for specialized rights.
 * It completes CollectionPropertyEditor, which handles more complex types
 * but handles maps only as collections of values, and StringMapPropertyEditor
 * which is for unmodifiable all-String maps. <P>
 * This class design is a merging of CollectionPropertyEditor, for the buttons handling,
 * and StringMapPropertyEditor for the map handling.
 * This class uses the property value 'restrictedKeyForSpecializedRight' at true
 * to detect the need to update the specialized rights.
 * It uses ACSFactoryUtilityImpl.getL_MapKeysStartsForSpecializedRights()
 * to get the triplet of the allowed values for the key. Then it calls
 * DialogBox.showBackedJLists() to get the user choices, and this method
 * calls back handleCreationDialog(). <P>
 * This calss is called by Gui1.BeamerManager.addPropertyEditor().
 * It is specified by the BeanInfo classes for the property descriptors
 * of StringMetaRightImpl.ACSRights, ACSImpl.SpecializedRightsSet and ConstraintsForRightValues.
 * It is used by Gui1.BeamerManager.addPropertyEditor(). <P>
 * An instance is constructed through the factory method valueOf() which calls
 * the super valueOf() method. It is possible to get a reusable empty instance.
 * java.beans.PropertyEditorManager is not used.
 * setValue(), setAsText() and getAsText() are not operational.
 * @version 0.7.0
 */
public class RightMapPropertyEditor extends BaseObjectMapPropertyEditor
  implements CreationDialogListener {

  /**
   * The 3 JLists for selectiong in addKeyElement() the main components
   * of a new key for a specialized right
   */
  JList[]                               l_jLists_ = null;
  
  /**
   * The text to get the subkey in a new key for a specialized right
   */
  JTextField                            textField_ = new JTextField();  
  
  //-----------------------------------------------------------------------

  /**
   * Constructor FOR USE ONLY through the valueOf() method,
   * EMPTY_INSTANCE and the subclasses.
   */
  RightMapPropertyEditor() {
    displayName_ = "";
  }

  /**
   * Factory method to use when a RightMapPropertyEditor is delegating to us.
   * It constructs the read method for the property, from the BeanInfo information.
   * The property must be a map of Strings as keys, and StringRight arrays as values.
   * Called by BeamerManager.addPropertyEditor().
   * Overrides the super method and calls it.
   * @param _source owning the property.
   * @param _propertyDescriptor is the descriptor of the property
   * @exception NullPointerException if _source or _displayName is null
   * @exception ProcessError if BeanInfo and PropertyDecriptor calls fail
   * @return a RightMapPropertyEditor instance
   */
  public static CommonPropertyEditor valueOf(BaseObject _source,
    PropertyDescriptor _propertyDescriptor)
    throws NullPointerException, ProcessError {

      RightMapPropertyEditor result =
        (RightMapPropertyEditor) BaseObjectMapPropertyEditor.valueOf(
          _source, _propertyDescriptor);

            // for testing purpose
             //System.out.println("RightMapPropertyEditor.valueOf() "+
               //      "after SimplePropertyEditor.valueOf(), for: "+
                 //       result.getDisplayName());

      // force to use a selection of the values in the beamer,
      // whatever the BeanInfo specification, on the contrary of the super class.
      result.limitedValuesMethod_ = null;
 
      return result;
  } // valueOf()

  // ----------------------- PUBLIC METHODS ---------------------------------

  /**
   * Returns a full custom Component that displays its property value,
   * with the relevant buttons following the property descriptor.
   * Called by BeamerManager.addPropertyEditor().
   * Calls changeDimensions().
   * @return a JPanel which contains one panel for the keys and their buttons,
   *   and one panel for the values and their buttons.
   */
  public java.awt.Component getCustomEditor() {

                      // for testing purpose
                 //System.out.println("RightMapPropertyEditor.getCustomEditor() for "+
                   //      propertyDescriptor_.getName()+" and the read method "+
                     //    readMethod_.getName());
      
    JPanel result = new JPanel();
    try {
      if (readMethod_.invoke(source_, (Object[])null) == null ) {

                      // for testing purpose
                 //System.out.println("in RightMapPropertyEditor.getCustomEditor(), "+
                  //"the read method is null for "+propertyDescriptor_.getName());
      
          return result;
      } // if
    } catch (InvocationTargetException j) {
      new ProcessError("from RightMapPropertyEditor.getCustomEditor() on the read method "+
        readMethod_.getName()+" - InvocationTargetException "+j.getTargetException());
      return result;
    } catch (Exception l) {
      new ProcessError("from RightMapPropertyEditor.getCustomEditor() on the read method "+
        readMethod_.getName()+" - exception "+l.toString());
      return result;
    }

    // initialization
    //JLabel                      keyTitle = new JLabel(" Keys " );
    //JLabel                      valueTitle = new JLabel(" Values for the key ");
    JList                       keyList;
    JList                       valueList;
    JPanel                      keyListPanel = new JPanel();
    JPanel                      valueListPanel = new JPanel();
    JPanel                      keyButtonPanel = null;
    JPanel                      valueButtonPanel = null;
    //JPanel                      titlePanel = new JPanel();
    JPanel                      mapPanel = new JPanel();
    boolean                     keyButtons = false;
    boolean                     valueButtons = false;    
    
    // an array of 9 booleans: withPreviousButton at 0, withViewButton,
    // withSelectButton, withNewButton, withDeleteButton, withRemoveButton,
    // withClearButton at 6, withNewVButton, withDeletVButton,   
    boolean[]                   l_buttonDisplay = getButtonsDisplaying();   
    
    /*
    keyTitle.setMaximumSize(COLOMN_DIMENSION);
    keyTitle.setMinimumSize(COLOMN_DIMENSION);
    keyTitle.setPreferredSize(COLOMN_DIMENSION);
    keyTitle.setFont(BeamerManager.plainFont);
    keyTitle.setToolTipText("keys in the map");    
    valueTitle.setMaximumSize(COLOMN_DIMENSION);
    valueTitle.setMinimumSize(COLOMN_DIMENSION);
    valueTitle.setPreferredSize(COLOMN_DIMENSION);
    valueTitle.setFont(BeamerManager.plainFont);
    valueTitle.setToolTipText("values for the selected key in the map");
     * */
    

    //titlePanel.add(keyTitle);
    //titlePanel.add(valueTitle); // good position as colomn titles?????
    
    try {
        setKeysModel();
    } catch (Exception e) {
        System.out.println("in RightMapPropertyEditor.getCustomEditor(), "+
          "exception from the initialize keys method: "+e);
    }

    try {
        if (!keysModel_.isEmpty())
            setValuesModel( (String)keysModel_.getElementAt(0) );
    } catch (Exception e) {
        System.out.println("in RightMapPropertyEditor.getCustomEditor(), "+
          "exception from the initialize values method: "+e);
    }
          // for testing purpose
          //System.out.println("in RightMapPropertyEditor.getCustomEditor(),"+
            //" the keys list size is: "+keysModel_.getSize());
          //if (!keysModel_.isEmpty()) System.out.println("the first key is:"+
            //keysModel_.get(0)+".");
          //System.out.println("in RightMapPropertyEditor.getCustomEditor(),"+
            //" the values list size is: "+valuesModel_.getSize());
          //if (!valuesModel_.isEmpty()) System.out.println("the first value is:"+
            //valuesModel_.get(0)+".");

    keyList = new JList(keysModel_);
    // valuesModel_ contains the values for the first key
    if (keysModel_.size() != 0) keyList.setSelectedIndex(0);
    keyList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    // fire events to valueChanged()
    keyList.addListSelectionListener(this);
    
    keyList.setBackground(InternalFrame.NON_EDITABLECOLOR);
    ((DefaultListCellRenderer)keyList.getCellRenderer()).
      setHorizontalAlignment(JLabel.RIGHT);
    ((DefaultListCellRenderer)keyList.getCellRenderer()).
      setHorizontalTextPosition(JLabel.RIGHT);
    keyList.setFont(BeamerManager.plainFont);
    keyScrollPane_ = new JScrollPane(keyList);

          // for testing purpose 
          //System.out.println("in RightMapPropertyEditor.getCustomEditor(),"+
            //" the font cell renderer in list is a: "+
            //((DefaultListCellRenderer)keyList.getCellRenderer()).getFont() );
              //" the keysModel_ is: "+keysModel_+
              //" and keyScrollPane_ is: "+keyScrollPane_);
                  
    // intializes the keys pane dimensions - must be consistent with changeDimensions()
    keyButtonPanel = setVerticalButtonsInEditor(l_buttonDisplay, true, false);
    if (keyButtonPanel != null) keyButtons = true;
    if (keyButtons)
        setListPanelDimension(8, keyScrollPane_, false, true, true);
    else
        setListPanelDimension(8, keyScrollPane_, false, false, false);

          // for testing purpose 
          //System.out.println("in RightMapPropertyEditor.getCustomEditor(),"+
            //      " keyButtons "+keyButtons+" and !keyButtons "+!keyButtons);
    
    keyScrollPane_.getHorizontalScrollBar().setValue(
      keyScrollPane_.getHorizontalScrollBar().getMaximum() );  // do not work well  
    //keyListPanel.add(keyTitle);
    keyListPanel.add(keyScrollPane_);
    keyListPanel.setBackground(Color.WHITE);
    keyListPanel.setBorder(new LineBorder(InternalFrame.FORECOLOR));
    
    valueList = new JList(valuesModel_);
    valueList.setBackground(InternalFrame.NON_EDITABLECOLOR);
    ((DefaultListCellRenderer)valueList.getCellRenderer()).
      setHorizontalAlignment(JLabel.RIGHT);
    ((DefaultListCellRenderer)valueList.getCellRenderer()).
      setHorizontalTextPosition(JLabel.RIGHT);
    valueList.setFont(BeamerManager.plainFont);
    // valueList has not this instance as ListSelectionListener
    valueScrollPane_ = new JScrollPane(valueList);
    
    // intializes the values pane dimensions - must be consistent with changeDimensions()
    valueButtonPanel = setVerticalButtonsInEditor(l_buttonDisplay, false, true);
    if (valueButtonPanel != null) valueButtons = true; // always true
    if (valueButtons)
        setListPanelDimension(8, valueScrollPane_, true, true, false);
    else // no use
        setListPanelDimension(8, valueScrollPane_, false, valueButtons, true);
    
    valueScrollPane_.getHorizontalScrollBar().setValue(
      valueScrollPane_.getHorizontalScrollBar().getMaximum() );  // do not work well 
    valueListPanel.add(valueScrollPane_);
    valueListPanel.setBackground(Color.WHITE);
    valueListPanel.setBorder(new LineBorder(InternalFrame.FORECOLOR));    
 
          // for testing purpose
          //System.out.println("in RightMapPropertyEditor.getCustomEditor(), "+
            //"just before the result build up.");
    
    // built up the result
    if (keyButtonPanel != null) keyListPanel.add(keyButtonPanel);
    if (valueButtonPanel != null) valueListPanel.add(valueButtonPanel);
    
    mapPanel.add(keyListPanel);
    mapPanel.add(valueListPanel);
    
    result.add(Box.createVerticalBox());
    //result.add(titlePanel);    
    result.setBackground(Color.white);
    result.setBorder(new EtchedBorder());
    result.add(mapPanel);

    boundComponent_ = keyList;
    boundComponentTwo_ = valueList;
    
    //if (propertyDescriptor_.isBound() ) boundComponent_ = keyList;

              // for testing purpose
         //System.out.println("in RightMapPropertyEditor.getCustomEditor(), "+
          //"the displayed list for "+propertyDescriptor_.getName()+" is:");
          //for (int i = 0; i < keysModel_.getSize(); i++) {
            //System.out.println("for "+i+": "+keysModel_.getElementAt(i));
         //}

    //titlePanel = null;
    mapPanel = null;
    //keyTitle = null;
    keyList = null;
    keyListPanel = null;
    //valueTitle = null;
    valueList = null;
    valueListPanel = null;
    keyButtonPanel = null;
    valueButtonPanel = null;    
    
    return result;
  } // getCustomEditor()
  
    /**
   * Updates the dimensions of the key and value panels.
   * Called by StringMapPropertyEditor.propertyChange() and overrides the same method
   * in this superclass. The Value buttons are mandatory.
   */
  public void changeDimensions() {
    JPanel                      keyButtonPanel = null;
    JPanel                      valueButtonPanel = null;
    boolean                     keyButtons = false;
    boolean                     valueButtons = false;  
    boolean[]                   l_buttonDisplay = getButtonsDisplaying();    
     
    keyButtonPanel = setVerticalButtonsInEditor(l_buttonDisplay, true, false);
    if (keyButtonPanel != null) keyButtons = true;
    if (keyButtons)
        setListPanelDimension(8, keyScrollPane_, false, true, true);
    else
        setListPanelDimension(8, keyScrollPane_, false, false, false);
        
    valueButtonPanel = setVerticalButtonsInEditor(l_buttonDisplay, false, true);
    setListPanelDimension(8, valueScrollPane_, true, true, false);
   
    keyButtonPanel = null;
    valueButtonPanel = null;
    l_buttonDisplay = null; 
  }  // changeDimensions()

/**
 * From a click on 'New', read the new key set through DialogBox.showBackedJComponents()
 * called by addKeyElement().
 * Call the 'add' method with the new key and an empty right array.
 * No operation if the text at the end of the key is null, empty or too large,
 * if a list has not been selected, or if the text is already a key.
 * @param _data true if the user clicks on OK, and false if the user clicks on Cancel
 */
  public void handleCreationDialog(boolean _data) {
    if (!_data) return;
    if (textField_.getText() == null || textField_.getText().trim().length() < 1 || 
        textField_.getText().length() > 100) {
            DialogBox.showErrorMessage(
                    "The key has to be entered, and it must have no more than 100 characters)");
            return;
    } // if
    boolean error = false;
    for (int i = 1; i < l_jLists_.length; i++) {
        if (l_jLists_[i] == null ) error = true;
    } // for i    
    if (error) {
            DialogBox.showErrorMessage(
                    "Sorry, you have to select at most one value in each list");
            return;
    } // if    
    
    // update newKeyElement_ from the user choices in each JList and in the textfield
    StringBuilder newKeyElement = new StringBuilder((String)l_jLists_[0].getSelectedValue());
    for (int i = 1; i < l_jLists_.length; i++) {
        newKeyElement.append(".");
        newKeyElement.append( (String)l_jLists_[i].getSelectedValue() );
    } // for i
    newKeyElement.append(".");
    newKeyElement.append( textField_.getText().trim() );
    
    textField_.setText("                 ");    
  
    if ( ((Map)getValue()).containsKey(newKeyElement.toString())) {
        DialogBox.showErrorMessage(
                    "Sorry, this key already exists.");
        newKeyElement = null;
        return;
    } // if
    
    try {
      addMethod_.invoke(source_, new Object[] {newKeyElement.toString(), new StringRight[0]} );
    } catch (InvocationTargetException i) {
          // show the message to the user
          int lastIndex = ((InvocationTargetException)i).getTargetException().toString().lastIndexOf(": ");
          if (lastIndex != 0)
            DialogBox.showErrorMessage(
              ((InvocationTargetException)i).getTargetException().toString().substring(lastIndex+1));
          else DialogBox.showErrorMessage(
            ((InvocationTargetException)i).getTargetException().toString()); 

          // CreateError is for the user errors
          //if (!(i.getTargetException() instanceof CreateError)) {
            //  System.out.println("in RightMapPropertyEditor.handleCreationDialog(), "+
              //  "the exception is an InvocationTargetException: ");
              //((InvocationTargetException)i).printStackTrace();
          //} // if
      
            // restore the base value in the editor
          propertyChange(new PropertyChangeEvent(source_, propertyDescriptor_.getName(), null, null));
          
          return;
    } catch (Exception e) {
          System.out.println("in RightMapPropertyEditor.handleCreationDialog(), exception: "+e);
    } // catch 

    // select the new key
    ((JList)boundComponent_).setSelectedValue(newKeyElement.toString(), true);
    
    newKeyElement = null;
  } // handleCreationDialog()

  /**
   * Get the values of variables for the instance.
   *
   * @return source, propertyName and listeners (if they have toString() methods).
   */
  public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("  RIGHT_MAP_PROPERTY_EDITOR.TOSTRING():");

        sb.append(" source_ = ");
        sb.append(source_);

        sb.append("; display name = ");
        sb.append(displayName_);

        sb.append("; listeners list = ");
        sb.append(listeners_);

        return " [" + sb + "] ";
  } // toString()

  /**
   * Finalizes the instance; the call is reserved to BeamerManager.
   */
  public void finalizeForProcess() {
    l_jLists_ = null;
    textField_ = null;
    super.finalizeForProcess();
  } // finalizeForProcess()

  // -------------------- PROTECTED METHODS -----------------------------------
  
  /**
   * Called by actionPerformed() when the user clicks on the button 'New' to
   * add a new key in the right map. This is only for specialized rights.
   * This method uses the property value 'restrictedKeyForSpecializedRight' at true
   * to detect the need to update the specialized rights. It uses
   * ACSFactoryUtilityImpl.getL_MapKeysStartsForSpecializedRights()
   * to get the triplet of the allowed values for the map key, then it calls
   * DialogBox.showBackedJLists() to get the user choices, and this method
   * calls back handleCreationDialog(). 
   * Overrides the super method.
   */
  protected void addKeyElement() {

                          // for testing and tracking purpose
                     //System.out.println("in RightMapPropertyEditor.addKeyElement(), "+
                            //"the property type is "+propertyTypeInGroup_);

    // the 'New' button is reserved to specialized rights
    if (propertyDescriptor_.getValue("restrictedKeyForSpecializedRight") != Boolean.TRUE)
      return;

    // construct l_jLists_ if necessary
    if (l_jLists_ == null) {
        l_jLists_ = new JList[3];
        List<String>[] l_slists = ACSFactoryUtilityImpl.getL_MapKeysStartsForSpecializedRights();
        DefaultListModel model = null;
        for (int i = 0; i < 3; i++) {
            model = new DefaultListModel();
            // update the JList
            for (Iterator ite = l_slists[i].iterator(); ite.hasNext(); ) {
                model.addElement(ite.next());            
            } // for ite
            l_jLists_[i] = new JList(model);
            // add a selection listener
            //l_jlists[i].addListSelectionListener(this);
        }// for i 
    } // if (l_jLists_
    
    textField_.setText("                 ");
    
    try {
        // call handleCreationDialog() when the user closes the data selection frame
        DialogBox.showBackedJLists(this, "Set a new criterion for specialized rights",
                l_jLists_, new String[] {"object", "domain", "subdomain", "interface/type"},
                false, textField_, false);
    } catch (SmallError s) {
        throw new InternalError("in RightMapPropertyEditor.addKeyElement(), exception: "+s);
    }
 } // addKeyElement()

  // -------------------- PRIVATE METHODS ----------------------------------- 

  
} // RightMapPropertyEditor

