//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui1;

import java.beans.*;
import javax.swing.*;
import java.lang.reflect.InvocationTargetException;
import java.awt.event.ActionEvent;
import java.util.Map;

import ARoad0.ErrW.ProcessError;
import ARoad0.ErrW.SmallError;
import ARoad0.ErrB.BaseError;

import ARoad0.gWork.ACS_Facade;
import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBaseInterface.ACSObject;
import ARoad0.gBaseInterface.ACS;

import ARoad0.gBase.NameImpl;

/**
 * This class is responsible for creating, by the mean of panels, maps where the keys
 * have a constrained first part, and an user-defined subkey, and where
 * the values are constrained String arrays. This class may also replace StringMapPropertyEditor
 * to display a map (String, String[]) when there is no button, because the value panel
 * has here a greater width. <P>
 * This class reads and updates a modifiable property map of a gBase JavaBean,
 * for which the type is a Map<String,String[]>.
 * The map updating is never done through a write method or a 'choiceMethodForNew'
 * in the property descriptor, but through 'add/remove' methods defined in the property
 * descriptor. <P>
 * The key 'choiceMethodForNew' of the property descriptor has to be used
 * to get the allowed values for the key starting,
 * then DialogBox.showBackedJLists() is called to get the user choices for the starting
 * and the subkey, and this method calls back handleCreationDialog().
 * Tests the key 'ForOneKeyAsName' in the property descriptor, and if true,
 * sets the key as a name.
 * A subkey has to be entered, when the key starting is ending with a point. <P>
 * This class is quite similar to RightMapPropertyEditor, but with Strings values,
 * and a strong control on the key structure, without using a constrained set of
 * allowed keys. <P>
 * See the <a href="{@docRoot}/copyright.html">Copyright</a>.
 */
public class StringTwoKeysMapPropertyEditor extends StringMapPropertyEditor
  implements CreationDialogListener {

  /**
   * The JList for selectiong in addKeyElement() the first part
   * of a new key
   */
  protected JList                                 jList_ = null;

  /**
   * The text to get the subkey, or second part in a new key
   */
  protected JTextField                            textField_ = new JTextField();

  //-----------------------------------------------------------------------

  /**
   * Constructor FOR USE ONLY through the valueOf() method,
   * EMPTY_INSTANCE and the subclasses.
   */
  StringTwoKeysMapPropertyEditor() {
    displayName_ = "";
  }

  /**
   * Factory method to use when a StringTwoKeysMapPropertyEditor is delegating to us.
   * It constructs the read method for the property, from the BeanInfo information.
   * The property must be a map of Strings as keys, and String arrays as values.
   * Called by BeamerManager.addPropertyEditor().
   * Overrides the super method and calls it.
   * @param _source owning the property.
   * @param _propertyDescriptor is the descriptor of the property
   * @exception NullPointerException if _source or _displayName is null
   * @exception ProcessError if BeanInfo and PropertyDecriptor calls fail
   * @return a StringTwoKeysMapPropertyEditor instance
   */
  public static CommonPropertyEditor valueOf(BaseObject _source,
    PropertyDescriptor _propertyDescriptor)
    throws NullPointerException, ProcessError {

      StringTwoKeysMapPropertyEditor result =
        (StringTwoKeysMapPropertyEditor) StringMapPropertyEditor.valueOf(
          _source, _propertyDescriptor);

            // for testing purpose
             //System.out.println("StringTwoKeysMapPropertyEditor.valueOf() "+
               //      "after StringMapPropertyEditor.valueOf(), for: "+
                 //       result.getDisplayName());

      result.maxKeySize_ = 50;
      //result.maxValueSize_ = 100;

                // for testing and tracking
                //System.out.println("StringTwoKeysMapPropertyEditor.valueOf(),"+
                  //      " keysModel_ at the end: "+result.keysModel_);

      return result;
  } // valueOf()

  // ----------------------- PUBLIC METHODS ---------------------------------


    /**
   * Sets the two panels for the keys and the values lists.
   * Calls SimplePropertyEditor.setListPanelDimension() and getMaximumValuesListSize().
   * @param _keyButton true if there is a vertical panel for key buttons.
   * @param _valueButton true if there is a vertical panel for value buttons.
   */
  public void setPanelDimension(boolean _keyButton, boolean _valueButton) {

      if (_keyButton)
          setListPanelDimension(keysModel_.getSize(), keyScrollPane_,
                  false, true, true);
      else
          setListPanelDimension(keysModel_.getSize(), keyScrollPane_,
                  true, true, false);

      if (_valueButton)
          setListPanelDimension(getMaximumValuesListSize(), valueScrollPane_,
                  false, true, false);
      else
          setListPanelDimension(getMaximumValuesListSize(), valueScrollPane_,
                  false, false, false);
  } // setPanelDimension()

  /**
   * Manages the events fired by the buttons 'New' and 'Delete' for the map keys,
   * and 'NewV', 'Select', 'Remove' and 'DeletV' for the map values.
   * Overrides the super method to handle 'New', 'DeleteV' and 'NewV',
   * and calls it for the buttons 'Delete', 'Select', 'Remove'.
   * Called by AssociationMapPropertyEditor.
   * @param _evt is the event fired by a button
   */
  public void actionPerformed(ActionEvent _evt) {

                      // for testing purpose
                 //System.out.println("StringTwoKeysMapPropertyEditor.actionPerformed(), "+
                  //"_evt from: "+((JButton)_evt.getSource()).getText());

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;

    String buttonText = ((JButton)_evt.getSource()).getText();

    if (!_evt.getSource().getClass().equals(JButton.class)) {
      throw new InternalError("actionPerformed received "+
        "in ActionMapPropertyEditor from a component which is not a button");
    }

    // for testing purpose
    // System.out.println("actionPerformed received in StringTwoKeysMapPropertyEditor from "+
    //  "the button "+((JButton)_evt.getSource()).getText());

    // only for a new key, not a new value
    if ( buttonText.equals( "  New  ")) {
      addKeyElement();
    } // if (((JButton)
    // only for a key, not a new value
    else if ( buttonText.equals("Delete")) {
      super.actionPerformed(_evt);
    }
    // for a value
    else if ( buttonText.equals("NewV")) {
      // user message if no selection in the keys JList
      if ( ((JList)boundComponent_).getSelectedValues().length != 1) {
        DialogBox.showErrorMessage("Please, select one key.");
      }
      else {
          addRestrictedValueElements( (String)keysModel_.getElementAt(
                ((JList)boundComponent_).getSelectedIndex() ) );
      } // else
    } // else if '
    // for a value
    else if ( buttonText.equals("DeletV")) {
      // user message if no selection in the values JList
      if ( ((JList)boundComponentTwo_).getSelectedValues().length == 0) {
        DialogBox.showErrorMessage("Sorry, no selected value(s).");
      }
      else removeValueElements(((JList)boundComponentTwo_).getSelectedValues(), true);
    } // else if ( buttonText
    // for a value
    else if ( buttonText.equals("Select")) {
      super.actionPerformed(_evt);
    } // else if 'Select'
    // for a value
    else if ( buttonText.equals("Remove" )) {
      super.actionPerformed(_evt);
    } // else if ( buttonText == "Remove"

    buttonText = null;
    return;
  } // actionPerformed()

/**
 * From a click on 'New', read the updated new map key which is set
 * through DialogBox.showBackedJComponents() from addKeyElement().
 * Calls the 'add' method with the new key and an empty right array.
 * No operation if the text at the end of the key is null or empty
 * when the selected item in the list is not ending with a point,
 * if a list has not been selected, or if the new key is a current key.
 * Used by AssociationMapPropertyEditor.
 * @param _data true if the user clicks on OK, and false if the user clicks on Cancel
 */
  public void handleCreationDialog(boolean _data) {
    if (!_data || jList_ == null) return;
    
    String          selection = (String)jList_.getSelectedValue();
    StringBuilder   newKeyElement = null;
    String          key = null;

    if (jList_.getModel().getSize() == 1)
        selection = (String)jList_.getModel().getElementAt(0);
    
    if (selection == null ) {
        DialogBox.showErrorMessage(
            "A value must be selected in the list");
        return;
    } // if

    if (selection.endsWith(".") &&
            (textField_.getText() == null || textField_.getText().trim().length() < 1)) {
        DialogBox.showErrorMessage(
            "A subkey has to be entered, when the key is ending with a point");
        return;
    } // if

    // updates newKeyElement_ from the user choices in the JList and in the textfield
    newKeyElement = new StringBuilder(selection);
    if ( selection.endsWith(".")) {
        // if keys may be updated, it may be for child ACS roots, and then with
        // only one starting: 'ACS.Name.<Value>.' followed by an ACS name in its String format
        if (propertyDescriptor_.getValue("ForOneKeyAsName") != null) {
            newKeyElement.append( " " + textField_.getText().trim() + "  " );

            try {
                NameImpl.getNameFromString(" " + textField_.getText().trim() + "  ");
            }
            catch (BaseError b) {
                DialogBox.showErrorMessage(
                        "This key does not end with a name.");
                newKeyElement = null;
                selection = null;
            }
        } // if  (propertyDescriptor_.getValue("ForOneKeyAsName")
        else {
            newKeyElement.append(textField_.getText().trim());
        }
    } // if

    textField_.setText("                 ");
    key = newKeyElement.toString();

    if ( ((Map)getValue()).containsKey(key)) {
        DialogBox.showErrorMessage(
                    "This key is already known.");
        newKeyElement = null;
        key = null;
        selection = null;
        return;
    } // if

                          // for testing and tracking purpose
                     //System.out.println("in StringTwoKeysMapPropertyEditor.handleCreationDialog(), "+
                       //     "the key is ["+key+"] and ForOneKeyAsName: "+
                         //   propertyDescriptor_.getValue("ForOneKeyAsName"));


    try {
      addMethod_.invoke(source_, new Object[] {key, new String[0]} );
    } catch (InvocationTargetException i) {
          // shows the message to the user
          int lastIndex = ((InvocationTargetException)i).getTargetException().toString().lastIndexOf(": ");
          if (lastIndex != 0)
            DialogBox.showErrorMessage(
              ((InvocationTargetException)i).getTargetException().toString().substring(lastIndex+1));
          else DialogBox.showErrorMessage(
            ((InvocationTargetException)i).getTargetException().toString());

          // CreateError is for the user errors
          //if (!(i.getTargetException() instanceof CreateError)) {
            //  System.out.println("in ActionMapPropertyEditor.handleCreationDialog(), "+
              //  "the exception is an InvocationTargetException: ");
              //((InvocationTargetException)i).printStackTrace();
          //} // if

            // restore the base value in the editor
          propertyChange(new PropertyChangeEvent(source_, propertyDescriptor_.getName(), null, null));

          return;
    } catch (Exception e) {
          System.out.println("in ActionMapPropertyEditor.handleCreationDialog(), exception: "+e);
    } // catch

    // sometimes, this does not work, when the new key is not key
    ((JList)boundComponent_).setSelectedValue(key, true);

    newKeyElement = null;
    key = null;
    selection = null;
  } // handleCreationDialog()

  /**
   * Gets the values of variables for the instance.
   *
   * @return source, propertyName and listeners (if they have toString() methods).
   */
  public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("  STRINGTWOKEYS_MAP_PROPERTY_EDITOR.TOSTRING():");

        sb.append(" source_ = ");
        sb.append(source_);

        sb.append("; display name = ");
        sb.append(displayName_);

        sb.append("; listeners list = ");
        sb.append(listeners_);

        return " [" + sb + "] ";
  } // toString()

  /**
   * Finalizes the instance; the call is reserved to BeamerManager.
   */
  public void finalizeForProcess() {
    jList_ = null;
    textField_ = null;
    super.finalizeForProcess();
  } // finalizeForProcess()

  // -------------------- PROTECTED METHODS -----------------------------------

  /**
   * Called by actionPerformed() when the user clicks on the button 'New' to
   * add a new key having the format 'keyStarting.subkey' in the map.
   * The key beginning is constrained, and the subkey is a free text to enter.
   * Tests the value 'ForOneKeyAsName' in the property descriptor, and if true,
   * allows only the key end 'xxxx.name'.
   * To get the restricted keys or key startings, it calls getRestrictedValues(),
   * with a null argument if the source is not an ACSObject.
   * Call DialogBox.showBackedJLists() to get the user choices, and this method
   * calls back handleCreationDialog().
   * Overrides the super method. Used by the subclass AssociationMapPropertyEditor.
   */
  protected void addKeyElement() {

                          // for testing and tracking purpose
                     //System.out.println("in StringTwoKeysMapPropertyEditor.addElement(), "+
                            //"the property type is "+propertyTypeInGroup_);
      
    if (!keysUpdateInMap_) {
        DialogBox.showErrorMessage("Keys cannot be changed in this property");
        return;
    }
    String[]            l_restrictedStartings = null;
    ACS                 acs = null;
    DefaultListModel    model = null;

    // constructs a new jLists_ since getRestrictedValues() may return a different value
    // at each call
    if (source_ instanceof ACSObject)
            acs = (ACS)ACS_Facade.getOpenACS().get(((ACSObject)source_).getAcsName() );
    l_restrictedStartings = getRestrictedValues(acs);

    if (l_restrictedStartings == null || l_restrictedStartings.length == 0) {
        acs = null;
        return;
    }

    model = new DefaultListModel();

    // if keys may be updated, it may be for child ACS roots, and then with
    // only one starting: 'ACS.Name.<Value>.'
    for (int i = 0; i < l_restrictedStartings.length; i++) {
        model.addElement( l_restrictedStartings[i] );
    } // for ite

    jList_ = new JList(model);

    textField_.setText("                 ");

    try {
        // call handleCreationDialog() when the user closes the data selection frame
        DialogBox.showBackedJLists(this, "Note: subkey may be constrained",
                new JList[] {jList_}, new String[] { "key", "subkey"},
                false, textField_, false);
    } catch (SmallError s) {
        throw new InternalError("in StringTwoKeysMapPropertyEditor.addKeyElement(), "+
                "exception: "+s);
    }

    l_restrictedStartings = null;
    acs = null;
    model = null;
 } // addKeyElement()

  // -------------------- PRIVATE METHODS -----------------------------------


} // StringTwoKeysMapPropertyEditor



