//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2002, 2004, 2007-2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import javax.swing.*;
import javax.swing.tree.*;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

import ARoad0.ErrW.*;
import ARoad0.Pattern.StaticClass;
import ARoad0.Pattern.MVC_Model;
import ARoad0.Pattern.MVC_View;

import ARoad0.gWork.ACS_Facade;

import ARoad0.gBaseInterface.*;

import ARoad0.gBase.ResourceImpl;
import ARoad0.gBase.DirectoryImpl;
import ARoad0.gBase.ACSFactoryUtilityImpl;
import ARoad0.gBase.UtilityImpl;

/**
 * This class is responsible for creating and for updating
 * viewable trees from an ACS. All the methods are static.
 * createGraphicalTree is the main method.
 * @version 0.7.0
 */
public class ACSTree implements MVC_Model, MVC_View, StaticClass {
  // variables
  // the only instance of the class, following singleton pattern
  private static ACSTree                instance__ = new ACSTree();

  private static final int              INITIAL_CAPACITY = 10; // the default value: 101

  // the TreeNode and TreeModel that are passed between createOwnResourceGraphicalTree()
  // and buildUpGraphicalDirectoryChildren()
  private static DefaultMutableTreeNode currentACSNode__ =
                                            new DefaultMutableTreeNode();
  private static DefaultTreeModel       acsTreeModel__ =
                                            new DefaultTreeModel(currentACSNode__);

  public static final Map     M_ACS_TREE_UPPER_NODES = new HashMap(10);

  // --------------------- static initialization -----------------------------
  static {
    M_ACS_TREE_UPPER_NODES.put("ACTORS", ACSFactoryUtilityImpl.ACTORS);
    M_ACS_TREE_UPPER_NODES.put("USERS", ACSFactoryUtilityImpl.USERS);
    M_ACS_TREE_UPPER_NODES.put("GROUPS", ACSFactoryUtilityImpl.GROUPS);
    M_ACS_TREE_UPPER_NODES.put("RESOURCES", ACSFactoryUtilityImpl.RESOURCES);
    M_ACS_TREE_UPPER_NODES.put("ACLENTRIES", ACSFactoryUtilityImpl.ACLENTRIES);
    M_ACS_TREE_UPPER_NODES.put("EEPARTIES", ACSFactoryUtilityImpl.EEPARTIES);
    M_ACS_TREE_UPPER_NODES.put("ECRESOURCES", ACSFactoryUtilityImpl.ECRESOURCES);
    M_ACS_TREE_UPPER_NODES.put("ACSRIGHTS", ACSFactoryUtilityImpl.ACSRIGHTS);
    M_ACS_TREE_UPPER_NODES.put("RESVFOLDERS", ACSFactoryUtilityImpl.RESVFOLDERS);
    M_ACS_TREE_UPPER_NODES.put("EPVFOLDERS", ACSFactoryUtilityImpl.EPVFOLDERS);
    M_ACS_TREE_UPPER_NODES.put("PRIVILEGES", ACSFactoryUtilityImpl.PRIVILEGES);
  }

  /**
  * only one private constructor
  */
  private ACSTree() {
  }

  /**
  * Single method to get an instance.
  */
//  public static synchronized ACSTree getInstance(){
//    return instance__;
//  } // getInstance()


  /**
  * Creates a new graphical tree of ACS objects.
  * Called by gDMak.ActionOpenMainFrame.
  * Calls createACSExplorer().
  * @exception ProcessError if _acs is null
  * @param    _acs to process
  * @return   JTree of objects in the ACS
  */
  public static final synchronized JTree createACSTree(ACS _acs)
    throws ProcessError {
          
                    // for testing
                    //System.out.println("ACSTree.createACSTree() for "+_acs.getName());

      // initializes the tree model with one root:
      ImmutableName           sRoot = _acs.getName();
      DefaultMutableTreeNode  root = new DefaultMutableTreeNode(sRoot);
      DefaultTreeModel        acsTreeModel = new DefaultTreeModel(root);
      JTree                   tree = new JTree(acsTreeModel);

      tree.setName(_acs.getName().toString());
        // adds the TreeCellEditor to display icons
      tree.setCellRenderer(new ExplorerTreeCellRenderer());

      acsTreeModel = null;
      sRoot = null;
      return createACSExplorer(_acs, tree, root);
  } // createACSTree()

  /**
  * Creates a graphical subtree of ACS objects.
  * Calls the createXXXGraphicalTree() method, following
  * the 10 value of "nodesInACSTree" in the ACS BeanInfo.
  * Calls checkACSTreeNode() for checking the values for the key 'Explorer.NoAcsNode'
  * from ACSImpl.getM_SpecializedTypesAndExplorer().
  * Calls createGraphicalTree().
  * Called by createACSTree() and TreeManager.createExplorer().
  * @param    _acs to process
  * @param    _tree to process, which may be the explorer tree or not
  * @param    _root of the ACS tree in _tree
  * @exception ProcessError if _acs is null or if "nodesInACSTree" return null
  *   in the BeanInfo of the ACS
  * @return JTree of objects in the ACS, as a subtree under _root
  */
  public static final synchronized JTree createACSExplorer(ACSRun _acs,
        JTree _tree, DefaultMutableTreeNode _root) throws ProcessError {
          
                    // for testing
                    //System.out.println("ACSTree.createACSExplorer() for "+_acs.getName());

    String[] l_mainNodes = (String[]) UtilityImpl.getFirstInstance().selectBeanInfo(_acs).
        getBeanDescriptor().getValue("nodesInACSTree");
          
                    // for testing
                    //System.out.println("ACSTree.createACSExplorer() for "+_acs.getName()+
                      //      " the BeanInfo is: "+
                        //    UtilityImpl.getFirstInstance().selectBeanInfo(_acs) );

    if (l_mainNodes == null || l_mainNodes.length == 0)
      throw new ProcessError("in ACSTree.createACSExplorer(), "+
        "the 'nodesInACSTree' value is null");

    DefaultTreeModel        acsTreeModel = (DefaultTreeModel) _tree.getModel();
    DefaultMutableTreeNode  node = null;
    int                     countOfNonVisibleNodes = 0;
    
    // creates the subnodes and the relevant graphical lists, after a
    // checking for the need for a node in the acs tree from the acs properties.
    for (int i = 0; i < l_mainNodes.length; i++ ) {
      if (!checkACSTreeNode(_acs, l_mainNodes[i])) {
          countOfNonVisibleNodes++;
          continue;
      }
      node = new DefaultMutableTreeNode(M_ACS_TREE_UPPER_NODES.get(
              l_mainNodes[i]), true);
      acsTreeModel.insertNodeInto(node, _root, i - countOfNonVisibleNodes);
      _tree = createGraphicalTree(_acs, _tree, node, l_mainNodes[i]);
    } // for
    
    // creates the subtree listener and registers it in TreeManager
    ACSTreeBaseListenerImpl baseListener =
                  new ACSTreeBaseListenerImpl(_acs, _tree, _root);
    TreeManager.addTreeBaseListener(baseListener);

    acsTreeModel = null;
    l_mainNodes = null;
    node = null;
    baseListener = null;
    
    return _tree;
  } // createACSExplorer(,,)

  /**
  * Checks the need of a node in the acs tree from the ACS properties.
  * ACLs, users, groups, ACS rights, resource virtual folders, eligible party virtual folders,
  * or privileges may be jumped if acs.getM_TypesAndGUIPolicy() contains the relevant value
  * for the key 'Explorer.NoAcsNode'.
  * Called by createACSExplorer().
  * @param _acs to process is an ACS or an ACSFactoryImpl. If ACSFactoryImpl,
  *    only the node 'ACSRIGHTS' is displayed.
  * @param _node is a node name from 
  *    ACSImplBeanInfo.getBeanDescriptor().getValue("nodesInACSTree")
  * @return true if the node has to be displayed in the acs tree
  */
  public static final boolean checkACSTreeNode(ACSRun _acs, String _node) {
      
      if (!(_acs instanceof ACS) ) {
          if (_node.equals("ACSRIGHTS")) return true;
          else return false;
      }
      
      ACS       acs = null;

      if (_acs instanceof ACS) acs = (ACS)_acs;
      else return false;

      Map       map = acs.getM_TypesAndGUIPolicy();
      String[]  l_val = null;
      boolean   nonode = false;  
      boolean   result = false;
      
      if (_node.equals("ACTORS")) {
          if (acs.getManageActors()) result = true;
      }      
      else if (_node.equals("RESOURCES")) {
          if (acs.getManageResources()) result = true;
      }
      else if (_node.equals("USERS")) {
          // test Explorer.NoAcsNode
          if (map != null && map.get("Explorer.NoAcsNode") != null) {
              l_val = (String[]) map.get("Explorer.NoAcsNode");
              for (int i = 0; i < l_val.length; i++) {
                  if (l_val[i].equals("ACSTree."+_node)) {
                      nonode = true;
                      break;
                  }
              } // for i
          } // if
                       
          if (!nonode && acs.getManageAccounts()) result = true;
      }      
      else if (_node.equals("GROUPS")) {
          // test Explorer.NoAcsNode
          if (map != null && map.get("Explorer.NoAcsNode") != null) {
              l_val = (String[]) map.get("Explorer.NoAcsNode");
              for (int i = 0; i < l_val.length; i++) {
                  if (l_val[i].equals("ACSTree."+_node)) {
                      nonode = true;
                      break;
                  }
              } // for i
          } // if
              
          if (!nonode && acs.getManageGroups()) result = true;
      } 
      else if (_node.equals("ACLENTRIES")) {
          // test Explorer.NoAcsNode
          if (map != null && map.get("Explorer.NoAcsNode") != null) {
              l_val = (String[]) map.get("Explorer.NoAcsNode");
              for (int i = 0; i < l_val.length; i++) {
          
                            // for testing
                            //System.out.println("         "+l_val[i] ); 
                  
                  if (l_val[i].equals("ACSTree."+_node)) {
                      nonode = true;
                      break;
                  }
              } // for i
          } // if
              
          if (!nonode && 
                  (acs.getManageInternalAcls() || acs.getManageExternalAcls())) 
              result = true;
          
                    // for testing
                    //System.out.println("ACSTree.checkACSTreeNode() for "+acs.getName()+
                      //      " and ACLENTRIES, result: "+result );          
      }
      else if (_node.equals("EEPARTIES") || _node.equals("ECRESOURCES")) {
          if (acs.getManageExternalAcls() ||
                  // test if the ACS may have inner external privileges
                  (acs.getManageExternalForOneToOneLinkedPrivileges() &&
                  (acs.getManageGrantingRightPrivileges() || acs.getManageDenyingRightPrivileges())))
              result = true;
      }       
      else if (_node.equals("ACSRIGHTS") ) {
          // test Explorer.NoAcsNode
          if (map != null && map.get("Explorer.NoAcsNode") != null) {
              l_val = (String[]) map.get("Explorer.NoAcsNode");
              for (int i = 0; i < l_val.length; i++) {
                  if (l_val[i].equals("ACSTree."+_node)) {
                      nonode = true;
                      break;
                  }
              } // for i
          } // if
             
          if (!nonode && acs.getManageAcsRights()) result = true;
      } // else if

      // new in 0.7.0
      else if (_node.equals("RESVFOLDERS") ) {
          // test Explorer.NoAcsNode
          if (map != null && map.get("Explorer.NoAcsNode") != null) {
              l_val = (String[]) map.get("Explorer.NoAcsNode");
              for (int i = 0; i < l_val.length; i++) {
                  if (l_val[i].equals("ACSTree."+_node)) {
                      nonode = true;
                      break;
                  }
              } // for i
          } // if
             
          if (!nonode && acs.getManageResourceVirtualFolders()) result = true;
      } // else if
      else if (_node.equals("EPVFOLDERS") ) {
          // test Explorer.NoAcsNode
          if (map != null && map.get("Explorer.NoAcsNode") != null) {
              l_val = (String[]) map.get("Explorer.NoAcsNode");
              for (int i = 0; i < l_val.length; i++) {
                  if (l_val[i].equals("ACSTree."+_node)) {
                      nonode = true;
                      break;
                  }
              } // for i
          } // if

          if (!nonode && acs.getManageEPVirtualFolders()) result = true;
      } // else if
      else if (_node.equals("PRIVILEGES") ) {
          // test Explorer.NoAcsNode
          if (map != null && map.get("Explorer.NoAcsNode") != null) {
              l_val = (String[]) map.get("Explorer.NoAcsNode");
              for (int i = 0; i < l_val.length; i++) {
                  if (l_val[i].equals("ACSTree."+_node)) {
                      nonode = true;
                      break;
                  }
              } // for i
          } // if

          if (!nonode && acs.getManagePrivilegeRights()) result = true;
      } // else if

      // for nodes from 'Explorer.SubAcsNode' and 'Explorer.AcsGroupTree'
      else result = true;

      map = null;
      l_val = null;
      nonode = false;      
      return result;
  } // checkACSTreeNode()

  /**
  * Creates a graphical tree for a standard node from one of the 10 value of 'nodesInACSTree'
  * in the ACS BeanInfo.
  * If it is the'RESOURCES' node, the user objects are not the base objects, but
  * their name last component. This avoids to link the base object to the node,
  * allowing the lazy loading of resources from the database files.
  * For the other nodes, the user objects in the nodes are the base objects,
  * excepting for the AclEntries, the Privileges and the external objects.
  * Calls all the createXXXXXXGraphicalTree().
  * Called by createACSExplorer().
  * @param  _acs to process is an ACS or an ACSFactoryImpl
  * @param  _tree to which ACS objects are added, with a DefaultTreeModel
  *           (not a TreeModel)
  * @param  _root is the node to which the graphical tree is added
  * @param  _rootName is the user value of _root, that is a string which
  *   must be a key in M_ACS_TREE_UPPER_NODES
  * @return _tree with a subtree of BaseObjects under _nodeRoot
  */
  public static final synchronized JTree createGraphicalTree(ACSRun _acs,
        JTree _tree, DefaultMutableTreeNode _root, String _rootName) {

    if (_rootName.equals("ACSRIGHTS")) {
      _tree = createAcsRightGraphicalTree(_acs, _tree, _root);
      return _tree; 
    }
    // for the ACSRun from ActionNewACSyst
    if (!(_acs instanceof ACS)) return _tree;
    ACS acs = (ACS)_acs;
    
    if (_rootName.equals("ACTORS")) {
      _tree = createOwnActorGraphicalTree(acs, _tree, _root);
      return _tree;
    }
    else if (_rootName.equals("USERS")) {
      _tree = createOwnUserGraphicalTree(acs, _tree, _root);
      return _tree;
    }
    else if (_rootName.equals("GROUPS")) {
      _tree = createOwnGroupGraphicalTree(acs, _tree, _root);
      return _tree;
    }
    else if (_rootName.equals("RESOURCES")) {
      _tree = createOwnResourceGraphicalTree(acs, _tree, _root);
      return _tree;
    }
    else if (_rootName.equals("ACLENTRIES")) {
      _tree = createOwnAclGraphicalTree(acs, _tree, _root);
      return _tree;
    }
    else if (_rootName.equals("EEPARTIES")) {
      _tree = createExtEligPartyGraphicalTree(acs, _tree, _root);
      return _tree;
    }
    else if (_rootName.equals("ECRESOURCES")) {
      _tree = createExtContResourceGraphicalTree(acs, _tree, _root);
      return _tree;
    }

    // new in 0.7.0
    else if (_rootName.equals("RESVFOLDERS")) {
      _tree = createOwnResourceVirtualFolderGraphicalTree(acs, _tree, _root);
      return _tree;
    }
    else if (_rootName.equals("EPVFOLDERS")) {
      _tree = createOwnEligiblePartyVirtualFolderGraphicalTree(acs, _tree, _root);
      return _tree;
    }
    else if (_rootName.equals("PRIVILEGES")) {
      _tree = createOwnPrivilegeGraphicalTree(acs, _tree, _root);
      return _tree;
    }
    
    return _tree;
  } // createGraphicalTree()

  /**
  * Creates a graphical flat list of own actors (which are also resources).
  * @param  _acs ACS to process
  * @param  _tree JTree to which ACS objects are added, with a DefaultTreeModel
  *           (not a TreeModel)
  * @param  _actRoot DefaultMutableTreeNode in _tree, to which the ACS actors
  *           are added
  * @return _tree with a subtree of actors
  */
  public static final synchronized JTree createOwnActorGraphicalTree(ACS _acs,
        JTree _tree, DefaultMutableTreeNode _actRoot) {

    // variables
        // array of tree nodes associated to ACS groupIDs
    DefaultMutableTreeNode[]  actorNode;
    Actor[]                   actorSet;

    DefaultTreeModel acsTreeModel = (DefaultTreeModel) _tree.getModel();

    // select, sort and test the ACS actors
    actorSet = ACS_Facade.getOwnActorArray(_acs);
    if (actorSet[0] == null ) return _tree;

    // set each actor in a list under _accRoot
    actorNode = new DefaultMutableTreeNode[actorSet.length];
    for (int  i=0 ; i < actorSet.length ; i++) {
      actorNode[i] = new DefaultMutableTreeNode(actorSet[i], true);
      acsTreeModel.insertNodeInto(actorNode[i], _actRoot, i);
    } // for

    actorNode = null;
    actorSet = null;
    acsTreeModel = null;
    return _tree;

  } // createActorGraphicalTree()

 /**
   * Creates a graphical subtree of own resources for an ACS.
   * The search is processed from up to down,
   * and from the first to the last child of each node.
   * Two trees are processed:<P>
   * - the Resources tree from ARoad0.gBase <P>
   * - the TreeNodes tree from javax.swing <P>
   * The user objects are not the base objects, but their name last component.
   * This choice avoids to link the base object to the node,
   * allowing a future lazy loading of resources from the database files.
   * One resource may be associated to only one TreeNode of the account tree.
   * Calls buildUpGraphicalDirectoryChildren().
   * @param   _acs to process
   * @param   _tree to which ACS objects are added, with a DefaultTreeModel
   *            (not a TreeModel)
   * @param   _resRoot DefaultMutableTreeNode in _tree, to which the ACS resources
   *            are added
   * @return  JTree with a subtree of the ACS resources.
   */
  public static final synchronized JTree createOwnResourceGraphicalTree(ACS _acs,
          JTree _tree, DefaultMutableTreeNode _resRoot) {

    acsTreeModel__ = (DefaultTreeModel) _tree.getModel();
     // We use DefaultMutableTreeNode because the
     // DefaultTreeModel can use that to build a complete tree
    Directory               currentDir;
        //(Directory)_acs.getEorM_OwnResources().get(_acs.getOwnResourcesRootName() );
    Resource                resource;
    DefaultMutableTreeNode  root;
    int                     count = _resRoot.getChildCount();
    boolean[]               isDirectory;
    boolean                 hasNonVisibleDirectory = true; // true when currentDir has a child
                                           // that is a directory with children to be
                                           // created in the tree.
    boolean                 directoryFound = false;
    Directory               parent = DirectoryImpl.EMPTY_INSTANCE;
    int                     index = 0;
    Resource                res;
    DefaultMutableTreeNode  node;
    ImmutableName           acsName = _acs.getName();

    // a general loop on all the _acs resources without parent
    for (Iterator ite = _acs.getEorM_OwnNoParentResources().values().iterator(); ite.hasNext() ; ) {
      resource = (Resource)ite.next();

                                // for testing
                                //System.out.println("in ACSTree.createOwnResourceGraphicalTree(), "+
                                  //      "current no-parent resource to process :" +
                                    //    resource.getName());

      if (!(resource instanceof Directory)) {
        root = new DefaultMutableTreeNode(resource.getName().lastComponent(), true);
          // adds root under _resRoot, as a leaf allowing children, following the leaf order
        acsTreeModel__.insertNodeInto(root, _resRoot, count);
        count++;
      }
      else { // (no-parent resource instanceof Directory)
        currentDir = (Directory) resource;

        root = new DefaultMutableTreeNode(currentDir.getName().lastComponent(), true);
          // currentACSNode__ is always associated to currentDir
        currentACSNode__ = root;
          // adds root to the tree, under _resRoot, before the leaves,
        //   and root may be a <ffffff_subacs> node
        acsTreeModel__.insertNodeInto(root, _resRoot, 0);

        hasNonVisibleDirectory = true; // true when currentDir has a child c that
                                               // is a directory with children not still
                                               // created in the tree.
        directoryFound = false;
        parent = DirectoryImpl.EMPTY_INSTANCE;
        index = 0;

        // In this while loop, currentDir is used, in a first step, to create
        // all children in all the first directories at each level of the tree.
        // Then, a second step searches for where there is still one directory
        // with children to create (there are two cases), and sets
        // currentDir to this directory. The first step is then run one more time.

        // we always have :     in java.Tree :          in ARoad.gBase :
        //                          no variable             parent
        //                              |                       |
        //                              - currentACSNode__      - currentDir

        while (hasNonVisibleDirectory && parent != null) {

                           // FIRST STEP

          hasNonVisibleDirectory = false;

          // isDirectory is a boolean array that indicates where there are
          // directories in currentDir. The array may be null.
          isDirectory = buildUpGraphicalDirectoryChildren(currentDir);

          // with the while loop at the first step, goes down to the first directory
          // of every directory, and creates in the tree
          // all the intermediate directories with their children
          if (isDirectory != null) {
            directoryFound = false;
            for (int i = 0 ; i < isDirectory.length ; i++) {
              if (isDirectory[i] == true && directoryFound == false) {
                hasNonVisibleDirectory = true;
                directoryFound = true;
                // jump to the upper level
                currentDir = (Directory)currentDir.getEorL_Children()[i];
                // currentACSNode__ is always associated to currentDir
                // by buildUpGraphicalDirectoryChildren(), with the same algorithm
              }
            } // for i
          } // if (isDirectory != null) {

                           // SECOND STEP

          // If (hasNonVisibleDirectory == true),
          // then currentDir is a directory to explore, and continue in the while loop.

          // if not, currentDir is set to the next directory of its parent
          // (the next sibling),
          // or to the next sibling of its parent, or more until the root
          if (!hasNonVisibleDirectory) {

              parent = (Directory)currentDir.getEorParent();

              // if currentDir is the root of the tree, stop the search
              if (parent == null) break;

              // catch the index where currentDir is, as a child of its parent
              index = Arrays.binarySearch(parent.getEorL_Children(), currentDir);
              
              if (index < 0 ) {
                throw new InternalError("in ACSTree.createOwnResourceGraphicalTree(), "+
                "incorrect synchronization for the searching of currentDir:"+currentDir+
                  " in parent: "+parent);
              }
              
              if (index == parent.getEorL_Children().length - 1) {

                // we are at the end of the parent siblings list.
                // parent has to be set to its parent before the next second while loop
                if (parent.getEorParent() != null) {
                  index = Arrays.binarySearch(((Directory)parent.getEorParent()).getEorL_Children(), parent);
                  parent = (Directory)parent.getEorParent();
                  currentACSNode__ = (DefaultMutableTreeNode)currentACSNode__.getParent();
                }
                else {
                  // stop the while loop
                  parent = null;
                }
              } // if (index == parent.getEorL_Children().length - 1)

              while (parent != null && !hasNonVisibleDirectory) {

                                        // FIRST CASE:

                // searching for the next directory through the parent children
                for (int i = index + 1 ; i < parent.getEorL_Children().length ; i++) {

                  if (parent.getEorL_Children()[i] instanceof Node) {

                    //System.out.println("detection of a new directory" );         //for test

                    // sets currentDir before a return at the top of the first while loop
                    currentDir = (Directory)parent.getEorL_Children()[i];

                                // for testing
                                //System.out.println("in ACSTree.createOwnResourceGraphicalTree(), "+
                                  //      "current directory :" + currentDir.getName()+"\n"+
                                  //      "current node will be :" +
                                  //      ((DefaultMutableTreeNode)currentACSNode__.getParent()).getUserObject());

                    // searching for the TreeNode associated to currentDir.
                    // The correct j is not always the current i, because
                    // the orders are not the same
                    res = null;
                    node = (DefaultMutableTreeNode)currentACSNode__.getParent();
                    for (int j = 0 ; j < node.getChildCount() ; j++) {

                      // get the resource through the sequence of Strings from the node 'RESOURCES'
                        // which defines the resource name
                      res = (Resource)_acs.getEorM_Resources().get(
                              ACSTreeUtilities.getResourceInTree( acsName,
                             (DefaultMutableTreeNode)node.getChildAt( j ) ));

                      //System.out.println("each resource associated to a DefaultMutableTreeNode under " + node + " :" + res);              //for test
                        // for testing
                      //System.out.println("in ACSTreecreateResourceGraphicalTree(), "+
                        //    "the loop on parent.getEorL_Children() \n gets "+
                        //    "res: "+res+"\n and current directory: " + currentDir);
                      
                      if ( res == currentDir) {

                                        // for testing
                                        //System.out.println("in ACSTreecreateResourceGraphicalTree(), "+
                                          //  "the loop on parent.getEorL_Children() is broken "+
                                            //"since res is current directory.");

                        currentACSNode__ = (DefaultMutableTreeNode)node.getChildAt( j );
                        break;
                      }
                    } // for j
                    if (currentDir != _acs.getEorM_Resources().get(
                            ACSTreeUtilities.getResourceInTree(
                            acsName, currentACSNode__ ))) {
                        System.out.println("in ACSTree.createResourceGraphicalTree(), "+
                                "current directory: " + currentDir + "; and current node in tree:" + 
                                _acs.getEorM_Resources().get(
                                    ACSTreeUtilities.getResourceInTree(
                                        acsName, currentACSNode__ )));
                        throw new InternalError("wrong search of the TreeNode "+
                        "associated to currentDir");
                    }
                    res = null;

                    // sets the boolean for repeting the first step
                    hasNonVisibleDirectory = true;

                  } // if (parent.getEorL_Children()[i] instanceof Node)

                  if(hasNonVisibleDirectory) break;
                } // for i

                                    // SECOND CASE:

                // if the first case search is negative,
                // goes down of one level in the tree,
                // and returns to the first case to find the next directory
                if (!hasNonVisibleDirectory) {

                  // parent has to be set to its parent
                  // and index is set for a new search
                  if (parent.getEorParent() != null) {
                    index = Arrays.binarySearch(((Directory)parent.getEorParent()).
                      getEorL_Children(), parent);
                    parent = (Directory)parent.getEorParent();
                    currentACSNode__ = (DefaultMutableTreeNode)currentACSNode__.
                      getParent();
                  } // return to the first case
                  else parent = null; // stop all the loops

                } // if (hasNonVisibleDirectory == false)

              } // while parent != null && hasNonVisibleDirectory == false

          } // if (!hasNonVisibleDirectory)

        } // while (hasNonVisibleDirectory) && (parent != null)
      } // else
    } // Iterator ite

    // for testing and tracking
    //System.out.println("in ACSTree.createOwnResourceGraphicalTree(), "+
    //  "the tree root children count are: " +root.getChildCount() );

    currentDir = null;
    root = null;
    parent = null;
    res = null;
    node = null;
    acsName = null;

    return _tree;
  } // createResourceGraphicalTree()

  /**
  * Creates a graphical tree of own groupIDs with all their members, including
  * other groupIDs. The to-display-by-type groups are put under their proper nodes under
  * the argument _accRoot.
  * On the contrary of the resource tree, one groupID object may be
  * associated there to several TreeNodes in the tree.
  * Calls the buildUpGroupTree method in ACSTreeUtilities.
  * Calls ACSTreeUtilities.getL_GroupTreesToDisplay() to create the groups tree roots,
  * and ACSFactoryUtilityImpl.getL_SubAcsToDisplay() to create the subACSs.
  * From the keys 'Explorer.SubAcsNode' and 'Explorer.AcsGroupTree' in the map of
  * ACSImpl.getM_SpecializedTypesAndExplorer(), creates the String nodes
  * '<fffff_subacs>' or '<fffff_tree>', the parents of the to-display-by-type nodes,
  * where fffff is the first word after 'GroupType.<byAcsCreator>'
  * in the group type for the groups to display directly under this node.
  * @param _acs to process
  * @param _tree to which ACS objects are added, with a DefaultTreeModel
  * @param _grpRoot in _tree, to which the ACS groups
  *           are added. May contains nodes for the typed groups.
  * @return _tree with a subtree of groups
  */
  public static final synchronized JTree createOwnGroupGraphicalTree(ACS _acs,
        JTree _tree, DefaultMutableTreeNode _grpRoot) {

    // variables
    GroupID[]                 l_groups = ACS_Facade.getOwnGroupArray(_acs);
    DefaultMutableTreeNode[]  l_groupNode;    
    boolean                   hasTypedGroups = false;
    List                      l_typedGroups = null;
    DefaultMutableTreeNode    node = null;
    // for the nodes '<fffff_subacs>' or '<fffff_tree>'
    List                      l_secondNodes = ACSFactoryUtilityImpl.getL_SubAcsToDisplay(_acs);
    String                    val = null;
    String                    nodeName = null;    
    int                       count = 0;
    
        // acsTreeModel__ is set again to have a public method
    acsTreeModel__ = (DefaultTreeModel) _tree.getModel();
 
    // from l_secondNodes, creates '<fffff_subacs>',
    // the parent of the groups tree roots,
    // where ffffff is the first word after 'GroupType.<byAcsCreator>'
    // in the group type
    if (l_secondNodes != null ) {
      for (Iterator ite = l_secondNodes.iterator(); ite.hasNext(); ) {
          val = (String)ite.next();

                    // for testing
                    //System.out.println("ACSTree.createOwnGroupGraphicalTree() with the subACS type "+
                      //      val);

          if (val.startsWith("GroupType.<byAcsCreator> ")) {
              if (val.substring(25).indexOf(" ") != -1)
                  nodeName = "<"+val.substring(25, 25 + val.substring(25).indexOf(" "))+"_subacs>";
              else
                  nodeName = "<"+val.substring(25)+"_subacs>";
              node = new DefaultMutableTreeNode(nodeName, true);
              acsTreeModel__.insertNodeInto(node, _grpRoot, count);
              hasTypedGroups = true;
              count++;
          } // if
      }  // for ite     
    } // if    
 
    // from l_secondNodes, creates '<fffff_tree>',
    // the parent of the groups tree roots,
    // where ffffff is the first word after 'GroupType.<byAcsCreator>'
    // in the group type
    l_secondNodes = ACSFactoryUtilityImpl.getL_GroupTreesToDisplay(_acs);
    if (l_secondNodes != null ) {
      for (Iterator ite = l_secondNodes.iterator(); ite.hasNext(); ) {
          val = (String)ite.next();

                    // for testing and tracking
                    //System.out.println("ACSTree.createOwnGroupGraphicalTree() "+
                      //      "with the groups tree type: "+val+" for the acs "+_acs.getName());

          if (val.startsWith("GroupType.<byAcsCreator> ")) {
              if (val.substring(25).indexOf(" ") != -1)
                  nodeName = "<"+val.substring(25, 25 + val.substring(25).indexOf(" "))+"_tree>";
              else
                  nodeName = "<"+val.substring(25)+"_tree>";
              node = new DefaultMutableTreeNode(nodeName, true);
              acsTreeModel__.insertNodeInto(node, _grpRoot, count);
              hasTypedGroups = true;
              count++;

                    // for testing and tracking
                    //System.out.println("ACSTree.createOwnGroupGraphicalTree() "+
                      //      "creates the groups tree: "+nodeName+" for the acs "+_acs.getName());

          } // if
      }  // for ite     
    } // if

    if (l_groups[0] == null ) {
        l_groups = null;
        node = null;
        l_secondNodes = null;
        return _tree;
    }

    l_typedGroups = new ArrayList(INITIAL_CAPACITY);
    l_groupNode = new DefaultMutableTreeNode[l_groups.length];
    // sets each to-display typed groupID in a tree node under a child of _grpRoot
    if (hasTypedGroups) {
        for (int  i = 0 ; i < l_groups.length ; i++) {

                    // for testing
                    //System.out.println("ACSTree.createOwnGroupGraphicalTree() " );

              if (!l_groups[i].getEPType().startsWith("<byAcsCreator> "))
                  continue;

              node = ACSTreeUtilities.getParentOfSubOrTreeTypedGroupID(
                            _grpRoot, l_groups[i]);

              if (node == null) continue;
              
              l_groupNode[i] = new DefaultMutableTreeNode(l_groups[i], true);
              acsTreeModel__.insertNodeInto(l_groupNode[i], node, node.getChildCount());
              l_typedGroups.add(l_groups[i]);
        } // for i
    } // if (hasTypedGroups) {
    
    // sets each no-typed groupID in a tree node under _grpRoot
    for (int  i = 0 ; i < l_groups.length ; i++) {
        
                // for testing
                //System.out.println("l_groupSet[i]: "+ l_groupSet[i].getName() );
        
      if (!l_typedGroups.contains(l_groups[i])) {
          l_groupNode[i] = new DefaultMutableTreeNode(l_groups[i], true);
          acsTreeModel__.insertNodeInto(
                  l_groupNode[i], _grpRoot, _grpRoot.getChildCount());
      } // if (!l_typedGroups
    } // for i

    // builds up the subtree under each l_groupNode[i], to-display typed or not
    for (int  i = 0 ; i < l_groups.length ; i++) {
       ACSTreeUtilities.buildUpGroupTree( _acs, (DefaultTreeModel) _tree.getModel(),
                l_groupNode[i], l_groups[i]);
    } // for i

    l_groupNode = null;
    l_groups = null;
    l_typedGroups = null;
    node = null;
    l_secondNodes = null;
    val = null;
    nodeName = null;      
    return _tree;
  } // createOwnGroupGraphicalTree()

  /**
  * Creates a graphical flat list of own userIDs.
  * @param _acs to process
  * @param _tree to which ACS objects are added, with a DefaultTreeModel
  * (not a TreeModel)
  * @param _accRoot is the node in _tree, to which the ACS userIDs
  * are added
  * @return _tree with a list of userIDs
  */
  public static final synchronized JTree createOwnUserGraphicalTree(ACS _acs,
        JTree _tree, DefaultMutableTreeNode _accRoot) {

    // variables
    DefaultMutableTreeNode    node = TreeManager.EMPTY_TREE_NODE;
    UserID[]                  userSet;
        // acsTreeModel__ is set again to have a public method
    acsTreeModel__ = (DefaultTreeModel) _tree.getModel();

    // selects and sorts the ACS UserIDs
    userSet = ACS_Facade.getOwnUserArray(_acs);
    if (userSet[0] != null ) {
      // set each userID directly under _accRoot
      for (int  i=0 ; i < userSet.length ; i++) {
        node = new DefaultMutableTreeNode(userSet[i], true);
        acsTreeModel__.insertNodeInto(node, _accRoot, i);
      } // for
    } // if

    userSet = null;
    node = null;
    return _tree;
  } // createOwnUserGraphicalTree()


  /**
  * Creates a graphical tree of own AclEntries, or simply a flat list if there is
  * no subACS resources with AclEntries.
  * @param _acs to process
  * @param _tree to which ACS objects are added, with a DefaultTreeModel
  * (not a TreeModel)
  * @param _root is the node in _tree, to which the ACS AclEntries are added
  * @return _tree with a list of AclEntry detailled names
  */
  public static final synchronized JTree createOwnAclGraphicalTree(ACS _acs,
        JTree _tree, DefaultMutableTreeNode _root) {

    // variables
    DefaultMutableTreeNode    node = null;
    DefaultMutableTreeNode    aclNode = null;
    ImmutableAclEntry[]       aclSet;
        // acsTreeModel__ is set again to have a public method
    acsTreeModel__ = (DefaultTreeModel) _tree.getModel();

    List                      l_secondNodes = ACSFactoryUtilityImpl.getL_SubAcsToDisplay(_acs);
    String                    val = null;
    String                    nodeName = null;    
    int                       count = 0;
    boolean                   hasTypedNodes = false;
    Object                    object = null;
    String                    toDisplayType = null;
    String                    type = null;
    String                    resType = null; 
    
        // acsTreeModel__ is set again to have a public method
    acsTreeModel__ = (DefaultTreeModel) _tree.getModel();
 
    // from l_secondNodes, creates the second nodes '<ffffff_subacs>'
    // where ffffff is the first word after 'ResourceType.<byAcsCreator>'
    // in the acl resource type, even if ACLs are not allowed to this resource type.
    if (l_secondNodes != null ) {
      for (Iterator ite = l_secondNodes.iterator(); ite.hasNext(); ) {
          val = (String)ite.next();

          if (!val.startsWith("ResourceType.<byAcsCreator> ")) continue;
          
          if (val.substring(28).indexOf(" ") != -1)
              nodeName = "<"+val.substring(28, 28 + val.substring(28).indexOf(" "))+"_subacs>";
          else
              nodeName = "<"+val.substring(28)+"_subacs>";

          node = new DefaultMutableTreeNode(nodeName, true);
          acsTreeModel__.insertNodeInto(node, _root, count);
          hasTypedNodes = true;
          count++;
      }  // for ite     
    } // if     
    
        // for testing and tracking
        //System.out.println("call of ACSTree.createOwnAclGraphicalTree()");

    // selects the ACS AclEntries
    aclSet = ACS_Facade.getOwnAclEntryArray(_acs);
    if (aclSet != null && aclSet.length != 0 && aclSet[0] != null ) {
      // puts each AclEntry directly under _root or under a SubACS node
      for (int i = 0 ; i < aclSet.length ; i++) {
          node = null;
          aclNode = _root;
          resType = aclSet[i].getEorResource().getResourceType();

          if (hasTypedNodes) {
              for (int j = 0; j < _root.getChildCount(); j++) {

                  object = ((DefaultMutableTreeNode)_root.getChildAt(j)).getUserObject();
                  if (!(object instanceof String)) continue;
                  toDisplayType = (String) object;
                  if (toDisplayType.length() < 9 || !toDisplayType.endsWith("_subacs>")) continue;
                        // '1' for "<" at the start
                  type = toDisplayType.substring(1, toDisplayType.indexOf("_subacs>"));
                  if (//resType.length() >= 24 + type.length() &&
                          resType.startsWith("<byAcsCreator> "+type)) {
                    node = (DefaultMutableTreeNode) _root.getChildAt(j);
                    break;
                  } // if

              } // for j
          } // if (hasTypedNodes)
          
          if (node != null) aclNode = node;
          //  else aclNode = _root;
    
                        // for testing and tracking
                        //System.out.println("ACSTree.createOwnAclGraphicalTree() at index "+
                          //      aclNode.getChildCount());
                        //System.out.println("--- add "+aclSet[i].getDetailledName());


          node = new DefaultMutableTreeNode(aclSet[i].getDetailledName(), false);
          acsTreeModel__.insertNodeInto(node, aclNode, aclNode.getChildCount()); 
      } // for i
    } // if

    aclSet = null;
    node = null;
    resType = null;
    aclNode = null;
    toDisplayType = null;
    val = null;
    nodeName = null;
    object = null;
    return _tree;
  } // createOwnAclGraphicalTree()

  /**
  * Creates 3 graphical subtrees of external eligible parties,
  * which are accounts (groupID and userID) and actors (that are also resources)
  * owned by another ACS, for which the ACS controls specific rights through
  * its AclEntries or Linked Privileges. The user object is the eligible party name,
  * not the eligible party.
  * @param _acs to process
  * @param _tree to which ACS objects are added, with a DefaultTreeModel
  *  (not a TreeModel)
  * @param _epRoot in _tree, to which the ACS eligible
  *  parties are added
  * @return JTree _tree with a list of account and actor names
  */
  public static final synchronized JTree createExtEligPartyGraphicalTree(
     ACS _acs, JTree _tree, DefaultMutableTreeNode _epRoot) {

    // variable
    DefaultTreeModel acsTreeModel = (DefaultTreeModel) _tree.getModel();

    // built up the 3 subnodes
    getExtEPSubTreeRoots(acsTreeModel, _epRoot);

    // builds up the 3 lists
    acsTreeModel = getExtEligibleActorTree(_acs, acsTreeModel,
      (DefaultMutableTreeNode)_epRoot.getChildAt(0));
    acsTreeModel = getExtEligibleUserTree(_acs, acsTreeModel,
      (DefaultMutableTreeNode)_epRoot.getChildAt(1));
    acsTreeModel = getExtEligibleGroupTree(_acs, acsTreeModel,
      (DefaultMutableTreeNode)_epRoot.getChildAt(2));

    acsTreeModel = null;

    return _tree;
  } // createExtEligPartyGraphicalTree()

  /**
  * Creates a graphical tree of external controlled resources
  * owned by another ACS, through the external AclEntries or Linked Privileges.
  * The user object in the tree is the resource name, not the resource.
  * Calls getExtContResourceSubTreeRoots(), getExtContActorTree() and getExtContResourceTree().
  * @param _acs to handle
  * @param _tree to which ACS objects are added, with a DefaultTreeModel
  *   (not a TreeModel)
  * @param _root in _tree, to which the ACS resourcess
  *   are added
  * @return JTree _tree with a list of resource names
  */
  public static final synchronized JTree createExtContResourceGraphicalTree(
    ACS _acs, JTree _tree, DefaultMutableTreeNode _root) {

    // variables
    DefaultTreeModel acsTreeModel = (DefaultTreeModel) _tree.getModel();

    // constructs the 2 subnodes
    getExtContResourceSubTreeRoots(acsTreeModel,_root);

    // creates a graphical flat list of external controlled actors
    acsTreeModel = getExtContActorTree(_acs, acsTreeModel,
        (DefaultMutableTreeNode)_root.getChildAt(0));

    // creates a graphical flat list of external controlled resources
    acsTreeModel = getExtContResourceTree(_acs, acsTreeModel,
      (DefaultMutableTreeNode)_root.getChildAt(1));

    acsTreeModel = null;

    return _tree;

  } // createExtContResourceGraphicalTree()

  /**
  * Creates a graphical flat list of ACS rights.
  * @param _acs to process is an ACS or an ACSFactoryImpl
  * @param _tree to which ACS objects are added, with a DefaultTreeModel
  * (not a TreeModel)
  * @param _accRoot is the node in _tree, to which the ACS rights are added
  * @return _tree with a list of acsrights
  */
  public static final synchronized JTree createAcsRightGraphicalTree(ACSRun _acs,
        JTree _tree, DefaultMutableTreeNode _accRoot) {

    // variables
    DefaultMutableTreeNode    node = TreeManager.EMPTY_TREE_NODE;
    StringRight[]             set;
        // acsTreeModel__ is set again to have a public method
    acsTreeModel__ = (DefaultTreeModel) _tree.getModel();

    // selects and sorts the ACS rights
    set = ACS_Facade.getAcsRightArray(_acs);
    if (set != null && set[0] != null ) {
      // set each right directly under _accRoot
      for (int  i=0 ; i < set.length ; i++) {
        node = new DefaultMutableTreeNode(set[i], false);
        acsTreeModel__.insertNodeInto(node, _accRoot, i);
      } // for
    } // if

    set = null;
    node = null;
    return _tree;
  } // createAcsRightGraphicalTree()
 
 /**
   * Creates a graphical subtree of own resource virtual folders for an ACS.
   * The searching is processed from up to down,
   * and from the first to the last child of each node. <P>
   * Two trees are processed: <P>
   * - the VirtualFolders tree or list from ARoad0.gBase <P>
   * - the TreeNodes tree from javax.swing <P>
   * The TreeNode user objects are the base objects.
   * One virtual folder may be associated to only one TreeNode of the tree.
   * Calls ACSTreeUtilities.buildUpVirtualFolderTree().
   * Called by createGraphicalTree().
   * @param   _acs to process
   * @param   _tree to which ACS objects are added, with a DefaultTreeModel
   *            (not a TreeModel)
   * @param   _vfRoot in _tree, to which the resource virtual folders
   *            are added
   * @return  JTree with a subtree of the ACS resource virtual folders.
   */
  public static final synchronized JTree createOwnResourceVirtualFolderGraphicalTree(
          ACS _acs, JTree _tree, DefaultMutableTreeNode _vfRoot) {

    // variables
    List<VirtualFolder> l_VFs = ACS_Facade.getOwnResourceVirtualFolderList(_acs);
    
    if (l_VFs.isEmpty()) {
        l_VFs = null;
        return _tree;
    }

    //VirtualFolder             vf = null;
    DefaultMutableTreeNode    node = null;

        // acsTreeModel__ is set again to have a public method
    acsTreeModel__ = (DefaultTreeModel) _tree.getModel();

    // sets each VF under _vfRoot, with their elements, and with a tree node if it exists
    for (VirtualFolder vf : l_VFs) {

                // for testing
                //System.out.println("vf: "+ vf.getName() );


        node = new DefaultMutableTreeNode(vf, true);
        acsTreeModel__.insertNodeInto(
              node, _vfRoot, _vfRoot.getChildCount());

        ACSTreeUtilities.buildUpVirtualFolderMembers(acsTreeModel__, node);

        // builds up the subtree under each node tree
        if (_acs.getManageVirtualFolderTrees() && vf.getEorL_Children() != null) {
           ACSTreeUtilities.buildUpVirtualFolderTree( _acs, acsTreeModel__,
                    node, vf);
        } // if
    } // for vf

    node = null;
    l_VFs = null;
    return _tree;
  } // createOwnResourceVirtualFolderGraphicalTree()

 /**
   * Creates a graphical subtree of own EligibleParty virtual folders for an ACS.
   * The searching is processed from up to down,
   * and from the first to the last child of each node. <P>
   * Two trees are processed: <P>
   * - the VirtualFolders tree or list from ARoad0.gBase <P>
   * - the TreeNodes tree from javax.swing <P>
   * The TreeNode user objects are the base objects.
   * One virtual folder may be associated to only one TreeNode of the tree.
   * Calls ACSTreeUtilities.buildUpVirtualFolderTree().
   * Called by createGraphicalTree().
   * @param   _acs to process
   * @param   _tree to which ACS objects are added, with a DefaultTreeModel
   *            (not a TreeModel)
   * @param   _vfRoot in _tree, to which the EligibleParty virtual folders
   *            are added
   * @return  JTree with a subtree of the EligibleParty virtual folders.
   */
  public static final synchronized JTree createOwnEligiblePartyVirtualFolderGraphicalTree(
          ACS _acs, JTree _tree, DefaultMutableTreeNode _vfRoot) {

    // variables
    List<VirtualFolder> l_VFs = ACS_Facade.getOwnEPVirtualFolderList(_acs);

    if (l_VFs.isEmpty()) {
        l_VFs = null;
        return _tree;
    }

    //VirtualFolder             vf = null;
    DefaultMutableTreeNode    node = null;

        // acsTreeModel__ is set again to have a public method
    acsTreeModel__ = (DefaultTreeModel) _tree.getModel();

    // sets each VF without parent in a tree node under _vfRoot
    for (VirtualFolder vf : l_VFs) {

        if (vf.getEorParent() != null) continue;

                        // for testing
                        //System.out.println("ACSTree.createOwnEligiblePartyVirtualFolderGraphicalTree()"+
                          //      " add vf: "+ vf.getName() );

        node = new DefaultMutableTreeNode(vf, true);
        acsTreeModel__.insertNodeInto(
              node, _vfRoot, _vfRoot.getChildCount());

        ACSTreeUtilities.buildUpVirtualFolderMembers(acsTreeModel__, node);

        // builds up the subtree under each node tree
        if (_acs.getManageVirtualFolderTrees() && vf.getEorL_Children() != null) {
           ACSTreeUtilities.buildUpVirtualFolderTree( _acs, acsTreeModel__,
                    node, vf);
        } // if
    } // for vf

    node = null;
    l_VFs = null;
    return _tree;
  } // createOwnEligiblePartyVirtualFolderGraphicalTree()
 
 /**
   * Creates a graphical list of own typed and linked privileges for an ACS.
   * If there are both typed and linked privileges, the typed ones are displayed first.
   * The user object in the node is the detailled name, like in AclEntries.
   * Called by createGraphicalTree().
   * @param   _acs to process
   * @param   _tree to which ACS objects are added, with a DefaultTreeModel
   *            (not a TreeModel)
   * @param   _priRoot in _tree, to which the privileges
   *            are added
   * @return  JTree with a subtree of the ACS privileges.
   */
  public static final synchronized JTree createOwnPrivilegeGraphicalTree(
          ACS _acs, JTree _tree, DefaultMutableTreeNode _priRoot) {

    // variables
    List<Privilege> l_PRIs = ACS_Facade.getOwnPrivilegeList(_acs);
    
    if (l_PRIs.isEmpty()) {
        l_PRIs = null;
        return _tree;
    }

    DefaultMutableTreeNode    node = null;

        // acsTreeModel__ is set again to have a public method
    acsTreeModel__ = (DefaultTreeModel) _tree.getModel();

    // sets each privilege in a tree node under _priRoot
    for (Privilege priv : l_PRIs) {

                // for testing
                //System.out.println("vf: "+ vf.getName() );


        node = new DefaultMutableTreeNode(priv.getDetailledName(), false);
        acsTreeModel__.insertNodeInto(
              node, _priRoot, _priRoot.getChildCount());

    } // for vf

    node = null;
    l_PRIs = null;
    return _tree;
  } // createOwnPrivilegeGraphicalTree()

  /**
  * Creates a graphical flat list of GroupIDs
  * owned by another ACS, for which the ACS controls specific rights
  * through its external AclEntries or Linked Privileges.
  * The user object is the eligible party name, not the eligible party.
  * Calls ACS_Facade.ACS_Facade.getExtEligibleGroupArray().
  * @param _acs to process
  * @param _treeModel to which ACS objects are added
  * @param _root in _treeModel, to which the ACS eligible
  *  parties are added
  * @return updated _treeModel
  */
  public static final synchronized DefaultTreeModel getExtEligibleGroupTree(
    ImmutableACS _acs, DefaultTreeModel _treeModel, DefaultMutableTreeNode _root) {

    // variables
        // array of tree nodes associated to the external objects
    DefaultMutableTreeNode[]  l_groupNode;
    GroupID[]                 l_group;

    // test if empty
    l_group = ACS_Facade.getExtEligibleGroupArray(_acs);
    if (l_group == null) return _treeModel;

    if (l_group.length != 0 ) {
        // set each groupID in a tree node under groupIDs
        l_groupNode = new DefaultMutableTreeNode[l_group.length];
        for (int  i = 0 ; i < l_group.length ; i++) {

          l_groupNode[i] = new DefaultMutableTreeNode(l_group[i].getName(), false);
          _treeModel.insertNodeInto(l_groupNode[i], _root, i);

          // built up the subtree under each l_groupNode[i]
          //ACSTreeUtilities.buildUpGroupTree(_acs, (DefaultTreeModel) _treeModel.getModel(),
          //  l_groupNode[i], l_group[i]);

        } // for i
    } // if l_group

    l_groupNode = null;
    l_group = null;

    return _treeModel;
  } // getExtEligibleGroupTree()

  /**
  * Creates a graphical flat list of Actors owned by another ACS,
  * for which the ACS controls specific rights
  * through its external AclEntries or Linked Privileges.
  * The user object is the eligible party name, not the eligible party.
  * Calls ACS_Facade.ACS_Facade.getExtEligibleActorArray().
  * @param _acs to process
  * @param _treeModel to which ACS objects are added
  * @param _root in _treeModel, to which the ACS eligible
  *  parties are added
  * @return updated _treeModel
  */
  public static final synchronized DefaultTreeModel getExtEligibleActorTree(
    ImmutableACS _acs, DefaultTreeModel _treeModel, DefaultMutableTreeNode _root) {

    // variables
        // array of nodes associated to the external objects
    DefaultMutableTreeNode[]  l_actorNode;
    Actor[]                   l_actor;
    short                     nullCount = 0;

    // test if empty
    l_actor = ACS_Facade.getExtEligibleActorArray(_acs);
    if (l_actor == null) return _treeModel;

    // displays the ACS external eligible (not controlled) actors
    if (l_actor.length != 0 ) {
      // sets each actor in a tree node under groupIDs
      l_actorNode = new DefaultMutableTreeNode[l_actor.length];
      for (int  i=0 ; i < l_actor.length ; i++) {
        // l_actor may be an empty Actor[] array, so test it
        if (l_actor[i].getName() != null) {

            // for testing and tracking
              //System.out.println("in ACSTree.getExtEligibleActorTree() -"+
                //" process:" + l_actor[i].getName());
            
          l_actorNode[i] = new DefaultMutableTreeNode(l_actor[i].getName(), false);
          _treeModel.insertNodeInto(l_actorNode[i], _root, i - nullCount);
        } // if
        else nullCount++;

      } // for
    } // if (l_actor

    l_actorNode = null;
    l_actor = null;

    return _treeModel;
  } // getExtEligibleActorTree()

  /**
  * Creates a graphical flat list of UserIDs owned by another ACS,
  * for which the ACS controls specific rights through
  * its external AclEntries or Linked Privileges.
  * The user object is the eligible party name, not the eligible party.
  * Calls ACS_Facade.ACS_Facade.getExtEligibleUserArray().
  * @param _acs to process
  * @param _treeModel to which ACS objects are added
  * @param _root in _treeModel, to which the ACS eligible
  *  parties are added
  * @return updated _treeModel
  */
  public static final synchronized DefaultTreeModel getExtEligibleUserTree(
    ImmutableACS _acs, DefaultTreeModel _treeModel, DefaultMutableTreeNode _root) {

    // variables
        // array of tree nodes associated to the external objects
    DefaultMutableTreeNode[]  l_userNode;
    UserID[]                  l_user;
    short                     nullCount = 0;

    l_user = ACS_Facade.getExtEligibleUserArray(_acs);
    if (l_user == null) {

                // for testing and tracking
              //System.out.println("in ACSTree.getExtEligibleUserTree(),"+
                //" no operation.");

        return _treeModel;
    }

    // display the ACS external userIDs
    if (l_user.length != 0 ) {
      // set each UserID in a tree node
      l_userNode = new DefaultMutableTreeNode[l_user.length];
      for (int  i = 0 ; i < l_user.length ; i++) {
        if (l_user[i].getName() != null) {
          l_userNode[i] = new DefaultMutableTreeNode(l_user[i].getName(), false);
          _treeModel.insertNodeInto(l_userNode[i], _root, i - nullCount);
        } // if
        else nullCount++;
      } // for
    } // if (l_user != null )

    l_userNode = null;
    l_user = null;

    return _treeModel;
  } // getExtEligibleUserTree()

  /**
  * Create a graphical flat list of external controlled actors
  * (which are also resources!) owned by another ACS.
  * The user object in tree is here the actor name, not the actor,
  * to display an extended name.
  * @param _acs to process
  * @param _treeModel to which ACS objects are added
  * @param _actRoot DefaultMutableTreeNode in _tree, to which the actors
  *   are added
  * @return updated _treeModel
  */
  public static final synchronized DefaultTreeModel getExtContActorTree(
    ImmutableACS _acs, DefaultTreeModel _treeModel, DefaultMutableTreeNode _actRoot) {

    // variables
        // array of tree nodes associated to ACS external actors
    DefaultMutableTreeNode[]  l_actorNode;
    Actor[]                   l_actor;

    // select, sort and test the ACS actors
    l_actor = ACS_Facade.getExtControlledActorArray(_acs);
    if (l_actor[0] == null ) return _treeModel;

    // set each actor in a tree node under _accRoot
    l_actorNode = new DefaultMutableTreeNode[l_actor.length];
    for (int  i=0 ; i < l_actor.length ; i++) {
      l_actorNode[i] = new DefaultMutableTreeNode(l_actor[i].getName(), false);
      _treeModel.insertNodeInto(l_actorNode[i], _actRoot, i);
    } // for

    l_actorNode = null;
    l_actor = null;

    return _treeModel;
  } // getExtContActorTree()

  /**
  * Create a graphical flat list of external controlled resources,
  * which are never actors, owned by another ACS.
  * The user object in tree is here the name, not the resource,
  * to display an extended name.
  * @param _acs to process
  * @param _treeModel to which ACS objects are added
  * @param _root in _treeModel, to which the resources are added
  * @return updated _treeModel
  */
  public static final synchronized DefaultTreeModel getExtContResourceTree(
    ImmutableACS _acs, DefaultTreeModel _treeModel, DefaultMutableTreeNode _root) {

    // variables
        // array of tree nodes associated to ACS external resources
    DefaultMutableTreeNode[]  l_resNode;
    Resource[]                l_res;

    // test if empty
    l_res = ACS_Facade.getExtControlledNonActorResourceArray(_acs);
    if (l_res[0] == null) return _treeModel;

    // create a graphical flat list of external non-actor controlled resources
        // select, sort and test the ACS other external resources
        // set each resource name in a tree node under eCOtherResources
    l_resNode = new DefaultMutableTreeNode[l_res.length];
    for (int  i=0 ; i < l_res.length ; i++) {
      l_resNode[i] = new DefaultMutableTreeNode(l_res[i].getName(), false);
          // getExtContResourceSubTreeRoots() returns 2 nodes
      _treeModel.insertNodeInto(l_resNode[i], _root, i);
    } // for

    l_resNode = null;
    l_res = null;

    return _treeModel;
  } // getExtContResourceTree()

  /**
  * Get the children under the 'External Controlled Resources'
  * node in an ACSTree.
  * @param    _treeModel to which ACS objects are added,
  *             with a DefaultTreeModel (not a TreeModel)
  * @param    _root in _tree, to which the ACS resources
  *             are added
  * @return   _treeModel completed
  */
  public static final DefaultTreeModel getExtContResourceSubTreeRoots(
        DefaultTreeModel _treeModel, DefaultMutableTreeNode _root) {

    // create a graphical flat list of external controlled actors
    DefaultMutableTreeNode  eCActors = new DefaultMutableTreeNode(
            ACSFactoryUtilityImpl.ECACTORS);
    _treeModel.insertNodeInto(eCActors, _root, 0);
    // create a graphical flat list of external controlled other resources
    DefaultMutableTreeNode  eCOtherResources =
        new DefaultMutableTreeNode(ACSFactoryUtilityImpl.ECOTHER_RESOURCES);
    _treeModel.insertNodeInto(eCOtherResources, _root, 1);

    eCActors = null;
    eCOtherResources = null;
    return _treeModel;
  } // getExtContResourceSubTreeRoots

  /**
  * Get the three standard nodes under the 'External Eligible Parties'
  * node in an ACSTree: EEACTORS, EGROUPS, EUSERS.
  * @param _treeModel to which ACS objects are added
  * @param _root in _tree, to which the ACS resources
  * are added
  * @return _treeModel completed
  */
  public static final DefaultTreeModel getExtEPSubTreeRoots(
        DefaultTreeModel _treeModel, DefaultMutableTreeNode _root) {
      // initialize the tree model with three roots:
      //  actors, groupIDs, userIDs
    DefaultMutableTreeNode  actors = new DefaultMutableTreeNode(
            ACSFactoryUtilityImpl.EEACTORS);
    DefaultMutableTreeNode  groupIDs = new DefaultMutableTreeNode(
            ACSFactoryUtilityImpl.EGROUPS);
    DefaultMutableTreeNode  userIDs = new DefaultMutableTreeNode(
            ACSFactoryUtilityImpl.EUSERS);

    _treeModel.insertNodeInto(actors, _root, 0);
    _treeModel.insertNodeInto(userIDs, _root, 1);
    _treeModel.insertNodeInto(groupIDs, _root, 2);

    actors = null;
    groupIDs = null;
    userIDs = null;
    return _treeModel;
  } // getExtEPThreeSubTreeRoots()


  /**
  * Get one of the twelve main nodes in an ACS tree,
  * in the explorer or an another ACS tree.
  * @param  _treeModel the explorer tree model, or the ACS tree model if
  *           it is for an ACS main frame
  * @param  _acs displayed in the ACS tree
  * @param  _title title or user string, of the required node; it has to be an ACSTree
  *           static final variable as ACS.ACTORS for instance
  * @param  _forExplorer true if it is in the explorer, false if it is
  *             in the ACS main frame
  * @return the required node in the ACS tree. May be null.
  */
  public static final DefaultMutableTreeNode getACSTreeMainNode(
    DefaultTreeModel _treeModel, ImmutableACS _acs,
    String _title, boolean _forExplorer) {

    // variables
    DefaultMutableTreeNode  root = (DefaultMutableTreeNode)_treeModel.getRoot();
    DefaultMutableTreeNode  result = root;  // no sens init
    DefaultMutableTreeNode  node = root;  // no sens init
    DefaultMutableTreeNode  aCSnode = root;  // no sens init

    if (_forExplorer) {

      // get the node "open ACS" in the explorer
      node = (DefaultMutableTreeNode)_treeModel.getChild(root,0);
      // select the right ACS tree node
      for (int i = 0; i < node.getChildCount(); i++) {
        aCSnode = (DefaultMutableTreeNode)node.getChildAt(i);
        if ( ((BaseObject)aCSnode.getUserObject()).getName().equals(_acs.getName()) ) {
              break;
        } // if
      } // for

      // test ACTORS = "Actors (rights user & access target)";
      if (_title.equals(ACSFactoryUtilityImpl.ACTORS)) result = (DefaultMutableTreeNode)
        aCSnode.getChildAt(0);
      // test USERS = "UserIDs (rights user)";
      else if (_title.equals(ACSFactoryUtilityImpl.USERS)) result = (DefaultMutableTreeNode)
        aCSnode.getChildAt(1);
      // test GROUPS = "GroupIDs tree (rights user)";
      else if (_title.equals(ACSFactoryUtilityImpl.GROUPS)) result = (DefaultMutableTreeNode)
        aCSnode.getChildAt(2);
      // test RESOURCES = "Resources tree (access target)";
      else if (_title.equals(ACSFactoryUtilityImpl.RESOURCES)) result = (DefaultMutableTreeNode)
        aCSnode.getChildAt(3);
      // test ACLENTRIES = "AclEntries: rights user >> access target";
      else if (_title.equals(ACSFactoryUtilityImpl.ACLENTRIES)) result = (DefaultMutableTreeNode)
        aCSnode.getChildAt(4);
      // test EEPARTIES = "External Eligible Parties (as rights user)";
      else if (_title.equals(ACSFactoryUtilityImpl.EEPARTIES)) result = (DefaultMutableTreeNode)
        aCSnode.getChildAt(5);
      // test EEACTORS = "External Eligible Actors";
      else if (_title.equals(ACSFactoryUtilityImpl.EEACTORS)) result = (DefaultMutableTreeNode)
        ((DefaultMutableTreeNode)aCSnode.getChildAt(5)).getChildAt(0);
      // test EGROUPS = "External GroupIDs";
      else if (_title.equals(ACSFactoryUtilityImpl.EGROUPS)) result = (DefaultMutableTreeNode)
        ((DefaultMutableTreeNode)aCSnode.getChildAt(5)).getChildAt(1);
      // test EUSERS = "External UserIDs";
      else if (_title.equals(ACSFactoryUtilityImpl.EUSERS)) result = (DefaultMutableTreeNode)
        ((DefaultMutableTreeNode)aCSnode.getChildAt(5)).getChildAt(2);
      // test ECRESOURCES = "External Controlled Resources (as access target)";
      else if (_title.equals(ACSFactoryUtilityImpl.ECRESOURCES)) result = (DefaultMutableTreeNode)
        aCSnode.getChildAt(6);
      // test ECACTORS = "External Controlled Actors";
      else if (_title.equals(ACSFactoryUtilityImpl.ECACTORS)) result = (DefaultMutableTreeNode)
        ((DefaultMutableTreeNode)aCSnode.getChildAt(6)).getChildAt(0);
      // test ECOTHER_RESOURCES = "Other External Controlled Resources";
      else if (_title.equals(ACSFactoryUtilityImpl.ECOTHER_RESOURCES)) result = (DefaultMutableTreeNode)
        ((DefaultMutableTreeNode)aCSnode.getChildAt(6)).getChildAt(1);
      else result = null;

      root = null;
      node = null;
      aCSnode = null;
      return result;
    } // if (_forExplorer)

    // if (!_forExplorer)
    // test ACTORS = "Actors (rights user & access target)";
    if (_title.equals(ACSFactoryUtilityImpl.ACTORS)) result = (DefaultMutableTreeNode)
      root.getChildAt(0);
    // test USERS = "UserIDs (rights user)";
    else if (_title.equals(ACSFactoryUtilityImpl.USERS)) result = (DefaultMutableTreeNode)
      root.getChildAt(1);
    // test GROUPS = "GroupIDs tree (rights user)";
    else if (_title.equals(ACSFactoryUtilityImpl.GROUPS)) result = (DefaultMutableTreeNode)
      root.getChildAt(2);
    // test RESOURCES = "Resources tree (access target)";
    else if (_title.equals(ACSFactoryUtilityImpl.RESOURCES)) result = (DefaultMutableTreeNode)
      root.getChildAt(3);
    // test ACLENTRIES = "AclEntries: rights user >> access target";
    else if (_title.equals(ACSFactoryUtilityImpl.ACLENTRIES)) result = (DefaultMutableTreeNode)
      root.getChildAt(4);
    // test EEPARTIES = "External Eligible Parties (as rights user)";
    else if (_title.equals(ACSFactoryUtilityImpl.EEPARTIES)) result = (DefaultMutableTreeNode)
      root.getChildAt(5);
    // test EEACTORS = "External Eligible Actors";
    else if (_title.equals(ACSFactoryUtilityImpl.EEACTORS)) result = (DefaultMutableTreeNode)
      ((DefaultMutableTreeNode)root.getChildAt(5)).getChildAt(0);
    // test EGROUPS = "External GroupIDs";
    else if (_title.equals(ACSFactoryUtilityImpl.EGROUPS)) result = (DefaultMutableTreeNode)
      ((DefaultMutableTreeNode)root.getChildAt(5)).getChildAt(1);
    // test EUSERS = "External UserIDs";
    else if (_title.equals(ACSFactoryUtilityImpl.EUSERS)) result = (DefaultMutableTreeNode)
      ((DefaultMutableTreeNode)root.getChildAt(5)).getChildAt(2);
    // test ECRESOURCES = "External Controlled Resources (as access target)";
    else if (_title.equals(ACSFactoryUtilityImpl.ECRESOURCES)) result = (DefaultMutableTreeNode)
      root.getChildAt(6);
    // test ECACTORS = "External Controlled Actors";
    else if (_title.equals(ACSFactoryUtilityImpl.ECACTORS)) result = (DefaultMutableTreeNode)
      ((DefaultMutableTreeNode)root.getChildAt(6)).getChildAt(0);
    // test ECOTHER_RESOURCES = "Other External Controlled Resources";
    else if (_title.equals(ACSFactoryUtilityImpl.ECOTHER_RESOURCES)) result = (DefaultMutableTreeNode)
      ((DefaultMutableTreeNode)root.getChildAt(6)).getChildAt(1);
    else result = null;

    root = null;
    node = null;
    aCSnode = null;
    return result;
  } // getACSTreeMainNode()


  //
  // --------------- PRIVATE STATIC METHODS ---------------------------
  //

 /**
  * Creates the non-null children as TreeNodes in a graphical tree.
  * Called by createOwnResourceGraphicalTree(), with strong links
  * through the updating of acsTreeModel__.
   * @param   _dir already created as graphical tree in acsTreeModel__
   * @return  boolean[] that indicates where there are directories
   *            as _dir children. The array index is from _dir.getEorL_Children(),
   *            not from the associated currentACSNode__,
   *            where null children are removed. Is null if there is no children.
   */
  private static final boolean[] buildUpGraphicalDirectoryChildren(Directory _dir) {

    DefaultMutableTreeNode  internalTreeNode = TreeManager.EMPTY_TREE_NODE;
    DefaultMutableTreeNode  leaf;
    ImmutableResource[]     l_childrenArray;
    short                   shiftingBack = 0;
    boolean                 directoryFound = false;
    boolean[]               isDirectory;
    boolean                 emptyDir = true;

    // check _dir, and no display if there is only an empty resource as child
    if (_dir == null || _dir.getEorL_Children() == null ||
      (_dir.getEorL_Children().length == 1 &&
        _dir.getEorL_Children()[0].equals(ResourceImpl.EMPTY_INSTANCE)) ) {

            // for testing and tracking
            //System.out.println("In ACSTree.buildUpGraphicalDirectoryChildren() call "+
            //  "for  the directory:" + _dir+". There is no children.");

        internalTreeNode = null;
        leaf = null;
        l_childrenArray = null;
        return null;
    } // if

    // for testing and tracking
    //System.out.println("in a ACSTree.buildUpGraphicalDirectoryChildren() call -"+
    //  " _dir children size is:" + _dir.getEorL_Children().length);

    // initializes and checks _dir children
    l_childrenArray = (ImmutableResource[]) _dir.getEorL_Children();
    for (int i = 0 ; i < l_childrenArray.length ; i++) {
      if (l_childrenArray[i] != null ||
        !l_childrenArray[i].equals(ResourceImpl.EMPTY_INSTANCE)) {
          emptyDir = false;
          break;
      } // if
    } // for

    if (!emptyDir) isDirectory = new boolean[l_childrenArray.length];
    else {
      // for testing and tracking
      //System.out.println("one buildUpGraphicalDirectoryChildren() call "+
        //"for  the directory:" + _dir+". All the children are empty.");

      internalTreeNode = null;
      leaf = null;
      l_childrenArray = null;
      return null;
    } // else

    // all is ok, so build up the tree without the null children
    for (int i = 0 ; i < l_childrenArray.length ; i++) {

      if (l_childrenArray[i] == null) shiftingBack++;

      else if (l_childrenArray[i] instanceof Node) {
        leaf = new DefaultMutableTreeNode(l_childrenArray[i].getName().lastComponent(), true);
        acsTreeModel__.insertNodeInto(leaf, currentACSNode__, i - shiftingBack);
        /* set the boolean returned to the caller */
        isDirectory[i] = true;
        if (directoryFound == false) {
          /* for setting currentACSNode__ to the first created TreeNode  in this loop */
          internalTreeNode = leaf;
          directoryFound = true;
        }
      } // else if (l_childrenArray[i] instanceof Node)

      else {
        isDirectory[i] = false;
        // leaf can't have children
        leaf = new DefaultMutableTreeNode(l_childrenArray[i].getName().lastComponent(), false);
        acsTreeModel__.insertNodeInto(leaf, currentACSNode__, i - shiftingBack);
      }

    } // for i

    // sets up currentACSNode__ to be used by createOwnResourceGraphicalTree()
    if (directoryFound == true) currentACSNode__ = internalTreeNode;

    // for testing and tracking
    //  System.out.println("in a ACSTree.buildUpGraphicalDirectoryChildren() call -"+
    //    " the result is normal and the size is:" + isDirectory.length);

    if (_dir.getNickName().equals("Virtual Objects")) {
        // for testing and tracking
        System.out.println("ACSTree.buildUpGraphicalDirectoryChildren() normal processing for "+
                _dir.getNickName()+
          " and the _dir children size is:" + _dir.getEorL_Children().length);
    }

    l_childrenArray = null;
    internalTreeNode = null;
    leaf = null;

    return isDirectory;
  } // buildUpGraphicalDirectoryChildren()


} // class ACSTree