//Title:        Access Road
//Version:      0.7.1
//Copyright:    Copyright 2002, 2004, 2007, 2008, 2009, 2010, 2012 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import javax.swing.*;
import javax.swing.tree.*;

import java.util.Enumeration;
import java.util.Arrays;
import java.util.Iterator;
import java.util.HashSet;

import ARoad0.Pattern.StaticClass;

import ARoad0.Gui1.InternalFrameModel;
import ARoad0.Gui1.DialogBox;

import ARoad0.gWork.ACS_Facade;
import ARoad0.gWork.ViewInBase_Facade;

import ARoad0.gBaseInterface.*;
import ARoad0.gBase.NameImpl;
import ARoad0.gBase.GroupIDMemberImpl;
import ARoad0.gBase.VirtualFolderImpl;
import ARoad0.gBase.ACSFactoryUtilityImpl;
import ARoad0.gBase.UtilityImpl;
import ARoad0.ErrB.BaseError;

/*
import ARoad0.ErrB.CreateError;
import ARoad0.gBase.Directory;
import ARoad0.gBase.DirectoryImpl;
import ARoad0.gBase.ACSImpl;
import ARoad0.gBase.Node;
import ARoad0.gBase.GroupIDImpl;
import ARoad0.gBase.UserID;
import ARoad0.gBase.Actor;
import ARoad0.gBase.ImmutableAclEntry;
*/

/**
 * This class is responsible for providing
 * useful methods for updating the ACS trees when a base object changes.
 * All methods are static methods.
 * @version 0.7.1
 */
public class ACSTreeUtilities implements StaticClass {
  // variables
  // the only instance of the class, following singleton pattern
  //private static ACSTreeUtilities       instance__ = new ACSTreeUtilities();

  private static final int              INITIAL_CAPACITY = 10; // the default value: 101

  // environment
  private static String                 lineSepar__ = System.getProperty("line.separator");

  // these TreeNode and TreeModel are passed between
  // buildUpGroupTree() and setGraphicalGroupIDMembers(),
  // or between buildUpVirtualFolderTree() and setGraphicalVirtualFolderMembers()
  private static DefaultMutableTreeNode currentACSNode__ =
                                        new DefaultMutableTreeNode();
  private static DefaultTreeModel       acsTreeModel__ =
                                        new DefaultTreeModel(currentACSNode__);

  /**
  * only one private constructor
  */
  private ACSTreeUtilities() {
  }

  /**
  * Only method to get an instance.
  */
//  public static synchronized ACSTreeUtilities getInstance(){
//    return instance__;
//  } // getInstance()


  /**
  * Removes the node associated to an ACS or a view in the explorer.
  * Used by ActionClose. Bug sometimes for view removing.
  * <P>
  * @param _objectName of the ACS or the view
  */
  public static final void removeACSorViewNode(ImmutableName _objectName) {
    // variables
    DefaultTreeModel        model;
    DefaultMutableTreeNode  root;
    DefaultMutableTreeNode  aCSnode;
    DefaultMutableTreeNode  viewNode;

    model = (DefaultTreeModel)InternalFrameModel.getTreeInExplorer().getModel();
    root = (DefaultMutableTreeNode)model.getRoot();
    DefaultMutableTreeNode  entityNode = root; // no sens init
    DefaultMutableTreeNode  searchedNode = root; // no sens init

    // get the nodes "open ACS" and "open views" in the explorer
    aCSnode = (DefaultMutableTreeNode)model.getChild(root,0);
    viewNode = (DefaultMutableTreeNode)model.getChild(root,1);

    if (ACS_Facade.knowsACS(_objectName)) {
      //entityNode = aCSnode;

      // iterate on 'Open Entities' node children
      // no iteration if entityNode is empty
          // caution: it is not coherent; for ACS, the user object is the acs,
          // and for view, the user object is the view name
      for (int i = 0 ; i < model.getChildCount(aCSnode) ; i++) {

        entityNode = (DefaultMutableTreeNode)model.getChild(aCSnode, i);
        if ( ((ACS)entityNode.getUserObject()).getName().equals(_objectName) ) {
          searchedNode = entityNode;
          break;
        } // if
      } // for i
    } //if
    else if (ViewInBase_Facade.knowsView(_objectName)) {
      //entityNode = viewNode;

      // iterate on 'Open Views' node direct and indirect children
      for (Enumeration ite = viewNode.depthFirstEnumeration(); ite.hasMoreElements(); ) {

        entityNode = (DefaultMutableTreeNode)ite.nextElement();
        if (!(entityNode.getUserObject() instanceof ViewInBase)) continue;
        if ( ((ViewInBase)entityNode.getUserObject()).getName().equals(_objectName) ) {
          searchedNode = entityNode;
          break;
        } // if
        
      } // for ite
    } // else if

    if (searchedNode != root && searchedNode != null) {
      model.removeNodeFromParent(searchedNode);
            
                    // for testing and tracking
                    //System.out.println("in ACSTreeUtilities.removeACSorViewNode(), "+
                      //"the object "+searchedNode.getUserObject()+" is removed.");      
      
    }

    model = null;
    root = null;
    aCSnode = null;
    viewNode = null;
    entityNode = null;
    searchedNode = null;
  } // removeACSorViewNode()


  /**
  * Removes all the children of a node in a tree,
  * since DefaultMutableTreeNode.removeAllChildren() does not work.
  * @param _treeModel of the tree
  * @param _rootNode for which the children have to be removed
  */
  public static final void removeAllChildren(
    DefaultTreeModel _treeModel, DefaultMutableTreeNode _rootNode) {
      // variables
      DefaultMutableTreeNode  node;
      HashSet                 l_childrenNodes = new HashSet(10);
      Iterator                iter;

      for (int i = 0 ; i < _treeModel.getChildCount(_rootNode) ; i++) {
        node = (DefaultMutableTreeNode)_treeModel.getChild(_rootNode, i);
        l_childrenNodes.add(node);
      }

      iter = l_childrenNodes.iterator();
      while (iter.hasNext()) {
        node = (DefaultMutableTreeNode)iter.next();

             // for testing and tracking
            //System.out.println("in ACSTreeUtilities.removeAllChildren(), remove "+
              //node.getUserObject());
        
        node.setUserObject(null);
        _treeModel.removeNodeFromParent(node);
      } // while

      l_childrenNodes = null;
      node = null;
      iter = null;
      return;
  } // removeAllChildren()

  /**
   * Inserts or removes a base object in a DefaultTreeModel.
   * No exception if the objet to remove does not exist.
   * If it is a tree of resources, and to avoid any direct link between
   * the resources in the base and the tree model (to allow lazy loading of resources),
   * the user object in the DefaultMutableTreeNode is the last component
   * of the user object name.
   * Otherwise, the user object is the base object.
   * The detection of the object to insert or to remove uses 
   * equals( getName().lastComponent() ) for the resources tree, 
   * and otherwise it uses directly equals() to compare it to every object in the tree.
   * If the object to insert is a node (Directory, VirtualFolder) or a group,
   * it sets a new DefaultMutableTreeNode allowing children. Else, no child is allowed.
   * Called by ACSTreeBaseListenerImpl and ActionClose.
   * @param   _treeModel of the objects tree
   * @param   _parentNode under which the user object
   *            is removed or inserted at the end of the list
   * @param   _obj to insert or to remove. Never null. Must be a BaseObject
   *            if _treeModel is a RESOURCES tree, that is isInResourceTree(_parentNode)
   *            returns true.
   * @param   _toInsert true to insert, false to remove
   * @see     #isInResourceTree
   * @return  the updated tree model
   */
  public static final DefaultTreeModel modifyObjectUnderNode(
    DefaultTreeModel _treeModel, DefaultMutableTreeNode _parentNode,
    Object _obj, boolean _toInsert) {

      if (_treeModel == null || _parentNode == null || _obj == null)
          return _treeModel;

      // initialization
      DefaultMutableTreeNode objNode;
      boolean found = false;
      int index = -1;
      boolean inResourceTree = isInResourceTree(_parentNode);
      if (inResourceTree && !(_obj instanceof BaseObject))
          throw new InternalError("in ACSTreeUtilites.modifyObjectUnderNode(), "+
                  "the argument _obj is not a BaseObject and it should be in the resources tree.");
      BaseObject obj = null;
      if (inResourceTree) obj = (BaseObject) _obj;

      if (_toInsert) {
        // checks if _obj is already there
        for (Enumeration e = _parentNode.children() ; e.hasMoreElements() ;) {
            if (inResourceTree) {
              if (((DefaultMutableTreeNode)e.nextElement()).getUserObject().equals(
                      obj.getName().lastComponent())) {
                found = true;
                break;
              } // if
            } else {
              if (((DefaultMutableTreeNode)e.nextElement()).getUserObject().equals(_obj)) {
                found = true;
                break;
              } // if                
            } // else
        } // for e
        if (found) {
            
                    // for testing and tracking
                    //System.out.println("in ACSTreeUtilities.modifyObjectUnderNode(), "+
                      //"the new object "+_obj+"to insert is already there.");
            
            return _treeModel;
        }

        // inserts the new node at the top of the list
            // a VirtualFolder is processed there
        if (_obj instanceof Node || _obj instanceof GroupID) { 
            if (inResourceTree)
                objNode = new DefaultMutableTreeNode(obj.getName().lastComponent(), true);
            else
                objNode = new DefaultMutableTreeNode(_obj, true);
            
                    // for testing and tracking
                    //System.out.println("in ACSTreeUtilities.modifyObjectUnderNode(), "+
                      //"the new Node or GroupID "+_obj+" is inserted");
            
            _treeModel.insertNodeInto(objNode, _parentNode, 0 );            
        } // if(_obj
        else { // inserts the new node at the end of the list if it is not a node
            
                    // for testing and tracking
                    //System.out.println("in ACSTreeUtilities.modifyObjectUnderNode(), "+
                      //"the new object (not Node nor GroupID) "+_obj+" is inserted");
            
              if (inResourceTree)
                  objNode = new DefaultMutableTreeNode(obj.getName().lastComponent(), false);
              else
                  objNode = new DefaultMutableTreeNode(_obj, false);
              
              _treeModel.insertNodeInto(objNode,
                  _parentNode, _treeModel.getChildCount(_parentNode) );
        } // else
      } // if  (_toInsert)

      
      else { // removes the node
        for (int i = 0 ; i < _treeModel.getChildCount(_parentNode) ; i++) {
          objNode = (DefaultMutableTreeNode)_treeModel.getChild(_parentNode, i);
          if (inResourceTree) {
              if ( obj.getName().lastComponent().equals(objNode.getUserObject()) ) {

                                     // for testing and tracking
                        //System.out.println("in ACSTreeUtilities.modifyObjectUnderNode(), "+
                          //"the object to remove is "+_obj+" and it is equal to "+
                          //objNode.getUserObject());

                      index = i;
                      //_treeModel.removeNodeFromParent(objNode );
                      //objNode.setUserObject(null);

                      break;
              } // if
              //else {
                                      // for testing and tracking
                        //System.out.println("in ACSTreeUtilities.modifyObjectUnderNode(), "+
                          //"the object to remove is "+_obj+" and it is compared to "+
                          //objNode.getUserObject());
              //} // else
          } else {
              if ( _obj.equals(objNode.getUserObject()) ) {

                                     // for testing and tracking
                        //System.out.println("in ACSTreeUtilities.modifyObjectUnderNode(), "+
                          //"the object to remove is "+_obj+" and it is equal to "+
                          //objNode.getUserObject());

                      index = i;
                      //_treeModel.removeNodeFromParent(objNode );
                      //objNode.setUserObject(null);

                      break;
              } // if              
          } // else if (inResourceTree)

        } // for
        
        if (index != -1) {
            
            objNode = (DefaultMutableTreeNode)_treeModel.getChild(_parentNode, index);
            
                                  // for testing and tracking
                    //System.out.println("in ACSTreeUtilities.modifyObjectUnderNode(), "+
                      //"the object to remove is "+_obj+" and it is found as: "+
                      //objNode.getUserObject());             
            
            objNode.setUserObject(null);
            _treeModel.removeNodeFromParent(objNode );

        } else {
                                  // for testing and tracking
                    //System.out.println("in ACSTreeUtilities.modifyObjectUnderNode(), "+
                      //"the object to remove is "+_obj+" and it is not found under "+
                      //_parentNode.getUserObject());
        } // else
      } // else, to remove the node

      objNode = null;
      return _treeModel;
  } // modifyObjectUnderNode()

  /**
   * Inserts a detailledName in a DefaultTreeModel, in a flat list
   * under a node, following the natural order of the argument class.
   * The argument is an AclEntry or a Privilege.
   * The method uses compareTo() on the object to insert it
   * at the right place into the current list of children.
   * Then, the same ordering logic is applied at the opening of the ACS
   * and at each adding of a DetailledName, for displaying thelist in the ACS tree.
   * This means the detailled name sorting is not applied, while
   * the user object node is the argument detailled name.
   * Called by ACSTreeBaseListenerImpl.
   * @param   _treeModel of the objects tree
   * @param   _parentNode under which the user object
   *            is removed or inserted at the end of the list
   * @param   _obj to insert. Never null.
   * @param   _acs ACS of _obj. Never null.
   * @return  the updated tree model
   * @throws InternalError if the argument _obj, or the user object of
   * a _parentNode child, is not a Comparable
   */
  public static final DefaultTreeModel addDetailledNameUnderNode(
    DefaultTreeModel _treeModel, DefaultMutableTreeNode _parentNode,
    DetailledName _obj, ACS _acs) {

        if (_treeModel == null || _parentNode == null || _obj == null)
          return _treeModel;

                    // for testing and tracking
                    //if (_acs.getNickName().equals("rbc3"))
                      //  System.out.println("ACSTreeUtilities.addDetailledNameUnderNode() call for "+
                        //    _obj);

        // initialization
        DefaultMutableTreeNode objNode;
        boolean found = false;
        int index = 0;

        DetailledName obj = null;
        String        name = null;

        // checks if _obj is already there
        for (Enumeration e = _parentNode.children() ; e.hasMoreElements() ;) {

              if (((DefaultMutableTreeNode)e.nextElement()).getUserObject().
                      equals(_obj.getDetailledName())) {
                found = true;
                break;
              } // if

        } // for e
        if (found) {

                    // for testing and tracking
                    //System.out.println("in ACSTreeUtilities.addDetailledNameUnderNode(), "+
                      //"the new object "+_obj+"to insert is already there.");

            return _treeModel;
        }
        
        try {
            // use the natural ordering on the DetailledName class to set the index,
            // and throw an InternalError if an instance is not Comparable
            for (Enumeration e = _parentNode.children() ; e.hasMoreElements() ;) {
                  name = (String)((DefaultMutableTreeNode)e.nextElement()).getUserObject();
                  // for a subACS node while the new ACL is outside the subACS
                  if (name.endsWith("_subacs>"))
                      continue;

                  obj = (DetailledName)ACS_Facade.getACSObjectFromDetailledName(_acs, name);

                  if (!(obj instanceof Comparable))
                      throw new BaseError("non comparable object: "+obj);
                  if (!(_obj instanceof Comparable))
                      throw new BaseError("non comparable object: "+_obj);

                  if (((Comparable)obj).compareTo((Comparable)_obj) < 0) {
                    index++;
                  } // if
                  else {
                      break;
                  }

            } // for e
        }
        catch (BaseError b) {
            throw new InternalError("ACSTreeUtilitites.addDetailledNameUnderNode() on a "+
                    "detailled name, for name: "+name+", exception: "+b.toString());
        }

                    // for testing and tracking 
                    //System.out.println("in ACSTreeUtilities.modifyObjectUnderNode(), "+
                      //"the new object (not Node nor GroupID) "+_obj+" is inserted");

        objNode = new DefaultMutableTreeNode(_obj.getDetailledName(), false);

        _treeModel.insertNodeInto(objNode,
          _parentNode, index );

        obj = null;
        name = null;
        objNode = null;
        return _treeModel;
  } // addDetailledNameUnderNode()

  /**
  * Gets the unique parent node associated to a base object (NOT an ACS or a view)
  * in a subtree from a node (in the explorer or another tree).
  * Used by ACSTreeBaseListenerImpl.changeOneResource and
  * changeOneEligibleParty methods.
  * @param    _rootNode in the tree which defines the subtree
  * @param    _object displayed only once in the subtree from _rootNode
  * @return   the parent node under _rootNode, or null
  */
  public static final DefaultMutableTreeNode getCurrentParent(
    DefaultMutableTreeNode _rootNode, BaseObject _object) {

    if (_object == null || _rootNode == null) return null;
    return (DefaultMutableTreeNode)getCurrentObjectNode(
            _rootNode, _object).getParent() ;
  } // getCurrentParent()

  /**
  * Gets the first tree node associated to a BaseObject (excepting for a view)
  * in a subtree from a node (in the explorer or another tree).
  * If the base object exists more than one time,
  * the result is the FIRST selection through _rootNode.breadthFirstEnumeration().
  * For exemple, when _rootNode is the root of the explorer and when _object is a resource,
  * the returned node is in the ACS resource tree in the explorer.
  * Does not use object methods from the base, to work after the _object deletion.
  * Used by getCurrentParent(), TreeManager.selectNodeInExplorer(), SimplePropertyEditor,
  * CollectionPropertyEditor, BeamerManager.
  * Calls TreeManager.getTreeNodeInNamingTree().
  * @param    _rootNode in the tree which defines the subtree
  * @param    _object displayed in the subtree from _rootNode
  * @return   the node for _object, or null
  */
  public static final DefaultMutableTreeNode getCurrentObjectNode(
    DefaultMutableTreeNode _rootNode, Object _object) {

    if (_object == null || _rootNode == null) return null;
    // variables
    DefaultMutableTreeNode  currentNode = null; 
    DefaultMutableTreeNode  result = null; 
    Enumeration             enumeration = null;
    boolean                 found = false;
    boolean                 inResourceTree = false;
    ACSObject               obj = null;
    ACS                     acs = null;
    Object                  object = null;
    
    if (_object instanceof Resource) {
        obj = (ACSObject)_object;
        acs = (ACS) ACS_Facade.getOpenACS().get(obj.getAcsName());
        inResourceTree = true;
    } // if
    else if (_object instanceof ImmutableACS) {
        object = _object;//).getName();
    }
    else if (_object instanceof DetailledName) {
        object = ((DetailledName)_object).getDetailledName();
    } else
        object = _object;

    // The resources tree may be very large and the searching then uses another process.
    // Otherwise, this enumeration is quite fast to find the node.
    if (!inResourceTree) {  
        enumeration = _rootNode.breadthFirstEnumeration();
        while (enumeration.hasMoreElements()) {
          currentNode = (DefaultMutableTreeNode) enumeration.nextElement();

          if ( currentNode.getUserObject().equals(object) ) {
                result = currentNode;
                found = true;
                break;
          } // if          
        } // while
        
        if (!found) result = null;

        currentNode = null;
        enumeration = null;
        object = null;
        return result;        
    } // if
    
    // else, searching in the may-be large resources tree
    Name name = (Name) acs.getName();
    
    // tests if _rootNode is in the ACS main frame tree
    JComponent compo = 
            (JComponent)InternalFrameModel.getJComponentsOfBaseObjects().get(name);
    if (compo != null && (compo instanceof JTree) && 
          _rootNode.isNodeAncestor( (TreeNode)((JTree)compo).getModel().getRoot() ) )
              currentNode = getACSTreeMainNode(
                  (DefaultTreeModel) ((JTree)compo).getModel(), 
                  acs, (String)ACSTree.M_ACS_TREE_UPPER_NODES.get("RESOURCES"), false);
    // else, it is in the explorer
    else currentNode = getACSTreeMainNode(
            (DefaultTreeModel) InternalFrameModel.getTreeInExplorer().getModel(), 
            acs, (String)ACSTree.M_ACS_TREE_UPPER_NODES.get("RESOURCES"), true);

                      // for testing and tracking
                      //System.out.println("in ACSTreeUtilities.getCurrentObjectNode(), "+
                        //"the searched object is :" + _object);

    result = TreeManager.getTreeNodeInNamingTree(
         acs.getName(), currentNode, obj.getName()); // new in 0.7.0

                                  // for testing and tracking
                    //System.out.println("in ACSTreeUtilities.getCurrentObjectNode(), "+
                      //"the result for acs "+acs.getName()+" , currentNode "+
                      //currentNode.getUserObject()+" and obj "+obj.getName());

    currentNode = null;
    enumeration = null;
    acs = null;
    obj = null; 
    name = null;
    return result;
  } // getCurrentObjectNode()

  /**
  * Gets the first tree node associated to a detailled name object
  * in a subtree from a node (in the explorer or another tree).
  * The detailled name does not contains the ACS name, and so
  * it may exist two identical viewable names for two different objects
  * in the tree.
  * When the object exists more than one time,
  * the result is get with the selection of the right ACS, and through an enumeration
  * of all the relevant children in the tree.
  * Used by TreeManager, CollectionPropertyEditor and BeamerManager.
  * Calls getACSTreeMainNode().
  * @param _object is a DetailledName object displayed in the explorer
  * @param _aCSname is the name of the ACS that manages _object
  * @return the node for _object, or null
  */
  public static final DefaultMutableTreeNode getDetailledNameNodeInExplorer(
    DetailledName _object, ImmutableName _aCSname) {

    if (_object == null || _aCSname == null) return null;
    // variables
    DefaultMutableTreeNode  currentNode = null; 
    DefaultMutableTreeNode  result = null;  
    ACS                     acs = (ACS) ACS_Facade.getOpenACS().get(_aCSname);
    Enumeration             enumeration = null;
    DefaultMutableTreeNode aclNode = getACSTreeMainNode(
            (DefaultTreeModel) InternalFrameModel.getTreeInExplorer().getModel(), 
            acs, (String)ACSTree.M_ACS_TREE_UPPER_NODES.get("ACLENTRIES"), true);
    DefaultMutableTreeNode privNode = getACSTreeMainNode(
            (DefaultTreeModel) InternalFrameModel.getTreeInExplorer().getModel(), 
            acs, (String)ACSTree.M_ACS_TREE_UPPER_NODES.get("PRIVILEGES"), true);
    
    if (aclNode == null && privNode == null) {
        acs = null;
        aclNode = null;
        privNode = null;
        return null;
    } // if

    if (aclNode != null) {
        enumeration = aclNode.breadthFirstEnumeration();
        // _object is a viewable name, for example an acl entry
        while (enumeration.hasMoreElements()) {
          currentNode = (DefaultMutableTreeNode) enumeration.nextElement();
          if ( currentNode.getUserObject().equals(_object.getDetailledName()) ) {

                result = currentNode;
                break;

          } // if
        } // while
    } // if
    
    if (result == null && privNode != null) {
        enumeration = privNode.breadthFirstEnumeration();
        while (enumeration.hasMoreElements()) {
          currentNode = (DefaultMutableTreeNode) enumeration.nextElement();
          if ( currentNode.getUserObject().equals(_object.getDetailledName()) ) {

                result = currentNode;
                break;

          } // if
        } // while
    } // if

                                  // for testing and tracking
                    //System.out.println("in ACSTreeUtilities.getDetailledNameNodeInExplorer(), "+
                      //"the result user object: "+result.getUserObject());

    currentNode = null;
    enumeration = null;
    acs = null;
    aclNode = null;
    return result;
  } // getDetailledNameNodeInExplorer()

  /**
  * Gets the unique parent node to associate to a resource
  * in a subtree from the ACSTree.RESOURCES node (in the explorer or another tree).
  * Used by ACSTreeBaseListenerImpl.changeOneResource().
  * Does not use the getEorParent() method of _resource,
  * since the resource is not always fully initialized, 
  * but uses the _resource name components to find the parent path in the tree.
  * @param    _acs displayed in the ACS tree
  * @param    _treeModel of the objects tree
  * @param    _resource displayed only once in the subtree from
  *             the ACSTree.RESOURCES node
  * @param    _isExplorer true if it is in the explorer tree, false if it is
  *             in the ACS main frame tree
  * @exception InternalError if the search is aborted
  * @return   the right future parent node, or null
  */
  public static final DefaultMutableTreeNode getParentOfResource(
    ImmutableACS _acs, DefaultTreeModel _treeModel,
    ImmutableResource _resource, boolean _isExplorer) {

    // first variable
    DefaultMutableTreeNode  resourceNode = (DefaultMutableTreeNode)
                              ACSTreeUtilities.getACSTreeMainNode(
                                _treeModel, _acs, ACSFactoryUtilityImpl.RESOURCES,
                                _isExplorer);

    // the first simplest case, when _resource has no parent
    if (_acs.getName().noOfComponents() == _resource.getName().noOfComponents() - 1) {
          return resourceNode;
    } // if

    // other variables
    Name                    name = (Name) _acs.getName();
    //ImmutableResource       resource = null;
    DefaultMutableTreeNode  result = resourceNode;
    boolean                 found_one = false;
    boolean                 found_two = false;
    Enumeration             childEnumeration = null;

              //System.out.println("in ACSTreeUtilities.getParentOfResource(), "+
                //" for the acs: " + _acs.getName()+
                //" and the _resource: " + _resource.getName());

    // goes down through the resources tree from currentNode
    // to find each node in the future parent path
    for (Iterator ite = ((Name)_resource.getName()).without( name ).
      immutableComponents().iterator();  ite.hasNext() ; ) {

          name.extended( (String)ite.next() );
          //resource = (ImmutableResource)_acs.getEorM_Resources().get(name);

              // for testing and tracking
              //System.out.println(" ---- the current resource name is :" + name.toString()+
                //", and the current result is: "+result.getUserObject());

          // searching for through the children to find the right path
          childEnumeration = result.children();
          found_one = false;
          while (childEnumeration.hasMoreElements()) {
            result = (DefaultMutableTreeNode) childEnumeration.nextElement();
            // tests to find the next node in the path
            if ( result.getUserObject().equals(name.lastComponent()) ) {
              found_one = true;
              break;
            } // if
          } // while
          if (!found_one ) {
            throw new InternalError("In ACSTreeUtilities.getParentOfResource(),"+
                    " for "+_resource.getName()+
                    ", aborted searching when the current Node is "+result.getUserObject());
          }
          if (name.noOfComponents() == _resource.getName().noOfComponents() - 1) {
            found_two = true;
            break;
          } // if (name.noOfComponents()

    } // for(Iterator ite

    if (!found_two) result = null;

    resourceNode = null;
    name = null;
    //resource = null;
    childEnumeration = null;
    return result;
  } // getParentOfResource()

  /**
  * Gets one of the main nodes in an ACS tree,
  * in the explorer or an another ACS tree.
  * The main nodes are defined by ACSTree.M_ACS_TREE_UPPER_NODES.
  * Creates the node before if necessary, and if the ACS displays the main node.
  * Called by ACSTreeBaseListenerImpl methods, ACSTree.createACSExplorer(),
  * getCurrentObjectNode() and getDetailledNameNodeInExplorer().
  * @see    ACSTree for the main nodes definitions
  * @param  _treeModel the explorer tree model, or the ACS tree model if
  *           it is for an ACS main frame
  * @param  _acs is an ACS or a partial ACSFactoryImpl 
   *          which is displayed in the acs tree
  * @param  _title title or user string, of the required node; it has to be
  *           defined in ACSTree.M_ACS_TREE_UPPER_NODES
  * @param  _forExplorer true if it is in the explorer tree, false if it is
  *             in the ACS main frame tree
  * @return the required node in the ACS tree. May be null.
  */
  public static final DefaultMutableTreeNode getACSTreeMainNode(
    DefaultTreeModel _treeModel, ACSRun _acs, String _title, boolean _forExplorer) {

    // variables
    DefaultMutableTreeNode  root = (DefaultMutableTreeNode)_treeModel.getRoot();
    DefaultMutableTreeNode  result = root;  // no sens init
    DefaultMutableTreeNode  node = root;  // no sens init
    DefaultMutableTreeNode  aCSnode = root; // correct only if !_forExplorer
    int                     nodeIndex = -1; // index of the node under aCSnode

    // initializes aCSnode in explorer
    if (_forExplorer) { // update aCSnode

      // searchings for the node TreeManager.OPEN_ACS in the explorer
      // which is always the first node
      node = (DefaultMutableTreeNode)_treeModel.getChild(root,0);
      // to set aCSnode, selects the right ACS tree node
      for (int i = 0; i < node.getChildCount(); i++) {
        if ( ((DefaultMutableTreeNode)node.getChildAt(i)).getUserObject() == _acs) {
            aCSnode = (DefaultMutableTreeNode) node.getChildAt(i);
            break;
        } // if
      } // for
      if (aCSnode == root) return null;

    } // if (_forExplorer)

                // for testing and tracking
                //System.out.println("in ACSTreeUtilities.getACSTreeMainNode(), "+
                  //"the ACS node is :" + aCSnode + " with " +
                  //aCSnode.getChildCount() + " children. Is it the explorer? " +
                  //_forExplorer+" and with title: "+_title);


    // sets nodeIndex in testing _title as a child under aCSnode
    for (int i = 0; i < aCSnode.getChildCount(); i++) {
      if (_title.equals( 
              ((DefaultMutableTreeNode)aCSnode.getChildAt(i)).getUserObject() )) {
        nodeIndex = i;
        result = (DefaultMutableTreeNode) aCSnode.getChildAt(nodeIndex);
        break;
      } // if
    } // for

          // for testing and tracking
          //System.out.println("in ACSTreeUtilities.getACSTreeMainNode(), "+
            //" aCSnode: "+aCSnode+" , and the current nodeIndex under this node is "+nodeIndex);

    if (nodeIndex != -1 ) {

        // for testing and tracking
         //System.out.println("in ACSTreeUtilities.getACSTreeMainNode(), "+
           // " the result is: "+result.getUserObject()+"|for the acs:"+_acs.getName()); 

      root = null;
      node = null;
      aCSnode = null; 
      
      return result;       
    } // if (nodeIndex  
    
    String[] l_nodesInACSTree =
      (String[]) UtilityImpl.getFirstInstance().selectBeanInfo(_acs).
                    getBeanDescriptor().getValue("nodesInACSTree");

    if (l_nodesInACSTree == null || l_nodesInACSTree.length == 0)
      throw new InternalError("in ACSTreeUtilities.getACSTreeMainNode(), "+
        "the 'nodesInACSTree' value is null.");
    
      // tests EEACTORS = "External Eligible Actors";
      if (_title.equals(ACSFactoryUtilityImpl.EEACTORS)) {

        // sets nodeIndex for ACSTree.EEPARTIES,
        // the parent node of ACSTree.EEACTORS
        for (int i = 0; i < l_nodesInACSTree.length; i++) {
          if (ACSFactoryUtilityImpl.EEPARTIES.equals(
            ACSTree.M_ACS_TREE_UPPER_NODES.get(l_nodesInACSTree[i]) )) {
              nodeIndex = i;
              break;
          }
        } // for

        if (nodeIndex == -1) result = null;
        else if (!((DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex)).isLeaf() )
          result = (DefaultMutableTreeNode)
            ((DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex)).getChildAt(0);
        else {
          // creates the result and its siblings since they do not exist
          result = new DefaultMutableTreeNode(ACSFactoryUtilityImpl.EEACTORS, true);
          _treeModel.insertNodeInto(result,
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 0 );
          _treeModel.insertNodeInto(new DefaultMutableTreeNode(ACSFactoryUtilityImpl.EGROUPS, true),
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 1 );
          _treeModel.insertNodeInto(new DefaultMutableTreeNode(ACSFactoryUtilityImpl.EUSERS, true),
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 2 );
        } // else

      } // else if

      // tests EGROUPS = "External GroupIDs";
      else if (_title.equals(ACSFactoryUtilityImpl.EGROUPS)) {

        // sets nodeIndex for ACSTree.EEPARTIES,
        // the parent node of ACSTree.EGROUPS
        for (int i = 0; i < l_nodesInACSTree.length; i++) {
          if (ACSFactoryUtilityImpl.EEPARTIES.equals(
            ACSTree.M_ACS_TREE_UPPER_NODES.get(l_nodesInACSTree[i]) )) {
              nodeIndex = i;
              break;
          }
        } // for

        if (nodeIndex == -1) result = null;
        else if (!((DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex)).isLeaf() )
          result = (DefaultMutableTreeNode)
            ((DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex)).getChildAt(1);
        else {
          // creates the result and its siblings since they do not exist
          result = new DefaultMutableTreeNode(ACSFactoryUtilityImpl.EGROUPS, true);
          _treeModel.insertNodeInto(new DefaultMutableTreeNode(ACSFactoryUtilityImpl.EEACTORS, true),
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 0 );
          _treeModel.insertNodeInto(result,
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 1 );
          _treeModel.insertNodeInto(new DefaultMutableTreeNode(ACSFactoryUtilityImpl.EUSERS, true),
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 2 );
        } // else

      }
      // tests EUSERS = "External UserIDs";
      else if (_title.equals(ACSFactoryUtilityImpl.EUSERS)) {

        // sets nodeIndex for ACSTree.EEPARTIES,
        // the parent node of ACSTree.EUSERS
        for (int i = 0; i < l_nodesInACSTree.length; i++) {
          if (ACSFactoryUtilityImpl.EEPARTIES.equals(
            ACSTree.M_ACS_TREE_UPPER_NODES.get(l_nodesInACSTree[i]) )) {
              nodeIndex = i;
              break;
          }
        } // for

        if (nodeIndex == -1) result = null;
        else if (!((DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex)).isLeaf() )
          result = (DefaultMutableTreeNode)
            ((DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex)).getChildAt(2);
        else {
          // creates the result and its siblings since they do not exist
          result = new DefaultMutableTreeNode(ACSFactoryUtilityImpl.EUSERS, true);
          _treeModel.insertNodeInto(new DefaultMutableTreeNode(ACSFactoryUtilityImpl.EEACTORS, true),
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 0 );
          _treeModel.insertNodeInto(new DefaultMutableTreeNode(ACSFactoryUtilityImpl.EGROUPS, true),
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 1 );
          _treeModel.insertNodeInto(result,
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 2 );
        } // else

      }
      // tests ECACTORS = "External Controlled Actors";
      else if (_title.equals(ACSFactoryUtilityImpl.ECACTORS)) {

        // sets nodeIndex for ACSTree.ECRESOURCES,
        // the parent node of ACSTree.ECACTORS
        for (int i = 0; i < l_nodesInACSTree.length; i++) {
          if (ACSFactoryUtilityImpl.ECRESOURCES.equals(
            ACSTree.M_ACS_TREE_UPPER_NODES.get(l_nodesInACSTree[i]) )) {
              nodeIndex = i;
              break;
          }
        } // for

        if (nodeIndex == -1) result = null;
        else if (!((DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex)).isLeaf() )
          result = (DefaultMutableTreeNode)
            ((DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex)).getChildAt(0);
        else {
          // creates the result and its siblings since they do not exist
          result = new DefaultMutableTreeNode(ACSFactoryUtilityImpl.ECACTORS, true);
          _treeModel.insertNodeInto(result,
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 0 );
          _treeModel.insertNodeInto(new DefaultMutableTreeNode(ACSFactoryUtilityImpl.ECOTHER_RESOURCES, true),
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 1 );
        } // else

      }
      // test ECOTHER_RESOURCES = "Other External Controlled Resources";
      else if (_title.equals(ACSFactoryUtilityImpl.ECOTHER_RESOURCES)) {

        // sets nodeIndex for ACSTree.ECRESOURCES,
        // the parent node of ACSTree.ECOTHER_RESOURCES
        for (int i = 0; i < l_nodesInACSTree.length; i++) {
          if (ACSFactoryUtilityImpl.ECRESOURCES.equals(
            ACSTree.M_ACS_TREE_UPPER_NODES.get(l_nodesInACSTree[i]) )) {
              nodeIndex = i;
              break;
          }
        } // for

        if (nodeIndex == -1) result = null;
        else if (!((DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex)).isLeaf() )
          result = (DefaultMutableTreeNode)
            ((DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex)).getChildAt(1);
        else {
          // creates the result and its siblings since they do not exist
          result = new DefaultMutableTreeNode(ACSFactoryUtilityImpl.ECOTHER_RESOURCES, true);
          _treeModel.insertNodeInto(new DefaultMutableTreeNode(ACSFactoryUtilityImpl.ECACTORS, true),
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 0 );
          _treeModel.insertNodeInto(result,
            (DefaultMutableTreeNode)aCSnode.getChildAt(nodeIndex), 1 );
        } // else

      } // else if

    if (result == root) result = null;
    root = null;
    node = null;
    aCSnode = null;
    l_nodesInACSTree = null;

              // for testing and tracking
              //System.out.println("in ACSTreeUtilities.getACSTreeMainNode(), "+
                //"for "+_title+", result: "+result);

    return result;
  } // getACSTreeMainNode()

  /**
  * Builds up a subtree for a GroupID.
  * It is a top-down searching, from the first to the last member of each node.
  * It is rather complex because two trees are managed:
  * the groupIDmember tree from ARoad0.gBase and the TreeNode tree from java.swing.
  * On the contrary of ACSTree.createOwnResourceGraphicalTree(),
  * one groupID object may there be associated to several TreeNodes of the group tree.
  * An another difference is the searching for parent. Here, the Resource.parent_
  * cannot be used, and TreeNode.parent_ replaces it.
  * Called by ACSTree.createOwnGroupGraphicalTree(), <P>
  * ACSTree.createExtEligPartyGraphicalTree(), <P>
  * ACSTreeBaseListenerImpl.changeGroupIDMember(). <P>
  * Calls setGraphicalGroupIDMembers().
  * When the group is new, it has only an empty member which is not displayed.
  * If the number of levels in the tree exceeds 15, it is probably a bug.
  * In that case, the process stops, the tree is uncomplete and
  * an error message is displayed to the user.
  * @param  _acs which owns _group
  * @param  _treeModel which owns _node
  * @param  _node having children to update
  * @param  _group in _acs, associated to _node in _treeModel
  * @return _treeModel with a subtree of nodes under _node
  */
  public static final DefaultTreeModel buildUpGroupTree(ACS _acs,
    DefaultTreeModel _treeModel, DefaultMutableTreeNode _node, GroupID _group) {

    // tests the ACSTree.GROUPS node in the explorer
    if (ACSTreeUtilities.getACSTreeMainNode(
      (DefaultTreeModel)InternalFrameModel.getTreeInExplorer().getModel(),
      _acs, (String)ACSTree.M_ACS_TREE_UPPER_NODES.get("GROUPS"), true) == null) {

             // for testing and tracking
            //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), for"+
              //_acs.getName()+", the ACSTree.GROUPS node in the explorer"+
              //" is not found for the group: "+_group);
          
          return _treeModel;
    }

    // variables
        // currentACSNode__ is always associated to currentGroup,
        // excepted after buildUpGraphicalGroupIDMembers()
        // where it may be the first created node for a groupID
    GroupID                     currentGroup = _group;
    ImmutableGroupIDMember[]    l_currentParentMembers;
    int                         currentLevel = 0;                 // process stops if exceeds 15
    boolean[]                   l_isGroup;                 // l_isGroup is a boolean array
                                       // that indicates where there are
                                      // groups in currentGroup
    boolean                     hasNonVisibleGroup = false; // true when currentGroup has a member m that
                                       // is a group with members not still created
                                       // in the tree, even if m is a group already
                                       // created with its member in an another
                                       // place of the tree.
        //greatParent is equal to "GroupIDs tree (rights user)", 
        // or to a sub-or-tree-typed groups node
    String                      greatParent = (String) ((DefaultMutableTreeNode)_node.
                                        getParent()).getUserObject(); 
    Object                      currentParent = greatParent; // 'parent' has to be understood
                                      // here with the idea that an eligible party
                                        // may have several parents, since a groupID
                                        // may be member of several groupIDs.
    int                         index = 0;
    GroupIDMember               ep = null;
    MutableTreeNode             node = null;
    
    currentACSNode__ = _node;
        // currentACSNode__ and acsTreeModel__ are used
        // by buildUpGraphicalGroupIDMembers()
    acsTreeModel__ = _treeModel;

                          // for testing and tracking
                          //System.out.println("ACSTreeUtilities.buildUpGroupTree(), "+
                            //"for the group "+_group);

    // creates the _group members and set currentGroup to the first GroupID
    // if it exists (then hasNonVisibleGroup == true), and runs the while loop
    l_isGroup = setGraphicalGroupIDMembers(_group);

    // with the while loop at the first step, goes down to the first group
    // of every group, and create in the tree
    // all the intermediate groups with their members
    for (int i = 0 ; i < l_isGroup.length ; i++) {
      if (l_isGroup[i]) {
          hasNonVisibleGroup = true;
          // jump to the upper level
          currentLevel++;
          currentParent = _group;
          currentGroup = (GroupID)_group.getEorL_Members()[i];
          // currentACSNode__ is always associated to this new currentGroup
          // after buildUpGraphicalGroupIDMembers();
          // see buildUpGraphicalGroupIDMembers() where the same algorithm is used.

                          // for testing and tracking
                          //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                            //"before the first step, new currentGroup: "+currentGroup);
          break;
      }
    } // for i

    // if _group has no group has member, the process stops there.
    
    // In this while loop, currentACSNode__ and currentGroup are used, in a first step,
    // to create all members in all the first groups at each level of the tree.
    // Then, a second step searches if there is still one group
    // with members to create (there are two cases), and if yes, sets currentACSNode__
    // and currentGroup to this directory. The first step is then run one more time
    // but for the detection of the next group to process.
    // The basic structure is:  in java.Tree:           in ARoad.gBase:
    //
    //                          no variable             currentParent
    //                              |                       |
    //                          currentACSNode__        currentGroup
    //
    // but the reference is always currentACSNode__ in all the steps and all the cases.

    while (hasNonVisibleGroup && !currentParent.equals(greatParent) ) {

                       // FIRST STEP:

                          // for testing and tracking
                          //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                            //"in the first step, currentGroup: "+currentGroup+
                            //", and currentACSNode__: "+currentACSNode__);

      hasNonVisibleGroup = false;
      // creates the subtree for currentGroup
      l_isGroup = setGraphicalGroupIDMembers(currentGroup);

      // with the while loop at the first step, go down to the first group
      for (int i = 0 ; i < l_isGroup.length ; i++) {
        if (l_isGroup[i]) {
          hasNonVisibleGroup = true;
          // jump to the upper level
          currentLevel++;
          // There is no need to update currentParent here: no use
          currentGroup = (GroupID)currentGroup.getEorL_Members()[i];

                          // for testing and tracking
                          //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                            //"in the first step, after setGraphicalGroupIDMembers(), "+
                            //"new currentGroup: "+currentGroup+", currentLevel = "+currentLevel);

          break;
        }
      } // for i
      // currentACSNode__ is always associated to currentGroup, even
      // if currentGroup has no group as child.
      // sees buildUpGraphicalGroupIDMembers() where the same algorithm is used.

      // stops the loop if too large
      if (currentLevel > 15) {
        DialogBox.showErrorMessage("There is a chain of more than 15 nested groups"+
          " in the ACS "+_acs.getName());
        break;
      }

                       // SECOND STEP when !hasNonVisibleGroup:

      // If (hasNonVisibleGroup),
      // then currentGroup is a Group to explore;
      // return to the start of the first while loop
      // to explore the first group which is a currentGroup member.

      // If not, currentGroup is set to the next Group of its current parent
      // (the next sibling),
      // or to the next sibling of its current parent, or more, until the root.
      // currentACSNode__ is associated to currentGroup.
      while (!hasNonVisibleGroup) {

                        // for testing and tracking
                        //System.out.println("in ACSTreeUtilities.buildUpGroupTree(),"+
                          //" currentGroup will be set to the next Group of its current parent."+
                          //"currentGroup: "+currentGroup+", and currentACSNode__:"+
                          //currentACSNode__.getUserObject() );
                      
                        // stop the loop
                      //DialogBox.showErrorMessage("Stop in the second step when !hasNonVisibleGroup,"+
                        //"currentACSNode__: "+currentACSNode__.getUserObject());
                        
                      
          // sets currentParent from currentACSNode__
          currentParent = ((DefaultMutableTreeNode)currentACSNode__.
            getParent()).getUserObject();
          // currentACSNode__ is at the right level, but without the next searched value.
          // currentGroup is synchonized to currentACSNode__.

          // old instruction: if currentGroup is _group,
          // there is no next Group of its current parent, stop all?
          
          // stop all the loops if currentParent.equals(accounts)
          if (currentParent.equals(greatParent) ) { 

                          // for testing and tracking
                          //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                            //"in the while hasNonVisibleGroup loop, end of the loop with currentParent: "
                            //+currentParent);

              hasNonVisibleGroup = true;
              currentParent = greatParent;
              break;
          } //if

          // else currentParent is a GroupID
          // catches the index where currentGroup is, as a member of its current parent
          l_currentParentMembers = ((GroupID)currentParent).getEorL_Members();
          //Arrays.sort(l_currentParentMembers);
          index = Arrays.binarySearch( l_currentParentMembers, currentGroup);
        
          if (index < 0 ) {
            throw new InternalError("incorrect synchronization for "+
              "the searching of currentACSNode__:"+currentACSNode__.getUserObject()+
              " in currentParent: "+currentParent);
          }
          
                          // for testing and tracking
                          //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                            //"SECOND STEP when !hasNonVisibleGroup, "+
                            //"index: "+index+", currentParent is: "+currentParent+
                            //" and currentGroup: "+currentGroup );
                          //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                            //" l_currentParentMembers: ");
                          //for (int i = 0; i < l_currentParentMembers.length; i++) {
                            //System.out.println("---- "+l_currentParentMembers[i].getName());
                          //} // for i

          /*
          if (currentParent.equals(accounts) ) {

                          // for testing and tracking
                          //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                          //  "in the while hasNonVisibleGroup loop, end of the loop. "
                          //  +currentParent);

              hasNonVisibleGroup = true;
              break;
          } //if
           **/

          while (index == l_currentParentMembers.length - 1) {

            // we are at the end of the current parent siblings list.
            // parent has to be set to its parent before the first case.
            // goes down currentParent through a going down of two levels
            // from currentACSNode__
            currentLevel--;
            //secondStepLevelsJump++;
            currentACSNode__ = (DefaultMutableTreeNode) currentACSNode__.getParent();
            currentParent = ((DefaultMutableTreeNode)currentACSNode__.
              getParent()).getUserObject();
            // currentACSNode__ is now at the right level, but it is not the right one
            // for the next search.
            // currentGroup is not synchonized

                          // for testing and tracking
                          //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                            //"in the while INDEX LOOP, index: "+index+", currentLevel:"+
                            //currentLevel+"- to stop, currentParent is compared to accounts,"+
                            //" and its value is: "+currentParent);

            // stops all the loops if currentParent.equals(accounts)
            if (currentParent.equals(greatParent) ) {
                  break;
            } //if
                          
            if (currentParent instanceof GroupID) {
                l_currentParentMembers = ((GroupID)currentParent).getEorL_Members();
                index = Arrays.binarySearch( l_currentParentMembers,
                  currentACSNode__.getUserObject());

                          // for testing and tracking
                          //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                            //"in the while INDEX LOOP");
                          //System.out.println("l_currentParentMembers: ");
                          //for (int i = 0; i < l_currentParentMembers.length; i++) {
                            //System.out.println("---- "+l_currentParentMembers[i].getName());
                          //} // for i
                          //System.out.println("and the searched member for index:"+
                            //currentACSNode__.getUserObject());
            } // if (currentParent

          } // while (index == currentParent.getEorL_Children().length - 1)

          // stops all the loops if currentParent.equals(accounts)
          if (currentParent.equals(greatParent) ) {

                          // for testing and tracking
                          //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                            //"in the while hasNonVisibleGroup loop, end of the loop with currentParent: "
                            //+currentParent);

              hasNonVisibleGroup = true;
              break;
          } //if

                                    // FIRST CASE

                          // for testing and tracking
                          //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                            //"second step, FIRST CASE: index: "+index+", currentParent is: "+
                            //currentParent+", currentACSNode__: "+currentACSNode__+
                            //", currentGroup: "+currentGroup+", currentLevel: "+currentLevel);
                          //System.out.println("second step, first case: and l_currentParentMembers: ");
                          //for (int i = 0; i < l_currentParentMembers.length; i++) {
                            //System.out.println("---- "+l_currentParentMembers[i].getName());
                          //} // for i

                          // stop the loop
                          //DialogBox.showErrorMessage("Stop in the second step, first case, currentGroup: "+currentGroup);
                
                          
          // searching for the next group through the parent members
          for (int i = index + 1 ; i < l_currentParentMembers.length ; i++) {

            if ( !(l_currentParentMembers[i] instanceof GroupID) ) continue;

                      // for testing and tracking
                      //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                        //"in second step, first case, detection of a new groupID: "+l_currentParentMembers[i] );
                      
            // sets currentGroup and currentACSNode__
            // before a return at the top of the first while loop
            // (currentParent is not updated because it is no more used) :
            
            // currentLevel++;
            
            //secondStepLevelsJump--;
            currentGroup = (GroupID) l_currentParentMembers[i];

                            // for testing and tracking
                            //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                              //"second step, first case: currentACSNode__: "+currentACSNode__+
                              //", currentGroup is: "+currentGroup+", currentLevel = "+currentLevel);

            // searches for the TreeNode associated to currentGroup:
            // The correct j is not always the current i, because
            // the orders may be different.
            node = (MutableTreeNode)currentACSNode__.getParent();
            for (int j = 0 ; j < node.getChildCount() ; j++) {
              ep = (GroupIDMember)((DefaultMutableTreeNode)node.
                getChildAt( j )).getUserObject();

                      // for testing and tracking
                      //System.out.println("each resource associated to a "+
                        //"DefaultMutableTreeNode under " + 
                        //((DefaultMutableTreeNode)node).getUserObject()+" :"+ep);

              if ( ep == currentGroup) {
                currentACSNode__ = (DefaultMutableTreeNode)node.getChildAt( j );
                break; // on j
              }
            } // for j
            if (currentGroup != currentACSNode__.getUserObject() ) {
              throw new InternalError("incorrect synchronization for "+
                "the TreeNode that is associated to currentGroup");
            }

            // sets the boolean for repeating the first step
            hasNonVisibleGroup = true;
            break; // return to the first step
          
          } // for i
          // else, return to the top of the !hasNonVisibleGroup while loop

                                // SECOND CASE:

            // if there is no groupID among the parent members
            // (the first case search is negative),
            // goes down one level in the tree,
            // and returns to the first case to find the next group
          if (!hasNonVisibleGroup) {
                  
                // currentParent has to be set to its currentParent,
                // and index is set for a new search
                currentLevel--;
                currentACSNode__ = (DefaultMutableTreeNode)currentACSNode__.getParent();
                currentGroup = (GroupID)((DefaultMutableTreeNode)currentACSNode__).
                  getUserObject();
                //currentParent = ((DefaultMutableTreeNode)currentACSNode__.
                //  getParent()).getUserObject();

                              // stop the loop
                              //DialogBox.showErrorMessage("Stop in the second step, second case, currentGroup is: "+
                                //currentGroup);
                
                            // for testing and tracking
                            //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                              //"second step, second case: currentACSNode__: "+currentACSNode__+
                              //", currentGroup is: "+currentGroup+", currentLevel = "+currentLevel);

                // stop all the loops if currentParent.equals(accounts)
                if (currentParent.equals(greatParent) ) {

                                // for testing and tracking
                                //System.out.println("in ACSTreeUtilities.buildUpGroupTree(), "+
                                  //"in the second step, second case, end of the loop. "
                                  //+currentParent);

                    hasNonVisibleGroup = true;
                    break;
                } //if

          } // if (!hasNonVisibleGroup)

      } // while (!hasNonVisibleGroup)

    } // while (hasNonVisibleGroup) && !currentParent.equals(accounts)

    l_currentParentMembers = null;
    currentParent = null;
    currentGroup = null;
    ep = null;
    l_isGroup = null;
    node = null;
    greatParent = null;

    return _treeModel;

  } // buildUpGroupTree()

  /**
  * Since the resource in an ACS tree is not the user object of the node,
  * gets the resource through the sequence of Strings from the node 'RESOURCES'
  * which defines the resource name.
  * Used by ACSTree.createOwnResourceGraphicalTree(), and
  * TreeManager.getTrueUserObjectInTreeExplorer().
  * @param    _acsName is the name of the acs which owned the resource
  * @param    _node of the resource in the tree
  * @return   the resource name. May be null.
  */
  public static final ImmutableName getResourceInTree(
    ImmutableName _acsName, DefaultMutableTreeNode _node) {

    if (_acsName == null || _node == null) return null;
    
    // variables
    Name                    name = new NameImpl();  
    
    TreeNode[] l_path = _node.getPath();

    // for testing    
    //for (int i = 0 ; i < l_path.length ; i++) {
      //  System.out.print(l_path[i].getClass().toString()+"\n");
    //}
    //DefaultMutableTreeNode[] l_path = (DefaultMutableTreeNode[])_node.getPath();    
    
    String resourcesRoot = (String)ACSTree.M_ACS_TREE_UPPER_NODES.get("RESOURCES");
    int resIndex = -1;
    
    for (int i = 0; i < l_path.length; i++) {
        if (resourcesRoot.equals( ((DefaultMutableTreeNode)l_path[i]).getUserObject()) ) {
            resIndex = i;
            continue;
        }
        if (resIndex != -1) {
          // from resourcesRoot node, build up the resource name 
            name.extended( (String) ((DefaultMutableTreeNode)l_path[i]).getUserObject());
        }
    } // i++
    
    
    l_path = null;
    resourcesRoot = null;
    
                            // for testing
                        //System.out.print("in ACSTreeUtilities.getResourceInTree(), "+
                          //      "the returned resource name, after the ACS name, is "+
                            //    name+"\n");
    
    return ((Name)_acsName.clone()).extended(name);
  } // getResourceInTree()  

  /**
  * Gets the parent node associated to a sub-or-tree-typed group
  * in the explorer or another ACS tree. A sub-or-tree-typed group has
  * its type which starts with '<byAcsCreator> xxx'. It has to be put directly under
  * a TreeNode where the user object has the format '<xxx_subacs>' or '<xxx_tree>'.
  * Otherwise, the group TreeNode is put directly under _rootNode by other methods.
  * Called by ACSTreeBaseListenerImpl.changeOneEligibleParty() and modifyGroupIDMembers(),
  * ACSTree.createOwnGroupGraphicalTree().
  * @param _rootNode TreeNode to which all the ACS groups are added.
  * @param    _grp is a sub-or-tree-typed GroupID to put
  * at the second level of groups in the ACS tree
  * @return   the parent node of the _grp TreeeNode is under _rootNode,
  * or null if _grp is not a typed group
  */
  public static final DefaultMutableTreeNode getParentOfSubOrTreeTypedGroupID(
    DefaultMutableTreeNode _rootNode, ImmutableGroupID _grp) {

      if (!_grp.getEPType().startsWith("<byAcsCreator> "))
          return null;

      DefaultMutableTreeNode result = null;

                        // for testing and tracking
                        //System.out.println(
                          //      "ACSTreeUtilities.getParentOfSubOrTreeTypedGroupID(), _grp: "+
                            //    _grp.getName() );
                        //System.out.println("          _grp.getEPType(): "+
                          //      _grp.getEPType() );
                        //if (_grp.getEPType().length() > 15)
                          //  System.out.println("          node type from _grp.getEPType() : "+
                            //    _grp.getEPType().substring(15));
                        //System.out.println(_rootNode.getUserObject()+" children: " );

      // sets the rsult in testing groups tree roots and subACSs under _rootNode
      for (int j = 0; j < _rootNode.getChildCount(); j++) {

                        // for testing and tracking
                        //System.out.println("          child at j = "+j+" is: "+
                          //      ((DefaultMutableTreeNode)_rootNode.getChildAt(j)).getUserObject());

          if (_grp.getEPType().substring(15).indexOf(" ") == -1 &&
                  ("<"+_grp.getEPType().substring(15)+"_subacs>").equals(
                    ((DefaultMutableTreeNode)_rootNode.getChildAt(j)).getUserObject() )) {
                result = (DefaultMutableTreeNode) _rootNode.getChildAt(j);
                break;
          } // if
          else if (_grp.getEPType().substring(15).indexOf(" ") != -1 &&
                  ("<"+_grp.getEPType().substring(
                  15, 15 + _grp.getEPType().substring(15).indexOf(" "))+"_subacs>").equals(
                    ((DefaultMutableTreeNode)_rootNode.getChildAt(j)).getUserObject() )) {
                result = (DefaultMutableTreeNode) _rootNode.getChildAt(j);
                break;
          } // if

          else if (_grp.getEPType().substring(15).indexOf(" ") == -1 &&
                  ("<"+_grp.getEPType().substring(15)+"_tree>").equals(
                    ((DefaultMutableTreeNode)_rootNode.getChildAt(j)).getUserObject() )) {
                result = (DefaultMutableTreeNode) _rootNode.getChildAt(j);
                break;
          } // if
          else if (_grp.getEPType().substring(15).indexOf(" ") != -1 &&
                  ("<"+_grp.getEPType().substring(
                  15, 15 + _grp.getEPType().substring(15).indexOf(" "))+"_tree>").equals(
                    ((DefaultMutableTreeNode)_rootNode.getChildAt(j)).getUserObject() )) {
                result = (DefaultMutableTreeNode) _rootNode.getChildAt(j);
                break;
          } // if

      } // for j

                        // for testing and tracking
                        //if (result != null)
                          //  System.out.println("         result : "+result.getUserObject());
                        //else
                          //  System.out.println("         result is null. ");

      return result;
  } // getCurrentParentOfSubOrTreeTypedGroupID()

  /**
  * Gets the parent node of a VirtualFolder in a tree,
  * in the explorer or another ACS tree. 
  * Called by ACSTreeBaseListenerImpl.changeOneEPVirtualFolder(),
  * changeOneResourceVirtualFolder() and changeOneBaseObjectInFolder().
  * @param _rootNode root of one of the two VirtualFolders tree in an ACS tree
  * @param _vf to put in or to remove from the VirtualFolder tree
  * @return the parent node of the _vf TreeeNode under _rootNode, or null
  */
  public static final DefaultMutableTreeNode getCurrentParentOfVirtualFolderInTree(
    DefaultMutableTreeNode _rootNode, ImmutableVirtualFolder _vf) {

      DefaultMutableTreeNode result = null;

      if (_vf.getEorParent() == null) return null;

                            // for testing and tracking
                            //System.out.println(
                              //      "ACSTreeUtilities.getCurrentParentOfVirtualFolderInTree(), _vf: "+
                                //    _vf.getName() );
                            //System.out.println(_rootNode.getUserObject()+" children: " );

      // sets the result under _rootNode
      for (Enumeration e = _rootNode.preorderEnumeration(); e.hasMoreElements();) {

          result = (DefaultMutableTreeNode)e.nextElement();

          if (result.getUserObject().equals( ACSFactoryUtilityImpl.EPVFOLDERS))
              continue;
          if (result.getUserObject().equals( ACSFactoryUtilityImpl.RESVFOLDERS))
              continue;

                            // for testing and tracking
                            //System.out.println("---------- child: "+
                              //      result.getUserObject());
                            //System.out.println("---------- child class: "+
                              //      result.getUserObject().
                                //    getClass());

             // result may be for instance a ResourceMySQL as member of a VirtualFolder
          if (result.getUserObject().equals(_vf.getEorParent())) {
                break;
          } // if

      } // for e

                        // for testing and tracking
                        //if (result != null)
                          //  System.out.println("         result : "+result.getUserObject());
                        //else
                          //  System.out.println("         result is null. ");

      return result;
  } // getCurrentParentOfVirtualFolderInTree()


  /**
  * Builds up the member nodes for a virtual folder. Members are not children.
  * Called by ACSTree.createOwnResourceVirtualFolderGraphicalTree(),
  * createOwnEligiblePartyVirtualFolderGraphicalTree(), and
  * ACSTreeListenerImpl.changeOneResource/EPVirtualFolder().
  * @param  _treeModel which owns _node; updated by this method
  * @param  _node has a VirtualFolder as object, which may have members, and _node
  * has no child for the moment
  * @return _treeModel updated with the VirtualFolder members under _node
  */
  public static final DefaultTreeModel buildUpVirtualFolderMembers(
    DefaultTreeModel _treeModel, DefaultMutableTreeNode _node) {

    DefaultMutableTreeNode    node = null;
    VirtualFolder             vf = (VirtualFolder)_node.getUserObject();

    // sets each member under _node
    for (Object obj : vf.getEorM_BaseObjects().values()) {

                // for testing
                //System.out.println("member: "+ obj.getName() );

        node = new DefaultMutableTreeNode(obj, false);
        _treeModel.insertNodeInto(node, _node, node.getChildCount());
    } // for obj

    node = null;
    vf = null;
    return _treeModel;
  } // buildUpVirtualFolderElements()

  /**
  * Builds up a subtree for a virtual folder, including both the children abd the members.
  * It is a top-down searching, from the first to the last member of each node.
  * It is rather complex because two trees and a list are managed: <P>
  * - the virtual folder subtree from ARoad0.gBase, <P>
  * - the TreeNode tree from java.swing, <P>
  * - the virtual folder members as a list from ARoad0.gBase. <P>
  * Like in ACSTree.createOwnResourceGraphicalTree(),
  * one virtual folder cannot be associated to several TreeNodes in the tree.
  * Called by ACSTree.createOwnResourceVirtualFolderGraphicalTree() and
  * createOwnEligiblePartyVirtualFolderGraphicalTree(). <P>
  * Calls setGraphicalVirtualFolderChildren() and buildUpVirtualFolderMembers()
  * which update acsTreeModel__. This method should not be called if _node has no child
  * (a child is not a folder element).
  * @param  _acs which owns _vf
  * @param  _treeModel which owns _node; updated by this method
  * @param  _node having children to update
  * @param  _vf in _acs, user object of _node in _treeModel, and without parent
  * @return _treeModel updated with a subtree of nodes under _node
  */
  public static final DefaultTreeModel buildUpVirtualFolderTree(ACS _acs,
    DefaultTreeModel _treeModel, DefaultMutableTreeNode _node, 
    ImmutableVirtualFolder _vf) {

    acsTreeModel__ = _treeModel;
    currentACSNode__ = _node;

    // never null
    ImmutableVirtualFolder[] l_firstChildren = (ImmutableVirtualFolder[]) _vf.getEorL_Children();
    // never null - this method may update currentACSNode__
    boolean[]                l_hasVFchildren = setGraphicalVirtualFolderChildren(_vf);

    // count of folders with children
    int                     count = 0;
    ImmutableVirtualFolder  vf;
    ImmutableVirtualFolder  currentVF;
    ImmutableVirtualFolder  fol;
    DefaultMutableTreeNode  node;
    boolean[]               l_isNode;
    boolean                 hasNonVisibleNode = true; // true when currentVF has a child that
                                           // is a VF with children to create in the tree.
    boolean                 nodeFound = false;
    ImmutableVirtualFolder  parent = null;
    int                     index = 0;

    // a general loop on all the virtual folders which are _node children (not members)
    for (int k = 0; k < l_firstChildren.length; k++) {

        if (!l_hasVFchildren[k]) continue; // no child to process
        currentVF = l_firstChildren[k];

                                    // for testing
                                    //System.out.println("ACSTreeUtilities.buildUpVirtualFolderTree(), "+
                                       //     "start of the k iteration for currentVF "+currentVF.getNickName() );

          // at k == 0, currentACSNode__ is associated to vf,
            // while it has to be associated to currentVF in all cases.
            // The TreeNode of currentVF is searched.

                                    // for testing
                                    //System.out.println("ACSTreeUtilities.buildUpVirtualFolderTree(), "+
                                      //      "just before the j iteration with currentVF: "+
                                        //    currentVF.getNickName()+
                                          //  ", with currentACSNode__ to update: "+
                                            //currentACSNode__.getUserObject()+
                                            //", and currentACSNode__ number of children: "+
                                            //currentACSNode__.getChildCount());

       
        for (int j = 0 ; j < _node.getChildCount() ; j++) {

                // gets the VF
            fol = (VirtualFolder)((DefaultMutableTreeNode)_node.
                  getChildAt( j ) ).getUserObject();

                                    // for testing
                                    //System.out.println(
                                      //      "j iteration for fol: "+fol.getNickName() );


            if ( fol == currentVF) {
                currentACSNode__ = (DefaultMutableTreeNode)_node.getChildAt( j );
                break;
            } // if
        } // for j

        count++;
        hasNonVisibleNode = true; // true when currentVF has a child c which
                                               // is a node with children not still
                                               // created in the tree.
        nodeFound = false;
        // this init is just to have parent.getEorParent() != null for starting the while loop
        parent = currentVF;
        index = 0;

                                    // for testing
                                    //System.out.println("ACSTreeUtilities.buildUpVirtualFolderTree(), "+
                                      //      "just before the while iteration"+
                                        //    ", with parent.getEorParent(): "+
                                          //  parent.getEorParent()+
                                            //", with currentACSNode__: "+
                                            //currentACSNode__.getUserObject());

        // In this while loop, currentVF is used, in a first step, to create
        // all children in all the first nodes at each level of the tree.
        // Then, a second step searches for where there is still one node
        // with children to create (there are two cases), and sets
        // currentVF to this directory. The first step is then run one more time.

        // we always have :     in java.Tree :          in ARoad.gBase :
        //                          no variable             parent
        //                              |                       |
        //                              - currentACSNode__      - currentVF

        while (hasNonVisibleNode && parent.getEorParent() != null) {

                           // FIRST STEP TO CREATE ALL THE FIRST-POSITION CHILDREN
                           // IN THE WHILE LOOP

          hasNonVisibleNode = false;

            // adds the currentVF members before the currentVF children,
            // contrary of Directories
          buildUpVirtualFolderMembers(acsTreeModel__, currentACSNode__);

          // updates currentACSNode__ to set it on the first currentVF child with whildren.
          // l_isNode is a boolean array that indicates where there are
          // with-children folders in currentVF. The array may be null.
          l_isNode = setGraphicalVirtualFolderChildren(currentVF);

          // with the while loop at the first step, goes down to the first node
          // of every node, and creates in the tree
          // all the intermediate nodes with their children
          if (l_isNode != null) {
            nodeFound = false;
            for (int i = 0 ; i < l_isNode.length ; i++) {
              if (l_isNode[i] && !nodeFound) {
                hasNonVisibleNode = true;
                nodeFound = true;
                // jump to the upper level
                currentVF = (VirtualFolder)currentVF.getEorL_Children()[i];
                // currentACSNode__ is always associated to currentVF
                // by setGraphicalVirtualFolderChildren(), with the same algorithm
              }
            } // for i
          } // if (l_isNode != null)

          // If hasNonVisibleNode is true,
          // then currentVF is a node to explore, so continue in the first step.

                           // SECOND STEP

          // if not, currentVF is set to the next node of its parent (the next sibling),
          // or to the next sibling of its parent, or more until the root is get
          if (!hasNonVisibleNode) {

              parent = (VirtualFolder)currentVF.getEorParent();

              // if currentVf is the root of the tree, stop the search
              if (parent == null) break;

              // index of currentVF as a child of its parent
              index = Arrays.binarySearch(parent.getEorL_Children(), currentVF);

              if (index < 0 ) {
                throw new InternalError("in ACSTreeUtilities.buildUpVirtualFolderTree(), "+
                "incorrect synchronization for the searching of currentVF: "+currentVF+
                  " in parent: "+parent);
              }

              if (index == parent.getEorL_Children().length - 1) {

                // we are at the end of the parent siblings list.
                // parent has to be set to its parent before the next second while loop
                if (parent.getEorParent() != null) {
                  index = Arrays.binarySearch(((VirtualFolder)parent.getEorParent()).
                          getEorL_Children(), parent);
                  parent = (VirtualFolder)parent.getEorParent();
                  currentACSNode__ = (DefaultMutableTreeNode)currentACSNode__.getParent();
                }
                else {
                  // stop the while loop
                  parent = null;
                }
              } // if (index == parent.getEorL_Children().length - 1)

              while (parent != null && !hasNonVisibleNode) {

                                        // FIRST CASE:

                // searching for the next node through the parent children
                for (int i = index + 1 ; i < parent.getEorL_Children().length ; i++) {

                  if (parent.getEorL_Children()[i].getEorL_Children() != null) {

                                    // for testing
                                    //System.out.println("in ACSTreeUtilities.buildUpVirtualFolderTree(), "+
                                      //      "detection of a new directory" );

                    // sets currentDir before a return at the top of the first while loop
                    currentVF = (VirtualFolder)parent.getEorL_Children()[i];

                    // searching for the TreeNode associated to currentVF.
                    // The correct j is not always the current i, because
                    // the orders are not the same
                    fol = null;
                    node = (DefaultMutableTreeNode)currentACSNode__.getParent();

                                            // for testing
                                            //System.out.println("in ACSTreeUtilities.buildUpVirtualFolderTree(), "+
                                              //      "currentVF :" + currentVF.getName()+"\n"+
                                                //    "-- currentVF parent for children iteration is :" +
                                                  //  node.getUserObject()+" with children number: "+
                                                    //node.getChildCount());


                    for (int j = 0 ; j < node.getChildCount() ; j++) {

                            // gets the VF
                        fol = (VirtualFolder)((DefaultMutableTreeNode)node.
                              getChildAt( j ) ).getUserObject();

                                            //System.out.println("iterate on a VF associated "+
                                              //      "to a DefaultMutableTreeNode under "+
                                                //    node + ":" + fol);              //for test
                                            // for testing
                                            //System.out.println("in ACSTreeUtilities.buildUpVirtualFolderTree(), "+
                                              //  "the loop on parent.getEorL_Children() \n gets "+
                                                //"fol: "+fol+"\n and current node: " + currentVF);

                        if ( fol == currentVF) {

                                            // for testing
                                            //System.out.println("in ACSTreeUtilities.buildUpVirtualFolderTree(), "+
                                              //  "end of the loop on parent.getEorL_Children(), "+
                                                //"since currentACSNode__ is updated.");


                            currentACSNode__ = (DefaultMutableTreeNode)node.getChildAt( j );
                            break;
                        } // if
                    } // for j
                    if (currentVF != currentACSNode__.getUserObject()) {
                        System.out.println("in ACSTreeUtilities.buildUpVirtualFolderTree(), "+
                                "currentVF: " + currentVF + "; and wrong search of the TreeNode:" +
                                currentACSNode__.getUserObject());
                        throw new InternalError("wrong search of the TreeNode "+
                        "associated to currentVF");
                    }
                    fol = null;

                    // sets the boolean for repeating the first step
                    hasNonVisibleNode = true;

                  } // if (parent.getEorL_Children()[i] instanceof Node)

                  if(hasNonVisibleNode) break;
                } // for i

                                    // SECOND CASE:

                // if the first case search is negative,
                // goes down of one level in the tree,
                // and returns to the first case to find the next directory
                if (!hasNonVisibleNode) {

                  // parent has to be set to its parent
                  // and index is set for a new search
                  if (parent.getEorParent() != null) {
                    index = Arrays.binarySearch(((VirtualFolder)parent.getEorParent()).
                      getEorL_Children(), parent);
                    parent = (VirtualFolder)parent.getEorParent();
                    currentACSNode__ = (DefaultMutableTreeNode)currentACSNode__.
                      getParent();
                  } // returns to the first case
                  else parent = null; // stop all the loops

                } // if (!hasNonVisibleNode)

              } // while parent != null && !hasNonVisibleNode

          } // if (!hasNonVisibleNode)

        } // while (hasNonVisibleNode) && parent.getEorParent() != null

    } // for k on _vf children

            // for testing and tracking
            //System.out.println("ACSTreeUtilities.buildUpVirtualFolderTree() end - "+
              //"the number of tree root children and members is: " +_node.getChildCount() );

    currentVF = null;
    parent = null;
    fol = null;
    node = null;
    l_isNode = null;

    l_firstChildren = null;

    return _treeModel;
  } // buildUpVirtualFolderTree()

  //
  // --------------- PROTECTED METHODS  -------------------------------
  //
  

  /**
  * Says if a node in a subtree belongs to the resources tree of an acs,
  * in the explorer or another tree.
  * Called by ACSTree.createResourceGraphicalTree(), and
  * TreeManager.analyzeSelectedNodeInExplorer().
  * @param _node in an ACS tree. Never null. May be the RESOURCES node.
  * @return true if the node is in a resources tree
  */
  protected static final boolean isInResourceTree(DefaultMutableTreeNode _node) {
     
      boolean result = false;

      Object[] l_path = _node.getUserObjectPath();
      for (int i = 0; i < l_path.length; i++) {  
        if (l_path[i].equals( ACSTree.M_ACS_TREE_UPPER_NODES.get("RESOURCES"))) {
            result = true;
            break;
        } else if (i > 3) break; // i = 3 in the explorer for 'RESOURCES' node
      } // for i
      return result;
  } // isInResourceTree()    
  
  //
  // --------------- PRIVATE STATIC METHODS ---------------------------
  //

  /**
   * Creates or analyzes the non-null and non-empty members as TreeNodes
   * in a graphical tree, to return the members which have no associated nodes
   * in the ACS tree. This method may be called several times, and it detects if
   * it is the first call or not, to adapt the process. 
   * Called by the buildUpGroupTree().
   * At the return, the static variable currentACSNode__ is the TreeNode
   * associated to the first GroupID in this loop which has not an associated node,
   * if it exists, or else, it is associated to _grp (that is, it is not changed).
   * @param _grp already created as graphical tree node in acsTreeModel__
   * @return boolean[] that indicates where there is a groupID
   * as _grp member which has no associated node.
   * The array index is that of _grp.getEorL_Members(),
   * not that of the associated currentACSNode__, when null members are removed.
   */
  private static final boolean[] setGraphicalGroupIDMembers(GroupID _grp) {

    DefaultMutableTreeNode      node;
    DefaultMutableTreeNode      grpNode = currentACSNode__;
    ImmutableGroupIDMember[]    l_members;
    short                       shiftingBack = 0;
    boolean                     groupFound = false;
    boolean[]                   l_isNoMemberNodesGroup;
    boolean                     afterFirstCall = false;

                  // for testing and tracking
                  //System.out.println(" ");
                  //System.out.println("in ACSTreeUtilities.setGraphicalGroupIDMembers()"+
                    //", _grp :" + _grp+", with currentACSNode__ "+currentACSNode__.getUserObject()+
                    //" having "+currentACSNode__.getChildCount()+" children.");
      
    l_members = _grp.getEorL_Members();
    if (l_members.length == 0) {
        l_members = null;
        return new boolean[0];
    }
    l_isNoMemberNodesGroup = new boolean[l_members.length];

    // set the variable for telling it is not the first processing for _grp
    if (currentACSNode__.getChildCount() != 0 ) {

                  // for testing and tracking
                  //System.out.println("in ACSTreeUtilities.setGraphicalGroupIDMembers()"+
                    //", detection of 'afterFirstCall'.");
        
        afterFirstCall = true;
    }

                  // for testing and tracking  
                  //System.out.println("the members are:");
                  //for (int i = 0; i < l_members.length; i++) {
                    //System.out.println(l_members[i]);    
                  //}
    
    for (int i = 0 ; i < l_members.length ; i++) {

              // for testing and tracking  
              //System.out.println("start of processing for i:"+i+" and shiftingBack: "+shiftingBack);
        
      if (l_members[i] == null || l_members[i].equals(GroupIDMemberImpl.EMPTY_INSTANCE)) {
        shiftingBack++;
        continue;
      }
      
      if (l_members[i] instanceof GroupID) {
        // if the node exists with some children and if the group has members,
        // l_isNoMemberNodesGroup[i] is not set to true
        // and the next member is analyzed.
        if (afterFirstCall && 
          ( ((GroupID)l_members[i]).getEorL_Members().length == 0 ||
          grpNode.getChildAt(i - shiftingBack).getChildCount() != 0) ) {

                  // for testing and tracking
                  //System.out.println("in ACSTreeUtilities.setGraphicalGroupIDMembers()"+
                    //", detection after first call of a processed group with all its children nodes,"+
                    //" for "+l_members[i]);
              
            continue;
        }
                  //System.out.println("before adding "+l_members[i]+
                          //", grpNode "+grpNode.getUserObject()+
                          //" has "+grpNode.getChildCount()+" children.");
        
        // create a mutable TreeNode for a groupID that may already has one
        // or more mutable TreeNodes associated to
        node = new DefaultMutableTreeNode(l_members[i], true);
        acsTreeModel__.insertNodeInto(node, grpNode, i - shiftingBack);
        // set the boolean returned to the caller
        l_isNoMemberNodesGroup[i] = true;
        
        if (!groupFound) {
          // for setting up currentACSNode__ to the TreeNode associated to
          // the first created GroupID in this loop.
          // else, keep the previous currentACSNode__ value 
          // for the buildUpGroupTree() second step.
          groupFound = true;
          
                      // for testing and tracking
                      //System.out.println("ACSTreeUtilities.setGraphicalGroupIDMembers()"+
                        //", groupFound true.");        
        
          currentACSNode__ = node;          

                      // for testing and tracking
                      //System.out.println("in ACSTreeUtilities.setGraphicalGroupIDMembers()"+
                        //", set l_isGroup to true for "+i+" with the group "+epArray[i]);
          
        }
      } // else if (l_members[i] instanceof GroupID)

      else {
          // if afterFirstCall true, the right leaf is already there
        if (afterFirstCall) continue; 
        
        l_isNoMemberNodesGroup[i] = false;
        node = new DefaultMutableTreeNode(l_members[i], false);// can't have members
        acsTreeModel__.insertNodeInto(node, grpNode, i - shiftingBack);
      }

    } // for i
    
                  // for testing and tracking
                  //System.out.println("ACSTreeUtilities.setGraphicalGroupIDMembers()"+
                    //", currentACSNode__ at the end :" + currentACSNode__.getUserObject());
    
    l_members = null;
    node = null;
    grpNode = null;

    return l_isNoMemberNodesGroup;
  } // setGraphicalGroupIDMembers()


 /**
  * Creates the non-null children as TreeNodes in a graphical tree.
  * Called by buildUpVirtualFolderTree(), with strong links
  * through the updating of acsTreeModel__ and currentACSNode__.
  * Caution: at the calling, acsTreeModel__ has to be the right model,
  * currentACSNode__ has to be the node of _vf, and at the end of the method,
  * currentACSNode__ is the node of the first child having children, if any.
  * @param _vf already created as graphical tree node in acsTreeModel__
  * @return boolean[] that indicates where there are folders with children
  * as _dir children. The array index is from _vf.getEorL_Children(),
  * not from the associated currentACSNode__,
  * where null children are removed. Is null if there is no children.
  */
  private static final boolean[] setGraphicalVirtualFolderChildren(
          ImmutableVirtualFolder _vf) {

    DefaultMutableTreeNode  internalTreeNode = null;
    DefaultMutableTreeNode  leaf;
    ImmutableVirtualFolder[] l_childrenArray = (ImmutableVirtualFolder[]) _vf.getEorL_Children();
    short                   shiftingBack = 0;
    boolean                 nodeFound = false;
    boolean[]               l_result;
    boolean                 emptyNode = true;

    // check _vf
    if (_vf == null || l_childrenArray == null ) {

            // for testing and tracking
            //System.out.println("In ACSTree.setGraphicalVirtualFolderChildren() call "+
            //  "for  the virtual folder:" + _vf+". There is no children.");

        return null;
    } // if

    internalTreeNode = TreeManager.EMPTY_TREE_NODE;

                    // for testing and tracking
                    //System.out.println("ACSTreeUtilities.setGraphicalVirtualFolderChildren() call -"+
                      //" for "+_vf.getName()+", the children size is:" +
                      //_vf.getEorL_Children().length);

    // checkups _vf children
    for (int i = 0 ; i < l_childrenArray.length ; i++) {
      if (l_childrenArray[i] != null ||
        !l_childrenArray[i].equals(VirtualFolderImpl.EMPTY_INSTANCE)) {
          emptyNode = false;
          break;
      } // if
    } // for

    if (!emptyNode) l_result = new boolean[l_childrenArray.length];
    else {
                      // for testing and tracking
                      //System.out.println("ACSTreeUtilities.setGraphicalVirtualFolderChildren(), "+
                        //      "one setGraphicalVirtualFolderChildren() call "+
                        //"for  the node:" + _vf.getName()+". All the children are empty.");

      internalTreeNode = null;
      l_childrenArray = null;
      return null;
    } // else

    //  builds up the tree without the null children
    for (int i = 0 ; i < l_childrenArray.length ; i++) {

      if (l_childrenArray[i] == null) {
          shiftingBack++;
          continue;
      }
    
      // the virtual folder may have no children, but members are always possible
      leaf = new DefaultMutableTreeNode(l_childrenArray[i], true);
      acsTreeModel__.insertNodeInto(leaf, currentACSNode__, i - shiftingBack);

        // adds the members before the children,
        // contrary of Directories
      buildUpVirtualFolderMembers(acsTreeModel__, leaf);

      if (l_childrenArray[i].getEorL_Children() != null) {
            // set the boolean returned to the caller
        l_result[i] = true;
        if (!nodeFound) {
            // for setting currentACSNode__ to the first created TreeNode  in this loop
          internalTreeNode = leaf;
          nodeFound = true;
        } // if
      } // if
      else {
        l_result[i] = false;
      }

    } // for i

    // sets currentACSNode__ to be used by buildUpOwnVirtualFolderTree()
    if (nodeFound) currentACSNode__ = internalTreeNode;

                            // for testing and tracking
                            //if (nodeFound)
                              //  System.out.println(
                                //    "ACSTreeUtilities.setGraphicalVirtualFolderChildren() end -"+
                                 //  " the result is normal, the size is: " + l_result.length+
                                 //   ", and the new currentACSNode__ is for "+
                                 //   currentACSNode__.getUserObject());
                            //else
                              //  System.out.println(
                                //    "ACSTreeUtilities.setGraphicalVirtualFolderChildren() end -"+
                                  //  " the result is normal, the size is: " + l_result.length+
                                    //", and currentACSNode__ is unchanged. ");

    l_childrenArray = null;
    internalTreeNode = null;
    leaf = null;

    return l_result;
  } // setGraphicalVirtualFolderChildren()

} // class ACSTreeUtilities