//Title:        Access Road
//Version:      0.6.5
//Copyright:    Copyright 2002, 2004, 2007, 2008, 2009 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.Pattern.MVC_Controller;

import java.beans.PropertyChangeListener;

/**
 * This interface is responsible for listening the events
 * fired by one main BaseObject (the main source) as ACSRun
 * and ViewInBase -and also the events fired by the main source objects-
 * for updating the unique graphic (tree or diagram) which is associated
 * to this BaseListener and which displays the main source objects.<P>
 * Each event source is registered only one time.
 * The main source opening and closing operations are not implied,
 * since these operations are handled in Gui1/Gui2 and the gDMak classes
 * ActionClose and ActionOpen.
 */
public interface BaseListener extends MVC_Controller, PropertyChangeListener {

 /**
 * Get the main event source associated to the instance
 * in its construction. It is an ACS or a view.
 * See the <a href="{@docRoot}/copyright.html">Copyright</a>.
 * @return name of the main source for PropertyChangeEvents. May be null.
 */
  public ImmutableName getEorMainEventSource();

 /**
 * To listen or not the events in propertyChange().
 * @param _listen true to listen the events in this instance, through propertyChange()
 * @return true if the previous value was different to this argument
 */
  public boolean setListenEvents(boolean _listen);

 /**
 * Unregister the instance as a listener for all registered sources.
 */
  public void finalizeForUser();

 /**
 * HashCode of the instance. It is used by the graphic manager to remove
 * a BaseListener.
 * @return int
 */
  public int hashCode();

} // BaseListener