//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2002, 2004, 2007-2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import java.awt.Component;
import javax.swing.Icon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;

import ARoad0.gBaseInterface.*;
import ARoad0.gBase.ACSFactoryUtilityImpl;

/**
 * This class is responsible for the look-an-feel setup into the explorer tree.
 */
public class ExplorerTreeCellRenderer extends CommonTreeCellRenderer {
    // variables
    private boolean     isACS = false;
    private boolean     isView = false;
    private boolean     isGroup = false;
    private boolean     isUser = false;
    private boolean     isActor = false;
    private boolean     isAcsNode = false;
    private boolean     isDirectoryInView = false; // only for directories as leaves in views
    private boolean     isSimpleResource = false;
    private boolean     isSubSystem = false; // for '_subacs>' nodes
    private boolean     isRight = false;
    private boolean     isVirtualFolder = false;
    private boolean     isTypedPrivilege = false;
    private boolean     isLinkedPrivilege = false;
    
    private boolean     toChange = false;
    
    // constructors

  /**
   * Public constructor. No work.
  */
  public ExplorerTreeCellRenderer() {

  } // ExplorerTreeCellRenderer()

    //
    // --------------- METHODS ---------------------
    //

 /**
 * Overrides the super method.
 * Called by super.getTreeCellRendererComponent().
 * @return the icon to display for ACS, view, group nodes
 */
  public Icon getClosedIcon() {
    if (toChange) {
      if (isACS) return acsIcon_; 
      else if (isAcsNode) return acsNodeIcon;
      else if (isView) return viewIcon;
      else if (isSubSystem) return subsystemIcon;
      else if (isVirtualFolder) return virtualFolderIcon;
      else return groupIcon;
    }
    else return super.getClosedIcon();
  } // getClosedIcon()

 /**
 * Overrides the super method.
 * Called by super.getTreeCellRendererComponent().
 * @return the icon to display for ACS, view, group nodes
 */
  public Icon getOpenIcon() {
    if (toChange) {
      if (isACS) return acsIcon_;
      else if (isAcsNode) return acsNodeIcon;
      else if (isView) return viewIcon;
      else if (isSubSystem) return subsystemIcon;
      else if (isVirtualFolder) return virtualFolderIcon;
      else return groupIcon;
    }
    else return super.getOpenIcon();
  } // getOpenIcon()

 /**
 * Overrides the super method. ACS and GroupID are leaves in the views.
 * Called by super.getTreeCellRendererComponent().
 * @return the icon to display the nodes
 */
  public Icon getLeafIcon() {
    if (toChange) {
      if (isSimpleResource) return sResIcon;
      else if (isACS) return acsIcon_;
      else if (isAcsNode) return acsNodeIcon;      
      else if (isView) return viewIcon; 
      else if (isSubSystem) return subsystemIcon;
      else if (isVirtualFolder) return virtualFolderIcon;
      else if (isGroup) return groupIcon;
      else if (isUser) return userIcon;
      else if (isActor) return actorIcon;
      else if (isDirectoryInView) return super.getClosedIcon();
      else if (isRight) return rightIcon;
      else if (isTypedPrivilege) return typedPrivilegeIcon;
      else if (isLinkedPrivilege) return linkedPrivilegeIcon;
      else return aclIcon;
    }
    // only for empty '<ffff_subacs>' under 'AclEntries'
    else return super.getClosedIcon();
  } // getLeafIcon()

/**
* Overrides the DefaultTreeCellRenderer method, for setting the right icon and calling
* this method at the end.
* Configures the renderer based on the passed-in components.
* The value is set from messaging the tree which ultimately invokes
* <code>toString</code> on <code>_value</code>.
* The icon is set based on leaf and expanded.
* For the Privileges, the node object is the detailled name, and this method
* tests if it starts with 'TYPED' to select the relevant icon for typed privileges.
* @param _tree to render
* @param _value DefaultMutableTreeNode
* @param _sel see the DefaultTreeCellRenderer method
* @param _expanded see the DefaultTreeCellRenderer method
* @param _leaf see the DefaultTreeCellRenderer method
* @param _row see the DefaultTreeCellRenderer method
* @param _hasFocus see the DefaultTreeCellRenderer method
* @return the Component
*/
  public final Component getTreeCellRendererComponent(JTree _tree, Object _value,
                                                boolean _sel,
                                                boolean _expanded,
                                                boolean _leaf, int _row,
                                                boolean _hasFocus) {

            // for testing and tracking
            //System.out.println("call of ExplorerTreeCellRenderer."+
            //  "getTreeCellRendererComponent() with the user object class"+
            //  " in the node: "+((DefaultMutableTreeNode)_value).getUserObject().getClass());
      
            // AclEntries and Privileges are displayed as Strings,
            // so they must be before SimpleResource,
            // and it is the default icon for changing
    if ( (((DefaultMutableTreeNode)_value).getParent() != null &&
            ((DefaultMutableTreeNode)((DefaultMutableTreeNode)_value).getParent()).
            getUserObject().equals(ACSFactoryUtilityImpl.ACLENTRIES) &&
            !((String)((DefaultMutableTreeNode)_value).getUserObject()).endsWith("_subacs>") ) ||
            
         (((DefaultMutableTreeNode)_value).getParent() != null &&
            ((DefaultMutableTreeNode)_value).getParent().getParent() != null &&
            ((DefaultMutableTreeNode)((DefaultMutableTreeNode)_value).getParent().getParent()).
            getUserObject().equals(ACSFactoryUtilityImpl.ACLENTRIES) )   ) {
      toChange = true;     
      isSimpleResource = false;
      isACS = false;
      isAcsNode = false;
      isView = false;
      isGroup = false;
      isUser = false; 
      isActor = false;
      isDirectoryInView = false;      
      isRight = false;
      isSubSystem = false;
      isVirtualFolder = false;
      isTypedPrivilege = false;
      isLinkedPrivilege = false;
    } // if
    else if ( (((DefaultMutableTreeNode)_value).getParent() != null &&
            ((DefaultMutableTreeNode)((DefaultMutableTreeNode)_value).getParent()).
            getUserObject().equals(ACSFactoryUtilityImpl.PRIVILEGES) )) {
      toChange = true;
      isSimpleResource = false;
      isACS = false;
      isAcsNode = false;
      isView = false;
      isSubSystem = false;
      isVirtualFolder = false;
      isGroup = false;
      isUser = false;
      isActor = false;
      isDirectoryInView = false;
      isRight = false;
      if (((String)((DefaultMutableTreeNode)_value).getUserObject()).
              startsWith("TYPED")) {
          isTypedPrivilege = true;
          isLinkedPrivilege = false;
      }
      else {
          isTypedPrivilege = false;
          isLinkedPrivilege = true;
      }
    }
    // tree upper nodes are displayed as Strings, so 
    // they must be tested before SimpleResource
    else if (ACSTree.M_ACS_TREE_UPPER_NODES.containsValue(
            ((DefaultMutableTreeNode)_value).getUserObject() ) ) {
      toChange = true;
      isSimpleResource = false;
      isACS = false;
      isAcsNode = true;
    }
    else if (TreeManager.M_EXPLORER_UPPER_NODES.containsValue(
            ((DefaultMutableTreeNode)_value).getUserObject() ) ) {
        toChange = false;
    }
    // to-display typed nodes are displayed as Strings, so
    // they must be tested before SimpleResource
    else if (        
             ((DefaultMutableTreeNode)_value).getPath().length == 5 &&
            ((DefaultMutableTreeNode)_value).getUserObject() instanceof String &&
            ((String)((DefaultMutableTreeNode)_value).getUserObject()).endsWith("_subacs>") ) {
      toChange = true;
      isSimpleResource = false;
      isACS = false;
      isAcsNode = false;
      isView = false;
      isSubSystem = true;
        /*
      toChange = true;
      isACS = false;
      isAcsNode = false;
      isView = false;
      isGroup = false;
      isUser = false; 
      isActor = false;
      isDirectoryInView = false;      
      isRight = false;
      isSubSystem = true;
      //isVirtualFolder = false;
      //isTypedPrivilege = false;
      //isLinkedPrivilege = false;
       */
    }
    // Resource and Directory under the Resources tree
    else if (((DefaultMutableTreeNode)_value).getUserObject() instanceof String &&
            ((DefaultMutableTreeNode)_value).isLeaf() ) {
      toChange = true;
      isSimpleResource = true;
    }
    // Directory under a ViewInBase
    else if ( ((DefaultMutableTreeNode)_value).getUserObject() instanceof Directory ) {
      toChange = true;
      isSimpleResource = false;
      isACS = false;
      isAcsNode = false;
      isView = false;
      isSubSystem = false;
      isVirtualFolder = false;
      isGroup = false;
      isUser = false;
      isActor = false;
      isDirectoryInView = true;
    }
    // Resource under a ViewInBase
    else if (((DefaultMutableTreeNode)_value).getUserObject() instanceof Resource) {
      toChange = true;
      isSimpleResource = true;
    }
    else if (((DefaultMutableTreeNode)_value).getUserObject() instanceof ImmutableACS) {
      toChange = true;
      isSimpleResource = false;
      isACS = true;

      acsIcon_ = ((ImmutableACS)((DefaultMutableTreeNode)_value).getUserObject()).getEorIcon();

      // the default value in CommonTreeCellRenderer is never used
      if (acsIcon_ == null)
          acsIcon_ = ACS_ICON;
      
    }
    else if (((DefaultMutableTreeNode)_value).getUserObject() instanceof ViewInBase) {
      toChange = true;
      isSimpleResource = false;
      isACS = false;
      isAcsNode = false;
      isView = true;
      //isSubSystem = false;
    }
    else if ( ((DefaultMutableTreeNode)_value).getUserObject() instanceof VirtualFolder ) {
      toChange = true;
      isSimpleResource = false;
      isACS = false;
      isAcsNode = false;
      isView = false;
      isSubSystem = false;
      isVirtualFolder = true;
    }
    else if (((DefaultMutableTreeNode)_value).getUserObject() instanceof GroupID) {        
      toChange = true;
      isSimpleResource = false;
      isACS = false;
      isAcsNode = false;      
      isView = false;
      isSubSystem = false;
      isVirtualFolder = false;
      isGroup = true;      
    }
    else if (((DefaultMutableTreeNode)_value).getUserObject() instanceof UserID) {        
      toChange = true;      
      isSimpleResource = false;
      isACS = false;
      isAcsNode = false;
      isView = false;
      isSubSystem = false;
      isVirtualFolder = false;
      isGroup = false;
      isUser = true;
    }
    else if ( ((DefaultMutableTreeNode)_value).getUserObject() instanceof StringRight ) {
      toChange = true;
      isSimpleResource = false;
      isACS = false;
      isAcsNode = false;
      isView = false;
      isSubSystem = false;
      isVirtualFolder = false;
      isGroup = false;
      isUser = false;
      isActor = false;
      isDirectoryInView = false;
      isRight = true;
    }
    // Directory under a ViewInBase node
    else toChange = false; 

    return super.getTreeCellRendererComponent(_tree, _value,
                                                _sel,
                                                _expanded,
                                                _leaf, _row,
                                                _hasFocus);

  } // getTreeCellRendererComponent()

} // ExplorerTreeCellRenderer