//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard 
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import java.awt.*;
import java.awt.geom.*;

import ARoad0.Gui1.*;
import ARoad0.gBaseInterface.Actor;
import ARoad0.gBaseInterface.BaseObject;


/**
* This class is responsible for managing the display of one Actor
* in a GraphicView.
*/
public class GraphicActor extends GraphicNode {
  // variables
  protected GraphicView         gView_;
  protected Actor               act_;
  protected static final int    PRIME = 1000003;
    // shape of a resource
  protected static final Shape  SHAPE = new GeneralPath(GeneralPath.WIND_EVEN_ODD, 10);

  public static final GraphicActor EMPTY_GRAPHIC_ACTOR = new GraphicActor();

  //
  // ----------------------------- constructors ---------------------------
  //

  /**
  * This empty constructor allows to draw an empty shape.
  * */
  public GraphicActor(  ) {

    ((GeneralPath)SHAPE).moveTo(NODE_SIZE+2, 2);
    ((GeneralPath)SHAPE).lineTo(NODE_SIZE+2, 3*(NODE_SIZE-2)/5);
    ((GeneralPath)SHAPE).lineTo(2, 3*(NODE_SIZE-2)/5);
        //((GeneralPath)SHAPE).lineTo(2, NODE_SIZE-2);
    ((GeneralPath)SHAPE).lineTo(2, 2);
    ((GeneralPath)SHAPE).closePath();

    setPreferredSize(new Dimension((int)getShape().getBounds2D().getWidth()+2,
      (int)getShape().getBounds2D().getHeight()));
    setMinimumSize(new Dimension((int)getShape().getBounds2D().getWidth()+2,
      (int)getShape().getBounds2D().getHeight()));
    setMaximumSize(new Dimension((int)getShape().getBounds2D().getWidth()+2,
      (int)getShape().getBounds2D().getHeight()));

  /*    addGraphicViewListener(new GraphicViewListener(  ) {
      public void graphicViewAdjusted(GraphicViewEvent e) {
        //
      }
    });
  */
    }

  /**
  * @param _gView GraphicView that manages this instance, as an External Object
  * Reference.
  * @param _act is the Actor associated to this instance, as an External Object
  * Reference.
  * */
  public GraphicActor(GraphicView _gView, Actor _act) {
    this();
    gView_ = _gView;
    act_ = _act;

/*    addMouseListener(new MouseAdapter(  ) {
      public void mousePressed(MouseEvent e) { spin(e); }
    });
    addMouseMotionListener(new MouseMotionAdapter(  ) {
      public void mouseDragged(MouseEvent e) { spin(e); }
    });
*/
  }

  /**
   * Is used by constructor GraphicActor(GraphicView, Resource)
   * for setting the size.
   * @return Shape drawn by the GraphicNode
   */
   public Shape getShape() {
    /*
    GeneralPath result = new GeneralPath(SHAPE);

    TextLayout name;
    String[] l_name = getDisplayableName();
    Font font = new Font("SansSerif", Font.TRUETYPE_FONT, 12);
    for ( int i = 0 ; i < l_name.length ; i++ ) {
      name = new TextLayout("   "+l_name[i],  font,
            new FontRenderContext(new AffineTransform(),true, false));
      result.append(name.getOutline(new AffineTransform()), false);
    } // for
    */
    return SHAPE;
   } // getShape()


  /**
   * @return the resource associated to the GraphicActor
   */
   public BaseObject getEorBaseObject() {
    return act_;
   } // getEorBaseObject()
   
  /**
   * For a GraphicNoThanView, call isReducedSource() to draw the shape in grey.
   * Override the GraphicNode method for drawing the shape
   * and the Actor name.
   * @param _g Graphics to be drawn.
   */
  public void paintComponent(Graphics _g) {

    Graphics2D g2 = (Graphics2D)_g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING ,
      RenderingHints.VALUE_ANTIALIAS_ON);
    GradientPaint gp = new GradientPaint(75, 75, Color.white, 95, 95, Color.blue, true);
    if (gView_ instanceof GraphicNoThanView && ((GraphicNoThanView)gView_).isReducedSource(act_))
        g2.setPaint(Color.LIGHT_GRAY);
    else
        g2.setPaint(gp);
    g2.setStroke(new BasicStroke(3) );

    if (act_ != null) {
      g2.draw(SHAPE);
      g2.setPaint(Color.blue);
      g2.setFont(BeamerManager.smallBoldUpFont);

      String[] l_name = getDisplayableName();
      String str;
      for ( int i = 0 ; i < l_name.length ; i++ ) {
          str = l_name[i].toUpperCase();
          g2.drawString(" "+str, 2, i*10 + (NODE_SIZE/5) ); //(2*NODE_SIZE / 5)
          if (i == 2) break;
      } // for (int i
      l_name = null;
      str = null;
    } // if
    g2 = null;
    gp = null;
    return;
  } // paintComponent()


  /**
   * For a GraphicNoThanView, call isReducedSource() to draw the shape.
   * Copy of paintComponent() with opposite colors.
   * @param _g Graphics to be drawn.
   */
  public void paintComponentForExport(Graphics _g) {

    Graphics2D g2 = (Graphics2D)_g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING ,
      RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setStroke(new BasicStroke(3) );
    GradientPaint gp = new GradientPaint(75, 75, Color.blue, 95, 95, Color.white, true);
    if (gView_ instanceof GraphicNoThanView && ((GraphicNoThanView)gView_).isReducedSource(act_))
        g2.setPaint(Color.WHITE);
    else
        g2.setPaint(gp);

    // set the composite rule
    //g2.setComposite(AlphaComposite.SrcIn);

    if (act_ != null) {
      g2.draw(SHAPE);
      g2.setPaint(Color.white);
      g2.setFont(BeamerManager.smallBoldUpFont);

      String[] l_name = getDisplayableName();
      String str;
      for ( int i = 0 ; i < l_name.length ; i++ ) {
          str = l_name[i].toUpperCase();
          g2.drawString(" "+str, 2, i*10 + (NODE_SIZE/5) ); //(2*NODE_SIZE / 5)
          if (i == 2) break;
      } // for (int i
      l_name = null;
      str = null;
    } // if
    g2 = null;
    gp = null;
    return;
  } // paintComponentForExport()

    /**
     * @return the hashcode value for this object, based on the resource.
     */
    public int hashCode() {
      int result = 0;

      if (act_ != null) {
        result = PRIME*result + act_.hashCode();
      }
      return result;
    } // hashCode()

  /**
   * @return String[] from act_.lastComponent(), cut in NAME_PART-size strings.
   */
   public String[] getDisplayableName() {
    String[] result = new String[0];
    if (act_ != null) {
      if (act_.getName().lastComponent().length() < NAME_PART)
        return new String[] {act_.getName().lastComponent()};
      else {

        int arraySize = 1 + ( act_.getName().lastComponent().length() -
          act_.getName().lastComponent().length()%NAME_PART )/NAME_PART;
        result = new String[arraySize];
        for (int i = 0 ; i < arraySize - 1 ; i++ ) {
          result[i] = act_.getName().lastComponent().
            substring(NAME_PART*i,NAME_PART*i+NAME_PART);
        } // for
        result[arraySize - 1] = act_.getName().lastComponent().substring(
          NAME_PART*(arraySize - 1),
          act_.getName().lastComponent().length());
      } // else

    } // if (act_ != null

    return result;
   } // getDisplayableName()

} // GraphicActor
