//Title:        Access Road
//Version:      0.6.0
//Copyright:    Copyright 2002, 2004, 2007, 2008, 2009 EURL ACCBEE
//Author:       Patrick Thazard ( )
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import java.awt.*;

import ARoad0.Gui1.*;
import ARoad0.gBaseInterface.BaseObject;


/**
* This class is responsible for managing the display of one rectangle in a GraphicView.
* It should able to contain other GraphicContainers.
* Used by the Architecture view.
* @version 0.6.0
*/
public class GraphicContainer extends GraphicNode {
  // variables
  protected GraphicView         gView_;
  protected BaseObject          baseObject_;
  protected static final int    PRIME = 1000003;
  /*
   * updated by GraphicNoThanView.setGraphicNodes() 
   */
  protected Shape               SHAPE = new Rectangle((int)NODE_SIZE,(int)NODE_SIZE/2);

  public static final GraphicContainer EMPTY_GRAPHIC_CONTAINER = new GraphicContainer();

  //
  // ----------------------------- constructors ---------------------------
  //

  /**
  * This empty constructor allows to draw an empty shape.
  * */
  public GraphicContainer(  ) {

    }

  /**
  * @param _gView that manages this instance, as an External Object
  * Reference.
  * @param _obj is the BaseObject associated to this instance, as an External Object
  * Reference.
  * */
  public GraphicContainer(GraphicView _gView, BaseObject _obj) {
    gView_ = _gView;
    baseObject_ = _obj;
 /*
    String     urlViewIcon = ARoad0.Gui1.Desktop.arFolder__+
                       "Gui1"+ARoad0.Gui1.Desktop.FILE_SEPAR+
                       "images"+ARoad0.Gui1.Desktop.FILE_SEPAR+"acs.gif";
    try {
        add(new JLabel(new ImageIcon( new URL(urlViewIcon) )));
    } catch (MalformedURLException m){
      System.out.println("in the GraphicContainer constructor, exception"+
        " on view icon: "+m.toString() );
    } // catch
*/
/*    addMouseListener(new MouseAdapter(  ) {
      public void mousePressed(MouseEvent e) { spin(e); }
    });
    addMouseMotionListener(new MouseMotionAdapter(  ) {
      public void mouseDragged(MouseEvent e) { spin(e); }
    });
*/
  }
  
  /**
  * In a GraphicNoThanView, calls the parent.
  * @param _e is the event
  * */
  //public void mouseClicked(MouseEvent _e) {
    //  ((GraphicView)getParent()).mouseClicked(_e);
  //}
  
  /**
   * @return the Shape drawn by the GraphicNode
   */
   public Shape getShape() {
    return SHAPE;
   } // getShape()

  /**
   * @return the BaseObject associated to the GraphicContainer
   */
   public BaseObject getEorBaseObject() {
    return baseObject_;
   } // getEorBaseObject()
   
  /**
   * Overread the GraphicNode method for drawing the shape
   * and the BaseObject name.
   * @param _g is the Graphics2D to be drawn.
   */
  public void paintComponent(Graphics _g) {

    Graphics2D g2 = (Graphics2D)_g;
    
    // paint the JPanel components: the icon - does not work
    super.paintComponent(_g);  
    
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING ,
      RenderingHints.VALUE_ANTIALIAS_ON);    
    //GradientPaint gp = new GradientPaint(75, 75, Color.white, 95, 95, Color.blue, true);
    //g2.setPaint(gp);
    g2.setStroke(new BasicStroke(3) );

    if (baseObject_ != null) {
        // g2.drawRect(4, 4, getWidth() - 4, getHeight() - 4);
      g2.setPaint(InternalFrame.FORECOLOR_2);  
      g2.draw(SHAPE);
      g2.setPaint(Color.blue);
      //g2.draw3DRect(2, 2, (int)NODE_SIZE, (int)NODE_SIZE/2, true);
      g2.setFont(BeamerManager.smallBoldUpFont);

      String[] l_name = getDisplayableName();
      String str;
      for ( int i = 0 ; i < l_name.length ; i++ ) {
          str = l_name[i].toUpperCase();
          g2.drawString(" "+str, 2, i*10 + (NODE_SIZE/5) ); //(2*NODE_SIZE / 5)
      } // for (int i
      l_name = null;
      str = null;
    } // if
    else {
      g2.setPaint(InternalFrame.FORECOLOR_2);  
      g2.draw(SHAPE);
      g2.setPaint(Color.blue);
    }
    g2 = null;
    //gp = null;
    return;
  } // paintComponent()


  /**
   * Copy of paintComponent() with opposite colors, without icon.
   * @param _g Graphics to be drawn.
   */
  public void paintComponentForExport(Graphics _g) {

    Graphics2D g2 = (Graphics2D)_g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING ,
      RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setStroke(new BasicStroke(3) );
    //GradientPaint gp = new GradientPaint(75, 75, Color.blue, 95, 95, Color.white, true);
    //g2.setPaint(gp);

    // set the composite rule
    //g2.setComposite(AlphaComposite.SrcIn);

    if (baseObject_ != null) {
      g2.draw(SHAPE);
      g2.setPaint(Color.white);
      //g2.draw3DRect(2, 2, (int)NODE_SIZE, (int)NODE_SIZE/2, true);
      g2.setFont(BeamerManager.smallBoldUpFont);

      String[] l_name = getDisplayableName();
      String str;
      for ( int i = 0 ; i < l_name.length ; i++ ) {
          str = l_name[i].toUpperCase();
          g2.drawString(" "+str, 2, i*10 + (NODE_SIZE/5) ); //(2*NODE_SIZE / 5)
      } // for (int i
      l_name = null;
      str = null;
    } // if
    else {
      g2.setPaint(InternalFrame.FORECOLOR_2);  
      g2.draw(SHAPE);
      g2.setPaint(Color.blue);
    }
    
    g2 = null;
    //gp = null;
    return;
  } // paintComponentForExport()

    /**
     * @return the hashcode value for this object, based on the BaseObject.
     */
    public int hashCode() {
      int result = 0;

      if (baseObject_ != null) {
        result = PRIME*result + baseObject_.hashCode();
      }
      return result;
    } // hashCode()

  /**
   * @return String[] from baseObject_.lastComponent(), cut in NAME_PART-size strings.
   */
   public String[] getDisplayableName() {
    String[] result = new String[0];
    if (baseObject_ != null) {
      if (baseObject_.getName().lastComponent().length() < NAME_PART)
        return new String[] {baseObject_.getName().lastComponent()};
      else {

        int arraySize = 1 + ( baseObject_.getName().lastComponent().length() -
          baseObject_.getName().lastComponent().length()%NAME_PART )/NAME_PART;
        result = new String[arraySize];
        for (int i = 0 ; i < arraySize - 1 ; i++ ) {
          result[i] = baseObject_.getName().lastComponent().
            substring(NAME_PART*i,NAME_PART*i+NAME_PART);
        } // for
        result[arraySize - 1] = baseObject_.getName().lastComponent().substring(
          NAME_PART*(arraySize - 1),
          baseObject_.getName().lastComponent().length());
      } // else

    } // if (baseObject_ != null

    return result;
   } // getDisplayableName()

} // GraphicContainer

