//Title:        Access Road
//Version:      0.6.0
//Copyright:    Copyright 2002, 2004, 2007-2009 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.JComponent;

import ARoad0.Pattern.MVC_View;

import ARoad0.gBaseInterface.BaseObject;


/**
* This class is responsible for managing the display of one graphic object
* in a GraphicView.
* It registers the entity and all its links to other entities in the GraphicView.
* @see ARoad0.gBase.ViewInBaseImpl
*/
public abstract class GraphicNode extends JComponent implements MVC_View {

  /**
   * size reference for the GraphicNode.
   */
  public static final float NODE_SIZE = 75;

  /**
   * space between 2 GraphicNodes columns.
   */
  public static final float COLUMN_SPACE = 70;//110;

  /**
   * space between 2 GraphicNodes rows.
   */
  public static final float ROW_SPACE = 15;//30;//36;

  /**
   * lenght of the name portion.
   */
  public static final int NAME_PART = 9;

  /**
   * lenght of the rights string in GraphicView.
   * in GraphicView, the rights string is cut to the value
   * if its lenght is superior to 4/3 of this value.
   */
  public static final int RIGHTS_PART = 15;

  /**
   * Point 0 in the draw of a view with 4 elements.
   */
  public static final Point POINT_4_0 =
    new Point(0, (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE));
  /**
   * Point 1 in the draw of a view with 4 elements.
   */
  public static final Point POINT_4_1 =
    new Point((int)((GraphicNode.COLUMN_SPACE)+GraphicNode.NODE_SIZE), 0);
  /**
   * Point 2 in the draw of a view with 4 elements.
   */
  public static final Point POINT_4_2 =
    new Point((int)((GraphicNode.COLUMN_SPACE*2)+(GraphicNode.NODE_SIZE*2)),
          (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE));

  /**
   * Point 3 in the draw of a view with 4 elements.
   */
  public static final Point POINT_4_3 =
    new Point((int)((GraphicNode.COLUMN_SPACE)+GraphicNode.NODE_SIZE),
          (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2)));

  /**
   * Point 0 in the draw of a view with 8 elements.
   */
  public static final Point POINT_8_0 =
     new Point(0, (int)((GraphicNode.ROW_SPACE)+(GraphicNode.NODE_SIZE)));

  /**
   * Point 1 in the draw of a view with 8 elements.
   */
  public static final Point POINT_8_1 =
     new Point(0, (int)((GraphicNode.ROW_SPACE*4)+(GraphicNode.NODE_SIZE*2)));

  /**
   * Point 2 in the draw of a view with 8 elements.
   */
  public static final Point POINT_8_2 =
     new Point((int)(GraphicNode.COLUMN_SPACE+GraphicNode.NODE_SIZE), 0);

  /**
   * Point 3 in the draw of a view with 8 elements.
   */
  public static final Point POINT_8_3 =
     new Point((int)(GraphicNode.COLUMN_SPACE+GraphicNode.NODE_SIZE),
          (int)((GraphicNode.ROW_SPACE*5)+(GraphicNode.NODE_SIZE*3)));

  /**
   * Point 0 in the draw of a view with 8 elements.
   */
  public static final Point POINT_8_4 =
     new Point((int)((GraphicNode.COLUMN_SPACE*2)+(GraphicNode.NODE_SIZE*2)),
        0);

  /**
   * Point 0 in the draw of a view with 8 elements.
   */
  public static final Point POINT_8_5 =
     new Point((int)((GraphicNode.COLUMN_SPACE*2)+(GraphicNode.NODE_SIZE*2)),
        (int)((GraphicNode.ROW_SPACE*5)+(GraphicNode.NODE_SIZE*3)));

  /**
   * Point 0 in the draw of a view with 8 elements.
   */
  public static final Point POINT_8_6 =
     new Point((int)((GraphicNode.COLUMN_SPACE*3)+(GraphicNode.NODE_SIZE*3)),
        (int)((GraphicNode.ROW_SPACE)+(GraphicNode.NODE_SIZE)));

  /**
   * Point 0 in the draw of a view with 8 elements.
   */
  public static final Point POINT_8_7 =
     new Point((int)((GraphicNode.COLUMN_SPACE*3)+(GraphicNode.NODE_SIZE*3)),
        (int)((GraphicNode.ROW_SPACE*4)+(GraphicNode.NODE_SIZE*2)));

  /**
   * Point 0 in the draw of a view with 12 elements.
   */
  public static final Point POINT_12_0 =
     new Point((int)GraphicNode.NODE_SIZE*2/3,
       (int)((GraphicNode.ROW_SPACE*4)+(GraphicNode.NODE_SIZE*3)));

  /**
   * Point 1 in the draw of a view with 12 elements.
   */
  public static final Point POINT_12_1 =
     new Point((int)GraphicNode.NODE_SIZE,
          (int)((GraphicNode.ROW_SPACE*6)+(GraphicNode.NODE_SIZE*4)));

  /**
   * Point 2 in the draw of a view with 12 elements.
   */
  public static final Point POINT_12_2 =
     new Point((int)GraphicNode.NODE_SIZE,
          (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2)));

  /**
   * Point 3 in the draw of a view with 12 elements.
   */
  public static final Point POINT_12_3 =
     new Point((int)(GraphicNode.COLUMN_SPACE+(GraphicNode.NODE_SIZE*2)),
          (int)((GraphicNode.ROW_SPACE*8)+(GraphicNode.NODE_SIZE*80/15)));

  /**
   * Point 4 in the draw of a view with 12 elements.
   */
  public static final Point POINT_12_4 =
     new Point((int)(GraphicNode.COLUMN_SPACE+(GraphicNode.NODE_SIZE*2)),
          (int)GraphicNode.NODE_SIZE*2/3);

  /**
   * Point 5 in the draw of a view with 12 elements.
   */
  public static final Point POINT_12_5 =
     new Point((int)((GraphicNode.COLUMN_SPACE*3)+(GraphicNode.NODE_SIZE*3)),
          (int)((GraphicNode.ROW_SPACE*8)+(GraphicNode.NODE_SIZE*6)));

  /**
   * Point 6 in the draw of a view with 12 elements.
   */
  public static final Point POINT_12_6 =
     new Point((int)((GraphicNode.COLUMN_SPACE*3)+(GraphicNode.NODE_SIZE*3)),
          0);

  /**
   * Point 7 in the draw of a view with 12 elements.
   */
  public static final Point POINT_12_7 =
     new Point((int)((GraphicNode.COLUMN_SPACE*5)+(GraphicNode.NODE_SIZE*4)),
          (int)((GraphicNode.ROW_SPACE*8)+(GraphicNode.NODE_SIZE*80/15)));

  /**
   * Point 8 in the draw of a view with 12 elements.
   */
  public static final Point POINT_12_8 =
     new Point((int)((GraphicNode.COLUMN_SPACE*5)+(GraphicNode.NODE_SIZE*4)),
          (int)GraphicNode.NODE_SIZE*2/3);

  /**
   * Point 9 in the draw of a view with 12 elements.
   */
  public static final Point POINT_12_9 =
     new Point((int)((GraphicNode.COLUMN_SPACE*6)+(GraphicNode.NODE_SIZE*5)),
          (int)((GraphicNode.ROW_SPACE*6)+(GraphicNode.NODE_SIZE*4)));

  /**
   * Point 10 in the draw of a view with 12 elements.
   */
  public static final Point POINT_12_10 =
     new Point((int)((GraphicNode.COLUMN_SPACE*6)+(GraphicNode.NODE_SIZE*5)),
          (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2)));

  /**
   * Point 11 in the draw of a view with 12 elements.
   */
  public static final Point POINT_12_11 =
     new Point((int)((GraphicNode.COLUMN_SPACE*6)+(GraphicNode.NODE_SIZE*32/6)),
          (int)((GraphicNode.ROW_SPACE*4)+(GraphicNode.NODE_SIZE*3)));


  /**
   * Array of the nodes locations for a 4 nodes view. No use.
   */
  public static final Point[] L_FOUR_NODES_POINTS = new Point[] {
    POINT_4_0,
    POINT_4_1,
    POINT_4_2,
    POINT_4_3};

  /**
   * Array of the nodes locations for a 8 nodes view. No use.
   */
  public static final Point[] L_EIGHT_NODES_POINTS = new Point[] {
    POINT_8_0,
    POINT_8_1,
    POINT_8_2,
    POINT_8_3,
    POINT_8_4,
    POINT_8_5,
    POINT_8_6,
    POINT_8_7};

  /**
   * Array of the 12 nodes locations in a circle. 
   */
  public static final Point[] L_TWELVE_NODES_POINTS = new Point[] {
    POINT_12_0,
    POINT_12_1,
    POINT_12_2,
    POINT_12_3,
    POINT_12_4,
    POINT_12_5,
    POINT_12_6,
    POINT_12_7,
    POINT_12_8,
    POINT_12_9,
    POINT_12_10,
    POINT_12_11};


  /**
   * Array of the nodes locations for a grid of 9 positions in a rectangular,
   * for a view with less than 5 nodes.
   */
  public static final Point[] L_NINE_GRID = new Point[] {
      // first line
      new Point(0, 0),
      new Point((int)(GraphicNode.COLUMN_SPACE+GraphicNode.NODE_SIZE), 0),
      new Point((int)(GraphicNode.COLUMN_SPACE*2+GraphicNode.NODE_SIZE*2), 0), 
      // second line
      new Point(0, 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
      new Point((int)(GraphicNode.COLUMN_SPACE+GraphicNode.NODE_SIZE), 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
      new Point((int)(GraphicNode.COLUMN_SPACE*2+GraphicNode.NODE_SIZE*2), 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
      // third line        
      new Point(0, 
              (int)(GraphicNode.ROW_SPACE*2+GraphicNode.NODE_SIZE*2)),
      new Point((int)((GraphicNode.COLUMN_SPACE)+GraphicNode.NODE_SIZE),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*2)+GraphicNode.NODE_SIZE*2),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),  
  };  

  /**
   * Array of the nodes locations for a grid of 9 positions in a rectangular,
   * for the Eligible Parties of a NoThan view.
   */
  public static final Point[] L_NOTHAN_NINE_GRID = new Point[] {
      // first line
      new Point(0, 0),
      new Point((int)(GraphicNode.COLUMN_SPACE/2+GraphicNode.NODE_SIZE), 0),
      new Point((int)(GraphicNode.COLUMN_SPACE+GraphicNode.NODE_SIZE*2), 0), 
      // second line
      new Point(0, 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
      new Point((int)(GraphicNode.COLUMN_SPACE/2+GraphicNode.NODE_SIZE), 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
      new Point((int)(GraphicNode.COLUMN_SPACE+GraphicNode.NODE_SIZE*2), 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
      // third line        
      new Point(0, 
              (int)(GraphicNode.ROW_SPACE*2+GraphicNode.NODE_SIZE*2)),
      new Point((int)((GraphicNode.COLUMN_SPACE)/2+GraphicNode.NODE_SIZE),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE)+GraphicNode.NODE_SIZE*2),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),  
  }; 
  
  /**
   * Array of the nodes locations for a grid of 4x4 positions in a rectangular,
   * for a 7-nodes view
   */
  public static final Point[] L_SIXTEEN_GRID = new Point[] {
      // first line
      new Point(0, 0),
      new Point((int)(GraphicNode.COLUMN_SPACE+GraphicNode.NODE_SIZE), 0),
      new Point((int)(GraphicNode.COLUMN_SPACE*2+GraphicNode.NODE_SIZE*2), 0), 
      new Point((int)(GraphicNode.COLUMN_SPACE*3+GraphicNode.NODE_SIZE*3), 0),
      // second line
      new Point(0, 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
      new Point((int)(GraphicNode.COLUMN_SPACE+GraphicNode.NODE_SIZE), 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
      new Point((int)(GraphicNode.COLUMN_SPACE*2+GraphicNode.NODE_SIZE*2), 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
      new Point((int)(GraphicNode.COLUMN_SPACE*3+GraphicNode.NODE_SIZE*3), 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
              // third line        
      new Point(0, 
              (int)(GraphicNode.ROW_SPACE*2+GraphicNode.NODE_SIZE*2)),
      new Point((int)((GraphicNode.COLUMN_SPACE)+GraphicNode.NODE_SIZE),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*2)+GraphicNode.NODE_SIZE*2),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),  
      new Point((int)((GraphicNode.COLUMN_SPACE*3)+GraphicNode.NODE_SIZE*3),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))), 
              // fourth line
      new Point(0, 
              (int)(GraphicNode.ROW_SPACE*3+GraphicNode.NODE_SIZE*3)),
      new Point((int)((GraphicNode.COLUMN_SPACE)+GraphicNode.NODE_SIZE),
              (int)((GraphicNode.ROW_SPACE*3)+(GraphicNode.NODE_SIZE*3))),
      new Point((int)((GraphicNode.COLUMN_SPACE*2)+GraphicNode.NODE_SIZE*2),
              (int)((GraphicNode.ROW_SPACE*3)+(GraphicNode.NODE_SIZE*3))),  
      new Point((int)((GraphicNode.COLUMN_SPACE*3)+GraphicNode.NODE_SIZE*3),
              (int)((GraphicNode.ROW_SPACE*3)+(GraphicNode.NODE_SIZE*3))), 
  };   

  /**
   * Array of the nodes locations for a grid of 5x5 positions in a rectangular,
   * for a 12-nodes view. Initialized in a static field.
   */
  public static final Point[] L_TWENTY_FIVE_GRID;
  
  /**
   * Array of the nodes locations for a grid of 7x7 positions in a rectangular,
   * for a 24-nodes view, with half spaces between locations. Initialized in
   * a static field.
   */
  public static final Point[] L_FOURTY_NINE_GRID;

  /**
   * Array of the nodes locations for a grid of 10x10 positions in a rectangular,
   * for a 48-nodes view, with half spaces between locations. Initialized in a static
   * field.
   */
  public static final Point[] L_ONE_HUNDRED_GRID; 
  
  /**
   * Array of the node locations for a grid of 7 positions in a rectangular,
   * for a 7-nodes sketch view. The index for the center is always (lenght - 1)/2
   * (3, there).
   */
  public static final Point[] L_SEVEN_NODES_SKETCH_GRID = new Point[] {
      // first line
      new Point(0, 0),
      new Point((int)(GraphicNode.COLUMN_SPACE+GraphicNode.NODE_SIZE), 0),
      new Point((int)(GraphicNode.COLUMN_SPACE*2+GraphicNode.NODE_SIZE*2), 0), 
      // second line only for the center node
      new Point((int)((GraphicNode.COLUMN_SPACE)+GraphicNode.NODE_SIZE), 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
              // third line        
      new Point(0, 
              (int)(GraphicNode.ROW_SPACE*2+GraphicNode.NODE_SIZE*2)),
      new Point((int)((GraphicNode.COLUMN_SPACE)+GraphicNode.NODE_SIZE),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*2)+GraphicNode.NODE_SIZE*2),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2)))
  };     

  /**
   * Array of the node locations for a grid of 11 positions in a rectangular,
   * for a 11-nodes sketch view. The index for the center is always (lenght - 1)/2
   * (5, there).
   */
  public static final Point[] L_ELEVEN_NODES_SKETCH_GRID = new Point[] {
      // first line
      new Point(0, 0),
      new Point((int)(GraphicNode.COLUMN_SPACE+GraphicNode.NODE_SIZE), 0),
      new Point((int)(GraphicNode.COLUMN_SPACE*2+GraphicNode.NODE_SIZE*2), 0), 
      new Point((int)(GraphicNode.COLUMN_SPACE*3+GraphicNode.NODE_SIZE*3), 0),
      new Point((int)(GraphicNode.COLUMN_SPACE*4+GraphicNode.NODE_SIZE*4), 0),
      // second line only for the center node
      new Point((int)((GraphicNode.COLUMN_SPACE*2)+GraphicNode.NODE_SIZE*2), 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
              // third line        
      new Point(0, 
              (int)(GraphicNode.ROW_SPACE*2+GraphicNode.NODE_SIZE*2)),
      new Point((int)((GraphicNode.COLUMN_SPACE)+GraphicNode.NODE_SIZE),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*2)+GraphicNode.NODE_SIZE*2),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*3)+GraphicNode.NODE_SIZE*3),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*4)+GraphicNode.NODE_SIZE*4),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2)))
  }; 

  /**
   * Array of the node locations for a grid of 21 positions in a rectangular,
   * for a 21-nodes sketch view. The index for the center is always (lenght - 1)/2
   * (10, there).
   */
  public static final Point[] L_TWENTYONE_NODES_SKETCH_GRID = new Point[] {
      // first line
      new Point(0, 0),
      new Point((int)(GraphicNode.COLUMN_SPACE+GraphicNode.NODE_SIZE), 0),
      new Point((int)(GraphicNode.COLUMN_SPACE*2+GraphicNode.NODE_SIZE*2), 0), 
      new Point((int)(GraphicNode.COLUMN_SPACE*3+GraphicNode.NODE_SIZE*3), 0),
      new Point((int)(GraphicNode.COLUMN_SPACE*4+GraphicNode.NODE_SIZE*4), 0),
      new Point((int)(GraphicNode.COLUMN_SPACE*5+GraphicNode.NODE_SIZE*5), 0),
      new Point((int)(GraphicNode.COLUMN_SPACE*6+GraphicNode.NODE_SIZE*6), 0), 
      new Point((int)(GraphicNode.COLUMN_SPACE*7+GraphicNode.NODE_SIZE*7), 0),
      new Point((int)(GraphicNode.COLUMN_SPACE*8+GraphicNode.NODE_SIZE*8), 0),
      new Point((int)(GraphicNode.COLUMN_SPACE*9+GraphicNode.NODE_SIZE*9), 0),
      // second line only for the center node
      new Point((int)((GraphicNode.COLUMN_SPACE*5)+GraphicNode.NODE_SIZE*5), 
              (int)(GraphicNode.ROW_SPACE+GraphicNode.NODE_SIZE)),
              // third line        
      new Point(0, 
              (int)(GraphicNode.ROW_SPACE*2+GraphicNode.NODE_SIZE*2)),
      new Point((int)((GraphicNode.COLUMN_SPACE)+GraphicNode.NODE_SIZE),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*2)+GraphicNode.NODE_SIZE*2),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*3)+GraphicNode.NODE_SIZE*3),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*4)+GraphicNode.NODE_SIZE*4),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*5)+GraphicNode.NODE_SIZE*5),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*6)+GraphicNode.NODE_SIZE*6),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*7)+GraphicNode.NODE_SIZE*7),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*8)+GraphicNode.NODE_SIZE*8),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2))),
      new Point((int)((GraphicNode.COLUMN_SPACE*9)+GraphicNode.NODE_SIZE*9),
              (int)((GraphicNode.ROW_SPACE*2)+(GraphicNode.NODE_SIZE*2)))
  }; 
  
  /**
   * Dimension of a 'See why' JTextPane
   */  
  public static final Dimension TAB_SIZE = new Dimension( 560, 2000 );
  
  /**
   * Dimension for a GraphicView
   */  
  public static final Dimension VIEW_INTAB_SIZE = new Dimension(
         550,
         300 );

  /**
   * Dimension of a 9 positions view for 3 nodes. Usefull when the view
   * is in a dedicated internal frame. No use.
   */
  public static final Dimension THREE_NODES_VIEW_SIZE = new Dimension(
         (int)(GraphicNode.NODE_SIZE*3+GraphicNode.COLUMN_SPACE*2),
         (int) (GraphicNode.NODE_SIZE*3+GraphicNode.ROW_SPACE*2)*2/3 );  
  
  /**
   * Dimension of a 9 positions view for 4 nodes. No use.
   */
  public static final Dimension FOUR_NODES_VIEW_SIZE = new Dimension(
         (int)(GraphicNode.NODE_SIZE*3+GraphicNode.COLUMN_SPACE*2),
         (int)(GraphicNode.NODE_SIZE*3+GraphicNode.ROW_SPACE*2));

  /**
  * Dimension of a 4x4 positions view for up to 7 nodes
  */
  public static final Dimension EIGHT_NODES_VIEW_SIZE = new Dimension(
         (int)(GraphicNode.NODE_SIZE*4+GraphicNode.COLUMN_SPACE*3),
         (int)(GraphicNode.NODE_SIZE*4+GraphicNode.ROW_SPACE*5));

  /**
  * Dimension of a 5x5 positions view for up to 12 nodes
  */
  public static final Dimension TWELVE_NODES_VIEW_SIZE = new Dimension(
         (int)(GraphicNode.NODE_SIZE*5+GraphicNode.COLUMN_SPACE*6),
         (int)(GraphicNode.NODE_SIZE*5+GraphicNode.ROW_SPACE*6));
  
  /**
  * Dimension of a 7x7 positions view for up to 24 nodes, using half-spaces between nodes
  */
  public static final Dimension TWENTY_FOUR_NODES_VIEW_SIZE = new Dimension(
         (int)(GraphicNode.NODE_SIZE*7+GraphicNode.COLUMN_SPACE*4),
         (int)(GraphicNode.NODE_SIZE*7+GraphicNode.ROW_SPACE*4));

  /**
  * Dimension of a 10x10 positions view for 46 nodes, using half-spaces between nodes
  */
  public static final Dimension FORTY_SIX_NODES_VIEW_SIZE = new Dimension(
         (int)(GraphicNode.NODE_SIZE*10+GraphicNode.COLUMN_SPACE*6),
         (int)(GraphicNode.NODE_SIZE*10+GraphicNode.ROW_SPACE*6));
  
  /**
  * Dimension of a 7 positions sketch view
  */
  public static final Dimension SEVEN_NODES_SKETCH_VIEW_SIZE = new Dimension(
          (int)(GraphicNode.NODE_SIZE*7+GraphicNode.COLUMN_SPACE*7),
          (int)(GraphicNode.NODE_SIZE*3+GraphicNode.ROW_SPACE*2));  

  /**
  * Dimension of a 11 positions sketch view
  */
  public static final Dimension ELEVEN_NODES_SKETCH_VIEW_SIZE = new Dimension(
         (int)(GraphicNode.NODE_SIZE*11+GraphicNode.COLUMN_SPACE*10),
         (int)(GraphicNode.NODE_SIZE*3+GraphicNode.ROW_SPACE*2));

  /**
  * Dimension of a 21 positions sketch view - like the previous one
  */
  public static final Dimension TWENTYONE_NODES_SKETCH_VIEW_SIZE = new Dimension(
         (int)(GraphicNode.NODE_SIZE*11+GraphicNode.COLUMN_SPACE*10),
         (int)(GraphicNode.NODE_SIZE*3+GraphicNode.ROW_SPACE*2));
  
  public static final int TWENTY_FIVE_GRID_WIDTH = 5;
  public static final int TWENTY_FIVE_GRID_HEIGHT = 5;
  public static final int FOURTY_NINE_GRID_WIDTH = 7;
  public static final int FOURTY_NINE_GRID_HEIGHT = 7;
  public static final int HUNDRED_GRID_WIDTH = 10;
  public static final int HUNDRED_GRID_HEIGHT = 10; 
  
  // Define 3 grids of node positions, 5x5, 7x7 and 10x10
  static {
    int width_index = 0;
    int height_index = 0;
    
       // for testing and tracking
       //System.out.println("GraphicNode initialization of static L_TWENTY_FIVE_GRID.");
    
    L_TWENTY_FIVE_GRID = new Point[25];  
    for (int i = 0; i < L_TWENTY_FIVE_GRID.length; i++) {
      
        // no half-space between nodes
        L_TWENTY_FIVE_GRID[i] = new Point(
              (int)(GraphicNode.COLUMN_SPACE*width_index+GraphicNode.NODE_SIZE*width_index),
              (int)((GraphicNode.ROW_SPACE*height_index)+(GraphicNode.NODE_SIZE*height_index)));
        
        // for testing and tracking
        //System.out.println("--------------- for "+i+" : "+L_TWENTY_FIVE_GRID[i]);
        
        width_index++;
        if (width_index == TWENTY_FIVE_GRID_WIDTH) {
          width_index = 0;
          height_index++;
        }
        
        if (height_index == TWENTY_FIVE_GRID_HEIGHT && i != L_TWENTY_FIVE_GRID.length - 1)
          throw new InternalError(
              "From GraphicNode static intialization of L_TWENTY_FIVE_GRID, i ="+i);

    } // for i

       // for testing and tracking
       //System.out.println("GraphicNode initialization of static L_FOURTY_NINE_GRID.");    
    
    width_index = 0;
    height_index = 0;    
    L_FOURTY_NINE_GRID = new Point[49];  
    for (int i = 0; i < L_FOURTY_NINE_GRID.length; i++) {
      
        // half-space between nodes
        L_FOURTY_NINE_GRID[i] = new Point(
              (int)(GraphicNode.COLUMN_SPACE*width_index/2+GraphicNode.NODE_SIZE*width_index),
              (int)((GraphicNode.ROW_SPACE*height_index/2)+(GraphicNode.NODE_SIZE*height_index)));

        
        // for testing and tracking
        //System.out.println("--------------- for "+i+" : "+L_FOURTY_NINE_GRID[i]);        
        
        width_index++;
        if (width_index == FOURTY_NINE_GRID_WIDTH) {
          width_index = 0;
          height_index++;
        }
        
        if (height_index == FOURTY_NINE_GRID_HEIGHT && i != L_FOURTY_NINE_GRID.length - 1)
          throw new InternalError(
              "From GraphicNode static intialization of L_FOURTY_NINE_GRID, i ="+i);

    } // for i

    width_index = 0;
    height_index = 0;     
    L_ONE_HUNDRED_GRID = new Point[100];
    for (int i = 0; i < L_ONE_HUNDRED_GRID.length; i++) {
      
        // half-space between nodes
        L_ONE_HUNDRED_GRID[i] = new Point(
              (int)(GraphicNode.COLUMN_SPACE*width_index/2+GraphicNode.NODE_SIZE*width_index),
              (int)((GraphicNode.ROW_SPACE*height_index/2)+(GraphicNode.NODE_SIZE*height_index)));
        
        width_index++;
        if (width_index == HUNDRED_GRID_WIDTH) {
          width_index = 0;
          height_index++;
        }
        
        if (height_index == HUNDRED_GRID_HEIGHT && i != L_ONE_HUNDRED_GRID.length - 1)
          throw new InternalError(
              "From GraphicNode static intialization of L_ONE_HUNDRED_GRID, i ="+i);

    } // for i
        
  } // static
  
  /**
   * No use. Overridden by the subclasses.
   * @return the BaseObject associated to the GraphicNode
   */
   public BaseObject getEorBaseObject() {
     return null;
   }

  /**
   * No use. Overridden by the subclasses.
   * @return the Shape drawn by the GraphicNode
   */
   public Shape getShape() {
     return new GeneralPath();
   }

  /**
   * Overridden by the subclasses.
   * @param _g Graphics to be drawn.
   */
  public void paintComponentForExport(Graphics _g) {
      
  }

  /**
   * No use. Overridden by the subclasses.
   * @return the node name to display in several parts
   */
   public String[] getDisplayableName() {
    return new String[0];
   } // getDisplayableName()
}