//Title:        Access Road
//Version:      0.6.0
//Copyright:    Copyright 2002, 2004, 2007, 2008, 2009 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import java.awt.*;
import java.awt.geom.*;

import ARoad0.Gui1.*;
import ARoad0.gBaseInterface.Resource;
import ARoad0.gBaseInterface.BaseObject;


/**
* This class is responsible for managing the display of one Resource
* in one GraphicView.
* @version 0.6.0
*/
public class GraphicResource extends GraphicNode {
  // variables
  protected GraphicView         gView_;
  protected Resource            res_;
  protected static final int    PRIME = 1000003;
    // shape of a resource
  protected static final Shape  SHAPE = new GeneralPath(GeneralPath.WIND_EVEN_ODD, 10);

  // constructors

  /**
  * This empty constructor allows to draw an empty shape.
  * */
  public GraphicResource(  ) {

    ((GeneralPath)SHAPE).moveTo(NODE_SIZE + 2, 2); // -2
    ((GeneralPath)SHAPE).lineTo(NODE_SIZE + 2, 3*(NODE_SIZE-2)/5); // -2
    ((GeneralPath)SHAPE).lineTo(2, 3*(NODE_SIZE-2)/5);
    ((GeneralPath)SHAPE).lineTo(2, 2);
    ((GeneralPath)SHAPE).closePath();

    setPreferredSize(new Dimension((int)getShape().getBounds2D().getWidth()+2,
      (int)getShape().getBounds2D().getHeight()));
    setMinimumSize(new Dimension((int)getShape().getBounds2D().getWidth()+2,
      (int)getShape().getBounds2D().getHeight()));
    setMaximumSize(new Dimension((int)getShape().getBounds2D().getWidth()+2,
      (int)getShape().getBounds2D().getHeight()));

  /*    addGraphicViewListener(new GraphicViewListener(  ) {
      public void graphicViewAdjusted(GraphicViewEvent e) {
        //
      }
    });
  */
    }

  /**
  * @param _gView manages this instance
  * @param _res associated to this instance
  * */
  public GraphicResource(GraphicView _gView, Resource _res) {
    this();
    gView_ = _gView;
    res_ = _res;
  }

  /**
   * @return Shape drawn by the GraphicNode
   */
   public Shape getShape() {
    return SHAPE;
   } // getShape()


  /**
   * @return the resource associated to the GraphicResource
   */
   public BaseObject getEorBaseObject() {
    return res_;
   } // getEorBaseObject()

  /**
   * The name from getDisplayableName() is drawn until 3 lines, no more.
   * Overrides the GraphicNode method for drawing the shape
   * and the Resource name.
   * @param _g Graphics to be drawn.
   */
  public void paintComponent(Graphics _g) {

    Graphics2D g2 = (Graphics2D)_g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING ,
      RenderingHints.VALUE_ANTIALIAS_ON);
    //GradientPaint gp = new GradientPaint(75, 75, Color.white, 95, 95, Color.blue, true);
    //g2.setPaint(gp);
    //g2.setStroke(new BasicStroke(3) );
    g2.setPaint(Color.blue);

    if (res_ != null) {
      g2.draw(SHAPE);
      g2.setFont(BeamerManager.smallBoldUpFont);

      String[] l_name = getDisplayableName();
      String str;
      for ( int i = 0 ; i < l_name.length ; i++ ) {
          str = l_name[i].toUpperCase();
          g2.drawString(" "+str, 2, i*10 + (NODE_SIZE/5) ); //(2*NODE_SIZE / 5)
          if (i == 2) break;
      } // for (int i
      l_name = null;
      str = null;
    } // if
    g2 = null;
    //gp = null;
    return;
  } // paintComponent()

  /**
   * Copy of paintComponent() with opposite colors.
   * @param _g Graphics to be drawn.
   */
  public void paintComponentForExport(Graphics _g) {

    Graphics2D g2 = (Graphics2D)_g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING ,
      RenderingHints.VALUE_ANTIALIAS_ON);
    //g2.setStroke(new BasicStroke(3) );
    //GradientPaint gp = new GradientPaint(75, 75, Color.blue, 95, 95, Color.white, true);
    //g2.setPaint(gp);
    g2.setPaint(Color.white);

    // set the composite rule
    //g2.setComposite(AlphaComposite.SrcIn);

    if (res_ != null) {
      g2.draw(SHAPE);
      g2.setFont(BeamerManager.smallBoldUpFont);

      String[] l_name = getDisplayableName();
      String str;
      for ( int i = 0 ; i < l_name.length ; i++ ) {
          str = l_name[i].toUpperCase();
          g2.drawString(" "+str, 2, i*10 + (NODE_SIZE/5) ); //(2*NODE_SIZE / 5)
          if (i == 2) break;
      } // for (int i
      l_name = null;
      str = null;
    } // if
    g2 = null;
    //gp = null;
    return;
  } // paintComponentForExport()

    /**
     * @return the hashcode value for this object, based on the resource.
     */
    public int hashCode() {
      int result = 0;

      if (res_ != null) {
        result = PRIME*result + res_.hashCode();
      }
      return result;
    } // hashCode()

  /**
   * Gets the presentation in N parts of the Resource name.
   * @return String[] from res_.lastComponent(), cut in NAME_PART-size strings.
   */
   public String[] getDisplayableName() {
    String[] result = new String[0];
    if (res_ != null) {
      if (res_.getName().lastComponent().length() < NAME_PART)
        return new String[] {res_.getName().lastComponent()};
      else {

        int arraySize = 1 + ( res_.getName().lastComponent().length() -
          res_.getName().lastComponent().length()%NAME_PART )/NAME_PART;
        result = new String[arraySize];
        for (int i = 0 ; i < arraySize - 1 ; i++ ) {
          result[i] = res_.getName().lastComponent().
            substring(NAME_PART*i,NAME_PART*i+NAME_PART);
        } // for
        result[arraySize - 1] = res_.getName().lastComponent().substring(
          NAME_PART*(arraySize - 1),
          res_.getName().lastComponent().length());
      } // else

    } // if (res_ != null

    return result;
   } // getDisplayableName()

} // GraphicResource