//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import java.awt.*;

import ARoad0.Gui1.*;
import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBaseInterface.StringRight;


/**
* This class is responsible for managing the displaying of a colored text in a GraphicView,
* from any BaseObject, a StringRight more specifically, or to display any String.<P>
* Used by GraphicNoThanView which calls setCriterionColorInGreen() to switch
* between green and red colors.<P>
* Used by GraphicView.addWaitingMessage() that uses a simple String to
* display a green text.
* @version 0.7.0
*/
public class GraphicText extends GraphicNode {
  // variables
  protected GraphicView         gView_;
  protected Object              object_;
  protected Color               textColor_;
  protected static final int    PRIME = 1000003;
  /*
   * no update so far
   */
  protected Shape               SHAPE = new Rectangle((int)NODE_SIZE*2,(int)NODE_SIZE/2);

  public static final GraphicText EMPTY_GRAPHIC_TEXT = new GraphicText();

  //
  // ----------------------------- constructors ---------------------------
  //

  /**
  * This empty constructor allows to draw an empty shape.
  * */
  public GraphicText(  ) {

  }

  /**
   * Set the default color to green.
  * @param _gView that manages this instance, as an External Object
  * Reference. Never null.
  * @param _obj may be the BaseObject associated to this instance, or a String.
  *     May be null.
  * */
  public GraphicText(GraphicView _gView, Object _obj) {
    gView_ = _gView;
    if (_obj instanceof BaseObject || _obj instanceof String)
        object_ = _obj;
    textColor_ = InternalFrame.FORECOLOR_3;
  }

  /**
   * The shape is a rectangle, but it is not used in the paint operations.
   * @return the Shape drawn by the GraphicNode
   */
   public Shape getShape() {
    return SHAPE;
   } // getShape()

  /**
   * The green color for the drawn criterion is InternalFrame.FORECOLOR_3, else
   * it is a red color: InternalFrame.FORECOLOR_4.
   * @return _true if the current color is green, false if it is red
   */
   public boolean isGreenCriterionColor() {
    if (textColor_ == InternalFrame.FORECOLOR_3 ) return true;
    else return false;
   } // isGreenCriterionColor()   

  /**
   * The green color for the drawn criterion is InternalFrame.FORECOLOR_3, else
   * it is a red color: InternalFrame.FORECOLOR_4.
   * @param _green true if the new color has to be green, false if it has to be red
   */
   public void setCriterionColorInGreen(boolean _green) {
    if (_green) {
                        // for testing and tracking
                    //System.out.println("In GraphicText.setCriterionColorInGreen(), green color for "+
                      //      object_.getName());
                    
        textColor_ = InternalFrame.FORECOLOR_3;
    }
    else {
                        // for testing and tracking
                    //System.out.println("In GraphicText.setCriterionColorInGreen(), red color for "+
                      //      object_.getName());
                    
        textColor_ = InternalFrame.FORECOLOR_4;
    }
   } // setCriterionColorInGreen()

  /**
   * When this class is used by GraphicNoThanView, the base object is a StringRight.
   * @return the BaseObject associated to the GraphicText, or null if the object
   *        is a String
   */
   public BaseObject getEorBaseObject() {
    if (object_ instanceof BaseObject) return (BaseObject)object_;
    else return null;
   } // getEorBaseObject()

  /**
   * When this class is used by GraphicNoThanView, the base object is a StringRight.
   * Is set to null by the GraphicNoThanView when the target is null.
   * @param _right associated to the GraphicText. May be null.
   */
   public void setEorBaseObject(StringRight _right) {
    object_ = _right;
   } // getEorBaseObject()
   
  /**
   * Overrides the GraphicNode method for drawing the shape and the BaseObject name.
   * @param _g is the Graphics2D to be drawn.
   */
  public void paintComponent(Graphics _g) {

                        // for testing and tracking
                    //System.out.println("Call to GraphicText.paintComponent()");

    Graphics2D g2 = (Graphics2D)_g;
    
    // paint the JPanel components: the icon - does not work
    super.paintComponent(_g);
    
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING ,
      RenderingHints.VALUE_ANTIALIAS_ON);    
    g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING ,
      RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
    g2.setFont(BeamerManager.boldButtonFont);
    g2.setStroke(GraphicView.BASIC_STROKE);

    if (object_ != null) {
      //g2.setPaint(Color.WHITE);  
      //g2.setFont(BeamerManager.smallBoldUpFont);
      //g2.draw(SHAPE);

                        // for testing and tracking
                    //System.out.println("In GraphicText.paintComponent(), textColor_: "+textColor_);
        
      g2.setPaint(textColor_);

      String str;
      if (object_ instanceof StringRight)
          str = ((StringRight)object_).getRight();
      else if (object_ instanceof BaseObject)
          str = ((BaseObject)object_).getName().lastComponent();
      else
          str = (String)object_;
      
      g2.drawString(" "+str, 2, 10 );

      str = null;
    } // if
    //else {
      //g2.setPaint(InternalFrame.BACKCOLOR_7);  
      //g2.draw(SHAPE);
      //g2.setPaint(Color.blue);
    //}
    g2 = null;
    //gp = null;
  } // paintComponent()

  /**
   * Copy of paintComponent() with opposite colors, without icon.
   * @param _g Graphics to be drawn.
   */
  public void paintComponentForExport(Graphics _g) {

    Graphics2D g2 = (Graphics2D)_g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING ,
      RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setStroke(new BasicStroke(3) );
    //GradientPaint gp = new GradientPaint(75, 75, Color.blue, 95, 95, Color.white, true);
    //g2.setPaint(gp);

    // set the composite rule
    //g2.setComposite(AlphaComposite.SrcIn);

    if (object_ != null) {
      g2.setPaint(textColor_);  
      //g2.setFont(BeamerManager.smallBoldUpFont);
      //g2.draw(SHAPE);

      String str;
      if (object_ instanceof StringRight)
          str = ((StringRight)object_).getRight();
      else if (object_ instanceof BaseObject)
          str = ((BaseObject)object_).getName().lastComponent();
      else
          str = (String)object_;
      
      g2.drawString(" "+str, 2, 10 );

      str = null;
    } // if
    //else {
      //g2.setPaint(InternalFrame.BACKCOLOR_7);  
      //g2.draw(SHAPE);
      //g2.setPaint(Color.blue);
    //}
    
    g2 = null;
    //gp = null;
  } // paintComponentForExport()

    /**
     * @return the hashcode value for this object, based on the BaseObject.
     */
    public int hashCode() {
      int result = 100001;

      if (object_ != null) {
        result = PRIME*result + object_.hashCode();
      }
      return result;
    } // hashCode()

  /**
   * @return String[] from object_.lastComponent(), cut in NAME_PART-size strings.
   */
   public String[] getDisplayableName() {
    String[] result = new String[0];
    if (object_ == null) return result;
    if (object_ instanceof BaseObject) {
      if (((BaseObject)object_).getName().lastComponent().length() < NAME_PART)
        return new String[] {((BaseObject)object_).getName().lastComponent()};
      else {

        int arraySize = 1 + ( ((BaseObject)object_).getName().lastComponent().length() -
          ((BaseObject)object_).getName().lastComponent().length()%NAME_PART )/NAME_PART;
        result = new String[arraySize];
        for (int i = 0 ; i < arraySize - 1 ; i++ ) {
          result[i] = ((BaseObject)object_).getName().lastComponent().
            substring(NAME_PART*i,NAME_PART*i+NAME_PART);
        } // for
        result[arraySize - 1] = ((BaseObject)object_).getName().lastComponent().substring(
          NAME_PART*(arraySize - 1),
          ((BaseObject)object_).getName().lastComponent().length());
      } // else
    } // if (object_ instanceof BaseObject
    else if (object_ instanceof String) {
      if (((String)object_).length() < NAME_PART)
        return new String[] {(String)object_};
      else {

        int arraySize = 1 + ((String)object_).length() -
          (((String)object_).length()%NAME_PART )/NAME_PART;
        result = new String[arraySize];
        for (int i = 0 ; i < arraySize - 1 ; i++ ) {
          result[i] = ((String)object_).
            substring(NAME_PART*i,NAME_PART*i+NAME_PART);
        } // for
        result[arraySize - 1] = ((String)object_).substring(
          NAME_PART*(arraySize - 1),
          ((String)object_).length());
      } // else
    } // if (object_ instanceof String

    return result;
   } // getDisplayableName()

} // GraphicText


