//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import java.awt.*;
import java.awt.geom.*;

import ARoad0.Gui1.*;
import ARoad0.gBaseInterface.VirtualFolder;
import ARoad0.gBaseInterface.BaseObject;


/**
* This class is responsible for managing the display of a VirtualFolder
* in a GraphicView.
* @version 0.7.0
*/
public class GraphicVirtualFolder extends GraphicNode {
  // variables
  protected GraphicView         gView_;
  protected VirtualFolder       vf_;
  protected static final int    PRIME = 1000003;
    // drawn shape of a virtual folder
  protected static final Shape  SHAPE = new GeneralPath(GeneralPath.WIND_EVEN_ODD, 10);
    // effective shape, after subtracting to the drawn shape
  protected static Shape        SUBTRACTED_SHAPE = null;
  
  static {

    ((GeneralPath)SHAPE).moveTo(2, 4);
    ((GeneralPath)SHAPE).lineTo(5, 2);
    ((GeneralPath)SHAPE).lineTo((NODE_SIZE-2)/2, 6);
    ((GeneralPath)SHAPE).lineTo(NODE_SIZE-5, 2);
    ((GeneralPath)SHAPE).lineTo(NODE_SIZE-2, 4);
    ((GeneralPath)SHAPE).lineTo(NODE_SIZE-8, 4*(NODE_SIZE-2)/5);
    ((GeneralPath)SHAPE).lineTo(8, 4*(NODE_SIZE-2)/5);
    ((GeneralPath)SHAPE).closePath();

      /*
       * old version
       *
    ((GeneralPath)SHAPE).moveTo(NODE_SIZE/2, 0);
    ((GeneralPath)SHAPE).curveTo(120, 10, 50, 80, NODE_SIZE-7, NODE_SIZE-2);

    ((GeneralPath)SHAPE).lineTo(7, NODE_SIZE-2);
    ((GeneralPath)SHAPE).curveTo(25, 80, -46, 10, NODE_SIZE/2, 0);

    ((GeneralPath)SHAPE).moveTo(1, 3*NODE_SIZE/4 - 6);
    ((GeneralPath)SHAPE).quadTo(7, NODE_SIZE-2, NODE_SIZE/2, NODE_SIZE - 2);
    ((GeneralPath)SHAPE).quadTo(NODE_SIZE-7, NODE_SIZE-2, NODE_SIZE - 1, 3*NODE_SIZE/4 - 6);

    ((GeneralPath)SHAPE).moveTo(NODE_SIZE/2, 0);
    ((GeneralPath)SHAPE).closePath();
    
    SUBTRACTED_SHAPE = new Area(SHAPE);
    ((Area)SUBTRACTED_SHAPE).subtract(new Area(new Rectangle2D.Double(
          NODE_SIZE +8, 2, NODE_SIZE/2, NODE_SIZE)));
       */
  } // static

  // constructors

  /**
  * This empty constructor allows to draw a shape whithout name.
  * */
  public GraphicVirtualFolder() {
    setPreferredSize(new Dimension((int)getShape().getBounds2D().getWidth()+2,
      (int)getShape().getBounds2D().getHeight()));
    setMinimumSize(new Dimension((int)getShape().getBounds2D().getWidth()+2,
      (int)getShape().getBounds2D().getHeight()));
    setMaximumSize(new Dimension((int)getShape().getBounds2D().getWidth()+2,
      (int)getShape().getBounds2D().getHeight()));
  }// GraphicVirtualFolder()

  /**
  * @param _gView manages this instance
  * @param _vf associated to this instance
  * */
  public GraphicVirtualFolder(GraphicView _gView, VirtualFolder _vf) {
    this();
    gView_ = _gView;
    vf_ = _vf;
  }

  /**
   * @return Shape drawn by the GraphicNode
   */
   public Shape getShape() {
       return SHAPE; // SUBTRACTED_SHAPE;
   } // getShape()


  /**
   * @return the VirtualFolder associated to the GraphicVirtualFolder
   */
   public BaseObject getEorBaseObject() {
    return vf_;
   } // getEorBaseObject()

  /**
   * The name from getDisplayableName() is drawn until 3 lines, no more.
   * Overrides the GraphicNode method for drawing the shape
   * and the VirtualFolder name.
   * @param _g Graphics to be drawn.
   */
  public void paintComponent(Graphics _g) {

    Graphics2D g2 = (Graphics2D)_g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING ,
      RenderingHints.VALUE_ANTIALIAS_ON);
    //GradientPaint gp = new GradientPaint(75, 75, Color.white, 95, 95, Color.blue, true);
    //g2.setPaint(gp);
    //g2.setStroke(new BasicStroke(3) );
    g2.setPaint(Color.blue);

    if (vf_ != null) {
      g2.draw(getShape());
      g2.setFont(BeamerManager.smallBoldUpFont);

      String[] l_name = getDisplayableName();
      String str;
      for ( int i = 0 ; i < l_name.length ; i++ ) {
          str = l_name[i].toUpperCase();
          g2.drawString(" "+str, 2, i*10 + (NODE_SIZE/5) ); //(2*NODE_SIZE / 5)
          if (i == 2) break;
      } // for (int i
      l_name = null;
      str = null;
    } // if
    g2 = null;
    //gp = null;
    return;
  } // paintComponent()

  /**
   * Copy of paintComponent() with opposite colors.
   * @param _g Graphics to be drawn.
   */
  public void paintComponentForExport(Graphics _g) {

    Graphics2D g2 = (Graphics2D)_g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING ,
      RenderingHints.VALUE_ANTIALIAS_ON);
    //g2.setStroke(new BasicStroke(3) );
    //GradientPaint gp = new GradientPaint(75, 75, Color.blue, 95, 95, Color.white, true);
    //g2.setPaint(gp);
    g2.setPaint(Color.white);

    // set the composite rule
    //g2.setComposite(AlphaComposite.SrcIn);

    if (vf_ != null) {
      g2.draw(getShape());
      g2.setFont(BeamerManager.smallBoldUpFont);

      String[] l_name = getDisplayableName();
      String str;
      for ( int i = 0 ; i < l_name.length ; i++ ) {
          str = l_name[i].toUpperCase();
          g2.drawString(" "+str, 2, i*10 + (NODE_SIZE/5) ); //(2*NODE_SIZE / 5)
          if (i == 2) break;
      } // for (int i
      l_name = null;
      str = null;
    } // if
    g2 = null;
    //gp = null;
    return;
  } // paintComponentForExport()

    /**
     * @return the hashcode value for this object, based on the virtual folder.
     */
    public int hashCode() {
      int result = 0;

      if (vf_ != null) {
        result = PRIME*result + vf_.hashCode();
      }
      return result;
    } // hashCode()

  /**
   * Get the presentation in N parts of the virtual folder name.
   * @return String[] from vf_.lastComponent(), cut in NAME_PART-size strings.
   */
   public String[] getDisplayableName() {
    String[] result = new String[0];
    if (vf_ != null) {
      if (vf_.getName().lastComponent().length() < NAME_PART)
        return new String[] {vf_.getName().lastComponent()};
      else {

        int arraySize = 1 + ( vf_.getName().lastComponent().length() -
          vf_.getName().lastComponent().length()%NAME_PART )/NAME_PART;
        result = new String[arraySize];
        for (int i = 0 ; i < arraySize - 1 ; i++ ) {
          result[i] = vf_.getName().lastComponent().
            substring(NAME_PART*i,NAME_PART*i+NAME_PART);
        } // for
        result[arraySize - 1] = vf_.getName().lastComponent().substring(
          NAME_PART*(arraySize - 1),
          vf_.getName().lastComponent().length());
      } // else

    } // if (vf_ != null

    return result;
   } // getDisplayableName()

} // GraphicVirtualFolder
