//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import javax.swing.*;
import javax.swing.tree.*;
import java.util.Collections;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.Enumeration;

import ARoad0.Pattern.StaticClass;
import ARoad0.Pattern.MVC_Model;
import ARoad0.Pattern.MVC_View;

import ARoad0.Gui1.InternalFrameModel;
import ARoad0.gWork.ACS_Facade;

import ARoad0.gBaseInterface.ImmutableNameISTree;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.Name;

import ARoad0.gBase.BaseManagerImpl;
import ARoad0.gBase.NameTreeImpl;

/**
 * This utility class is responsible for creating and for updating
 * a viewable tree for the IS structure. All the methods are static.
 * @version 0.7.0
 */
public class ISTree implements MVC_Model, MVC_View, StaticClass {
  // variables
  // the only instance of the class, following singleton pattern
  //private static ISTree                 instance__ = new ISTree();

  private static final int              INITIAL_CAPACITY = 500; // the default value: 101


  // --------------------- static initialization -----------------------------
  static {

  }

  /**
  * only one private constructor
  */
  private ISTree() {
  }

  /**
  * Single method to get an instance.
  */
//  public static synchronized ISTree getInstance(){
//    return instance__;
//  } // getInstance()

  /**
  * Creates a graphical subtree of IS objects, where the last component of the
  * object name is the user object for the tree node.
  * Called by TreeManager.createStructureTree(), ISTreeBaseListenerImpl.
  * @param    _tree to process
  * @param    _iSname name of the IS. Never null.
  * @param    _name of the IS node. Never null. It is _iSname to create the IS in the tree.
  * @param    _parent of _tree under which a node has to be created for _name
  * @return   updated _tree, with a subtree under the new _name node
  */
  public static final JTree createISnodeTree(JTree _tree,
          ImmutableName _iSname, ImmutableName _name,
          DefaultMutableTreeNode _parent) {

    DefaultTreeModel        iSTreeModel = (DefaultTreeModel) _tree.getModel();
    DefaultMutableTreeNode  firstNode = null;
    DefaultMutableTreeNode  node = null;
    DefaultMutableTreeNode  parent = null;
    int                     count = _parent.getChildCount();
    ImmutableNameISTree     nametree = (ImmutableNameISTree)BaseManagerImpl.getInstance().
                                            getISTree(_iSname);

                      // for testing and tracking
                      //System.out.println("in ISTree.createISnodeTree(), _iSname:"+ _iSname+
                        //      ", _name: "+_name+" nametree: "+nametree+" and all IS names:");
                      
                      //for (ImmutableName nam : BaseManagerImpl.getInstance().getL_IS()) {
                        //  System.out.println("---------- "+nam+" equals to _iSname: "+
                          //        nam.equals(_iSname));
                      //}

                      // for testing and tracking
                      //System.out.println("in ISTree.createISnodeTree(), _parent: "+
                        //      _parent.getUserObject()+", insertion at count: "+count);

    List<ImmutableName> list =  nametree.getL_AllChildren(_name);

                      // for testing and tracking
                      //System.out.println("call to ISTree.createISnodeTree(): "+
                        //"for: "+_name+", and the list of direct children is: ");
                      //for (ImmutableName name : list) {
                        //  System.out.println("-------- "+name);
                      //}

    Map<ImmutableName, DefaultMutableTreeNode> m_treenodes = new HashMap(INITIAL_CAPACITY);

    // to start with the direct children
    Collections.reverse(list);

    firstNode = new DefaultMutableTreeNode(_name.lastComponent(), true);
    iSTreeModel.insertNodeInto(firstNode, _parent, count);

    // since there is no method to parse iSTreeModel,
    // uses a map of the current DefaultMutableTreeNodes
    for (ImmutableName name : list ) {
      node = new DefaultMutableTreeNode(name.lastComponent(), true);
      m_treenodes.put(name, node);

      if (nametree.getParent(name).equals(_name)) {
          iSTreeModel.insertNodeInto(node, firstNode, 0);
          //count++;
      } // if
      else {
          parent = m_treenodes.get(nametree.getParent(name));
          iSTreeModel.insertNodeInto(node, parent, 0);
      } // else
    } // for list

    // change the selection in the tree for the _name node
    _tree.setSelectionPath(new TreePath( firstNode ));

    iSTreeModel = null;
    firstNode = null;
    node = null;
    parent = null;
    m_treenodes = null;
    nametree = null;
    list = null;

    return _tree;
  } // createISnodeTree(,,)


  /**
  * Gets the parent TreeNode from the element name, through a searching from the root.
  * _name may not have a current TreeNode in the IS structure tree.
  * No use.
  * @param  _name of an NameISTree element in a registered IS
  * @return the parent. May be null.
  */
  public static final DefaultMutableTreeNode getParentInISTree(ImmutableName _name) {
      // variables
      ImmutableName iSname = BaseManagerImpl.getInstance().getISofName(_name);
      ImmutableName parent = ((NameTreeImpl)BaseManagerImpl.getInstance().
                                getISTree(iSname)).getParent(_name);

      if (parent == null) {
          iSname = null;
          return null;
      }

      return getTreeNodeInISTree(parent);
  } // getParentInISTree()


  /**
  * Gets the TreeNode from the IS node name, through a searching from the root.
  * Called by ISTreeBaseListenerImpl. Calls TreeManager.getTreeNodeInNamingTree().
  * @param  _name of an IS element
  * @return the node. May be null.
  */
  public static final DefaultMutableTreeNode getTreeNodeInISTree(ImmutableName _name) {
      // variables
      Name                      ISName = null;
      boolean                   found = false;
      Enumeration               childEnumeration = ((DefaultMutableTreeNode)
              InternalFrameModel.getTreeInStructure().getModel().getRoot()).children();
      DefaultMutableTreeNode    currentNode = null;

                      // for testing and tracking
                      //System.out.println("call to ISTree.getTreeNodeInISTree() for "+_name);

      if (_name.noOfComponents() != 1) {
          for (Object name : ACS_Facade.getL_IS()) {

                          // for testing and tracking
                          //System.out.println("----- open IS: "+name);

              if (_name.component(0).equals(((ImmutableName)name).component(0))) {
                  ISName = (Name)name;
                  break;
              }
          } // for
      }
      else ISName = (Name)_name;

      if (ISName == null) {

                      // for testing and tracking
                      System.out.println("in ISTree.getTreeNodeInISTree(), ISname is not found"+
                              " for "+_name);

          childEnumeration = null;
          return null;
      }

      if (!childEnumeration.hasMoreElements()) {

                      // for testing and tracking
                      //System.out.println("in ISTree.getTreeNodeInISTree(), childEnumeration"+
                        //      " has no items for "+_name);

          childEnumeration = null;
          return null;
      }

      // searching for through the IS nodes to find the right IS
      while (childEnumeration.hasMoreElements()) {
        currentNode = (DefaultMutableTreeNode) childEnumeration.nextElement();

                      // for testing and tracking
                      //System.out.println("in ISTree.getTreeNodeInISTree(), enumeration on IS node "+
                        //      currentNode.toString());

        // tests to find the next node in the path
        if ( currentNode.getUserObject().equals(ISName.lastComponent()) ) {
          found = true;
          break;
        } // if
      } // while
      if (!found ) {
          childEnumeration = null;
          currentNode = null;

                      // for testing and tracking
                      System.out.println("in ISTree.getTreeNodeInISTree(), IS is not found "+
                              " after the child enumeration on "+
                              "(InternalFrameModel.getTreeInStructure().getModel().getRoot()).children(),"+
                              " for the ISName "+ISName);

          ISName = null;
          return null;
      }

      return TreeManager.getTreeNodeInNamingTree(ISName, currentNode, _name);
  } // getTreeNodeInISTree()

} // class ISTree