//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import java.awt.Component;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
//import javax.swing.ImageIcon;
import javax.swing.Icon;

import ARoad0.ErrB.BaseError;

import ARoad0.gBase.BaseManagerImpl;
import ARoad0.gBase.ISFactoryUtilityImpl;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.ImmutableNameISTree;
import ARoad0.gBaseInterface.ImmutableACS;
import ARoad0.gWork.ACS_Facade;

/**
 * This class is responsible for the look-an-feel setup for the IS structure tree.
 * It adds the icons to the super class DefaultTreeCellRenderer.
 * @version 0.7.0
 */
public class ISTreeCellRenderer extends CommonTreeCellRenderer {

    // constructors

  /**
   * Public constructor. No work.
  */
  public ISTreeCellRenderer() {

  } // ISTreeCellRenderer()

    //
    // --------------- METHODS ---------------------
    //

/**
* Overrides the DefaultTreeCellRenderer method, for setting the right icon and calling
* this method at the end.
* Configures the renderer based on the passed in components.
* The value is set from messaging the tree which ultimately invokes
* <code>toString</code> on <code>_value</code>.
* The icon set is based on leaf and expanded, the open state of the node and
* its type. If it is an ACS, a dedicated icon may be used, if any, through
* a call to ISTree.
* @param _tree to render
* @param _value DefaultMutableTreeNode
* @param _sel see the DefaultTreeCellRenderer method
* @param _expanded see the DefaultTreeCellRenderer method
* @param _leaf see the DefaultTreeCellRenderer method
* @param _row see the DefaultTreeCellRenderer method
* @param _hasFocus see the DefaultTreeCellRenderer method
*/
  public final Component getTreeCellRendererComponent(JTree _tree, Object _value,
                                                boolean _sel,
                                                boolean _expanded,
                                                boolean _leaf, int _row,
                                                boolean _hasFocus) {

      if (_value == _tree.getModel().getRoot()) {
          setOpenIcon(structureIcon);
          setClosedIcon(structureIcon);
          setLeafIcon(structureIcon);
          return super.getTreeCellRendererComponent(_tree, _value,
                                                _sel,
                                                _expanded,
                                                _leaf, _row,
                                                _hasFocus);
      } // if

      ImmutableName name = TreeManager.getTrueUserObjectNameInStructureTree(
                            (DefaultMutableTreeNode) _value);
      ImmutableName iSname = BaseManagerImpl.getInstance().getISofName(name);
      String        type = null;
      boolean       openNode = false;
      ImmutableNameISTree    nametree = BaseManagerImpl.getInstance().getISTree(iSname);
      ImmutableACS  acs = null;
      Icon          icon = null;

      if (nametree == null) {
            // for testing and tracking
            System.out.println("ISTreeCellRenderer."+
              "getTreeCellRendererComponent() error: finds no ImmutableNameISTree"+
              " for the node: "+((DefaultMutableTreeNode)_value).getUserObject()+
              ", iSname: "+iSname+
              ", node name: "+name);
      } // if

      try {
          type = nametree.getType(name);
          openNode = nametree.isOpen(name);
      }
      catch (BaseError b) {
           throw new InternalError("ISTreeCellRenderer.getTreeCellRendererComponent(): "+
                   b.toString());
      }

      if (type == null)
          throw new InternalError("ISTreeCellRenderer.getTreeCellRendererComponent(): "+
                  name+" has no node type.");


      if (ISFactoryUtilityImpl.INFORMATION_SYSTEM_NODE.equals(type)) {
        if (_leaf) {
            if (openNode)
                setLeafIcon(isIcon);
            else
                setLeafIcon(closed_isIcon);
        }
        else {
            if (openNode) {
                if (_expanded) setOpenIcon(isIcon);
                else setClosedIcon(isIcon);
            }
            else {
                if (_expanded) setOpenIcon(closed_isIcon);
                else setClosedIcon(closed_isIcon);
            } // else
        } // else
      } // if IS_NODE

      else if (ISFactoryUtilityImpl.ACS_NODE.equals(type)) {

                                            // for testing and tracking
                                            //System.out.println("call to ISTreeCellRenderer."+
                                              //"getTreeCellRendererComponent()"+
                                              //" for the ACS "+name);

        acs = (ImmutableACS) ACS_Facade.getOpenACS().get(name);

        if (acs == null) {
            if (_leaf)
                setLeafIcon(closed_ACS_ICON);
            else {
                if (_expanded) setOpenIcon(closed_ACS_ICON);
                else setClosedIcon(closed_ACS_ICON);
            }
        } // if (acs == null)

        else if (_leaf) {
            // openNode is true
            icon = acs.getEorIcon();

            if (icon != null) {

                            // for testing and tracking
                            //System.out.println("call to ISTreeCellRenderer."+
                              //"getTreeCellRendererComponent() set the icon "+icon+
                              //" for the ACS "+name);

                setLeafIcon(icon);

            }
            else {

                setLeafIcon(ACS_ICON);
            }
        }

        else {
            // openNode is true
            icon = acs.getEorIcon();

            if (icon != null) {
                if (_expanded) setOpenIcon(icon);
                else setClosedIcon(icon);
            }
            else {
                if (_expanded) setOpenIcon(ACS_ICON);
                else setClosedIcon(ACS_ICON);
            }
        } // else
      } // if ACS_NODE
      
      else if (ISFactoryUtilityImpl.PHYSICAL_COMPONENT_NODE.equals(type)) {
        if (_leaf) {
            if (openNode)
                setLeafIcon(physicalcomponentIcon);
            else
                setLeafIcon(closed_physicalcomponentIcon);
        }
        else {
            if (openNode) {
                if (_expanded) setOpenIcon(physicalcomponentIcon);
                else setClosedIcon(physicalcomponentIcon);
            }
            else {
                if (_expanded) setOpenIcon(closed_physicalcomponentIcon);
                else setClosedIcon(closed_physicalcomponentIcon);
            } // else
        } // else
      } // if PHYSICAL_NODE

      else if (ISFactoryUtilityImpl.LOGICAL_COMPONENT_NODE.equals(type)) {
        if (_leaf) {
            if (openNode)
                setLeafIcon(logicalcomponentIcon);
            else
                setLeafIcon(closed_logicalcomponentIcon);
        }
        else {
            if (openNode) {
                if (_expanded) setOpenIcon(logicalcomponentIcon);
                else setClosedIcon(logicalcomponentIcon);
            }
            else {
                if (_expanded) setOpenIcon(closed_logicalcomponentIcon);
                else setClosedIcon(closed_logicalcomponentIcon);
            } // else
        } // else
      } // if LOGICAL_NODE

      else if (ISFactoryUtilityImpl.SUB_ACS_NODE.equals(type)) {
        if (_leaf) {
            if (openNode)
                setLeafIcon(subacsIcon);
            else
                setLeafIcon(closed_subacsIcon);
        }
        else {
            if (openNode) {
                if (_expanded) setOpenIcon(subacsIcon);
                else setClosedIcon(subacsIcon);
            }
            else {
                if (_expanded) setOpenIcon(closed_subacsIcon);
                else setClosedIcon(closed_subacsIcon);
            } // else
        } // else
      } // if SUBACS_NODE

      else if (ISFactoryUtilityImpl.COMPOSITE_ACS_NODE.equals(type) ||
              ISFactoryUtilityImpl.COMPOSITE_SUB_ACS_NODE.equals(type) ) {
        if (_leaf) {
            if (openNode)
                setLeafIcon(compositeIcon);
            else
                setLeafIcon(closed_compositeIcon);
        }
        else {
            if (openNode) {
                if (_expanded) setOpenIcon(compositeIcon);
                else setClosedIcon(compositeIcon);
            }
            else {
                if (_expanded) setOpenIcon(closed_compositeIcon);
                else setClosedIcon(closed_compositeIcon);
            } // else
        } // else
      } // if COMPOSITE_NODE

      else {
                                // for testing and tracking
                                //System.out.println("ISTreeCellRenderer."+
                                  //"getTreeCellRendererComponent() does not find the type "+type+
                                  //" with the user object class"+
                                  //((DefaultMutableTreeNode)_value).getUserObject().getClass());
      } // else

      name = null;
      type = null;
      iSname = null;
      nametree = null;
      acs = null;
      icon = null;

      Component result = super.getTreeCellRendererComponent(_tree, _value,
                                                _sel,
                                                _expanded,
                                                _leaf, _row,
                                                _hasFocus);

                                // for testing and tracking
                                //System.out.println("call to ISTreeCellRenderer."+
                                  //"getTreeCellRendererComponent() returns "+result+
                                  //" with the user object class"+
                                  //((DefaultMutableTreeNode)_value).getUserObject().getClass());

      return result;
  } // getTreeCellRendererComponent()

} // ISTreeCellRenderer