//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import javax.swing.*;
import javax.swing.tree.*;
import java.util.Enumeration;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Collections;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import ARoad0.Gui1.*;
import ARoad0.ErrW.*;
import ARoad0.ErrB.BaseError;
import ARoad0.Pattern.StaticClass;
import ARoad0.Pattern.MVC_Model;

import ARoad0.gWork.ACS_Facade;
import ARoad0.gWork.ViewInBase_Facade;

import ARoad0.gBaseInterface.*;

import ARoad0.gBase.BaseManagerImpl;
import ARoad0.gBase.ACSFactoryImpl;
import ARoad0.gBase.AclEntryImpl;
import ARoad0.gBase.NameImpl;
import ARoad0.gBase.PrivilegeAbst;
import ARoad0.gBase.ACSFactoryUtilityImpl;
import ARoad0.gBase.ISFactoryUtilityImpl;

/**
 * This class is responsible for managing viewable trees for an ACS or a view,
 * and more precisely the TreeBaseListeners associated to these trees,
 * and for managing the Explorer tree creation. It does not handle the creation
 * of the global structure tree, but manages its tree listener.
 * It is invoked by several Action classes.
 * All methods are static, following the StaticClass pattern.
 * @version 0.7.0
 */
public class TreeManager implements MVC_Model, StaticClass {

  // the only instance of the class, following singleton pattern
  //private static TreeManager            instance__ = new TreeManager();

  private static final int              INITIAL_CAPACITY = 10; // the default value: 101

  // list of TreeBaseListeners to control
  private static List                   l_treeBaseListener__ =
                                        new ArrayList(INITIAL_CAPACITY);

  // these strings signal the main nodes in the Explorer.
  public static final String  OPEN_ENTITIES = "Open Entities";
  public static final String  OPEN_ACS = "Open Access Control Systems";
  public static final String  OPEN_VIEWS = "Open Views for open ACS";
  public static final String  FULL_VIEWS = "Full Views";
  public static final String  NOTHAN_VIEWS = "NoMore-NoLess Views";
  public static final String  META_RIGHTS = "Types of Generic Right";
  public static final String  GRANTING_RIGHTS = "Granting Rights";
  public static final String  DENYING_RIGHTS = "Denying Rights";  
  
  public static final Map     M_EXPLORER_UPPER_NODES = new HashMap(10);

  public static final Name    STRUCTURE_ROOT_NAME = new NameImpl(
                                ISFactoryUtilityImpl.STRUCTURE_ROOT_STRING);

  // --------------------- static initialization -----------------------------
  static {
    M_EXPLORER_UPPER_NODES.put("OPEN_ENTITIES", OPEN_ENTITIES);
    M_EXPLORER_UPPER_NODES.put("OPEN_ACS", OPEN_ACS);
    M_EXPLORER_UPPER_NODES.put("OPEN_VIEWS", OPEN_VIEWS);
    M_EXPLORER_UPPER_NODES.put("FULL_VIEWS", FULL_VIEWS);
    M_EXPLORER_UPPER_NODES.put("NOTHAN_VIEWS", NOTHAN_VIEWS);
    M_EXPLORER_UPPER_NODES.put("META_RIGHTS", META_RIGHTS);
    M_EXPLORER_UPPER_NODES.put("GRANTING_RIGHTS", GRANTING_RIGHTS);    
    M_EXPLORER_UPPER_NODES.put("DENYING_RIGHTS", DENYING_RIGHTS);    
  }
  
  
  // public empty node
  public static final DefaultMutableTreeNode EMPTY_TREE_NODE = new DefaultMutableTreeNode();

  /**
  * only one private constructor
  */
  private TreeManager() {
  }

  /**
  * Only method to get an instance.
  */
//  public static synchronized TreeManager getInstance(){
//    return instance__;
//  } // getInstance()

  /** 
  * Creates a graphical tree of open objects for the explorer.
  * The display order is open ACS, then open Views, then Metarights 
  * which are always opened.
  * Calls ACSTree.createACSExplorer() and ViewTree.createViewExplorer().
  * Called by InternalFrame.openExplorer().
  * @exception BaseError if an ACS in the base is null
  * @return JTree of open objects in the base
  */
  public static final synchronized JTree createExplorer() throws BaseError {
    // variables
    DefaultMutableTreeNode  root = new DefaultMutableTreeNode(OPEN_ENTITIES);
    DefaultMutableTreeNode  root2 = root; // nonsense init
    DefaultMutableTreeNode  root3 = root; // nonsense init
    ACS                     acs;
    ViewInBase              view;
    int                     count = 0;

    // creates the tree
    DefaultTreeModel        treeModel = new DefaultTreeModel(root);
    JTree                   tree = new JTree(treeModel);
    //tree.setRootVisible(false);   // doesn't work?

    tree.setName("Explorer");

    // puts DialogBox as a tree node selection listener
    // for the selectionDialog() method
    tree.getSelectionModel().setSelectionMode(
        TreeSelectionModel.SINGLE_TREE_SELECTION);
    tree.addTreeSelectionListener(DialogBox.getInstance() );

    // adds the TreeCellEditor to display icons
    tree.setCellRenderer(new ExplorerTreeCellRenderer());

    // creates open ACS trees
    root2 = new DefaultMutableTreeNode(OPEN_ACS, true);
    treeModel.insertNodeInto(root2, root, 0);
    for (Iterator ite =
      ACS_Facade.getOpenACS().values().iterator() ;
      ite.hasNext() ; ) {
        acs = (ACS)ite.next();
        // creates the ACS node
        root3 = new DefaultMutableTreeNode(acs, true);
        treeModel.insertNodeInto(root3, root2, count);
        count++;
        // creates the subtree
        try {
          tree = ACSTree.createACSExplorer( acs, tree, root3 );
        } catch (ProcessError p) {
          throw new BaseError("ACS_Facade.getOpenACS() provides one null ACS.");
        }
    } // for (Iterator ite

    // creates open View trees
    root2 = new DefaultMutableTreeNode(OPEN_VIEWS, true);
    treeModel.insertNodeInto(root2, root, 1);
    // create the Architecture node, and create the subtree
    if (ViewInBase_Facade.getArchitectureView() != null)
        ViewTree.createViewExplorer( ViewInBase_Facade.getArchitectureView() );
    for (Iterator ite =
      ViewInBase_Facade.getAllStandardOpenViews().values().iterator() ;
      ite.hasNext() ; ) {
        view = (ViewInBase)ite.next();
        // creates the view node, and create the subtree
        ViewTree.createViewExplorer( view );
    } // for (Iterator ite

    // creates metaright lists
    root2 = new DefaultMutableTreeNode(META_RIGHTS, true);
    treeModel.insertNodeInto(root2, root, 2);
    root = root2;
    root2 = new DefaultMutableTreeNode(GRANTING_RIGHTS, true);
    root3 = new DefaultMutableTreeNode(DENYING_RIGHTS, true);    
    treeModel.insertNodeInto(root2, root, 0);
    treeModel.insertNodeInto(root3, root, 1);
    List l_rights = BaseManagerImpl.getInstance().getL_MetaRights();
    count = 0;
    int index = 0;
    StringMetaRight right = null;
    for (Iterator ite = l_rights.iterator(); ite.hasNext(); ) {
        right = (StringMetaRight) ite.next();
        root = new DefaultMutableTreeNode(right, false); 
        
        if (right.isGrantingRight()) {
            treeModel.insertNodeInto(root, root2, count);
            count++;
        } else {
            treeModel.insertNodeInto(root, root3, index - count);            
        } // else
        index++;
    } // for (Iterator ite   
    
    root = null;
    root2 = null;
    root3 = null;
    treeModel = null;
    acs = null;
    view = null;
    
    return tree;
  } // createExplorer()

  /**
  * Creates a new graphical tree for the IS structure of all the information systems,
  * even the closed ones. They are get from gBase.BaseManagerImpl.
  * It is a tree of DefaultMutableTreeNodes with Strings as user objects.
  * The root user object is STRUCTURE_ROOT_NAME.
  * Creates the tree listener and registers it.
  * Called by gDMak.ActionOpenGlobalStructure.openFrame().
  * Calls BaseManagerImpl.getL_IS().
  * Calls ISTree.createISnodeTree() to create the IS subtree following the order
  * of BaseManagerImpl.getL_IS(), and this is important to display the ACS
  * in the same order in ActionOpenACSyst.displayOneACS().
  * @return JTree of names in the base
  */
  public static final synchronized JTree createStructureTree()
    throws ProcessError {
      // initializes the tree model with one root:
      DefaultMutableTreeNode  root = new DefaultMutableTreeNode(
                                        ISFactoryUtilityImpl.STRUCTURE_ROOT_STRING);
      DefaultTreeModel        iSTreeModel = new DefaultTreeModel(root);
      JTree                   tree = new JTree(iSTreeModel);

      tree.setName(ISFactoryUtilityImpl.STRUCTURE_ROOT_STRING);
      tree.getSelectionModel().setSelectionMode(
        TreeSelectionModel.SINGLE_TREE_SELECTION);

      // hidden root
      //tree.setRootVisible(false);
      tree.setShowsRootHandles(true);

        // adds the TreeCellEditor to display icons
      tree.setCellRenderer(new ISTreeCellRenderer());

      for (ImmutableName name : BaseManagerImpl.getInstance().getL_IS()) {
        ISTree.createISnodeTree(tree, name, name, root);
      }

      // creates the tree listener and registers it
      ISTreeBaseListenerImpl baseListener =
                  new ISTreeBaseListenerImpl(tree, root);
      addTreeBaseListener(baseListener);

      root = null;
      iSTreeModel = null;
      baseListener = null;

      return tree;
  } // createStructureTree()

  /**
  * Adds a listener to the list of baseListeners associated to JTrees in the GUI.
  * TreeBaseListeners assume PropertyChangeEvents from the base objects.
  * Called by ACSTree and ViewTree.
  * @param _listener TreeBaseListener
  */
  public static final void addTreeBaseListener(TreeBaseListener _listener) {
    synchronized (l_treeBaseListener__) {l_treeBaseListener__.add(_listener);}
  } // addTreeBaseListener()

  /**
  * Gets the list of baseListeners associated to JTrees in the GUI.
  * BaseListeners handle PropertyChangeEvents from the base objects.
  * @return unmodifiableList of treeBaseListeners
  */
  public static final List getL_TreeBaseListener() {
    return Collections.unmodifiableList( l_treeBaseListener__ );
  } // getL_TreeBaseListener()

  /**
  * Detect an ACSRun or ViewInBase that has at least one tree base listener.
  * Uses listener.getEorMainEventSource() to find the right listener.
  * Called by gWork.RightsMediatorImpl.propertyChange().
  * @since 0.7.0
  * @param _sourceName of an ACSRun or ViewInBase
  * @return true if _sourceName is the source name of a registered listener
  */
  public static final boolean haveAtLeastOneTreeBaseListener(ImmutableName _sourceName) {
      TreeBaseListener  listener = null;
      boolean           find = false;
      
      // search the right baseListener
      synchronized (l_treeBaseListener__) {
        for (ListIterator ite = l_treeBaseListener__.listIterator() ; ite.hasNext() ; ) {
          listener = (TreeBaseListener) ite.next();
          if (listener.getEorMainEventSource().equals(_sourceName) ) {
                find = true;
                break;
          } // if
        } // for
      } // synchro
      
      listener = null;
      return find;
  } // haveAtLeastOneTreeBaseListener()

  /**
  * Removes the BaseListener associated to a main ACS JTree,
  * defined as the JTree which displays all the ACS in a specific frame,
  * to complete the Explorer displaying.
  * To detect the right baseListener, tests if the ACS tree root
  * is the ACS name, as do the main ACS tree only.
  * Called by the Gui1.InternalFrame inner class for ACS frame closing.
  * The associated JTree may be still visible; this is not controlled here.
  * @see ARoad0.gDMak.ActionClose
  * @param _name the last components (last 4, generally) of the name for the ACSRun which is
  * associated to the baseListener as the main source event
  * @exception SmallError if the baseListener is unknown
  */
  public static final void removeACSMainTreeBaseListener(ImmutableName _name)
    throws SmallError {
      // variables
      TreeBaseListener listener = null;//new ACSTreeBaseListenerImpl();
      DefaultMutableTreeNode treeRoot = null;//new DefaultMutableTreeNode();
      boolean find = false;

                      // for testing and tracking
                      //System.out.println("start of TreeManager.removeACSMainTreeBaseListener() "+
                        //      "for _name: "+_name);

      // searching for the right baseListener
      synchronized (l_treeBaseListener__) {
        for (ListIterator ite = l_treeBaseListener__.listIterator() ; ite.hasNext() ; ) {
          listener = (TreeBaseListener) ite.next();
          treeRoot = (DefaultMutableTreeNode)listener.getEorTreeModel().getRoot();

                      // for testing and tracking
                      //System.out.println("comparison of treeRoot.getUserObject(): "+
                        //      treeRoot.getUserObject()+" and listener.getEorMainEventSource() "+
                          //    listener.getEorMainEventSource());

          if (listener.getEorMainEventSource().lastComponent().equals(
                  _name.lastComponent()) ){
              // test if the ACS tree root is the ACS name,
              // as do the main ACS tree only
              if (treeRoot.getUserObject().equals(listener.getEorMainEventSource() )) {
                  find = true;
                  break;
              } // if (treeRoot
          } // if (listener.
        } // for

        // exception
        if ( !find ) throw new SmallError(
          "in TreeManager.removeACSMainTreeBaseListener(), the baseListener is unknown.");
        else {
        // remove the listener
          listener.finalizeForUser();
          l_treeBaseListener__.remove(listener);
        }
      } // synchro

      listener = null;
  } // removeACSMainTreeBaseListener()

  /**
  * Removes the baseListener associated to a JTree in the GUI.
  * Called by removeAllTreeBaseListenerFor().
  * The associated JTree may be updated or still visible; this is not controlled.
  * @param _sourceName of the ACSRun or ViewInBase associated to the baseListener
  * as the main source event
  * @param _model (usually DefaultTreeModel) associated to the JTree
  * displaying _source
  * @exception SmallError if the baseListener is unknown
  */
  public static final void removeTreeBaseListener(ImmutableName _sourceName,
    TreeModel _model) throws SmallError {
      // variables
      TreeBaseListener listener = null;
      boolean find = false;
      // search the right baseListener
      synchronized (l_treeBaseListener__) {
        for (ListIterator ite = l_treeBaseListener__.listIterator() ; ite.hasNext() ; ) {
          listener = (TreeBaseListener) ite.next();
          if (listener.getEorMainEventSource().equals(_sourceName) &&
              listener.getEorTreeModel() == _model) {
                find = true;
                break;
          } // if
        } // for
        // exception
        if (find == false) throw new SmallError(
          "in TreeManager.removeTreeBaseListener(), the baseListener is unknown.");

        // remove the listener
        listener.finalizeForUser();
        l_treeBaseListener__.remove(listener);
      } // synchro
      listener = null;
  } // removeTreeBaseListener()

  /**
  * Removes in the list of baseListeners all those associated to a specified
  * event source.
  * Called by gDMak.ActionClose and ActionNewACSyst.
  * The associated JTrees may be updated or still visible; this is not controlled.
  * @param _sourceName of the BaseManager, ACSRun or ViewInBase associated to the baseListener
  * as the main event source
  * @exception SmallError if the baseListener is unknown
  */
  public static final void removeAllTreeBaseListenerFor(ImmutableName _sourceName)
    throws SmallError {
      // variables
      TreeBaseListener  listener = null;
      List              list = new ArrayList(INITIAL_CAPACITY);
      boolean           find = false;
      // searching for the right baseListeners
      for (ListIterator ite = l_treeBaseListener__.listIterator() ; ite.hasNext() ; ) {
        listener = (TreeBaseListener) ite.next();
        if (listener.getEorMainEventSource().equals(_sourceName) ) {
          list.add(listener);
          find = true;
        }
      } // for
      // exception
      if (!find) {

                  // for testing and tracking
                  //System.out.println("in TreeManager.removeACSMainTreeBaseListener(), "+
                    //"the source is unknown: "+_sourceName);

          listener = null;
          list = null;
          return;
      }
      else {

                  // for testing and tracking
                  //System.out.println("in TreeManager.removeACSMainTreeBaseListener(), "+
                    //"the source is removed: "+_sourceName);
      }

          //throw new SmallError(
            //"in TreeManager.removeAllTreeBaseListenerFor(source), the source is unknown.");
      
      // removes the listeners
      for (ListIterator ite = list.listIterator() ; ite.hasNext() ; ) {
        listener = (TreeBaseListener) ite.next();
        removeTreeBaseListener(listener.getEorMainEventSource(),
            listener.getEorTreeModel() );
      } // for
      list = null;
      listener = null;
  } // removeAllTreeBaseListenerFor()

  /**
  * Analyzes the selected node in the explorer to get the main information.
  * The returned array has the format: <P>
  *   0- String type of the selected base object: <P>
  *     - possible values are "acs", "acsrun", "resource", "actor", "groupid", "userid", <P>
  *     "aclentry", "view", "metaright", "acsright", "virtualfolder", "privilege", <P>
  *     or "" if it is an another selected node <P>
  *     - is null if there is no selected object in the explorer <P>
  *   1- ImmutableName of the base object - may be null.
  *     If it is an AclEntry or a Privilege, the name is not the detailled name <P>
  *   2- JInternalFrame open main frame of the base object - may be null <P>
  * Called by ActionUtilities, ActionOpenBeamer and ActionOpenSketcher.
  * @return Object[] of three objects: <P>
  *   Returns {null,null,null} if no frame is selected, or a no-workable frame,
  *   or if the selected node in the explorer is a main or a second node
  *   ('OPEN_ACS' node, or '<network_subacs>' under RESOURCES for instance).
  */
  public static final Object[] analyzeSelectedNodeInExplorer() {

    // if there is no selection in the tree
    if (InternalFrameModel.getTreeInExplorer().
      getSelectionPath() == null) return new Object[] {null, null, null};

    DefaultMutableTreeNode  selectedNodeInExplorer =
      (DefaultMutableTreeNode)InternalFrameModel
      .getTreeInExplorer().getSelectionPath().getLastPathComponent();
    TreeNode[]              l_selectedPath = selectedNodeInExplorer.getPath();


                        // for testing and tracking
                     //System.out.println("in TreeManager.analyzeSelectedNodeInExplorer(), "+
                      //"selectedNodeInExplorer is: "+selectedNodeInExplorer);

    // if it is the tree root or one of the main nodes
    if ( M_EXPLORER_UPPER_NODES.containsValue( selectedNodeInExplorer.getUserObject()) ||
            ACSTree.M_ACS_TREE_UPPER_NODES.containsValue(selectedNodeInExplorer.getUserObject())) {
                selectedNodeInExplorer = null;
                l_selectedPath = null;
                return new Object[] {null, null, null};
    } // if
    
    // if it is a second node under GROUPS (not RESOURCES) like '<network_subacs>'
    // or '<capability_tree>'
    if (l_selectedPath.length == 5 && 
            ((DefaultMutableTreeNode)l_selectedPath[3]).getUserObject().equals(
                ACSFactoryUtilityImpl.GROUPS) &&
            ((DefaultMutableTreeNode)l_selectedPath[4]).getUserObject() instanceof String &&
            (((String)((DefaultMutableTreeNode)l_selectedPath[4]).getUserObject()).endsWith("_subacs>") ||
            ((String)((DefaultMutableTreeNode)l_selectedPath[4]).getUserObject()).endsWith("_tree>"))) {
                selectedNodeInExplorer = null;
                l_selectedPath = null;
                return new Object[] {null, null, null};        
    } // if

    if (selectedNodeInExplorer == null) {
                l_selectedPath = null;
                return new Object[] {null, null, null};  
    }

    // initialization
    String                  type = "";
    JInternalFrame          mainFrame = null;
    ImmutableName           name = NameImpl.EMPTY_INSTANCE;

    boolean                 external = false;
    boolean                 isInResourceTree = false;
    String                  displayedName = "";
    Object                  obj = null;
    ImmutableName[]         l_Names = new ImmutableName[2];

    // DETECTS AN ACSRUN OR ACS UNDER OPEN_ACS
    if (((DefaultMutableTreeNode)selectedNodeInExplorer.getParent())
        .getUserObject().equals(OPEN_ACS)) {
          if (selectedNodeInExplorer.getUserObject() instanceof ACSFactoryImpl)
              type = "acsrun";
          else type = "acs";
          // the ACS name is the name associated to the selected node
          name = (ImmutableName)
            ((ACSRun)selectedNodeInExplorer.getUserObject()).getName();
          // gets the main frame associated to this name
          mainFrame = (JInternalFrame)InternalFrameModel
            .getMainFramesOfBaseObjects().get(name);
    } // if
    
    // DETECTS ACS in the Architecture view
    else if (((DefaultMutableTreeNode)selectedNodeInExplorer.getParent())
        .getUserObject().equals( ViewInBase_Facade.getArchitectureView() )) {
          type = "acs";
          // the ACS name is the name associated to the selected node
          name = (ImmutableName)
            ((ACS)selectedNodeInExplorer.getUserObject()).getName();
          // gets the main frame associated to this name
          mainFrame = (JInternalFrame)InternalFrameModel
            .getMainFramesOfBaseObjects().get(name);

                        // for testing and tracking
                     //System.out.println("in TreeManager.analyzeSelectedNodeInExplorer(), "+
                      //"an ACS in the 'Architecture' view is selected: "+name);          
          
    } // if

    // DETECTS A BASEOBJECT IN A STANDARD VIEW
    else if ( selectedNodeInExplorer.getLevel() > 3 &&
      ((DefaultMutableTreeNode)selectedNodeInExplorer.getParent().getParent().getParent())
        .getUserObject().equals(OPEN_VIEWS)) {
          if (selectedNodeInExplorer.getUserObject() instanceof ImmutableEligibleParty) {
            if (selectedNodeInExplorer.getUserObject() instanceof ImmutableResource)
              type = "actor";
            else if (selectedNodeInExplorer.getUserObject() instanceof ImmutableGroupID)
              type = "groupid";
            else if (selectedNodeInExplorer.getUserObject() instanceof ImmutableUserID)
              type = "userid";
          } // if
          else if (selectedNodeInExplorer.getUserObject() instanceof ImmutableResource) {
              type = "resource";
          } // else
          else if (selectedNodeInExplorer.getUserObject() instanceof VirtualFolder) {
              type = "virtualfolder";
          } // else
          // the name is the name associated to the selected node
          name = ((BaseObject)selectedNodeInExplorer.getUserObject()).getName();
          // gets the main frame associated to this name
          mainFrame = null;


                        // for testing and tracking
                     //System.out.println("in TreeManager.analyzeSelectedNodeInExplorer(), "+
                      //"a BaseObject in a view is selected, and its BaseObject name is: "+name);

    } // else if

    // DETECTS A STANDARD VIEW or the Architecture view
    else if ((selectedNodeInExplorer.getLevel() > 2 &&
            ((DefaultMutableTreeNode)selectedNodeInExplorer.getParent().getParent())
                .getUserObject().equals(OPEN_VIEWS) ) ||
            selectedNodeInExplorer.getUserObject().equals(ViewInBase_Facade.getArchitectureView()) ) {
          type = "view";
          // the view name is the name associated to the selected node
          name = ((BaseObject)selectedNodeInExplorer.getUserObject()).getName();
          // gets the main frame associated to this name
          mainFrame = (JInternalFrame)InternalFrameModel
            .getMainFramesOfBaseObjects().get(name);


                        // for testing and tracking
                     //System.out.println("in TreeManager.analyzeSelectedNodeInExplorer(), "+
                      //"a view is selected and its name is: "+name+" with the main frame "+mainFrame);

    } // else if
    
    // DETECTS A METARIGHT
    else if ( ((DefaultMutableTreeNode)selectedNodeInExplorer.getParent().getParent())
        .getUserObject().equals(META_RIGHTS) ) {
          type = "metaright";
          // the name is the name associated to the selected node
          name = ((BaseObject)selectedNodeInExplorer.getUserObject()).getName();
          // gets the main frame associated to this name
          mainFrame = null;


                        // for testing and tracking
                     //System.out.println("in TreeManager.analyzeSelectedNodeInExplorer(), "+
                     // "a BaseObject in a view is selected, and its BaseObject name is: "+name);

    } // else if
    
    // DETECTS AN ACL ENTRY at the depth 4 or 5
    // and the returned name is the BaseObject name of the AclEntry,
    // that is, in short: AcsName, ResourceName, EligiblePartyName, grant/deny
    // (see gBase.AclEntryImpl.getName())
    else if (l_selectedPath.length > 4 &&
      ((DefaultMutableTreeNode)l_selectedPath[3]).getUserObject().equals(
        ACSFactoryUtilityImpl.ACLENTRIES) ) {
              // DETECT AN ACL ENTRY
              if (  // simple ACL - test of String is unuseful
                      (l_selectedPath.length == 5 &&
                        !((String)((DefaultMutableTreeNode)l_selectedPath[4]).
                          getUserObject()).endsWith("_subacs>")) ||
                    // or ACL of a SubACS
                      l_selectedPath.length > 5 ) {

                    type = "aclentry";
                    displayedName = (String) selectedNodeInExplorer.getUserObject();

                      // finds the ep and resource names
                    try {
                      name = AclEntryImpl.EMPTY_INSTANCE.getNameFromDetailledName(
                              // finds the ACS at the depth 2 in the explorer tree
                              (ACS)((DefaultMutableTreeNode)l_selectedPath[2]).getUserObject(),
                              displayedName);
                    } catch (BaseError b) {
                      throw new InternalError("in TreeManager.analyzeSelectedNodeInExplorer(), "+
                        "the viewable name of the AclEntry is not correct");
                    }

                                // for testing and tracking
                             //System.out.println("in TreeManager.analyzeSelectedNodeInExplorer(), "+
                              //"an aclEntry is selected and its BaseObject name is: "+name);

                    mainFrame = null;
              } // if ((l_selectedPath.length == 5
              else {
                  // '_subacs>' node
                  name = null;
                  mainFrame = null;
                  selectedNodeInExplorer = null;
                  l_selectedPath = null;
                  displayedName = null;
                  l_Names = null;
                  return new Object[] {"", null, null};                  
              } // else
    } // else if

    // DETECTS A PRIVILEGE at the depth 4 or 5
    // and the returned name is the BaseObject name of the Privilege
    // (see gBase.PrivilegeForTypeImpl.getName() et gBase.PrivilegeForLinksImpl.getName()).
    // The LinkedPrivilege linked objects are detected further.
    else if (l_selectedPath.length == 5 &&
      ((DefaultMutableTreeNode)l_selectedPath[3])
          .getUserObject().equals(ACSFactoryUtilityImpl.PRIVILEGES) ) {
              // DETECT A PRIVILEGE

                    type = "privilege";
                    displayedName = (String) selectedNodeInExplorer.getUserObject();

                      // finds the type and the short name
                    try {
                      name = PrivilegeAbst.EMPTY_INSTANCE.getNameFromDetailledName(
                              // finds the ACS name at the depth 2 in the explorer tree
                              (ACS)((DefaultMutableTreeNode)l_selectedPath[2]).
                                getUserObject(), displayedName);
                    } catch (BaseError b) {
                      throw new InternalError("in TreeManager.analyzeSelectedNodeInExplorer(), "+
                        "the viewable name of the Privilege is not correct");
                    }

                                // for testing and tracking
                             //System.out.println("in TreeManager.analyzeSelectedNodeInExplorer(), "+
                             // "an aclEntry is selected and its BaseObject name is: "+name);

                    mainFrame = null;
    } // else if

    // DETECTS AN INTERNAL ENTITY or AN EXTERNAL ENTITY if its path contains a known node
    // at the depth 3 or 4, and if there are some entities below this node
    else if (l_selectedPath.length > 4) {
        
          // ----------- start of 'type' init ------------------------
          if ( ((DefaultMutableTreeNode)l_selectedPath[3])
          .getUserObject().equals(ACSFactoryUtilityImpl.ACTORS) ) type = "actor";
          else if ( ((DefaultMutableTreeNode)l_selectedPath[3])
          .getUserObject().equals(ACSFactoryUtilityImpl.USERS) ) type = "userid";
          else if ( ((DefaultMutableTreeNode)l_selectedPath[3])
          .getUserObject().equals(ACSFactoryUtilityImpl.GROUPS) ) type = "groupid";
          else if ( ((DefaultMutableTreeNode)l_selectedPath[3])
          .getUserObject().equals(ACSFactoryUtilityImpl.RESOURCES) ) {
              type = "resource";
              isInResourceTree = true;
          }
          else if ( ((DefaultMutableTreeNode)l_selectedPath[3])
          .getUserObject().equals(ACSFactoryUtilityImpl.ACSRIGHTS) ) type = "acsright";

          // VirtualFolders, their members and children in a tree (which are VirtualFolders)
          else if ( ((DefaultMutableTreeNode)l_selectedPath[3])
            .getUserObject().equals(ACSFactoryUtilityImpl.RESVFOLDERS) ||
                  ((DefaultMutableTreeNode)l_selectedPath[3])
            .getUserObject().equals(ACSFactoryUtilityImpl.EPVFOLDERS)) {
              if (l_selectedPath.length == 5)
                  type = "virtualfolder";
              else if (selectedNodeInExplorer.getUserObject() instanceof ImmutableEligibleParty) {
                if (selectedNodeInExplorer.getUserObject() instanceof ImmutableResource)
                  type = "actor";
                else if (selectedNodeInExplorer.getUserObject() instanceof ImmutableGroupID)
                  type = "groupid";
                else if (selectedNodeInExplorer.getUserObject() instanceof ImmutableUserID)
                  type = "userid";
              } // if
              else if (selectedNodeInExplorer.getUserObject() instanceof ImmutableResource) {
                  type = "resource";
              } // else
              else if (selectedNodeInExplorer.getUserObject() instanceof VirtualFolder) {
                  type = "virtualfolder";
              } // else
          } // else if

          // LinkedPrivilege linked objects
          else if ( l_selectedPath.length > 5 && ((DefaultMutableTreeNode)l_selectedPath[3])
            .getUserObject().equals(ACSFactoryUtilityImpl.PRIVILEGES) ) {
              if (selectedNodeInExplorer.getUserObject() instanceof ImmutableEligibleParty) {
                if (selectedNodeInExplorer.getUserObject() instanceof ImmutableResource)
                  type = "actor";
                else if (selectedNodeInExplorer.getUserObject() instanceof ImmutableGroupID)
                  type = "groupid";
                else if (selectedNodeInExplorer.getUserObject() instanceof ImmutableUserID)
                  type = "userid";
              } // if
              else if (selectedNodeInExplorer.getUserObject() instanceof ImmutableResource) {
                  type = "resource";
              } // else
              else if (selectedNodeInExplorer.getUserObject() instanceof VirtualFolder) {
                  type = "virtualfolder";
              } // else
          } // else if
          
          else if ( ((DefaultMutableTreeNode)l_selectedPath[3])
          .getUserObject().equals(ACSFactoryUtilityImpl.EEPARTIES) ) {

            if (selectedNodeInExplorer.getUserObject().equals(ACSFactoryUtilityImpl.EEACTORS)
              || selectedNodeInExplorer.getUserObject().equals(ACSFactoryUtilityImpl.EGROUPS)
              || selectedNodeInExplorer.getUserObject().equals(ACSFactoryUtilityImpl.EUSERS)) {
                return new Object[] {null, null, null};
            } // if

            external = true;
            if (l_selectedPath.length > 5 &&
              ((DefaultMutableTreeNode)l_selectedPath[4])
              .getUserObject().equals(ACSFactoryUtilityImpl.EEACTORS) ) type = "actor";
            else if (l_selectedPath.length > 5 &&
              ((DefaultMutableTreeNode)l_selectedPath[4])
              .getUserObject().equals(ACSFactoryUtilityImpl.EGROUPS) ) type = "groupid"; // false for userids under the group node
            else if (l_selectedPath.length > 5 &&
              ((DefaultMutableTreeNode)l_selectedPath[4])
              .getUserObject().equals(ACSFactoryUtilityImpl.EUSERS) ) type = "userid";
          } // if

          else if ( ((DefaultMutableTreeNode)l_selectedPath[3])
          .getUserObject().equals(ACSFactoryUtilityImpl.ECRESOURCES) ) {

            if (selectedNodeInExplorer.getUserObject().equals(ACSFactoryUtilityImpl.ECACTORS)
              || selectedNodeInExplorer.getUserObject().equals(ACSFactoryUtilityImpl.ECOTHER_RESOURCES)) {
                return new Object[] {null, null, null};
            } // if

            external = true;
            if (l_selectedPath.length > 5 &&
              ((DefaultMutableTreeNode)l_selectedPath[4])
              .getUserObject().equals(ACSFactoryUtilityImpl.ECACTORS) ) type = "actor";
            else type = "resource";
          } // else if
          
          // ----------- end of 'type' init ------------------------

          // the name is derived from the selected node path,
          // and the process changes for the internal resources
          if (!external && isInResourceTree) {
              
              ACS acs = (ACS)((DefaultMutableTreeNode)l_selectedPath[2])
                 .getUserObject();
              name = ACSTreeUtilities.getResourceInTree(acs.getName(), 
                      (DefaultMutableTreeNode)selectedNodeInExplorer);
              
          } else if (!external) {
              name = ((BaseObject) selectedNodeInExplorer.getUserObject()).getName();

                        // for testing and tracking
                     //System.out.println("in TreeManager.analyzeSelectedNodeInExplorer(), "+
                      //"the non-external/non-resource BaseObject name is: "+name);              
              
              // for an account under a group
              if (type.equals("groupid") && 
                      selectedNodeInExplorer.getUserObject() instanceof ImmutableUserID)
                            type = "userid";
          } else name = (ImmutableName) selectedNodeInExplorer.getUserObject();

          mainFrame = null;
    } // else if
    
    else {
      name = null;
      mainFrame = null;
      selectedNodeInExplorer = null;
      l_selectedPath = null;
      displayedName = null;
      l_Names = null;
      obj = null;
      return new Object[] {"", null, null};
    } // else

    selectedNodeInExplorer = null;
    l_selectedPath = null;
    displayedName = null;
    l_Names = null;
    obj = null;
            
                            // for testing and tracking
                            //System.out.println("in TreeManager.analyzeSelectedNodeInExplorer()"+
                              //  " the selected node in the explorer is: ");
                                //System.out.println("---------- type: "+type);
                                //System.out.println("---------- name: "+name);
                                //System.out.println("---------- mainFrame: "+mainFrame);
    
    return new Object[] {type, name, mainFrame};
  } // analyzeSelectedNodeInExplorer()

  /**
   * Selects the object node if it exists and return +1, or return -1 if it is not found.
   * Does not select the standard nodes like 'Open Entities'.
   * Calls ACSTreeUtilities.getDetailledNameNodeInExplorer() and getCurrentObjectNode().
   * @param _selectedObject displayed in the explorer and to select
   * @return +1 if found and -1 otherwise
   */
  public static final int selectNodeInExplorer(BaseObject _selectedObject) {
    // variables
    DefaultMutableTreeNode node = null; 

                            // for testing and tracking
                            //System.out.println("TreeManager.selectNodeInExplorer(): starting ");

    
    // changes the selection in the explorer
    if (_selectedObject instanceof DetailledName && _selectedObject instanceof ACSObject) {
        node = ACSTreeUtilities.getDetailledNameNodeInExplorer(
          (DetailledName) _selectedObject, ((ACSObject)_selectedObject).getAcsName());
    } // if
    else if (_selectedObject instanceof BaseObject) {
        node = ACSTreeUtilities.getCurrentObjectNode(
          (DefaultMutableTreeNode) InternalFrameModel.getTreeInExplorer().getModel().getRoot(),
          _selectedObject);
    } // if
    
    if (node == null) {
        
            // for testing and tracking
            System.out.println("TreeManager.selectNodeInExplorer() "+
              "null node to display ");

        node = null;
        return -1;
    }
            
    TreePath path = new TreePath(node.getPath());
    try {
        InternalFrameModel.getTreeInExplorer().setSelectionPath(path);
    }
    catch (Exception e) {
        System.out.println("in TreeManager.selectNodeInExplorer(), "+
                "InternalFrameModel.getTreeInExplorer().setSelectionPath(path) throws:");
        e.printStackTrace();
    }

                      // for testing and tracking
                      //System.out.println("TreeManager.selectNodeInExplorer(): the new selected path is: "+
                        //      path.toString());

    node = null;
    path = null;          
    return +1;
  } // selectNodeInExplorer()
  
  /**
  * Gets the true user object in the explorer, managing all the possible cases.
  * For resources in the acs resource tree, the resource is returned
  * after its name has been rebuilt up.
  * For the DetailledNames of AclEntries and Privileges, the getACSObjectFromDetailledName
  * method of ACS_Facade is called.
  * For the Names of external entities of an acs, the entity is returned.
  * For main nodes in an acs tree, the title of the node is returned.
  * For other nodes, the BaseObject is the node user object. 
  * Called by DialogBox.valueChanged().  
  * @param _node of the tree explorer or of an acs main frame. 
  * @return the user object, which may be a String (main nodes) or
  *    a BaseObject. May ne null.
  */
  public static final Object getTrueUserObjectInTreeExplorer(DefaultMutableTreeNode _node) {
    if (_node == null) return null;
    Object obj = _node.getUserObject();

    if (M_EXPLORER_UPPER_NODES.containsValue(obj) ||
            ACSTree.M_ACS_TREE_UPPER_NODES.containsValue(obj) ) {
                 return obj;
    }
    // the most common case
    else if (obj instanceof BaseObject) {
        return obj;    
    // case of external entities in acs
    } else if (obj instanceof ImmutableName) {
        try {
            return ACS_Facade.getEorBaseObject((ImmutableName) obj);
        } catch (BaseError b) {
            throw new InternalError("in TreeManager.getTrueUserObjectInTreeExplorer()"+
                    "for the name "+
                    obj+" , this exception is thrown: "+b.toString());
        } // catch
    // for AclEntries, the parent node is 'ACLENTRIES' and obj is a String
    } else if (((DefaultMutableTreeNode)_node.getParent()).getUserObject().equals(
        ACSTree.M_ACS_TREE_UPPER_NODES.get("ACLENTRIES") )) {

            ACS acs = (ACS)((DefaultMutableTreeNode)_node.getParent().
                    getParent()).getUserObject();
            try {
                obj = ACS_Facade.getACSObjectFromDetailledName(acs, (String) obj);
            } catch (BaseError b) {
                throw new InternalError("in TreeManager.getTrueUserObjectInTreeExplorer()"+
                        "for the aclentry "+_node.getUserObject()+
                        " , the exception is thrown: "+b.toString());
            } // catch
            acs = null;
            return obj;
    // for Privileges, the parent node is 'PRIVILEGES' and obj is a String
    } else if (((DefaultMutableTreeNode)_node.getParent()).getUserObject().equals(
        ACSTree.M_ACS_TREE_UPPER_NODES.get("PRIVILEGES") )) {

            ACS acs = (ACS)((DefaultMutableTreeNode)_node.getParent().
                    getParent()).getUserObject();
            try {
                obj = ACS_Facade.getACSObjectFromDetailledName(acs, (String) obj);
            } catch (BaseError b) {
                throw new InternalError("in TreeManager.getTrueUserObjectInTreeExplorer()"+
                        "for the privilege "+_node.getUserObject()+
                        " , the exception is thrown: "+b.toString());
            } // catch
            acs = null;
            return obj;

    } else if (!ACSTreeUtilities.isInResourceTree(_node)) return null;
    
    // The user object is a resource to find.
    // searches for the 'RESOURCES' node, just under the ACS node, to find the acs name
    Object[] l_path = _node.getUserObjectPath();
    int index = -1;
    for (int i = 0; i < l_path.length; i++) {
        if (l_path[i].equals(ACSTree.M_ACS_TREE_UPPER_NODES.get("RESOURCES"))) {
            index = i;
            break;
        }
    } // for i
    if (index == -1) throw new InternalError("in TreeManager.getTrueUserObjectInTreeExplorer()"+
            " , the node in the resources acs tree cannot be found.");
    
    // uses index-1 to have the acs, in the explorer or in the acs main frame
    ImmutableName acsName = ((ACS)l_path[index-1]).getName();
    ImmutableName name = ACSTreeUtilities.getResourceInTree(acsName, _node);
    Resource result = (Resource)((ACS)l_path[index-1]).getEorM_Resources().get(name);
    
    l_path = null;
    acsName = null;
    name = null;
    obj = null;

                            // for testing
                        //System.out.print("in TreeManager.getTrueUserObjectInTreeExplorer(), "+
                          //      "the result is "+result+"\n");


    return result;
  } // getTrueUserObjectInTreeExplorer()
  
  /**
  * Analyzes the selected node in the global structure, if any.
  * Called by ActionOpenGlobalStructure.valueChanged().
  * @return DefaultMutableTreeNode selected node in the structure. May be null.
  */
  public static final DefaultMutableTreeNode analyzeSelectedNodeInStructure() {
    if (InternalFrameModel.getTreeInStructure() == null) return null;
      // if there is no selection in the tree
    if (InternalFrameModel.getTreeInStructure().getSelectionPath() == null)
        return null;

    return (DefaultMutableTreeNode) InternalFrameModel
      .getTreeInStructure().getSelectionPath().getLastPathComponent();
  } // analyzeSelectedNodeInStructure()

  /**
  * Since the element in the IS structure is not the user object of the node,
  * gets the element through the tree path of Strings from the IS node.
  * Called by ActionOpenGlobalStructure.valueChanged() and
  * ISTreeCellRenderer.getTreeCellRendererComponent().
  * @param    _node is in the IS structure tree
  * @return   the element name. May be the IS name. May be null, mainly if _node
  *     is not under the IS node or if an argument is null.
  */
  public static final ImmutableName getTrueUserObjectNameInStructureTree(
          DefaultMutableTreeNode _node) {

      if (_node == null) return null;

      Name      name = new NameImpl();
      Object[]  l_components = (Object[])_node.getUserObjectPath();

      if (l_components.length == 1) { // for the root STRUCTURE_ROOT_STRING
          return name.extended((String)l_components[0]);
      } // if

      for (int i = 1; i < l_components.length; i++) {
          name.extended((String)l_components[i]);
      } // for i

                // for testing
             //System.out.print("in TreeManager.getTrueUserObjectInStructureTree(), "+
              //      "the returned name is "+result);

    l_components = null;
    return name;
  } // getTrueUserObjectNameInStructureTree()

  /**
  * Gets the TreeNode for a Name through the tree path of Strings from a ancestor node
  * which has to be an ancestor of the result.
  * A naming tree is used in the subtree of resources for an ACS in the explorer,
  * and also in the IS structure tree.
  * The element in the naming tree is not the user object of the node,
  * but it is a Name for which each component is in a TreeNode to set a tree path.
  * This is the inverse method of getTrueUserObjectNameInStructureTree().
  * Called by ACSTreeUtilities.getCurrentObjectNode() and ISTree.
  * @param    _ancestorName is the name of the ACS or the IS which owns _name
  * @param    _rootNode is the root of the naming tree. It is associated
  *     to the resources node for an ACS in the explorer, or to the IS.
  * @param    _name for which the TreeNode under _ancestorNode is returned
  * @return   the TreeNode in the tree for _name. May be null.
  */
  public static final DefaultMutableTreeNode getTreeNodeInNamingTree(
          ImmutableName _ancestorName, DefaultMutableTreeNode _rootNode,
          ImmutableName _name) {

    // for ISTree when _name is the IS name
    if (_name.equals(_ancestorName)) return _rootNode;

    Name                    name = (Name)_ancestorName.clone();
    DefaultMutableTreeNode  result = null;
    Enumeration             enumeration = null;
    boolean                 found = false;
    boolean                 found_two = false;
    DefaultMutableTreeNode  currentNode = _rootNode;

                      // for testing and tracking
                      //System.out.println("in TreeManager.getTreeNodeInNamingTree(), "+
                        //"_name is: " + _name);
                        //", the current name is: " + name+
                        //", and ((Name)_name.clone()).without( (Name)_ancestorName ): "+
                        //((Name)_name.clone()).without( (Name)_ancestorName ));

    // following the obj name components,
    // goes down through the naming tree from currentNode
    // to find each node in the path to the result
    for (Iterator ite = ((Name)_name.clone()).without( (Name)_ancestorName ).
      immutableComponents().iterator();  ite.hasNext() ; ) {

          name.extended( (String)ite.next() );

                      // for testing and tracking
                      //System.out.println("-------- the current name is: " + name);
                      //System.out.println("-------- the current node is: "+
                        //      currentNode.getUserObject());

          // searching through the children to find the right path
          enumeration = currentNode.children();
          found = false;
          while (enumeration.hasMoreElements()) {
            currentNode = (DefaultMutableTreeNode) enumeration.nextElement();

                      // for testing and tracking
                      //System.out.println("-------- the enumeration name is: " + name);

            // tests to find the next node in the path
            if ( currentNode.getUserObject().equals(name.lastComponent()) ) {
              found = true;
              break;
            } // if
          } // while

          if (!found ) {
            throw new InternalError("In TreeManager.getTreeNodeInNamingTree(),"+
              " wrong searching when the current resource is: "+name+
              " and the current node is "+currentNode.getUserObject() );
          }
          if (name.noOfComponents() == _name.noOfComponents() ) {
            found_two = true;
            break;
          } // if (name.noOfComponents()

    } // for(Iterator ite

    if (!found_two) result = null;
    else result = currentNode;


              // for testing and tracking
              //if (result != null) {
                //  System.out.println("in TreeManager.getTreeNodeInNamingTree(), "+
                  //  "_name is: " + _name+
                    //", _ancestorName: " + _ancestorName+
                    //", and the result is: "+result.getUserObject());
              //}

    currentNode = null;
    name = null;
    enumeration = null;
    return result;
  } // getTreeNodeInNamingTree()

} // class TreeManager