//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Gui2;

import javax.swing.*;
import javax.swing.tree.*;
import java.util.Iterator;
import java.util.Map;

import ARoad0.Gui1.*;
import ARoad0.Pattern.StaticClass;
import ARoad0.gDMak.ActionOpenArchitecture;

import ARoad0.gBaseInterface.ViewInBase;
import ARoad0.gBaseInterface.NoThanViewInBase;
import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gWork.ViewInBase_Facade;

/**
 * This class is responsible for constructing viewable trees for a view.
 */
public class ViewTree implements StaticClass {
  // variables
  // the only instance of the class, following singleton pattern
  private static ViewTree             instance__ = new ViewTree();

  private static final int            INITIAL_CAPACITY = 10; // the default value: 101

  /**
  * only one private constructor
  */
  private ViewTree() {
  }

  /**
  * Only method to get an instance.
  */
//  public static synchronized ViewTree getInstance(){
//    return instance__;
//  } // getInstance()

  /**
  * Creates a graphical flat list of an open view,
  * and creates the ViewTreeBaseListener associated to the view.
  * Inserts the view 'Architecture' at the index 0 under the node TreeManager.OPEN_VIEWS.
  * Inserts a full view under the node TreeManager.FULL_VIEWS, which is at the index 1
  * under the node OPEN_VIEWS.
  * Inserts a NoThan view under the node TreeManager.NOTHAN_VIEWS,
  * which is at the index 2 under the node OPEN_VIEWS.
  * Called by ActionNewView, ActionOpenGraphicView, ActionOpenArchitecture.
  * Calls _view.getEorM_BaseObjects().
  * @param _view to process
  */
  public static final synchronized void createViewExplorer(ViewInBase _view) {
    // variables
    DefaultMutableTreeNode    node = null;
    BaseObject                object;
    Map                       map = _view.getEorM_BaseObjects();
    DefaultMutableTreeNode    newNode = new DefaultMutableTreeNode(_view, true);
    JTree                     eTree = InternalFrameModel.getTreeInExplorer();
    DefaultTreeModel          explorerTreeModel =(DefaultTreeModel)
                                eTree.getModel();
    // to update the explorer, gets the OPEN_VIEWS node in Explorer
    DefaultMutableTreeNode    viewsNode = (DefaultMutableTreeNode)
                                ((DefaultMutableTreeNode)explorerTreeModel.
                                    getRoot()).getChildAt(1);
    int                       index;
    // openViewsChildrenNunber is the number of currently displayed nodes under OPEN_VIEWS
    int                       openViewsChildrenNunber = explorerTreeModel.getChildCount( viewsNode );
    int                       fullViewsIndex;
    boolean                   withArchitecture = false;
    boolean                   withFullViewsNode = false;
    boolean                   indexFound = false;

    if ( ViewInBase_Facade.getArchitectureView() != null)
        withArchitecture = true;
    
    if (ActionOpenArchitecture.ARCHI_VIEW_NAME.equals(_view.getName())) 
        explorerTreeModel.insertNodeInto( newNode, viewsNode, 0);
    else {
        
        if (!ViewInBase_Facade.isNoThanViewName(_view.getName())) {
            if (withArchitecture) {
                fullViewsIndex = 1;
                if (openViewsChildrenNunber == 1 ||
                    !((DefaultMutableTreeNode)viewsNode.
                    getChildAt(fullViewsIndex)).getUserObject().equals(TreeManager.FULL_VIEWS)) {
                        
                        // creates the FULL_VIEWS node first at fullViewsIndex
                        explorerTreeModel.insertNodeInto(
                                new DefaultMutableTreeNode(TreeManager.FULL_VIEWS, true), 
                                viewsNode, fullViewsIndex);
                        
                } // if (openViewsChildrenNunber == 1
            } 
            else { // no Architecture
                fullViewsIndex = 0;
                if (openViewsChildrenNunber == 0 ||
                    !((DefaultMutableTreeNode)viewsNode.
                    getChildAt(fullViewsIndex)).getUserObject().equals(TreeManager.FULL_VIEWS)) {
                        
                        // creates the FULL_VIEWS node first at fullViewsIndex
                        explorerTreeModel.insertNodeInto(
                                new DefaultMutableTreeNode(TreeManager.FULL_VIEWS, true), 
                                viewsNode, fullViewsIndex);
                        
                } // if (openViewsChildrenNunber == 1
            } // else
            // in any cases, inserts the _view node under FULL_VIEWS
            // in the alphabetic order
            // uses 'node' to avoid any error in insertNodeInto() for 10% of cases!
            node = (DefaultMutableTreeNode) viewsNode.getChildAt(fullViewsIndex);
            
            index = 0;
            indexFound = false;
            while (!indexFound) {
                if (node.getChildCount() == index) {
                    indexFound = true;
                    continue;
                }
                if (((BaseObject)((DefaultMutableTreeNode)node.getChildAt(index)).getUserObject()).
                        getNickName().compareTo(
                        ((BaseObject)newNode.getUserObject()).getNickName()) > 0) {
                    indexFound = true;
                    continue;
                }
                else index++;
            } // while
            explorerTreeModel.insertNodeInto( newNode, node, index);
        } // if
        else { // for NOTHAN view
            
            // the FULL_VIEWS node is always created before the NOTHAN_VIEWS
            if (withArchitecture) {

                            // for testing and tracking
                            //System.out.println("ViewTree.createViewExplorer() processes a NOTHAN view"+
                              //      " when there is the Architecture node");
                            
                fullViewsIndex = 1;
                if (openViewsChildrenNunber > 1 &&
                        ((DefaultMutableTreeNode)viewsNode.
                            getChildAt(fullViewsIndex)).getUserObject().equals(TreeManager.FULL_VIEWS))
                        withFullViewsNode = true;
                
                if (withFullViewsNode && openViewsChildrenNunber == 2) {

                            // for testing and tracking
                            //System.out.println("ViewTree.createViewExplorer() create the NOTHAN_VIEWS node"+
                              //      " at fullViewsIndex + 1 with fullViewsIndex = "+fullViewsIndex);
                            
                        // creates the NOTHAN_VIEWS node at 2
                        explorerTreeModel.insertNodeInto(
                                new DefaultMutableTreeNode(TreeManager.NOTHAN_VIEWS, true), 
                                viewsNode, fullViewsIndex + 1);
                        
                } // if (openViewsChildrenNunber == 1
                else if (openViewsChildrenNunber == 1) {

                            // for testing and tracking
                            //System.out.println("ViewTree.createViewExplorer() create the FULL_VIEWS node, then"+
                              //      " the NOTHAN_VIEWS node at fullViewsIndex + 1 with fullViewsIndex = "+
                                //    fullViewsIndex);
                            
                    // creates the FULL_VIEWS node first at 1
                        explorerTreeModel.insertNodeInto(
                                new DefaultMutableTreeNode(TreeManager.FULL_VIEWS, true), 
                                viewsNode, fullViewsIndex);                         
                        // create the NOTHAN_VIEWS node first at 2
                        explorerTreeModel.insertNodeInto(
                                new DefaultMutableTreeNode(TreeManager.NOTHAN_VIEWS, true), 
                                viewsNode, fullViewsIndex + 1);
                        
                } // if (openViewsChildrenNunber == 1
                
            } 
            else { // no Architecture

                            // for testing and tracking
                            //System.out.println("ViewTree.createViewExplorer() processes a NOTHAN view"+
                              //      " when there is NOT the Architecture node");
                            
                fullViewsIndex = 0;
                if (openViewsChildrenNunber != 0 &&
                        ((DefaultMutableTreeNode)viewsNode.
                            getChildAt(fullViewsIndex)).getUserObject().equals(TreeManager.FULL_VIEWS))
                        withFullViewsNode = true;
                
                if (withFullViewsNode && openViewsChildrenNunber == 1 ) {

                            // for testing and tracking
                            //System.out.println("ViewTree.createViewExplorer() create the NOTHAN_VIEWS node"+
                              //      " at fullViewsIndex + 1 with fullViewsIndex = "+fullViewsIndex);
                    
                        // create the NOTHAN_VIEWS node first at 1
                        explorerTreeModel.insertNodeInto(
                                new DefaultMutableTreeNode(TreeManager.NOTHAN_VIEWS, true), 
                                viewsNode, fullViewsIndex + 1);
                        
                } // if (openViewsChildrenNunber == 1
                else if (openViewsChildrenNunber == 0) {

                            // for testing and tracking
                            //System.out.println("ViewTree.createViewExplorer() create the FULL_VIEWS node, then"+
                              //      " the NOTHAN_VIEWS node at fullViewsIndex + 1 with fullViewsIndex = "+
                                //    fullViewsIndex);
                            
                        // creates the FULL_VIEWS node first at 0
                        explorerTreeModel.insertNodeInto(
                                new DefaultMutableTreeNode(TreeManager.FULL_VIEWS, true), 
                                viewsNode, fullViewsIndex);                        
                        // create the NOTHAN_VIEWS node first at 1
                        explorerTreeModel.insertNodeInto(
                                new DefaultMutableTreeNode(TreeManager.NOTHAN_VIEWS, true), 
                                viewsNode, fullViewsIndex + 1);
                        
                } // if (openViewsChildrenNunber == 1
                
            } // else
            // in any cases, inserts the _view node under NOTHAN_VIEWS
            // in the alphabetic order
            node = (DefaultMutableTreeNode) viewsNode.getChildAt(fullViewsIndex + 1);
            
            index = 0;
            indexFound = false;
            while (!indexFound) {
                if (node.getChildCount() == index) {
                    indexFound = true;
                    continue;
                }
                if (((BaseObject)((DefaultMutableTreeNode)node.getChildAt(index)).getUserObject()).
                        getNickName().compareTo(
                        ((BaseObject)newNode.getUserObject()).getNickName()) > 0) {
                    indexFound = true;
                    continue;
                }
                else index++;
            } // while
            explorerTreeModel.insertNodeInto( newNode, node, index); 
        } // else
    } // else

    // creates the tree listener (the observer and the graphic view listener, if any,
    // are created by ViewModel)
    ViewTreeBaseListenerImpl baseListener = new ViewTreeBaseListenerImpl(_view);
    TreeManager.addTreeBaseListener(baseListener);

    // displays the view objects
    index = 0;
    // adds the target if any
    if (_view instanceof NoThanViewInBase && ((NoThanViewInBase)_view).getEorFirstAccessTarget() != null) {
      node = new DefaultMutableTreeNode(((NoThanViewInBase)_view).getEorFirstAccessTarget(), false);
      explorerTreeModel.insertNodeInto(node, newNode, index);
      index++;        
    }
    // adds the standard view objects
    for (Iterator ite = map.values().iterator() ; ite.hasNext() ; ) {
      object = (BaseObject)ite.next();
      node = new DefaultMutableTreeNode(object, false);
      explorerTreeModel.insertNodeInto(node, newNode, index);
      index++;
    } // for

    // changes the selection in the explorer, and in the beamer
    eTree.setSelectionPath(new TreePath( newNode.getPath() ));

    node = null;
    map = null;
    explorerTreeModel = null;
    eTree = null;
    viewsNode = null;
  } // createViewExplorer()

  /**
  * Updates the graphical flat list of elements in an open view.
  * The ViewTreeBaseListener associated to the view calls this method, and it is not changed.
  * @param _view ViewInBase to process
  * @param _tree JTree to which ACS objects are added, with a DefaultTreeModel
  * @param _root empty node in _tree, associated to the view,
  *   to which the flat list of view elements is added
  * @return JTree _tree with a list of views
  */
  public static final synchronized JTree fillViewNodeInExplorer(ViewInBase _view,
        JTree _tree, DefaultMutableTreeNode _root) {
    // variables
    DefaultMutableTreeNode    node = null;
    BaseObject                object;
    Map                       map = _view.getEorM_BaseObjects();
    DefaultTreeModel          viewTreeModel = (DefaultTreeModel)_tree.getModel();
    int                       index = 0;

    if (map.isEmpty()) {
        if (!(_view instanceof NoThanViewInBase)) return _tree;
        else if (((NoThanViewInBase)_view).getEorFirstAccessTarget() == null)
            return _tree;
    }

    // displays the view objects
        // adds the target if any
    if (_view instanceof NoThanViewInBase && ((NoThanViewInBase)_view).
            getEorFirstAccessTarget() != null) {
      node = new DefaultMutableTreeNode(((NoThanViewInBase)_view).getEorFirstAccessTarget(), false);
      viewTreeModel.insertNodeInto(node, _root, index);
      index++;        
    }
        // adds the standard objects
    for (Iterator ite = map.values().iterator() ; ite.hasNext() ; ) {
      object = (BaseObject)ite.next();
      node = new DefaultMutableTreeNode(object, false);
      viewTreeModel.insertNodeInto(node, _root, index);
      index++;
    } // for

    node = null;
    map = null;
    viewTreeModel = null;
    object = null;

    return _tree;
  } // fillViewNodeInExplorer()

} // class ViewTree