//Title:        Access Road
//Version:      0.7.0
//Copyright:    Copyright 2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Pattern;

import ARoad0.ErrB.BaseError;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBase.ACSImpl;

/**
 * This interface is responsible for describing an AcsFactory in the gWork package
 * of an AcsAddon. It is used for creating the ACS of any AcsAddon.<P>
 * The descriptor of the AcsAddon, to set in the menu for creating a new ACS, is
 * returned by the method getPublicAcsAddonName(). <P>
 * Caution: This property has to be defined in every implementation of this interface. <P>
 * The BaseManagerImpl class, in the generic gBase package, is responsible to return
 * the method getL_AcsAddonDescriptors() describing all the ACS addon editors and names.
 * In this returned list, the form of a value is 'Editor: editor_name - AcsAddon: addon_name'.
 * BaseManagerImpl checks that the 4 standard directories of an AcsAddon are
 * there in the packages tree. These directories are gBase, gBaseBeanInfo, gBaseInterface,
 * and gWork. It also controls that the standard file AcsFactoryXXX is in the directory 'gWork',
 * where 'XXX' is the AcsAddon name. Otherwise, the BaseManagerImpl instance
 * cannot be initialized, an error message is thrown and the program stops.<P>
 * An AcsAddon must have a 'acsaddon_guide.html' file in the
 * AcsAddon gBase directory, as the first user guide. In this same directory,
 * it may provide an icon for the ACS, at the format JPEG.
 * There is no explicit management of the AcsAddon version, but is is possible to add
 * a version number in the AcsAddon name.
 * @see #getPublicAcsAddonName() for the definition of the ACS type
 * @version 0.7.0
 */
public interface AcsFactoryForAcsAddon extends Singleton {

  /**
   * Creates a specific ACS for an AcsAddon.
   * The method configure() is called after this method by
   * ActionNewACSyst.fillWithAcsObjects().
   * The ACS vocabulary is set by the AcsAddon ACSFactory directly.
   * @param _iSname the IS of the new ACS. May be null.
   * @param _name of the ACS to create or open.
   *   No test of null value nor empty.
   * @param _type must be the right type for the AcsAddon
   * @param _parent is the parent node of the new ACS in the IS. Never null.
   * @exception BaseError if the ACS is known, the ACS is open,
   *   the core ACS is null when it is mandatory, or
   *   ClassNotFoundException, IOException with
   *   invalidClassException if the ACSImpl class HAS CHANGED,
   *   or if the type or the specified structure are not correct.
   * @return the new ACS
   */
  public ACSImpl create(ImmutableName _iSname,
          ImmutableName _name, String _type, ImmutableName _parent) throws BaseError;

  /**
  * Creates the initial components in a new ACS.
  * Called by ActionNewACSyst.fillWithAcsObjects().
  * @param _result to construct
  * @return ACS with the initial and standard set of components.
  */
  public ACSImpl configure(ACSImpl _result);

  /**
   * Gets the descriptor of the AcsAddon, to set in the menu for creating a new ACS.
   * This method has to return a static variable, because it is called after a call
   * to the empty constructor. This variable is usually called PUBLIC_ACS_ADDON_NAME.
   * It has to contain a '(' character. ActionNewACSyst uses it to define the ACS type,
   * as the beginning of the text up to this first '(' character. For instance,
   * 'Linux Ubuntu 8.04 (by Accbee, France, EU)' is the value of PUBLIC_ACS_ADDON_NAME
   * in the class AcsFactoryUbuntu that implements this interface, and
   * 'Linux Ubuntu 8.04' is the type of the ACS this class creates.
   */
  public String getPublicAcsAddonName();

} // AcsFactoryForAcsAddon
