//Title:        Access Road
//Version:      0.6.0
//Copyright:    Copyright 2002, 2004, 2007-2009 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.Pattern;

import java.lang.reflect.Method;
import java.beans.BeanInfo;

import ARoad0.gBaseInterface.BaseBeanManager;

/**
 * This interface is responsible for describing a BaseObject class
 * and for helping to implement the BeanInfo interface.
 * It is used for displaying and editing the BaseObject properties of a gBase class.
 * @version 0.5.1
 */
public interface BeanInfoPattern extends BeanInfo {

    /**
     * Get the instance, if it exists, which fires an event when
     * the bean is closed or deleted.
     * @return the relevant BaseBeanManager object. Return null
     *  when there is an active BeanInfoPattern.getBaseBeanManagerGetMethod(),
     *  or when it is not applicable.
     */
    public BaseBeanManager getBaseBeanManagerInstance();
      // return YYY.getInstance() or return new YYY();

    /**
     * Get the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, if it exists,
     * which fires an event when the bean instance is closed or deleted.
     * @return the get method. Is null when there is
     *  an active BeanInfoPattern.getBaseBeanManagerInstance(),
     *  or when it is not applicable.
     */
    public Method getBaseBeanManagerGetMethod();

    /**
     * Get the property name for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * This is used to register the BeamerManager as a PropertyChangeListener.
     * @return the property name. Is null when it is not applicable,
     *  that is BeanInfoPattern.getBaseBeanManagerInstance()
     *  and BeanInfoPattern.getBaseBeanManagerGetMethod()
     *  return null.
     */
    public String getBaseBeanManagerBoundProperty();
      // return new String("propertyName");


} // BeanInfoPattern