//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.BaseBeanManager;
import ARoad0.gBaseInterface.StringRight;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBase.*;
import java.beans.SimpleBeanInfo;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.lang.reflect.Method;

import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the ACSFactoryImpl class.
 * It is used for displaying and editing most of the properties of this class,
 * after a first editing through ACSpreFactoryImplBeanInfo.
 * @version 0.7.0
 */
public class ACSFactoryImplBeanInfo extends SimpleBeanInfo implements BeanInfoPattern {

  public ACSFactoryImplBeanInfo() {}

  /**
   * Gets the description of the bean.
   * Some standard values are set:
   * - 'hidden' is false since the bean is not hidden to the users
   * - 'expert' is false since the bean is not reserved to experts
   * <P>
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of ACSImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(ACSFactoryImpl.class );
    result.setDisplayName("ACS Factory");

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of ACSImpl
    result.setValue("withTabbedPanes", Boolean.TRUE );

    // these strings signal the nodes to use in an ACS tree
    // to display this ACS, following the ACSTree.M_ACS_TREE_UPPER_NODES
    // in the Gui2 package.
    result.setValue("nodesInACSTree", new String[]
      {"ACSRIGHTS"});
    
    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    result.setHidden(false);
    result.setExpert(false);

    return result;
  } // getBeanDescriptor()

  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue():<P>
   * - 'tabbedPaneName' is the name (String value) of the pane
   *    where this property has to be displayed in the beamer.
   *    Is "" if 'withTabbedPanes' is false in the BeanDescriptor.
   * - 'panelGroupName' is the name (String value) of the group
   *    where this property has to be displayed with other related properties.
   *    May be "".<P>
   * The order of the property descriptors is:
   * Name, 
   * ISname, 
   * ParentName, 
   * VisibleSpace, 
   * HubProperty, 
   * ResourceTypes, 
   * EPTypes,
   * ACSType, 
   * 
   * GORightsSet, 
   * AclRightsSet, 
   * BridgeRightsSet, 
   * SpecializedRightsSet,
   * TypesAndExplorerSet,
   * AssociationForNewObjectsMap, 
   * Rights, 
   * 
   * ManageResources, 
   * ManageResourceTrees, 
   * ManageAccounts, 
   * ManageGroups, 
   * ManageGroupTrees, 
   * ManageInternalAcls, 
   * ManageExternalAcls, 
   * ManageActors, 
   * ManageBridges, 
   * ManageSecondaryGroupsForActor, 
   * deprecated: ManageNxDirectoryEPs,
   * deprecated: ManageNxExecutables,
   * ManageVirtualFolders, 
   * ManageAGORight, 
   * deprecated: ManageOtherRights,
   * ManagePrivilegeRights, 
   * ManageAcsRights, 
   * ManageMetaRights,
   *
   *    // new in 070
   * 
   * TiedToParentACS,
   * SubAndGroupPolicy,
   * AuthorizationServer,
   * M_BelongsToComposite,
   * Alternates,
   * RunningOnParentACS,
   * OpeningAndClosingRoots,
   * OpeningNoClosingRoots,
   * IntegrityRoots,
   * ComponentNodes,
   * CompositeNodes,
   * Icon,
   * CommentsOnProperties,
   * AcsVocabulary,
   *
   * IncompleteStructureModeling,
   * IncompleteBehaviorModeling,
   *
   * ManageSimpleLinkedPrivileges,
   * ManageSimpleTypedPrivileges,
   * ManageGrantingRightPrivileges,
   * ManageDenyingRightPrivileges,
   * ManagePrivilegesInVirtualFolders,
   * ManagePrivilegeInResources,
   * ManageResourceVirtualFolders,
   * ManageEPVirtualFolders,
   * ManageSimpleResourceVirtualFolders,
   * ManageDirectoryInResourceVirtualFolders,
   * ManageDirectoryEPinEPVirtualFolders,
   * ManageActorInEPVirtualFolders,
   * ManageACLRightInheritance,
   * ManageAGORightInheritance,
   * ManageLPRIRightInheritance,
   * ManageVirtualFolderTrees,
   * ManageOneTimeInVirtualFolderTreeForMember,
   * VirtualFolderRightsPropagationToMembers,
   * ManageSoftAlias,
   * ManageInternalHardAlias,
   * ManageFlexibleConditionalAcls,
   * ManageRightDefinedConditionalAcls,
   *
   * ResVirtualFolderTypes,
   * EPVirtualFolderTypes,
   * PrivilegeRightsSet,
   * PrivilegeForTypeTypes,
   * PrivilegeForLinksTypes
   * 
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pd;

    try {
      // ------------------- property "Name" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "Name", ACSFactoryImpl.class, "getName", null);
      pd_one.setDisplayName("New ACS name");
      //pd_one.setShortDescription("   ");
      //pd_one.setBound(false);
      //pd_one.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_one.setValue("panelGroupName", "  Main attributes  " );

      // ------------------- property "ISname" ------------------------
      PropertyDescriptor pd_seventy = new PropertyDescriptor(
        "ISname", ACSFactoryImpl.class, "getISName", null);
      pd_seventy.setDisplayName("Name of the IS");
      //pd_seventy.setShortDescription("   ");
      //pd_seventy.setBound(false);
      //pd_seventy.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventy.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventy.setValue("panelGroupName", "  inter-ACS Structure  " );     

      // ------------------- property "ParentName" ------------------------
      PropertyDescriptor pd_fourtyFive = new PropertyDescriptor(
        "ParentName", ACSFactoryImpl.class, "getParentName", null);
      pd_fourtyFive.setDisplayName("Parent name");
      //pd_fourtyFive.setShortDescription("   ");
      //pd_fourtyFive.setBound(false);
      //pd_fourtyFive.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyFive.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtyFive.setValue("panelGroupName", "  inter-ACS Structure  " );
      
      // ------------------- property "VisibleSpace" ------------------------
      PropertyDescriptor pd_fourtyThree = new PropertyDescriptor(
        "VisibleSpace", ACSFactoryImpl.class, "getVisibleSpace", "setVisibleSpace");
      pd_fourtyThree.setDisplayName("IS Visible Context ");
      pd_fourtyThree.setShortDescription("the IS context from the ACS point of view");
      //pd_fourtyThree.setBound(false);
      //pd_fourtyThree.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyThree.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtyThree.setValue("panelGroupName", "  inter-ACS Structure  " );
      // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor.
      // The restricted values are get from 'staticSetOfItems', for Strings only,
      // secondly from 'choiceMethodForNew' like here,
      // or through a search in the ACS by the PropertyEditor.
      pd_fourtyThree.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'staticSetOfItems' is the array of possible String values for the property.
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method.
            //pd_fourtyThree.setValue("staticSetOfItems",  );
      // 'choiceMethodForNew' defines a specific instance/method to invoke 
      // to let the user defines the new restricted key, in a map, or the new instance, 
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if 
      // there is an add method or a write method. With 'isRestrictedArgument', 
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      pd_fourtyThree.setValue("choiceMethodForNew", new Object[] {
        NameISTreeImpl.getInstance(), "getExtendedVisibleSpaceConstants"} );
      
      // ------------------- property "HubProperty" ------------------------
      PropertyDescriptor pd_fourtyFour = new PropertyDescriptor(
        "HubProperty", ACSFactoryImpl.class, "getHubProperty", "setHubProperty");
      pd_fourtyFour.setDisplayName("Is a hub for children");
      pd_fourtyFour.setShortDescription("the ACS allowed children to communicate");
      //pd_fourtyFour.setBound(false);
      //pd_fourtyFour.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyFour.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtyFour.setValue("panelGroupName", "  inter-ACS Structure  " );
      
      // ------------------- property "ResourceTypes" --------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "ResourceTypes", ACSFactoryImpl.class, "getL_ResourceTypes", "setL_ResourceTypes");
      pd_two.setDisplayName("Resource types");
      pd_two.setShortDescription("The allowed types for the resources");
      pd_two.setBound(true);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_two.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_seven.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  Types  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_two.setValue("panelGroupName", "  Types for the updating policy, display "+
              "policy and specialized rights ");
      // 'witViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer.
      pd_two.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_two.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_two.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_two.setValue("withDeleteButton", Boolean.TRUE );
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_seven.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible list.
      pd_two.setValue("limitedSizeForPresentation", new Integer(5));

      // ------------------- property "EPTypes" --------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "EPTypes", ACSFactoryImpl.class, "getL_EPTypes", "setL_EPTypes");
      pd_three.setDisplayName("Eligible Party types");
      pd_three.setShortDescription("The allowed types for the eligible parties");
      pd_three.setBound(true);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_three.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_eight.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  Types  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_three.setValue("panelGroupName", "  Types for the updating policy, display "+
              "policy and specialized rights ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      pd_three.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // False if it is not editable trough a constructor or another method.
      pd_three.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_three.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_three.setValue("withDeleteButton", Boolean.TRUE );
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_eight.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible list.
      pd_three.setValue("limitedSizeForPresentation", new Integer(5));
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the String values of the previous property for this current property,
      // following the order of the property descriptors returned by getPropertyDescriptors().
      pd_three.setValue("withPreviousButton", Boolean.TRUE );

      // ------------------- property "ResVirtualFolderTypes" --------------------
      PropertyDescriptor pd_seventyThree = new PropertyDescriptor(
        "ResVirtualFolderTypes", ACSFactoryImpl.class, "getL_ResourceVirtualFolderTypes",
        "setL_ResourceVirtualFolderTypes");
      pd_seventyThree.setDisplayName("Simple Res. VF types");
      pd_seventyThree.setShortDescription("The allowed types for the virtual folders "+
              "of simple Resources");
      pd_seventyThree.setBound(true);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_seventyThree.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_seventyThree.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventyThree.setValue("tabbedPaneName", "  Types  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventyThree.setValue("panelGroupName", "  Types for the updating policy, display "+
              "policy and specialized rights ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      pd_seventyThree.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // False if it is not editable trough a constructor or another method.
      pd_seventyThree.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_seventyThree.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_seventyThree.setValue("withDeleteButton", Boolean.TRUE );
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_seventyThree.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible list.
      pd_seventyThree.setValue("limitedSizeForPresentation", new Integer(5));
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the String values of the previous property for this current property,
      // following the order of the property descriptors returned by getPropertyDescriptors().
      pd_seventyThree.setValue("withPreviousButton", Boolean.TRUE );

      // ------------------- property "EPVirtualFolderTypes" --------------------
      PropertyDescriptor pd_seventyEight = new PropertyDescriptor(
        "EPVirtualFolderTypes", ACSFactoryImpl.class, "getL_EPVirtualFolderTypes", 
        "setL_EPVirtualFolderTypes");
      pd_seventyEight.setDisplayName("RU Res. VF types");
      pd_seventyEight.setShortDescription("The allowed types for the virtual folders"+
              " of Right User Resources");
      pd_seventyEight.setBound(true);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_seventyEight.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_seventyEight.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventyEight.setValue("tabbedPaneName", "  Types  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventyEight.setValue("panelGroupName", "  Types for the updating policy, display "+
              "policy and specialized rights ");
      // 'withViewButton' is the boolean which set if the editor
      // has to display an 'View' button
      pd_seventyEight.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // False if it is not editable trough a constructor or another method.
      pd_seventyEight.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_seventyEight.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_seventyEight.setValue("withDeleteButton", Boolean.TRUE );
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_seventyEight.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible list.
      pd_seventyEight.setValue("limitedSizeForPresentation", new Integer(5));
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the String values of the previous property for this current property,
      // following the order of the property descriptors returned by getPropertyDescriptors().
      pd_seventyEight.setValue("withPreviousButton", Boolean.TRUE );

      // ------------------- property "ACSType" ------------------------
      PropertyDescriptor pd_nine = new PropertyDescriptor(
        "ACSType", ACSFactoryImpl.class, "getACSType", null);
      pd_nine.setDisplayName("ACS type");
      pd_nine.setShortDescription("the ACS type may be GENERIC, LINUX, ... "+
        "or may be user-defined. The created ACS has a type ended with the policy checksum.");
      //pd_nine.setBound(true);
      //pd_nine.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_nine.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_nine.setValue("panelGroupName", "  Types  " );
      
      // ------------------- property "ARightsSet" --------------------
      PropertyDescriptor pd_five = new PropertyDescriptor(
        "ARightsSet", ACSFactoryImpl.class, "getL_AccountRightsSet", "setL_AccountRightsSet");
      pd_five.setDisplayName("Rights for Accounts");
      pd_five.setShortDescription("The allowed rights for the accounts (or userIDs)");
      pd_five.setBound(true);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_five.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is null to allow a StringRight, not a StringRightImpl
      // or a StringMetaRightImpl.
        //pd_five.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_five.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_five.setValue("panelGroupName", "  ACS Allowed Rights  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer.
      pd_five.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_five.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_five.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_five.setValue("withRemoveButton", Boolean.TRUE );
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // If the ACS limits the values, these BeanInfo values are actived only if
      // they are allowed by the ACS, and the ACS values must be in this BeanInfo
      // array to be actived.
      //pd_five.setValue("staticSetOfItems", new String[]
        //{"w", "x"} );
      // 'limitedSizeForPresentation' is the maximal size of the visible list.
      // The value 4 is not used, but the list is displayed for that.
      pd_five.setValue("limitedSizeForPresentation", new Integer(8));  
      
      // ------------------- property "GORightsSet" --------------------
      PropertyDescriptor pd_six = new PropertyDescriptor(
        "GORightsSet", ACSFactoryImpl.class, "getL_GroupRightsSet",
        "setL_GroupRightsSet");
      pd_six.setDisplayName("Rights for Grp");
      pd_six.setShortDescription("The allowed 'Group' in the AG rights");
      pd_six.setBound(true);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_six.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is null to allow a StringRight, not a StringRightImpl
      // or a StringMetaRightImpl.
        //pd_six.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_six.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_six.setValue("panelGroupName", "  ACS Allowed Rights  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer.
      pd_six.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_six.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_six.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_six.setValue("withRemoveButton", Boolean.TRUE );
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_six.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible list.
      // The value 4 is not used, but the list is displayed for that.
      pd_six.setValue("limitedSizeForPresentation", new Integer(8)); 
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the String values of the previous property for this current property,
      // following the order of the property descriptors returned by getPropertyDescriptors().
      pd_six.setValue("withPreviousButton", Boolean.TRUE );   

      // ------------------- property "AclRightsSet" --------------------
      PropertyDescriptor pd_seven = new PropertyDescriptor(
        "AclRightsSet", ACSFactoryImpl.class, "getL_AclRightsSet", "setL_AclRightsSet");
      pd_seven.setDisplayName("Rights for 'ACL'");
      pd_seven.setShortDescription("The allowed rights for the Access Control Lists");
      pd_seven.setBound(true);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_seven.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is null to allow a StringRight, not a StringRightImpl
      // or a StringMetaRightImpl.
        //pd_seven.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seven.setValue("tabbedPaneName", "  Rights 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_seven.setValue("panelGroupName", "  ACS Allowed Rights  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer.
      pd_seven.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_seven.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_seven.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_seven.setValue("withRemoveButton", Boolean.TRUE );
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_seven.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible list.
      // The value 4 is not used, but the list is displayed for that.
      pd_seven.setValue("limitedSizeForPresentation", new Integer(8));
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the String values of the previous property for this current property,
      // following the order of the property descriptors returned by getPropertyDescriptors().
      pd_seven.setValue("withPreviousButton", Boolean.TRUE );
  
      // ------------------- property "BridgeRightsSet" -------------------- 
      PropertyDescriptor pd_ten = new PropertyDescriptor(
        "BridgeRightsSet", ACSFactoryImpl.class, "getL_bridgeRightsSet",
        "setL_bridgeRightsSet");
      pd_ten.setDisplayName("Rights for Bridges");
      pd_ten.setShortDescription("The allowed rights for the Bridges");
      pd_ten.setBound(true);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_ten.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is null to allow a StringRight, not a StringRightImpl
      // or a StringMetaRightImpl.
        //pd_ten.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ten.setValue("tabbedPaneName", "  Rights 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_ten.setValue("panelGroupName", "  ACS Allowed Rights  ");
      // 'withViewButton' is the boolean which set if the CollectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer.
      pd_ten.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_ten.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_ten.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_ten.setValue("withRemoveButton", Boolean.TRUE );
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_ten.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible list.
      // The value 4 is not used, but the list is displayed for that.
      pd_ten.setValue("limitedSizeForPresentation", new Integer(8));
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the String values of the previous property for this current property,
      // following the order of the property descriptors returned by getPropertyDescriptors().
      pd_ten.setValue("withPreviousButton", Boolean.TRUE );
      
      /*
      // ------------------ property "HaveSpecializedRights" --------------- 
      PropertyDescriptor pd_twenty = new PropertyDescriptor(
        "HaveSpecializedRights", ACSFactoryImpl.class, "getHaveSpecializedRights", null);
      pd_twenty.setDisplayName("has specialized rights");
      pd_twenty.setShortDescription("true if some detailled rights have been set.");
      pd_twenty.setBound(true);
      //pd_twenty.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twenty.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_twenty.setValue("panelGroupName", "  inter-ACS Structure  " );
       * */
  
      // ------------------- property "SpecializedRightsSet" --------------------
      PropertyDescriptor pd_seventyOne = new PropertyDescriptor(
        "SpecializedRightsSet", ACSFactoryImpl.class, "getM_SpecializedRights",
        null);
      pd_seventyOne.setDisplayName("Detailled sets for rights");
      pd_seventyOne.setShortDescription("Specialized rights for interfaces or types, "+
              "for targets or sources");
      pd_seventyOne.setBound(true);
      // 'isRightMap' is TRUE when the right property editor is RightMapPropertyEditor
      pd_seventyOne.setValue("isRightMap", Boolean.TRUE );      
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, it defines the type of the elementary object to handle. 
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_seventyOne.setValue("valueTypeInGroup", new Object[] {StringRight.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventyOne.setValue("tabbedPaneName", "  Rights 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventyOne.setValue("panelGroupName", 
              "  Specialized AG/ACL/Bridge Rights - They override Rights and Rights 2 (option) ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_seventyOne.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_seventyOne.setValue("withPreviousButton", Boolean.TRUE );
      // 'withViewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_seventyOne.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'Add New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_seventyOne.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the RightMapPropertyEditor
      // has to display 2 'Select' buttons if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_seventyOne.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the RightMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_seventyOne.setValue("withRemoveButton", Boolean.TRUE );
      pd_seventyOne.setValue("withDeleteButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the fixed argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method to initialize a l_fixedValue from this object.
      // Used by SimplePropertyEditor.getKnownValuesForNewElement(), which
      // may use a subset of the returned arguments if the class of the object to create
      // waits a smaller set of arguments.
      // Caution: if 'choiceMethodForNew' exists, then the constrained arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
        //pd_seventyOne.setValue("withKnownArgumentsForNew", new Object[] {null, null} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_seventyOne.setValue("addMethod", "addSpecializedRights"); 
      // 'restrictedKeyForSpecializedRight' indicates if the key is restricted
      // to some specified values for the map key of a new specialized rights.
      pd_seventyOne.setValue("restrictedKeyForSpecializedRight", Boolean.TRUE );
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
      pd_seventyOne.setValue("removeMethod", "removeSpecializedRights");     
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_seventyOne.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      pd_seventyOne.setValue("limitedSizeForPresentation", new Integer(8));
   
      // ------------------- property "TypesAndExplorerSet" --------------------
      PropertyDescriptor pd_seventyTwo = new PropertyDescriptor(
        "TypesAndExplorerSet", ACSFactoryImpl.class, "getM_TypesAndExplorer",
        null);
      pd_seventyTwo.setDisplayName("Detailled sets for values");
      pd_seventyTwo.setShortDescription("Specialized values for the explorer and ACS object policies");
      pd_seventyTwo.setBound(true);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_seventyTwo.setValue("isStringMap", Boolean.TRUE );      
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_seventyTwo.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventyTwo.setValue("tabbedPaneName", "  Constraints  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventyTwo.setValue("panelGroupName", "  A. Types and their constraints for the explorer "+
              "and ACS object policies (option) ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_seventyTwo.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_seventyTwo.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_seventyTwo.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display a 'New' button. False if it is not editable trough a constructor.
      pd_seventyTwo.setValue("withNewButton", Boolean.TRUE ); // for map keys
      // 'withSelectButton' is the boolean which set if the StringMapPropertyEditor
      // has to display a 'Select' button.
      // False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_seventyTwo.setValue("withSelectButton", Boolean.TRUE ); // for map values
      // 'withDeleteButton' is the boolean which is set if the StringMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_seventyTwo.setValue("withRemoveButton", Boolean.TRUE ); // for map values
      pd_seventyTwo.setValue("withDeleteButton", Boolean.TRUE ); // for map keys
       // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor.
      pd_seventyTwo.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke 
      // to let the user defines the new restricted key, in a map, or the new instance, 
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if 
      // there is an add method or a write method. With 'isRestrictedArgument', 
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke, 
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      pd_seventyTwo.setValue("choiceMethodForNew", new Object[] {
        null, "getL_MapKeysForTypePolicies"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined (only for the first value).
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
      pd_seventyTwo.setValue("limitedValuesMethod",
              "getL_LimitedValuesForTypesAndExplorer" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_seventyTwo.setValue("addMethod", "addTypesAndExplorer");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
      pd_seventyTwo.setValue("removeMethod", "removeTypesAndExplorer");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      pd_seventyTwo.setValue("limitedSizeForPresentation", new Integer(8));
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      pd_seventyTwo.setValue("MaxValueSize", new Integer(400));
   
      // ------------------- property "AssociationForNewObjectsMap" --------------------
      PropertyDescriptor pd_seventyFour = new PropertyDescriptor(
        "AssociationForNewObjectsMap", ACSFactoryImpl.class, "getM_AssociationsOnNewObject",
        null);
      pd_seventyFour.setDisplayName("Associations for new objects");
      pd_seventyFour.setShortDescription("Associations for some new objects");
      pd_seventyFour.setBound(true);
      // 'isActionMap' is TRUE when the right property editor is AssociationMapPropertyEditor
      pd_seventyFour.setValue("isActionMap", Boolean.TRUE );      
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_seventyFour.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventyFour.setValue("tabbedPaneName", "  Constraints  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventyFour.setValue("panelGroupName", "  B. Scripts "+
              "at the creation of some ACS objects - for advanced users  ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_seventyFour.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_seventyFour.setValue("withPreviousButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the PropertyEditor
      // has to display a 'New' button. False if it is not editable trough a constructor.
      pd_seventyFour.setValue("withNewButton", Boolean.TRUE );
      // 'withNewVButton' is the boolean which set if the PropertyEditor
      // has to display a 'NewV' button if the map value (not key) is editable
      // in the beamer. False if it is not editable.
      pd_seventyFour.setValue("withNewVButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the PropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      // 'withDeletVButton' matches with 'NewV'; it is also for map values only.
      pd_seventyFour.setValue("withDeleteButton", Boolean.TRUE );
      pd_seventyFour.setValue("withDeletVButton", Boolean.TRUE );
      // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor.
      pd_seventyFour.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' is mandatory when 'isActionMap'/'isStringTwoKeysMap'
      // and 'KeysUpdateInMap' are true.
      // It defines a specific instance/method to invoke 
      // for defining the restricted keys or key startings, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if 
      // there is an add method or a write method. With 'isRestrictedArgument', 
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      pd_seventyFour.setValue("choiceMethodForNew", new Object[] {
        AssociateUtilityImpl.getInstance(), "getL_MapKeyStartsForAssociationsOnNewObject"} );
      // 'staticSetOfItems' is the array of possible keys for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. 
        //pd_seventyFour.setValue("staticSetOfItems",
              //new String[] {"Explorer.AcsNode", "Explorer.NoAcsNode"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined.
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined (only for the first value).
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
      pd_seventyFour.setValue("limitedValuesMethod", "getL_LimitedFirstValuesForAssociation" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_seventyFour.setValue("addMethod", "addAssociationsToNewObject"); 
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
      pd_seventyFour.setValue("removeMethod", "removeAssociationsToNewObject");
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      pd_seventyFour.setValue("MaxValueSize", new Integer(990));
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      pd_seventyFour.setValue("limitedSizeForPresentation", new Integer(10));
      // 'horizontalTextPositionToLeft' is the boolean which is set if 
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value. 
      pd_seventyFour.setValue("horizontalTextPositionToLeft", Boolean.TRUE );
      
      // ------------------- property "Rights" --------------------
      PropertyDescriptor pd_fifty = new PropertyDescriptor(
        "OneAcsRight", ACSFactoryImpl.class, "getL_Rights",
        null);
      pd_fifty.setDisplayName("List of ACS rights");
      pd_fifty.setShortDescription("All the ACS rights which are workable for this ACS");
      pd_fifty.setBound(true);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor
      // If the display name is "", it is not displayed.
      pd_fifty.setValue("isCollection", Boolean.TRUE );      
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_fifty.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fifty.setValue("tabbedPaneName", "  Rights 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_fifty.setValue("panelGroupName", "  ACS Allowed Rights  ");
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_fifty.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer.
      pd_fifty.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'Add New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_fifty.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the RightMapPropertyEditor
      // has to display 2 'Select' buttons if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_fifty.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the RightMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
        //pd_fifty.setValue("withRemoveButton", Boolean.FALSE );
        //pd_seventyOne.setValue("withDeleteButton", Boolean.FALSE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the fixed argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Used by SimplePropertyEditor.getFixedValuesForNewElement(), which
      // may use a subset of the returned arguments if the class of the object to create
      // waits a smaller set of arguments.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
          //pd_fifty.setValue("withKnownArgumentsForNew", new Object[] {null, null} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_fifty.setValue("addMethod", "addSpecializedRights"); 
      // 'restrictedKeyForSpecializedRight' indicates if the key is restricted
      // to some specified values for the map key of a new specialized rights.
        //pd_fifty.setValue("restrictedKeyForSpecializedRight", Boolean.TRUE );
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
        //pd_fifty.setValue("removeMethod", "removeSpecializedRights");     
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_fifty.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      pd_fifty.setValue("limitedSizeForPresentation", new Integer(8));      
      
      // ------------------ property "ManageResources" ---------------
      PropertyDescriptor pd_twelve = new PropertyDescriptor(
        "ManageResources", ACSFactoryImpl.class, "getManageResources", "setManageResources");
      pd_twelve.setDisplayName("Resources");
      pd_twelve.setShortDescription("Resources, in trees or not, are the main access targets.");
      pd_twelve.setBound(true);
      //pd_twelve.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twelve.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_twelve.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );

      // ------------------ property "ManageResourceTrees" ---------------
      PropertyDescriptor pd_thirteen = new PropertyDescriptor(
        "ManageResourceTrees", ACSFactoryImpl.class, "getManageResourceTrees",
        "setManageResourceTrees");
      pd_thirteen.setDisplayName("Resources Tree");
      pd_thirteen.setShortDescription("True if the ACS manages an inner tree of resources.");
      pd_thirteen.setBound(true);
      //pd_thirteen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirteen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_thirteen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );

      // ------------------ property "ManageAccounts" ---------------
      PropertyDescriptor pd_fourteen = new PropertyDescriptor(
        "ManageAccounts", ACSFactoryImpl.class, "getManageAccounts", "setManageAccounts");
      pd_fourteen.setDisplayName("Accounts - Users ('A')");
      pd_fourteen.setShortDescription("Accounts are right users, owners of resources"+
              " and running contexts of processes; they may be personal user accounts.");
      pd_fourteen.setBound(true);
      //pdfourteen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourteen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourteen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
 
      // ------------------ property "ManageGroups" ---------------
      PropertyDescriptor pd_fifteen = new PropertyDescriptor(
        "ManageGroups", ACSFactoryImpl.class, "getManageGroups", "setManageGroups");
      pd_fifteen.setDisplayName("Account Groups ('G')");
      pd_fifteen.setShortDescription("Groups are sets of accounts, "+
              "in group trees or not; they are right users.");
      pd_fifteen.setBound(true);
      //pd_fifteen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fifteen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fifteen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
 
      // ------------------ property "ManageGroupTrees" ---------------
      PropertyDescriptor pd_sixteen = new PropertyDescriptor(
        "ManageGroupTrees", ACSFactoryImpl.class, "getManageGroupTrees", "setManageGroupTrees");
      pd_sixteen.setDisplayName("Nested Account Groups");
      pd_sixteen.setShortDescription("To manage nested account groups, "+
              "where a group may be member of other groups.");
      pd_sixteen.setBound(true);
      //pd_sixteen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixteen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_sixteen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
 
      // ------------------ property "ManageInternalAcls" ---------------
      PropertyDescriptor pd_seventeen = new PropertyDescriptor(
        "ManageInternalAcls", ACSFactoryImpl.class, "getManageInternalAcls",
        "setManageInternalAcls");
      pd_seventeen.setDisplayName("Internal ACLs");
      pd_seventeen.setShortDescription("An internal Access Control List defines dynamic rights"+
              " with a rights user and an access target (a resource) from this ACS.");
      pd_seventeen.setBound(true);
      //pd_seventeen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventeen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventeen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
  
      // ------------------ property "ManageExternalAcls" ---------------
      PropertyDescriptor pd_eighteen = new PropertyDescriptor(
        "ManageExternalAcls", ACSFactoryImpl.class, "getManageExternalAcls",
        "setManageExternalAcls");
      pd_eighteen.setDisplayName("External ACLs");
      pd_eighteen.setShortDescription("An external Access Control List defines dynamic rights"+
              " with a rights user and/or an access target (a resource) from another ACS.");
      pd_eighteen.setBound(true);
      //pd_eighteen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eighteen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eighteen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
  
      // ------------------ property "ManageActors" ---------------
      PropertyDescriptor pd_nineteen = new PropertyDescriptor(
        "ManageActors", ACSFactoryImpl.class, "getManageActors", "setManageActors");
      pd_nineteen.setDisplayName("Actors");
      pd_nineteen.setShortDescription("Actors are both resources and right users; "+
              "they are processes running generally under an Account/Group context");
      pd_nineteen.setBound(true);
      //pd_nineteen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_nineteen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_nineteen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      
      // ------------------ property "ManageBridges" ---------------
      PropertyDescriptor pd_twentyFour = new PropertyDescriptor(
        "ManageBridges", ACSFactoryImpl.class, "getManageBridges", "setManageBridges");
      pd_twentyFour.setDisplayName("Bridges (BDG)");
      pd_twentyFour.setShortDescription("Bridges are links"+
              "between 2 actors of the 2 different ACS, where the source executes the target.");
      pd_twentyFour.setBound(true);
      //pd_twentyFour.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twentyFour.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_twentyFour.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
    
      // ------------------ property "ManageSecondaryGroupsForActor" ---------------
      PropertyDescriptor pd_thirtyOne = new PropertyDescriptor(
        "ManageSecondaryGroupsForActor", ACSFactoryImpl.class, "getManageSecGroupsForActor", 
        "setManageSecGroupsForActor");
      pd_thirtyOne.setDisplayName("Actor Secondary Groups");
      pd_thirtyOne.setShortDescription("True if an actor may be member of secondary groups, "+
              "as right user.");
      pd_thirtyOne.setBound(true);
      //pd_thirtyOne.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtyOne.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_thirtyOne.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
     
      // ------------------ property "ManageDirectoryEPs" ---------------
      PropertyDescriptor pd_thirtyFour = new PropertyDescriptor(
        "ManageDirectoryEPs", ACSFactoryImpl.class, "getManageDirectoryEPs", 
        null);
      pd_thirtyFour.setDisplayName("Right User Directories");
      pd_thirtyFour.setShortDescription("True if some directories be right users "+
              "for being a privilege source");
      pd_thirtyFour.setBound(false);
      //pd_thirtyFour.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtyFour.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_thirtyFour.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );

      
      // ------------------ property "ManageVirtualFolders" --------------- 
      PropertyDescriptor pd_thirtySeven = new PropertyDescriptor(
        "ManageVirtualFolders", ACSFactoryImpl.class, "getManageVirtualFolders", 
        "setManageVirtualFolders");
      pd_thirtySeven.setDisplayName("Virtual Folders (VF)");
      pd_thirtySeven.setShortDescription("Virtual folders are containers of resources; "+
              "they are privilege targets");
      pd_thirtySeven.setBound(true);
      //pd_thirtySeven.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtySeven.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_thirtySeven.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      
      // ------------------ property "ManageAGORight" ---------------
      PropertyDescriptor pd_twentyEight = new PropertyDescriptor(
        "ManageAGORight", ACSFactoryImpl.class, "getManageAGORights", "setManageAGORights");
      pd_twentyEight.setDisplayName("Account and Group Rights");
      pd_twentyEight.setShortDescription("True if this ACS manages "+
              "Account Rights and/or the Group Rights (called AG rights) of a Resource");
      pd_twentyEight.setBound(true);
      //pd_twentyEight.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twentyEight.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_twentyEight.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      
      // ------------------ property "ManagePrivilegeRights" ---------------
      PropertyDescriptor pd_thirtySix = new PropertyDescriptor(
        "ManagePrivilegeRights", ACSFactoryImpl.class, "getManagePrivilegeRights", 
        "setManagePrivilegeRights");
      pd_thirtySix.setDisplayName("Privilege Rights (PRI)");
      pd_thirtySix.setShortDescription("A privilege defines some static rights "+
              "from a set of user rights to a set of resources or virtual folders.");
      pd_thirtySix.setBound(true);
      //pd_thirtySix.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtySix.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_thirtySix.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      
      // ------------------ property "ManageAcsRights" --------------- 
      PropertyDescriptor pd_sixty = new PropertyDescriptor(
        "ManageAcsRights", ACSFactoryImpl.class, "getManageAcsRights", 
        null);
      pd_sixty.setDisplayName("ACS Rights");
      pd_sixty.setShortDescription("true if there are rights that are proper to this ACS.");
      pd_sixty.setBound(true);
      //pd_sixty.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixty.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_sixty.setValue("panelGroupName", "  inter-ACS Structure  " );
     
      /*
      // ------------------ property "ManageMetaRights" --------------- 
      PropertyDescriptor pd_eightySeven = new PropertyDescriptor(
        "ManageMetaRights", ACSFactoryImpl.class, "getManageMetaRights", 
        null);
      pd_eightySeven.setDisplayName("Generic Rights");
      pd_eightySeven.setShortDescription("true if this ACS uses directly metarights, "+
              "also called generic rights since there are not owned by one ACS.");
      pd_eightySeven.setBound(false);
      //pd_eightySeven.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightySeven.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightySeven.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      */

      // ------------------ property "TiedToParentACS" ---------------
      PropertyDescriptor pd_fourty = new PropertyDescriptor(
        "TiedToParentACS", ACSFactoryImpl.class, "getTiedToParentACS",
        "setTiedToParentACS");
      pd_fourty.setDisplayName("Tied to ACS parent");
      pd_fourty.setShortDescription("True if the ACS is tied to its ACS parent.");
      pd_fourty.setBound(true);
      //pd_fourty.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourty.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourty.setValue("panelGroupName", "  inter-ACS Structure  " );

      // ------------------- property "SubAndGroupPolicy" --------------------
      PropertyDescriptor pd_fourtyOne = new PropertyDescriptor(
        "SubAndGroupPolicy", ACSFactoryImpl.class, "getM_SubAndGroupPolicy",
        null);
      pd_fourtyOne.setDisplayName("SubACSs/Group roots");
      pd_fourtyOne.setShortDescription("The subACSs and the group tree roots");
      pd_fourtyOne.setBound(true);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_fourtyOne.setValue("isStringMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_fourtyOne.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyOne.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtyOne.setValue("panelGroupName", "  A. The subACSs and "+
              "the group tree roots (option) ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_fourtyOne.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_fourtyOne.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_fourtyOne.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display a 'New' button. False if it is not editable trough a constructor.
      pd_fourtyOne.setValue("withNewButton", Boolean.TRUE ); // for map keys
      // 'withSelectButton' is the boolean which set if the StringMapPropertyEditor
      // has to display a 'Select' button.
      // False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_fourtyOne.setValue("withSelectButton", Boolean.TRUE ); // for map values
      // 'withDeleteButton' is the boolean which is set if the StringMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_fourtyOne.setValue("withRemoveButton", Boolean.TRUE ); // for map values
      pd_fourtyOne.setValue("withDeleteButton", Boolean.TRUE ); // for map keys
       // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor.
      pd_fourtyOne.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke
      // to let the user defines the new restricted key, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
        //pd_fourtyOne.setValue("choiceMethodForNew", new Object[] {
            //null, "getL_MapKeysStartsForTypePolicies"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined (only for the first value).
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
      pd_fourtyOne.setValue("limitedValuesMethod", "getL_LimitedValuesForSubAndGroupPolicy" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_fourtyOne.setValue("addMethod", "addSubAndGroupPolicy");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some values of a key.
      pd_fourtyOne.setValue("removeMethod", "removeSubAndGroupPolicy");
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      pd_fourtyOne.setValue("MaxValueSize", new Integer(2));
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      pd_fourtyOne.setValue("limitedSizeForPresentation", new Integer(4));
     
      // ------------------ property "AuthorizationServer" --------------- 
      PropertyDescriptor pd_fourtyTwo = new PropertyDescriptor(
        "AuthorizationServer", ACSFactoryImpl.class, "getIsAuthorizationServer",
        "setIsAuthorizationServer");
      pd_fourtyTwo.setDisplayName("Authorization Server");
      pd_fourtyTwo.setShortDescription("True if this ACS manages some rights other ACS "+
              "have to applied");
      pd_fourtyTwo.setBound(true);
      //pd_fourtyTwo.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyTwo.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtyTwo.setValue("panelGroupName", "  inter-ACS Structure  " ); 

      // ------------------- property "BelongsToComposite" --------------------
      PropertyDescriptor pd_fourtySix = new PropertyDescriptor(
        "BelongsToComposite", ACSFactoryImpl.class, "getM_BelongsToComposite",
        null);
      pd_fourtySix.setDisplayName("Composite Links");
      pd_fourtySix.setShortDescription("ACS or subACS which belongs to a composite");
      pd_fourtySix.setBound(true);
      // 'isBaseObjectMap' is TRUE when the right property editor is BaseObjectMapPropertyEditor
      pd_fourtySix.setValue("isBaseObjectMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_fourtySix.setValue("valueTypeInGroup", new Object[] {ImmutableName.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtySix.setValue("tabbedPaneName", "  Structure 4  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtySix.setValue("panelGroupName", "  C. ACS or subACS which belongs to a current "+
              "or a new composite (option) ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_fourtySix.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_fourtySix.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_fourtySix.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display a 'New' button. False if it is not editable trough a constructor.
      pd_fourtySix.setValue("withNewButton", Boolean.TRUE ); // for map keys
      // 'withSelectButton' is the boolean which set if the StringMapPropertyEditor
      // has to display a 'Select' button.
      // False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_fourtySix.setValue("withSelectButton", Boolean.TRUE ); // for map values
      // 'withDeleteButton' is the boolean which is set if the StringMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_fourtySix.setValue("withRemoveButton", Boolean.TRUE ); // for map values
      pd_fourtySix.setValue("withDeleteButton", Boolean.TRUE ); // for map keys
       // 'isRestrictedArgument' is true if it is a constrained _key_ that is not
      // in the constructor.
      pd_fourtySix.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke
      // to let the user defines the new restricted key, in a map, or the new instance,
      // for instance to chose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      pd_fourtySix.setValue("choiceMethodForNew", new Object[] {
            null, "getL_currentAcsAndSubAcsNickNames"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined.
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
      pd_fourtySix.setValue("limitedValuesMethod", "getL_LimitedValuesForBelongsToComposite" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_fourtySix.setValue("addMethod", "addBelongsToComposite");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some values of a key.
      pd_fourtySix.setValue("removeMethod", "removeBelongsToComposite");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      pd_fourtySix.setValue("limitedSizeForPresentation", new Integer(4));
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      pd_fourtySix.setValue("MaxValueSize", new Integer(1));

      // ------------------- property "Alternates" --------------------
      PropertyDescriptor pd_fourtySeven = new PropertyDescriptor(
        "Alternates", ACSFactoryImpl.class, "getM_Alternates",
        null);
      pd_fourtySeven.setDisplayName("Alternates");
      pd_fourtySeven.setShortDescription("Alternates for ACS and subACS");
      pd_fourtySeven.setBound(true);
      // 'isStringTwoKeysMap' is TRUE when the right property editor is StringTwoKeysMapPropertyEditor
      pd_fourtySeven.setValue("isStringTwoKeysMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_fourtySeven.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtySeven.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtySeven.setValue("panelGroupName", "  B. The alternates (option) "+
              "The subkey is a full alternate name (ends with '::  ') ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_fourtySeven.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_fourtySeven.setValue("withPreviousButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the PropertyEditor
      // has to display a 'New' button. False if it is not editable trough a constructor.
      pd_fourtySeven.setValue("withNewButton", Boolean.TRUE );
      // 'withNewVButton' is the boolean which set if the PropertyEditor
      // has to display a 'NewV' button if the map value (not key) is editable
      // in the beamer. False if it is not editable.
      pd_fourtySeven.setValue("withNewVButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the PropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      // 'withDeletVButton' matches with 'NewV'; it is also for map values only.
      pd_fourtySeven.setValue("withDeleteButton", Boolean.TRUE );
      pd_fourtySeven.setValue("withDeletVButton", Boolean.TRUE );
      // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor.
      pd_fourtySeven.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' is mandatory when 'isActionMap'/'isStringTwoKeysMap'
      // and 'KeysUpdateInMap' are true.
      // It defines a specific instance/method to invoke
      // for defining the restricted keys or key startings, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      pd_fourtySeven.setValue("choiceMethodForNew", new Object[] {
        null, "getL_MapKeysStartsForAlternates"} );
      // 'staticSetOfItems' is the array of possible keys for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method.
        //pd_fourtySeven.setValue("staticSetOfItems",
              //new String[] {"Explorer.AcsNode", "Explorer.NoAcsNode"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined.
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
      pd_fourtySeven.setValue("limitedValuesMethod", "getL_LimitedValuesForAlternates" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_fourtySeven.setValue("addMethod", "addAlternate");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some values of a key.
      pd_fourtySeven.setValue("removeMethod", "removeAlternate");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      pd_fourtySeven.setValue("limitedSizeForPresentation", new Integer(4));
      // 'horizontalTextPositionToLeft' is the boolean which is set if
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value.
      pd_fourtySeven.setValue("horizontalTextPositionToLeft", Boolean.TRUE );
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      pd_fourtySeven.setValue("MaxValueSize", new Integer(6));

      // ------------------- property "OpeningAndClosingRoots" --------------------
      PropertyDescriptor pd_fiftyOne = new PropertyDescriptor(
        "OpeningAndClosingRoots", ACSFactoryImpl.class, "getM_OpeningAndClosingRoots",
        null);
      pd_fiftyOne.setDisplayName("Opening and closing roots");
      pd_fiftyOne.setShortDescription("Roots in ACS ancestors to set at the ACS opening and closing");
      pd_fiftyOne.setBound(true);
      // 'isActionMap' is TRUE when the right property editor is AssociationMapPropertyEditor
      pd_fiftyOne.setValue("isActionMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_fiftyOne.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftyOne.setValue("tabbedPaneName", "  Roots  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fiftyOne.setValue("panelGroupName", "  A. Scripts for roots to set at the "+
              "ACS opening and closing - for advanced users  ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_fiftyOne.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_fiftyOne.setValue("withPreviousButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the PropertyEditor
      // has to display a 'New' button. False if it is not editable trough a constructor.
      pd_fiftyOne.setValue("withNewButton", Boolean.TRUE );
      // 'withNewVButton' is the boolean which set if the PropertyEditor
      // has to display a 'NewV' button if the map value (not key) is editable
      // in the beamer. False if it is not editable.
      pd_fiftyOne.setValue("withNewVButton", Boolean.TRUE );
      // 'withDeletVButton' is the boolean which is set if the PropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      // 'withDeletVButton' matches with 'NewV'; it is also for map values only.
      pd_fiftyOne.setValue("withDeleteButton", Boolean.TRUE );
      pd_fiftyOne.setValue("withDeletVButton", Boolean.TRUE );
      // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor.
      pd_fiftyOne.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' is mandatory when 'isActionMap'/'isStringTwoKeysMap'
      // and 'KeysUpdateInMap' are true.
      // It defines a specific instance/method to invoke
      // for defining the restricted keys or key startings, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      pd_fiftyOne.setValue("choiceMethodForNew", new Object[] {
        AssociateUtilityImpl.getInstance(), "getL_MapKeyStartsForAllRoots"} );
      // 'ForOneKeyAsName' is not null to specify the String format of a Name for the key.
      // The value has simply to be non-null to be effective.
      pd_fiftyOne.setValue("ForOneKeyAsName", new Boolean(true) );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined (only for the first value).
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
      pd_fiftyOne.setValue("limitedValuesMethod", "getL_LimitedFirstValuesForAssociation" );
      // 'limitedValuesMethodKey' is the key to use in an AssociationMapPropertyEditor only,
      // as the argument of the limited values method. Null for the association to a new object.
      pd_fiftyOne.setValue("limitedValuesMethodKey", "ForChildACS" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_fiftyOne.setValue("addMethod", "addOpeningAndClosingRoots");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
      pd_fiftyOne.setValue("removeMethod", "removeOpeningAndClosingRoots");
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      pd_fiftyOne.setValue("MaxValueSize", new Integer(990));
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      pd_fiftyOne.setValue("limitedSizeForPresentation", new Integer(4));
      // 'horizontalTextPositionToLeft' is the boolean which is set if
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value.
      pd_fiftyOne.setValue("horizontalTextPositionToLeft", Boolean.TRUE );

      // ------------------- property "OpeningNoClosingRoots" --------------------
      PropertyDescriptor pd_fiftyTwo = new PropertyDescriptor(
        "OpeningNoClosingRoots", ACSFactoryImpl.class, "getM_OpeningNoClosingRoots",
        null);
      pd_fiftyTwo.setDisplayName("Opening without closing roots");
      pd_fiftyTwo.setShortDescription("Roots in ACS ancestors to set only at the ACS opening");
      pd_fiftyTwo.setBound(true);
      // 'isActionMap' is TRUE when the right property editor is AssociationMapPropertyEditor
      pd_fiftyTwo.setValue("isActionMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_fiftyTwo.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftyTwo.setValue("tabbedPaneName", "  Roots  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fiftyTwo.setValue("panelGroupName", "  B. Scripts for roots to set "+
              "at the ACS opening only - for advanced users  ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_fiftyTwo.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_fiftyTwo.setValue("withPreviousButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the PropertyEditor
      // has to display a 'New' button. False if it is not editable trough a constructor.
      pd_fiftyTwo.setValue("withNewButton", Boolean.TRUE );
      // 'withNewVButton' is the boolean which set if the PropertyEditor
      // has to display a 'NewV' button if the map value (not key) is editable
      // in the beamer. False if it is not editable.
      pd_fiftyTwo.setValue("withNewVButton", Boolean.TRUE );
      // 'withDeletVButton' is the boolean which is set if the PropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      // 'withDeletVButton' matches with 'NewV'; it is also for map values only.
      pd_fiftyTwo.setValue("withDeleteButton", Boolean.TRUE );
      pd_fiftyTwo.setValue("withDeletVButton", Boolean.TRUE );
      // 'isRestrictedArgument' is true if it is a constrained property for keys that is not
      // in the constructor.
      pd_fiftyTwo.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' is mandatory when 'isActionMap'/'isStringTwoKeysMap'
      // and 'KeysUpdateInMap' are true.
      // It defines a specific instance/method to invoke
      // for defining the restricted keys or key startings, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      pd_fiftyTwo.setValue("choiceMethodForNew", new Object[] {
        AssociateUtilityImpl.getInstance(), "getL_MapKeyStartsForAllRoots"} );
      // 'ForOneKeyAsName' is not null to specify the String format of a Name for the key.
      // The value has simply to be non-null to be effective.
      pd_fiftyTwo.setValue("ForOneKeyAsName", new Boolean(true) );
      // 'limitedValuesMethodKey' is the key to use in an AssociationMapPropertyEditor only,
      // as the argument of the limited values method. Null for the association to a new object.
      pd_fiftyTwo.setValue("limitedValuesMethodKey", "ForChildACS" );
      // 'staticSetOfItems' is the array of possible keys for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method.
        //pd_fiftyTwo.setValue("staticSetOfItems",
              //new String[] {"Explorer.AcsNode", "Explorer.NoAcsNode"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined (only for the first value).
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
      pd_fiftyTwo.setValue("limitedValuesMethod", "getL_LimitedFirstValuesForAssociation" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_fiftyTwo.setValue("addMethod", "addOpeningNoClosingRoots");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
      pd_fiftyTwo.setValue("removeMethod", "removeOpeningNoClosingRoots");
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      pd_fiftyTwo.setValue("MaxValueSize", new Integer(990));
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      pd_fiftyTwo.setValue("limitedSizeForPresentation", new Integer(4));
      // 'horizontalTextPositionToLeft' is the boolean which is set if
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value.
      pd_fiftyTwo.setValue("horizontalTextPositionToLeft", Boolean.TRUE );

      // ------------------- property "IntegrityRoots" --------------------
      PropertyDescriptor pd_fiftyThree = new PropertyDescriptor(
        "IntegrityRoots", ACSFactoryImpl.class, "getM_IntegrityRoots",
        null);
      pd_fiftyThree.setDisplayName("Integrity roots");
      pd_fiftyThree.setShortDescription("Scripts for roots in ACS ancestors which "+
              "ensure the ACS integrity");
      pd_fiftyThree.setBound(true);
      // 'isActionMap' is TRUE when the right property editor is AssociationMapPropertyEditor
      pd_fiftyThree.setValue("isActionMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_fiftyThree.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftyThree.setValue("tabbedPaneName", "  Roots  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fiftyThree.setValue("panelGroupName", "  C. Roots in ACS ancestors which ensure "+
              "the ACS integrity - for advanced users  ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_fiftyThree.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_fiftyThree.setValue("withPreviousButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the PropertyEditor
      // has to display a 'New' button. False if it is not editable trough a constructor.
      pd_fiftyThree.setValue("withNewButton", Boolean.TRUE );
      // 'withNewVButton' is the boolean which set if the PropertyEditor
      // has to display a 'NewV' button if the map value (not key) is editable
      // in the beamer. False if it is not editable.
      pd_fiftyThree.setValue("withNewVButton", Boolean.TRUE );
      // 'withDeletVButton' is the boolean which is set if the PropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      // 'withDeletVButton' matches with 'NewV'; it is also for map values only.
      pd_fiftyThree.setValue("withDeleteButton", Boolean.TRUE );
      pd_fiftyThree.setValue("withDeletVButton", Boolean.TRUE );
      // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor.
      pd_fiftyThree.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' is mandatory when 'isActionMap'/'isStringTwoKeysMap'
      // and 'KeysUpdateInMap' are true.
      // It defines a specific instance/method to invoke
      // for defining the restricted keys or key startings, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      pd_fiftyThree.setValue("choiceMethodForNew", new Object[] {
        AssociateUtilityImpl.getInstance(), "getL_MapKeyStartsForAllRoots"} );
      // 'ForOneKeyAsName' is not null to specify the String format of a Name for the key.
      // The value has simply to be non-null to be effective.
      pd_fiftyThree.setValue("ForOneKeyAsName", new Boolean(true) );
      // 'staticSetOfItems' is the array of possible keys for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method.
        //pd_fiftyThree.setValue("staticSetOfItems",
              //new String[] {"Explorer.AcsNode", "Explorer.NoAcsNode"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined (only for the first value).
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
      pd_fiftyThree.setValue("limitedValuesMethod", "getL_LimitedFirstValuesForAssociation" );
      // 'limitedValuesMethodKey' is the key to use in an AssociationMapPropertyEditor only,
      // as the argument of the limited values method. Null for the association to a new object.
      pd_fiftyThree.setValue("limitedValuesMethodKey", "ForChildACS" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_fiftyThree.setValue("addMethod", "addIntegrityRoots");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
      pd_fiftyThree.setValue("removeMethod", "removeIntegrityRoots");
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      pd_fiftyThree.setValue("MaxValueSize", new Integer(990));
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      pd_fiftyThree.setValue("limitedSizeForPresentation", new Integer(4));
      // 'horizontalTextPositionToLeft' is the boolean which is set if
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value.
      pd_fiftyThree.setValue("horizontalTextPositionToLeft", Boolean.TRUE );

      // ------------------- property "ComponentNodes" --------------------
      PropertyDescriptor pd_fourtyEight = new PropertyDescriptor(
        "ComponentNodes", ACSFactoryImpl.class, "getM_ComponentNodes",
        null);
      pd_fourtyEight.setDisplayName("New components");
      pd_fourtyEight.setShortDescription("The logical or physical components to create"+
              " under the new ACS or its parent");
      pd_fourtyEight.setBound(true);
      // 'isStringMap' is TRUE when the right property editor is StringTwoKeysMapPropertyEditor
      pd_fourtyEight.setValue("isStringTwoKeysMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_fourtyEight.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyEight.setValue("tabbedPaneName", "  Structure 4  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtyEight.setValue("panelGroupName", "  A. New physical/logical components"+
              " under the ACS or its parent (option) ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_fourtyEight.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_fourtyEight.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_fourtyEight.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display a 'New' button. False if it is not editable trough a constructor.
      pd_fourtyEight.setValue("withNewButton", Boolean.TRUE ); // for map keys
      // 'withSelectButton' is the boolean which set if the StringMapPropertyEditor
      // has to display a 'Select' button.
      // False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_fourtyEight.setValue("withSelectButton", Boolean.TRUE ); // for map values
      // 'withDeleteButton' is the boolean which is set if the StringMapPropertyEditor
      // has to display a 'Delete' button.
      // If false, there are 'Remove' buttons to remove from the property.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_fourtyEight.setValue("withRemoveButton", Boolean.TRUE ); // for map values
      pd_fourtyEight.setValue("withDeleteButton", Boolean.TRUE ); // for map keys
       // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor.
      pd_fourtyEight.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke
      // to let the user defines the new restricted key, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      pd_fourtyEight.setValue("choiceMethodForNew", new Object[] {
            null, "getL_MapKeysStartsForComponents"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property,
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
      pd_fourtyEight.setValue("limitedValuesMethod", "getL_LimitedValuesForComponentNodes" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_fourtyEight.setValue("addMethod", "addComponentNode");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some values of a key.
      pd_fourtyEight.setValue("removeMethod", "removeComponentNode");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      pd_fourtyEight.setValue("limitedSizeForPresentation", new Integer(8));
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      // The default is 100 for a StringMap or one of its subclass.
      pd_fourtyEight.setValue("MaxValueSize", new Integer(1));

      // ------------------- property "CompositeNodes" --------------------
      PropertyDescriptor pd_fourtyNine = new PropertyDescriptor(
        "CompositeNodes", ACSFactoryImpl.class, "getM_CompositeNodes",
        null);
      pd_fourtyNine.setDisplayName("New composites");
      pd_fourtyNine.setShortDescription("The composites to create in the IS");
      pd_fourtyNine.setBound(true);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_fourtyNine.setValue("isStringMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_fourtyNine.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyNine.setValue("tabbedPaneName", "  Structure 4  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtyNine.setValue("panelGroupName", "  B. New composites of the IS -"+
              " may be populated in C. (option) ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_fourtyNine.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_fourtyNine.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_fourtyNine.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display a 'New' button. False if it is not editable trough a constructor.
      pd_fourtyNine.setValue("withNewButton", Boolean.TRUE ); // for map keys
      // 'withSelectButton' is the boolean which set if the StringMapPropertyEditor
      // has to display a 'Select' button.
      // False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_fourtyNine.setValue("withSelectButton", Boolean.TRUE ); // for map values
      // 'withDeleteButton' is the boolean which is set if the StringMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_fourtyNine.setValue("withRemoveButton", Boolean.TRUE ); // for map values
      pd_fourtyNine.setValue("withDeleteButton", Boolean.TRUE ); // for map keys
       // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor.
      pd_fourtyNine.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke
      // to let the user defines the new restricted key, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
        //pd_fourtyNine.setValue("choiceMethodForNew", new Object[] {
          //  null, "getL_MapKeysStartsForComponents"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property,
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
      pd_fourtyNine.setValue("limitedValuesMethod", "getL_LimitedValuesForCompositeNodes" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_fourtyNine.setValue("addMethod", "addCompositeNode");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some values of a key.
      pd_fourtyNine.setValue("removeMethod", "removeCompositeNode");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      pd_fourtyNine.setValue("limitedSizeForPresentation", new Integer(8));
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      pd_fourtyNine.setValue("MaxValueSize", new Integer(1));

      // ------------------- property "Icon" ------------------------
      PropertyDescriptor pd_fiftyFour = new PropertyDescriptor(
        "Icon", ACSFactoryImpl.class, "getIcon", "setIcon");
      pd_fiftyFour.setDisplayName("ACS icon (option)");
      pd_fiftyFour.setShortDescription("The ACS icon 'XXXX.gif' under ARoad0.Gui1.images");
      pd_fiftyFour.setBound(true);
      //pd_fiftyFour.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftyFour.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_fiftyFour.setValue("panelGroupName", "  Types  " );

      // ------------------- property "CommentsOnProperties" --------------------
      PropertyDescriptor pd_seventyFive = new PropertyDescriptor(
        "CommentsOnProperties", ACSFactoryImpl.class, "getM_CommentsOnProperties",
        null);
      pd_seventyFive.setDisplayName("Comments on properties");
      pd_seventyFive.setShortDescription(
              "Comments on how this ACS simulates the real system/software");
      pd_seventyFive.setBound(true);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_seventyFive.setValue("isStringMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_seventyFive.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventyFive.setValue("tabbedPaneName", "  See Why  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventyFive.setValue("panelGroupName",
              "  A. How this ACS simulates the real system/software, "+
              "for each property (option) ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_seventyFive.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_seventyFive.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_seventyFive.setValue("withViewButton", Boolean.FALSE );
      // 'withNewVButton' is the boolean which set if the PropertyEditor
      // has to display a 'NewV' button if the map value (not key) is editable
      // in the beamer. False if it is not editable.
      pd_seventyFive.setValue("withNewButton", Boolean.TRUE );
      pd_seventyFive.setValue("withNewVButton", Boolean.TRUE );
      // 'withDeleteVButton' is the boolean which is set if the PropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      // 'withDeletVButton' matches with 'NewV'; it is also for map values only.
      pd_seventyFive.setValue("withDeleteButton", Boolean.TRUE );
      pd_seventyFive.setValue("withDeletVButton", Boolean.TRUE );
       // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor.
      pd_seventyFive.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke
      // to let the user defines the new restricted key, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      pd_seventyFive.setValue("choiceMethodForNew", new Object[] {
        null, "getL_MapKeysForCommentsOnProperties"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined (only for the first value).
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
        //pd_seventyFive.setValue("limitedValuesMethod",
          //    "getL_LimitedValuesForTypesAndExplorer" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_seventyFive.setValue("addMethod", "addCommentsOnProperties");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
      pd_seventyFive.setValue("removeMethod", "removeCommentsOnProperties");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      pd_seventyFive.setValue("limitedSizeForPresentation", new Integer(16));
      // 'horizontalTextPositionToLeft' is the boolean which is set if 
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value. 
      pd_seventyFive.setValue("horizontalTextPositionToLeft", Boolean.TRUE );

      // ------------------- property "AcsVocabulary" --------------------
      PropertyDescriptor pd_seventyNine = new PropertyDescriptor(
        "AcsVocabulary", ACSFactoryImpl.class, "getM_AcsVocabulary",
        null);
      pd_seventyNine.setDisplayName("Vocabulary");
      pd_seventyNine.setShortDescription(
              "Used classes, including AcsAddon classes, and terms");
      pd_seventyNine.setBound(true);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_seventyNine.setValue("isStringMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_seventyNine.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventyNine.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventyNine.setValue("panelGroupName",
              "  C. What classes are used by the ACS, with the AcsAddon classes and terms (option) ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_seventyNine.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_seventyNine.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_seventyNine.setValue("withViewButton", Boolean.FALSE );
      // 'withNewVButton' is the boolean which set if the PropertyEditor
      // has to display a 'NewV' button if the map value (not key) is editable
      // in the beamer. False if it is not editable.
      pd_seventyNine.setValue("withNewButton", Boolean.TRUE );
      pd_seventyNine.setValue("withNewVButton", Boolean.TRUE );
      // 'withDeleteVButton' is the boolean which is set if the PropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      // 'withDeletVButton' matches with 'NewV'; it is also for map values only.
      pd_seventyNine.setValue("withDeleteButton", Boolean.TRUE );
      pd_seventyNine.setValue("withDeletVButton", Boolean.TRUE );
       // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor.
      pd_seventyNine.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke
      // to let the user defines the new restricted key, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      pd_seventyNine.setValue("choiceMethodForNew", new Object[] {
        null, "getL_MapKeysForAcsVocabulary"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined (only for the first value).
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
        //pd_seventyFive.setValue("limitedValuesMethod",
          //    "getL_LimitedValuesForTypesAndExplorer" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_seventyNine.setValue("addMethod", "addAcsVocabulary");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
      pd_seventyNine.setValue("removeMethod", "removeAcsVocabulary");
      // 'horizontalTextPositionToLeft' is the boolean which is set if 
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value. 
      pd_seventyNine.setValue("horizontalTextPositionToLeft", Boolean.TRUE );
      // 'MaxValueSize' is the maximal size of the values list for each map key.
        //pd_seventyNine.setValue("MaxValueSize", new Integer(1));
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      pd_seventyNine.setValue("limitedSizeForPresentation", new Integer(6));
      
      // ------------------ property "IncompleteStructureModeling" ---------------
      PropertyDescriptor pd_eightyOne = new PropertyDescriptor(
        "IncompleteStructureModeling", ACSFactoryImpl.class, "getIncompleteStructureModeling",
        "setIncompleteStructureModeling");
      pd_eightyOne.setDisplayName("Incomplete objects list");
      pd_eightyOne.setShortDescription("true if the list of ACS objects is incomplete.");
      pd_eightyOne.setBound(true);
      //pd_eightyOne.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyOne.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_eightyOne.setValue("panelGroupName", "  inter-ACS Structure  " );

      // ------------------ property "IncompleteBehaviorModeling" ---------------
      PropertyDescriptor pd_eightyTwo = new PropertyDescriptor(
        "IncompleteBehaviorModeling", ACSFactoryImpl.class, "getIncompleteBehaviorModeling",
        "setIncompleteBehaviorModeling");
      pd_eightyTwo.setDisplayName("Incomplete behavior");
      pd_eightyTwo.setShortDescription("true if the modeling of the ACS behavior is incomplete.");
      pd_eightyTwo.setBound(true);
      //pd_eightyTwo.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyTwo.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_eightyTwo.setValue("panelGroupName", "  inter-ACS Structure  " );

      // ------------------ property "ManageIfAllSourceLinkedPrivileges" ---------------
      PropertyDescriptor pd_100 = new PropertyDescriptor(
        "ManageIfAllSourceLinkedPrivilegess", ACSFactoryImpl.class,
        "getManageIfAllSourceLinkedPrivileges",
        "setManageIfAllSourceLinkedPrivileges");
      pd_100.setDisplayName("If-All-sources LPRI");
      pd_100.setShortDescription("true if the If-All-Sources linked privileges are managed.");
      pd_100.setBound(true);
      //pd_100.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_100.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_100.setValue("panelGroupName", "  How privileges "+
              "are handled by this ACS, if any  ");

      // ------------------ property "ManageInternalForOneToOneLinkedPrivileges" ---------------
      PropertyDescriptor pd_101 = new PropertyDescriptor(
        "ManageInternalForOneToOneLinkedPrivileges", ACSFactoryImpl.class,
        "getManageInternalForOneToOneLinkedPrivileges",
        "setManageInternalForOneToOneLinkedPrivileges");
      pd_101.setDisplayName("Internal One-to-One LPRI");
      pd_101.setShortDescription("true if the internal One-To-One linked privileges are managed.");
      pd_101.setBound(true);
      //pd_101.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_101.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_101.setValue("panelGroupName", "  How privileges "+
              "are handled by this ACS, if any  ");

      // ------------------ property "ManageExternalForOneToOneLinkedPrivileges" ---------------
      PropertyDescriptor pd_102 = new PropertyDescriptor(
        "ManageExternalForOneToOneLinkedPrivileges", ACSFactoryImpl.class,
        "getManageExternalForOneToOneLinkedPrivileges",
        "setManageExternalForOneToOneLinkedPrivileges");
      pd_102.setDisplayName("External One-to-One LPRI");
      pd_102.setShortDescription("true if the external One-To-One linked privileges are managed.");
      pd_102.setBound(true);
      //pd_102.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_102.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_102.setValue("panelGroupName", "  How privileges "+
              "are handled by this ACS, if any  ");

      // ------------------ property "ManageClassedTypedPrivileges" ---------------
      PropertyDescriptor pd_103 = new PropertyDescriptor(
        "ManageClassedTypedPrivileges", ACSFactoryImpl.class,
        "getManageClassedTypedPrivileges",
        "setManageClassedTypedPrivileges");
      pd_103.setDisplayName("Classed TPRI");
      pd_103.setShortDescription("true if the classed typed privileges are managed.");
      pd_103.setBound(true);
      //pd_103.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_103.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_103.setValue("panelGroupName", "  How privileges "+
              "are handled by this ACS, if any  ");

      // ------------------ property "ManageClassedSpecificTypedPrivileges" ---------------
      PropertyDescriptor pd_104 = new PropertyDescriptor(
        "ManageClassedSpecificTypedPrivileges", ACSFactoryImpl.class,
        "getManageClassedSpecificTypedPrivileges",
        "setManageClassedSpecificTypedPrivileges");
      pd_104.setDisplayName("Specific TPRI");
      pd_104.setShortDescription("true if the specific typed privileges are managed "+
              "for designated source and target.");
      pd_104.setBound(true);
      //pd_104.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_104.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_104.setValue("panelGroupName", "  How privileges "+
              "are handled by this ACS, if any  ");

      // ------------------ property "ManageAuthorizationServerTypedPrivileges" ---------------
      PropertyDescriptor pd_105 = new PropertyDescriptor(
        "ManageAuthorizationServerTypedPrivileges", ACSFactoryImpl.class,
        "getManageAuthorizationServerTypedPrivileges",
        "setManageAuthorizationServerTypedPrivileges");
      pd_105.setDisplayName("Authorization server TPRI");
      pd_105.setShortDescription("true if the typed privileges are managed "+
              "for authorization server.");
      pd_105.setBound(true);
      //pd_105.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_105.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_105.setValue("panelGroupName", "  How privileges "+
              "are handled by this ACS, if any  ");

      // ------------------ property "ManagePrivilegesInVirtualFolders" ---------------
      PropertyDescriptor pd_106 = new PropertyDescriptor(
        "ManagePrivilegesInVirtualFolders", ACSFactoryImpl.class,
        "getManagePrivilegesInVirtualFolders", "setManagePrivilegesInVirtualFolders");
      pd_106.setDisplayName("Privileges in VirtualFolders");
      pd_106.setShortDescription("true if the privileges are managed in the virtual folders.");
      pd_106.setBound(true);
      //pd_eightyThree.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_106.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_106.setValue("panelGroupName", "  How privileges "+
              "are handled by this ACS, if any  ");

      // ------------------ property "ManagePrivilegesInResources" ---------------
      PropertyDescriptor pd_107 = new PropertyDescriptor(
        "ManagePrivilegesInResources", ACSFactoryImpl.class,
        "getManagePrivilegesInResources", "setManagePrivilegesInResources");
      pd_107.setDisplayName("Privileges in Resources");
      pd_107.setShortDescription("true if the privileges are managed in the resources.");
      pd_107.setBound(true);
      //pd_eightyThree.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_107.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_107.setValue("panelGroupName", "  How privileges "+
              "are handled by this ACS, if any  ");
      
      // ------------------ property "ManageSimpleLinkedPrivileges" ---------------
      PropertyDescriptor pd_eightyThree = new PropertyDescriptor(
        "ManageSimpleLinkedPrivileges", ACSFactoryImpl.class, "getManageSimpleLinkedPrivileges",
        "setManageSimpleLinkedPrivileges");
      pd_eightyThree.setDisplayName("Simple Linked Privileges");
      pd_eightyThree.setShortDescription("true if the simplest linked privileges are managed.");
      pd_eightyThree.setBound(true);
      //pd_eightyThree.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyThree.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightyThree.setValue("panelGroupName", "  How privileges "+
              "are handled by this ACS, if any  " );

      // ------------------ property "ManageSimpleTypedPrivileges" ---------------
      PropertyDescriptor pd_eightyFour = new PropertyDescriptor(
        "ManageSimpleTypedPrivileges", ACSFactoryImpl.class, "getManageSimpleTypedPrivileges",
        "setManageSimpleTypedPrivileges");
      pd_eightyFour.setDisplayName("Simple Typed Privileges");
      pd_eightyFour.setShortDescription("true if the simplest typed privileges are managed.");
      pd_eightyFour.setBound(true);
      //pd_eightyFour.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyFour.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightyFour.setValue("panelGroupName", "  How privileges "+
              "are handled by this ACS, if any  " );

      // ------------------ property "ManageGrantingRightPrivileges" ---------------
      PropertyDescriptor pd_eightyFive = new PropertyDescriptor(
        "ManageGrantingRightPrivileges", ACSFactoryImpl.class, "getManageGrantingRightPrivileges",
        "setManageGrantingRightPrivileges");
      pd_eightyFive.setDisplayName("Granting privileges");
      pd_eightyFive.setShortDescription("true if the granting privileges are managed.");
      pd_eightyFive.setBound(true);
      //pd_eightyFive.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyFive.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightyFive.setValue("panelGroupName", "  How privileges "+
              "are handled by this ACS, if any  " );

      // ------------------ property "ManageDenyingRightPrivileges" ---------------
      PropertyDescriptor pd_eightySix = new PropertyDescriptor(
        "ManageDenyingRightPrivileges", ACSFactoryImpl.class, "getManageDenyingRightPrivileges",
        "setManageDenyingRightPrivileges");
      pd_eightySix.setDisplayName("Denying privileges");
      pd_eightySix.setShortDescription("true if the denying privileges are managed.");
      pd_eightySix.setBound(true);
      //pd_eightySix.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightySix.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightySix.setValue("panelGroupName", "  How privileges "+
              "are handled by this ACS, if any  " );

      // ------------------ property "ManageResourceVirtualFolders" ---------------
      PropertyDescriptor pd_eightySeven = new PropertyDescriptor(
        "ManageResourceVirtualFolders", ACSFactoryImpl.class,
        "getManageResourceVirtualFolders",
        "setManageResourceVirtualFolders");
      pd_eightySeven.setDisplayName("VF of simple Resources");
      pd_eightySeven.setShortDescription("true if the Resources are managed"+
              " in Virtual Folders.");
      pd_eightySeven.setBound(true);
      //pd_eightySeven.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightySeven.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightySeven.setValue("panelGroupName", "  How virtual folders "+
              "are handled by this ACS, if any  " );

      // ------------------ property "ManageEPVirtualFolders" ---------------
      PropertyDescriptor pd_eightyEight = new PropertyDescriptor(
        "ManageEPVirtualFolders", ACSFactoryImpl.class, "getManageEPVirtualFolders",
        "setManageEPVirtualFolders");
      pd_eightyEight.setDisplayName("VF of Right User Resources");
      pd_eightyEight.setShortDescription("true if the Actors "+
              "and/or the DirectoryEPs are managed in some Virtual Folders.");
      pd_eightyEight.setBound(true);
      //pd_eightyEight.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyEight.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightyEight.setValue("panelGroupName", "  How virtual folders "+
              "are handled by this ACS, if any  " );

      // ------------------ property "ManageSimpleResourceVirtualFolders" ---------------
      PropertyDescriptor pd_eightyNine = new PropertyDescriptor(
        "ManageSimpleResourceVirtualFolders", ACSFactoryImpl.class,
        "getManageSimpleResourceVirtualFolders",
        "setManageSimpleResourceVirtualFolders");
      pd_eightyNine.setDisplayName("Simplest items in Res-VF");
      pd_eightyNine.setShortDescription("true if the no-directory resources are managed"+
              " in Resource Virtual Folders.");
      pd_eightyNine.setBound(true);
      //pd_eightyNine.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyNine.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightyNine.setValue("panelGroupName", "  How virtual folders "+
              "are handled by this ACS, if any  " );

      // ------------------ property "ManageDirectoryInResourceVirtualFolders" ---------------
      PropertyDescriptor pd_ninetyOne = new PropertyDescriptor(
        "ManageDirectoryInResourceVirtualFolders", ACSFactoryImpl.class,
        "getManageDirectoryInResourceVirtualFolders", "setManageDirectoryInResourceVirtualFolders");
      pd_ninetyOne.setDisplayName("Directories in Res-VF");
      pd_ninetyOne.setShortDescription("true if the directories are managed in"+
              " the Virtual Folders.");
      pd_ninetyOne.setBound(true);
      //pd_ninetyOne.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetyOne.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetyOne.setValue("panelGroupName", "  How virtual folders "+
              "are handled by this ACS, if any  " );

      // ------------------ property "ManageDirectoryEPinEPVirtualFolders" ---------------
      PropertyDescriptor pd_ninetyTwo = new PropertyDescriptor(
        "ManageDirectoryEPinEPVirtualFolders", ACSFactoryImpl.class,
        "getManageDirectoryEPinEPVirtualFolders",
        "setManageDirectoryEPinEPVirtualFolders");
      pd_ninetyTwo.setDisplayName("DirectoryEPs in RU-VF");
      pd_ninetyTwo.setShortDescription("true if the right user directoriess are managed"+
              " in the Right User Virtual Folders.");
      pd_ninetyTwo.setBound(true);
      //pd_ninetyTwo.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetyTwo.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetyTwo.setValue("panelGroupName", "  How virtual folders "+
              "are handled by this ACS, if any  " );

      // ------------------ property "ManageActorInEPVirtualFolders" ---------------
      PropertyDescriptor pd_ninetyThree = new PropertyDescriptor(
        "ManageActorInEPVirtualFolders", ACSFactoryImpl.class,
        "getManageActorInEPVirtualFolders",
        "setManageActorInEPVirtualFolders");
      pd_ninetyThree.setDisplayName("Actors in RU-VF");
      pd_ninetyThree.setShortDescription("true if the actors are managed in"+
              " the Right User Virtual Folders.");
      pd_ninetyThree.setBound(true);
      //pd_ninetyThree.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetyThree.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetyThree.setValue("panelGroupName", "  How virtual folders "+
              "are handled by this ACS, if any  " );

      // ------------------ property "ManageACLRightInheritance" ---------------
      PropertyDescriptor pd_ninetyFour = new PropertyDescriptor(
        "ManageACLRightInheritance", ACSFactoryImpl.class,
        "getManageACLRightsInheritance",
        "setManageACLRightsInheritance");
      pd_ninetyFour.setDisplayName("ACL inheritance");
      pd_ninetyFour.setShortDescription("true if the AclEntry inheritance is managed"+
              " in the resources tree.");
      pd_ninetyFour.setBound(true);
      //pd_ninetyFour.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetyFour.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetyFour.setValue("panelGroupName", 
              "  The main concepts which are managed or not by this ACS  " );

      // ------------------ property "ManageAGORightInheritance" ---------------
      PropertyDescriptor pd_ninetyFive = new PropertyDescriptor(
        "ManageAGORightInheritance", ACSFactoryImpl.class,
        "getManageAGORightsInheritance",
        "setManageAGORightsInheritance");
      pd_ninetyFive.setDisplayName("Account/Group rights inheritance");
      pd_ninetyFive.setShortDescription("true if the Account/Group rights inheritance is managed"+
              " in the resources tree.");
      pd_ninetyFive.setBound(true);
      //pd_ninetyFive.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetyFive.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetyFive.setValue("panelGroupName",
              "  The main concepts which are managed or not by this ACS  ");

      // ------------------ property "ManageLPRIRightInheritance" ---------------
      PropertyDescriptor pd_ninetySeven = new PropertyDescriptor(
        "ManageLPRIRightInheritance", ACSFactoryImpl.class,
        "getManageLPRIRightsInheritance",
        "setManageLPRIRightsInheritance");
      pd_ninetySeven.setDisplayName("PRI inheritance");
      pd_ninetySeven.setShortDescription("true if the Linked Privileges inheritance is managed"+
              " in the resources or virtual folders tree.");
      pd_ninetySeven.setBound(true);
      //pd_ninetySeven.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetySeven.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetySeven.setValue("panelGroupName",
              "  The main concepts which are managed or not by this ACS  ");

      // ------------------ property "ManageVirtualFolderTrees" ---------------
      PropertyDescriptor pd_ninetySix = new PropertyDescriptor(
        "ManageVirtualFolderTrees", ACSFactoryImpl.class,
        "getManageVirtualFolderTrees",
        "setManageVirtualFolderTrees");
      pd_ninetySix.setDisplayName("Virtual Folders Trees");
      pd_ninetySix.setShortDescription("true if 2 trees are managed"+
              " for the 2 main types of Virtual Folders.");
      pd_ninetySix.setBound(true);
      //pd_ninetySix.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetySix.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetySix.setValue("panelGroupName", 
              "  The main concepts which are managed or not by this ACS  " );

      // ------------------ property "ManageOneTimeInVirtualFolderTreeForMember"
      PropertyDescriptor pd_108 = new PropertyDescriptor(
        "ManageOneTimeInVirtualFolderTreeForMember", ACSFactoryImpl.class,
        "getManageOneTimeInVirtualFolderTreeForMember",
        "setManageOneTimeInVirtualFolderTreeForMember");
      pd_108.setDisplayName("One time in tree");
      pd_108.setShortDescription("true if a member is no more than one time"+
              " in each virtual folder tree.");
      pd_108.setBound(true);
      //pd_108.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_108.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_108.setValue("panelGroupName", 
              "  How virtual folders "+
              "are handled by this ACS, if any  " );

      // ------------------ property "ManageVirtualFolderRightsPropagationToMembers" 
      PropertyDescriptor pd_111 = new PropertyDescriptor(
        "ManageVirtualFolderRightsPropagationToMembers", ACSFactoryImpl.class,
        "getManageVirtualFolderRightsPropagationToMembers",
        "setManageVirtualFolderRightsPropagationToMembers");
      pd_111.setDisplayName("Right transfer to members");
      pd_111.setShortDescription("true if a virtual folder transfers its target rights"+
              "to each of its members.");
      pd_111.setBound(true);
      //pd_111.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_111.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_111.setValue("panelGroupName", 
              "  How virtual folders "+
              "are handled by this ACS, if any  " );

      // ------------------ property "ManageInternalHardAlias" ---------------
      PropertyDescriptor pd_109 = new PropertyDescriptor(
        "ManageInternalHardAlias", ACSFactoryImpl.class,
        "getManageInternalHardAlias",
        "setManageInternalHardAlias");
      pd_109.setDisplayName("Hard aliasing");
      pd_109.setShortDescription("true if an acs object may be the alias of another one"+
              " from the same acs  (like a Linux symbolic link).");
      pd_109.setBound(true);
      //pd_109.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_109.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_109.setValue("panelGroupName", 
              "  The main concepts which are managed or not by this ACS  " );

      // ------------------ property "ManageSoftAlias" ---------------
      PropertyDescriptor pd_110 = new PropertyDescriptor(
        "ManageSoftAlias", ACSFactoryImpl.class,
        "getManageSoftAlias", null);
        //"setManageSoftAlias");
      pd_110.setDisplayName("Soft aliasing");
      pd_110.setShortDescription("true if an acs object may be the alias of an"+
              "external object, following the acs proper rules"+
              " (like a Linux hard link).");
      pd_110.setBound(false);
      //pd_110.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_110.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_110.setValue("panelGroupName", 
              "  The main concepts which are managed or not by this ACS  " );

      // ------------------ property "ManageFlexibleConditionalAcls" ---------------
      PropertyDescriptor pd_112 = new PropertyDescriptor(
        "ManageFlexibleConditionalAcls", ACSFactoryImpl.class,
        "getManageFlexibleConditionalAcls",
        "setManageFlexibleConditionalAcls");
      pd_112.setDisplayName("Simple-condition ACL");
      pd_112.setShortDescription("true if an AclEntry may be conditional, desabling the rights "+
              "if the source is not member of each condition group.");
      pd_112.setBound(true);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_112.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_112.setValue("panelGroupName", 
              "  The main concepts which are managed or not by this ACS  " );

      // ------------------ property "ManageRightDefinedConditionalAcls" ---------------
      PropertyDescriptor pd_113 = new PropertyDescriptor(
        "ManageRightDefinedConditionalAcls", ACSFactoryImpl.class,
        "getManageRightDefinedConditionalAcls",
        "setManageRightDefinedConditionalAcls");
      pd_113.setDisplayName("Right-condition ACL");
      pd_113.setShortDescription("true if an AclEntry may be conditional, desabling the rights "+
              "if the source is not member of each right-defined condition group.");
      pd_113.setBound(true);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_113.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_113.setValue("panelGroupName", 
              "  The main concepts which are managed or not by this ACS  " );

      // ------------------- property "PrivilegeRightsSet" --------------------
      PropertyDescriptor pd_eight = new PropertyDescriptor(
        "PrivilegeRightsSet", ACSFactoryImpl.class, "getM_PrivilegeRights",
        null);
      pd_eight.setDisplayName("Privilege rights");
      pd_eight.setShortDescription(
              "The allowed standard or specialized rights for the Privileges");
      pd_eight.setBound(true);
      // 'isRightMap' is TRUE when the right property editor is RightMapPropertyEditor
      pd_eight.setValue("isBaseObjectMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, it defines the type of the elementary object to handle.
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_eight.setValue("valueTypeInGroup", new Object[] {StringRight.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eight.setValue("tabbedPaneName", "  Rights 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eight.setValue("panelGroupName",
              "  Privilege Rights - sets standard and specialized rights (option) ");
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_eight.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_eight.setValue("withPreviousButton", Boolean.TRUE );
      // 'withViewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_eight.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'Add New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_eight.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the RightMapPropertyEditor
      // has to display 2 'Select' buttons if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_eight.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the RightMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_eight.setValue("withRemoveButton", Boolean.TRUE );
      pd_eight.setValue("withDeleteButton", Boolean.TRUE );
      // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor.
      // The restricted values are get from 'staticSetOfItems', for Strings only,
      // secondly from 'choiceMethodForNew' like here,
      // or through a search in the ACS by the PropertyEditor.
      pd_eight.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke
      // to let the user defines the new restricted key, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      pd_eight.setValue("choiceMethodForNew", new Object[] {
        null, "getL_MapKeysForPrivilegeRights"} );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the fixed argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method to initialize a l_fixedValue from this object.
      // Used by SimplePropertyEditor.getKnownValuesForNewElement(), which
      // may use a subset of the returned arguments if the class of the object to create
      // waits a smaller set of arguments.
      // Caution: if 'choiceMethodForNew' exists, then the constrained arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
        //pd_eight.setValue("withKnownArgumentsForNew", new Object[] {null, null} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_eight.setValue("addMethod", "addPrivilegeRights");
      // 'restrictedKeyForSpecializedRight' indicates if the key is restricted
      // to some specified values for the map key of a new specialized rights.
        //pd_eight.setValue("restrictedKeyForSpecializedRight", Boolean.TRUE );
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
      pd_eight.setValue("removeMethod", "removePrivilegeRights");
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_eight.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      pd_eight.setValue("limitedSizeForPresentation", new Integer(8));

      // ------------------- property "PrivilegeForTypeTypes" --------------------
      PropertyDescriptor pd_seventySix = new PropertyDescriptor(
        "PrivilegeForTypeTypes", ACSFactoryImpl.class,
        "getL_PrivilegeForTypeTypes", "setL_PrivilegeForTypeTypes");
      pd_seventySix.setDisplayName("PrivilegeForType types");
      pd_seventySix.setShortDescription("The allowed types for the PrivilegeForTypes,"+
              "not for PrivilegeForLinks");
      pd_seventySix.setBound(true);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor
      // If the display name is "", it is not displayed.
      pd_seventySix.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_seventySix.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventySix.setValue("tabbedPaneName", "  Types 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventySix.setValue("panelGroupName", "  Allowed Types for "+
              "privileges ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      pd_seventySix.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // False if it is not editable trough a constructor or another method.
      // If all the button booleans are false, the display name is not displayed.
      pd_seventySix.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_seventySix.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_seventySix.setValue("withDeleteButton", Boolean.TRUE );
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_seventySix.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible list.
      pd_seventySix.setValue("limitedSizeForPresentation", new Integer(5));
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the String values of the previous property for this current property,
      // following the order of the property descriptors returned by getPropertyDescriptors().
      pd_seventySix.setValue("withPreviousButton", Boolean.TRUE );

      // ------------------- property "PrivilegeForLinksTypes" --------------------
      PropertyDescriptor pd_seventySeven = new PropertyDescriptor(
        "PrivilegeForLinksTypes", ACSFactoryImpl.class,
        "getL_PrivilegeForLinksTypes", "setL_PrivilegeForLinksTypes");
      pd_seventySeven.setDisplayName("PrivilegeForLinks types");
      pd_seventySeven.setShortDescription("The allowed types for the PrivilegeForLinks,"+
              "not for PrivilegeForTypes");
      pd_seventySeven.setBound(true);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // If the display name is "", it is not displayed.
      pd_seventySeven.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_seventySeven.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventySeven.setValue("tabbedPaneName", "  Types 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventySeven.setValue("panelGroupName", "  Allowed Types for "+
              "privileges ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      pd_seventySeven.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // False if it is not editable trough a constructor or another method.
      pd_seventySeven.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_seventySeven.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_seventySeven.setValue("withDeleteButton", Boolean.TRUE );
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_seventySeven.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible list.
      pd_seventySeven.setValue("limitedSizeForPresentation", new Integer(5));
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the String values of the previous property for this current property,
      // following the order of the property descriptors returned by getPropertyDescriptors().
      pd_seventySeven.setValue("withPreviousButton", Boolean.TRUE );

      // ---------------------------------------------------------------------     
      // --------------------------------------------------------------------- 
      // The order of descriptors sets the display by the beamer,
      // and the capacity for the 'Previous' button to select
      // a collection in the previous property
      l_pd = new PropertyDescriptor[] {
        pd_one, 
        pd_fiftyFour, // Icon

        // inter-ACS structure
        pd_seventy, pd_fourtyFive,
        pd_fourtyThree, pd_fourty, 
        pd_fourtyTwo,  // AuthorizationServer
        pd_sixty, // manageAcsRights,
        pd_fourtyFour, // hub for children
        pd_eightyOne, pd_eightyTwo, // Incomplete Structure or Behavior Modeling

        // pd_twenty, // haveSpecializedRights 
        
        // ACS Structure
        pd_twelve, pd_thirteen,
        pd_fourteen, pd_fifteen, 
        pd_twentyEight, // AGO rights
        pd_nineteen, // Actors
        pd_seventeen, pd_eighteen, // ACLs
        pd_112, pd_113, // conditional ACLs
        pd_twentyFour, pd_thirtyOne, // Bridges
        pd_sixteen, pd_thirtyFour,
        pd_thirtySix, pd_thirtySeven, 
        pd_ninetySix, // VF trees
        pd_ninetyFive, pd_ninetyFour, pd_ninetySeven, // rights inheritance
        pd_109, pd_110, // alias

        // Structure 2
        pd_eightyFive, pd_eightySix, // granting/denying privileges
        pd_106, pd_107, // privileges for Resources and VirtualFolders
        pd_eightyThree, pd_eightyFour, // simple typed/linked privileges
        pd_100, pd_103, pd_101, // linked and typed privileges
        pd_104, pd_102, pd_105, // linked and typed privileges
        pd_eightySeven, pd_eightyEight, // virtual folders
        pd_eightyNine, // virtual folders
        pd_ninetyTwo, pd_ninetyOne, // virtual folders
        pd_ninetyThree, // virtual folders
        pd_108, pd_111, // virtual folders: OneTimeInVFTree, RightsPropagationToMembers

        // Structure 3
        pd_fourtyOne, pd_fourtySeven, // SubAcsPolicy, Alternates
        pd_seventyNine, // ACS vocabulary

        // Structure 4
        pd_fourtyEight, pd_fourtyNine, pd_fourtySix, // components, composites, belongs to composite

        // ACS Types
        pd_nine, // ACS type
        pd_two, pd_three, // Resources, EP
        pd_seventyThree, pd_seventyEight, // Resource and EPVirtualFolder types
        pd_seventySix, pd_seventySeven, // 2 Privilege types

        // Rights processing - this order allows 'Prev' button to work
        pd_five, pd_six, pd_seven, // A, GO, ACL rights
        pd_eight, pd_ten, // Privileges, Bridges
        pd_seventyOne, pd_fifty, // specialized rights, acs rights

        // Roots
        pd_fiftyOne, pd_fiftyTwo, pd_fiftyThree, // 3 roots in parent acs

        // Constraints
        pd_seventyTwo, pd_seventyFour,  // specialized types, associations on new object

        // See why
        pd_seventyFive

      };
      return l_pd;
    } catch (IntrospectionException e) {
      new ProcessError("In ACSFactoryImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In ACSFactoryImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

//  public int getDefaultPropertyIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultPropertyIndex() not yet implemented.");
//  }
//  public MethodDescriptor[] getMethodDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getMethodDescriptors() not yet implemented.");
//  }
//  public BeanInfo[] getAdditionalBeanInfo() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getAdditionalBeanInfo() not yet implemented.");
//  }
//  public Image getIcon(int iconKind) {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getIcon() not yet implemented.");
//  }


    /**
     * Get the instance, if it exists, which fires an event when
     * the bean is closed or deleted. No used here.
     * @return a BaseManager object.
     */
    public BaseBeanManager getBaseBeanManagerInstance() {
      return BaseManagerImpl.getInstance();
    }

    /**
     * Get the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, if it exists,
     * which fires an event when the bean instance is closed or deleted.
     * @return null
     */
    public Method getBaseBeanManagerGetMethod() {
      return null;
    }

    /**
     * Get the property name for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * @return "openACS"
     */
    public String getBaseBeanManagerBoundProperty() {
      return "";
    }


} // ACSFactoryImplBeanInfo