//Title:        Access Road
//Version:      0.7.1
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010, 2012 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.*;

import ARoad0.gBase.*;

import java.beans.SimpleBeanInfo;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.lang.reflect.Method;

import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;
import ARoad0.gDMak.ActionNewResource;
import ARoad0.gDMak.ActionNewEligibleParty;

/**
 * This class is responsible for describing the ACSImpl class.
 * It is used for displaying and editing the properties of this class.
 * @version 0.7.1
 */
public class ACSImplBeanInfo extends SimpleBeanInfo implements BeanInfoPattern {

  public ACSImplBeanInfo() {}

  /**
   * Gets the description of the bean.
   * Some standard values are set:
   * - 'hidden' is false since the bean is not hidden to the users
   * - 'expert' is false since the bean is not reserved to experts
   * <P>
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of ACSImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(ACSImpl.class );
    result.setDisplayName("ACS");

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of ACSImpl
    result.setValue("withTabbedPanes", Boolean.TRUE );

    // these strings signal the nodes to use in an ACS tree
    // to display this ACS, derived from the ACSTree.M_ACS_TREE_UPPER_NODES
    // in the Gui2 package.
    result.setValue("nodesInACSTree", new String[]
      {"RESOURCES", "RESVFOLDERS", "EPVFOLDERS", "USERS", "GROUPS", "ACLENTRIES",
       "PRIVILEGES", "EEPARTIES", "ECRESOURCES", "ACSRIGHTS"});

    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    result.setHidden(false);
    result.setExpert(false);

    return result;
  } // getBeanDescriptor()


  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue().
   * To read a property, use the read method in the property descriptor. <P>
   * To update a property, there are 3 means, in this order: <P>
   * - if it is not a single value, use the "choiceMethodForNew" value to add one value <P>
   * - or, if it is not a single value, use the "addMethod" value to add one value
   *  and "removeMethod", for BaseObjects only, <P>
   * - or, in any cases, use the write method in the property descriptor <P>
   * To set the position of the editor: <P>
   * - 'tabbedPaneName' is the name (String value) of the pane
   *    where this property has to be displayed in the beamer.
   *    Is "" if 'withTabbedPanes' is false in the BeanDescriptor. <P>
   * - 'panelGroupName' is the name (String value) of the group
   *    where this property has to be displayed with other related properties.
   *    May be "". 
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

/*
   * The order of the property descriptors is, where the first colomn is for
   * ACSFactoryImplBeanInfo descriptors ('Not there:' means that it is not an ACS
   * property in this BeanInfo, although it is an ACSFactory property):
   * <P>
   * Name,
   * ISname,
   * ParentName,
   *    Comment,
   *    NumberOfAcsObjects,
   * Controlled,
   *    ExternalOriented,
   * M_OwnResources,
   *    M_ExternalResources,
   *    M_OwnEligibleParties,
   *    M_ExternalEligibleParties,
   *    ExternalOrientedAclEntry, // supprimé en 0.7.1
   *    BridgeSources,
   *    BridgeTargets,
   * Not there: VisibleSpace,
   * Not there: HubProperty,
   * Not there: ResourceTypes,
   * Not there: EPTypes,
   * ACSType,
   *
   * Not there: GORightsSet,
   * Not there: AclRightsSet,
   * Not there: BridgeRightsSet,
   * HaveSpecializedRights,
   * SpecializedRightsSet,
   * ConstraintsForTypesAndGUIPolicy,
   * TypeOrientedPolicy,
   * ConstraintsForRightValues,
   * AssociationForNewObjectsMap,
   * Not there: OneAcsRight,
   * M_RightsOfInternalKey,
   *
   * ManageResources,
   * ManageResourceTrees,
   * ManageAccounts,
   * ManageGroups,
   * ManageGroupTrees,
   * ManageInternalAcls,
   * ManageExternalAcls,
   * ManageActors,
   * ManageBridges,
   * ManageSecondaryGroupsForActor,
   * deprecated: ManageNxDirectoryEPs,
   * deprecated: ManageNxExecutables,
   * ManageVirtualFolders,
   * ManageAGORight,
   * deprecated: ManageOtherRights,
   * ManagePrivilegeRights,
   * ManageGrantRights,
   * ManageDenyRights,
   * ManageAcsRights,
   * ManageMetaRights,
   *
   * // new in 070
   *
   * RightServer
   * ImmutableRules
   * OneNoRightProxyActor
   * TiedChildACSs
   * M_SubAcsNodePolicy
   * M_BelongsToComposite
   * L_RootsForOpening
   * L_RootsForClosing
   * L_RootsForIntegrity
   * M_RootsForAcsOpeningAndClosing
   * M_RootsForAcsOpeningNoClosing
   * M_RootsForAcsIntegrity
   * Icon
   * CommentsOnProperties
   * AcsVocabulary
   * Editor
   *
   * IncompleteStructureModeling,
   * IncompleteBehaviorModeling,
   * ManagePrivilegesInResources,
   * ManagePrivilegesInVirtualFolders,
   * ManageSimpleLinkedPrivileges,
   * ManageSimpleTypedPrivileges,
   * ManageIfAllSourceLinkedPrivileges,
   * ManageInternalForOneToOneLinkedPrivileges,
   * ManageExternalForOneToOneLinkedPrivileges,
   * ManageClassedTypedPrivileges,
   * ManageClassedSpecificTypedPrivileges,
   * ManageAuthorizationServerTypedPrivileges,
   * ManageGrantingRightPrivileges,
   * ManageDenyingRightPrivileges,
   * ManageResourceVirtualFolders,
   * ManageEPVirtualFolders,
   * ManageSimpleResourceVirtualFolders,
   * ManageDirectoryInResourceVirtualFolders,
   * ManageDirectoryEPinEPVirtualFolders,
   * ManageActorInEPVirtualFolders,
   * ManageACLRightInheritance,
   * ManageAGORightInheritance,
   * ManageLPRIRightInheritance,
   * ManageVirtualFolderTrees,
   * ManageOneTimeInVirtualFolderTreeForMember,
   * ManageVirtualFolderRightsPropagationToMembers,
   * ManageExternalSoftAlias,
   * ManageInternalHardAlias,
   * ManageFlexibleConditionalAcls,
   * ManageRightDefinedConditionalAcls,
   *
   * OwnResourceFoldersRootName,
   * OwnEPFoldersRootName,
   * OneResourceVirtualFolder,
   * OneEPVirtualFolder,
   * OneTypedPrivilege,
   * OneLinkedPrivilege,
   * Not there: ResVirtualFolderTypes,
   * Not there: EPVirtualFolderTypes,
   * Not there: PrivilegeForTypeTypes,
   * Not there: PrivilegeForLinksTypes,
   * Not there: PrivilegeRightsSet,
   *
   * // new in 0.7.1
   * 
   * OneController
   *
 */

    PropertyDescriptor[] l_pd;

    try {
      // ------------------- property "Name" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "Name", ACSImpl.class, "getName", null);
      pd_one.setDisplayName("ACS name");
      //pd_one.setShortDescription("   ");
      //pd_one.setBound(false);
      //pd_one.setConstrained(false);
      //pd_one.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_one.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_one.setValue("BaseBeanManager", null);

      // ------------------- property "ISname" ------------------------
      PropertyDescriptor pd_seventy = new PropertyDescriptor(
        "ISname", ACSImpl.class, "getISName", null);
      pd_seventy.setDisplayName("IS name");
      //pd_seventy.setShortDescription("   ");
      //pd_seventy.setBound(false);
      //pd_seventy.setConstrained(false);
      //pd_seventy.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventy.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_seventy.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_seventy.setValue("BaseBeanManager", null);      

      // ------------------- property "ParentName" ------------------------
      PropertyDescriptor pd_fourtyFive = new PropertyDescriptor(
        "ParentName", ACSImpl.class, "getParentName", null);
      pd_fourtyFive.setDisplayName("ACS parent");
      //pd_fourtyFive.setShortDescription("   ");
      //pd_fourtyFive.setBound(false);
      //pd_fourtyFive.setConstrained(false);
      //pd_fourtyFive.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyFive.setValue("tabbedPaneName", "  Context  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_fourtyFive.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_fourtyFive.setValue("BaseBeanManager", null); 
      
      // ------------------- property "Comment" --------------------
      PropertyDescriptor pd_thirtyTwo = new PropertyDescriptor(
        "Comment", ACSImpl.class, "getComment", "setComment");
      pd_thirtyTwo.setDisplayName("Comment");
      pd_thirtyTwo.setShortDescription("free user comments");
      pd_thirtyTwo.setBound(true);
      //pd_thirtyTwo.setConstrained(false);
      //pd_thirtyTwo.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtyTwo.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_thirtyTwo.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the ResourceImpl instance
      // does it, or when is is not applicable.
      //pd_thirtyTwo.setValue("BaseBeanManager", ); getNumberOfAcsObjects()
 
      // ------------------- property "NumberOfResources" --------------------
      PropertyDescriptor pd_thirtySeven = new PropertyDescriptor(
        "NumberOfResources", ACSImpl.class, "getNumberOfResources", null);
      pd_thirtySeven.setDisplayName("Number of resources");
      pd_thirtySeven.setShortDescription("Caution: No updating of this value "+
              "until the next display in the beamer.");
      pd_thirtySeven.setBound(false);
      //pd_thirtySeven.setConstrained(false);
      //pd_thirtySeven.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtySeven.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_thirtySeven.setValue("panelGroupName", "  Capacity  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the ResourceImpl instance
      // does it, or when is is not applicable.
      //pd_thirtySeven.setValue("BaseBeanManager", ); 
 
      // ------------------- property "NumberOfEPs" --------------------
      PropertyDescriptor pd_seventyThree = new PropertyDescriptor(
        "NumberOfEPs", ACSImpl.class, "getNumberOfEPs", null);
      pd_seventyThree.setDisplayName("Number of right users");
      pd_seventyThree.setShortDescription("Caution: No updating of this value "+
              "until the next display in the beamer.");
      pd_seventyThree.setBound(false);
      //pd_seventyThree.setConstrained(false);
      //pd_seventyThree.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventyThree.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_seventyThree.setValue("panelGroupName", "  Capacity  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the ResourceImpl instance
      // does it, or when is is not applicable.
      //pd_seventyThree.setValue("BaseBeanManager", );
      
      // ------------------- property "Controlled" --------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "Controlled", ACSImpl.class, "getIsControlled", null);
      pd_two.setDisplayName("Other ACS control(s) it");
      pd_two.setShortDescription("true if some access targets of this ACS "+
        "are controlled by another ACS through AclEntries or Linked Privileges.");
      pd_two.setBound(true);
      //pd_two.setConstrained(false);
      //pd_two.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  Context  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_two.setValue("panelGroupName", "    ACL-based or PRI-based controls between ACS " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_two.setValue("BaseBeanManager", null);

      // ------------------ property "ExternalOriented" ---------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "ExternalOriented", ACSImpl.class, "getIsExternalOriented", null);
      pd_three.setDisplayName("Controls other ACS");
      pd_three.setShortDescription("true if this ACS has external AclEntries or "+
              "Linked Privileges which control a source or a target of another ACS");
      pd_three.setBound(true);
      //pd_three.setConstrained(false);
      //pd_three.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  Context  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_three.setValue("panelGroupName", "    ACL-based or PRI-based controls between ACS " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_three.setValue("BaseBeanManager", null);


      // ------------------- property "getL_AcsControllers" --------------------
      PropertyDescriptor pd_seven = new PropertyDescriptor(
        "OneController", ACSImpl.class, "getL_AcsControllers", null);

      pd_seven.setDisplayName("They control this ACS");
      pd_seven.setShortDescription("These ACS have external AclEntries or Linked Privileges"+
              " having target(s) owned by this ACS.");
      pd_seven.setBound(true);
      //pd_seven.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor or a StringMapPropertyEditor.
      // If the display name is "", it is not displayed.
      pd_seven.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle.
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_seven.setValue("valueTypeInGroup", new Object[] {ImmutableName.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seven.setValue("tabbedPaneName", "  Context  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seven.setValue("panelGroupName", "    ACL-based or PRI-based controls between ACS " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_seven.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the CollectionPropertyEditor
      // has to display a 'View' button if the property is viewable in the beamer.
      pd_seven.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_seven.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Select' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_seven.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_seven.setValue("withDeleteButton", Boolean.FALSE );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_seven.setValue("limitedSizeForPresentation", new Integer(5));

      // ------------------- property "M_OwnResources" --------------------
      PropertyDescriptor pd_five = new PropertyDescriptor(
        "M_OwnResources", ACSImpl.class, "getEorM_OwnResources", null);
      
      pd_five.setDisplayName("Own Resources");
      pd_five.setShortDescription("The resources which are owned by the ACS.");
      pd_five.setBound(true);
      //pd_five.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor or a StringMapPropertyEditor.
      // If the display name is "", it is not displayed.
      pd_five.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle.
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_five.setValue("valueTypeInGroup", new Object[] {ImmutableResource.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_five.setValue("tabbedPaneName", "  Access Targets  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_five.setValue("panelGroupName", "  Own and external ones  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_five.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable in the beamer.
      pd_five.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_five.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Select' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_five.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_five.setValue("withDeleteButton", Boolean.TRUE );

      // 'choiceMethodForNew' defines a specific instance/method to invoke 
      // to let the user defines the new restricted key, in a map, or the new instance, 
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if 
      // there is an add method or a write method. With 'isRestrictedArgument', 
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      // The array is equal to 'new Object[1]' when the value 'withKnownArgumentsForNew' is null.
      pd_five.setValue("choiceMethodForNew", new Object[] {
        ActionNewResource.getInstance(), "createResource"} );

      // 'withKnownArgumentsForNew' allows to define the constructor/add/copy arguments
      // which has to be preset - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the fixed argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Third case for the fixed argument: the string 'thisname' is used
      // to point to the value returned from getName() by the object in the beamer.
      // Fourth case, the string 'true' or 'false' is used
      // to define a boolean value.
      // Fifth case, if none of the previous cases matches: takes directly
      // the fixed argument if it is a String.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property.
      // Caution: the known arguments have to be Objects.
      // Caution: the number of arguments to use may be superior.
      pd_five.setValue("withKnownArgumentsForNew", new Object[] {null, "this"} );

      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser", at the index 0.
      // Else, the returned array defines the name of the remove method, at the index 0.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_five.setValue("removeMethod", "finalizeForUser");

      // ------------------- property "OneExternalResource" --------------------
      PropertyDescriptor pd_six = new PropertyDescriptor(
        "OneExternalResource", ACSImpl.class, "getEorM_ExternalResources", null);

      pd_six.setDisplayName("External Resources");
      pd_six.setShortDescription("The external resources which are controlled by this ACS "+
              "external AclEntries or Linked Privileges.");
      pd_six.setBound(true);
      //pd_six.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor or a StringMapPropertyEditor.
      // If the display name is "", it is not displayed.
      pd_six.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_six.setValue("valueTypeInGroup", new Object[] {ImmutableResource.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_six.setValue("tabbedPaneName", "  Access Targets  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_six.setValue("panelGroupName", "  External resources into "+
              "an own external ACL or LPRI  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_six.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable in the beamer.
      pd_six.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_six.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_six.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_six.setValue("withDeleteButton", Boolean.FALSE );

      // ------------------- property "M_OwnEligibleParties" --------------------
      PropertyDescriptor pd_nine = new PropertyDescriptor(
        "M_OwnEligibleParties", ACSImpl.class, "getEorM_OwnEligibleParties", null);

      pd_nine.setDisplayName("Own Eligible Parties");
      pd_nine.setShortDescription("The eligible parties which are owned by the ACS.");
      pd_nine.setBound(true);
      //pd_nine.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_nine.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_nine.setValue("valueTypeInGroup", new Object[] {ImmutableEligibleParty.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_nine.setValue("tabbedPaneName", "  Right Users  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_nine.setValue("panelGroupName", "  Own and external ones  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_nine.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable in the beamer.
      pd_nine.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_nine.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_nine.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_nine.setValue("withDeleteButton", Boolean.TRUE );

      // 'choiceMethodForNew' defines a specific instance/method to invoke 
      // to let the user defines the new restricted key, in a map, or the new instance, 
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if 
      // there is an add method or a write method. With 'isRestrictedArgument', 
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      // The array is equal to 'new Object[1]' when the value 'withKnownArgumentsForNew' is null.
      pd_nine.setValue("choiceMethodForNew", new Object[] {
        ActionNewEligibleParty.getInstance(), "createEligibleParty"} );

      // 'withKnownArgumentsForNew' allows to define the constructor/add/copy arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the knwon argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_knownValue from this object.
      // Third case for the known argument: the string 'thisname' is used
      // to point to the value returned from getName() by the object in the beamer.
      // Fourth case, the string 'true' or 'false' is used
      // to define a boolean value.
      // Fifth case, if none of the previous cases matches: takes directly
      // the fixed argument if it is a String.
      // Used by SimplePropertyEditor.getKnownValuesForNewElement(), which
      // may use a subset of the returned arguments if the class of the object to create
      // waits a smaller set of arguments.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property.
      // Caution: the known arguments have to be Objects.
      pd_nine.setValue("withKnownArgumentsForNew", new Object[] {null, "this", null,
        null, null, null, null} );

      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser", at the index 0.
      // Else, the returned array defines the name of the remove method, at the index 0.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_nine.setValue("removeMethod", "finalizeForUser");

      // ------------------- property "OneExternalEligibleParty" --------------------
      PropertyDescriptor pd_eight = new PropertyDescriptor(
        "OneExternalEligibleParty", ACSImpl.class, "getEorM_ExternalEligibleParties", null);

      pd_eight.setDisplayName("External EParties");
      pd_eight.setShortDescription("The external eligible parties which are controlled "+
              "by this ACS external AclEntries or Linked Privileges.");
      pd_eight.setBound(true);
      //pd_eight.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_eight.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_eight.setValue("valueTypeInGroup", new Object[] {ImmutableEligibleParty.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eight.setValue("tabbedPaneName", "  Right Users  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eight.setValue("panelGroupName", 
              "  External Eligible Parties into an own external ACL or LPRI  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_eight.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable in the beamer.
      pd_eight.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_eight.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_eight.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_eight.setValue("withDeleteButton", Boolean.FALSE );

      // ------------------- property "ExternalOrientedAclEntry" -------------------- supprimée
      //PropertyDescriptor pd_seven = new PropertyDescriptor(
        //"ExternalOrientedAclEntry", ACSImpl.class, "getEorL_ExternalAclEntries", null);

      // ------------------- property "BridgeSources" --------------------
      PropertyDescriptor pd_ten = new PropertyDescriptor(
        "BridgeSources", ACSImpl.class, "getEorL_OwnActorBridgeSources", null);

      pd_ten.setDisplayName("Own bridge sources");
      pd_ten.setShortDescription("The own actors which are sources for an ACS bridge.");
      pd_ten.setBound(true);
      //pd_ten.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_ten.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_ten.setValue("valueTypeInGroup", new Object[] {ImmutableActor.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ten.setValue("tabbedPaneName", "  Context  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ten.setValue("panelGroupName", "   ACS Bridges  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_ten.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable in the beamer.
      pd_ten.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "BridgeTargets" --------------------
      PropertyDescriptor pd_eleven = new PropertyDescriptor(
        "BridgeTargets", ACSImpl.class, "getEorL_OwnActorBridgeTargets", null);

      pd_eleven.setDisplayName("Own bridge targets");
      pd_eleven.setShortDescription("The own actors which are targets for an ACS bridge.");
      pd_eleven.setBound(true);
      //pd_eleven.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_eleven.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_eleven.setValue("valueTypeInGroup", new Object[] {ImmutableActor.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eleven.setValue("tabbedPaneName", "  Context  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eleven.setValue("panelGroupName", "   ACS Bridges  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_eleven.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable in the beamer.
      pd_eleven.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "ACSType" ------------------------
      PropertyDescriptor pd_twentySix = new PropertyDescriptor(
        "ACSType", ACSImpl.class, "getACSType", null);
      pd_twentySix.setDisplayName("Type | policy checksum");
      pd_twentySix.setShortDescription("the ACS type and the immutable ACS policy checksum"+
              " (depends on the ACS structure, the subACS, the types policy, the rights policy).");
      //pd_twentySix.setBound(false);
      //pd_twentySix.setConstrained(false);
      //pd_twentySix.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twentySix.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_twentySix.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_twentySix.setValue("BaseBeanManager", null);
      
      // ------------------ property "HaveSpecializedRights" --------------- 
      PropertyDescriptor pd_twenty = new PropertyDescriptor(
        "HaveSpecializedRights", ACSImpl.class, "getHaveSpecializedRights", null);
      pd_twenty.setDisplayName("Has specialized rights");
      pd_twenty.setShortDescription("true if has source-oriented or target-oriented rights,"+
              " beyond the standard ones (AGO, ACL,...).");
      pd_twenty.setBound(false);
      //pd_twenty.setConstrained(false);
      //pd_twenty.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twenty.setValue("tabbedPaneName", "  Rights 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_twenty.setValue("panelGroupName", "  Structure  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_twenty.setValue("BaseBeanManager", null); 

      // ------------------- property "SpecializedRightsSet" --------------------
      PropertyDescriptor pd_seventyOne = new PropertyDescriptor(
        "SpecializedRightsSet", ACSImpl.class, "getM_SpecializedRights",
        null);
      pd_seventyOne.setDisplayName("Specialized rights");
      pd_seventyOne.setShortDescription("no use");
      pd_seventyOne.setBound(false);
        //pd_seventyOne.setConstrained(false);
      // 'isRightMap' is TRUE when the right property editor is RightMapPropertyEditor
      pd_seventyOne.setValue("isRightMap", Boolean.TRUE );      
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle. 
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_seventyOne.setValue("valueTypeInGroup", new Object[] {StringRight.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventyOne.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventyOne.setValue("panelGroupName", 
              "    B. Specialized Rights - They override the standard rights  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when is is not applicable.
        //pd_seventyOne.setValue("BaseBeanManager", );
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_seventyOne.setValue("KeysUpdateInMap", Boolean.FALSE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_seventyOne.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. 
      pd_seventyOne.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'Add New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_seventyOne.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the RightMapPropertyEditor
      // has to display 2 'Select' buttons if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_seventyOne.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the RightMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_seventyOne.setValue("withRemoveButton", Boolean.FALSE );
      pd_seventyOne.setValue("withDeleteButton", Boolean.FALSE );    
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_seventyOne.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.      
      pd_seventyOne.setValue("limitedSizeForPresentation", new Integer(10));

      // ------------------- property "TypesAndGUIPolicy" ------------------------
      PropertyDescriptor pd_twentySeven = new PropertyDescriptor(
        "TypesAndGUIPolicy", ACSImpl.class, "getM_TypesAndGUIPolicy", null);
      pd_twentySeven.setDisplayName("All the ACS object types");
      pd_twentySeven.setShortDescription("Allowed type values, and GUI display/creation policy.");
      //pd_twentySeven.setBound(false);
      //pd_twentySeven.setConstrained(false);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_twentySeven.setValue("isStringMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle. 
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_twentySeven.setValue("valueTypeInGroup", new Object[] {String.class} );      
      //pd_twentySeven.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twentySeven.setValue("tabbedPaneName", "  Types  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_twentySeven.setValue("panelGroupName",
              "  A. All the ACS object TYPES, and the 'CreationByBeamer' type POLICY  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_twentySeven.setValue("BaseBeanManager", null);
 
      // ------------------- property "TypeOrientedPolicy" ------------------------
      PropertyDescriptor pd_twentyNine = new PropertyDescriptor(
        "TypeOrientedPolicy", ACSImpl.class, "getM_TypeOrientedPolicy", null);
      pd_twentyNine.setDisplayName(
              "Container, resource, ACL and privilege type-oriented policies");
      pd_twentyNine.setShortDescription(
              "Varied constraints on properties from the types.");
      //pd_twentyNine.setBound(false);
      //pd_twentyNine.setConstrained(false);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_twentyNine.setValue("isStringMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle. 
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_twentyNine.setValue("valueTypeInGroup", new Object[] {String.class} );      
      //pd_twentyNine.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twentyNine.setValue("tabbedPaneName", "  Types  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_twentyNine.setValue("panelGroupName", 
              "  B. Container / Resource / ACL / Privilege type POLICIES - for advanced users  " );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.      
      pd_twentyNine.setValue("limitedSizeForPresentation", new Integer(8));
      // 'MaxKeySize' is the maximal size of the key lists. Analyzed by StringMapPropertyEditor.
      // The default value is 50.
      pd_twentyNine.setValue("MaxKeySize", new Integer(100));
      
      // ------------------- property "ConstraintsForRightValues" ------------------------
      PropertyDescriptor pd_fiftyOne = new PropertyDescriptor(
        "ConstraintsForRightValues", ACSImpl.class, "getM_StandardRights", null);
      pd_fiftyOne.setDisplayName("Standard Rights");
      pd_fiftyOne.setShortDescription("no use");
      pd_fiftyOne.setBound(false);
      //pd_fiftyOne.setConstrained(false);
      // 'isRightMap' is TRUE when the right property editor is RightMapPropertyEditor
      pd_fiftyOne.setValue("isRightMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle. 
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_fiftyOne.setValue("valueTypeInGroup", new Object[] {StringRight.class} );      
      //pd_fiftyOne.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftyOne.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fiftyOne.setValue("panelGroupName", 
              "    A. Standard Rights  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_fiftyOne.setValue("BaseBeanManager", null);
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_fiftyOne.setValue("KeysUpdateInMap", Boolean.FALSE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_fiftyOne.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_fiftyOne.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'Add New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_fiftyOne.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the RightMapPropertyEditor
      // has to display 2 'Select' buttons if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_fiftyOne.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the RightMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_fiftyOne.setValue("withRemoveButton", Boolean.FALSE );
      pd_fiftyOne.setValue("withDeleteButton", Boolean.FALSE );    
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_fiftyOne.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.      
      pd_fiftyOne.setValue("limitedSizeForPresentation", new Integer(10));
      
      // ------------------- property "M_RightsOfInternalKey" --------------------
      PropertyDescriptor pd_fifty = new PropertyDescriptor(
        "M_RightsOfInternalKey", ACSImpl.class, "getM_RightsOfInternalKey", null);
      pd_fifty.setDisplayName("All the rights");
      pd_fifty.setShortDescription("ACS rights and metarights allowed by the ACS");
      pd_fifty.setBound(false);
        //pd_fifty.setConstrained(false);
      // 'isBaseObjectMap' is TRUE when the right property editor is BaseObjectMapPropertyEditor
      // If the display name is "", it is not displayed.
      pd_fifty.setValue("isBaseObjectMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, the value is the type of the values in the set.
      pd_fifty.setValue("valueTypeInGroup", new Object[] {StringRight.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fifty.setValue("tabbedPaneName", "  Rights 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fifty.setValue("panelGroupName", 
              "    All Rights, by Strength of the generic right (whatever the rights network)  ");
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      // If all the button booleans are false, the display name is not displayed.
      pd_fifty.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_fifty.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'Add New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_fifty.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the RightMapPropertyEditor
      // has to display 2 'Select' buttons if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_fifty.setValue("withSelectButton", Boolean.FALSE );     
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_fifty.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.      
      pd_fifty.setValue("limitedSizeForPresentation", new Integer(16));
      
      // ------------------ property "ManageResources" ---------------
      PropertyDescriptor pd_twelve = new PropertyDescriptor(
        "ManageResources", ACSImpl.class, "getManageResources", null);
      pd_twelve.setDisplayName("Resources");
      pd_twelve.setShortDescription("Resources, in trees or not, are the main access targets.");
      pd_twelve.setBound(false);
      //pd_twelve.setConstrained(false);
      //pd_twelve.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twelve.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_twelve.setValue("panelGroupName", 
              "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_twelve.setValue("BaseBeanManager", null);

      // ------------------ property "ManageResourceTrees" ---------------
      PropertyDescriptor pd_thirteen = new PropertyDescriptor(
        "ManageResourceTrees", ACSImpl.class, "getManageResourceTrees", null);
      pd_thirteen.setDisplayName("Resources Tree");
      pd_thirteen.setShortDescription("True if the ACS manages an inner tree of resources.");
      pd_thirteen.setBound(false);
      //pd_thirteen.setConstrained(false);
      //pd_thirteen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirteen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_thirteen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_thirteen.setValue("BaseBeanManager", null);

      // ------------------ property "ManageAccounts" ---------------
      PropertyDescriptor pd_fourteen = new PropertyDescriptor(
        "ManageAccounts", ACSImpl.class, "getManageAccounts", null);
      pd_fourteen.setDisplayName("Accounts - Users ('A')");
      pd_fourteen.setShortDescription("Accounts are right users, owners of resources"+
              " and running contexts of processes; they may be personal user accounts.");
      pd_fourteen.setBound(false);
      //pd_fourteen.setConstrained(false);
      //pdfourteen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourteen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourteen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_fourteen.setValue("BaseBeanManager", null);
 
      // ------------------ property "ManageGroups" ---------------
      PropertyDescriptor pd_fifteen = new PropertyDescriptor(
        "ManageGroups", ACSImpl.class, "getManageGroups", null);
      pd_fifteen.setDisplayName("Account Groups ('G')");
      pd_fifteen.setShortDescription("Groups are sets of accounts, "+
              "in group trees or not; they are right users.");
      pd_fifteen.setBound(false);
      //pd_fifteen.setConstrained(false);
      //pd_fifteen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fifteen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fifteen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_fifteen.setValue("BaseBeanManager", null);
 
      // ------------------ property "ManageGroupTrees" ---------------
      PropertyDescriptor pd_sixteen = new PropertyDescriptor(
        "ManageGroupTrees", ACSImpl.class, "getManageGroupTrees", null);
      pd_sixteen.setDisplayName("Nested Account Groups");
      pd_sixteen.setShortDescription("To manage nested account groups, "+
              "where a group may be member of other groups.");
      pd_sixteen.setBound(false);
      //pd_sixteen.setConstrained(false);
      //pd_sixteen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixteen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_sixteen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_sixteen.setValue("BaseBeanManager", null);
 
      // ------------------ property "ManageInternalAcls" ---------------
      PropertyDescriptor pd_seventeen = new PropertyDescriptor(
        "ManageInternalAcls", ACSImpl.class, "getManageInternalAcls", null);
      pd_seventeen.setDisplayName("Internal AclEntries ('ACL')");
      pd_seventeen.setShortDescription("An internal Access Control List defines dynamic rights"+
              " with a rights user and an access target (a resource) from this ACS.");
      pd_seventeen.setBound(false);
      //pd_seventeen.setConstrained(false);
      //pd_seventeen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventeen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventeen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_seventeen.setValue("BaseBeanManager", null);    
      
      // ------------------ property "ManageExternalAcls" ---------------
      PropertyDescriptor pd_eighteen = new PropertyDescriptor(
        "ManageExternalAcls", ACSImpl.class, "getManageExternalAcls", null);
      pd_eighteen.setDisplayName("External AclEntries ('ACL')");
      pd_eighteen.setShortDescription("An external Access Control List defines dynamic rights"+
              " with a rights user and/or an access target (a resource) from another ACS.");
      pd_eighteen.setBound(false);
      //pd_eighteen.setConstrained(false);
      //pd_eighteen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eighteen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eighteen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_eighteen.setValue("BaseBeanManager", null);
  
      // ------------------ property "ManageActors" ---------------
      PropertyDescriptor pd_nineteen = new PropertyDescriptor(
        "ManageActors", ACSImpl.class, "getManageActors", null);
      pd_nineteen.setDisplayName("Actors");
      pd_nineteen.setShortDescription("Actors are both resources and right users; "+
              "they are active objects running generally under an Account/Group context");
      pd_nineteen.setBound(false);
      //pd_nineteen.setConstrained(false);
      //pd_nineteen.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_nineteen.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_nineteen.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_nineteen.setValue("BaseBeanManager", null);
      
      // ------------------ property "ManageBridges" ---------------
      PropertyDescriptor pd_twentyFour = new PropertyDescriptor(
        "ManageBridges", ACSImpl.class, "getManageBridges", null);
      pd_twentyFour.setDisplayName("Bridges ('BDG')");
      pd_twentyFour.setShortDescription("Bridges are links "+
              "between 2 actors of the 2 different ACS.");
      pd_twentyFour.setBound(false);
      //pd_twentyFour.setConstrained(false);
      //pd_twentyFour.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twentyFour.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_twentyFour.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_twentyFour.setValue("BaseBeanManager", null);

      // ------------------ property "ManageSecGroupsForActor" ---------------
      PropertyDescriptor pd_thirtyOne = new PropertyDescriptor(
        "ManageSecGroupsForActor", ACSImpl.class, "getManageSecGroupsForActor", null);
      pd_thirtyOne.setDisplayName("Actor Secondary Groups");
      pd_thirtyOne.setShortDescription("True if an actor may be member of secondary groups, "+
              "as right user.");
      pd_thirtyOne.setBound(false);
      //pd_thirtyOne.setConstrained(false);
      //pd_thirtyOne.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtyOne.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_thirtyOne.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_thirtyOne.setValue("BaseBeanManager", null);
       
      // ------------------ property "ManageDirectoryEPs" ---------------
      PropertyDescriptor pd_fiftyFour = new PropertyDescriptor(
        "ManageDirectoryEPs", ACSImpl.class, "getManageDirectoryEPs", null);
      pd_fiftyFour.setDisplayName("Right User Directories");
      pd_fiftyFour.setShortDescription("True if some directories be right users "+
              "for being a privilege source");
      pd_fiftyFour.setBound(false);
      //pd_fiftyFour.setConstrained(false);
      //pd_fiftyFour.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftyFour.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fiftyFour.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_fiftyFour.setValue("BaseBeanManager", null);
       
      // ------------------ property "ManageVirtualFolders" ---------------
      PropertyDescriptor pd_fiftyFive = new PropertyDescriptor(
        "ManageDirectoryEPs", ACSImpl.class, "getManageVirtualFolders", null);
      pd_fiftyFive.setDisplayName("Virtual Folders ('VF')");
      pd_fiftyFive.setShortDescription("Virtual folders are dynamic containers of resources; "+
              "they are privilege targets");
      pd_fiftyFive.setBound(false);
      //pd_fiftyFive.setConstrained(false);
      //pd_fiftyFive.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftyFive.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fiftyFive.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_fiftyFive.setValue("BaseBeanManager", null);
      
      // ------------------ property "ManageAGORights" ---------------
      PropertyDescriptor pd_twentyEight = new PropertyDescriptor(
        "ManageAGORights", ACSImpl.class, "getManageAGORights", null);
      pd_twentyEight.setDisplayName("Account and Group Rights");
      pd_twentyEight.setShortDescription("True if this ACS manages "+
              "Account Rights and/or the Group Rights (called AG rights) of a Resource");
      pd_twentyEight.setBound(false);
      //pd_twentyEight.setConstrained(false);
      //pd_twentyEight.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twentyEight.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_twentyEight.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_twentyEight.setValue("BaseBeanManager", null);
      
      // ------------------ property "ManagePrivilegeRights" ---------------
      PropertyDescriptor pd_thirtySix = new PropertyDescriptor(
        "ManagePrivilegeRights", ACSImpl.class, "getManagePrivilegeRights",
        null);
      pd_thirtySix.setDisplayName("Privilege Rights ('PRI')");
      pd_thirtySix.setShortDescription("A privilege defines some static rights "+
              "from a set of user rights to a set of resources or virtual folders.");
      pd_thirtySix.setBound(false);
      //pd_thirtySix.setConstrained(false);
      //pd_thirtySix.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtySix.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_thirtySix.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_thirtySix.setValue("BaseBeanManager", null);  
      
      // ------------------ property "ManageGrantRights" --------------- 
      PropertyDescriptor pd_thirtyEight = new PropertyDescriptor(
        "ManageGrantRights", ACSImpl.class, "getManageGrantRights",
        null);
      pd_thirtyEight.setDisplayName("Granting Rights");
      pd_thirtyEight.setShortDescription("true if the granting rights are managed.");
      pd_thirtyEight.setBound(false);
      //pd_thirtyEight.setConstrained(false);
      //pd_thirtyEight.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtyEight.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_thirtyEight.setValue("panelGroupName", "    C. Rights State  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_thirtyEight.setValue("BaseBeanManager", null);
     
      // ------------------ property "ManageDenyRights" --------------- 
      PropertyDescriptor pd_thirtyNine = new PropertyDescriptor(
        "ManageDenyRights", ACSImpl.class, "getManageDenyRights",
        null);
      pd_thirtyNine.setDisplayName("Denying Rights");
      pd_thirtyNine.setShortDescription("true if the denying rights are managed.");
      pd_thirtyNine.setBound(false);
      //pd_thirtyNine.setConstrained(false);
      //pd_thirtyNine.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtyNine.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_thirtyNine.setValue("panelGroupName", "    C. Rights State  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_thirtyNine.setValue("BaseBeanManager", null);       
      
      // ------------------ property "ManageAcsRights" --------------- 
      PropertyDescriptor pd_fiftyTwo = new PropertyDescriptor(
        "ManageGrantRights", ACSImpl.class, "getManageAcsRights",
        null);
      pd_fiftyTwo.setDisplayName("Have ACS Rights");
      pd_fiftyTwo.setShortDescription("true if there are rights that are proper to this ACS");
      pd_fiftyTwo.setBound(false);
      //pd_fiftyTwo.setConstrained(false);
      //pd_fiftyTwo.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftyTwo.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fiftyTwo.setValue("panelGroupName", "    C. Rights State  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_fiftyTwo.setValue("BaseBeanManager", null);
     
      // ------------------ property "ManageMetaRights" --------------- 
      PropertyDescriptor pd_fiftyThree = new PropertyDescriptor(
        "ManageDenyRights", ACSImpl.class, "getManageMetaRights",
        null);
      pd_fiftyThree.setDisplayName("Have Generic Rights");
      pd_fiftyThree.setShortDescription("true if some metarights are used.");
      pd_fiftyThree.setBound(false);
      //pd_fiftyThree.setConstrained(false);
      //pd_fiftyThree.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftyThree.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fiftyThree.setValue("panelGroupName", "    C. Rights State  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_fiftyThree.setValue("BaseBeanManager", null);   
     
      // ------------------ property "RightServer" --------------- 
      PropertyDescriptor pd_fourtyTwo = new PropertyDescriptor(
        "RightServer", ACSImpl.class, "getIsAuthorizationServer",
        null);
      pd_fourtyTwo.setDisplayName("Is an Authorization Server");
      pd_fourtyTwo.setShortDescription("True if this ACS manages some rights other ACS "+
              "have to applied");
      pd_fourtyTwo.setBound(false);
      //pd_fourtyTwo.setConstrained(false);
      //pd_fourtyTwo.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyTwo.setValue("tabbedPaneName", "  Rights 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_fourtyTwo.setValue("panelGroupName", "  Capacity  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_fourtyTwo.setValue("BaseBeanManager", null);  
     
      // ------------------ property "ImmutableRules" --------------- 
      PropertyDescriptor pd_fourtyThree = new PropertyDescriptor(
        "ImmutableRules", ACSImpl.class, "getL_ImmutableRules", null);
       // If the display name is "", the property name is not displayed.
      pd_fourtyThree.setDisplayName("");
      pd_fourtyThree.setShortDescription(
              "These immutable rules drive the simulation of the structure, the behavior "+
              "and the rights for this ACS.");
      pd_fourtyThree.setBound(false);
      //pd_ten.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_fourtyThree.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_fourtyThree.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyThree.setValue("tabbedPaneName", "  Rules  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtyThree.setValue("panelGroupName", "  Rules for the ACS structure and behavior -"+
              "for advanced users (* for generic rules)  ");
      // 'withViewButton' is the boolean which is set if the CollectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer.
      pd_fourtyThree.setValue("withViewButton", Boolean.FALSE );
      // 'horizontalTextPositionToLeft' is the boolean which is set if 
      // the collectionPropertyEditor has to display the text from the left
      // in the beamer. The Right position is the default value. 
      pd_fourtyThree.setValue("horizontalTextPositionToLeft", Boolean.TRUE );
       // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.      
      pd_fourtyThree.setValue("limitedSizeForPresentation", new Integer(32));
   
      // ------------------ property "OneNoRightProxyActor" --------------- 
      PropertyDescriptor pd_fourtySix = new PropertyDescriptor(
        "OneNoRightProxyActor", ACSImpl.class, "getEorL_NoRightProxyActors", null);
      pd_fourtySix.setDisplayName("No right-proxy actors");
      pd_fourtySix.setShortDescription("These programs are strictly coded and "+
              "do not offer access to other targets for those which execute them.");
      pd_fourtySix.setBound(true);
      //pd_ten.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_fourtySix.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_fourtySix.setValue("valueTypeInGroup", new Object[] {ImmutableActor.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtySix.setValue("tabbedPaneName", "  Proxies  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtySix.setValue("panelGroupName", "  Execution of these programs"+
              " does not provide any access to their launcher ");
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer.
      pd_fourtySix.setValue("withViewButton", Boolean.TRUE );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_fourtySix.setValue("limitedSizeForPresentation", new Integer(16));
      // 'horizontalTextPositionToLeft' is the boolean which is set if 
      // the collectionPropertyEditor has to display the text from the left
      // in the beamer. The Right position is the default value. 
      //pd_fourtySix.setValue("horizontalTextPositionToLeft", Boolean.TRUE );

      // ------------------ property "TiedChildACSs" ---------------
      PropertyDescriptor pd_fourtySeven = new PropertyDescriptor(
        "TiedChildACSs", ACSImpl.class, "getL_TiedChildACS", null);
      pd_fourtySeven.setDisplayName("Tied Child ACSs");
      pd_fourtySeven.setShortDescription(
              "A tied child ACS has to be opened when this instance is opened.");
      pd_fourtySeven.setBound(true);
      //pd_ten.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_fourtySeven.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_fourtySeven.setValue("valueTypeInGroup", new Object[] {ImmutableName.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtySeven.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_fourtySeven.setValue("panelGroupName", "  Actions  ");
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer.
      pd_fourtySeven.setValue("withViewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the PropertyEditor
      // has to display 2 'Select' buttons if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
        //pd_fourtySeven.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the PropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
        //pd_fourtySeven.setValue("withRemoveButton", Boolean.TRUE );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_fourtySeven.setValue("limitedSizeForPresentation", new Integer(8));

      // ------------------ property "M_SubAcsNodePolicy" --------------- 
      PropertyDescriptor pd_fourtyEight = new PropertyDescriptor(
        "M_SubAcsNodePolicy", ACSImpl.class, "getM_SubAndGroupPolicy", null);
      pd_fourtyEight.setDisplayName("SubACSs and Groups tree roots");
      pd_fourtyEight.setShortDescription(
              "SubACS and groups tree names, and explorer nodes for each.");
      pd_fourtyEight.setBound(false);
      //pd_fourtyEight.setConstrained(false);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_fourtyEight.setValue("isStringMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle. 
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_fourtyEight.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyEight.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtyEight.setValue("panelGroupName", "  A. SubACSs and Groups tree roots  ");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.      
      pd_fourtyEight.setValue("limitedSizeForPresentation", new Integer(8));

      // ------------------ property "M_BelongsToComposite" --------------- 
      PropertyDescriptor pd_fourtyNine = new PropertyDescriptor(
        "M_BelongsToComposite", ACSImpl.class, "getM_BelongsToComposite", null);
      pd_fourtyNine.setDisplayName("ACS or subACS which belongs to a composite (not still implemented)");
      pd_fourtyNine.setShortDescription(
              "For the ACS and each SubACS, say if it is in a composite ACS, but "+
              "this is a work in progress for future versions of Access Road.");
      pd_fourtyNine.setBound(false);
      //pd_fourtyNine.setConstrained(false);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_fourtyNine.setValue("isStringMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle.
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_fourtyNine.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyNine.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtyNine.setValue("panelGroupName", "  B. ACS or subACS which belongs to a composite  ");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.      
      pd_fourtyNine.setValue("limitedSizeForPresentation", new Integer(8));
      
      // ------------------- property "L_RootsForOpening" --------------------
      PropertyDescriptor pd_sixtyOne = new PropertyDescriptor(
        "L_RootsForOpening", ACSImpl.class, "getL_RootsForOpening", null);

      pd_sixtyOne.setDisplayName("Roots for opening");
      pd_sixtyOne.setShortDescription("These roots in a parent ACS "+
              "must be there at the opening of this ACS.");
      pd_sixtyOne.setBound(false);
      //pd_sixtyOne.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_sixtyOne.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle.
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_sixtyOne.setValue("valueTypeInGroup", new Object[] {ImmutableName.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixtyOne.setValue("tabbedPaneName", "  Roots  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_sixtyOne.setValue("panelGroupName", "   ACS Roots  " );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer.
      pd_sixtyOne.setValue("withViewButton", Boolean.TRUE );
      // 'horizontalTextPositionToLeft' is the boolean which is set if
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value.
        //pd_sixtyOne.setValue("horizontalTextPositionToLeft", Boolean.TRUE );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The common values are 5, 8, 10 (standard height), 16, 32.
      pd_sixtyOne.setValue("limitedSizeForPresentation", new Integer(5));

      // ------------------- property "L_RootsForClosing" --------------------
      PropertyDescriptor pd_sixtyTwo = new PropertyDescriptor(
        "L_RootsForClosing", ACSImpl.class, "getL_RootsForClosing", null);

      pd_sixtyTwo.setDisplayName("Roots for closing");
      pd_sixtyTwo.setShortDescription("These roots in a parent ACS "+
              "are deleted at the closing of this ACS.");
      pd_sixtyTwo.setBound(false);
      //pd_sixtyTwo.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_sixtyTwo.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle.
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_sixtyTwo.setValue("valueTypeInGroup", new Object[] {ImmutableName.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixtyTwo.setValue("tabbedPaneName", "  Roots  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_sixtyTwo.setValue("panelGroupName", "   ACS Roots  " );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer.
      pd_sixtyTwo.setValue("withViewButton", Boolean.TRUE );
      // 'horizontalTextPositionToLeft' is the boolean which is set if
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value.
        //pd_sixtyTwo.setValue("horizontalTextPositionToLeft", Boolean.TRUE );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The common values are 5, 8, 10 (standard height), 16, 32.
      pd_sixtyTwo.setValue("limitedSizeForPresentation", new Integer(5));

      // ------------------- property "L_RootsForIntegrity" --------------------
      PropertyDescriptor pd_fiftySix = new PropertyDescriptor(
        "L_RootsForIntegrity", ACSImpl.class, "getL_RootsForIntegrity", null);

      pd_fiftySix.setDisplayName("Roots for integrity");
      pd_fiftySix.setShortDescription("These roots in a parent ACS "+
              "ensure the integrity of this ACS.");
      pd_fiftySix.setBound(false);
      //pd_fiftySix.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_fiftySix.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle.
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_fiftySix.setValue("valueTypeInGroup", new Object[] {ImmutableName.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftySix.setValue("tabbedPaneName", "  Roots  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fiftySix.setValue("panelGroupName", "   ACS Roots  " );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer.
      pd_fiftySix.setValue("withViewButton", Boolean.TRUE );
      // 'horizontalTextPositionToLeft' is the boolean which is set if
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value.
        //pd_fiftySix.setValue("horizontalTextPositionToLeft", Boolean.TRUE );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The common values are 5, 8, 10 (standard height), 16, 32.
      pd_fiftyThree.setValue("limitedSizeForPresentation", new Integer(5));
      
      // ------------------- property "AssociationForNewObjectsMap" --------------------
      PropertyDescriptor pd_seventyFour = new PropertyDescriptor(
        "AssociationForNewObjectsMap", ACSImpl.class, "getM_AssociationsOnNewObject",
        null);
      pd_seventyFour.setDisplayName("Actions for association to a new object");
      pd_seventyFour.setShortDescription("Associations for new objects");
      pd_seventyFour.setBound(false);
        //pd_seventyFour.setConstrained(false); 
      // 'isActionMap' is TRUE when the right property editor is AssociationMapPropertyEditor
      pd_seventyFour.setValue("isActionMap", Boolean.TRUE );      
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle. 
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_seventyFour.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventyFour.setValue("tabbedPaneName", "  Actions  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventyFour.setValue("panelGroupName", 
              "    Actions to do at the creation of an object - for advanced users  ");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The common values are 5, 8, 10 (standard height), 16, 32.
      pd_seventyFour.setValue("limitedSizeForPresentation", new Integer(32));
      // 'horizontalTextPositionToLeft' is the boolean which is set if 
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value. 
      pd_seventyFour.setValue("horizontalTextPositionToLeft", Boolean.TRUE );

      // ------------------ property "M_RootsForAcsOpeningAndClosing" --------------- 
      PropertyDescriptor pd_fiftySeven = new PropertyDescriptor(
        "M_RootsForAcsOpeningAndClosing", ACSImpl.class,
        "getM_RootCommandsForOpeningAndClosing", null);
      pd_fiftySeven.setDisplayName("Actions on roots for ACS opening and closing");
      pd_fiftySeven.setShortDescription(
              "Parent objects/properties which are necessary to run this ACS;"+
                " closed/updated at its closing.");
      pd_fiftySeven.setBound(false);
      //pd_fiftySeven.setConstrained(false);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_fiftySeven.setValue("isActionMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle. 
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_fiftySeven.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftySeven.setValue("tabbedPaneName", "  Root Actions  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fiftySeven.setValue("panelGroupName", "    A. Actions for ACS opening and closing"+
              "- for advanced users  ");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The common values are 5, 8, 10 (standard height), 16, 32.
      pd_fiftySeven.setValue("limitedSizeForPresentation", new Integer(10));
      // 'horizontalTextPositionToLeft' is the boolean which is set if
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value.
      pd_fiftySeven.setValue("horizontalTextPositionToLeft", Boolean.TRUE );
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      // The default is 100 for a StringMap or one of its subclass.
      pd_fiftySeven.setValue("MaxValueSize", new Integer(500));

      // ------------------ property "M_RootsForAcsOpeningNoClosing" --------------- 
      PropertyDescriptor pd_fiftyEight = new PropertyDescriptor(
        "M_RootsForAcsOpeningNoClosing", ACSImpl.class,
        "getM_RootCommandsForOpeningNoClosing", null);
      pd_fiftyEight.setDisplayName("Actions on roots for ACS opening only");
      pd_fiftyEight.setShortDescription(
              "Parent objects/properties which are necessary to run this ACS;"+
                " may be opened when the ACS is closed.");
      pd_fiftyEight.setBound(false);
      //pd_fiftyEight.setConstrained(false);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_fiftyEight.setValue("isActionMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle. 
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_fiftyEight.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftyEight.setValue("tabbedPaneName", "  Root Actions  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fiftyEight.setValue("panelGroupName", "    B. Actions for ACS opening only, not for closing"+
              "- for advanced users  ");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The common values are 5, 8, 10 (standard height), 16, 32.
      pd_fiftyEight.setValue("limitedSizeForPresentation", new Integer(10));
      // 'horizontalTextPositionToLeft' is the boolean which is set if
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value.
      pd_fiftyEight.setValue("horizontalTextPositionToLeft", Boolean.TRUE );
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      // The default is 100 for a StringMap or one of its subclass.
      pd_fiftyEight.setValue("MaxValueSize", new Integer(500));
      
      // ------------------ property "M_RootsForAcsIntegrity" --------------- 
      PropertyDescriptor pd_fiftyNine = new PropertyDescriptor(
        "M_RootsForAcsIntegrity", ACSImpl.class, "getM_RootCommandsForIntegrity", null);
      pd_fiftyNine.setDisplayName("Actions on roots for ACS integrity");
      pd_fiftyNine.setShortDescription(
              "Parent objects/properties which ensure the integrity of this ACS.");
      pd_fiftyNine.setBound(false);
      //pd_fiftyNine.setConstrained(false);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_fiftyNine.setValue("isActionMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle. 
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_fiftyNine.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fiftyNine.setValue("tabbedPaneName", "  Root Actions  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fiftyNine.setValue("panelGroupName", 
              "    C. Actions for ACS integrity - for advanced users  ");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The common values are 5, 8, 10 (standard height), 16, 32.
      pd_fiftyNine.setValue("limitedSizeForPresentation", new Integer(10));
      // 'horizontalTextPositionToLeft' is the boolean which is set if
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value.
      pd_fiftyNine.setValue("horizontalTextPositionToLeft", Boolean.TRUE );
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      // The default is 100 for a StringMap or one of its subclass.
      pd_fiftyNine.setValue("MaxValueSize", new Integer(500));

      // ------------------- property "Icon" ------------------------ getEditorAndAddonNames
      PropertyDescriptor pd_sixty = new PropertyDescriptor(
        "Icon", ACSImpl.class, "getIconName", null);
      pd_sixty.setDisplayName("ACS icon");
      pd_sixty.setShortDescription("The name of the dedicated ACS icon");
      pd_sixty.setBound(false);
      //pd_sixty.setConstrained(false);
      //pd_sixty.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixty.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_sixty.setValue("panelGroupName", "  Types  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_sixty.setValue("BaseBeanManager", null);

      // ------------------- property "RootsForChildACS" --------------------
      PropertyDescriptor pd_sixtyThree = new PropertyDescriptor(
        "RootsForChildACS", ACSImpl.class, "getM_InnerRootsForChildACS",
        null);
      pd_sixtyThree.setDisplayName("Child ACS roots");
      pd_sixtyThree.setShortDescription("Map of the child ACS roots which are in this ACS");
      pd_sixtyThree.setBound(true);
      // 'isBaseObjectMap' is TRUE when the right property editor is BaseObjectMapPropertyEditor
      pd_sixtyThree.setValue("isBaseObjectMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, the value is the type of the values in the collection or in the map,
      // or for a StringMapPropertyEditor or one of its subclass,
      // it is the type of the value to select or view.
      pd_sixtyThree.setValue("valueTypeInGroup", new Object[] {ACSObject.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixtyThree.setValue("tabbedPaneName", "  Context  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_sixtyThree.setValue("panelGroupName", 
              "    Child ACS roots in this ACS for opened or closed child ACS ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when is is not applicable.
        //pd_sixtyThree.setValue("BaseBeanManager", );
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_sixtyThree.setValue("KeysUpdateInMap", Boolean.FALSE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_sixtyThree.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_sixtyThree.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display a 'New' button. False if it is not editable trough a constructor.
      pd_sixtyThree.setValue("withNewButton", Boolean.FALSE ); // for map keys
      // 'withSelectButton' is the boolean which set if the StringMapPropertyEditor
      // has to display a 'Select' button.
      // False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_sixtyThree.setValue("withSelectButton", Boolean.FALSE ); // for map values
      // 'withDeleteButton' is the boolean which is set if the StringMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_sixtyThree.setValue("withRemoveButton", Boolean.FALSE ); // for map values
      pd_sixtyThree.setValue("withDeleteButton", Boolean.FALSE ); // for map keys
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The common values are 5, 8, 10 (standard height), 16, 32.
      pd_sixtyThree.setValue("limitedSizeForPresentation", new Integer(8));

      // ------------------- property "CommentsOnProperties" --------------------
      PropertyDescriptor pd_sixtyFour = new PropertyDescriptor(
        "CommentsOnProperties", ACSImpl.class, "getM_CommentsOnProperties",
        null);
      pd_sixtyFour.setDisplayName("Comments on the simulation");
      pd_sixtyFour.setShortDescription(
              "Comments on how this ACS simulates the real system/software");
      pd_sixtyFour.setBound(true);
      // 'isStringTwoKeysMap' is TRUE when the right property editor is StringTwoKeysMapPropertyEditor
      // replace 'isStringMap' to have a value panel with a greater width
      pd_sixtyFour.setValue("isStringTwoKeysMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_sixtyFour.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixtyFour.setValue("tabbedPaneName", "  See Why  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_sixtyFour.setValue("panelGroupName",
              "  A. Designer comments on how this ACS simulates the real system/software ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when is is not applicable.
        //pd_sixtyFour.setValue("BaseBeanManager", );
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_sixtyFour.setValue("KeysUpdateInMap", Boolean.FALSE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_sixtyFour.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_sixtyFour.setValue("withViewButton", Boolean.FALSE );
      // 'withNewButton' is the boolean which set if the PropertyEditor
      // has to display a 'New' button. False if it is not editable trough a constructor.
      pd_sixtyFour.setValue("withNewButton", Boolean.FALSE );
      // 'withNewVButton' is the boolean which set if the PropertyEditor
      // has to display a 'NewV' button if the map value (not key) is editable
      // in the beamer. False if it is not editable.
      pd_sixtyFour.setValue("withNewVButton", Boolean.FALSE );
      // 'withDeleteVButton' is the boolean which is set if the PropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      // 'withDeletVButton' matches with 'NewV'; it is also for map values only.
      pd_sixtyFour.setValue("withDeleteButton", Boolean.FALSE );
      pd_sixtyFour.setValue("withDeletVButton", Boolean.FALSE );
       // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor (else, see 'constrRestrictedArgument' in the BeanDescriptor).
        //pd_sixtyFour.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke
      // to let the user defines the new restricted key, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
          //pd_sixtyFour.setValue("choiceMethodForNew", new Object[] {
            //null, "getL_MapKeysForCommentsOnProperties"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined (only for the first value).
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
        //pd_sixtyFour.setValue("limitedValuesMethod",
          //    "getL_LimitedValuesForTypesAndExplorer" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_sixtyFour.setValue("addMethod", "addCommentsOnProperties");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
        //pd_sixtyFour.setValue("removeMethod", "removeCommentsOnProperties");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      pd_sixtyFour.setValue("limitedSizeForPresentation", new Integer(32));
      // 'horizontalTextPositionToLeft' is the boolean which is set if 
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value. 
      pd_sixtyFour.setValue("horizontalTextPositionToLeft", Boolean.TRUE );

      // ------------------- property "AcsVocabulary" --------------------
      PropertyDescriptor pd_seventyNine = new PropertyDescriptor(
        "AcsVocabulary", ACSImpl.class, "getM_AcsVocabulary",
        null);
      pd_seventyNine.setDisplayName("Vocabulary");
      pd_seventyNine.setShortDescription(
              "Used classes, including AcsAddon classes, and terms");
      pd_seventyNine.setBound(true);
      // 'isStringMap' is TRUE when the right property editor is StringMapPropertyEditor
      pd_seventyNine.setValue("isStringMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_seventyNine.setValue("valueTypeInGroup", new Object[] {String.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventyNine.setValue("tabbedPaneName", "  Structure 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventyNine.setValue("panelGroupName",
              "  C. What editor, version and classes ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when is is not applicable.
        //pd_seventyFive.setValue("BaseBeanManager", );
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
            //pd_seventyNine.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_seventyNine.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the StringMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_seventyNine.setValue("withViewButton", Boolean.FALSE );
      // 'withNewVButton' is the boolean which set if the PropertyEditor
      // has to display a 'NewV' button if the map value (not key) is editable
      // in the beamer. False if it is not editable.
      pd_seventyNine.setValue("withNewButton", Boolean.FALSE );
      // 'withDeleteVButton' is the boolean which is set if the PropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      // 'withDeletVButton' matches with 'NewV'; it is also for map values only.
      pd_seventyNine.setValue("withDeleteButton", Boolean.FALSE );
      pd_seventyNine.setValue("withDeletVButton", Boolean.FALSE );
       // 'isRestrictedArgument' is true if it is a constrained key that is not
      // in the constructor (else, see 'constrRestrictedArgument' in the BeanDescriptor).
            //pd_seventyNine.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke
      // to let the user defines the new restricted key, in a map, or the new instance,
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if
      // there is an add method or a write method. With 'isRestrictedArgument',
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
            //pd_seventyNine.setValue("choiceMethodForNew", new Object[] {
                //null, "getL_MapKeysForAcsVocabulary"} );
      // 'limitedValuesMethod' is the array of possible values for each possible key
      // which is defined in 'staticSetOfItems' for the property, or for the static key
      // set by 'limitedValuesMethodKey' if it is defined (only for the first value).
      // It is an equivallent to 'isRestrictedArgument' for the allowed keys.
      // The type of the array has to be the type of the returned elements
      // for the values in the 'get' property method.
        //pd_seventyFive.setValue("limitedValuesMethod",
          //    "getL_LimitedValuesForTypesAndExplorer" );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
            //pd_seventyNine.setValue("addMethod", "addAcsVocabulary");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
            //pd_seventyNine.setValue("removeMethod", "removeAcsVocabulary");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      pd_seventyNine.setValue("limitedSizeForPresentation", new Integer(5));
      // 'horizontalTextPositionToLeft' is the boolean which is set if 
      // the PropertyEditor has to display the values from the left
      // in the beamer. The Right position is the default value. 
      pd_seventyNine.setValue("horizontalTextPositionToLeft", Boolean.TRUE );
      // 'MaxValueSize' is the maximal size of the values list for each map key.
      pd_seventyNine.setValue("MaxValueSize", new Integer(1));

      // ------------------- property "Editor" ------------------------
      PropertyDescriptor pd_114 = new PropertyDescriptor(
        "Editor", ACSImpl.class, "getEditorAndAddonNames", null);
      pd_114.setDisplayName("ACS editor, add-on name");
      pd_114.setShortDescription("The ACS editor and the ACS add-on name");
      pd_114.setBound(false);
      //pd_114.setConstrained(false);
      //pd_114.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_114.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_114.setValue("panelGroupName", "  Types  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_114.setValue("BaseBeanManager", null);

      // ------------------ property "IncompleteStructureModeling" ---------------
      PropertyDescriptor pd_eightyOne = new PropertyDescriptor(
        "IncompleteStructureModeling", ACSImpl.class,
        "getIncompleteStructureModeling",
        "setIncompleteStructureModeling");
      pd_eightyOne.setDisplayName("Incomplete objects list");
      pd_eightyOne.setShortDescription("true if the modeling of the ACS objects list is incomplete.");
      pd_eightyOne.setBound(false);
      //pd_eightyOne.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyOne.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_eightyOne.setValue("panelGroupName", "  inter-ACS Structure  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_eightyOne.setValue("BaseBeanManager", null);

      // ------------------ property "IncompleteBehaviorModeling" ---------------
      PropertyDescriptor pd_eightyTwo = new PropertyDescriptor(
        "IncompleteBehaviorModeling", ACSImpl.class,
        "getIncompleteBehaviorModeling",
        null);
      pd_eightyTwo.setDisplayName("Incomplete behavior");
      pd_eightyTwo.setShortDescription("true if the modeling of the ACS behavior is incomplete.");
      pd_eightyTwo.setBound(false);
      //pd_eightyTwo.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyTwo.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_eightyTwo.setValue("panelGroupName", "  inter-ACS Structure  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_eightyTwo.setValue("BaseBeanManager", null);

      // ------------------ property "ManagePrivilegesInVirtualFolders" ---------------
      PropertyDescriptor pd_106 = new PropertyDescriptor(
        "ManagePrivilegesInVirtualFolders", ACSImpl.class,
        "getManagePrivilegesInVirtualFolders",
        null);
      pd_106.setDisplayName("Privileges in VirtualFolders");
      pd_106.setShortDescription("true if the privileges are managed in the virtual folders.");
      pd_106.setBound(false);
      //pd_eightyThree.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_106.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_106.setValue("panelGroupName", "  A - Privileges (LPRI and TPRI)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_106.setValue("BaseBeanManager", null);

      // ------------------ property "ManagePrivilegesInResources" ---------------
      PropertyDescriptor pd_107 = new PropertyDescriptor(
        "ManagePrivilegesInResources", ACSImpl.class,
        "getManagePrivilegesInResources",
        null);
      pd_107.setDisplayName("Privileges in Resources");
      pd_107.setShortDescription("true if the privileges are managed in the resources.");
      pd_107.setBound(false);
      //pd_eightyThree.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_107.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_107.setValue("panelGroupName", "  A - Privileges (LPRI and TPRI)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_eightyThree.setValue("BaseBeanManager", null);

      // ------------------ property "ManageSimpleLinkedPrivileges" ---------------
      PropertyDescriptor pd_eightyThree = new PropertyDescriptor(
        "ManageSimpleLinkedPrivileges", ACSImpl.class,
        "getManageSimpleLinkedPrivileges",
        null);
      pd_eightyThree.setDisplayName("Simple Linked PRI ('LPRI')");
      pd_eightyThree.setShortDescription("true if the simplest linked privileges are managed.");
      pd_eightyThree.setBound(false);
      //pd_eightyThree.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyThree.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightyThree.setValue("panelGroupName", "  A - Privileges (LPRI and TPRI)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_eightyThree.setValue("BaseBeanManager", null);

      // ------------------ property "ManageSimpleTypedPrivileges" ---------------
      PropertyDescriptor pd_eightyFour = new PropertyDescriptor(
        "ManageSimpleTypedPrivileges", ACSImpl.class,
        "getManageSimpleTypedPrivileges",
        null);
      pd_eightyFour.setDisplayName("Simple Typed PRI ('TPRI')");
      pd_eightyFour.setShortDescription("true if the simplest typed privileges are managed.");
      pd_eightyFour.setBound(false);
      //pd_eightyFour.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyFour.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightyFour.setValue("panelGroupName", "  A - Privileges (LPRI and TPRI)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_eightyFour.setValue("BaseBeanManager", null);

      // ------------------ property "ManageIfAllSourceLinkedPrivileges" ---------------
      PropertyDescriptor pd_100 = new PropertyDescriptor(
        "ManageIfAllSourceLinkedPrivilegess", ACSImpl.class,
        "getManageIfAllSourceLinkedPrivileges",
        null);
      pd_100.setDisplayName("If-All-sources LPRI");
      pd_100.setShortDescription("true if the If-All-Sources linked privileges are managed.");
      pd_100.setBound(false);
      //pd_100.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_100.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_100.setValue("panelGroupName", "  A - Privileges (LPRI and TPRI)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_100.setValue("BaseBeanManager", null);

      // ------------------ property "ManageInternalForOneToOneLinkedPrivileges" ---------------
      PropertyDescriptor pd_101 = new PropertyDescriptor(
        "ManageInternalForOneToOneLinkedPrivileges", ACSImpl.class,
        "getManageInternalForOneToOneLinkedPrivileges",
        null);
      pd_101.setDisplayName("Internal One-to-One LPRI");
      pd_101.setShortDescription("true if the internal One-To-One linked privileges are managed.");
      pd_101.setBound(false);
      //pd_101.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_101.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_101.setValue("panelGroupName", "  A - Privileges (LPRI and TPRI)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_101.setValue("BaseBeanManager", null);

      // ------------------ property "ManageExternalForOneToOneLinkedPrivileges" ---------------
      PropertyDescriptor pd_102 = new PropertyDescriptor(
        "ManageExternalForOneToOneLinkedPrivileges", ACSImpl.class,
        "getManageExternalForOneToOneLinkedPrivileges",
        null);
      pd_102.setDisplayName("External One-to-One LPRI");
      pd_102.setShortDescription("true if the external One-To-One linked privileges are managed,"+
              " at least for the own sources.");
      pd_102.setBound(false);
      //pd_102.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_102.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_102.setValue("panelGroupName", "  A - Privileges (LPRI and TPRI)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_102.setValue("BaseBeanManager", null);

      // ------------------ property "ManageClassedTypedPrivileges" ---------------
      PropertyDescriptor pd_103 = new PropertyDescriptor(
        "ManageClassedTypedPrivileges", ACSImpl.class,
        "getManageClassedTypedPrivileges",
        null);
      pd_103.setDisplayName("Classed TPRI");
      pd_103.setShortDescription("true if the classed typed privileges are managed.");
      pd_103.setBound(false);
      //pd_103.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_103.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_103.setValue("panelGroupName", "  A - Privileges (LPRI and TPRI)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_103.setValue("BaseBeanManager", null);

      // ------------------ property "ManageClassedSpecificTypedPrivileges" ---------------
      PropertyDescriptor pd_104 = new PropertyDescriptor(
        "ManageClassedSpecificTypedPrivileges", ACSImpl.class,
        "getManageClassedSpecificTypedPrivileges",
        null);
      pd_104.setDisplayName("Specific TPRI");
      pd_104.setShortDescription("true if the specific typed privileges are managed "+
              "for designated source and target.");
      pd_104.setBound(false);
      //pd_104.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_104.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_104.setValue("panelGroupName", "  A - Privileges (LPRI and TPRI)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_104.setValue("BaseBeanManager", null);

      // ------------------ property "ManageAuthorizationServerTypedPrivileges" ---------------
      PropertyDescriptor pd_105 = new PropertyDescriptor(
        "ManageAuthorizationServerTypedPrivileges", ACSImpl.class,
        "getManageAuthorizationServerTypedPrivileges",
        null);
      pd_105.setDisplayName("Authorization server TPRI");
      pd_105.setShortDescription("true if the typed privileges are managed "+
              "for authorization server.");
      pd_105.setBound(false);
      //pd_105.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_105.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_105.setValue("panelGroupName", "  A - Privileges (LPRI and TPRI)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_eightyFour.setValue("BaseBeanManager", null);

      // ------------------ property "ManageGrantingRightPrivileges" ---------------
      PropertyDescriptor pd_eightyFive = new PropertyDescriptor(
        "ManageGrantingRightPrivileges", ACSImpl.class,
        "getManageGrantingRightPrivileges",
        null);
      pd_eightyFive.setDisplayName("Granting Privileges");
      pd_eightyFive.setShortDescription("true if the granting privileges are managed.");
      pd_eightyFive.setBound(false);
      //pd_eightyFive.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyFive.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightyFive.setValue("panelGroupName", "  A - Privileges (LPRI and TPRI)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_eightyFive.setValue("BaseBeanManager", null);

      // ------------------ property "ManageDenyingRightPrivileges" ---------------
      PropertyDescriptor pd_eightySix = new PropertyDescriptor(
        "ManageDenyingRightPrivileges", ACSImpl.class,
        "getManageDenyingRightPrivileges",
        null);
      pd_eightySix.setDisplayName("Denying Privileges");
      pd_eightySix.setShortDescription("true if the denying privileges are managed.");
      pd_eightySix.setBound(false);
      //pd_eightySix.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightySix.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightySix.setValue("panelGroupName", "  A - Privileges (LPRI and TPRI)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_eightySix.setValue("BaseBeanManager", null);

      // ------------------ property "ManageResourceVirtualFolders" ---------------
      PropertyDescriptor pd_eightySeven = new PropertyDescriptor(
        "ManageResourceVirtualFolders", ACSImpl.class,
        "getManageResourceVirtualFolders",
        null);
      pd_eightySeven.setDisplayName("Simple-Resources ('Res-VF')");
      pd_eightySeven.setShortDescription("true if there are Virtual Folders"+
              " for resources which are not right users");
      pd_eightySeven.setBound(false);
      //pd_eightySeven.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightySeven.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightySeven.setValue("panelGroupName", "  B - Virtual Folders (Res-VF and RU-VF)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_eightySeven.setValue("BaseBeanManager", null);

      // ------------------ property "ManageEPVirtualFolders" ---------------
      PropertyDescriptor pd_eightyEight = new PropertyDescriptor(
        "ManageEPVirtualFolders", ACSImpl.class,
        "getManageEPVirtualFolders",
        null);
      pd_eightyEight.setDisplayName("Right-User Resour. ('RU-VF')");
      pd_eightyEight.setShortDescription("true if the Actors "+
              "and/or the DirectoryEPs are managed in some Virtual Folders.");
      pd_eightyEight.setBound(false);
      //pd_eightyEight.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyEight.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightyEight.setValue("panelGroupName", "  B - Virtual Folders (Res-VF and RU-VF)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_eightyEight.setValue("BaseBeanManager", null);

      // ------------------ property "ManageSimpleResourceVirtualFolders" ---------------
      PropertyDescriptor pd_eightyNine = new PropertyDescriptor(
        "ManageSimpleResourceVirtualFolders", ACSImpl.class,
        "getManageSimpleResourceVirtualFolders",
        null);
      pd_eightyNine.setDisplayName("Simple items in Res-VF");
      pd_eightyNine.setShortDescription("true if the no-directory resources are managed"+
              " in Resource Virtual Folders.");
      pd_eightyNine.setBound(false);
      //pd_eightyNine.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eightyNine.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eightyNine.setValue("panelGroupName", "  B - Virtual Folders (Res-VF and RU-VF)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_eightyNine.setValue("BaseBeanManager", null);

      // ------------------ property "ManageDirectoryInResourceVirtualFolders" ---------------
      PropertyDescriptor pd_ninetyOne = new PropertyDescriptor(
        "ManageDirectoryInResourceVirtualFolders", ACSImpl.class,
        "ManageDirectoryInResourceVirtualFolders",
        null);
      pd_ninetyOne.setDisplayName("Directories in Res-VF");
      pd_ninetyOne.setShortDescription("true if the simple directories are managed in"+
              " Resource Virtual Folders.");
      pd_ninetyOne.setBound(false);
      //pd_ninetyOne.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetyOne.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetyOne.setValue("panelGroupName", "  B - Virtual Folders (Res-VF and RU-VF)  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_ninetyOne.setValue("BaseBeanManager", null);

      // ------------------ property "ManageDirectoryEPinEPVirtualFolders" ---------------
      PropertyDescriptor pd_ninetyTwo = new PropertyDescriptor(
        "ManageDirectoryEPinEPVirtualFolders", ACSImpl.class,
        "getManageDirectoryEPinEPVirtualFolders",
        null);
      pd_ninetyTwo.setDisplayName("DirectoryEPs in RU-VF");
      pd_ninetyTwo.setShortDescription("true if the right user directories are managed"+
              " in the Right User Virtual Folders.");
      pd_ninetyTwo.setBound(false);
      //pd_ninetyTwo.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetyTwo.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetyTwo.setValue("panelGroupName", "  B - Virtual Folders (Res-VF and RU-VF)  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_ninetyTwo.setValue("BaseBeanManager", null);

      // ------------------ property "ManageActorInEPVirtualFolders" ---------------
      PropertyDescriptor pd_ninetyThree = new PropertyDescriptor(
        "ManageActorInEPVirtualFolders", ACSImpl.class,
        "getManageActorInEPVirtualFolders",
        null);
      pd_ninetyThree.setDisplayName("Actors in RU-VF");
      pd_ninetyThree.setShortDescription("true if the actors are managed in"+
              " the Right User Virtual Folders.");
      pd_ninetyThree.setBound(false);
      //pd_ninetyThree.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetyThree.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetyThree.setValue("panelGroupName", "  B - Virtual Folders (Res-VF and RU-VF)  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_ninetyThree.setValue("BaseBeanManager", null);

      // ------------------ property "ManageACLRightInheritance" ---------------
      PropertyDescriptor pd_ninetyFour = new PropertyDescriptor(
        "ManageACLRightInheritance", ACSImpl.class,
        "getManageACLRightsInheritance",
        null);
      pd_ninetyFour.setDisplayName("ACL inheritance");
      pd_ninetyFour.setShortDescription("true if the ACL inheritance is managed"+
              " in the resources tree.");
      pd_ninetyFour.setBound(false);
      //pd_ninetyFour.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetyFour.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetyFour.setValue("panelGroupName",
              "  The main concepts which are managed or not by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_ninetyFour.setValue("BaseBeanManager", null);

      // ------------------ property "ManageAGORightInheritance" ---------------
      PropertyDescriptor pd_ninetyFive = new PropertyDescriptor(
        "ManageAGORightInheritance", ACSImpl.class,
        "getManageAGORightsInheritance", null);
      pd_ninetyFive.setDisplayName("AG rights inheritance");
      pd_ninetyFive.setShortDescription("true if the Account/Group rights inheritance is managed"+
              " in the resources tree.");
      pd_ninetyFive.setBound(false);
      //pd_ninetyFive.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetyFive.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetyFive.setValue("panelGroupName",
              "  The main concepts which are managed or not by this ACS  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_ninetyFive.setValue("BaseBeanManager", null);

      // ------------------ property "ManageLPRIRightInheritance" ---------------
      PropertyDescriptor pd_ninetySeven = new PropertyDescriptor(
        "ManageLPRIRightInheritance", ACSImpl.class,
        "getManageLPRIRightsInheritance", null);
      pd_ninetySeven.setDisplayName("PRI inheritance");
      pd_ninetySeven.setShortDescription("true if the linked privileges inheritance is managed"+
              " in the resources or virtual folders tree.");
      pd_ninetySeven.setBound(false);
      //pd_ninetySeven.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetySeven.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetySeven.setValue("panelGroupName",
              "  The main concepts which are managed or not by this ACS  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_ninetySeven.setValue("BaseBeanManager", null);

      // ------------------ property "ManageVirtualFolderTrees" ---------------
      PropertyDescriptor pd_ninetySix = new PropertyDescriptor(
        "ManageVirtualFolderTrees", ACSImpl.class,
        "getManageVirtualFolderTrees", null);
      pd_ninetySix.setDisplayName("Virtual Folders Trees");
      pd_ninetySix.setShortDescription("true if 2 trees are managed for"+
              " the 2 main types of Virtual Folders.");
      pd_ninetySix.setBound(false);
      //pd_ninetySix.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ninetySix.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ninetySix.setValue("panelGroupName", "  The main concepts which are managed or not by this ACS  "); //"  B - Virtual Folder Trees  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_ninetySix.setValue("BaseBeanManager", null);

      // ------------------ property "ManageOneTimeInVirtualFolderTreeForMember" ---------------
      PropertyDescriptor pd_108 = new PropertyDescriptor(
        "ManageOneTimeInVirtualFolderTreeForMember", ACSImpl.class,
        "getManageOneTimeInVirtualFolderTreeForMember", null);
      pd_108.setDisplayName("One time in tree");
      pd_108.setShortDescription("true if a member is no more than one time"+
              " in each virtual folder tree.");
      pd_108.setBound(false);
      //pd_108.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_108.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_108.setValue("panelGroupName", 
              "  B - Virtual Folders (Res-VF and RU-VF)  " );

      // ------------------ property "ManageVirtualFolderRightsPropagationToMembers" 
      PropertyDescriptor pd_111 = new PropertyDescriptor(
        "ManageVirtualFolderRightsPropagationToMembers", ACSImpl.class,
        "getManageVirtualFolderRightsPropagationToMembers", null);
      pd_111.setDisplayName("Right transfer to members");
      pd_111.setShortDescription("true if a virtual folder transfers its target rights"+
              " to each of its members.");
      pd_111.setBound(true);
      //pd_111.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_111.setValue("tabbedPaneName", "  Structure 3  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_111.setValue("panelGroupName", 
              "  B - Virtual Folders (Res-VF and RU-VF)  " );

      // ------------------ property "ManageInternalHardAlias" ---------------
      PropertyDescriptor pd_109 = new PropertyDescriptor(
        "ManageInternalHardAlias", ACSImpl.class,
        "getManageInternalHardAlias", null);
      pd_109.setDisplayName("Hard aliasing");
      pd_109.setShortDescription("true if an acs object may be the alias of another one"+
              " from the same acs (like a Linux symbolic link).");
      pd_109.setBound(false);
      //pd_109.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_109.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_109.setValue("panelGroupName", 
              "  The main concepts which are managed or not by this ACS  " );

      // ------------------ property "ManageSoftAlias" ---------------
      PropertyDescriptor pd_110 = new PropertyDescriptor(
        "ManageSoftAlias", ACSImpl.class,
        "getManageSoftAlias", null);
      pd_110.setDisplayName("Soft aliasing");
      pd_110.setShortDescription("true if an acs object may be the alias of an"+
              " external or internal object, following the own ACS rules"+
              " (like a Linux hard link).");
      pd_110.setBound(false);
      //pd_110.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_110.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_110.setValue("panelGroupName", 
              "  The main concepts which are managed or not by this ACS  " );

      // ------------------ property "ManageFlexibleConditionalAcls" ---------------
      PropertyDescriptor pd_112 = new PropertyDescriptor(
        "ManageFlexibleConditionalAcls", ACSImpl.class,
        "getManageFlexibleConditionalAcls", null);
      pd_112.setDisplayName("Simple-condition ACL");
      pd_112.setShortDescription("true if an AclEntry may be conditional, desabling the rights "+
              "if the source is not member of each condition group.");
      pd_112.setBound(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_112.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_112.setValue("panelGroupName", 
              "  The main concepts which are managed or not by this ACS  " );

      // ------------------ property "ManageRightDefinedConditionalAcls" ---------------
      PropertyDescriptor pd_113 = new PropertyDescriptor(
        "ManageRightDefinedConditionalAcls", ACSImpl.class,
        "getManageRightDefinedConditionalAcls", null);
      pd_113.setDisplayName("Right-condition ACL");
      pd_113.setShortDescription("true if an AclEntry may be conditional, desabling the rights "+
              "if the source is not member of each right-defined condition group.");
      pd_113.setBound(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_113.setValue("tabbedPaneName", "  Structure  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_113.setValue("panelGroupName", 
              "  The main concepts which are managed or not by this ACS  " );

      // ------------------ property "OwnEPFoldersRootName" ---------------
      PropertyDescriptor pd_sixtyFive = new PropertyDescriptor(
        "OwnEPFoldersRootName", ACSImpl.class,
        "getOwnEPFoldersRootName", null);
      pd_sixtyFive.setDisplayName("Root of EP-VF");
      pd_sixtyFive.setShortDescription("The root of the"+
              " Eligible Party Virtual Folders tree.");
      pd_sixtyFive.setBound(true);
      //pd_sixtyFive.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixtyFive.setValue("tabbedPaneName", "  Right Users  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_sixtyFive.setValue("panelGroupName", "   " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_sixtyFive.setValue("BaseBeanManager", null);

      // ------------------ property "OwnResourceFoldersRootName" ---------------
      PropertyDescriptor pd_sixtySix = new PropertyDescriptor(
        "OwnResourceFoldersRootName", ACSImpl.class,
        "getOwnResourceFoldersRootName", null);
      pd_sixtySix.setDisplayName("Root of Res-VF");
      pd_sixtySix.setShortDescription("The root of the"+
              " Resource Virtual Folders tree.");
      pd_sixtySix.setBound(true);
      //pd_sixtySix.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixtySix.setValue("tabbedPaneName", "  Access Targets  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_sixtySix.setValue("panelGroupName", "  How Virtual Folders "+
        //      "are handled by this ACS  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_sixtySix.setValue("BaseBeanManager", null);

      // ------------------- property "OneResourceVirtualFolder" --------------------
      PropertyDescriptor pd_sixtySeven = new PropertyDescriptor(
        "OneResourceVirtualFolder", ACSImpl.class, "getEorL_ResourceVirtualFolders", null);
      pd_sixtySeven.setDisplayName("VFolders of Resources");
      pd_sixtySeven.setShortDescription("The virtual folders of Resources.");
      pd_sixtySeven.setBound(true);
      //pd_sixtySeven.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_sixtySeven.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle.
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_sixtySeven.setValue("valueTypeInGroup", new Object[] {VirtualFolder.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixtySeven.setValue("tabbedPaneName", "  Access Targets  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_sixtySeven.setValue("panelGroupName", "  List  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when is is not applicable.
          //pd_sixtySeven.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the CollectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer.
      pd_sixtySeven.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_sixtySeven.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_sixtySeven.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_sixtySeven.setValue("withDeleteButton", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke 
      // to let the user defines the new restricted key, in a map, or the new instance, 
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if 
      // there is an add method or a write method. With 'isRestrictedArgument', 
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      // The array is equal to 'new Object[1]' when the value 'withKnownArgumentsForNew' is null.
        //pd_sixtySeven.setValue("choiceMethodForNew", new Object[] {
          //  VirtualFolderImpl.EMPTY_INSTANCE, "valueOf"} );
      // 'withKnownArgumentsForNew' allows to define the constructor/add/copy arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the fixed argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor in the beamer,
      // and call the read method to initialize a l_fixedValue from this object.
      // Third case for the fixed argument: the string 'thisname' is used
      // to point to the value returned from getName() by the object in the beamer.
      // Fourth case, the string 'true' or 'false' is used
      // to define a boolean value.
      // Fifth case, if none of the previous cases matches: takes directly
      // the fixed argument if it is a String.
      // Used by SimplePropertyEditor.getFixedValuesForNewElement(), which
      // may use a subset of the returned arguments if the class of the object to create
      // waits a smaller set of arguments.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
      pd_sixtySeven.setValue("withKnownArgumentsForNew", new Object[] {
          null, "this", null, null, "true"} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_sixtySeven.setValue("addMethod", "addEorChild");
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_sixtySeven.setValue("removeMethod", "finalizeForUser");

      // ------------------- property "OneEPVirtualFolder" --------------------
      PropertyDescriptor pd_sixtyEight = new PropertyDescriptor(
        "OneEPVirtualFolder", ACSImpl.class, "getEorL_EPVirtualFolders", null);
      pd_sixtyEight.setDisplayName("Right-User VFolders");
      pd_sixtyEight.setShortDescription("The virtual folders of Right User Resources");
      pd_sixtyEight.setBound(true);
      //pd_sixtyEight.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      // If the display name is "", it is not displayed.
      pd_sixtyEight.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle.
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_sixtyEight.setValue("valueTypeInGroup", new Object[] {VirtualFolder.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixtyEight.setValue("tabbedPaneName", "  Right Users  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_nine.setValue("panelGroupName", "  List  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when is is not applicable.
          //pd_sixtyEight.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the CollectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer.
      pd_sixtyEight.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_sixtyEight.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_sixtyEight.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_sixtyEight.setValue("withDeleteButton", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke 
      // to let the user defines the new restricted key, in a map, or the new instance, 
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if 
      // there is an add method or a write method. With 'isRestrictedArgument', 
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke,
      // or null if it is the source, and the name of the method (for instance,
      // ActionNewResource.getInstance(), "createResource").
      // The method have no argument and returns an array String, a List or a Set of Strings.
      // The array is equal to 'new Object[1]' when the value 'withKnownArgumentsForNew' is null.
          //pd_sixtyEight.setValue("choiceMethodForNew", new Object[] {
            //    VirtualFolderImpl.EMPTY_INSTANCE, "valueOf"} );
      // 'withKnownArgumentsForNew' allows to define the constructor/add/copy arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the fixed argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor in the beamer,
      // and call the read method to initialize a l_fixedValue from this object.
      // Third case for the fixed argument: the string 'thisname' is used
      // to point to the value returned from getName() by the object in the beamer.
      // Fourth case, the string 'true' or 'false' is used
      // to define a boolean value.
      // Fifth case, if none of the previous cases matches: takes directly
      // the fixed argument if it is a String.
      // Used by SimplePropertyEditor.getFixedValuesForNewElement(), which
      // may use a subset of the returned arguments if the class of the object to create
      // waits a smaller set of arguments.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (for instance, children of Directory).
      // Caution: the known arguments have to be Objects.
      pd_sixtyEight.setValue("withKnownArgumentsForNew", new Object[] {
            null, "this", null, null, "false"} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_sixtyEight.setValue("addMethod", "addEorChild");
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_sixtyEight.setValue("removeMethod", "finalizeForUser");

      // ------------------- property "OneTypedPrivilege" --------------------
      PropertyDescriptor pd_sixtyNine = new PropertyDescriptor(
        "OneTypedPrivilege", ACSImpl.class, "getEorL_PrivilegeForTypes", null);
      pd_sixtyNine.setDisplayName("Typed privileges");
      pd_sixtyNine.setShortDescription("Own typed privileges of the ACS");
      pd_sixtyNine.setBound(true);
        //pd_sixtyNine.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor
      // If the display name is "", it is not displayed.
      pd_sixtyNine.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle.
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      // as DetailledName, is used only in a CollectionPropertyEditor
      // to display the detailled name rather than the name.
      pd_sixtyNine.setValue("valueTypeInGroup", new Object[] {PrivilegeForType.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixtyNine.setValue("tabbedPaneName", "  Rights 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_sixtyNine.setValue("panelGroupName", "    Privileges  ");
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_sixtyNine.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer.
      pd_sixtyNine.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'Add New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_sixtyNine.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the RightMapPropertyEditor
      // has to display 2 'Select' buttons if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_sixtyNine.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_sixtyNine.setValue("withDeleteButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor/add/copy arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the fixed argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor in the beamer,
      // and call the read method to initialize a l_fixedValue from this object.
      // Third case for the fixed argument: the string 'thisname' is used
      // to point to the value returned from getName() by the object in the beamer.
      // Fourth case, the string 'true' or 'false' is used
      // to define a boolean value.
      // Fifth case, if none of the previous cases matches: takes directly
      // the fixed argument if it is a String.
      // Used by SimplePropertyEditor.getFixedValuesForNewElement(), which
      // may use a subset of the returned arguments if the class of the object to create
      // waits a smaller set of arguments.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
      pd_sixtyNine.setValue("withKnownArgumentsForNew", new Object[] {
          "this", null, null, null, null, null, null} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_sixtyNine.setValue("addMethod", "addEorChild");
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_sixtyNine.setValue("removeMethod", "finalizeForUser");
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_sixtyNine.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_sixtyNine.setValue("limitedSizeForPresentation", new Integer(8));

      // ------------------- property "OneLinkedPrivilege" --------------------
      PropertyDescriptor pd_seventyTwo = new PropertyDescriptor(
        "OneLinkedPrivilege", ACSImpl.class, "getEorL_PrivilegeForLinks", null);
      pd_seventyTwo.setDisplayName("Linked privileges");
      pd_seventyTwo.setShortDescription("Own linked privileges of the ACS");
      pd_seventyTwo.setBound(true);
        //pd_seventyTwo.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor
      // If the display name is "", it is not displayed.
      pd_seventyTwo.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is an array, a collection or a map.
      // Then, it defines the type of the elementary object to handle.
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      // as DetailledName, is used only in a CollectionPropertyEditor
      // to display the detailled name rather than the name.
      pd_seventyTwo.setValue("valueTypeInGroup", new Object[] {PrivilegeForLinks.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventyTwo.setValue("tabbedPaneName", "  Rights 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventyTwo.setValue("panelGroupName", "    Privileges  ");
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_seventyTwo.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer.
      pd_seventyTwo.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'Add New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_seventyTwo.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the RightMapPropertyEditor
      // has to display 2 'Select' buttons if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_seventyTwo.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_seventyTwo.setValue("withDeleteButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor/add/copy arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the fixed argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor in the beamer,
      // and call the read method to initialize a l_fixedValue from this object.
      // Third case for the fixed argument: the string 'thisname' is used
      // to point to the value returned from getName() by the object in the beamer.
      // Fourth case, the string 'true' or 'false' is used
      // to define a boolean value.
      // Fifth case, if none of the previous cases matches: takes directly
      // the fixed argument if it is a String.
      // Used by SimplePropertyEditor.getFixedValuesForNewElement(), which
      // may use a subset of the returned arguments if the class of the object to create
      // waits a smaller set of arguments.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
      pd_seventyTwo.setValue("withKnownArgumentsForNew", new Object[] {
          "this", null, null, null, null} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_seventyTwo.setValue("addMethod", "addEorChild");
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_seventyTwo.setValue("removeMethod", "finalizeForUser");
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_seventyTwo.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_seventyTwo.setValue("limitedSizeForPresentation", new Integer(8));

      // ---------------------------------------------------------------------
      
      l_pd = new PropertyDescriptor[] {
        // General
        pd_one, pd_seventy, pd_fourtyFive,
        pd_twentySix,
        pd_sixty, // icon
        pd_thirtyTwo, // Comment
        pd_114, // editor
        pd_thirtySeven, pd_seventyThree,
        pd_eightyOne, pd_eightyTwo, // IncompleteStructure/BehaviorModeling

        // Context
            // constrained child ACS
        pd_sixtyThree,
            // ACS controls
        pd_two, pd_three, pd_seven,
            // bridges
        pd_ten, pd_eleven,
        
        // Structure
        pd_twelve, pd_thirteen, 
        pd_fourteen, pd_fifteen, 
        pd_twentyEight, 
        pd_nineteen,
        pd_seventeen, pd_eighteen, // ACLs
        pd_112, pd_113, // conditional ACLs
        pd_twentyFour, pd_thirtyOne,         
        pd_sixteen, pd_fiftyFour,
        pd_thirtySix, // privileges
        pd_fiftyFive, pd_ninetySix, // virtual folders, VF trees
        pd_ninetyFive, pd_ninetyFour, pd_ninetySeven, // rights inheritance
        pd_109, pd_110, // alias
        
        // Structure 2
        pd_fourtySeven, pd_fourtyEight, // tied child, subACS
        pd_fourtyNine, // belongs to composite
        pd_seventyNine, // AcsVocabulary

        // Structure 3
        pd_eightyFive, pd_eightySix, // granting/denying privileges
        pd_106, pd_107, // privileges for Resources and VirtualFolders
        pd_eightyThree, pd_eightyFour, // simple typed/linked privileges
        pd_100, pd_103, pd_101, // linked and typed privileges
        pd_104, pd_102, pd_105, // linked and typed privileges
        pd_eightySeven, pd_eightyEight, // virtual folders
        pd_eightyNine, // virtual folders
        pd_ninetyTwo, pd_ninetyOne, // virtual folders
        pd_ninetyThree, // virtual folders
        pd_108, pd_111, // virtual folders: OneTimeInVFTree, RightsPropagationToMembers
        
        // Types 
        pd_twentySeven,
        pd_twentyNine,
        
        // Rights
        pd_seventyOne, pd_fiftyOne,
        
        // Rights 2
        pd_fourtyTwo, pd_twenty, // authorization server, specialized rights
        pd_fifty, // all rights list
        pd_thirtyEight, pd_thirtyNine, // grant/deny rights
        pd_fiftyTwo, pd_fiftyThree, // ACS and meta rights
        pd_sixtyNine, pd_seventyTwo, // 2 privileges

        // Right Users
        pd_nine, pd_eight,
        pd_sixtyFive, // OwnEPFoldersRootName
        pd_sixtyEight, // EP VirtualFolders
        
        // Access Targets
        pd_five, pd_six,
        pd_sixtySix, // OwnResourceFoldersRootName
        pd_sixtySeven, // Resource VirtualFolders
                
        // Rules
        pd_fourtyThree,
        
        // Proxies
        pd_fourtySix,

        // Actions
        pd_seventyFour, pd_fiftyNine,
        
        // Roots
        pd_sixtyOne, pd_sixtyTwo, pd_fiftySix,

        // Root Actions
        pd_fiftySeven, pd_fiftyEight,

        // See Why
        pd_sixtyFour // CommentsOnProperties
                         
      };
      return l_pd;
    } catch (IntrospectionException e) {
      new ProcessError("In ACSImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In ACSImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

    /**
     * Get the instance, if it exists, which fires an event when
     * the bean is closed or deleted.
     * @return a BaseManager object.
     */
    public BaseBeanManager getBaseBeanManagerInstance() {
      return BaseManagerImpl.getInstance();
    }

    /**
     * Get the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, if it exists,
     * which fires an event when the bean instance is closed or deleted.
     * @return null
     */
    public Method getBaseBeanManagerGetMethod() {
      return null;
    }

    /**
     * Get the property name for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * @return "openACS"
     */
    public String getBaseBeanManagerBoundProperty() {
      return "openACS";
    }


} // ACSImplBeanInfo