//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.BaseBeanManager;
import ARoad0.gBase.*;
import java.beans.SimpleBeanInfo;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.lang.reflect.Method;

import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for editing the first properties of a ACSFactoryImpl instance.
 * It is used for the first properties which have to be set first.
 * These properties include mainly the ACS rights in this generic version.
 * For an AcsAddon, this class may be subclassed to edit some AcsAddon properties.
 * Otherwise, this generic class is selected in the creation process of the AcsAddon ACS.
 * gDMak.ActionNewACSyst uses ACSFactoryImplBeanInfo for editing most of the new ACS properties,
 * after the call to this class or its subclass.
 * @version 0.7.0
 */
public class ACSpreFactoryImplBeanInfo extends SimpleBeanInfo implements BeanInfoPattern {

  public ACSpreFactoryImplBeanInfo() {}

  /**
   * Gets the description of the bean.
   * Some standard values are set:
   * - 'hidden' is false since the bean is not hidden to the users
   * - 'expert' is false since the bean is not reserved to experts
   * <P>
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of ACSImpl
   *  <P>
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(ACSFactoryImpl.class );
    result.setDisplayName("ACS preFactory");

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of ACSImpl
    result.setValue("withTabbedPanes", Boolean.TRUE );

    // these strings signal the nodes to use in an ACS tree
    // to display this ACS, following the ACSTree.M_ACS_TREE_UPPER_NODES
    // in the Gui2 package.
    result.setValue("nodesInACSTree", new String[]
      {"ACSRIGHTS"});

    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    result.setHidden(false);
    result.setExpert(false);

    return result;
  } // getBeanDescriptor()

//  public EventSetDescriptor[] getEventSetDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getEventSetDescriptors() not yet implemented.");
//  }
//  public int getDefaultEventIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultEventIndex() not yet implemented.");
//  }


  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue():<P>
   * - 'tabbedPaneName' is the name (String value) of the pane
   *    where this property has to be displayed in the beamer.
   *    Is "" if 'withTabbedPanes' is false in the BeanDescriptor.
   * - 'panelGroupName' is the name (String value) of the group
   *    where this property has to be displayed with other related properties.
   *    May be "".<P>
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pd;

    try {
      // ------------------- property "Name" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "Name", ACSFactoryImpl.class, "getName", null);
      pd_one.setDisplayName("Selected ACS name");
      //pd_one.setShortDescription("   ");
      //pd_one.setBound(false);
      //pd_one.setConstrained(false);
      //pd_one.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_one.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_one.setValue("BaseBeanManager", null);

      // ------------------- property "ISname" ------------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "ISname", ACSFactoryImpl.class, "getISName", "setISName");
      pd_four.setDisplayName("Name of the IS");
      //pd_four.setShortDescription("   ");
      pd_four.setBound(true);
      //pd_four.setConstrained(false);
      //pd_four.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_four.setValue("panelGroupName", "  Structure  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_four.setValue("BaseBeanManager", null);      

      // ------------------- property "ParentName" ------------------------
      PropertyDescriptor pd_fourtyFive = new PropertyDescriptor(
        "ParentName", ACSFactoryImpl.class, "getParentName", "setParentName");
      pd_fourtyFive.setDisplayName("Parent name ");
      pd_fourtyFive.setShortDescription("The parent of the new ACS may be a component or an ACS");
      pd_fourtyFive.setBound(true);
      //pd_fourtyFive.setConstrained(false);
      //pd_fourtyFive.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourtyFive.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fourtyFive.setValue("panelGroupName", "  Structure  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_fourtyFive.setValue("BaseBeanManager", null); 
      
      // ------------------- property "ACSType" ------------------------
      PropertyDescriptor pd_nine = new PropertyDescriptor(
        "ACSType", ACSFactoryImpl.class, "getACSType", null);
      pd_nine.setDisplayName("Selected ACS type");
      pd_nine.setShortDescription("the ACS type may be GENERIC, LINUX, ... "+
        "or may be user-defined");
      //pd_nine.setBound(false);
      //pd_nine.setConstrained(false);
      //pd_nine.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_nine.setValue("tabbedPaneName", "  Main  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_nine.setValue("panelGroupName", "  Types  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_nine.setValue("BaseBeanManager", null);
 
      // ------------------- property "Rights" --------------------
      PropertyDescriptor pd_fifty = new PropertyDescriptor(
        "OneAcsRight", ACSFactoryImpl.class, "getL_Rights",
        null);
      pd_fifty.setDisplayName("ACS rights");
      pd_fifty.setShortDescription("The ACS rights to be used in the new ACS");
      pd_fifty.setBound(true);
        //pd_fifty.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor
      pd_fifty.setValue("isCollection", Boolean.TRUE );      
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_fifty.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fifty.setValue("tabbedPaneName", "  ACS Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_fifty.setValue("panelGroupName", "  Creations has to be done from the GREATER right"+
              " to the LESSER right ");
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_fifty.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_fifty.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'Add New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_fifty.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the RightMapPropertyEditor
      // has to display 2 'Select' buttons if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_fifty.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the RightMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
        //pd_fifty.setValue("withRemoveButton", Boolean.FALSE );
      pd_fifty.setValue("withDeleteButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be preset - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the preset argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Third case for the fixed argument: the string 'thisname' is used
      // to point to the value returned from getName() by the object in the beamer.
      // Used by SimplePropertyEditor.getKnownValuesForNewElement(), which
      // may use a subset of the returned arguments if the class of the object to create
      // waits a smaller set of arguments.
      pd_fifty.setValue("withKnownArgumentsForNew", new Object[] {
        null, null, null, null, null, "this"} );
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser", at the index 0.
      // Else, the returned array defines the name of the remove method, at the index 0.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_fifty.setValue("removeMethod", "finalizeForUser");    
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_eleven.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      pd_fifty.setValue("limitedSizeForPresentation", new Integer(8));
      
      // --------------------------------------------------------------------- 
      // The order of descriptors sets the display by the beamer,
      // and the capacity for the 'Previous' button to select
      // a collection in the previous property
      l_pd = new PropertyDescriptor[] {
        // ACS Rights
        pd_fifty,
        // Main
        pd_one, pd_nine, pd_fourtyFive, pd_four
        };
      return l_pd;
    } catch (IntrospectionException e) {
      new ProcessError("In ACSpreFactoryImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In ACSpreFactoryImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

//  public int getDefaultPropertyIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultPropertyIndex() not yet implemented.");
//  }
//  public MethodDescriptor[] getMethodDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getMethodDescriptors() not yet implemented.");
//  }
//  public BeanInfo[] getAdditionalBeanInfo() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getAdditionalBeanInfo() not yet implemented.");
//  }
//  public Image getIcon(int iconKind) {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getIcon() not yet implemented.");
//  }


    /**
     * Get the instance, if it exists, which fires an event when
     * the bean is closed or deleted. No used here.
     * @return a BaseManager object.
     */
    public BaseBeanManager getBaseBeanManagerInstance() {
      return BaseManagerImpl.getInstance();
    }

    /**
     * Get the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, if it exists,
     * which fires an event when the bean instance is closed or deleted.
     * @return null
     */
    public Method getBaseBeanManagerGetMethod() {
      return null;
    }

    /**
     * Get the property name for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * @return "openACS"
     */
    public String getBaseBeanManagerBoundProperty() {
      return "";
    }


} // ACSpreFactoryImplBeanInfo
