//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.BaseBeanManager;
import ARoad0.gBaseInterface.AclEntry;
import ARoad0.gBaseInterface.StringRight;
import ARoad0.gBase.*;
import java.beans.SimpleBeanInfo;
import java.beans.BeanDescriptor;
import ARoad0.gBaseInterface.ImmutableGroupID;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.lang.reflect.Method;

import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the AclEntryImpl class.
 * It is used for displaying and editing the properties of AclEntryImpl.
 * The closing and deleting operations are managed by the resource which
 * controls the AclEntry, through the 'AclEntries' bound property.
 * This BeanInfo is one of the most complex in gBaseBeanInfo.
 * @version 0.7.0
 */
public class AclEntryImplBeanInfo extends SimpleBeanInfo
  implements BeanInfoPattern {

  public AclEntryImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of AclEntryImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(AclEntry.class );
    result.setDisplayName("ACL");

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of AclEntryImpl
    result.setValue("withTabbedPanes", Boolean.TRUE );

    // 'commonBeanClass' is the class which has the bean info
    // defining getCommonBeanDescriptor(),
    // and then which is the class or the superclass of the bean.
    result.setValue("commonBeanClass", AclEntryImpl.class );
    
    // ----------------------- delete parameters ---------------------------
    
    // 'withDeleteMethod' is true to display the 'delete' button in the beamer
    result.setValue("withDeleteMethod", Boolean.TRUE );

    // When the bean is not a FinalizedObject,
    // 'externalFinalizeProperty' is the string name of the property, if it exists,
    // under which the finalize method for this object is described
    // in the BeanInfo of the BaseBeanManager
    // (see getBaseBeanManagerInstance() and getBaseBeanManagerGetMethod() ).
    result.setValue("externalFinalizeProperty", new String[]{"AclRights"} );

    // ----------------------- constructor parameters ---------------------------
    
    // 'constrArgumentPropertyNames' is the list of property names
    // for the class constructor or the add method
    result.setValue("constrArgumentPropertyNames", new String[]{
      "ACS", "EligibleParty", "AclRights", "PositiveRight", "ConditionSourceGroups"} );

    // 'constrArgumentFromSelect' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "constrArgumentPropertyNames", or, as here,
    // in the order of the constructor method (in ResourceImpl)
    result.setValue("constrArgumentFromSelect", new Boolean[]{
      Boolean.TRUE, Boolean.TRUE, Boolean.FALSE, Boolean.FALSE, Boolean.TRUE} );

    // 'constrArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another constructor argument,
    // what is this another argument which provides the first part of the name.
    result.setValue("constrArgumentConcatenedNameSource", new Integer[]{
      null, null, null, null, null} );

    // 'constrNullArgumentPossible' indicates if an argument may be null in the constructor
    result.setValue("constrNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.TRUE} );

    // 'constrOptionalArgument' indicates if an argument is optional in the
    // call to the constructor when it is null.
    result.setValue("constrOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'constrRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names. Is optional.
    // If it is true, the array of restricted values for the argument is searched
    // through BaseUtilityImpl.getRestrictedValues():
    // - first, in 3 ACS maps which define the constraints from the ACS,
    // and then "constrArgumentPropertyNames" must contain the property descriptor name
    // The name words may be separated by spaces in "constrArgumentPropertyNames",
    // although there is no separation in the ACS maps.
    // - second, in the value "staticSetOfItems" of the property in this BeanInfo.
    result.setValue("constrRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE, Boolean.FALSE, Boolean.FALSE} );
    
    // Only if 'constrNonVisibleArgument' is true for an argument,
    // 'constrRestrictedFromOtherArgumentMessage' may be null to give to the user
    // a message, when he needs to enter another argument to allow the definition
    // of the constraints for this argument.
    // Exemple of message: "<Set the ACS first>" from the value 'ACS'.
    result.setValue("constrRestrictedFromOtherArgumentMessage", new String[]{
      null, null, "ACS", null, null} );
    
    // ----------------------- standard parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    result.setHidden(false);
    result.setExpert(false);

    return result;
  } // getBeanDescriptor()

//  public EventSetDescriptor[] getEventSetDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getEventSetDescriptors() not yet implemented.");
//  }
//  public int getDefaultEventIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultEventIndex() not yet implemented.");
//  }


  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue():<P>
   * - 'tabbedPaneName' is the name (String value) of the pane
   *    where this property has to be displayed in the beamer.
   *    Is "" if 'withTabbedPanes' is false in the BeanDescriptor.
   * - 'panelGroupName' is the name (String value) of the group
   *    where this property has to be displayed with other related properties.
   *    May be "".<P>
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pd;

    try {

      // ------------------- property "EligibleParty" ------------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "EligibleParty", AclEntryImpl.class, "getEorEP", null);
      pd_two.setDisplayName("Right User (RU)");
      //pd_two.setShortDescription("   ");
      pd_two.setBound(false);
      //pd_two.setConstrained(false);
      //pd_two.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_two.setValue("panelGroupName", "  User and target  " );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_two.setValue("withViewButton", Boolean.TRUE );
      
      // ------------------- property "Resource" ------------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "Resource", AclEntryImpl.class, "getEorResource", null);
      pd_four.setDisplayName("Access Target");
      //pd_four.setShortDescription("   ");
      pd_four.setBound(false);
      //pd_four.setConstrained(false);
      //pd_four.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_four.setValue("panelGroupName", "  User and target  " );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_four.setValue("withViewButton", Boolean.TRUE );
      
      // ------------------- property "PositiveRight" --------------------
      PropertyDescriptor pd_eight = new PropertyDescriptor(
        "PositiveRight", AclEntryImpl.class, "getPositiveRight", null);
      pd_eight.setDisplayName("Rights sens");
      pd_eight.setShortDescription("True if the rights grant "+
        "the access to the target, and false if they deny it.");
      pd_eight.setBound(false);
      //pd_eight.setConstrained(false);
      //pd_eight.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eight.setValue("tabbedPaneName", "  ACL  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eight.setValue("panelGroupName", "  Rights  ");
      // 'tags' is the name (String value) of the couple of strings,
      // with a size limited to 12 caracters,
      // to be displayed when the property is true or false, in this order.
      // Cannot be empty if it exists, and the default value for the property
      // editor is ("yes", "no") when 'tags' does not exist.
      pd_eight.setValue("tags", new String[] {"Grant", "Deny"} );

      // ------------------- property "Rights" --------------------
      // the property descriptor name has to be the argument name in the constructor: AclRights
      PropertyDescriptor pd_seven = new PropertyDescriptor(
        "AclRights", AclEntryImpl.class, "getNoConditionalRight", "setL_RightsForUser");
      pd_seven.setDisplayName("List of rights");
      pd_seven.setShortDescription("The granting or denying"+
        " rights on the access target - effective rights if not conditional.");
      pd_seven.setBound(true);
      //pd_seven.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_seven.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_seven.setValue("valueTypeInGroup", new Object[] {StringRight.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seven.setValue("tabbedPaneName", "  ACL  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seven.setValue("panelGroupName", "  Rights  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when is is not applicable.
        //pd_seven.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. 
      pd_seven.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_seven.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_seven.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select' with there is no 'Clear'
      // when the property is an external object reference.
      pd_seven.setValue("withRemoveButton", Boolean.TRUE );
       // 'isRestrictedArgument' is true if it is a constrained property that is not
      // in the constructor (else, see 'constrRestrictedArgument' in the BeanDescriptor).
      // The restricted values are get from 'staticSetOfItems', for Strings only,
      // secondly from 'choiceMethodForNew', or through a search in the ACS by the PropertyEditor
      // like here.
      pd_seven.setValue("isRestrictedArgument", Boolean.TRUE );
      
      // ------------------- property "ACS" --------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "ACS", AclEntryImpl.class, "getEorACS", null);
      pd_one.setDisplayName("ACL manager");
      pd_one.setShortDescription("This ACS is not always the"+
        " access target ACS or the rights user ACS.");
      pd_one.setBound(false);
      //pd_one.setConstrained(false);
      //pd_one.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_one.setValue("panelGroupName", "  ACS attributes  ");
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_one.setValue("withViewButton", Boolean.TRUE );
      
      // ------------------- property "NameOfEpACS" --------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "NameOfEpACS", AclEntryImpl.class, "getNameOfEpACS", null);
      pd_three.setDisplayName("Right User manager");
      pd_three.setShortDescription("This ACS is not always the"+
        " access target or the aclEntry ACS.");
      pd_three.setBound(false);
      //pd_three.setConstrained(false);
      //pd_three.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  General  ");
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_three.setValue("panelGroupName", "  ACS attributes  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_three.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "NameOfResACS" --------------------
      PropertyDescriptor pd_five = new PropertyDescriptor(
        "NameOfResACS", AclEntryImpl.class, "getNameOfResACS", null);
      pd_five.setDisplayName("Target manager");
      pd_five.setShortDescription("This ACS is not always the"+
        " rights user ACS or the aclEntry ACS.");
      pd_five.setBound(false);
      //pd_five.setConstrained(false);
      //pd_five.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_five.setValue("tabbedPaneName", "  General  ");
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_five.setValue("panelGroupName", "  ACS attributes  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_five.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "Comment" --------------------
      PropertyDescriptor pd_six = new PropertyDescriptor(
        "Comment", AclEntryImpl.class, "getComment", "setComment");
      pd_six.setDisplayName("Comment");
      pd_six.setShortDescription("user comments");
      pd_six.setBound(true);
      //pd_six.setConstrained(false);
      //pd_six.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_six.setValue("tabbedPaneName", "  General  ");
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_six.setValue("panelGroupName", "  Main attributes  ");

      // ------------------- property "ConditionSourceGroups" --------------------
      PropertyDescriptor pd_ten = new PropertyDescriptor(
        "ConditionSourceGroups", AclEntryImpl.class, "getL_ConditionSourceGroups", null);

      pd_ten.setDisplayName("Condition groups");
      pd_ten.setShortDescription("If this list is not empty, to enable the rights,"+
              " the Rights User has to be (in)direct member of each group.");
      pd_ten.setBound(true);
      //pd_ten.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_ten.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_ten.setValue("valueTypeInGroup", new Object[] {ImmutableGroupID.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ten.setValue("tabbedPaneName", "  ACL  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ten.setValue("panelGroupName", "   The rights are "+
              "disabled if the right user is not (in)direct member of each group   " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_fifty.setValue("panelGroupName", "  ACS Allowed Rights  ");
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_ten.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_ten.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'Add New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_ten.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the RightMapPropertyEditor
      // has to display 2 'Select' buttons if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_ten.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the RightMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_ten.setValue("withRemoveButton", Boolean.TRUE );
        //pd_seventyOne.setValue("withDeleteButton", Boolean.FALSE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the fixed argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Used by SimplePropertyEditor.getFixedValuesForNewElement(), which
      // may use a subset of the returned arguments if the class of the object to create
      // waits a smaller set of arguments.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
          //pd_ten.setValue("withKnownArgumentsForNew", new Object[] {null, null} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property.
      // As it is a RightMapPropertyEditor, the arguments has to be a String for the key,
      // and a StringRight array for the value.
      // The returned value defines the name of the add method used both for
      // defining a new key and some new rights for an existing key.
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_ten.setValue("addMethod", "addForUserConditionSourceGroup");
      // 'removeMethod' defines the 'remove' method for an element of the property.
      // As it is a MapPropertyEditor, the removeMethod arguments are the addMethod arguments.
      // These elements allow the PropertyEditor to call the remove method
      // of the property, both for removing a key and then all its values, or
      // for removing some StringRights of a key.
      pd_ten.setValue("removeMethod", "removeForUserConditionSourceGroup");
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_seventyOne.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      pd_ten.setValue("limitedSizeForPresentation", new Integer(5));
      pd_ten.setValue("MaxSize", new Integer(4));
      
      // ------------------- property "EnabledRights" --------------------
      PropertyDescriptor pd_nine = new PropertyDescriptor(
        "EnabledRights", AclEntryImpl.class, "getEnabledRights", null);
      pd_nine.setDisplayName("Enabled rights");
      pd_nine.setShortDescription(
              "False only for a conditional ACL, if the right user is not an actor"+
              " and if it is not member of a condition group.");
      pd_nine.setBound(true);
      //pd_nine.setConstrained(false);
      //pd_nine.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_nine.setValue("tabbedPaneName", "  ACL  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_nine.setValue("panelGroupName", "  Rights  ");
      // 'tags' is the name (String value) of the couple of strings,
      // with a size limited to 12 caracters,
      // to be displayed when the property is true or false, in this order.
      // Cannot be empty if it exists, and the default value for the property
      // editor is ("yes", "no") when 'tags' does not exist.
      //pd_nine.setValue("tags", new String[] {"Grant", "Deny"} );
      
      //
      //   ------------ End of properties definition ----------------------------
      //

      l_pd = new PropertyDescriptor[]
        {pd_two, pd_four, pd_eight, pd_nine, pd_seven, pd_one, pd_three, pd_five, pd_six, pd_ten };
      return l_pd;
    } catch (IntrospectionException e) {
      new ProcessError("In AclEntryImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In AclEntryImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

//  public int getDefaultPropertyIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultPropertyIndex() not yet implemented.");
//  }
//  public MethodDescriptor[] getMethodDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getMethodDescriptors() not yet implemented.");
//  }
//  public BeanInfo[] getAdditionalBeanInfo() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getAdditionalBeanInfo() not yet implemented.");
//  }
//  public Image getIcon(int iconKind) {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getIcon() not yet implemented.");
//  }


    /**
     * Get the instance, if it exists, which fires an event when
     * the bean is closed or deleted.
     * @return null.
     */
    public BaseBeanManager getBaseBeanManagerInstance() {
      return null;
    }

    /**
     * Get the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, that is the Resource here,
     * which fires an event when the AclEntry is closed or deleted.
     * @return the get method which provides the resource
     *  that manages the AclEntry.
     */
    public Method getBaseBeanManagerGetMethod() {
      Object result = new AclEntryImpl(); // nonsens init
      try {
        result = AclEntryImpl.class.getMethod("getEorResource", (Class<?>[])null);
      } catch (NoSuchMethodException n) {
        new ProcessError("in AclEntryImplBeanInfo."+
          "getBaseBeanManagerGetMethod(): "+n.toString());
      } catch (SecurityException s) {
      }
      return (Method) result;
    }


    /**
     * Get the property name 'AclRights' for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * This is used to register the BeamerManager as a PropertyChangeListener.
     * @return the property name.
     */
    public String getBaseBeanManagerBoundProperty() {
      return new String("AclRights");
    }


} // AclEntryImplBeanInfo