//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.UserID;
import ARoad0.gBaseInterface.Directory;
import ARoad0.gBaseInterface.ACS;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.ImmutableActor;
import ARoad0.gBaseInterface.Actor;
import ARoad0.gBase.*;
import ARoad0.gBaseInterface.ImmutableResource;
import ARoad0.gBaseInterface.PrivilegeForType;
import ARoad0.gBaseInterface.PrivilegeForLinks;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the ActorImpl class.
 * It is used for displaying and editing the properties of the class.
 * Caution: any change must be copied in EligiblePartyImplBeanInfo.
 */
public class ActorImplBeanInfo extends ResourceImplBeanInfo {

  public ActorImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor.
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(Actor.class );
    result.setDisplayName("Actor");

    result = getCommonBeanDescriptor(result);

    return result;
  } // getBeanDescriptor()


  /**
   * Get the common description of the bean.
   * Used by subclasses. Reuse super.getCommonBeanDescriptor().
   * @param _result is the BeanDescriptor to configure
   * @return the bean descriptor
   */
  protected BeanDescriptor getCommonBeanDescriptor(BeanDescriptor _result) {

    super.getCommonBeanDescriptor(_result);
    
    // ----------------------- constructor parameters ---------------------------
    
    // 'constrArgumentClasses' is the list of argument classes
    // for the class constructor
    _result.setValue("constrArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class, Directory.class, UserID.class, String.class,
      ImmutableActor.class} );

    // 'constrArgumentPropertyNames' is the list of property names
    // for the class constructor.
    _result.setValue("constrArgumentPropertyNames", new String[]{
      "Name", "ACS", "Parent Node", "UserID", "Resource Type", 
      "ACS Bridge Target"} );

    // 'constrArgumentFromSelect' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "constrArgumentPropertyNames", or, as here,
    // in the order of the constructor method in ResourceImpl
    _result.setValue("constrArgumentFromSelect", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.TRUE, Boolean.TRUE, Boolean.FALSE,
      Boolean.TRUE} );

    // 'constrArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another constructor argument,
    // what is this another argument which provides the first part of the name
    // if it is not null.
    _result.setValue("constrArgumentConcatenedNameSource", new Integer[]{
      new Integer(2), null, null, null, null, null} );

    // 'constrNullArgumentPossible' indicates if an argument may be null
    // in the constructor
    _result.setValue("constrNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE, Boolean.TRUE, Boolean.FALSE,
      Boolean.TRUE} );

    // 'constrOptionalArgument' indicates if an argument must not be used in the
    // call to the constructor when it is null.
    _result.setValue("constrOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE,
      Boolean.FALSE} );

    // 'constrRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names. Is optional.
    // If it is true, the array of restricted values for an argument is searched:
    // - first, in the ACS.getM_staticSetOfItems() map,
    // and then "constrArgumentPropertyNames" must contain the property descriptor name.
    // The name words may be separated by spaces in "constrArgumentPropertyNames",
    // although there is no separation in the ACS.getM_staticSetOfItems() map.
    // - second, in the value "staticSetOfItems" of the property in this BeanInfo.
    _result.setValue("constrRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.TRUE,
      Boolean.FALSE} );

    // 'constrNonVisibleArgument' indicates if an argument is masqued
    // in the constructor window and sets to null in the constructor.
    // True when the ACS does not manage the property, and then, 
    // this value gets the name of the ACS method which returns a boolean 'false'
    // when the argument has to be null and to be not displayed. If
    // the returned value is 'true', the argument is displayed as usual.
    // For each non null value:
    // "constrNullArgumentPossible" has to be Boolean.TRUE,
    // "constrOptionalArgument" has to be Boolean.FALSE.
    _result.setValue("constrNonVisibleArgument", new String[]{
      null, null, "getManageResourceTrees", "getManageAccounts", null, 
      "getManageBridges"} );    
    
    // ----------------------- copy parameters ---------------------------

      
    return _result;
  } // getCommonBeanDescriptor()

//  public EventSetDescriptor[] getEventSetDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getEventSetDescriptors() not yet implemented.");
//  }
//  public int getDefaultEventIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultEventIndex() not yet implemented.");
//  }


  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue().
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;

    try {
      // ------------------- property "AclConnectedResources" --------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "AclConnectedResources", ActorImpl.class, "getEorL_AclConnectedResources", null);
      pd_one.setDisplayName("Resources in ACL");
      pd_one.setShortDescription("The internal and external resources connected "+
        "through aclEntries.");
      pd_one.setBound(true);
        //pd_one.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_one.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_one.setValue("valueTypeInGroup", new Object[] {ImmutableResource.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  As Actor  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_one.setValue("panelGroupName", "  C. In the ACL, as RIGHT USER  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_one.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_one.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_one.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_one.setValue("withDeleteButton", Boolean.FALSE );

      // ------------------- property "SecondaryGroups" --------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "SecondaryGroups", ActorImpl.class, "getEorL_SecondaryGroup",
        "setEorL_SecondaryGroupForUser");
      pd_two.setDisplayName("Secondary groups");
      pd_two.setShortDescription("The secondary groupIDs which have this actor"+
      " as member through its current account.");
      pd_two.setBound(true);
      //pd_two.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_two.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_two.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  As Actor  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_two.setValue("panelGroupName", "  B. AG context as Actor  " );
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_two.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_two.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_two.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_two.setValue("withRemoveButton", Boolean.TRUE );

      // ------------------- property "l_bridgeSources" --------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "l_bridgeSources", ActorImpl.class, "getEorL_bridgeSources", null);
      pd_three.setDisplayName("Is bridge target for");
      pd_three.setShortDescription("The other-ACS entities which are connected to"+
              " this ACS through this entity.");
      pd_three.setBound(true);
      //pd_three.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_three.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_three.setValue("valueTypeInGroup", new Object[] {ImmutableActor.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  As Actor  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_three.setValue("panelGroupName", "  D. Bridge Links to other ACS  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_three.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_three.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_three.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_three.setValue("withDeleteButton", Boolean.FALSE );

      /*
      // ------------------- property "l_isAGateFor" --------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "l_isAGateFor", ActorImpl.class, "getEorL_isAGateFor", "setEorL_isAGateFor");
      pd_four.setDisplayName("Is a gate for actors");
      pd_four.setShortDescription("A gate is a kind of proxy used by others actors.");
      pd_four.setBound(true);
      //pd_four.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_four.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_four.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  As Actor  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_four.setValue("panelGroupName", "  Actor Links  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_four.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_four.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_four.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_four.setValue("withDeleteButton", Boolean.TRUE );
      */

      // ------------------- property "CurrentUserID" ------------------------
      PropertyDescriptor pd_five = new PropertyDescriptor(
        "CurrentUserID", ActorImpl.class, "getEorCurrentUserID", "setEorCurrentUserIDForUser");
      pd_five.setDisplayName("Started from account");
      pd_five.setShortDescription("The actor is started by the ACS under this current account"+
        ", without link to the actor owner.");
      pd_five.setBound(true);
      //pd_five.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_five.setValue("tabbedPaneName", "  As Actor  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_five.setValue("panelGroupName", "  B. AG context as Actor  " );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_five.setValue("withViewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_five.setValue("withSelectButton", Boolean.TRUE );
      // 'withClearButton' is the boolean which set if the PropertyEditor
      // has to display a Clear button
      pd_five.setValue("withClearButton", Boolean.TRUE );
      
      // ------------------- property "CurrentGroupID" ------------------------
      PropertyDescriptor pd_six = new PropertyDescriptor(
        "CurrentGroupID", ActorImpl.class, "getEorCurrentGroupID", "setEorCurrentGroupIDForUser");
      pd_six.setDisplayName("Started from group");
      pd_six.setShortDescription("This actor is started under this current groupID, "+
              "sometimes as the primary group of the current userID.");
      pd_six.setBound(true);
      //pd_six.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_six.setValue("tabbedPaneName", "  As Actor  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_six.setValue("panelGroupName", "  B. AG context as Actor  " );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_six.setValue("withViewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_six.setValue("withSelectButton", Boolean.TRUE );
      // 'withClearButton' is the boolean which set if the PropertyEditor
      // has to display a Clear button
      pd_six.setValue("withClearButton", Boolean.TRUE );

      // ------------------- property "ACSBridgeTarget" ------------------------
      PropertyDescriptor pd_seven = new PropertyDescriptor(
        "ACSBridgeTarget", ActorImpl.class, "getEorACSBridgeTarget", null);
      pd_seven.setDisplayName("Bridge target ");
      pd_seven.setShortDescription("An external actor through which this ACS is connected"+
              "to another one.");
      pd_seven.setBound(true);
      //pd_seven.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seven.setValue("tabbedPaneName", "  As Actor  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seven.setValue("panelGroupName", "  D. Bridge Links to other ACS  ");
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_seven.setValue("withViewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      //pd_seven.setValue("withSelectButton", Boolean.TRUE );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_seven.setValue("panelGroupName", "  Main attributes  " );
      // 'withClearButton' is the boolean which set if the PropertyEditor
      // has to display a Clear button
      //pd_seven.setValue("withClearButton", Boolean.TRUE );      
      
      /*
      // ------------------- property "isACSBridge" --------------------
      PropertyDescriptor pd_nine = new PropertyDescriptor(
        "isACSBridge", ActorImpl.class, "getIsACSBridge", null);
      pd_nine.setDisplayName("Is a bridge source");
      pd_nine.setShortDescription("true if the actor is a bridge to another ACS, "+
        " and then, the target is shown in the 'General' panel.");
      pd_nine.setBound(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_nine.setValue("tabbedPaneName", "  As Actor  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_nine.setValue("panelGroupName", "  Actor values  " );
       **/ 

      // ------------------- property "EPType" ------------------------
      PropertyDescriptor pd_ten = new PropertyDescriptor(
        "EPType", ActorImpl.class, "getEPType", "setEPType");
      pd_ten.setDisplayName("Right User type ");
      pd_ten.setShortDescription("This type is managed by the ACS, and it is also the resource type. ");
      pd_ten.setValue("tabbedPaneName", "  General  " );
      pd_ten.setBound(true);
       // 'isRestrictedArgument' is true if it is a constrained property that is not
      // in the constructor (else, see 'constrRestrictedArgument' in the BeanDescriptor).
      // The restricted values are get from 'staticSetOfItems', for Strings only,
      // secondly from 'choiceMethodForNew', or through a search in the ACS
      // by the PropertyEditor like here.
      pd_ten.setValue("isRestrictedArgument", Boolean.TRUE );

      // ------------------- property "IsRightProxy" ------------------------
      PropertyDescriptor pd_eleven = new PropertyDescriptor(
        "IsRightProxy", ActorImpl.class, "getIsRightProxy", "setIsRightProxy");
      pd_eleven.setDisplayName("Is a Right Proxy");
      pd_eleven.setShortDescription("False only if it is proved that "+
              "this Actor NEVER gives to its accessors, some rights on the targets it may access. ");
      pd_eleven.setValue("tabbedPaneName", "  As Actor  " );
      pd_eleven.setValue("panelGroupName", "  A. Main values (note: see also the 'Privileges as RIGHT USER')  " );
      pd_eleven.setBound(true);
      
      // ------------------- property "OneSourceTypedPrivilege" --------------------
      PropertyDescriptor pd_twelve = new PropertyDescriptor(
        "OneSourceTypedPrivilege", ActorImpl.class,
        "getEorL_SourcePrivilegeForTypes", null);
      // caution: this descriptor name is used in BeamerManager to forbid the
      // Select and Remove buttons
      pd_twelve.setDisplayName("Typed Privileges");
      pd_twelve.setShortDescription("The typed privileges of this privilege source.");
      pd_twelve.setBound(true);
      //pd_twelve.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_twelve.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      // as DetailledName, is used only in a CollectionPropertyEditor
      // to display the detailled name rather than the name.
      pd_twelve.setValue("valueTypeInGroup", new Object[] {PrivilegeForType.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twelve.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_twelve.setValue("panelGroupName", "  B. Privileges as RIGHT USER  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
        //pd_twelve.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_twelve.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_twelve.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'. 'Select' implies 'Remove'.
      pd_twelve.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes automatically with 'Select'
      // when the property is an external object reference.
      pd_twelve.setValue("withRemoveButton", Boolean.FALSE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be preset - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the known argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
        //pd_seven.setValue("withKnownArgumentsForNew", new Object[] {
          //null, null, null, null, "this"} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_twelve.setValue("addMethod", "addEorPrivilegeForLinksToTarget");
          //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty", "Rights"} );
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        //pd_twelve.setValue("removeMethod", "removeEorPrivilegeToTarget");
            //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty"} );
      // Following the KeyPropertiesInName pattern,
      // 'removeArgumentsFromName' defines how to find the right arguments for
      // the 'remove' method for the property, when the name of the object
      // to remove is known, and when finalizeForUser() is not used.
      // If "removeArgumentsFromName" is null, the remove method is invoked
      // with the object to remove as single argument.
      // The returned array defines:
      // - the name of the class to call, at the index 0
      // - the name of the static method to call, at the index 1, with
      //    the name of the object to remove as argument, and with
      //    an Object[] array as returned value
      // - in the Object[] array returned as value by the static method,
      //    the sequence of index where are the n arguments to use
      //    in the remove method, in the correct order
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        // pd_twelve.setValue("removeArgumentsFromName", ... );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_twelve.setValue("limitedSizeForPresentation", new Integer(5));
      
      // ------------------- property "OneSourceLinkedPrivilege" --------------------
      PropertyDescriptor pd_thirteen = new PropertyDescriptor(
        "OneSourceLinkedPrivilege", ActorImpl.class,
        "getEorL_SourcePrivilegeForLinks", null);
      // caution: this descriptor name is used in BeamerManager to forbid the
      // Select and Remove buttons
      pd_thirteen.setDisplayName("Linked Privileges");
      pd_thirteen.setShortDescription("The linked privileges of this privilege source.");
      pd_thirteen.setBound(true);
      //pd_thirteen.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_thirteen.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      // as DetailledName, is used only in a CollectionPropertyEditor
      // to display the detailled name rather than the name.
      pd_thirteen.setValue("valueTypeInGroup", new Object[] {PrivilegeForLinks.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirteen.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_thirteen.setValue("panelGroupName", "  B. Privileges as RIGHT USER  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_thirteen.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_thirteen.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_thirteen.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'. 'Select' implies 'Remove'.
        //pd_thirteen.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes automatically with 'Select'
      // when the property is an external object reference.
        //pd_thirteen.setValue("withRemoveButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be preset - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the known argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
        //pd_thirteen.setValue("withKnownArgumentsForNew", new Object[] {
          //null, null, null, null, "this"} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_thirteen.setValue("addMethod", "addEorPrivilegeForLinksToSource");
          //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty", "Rights"} );
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        //pd_thirteen.setValue("removeMethod", "removeEorPrivilegeToSource");
            //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty"} );
      // Following the KeyPropertiesInName pattern,
      // 'removeArgumentsFromName' defines how to find the right arguments for
      // the 'remove' method for the property, when the name of the object
      // to remove is known, and when finalizeForUser() is not used.
      // If "removeArgumentsFromName" is null, the remove method is invoked
      // with the object to remove as single argument.
      // The returned array defines:
      // - the name of the class to call, at the index 0
      // - the name of the static method to call, at the index 1, with
      //    the name of the object to remove as argument, and with
      //    an Object[] array as returned value
      // - in the Object[] array returned as value by the static method,
      //    the sequence of index where are the n arguments to use
      //    in the remove method, in the correct order
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        // pd_thirteen.setValue("removeArgumentsFromName", ... );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_thirteen.setValue("limitedSizeForPresentation", new Integer(5));
      
      //
      // ------------End of ActorImpl properties definition --------------------
      //

      // start from the parent class bean info
      l_pdFromParent = super.getPropertyDescriptors();
      l_result = new PropertyDescriptor[l_pdFromParent.length + 10];
      System.arraycopy(l_pdFromParent, 0, l_result, 0, l_pdFromParent.length);   
      l_result[l_result.length - 10] = pd_twelve;
      l_result[l_result.length - 9] = pd_thirteen;
      l_result[l_result.length - 8] = pd_eleven;
      l_result[l_result.length - 7] = pd_seven;
      l_result[l_result.length - 6] = pd_ten;
      //l_result[l_result.length - 7] = pd_nine;     
      l_result[l_result.length - 5] = pd_one;
      l_result[l_result.length - 4] = pd_five;
      //l_result[l_result.length - 3] = pd_four;
      l_result[l_result.length - 3] = pd_six;
      l_result[l_result.length - 2] = pd_two;
      l_result[l_result.length - 1] = pd_three;

      l_pdFromParent = null;
      return l_result;
    } catch (IntrospectionException e) {
      new ProcessError("In ActorImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In ActorImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

} // ActorImplBeanInfo