//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.*;

import ARoad0.gBase.*;

import java.beans.SimpleBeanInfo;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.lang.reflect.Method;

import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the BasicImpl class.
 * It is used for displaying and editing the properties,
 * to create it and to copy it.
 * @version 0.7.0
 */
public class BasicImplBeanInfo extends SimpleBeanInfo
  implements BeanInfoPattern {

  public BasicImplBeanInfo() {}

  /**
   * Gets the description of the bean.
   * The properties to handle in the GUI are set in the descriptor.
   * They are get through BeanDescriptor.getValue().
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
      // used by BaseUtilityImpl.getSuperInterfacesFromBeanInfo()
    BeanDescriptor result = new BeanDescriptor(Basic.class );
    result.setDisplayName("Basic");

    return getCommonBeanDescriptor(result);
  } // getBeanDescriptor()

  /**
   * Gets the common description of the bean.
   * Overriden by the subclasses.
   * @param _result is the BeanDescriptor to configure
   * @return the bean descriptor
   */
  protected BeanDescriptor getCommonBeanDescriptor(BeanDescriptor _result) {

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of BasicImpl
    _result.setValue("withTabbedPanes", Boolean.TRUE );

    // 'withDeleteMethod' is true to display the 'delete' button in the beamer
    _result.setValue("withDeleteMethod", Boolean.TRUE );

    // 'commonBeanClass' is the class which has the bean info defining getCommonBeanDescriptor(),
    // and then which is the class or the superclass of the bean
    //_result.setValue("commonBeanClass", BasicImpl.class );

    // ----------------------- constructor parameters ---------------------------

    // 'constrArgumentClasses' is the list of argument classes
    // for the class constructor
    _result.setValue("constrArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class} );

    // 'constrArgumentPropertyNames' is the list of property names
    // for the class constructor
    _result.setValue("constrArgumentPropertyNames", new String[]{
      "Name", "ACS"} );

    // 'constrArgumentFromSelect' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "constrArgumentPropertyNames", or, as here,
    // in the order of the constructor method in BasicImpl
    _result.setValue("constrArgumentFromSelect", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE} );

    // 'constrArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another constructor argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    _result.setValue("constrArgumentConcatenedNameSource", new Integer[]{
      null, null} );

    // 'constrNullArgumentPossible' indicates if an argument may be null in the constructor
    _result.setValue("constrNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE} );

    // 'constrOptionalArgument' indicates if an argument must not be used in the
    // call to the constructor when it is null.
    _result.setValue("constrOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE} );

    // 'constrRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names. Is optional.
    // If it is true, the array of restricted values for an argument is searched
    // through BaseUtilityImpl.getRestrictedValues():
    // - first, in 3 ACS maps which define the constraints from the ACS,
    // and then "constrArgumentPropertyNames" must contain the property descriptor name.
    // The name words may be separated by spaces in "constrArgumentPropertyNames",
    // although there is no separation in the ACS map.
    // - second, in the value "staticSetOfItems" of the property in this BeanInfo.
    _result.setValue("constrRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE} );

    // 'constrNonVisibleArgument' indicates if an argument is masqued
    // in the constructor window and sets to null in the constructor.
    // True when the ACS does not manage the property, and then,
    // this value gets the name of the ACS method which returns a boolean 'false'
    // when the argument has to be null and to be not displayed. If
    // the returned value is 'true', the argument is displayed as usual.
    // For each non null value:
    // "constrNullArgumentPossible" has to be Boolean.TRUE,
    // "constrOptionalArgument" has to be Boolean.FALSE.
    _result.setValue("constrNonVisibleArgument", new String[]{
      null, null} );

    // ----------------------- copy parameters ---------------------------

    // 'copyArgumentClasses' is the list of argument classes
    // for the copy method, excepted for _toRegister at the end,
    //  which is another argument set to true.
    _result.setValue("copyArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class} );

    // 'copyArgumentPropertyNames' is the list of property names
    // for the copy method, excepted for _toRegister which is set to true
    // following the copy() pattern.
    _result.setValue("copyArgumentPropertyNames", new String[]{
      "Name", "ACS"} );

    // 'copyArgumentFromExplorer' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "copyArgumentPropertyNames"
    _result.setValue("copyArgumentFromExplorer", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE} );

    // 'copyArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    _result.setValue("copyArgumentConcatenedNameSource", new Integer[]{
      null, null} );

    // 'copyNullArgumentPossible' indicates if an argument may be null
    //  in the method
    _result.setValue("copyNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE} );

    // 'copyOptionalArgument' indicates if an argument must not be used
    //  in the call to the method when it is null.
    _result.setValue("copyOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE} );

    // 'copyRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names.
    //  Is optional.
    // If it is true, the array of restricted values for an argument is searched:
    // - first, in the value "staticSetOfItems" of the property in this BeanInfo,
    // - second, in the ACS.getM_staticSetOfItems() map
    // and then "constrArgumentPropertyNames" must contain the property descriptor name
    // (but the name words may be separated by spaces in "constrArgumentPropertyNames").
    _result.setValue("copyRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE} );

    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    _result.setHidden(false);
    _result.setExpert(true);

    return _result;
  } // getCommonBeanDescriptor()



//  public EventSetDescriptor[] getEventSetDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getEventSetDescriptors() not yet implemented.");
//  }
//  public int getDefaultEventIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultEventIndex() not yet implemented.");
//  }


  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue(). 'Type' is not described since it is
   * used by the subclasses.
   * To read a property, use the read method in the prooperty descriptor. <P>
   * To update a property, there are 3 means, in this order: <P>
   * - if it is not a single value, use the "choiceMethodForNew" value to add one value <P>
   * - or, if it is not a single value, use the "addMethod" value to add one value
   *  and "removeMethod", for BaseObjects only, <P>
   * - or, in any cases, use the write method in the property descriptor
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pd;

    try {
      // ------------------- property "Name" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "Name", BasicImpl.class, "getName", null);
      pd_one.setDisplayName("Name");
      pd_one.setShortDescription("The unique name of this element in Access Road ");
      //pd_one.setBound(false);
      //pd_one.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
            //pd_one.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_one.setValue("BaseBeanManager", );

      // ------------------- property "ACS" ------------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "ACS", BasicImpl.class, "getEorACS", null);
      pd_two.setDisplayName("ACS");
      pd_two.setShortDescription("The access control system which manages this element");
      //pd_two.setBound(false);
      //pd_two.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_two.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_two.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_two.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "Comment" --------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "Comment", BasicImpl.class, "getComment", "setComment");
      pd_four.setDisplayName("Comments");
      pd_four.setShortDescription("free user comments");
      pd_four.setBound(true);
      //pd_four.setConstrained(false);
      //pd_four.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_four.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_four.setValue("BaseBeanManager", );

      // ------------------- property "ImpliedOpenViews" ------------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "ImpliedOpenViews", BasicImpl.class, "getImpliedViews", null);
      pd_three.setDisplayName("Is in open views");
      pd_three.setShortDescription("These open views contain this object.");
      pd_three.setBound(true);
        //pd_three.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and the default value is SimplePropertyEditor.
      pd_three.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_three.setValue("valueTypeInGroup", new Object[] {ViewInBase.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      // caution: "panelGroupName" is superior to "tabbedPaneName" in the BeamerManager analysis.
        //pd_three.setValue("panelGroupName", "  Displayed in views  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_three.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_three.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_three.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_three.setValue("withDeleteButton", Boolean.FALSE );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      pd_three.setValue("limitedSizeForPresentation", new Integer(4));

      // ------------------- property "IS_Name" ------------------------
      PropertyDescriptor pd_five = new PropertyDescriptor(
        "IS_Name", BasicImpl.class, "getIS_Name", null);
      pd_five.setDisplayName("IS name");
      pd_five.setShortDescription("The Information System name");
      pd_five.setValue("tabbedPaneName", "  General  " );
      pd_five.setBound(true);
       // 'isRestrictedArgument' is true if it is a constrained property that is not
      // in the constructor (else, see 'constrRestrictedArgument' in the BeanDescriptor).
      // The restricted values are get from 'staticSetOfItems', for Strings only,
      // secondly from 'choiceMethodForNew', or through a search in the ACS by the PropertyEditor
      // like here.
      pd_five.setValue("isRestrictedArgument", Boolean.FALSE );

      // ------------------- property "VirtualFolders" ------------------------
      PropertyDescriptor pd_six = new PropertyDescriptor(
        "VirtualFolders", BasicImpl.class, "getVirtualFolders", null);
      pd_six.setDisplayName("Is in virtual folders");
      pd_six.setShortDescription("These virtual folders contain this object.");
      pd_six.setBound(true);
        //pd_six.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and the default value is SimplePropertyEditor.
      pd_six.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_six.setValue("valueTypeInGroup", new Object[] {VirtualFolder.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_six.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      // caution: "panelGroupName" is superior to "tabbedPaneName" in the BeamerManager analysis.
        //pd_six.setValue("panelGroupName", "  B - This object may be in virtual folders  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_six.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_six.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_six.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'Delete' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_six.setValue("withDeleteButton", Boolean.FALSE );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0, which allows, if it is alone,
      //      to have the right BeanInfo and its constructor description from the class
      //      of the return value of this add method,
                // - optionnaly, the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - optionnaly, the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to call BaseObjectHandler which
      // displays a dialog to enter the arguments and call the add method.
        //pd_six.setValue("addMethod", "addInVFolderList");
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        //pd_six.setValue("removeMethod", "removeInVFolderList");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      pd_six.setValue("limitedSizeForPresentation", new Integer(4));

      // ------------------- property "HardAliasReference" ------------------------
      PropertyDescriptor pd_twelve = new PropertyDescriptor(
        "HardAliasReference", BasicImpl.class,
        "getEorReferenceAsInternalHardAlias", null);
      pd_twelve.setDisplayName("The hard reference");
      pd_twelve.setShortDescription("The reference "+
              "of this object that is an hard alias");
      pd_twelve.setValue("tabbedPaneName", "  Alias  " );
      pd_twelve.setBound(true);

      // ------------------- property "HardAlias" ------------------------
      PropertyDescriptor pd_thirteen = new PropertyDescriptor(
        "HardAlias", BasicImpl.class,
        "getEorAliasAsInternalHardReference", null);
      pd_thirteen.setDisplayName("The hard alias");
      pd_thirteen.setShortDescription("The hard alias "+
              "of this object that is a reference");
      pd_thirteen.setValue("tabbedPaneName", "  Alias  " );
      pd_thirteen.setBound(true);

      // ------------------- property "SoftAlias" --------------------
      PropertyDescriptor pd_fifteen = new PropertyDescriptor(
        "SoftAlias", BasicImpl.class, "getL_SoftAlias", null);
      pd_fifteen.setDisplayName("Soft aliases");
      pd_fifteen.setShortDescription("The external soft aliases of this reference");
      pd_fifteen.setBound(true);
      pd_fifteen.setValue("isCollection", Boolean.TRUE );
      pd_fifteen.setValue("tabbedPaneName", "  Alias  " );
      pd_fifteen.setValue("withViewButton", Boolean.TRUE );
      pd_fifteen.setValue("withNewButton", Boolean.FALSE );
      pd_fifteen.setValue("withSelectButton", Boolean.FALSE );
      pd_fifteen.setValue("withDeleteButton", Boolean.FALSE );
      pd_fifteen.setValue("limitedSizeForPresentation", new Integer(5));


      // ------- End of properties description ---------------------------------

      // The order in l_pd defines the display order/
      // 'Type' property is not there, since it is masked by resource and eligible party types
      l_pd = new PropertyDescriptor[]
        {pd_one, pd_five, pd_two, pd_four,  // General
          pd_six, pd_three,            // 'VirtualFolders', 'OpenViews'
          pd_twelve, pd_thirteen, pd_fifteen // 'HardAliasReference', 'HardAlias', 'SoftAlias'
        };
      return l_pd;
    } catch (IntrospectionException e) {
      new ProcessError("In BasicImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In BasicImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

//  public int getDefaultPropertyIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultPropertyIndex() not yet implemented.");
//  }
//  public MethodDescriptor[] getMethodDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getMethodDescriptors() not yet implemented.");
//  }
//  public BeanInfo[] getAdditionalBeanInfo() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getAdditionalBeanInfo() not yet implemented.");
//  }
//  public Image getIcon(int iconKind) {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getIcon() not yet implemented.");
//  }

    /**
     * Get the instance, if it exists, which fires an event when
     * the bean is closed or deleted.
     * @return null.
     */
    public BaseBeanManager getBaseBeanManagerInstance() {
      return null;
    }

    /**
     * Get the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, if it exists,
     * which fires an event when the bean instance is closed or deleted.
     * @return the get method. Is null when there is
     *  an active BeanInfoPattern.getBaseBeanManagerInstance(),
     *  or when it is not applicable.
     */
    public Method getBaseBeanManagerGetMethod() {
      Object result = BasicImpl.EMPTY_INSTANCE; // non-sens init
      try {
        result = BasicImpl.class.getMethod("getEorACS", (Class<?>[])null);
      } catch (NoSuchMethodException n) {
        new ProcessError("in BasicImplBeanInfo."+
          "getBaseBeanManagerGetMethod(): "+n.toString());
      } catch (SecurityException s) {
      }
      return (Method) result;
    }


    /**
     * Get the property name for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * This is used to register the BeamerManager as a PropertyChangeListener.
     * @return null
     */
    public String getBaseBeanManagerBoundProperty() {
      return null;
    }


} // BasicImplBeanInfo
