//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBase.*;
import ARoad0.gBaseInterface.Directory;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

import ARoad0.ErrW.ProcessError;
import ARoad0.gDMak.ActionNewResource;

/**
 * This class is responsible for describing the DirectoryImpl class.
 * It is used for displaying and editing the properties of DirectoryImpl,
 * to create it and to copy it.
 */
public class DirectoryImplBeanInfo extends ResourceImplBeanInfo {

  public DirectoryImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor. There is no method to change
   * the class of a BeanDescriptor, so, to reuse the BeanInfo of the mother class,
   * the values are copied in a new BeanDescriptor.
   * They are get with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of ResourceImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(Directory.class );
    result.setDisplayName("Directory");

    result = getCommonBeanDescriptor(result);

    return result;
  } // getBeanDescriptor()

  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue().
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;

    try {
      // ------------------- property "Children" --------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "Children", DirectoryImpl.class, "getEorL_Children", null);
      pd_one.setDisplayName("Children");
      pd_one.setShortDescription("The direct children of the directory.");
      pd_one.setBound(true);
      //pd_one.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_one.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_one.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  As Directory  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_one.setValue("panelGroupName", "  List  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when is is not applicable.
          //pd_one.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. 
      pd_one.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_one.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_one.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_one.setValue("withDeleteButton", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke 
      // to let the user defines the new restricted key, in a map, or the new instance, 
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if 
      // there is an add method or a write method. With 'isRestrictedArgument', 
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke, or null if it is the source,
      // and the name of the method (for instance, 
      // ActionNewResource.createResource() or NameLinkTreeImpl.getVisibleSpaceConstants()).
      // The method has no argument.
      // The array is equal to 'new Object[1]' when the value 'withKnownArgumentsForNew' is null.
      pd_one.setValue("choiceMethodForNew", new Object[] {
        ActionNewResource.getInstance(), "createResource"} );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // The string 'this' is used to point to the object in the beamer.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property.
      // Caution: the known arguments have to be Objects.
      // Caution: the number of arguments may be superior.
      pd_one.setValue("withKnownArgumentsForNew", new Object[] {null, "ACS", "this"} );
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_one.setValue("removeMethod", "finalizeForUser");

      // ------------------- property "EnforceChildGID" --------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "EnforceChildGID", DirectoryImpl.class, "getEnforceChildGID", "setEnforceChildGID");
      pd_two.setDisplayName("Enforce child group");
      pd_two.setShortDescription("If true, this node enforces its group (but not the group rights)"+
              " for a new child, as setgid in *NX systems.");
      pd_two.setBound(true);
      //pd_two.setConstrained(false);
      //pd_two.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  As Directory  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_two.setValue("panelGroupName", "  Child management  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_two.setValue("BaseBeanManager", null);

      //
      // ------------End of DirectoryImpl properties definition --------------------
      //

      // start from the parent class bean info
      l_pdFromParent = super.getPropertyDescriptors();
      l_result = new PropertyDescriptor[l_pdFromParent.length + 2];
      System.arraycopy(l_pdFromParent, 0, l_result, 0, l_pdFromParent.length);
      l_result[l_result.length - 2] = pd_one;
      l_result[l_result.length - 1] = pd_two;

      l_pdFromParent = null;
      return l_result;
    } catch (IntrospectionException e) {
      new ProcessError("In DirectoryImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In DirectoryImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()


} // DirectoryImplBeanInfo