//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.EligibleParty;
import ARoad0.gBaseInterface.Directory;
import ARoad0.gBaseInterface.ACS;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.ImmutableACS;
import ARoad0.gBaseInterface.PrivilegeForType;
import ARoad0.gBaseInterface.PrivilegeForLinks;
import ARoad0.gBase.*;
import ARoad0.gBaseInterface.ImmutableResource;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the EligiblePartyImpl class.
 * It is used for displaying and editing the properties of EligiblePartyImpl.
 * Caution: any change must be copied in ActorImplBeanInfo.
 * @version 0.7.0
 * @since 0.5.1
 */
public class EligiblePartyImplBeanInfo extends BasicImplBeanInfo
  implements BeanInfoPattern {

  public EligiblePartyImplBeanInfo() {}

  /**
   * Gets the description of the bean.
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of EligiblePartyImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(EligibleParty.class );
    result.setDisplayName("Generic Eligible Party");

    result = getCommonBeanDescriptor(result);
      
    return result;
  } // getBeanDescriptor()

  /**
   * Gets the common description of the bean.
   * Not used by subclasses.
   * @param _result is the BeanDescriptor to configure
   * @return the bean descriptor
   */
  protected BeanDescriptor getCommonBeanDescriptor(BeanDescriptor _result) {

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value)
    _result.setValue("withTabbedPanes", Boolean.TRUE );

    // 'withDeleteMethod' is true to display the 'delete' button in the beamer
    _result.setValue("withDeleteMethod", Boolean.TRUE );

    // 'commonBeanClass' is the class which has the bean info defining getCommonBeanDescriptor(),
    // and then which is the class or a superclass of the bean
    _result.setValue("commonBeanClass", EligiblePartyImpl.class );
    
    // ----------------------- constructor parameters ---------------------------
    
    // 'constrArgumentClasses' is the list of argument classes
    // for the class constructor
    _result.setValue("constrArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class, String.class} );
    
    // 'constrArgumentPropertyNames' is the list of property names
    // for the class constructor, even when the new instance
    // is created through a constructor, and not through another class
    _result.setValue("constrArgumentPropertyNames", new String[]{
      "Name", "ACS", "EPType"} );

    // 'constrArgumentFromSelect' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "constrArgumentPropertyNames", or, as here,
    // in the order of the constructor method in ResourceImpl
    _result.setValue("constrArgumentFromSelect", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.FALSE} );

    // 'constrArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another constructor argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    _result.setValue("constrArgumentConcatenedNameSource", new Integer[]{
      new Integer(1), null, null} );

    // 'constrNullArgumentPossible' indicates if an argument may be null in the constructor
    _result.setValue("constrNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'constrOptionalArgument' indicates if an argument must not be used in the
    // call to the constructor when it is null.
    _result.setValue("constrOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'constrRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names. Is optional.
    // If it is true, the array of restricted values for an argument is searched
    // through BaseUtilityImpl.getRestrictedValues():
    // - first, in 3 ACS maps which define the constraints from the ACS,
    // and then "constrArgumentPropertyNames" must contain the property descriptor name.
    // The name words may be separated by spaces in "constrArgumentPropertyNames",
    // although there is no separation in the ACS map.
    // - second, in the value "staticSetOfItems" of the property in this BeanInfo.
    _result.setValue("constrRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE} );
      
    // ----------------------- copy parameters ---------------------------

    // 'copyArgumentClasses' is the list of argument classes
    // for the copy method, excepted for _toRegister at the end,
    //  which is another argument set to true.
    _result.setValue("copyArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class, Directory.class} );
      
    // 'copyArgumentPropertyNames' is the list of property names
    // for the copy method,
    // excepted for _toRegister which is set to true
    // following the copy() pattern,
    // and excepted for 'Parent Node' which is set to ""
    // to not display it in the user frame creation.
    _result.setValue("copyArgumentPropertyNames", new String[]{
      "Name", "ACS", ""} );

    // 'copyArgumentFromExplorer' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "copyArgumentPropertyNames"
    _result.setValue("copyArgumentFromExplorer", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.TRUE} );

    // 'copyArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    _result.setValue("copyArgumentConcatenedNameSource", new Integer[]{
      new Integer(2), null, null} );

    // 'copyNullArgumentPossible' indicates if an argument may be null
    //  in the method
    _result.setValue("copyNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE} );

    // 'copyOptionalArgument' indicates if an argument must not be used
    //  in the call to the method when it is null.
    _result.setValue("copyOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'copyRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names.
    //  Is optional.
    // If it is true, the array of restricted values for an argument
    // is in the value "staticSetOfItems" of the property in this BeanInfo.
    _result.setValue("copyRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    _result.setHidden(false);
    _result.setExpert(true);

    return _result;
  } // getCommonBeanDescriptor()

//  public EventSetDescriptor[] getEventSetDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getEventSetDescriptors() not yet implemented.");
//  }
//  public int getDefaultEventIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultEventIndex() not yet implemented.");
//  }


  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue():<P>
   * - 'tabbedPaneName' is the name (String value) of the pane
   *    where this property has to be displayed in the beamer.
   *    Is "" if 'withTabbedPanes' is false in the BeanDescriptor.
   * - 'panelGroupName' is the name (String value) of the group
   *    where this property has to be displayed with other related properties.
   *    May be "".<P>
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;

    try {
      /*
      // ------------------- property "IsAlias" --------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "IsAlias", EligiblePartyImpl.class, "getIsAlias", null);
      pd_four.setDisplayName("Is an alias");
      pd_four.setShortDescription("true if the eligible party is the alias"+
        " of another eligible party owned by this ACS or not.");
      pd_four.setBound(false);
      //pd_four.setConstrained(false);
      //pd_four.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_four.setValue("panelGroupName", "  Main attributes  " );

      // ------------------- property "AliasOf" ------------------------
      PropertyDescriptor pd_six = new PropertyDescriptor(
        "AliasOf", EligiblePartyImpl.class, "getEorTrueEP", null);
      pd_six.setDisplayName("Is alias of");
      pd_six.setShortDescription("This is the eligible party for which this object is an alias.");
      pd_six.setBound(false);
      //pd_six.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_six.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_six.setValue("panelGroupName", "  Main attributes  " );
       */

      // ------------------- property "AclConnectedResources" --------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "AclConnectedResources", EligiblePartyImpl.class, "getEorL_AclConnectedResources", null);
      pd_one.setDisplayName("Resources in ACL");
      pd_one.setShortDescription("The internal and external resources connected"+
        " through aclEntries.");
      pd_one.setBound(true);
        //pd_one.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_one.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_one.setValue("valueTypeInGroup", new Object[] {ImmutableResource.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  ACL  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_one.setValue("panelGroupName", "  In the ACL, as RIGHT USER  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_one.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_one.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_one.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_one.setValue("withDeleteButton", Boolean.FALSE );

      // ------------------- property "EPType" ------------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "EPType", EligiblePartyImpl.class, "getEPType", "setEPType");
      pd_two.setDisplayName("Right User type");
      pd_two.setShortDescription("If the type is not immutable, "+
              "the new type must be associated to the old one.");
      pd_two.setValue("tabbedPaneName", "  General  " );
      pd_two.setBound(true);
       // 'isRestrictedArgument' is true if it is a constrained property that is not
      // in the constructor (else, see 'constrRestrictedArgument' in the BeanDescriptor).
      // The restricted values are get from 'staticSetOfItems', for Strings only,
      // secondly from 'choiceMethodForNew', or through a search in the ACS by the PropertyEditor
      // like here.
      pd_two.setValue("isRestrictedArgument", Boolean.TRUE );

      // ------------------- property "M_ConstrainedChildACS" --------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "M_ConstrainedChildACS", EligiblePartyImpl.class, 
        "getM_ConstrainedChildACS", null);

      pd_three.setDisplayName("Supported child ACS");
      pd_three.setShortDescription("This object supports "+
              "the execution or the integrity of these child ACSs.");
      pd_three.setBound(true);
      //pd_three.setConstrained(false);
      // set the type of property editor to use in the Gui1 package.
      pd_three.setValue("isBaseObjectMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, it defines the type of the elementary object to handle. 
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_three.setValue("valueTypeInGroup", new Object[] {ImmutableACS.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_three.setValue("panelGroupName", "    Supports the execution "+
              "or the integrity of (opened or closed) child ACSs  " );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. 
      pd_three.setValue("withViewButton", Boolean.TRUE );
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
        //pd_three.setValue("KeysUpdateInMap", Boolean.TRUE);
      // 'isRestrictedArgument' is true if it is a constrained property that is not
      // in the constructor (else, see 'constrRestrictedArgument' in the BeanDescriptor).
      // The restricted values are get from 'staticSetOfItems', for Strings only,
      // secondly from 'choiceMethodForNew' like here,
      // or through a search in the ACS by the PropertyEditor.
        //pd_three.setValue("isRestrictedArgument", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke 
      // to let the user defines the new restricted key, in a map, or the new instance, 
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if 
      // there is an add method or a write method. With 'isRestrictedArgument', 
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke, or null if it is the source,
      // and the name of the method (for instance, 
      // ActionNewResource.createResource() or NameLinkTreeImpl.getVisibleSpaceConstants()).
      // The method has no argument.
        //pd_three.setValue("choiceMethodForNew", new Object[] {
            //ACSFactoryUtilityImpl.getInstance(), "getKeysForConstrainedChildACSMap"} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0, which allows, if it is alone,
      //      to have the right BeanInfo and its constructor description from the class 
      //      of the return value of this add method,
                // - option 1: the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - option 2: the property names of the add method arguments, 
                //      in the BeanInfo class, in the correct order
      // These elements allow the PropertyEditor to call BaseObjectHandler which 
      // displays a dialog to enter the arguments and call the add method.
        //pd_three.setValue("addMethod", "addAclEntry");
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property. To remove a key, use this method with a null argument
      // for the value.
        //pd_three.setValue("removeMethod", "removeAclEntry");
      
      // ------------------- property "OneSourceTypedPrivilege" --------------------
      PropertyDescriptor pd_seven = new PropertyDescriptor(
        "OneSourceTypedPrivilege", EligiblePartyImpl.class,
        "getEorL_SourcePrivilegeForTypes", null);
      // caution: this descriptor name is used in BeamerManager to forbid the
      // Select and Remove buttons
      pd_seven.setDisplayName("Typed Privileges");
      pd_seven.setShortDescription("The typed privileges of this privilege source.");
      pd_seven.setBound(true);
      //pd_seven.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_seven.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      // as DetailledName is used only in a CollectionPropertyEditor
      // to display the detailled name rather than the name.
      pd_seven.setValue("valueTypeInGroup", new Object[] {PrivilegeForType.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seven.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seven.setValue("panelGroupName", "  A - Privileges as RIGHT USER  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_seven.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_seven.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_seven.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'. 'Select' implies 'Remove'.
      pd_seven.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes automatically with 'Select'
      // when the property is an external object reference.
      pd_seven.setValue("withRemoveButton", Boolean.FALSE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be preset - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the known argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
        //pd_seven.setValue("withKnownArgumentsForNew", new Object[] {
          //null, null, null, null, "this"} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_seven.setValue("addMethod", "addEorPrivilegeForLinksToTarget");
          //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty", "Rights"} );
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        //pd_seven.setValue("removeMethod", "removeEorPrivilegeToTarget");
            //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty"} );
      // Following the KeyPropertiesInName pattern,
      // 'removeArgumentsFromName' defines how to find the right arguments for
      // the 'remove' method for the property, when the name of the object
      // to remove is known, and when finalizeForUser() is not used.
      // If "removeArgumentsFromName" is null, the remove method is invoked
      // with the object to remove as single argument.
      // The returned array defines:
      // - the name of the class to call, at the index 0
      // - the name of the static method to call, at the index 1, with
      //    the name of the object to remove as argument, and with
      //    an Object[] array as returned value
      // - in the Object[] array returned as value by the static method,
      //    the sequence of index where are the n arguments to use
      //    in the remove method, in the correct order
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        // pd_seven.setValue("removeArgumentsFromName", ... );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_seven.setValue("limitedSizeForPresentation", new Integer(5));
      
      // ------------------- property "OneSourceLinkedPrivilege" --------------------
      PropertyDescriptor pd_eight = new PropertyDescriptor(
        "OneSourceLinkedPrivilege", EligiblePartyImpl.class,
        "getEorL_SourcePrivilegeForLinks", null);
      // caution: this descriptor name is used in BeamerManager to forbid the
      // Select and Remove buttons
      pd_eight.setDisplayName("Linked Privileges");
      pd_eight.setShortDescription("The linked privileges of this privilege source.");
      pd_eight.setBound(true);
      //pd_eight.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_eight.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      // as DetailledName, is used only in a CollectionPropertyEditor
      // to display the detailled name rather than the name.
      pd_eight.setValue("valueTypeInGroup", new Object[] {PrivilegeForLinks.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eight.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eight.setValue("panelGroupName", "  A - Privileges as RIGHT USER  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_eight.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_eight.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_eight.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'. 'Select' implies 'Remove'.
        //pd_eight.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes automatically with 'Select'
      // when the property is an external object reference.
        //pd_eight.setValue("withRemoveButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be preset - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the known argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
        //pd_eight.setValue("withKnownArgumentsForNew", new Object[] {
          //null, null, null, null, "this"} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_eight.setValue("addMethod", "addEorPrivilegeForLinksToSource");
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        //pd_eight.setValue("removeMethod", "removeEorPrivilegeToSource");
      // Following the KeyPropertiesInName pattern,
      // 'removeArgumentsFromName' defines how to find the right arguments for
      // the 'remove' method for the property, when the name of the object
      // to remove is known, and when finalizeForUser() is not used.
      // If "removeArgumentsFromName" is null, the remove method is invoked
      // with the object to remove as single argument.
      // The returned array defines:
      // - the name of the class to call, at the index 0
      // - the name of the static method to call, at the index 1, with
      //    the name of the object to remove as argument, and with
      //    an Object[] array as returned value
      // - in the Object[] array returned as value by the static method,
      //    the sequence of index where are the n arguments to use
      //    in the remove method, in the correct order
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        // pd_eight.setValue("removeArgumentsFromName", ... );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_eight.setValue("limitedSizeForPresentation", new Integer(5));

      //
      // ----------- End of properties ----------------------------------------
      //

      // start from the parent class bean info
      l_pdFromParent = super.getPropertyDescriptors(); // 9 items
      l_result = new PropertyDescriptor[l_pdFromParent.length + 5];
      System.arraycopy(l_pdFromParent, 0, l_result, 0, 4);
      l_result[4] = pd_two; // EP Type
      l_result[5] = l_pdFromParent[4]; // Virtual Folders
      l_result[6] = l_pdFromParent[5]; // Open Views
      l_result[7] = l_pdFromParent[6]; //
      l_result[8] = l_pdFromParent[7]; // 
      l_result[9] = l_pdFromParent[8]; //

      // caution for updating: GroupID and UserID BeanInfos are moving all the descriptors
      // after these 8 first ones, to put the 'AG' pane at the right place,
      // before the 'ACL' and 'Privileges' panes.
      PropertyDescriptor[] l_pd = new PropertyDescriptor[] {
          pd_three, // constrained child ACS
          pd_one, // ACL
          pd_seven, pd_eight // Privileges
      };
      System.arraycopy(l_pd, 0, l_result, 10, l_pd.length);

      l_pdFromParent = null;
      l_pd = null;
      return l_result;

    } catch (IntrospectionException e) {
      new ProcessError("In EligiblePartyImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In EligiblePartyImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

//  public int getDefaultPropertyIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultPropertyIndex() not yet implemented.");
//  }
//  public MethodDescriptor[] getMethodDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getMethodDescriptors() not yet implemented.");
//  }
//  public BeanInfo[] getAdditionalBeanInfo() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getAdditionalBeanInfo() not yet implemented.");
//  }
//  public Image getIcon(int iconKind) {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getIcon() not yet implemented.");
//  }

    /**
     * Get the property name for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * This is used to register the BeamerManager as a PropertyChangeListener.
     * @return the property name.
     */
    public String getBaseBeanManagerBoundProperty() {
      return new String("m_OwnEligibleParties");
    }

} // EligiblePartyImplBeanInfo