//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBase.*;
import ARoad0.gBaseInterface.GroupID;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the GroupIDImpl class.
 * It is used for guiding the displaying and the editing of the properties
 * of GroupIDImpl, by the Gui1 classes as CommonPropertyEditor and BaseObjectCreator.
 * @version 0.7.0
 * @since 0.5.1
 */
public class GroupIDImplBeanInfo extends GroupIDMemberImplBeanInfo {

  /**
   * Single constructor
   */
  public GroupIDImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Some standard values are set:
   * - 'hidden' is false since the bean is not hidden to the users
   * - 'expert' is false since the bean is not reserved to experts
   * <P>
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue().
   * <P>
   * Used by Gui1.BaseObjectCreator for the GroupIDImpl constructor:
   * - 'constrArgumentClasses' is a Class array of the argument classes.
   * - 'constrArgumentPropertyNames' is the list of property names.
   *    A name is null if it is not a property. Then, it is selected in the explorer.
   *    A name may be from a super class.
   * - 'constrArgumentFromSelect' is the list of cases
   *    where the argument is selected by the user in the explorer.
   * - 'constrArgumentConcatenedNameSource' indicates, when an argument
   *    is entered by the user as a relative name
   *    which has then to be concatenate to another constructor argument,
   *    what is this another argument which provides the first part of the name.
   *    In this case, the relative name is always at the end,
   *    and the number of the another argument is get as an Integer
   *    following the order of 'constrArgumentClasses'.
   *    If the another argument is not a name, it must be a BaseObject,
   *    and so the relevant name is get through the method getName().
   *  <P>
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(GroupID.class );
    result.setDisplayName("Group");

    result = getCommonBeanDescriptor(result);
    
    return result;
  } // getBeanDescriptor()

//  public EventSetDescriptor[] getEventSetDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getEventSetDescriptors() not yet implemented.");
//  }
//  public int getDefaultEventIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultEventIndex() not yet implemented.");
//  }


  /**
   * Return the property descriptors for all the super class and for this class.
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue():<P>
   * - 'tabbedPaneName' is the name (String value) of the pane
   *    where this property has to be displayed in the beamer.
   *    Is "" if 'withTabbedPanes' is false in the BeanDescriptor.
   * - 'panelGroupName' is the name (String value) of the group
   *    where this property has to be displayed with other related properties.
   *    May be "".<P>
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;

    try {

      // ------------------- property "l_Members" --------------------
      PropertyDescriptor pd_five = new PropertyDescriptor(
        "l_Members", GroupIDImpl.class, "getEorL_Members", null);

      pd_five.setDisplayName("Direct members");
      pd_five.setShortDescription("The direct members of this group.");
      pd_five.setBound(true);
      //pd_five.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_five.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_five.setValue("valueTypeInGroup", new Object[] {Resource.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_five.setValue("tabbedPaneName", "  Members  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_five.setValue("panelGroupName", "  Own and external ones  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_five.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_five.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_five.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_five.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select' if 'Clear' is not used,
      // when the property is an external object reference.
      pd_five.setValue("withRemoveButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the fixed argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
        //pd_five.setValue("withKnownArgumentsForNew", new Object[] {
          //null, null, null, null, "this"} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_five.setValue("addMethod", "addEorMember");
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_five.setValue("removeMethod", "removeEorMember");
            //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty"} );
      // Following the KeyPropertiesInName pattern,
      // 'removeArgumentsFromName' defines how to find the right arguments for
      // the 'remove' method for the property, when the name of the object
      // to remove is known, and when finalizeForUser() is not used.
      // If "removeArgumentsFromName" is null, the remove method is invoked
      // with the object to remove as single argument.
      // The returned array defines:
      // - the name of the class to call, at the index 0
      // - the name of the static method to call, at the index 1, with
      //    the name of the object to remove as argument, and with
      //    an Object[] array as returned value
      // - in the Object[] array returned as value by the static method,
      //    the sequence of index where are the n arguments to use
      //    in the remove method, in the correct order
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        // pd_five.setValue("removeArgumentsFromName", ... );

      // ------------------- property "l_GroupIDCurrentActors" --------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "l_GroupIDCurrentActors", GroupIDImpl.class, "getEorL_GroupIDCurrentActors", null);

      pd_one.setDisplayName("Current actors");
      pd_one.setShortDescription("The actors currently attached to this groupID.");
      pd_one.setBound(true);
      //pd_one.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_one.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_one.setValue("valueTypeInGroup", new Object[] {Actor.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  AG  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_one.setValue("panelGroupName", "  Links  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_one.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_one.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_one.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_one.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_one.setValue("withDeleteButton", Boolean.FALSE );


      // ------------------- property "l_GroupIDOwnResources" --------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "l_GroupIDOwnResources", GroupIDImpl.class, "getEorL_GroupIDOwnResources", null);

      pd_two.setDisplayName("Contains resources");
      pd_two.setShortDescription("The resources of the ACS which are in this group.");
      pd_two.setBound(true);
      //pd_two.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_two.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_two.setValue("valueTypeInGroup", new Object[] {Resource.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  Resources  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_two.setValue("panelGroupName", "  Links  " );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_two.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_two.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_two.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_two.setValue("withDeleteButton", Boolean.FALSE );


      // ------------------- property "l_GroupIDCurrentActorsAsSecondaryGroup" -------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "l_GroupIDCurrentActorsAsSecondaryGroup", GroupIDImpl.class,
        "getEorL_GroupIDCurrentActorsAsSecondaryGroup", null);

      pd_three.setDisplayName("Second. group/actors");
      pd_three.setShortDescription("The actors that have this groupID as"+
        " a secondary groupID as rights users.");
      pd_three.setBound(true);
      //pd_three.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_three.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_three.setValue("valueTypeInGroup", new Object[] {Resource.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  AG  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_three.setValue("panelGroupName", "  Links  " );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_three.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_three.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_three.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_three.setValue("withDeleteButton", Boolean.FALSE );

      /*
      // ------------------- property "Order" --------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "Order", GroupIDImpl.class, "getOrder", null);
      pd_four.setDisplayName("GID number ");
      pd_four.setShortDescription("The GID may determine the priority order in access decisions.");
      pd_four.setBound(true);
      //pd_four.setConstrained(false);
      //pd_four.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  AG  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_four.setValue("panelGroupName", "  Main attributes  " );   
       * */   
      
      //
      // ------------End of GroupIDImpl properties definition --------------------
      //

      // start from the parent class bean info, and
      // put the inner descriptor after the 8 first ones for the 'General' pane,
      // to put the 'AG' pane at the third place
      l_pdFromParent = super.getPropertyDescriptors();
      l_result = new PropertyDescriptor[l_pdFromParent.length + 4];
      System.arraycopy(l_pdFromParent, 0, l_result, 0, 8);
      l_result[8] = pd_one;
      l_result[9] = pd_three;
      System.arraycopy(l_pdFromParent, 8, l_result, 10, l_pdFromParent.length - 8);
      l_result[l_result.length - 2] = pd_five;
      l_result[l_result.length - 1] = pd_two;
      //l_result[l_result.length - 1] = pd_four;

      l_pdFromParent = null;
      return l_result;
    } catch (IntrospectionException e) {
      new ProcessError("In GroupIDImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In GroupIDImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

//  public int getDefaultPropertyIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultPropertyIndex() not yet implemented.");
//  }
//  public MethodDescriptor[] getMethodDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getMethodDescriptors() not yet implemented.");
//  }
//  public BeanInfo[] getAdditionalBeanInfo() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getAdditionalBeanInfo() not yet implemented.");
//  }
//  public Image getIcon(int iconKind) {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getIcon() not yet implemented.");
//  }

} // GroupIDImplBeanInfo