//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.EligibleParty;
import ARoad0.gBase.*;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the ACS node in a nameISTree
 * to display in the GUI. It is used for displaying the properties without editing.
 * @version 0.7.0
 */
public class ISAcsBeanInfo extends ISSubACSBeanInfo {

  public ISAcsBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of EligiblePartyImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(EligibleParty.class );
    result.setDisplayName("IS ACS");

    result = getCommonBeanDescriptor(result);

    return result;
  } // getBeanDescriptor()

  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue():<P>
   * - 'tabbedPaneName' is the name (String value) of the pane
   *    where this property has to be displayed in the beamer.
   *    Is "" if 'withTabbedPanes' is false in the BeanDescriptor.
   * - 'panelGroupName' is the name (String value) of the group
   *    where this property has to be displayed with other related properties.
   *    May be "".<P>
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;

    try {
      // ------------------- property "isAuthorizationServer" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "AuthorizationServer", ISNodeImpl.class, "isAuthorizationServer", null);
      pd_one.setDisplayName("Authorization server");
      pd_one.setShortDescription("This IS node is an authorization server for other ACS");
      pd_one.setBound(true);
      //pd_one.setConstrained(false);
      //pd_one.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_one.setValue("panelGroupName", "  Main attributes  " );

      // ------------------- property "TiedChild" ------------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "TiedChild", ISNodeImpl.class, "isTiedChild", null);
      pd_two.setDisplayName("Tied child");
      pd_two.setShortDescription("This ACS node is tied to its parent for openings and closings");
      pd_two.setBound(true);
      //pd_two.setConstrained(false);
      //pd_two.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_two.setValue("panelGroupName", "  Main attributes  " );

      //
      // ------------End of properties definition --------------------
      //

      // start from the parent class bean info
      l_pdFromParent = super.getPropertyDescriptors();
      l_result = new PropertyDescriptor[l_pdFromParent.length + 2];
      System.arraycopy(l_pdFromParent, 0, l_result, 0, l_pdFromParent.length);
      l_result[l_result.length - 2] = pd_one;
      l_result[l_result.length - 1] = pd_two;

      return l_result;
    } catch (IntrospectionException e) {
      new ProcessError("In ISAcsBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In ISAcsBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

} // ISAcsBeanInfo



