//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.EligibleParty;
import ARoad0.gBaseInterface.BaseBeanManager;
import ARoad0.gBase.*;
import java.beans.SimpleBeanInfo;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.lang.reflect.Method;

import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the component node in a nameISTree
 * to display in the GUI. It is used for displaying the properties without editing.
 * @version 0.7.0
 */
public class ISComponentBeanInfo extends SimpleBeanInfo
  implements BeanInfoPattern {

  public ISComponentBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of EligiblePartyImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(EligibleParty.class );
    result.setDisplayName("Generic IS Component");

    result = getCommonBeanDescriptor(result);

    return result;
  } // getBeanDescriptor()

  /**
   * Get the common description of the bean.
   * Not used by subclasses.
   * @param _result is the BeanDescriptor to configure
   * @return the bean descriptor
   */
  protected BeanDescriptor getCommonBeanDescriptor(BeanDescriptor _result) {

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value)
    _result.setValue("withTabbedPanes", Boolean.TRUE );

    // 'withDeleteMethod' is true to display the 'delete' button in the beamer
    _result.setValue("withDeleteMethod", Boolean.FALSE );

    // 'commonBeanClass' is the class which has the bean info defining getCommonBeanDescriptor(),
    // and then which is the class or a superclass of the bean
    _result.setValue("commonBeanClass", ISNodeImpl.class );

    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    _result.setHidden(false);
    _result.setExpert(true);

    return _result;
  } // getCommonBeanDescriptor()

  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue():<P>
   * - 'tabbedPaneName' is the name (String value) of the pane
   *    where this property has to be displayed in the beamer.
   *    Is "" if 'withTabbedPanes' is false in the BeanDescriptor.
   * - 'panelGroupName' is the name (String value) of the group
   *    where this property has to be displayed with other related properties.
   *    May be "".<P>
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pd;

    try {
      // ------------------- property "OpenNode" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "OpenNode", ISNodeImpl.class, "isOpen", null);
      pd_one.setDisplayName("Open");
      pd_one.setShortDescription("This element is open or closed");
      pd_one.setBound(true);
      //pd_one.setConstrained(false);
      //pd_one.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  Basic  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_one.setValue("panelGroupName", "  Main attributes  " );

      // ------------------- property "NodeType" --------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "NodeType", ISNodeImpl.class, "getType", null);
      pd_two.setDisplayName("Type");
      pd_two.setShortDescription("The type of this IS element");
      pd_two.setBound(true);
      //pd_two.setConstrained(false);
      //pd_two.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  Basic  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_two.setValue("panelGroupName", "  Main attributes  " );

      // ------------------- property "HubNode" --------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "HubNode", ISNodeImpl.class, "isHubNode", null);
      pd_three.setDisplayName("Is hub");
      pd_three.setShortDescription("This element is a hub for its children exchanges.");
      pd_three.setBound(true);
      //pd_one.setConstrained(false);
      //pd_three.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  Basic  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_three.setValue("panelGroupName", "  Main attributes  " );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      //pd_three.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "VisibleSpace" --------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "VisibleSpace", ISNodeImpl.class, "getExtendedVisibleSpace", null);
      pd_four.setDisplayName("Visible space");
      pd_four.setShortDescription("The space which this element may view in the IS");
      pd_four.setBound(true);
      //pd_four.setConstrained(false);
      //pd_four.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  Basic  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_four.setValue("panelGroupName", "  Main attributes  " );


      //
      // ----------- End of properties ----------------------------------------
      //

      l_pd = new PropertyDescriptor[] {pd_one, pd_three, pd_two, pd_four};
      return l_pd;
    } catch (IntrospectionException e) {
      new ProcessError("In ISComponentBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In ISComponentBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

    /**
     * Get the instance, if it exists, which fires an event when
     * the bean is closed or deleted.
     * @return null.
     */
    public BaseBeanManager getBaseBeanManagerInstance() {
      return null;
    }

    /**
     * Get the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, if it exists,
     * which fires an event when the bean instance is closed or deleted.
     * @return the get method. Is null when there is
     *  an active BeanInfoPattern.getBaseBeanManagerInstance(),
     *  or when it is not applicable.
     */
    public Method getBaseBeanManagerGetMethod() {
      return null;
    }

    /**
     * Get the property name for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * This is used to register the BeamerManager as a PropertyChangeListener.
     * @return the property name.
     */
    public String getBaseBeanManagerBoundProperty() {
      return null;
    }

} // ISComponentBeanInfo
