//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

import ARoad0.gBase.*;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.EligibleParty;

import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the NoThanViewInBaseImpl class.
 * It is used for displaying and editing the properties of the class.
 * @version 0.7.0
 */
public class NoThanViewInBaseImplBeanInfo extends EPRViewInBaseImplBeanInfo
  implements BeanInfoPattern {

  public NoThanViewInBaseImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * <P>
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue().
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(NoThanViewInBaseImpl.class );
    result.setDisplayName("NoMore-NoLess View");

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of NoThanViewInBaseImpl
    result.setValue("withTabbedPanes", Boolean.TRUE );

    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    result.setHidden(false);
    result.setExpert(false);

    return result;
  } // getBeanDescriptor()

  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue()
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_pd;

    try {

      // ------------------- property "FirstAccessTarget" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "FirstAccessTarget", NoThanViewInBaseImpl.class, 
        "getEorFirstAccessTarget", "setEorFirstAccessTarget");
      pd_one.setDisplayName("Access Target ");
      pd_one.setShortDescription("The target on which Eligible Parties have "+
              "rights to be controlled");
      pd_one.setBound(true);
      //pd_one.setConstrained(false);
      //pd_one.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_one.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the NoThanViewInBaseImpl instance
      // does it, or when is is not applicable.
      //pd_one.setValue("BaseBeanManager", null);
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_one.setValue("withViewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_one.setValue("withSelectButton", Boolean.TRUE );
      // 'withClearButton' is the boolean which set if the PropertyEditor
      // has to display a Clear button. False since the set method does not
      // accept anull argument.
      pd_one.setValue("withClearButton", Boolean.FALSE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be fixed
      // when a new instance is created through the 'New' button.
      // The string 'this' is used to point to the object in the beamer.
      // Caution: the known arguments have to be Objects.
      //pd_six.setValue("withKnownArgumentsForNew", new Object[] {null, "ACS", null} );
      
      // ------------------- property "ConnectedACS" from ViewInBaseBeanInfo  ----------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "ConnectedACS", NoThanViewInBaseImpl.class, "getEorL_ConnectedACSNames", null);
      pd_two.setDisplayName("EP Connected ACSs");
      pd_two.setShortDescription("ACS having EPs in the view, excluding the target ACS.");
      pd_two.setBound(true);
        //pd_two.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_two.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_two.setValue("valueTypeInGroup", new Object[] {ImmutableName.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  Context  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_two.setValue("panelGroupName", "  Links in ACL, as rights user  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_two.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_two.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_two.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_two.setValue("withDeleteButton", Boolean.FALSE );

      // ------------------- property "OneBaseObjectInFolder" from ViewInBaseBeanInfo  ----------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "OneBaseObjectInFolder", NoThanViewInBaseImpl.class, "getEorM_BaseObjects", null);
      // caution: this descriptor name is used in BeamerManager to forbid the
      // Select and Remove buttons
      pd_three.setDisplayName("Basic Right Users ");
      pd_three.setShortDescription("Access sources for which rights are to be controlled, including "+
              "their indirect members and running-under actors.");
      pd_three.setBound(true);
      //pd_three.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_three.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_three.setValue("valueTypeInGroup", new Object[] {EligibleParty.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_three.setValue("panelGroupName", "    " );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_three.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_three.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'. 'Select' implies 'Remove'.
      pd_three.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes automatically with 'Select'
      // when the property is an external object reference.
      pd_three.setValue("withRemoveButton", Boolean.TRUE );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_three.setValue("addMethod", "addEorBaseObject");
          //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty", "Rights"} );
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_three.setValue("removeMethod", "removeEorBaseObject");

      // ------------------- property "OneExcludedEP" ----------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "OneExcludedEP", NoThanViewInBaseImpl.class, "getExcludedSources", null);
      pd_four.setDisplayName("Excluded R. Users");
      pd_four.setShortDescription("Should be in the checking perimeter"+
              ", but is explictly excluded from it.");
      pd_four.setBound(true);
        //pd_four.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_four.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_four.setValue("valueTypeInGroup", new Object[] {EligibleParty.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_four.setValue("panelGroupName", "  Links in ACL, as rights user  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_four.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_four.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_four.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_four.setValue("withRemoveButton", Boolean.TRUE );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_four.setValue("addMethod", "addEorExcludedSource");
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_four.setValue("removeMethod", "removeEorExcludedSource");

      // ------------------- property "OneMoreCriterion" ------------------------
      PropertyDescriptor pd_five = new PropertyDescriptor(
        "OneMoreCriterion", NoThanViewInBaseImpl.class, 
        "getMoreCriterion", "setMoreCriterion");
      pd_five.setDisplayName("No-More-Than right");
      pd_five.setShortDescription("The maximal right that each checking perimeter EP "+
              "should have on the target");
      pd_five.setBound(true);
      //pd_five.setConstrained(false);
      //pd_five.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_five.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_five.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the NoThanViewInBaseImpl instance
      // does it, or when is is not applicable.
      //pd_five.setValue("BaseBeanManager", null);
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the ResourceImpl instance
      // does it, or when is is not applicable.
      //pd_six.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_five.setValue("withViewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_five.setValue("withSelectButton", Boolean.TRUE );
      // 'withClearButton' is the boolean which set if the PropertyEditor
      // has to display a Clear button
      pd_five.setValue("withClearButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be fixed
      // when a new instance is created through the 'New' button.
      // The string 'this' is used to point to the object in the beamer.
      // Caution: the known arguments have to be Objects.
      //pd_six.setValue("withKnownArgumentsForNew", new Object[] {null, "ACS", null} );
      
      // ------------------- property "OneLessCriterion" ------------------------
      PropertyDescriptor pd_six = new PropertyDescriptor(
        "OneLessCriterion", NoThanViewInBaseImpl.class, 
        "getLessCriterion", "setLessCriterion");
      pd_six.setDisplayName("No-Less-Than right");
      pd_six.setShortDescription("The minimal right that each checking perimeter EP "+
              "should have on the target");
      pd_six.setBound(true);
      //pd_six.setConstrained(false);
      //pd_six.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_six.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_six.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the NoThanViewInBaseImpl instance
      // does it, or when is is not applicable.
      //pd_six.setValue("BaseBeanManager", null);
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the ResourceImpl instance
      // does it, or when is is not applicable.
      //pd_six.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_six.setValue("withViewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_six.setValue("withSelectButton", Boolean.TRUE );
      // 'withClearButton' is the boolean which set if the PropertyEditor
      // has to display a Clear button
      pd_six.setValue("withClearButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be fixed
      // when a new instance is created through the 'New' button.
      // The string 'this' is used to point to the object in the beamer.
      // Caution: the known arguments have to be Objects.
      //pd_six.setValue("withKnownArgumentsForNew", new Object[] {null, "ACS", null} );

      // ------------------- property "l_CheckingPerimeter"  ----------
      PropertyDescriptor pd_seven = new PropertyDescriptor(
        "l_CheckingPerimeter", NoThanViewInBaseImpl.class, "getCheckingPerimeter", null);
      pd_seven.setDisplayName("Checking Perimeter ");
      pd_seven.setShortDescription("Right users implied in the rights checking, including "+
              "the basic right users, their members/running-under actors, but without the excluded users.");
      pd_seven.setBound(true);
      //pd_seven.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_seven.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_seven.setValue("valueTypeInGroup", new Object[] {EligibleParty.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seven.setValue("tabbedPaneName", "  Context  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_three.setValue("panelGroupName", "    " );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_seven.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_seven.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'. 'Select' implies 'Remove'.
      pd_seven.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes automatically with 'Select'
      // when the property is an external object reference.
      pd_seven.setValue("withRemoveButton", Boolean.FALSE );

      // ------------------- property "l_ReducedEPsInViewEPs"  ----------
      PropertyDescriptor pd_eight = new PropertyDescriptor(
        "l_ReducedEPsInViewEPs", NoThanViewInBaseImpl.class, "getReducedSourcesInViewSources", null);
      pd_eight.setDisplayName("Reduced R. Users ");
      pd_eight.setShortDescription("Basic right users associated to at most one excluded user." +
              " So, they are not fully analyzed by the view.");
      pd_eight.setBound(true);
      //pd_three.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_eight.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_eight.setValue("valueTypeInGroup", new Object[] {EligibleParty.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eight.setValue("tabbedPaneName", "  Context  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_eight.setValue("panelGroupName", "    " );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_eight.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_eight.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'. 'Select' implies 'Remove'.
      pd_eight.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes automatically with 'Select'
      // when the property is an external object reference.
      pd_eight.setValue("withRemoveButton", Boolean.FALSE );
      
      // ------- End of properties description --------------------------------- 
      
      // replaces the 'l_BaseObjects', 'ConnectedACS' properties in the super BeanInfo
      // to change the display name and the short description
      l_pdFromParent = super.getPropertyDescriptors();
      l_pd = new PropertyDescriptor[11];

      l_pd[0] = l_pdFromParent[0];      
      l_pd[1] = pd_one; 
      l_pd[2] = l_pdFromParent[2];
      l_pd[3] = l_pdFromParent[3];
      l_pd[4] = pd_five;
      l_pd[5] = pd_six;
      l_pd[6] = pd_two;
      l_pd[7] = pd_three;
      l_pd[8] = pd_four;
      l_pd[9] = pd_seven;
      l_pd[10] = pd_eight;
      
      l_pdFromParent = null;
      return l_pd;
    } catch (IntrospectionException e) {
      new ProcessError("In NoThanViewInBaseImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In NoThanViewInBaseImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch
 } // getPropertyDescriptors()

//  public int getDefaultPropertyIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultPropertyIndex() not yet implemented.");
//  }
//  public MethodDescriptor[] getMethodDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getMethodDescriptors() not yet implemented.");
//  }
//  public BeanInfo[] getAdditionalBeanInfo() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getAdditionalBeanInfo() not yet implemented.");
//  }
//  public Image getIcon(int iconKind) {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getIcon() not yet implemented.");
//  }

    /**
     * Get the instance, if it exists, which fires an event when
     * the bean is closed or deleted.
     * @return a BaseManager object.
     */
/*    public BaseBeanManager getBaseBeanManagerInstance() {
      return BaseManagerImpl.getInstance();
    }
*/
    /**
     * Get the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, if it exists,
     * which fires an event when the bean instance is closed or deleted.
     * @return null
     */
/*    public Method getBaseBeanManagerGetMethod() {
      return null;
    }
*/
    /**
     * Get the property name for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * @return "OpenViews"
     */
/*    public String getBaseBeanManagerBoundProperty() {
      return "OpenViews";
    }
*/

} // NoThanViewInBaseImplBeanInfo