//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls


/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details (gpl.html at the root of Access Road directory).
This license is applicable for 20 years since the publication of the program.
The applicable laws are the French ones, and the relevant court is at Nanterre
(92), France.
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation, Inc.,
675 Mass Ave, Cambridge, MA 02139, USA.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.*;
import ARoad0.gBase.PrivilegeForTypeImpl;
import ARoad0.gBase.PrivilegeAbst;

import java.beans.SimpleBeanInfo;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.lang.reflect.Method;

import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the PrivilegeForTypeImpl class.
 * It is used for displaying and editing the properties of PrivilegeForTypeImpl.
 * The closing and deleting operations are managed by the ACS.
 * See the <a href="{@docRoot}/copyright.html">Copyright</a>.
 * @version 0.7.0
 */
public class PrivilegeForTypeImplBeanInfo extends SimpleBeanInfo
  implements BeanInfoPattern {

  public PrivilegeForTypeImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue():
   * - 'withTables' is true (Boolean value) since some properties have
   *  to be displayed in table(s), and not only in panels (property, value)
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of PrivilegeForTypeImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(PrivilegeForType.class );
    result.setDisplayName("Typed Privilege");

    // 'withTables' is true since some properties have to be displayed in tables,
    // and not only in panels (property, value)
      //result.setValue("withTables", new Boolean(true) );

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of PrivilegeForTypeImpl
    result.setValue("withTabbedPanes", Boolean.TRUE );

    // 'commonBeanClass' is the class which has the bean info defining getCommonBeanDescriptor(),
    // and which is used by BaseUtilityimpl.getSuperInterfacesFromBeanInfo()
    // to get the class or the superclass of the bean
    result.setValue("commonBeanClass", PrivilegeForTypeImpl.class );
    
    // ----------------------- delete parameters ---------------------------
    
    // 'withDeleteMethod' is true to display the 'delete' button in the beamer
    result.setValue("withDeleteMethod", Boolean.TRUE );

    // When the bean is not a FinalizedObject,
    // 'externalFinalizeProperty' is the string name of the property, if it exists,
    // under which the finalizing method for this object is described
    // in the BeanInfo of the BaseBeanManager
    // (see getBaseBeanManagerInstance() and getBaseBeanManagerGetMethod() ).
        //result.setValue("externalFinalizeProperty", new String[]{"AclEntries"} );

    // ----------------------- constructor parameters ---------------------------

    // 'constrArgumentClasses' is the list of argument classes
    // for the class constructor
    result.setValue("constrArgumentClasses", new Class[] {
      ACS.class, String.class, String.class, String.class,
      String.class, StringRight[].class, String.class} );

    // 'constrArgumentPropertyNames' is the list of property names
    // for the class constructor or the add method
    result.setValue("constrArgumentPropertyNames", new String[]{
      "ACS", "NickName", "SourceType", "TargetType",
      "MainType", "PrivilegeRights", "SecondType"} );

    // 'constrArgumentFromSelect' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "constrArgumentPropertyNames", or, as here,
    // in the order of the constructor method in ResourceImpl
    result.setValue("constrArgumentFromSelect", new Boolean[]{
      Boolean.TRUE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE,
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'constrArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another constructor argument,
    // what is this another argument which provides the first part of the name.
    result.setValue("constrArgumentConcatenedNameSource", new Integer[]{
      null, null, null, null, null, null, null} );

    // 'constrNullArgumentPossible' indicates if an argument may be null in the constructor
    result.setValue("constrNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE,
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'constrOptionalArgument' indicates if an argument is optional in the
    // call to the constructor when it is null.
    result.setValue("constrOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE,
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'constrRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names. Is optional.
    // If it is true, the array of restricted values for an argument is searched
    // through BaseUtilityImpl.getRestrictedValues():
    // - first, in 3 ACS maps which define the constraints from the ACS,
    // and then "constrArgumentPropertyNames" must contain the property descriptor name.
    // The name words may be separated by spaces in "constrArgumentPropertyNames",
    // although there is no separation in the ACS map.
    // - second, in the value "staticSetOfItems" of the property in this BeanInfo.
    result.setValue("constrRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE, Boolean.TRUE,
      Boolean.TRUE, Boolean.TRUE, Boolean.TRUE} );

    // Only if 'constrNonVisibleArgument' is true for an argument,
    // 'constrRestrictedFromOtherArgumentMessage' is optionally not null to give to the user
    // a message, when he needs to enter another argument to allow the definition
    // of the constraints for this argument.
    // Exemple of message: "<Set the ACS first>" from the value 'ACS'.
    result.setValue("constrRestrictedFromOtherArgumentMessage", new String[]{
      null, null, null, null,
      null, "Main Type", null} );

    // ----------------------- standard parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    result.setHidden(false);
    result.setExpert(true);

    return result;
  } // getBeanDescriptor()

  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue().
   * To read a property, use the read method in the prooperty descriptor. <P>
   * To update a property, there are 3 means, in this order: <P>
   * - if it is not a single value, use the "choiceMethodForNew" value to add one value <P>
   * - or, if it is not a single value, use the "addMethod" value to add one value
   *  and "removeMethod", for BaseObjects only, <P>
   * - or, in any cases, use the write method in the property descriptor
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pd;

    try {

      // ------------------- property "NickName" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "NickName", PrivilegeForTypeImpl.class, "getNickName", null);
      pd_one.setDisplayName("Nick Name");
      pd_one.setShortDescription("The nick name of the privilege ");
      //pd_one.setBound(false);
      //pd_one.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
            //pd_one.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_one.setValue("BaseBeanManager", );

        // ------------------ from PrivilegeAbst ---------------------------

      // ------------------- property "ACS" ------------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "ACS", PrivilegeForTypeImpl.class, "getEorACS", null);
      pd_two.setDisplayName("ACS");
      pd_two.setShortDescription("The access control system which manages this element");
      //pd_two.setBound(false);
      //pd_two.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_two.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_two.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_two.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "Comment" --------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "Comment", PrivilegeForTypeImpl.class, "getComment", "setComment");
      pd_four.setDisplayName("Comments");
      pd_four.setShortDescription("free user comments");
      pd_four.setBound(true);
      //pd_four.setConstrained(false);
      //pd_four.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_four.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_four.setValue("BaseBeanManager", );

      // ------------------- property "MainType" --------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "MainType", PrivilegeForTypeImpl.class, "getMainType", null);
      pd_three.setDisplayName("Main Type");
      pd_three.setShortDescription("The main type defines the behavior of the privilege");
      pd_three.setBound(false);
      //pd_three.setConstrained(false);
      //pd_three.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_three.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_three.setValue("BaseBeanManager", );

      // ------------------- property "PrivilegeRights" --------------------
      PropertyDescriptor pd_seven = new PropertyDescriptor(
        "PrivilegeRights", PrivilegeForTypeImpl.class, "getL_Rights", null);
      pd_seven.setDisplayName("List of rights");
      pd_seven.setShortDescription("The immutable privilege rights");
      pd_seven.setBound(false);
      //pd_seven.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_seven.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      //pd_seven.setValue("valueTypeInGroup", new Object[] {...} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seven.setValue("tabbedPaneName", "  Rights 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seven.setValue("panelGroupName", "  Rights  - "+
              " From typed sources to typed targets");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the PrivilegeForTypeImpl instance
      // does it, or when is is not applicable.
        //pd_seven.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_seven.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_seven.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_seven.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_seven.setValue("withDeleteButton", Boolean.FALSE );
      // 'BeanClass' is the BaseObject class which manages this property.
      // Currently, it is used for ACS restricted values set management to found
      // the 'Privilege' interface.
      pd_seven.setValue("BeanClass", "ARoad0.gBase.PrivilegeForTypeImpl" );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      pd_seven.setValue("limitedSizeForPresentation", new Integer(4));

      // ------------------- property "PositiveRight" --------------------
      PropertyDescriptor pd_eight = new PropertyDescriptor(
        "PositiveRight", PrivilegeForTypeImpl.class, "getPositiveRight", null);
      pd_eight.setDisplayName("Sens of rights");
      pd_eight.setShortDescription("'Grant' if the rights grant "+
        "the access, and 'Deny' otherwise.");
      pd_eight.setBound(false);
      //pd_eight.setConstrained(false);
      //pd_eight.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eight.setValue("tabbedPaneName", "  Rights 2  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_eight.setValue("panelGroupName", "  Context  ");
      // 'tags' is the name (String value) of the couple of strings,
      // with a size limited to 12 caracters,
      // to be displayed when the property is true or false, in this order.
      // Cannot be empty if it exists, and the default value for the property
      // editor is ("TRUE", "False") when 'tags' does not exist.
      pd_eight.setValue("tags", new String[] {"Grant", "Deny"} );
 
      // ------------------- property "SecondType" --------------------
      PropertyDescriptor pd_six = new PropertyDescriptor(
        "SecondType", PrivilegeForTypeImpl.class, "getSecondType", "setSecondType");
      pd_six.setDisplayName("Second Type");
      pd_six.setShortDescription("The second type is used just like a Resource type");
      pd_six.setBound(true);
      //pd_six.setConstrained(false);
      //pd_six.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_six.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_six.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_six.setValue("BaseBeanManager", );

      // ------------------- property "SeeWhy" --------------------
      PropertyDescriptor pd_nine = new PropertyDescriptor(
        "SeeWhy", PrivilegeForTypeImpl.class, "getSeeWhy", "setSeeWhy");
      pd_nine.setDisplayName("See Why");
      pd_nine.setShortDescription("To explain the role of this privilege");
      pd_nine.setBound(true);
      //pd_nine.setConstrained(false);
      //pd_nine.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_nine.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_nine.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_nine.setValue("BaseBeanManager", );

        // ------------------ end of from PrivilegeAbst ---------------------------

      // ------------------- property "Classed" --------------------
      PropertyDescriptor pd_ten = new PropertyDescriptor(
        "Classed", PrivilegeForTypeImpl.class, "getClassed", null);
      pd_ten.setDisplayName("Is classed");
      pd_ten.setShortDescription(
              "true if there are class contraints on sources and targets.");
      pd_ten.setBound(false);
      //pd_ten.setConstrained(false);
      //pd_ten.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_ten.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_ten.setValue("panelGroupName", "  Context  ");
      // 'tags' is the name (String value) of the couple of strings,
      // with a size limited to 12 caracters,
      // to be displayed when the property is true or false, in this order.
      // Cannot be empty if it exists, and the default value for the property
      // editor is ("TRUE", "False") when 'tags' does not exist.
      pd_ten.setValue("tags", new String[] {"YES", "No"} );

      // ------------------- property "Specific" --------------------
      PropertyDescriptor pd_eleven = new PropertyDescriptor(
        "Specific", PrivilegeForTypeImpl.class, "getSpecific", null);
      pd_eleven.setDisplayName("Is specific");
      pd_eleven.setShortDescription("true if there is one specific source"+
              " and one specific target.");
      pd_eleven.setBound(false);
      //pd_eleven.setConstrained(false);
      //pd_eleven.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eleven.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_eleven.setValue("panelGroupName", "  Context  ");
      // 'tags' is the name (String value) of the couple of strings,
      // with a size limited to 12 caracters,
      // to be displayed when the property is true or false, in this order.
      // Cannot be empty if it exists, and the default value for the property
      // editor is ("TRUE", "False") when 'tags' does not exist.
      pd_eleven.setValue("tags", new String[] {"YES", "No"} );

      // ------------------- property "ForSeed" --------------------
      PropertyDescriptor pd_twelve = new PropertyDescriptor(
        "ForSeed", PrivilegeForTypeImpl.class, "getForSeed", null);
      pd_twelve.setDisplayName("For Seed");
      pd_twelve.setShortDescription("true if the privilege "+
        "is managed in an authorization server, that is a special type of ACS.");
      pd_twelve.setBound(false);
      //pd_twelve.setConstrained(false);
      //pd_twelve.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_twelve.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_twelve.setValue("panelGroupName", "  Context  ");
      // 'tags' is the name (String value) of the couple of strings,
      // with a size limited to 12 caracters,
      // to be displayed when the property is true or false, in this order.
      // Cannot be empty if it exists, and the default value for the property
      // editor is ("TRUE", "False") when 'tags' does not exist.
      pd_twelve.setValue("tags", new String[] {"YES", "No"} );

      // ------------------- property "PrivilegeSourceClass" ------------------------
      PropertyDescriptor pd_thirteen = new PropertyDescriptor(
        "PrivilegeSourceClass", PrivilegeForTypeImpl.class,
        "getSourceClass", "setSourceClassOneTime");
      pd_thirteen.setDisplayName("Right User class");
      pd_thirteen.setShortDescription("The class of the sources; "+
              "the user cannot change them");
      pd_thirteen.setBound(true);
      //pd_thirteen.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirteen.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
            //pd_thirteen.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_thirteen.setValue("BaseBeanManager", );

      // ------------------- property "PrivilegeTargetClass" ------------------------
      PropertyDescriptor pd_fourteen = new PropertyDescriptor(
        "PrivilegeTargetClass", PrivilegeForTypeImpl.class,
        "getTargetClass", "setTargetClassOneTime");
      pd_fourteen.setDisplayName("Target class");
      pd_fourteen.setShortDescription("The class of the targets; "+
              "the user cannot change them");
      pd_fourteen.setBound(true);
      //pd_fourteen.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourteen.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
            //pd_fourteen.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_fourteen.setValue("BaseBeanManager", );

      // ------------------- property "SourceType" ------------------------
      PropertyDescriptor pd_fifteen = new PropertyDescriptor(
        "SourceType", PrivilegeForTypeImpl.class, "getSourceType", null);
      pd_fifteen.setDisplayName("Right User type");
      pd_fifteen.setShortDescription("The type of the sources, "+
              "as the single criterion to select the sources having this privilege");
      pd_fifteen.setBound(false);
      //pd_fifteen.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fifteen.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
            //pd_fifteen.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_fifteen.setValue("BaseBeanManager", );

      // ------------------- property "TargetType" ------------------------
      PropertyDescriptor pd_sixteen = new PropertyDescriptor(
        "TargetType", PrivilegeForTypeImpl.class, "getTargetType", null);
      pd_sixteen.setDisplayName("Target type");
      pd_sixteen.setShortDescription("The type of the targets, "+
              "as the single criterion to select the targets of this privilege");
      pd_sixteen.setBound(false);
      //pd_sixteen.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_sixteen.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
            //pd_sixteen.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
      //pd_sixteen.setValue("BaseBeanManager", );

      // ------------------- property "ExternalPrivilege" ------------------------
      PropertyDescriptor pd_seventeen = new PropertyDescriptor(
        "ExternalPrivilege", PrivilegeForTypeImpl.class, "isExternalPrivilege", null);
      pd_seventeen.setDisplayName("Is external");
      pd_seventeen.setShortDescription("true if the privilege is applicable "+
              "to a source or to a target from another ACS");
      pd_seventeen.setBound(true);
      //pd_seventeen.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seventeen.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_seventeen.setValue("panelGroupName", "  Context  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the BasicImpl instance
      // does it, or when is is not applicable.
        //pd_seventeen.setValue("BaseBeanManager", );
      // 'tags' is the name (String value) of the couple of strings,
      // with a size limited to 12 caracters,
      // to be displayed when the property is true or false, in this order.
      // Cannot be empty if it exists, and the default value for the property
      // editor is ("TRUE", "False") when 'tags' does not exist.
      pd_seventeen.setValue("tags", new String[] {"YES", "No"} );

      //
      //   ------------ End of properties definition ----------------------------
      //

      l_pd = new PropertyDescriptor[]
         // General
        {pd_one, pd_two, pd_four, pd_nine, pd_three, pd_six,
         // Rights
        pd_eight, pd_seven,
         // ForType
        pd_ten, pd_eleven, pd_seventeen, pd_twelve, pd_thirteen, pd_fourteen,
        pd_fifteen, pd_sixteen};

      return l_pd;

    } catch (IntrospectionException e) {
      new ProcessError("In PrivilegeForTypeImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In PrivilegeForTypeImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

    /**
     * Gets the instance, if it exists, which fires an event when
     * the bean is closed or deleted.
     * @return null.
     */
    public BaseBeanManager getBaseBeanManagerInstance() {
      return null;
    }

    /**
     * Gets the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, that is the ACS here,
     * which fires an event when the PrivilegeForLinks is closed or deleted.
     * @return the get method which provides the resource
     *  that manages the PrivilegeForLinks.
     */
    public Method getBaseBeanManagerGetMethod() {
      Object result = null;
      try {
        result = PrivilegeAbst.class.getMethod("getEorACS", (Class<?>[])null);
      } catch (NoSuchMethodException n) {
        new ProcessError("in PrivilegeForTypeImplBeanInfo."+
          "getBaseBeanManagerGetMethod(): "+n.toString());
      } catch (SecurityException s) {
      }
      return (Method) result;
    }

    /**
     * Gets the property name 'Privileges' for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * This is used to register the BeamerManager as a PropertyChangeListener.
     * @return the property name.
     */
    public String getBaseBeanManagerBoundProperty() {
      return new String("Privileges");
    }

} // PrivilegeForTypeImplBeanInfo