//Title:        Access Road
//Version:      0.6.0
//Copyright:    2002, 2004, 2007, 2008, 2009 EURL ACCBEE
//Author:       Patrick Thazard ( )
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.*;
import ARoad0.gBase.*;
import ARoad0.ErrW.ProcessError;

import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.lang.reflect.Method;

/**
 * This class is responsible for describing the StringMetaRightImpl class.
 * It is used for displaying and editing the properties of StringMetaRightImpl,
 * to create it and to copy it.
 * @version 0.6.0
 */
public class StringMetaRightImplBeanInfo extends StringRightImplBeanInfo {

  public StringMetaRightImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue().
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(StringMetaRight.class );
    result.setDisplayName("StringMetaRight");
      
    return getCommonBeanDescriptor(result);
  } // getBeanDescriptor()

  /**
   * Get the common description of the bean. Overwrite the StringRightBeanInfo method.
   * Used by subclasses.
   * @param _result is the BeanDescriptor to configure
   * @return the bean descriptor
   */
  protected BeanDescriptor getCommonBeanDescriptor(BeanDescriptor _result) {

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of StringMetaRightImpl
    _result.setValue("withTabbedPanes", Boolean.TRUE );

    // 'withDeleteMethod' is true to display the 'delete' button in the beamer
    _result.setValue("withDeleteMethod", Boolean.FALSE );

    // 'commonBeanClass' is the class which has the bean info defining getCommonBeanDescriptor(),
    // and then which is the class or the superclass of the bean
    _result.setValue("commonBeanClass", StringMetaRightImpl.class );
    
    // ----------------------- constructor parameters ---------------------------
    
    // 'constrArgumentClasses' is the list of argument classes
    // for the class constructor
    _result.setValue("constrArgumentClasses", new Class[]{
      String.class, Class.class, Integer.class, Integer.class, Integer.class,
      Boolean.class} );
    
    // 'constrArgumentPropertyNames' is the list of property names
    // for the class constructor
    _result.setValue("constrArgumentPropertyNames", new String[]{
      "Kernel Right", "Target Class", "Internal Key", "Power", "Field",
      "Granting Right"} );

    // 'constrArgumentFromSelect' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "constrArgumentPropertyNames", or, as here,
    // in the order of the constructor method in StringMetaRightImpl
    _result.setValue("constrArgumentFromSelect", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE,
      Boolean.FALSE} );

    // 'constrArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another constructor argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    _result.setValue("constrArgumentConcatenedNameSource", new Integer[]{
      null, null, null, null, null, null} );

    // 'constrNullArgumentPossible' indicates if an argument may be null in the constructor
    _result.setValue("constrNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE,
      Boolean.FALSE} );

    // 'constrOptionalArgument' indicates if an argument must not be used in the
    // call to the constructor when it is null.
    _result.setValue("constrOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE,
      Boolean.FALSE} );

    // 'constrRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names. Is optional.
    // If it is true, the array of restricted values for an argument is searched:
    // - first, in the ACS.getM_staticSetOfItems() map,
    // and then "constrArgumentPropertyNames" must contain the property descriptor name.
    // The name words may be separated by spaces in "constrArgumentPropertyNames",
    // although there is no separation in the ACS.getM_staticSetOfItems() map.
    // - second, in the value "staticSetOfItems" of the property in this BeanInfo.
    _result.setValue("constrRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE,
      Boolean.FALSE} );
    
    // 'constrNonVisibleArgument' indicates if an argument is masqued
    // in the constructor window and sets to null in the constructor.
    // True when the ACS does not manage the property, and then, 
    // this value gets the name of the ACS method which returns a boolean 'false'
    // when the argument has to be null and to be not displayed. If
    // the returned value is 'true', the argument is displayed as usual.
    // For each non null value:
    // "constrNullArgumentPossible" has to be Boolean.TRUE,
    // "constrOptionalArgument" has to be Boolean.FALSE.
    _result.setValue("constrNonVisibleArgument", new String[]{
      null, null, null, null, null, null} ); 

    // ----------------------- copy parameters TODO ---------------------------

    // 'copyArgumentClasses' is the list of argument classes
    // for the copy method, excepted for _toRegister at the end,
    //  which is another argument set to true.
    _result.setValue("copyArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class, Directory.class} );
      
    // 'copyArgumentPropertyNames' is the list of property names
    // for the copy method, excepted for _toRegister which is set to true
    // following the copy() pattern.
    _result.setValue("copyArgumentPropertyNames", new String[]{
      "Name", "ACS", "Parent Node"} );

    // 'copyArgumentFromExplorer' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "copyArgumentPropertyNames"
    _result.setValue("copyArgumentFromExplorer", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.TRUE} );

    // 'copyArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    _result.setValue("copyArgumentConcatenedNameSource", new Integer[]{
      new Integer(2), null, null} );

    // 'copyNullArgumentPossible' indicates if an argument may be null
    //  in the method
    _result.setValue("copyNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE} );

    // 'copyOptionalArgument' indicates if an argument must not be used
    //  in the call to the method when it is null.
    _result.setValue("copyOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'copyRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names.
    //  Is optional.
    // If it is true, the array of restricted values for an argument is searched:
    // - first, in the value "staticSetOfItems" of the property in this BeanInfo,
    // - second, in the ACS.getM_staticSetOfItems() map
    // and then "constrArgumentPropertyNames" must contain the property descriptor name
    // (but the name words may be separated by spaces in "constrArgumentPropertyNames").
    _result.setValue("copyRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );
      
    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    _result.setHidden(false);
    _result.setExpert(true);

    return _result;
  } // getCommonBeanDescriptor()

  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue().
   * To update a property, there are 3 means, in this order:
   * - if it is not a single value, use the "choiceMethodForNew" value to add one value
   * - or, if it is not a single value, use the "addMethod" value to add one value
   *  and "removeMethod", for BaseObjects only,
   * - or, in any cases, use the write method in the property descriptor
   *  <P>
   * Replace the 'isParent', 'isSlave' and 'Comment' properties in the super BeanInfo
   * to remove the set method in them.
   * Replace 'Right' and 'Internal key' to change the display name,
   * Remove the 'ACS', 'TargetClassType', 'isDesactivated' and 'GenericRight' properties.
   * Add the 'InFloatingSubNetwork', 'ACSRights' properties,
   * Move 'NickName', 'TargetClass' and 'isGrantingRight' to have a better look.
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {
    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_pd;  
    
    try {
      
      // ------------------- property "Right" ------------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "Right", StringRight.class, "getRight", null);
      pd_two.setDisplayName("Generic Right ");
      pd_two.setShortDescription("The operational granting or denying generic right ");
      //pd_two.setBound(false);
      //pd_two.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
            //pd_two.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringRightImpl instance
      // does it, or when is is not applicable.
      //pd_two.setValue("BaseBeanManager", );
        
        /*
      // ------------------- property "Power" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "Power", StringMetaRightImpl.class, "getPower", null);
      pd_one.setDisplayName("Power");
      pd_one.setShortDescription("is the line value in the matrix (power, field)"+ 
         " which helps to locate the metaright.");
      //pd_one.setBound(false);
      //pd_one.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_one.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringMetaRightImpl instance
      // does it, or when is is not applicable.
      //pd_one.setValue("BaseBeanManager", );
         * */
      
      /*
      // ------------------- property "Field" ------------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "Field", StringMetaRightImpl.class, "getField", null);
      pd_two.setDisplayName("Field");
      pd_two.setShortDescription("is the colomn value in the matrix (power, field) "+
         "which helps to locate the near metarights.");
      //pd_two.setBound(false);
      //pd_two.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
            //pd_two.setValue("panelGroupName", "  Rights  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringMetaRightImpl instance
      // does it, or when is is not applicable.
      //pd_two.setValue("BaseBeanManager", );
       * */
      
      // ------------------- property "ACSRights" ------------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "ACSRights", StringMetaRightImpl.class, "getM_ACSRights", null);
      pd_three.setDisplayName("Right ");
      pd_three.setShortDescription("For each ACS, the rights which are equivallent"+
          "to this generic right.");
      pd_three.setBound(true);
      //pd_three.setConstrained(false);
      // 'isRightMap' is TRUE when the right property editor is RightMapPropertyEditor
      pd_three.setValue("isRightMap", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, it defines the type of the elementary object to handle. 
      // If the map value is a collection or an array, the given type
      // is the type of the element in this collection or array.
      pd_three.setValue("valueTypeInGroup", new Object[] {StringRight.class} );
      pd_three.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_three.setValue("panelGroupName", "  Associated ACS rights  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringMetaRightImpl instance
      // does it, or when is is not applicable.
      //pd_three.setValue("BaseBeanManager", );
      // 'KeysUpdateInMap' is true if the key list may be updated. If it is false,
      // and if there is some updating methods, only the value list in the map
      // may be updated for any key.
      pd_three.setValue("KeysUpdateInMap", Boolean.FALSE);
      // 'withPreviousButton' is set to true to display a 'Previous' button
      // to copy all the values of the previous key in the map,
      // for this current key (and not 'property', like in other PropertyEditors),
      // following the order of the key descriptors returned by getKeys().
      pd_three.setValue("withPreviousButton", Boolean.FALSE );
      // 'withViewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'View' button if the property value (not key) is viewable
      // in the beamer. False if it is not editable.
      pd_three.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the RightMapPropertyEditor
      // has to display an 'Add New' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_three.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the RightMapPropertyEditor
      // has to display 2 'Select' buttons if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_three.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the RightMapPropertyEditor
      // has to display 2 'Delete' buttons if the property is viewable
      // in the beamer. If false, there are 'Remove' buttons.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_three.setValue("withRemoveButton", Boolean.FALSE );
      pd_three.setValue("withDeleteButton", Boolean.FALSE );    
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. No use, since the ACS defines the set.
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
        //pd_fiftyOne.setValue("staticSetOfItems", new String[0] );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      pd_three.setValue("limitedSizeForPresentation", new Integer(4));
      
      // ------------------- property "InFloatingSubNetwork" --------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "InFloatingSubNetwork", StringMetaRightImpl.class, "getInFloatingSubNetwork", null);
      pd_four.setDisplayName("Is in a subnetwork");
      pd_four.setShortDescription("A floating subnetwork is, for a specific ACS,"+
        " without any association to most of the metarights.");
      pd_four.setBound(true);
      //pd_four.setConstrained(false);
      //pd_four.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_four.setValue("panelGroupName", "  Hard Links  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringMetaRightImpl instance
      // does it, or when is is not applicable.
      //pd_four.setValue("BaseBeanManager", );
      
      // ------------------- property "isParent" --------------------
      // overwrites the RightString property to avoid any update
      PropertyDescriptor pd_thirteen = new PropertyDescriptor(
        "isParent", StringMetaRight.class, "isParent", null);
      pd_thirteen.setDisplayName("Is a parent right");
      pd_thirteen.setShortDescription("True if it is a parent in a two-level right");
      pd_thirteen.setBound(true);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirteen.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_thirteen.setValue("panelGroupName", "  Hard Links  " );
      
      // ------------------- property "isSlave" --------------------
      // overwrites the RightString property to avoid any update
      PropertyDescriptor pd_fourteen = new PropertyDescriptor(
        "isSlave", StringMetaRight.class, "isSlave", null);
      pd_fourteen.setDisplayName("Is a slave right");
      pd_fourteen.setShortDescription("True if it is a slave in a two-level right");
      pd_fourteen.setBound(true);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourteen.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_fourteen.setValue("panelGroupName", "  Hard Links  " );
      
      // ------------------- property "Internal Key" --------------------
      PropertyDescriptor pd_thirtyFive = new PropertyDescriptor(
        "InternalKey", StringRight.class, "getInternalKey", null);
      pd_thirtyFive.setDisplayName("Strength (1 to 99,999)");
      pd_thirtyFive.setShortDescription("Key used to compare rights.");
      pd_thirtyFive.setBound(false);
      //pd_thirtyFive.setConstrained(false);
      //pd_thirtyFive.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtyFive.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_thirtyFive.setValue("panelGroupName", "  Capacity  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the ResourceImpl instance
      // does it, or when is is not applicable.
      //pd_thirtyFive.setValue("BaseBeanManager", );       
 
      // ------------------- property "Comment" --------------------
      PropertyDescriptor pd_thirtySix = new PropertyDescriptor(
        "Comment", StringRight.class, "getComment", null);
      pd_thirtySix.setDisplayName("Comments ");
      pd_thirtySix.setShortDescription("From the Access Road team");
      pd_thirtySix.setBound(true);
      //pd_thirtySix.setConstrained(false);
      //pd_thirtySix.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtySix.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_four.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringRightImpl instance
      // does it, or when is is not applicable.
      //pd_thirtySix.setValue("BaseBeanManager", );
      
      // ------- End of properties description ---------------------------------

      // replace the 'isParent', 'isSlave' and 'Comment' properties in the super BeanInfo
      // to remove the set method in them,
      // replace 'Right' and 'Internal key' to change the display name,
      // remove the 'ACS', 'TargetClassType', 'isDesactivated' and 'GenericRight' properties,
      // add the 'InFloatingSubNetwork', 'ACSRights' properties,
      // move 'NickName', 'TargetClass' and 'isGrantingRight' to improve the look
      l_pdFromParent = super.getPropertyDescriptors();
      l_pd = new PropertyDescriptor[l_pdFromParent.length - 2];
      int count = 0;
      PropertyDescriptor nickName = null;
      PropertyDescriptor targetClass = null; 
      PropertyDescriptor isGrantingRight = null;
      
      // i from 1 to have pd_two at 0
      for (int i = 1; i < l_pdFromParent.length; i++) {
          if (l_pdFromParent[i].getName().equals("ACS")) {
              count++;
              continue;
          } // if
          else if (l_pdFromParent[i].getName().equals("Right")) {
              count++;
              continue;
          } // if
          else if (l_pdFromParent[i].getName().equals("NickName")) {
              count++;
              nickName = l_pdFromParent[i];
              continue;
          } // if
          else if (l_pdFromParent[i].getName().equals("TargetClass")) {
              count++;
              targetClass = l_pdFromParent[i];
              continue;
          } // if
          else if (l_pdFromParent[i].getName().equals("isGrantingRight")) {
              count++;
              isGrantingRight = l_pdFromParent[i];
              continue;
          } // if
          else if (l_pdFromParent[i].getName().equals("isParent")) {
              count++;
              continue;
          } // if
          else if (l_pdFromParent[i].getName().equals("isSlave")) {
              count++;
              continue;
          } // if  
          else if (l_pdFromParent[i].getName().equals("Comment")) {
              count++;
              continue;
          } // if 
          else if (l_pdFromParent[i].getName().equals("TargetClassType")) {
              count++;
              continue;
          } // if  
          else if (l_pdFromParent[i].getName().equals("InternalKey")) {
              count++;
              continue;
          } // if 
          else if (l_pdFromParent[i].getName().equals("GenericRight")) {
              count++;
              continue;
          } // if
          else if (l_pdFromParent[i].getName().equals("isDesactivated")) {
              count++;
              continue;
          } // if 
          else {
              l_pd[i - count] = l_pdFromParent[i];
          } // else
      } // for i

      l_pd[0] = pd_two;      
      l_pd[l_pd.length - 9] = nickName; 
      l_pd[l_pd.length - 8] = pd_thirtySix;       
      l_pd[l_pd.length - 7] = isGrantingRight;
      l_pd[l_pd.length - 6] = pd_four; 
      l_pd[l_pd.length - 5] = pd_three;
      l_pd[l_pd.length - 4] = pd_fourteen;
      l_pd[l_pd.length - 3] = pd_thirtyFive;
      l_pd[l_pd.length - 2] = pd_thirteen;
      //l_pd[l_pd.length - 3] = pd_one;
      //l_pd[l_pd.length - 3] = pd_two;     
      l_pd[l_pd.length - 1] = targetClass;       
      
      l_pdFromParent = null;
      nickName =  null;
      isGrantingRight = null;
      targetClass = null;
      return l_pd;
    } catch (IntrospectionException e) {
      new ProcessError("In StringMetaRightBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In StringMetaRightBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

//  public int getDefaultPropertyIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultPropertyIndex() not yet implemented.");
//  }
//  public MethodDescriptor[] getMethodDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getMethodDescriptors() not yet implemented.");
//  }
//  public BeanInfo[] getAdditionalBeanInfo() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getAdditionalBeanInfo() not yet implemented.");
//  }
//  public Image getIcon(int iconKind) {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getIcon() not yet implemented.");
//  }


    /**
     * Get the instance, if it exists, which fires an event when
     * the bean is closed or deleted.
     * @return null.since metarights cannot be deleted by the user
     */
    public BaseBeanManager getBaseBeanManagerInstance() {
      return null;
    }

    /**
     * Get the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, if it exists,
     * which fires an event when the bean instance is closed or deleted.
     * Return null when there is
     * an active BeanInfoPattern.getBaseBeanManagerInstance(),
     * or when it is not applicable.
     * @return the get method. Nul there .since metarights cannot be deleted by the user.
     */
    public Method getBaseBeanManagerGetMethod() {
      return null;
    }


    /**
     * Get the property name for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * This is used to register the BeamerManager as a PropertyChangeListener.
     * @return the property name. Is always null there.
     */
    public String getBaseBeanManagerBoundProperty() {
      return null;
    }


} // StringMetaRightBeanInfo
