//Title:        Access Road
//Version:      0.6.0
//Copyright:    2002, 2004, 2007, 2008, 2009 EURL ACCBEE
//Author:       Patrick Thazard ( )
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.*;
import ARoad0.gBase.*;
import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;

import java.beans.SimpleBeanInfo;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.lang.reflect.Method;

/**
 * This class is responsible for describing the StringRightImpl class.
 * It is used for displaying and editing the properties of StringRightImpl,
 * to create it and to copy it.
 * @version 0.6.0
 */
public class StringRightImplBeanInfo extends SimpleBeanInfo
  implements BeanInfoPattern {

  public StringRightImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue().
   * Used by BaseObjectHandler for constructing acsrights, and eventually
   * by subclasses, but not by StringMetaRightBeanInfo.
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(StringRight.class );
    result.setDisplayName("StringRight");

                // for testing and tracking
                //System.out.println("call to StringRightImplBeanInfo.getBeanDescriptor()");  
    
    return getCommonBeanDescriptor(result);
  } // getBeanDescriptor()

  /**
   * Get the common description of the bean.
   * Used by BaseObjectHandler for constructing acsrights, and eventually
   * by subclasses, but not by StringMetaRightBeanInfo.
   * @param _result is the BeanDescriptor to configure
   * @return the bean descriptor
   */
  protected BeanDescriptor getCommonBeanDescriptor(BeanDescriptor _result) {

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of StringRightImpl
    _result.setValue("withTabbedPanes", Boolean.TRUE );

    // 'withDeleteMethod' is true to display the 'delete' button in the beamer
    //_result.setValue("withDeleteMethod", Boolean.TRUE );

    // 'commonBeanClass' is the class which has the bean info defining getCommonBeanDescriptor(),
    // and then which is the class or the superclass of the bean
    _result.setValue("commonBeanClass", StringRightImpl.class );
    
    // ----------------------- constructor parameters ---------------------------
  
    /*
  public StringRightImpl(String _kernelRight,
    StringMetaRightImpl _meta, Integer _internalKey, StringRightImpl[] _l_upperRights,
    String _targetClass, String _targetClassType, ACSRun _acsRun)
     */
    
    
    // 'constrArgumentClasses' is the list of argument classes
    // for the class constructor
    _result.setValue("constrArgumentClasses", new Class[]{
      String.class, StringMetaRight.class, Integer.class, StringRightImpl[].class,
      String.class, ACSRun.class} );
    
    // 'constrArgumentPropertyNames' is the list of property names
    // for the class constructor
    _result.setValue("constrArgumentPropertyNames", new String[]{
      "Right", "Generic Right", "Internal Key", "Upper Rights",
      "Target Class Type", "ACS"} );

    // 'constrArgumentFromSelect' is the list of cases
    // where the argument is selected from the explorer or a special method,
    // in the order of "constrArgumentPropertyNames", or, as here,
    // in the order of the constructor method in StringRightImpl
    _result.setValue("constrArgumentFromSelect", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.FALSE, Boolean.TRUE,
      Boolean.FALSE, Boolean.FALSE} );

    // 'constrArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another constructor argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    _result.setValue("constrArgumentConcatenedNameSource", new Integer[]{
      null, null, null, null, null, null} );

    // 'constrNullArgumentPossible' indicates if an argument may be null in the constructor
    _result.setValue("constrNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.TRUE,
      Boolean.TRUE, Boolean.FALSE} );

    // 'constrOptionalArgument' indicates if an argument must not be used in the
    // call to the constructor when it is null.
    _result.setValue("constrOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE,
      Boolean.FALSE, Boolean.FALSE} );

    // 'constrRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names. Is optional.
    // If it is true, the array of restricted values for an argument is searched
    // through BaseUtilityImpl.getRestrictedValues():
    // - first, in 3 ACS maps which define the constraints from the ACS,
    // and then "constrArgumentPropertyNames" must contain the property descriptor name.
    // The name words may be separated by spaces in "constrArgumentPropertyNames",
    // although there is no separation in the ACS.getM_staticSetOfItems() map.
    // - second, in the value "staticSetOfItems" of the property in this BeanInfo.
    _result.setValue("constrRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.TRUE,
      Boolean.FALSE, Boolean.FALSE} );
    
    // 'constrNonVisibleArgument' indicates if an argument is masqued
    // in the constructor window and sets to null in the constructor.
    // True when the ACS does not manage the property, and then, 
    // this value gets the name of the ACS method which returns a boolean 'false'
    // when the argument has to be null and to be not displayed. If
    // the returned value is 'true', the argument is displayed as usual.
    // For each non null value:
    // "constrNullArgumentPossible" has to be Boolean.TRUE,
    // "constrOptionalArgument" has to be Boolean.FALSE.
    _result.setValue("constrNonVisibleArgument", new String[]{
      null, null, null, null, null, null} );
    
    // Only if 'constrRestrictedArgument' is true for an argument,
    // 'constrRestrictedFromOtherArgumentMessage' is optionally not null to give to the user
    // a message, when he needs to enter another argument to allow the definition
    // of the constraints for this argument.
    // Exemple of message: "<Set the ACS first>" from the value 'ACS'.
    // For each non null value:
    // "constrNonVisibleArgument" has to be Boolean.TRUE.
    _result.setValue("constrRestrictedFromOtherArgumentMessage", new String[]{
      null, null, null, "Strength", null, null} );
    
    // ----------------------- copy parameters TODO ---------------------------

    // 'copyArgumentClasses' is the list of argument classes
    // for the copy method, excepted for _toRegister at the end,
    //  which is another argument set to true.
    _result.setValue("copyArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class, Directory.class} );
      
    // 'copyArgumentPropertyNames' is the list of property names
    // for the copy method, excepted for _toRegister which is set to true
    // following the copy() pattern.
    _result.setValue("copyArgumentPropertyNames", new String[]{
      "Name", "ACS", "Parent Node"} );

    // 'copyArgumentFromExplorer' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "copyArgumentPropertyNames"
    _result.setValue("copyArgumentFromExplorer", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.TRUE} );

    // 'copyArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    _result.setValue("copyArgumentConcatenedNameSource", new Integer[]{
      new Integer(2), null, null} );

    // 'copyNullArgumentPossible' indicates if an argument may be null
    //  in the method
    _result.setValue("copyNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE} );

    // 'copyOptionalArgument' indicates if an argument must not be used
    //  in the call to the method when it is null.
    _result.setValue("copyOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'copyRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names.
    //  Is optional.
    // If it is true, the array of restricted values for an argument is searched:
    // - first, in the value "staticSetOfItems" of the property in this BeanInfo,
    // - second, in the ACS.getM_staticSetOfItems() map
    // and then "constrArgumentPropertyNames" must contain the property descriptor name
    // (but the name words may be separated by spaces in "constrArgumentPropertyNames").
    _result.setValue("copyRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );
      
    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    _result.setHidden(false);
    _result.setExpert(true);

    return _result;
  } // getCommonBeanDescriptor()

  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue().
   * The PropertyDescriptors are based on StringRight.class rather than StringRightImpl.class
   * to allow StringMetaRightBeanInfo to reuse the properties excepting 'ACS'.
   * <P>
   * To update a property, there are 3 means, in this order:
   * - if it is not a single value, use the "choiceMethodForNew" value to add one value
   * - or, if it is not a single value, use the "addMethod" value to add one value
   *  and "removeMethod", for BaseObjects only,
   * - or, in any cases, use the write method in the property descriptor
   *  <P>
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pd;  
    
    try {

      // ------------------- property "ACS" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "ACS", StringRight.class, "getAcsName", null);
      pd_one.setDisplayName("ACS ");
      pd_one.setShortDescription("The access control system which manages this right,"+
              " if not a metaright");
      //pd_one.setBound(false);
      //pd_one.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_one.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringRightImpl instance
      // does it, or when is is not applicable.
      //pd_one.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_one.setValue("withViewButton", Boolean.TRUE );
      
      // ------------------- property "Right" ------------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "Right", StringRight.class, "getRight", null);
      pd_two.setDisplayName("ACS Right ");
      pd_two.setShortDescription("The operational granting or denying ACS right ");
      //pd_two.setBound(false);
      //pd_two.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
            //pd_two.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringRightImpl instance
      // does it, or when is is not applicable.
      //pd_two.setValue("BaseBeanManager", );
      
      // ------------------- property "NickName" ------------------------
      PropertyDescriptor pd_thirtySix = new PropertyDescriptor(
        "NickName", StringRight.class, "getNickName", null);
      pd_thirtySix.setDisplayName("Nick Name ");
      pd_thirtySix.setShortDescription("The shorter name used in views ");
      //pd_thirtySix.setBound(false);
      //pd_thirtySix.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtySix.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
            //pd_thirtySix.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringRightImpl instance
      // does it, or when is is not applicable.
      //pd_thirtySix.setValue("BaseBeanManager", );
      
      // ------------------- property "Internal Key" --------------------
      PropertyDescriptor pd_thirtyFive = new PropertyDescriptor(
        "InternalKey", StringRight.class, "getInternalKey", null);
      pd_thirtyFive.setDisplayName("Strength (1 to 999)");
      pd_thirtyFive.setShortDescription("Key used to compare rights.");
      pd_thirtyFive.setBound(false);
      //pd_thirtyFive.setConstrained(false);
      //pd_thirtyFive.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirtyFive.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_thirtyFive.setValue("panelGroupName", "  Force  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the ResourceImpl instance
      // does it, or when is is not applicable.
      //pd_thirtyFive.setValue("BaseBeanManager", ); 
      
      // ------------------- property "TargetClass" ------------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "TargetClass", StringRight.class, "getTargetClass", null);
      pd_three.setDisplayName("Target Class ");
      pd_three.setShortDescription("The class of the target on which"+
        " the right may be applicable");
      //pd_three.setBound(false);
      //pd_three.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_three.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringRightImpl instance
      // does it, or when is is not applicable.
        //pd_three.setValue("BaseBeanManager", );
      // 'staticSetOfItems' is the array of possible values for the property,
      // The type of the array has to be the type of the returned elements
      // in the 'get' property method. 
      // Currently, it must be a string because BaseUtilityImpl.getEorBaseObject(_name)
      // is restricted to that, and it is not tested otherwise.
      // Caution: the first value may be 'null' as the default value.
        //pd_three.setValue("staticSetOfItems", new String[] {
        //null, "ARoad0.gBaseInterface.Resource",
        //"ARoad0.gBaseInterface.Directory", "ARoad0.gBaseInterface.Actor",
        //"ARoad0.gBaseInterface.Executable", "ARoad0.gBaseInterface.EligibleParty",
        //"ARoad0.gBaseInterface.UserID", "ARoad0.gBaseInterface.GroupIDMember",
        //"ARoad0.gBaseInterface.GroupID", "ARoad0.gBaseInterface.ACS"} );      

      // ------------------- property "Comment" --------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "Comment", StringRight.class, "getComment", "setComment");
      pd_four.setDisplayName("Comments ");
      pd_four.setShortDescription("user comments");
      pd_four.setBound(true);
      //pd_four.setConstrained(false);
      //pd_four.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_four.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringRightImpl instance
      // does it, or when is is not applicable.
      //pd_four.setValue("BaseBeanManager", );

      // ------------------- property "TargetClassType" --------------------
      //  This property is controlled in the StringRightImpl constructor
      //  from the values that the ACS allows for the target class.
      PropertyDescriptor pd_five = new PropertyDescriptor(
        "TargetClassType", StringRight.class, "getTargetClassType", null);
      pd_five.setDisplayName("Type, only if classed ");
      pd_five.setShortDescription("If the Generic Right is classed, the type on which "+
        "the right is applicable");
      //pd_five.setBound(false);
      //pd_five.setConstrained(false);
      //pd_five.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_five.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_five.setValue("panelGroupName", "  Group and user owners  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringRightImpl instance
      // does it, or when is is not applicable.
      //pd_five.setValue("BaseBeanManager", );

      // ------------------- property "MetaRight" --------------------
      // BeamerManager.getNoDisplayThisProperty() leaves this property for metarights
      PropertyDescriptor pd_six = new PropertyDescriptor(
        "GenericRight", StringRight.class, "getEorMetaRight", null);
      pd_six.setDisplayName("Generic Right");
      pd_six.setShortDescription("The generic right (also called metaright) linked to this right,"+
              " for 'all' objects or 'classed' for one interface.");
      //pd_six.setBound(false);
      //pd_six.setConstrained(false);
      //pd_six.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_six.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_six.setValue("panelGroupName", "  Group and user owners  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the StringRightImpl instance
      // does it, or when is is not applicable.
      //pd_six.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_six.setValue("withViewButton", Boolean.TRUE );
      
      // ------------------- property "LowerRights" --------------------
      PropertyDescriptor pd_seven = new PropertyDescriptor(
        "LowerRights", StringRight.class, "getLowerRights", null);
      pd_seven.setDisplayName("Lower Rights ");
      pd_seven.setShortDescription("The less-powerfull rights"+
        " which are directly connected to this right.");
      pd_seven.setBound(true);
      //pd_seven.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_seven.setValue("isCollection", Boolean.TRUE );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_seven.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      // caution: "panelGroupName" is superior to "tabbedPaneName" in the BeamerManager analysis.
      pd_seven.setValue("panelGroupName", "  Rights Network  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when is is not applicable.
          //pd_seven.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_seven.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the PropertyEditor
      // has to display a New button if the property is viewable
      // in the beamer.
      pd_seven.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_seven.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_seven.setValue("withDeleteButton", Boolean.FALSE );

      // ------------------- property "UpperRights" --------------------
      PropertyDescriptor pd_eight = new PropertyDescriptor(
        "UpperRights", StringRight.class, "getUpperRights", null);
      pd_eight.setDisplayName("Upper Rights ");
      pd_eight.setShortDescription("The more-powerfull rights"+
        " which are directly connected to this right.");
      pd_eight.setBound(true);
      //pd_eight.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_eight.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_eight.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eight.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      // caution: "panelGroupName" is superior to "tabbedPaneName" in the BeamerManager analysis.
      pd_eight.setValue("panelGroupName", "  Rights Network  ");
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the instance
      // does it, or when is is not applicable.
        //pd_eight.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_eight.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_eight.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_eight.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_eight.setValue("withDeleteButton", Boolean.FALSE );

      /*
      // ------------------- property "isTargetClassed" --------------------
      PropertyDescriptor pd_nine = new PropertyDescriptor(
        "isTargetClassed", StringRight.class, "isTargetClassed", null);
      pd_nine.setDisplayName("Have a target class");
      pd_nine.setShortDescription("True if the target has to be a given class");
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_nine.setValue("tabbedPaneName", "  Rights  " );
      
      // ------------------- property "isTargetClassTyped" --------------------
      PropertyDescriptor pd_ten = new PropertyDescriptor(
        "isTargetClassTyped", StringRight.class, "isTargetClassTyped", null);
      pd_ten.setDisplayName("Have a target type");
      pd_ten.setShortDescription("True if the target has to be a given type in its class");
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.  
      pd_ten.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_ten.setValue("panelGroupName", "  Hard Links  " );
      */
      // ------------------- property "isDesactivated" --------------------
      PropertyDescriptor pd_eleven = new PropertyDescriptor(
        "isDesactivated", StringRight.class, "isDesactivated", "setDesactivated");
      pd_eleven.setDisplayName("Not workable");
      pd_eleven.setShortDescription("True if the right is not workable - CAUTION: changes"+
              " do not produce immediate updates in the open views.");
      pd_eleven.setBound(true);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer. 
      pd_eleven.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_eleven.setValue("panelGroupName", "  Force  " );
      
      // ------------------- property "isEmpty" --------------------
      //PropertyDescriptor pd_twelve = new PropertyDescriptor(
        //"isEmpty", StringRight.class, "isEmpty", null);
      //pd_twelve.setDisplayName("is empty");
      //pd_twelve.setShortDescription("True if the right is empty");
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer. 
      //pd_twelve.setValue("tabbedPaneName", "  Rights  " );

      // ------------------- property "isParent" --------------------
      PropertyDescriptor pd_thirteen = new PropertyDescriptor(
        "isParent", StringRight.class, "isParent", "setParent");
      pd_thirteen.setDisplayName("Is a parent right");
      pd_thirteen.setShortDescription("True if it is a parent in a two-level right");
      pd_thirteen.setBound(true);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_thirteen.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_thirteen.setValue("panelGroupName", "  Hard Links  " );
      
      // ------------------- property "isSlave" --------------------
      PropertyDescriptor pd_fourteen = new PropertyDescriptor(
        "isSlave", StringRight.class, "isSlave", "setSlave");
      pd_fourteen.setDisplayName("Is a slave right");
      pd_fourteen.setShortDescription("True if it is a slave in a two-level right");
      pd_fourteen.setBound(true);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fourteen.setValue("tabbedPaneName", "  Rights  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_fourteen.setValue("panelGroupName", "  Hard Links  " );
      
      // ------------------- property "isGrantingRight" --------------------
      PropertyDescriptor pd_fifteen = new PropertyDescriptor(
        "isGrantingRight", StringRight.class, "isGrantingRight", null);
      pd_fifteen.setDisplayName("Is a granting right");
      pd_fifteen.setShortDescription("True if the right is positive");
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_fifteen.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_fifteen.setValue("panelGroupName", "  Force  " );
      
      // ------- End of properties description ---------------------------------

      // The order in l_pd defines the display finest order, plus the values in
      // "tabbedPaneName" and "panelGroupName" for each property
      l_pd = new PropertyDescriptor[]
        {

          pd_two, pd_thirtySix, pd_four, pd_fifteen, pd_eleven, 
          pd_three, pd_five, pd_six,
          // Rights
          pd_thirteen, pd_thirtyFive, pd_fourteen,
          pd_eight, pd_seven, pd_one     
      };
      return l_pd;
    } catch (IntrospectionException e) {
      new ProcessError("In StringRightBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In StringRightBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

//  public int getDefaultPropertyIndex() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getDefaultPropertyIndex() not yet implemented.");
//  }
//  public MethodDescriptor[] getMethodDescriptors() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getMethodDescriptors() not yet implemented.");
//  }
//  public BeanInfo[] getAdditionalBeanInfo() {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getAdditionalBeanInfo() not yet implemented.");
//  }
//  public Image getIcon(int iconKind) {
    /**@todo: Implement this java.beans.BeanInfo method*/
//    throw new java.lang.UnsupportedOperationException("Method getIcon() not yet implemented.");
//  }


    /**
     * Get the instance, if it exists, which fires an event when
     * the bean is closed or deleted.
     * @return null since getBaseBeanManagerGetMethod() gives the data
     */
    public BaseBeanManager getBaseBeanManagerInstance() {
      return null;
    }

    /**
     * Get the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, if it exists,
     * which fires an event when the bean instance is closed or deleted.
     * @return the get method. Is null when there is
     *  an active BeanInfoPattern.getBaseBeanManagerInstance(),
     *  or when it is not applicable.
     */
    public Method getBaseBeanManagerGetMethod() {
      Object result = null;
      try {
        result = StringRightImpl.class.getMethod("getEorACS", (Class<?>[])null);
      } catch (NoSuchMethodException n) {
        new ProcessError("in StringRightImplBeanInfo."+
          "getBaseBeanManagerGetMethod(): "+n.toString());
      } catch (SecurityException s) {
      }
      return (Method) result;
    }


    /**
     * Get the property name for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * This is used to register the BeamerManager as a PropertyChangeListener.
     * @return the property name.
     */
    public String getBaseBeanManagerBoundProperty() {
      return new String("OneRight");
    }


} // StringRightBeanInfo