//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBase.*;
import ARoad0.gBaseInterface.UserID;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;

import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the UserIDImpl class.
 * It is used for displaying and editing the properties of UserIDImpl.
 */
public class UserIDImplBeanInfo extends GroupIDMemberImplBeanInfo {

  public UserIDImplBeanInfo() {}

  /**
   * Get the description of the bean.
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue():
   * - 'withTabbedPanes' is true (Boolean value) since several tabbed panes
   *  are necessary to display all the panels (property, value) of UserIDImpl
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(UserID.class );
    result.setDisplayName("UserID");

    result = getCommonBeanDescriptor(result);

    return result;
  } // getBeanDescriptor()

  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue()
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pdFromParent;
    PropertyDescriptor[] l_result;

    try {

      // ------------------- property "l_UserIDCurrentActors" --------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "l_UserIDCurrentActors", UserIDImpl.class, "getEorL_UserIDCurrentActors", null);

      pd_one.setDisplayName("Current actors ");
      pd_one.setShortDescription("The actors currently attached to this userID.");
      pd_one.setBound(true);
      //pd_one.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_one.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_one.setValue("valueTypeInGroup", new Object[] {Actor.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  AG  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_one.setValue("panelGroupName", "  Links  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_one.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_one.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_one.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_one.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_one.setValue("withDeleteButton", Boolean.FALSE );


      // ------------------- property "l_UserIDOwnResources" --------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "l_UserIDOwnResources", UserIDImpl.class, "getEorL_UserIDOwnResources", null);

      pd_two.setDisplayName("Own resources ");
      pd_two.setShortDescription("The resources in the ACS which are own by this userID,"+
        "or the actors which have this userID currently.");
      pd_two.setBound(true);
      //pd_two.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_two.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_two.setValue("valueTypeInGroup", new Object[] {Resource.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  Resources  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_two.setValue("panelGroupName", "  Links  " );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_two.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_two.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_two.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_two.setValue("withDeleteButton", Boolean.FALSE );

      // ------------------- property "PrimGroup" --------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "PrimGroup", UserIDImpl.class, "getEorPrimGroup", "setEorPrimGroup");
      pd_three.setDisplayName("Primary group ");
      pd_three.setShortDescription("The primary group of the userID");
      pd_three.setBound(true);
      //pd_three.setConstrained(false);
      //pd_three.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  AG  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_three.setValue("panelGroupName", "  Main attributes  " );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_three.setValue("withViewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_three.setValue("withSelectButton", Boolean.TRUE );
      // 'withClearButton' is the boolean which set if the PropertyEditor
      // has to display a Clear button
      pd_three.setValue("withClearButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be fixed
      // when a new instance is created through the 'New' button.
      // The string 'this' is used to point to the object in the beamer.
      // Caution: the known arguments have to be Objects.
      //pd_three.setValue("withKnownArgumentsForNew", new Object[] {null, "ACS", null} );

      // ------------------- property "Order" --------------------
      PropertyDescriptor pd_five = new PropertyDescriptor(
        "Order", UserIDImpl.class, "getOrder", "setOrder");
      pd_five.setDisplayName("UID number ");
      pd_five.setShortDescription("The UID may determine the priority order in the rights.");
      pd_five.setBound(true);
      //pd_five.setConstrained(false);
      //pd_five.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_five.setValue("tabbedPaneName", "  AG  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_five.setValue("panelGroupName", "  Main attributes  " );

      //
      // ------------End of UserIDImpl properties definition --------------------
      //

      // start from the parent class bean info, and
      // put the inner descriptor after the 8 first ones for the 'General' pane,
      // to put the 'AG' pane at the third place
      l_pdFromParent = super.getPropertyDescriptors();
      l_result = new PropertyDescriptor[l_pdFromParent.length + 4];
      System.arraycopy(l_pdFromParent, 0, l_result, 0, 8);
      l_result[8] = pd_one;
      l_result[9] = pd_three;
      l_result[10] = pd_five;
      System.arraycopy(l_pdFromParent, 8, l_result, 11, l_pdFromParent.length - 8);
      l_result[l_result.length - 1] = pd_two;            

      l_pdFromParent = null;
      return l_result;
    } catch (IntrospectionException e) {
      new ProcessError("In UserIDImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In UserIDImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

} // UserIDImplBeanInfo