//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBaseInterface.BaseBeanManager;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBase.*;
import java.beans.SimpleBeanInfo;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.lang.reflect.Method;

import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the ViewInBaseImpl class.
 * It is used for displaying and editing the properties of ViewInBaseImpl.
 * There is no BeanInfo for the abstract super class FolderAbst.
 * @version 0.7.0
 * @since 0.5.1
 */
public class ViewInBaseImplBeanInfo extends SimpleBeanInfo implements BeanInfoPattern {

  public ViewInBaseImplBeanInfo() {}

  /**
   * Gets the description of the bean.
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue().
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
    BeanDescriptor result = new BeanDescriptor(ViewInBaseImpl.class );
    result.setDisplayName("Basic View");

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of ViewInBaseImpl
    result.setValue("withTabbedPanes", Boolean.TRUE );

    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    result.setHidden(false);
    result.setExpert(false);

    return result;
  } // getBeanDescriptor()

  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue()
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pd;

    try {
      // ------------------- property "Name" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "Name", ViewInBaseImpl.class, "getName", null);
      pd_one.setDisplayName("Name ");
      //pd_one.setShortDescription("   ");
      //pd_one.setBound(false);
      //pd_one.setConstrained(false);
      //pd_one.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_one.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ViewInBaseImpl instance
      // does it, or when is is not applicable.
      //pd_one.setValue("BaseBeanManager", null);

      // ------------------- property "ConnectedACS" --------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "ConnectedACS", ViewInBaseImpl.class, "getEorL_ConnectedACSNames", null);
      // caution: this descriptor name is used in BeamerManager to forbid the
      // property      
      pd_two.setDisplayName("Connected ACS ");
      pd_two.setShortDescription("The ACS which have objects in the view.");
      pd_two.setBound(true);
        //pd_two.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_two.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_two.setValue("valueTypeInGroup", new Object[] {ImmutableName.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_two.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_two.setValue("panelGroupName", "  Links in ACL, as rights user  ");
      // 'withViewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_two.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_two.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable, if it is a String list,
      // or if the only write method is a removing method.
      pd_two.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_two.setValue("withDeleteButton", Boolean.FALSE );

      // ------------------- property "OneBaseObjectInFolder" --------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "OneBaseObjectInFolder", ViewInBaseImpl.class, "getEorM_BaseObjects", null);
      // caution: this descriptor name is used in BeamerManager to forbid the
      // Select and Remove buttons
      pd_three.setDisplayName("Elements ");
      pd_three.setShortDescription("The base objects which are elements of this view.");
      pd_three.setBound(true);
      //pd_three.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_three.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_three.setValue("valueTypeInGroup", new Object[] {BaseObject.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_three.setValue("panelGroupName", "    " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_three.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_three.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_three.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'. 'Select' implies 'Remove'.
      pd_three.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes automatically with 'Select'
      // when the property is an external object reference.
      pd_three.setValue("withRemoveButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be preset - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the known argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
        //pd_three.setValue("withKnownArgumentsForNew", new Object[] {
          //null, null, null, null, "this"} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_three.setValue("addMethod", "addEorBaseObject");
          //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty", "Rights"} );
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_three.setValue("removeMethod", "removeEorBaseObject");
            //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty"} );
      // Following the KeyPropertiesInName pattern,
      // 'removeArgumentsFromName' defines how to find the right arguments for
      // the 'remove' method for the property, when the name of the object
      // to remove is known, and when finalizeForUser() is not used.
      // If "removeArgumentsFromName" is null, the remove method is invoked
      // with the object to remove as single argument.
      // The returned array defines:
      // - the name of the class to call, at the index 0
      // - the name of the static method to call, at the index 1, with
      //    the name of the object to remove as argument, and with
      //    an Object[] array as returned value
      // - in the Object[] array returned as value by the static method,
      //    the sequence of index where are the n arguments to use
      //    in the remove method, in the correct order
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        // pd_three.setValue("removeArgumentsFromName", ... );

      // ------------------- property "Comment" --------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "Comment", ViewInBaseImpl.class, "getComment", "setComment");
      pd_four.setDisplayName("Comments ");
      pd_four.setShortDescription("free user comments");
      pd_four.setBound(true);
      //pd_four.setConstrained(false);
      //pd_four.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_four.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the ResourceImpl instance
      // does it, or when is is not applicable.
      //pd_four.setValue("BaseBeanManager", );

      // ------------------- property "CompleteResults" --------------------
      PropertyDescriptor pd_five = new PropertyDescriptor(
        "CompleteResults", ViewInBaseImpl.class, "getCompleteResults", null);
      pd_five.setDisplayName("Complete results");
      pd_five.setShortDescription("false if at least one ACS structure modeling "+
        "is incomplete for an used ACS");
      pd_five.setBound(true);
      //pd_five.setConstrained(false);
      //pd_five.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_five.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      pd_five.setValue("panelGroupName", "  Complete results if all the ACS "+
              "have complete behavior and objects list " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the ACSImpl instance
      // does it, or when is is not applicable.
      //pd_five.setValue("BaseBeanManager", null);

      l_pd = new PropertyDescriptor[] {pd_one, pd_three, pd_four, pd_five, pd_two};

      return l_pd;
    } catch (IntrospectionException e) {
      new ProcessError("In ViewInBaseImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In ViewInBaseImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

    /**
     * Gets the instance, if it exists, which fires an event when
     * the bean is closed or deleted.
     * @return a BaseManager object.
     */
    public BaseBeanManager getBaseBeanManagerInstance() {
      return BaseManagerImpl.getInstance();
    }

    /**
     * Gets the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, if it exists,
     * which fires an event when the bean instance is closed or deleted.
     * @return null
     */
    public Method getBaseBeanManagerGetMethod() {
      return null;
    }

    /**
     * Gets the property name for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * @return "OpenViews"
     */
    public String getBaseBeanManagerBoundProperty() {
      return "OpenViews";
    }


} // ViewInBaseImplBeanInfo