//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseBeanInfo;

import ARoad0.gBase.VirtualFolderImpl;
import ARoad0.gBaseInterface.*;
import java.beans.SimpleBeanInfo;
import java.beans.BeanDescriptor;
import java.beans.PropertyDescriptor;
import java.beans.IntrospectionException;
import java.lang.reflect.Method;

import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;

/**
 * This class is responsible for describing the VirtualFolderImpl class.
 * It is used for displaying and editing the properties of VirtualFolderImpl.
 * There is no BeanInfo for the abstract super class FolderAbst.
 * @version 0.7.0
 */
public class VirtualFolderImplBeanInfo extends SimpleBeanInfo implements BeanInfoPattern {

  public VirtualFolderImplBeanInfo() {}

  /**
   * Gets the description of the bean.
   * Specific values are set in the descriptor. They are get
   * with BeanDescriptor.getValue().
   * @return the bean descriptor
   */
  public BeanDescriptor getBeanDescriptor() {
            // used by BaseUtilityImpl.getSuperInterfacesFromBeanInfo()
    BeanDescriptor result = new BeanDescriptor(VirtualFolder.class );
    result.setDisplayName("Virtual Folder");

    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    result.setHidden(false);
    result.setExpert(false);

    return getCommonBeanDescriptor(result);
  } // getBeanDescriptor()

  /**
   * Get the common description of the bean.
   * Used by the subclasses.
   * @param _result is the BeanDescriptor to configure
   * @return the bean descriptor
   */
  protected BeanDescriptor getCommonBeanDescriptor(BeanDescriptor _result) {

    // 'withTabbedPanes' is true since several tabbed panes are necessary
    // to display all the panels (property, value) of ResourceImpl
    _result.setValue("withTabbedPanes", Boolean.TRUE );

    // 'withDeleteMethod' is true to display the 'delete' button in the beamer
    _result.setValue("withDeleteMethod", Boolean.TRUE );

    // 'commonBeanClass' is the class which has the bean info defining getCommonBeanDescriptor(),
    // and then which is the class or the superclass of the bean.
    _result.setValue("commonBeanClass", VirtualFolderImpl.class );
    
    // ----------------------- constructor parameters ---------------------------

    // 'constrArgumentClasses' is the list of argument classes
    // for the class constructor
    _result.setValue("constrArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class, VirtualFolder.class, String.class, Boolean.class} );
    
    // 'constrArgumentPropertyNames' is the list of property names
    // for the class constructor
    _result.setValue("constrArgumentPropertyNames", new String[]{
      "Name", "ACS", "Parent Node", "Virtual Folder Type", "Main VF Type"} );

    // 'constrArgumentFromSelect' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "constrArgumentPropertyNames", or, as here,
    // in the order of the constructor method
    _result.setValue("constrArgumentFromSelect", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE, Boolean.FALSE, Boolean.FALSE} );

    // 'constrArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another constructor argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    _result.setValue("constrArgumentConcatenedNameSource", new Integer[]{
      new Integer(2), null, null, null, null} );

    // 'constrNullArgumentPossible' indicates if an argument may be null in the constructor
    _result.setValue("constrNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE, Boolean.FALSE, Boolean.FALSE} );

    // 'constrOptionalArgument' indicates if an argument must not be used in the
    // call to the constructor when it is null.
    _result.setValue("constrOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'constrRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names. Is optional.
    // If it is true, the array of restricted values for an argument is found
    // through BaseUtilityImpl.getRestrictedValues():
    // - first, in 3 ACS maps which define the constraints from the ACS,
    // and then "constrArgumentPropertyNames" must contain the property descriptor name.
    // The name words may be separated by spaces in "constrArgumentPropertyNames",
    // although there is no separation in the ACS map.
    // - second, in the value "staticSetOfItems" of the property in this BeanInfo.
    _result.setValue("constrRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE, Boolean.TRUE, Boolean.FALSE} );
    
    // 'constrNonVisibleArgument' indicates if an argument is masqued
    // in the constructor window and sets to null in the constructor.
    // True when the ACS does not manage the property, and then, 
    // this value gets the name of the ACS method which returns a boolean 'false'
    // when the argument has to be null and to be not displayed. If
    // the returned value is 'true', the argument is displayed as usual.
    // For each non null value:
    // "constrNullArgumentPossible" has to be Boolean.TRUE,
    // "constrOptionalArgument" has to be Boolean.FALSE.
    _result.setValue("constrNonVisibleArgument", new String[]{
      null, null, "getManageVirtualFolderTrees", null, null} );

    // ----------------------- copy parameters ---------------------------

    // 'copyArgumentClasses' is the list of argument classes
    // for the copy method, excepted for _toRegister at the end,
    //  which is another argument set to true.
    _result.setValue("copyArgumentClasses", new Class[]{
      ImmutableName.class, ACS.class, VirtualFolder.class} );
      
    // 'copyArgumentPropertyNames' is the list of property names
    // for the copy method, excepted for _toRegister which is set to true
    // following the copy() pattern.
    _result.setValue("copyArgumentPropertyNames", new String[]{
      "Name", "ACS", "Parent Node"} );

    // 'copyArgumentFromExplorer' is the list of cases
    // where the argument is selected in the explorer,
    // in the order of "copyArgumentPropertyNames"
    _result.setValue("copyArgumentFromExplorer", new Boolean[]{
      Boolean.FALSE, Boolean.TRUE, Boolean.TRUE} );

    // 'copyArgumentConcatenedNameSource' indicates, when an argument
    // is entered by the user as a relative name
    // which has then to be concatenate to another argument,
    // what is this another argument which provides the first part of the name
    // if it is not null. The relative name is an ImmutableName or a String.
    _result.setValue("copyArgumentConcatenedNameSource", new Integer[]{
      new Integer(2), null, null} );

    // 'copyNullArgumentPossible' indicates if an argument may be null
    //  in the method
    _result.setValue("copyNullArgumentPossible", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.TRUE} );

    // 'copyOptionalArgument' indicates if an argument must not be used
    //  in the call to the method when it is null.
    _result.setValue("copyOptionalArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );

    // 'copyRestrictedArgument' indicates if an argument is restricted
    // to some specified values, which are strings or BaseObject names.
    //  Is optional.
    // If it is true, the array of restricted values for an argument is searched:
    // - first, in the value "staticSetOfItems" of the property in this BeanInfo,
    // - second, in the ACS.getM_staticSetOfItems() map
    // and then "constrArgumentPropertyNames" must contain the property descriptor name
    // (but the name words may be separated by spaces in "constrArgumentPropertyNames").
    _result.setValue("copyRestrictedArgument", new Boolean[]{
      Boolean.FALSE, Boolean.FALSE, Boolean.FALSE} );
      
    // ----------------------- other parameters ---------------------------

    // 'hidden' is true if the bean is hidden to the users, without
    // graphical user interface
    _result.setHidden(false);
    _result.setExpert(true);

    return _result;
  } // getCommonBeanDescriptor()


  /**
   * Each property descriptor has specific values that are get
   * with PropertyDescriptor.getValue()
   * @return array of property descriptors
   */
  public PropertyDescriptor[] getPropertyDescriptors() {

    PropertyDescriptor[] l_pd;

    try {
      // ------------------- property "Name" ------------------------
      PropertyDescriptor pd_one = new PropertyDescriptor(
        "Name", VirtualFolderImpl.class, "getName", null);
      pd_one.setDisplayName("Name ");
      //pd_one.setShortDescription("   ");
      //pd_one.setBound(false);
      //pd_one.setConstrained(false);
      //pd_one.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_one.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_one.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. Is null when the VirtualFolderImpl instance
      // does it, or when is is not applicable.
      //pd_one.setValue("BaseBeanManager", null);

      // ------------------- property "OneBaseObjectInFolder" --------------------
      PropertyDescriptor pd_three = new PropertyDescriptor(
        "OneBaseObjectInFolder", VirtualFolderImpl.class, "getEorM_BaseObjects", null);
      // caution: this descriptor name is used in BeamerManager to forbid or not
      // the Select and Remove buttons
      pd_three.setDisplayName("Elements");
      pd_three.setShortDescription("The ACS objects which are elements of this folder.");
      pd_three.setBound(true);
      //pd_three.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor
      pd_three.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_three.setValue("valueTypeInGroup", new Object[] {BaseObject.class} );

      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_three.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_three.setValue("panelGroupName", "    " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_three.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_three.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_three.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'. 'Select' implies 'Remove'.
      pd_three.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes automatically with 'Select'
      // when the property is an external object reference.
      pd_three.setValue("withRemoveButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be preset - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the known argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
        //pd_three.setValue("withKnownArgumentsForNew", new Object[] {
          //null, null, null, null, "this"} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
      pd_three.setValue("addMethod", "addEorBaseObject");
          //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty", "Rights"} );
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_three.setValue("removeMethod", "removeEorBaseObject");
            //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty"} );
      // Following the KeyPropertiesInName pattern,
      // 'removeArgumentsFromName' defines how to find the right arguments for
      // the 'remove' method for the property, when the name of the object
      // to remove is known, and when finalizeForUser() is not used.
      // If "removeArgumentsFromName" is null, the remove method is invoked
      // with the object to remove as single argument.
      // The returned array defines:
      // - the name of the class to call, at the index 0
      // - the name of the static method to call, at the index 1, with
      //    the name of the object to remove as argument, and with
      //    an Object[] array as returned value
      // - in the Object[] array returned as value by the static method,
      //    the sequence of index where are the n arguments to use
      //    in the remove method, in the correct order
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        // pd_three.setValue("removeArgumentsFromName", ... );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_three.setValue("limitedSizeForPresentation", new Integer(16));

      // ------------------- property "VirtualFolderType" ------------------------
      PropertyDescriptor pd_two = new PropertyDescriptor(
        "VirtualFolderType", VirtualFolderImpl.class, "getType", "setType");
      pd_two.setDisplayName("Virtual Folder Type");
      pd_two.setShortDescription("The secondary type of the virtual folder, after mainVFType."+
              " It is used in the ACS type policy.");
      pd_two.setValue("tabbedPaneName", "  General  " );
      pd_two.setBound(true);
       // 'isRestrictedArgument' is true if it is a constrained property that is not
      // in the constructor (else, see 'constrRestrictedArgument' in the BeanDescriptor).
      // The restricted values are get from 'staticSetOfItems', for Strings only,
      // secondly from 'choiceMethodForNew', or through a search in the ACS by the PropertyEditor
      // like here.
      pd_two.setValue("isRestrictedArgument", Boolean.TRUE );

      // ------------------- property "Comment" --------------------
      PropertyDescriptor pd_four = new PropertyDescriptor(
        "Comment", VirtualFolderImpl.class, "getComment", "setComment");
      pd_four.setDisplayName("Comments");
      pd_four.setShortDescription("free user comments");
      pd_four.setBound(true);
      //pd_four.setConstrained(false);
      //pd_four.setPropertyEditorClass(ARoad0.Gui1.SimplePropertyEditor.class);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_four.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
      //pd_four.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the ResourceImpl instance
      // does it, or when is is not applicable.
      //pd_four.setValue("BaseBeanManager", );

      // ------------------- property "ACS" ------------------------
      PropertyDescriptor pd_five = new PropertyDescriptor(
        "ACS", VirtualFolderImpl.class, "getEorACS", null);
      pd_five.setDisplayName("ACS");
      pd_five.setShortDescription("The access control system which manages this element");
      //pd_five.setBound(false);
      //pd_five.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_five.setValue("tabbedPaneName", "  General  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_five.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the ResourceImpl instance
      // does it, or when is is not applicable.
      //pd_five.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_five.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "ParentNode" ------------------------
      PropertyDescriptor pd_six = new PropertyDescriptor(
        "ParentNode", VirtualFolderImpl.class, "getEorParent", null);
      pd_six.setDisplayName("Parent");
      pd_six.setShortDescription("The node which is the parent in "+
        "the tree structure");
      //pd_six.setBound(false);
      //pd_six.setConstrained(false);
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_six.setValue("tabbedPaneName", "  As Node  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_six.setValue("panelGroupName", "  Main attributes  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the ResourceImpl instance
      // does it, or when is is not applicable.
      //pd_six.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which set if the PropertyEditor
      // has to display an 'See' button if the property is viewable
      // in the beamer.
      pd_six.setValue("withViewButton", Boolean.TRUE );

      // ------------------- property "MainVFType" ------------------------
      PropertyDescriptor pd_seven = new PropertyDescriptor(
        "MainVFType", VirtualFolderImpl.class, "getMainVFType", null);
      pd_seven.setDisplayName("Main VF Type");
      pd_seven.setShortDescription("Only for Resources, or for Right User Resources.");
      pd_seven.setValue("tabbedPaneName", "  General  " );
      pd_seven.setBound(false);
      // 'tags' is the name (String value) of the couple of strings,
      // with a size limited to 12 characters,
      // to be displayed when the property is true or false, in this order.
      // Cannot be empty if it exists, and the default value for the property
      // editor is ("yes", "no") when 'tags' does not exist.
      pd_seven.setValue("tags", new String[] {"Resources", "RU Resources"});
      
      // ------------------- property "OneTargetLinkedPrivilege" --------------------
      PropertyDescriptor pd_eight = new PropertyDescriptor(
        "OneTargetLinkedPrivilege", VirtualFolderImpl.class,
        "getEorL_TargetPrivilegeForLinks", null);
      // caution: this descriptor name is used in BeamerManager to forbid the
      // Select and Remove buttons
      pd_eight.setDisplayName("Linked Privileges");
      pd_eight.setShortDescription("The linked privileges of this folder as access target.");
      pd_eight.setBound(true);
      //pd_eight.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_eight.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_eight.setValue("valueTypeInGroup", new Object[] {PrivilegeForLinks.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eight.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_eight.setValue("panelGroupName", "    " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_eight.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_eight.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_eight.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'. 'Select' implies 'Remove'.
        //pd_eight.setValue("withSelectButton", Boolean.TRUE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes automatically with 'Select'
      // when the property is an external object reference.
        //pd_eight.setValue("withRemoveButton", Boolean.TRUE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be preset - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the known argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
        //pd_eight.setValue("withKnownArgumentsForNew", new Object[] {
          //null, null, null, null, "this"} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_eight.setValue("addMethod", "addEorPrivilegeForLinksToTarget");
          //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty", "Rights"} );
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        //pd_eight.setValue("removeMethod", "removeEorPrivilegeToTarget");
            //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty"} );
      // Following the KeyPropertiesInName pattern,
      // 'removeArgumentsFromName' defines how to find the right arguments for
      // the 'remove' method for the property, when the name of the object
      // to remove is known, and when finalizeForUser() is not used.
      // If "removeArgumentsFromName" is null, the remove method is invoked
      // with the object to remove as single argument.
      // The returned array defines:
      // - the name of the class to call, at the index 0
      // - the name of the static method to call, at the index 1, with
      //    the name of the object to remove as argument, and with
      //    an Object[] array as returned value
      // - in the Object[] array returned as value by the static method,
      //    the sequence of index where are the n arguments to use
      //    in the remove method, in the correct order
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        // pd_eight.setValue("removeArgumentsFromName", ... );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_eight.setValue("limitedSizeForPresentation", new Integer(5));
      
      // ------------------- property "OneTargetTypedPrivilege" --------------------
      PropertyDescriptor pd_eleven = new PropertyDescriptor(
        "OneTargetTypedPrivilege", VirtualFolderImpl.class, "getEorL_TargetPrivilegeForTypes", null);
      // caution: this descriptor name is used in BeamerManager to forbid the
      // Select and Remove buttons
      pd_eleven.setDisplayName("Typed Privileges");
      pd_eleven.setShortDescription("The typed privileges of this folder as access target.");
      pd_eleven.setBound(true);
      //pd_eleven.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_eleven.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
      pd_eleven.setValue("valueTypeInGroup", new Object[] {PrivilegeForType.class} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_eleven.setValue("tabbedPaneName", "  Privileges  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_eleven.setValue("panelGroupName", "    " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when it is not applicable.
          //pd_eleven.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display a 'View' button if the property is viewable
      // in the beamer. False if it is not editable.
      pd_eleven.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_eleven.setValue("withNewButton", Boolean.FALSE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'. 'Select' implies 'Remove'.
      pd_eleven.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes automatically with 'Select'
      // when the property is an external object reference.
      pd_eleven.setValue("withRemoveButton", Boolean.FALSE );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be preset - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the known argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor of the object
      // in the beamer, and call the read method
      // to initialize a l_fixedValue from this object.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (exemple: children of Directory).
      // Caution: the known arguments have to be Objects.
        //pd_eleven.setValue("withKnownArgumentsForNew", new Object[] {
          //null, null, null, null, "this"} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_eleven.setValue("addMethod", "addEorPrivilegeForLinksToTarget");
          //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty", "Rights"} );
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method. The property has to be a BaseObject.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        //pd_eleven.setValue("removeMethod", "removeEorPrivilegeToTarget");
            //"ARoad0.gBase.AclEntryImplBeanInfo", "ACS", "EligibleParty"} );
      // Following the KeyPropertiesInName pattern,
      // 'removeArgumentsFromName' defines how to find the right arguments for
      // the 'remove' method for the property, when the name of the object
      // to remove is known, and when finalizeForUser() is not used.
      // If "removeArgumentsFromName" is null, the remove method is invoked
      // with the object to remove as single argument.
      // The returned array defines:
      // - the name of the class to call, at the index 0
      // - the name of the static method to call, at the index 1, with
      //    the name of the object to remove as argument, and with
      //    an Object[] array as returned value
      // - in the Object[] array returned as value by the static method,
      //    the sequence of index where are the n arguments to use
      //    in the remove method, in the correct order
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
        // pd_eleven.setValue("removeArgumentsFromName", ... );
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_eleven.setValue("limitedSizeForPresentation", new Integer(5));

      // ------------------- property "Children" --------------------
      PropertyDescriptor pd_nine = new PropertyDescriptor(
        "Children", VirtualFolderImpl.class, "getEorL_Children", null);
      pd_nine.setDisplayName("Children");
      pd_nine.setShortDescription("The direct children of the virtual folder.");
      pd_nine.setBound(true);
      //pd_nine.setConstrained(false);
      // 'isCollection' is TRUE when the right property editor is CollectionPropertyEditor,
      // and is FALSE when it is SimplePropertyEditor.
      pd_nine.setValue("isCollection", Boolean.TRUE );
      // 'valueTypeInGroup' is not null only when the property is a collection or a map.
      // Then, the value is the type of the values in the collection or in the map.
        //pd_nine.setValue("valueTypeInGroup", new Object[] {...} );
      // 'tabbedPaneName' is the name (String value) of the pane where this property
      // has to be displayed in the beamer.
      pd_nine.setValue("tabbedPaneName", "  As Node  " );
      // 'panelGroupName' is the name (String value) of the group where this property
      // has to be displayed with other related properties. May be empty.
        //pd_nine.setValue("panelGroupName", "  List  " );
      // 'BaseBeanManager' is the manager (Object value) which fires an event
      // when the property is closed or deleted. No value when the AclEntryImpl instance
      // does it, or when is is not applicable.
          //pd_nine.setValue("BaseBeanManager", );
      // 'withViewButton' is the boolean which is set if the CollectionPropertyEditor
      // has to display an 'View' button if the property is viewable
      // in the beamer. 
      pd_nine.setValue("withViewButton", Boolean.TRUE );
      // 'withNewButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add New Value(s)' button if the property is viewable
      // in the beamer. False if it is not editable trough a constructor.
      pd_nine.setValue("withNewButton", Boolean.TRUE );
      // 'withSelectButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Add Selection(s)' button if the property is viewable
      // in the beamer. False if it is not editable or if it is a String list,
      // or if the only write method is a removing method.
      // It is possible to have 'New' and 'Select'.
      pd_nine.setValue("withSelectButton", Boolean.FALSE );
      // 'withDeleteButton' is the boolean which is set if the collectionPropertyEditor
      // has to display an 'Delete Selection(s)' button if the property is viewable
      // in the beamer. If false, there is a 'Remove Selection(s)' button.
      // 'Delete' goes generally with 'New'.
      // On the contrary, 'Remove' goes generally with 'Select'
      // when the property is an external object reference.
      pd_nine.setValue("withDeleteButton", Boolean.TRUE );
      // 'choiceMethodForNew' defines a specific instance/method to invoke 
      // to let the user defines the new restricted key, in a map, or the new instance, 
      // for instance to choose a sub-class
      // of the property class or to select among the restricted values.
      // With 'withKnownArgumentsForNew', the specific method is used even if 
      // there is an add method or a write method. With 'isRestrictedArgument', 
      // and only if 'staticSetOfItems' is null,
      // the specific method is used to get the restricted String values.
      // The value is an array of two Objects; the instance to invoke, or null if it is the source,
      // and the name of the method (for instance, 
      // ActionNewResource.createResource() or NameLinkTreeImpl.getVisibleSpaceConstants()).
      // The method has no argument.
      // The array is equal to 'new Object[1]' when the value 'withKnownArgumentsForNew' is null.
        //pd_nine.setValue("choiceMethodForNew", new Object[] {
          //  xxxx, "valueOf"} );
      // 'withKnownArgumentsForNew' allows to define the constructor arguments
      // which has to be fixed - with 'choiceMethodForNew' or not, add method or not -
      // when a new instance is created through the 'New' button.
      // First case for the fixed argument: the string 'this' is used
      // to point to the object in the beamer;
      // but this object is NOT get as argument if the 'this' index in the array
      // is after the last specified argument for the constructor or the add method.
      // Second case: search the right property descriptor in the beamer,
      // and call the read method to initialize a l_fixedValue from this object.
      // Third case for the fixed argument: the string 'thisname' is used
      // to point to the value returned from getName() by the object in the beamer.
      // Fourth case, if none of the previous cases matches: takes directly
      // the fixed argument if it is a String.
      // Used by SimplePropertyEditor.getFixedValuesForNewElement(), which
      // may use a subset of the returned arguments if the class of the object to create
      // waits a smaller set of arguments.
      // Caution: if 'choiceMethodForNew' exists, then the known arguments
      // are applied to the constructor even if the type of the new instance
      // is not the type of the property (for instance, children of Directory).
      // Caution: the known arguments have to be Objects.
      pd_nine.setValue("withKnownArgumentsForNew", new Object[] {
          null, "ACS", "this", null, null} );
      // 'addMethod' defines the 'add' method to invoke on the object for the property,
      // when there is no write method.  The property has to be a BaseObject.
      // The returned array defines:
      // - the name of the add method, at the index 0
                // - the name of the BeanInfo class where the add method arguments
                //      are defined, at the index 1
                // - the property names of the add method arguments, in the BeanInfo class,
                //      in the correct order
      // These elements allow the PropertyEditor to display a window to create
      // all the required arguments and to call the add method of the property.
        //pd_nine.setValue("addMethod", "addEorChild");
      // 'removeMethod' defines the 'remove' method for an element of the property,
      // even if there is a write method.
      // If finalizeForUser() exists, is public, and if there is "withDeleteButton",
      // the returned value is "finalizeForUser".
      // Else, the returned array defines the name of the remove method.
      // These elements allow the PropertyEditor to call the remove method
      // of the property.
      pd_nine.setValue("removeMethod", "finalizeForUser");
      // 'limitedSizeForPresentation' is the maximal size of the visible map and value lists.
      // Not ridden if less than 4.
      // The casual values are 5, 8, 10 (standard height), 16, 32.
      pd_nine.setValue("limitedSizeForPresentation", new Integer(8));

      // ------------------- property "MemberOneTimeInTree" ------------------------
      PropertyDescriptor pd_ten = new PropertyDescriptor(
        "MemberOneTimeInTree", VirtualFolderImpl.class, "getMemberOneTimeInTree", null);
      pd_ten.setDisplayName("Member one time in the tree");
      pd_ten.setShortDescription("In the virtual folders tree, "+
              "true if a member may be no more than one time.");
      pd_ten.setValue("tabbedPaneName", "  As Node  " );
      pd_ten.setBound(false);

      // ------------------- property "HardAliasReference" ------------------------
      PropertyDescriptor pd_twelve = new PropertyDescriptor(
        "HardAliasReference", VirtualFolderImpl.class,
        "getEorReferenceAsInternalHardAlias", null);
      pd_twelve.setDisplayName("This-alias reference");
      pd_twelve.setShortDescription("The reference "+
              "of this object that is an hard alias");
      pd_twelve.setValue("tabbedPaneName", "  Alias  " );
      pd_twelve.setBound(true);

      // ------------------- property "HardAlias" ------------------------
      PropertyDescriptor pd_thirteen = new PropertyDescriptor(
        "HardAlias", VirtualFolderImpl.class,
        "getEorAliasAsInternalHardReference", null);
      pd_thirteen.setDisplayName("The alias");
      pd_thirteen.setShortDescription("The hard alias "+
              "of this object that is a reference");
      pd_thirteen.setValue("tabbedPaneName", "  Alias  " );
      pd_thirteen.setBound(true);

      // ------------------- property "SoftAlias" --------------------
      PropertyDescriptor pd_fifteen = new PropertyDescriptor(
        "SoftAlias", VirtualFolderImpl.class, "getL_SoftAlias", null);
      pd_fifteen.setDisplayName("Soft aliases");
      pd_fifteen.setShortDescription("The external soft aliases of this reference");
      pd_fifteen.setBound(true);
      pd_fifteen.setValue("isCollection", Boolean.TRUE );
      pd_fifteen.setValue("tabbedPaneName", "  Alias  " );
      pd_fifteen.setValue("withViewButton", Boolean.TRUE );
      pd_fifteen.setValue("withNewButton", Boolean.FALSE );
      pd_fifteen.setValue("withSelectButton", Boolean.FALSE );
      pd_fifteen.setValue("withDeleteButton", Boolean.FALSE );
      pd_fifteen.setValue("limitedSizeForPresentation", new Integer(5));

      // ------- End of properties description ---------------------------------
      
      l_pd = new PropertyDescriptor[] {pd_one, pd_five, pd_two, pd_four, pd_seven, // General
            pd_three, // 'OneBaseObjectInFolder' in General
            pd_twelve, pd_thirteen, pd_fifteen, // 'HardAliasReference', 'HardAlias', 'SoftAlias'
            pd_eleven, pd_eight, // As PrivilegeTarget
            pd_six, pd_nine, pd_ten // As Node
      };

      return l_pd;
    } catch (IntrospectionException e) {
      new ProcessError("In VirtualFolderImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In VirtualFolderImplBeanInfo.getPropertyDescriptors(), "+
        "IntrospectionException: "+e);
    } // catch

  } // getPropertyDescriptors()

    /**
     * Gets the instance, if it exists, which fires an event when
     * the bean is closed or deleted.
     * @return null
     */
    public BaseBeanManager getBaseBeanManagerInstance() {
      return null;
    }

    /**
     * Gets the bean method, without parameters, that allows to get
     * the BaseBeanManager instance, if it exists,
     * which fires an event when the bean instance is closed or deleted.
     * @return the get method. Is null when there is
     *  an active BeanInfoPattern.getBaseBeanManagerInstance(),
     *  or when it is not applicable
     */
    public Method getBaseBeanManagerGetMethod() {
      Object result = null;
      try {
        result = VirtualFolderImpl.class.getMethod("getEorACS", (Class<?>[])null);
      } catch (NoSuchMethodException n) {
        new ProcessError("in VirtualFolderImplBeanInfo."+
          "getBaseBeanManagerGetMethod(): "+n.toString());
      } catch (SecurityException s) {
      }
      return (Method) result;
    }

    /**
     * Gets the property name for which a change fires an PropertyChangeEvent
     * in the BaseBeanManager, when the bean instance is closed or deleted.
     * @return "OneVirtualFolder"
     */
    public String getBaseBeanManagerBoundProperty() {
      return "OneVirtualFolder";
    }


} // VirtualFolderImplBeanInfo
