//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

import java.util.List;

import ARoad0.gBase.*;

/**
* This interface is responsible for modifying
* the access control system (ACS) through specific
* methods. As a BaseBeanManager, fire closing/deleting events on ACS BaseObjects.
* @version 0.7.0
* @see ARoad0.gBase.ACSImpl
*/
public interface ACS extends ImmutableACS {
  
  
  /**
  * Set any comment related to this acs. It is a bound property.
  * @param _s comment which is associated to the acs.
  */
  public void setComment(String _s);
  
  /**
   * A tied child ACS has to be opened when this instance is opened.
   * Like any child ACS, it has to be closed when this instance is closed, and 
   * it may be closed alone.
   * @param _l_names names of the child ACS which are tied up to this instance
   */
  public void setL_TiedChildACS(List<ImmutableName> _l_names);

  /**
   * Note: if the structure map does not contain the key
   * 'access metarule: incomplete modeling of the ACS structure', this method
   * throws a NullPointerException.
   * @since 0.7.0
   * @param _b true if the modeling of the ACS structure is incomplete.
   */
  public void setIncompleteStructureModeling(boolean _b);
  
} // ACS