//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

import java.util.Map;

/**
 * This interface is responsible for describing the bound properties of an ACSObject
 * in an ACS Addon package.
 * The listeners are gBase.ViewInBaseImpl and gWork.RightsMediatorImpl.
 * For instance, it is necessary to listen the event 'OtherRights'
 * in the AcsAddon Ubuntu for the class ResourceUbuntuImpl.
 * @version 0.7.0
 */
public interface ACSObjectAddon extends ACSObject {

  /**
   * Gets the specific events to listen, because these events should change the view rights
   * of an ACSObject in an ACS Addon. This method is not ridden by the beamer.
   * These events are listened to update the views containing these ACSObjects.
   * This method does not cover the parent properties of a Resource or a VirtualFolder
   * (see ARoad0.gBase.AcsAddon.getAddonParentPropertiesToListen()).
   * To define the events, the map key may have the following values: <P>
   * - 'THIS' to say that the single object to listen is the ACSObject, <P>
   * - 'ACS.method_name' to say that the list of objects to listen is returned
   * by the method 'method_name' in the Addon ACS class. <P>
   * For each map key, the values have the following format: <P>
   * 'AAA.method_name IF BBB' <P>
   * where: <P>
   * - AAA: THIS (mandatory), <P>
   * - method_name: 'get' method name of the ACSObject that is associated,
   * in the relevant BeanInfo class, to the event to listen (mandatory) <P>
   * - IF BBB: is optional, where BBB is TRUE, or FALSE, or NULL, or NOT-NULL, to say that
   * the event must be listened if and only if the returned value of the method_name is
   * respectively a boolean TRUE, or FALSE, or an object NULL, or NOT-NULL. <P>
   * When the map value starts with 'ACS', the ACS method to call has the ACSObject has
   * single argument. Otherwise, the ACS or the ACSObject method has no argument.
   * When the map value starts with 'ACS', the ACS method returns a list
   * containing objects that are both BaseBeanManager and BaseObjects,
   * just like Basics and VirtualFolders.
   * Called by ARoad0.gBase.UtilityImpl.changeAddonInnerPropertyChangeListener().
   * @see ARoad0.gBaseInterface.AcsAddon#getAddonParentPropertiesToListen()
   * @return map to define the specific ACS Addon events to listen. May be null.
   */
  public Map<String, String> getAddonInnerPropertiesToListen();

} // ACSObjectAddon

