//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

/**
* This is an interface to inform that it is an object managed by the Access Road base,
* and that it may be in a ViewInBase.
* ImmutableName and its extensions are not implied, nor BaseManagerImpl.
* @version 0.7.0
*/
public interface BaseObject {
  /**
   * The name is unique for the Access Road program only for the ACSObjects.
   * @return a clone of the name. May be empty, but not null.
   */
  public ImmutableName getName();

  /**
   * The full name is unique for the Access Road program.
   * @return a clone of the full name. May be empty, but not null.
   */
    public ImmutableName getFullName();

  /**
   * This short name is NOT unique for the Access Road program nor the ACS if
   * it is an ACS object.
   * It is often the last component name.
   * @return the nick name. May be null.
   */
    public String getNickName();

  /**
   * @param _obj to analyze
   * @return true if the two objects are identical instances
   * of two identical classes
   */
  public boolean equals(Object _obj);

   /**
   * @return int: the hashcode value for this instance
   */
  public int hashCode();

}