//Title:        Access Road
//Version:      0.7.0
//Copyright:    2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation, purely the version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details (gpl.html at the root of Access Road directory).
This license is applicable for 20 years since the publication of the program.
The applicable laws are the french ones, and the relevant court is at Nanterre (92).
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation, Inc.,
675 Mass Ave, Cambridge, MA 02139, USA.
*/

package ARoad0.gBaseInterface;

import ARoad0.ErrB.UpDateError;

/**
 * This interface is responsible for modeling the member of a folder (VirtualFolder
 * or ViewInBase) in an ACS, and it ensures the handling of Privileges.
 * It is the super class of EligibleParty and Resource.
 * finalizeForBase() or finalizeForUser() HAVE TO BE CALLED BEFORE DELETING the object.
 * It is a javabean with the following bound properties:
 * 'Comment', 'ImpliedOpenViews', 'VirtualFolders'.
 * The listeners are proper to this instance.
 * They are called in any order.
 * See the <a href="{@docRoot}/copyright.html">Copyright</a>.
 * @version 0.7.0
 */
public interface Basic extends ImmutableBasic, BaseBeanManager {

/**
 * @param _s is the comment which is associated to the resource.
 */
public void setComment (String _s);

/**
* The type is used by PrivilegeForType instances, so changing the type provides
* a change in privileges, if any, as access source or as access target.
* If the object is a Resource, it is equal to the resource type.
* If the object is an EligibleParty, it is equal to the EP type.
* Called by the ResourceImpl, EligiblePartyImpl and ActorImpl constructors.
* Fires a property change on 'Type', but this method is not called from the GUI.
* @param _type as for instance "Cayro_file" or "tablespace".
*  May be null or superior to 30, but no action then.
*/
//public void setType(String _type);

/**
 * @param _L is an array of Access names which use the resource. Internal variables CAN
 * be updated through the updating of the method argument after the call
 * of this set method
 */
//	public void setL_Access (Name[] _L);

   /**
   * This method removes the basic from the ACS lists.
   * An unused object has to be deleted to inform the others objects, and this method
   * HAS TO BE USED before deleting the object.
   * This method is also called by finalize(), and so may be called twice.
   * @exception UpDateError if the basic is unknown of the main ACS (if
   * it is a clone without setName(), typically)
   */
  public void finalizeForUser() throws UpDateError;

   /**
    * Protected method.
   * Registers the view as containing this BaseObject.
   * @param _view contains this object
   */
  //public void addInViewList(ViewInBase _view);

   /**
    * Protected method.
   * Unregisters the view when it does not contain
   * this BaseObject.
   * @param _view does no more contain this object
   * @return boolean false if the view is unknown
   */
  //public boolean removeInViewList(ViewInBase _view);

   /**
    * Protected method.
   * Registers the virtual folder as containing this BaseObject.
   * @exception BaseError if the ACS does not manage the virtual folders.
   * @param _folder contains this object
   */
  //public void addInVFolderList(VirtualFolder _folder) throws BaseError;

   /**
    * Protected method.
   * Unregisters the virtual folder when it does not contain
   * this BaseObject.
   * @param _folder does no more contain this object
   * @exception BaseError if the ACS does not manage the virtual folders.
   * @return boolean false if the folder is unknown
   */
  //public boolean removeInVFolderList(VirtualFolder _folder) throws BaseError;
  
} // Basic

