//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

import ARoad0.ErrB.CreateError;

/**
 * This interface is responsible to mark an acs object following
 * the BeanInfo class parameters of the object class for the copy operation.
 * @version 0.7.0
 */
public interface Copyable extends BaseObject {
  /**
  * This method provides a copy of the instance
  * with another name. Each internal variable that is an external
  * object reference have a shallow copy or a deep copy.
  * @param _name copy name. Usually creates as an extension of _parent name.
  * @param _acs main ACS of the copy to create. May not be the ACS
  *   of this instance.
  * @param _parent DirectoryImpl which belongs to _acs and will have the copy.
  *   May be null for EligibleParty. Must be null for some implementations.
  * @param _toRegisterInManager false if this method must not register
  *  the instance in the ACS.
  * @return a copy of the object, in the same class.
  * @exception CreateError of the constructor with the same parameters,
  *   or if _parent ACS is not _acs, or addAclEntry() exceptions.
  **/
  public Object copy(ImmutableName _name, ACS _acs, Directory _parent,
    Boolean _toRegisterInManager) throws CreateError;

}
