//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

 /**
* The directory is composed of directories and files as children,
* and it is a kind of resource.
* It manages fully the AG inherited rights of its children.
* It manages only partially the ACL and PRI inherited rights of its children,
* and this may be completed by the AcsAddons.
* There are three flows of data to manage the inherited AclEntries: <P>
* - one at the ACL creation/deletion, and this is not done in the generic gBaseInterface, <P>
* - one at their inner rights editing, and this is the responsability of this interface, <P>
* - and the last flow, only for the condition ACL, at the editing of the membership
* relation between the source and a condition group, and this is the responsability
* of this interface. <P>
* There are two flows of data to manage the inherited Privileges: <P>
* - one at the privilege creation/deletion, and this is not done in the generic gBaseInterface, <P>
* - one at their sources editing, and this is the responsability of this interface.
* @see ARoad0.gBaseInterface.Resource
*/
public interface Directory extends Node, Resource, ImmutableDirectory
{
    /**
   * @param _a true if the groupID is changed for the newly-created files
   * to this directory groupID, and the newly-created sub-directories have
   * this property set to true.
*/
public void setEnforceChildGID(boolean _a);

    /**
    * Resets the inherited AclEntry rights in the children, if the AclEntry
    * inheritance is applicable.
    * There are three flows of data to manage the inherited AclEntries: <P>
    * - one at the ACL creation/deletion, and this is not done in the generic gBase package, <P>
    * - one at their inner rights editing, <P>
    * - and the last flow, only for the condition ACL, at the editing of the membership
    * relation between the source and a condition group. <P>
    * This method is called for handling the second and the third cases,
    * to update the children of the Directory the AclEntry which is the target of the AclEntry.
    * Not described in the BeanInfo since the user does not call it.
    * Note: the creation of the inherited AclEntry in a child is not handled by
    * the generic gBase package, while the rights updating of the inherited AclEntry
    * is handled through this method.
    * @param _acl inherited AclEntry with updated rights
    */
  public void resetInheritedAclEntryRightsOfChildren(AclEntry _acl);

    /**
    * Resets the inherited Privilege rights in the children, if the Privilege
    * inheritance is applicable.
    * There are two flows of data to manage the inherited Privileges: <P>
    * - one at the privilege creation/deletion, and this is not done in the generic gBase package, <P>
    * - one at their sources editing, <P>
    * This method is called for handling the second case,
    * to update the children of the Directory which is the target of the Privilege.
    * Not described in the BeanInfo since the user does not call it.
    * Note: the creation of the inherited Privilege in a child is not handled by
    * the generic gBase package, while the rights updating of the inherited Privilege
    * is handled through this method.
    * @param _lpri inherited privilege with updated sources
    */
  public void resetInheritedPrivilegeRightsOfChildren(PrivilegeForLinks _lpri);

  /**
  * This method provides a copy of the instance
  * with another name, with copy of the children.
  * The children are copied only if they are
  * ResourceImpl, and not simply leaves, and then each new ResourceImpl
  * is set with a name built up from _name and the last component
  * of the copied resource.
  * If a child is not a resource, there is no error and it is not copied.
  * The directory name is structured as this:
  * "_name//isAliasOf//_aliasRef.getName()", where
  * // represents the separator between two name components (@see Name).
  * _aliasRef may belong to another ACS than this instance ACS.
  * <P>
  * @param _name ImmutableName of the directory. Must not be null.
  * @param _aliasRef Directory for which this instance is an alias.
  * @return instance copy
  * @exception CreateError: see DirectoryImpl(
  *   _name, _acs, _parent, _aliasRef) exception
  **/
  //public DirectoryImpl deepCopy(ImmutableName _name, Directory _aliasRef)
    //throws CreateError;        

}

