//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

import ARoad0.ErrB.UpDateError;
import ARoad0.ErrB.CreateError;


/**
* This interface is is responsible for modeling the simplest rights user
* for which an ACS grants access or denies access to
* some targets through AclEntries or Privileges.
* This kind of entity may be an account (UserID), a group (GroupID) or an actor (Actor).
* This class manages the resources connected to the EligibleParty
* through AclEntries, and it manages its Privileges.
* This kind of entity may be an account (UserID),
* a group (GroupID), a program (Executable),
* an external network or a server (Actor), and so on.
* This interface manages the resources connected to
* the eligible party through aclEntries.
* It is a javabean with the following bound properties:
* 'AclConnectedResources'.
* The listeners are outside gBase and transient (not serialized),
* and they are called in any order.
* @version 0.7.0
*/
public interface EligibleParty extends ImmutableEligibleParty, Basic, PrivilegeSource {

  /**
   * If _st is allowed by the ACS, set the type of this eligible party.
   * This method is used to switch from an <immutable> type
   * which forbids the updates of a protected property, and to return to this protecting type.
   * Then , it is possible to switch for example from 'actor' to
   * '<immutable> actor', but not to '<immutable> group'.
   * From the default value '<undefined>', any first update may be done.
   * Fire a PropertyChange for 'EPType'.
   * @param _st type of the resource.
   * @exception UpDateError if the type is null, equal to <byAcsCreator>,
   *    or not known by the ACS, or not associated to the previous type.
   */
  public void setEPType(String _st) throws UpDateError;     

  /**
  * This method provides a copy of the instance as Copyable.copy(),
  * but without copying the internal aclEntries.
  * It is used in ACSImpl.copy().
  * Each internal variable that is an external
  * object reference have a shallow copy (no deep copy),
  * but without copy for the internal aclEntry nor the list of ViewInBase
  * that contains this instance.
  * @param _name copy name. Usually creates as an extension of _parent name.
  * @param _acs main ACS of the copy to create. May not be the ACS
  *   of this instance.
  * @param _parent DirectoryImpl which belongs to _acs and will have the copy.
  *   May be null. Must be null for some implementations.
  * @param _toRegisterInManager false if this method must not register
  *  the instance in the ACS.
  * @return a copy of the object, in the same class.
  * @exception CreateError of the constructor with the same parameters,
  *   or if _parent ACS is not _acs, or addAclEntry() exceptions.
  **/
  public Object copyWithoutInternalAcl(ImmutableName _name, ACS _acs, Directory _parent,
    Boolean _toRegisterInManager) throws CreateError;

}