//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

import ARoad0.ErrB.UpDateError;

/**
* This is an interface to inform that there is a public finaliseForBase() method.
* Does not allows the user to delete this object, on the contrary of
* FinalizeObjectForUser.finalizeForUser() which is designed to be declared in the BeanInfo
* of the gBase classes, and then, to be used by the beamer to delete the object.
* @see FinalizedObjectForUser
*/
public interface FinalizedObject {

   /**
   * An unused object has to be deleted to inform the others objects, and this method
   * HAS TO BE USED before deleting the ACS object.
   * Called by ACSImpl.finalizeForBase() if it is not an embedded object like an AclEntry.
   * This method is also called by FinalizeObjectForUser.finalizeForUser(), finalize(),
   * and so may be called twice.
   * @exception UpDateError if the resource is unknown of the main ACS (if
   * it is a clone without setName(), typically)
   */
   public void finalizeForBase() throws UpDateError;
}
