//Title:        Access Road
//Version:      0.7.0
//Copyright:    2007, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation, purely the version 2 of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details (gpl.html at the root of Access Road directory).
This license is applicable for 20 years since the publication of the program.
The applicable laws are the French ones. The relevant court is at Nanterre (92).
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation, Inc.,
675 Mass Ave, Cambridge, MA 02139, USA.
*/

package ARoad0.gBaseInterface;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import ARoad0.ErrB.CreateError;

/**
* This interface is responsible for managing a folder of BaseObjects
* (even if most of the code is adapted to ACSObjects folder), where the objects
* are added and removed from the folder without object deleting.
* It is the super interface of ViewInBase and VirtualFolder.
* Not Copyable since the copy() method are protected.
* See the <a href="{@docRoot}/copyright.html">Copyright</a>.
* @see ARoad0.gBaseInterface.ViewInBase
* @see ARoad0.gBaseInterface.VirtualFolder
*/
public interface Folder extends ImmutableFolder, PropertyChangeListener {

  /**
   * Adds a base object to the folder.
   * Fire the events 'OneBaseObjectInfolder'.
   * @param _obj BaseObject to add in the folder
   * @exception CreateError if the name is null, or the object name is null or empty,
   * or if the name is already known by the folder.
   */
  public void addEorBaseObject(BaseObject _obj)
    throws CreateError;

  /**
   * Removes a base object to the folder.
   * Fire the events 'OneBaseObjectInfolder' and, sometimes, 'ConnectedACS'.
   * @param _obj non-null BaseObject to remove in the folder
   * @exception CreateError if the object name is empty, or
   * the object is not known by the folder.
   */
  public void removeEorBaseObject(BaseObject _obj)
    throws CreateError;
  
  /**
  * Sets any comment related to this folder. It is a bound property.
  * @param _s comment which is associated to the folder.
  */
  public void setComment(String _s);

  /**
   * Observers to the changes in the open ACS and the base objects list.
   * Overridden method.
   * @param _evt PropertyChangeEvent
   */
  public void propertyChange(PropertyChangeEvent _evt);

  /**
  * Add a listener to the bean.
  * @param _l PropertyChangeListener to add. Caution: the listener must NOT
  *   have specific hashCode() and equals() methods, to work with HashSet
  *   in the removing operations.
  */
  public void addPropertyChangeListener(PropertyChangeListener _l);

  /**
  * Adds a listener to the bean for the properties 'OneBaseObjectInfolder'.
  * @param _propertyName is the name of the property
  * @param _l PropertyChangeListener to add. Caution: the listener must NOT
  *   have specific hashCode() and equals() methods, to work with HashSet
  *   in the removing operations.
  */
  public void addPropertyChangeListener(String _propertyName,
    PropertyChangeListener _l);

  /**
  * Removes a listener to the bean.
  * @param _l PropertyChangeListener to add
  */
  public void removePropertyChangeListener(PropertyChangeListener _l);

  /**
  * remove a listener to the bean for a named property.
  * @param _propertyName String is the name of the property to listen
  * @param _l PropertyChangeListener to add
  */
  public void removePropertyChangeListener(String _propertyName,
    PropertyChangeListener _l);

} // Folder