//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

import ARoad0.ErrB.CreateError;

/**
 * This interface is responsible for modeling a GroupIDMember that owns a set of members,
 * and which has Account/Group rights on Resources.
 * Typically, it handles a set of userIDs and GroupIDs.
 * The membership links between the groupID and each member are managed by
 * the groupID instance.
 * Resources may also be registered as owned by a groupID. Aa rights user,
 * an actor may also have a groupID as a current group, and it is then registered
 * in a third list. An actor may have several secondary groups as rights user,
 * in a fourth list.
 */
public interface GroupID extends ImmutableGroupID, GroupIDMember {
  /**
  * Defines the groupID members, and update the members.
  * Controls the presence of a loop between groupIDs.
  * @param _L  is the members of the GroupID.  May be null.
  *  The new children array is sorted with ImmutableEligibleParty.compareTo();
  *  null and emptty instances are removed.
  * @exception CreateError if _gm is a GroupID which has this instance has
  *   direct or indirect member, or if the type does not allow the operation
  */
  public void setEorL_Members(GroupIDMember[] _L) throws CreateError;
  /**
  * Adds a GroupIDMember as a member of this groupID, and
  * update the member through the addGroupToGM method.
  * Control the presence of a loop between groupIDs, including aliasings.
  * @param _ep new member of the GroupID. No action if null or empty.
  * @exception CreateError if _gm is a GroupID which has this instance has
  *   direct or indirect member, or if the type does not allow the operation
  */
  public void addEorMember(GroupIDMember _ep) throws CreateError;
  /**
  * Removes a groupID member, and update it.
  * @param _ep member of the GroupID. No action if unknown, null or empty.
  * @exception CreateError if the type does not allow the operation
  */
  public void removeEorMember(GroupIDMember _ep) throws CreateError;
  /**
  * Set the order of the GroupID which determines its priority
  * among in access control decisions - on Unix : 16-bit group identifier (GID).
  * @param _i is the order of the groupID - on Unix : 16-bit group identifier (GID)
  */
  public void setOrder(int _i);

}