//Title:        Access Road
//Version:      0.7.0
//Copyright:    2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation, purely the version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details (gpl.html at the root of Access Road directory).
This license is applicable for 20 years since the publication of the program.
The applicable laws are the french ones, and the relevant court is at Nanterre (92).
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation, Inc.,
675 Mass Ave, Cambridge, MA 02139, USA.
*/

package ARoad0.gBaseInterface;

import java.util.List;
import java.beans.PropertyChangeListener;

/**
 * This interface is responsible for modeling the member of a folder (VirtualFolder
 * or ViewInBase) in an ACS.
 * It is the super interface of EligibleParty and Resource.
 * See the <a href="{@docRoot}/copyright.html">Copyright</a>.
 * @version 0.7.0
 */
public interface ImmutableBasic extends ACSObject, PrivilegeUser, Copyable,
    Comparable, BoundBean, FinalizedObjectForUser, Cloneable {

    /**
    * The Basic belongs to an ACS which belongs
    * to an information system (IS). The IS name is the first component
    * of the ACS name.
    * @return ImmutableName of the IS
    */
    public ImmutableName getIS_Name();

    /**
    * @return String: comment which is associated to the basic
    */
    public  String getComment();

    /**
    * Get the current open views containing this BaseObject.
    * @return List of ViewInBase
    */
     public List getImpliedViews();

    /**
    * Get the virtual folders containing this BaseObject.
    * @return List of VirtualFolders
    */
     public List getVirtualFolders();

    /**
     * @return long serial version uid for java serialization.
     */
    public long getSerialVersionUID();

  /**
   * Get the unique ACSObject, if any, of this ACS as reference for which this object
   * is an internal hard alias.
   * @return the hard reference. May be null.
   */
  public ACSObject getEorReferenceAsInternalHardAlias();

  /**
   * Get the unique ACSObject, if any, of this ACS for which this object
   * is an internal hard reference.
   * This class implements the common alias methods for its subclasses which
   * are ImmutableSource or ImmutableTarget.
   * @return the hard alias. May be null.
   */
  public ACSObject getEorAliasAsInternalHardReference();

  /**
  * Gets a sorted array of names of the internal or external soft alias.
   * This class implements the common alias methods for its subclasses which
   * are ImmutableSource or ImmutableTarget.
  * @return ImmutableName[] of the soft alias. May be null but not empty.
  */
  public ImmutableName[] getL_SoftAlias();
  
  /**
  * Gets a sorted array of names of the ACS of the soft references.
  * @return ImmutableName[] of the ACS. May be null but not empty.
  */
  public ImmutableName[] getL_AcsOfSoftReferences();

  /**
   * This property is not displayed in the GUI.
   * This class implements the common alias methods for its subclasses which
   * are ImmutableSource or ImmutableTarget.
  * @return true if there is at least one external or internal soft alias.
  */
  public boolean getHasSoftAlias();

  /**
   * Returns the change listeners for a property.  Used by the AcsAddons.
   * @param _propertyName name of the listened property
   * @return the change listeners. May be null or empty.
   */
  public PropertyChangeListener[] getPropertyChangeListeners(String _propertyName);


} // ImmutableBasic

