//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/


package ARoad0.gBaseInterface;

import java.util.List;
import java.util.Map;

/**
* This interface is responsible for modeling the simplest rights user
* for which an ACS grants access or denies access to
* some targets through AclEntries or Privileges.
* This kind of entity may be an account (UserID),
* a group (GroupID), an external network or a server (Actor).
* This interface manages the access targets connected to
* the eligible party through AclEntries or Privileges.
* @version 0.7.0
*/
public interface ImmutableEligibleParty extends ImmutableBasic, ImmutablePrivilegeSource {

  /**
   * @return type of the eligible party
   */
  public  String getEPType();

  /**
   * @return boolean: true if this eligibleParty is an eligibleParty alias,
   *  created through a specific constructor.
   */
  //public boolean getIsAlias();

  /**
   * Get the eligible party for which this eligible party is an alias.
   * @return a reference on the EP. May be null, even if
   *  getIsAlias() returns true since there is no control when the true EP
   *  is finalized.
   */
  //public EligibleParty getEorTrueEP();

   /**
	 * @return Icon
	 */
//	public  Icon getIcon();

  /**
   * Get the resources connected to this eligible party.
   *  The resources may belong to differents ACS,
   *  as the aclEntries. One resource may have differents aclEntries
   *  (of different ACS) which connect it to this eligible party;
   *  then, the returned list contains only one reference to this resource.
   * Caution: this method is associated to the protected methods
   *  add/removeAclEntryToEP(AclEntry _acl), which have to be implemented
   *  by any class that implements this interface. These protected method
   *  are used in AclEntryImpl.finalizeForUser(short _s), which have to know
   *  all the implementations of this interface.
   * @return the unmodifiable list of references on resources. May be null.
   */
  public List<ImmutableResource> getEorL_AclConnectedResources();

  /**
   * A constrained (opened or closed) child ACS has one or several
   * of the following relations to an eligible party: <P>
   * <ForChildAcsOpeningAndClosing>, <ForChildAcsOpeningNoClosing>
   * or <ForChildAcsIntegrity>.<P>
   * Each relation is associated to a key with the same name.
   * @return a map of 3 ACS name lists for the 3 keys <ForChildAcsOpeningAndClosing>,
   *    <ForChildAcsOpeningNoClosing> and <ForChildAcsIntegrity>. May be empty but
   *    never null.
   */
  public Map<String, ImmutableName[]> getM_ConstrainedChildACS();

}
