//Title:        Access Road
//Version:      0.7.0
//Copyright:    2006, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation, purely the version 2 of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details (gpl.html at the root of Access Road directory).
This license is applicable for 20 years since the publication of the program.
The applicable laws are the French ones. The relevant court is at Nanterre (92).
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation, Inc.,
675 Mass Ave, Cambridge, MA 02139, USA.
*/

package ARoad0.gBaseInterface;

import java.util.SortedMap;
import java.util.List;

/**
* This immutable interface is responsible for managing a folder of BaseObjects
* (even if most of the code is adapted to ACSObjects folder), where the objects
* are added and removed from the folder without object deleting.
* It is the super interface of ViewInBase and VirtualFolder. On the other hand,
* Node is a container where the children are deleted when the node is deleted.
* See the <a href="{@docRoot}/copyright.html">Copyright</a>.
* @see ViewInBase
* @see VirtualFolder
* @see ImmutableNode
*/
public interface ImmutableFolder extends BaseObject, BoundBean {

  /**
   * @param _obj BaseObject to test
   * @return boolean true if the folder references this instance
   */
  public boolean containsBaseObject(BaseObject _obj);

  /**
   * Gets all the base objects of the folder.
   * @return unmodifiableSortedMap of the folder BaseObjects.
   */
  public SortedMap getEorM_BaseObjects();

  /**
  * @return String comment which is associated to the folder. May be null.
  */
  public String getComment();

  /**
   * The type is generally defined by the user.
   * For VirtualFolders only, it is used by PrivilegeForType instances.
   * If the user does not change it, it is equal to the virtual folder type
   * if the object is a VirtualFolder, or to the ViewInBase type
   * if it is a ViewInBase.
   * @return String type of the folder
   */
  public String getType();

  /**
  * @return true if contains no objects
  */
  public boolean isEmpty();

  /**
  * @return boolean true if the folder is connected to ACSObjects
  * belonging to several ACS.
  */
  public boolean getConnectedToSeveralACS();

  /**
  * @return List of names of the ACS having ACS Objects displayed in the folder.
   * Never null.
  */
  public List getEorL_ConnectedACSNames();

} // ImmutableFolder