//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

/**
* This immutable interface is responsible for modeling a leaf in a tree.
* It is used in resources trees and virtual folder trees. It is not used
* in NameTree, which is based on specific nodes.
*/
public interface ImmutableLeaf extends ACSObject, Comparable, BoundBean,
  FinalizedObjectForUser {
    
  /**
   * Gets the current parent node of the object
   * @return a reference on the current parent node
   */
  public  ImmutableNode getEorParent();

  /**
   * From the Comparable interface.
   * @param o Object to compare must be an ImmutableResource (true for Resource)
   * @exception ClassCastException - if the specified object's type
   * prevents it from being compared to this current Object
   * @return 0 is the resources are equals, a negative integer if this object
   * is lesser than the argument, and a positive integer if this object
   * is greater than the argument.
   * The comparison is based on the name compareTo() method of each
   * object when they are not equals.
   */
  public int compareTo(Object o) throws ClassCastException;

}

