//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation, purely version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details (gpl.html at the root of Access Road directory).
This license is applicable for 20 years since the publication of the program.
The applicable laws are the French ones. The relevant court is at Nanterre (92).
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation, Inc.,
675 Mass Ave, Cambridge, MA 02139, USA.
*/

package ARoad0.gBaseInterface;

import java.util.List;

 /**
 * This interface is responsible for modeling
 * the source which is connected to Privileges.
 * A privilege instance is crontolled by the ACS, and it is
 * then appliable to any couple (source, target) which
 * belong to the ACS.
 * There are different types of privilege (defined in PrivilegeAbst
 * class) and interfaces. 
 * See the <a href="{@docRoot}/copyright.html">Copyright</a>.
 * @see ARoad0.gBase.PrivilegeAbst
 */
public interface ImmutablePrivilegeSource extends PrivilegeUser, ImmutableSource {

   /**
   * Gets all the privileges which are associated to this source.
   * There is two types of privilege links (allowed or not by the ACS): <P>
   * - PrivilegeForLinks: linked privilege established for a couple
   * (ACSObject, privilege), <P>
   * - PrivilegeForType: typed privilege, where the ACSObject and the privilege
   * has a common type, but without the TYPED_FOR_SEED and
   * TYPED_CLASSED_SPECIFIC privileges. <P>
   * @return List of Privileges. May be empty, but never null.
   */
  public List getEorL_SourcePrivileges();

   /**
   * Gets all the PrivilegeForTypes of this source.
   * Does not include the TYPED_FOR_SEED and the TYPED_CLASSED_SPECIFIC privileges.
   * @return List of PrivilegeForType. May be empty, but never null.
   */
  public List getEorL_SourcePrivilegeForTypes();

   /**
   * Gets all the PrivilegeForLinks linked to this source.
   * Note: for LINKED_IF_ALL linked privileges, the presence of the privilege
   * does not mean that the master has the privilege rights.
   * @return List of PrivilegeForLinks. May be empty, but never null.
   */
  public List getEorL_SourcePrivilegeForLinks();
  
} // ImmutablePrivilegeSource