//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation, purely version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details (gpl.html at the root of Access Road directory).
This license is applicable for 20 years since the publication of the program.
The applicable laws are the French ones. The relevant court is at Nanterre (92).
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation, Inc.,
675 Mass Ave, Cambridge, MA 02139, USA.
*/

package ARoad0.gBaseInterface;

 /**
 * This interface is responsible for modeling
 * the source that is implied in an access.
 * See the <a href="{@docRoot}/copyright.html">Copyright</a>.
 */
public interface ImmutableSource extends ACSObject, BaseBeanManager, Comparable {

    /**
    * The type is defined at the creation of the object, then it may be updated by the user.
    * It has several uses: <P>
    * - to define the type updating policy from the ACS, <P>
    * - for defining the authorized rights for each kind of rights (ACL, AGO...), <P>
    * - to manage the capacity to delete the instance, <P>
    * - and by the PrivilegeForType instances to apply their rights. <P>
    * It is the resource type if the object is a Resource, the EP type
    * if it is an EP, or it is the VirtualFolder type.
    * @return String of type, as for exemple "executable" or "file".
    *  Never null.
    */
    public String getType();

  /**
   * Get the unique ACSObject, if any, of this ACS for which this object
   * is an internal hard alias. There are internal aliasing
   * in an ACS, and also external aliasing.
   * It is the responsibility of the ACS associations to set an internal alias
   * for one new ACSObject, as an alias or as an alias reference.
   * AcsAddons are not required for the hard aliasing, but they may use it.
   * @return the reference. May be null.
   */
  public ACSObject getEorReferenceAsInternalHardAlias();

  /**
   * Get the unique ACSObject, if any, of this ACS for which this object
   * is an internal hard reference.
   * @return the hard alias. May be null.
   */
  public ACSObject getEorAliasAsInternalHardReference();

  /**
   * This property is not displayed by the generic BeanInfo classes.
   * The soft Alias relation is always set by a gBase class
   * in an AcsAddon, never by the generic gBase classes. In the two alias relations,
   * each ACS is responsible to script or to code its proper use of the alias types.
   * An AcsAddon should provide it to the GUI.
   * The sketch view does not handle the soft alias and the reference,
   * while it handles the hard alias and the hard reference.
  * @return true if there is at least one internal or external soft alias.
  */
  public boolean getHasSoftAlias();

  /**
  * Gets a sorted array of names of all the soft alias.
   * The sketch view does not handle the soft alias and the reference,
   * while it handles the hard alias and the hard reference.
  * @return ImmutableName[]. May be null but not empty.
  */
  public ImmutableName[] getL_SoftAlias();

  /**
  * Gets a sorted array of names of the ACS of the soft references.
  * @return ImmutableName[] of the ACS. May be null but not empty.
  */
  public ImmutableName[] getL_AcsOfSoftReferences();

} // ImmutableSource

