//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

/**
* This interface is the immutable version of UserID.
* It is responsible for modeling an account in an ACS,
* as a kind of GroupIDMember that owns some resources and a primary groupID.
* It has some current actors, that are programs starting their execution under it.
*/
public interface ImmutableUserID extends ImmutableGroupIDMember
{
  /**
   * The primary group is used by an Actor that has a current UserID, to set
   * the primary group as current GroupID of the Actor.
   * @return GroupID: the primary group of the userID,
   *  following Linux/Unix concepts. May be null.
   */
  public GroupID getEorPrimGroup ();

  /**
   * @return the UID of the userID - on Unix : 16-bit user identifier
   */
  public int getOrder();

  /**
  * An own resource is a same-ACS resource that have this as userID.
  * If it is an Actor, it may be a current actor of this userID or not.
  * @return array of the own resources. May be empty, but never null.
   */
  public Resource[] getEorL_UserIDOwnResources();

  /**
   * A current actor is running under this userID whatever its caller.
   * It may be an own resource of this userID or not. 
   * @return array of the current actors attached to this userID.
   *  May be empty, but never null.
  */
  public Actor[] getEorL_UserIDCurrentActors();

  /**
  * @return true if it owns the resource.
  */
  public boolean ownsResource(Resource _res);
  
}

