//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

 /**
  * This interface is responsible for managing the names
  * for all the base objects. <P>
  *
  * Examples of name are "http://tpaconseil.com", "c:\program files\access road",
  * "net/users/john". The separator character depends on the ACS. <P>
  *
  * Name follows the OMG CORBA Object Services Specification (OMG-COSS)
  * for a naming service which names an object for later retrieval.
  * Methods added to the OMG specification are extended(ImmutableName),
  * extended(String), clone(), equals(), hashCode(), getSerialVersion(),
  * getNameFromString().
  * It does NOT follow the JNDI name interface. <P>
  *
  * The name is an ordered list of components in a java.util.ArrayList.
  * Each component is a string that may be null. Case must be respected.
  * ':: ' is a forbidden component, because it is used in toString()
  * and getNameFromString(). It is controlled in every creating or updating method.
  * <P>
  * The usual struture of an entity name is (IS name).(ACS name).(entity name)
  * Normally, the usual name of an instance of Name is equals
  * to name.lastComponent().
  * <P>
  * This interface extends ImmutableName that contains all the get methods and
  * the equivalent ones (clone(), equals()...).
  * <P>
  * To remove any component, use without(). To add a component at the end,
  * use extended(). To modify the last component, use without() then extended().
  * For more complex modifying operations, use the list returned by components().
  * With threads,
  * always call these methods in a block synchronized on the Name instance.
  * <P>
  * If the name is used as a key in a map, it is much safer to avoid a mutable
  * name. Use then the ImmutableName interface to construct the key. But
  * the update will remain possible by casting the ImmutableName
  * in Name.
  * @version 0.5.0
  */
public interface Name extends ImmutableName {

  /**
   * @param _name is removed from the current name. _name may be at the start,
   *    the end or the middle of the current name.
   *    If there are several sections of the current name that are strictly
   *    identical to _name, without() removes the last matched part,
   *    at the end of the current name.
   * @return the updated name of the current name.
   *    With threads, this non-synchronized method (for performance)
   *    must be used in a synchronized block. May be null.
   */
    public  Name without(Name _name);

  /**
    * For extending the current name.
    * Noop if the argument is null or empty, or if ':: ' is in the argument,
    * because it is used in toString() and getNameFromString().
    * With threads, this method must be used in a synchronized block.
    * @param _nc adds a new last component to the current instance. The string
     *        added is not backed by the argument, so modify the new component
     *        doesn't modify the initial string used as argument in this method.
     *        With threads, this method must be used in a synchronized block
     * @return this instance
     */
    public Name extended(String _nc);

    /**
    * For extending the current name.
    * The name is unchanged if _name is null or an empty instance,
    * or has a component which contains ":: " (see getNameFromString()).
    * @param _name adds a new name at the end of the current instance,
    *   with string components.
    * @return this instance
    * @since 0.5.1
    */
  public Name extended(ImmutableName _name);

  /**
   * Returns a new independant Name instance as an extract of the current Name.
   * @param _start inclusive fromIndex
   * @param _end exclusive toIndex
   * @return an extract of the current name. May be null.
   */
  public Name subname(int _start, int _end);

} // Name class