//Title:        Access Road
//Version:      0.7.0
//Copyright:    2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls


/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation, purely version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details (gpl.html at the root of Access Road directory).
This license is applicable for 20 years since the publication of the program.
The applicable laws are the French ones. The relevant court is at Nanterre (92).
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation, Inc.,
675 Mass Ave, Cambridge, MA 02139, USA.
*/

package ARoad0.gBaseInterface;

import ARoad0.ErrB.UpDateError;

 /**
 * This interface is used for modeling the Privilege, which, like ACL or AGO rights,
 * is a type of granted or denied access right delivered to an access source
 * for accessing to a target.
 * A privilege instance is controlled by an ACS, and delivers immutable rights
 * applicable to couples (source or sources set or source containers set, 
 * target or targets set), which may be external (do not belong to the privilege ACS)
 * in some cases. The rights of a privilege are immutable
 * after its creation, while the AclEntry has immutable source and target,
 * but with editable rights after the creation.
 * The two types of privileges are typed privileges and linked privileges.
 * See the <a href="{@docRoot}/copyright.html">Copyright</a>.
 * @see ARoad0.gBase.PrivilegeAbst
 * @see ARoad0.gBase.PrivilegeForLinksImpl
 * @see ARoad0.gBase.PrivilegeForTypeImpl
 */
public interface Privilege extends ImmutablePrivilege {

   /**
    * @param _longPrivilege is the description which is displayed to the user.
    *  May be null.
    */
    public void setSeeWhy(String _longPrivilege);

  /**
  * Set any comment related to this privilege.
  * @param _s comment which is associated to the resource.
  */
  public void setComment(String _s);

  /**
   * If the new value is allowed by the ACS, set the second type of this privilege.
   * This method is used to switch from an <immutable> type
   * which forbids the updates of a protected property, and to return to this protecting type.
   * Then, it is possible to switch for instance from 'read' to
   * '<immutable> read', but not to '<immutable> write'.
   * From the default value '<undefined>', any first update may be done.
   * '<12345678901234567890>' is a program-reserved value. Even if not
   * allowed by the ACS, '<12345678901234567890>' is the only new value
   * for which a current type containing '<byAcsCreator>' may be changed.
   * From '<12345678901234567890>', any new value may be set from the allowed ones,
   * and this starting value is the only one which allows to set a value containing
   * '<byAcsCreator>'. Calls ACS.getM_TypesAndGUIPolicy() to have the allowed values
   * for the key 'PrivilegeForType.SecondType' if the instance is a PrivilegeForType,
   * or 'PrivilegeForLinks.SecondType' if the instance is a PrivilegeForLinks.
   * @param _st second type of the privilege. It is not the main type.
   * @exception UpDateError if the type is null, starts with <byAcsCreator>,
   *    or not known by the ACS, or not associated to the previous type.
   */
  public void setSecondType(String _st) throws UpDateError;
} // Privilege