//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

import ARoad0.ErrB.UpDateError;

/**
* This interface is responsible for modeling an account in an ACS,
* as a kind of GroupIDMember that owns some resources and a primary groupID.
* It has some current actors, that are programs starting their execution under it.
* @version 0.7.0
*/
public interface UserID extends ImmutableUserID, GroupIDMember
{
    /**
   * Set a primary group, and it must have this instance as member.
   * The primary group is used by an Actor that has a current UserID, to set
   * the primary group as current GroupID of the Actor.
   * Update the own actors which have the previous primary group as current group.
   * Remove the current primary group in the current actors which have it as
   * their current group. Fire the PropertyChange 'PrimGroup'.
   * Noop and error mesage to the user if the ep type contains <byAcsCreator>
   * or <immutable>  after the creation.
   * To do it in that case, remove this instance from all its groups,
   * and then, the first registered group will be the primary group.
   * Read the ACS 'GroupID.NoPrimaryGroup' values to check if the argument 
   * may be a primary group.
   * @param _g the primary group of the userID.
   *   May be null.
   * @exception UpDateError if _g does not belong to the main ACS
   *  of this resource or cannot be a primary group, or does not contain this instance,
   *  or if the type is immutable after the first creation.
     */
    public  void setEorPrimGroup (GroupID _g) throws UpDateError;

    /**
     * Set the strenght of the UserID which defines its priority order
     * in access control decisions.
     * @param _i is the order of the userID - on Unix: 16-bit user identifier (UID)
     * @exception UpDateError if immutable
     */
    public void setOrder(int _i)  throws UpDateError;

   /**
   * The super method removes the userID in the ACS lists (main and controllers),
   * and removes in the resources connected through an aclEntry.
   * This method removes this instance in own resources and in current actors,
   * so they have no more UserIDs connected to them.
   * This method HAS TO BE USED before deleting the userID.
   * @exception UpDateError if the userID is unknown of the main ACS
   * (if it is a clone, typically)
   */
  public void finalizeForUser() throws UpDateError;

}

