//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gBaseInterface;

/**
* This interface is responsible for managing in the base the views
* about access control issues.
* It is a listener of the ACS openings and closings.
* @version 0.7.0
*/
public interface ViewInBase extends Folder {

  /**
   * False if at least one ACS structure modeling is incomplete for an used ACS.
   * The modeling of the ACS structure is incomplete when some ACSObjects miss
   * while they participate to the decision for some important access controls,
   * or simply when these missed ACSObjects are important for the overall ACS security,
   * or when their important relations with some other ACSObjects are not set.
   * On the other hand, the model remains complete if some secondary ACSObjects
   * are not modelled, like for instance image files in an operating system file system.
   * When the model of an ACS structure is incomplete, all the views which
   * use this ACS are set incomplete in their results.
   * @return false if the result is incomplet since some ACS structure is incomplete.
   */
  public boolean getCompleteResults();


} // ViewInBase