//Title:        Access Road
//Version:      0.7.0
//Copyright:    2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import java.awt.event.*;
import java.net.*;
import java.io.IOException;
import java.util.List;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.gBase.BaseManagerImpl;
import ARoad0.ErrW.SmallError;

/**
 * An action that displays the guides of the registered AcsAddons.
 * They have to be named 'acsaddon_guide.html', in the ACS addon gBase directory.
 */
public class ActionAcsAddonGuides extends AbstractAction implements Singleton {
  /**
   * The list from BaseManagerImpl.getInstance().getL_AcsAddonDescriptors()
   */
  List<String>                      l_addon_ = null;

  /* the only instance of the class, following singleton pattern */
  private static ActionAcsAddonGuides      instance__ = new ActionAcsAddonGuides();


  /**
   * with this private constructor, the compiler won't generate
   * a default public constructor.
   */
  private ActionAcsAddonGuides() {
    super("ACS Addon Guides");

    l_addon_ = BaseManagerImpl.getInstance().getL_AcsAddonDescriptors();
  }

  /**
   * single public method to get the unique instance
   * @return the unique instance
   */
  public static synchronized ActionAcsAddonGuides getInstance(){
    return instance__;
  } // getInstance()

  /**
   * Run the default browser to display the ACS addon guide the user selects.
   * @param ev ActionEvent
   */
  public void actionPerformed(ActionEvent ev) {

    if (l_addon_.isEmpty()) {
        DialogBox.showOkMessage("No registered ACS addon.");
        return;
    }

    String  tutorial = "acsaddon_guide.html";
    int     choice = -1;
    String  editor = null;
    String  addon = null;

    try {
        choice = DialogBox.showOneSelectionOneDialog(
                "ACS addon", l_addon_.toArray(new String[0]));
    }
    catch (SmallError s) {
        throw new InternalError("From ActionAcsAddonGuides to select an ACS addon: "+s);
    }

    if (choice == -1) {
        DialogBox.showOkMessage("No selected ACS addon.");
        return;
    }

    //The form of a value is 'Editor: editor_name - AcsAddon: addon_name'
    editor = l_addon_.get(choice).substring(8, l_addon_.get(choice).indexOf(" - AcsAddon: "));
    addon = l_addon_.get(choice).substring(l_addon_.get(choice).indexOf("AcsAddon: ")+10);

    // Desktop is in java 1.6
    if ( java.awt.Desktop.isDesktopSupported() ) {

            java.awt.Desktop desktop = java.awt.Desktop.getDesktop();

            if (desktop.isSupported(java.awt.Desktop.Action.BROWSE)) {

               try {
                    desktop.browse(
                        Desktop.AR_URI.resolve("AcsAddon"+Desktop.URL_SEPAR+editor+
                            Desktop.URL_SEPAR+addon+Desktop.URL_SEPAR+"gBase"+
                            Desktop.URL_SEPAR+tutorial) );
               } catch (MalformedURLException e1) {
                    DialogBox.showAlarmMessage("Access Road may have found"+
                           " the default browser on your computer for"+tutorial+
                           ", but there is the exception: "+e1.getMessage());
                    return;
               } catch (IOException e3) {
                    DialogBox.showAlarmMessage("Access Road may have found"+
                           " the default browser on your computer for"+tutorial+
                           ", but there is the exception: "+e3.getMessage());
                    return;
               } catch (SecurityException e4) {
                    DialogBox.showAlarmMessage("Access Road cannot run"+
                           " the default browser on your computer, since the security configuration"+
                           " does not authorize it. The exception is: "+e4.getMessage());
                    return;
               } // catch

               DialogBox.showOkMessage("The guide is displayed in your default web browser.");
            } // if
            else  DialogBox.showAlarmMessage("Access Road cannot launch"+
                           " the default browser on your computer. This"+
                           " is not supported by the java virtual machine.");
    } // if
    else { // Desktop not supported, try Firefox on *nx, or the Windows default browser
        try {
           Process proc = null;
           String name = System.getProperty ( "os.name" );
           //String version = System.getProperty ( "os.version" );

                                    // for testing and tracking
                                  //System.out.println("ActionAcsAddonGuides - os.name is: "+name);

           if (name.indexOf("Linux") != -1 || name.indexOf("Unix") != -1 || name.indexOf("Solaris") != -1)
              proc = Runtime.getRuntime().exec("firefox "+
                   ARoad0.Gui1.Desktop.AR_DIR+Desktop.FILES_SEPAR+
                   "AcsAddon"+Desktop.URL_SEPAR+editor+
                   Desktop.URL_SEPAR+addon+Desktop.URL_SEPAR+"gBase"+
                   Desktop.FILES_SEPAR+tutorial);

           else if (name.indexOf("Windows") != -1 )
              proc = Runtime.getRuntime().exec("start "+
                   ARoad0.Gui1.Desktop.AR_DIR+Desktop.FILES_SEPAR+
                   "AcsAddon"+Desktop.URL_SEPAR+editor+
                   Desktop.URL_SEPAR+addon+Desktop.URL_SEPAR+"gBase"+
                   Desktop.FILES_SEPAR+tutorial);
           // on Windows NT, start cmd.exe because start is not
           // an external command but internal, you need to start the
           // command interpreter before: "cmd.exe /c " + "start "+...

           else {
               DialogBox.showAlarmMessage("Access Road cannot launch"+
                           " the default browser on your computer. The"+
                           " operating system and Firefox are not detected.");
               return;
           } // else


           DialogBox.showOkMessage("The guide is displayed in the web browser.");
           proc = null;
           //version = null;
        } catch (IOException e) {
           DialogBox.showAlarmMessage("Access Road cannot launch"+
                           " the default browser on your computer. The exception"+
                           " is: "+e.toString());
        }
    } // else


  } // actionPerformed(ActionEvent)

} // class ActionAcsAddonGuides
