//Title:        Access Road
//Version:      0.6.0
//Copyright:    2002, 2004, 2007, 2008, 2009 EURL ACCBEE
//Author:       Patrick Thazard ( )
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import java.awt.event.*;
import java.net.*;
import java.io.IOException;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;

/**
 * An action that displays the tutorials.
 * @version 0.6.0
 */
public class ActionBasicTutorial extends AbstractAction implements Singleton {

  /* the only instance of the class, following singleton pattern */
  private static ActionBasicTutorial      instance__ = new ActionBasicTutorial();


  /**
   * with this private constructor, the compiler won't generate
   * a default public constructor.
   */
  private ActionBasicTutorial() {
    super("Getting Started");
  }
  /**
   * constructor()
   * @return ActionBasicTutorial
   */
  public static synchronized ActionBasicTutorial getInstance(){
    return instance__;
  } // getInstance()

  /**
   * Run the default browser to display the basic tutorial.
   * Use the JRE 1.6 Desktop or the operating systems Linux, Unix, Solaris and Windows.
   * @param ev ActionEvent
   */
  public void actionPerformed(ActionEvent ev) {
    String tutorial = "tutorials.html";
    //CanisMinor c = new CanisMinor(tutorial, true);
    
    // Desktop is in java 1.6
    if ( java.awt.Desktop.isDesktopSupported() ) {

            java.awt.Desktop desktop = java.awt.Desktop.getDesktop();

            if (desktop.isSupported(java.awt.Desktop.Action.BROWSE)) {

               try {
                    desktop.browse(          
                        Desktop.AR_URI.resolve("Gui1"+Desktop.URL_SEPAR+"help"+
                        Desktop.URL_SEPAR+tutorial) ); 
               } catch (MalformedURLException e1) {
                    DialogBox.showAlarmMessage("Access Road may have found"+
                           " the default browser on your computer for"+tutorial+
                           ", but there is the exception: "+e1.getMessage());
                    return;
               } catch (IOException e3) {
                    DialogBox.showAlarmMessage("Access Road may have found"+
                           " the default browser on your computer, but there is"+
                           " the exception: "+e3.getMessage());
                    return;
               } catch (SecurityException e4) {
                    DialogBox.showAlarmMessage("Access Road cannot run"+
                           " the default browser on your computer, since the security configuration"+
                           " does not authorize it. The exception is: "+e4.getMessage());
                    return;
               } // catch
               
               DialogBox.showOkMessage("The tutorials are displayed in your default web browser.");
            } // if
            else  DialogBox.showAlarmMessage("Access Road cannot launch"+
                           " the default browser on your computer. This"+
                           " is not supported by the java virtual machine.");
    } // if 
    else { // Desktop not supported, try Firefox or windows default browser
        try {
           Process proc = null;
           String name = System.getProperty ( "os.name" );
           //String version = System.getProperty ( "os.version" );
           
                                    // for testing and tracking
                                  //System.out.println("ActionBasicTutorial - os.name is: "+name);
           
           if (name.indexOf("Linux") != -1 || name.indexOf("Unix") != -1 || name.indexOf("Solaris") != -1)
              proc = Runtime.getRuntime().exec("firefox "+
                   ARoad0.Gui1.Desktop.AR_DIR+Desktop.FILES_SEPAR+
                   "Gui1"+Desktop.FILES_SEPAR+"help"+Desktop.FILES_SEPAR+tutorial);
           
           else if (name.indexOf("Windows") != -1 )
              proc = Runtime.getRuntime().exec("start "+
                   ARoad0.Gui1.Desktop.AR_DIR+Desktop.FILES_SEPAR+
                   "Gui1"+Desktop.FILES_SEPAR+"help"+Desktop.FILES_SEPAR+tutorial);
           // on Windows NT, start cmd.exe because start is not
           // an external command but internal, you need to start the
           // command interpreter before: "cmd.exe /c " + "start "+...

           else {
               DialogBox.showAlarmMessage("Access Road cannot launch"+
                           " the default browser on your computer. The"+
                           " operating system and Firefox are not detected.");
               return;
           } // else

               
           DialogBox.showOkMessage("The tutorials are displayed in the web browser.");
           proc = null;
           //version = null;
        } catch (IOException e) {
           DialogBox.showAlarmMessage("Access Road cannot launch"+
                           " the default browser on your computer. The exception"+
                           " is: "+e.toString());
        }
    } // else

    
    return;
  } // actionPerformed(ActionEvent)

} // class ActionBasicTutorial