//Title:        Access Road
//Version:      0.6.0
//Copyright:    2002, 2004, 2007, 2008, 2009 EURL ACCBEE
//Author:       Patrick Thazard ( )
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.event.*;
import java.awt.*;
import java.beans.*;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.Gui2.*;
import ARoad0.gWork.*;
import ARoad0.ErrW.*;
import ARoad0.ErrB.*;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.Directory;
import ARoad0.gBase.EPRViewInBaseImpl;
import ARoad0.gBaseInterface.ImmutableGroupID;
import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBaseInterface.ACS;
import ARoad0.gBase.ACSImpl;
import ARoad0.gBaseInterface.Copyable;
import ARoad0.gBaseInterface.GroupID;
import ARoad0.gBaseInterface.EPRViewInBase;
import ARoad0.gBaseInterface.NoThanViewInBase;

/**
 * An action that copies a view in a new instance with a new name,
 * and saves it then opens it.
 * @version 0.6.0
 */
public class ActionCopy extends AbstractAction implements Singleton,
 NewBaseObjectListener {

/*
 * For an ACS, copies all its objects. If an own actor is a bridge source,
 * a message to the user requests to also copy the bridge target,
 * to create a bridge link from the the copied ACS
 * to the initial bridge target ACS.
 * A new ACS object may have a new ACS and, if it is a resource, a new parent.
 * For a Directory, copies the children tree.
 * For a GroupID, copies all the members tree, and registers the copy
 * in every GroupID as member, if it is the same ACS,
 * or else in every GroupID which has a sililar name - that is ending
 * to the same component.
 * The alias selection is quite open:
 * - for Resource subclasses, the alias may be any Resource
 * - for EligibleParty subclasses, the alias may be any EligibleParty
 * - for Actor subclasses, the alias may be any Actor
 * Uses BaseObjectHandler and Copyable for copying directory children
 * and groupId members.
 */
    
  protected static final int    INITIAL_CAPACITY = 3;
    
  /* the only instance of the class, following singleton pattern */
  private static ActionCopy     instance__ = new ActionCopy();

  // environment
  private static String         currentDir__ = System.getProperty("user.dir");
  private static String         userDir__ = System.getProperty("user.home");
  private static String         fileSepar__ = System.getProperty("file.separator");
  private static String         lineSepar__ = System.getProperty("line.separator");

  /**
   * The object to copy, only if it is a Directory.
   */
  protected Directory dirToCopy_;

  /**
   * The object to copy, only if it is a GroupID.
   */
  protected GroupID groupToCopy_;
  
  /**
   * The value of the copy source.
   */
  protected BaseObject source_;
  
  /**
   * with this private constructor, the compiler won't generate
   * a default public constructor
   */
  private ActionCopy() {
    super("Copy View");
  }

  /**
   * constructor()
   */
  public static synchronized ActionCopy getInstance(){
    return instance__;
  } // getInstance()
  
  /**
   * Gets the value of the copy source.
   *
   * @return the source of the property.
   */
  public BaseObject getSource() {
    return source_;
  }  // getSource()  

  /**
  * Copies the selected frame object.
  * No action if another user dialog is currently open.
  * Manages the actions fired from:
   * - from 'View Copy' in the main menu when, in the explorer,
   *    an object is selected
  * A control validates the selected frame or node by verifying if
  * it is associated to a right object.
  * @param _ev ActionEvent
  */
  public void actionPerformed(ActionEvent _ev) {      

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
      
    // initialization
    //boolean                 userChoice = false;
    Object[]            l_analyzedEvent = 
      ActionUtilities.getInstance().analyzeActionEvent(_ev);
    ImmutableName       name = ((ImmutableName)l_analyzedEvent[1]);

          // for testing and tracking
    //System.out.println("in ActionCopy.actionPerformed()"+
    //  "l_analyzedEvent: ");
    //for (int i = 0 ; i < l_analyzedEvent.length ; i++) {
    //  System.out.println(l_analyzedEvent[i].toString());
    //}
    
  //     ("acs", "resource", "actor", "groupid", "userid",
 //      "aclentry", "view", "beamer"
    
    if (l_analyzedEvent[0] == null ) {
            // display a warning message
            DialogBox.showErrorMessage(
              "No selection or unworkable selection.");
            l_analyzedEvent = null;
            name = null;
            return;
    } // if
    /*
    else if (
      ((String)l_analyzedEvent[0]).equals("aclentry") ) {
            // display a warning message
            DialogBox.showErrorMessage(
              "This object cannot be copied.");
            l_analyzedEvent = null;
            name = null;
            return;
    } // if
    else if (
      ((String)l_analyzedEvent[0]).equals("beamer") ) {
            // display a warning message
            DialogBox.showErrorMessage(
              "To copy an object, select the explorer first.");
            l_analyzedEvent = null;
            name = null;
            return;
    } // if
     else if (
      ((String)l_analyzedEvent[0]).equals("acs") ) {
            if ( ((ACS)ACS_Facade.getOpenACS().get(name)).getContainsBridgeTarget() ) {
                DialogBox.showOkMessage("This ACS has bridge target(s), and "+
                  "these ACS bridge link(s) will not be copied.");
            }
            
            copyACSorView(name);
            l_analyzedEvent = null;
            name = null;
            return;
    } // if

     */
    else if (((String)l_analyzedEvent[0]).equals("view")) {            
            copyACSorView(name);
            l_analyzedEvent = null;
            name = null;
            return;
    } // else if
    else {
            DialogBox.showErrorMessage("The selection in the explorer is not a view.");
            l_analyzedEvent = null;
            name = null;
            return;
    }
    
    // else, it is an ACS objet to copy
    /*
    // sets the copy parameters
    Object[] l_fixedValue = new Object[] {null, null, null, null};
    
    try {
        source_ = ACS_Facade.getEorBaseObject(name);
    } catch (BaseError b) {
        new InternalError("in ActionCopy.actionPerformed(): "+b.toString());
    }

    if (!(source_ instanceof Copyable)) {
            // display a warning message
            DialogBox.showErrorMessage(
              "This object cannot be copied.");
            return;
    } // if
    
    if (source_ instanceof Directory) dirToCopy_ = (Directory)source_;
    else if (source_ instanceof GroupID) groupToCopy_ = (GroupID)source_;
    
    // requests to the user the parameters,
    // uses the copy() method, and
    // calls back the handleNewBaseObject() method
    // which uses dirToCopy_ or groupToCopy_.
    // double argument obj to be different from
    // the first valueOf method
    BaseObjectHandler.valueOf(this, (Copyable)source_, (Copyable)source_).
      getBaseObjectEditor(l_fixedValue);

    l_fixedValue = null;
    l_analyzedEvent = null;
    name = null;
    return;
     * */
  } // actionpPerformed(ActionEvent) 


  /**
  * For a view only, copies, saves and opens it.
  * @param _name of the ACS or view
  */
  public void copyACSorView(ImmutableName _name) {
    
    final ImmutableName     name = _name;
    //boolean isACS = false;
    //if (ACS_Facade.getOpenACS().containsKey(name) ) isACS = true;
    //final boolean isAnACS = isACS;
    
    // requests to the user 
    String elementName = "Copy "+name+"- Enter the name: ";
    elementName = DialogBox.showSingleEditor(elementName); 

    if (elementName.trim().length() == 0) {
        DialogBox.showAlarmMessage("The view copy has no name. The copy is aborted.");
        return;
    }

    ImmutableName copyName = name.context().extended(elementName);
    try {
        //if (!isAnACS) {

       EPRViewInBase view = null;
       
       if (!ViewInBase_Facade.isNoThanViewName(copyName))
           view = ((EPRViewInBase)ViewInBase_Facade.getAllStandardOpenViews().
               get(name)).copyAndSave(copyName);
       else
           view = ((NoThanViewInBase)ViewInBase_Facade.getAllStandardOpenViews().
               get(name)).copyAndSave(copyName);
          
       ActionOpenGraphicView.openViewInGui(view, true, null);

        //} 

        /*
        else if (isAnACS) {

          source_ = (ACSImpl)ACS_Facade.getOpenACS().get(name);
          ACS acs = null;
          //ACSImpl.copy(
            //copyName,
            //(ACSImpl)source_,
            //true);

          //copyACSBridges(initialACS, acs);

          JTree                   eTree = InternalFrameModel.getTreeInExplorer();
          DefaultTreeModel        eTreeModel = (DefaultTreeModel) eTree.getModel();
          DefaultMutableTreeNode  acsNode = null;
          int                     index;

          // updates the explorer
              // index is the number of presently displayed ACS in Explorer
          index = eTreeModel.getChildCount(((DefaultMutableTreeNode)eTreeModel.
              getRoot()).getChildAt(0) );

          acsNode = new DefaultMutableTreeNode(acs );
          eTreeModel.insertNodeInto(
              acsNode,
              (DefaultMutableTreeNode)(((DefaultMutableTreeNode)eTreeModel.
                getRoot()).getChildAt(0)),
              index);

                  // for testing and tracking
                  //System.out.println("in ActionCopy.copyACSorView(),"+
                  //  "add an ACS: "+acs.getName()+", at the index: "+index+
                  //  ", under the node: "+((DefaultMutableTreeNode)eTreeModel.
                  //  getRoot()).getChildAt(0)+", under the tree root: "+eTreeModel.
                  //  getRoot());

          ACSTree.createACSExplorer(acs, eTree, acsNode);

          // changes the selection in the explorer, and in the beamer
          eTree.setSelectionPath(new TreePath( acsNode.getPath() ));
          // if the beamer is closed, open it with this acs
          if ( !InternalFrameModel.withBeamer() ) {
            try {
              InternalFrame.openBeamer(
                BeamerManager.getInstance().updateBeamerPane(acs));
            } catch (ProcessError p) {
              DialogBox.showAlarmMessage("The beamer can't detect"+
                " the acs to display.");
            }
          } // if

          eTree = null;
          eTreeModel = null;
          acsNode = null;
          acs = null;
          copyName = null;

        } // else if
         * */
    //} catch (ProcessError p) {
      //  throw new InternalError("in ActionCopy: "+p.toString());
    } catch (BaseError b) {
        throw new InternalError("in ActionCopy, the element is not a base object: "+b);
    //} catch (CreateError c) {
        //throw new InternalError("in ActionSaveAs, the copy throws: "+c);
    }    
    
    copyName = null;
    elementName = null;
  } // copyACSorView()
  
  /**
  * This method is called from actionPerformed(). 
  * It is active only for GroupIDImpl and DirectoryImpl copy:
  * - when _obj is a GroupIDImpl, calls BaseUtilityImpl.copyGroupMembers()
  * to copy all the direct and indirect groupToCopy_ members,
  * then registers the current ACSTreeBaseListenerImpls as listeners
  * for the 'l_Members' bound property of each GroupID,
  * and to register all the groupToCopy_ GroupIDMembers in _obj.
  * - when _obj is a DirectoryImpl,
  * to copy all the children and children of children with
  * the standard copy() method of the Copyable interface. 
  * <P>
  * Called by BaseObjectHandler for actionPerformed().
  * @param _obj is the instance which has been created
  */
  public void handleNewBaseObject(BaseObject _obj) {
/*
            // for testing and tracking
            //System.out.println("Call to ActionNewEligibleParty. "+
            //"New eligible party:"+ _obj.toString() );
      
    GroupID[] l_grpToListen_one = null;
    GroupID[] l_grpToListen_two = null;
    GroupID[] l_grpToListen_three = new GroupID[0]; //
    GroupID[] l_grpToListen_four = null; // transient array
    int iter = 0;
     
    // registers the ACSTreeBaseListenerImpls and creates the members.
    // To see the new members in the explorer, each copied groupID must register
    // the listeners before the creation of its members. 
    if (_obj instanceof ImmutableGroupID) { 
         ActionUtilities.getInstance().registerACSTreeListenerInGroup(
           (ImmutableGroupID)_obj);
         
         // this is the structure of the array returned by BaseUtilityImpl.copyOrAddGroupMembers(),
         // that is a list of couples (groupToCopy, copiedGroup)
         l_grpToListen_one = new GroupID[] {groupToCopy_, (GroupID)_obj};
         
         while (l_grpToListen_one.length != 0) {
             for (int i = 0; i < l_grpToListen_one.length/2; i++) {
                 
                 if (l_grpToListen_one[2*i] == null || 
                   l_grpToListen_one[2*i].getEorL_Members().length == 0) continue;

                 // fires event to each "l_Members" listener   
                 l_grpToListen_two = BaseUtilityImpl.copyOrAddGroupMembers(
                   l_grpToListen_one[2*i], l_grpToListen_one[2*i + 1]);

                 //if (l_grpToListen_two.length is not pair) throw new InternalError(
                   //"in ActionCopy.handleNewBaseObject(), BaseUtilityImpl.copyGroupsTree()"+
                   //" has returned a wrong array.");

                             // for testing and tracking
                             //System.out.println("In ActionCopy.handleNewBaseObject(), for loop "+iter);
                             //for (int j = 0; j < l_grpToListen_two.length; j++) {  
                                //System.out.println(" ------------- "+l_grpToListen_two[j].getName());
                             //} // for j
                 
                 if (l_grpToListen_two.length == 0) continue;

                 // if it is the same ACS, the GroupIDMembers are added to _obj,
                 // not copied. Else,
                 // registers the new GroupIDs as listeners
                 if (groupToCopy_.getEorACS() != ((GroupID)_obj).getEorACS()) {
                     for (int j = 0; j < l_grpToListen_two.length/2; j++) {  
                        // works even if the listener has been registered
                        ActionUtilities.getInstance().registerACSTreeListenerInGroup(
                          l_grpToListen_two[2*j + 1]);
                     } // for j
                 } // if

                 // adds l_grpToListen_two to l_grpToListen_three
                 l_grpToListen_four = new GroupID[l_grpToListen_three.length + 
                   l_grpToListen_two.length];
                 System.arraycopy(l_grpToListen_three, 0, l_grpToListen_four, 
                   0, l_grpToListen_three.length);
                 System.arraycopy(l_grpToListen_two, l_grpToListen_three.length,
                   l_grpToListen_four, l_grpToListen_three.length, l_grpToListen_two.length);
                 l_grpToListen_three = l_grpToListen_four;
             } // for i < l_grpToListen_one
             
             l_grpToListen_one = l_grpToListen_three;
             l_grpToListen_three = new GroupID[0];
         } // while
         
         // resets this method
         groupToCopy_ = null;
    } // if (_obj instanceof ImmutableGroupID)

     
    // creates the children and the children of children
    if (_obj instanceof Directory && dirToCopy_ != null) {
            BaseUtilityImpl.copyDirectoryChildren(dirToCopy_, (Directory)_obj);
            dirToCopy_ = null;
    } // if (_obj instanceof Directory
     
     
    l_grpToListen_one = null; 
    l_grpToListen_two = null;
    l_grpToListen_three = null;
    l_grpToListen_four = null;
    */
    return;
  } // handleNewBaseObject()
  
  /**
  * This method is called from copyACSorView().
  * Requets to the user to copy all the bridge targets of the ACS.
  * @param _initialACS
  * @param _copiedACS
  */
  /*
  private void copyACSBridges(ACS _initialACS, ACS _copiedACS) {

     if (!_initialACS.getContainsBridgeSource()) return;
     
     Actor source;
     ImmutableActor target;
     String elementName;
     // sets the copy parameters
     Object[] l_fixedValue;

     for (Iterator ite = _initialACS.getEorL_OwnActorBridgeSources().iterator(); ite.hasNext();) {
        source = (Actor)ite.next();
        target = source.getEorACSBridgeTarget();
        elementName = "After the ACS copy, do you want to also copy this bridge to another ACS?";

        if (!DialogBox.showConfirmDialog(elementName, "The bridge target to copy is "+
          target.getName())) continue;

        copiedSource_ = _copiedACS.getEorM_OwnActors().get();
        
        l_fixedValue = new Object[] {null, (ACS)target.getEorACS(),
          (Directory)target.getEorParent(), null};
        
        // requests to the user the parameters,
        // uses the copy() method, and
        // calls back the handleNewBaseObject() method
        BaseObjectHandler.valueOf(this, target).
          getBaseObjectEditor(l_fixedValue);
        
        
        // TO DO: set the new bridge

     } // for
     
     elementName = null;
     source = null;
     target = null;
     l_fixedValue = null;
     return;
  } // copyACSBridges()
  */
  
} // class ActionCopy