//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import java.awt.event.*;
import ARoad0.Pattern.*;

import ARoad0.Gui1.*;
import ARoad0.gWork.ACS_Facade;
import ARoad0.gWork.ViewInBase_Facade;
import ARoad0.ErrB.BaseError;
//import ARoad0.ErrW.ProcessError;
//import javax.swing.SwingWorker;

/**
 * An action that exits Access Road and asks to the user
 * if a saving of each ACS is required.
 * Do not close the HTTP help pages since they are opened in the web browser.
 * Called from the main menu and when the main frame is closing.
 * @since 0.5.0
 * @version 0.7.0
 */
public class ActionExit extends AbstractAction implements Singleton {

  /* the only instance of the class, following singleton pattern */
  private static ActionExit         instance__ = new ActionExit();

  private static ViewInBase_Facade  ViewBase__ = ViewInBase_Facade.getInstance();

  /**
   * with this private constructor, the compiler won't generate
   * a default public constructor
   */
  private ActionExit() {
    super("Exit");
  }

  /**
   * constructor()
   * @return ActionExit
   */
  public static synchronized ActionExit getInstance(){
    return instance__;
  } // getInstance()

  /**
   * Exit without user confirmation.
   * Answer to select the saving of the open views, ACS and GUI configuration.
   * Closes the open views and ACS, then the desktop, including an open sketch view.
   * @param ev called from the main menu and when the main frame is closing.
   */
  public void actionPerformed(ActionEvent ev) {

                        // for testing and tracking
                        //System.out.println("Call to ActionExit.actionPerformed()");

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    //if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
    
    if (ACS_Facade.getOpenACS().isEmpty() ) {
      ARoad0.Gui1.Desktop.getInstance().setVisible(false);
      ARoad0.Gui1.Desktop.getInstance().dispose();
      System.exit(0);
      return;
    }

    // initialization
    JLabel          label;
    JPanel          panel;
    JDialog         dialog;
    JOptionPane     pane;
    label = new JLabel("Access Road is closing. "+
            "Do you want to save all the ACS, views and window locations?");
    label.setAlignmentX(label.CENTER_ALIGNMENT);
    panel = new JPanel();
    panel.add( label );
    //ViewInBase      view = null;
    //Map             m_GUIconfig = null;
    //Map             m_Config = new HashMap(10);
    //JInternalFrame[] allframes = null;
    //Map             map = null;
    //ImmutableName   name;
    boolean         saveAll = false;

    try {
      // display a question about ACS and views saving if there are some open ACS
      dialog = DialogBox.showYesNoCancelDialog("QUESTION", panel);

      // here, dialog waits an enter from the user,
      // then the next instructions are processed.
      pane = (JOptionPane)dialog.getContentPane().getComponent(0);
      
        // if YES, save the ACS, views, GUI configuration for all open frames
      if (pane.getValue() != null &&
          ((Integer)pane.getValue()).intValue() == JOptionPane.YES_OPTION) {
              saveAll = true;
              
                        // for testing and tracking
                        //System.out.println("ActionExit.actionPerformed(): saveAll ");
      }

      //DialogBox.showOkMessage("Please wait, while Access Road is closing.");
      //DialogBox.showNonModalOkMessage("Please wait, while Access Road is closing.");

       if (saveAll) {
            ActionSaveAll.getInstance().saveAll(false);
       } // if
       //else {
         //   try {
           //     Thread.currentThread().sleep(8000);
           // } catch (InterruptedException i) {
             //   System.out.println("in ActionExit, interruption on sleep: ");
             //   i.printStackTrace();
           // }
             // ARoad0.Gui1.Desktop.getInstance().setVisible(false);
             // ARoad0.Gui1.Desktop.getInstance().dispose();
             // System.exit(0);
            //return;
       //} // else

      /*
       // display a message in a worker thread to do no stop the exit
       SwingWorker worker_ = new SwingWorker<Void, Void>() {
            @Override
            public Void doInBackground() {
                // call in a worker thread - TODO: this method does not work
                DialogBox.showNonModalOkMessage("Please wait while Access Road is closing.");

                        // for testing and tracking
                        //System.out.println("End of SwingWorker.doInBackground() for "+
                          //      property_.getEorDataModel().getName());

                return null;
            } // doInBackground()

            //@Override
            //public void done() {
                // in the Event Dispatch Thread

                        // for testing and tracking
                        //System.out.println("Call to SwingWorker.done() for ");
                //try {
                    //get();
                //} catch (InterruptedException ignore) {}
                //catch (java.util.concurrent.ExecutionException e) {
                    //String why = null;
                    //Throwable cause = e.getCause();
                    //if (cause != null) {
                        //why = cause.getMessage();
                    //} else {
                        //why = e.getMessage();
                    //}
                    //System.err.println("Error in SwingWorker.done() for "+
                                //property_.getEorDataModel().getName()+" is: "+ e.toString());
                    //e.printStackTrace();
                //} // catch
            //} // done()

        }; // new SwingWorker<Void, Void>()

        worker_.execute();
       */

        ACS_Facade.closeDefaultBase();
       
    } catch (BaseError b) {
      DialogBox.showErrorMessage("Error in the closing of some ACS or views - "+
        b);
    //} catch (ProcessError p) {
      //DialogBox.showErrorMessage("Error in the saving of the GUI configuration - "+
        //p);
    }

    panel = null;
    label = null;
    dialog = null;
    pane = null;

                    // for testing and tracking
                //System.out.println("in ActionExit, exit. ");

    //try {
      //  Thread.currentThread().sleep(5000);
    //} catch (InterruptedException i) {
      //  System.out.println("in ActionExit, interruption on sleep: ");
      //  i.printStackTrace();
    //}

    ARoad0.Gui1.Desktop.getInstance().setVisible(false);
    ARoad0.Gui1.Desktop.getInstance().dispose();
    System.exit(0);
  } // actionPerformed(ActionEvent)

} // class ActionExit