//Title:        Access Road
//Version:      0.7.2
//Copyright:    2002, 2012 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;
import java.io.File;
import java.io.IOException;
import java.io.FileNotFoundException;
import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.gBaseInterface.ImmutableName;

/**
 * An action that saves an explorer, a structure, a view or a beamer image
 * in the PNG format.
 * @version 0.7.2
 */
public class ActionExportPNG extends AbstractAction implements Singleton {

  /* the only instance of the class, following singleton pattern */
  private static ActionExportPNG instance_ = new ActionExportPNG();

  private JDesktopPane desk; // the desktop to work with


  /**
   * with this private constructor, the compiler won't generate
   * a default public constructor
   */
  private ActionExportPNG() {
    super("Export image");
  } /**
   * constructor()
   */
  public static synchronized ActionExportPNG getInstance(){
    return instance_;
  } // getInstance()

  /**
   * Selects the view if it is the object to export, then the beamer, then the structure
   * and finally selects the explorer. Call exportPNG().
   * @param _ev is the ActionEvent
   */
  public void actionPerformed(ActionEvent _ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
    
    // initialization
    Object[]               l_analyzedEvent = new Object[3];

    l_analyzedEvent = ActionUtilities.getInstance().analyzeActionEvent(_ev);

                      // for testing and tracking
                //System.out.println("in ActionExportPNG.actionPerformed()"+
                  //"l_analyzedEvent: ");
                //for (int i = 0 ; i < l_analyzedEvent.length ; i++) {
                  //if (l_analyzedEvent[i] != null)
                    //  System.out.println(l_analyzedEvent[i].toString());
                //}
    
    if (l_analyzedEvent[0] == null &&
          !InternalFrameModel.getActiveFrame().getTitle().trim().equals("Explorer") &&
          !InternalFrameModel.getActiveFrame().getTitle().trim().equals("IS Structure") ) {
                // displays a warning message
                DialogBox.showErrorMessage(
                  "No selection or unworkable selection.");
                l_analyzedEvent = null;
                return;
    } // if
    
    if ((l_analyzedEvent[0] != null &&
      !((String)l_analyzedEvent[0]).equals("view") &&
      !((String)l_analyzedEvent[0]).equals("beamer") &&
      !((String)l_analyzedEvent[0]).equals("structure")) &&
      !InternalFrameModel.getActiveFrame().getTitle().trim().equals("Explorer")
      ) {
        // displays a warning message
        DialogBox.showErrorMessage(
          "A view, the explorer, the structure or the beamer should be selected "+
            "to be exported in a PNG image.");
        return;
    }

    ImmutableName name = (ImmutableName) l_analyzedEvent[1];

    String str = null;
    if (name != null && name.noOfComponents() < 2 ) str = name.lastComponent();
    else if (name != null) {
        str = name.context().lastComponent() + "_" + name.lastComponent();
    }
                    //System.out.println("in ActionExportPNG.actionPerformed(), "+
                      //"InternalFrameModel.getActiveFrame(): "+InternalFrameModel.getActiveFrame().getTitle().trim());

    if (l_analyzedEvent[0] != null && ((String)l_analyzedEvent[0]).equals("view") ) {

      exportPNG((JComponent)InternalFrameModel.
        getJComponentsOfBaseObjects().get(l_analyzedEvent[1]), "view", str, true);

    } else if (l_analyzedEvent[0] != null && ((String)l_analyzedEvent[0]).equals("beamer")) {

                    //System.out.println("in ActionExportPNG.actionPerformed() for beamer, "+
                      //"bemer pane: "+InternalFrameModel.getBeamerPane());

      exportPNG(InternalFrameModel.getBeamerPane(), "beamer", str, true);
    } else if (l_analyzedEvent[0] != null && ((String)l_analyzedEvent[0]).equals("structure")) {

                    //System.out.println("in ActionExportPNG.actionPerformed() for the structure, "+
                      //"structure pane: "+InternalFrameModel.getStructurePane());

      exportPNG(InternalFrameModel.getTreeInStructure(), "structure", null, true);

    } else { // the explorer is the active frame

      if (InternalFrameModel.withExplorer())
          exportPNG(InternalFrameModel.getTreeInExplorer(), "explorer", null, true);

    } // else

    name = null;
    str = null;
  } // actionPerformed(ActionEvent)

  /**
   * Exports in a file an image derived from the local configuration.
   * For a NMNL view, removes the form '<>' in the view name.
   * @param _component to export in PNG
   * @param _type is equals to "view", "explorer", "beamer", "structure"
   * @param _name is the exported object name. May be null if 'explorer'.
   * @param _toFile true to export to the local file system, false to simply return the image
   * @return exported image
   */
  public BufferedImage exportPNG(JComponent _component, String _type,
    String _name, boolean _toFile) {
                    
                        // for testing and tracking
                    //System.out.println("in ActionExportPNG.exportPNG(), "+
                      //"InternalFrameModel.getActiveFrame(): "+
                      //InternalFrameModel.getActiveFrame().getTitle().trim());
                    
    String str = null;
    File f = null;
    GraphicsEnvironment local = GraphicsEnvironment.getLocalGraphicsEnvironment();
    GraphicsDevice screen = local.getDefaultScreenDevice();
    GraphicsConfiguration configuration = screen.getDefaultConfiguration();
    
                    //System.out.println("in ActionExportPNG.exportPNG(), "+
                      //"_component: "+((JInternalFrame)_component).getTitle().trim());    
                    //System.out.println("in ActionExportPNG.exportPNG(), "+
                      //"configuration: "+configuration); 
                    
    BufferedImage image = configuration.createCompatibleImage(
      _component.getWidth(), _component.getHeight());
    Graphics2D graphics = image.createGraphics();
    _component.paint(graphics);

    if (_type.equals("view") && _name.indexOf("<") != -1) {
        // remove '<' and '>'
      str = _name.substring(0, _name.indexOf("<")) + 
              _name.substring(_name.indexOf("<") + 1, _name.indexOf(">")) +
              _name.substring(_name.indexOf(">") + 1);
      str = "Access_Road_view_" + str + ".png";
    }
    else if (_type.equals("view")) {
      str = "Access_Road_view_" + _name + ".png";
    }
    else if (_type.equals("beamer")) {
      str = "Access_Road_beamer_on_" + _name + ".png";
    }
    else if (_type.equals("explorer")) {
      str = "Access_Road_explorer.png";
    }
    else if (_type.equals("structure")) {
      str = "Access_Road_full_structure.png";
    } // else// else

    f = new File(System.getProperty("user.dir") + ARoad0.Gui1.Desktop.FILES_SEPAR +
      "export" + ARoad0.Gui1.Desktop.FILES_SEPAR + str);

    if (_toFile) {
      if (!DialogBox.showConfirmDialog(
              "Export in the file "+str+"?",
              "If a file has this name in 'export', it will be deleted.")) {
        f = null;
        str = null;
        local = null;
        screen = null;
        configuration = null;
        graphics = null;
        return null;
      } // if
        
      try {
        ImageIO.write(image, "png", f);
        DialogBox.showOkMessage(
          "Exporting of "+str);
      } catch (FileNotFoundException fnfe) {
        // display a message
        DialogBox.showErrorMessage(
          "No exporting. The 'export' directory is not "+
          "directly under the Access Road working directory.");
          fnfe.printStackTrace();
      } catch (IOException io) {
        // display a message
        DialogBox.showErrorMessage(
          "No exporting. The 'export' directory may be not "+
          "directly under the Access Road working directory.");;
      } catch (IllegalArgumentException iae) {
        // display a message
        DialogBox.showErrorMessage(
          "No exporting. There is a bug.");
      }
    } // if (_graphics == null) {

    f = null;
    str = null;
    local = null;
    screen = null;
    configuration = null;
    graphics = null;
    return image;
  } // exportPNG()

  } // class ActionExportPNG