//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.border.BevelBorder;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import java.awt.event.*;
import javax.swing.border.LineBorder;
import java.awt.*;
import java.beans.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Arrays;
import java.util.Collections;
import java.util.SortedMap;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Constructor;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.Gui2.*;
import ARoad0.ErrB.BaseError;
import ARoad0.ErrB.UpDateError;
import ARoad0.ErrB.CreateError;
import ARoad0.ErrW.*;

import ARoad0.gWork.ACS_Facade;
import ARoad0.gWork.AcsFactory;
import ARoad0.gBaseInterface.*;
import ARoad0.gBase.*;

/**
 * An action that creates a new access control system and the related new components
 * in an information system.
 * This class manages all the dialog boxes required for the user input,
 * controls the inputs, creates through gBase.ACSFactoryImpl the ACS in the base,
 * invokes ACSTree.createACSExplorer(),
 * then displays the new ACS resources in a tree with InternalFrame.showTreeFrame(),
 * and updates the explorer through InternalFrame.
 * @version 0.7.0
 */
public class ActionNewACSyst extends AbstractAction implements Singleton,
    CreationDialogListener {

  /**
   *  the only instance of the class, following singleton pattern
   */
  private static ActionNewACSyst    instance__ = new ActionNewACSyst();

  // variables for the dialog
  private static JDialog            dialog__ = null;
  
  /**
   * Current CommonPropertyEditors in the user dialog. The key is the display name.
   */
  private Map                       m_currentPropertyEditors_ = new HashMap(40);

  // variables for the user data
  private JTextField                isField_;
  private JTextField                acsField_;
  private Name                      parent_ = null;
  private Dimension                 dimension_ = new Dimension(150, 25); 
  private ButtonGroup               group_;
  private Name                      name_ = null;
  private Name                      iSname_ = null;
  private String                    nickName_ = null;

  /**
   * Current factory when the class instance is working for building up an ACS
   * from the user choices
   */
  private ACSFactoryImpl            factory_ = null;
  /**
   * The list from getL_AcsAddonDescriptors in the class BaseManagerImpl.
   */
  List<String>                      l_addonDescriptors_ = null;
  /**
   * In createUserDefinedACS() and createFirstACSproperties(),
   * used to detect the frame closing from the okButton
   */
  private boolean                   toFinalize_ = true;
  
  // environment
  private static String             lineSepar__ = System.getProperty("line.separator");

  // for setNewComponent()
  private JList                     componentList_ = null;
  private JTextField                componentField_ = new JTextField();
  private ImmutableName             currentAncestor_ = null;

  /**
  * with this private constructor, the compiler won't generate
  * a default public constructor
  */
  private ActionNewACSyst() {
    super("New Access Control System");

    //initialization
    DefaultListModel componentsModel = new DefaultListModel();
    for (String type : ACSFactoryUtilityImpl.getL_LimitedValuesForComponentNodes(null, null)) {
        componentsModel.addElement( type );
    } // for ite

    componentList_ = new JList(componentsModel);
    componentField_.setText("                 ");
    
    l_addonDescriptors_ = BaseManagerImpl.getInstance().getL_AcsAddonDescriptors();
  } // ActionNewACSyst()

  /**
   * single public method to get the unique instance
   * @return the unique instance
   */
  public static synchronized ActionNewACSyst getInstance() {
    return instance__;
  } // getInstance()

  /**
  * Manages the actions fired from the main menu,
  * and displays a dialog box. Controls the user imputs (ACS name and type).
  * @param ev ActionEvent
  */
  public void actionPerformed(ActionEvent ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
    if (acsField_ != null) {
        DialogBox.showErrorMessage(
            "There is an on-going ACS creation. It should be cancelled first.");
        return;
    } // if
    
    // variables for the dialog box
    JOptionPane               pane;
    JPanel                    panel;
    JPanel                    panel2;
    JRadioButton              rbutton;
    char                      a = 'A';
    char                      r = 'R';
    char                      y = 'Y';
    char                      u = 'U';
    char                      m = 'M';
    char                      b = 'B';
    char                      o = 'O';
    
    // fired from the main menu, so
    // displays a dialog box for the ACS name input

      // initialization to each new dialog box
    isField_ = new JTextField();
    acsField_ = new JTextField();
    isField_.setActionCommand("isField_");
    acsField_.setActionCommand("acsField_");
    isField_.setPreferredSize(dimension_);
    isField_.setMaximumSize(dimension_);
    isField_.setMinimumSize(dimension_);
    acsField_.setPreferredSize(dimension_);
    acsField_.setMaximumSize(dimension_);
    acsField_.setMinimumSize(dimension_);
    
    panel = new JPanel();
    panel.setFont(BeamerManager.plainFont);
    panel2 = new JPanel();
    BoxLayout box = new BoxLayout(panel2, BoxLayout.Y_AXIS);
    panel2.setLayout(box);
    group_ = new ButtonGroup();

    JLabel label3;
    JLabel label2;
    JLabel label;

    label3 = new JLabel("Use Mouse or Tab to edit");
    label3.setAlignmentX(label3.CENTER_ALIGNMENT);
    panel.add( label3 );
    label3 = new JLabel(" "); // to do a space
    label3.setAlignmentX(label3.CENTER_ALIGNMENT);
    panel.add( label3 );
    label3 = new JLabel("Name of the ACS IS"); 
    label3.setAlignmentX(label3.CENTER_ALIGNMENT);
    panel.add( label3 );
    isField_.setAlignmentX(isField_.CENTER_ALIGNMENT);
    panel.add(isField_);
    label2 = new JLabel(" "); // to do a space
    label2.setAlignmentX(label2.CENTER_ALIGNMENT);
    panel.add( label2 );
    label2 = new JLabel("Nick name of the ACS");
    label2.setAlignmentX(label2.CENTER_ALIGNMENT);
    panel.add(label2 );
    acsField_.setAlignmentX(acsField_.CENTER_ALIGNMENT);
    panel.add(acsField_);
    label = new JLabel(" "); // to do a space
    label.setAlignmentX(label.CENTER_ALIGNMENT);
    panel.add( label );
    label = new JLabel("Select one button");
    label.setAlignmentX(label.CENTER_ALIGNMENT);
    panel.add( label );

    String          editor = null;
    String          addon = null;
    String          acsAddonName = null;
    Class           factory = null;

    try {

        for (String value : l_addonDescriptors_) {

                        // for testing purpose
                        //System.out.println(" in ActionNewACSyst.actionPerformed((), "+
                          //      "process the AcsAddon: "+value);

            //The form of a value is 'Editor: editor_name - AcsAddon: addon_name'
            editor = value.substring(8, value.indexOf(" - AcsAddon: "));
            addon = value.substring(value.indexOf("AcsAddon: ")+10);

            factory = Class.forName(
                    "ARoad0.AcsAddon."+editor+"."+addon+"."+"gWork.AcsFactory"+addon);

            acsAddonName = ((AcsFactoryForAcsAddon)factory.newInstance()).
                                getPublicAcsAddonName();

            rbutton  = new JRadioButton(acsAddonName);
            panel2.add(rbutton );
            rbutton.setActionCommand(acsAddonName);
            rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
            rbutton.setBackground(InternalFrame.BACKCOLOR_2);
            rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
            group_.add(rbutton);

        } // for value

    }
    catch (ClassNotFoundException c) {
        throw new InternalError(" The AcsAddon class is not found: "+c.toString());
    }
    catch (InstantiationException i) {
        throw new InternalError(" The AcsAddon class is not instantiatied: "+i.toString());
    }
    catch (IllegalAccessException k) {
        throw new InternalError(" The AcsAddon class is not instantiatied: "+k.toString());
    }

    rbutton  = new JRadioButton("RBAC application (by Accbee, France, EU)");
    panel2.add(rbutton );
    rbutton.setActionCommand("RBAC application");
    //rbutton.setMnemonic(r);
    rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
    rbutton.setBackground(InternalFrame.BACKCOLOR_2);
    rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
    group_.add(rbutton);
    rbutton = new JRadioButton("Derived from an open ACS, with the default components");
    panel2.add(rbutton);
    rbutton.setActionCommand("Derived from an open ACS");
    //rbutton.setMnemonic(a);
    rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
    rbutton.setBackground(InternalFrame.BACKCOLOR_2);
    rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
    group_.add(rbutton);
    rbutton  = new JRadioButton("Empty ACS to specify completely by hand");
    panel2.add(rbutton );
    rbutton.setActionCommand("Empty ACS to specify completely by hand");
    //rbutton.setMnemonic(o);
    rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
    rbutton.setBackground(InternalFrame.BACKCOLOR_2);
    rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
    group_.add(rbutton);
    rbutton  = new JRadioButton("An ACS for testing VirtualFolders and Privileges");
    panel2.add(rbutton );
    rbutton.setActionCommand("An ACS for testing VirtualFolders and Privileges");
    //rbutton.setMnemonic(o);
    rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
    rbutton.setBackground(InternalFrame.BACKCOLOR_2);
    rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
    group_.add(rbutton);
    rbutton  = new JRadioButton("An ACS for testing AG/ACL/PRI rights and AG inheritance");
    panel2.add(rbutton );
    rbutton.setActionCommand("An ACS for testing AG/ACL/PRI rights and AG inheritance");
    //rbutton.setMnemonic(o);
    rbutton.setAlignmentX(rbutton.LEFT_ALIGNMENT);
    rbutton.setBackground(InternalFrame.BACKCOLOR_2);
    rbutton.setForeground(InternalFrame.FORECOLOR_BLACK);
    group_.add(rbutton);
    
    panel2.setBackground(InternalFrame.BACKCOLOR_2);
    panel2.setForeground(InternalFrame.FORECOLOR);
    panel.add(panel2);
    //panel.setSize(200, 75);

    // shows the dialog box
    dialog__ = DialogBox.showOkCancelDialog("Create a new ACS", panel);

    // here, dialog waits an enter from the user,
    // then the next instructions are processed.
      // if the user enters a "return" in an editable field,
      // it is also processed here as OK_OPTION.

    pane = (JOptionPane)dialog__.getContentPane().getComponent(0);
    // no action if CLOSED, and closes the dialog box
    if ( pane.getValue() == null) {
        acsField_ = null;
        return;
    }
    // else, it is an Integer
    int value = ((Integer)pane.getValue()).intValue();

      // no action if CANCEL, and closes the dialog box
    if (value == JOptionPane.CANCEL_OPTION) {
        acsField_ = null;
        return;
    }
      // if OK, action and close the dialog box
    if (value == JOptionPane.OK_OPTION) prepareACS();

    factory = null;
    pane = null;
    panel = null;
    panel2 = null;
    rbutton = null;
    label3 = null;
    label2 = null;
    label = null;
    box = null;
  } // actionPerformed(ActionEvent)
  
  /**
   * Invoked by actionPerformed() to check and to select the parent ACS.
   * If the ACS-name field is empty, an error message is displayed.
   * Its IS name is set to "DefaultIS" if it is empty. If the complete ACS name
   * is already known by the base, an error message is displayed.
   * There is no control of space character at the end of any name
   * entered by the user, nor control of the name size.
   * Calls DialogBox.showExplorerSelectionDialog() which calls handleSelectionDialog().
   * */
   private void prepareACS() {
    // displays a warning message if the ACS name is empty
    if ( acsField_.getText() == null || acsField_.getText().trim().length() == 0 ) {
        DialogBox.showErrorMessage("The name is empty."+
          lineSepar__+"Enter a value please.");
        endACScreation(null);
        return;
    }

    // displays a warning message if no radio button is selected
    if (group_.getSelection() == null) {
        DialogBox.showOkMessage("No selection."+
          " Choose a selection to create a new ACS.");
        endACScreation(null);
        return;
    }

    // select IS name if it is empty or too short
    if (isField_ == null || isField_.getText() == null || 
            isField_.getText().trim().length() == 0 ) {
        iSname_ = null;
    }
    else {
        iSname_ = new NameImpl(isField_.getText().trim() );
    }

    // acsField_ is the new ACS nickName
    nickName_ = acsField_.getText().trim();

    // validateName(name) display an user warning message if a name component
    // contains ':: ', or '>>', or '<>' at the start and the end
    if (iSname_ == null) {
        if (!ACS_Facade.validateFrameName(new NameImpl(nickName_)) ) {
            DialogBox.showErrorMessage(
              "A name cannot contain ':: ', or '>>', or '<>' at the start and the end.");
            endACScreation(null);
            return;
        }
    }
    else if (!ACS_Facade.validateFrameName( new NameImpl(nickName_) ) ||
            !ACS_Facade.validateFrameName(iSname_)) {
        DialogBox.showErrorMessage(
          "A name cannot contain ':: ', or '>>', or '<>' at the start and the end.");
        endACScreation(null);
        return;
    }
    
    InternalFrame.openExplorer();
    
    if (iSname_ == null) {
        name_ = new NameImpl(nickName_);
        parent_ = null;
        createACS();
        return;
    }
    
    setCurrentAncestor();
  } // prepareACS()
  
  /**
  * Called by prepareACS() to select the current ancestor in the IS.
  * Calls setNewComponent() to create new logical or physical components
  * as ancestors of the new ACS, at the request of the user. Calls createACS().
  */
  public void setCurrentAncestor() {

        // for testing purpose
        //System.out.println("setCurrentAncestor() called in ActionNewACSyst");
      
    ImmutableNameISTree nametree = BaseManagerImpl.getInstance().getISTree(iSname_);
    Name                currentAncestor = null;
    boolean             newComponents = false;

    if (nametree == null) {
        newComponents = DialogBox.showConfirmDialog(
                "Do you want to create new component(s) first?",
                "They will be put between the IS root and the new ACS.");

        if (!newComponents) {
            parent_ = iSname_;
            name_ = ((Name)((NameImpl)parent_).clone()).extended(nickName_);
            createACS();
        } // if
        else {
            try {
                BaseManagerImpl.getInstance().addIS(iSname_, true);
            }
            catch (BaseError b) {
                throw new InternalError("ActionNewACSyst.setCurrentAncestor(): "+b.toString());
            }
            
            setNewComponent(iSname_);
        } // else

        return;
    } // if (nametree == null

    Set                 l_nodes = nametree.nodes();
    List<Comparable>    l_currentNodes = new ArrayList( l_nodes.size() );
    int                 index = -1;
    List<String>        l_allowedTypes = Arrays.asList(
                            ISFactoryUtilityImpl.INFORMATION_SYSTEM_NODE,
                            ISFactoryUtilityImpl.PHYSICAL_COMPONENT_NODE,
                            ISFactoryUtilityImpl.LOGICAL_COMPONENT_NODE,
                            ISFactoryUtilityImpl.ACS_NODE);

    try {
        for (Object name : l_nodes) {
            if (!l_allowedTypes.contains(nametree.getType((ImmutableName)name)))
                continue;
            l_currentNodes = BaseUtilityImpl.addObjectInDecreasingSort(
                    l_currentNodes, (ImmutableName) name);
        } // for name
    }
    catch (BaseError b) {
        throw new InternalError("ActionNewACSyst.setCurrentAncestor(): "+b);
    }

    Collections.reverse(l_currentNodes);

    try {
        if (l_currentNodes.size() < 25) {
            index = DialogBox.showOneSelectionOneDialog(
                "direct parent or the most-near ancestor",
                l_currentNodes.toArray(new ImmutableName[0]));
            if (index == -1) {
                l_nodes = null;
                l_allowedTypes = null;
                l_currentNodes = null;
                endACScreation(null);
                return;
            }
            currentAncestor = (Name)l_currentNodes.toArray(new ImmutableName[0])[index];
        } // if
        else {
            Set<ImmutableName> l_someNodes = new HashSet(25);
            int count = 0;

            while (index == -1) {
                for (Object name : l_currentNodes) {
                    l_someNodes.add((ImmutableName)name);
                    count++;
                    if (count > 25) break;
                }
                count = 0;
                l_currentNodes.removeAll(l_someNodes);

                index = DialogBox.showOneSelectionOneDialog(
                    "direct parent or the most-near ancestor, or skip this list",
                    l_someNodes.toArray(new ImmutableName[0]));
                if (index != -1)
                    currentAncestor = (Name)l_someNodes.toArray(new ImmutableName[0])[index];
            } // while

            l_someNodes = null;
        } // else
    }
    catch (SmallError s) {
        throw new InternalError("ActionNewACSyst.setCurrentAncestor(): "+s);
    }

    if (currentAncestor == null) {
        newComponents = DialogBox.showConfirmDialog(
                "Do you want to create new component(s) first?",
                "They will be put between the IS root and the new ACS.");
    }
    else {
        newComponents = DialogBox.showConfirmDialog(
                "Do you want to create new component(s) first?",
                "They will be put between the selected ancestor and the new ACS.");
    }

    if (!newComponents) {
        nametree = null;
        l_currentNodes = null;

        if (currentAncestor == null) parent_ = iSname_;
        else parent_ = currentAncestor;

        name_ = ((Name)((NameImpl)parent_).clone()).extended(nickName_);
        createACS();
        return;
    } // if

    if (currentAncestor == null)
        setNewComponent(iSname_);
    else
        setNewComponent(currentAncestor);

    nametree = null;
    currentAncestor = null;
    l_currentNodes = null;
    l_allowedTypes = null;
  } // setCurrentAncestor()

  /**
  * Called by setCurrentAncestor() to create a new logical or physical component
  * as an ancestor of the new ACS. Calls the reentrant method DialogBox.showBackedJLists()
  * which calls handleCreationDialog(), and this method may calls back setNewComponent()
  * to create another component as a child of the previous one. Sets currentAncestor_.
  * @param _currentAncestor current node in the IS nametree
  */
  public void setNewComponent(ImmutableName _currentAncestor) {

        // for testing purpose
        //System.out.println("setNewComponent() called in ActionNewACSyst");

    currentAncestor_ = _currentAncestor;

    try {
        // calls handleCreationDialog() when the user closes the data selection frame
        DialogBox.showBackedJLists(this, "NEW COMPONENT under "+_currentAncestor.lastComponent(),
                new JList[] {componentList_}, new String[] { "type", "name"},
                false, componentField_, true);
    } catch (SmallError s) {
        s.printStackTrace();
        throw new InternalError("ActionNewACSyst.setNewComponent(), "+
                "exception: "+s);
    }
  } // setNewComponent()

/**
 * Called by DialogBox.showBackedJLists() for setNewComponent() which may be recalled.
 * Then it is a partially reentrant method which handles componentField_ and
 * componentList_ at each invocation. It sets name_ and parent_ before the call to createACS().
 * Calls ISFactoryUtilityImpl.getChildTypesForISnode() and
 * BaseManagerImpl.addComponentNodeToIS(). At the end of the loop, calls createACS().
 * @param _data true if the user clicks on OK, and false if the user clicks on Cancel
 */
  public void handleCreationDialog(boolean _data) {
    if (!_data || componentList_ == null) {
        DialogBox.showOkMessage(
            currentAncestor_.lastComponent()+" will be the new ACS parent");
        parent_ = (Name)currentAncestor_;
        name_ = ((Name)((NameImpl)parent_).clone()).extended(nickName_);
        createACS();
        return;
    } // if
    
    String selection = (String)componentList_.getSelectedValue();

    if (componentList_.getModel().getSize() == 1)
        selection = (String)componentList_.getModel().getElementAt(0);
    
    if (selection == null ) {
        DialogBox.showOkMessage(
            "No selected type - "+currentAncestor_.lastComponent()+
            " will be the new ACS parent");
        parent_ = (Name)currentAncestor_;
        name_ = ((Name)((NameImpl)parent_).clone()).extended(nickName_);
        createACS();
        return;
    } // if

    if ((componentField_.getText() == null || componentField_.getText().trim().length() < 1)) {
        DialogBox.showOkMessage(
            "No name - "+currentAncestor_.lastComponent()+
            " will be the new ACS parent");
        parent_ = (Name)currentAncestor_;
        name_ = ((Name)((NameImpl)parent_).clone()).extended(nickName_);
        createACS();
        return;
    } // if
    else if (!ACS_Facade.validateFrameName(new NameImpl(componentField_.getText().trim()) ) ) {
        DialogBox.showErrorMessage(
          "A name cannot contain ':: ', or '>>', or '<>' at the start and the end.");

        // try to enter a new node
        componentList_.clearSelection();
        componentField_.setText("                 ");
        setNewComponent(currentAncestor_);
        return;
    } // else


    ImmutableName       newAncestor = ((Name)currentAncestor_.clone()).
                                        extended(componentField_.getText().trim() );
    ImmutableNameISTree nametree = BaseManagerImpl.getInstance().getISTree(iSname_);
    List<String>        l_authorizedChildren = null;

    if ( nametree.isNode(newAncestor) || nametree.isArea(newAncestor)) {
        DialogBox.showErrorMessage(
                    "Sorry, this new node name is already known in this information system.");

        // try to enter a new node
        componentList_.clearSelection();
        componentField_.setText("                 ");
        setNewComponent(currentAncestor_);
        return;
    } // if

    try {
        l_authorizedChildren = ISFactoryUtilityImpl.
            getChildTypesForISnode(nametree.getType(currentAncestor_));

        if ((!selection.equals("physical") &&
                !l_authorizedChildren.contains(ISFactoryUtilityImpl.LOGICAL_COMPONENT_NODE)) ||
                (selection.equals("physical") &&
                !l_authorizedChildren.contains(ISFactoryUtilityImpl.PHYSICAL_COMPONENT_NODE))) {
            DialogBox.showErrorMessage(
                        "Sorry, the parent node '"+currentAncestor_.lastComponent()+
                        "' forbids '"+selection+"' for its children.");

            // try to enter a new node
            componentList_.clearSelection();
            componentField_.setText("                 ");
            setNewComponent(currentAncestor_);
            return;
        } // if

        if (!selection.equals("physical"))
            BaseManagerImpl.getInstance().addComponentNodeToIS(
                    iSname_, newAncestor,
                    ISFactoryUtilityImpl.LOGICAL_COMPONENT_NODE);
        else
            BaseManagerImpl.getInstance().addComponentNodeToIS(
                    iSname_, newAncestor,
                    ISFactoryUtilityImpl.PHYSICAL_COMPONENT_NODE);
    } catch (BaseError b) {
        DialogBox.showErrorMessage(
          "End of operation - The creation of the last component throws: "+b.toString());
        b.printStackTrace();

                // for testing and tracking purpose
                System.out.println("ActionNewACSyst.handleCreationDialog() error,"+
                        " currentAncestor_: "+currentAncestor_);
                System.out.println("iSname_: "+iSname_);
                System.out.println("nickName_: "+nickName_);
                System.out.println("componentField_: "+componentField_.getText());
                System.out.println("newAncestor: "+newAncestor);

        endACScreation(null);
        return;
    } // catch

    if (DialogBox.showConfirmDialog("Do you want to create another component?",
            "It will be set as a child of the last new component.")) {
        componentList_.clearSelection();
        componentField_.setText("                 ");
        setNewComponent(newAncestor);
    }
    else {
        parent_ = (Name)newAncestor;
        name_ = ((Name)((NameImpl)parent_).clone()).extended(nickName_);

                // for testing purpose
                //System.out.println(" in ActionNewACSyst.handleCreationDialog(), "+
                  //      "call to createACS() with parent_ "+parent_+", and name_ "+name_+
                    //    ", for an ACS type "+group_.getSelection().getActionCommand());

        createACS();
    }

    newAncestor = null;
    selection = null;
    l_authorizedChildren = null;
    nametree = null;
  } // handleCreationDialog()

  /**
   * Called by prepareACS(), setCurrentAncestor() and handleSelectionDialog()
   * to create the ACS. 
   * For an AcsAddon, calls the correct implementation of AcsFactoryForAcsAddon.
   * The ACS type is the beginning of the Addon name, up to a '(' character.
   * iSname_ and parent_ may be null.
   * Calls createUserDefinedACS(), createFirstACSproperties(), endACScreation(),
   * selectNoNX_ACSType() and fillAcsRightsForDerivedACS(),
   * AcsFactory.createRBAC(), createTestForVFandPRI() and createTestOfAll(),
   * AcsFactoryForAcsAddon.create().
   * */
   private void createACS() {
    // variables
    ACSImpl acs = null;
    
    try {
      if (group_.getSelection().getActionCommand().equals("RBAC application") ) { 
          // creates a role-based access control application on an ACS operating system
              acs = createRBAC(name_);
              endACScreation(acs);
      } 
      else if (group_.getSelection().getActionCommand().equals(
              "An ACS for testing VirtualFolders and Privileges")) {
          // creates an ACS Test of VirtualFolders and Privileges 
              acs = AcsFactory.createTestForVFandPRI(iSname_, name_, 
                      "An ACS for testing VirtualFolders and Privileges",
                      parent_);
              endACScreation(acs);
      } // else
      else if (group_.getSelection().getActionCommand().equals(
              "An ACS for testing AG/ACL/PRI rights and AG inheritance")) {
          // creates an ACS Test of VirtualFolders and Privileges 
              acs = AcsFactory.createTestOfAll(iSname_, name_,
                      "An ACS for testing AG/ACL/PRI rights and AG inheritance",
                      parent_);
              endACScreation(acs);
      } // else
      else if (group_.getSelection().getActionCommand().equals(
              "Empty ACS to specify completely by hand") ) {
        // selects the type
        String  type = selectNoNX_ACSType();

        if (type.equals("no choice") ) {
            endACScreation(null);
            return;
        } // if (type

        // creates the ACS factory, which may be used by getCurrentACSFactory()
        factory_ = new ACSFactoryImpl(iSname_, name_, type, parent_);
        createFirstACSproperties(null);

        type = null;
      } 
      else if (group_.getSelection().getActionCommand().equals(
              "Derived from an open ACS") ) {
        // selects the reference ACS
        Set     l_acs = new HashSet(ACS_Facade.getOpenACS().values());
        String  type = null;
        Class   factory = null;
        Constructor  constructor = null;

        if (l_acs.isEmpty()) {
            DialogBox.showErrorMessage("Please, open first the reference ACS to derive from.");
            acs = null;
            endACScreation(null);
            return;
        } // if (type
        
        int index = -1;
        
        if (l_acs.size() > 1) {
            index = DialogBox.showOneSelectionOneDialog(
                "ACS to derive from", l_acs.toArray());
            if (index != -1)
                acs = (ACSImpl)l_acs.toArray()[index];
        }
        else
            acs = (ACSImpl)l_acs.iterator().next();

        if (acs == null)  {
            DialogBox.showErrorMessage("No ACS creation.");
            acs = null;
            endACScreation(null);
            return;
        } // if (type

        type = acs.getACSType();

        // creates the ACS factory, which may be used by getCurrentACSFactory(),
        // with the right ACSFactoryImpl class, even from an AcsAddon
        if (acs.getM_AcsVocabulary().get(ACSFactoryUtilityImpl.ACSFACTORY_CLASS)[0].
                equals("ARoad0.gBase.ACSFactoryImpl"))
            factory_ = new ACSFactoryImpl(acs.getISName(), name_, type, parent_);
        else {
            try {
                factory = Class.forName(
                    acs.getM_AcsVocabulary().get(ACSFactoryUtilityImpl.ACSFACTORY_CLASS)[0]);
                constructor = factory.getConstructor(new Class[] {
                        Class.forName("ARoad0.gBaseInterface.ImmutableName"),
                        Class.forName("ARoad0.gBaseInterface.ImmutableName"),
                        String.class,
                        Class.forName("ARoad0.gBaseInterface.ImmutableName")
                    });
                factory_ = (ACSFactoryImpl) constructor.newInstance(new Object[] {
                        acs.getISName(), name_, type, parent_
                    });
            }
            catch (ClassNotFoundException c) {
                throw new InternalError(" The AcsAddon ACSFactoryImpl class is not found: "+c.toString());
            }
            catch (InstantiationException i) {
                throw new InternalError(" The AcsAddon ACSFactoryImpl class is not instantiated: "+i.toString());
            }
            catch (IllegalAccessException k) {
                throw new InternalError(" The AcsAddon ACSFactoryImpl class is not instantiated: "+k.toString());
            }
            catch (NoSuchMethodException n) {
                throw new InternalError(" The AcsAddon ACSFactoryImpl class is not instantiated: "+n.toString());
            }
            catch (InvocationTargetException g) {
                throw new InternalError(" The AcsAddon ACSFactoryImpl class is not instantiated: "+g.toString());
            }
        } // else
        
        // fills factory_ with copies of the ACS rights from acs
        fillAcsRightsForDerivedACS(acs);
        
        // uses the detection of non-null acs to know if it is an acs '"derived from '
        createFirstACSproperties(acs);

        acs = null;              
        l_acs = null;
        type = null;
        factory = null;
        constructor = null;
      }
      
      // ----------------- From an AcsAddon
      else {
            String    acsAddonName = null;
            String    editor = null;
            String    addon = null;
            Class     factory = null;
            Method    constructor = null;

            try {
                for (String value : l_addonDescriptors_) {

                    //The form of a value is 'Editor: editor_name - AcsAddon: addon_name'
                    editor = value.substring(8, value.indexOf(" - AcsAddon: "));
                    addon = value.substring(value.indexOf("AcsAddon: ")+10);

                    factory = Class.forName(
                            "ARoad0.AcsAddon."+editor+"."+addon+"."+"gWork.AcsFactory"+addon);

                    acsAddonName = ((AcsFactoryForAcsAddon)factory.newInstance()).
                                        getPublicAcsAddonName();

                    if (group_.getSelection().getActionCommand().equals(acsAddonName))
                            break;

                } // for value

                constructor = factory.getDeclaredMethod("getInstance", (Class[])null);

                acs = ((AcsFactoryForAcsAddon)constructor.invoke(factory, (Object[])null)).
                        create(iSname_, name_, 
                        // the ACS type is the beginning of the Addon name, up to a '(' character
                        acsAddonName.substring(0, acsAddonName.indexOf(" (")),
                        parent_);
                
                endACScreation(acs);

            }
            catch (ClassNotFoundException c) {
                throw new InternalError(" The AcsAddon class is not found: "+c.toString());
            }
            catch (InstantiationException i) {
                throw new InternalError(" The AcsAddon class is not instantiated: "+i.toString());
            }
            catch (IllegalAccessException k) {
                throw new InternalError(" The AcsAddon class is not instantiated: "+k.toString());
            }
            catch (NoSuchMethodException n) {
                throw new InternalError(" The AcsAddon class is not instantiated: "+n.toString());
            }
            catch (InvocationTargetException g) {
                throw new InternalError(" The AcsAddon class is not instantiated: "+g.toString());
            }

            factory = null;
            constructor = null;
            acs = null;
      } // else, for an AcsAddon

      
    } catch (BaseError b) {
        System.out.println((String) b.getLastException());
        
        DialogBox.showErrorMessage((String) b.getLastException() );       
        acs = null;
        endACScreation(null);
        return;
    } catch (SmallError s) {
        s.printStackTrace();
        
        DialogBox.showErrorMessage((String) s.getMessage() );       
        acs = null;
        endACScreation(null);
        return;
    } // catch
  } // createACS
   
  /**
   * Called by createACS() to create the specific ACS rights, or to
   * display an AcsAddon frame for some specific properties to edit.
   * The ACS rights, if any, have been already created in the argument.
   * Calls addPreACSinGUI(), removePreACSinGUI(), createUserDefinedACS().
   * Based on gBase.ACSFactoryImpl and its subclasses in the AcsAddons.
   * @param _refAcs is the reference ACS from which the new acs is derived. Null else.
   * */
  private void createFirstACSproperties(ACSImpl _refAcs) {
    // variables
    final JPanel        panel = new JPanel();
    final JTabbedPane   tPane = new JTabbedPane();
    final ACSImpl       refAcs = _refAcs;
    Class               beanInfoClass = null;
    String              rightClassName = null;
    
    try { 
        // stores factory_ for ActionOpenBeamer and BeamerManager.getBaseObject()
        ActionUtilities.getInstance().setACSFactory(factory_);        
        // creates the temporary ACS node in the explorer
        addPreACSinGUI(factory_);
    } catch (BaseError b) {
        DialogBox.showErrorMessage((String) b.toString() );
        return;
    } // catch

    rightClassName = UtilityImpl.getFirstInstance().
            getBeanInfoNameFromClass(factory_.getClass().getName());
    // add 'pre' to have 'gBaseBeanInfo.ACSpreFactory'
    rightClassName = rightClassName.substring(0,
            rightClassName.indexOf("gBaseBeanInfo.") + 14)+
            "ACSpre"+
            rightClassName.substring(rightClassName.indexOf("gBaseBeanInfo.") + 17);

    // if factory_ is from an AcsAddon, 'XXX.gBaseBeanInfo.ACSpreFactoryXXXImplBeanIndo'
    // may do not exist, and then the generic BeanInfo is used
    try {
        beanInfoClass = Class.forName(rightClassName);
    }
    catch (ClassNotFoundException c) {
    } // catch

    if (beanInfoClass == null) {
        try {
            beanInfoClass = Class.forName("ARoad0.gBaseBeanInfo.ACSpreFactoryImplBeanInfo");
        }
        catch (ClassNotFoundException c) {
            throw new InternalError("ActionNewACsyst.createFirstACSproperties(),"+
                    " no ARoad0.gBaseBeanInfo.ACSpreFactoryImplBeanInfo class");
        } // catch
    } // if

    try {
        // creates the ACS rights or other first properties from ACSpreFactoryImplBeanInfo
        // updates m_currentPropertyEditors_
        // updates the explorer with the new ACS rights, but not the other properties
        BeamerManager.setJTabbedPane(
          factory_,
          ((BeanInfoPattern)beanInfoClass.newInstance()).getPropertyDescriptors(),
          tPane,
          m_currentPropertyEditors_,
          null);

        panel.add( tPane, BorderLayout.CENTER ); 
        
        JButton continueButton = new JButton("  Continue  ");
        continueButton.setBorder(new LineBorder(InternalFrame.BACKCOLOR_2, 10));
        continueButton.setBorder(new BevelBorder(BevelBorder.RAISED));
        continueButton.setPreferredSize(new Dimension( 100, 40));        
        panel.add( continueButton, BorderLayout.SOUTH );

        // The frame has the dimension (520, 380).
        final JInternalFrame  frame = InternalFrame.showEmptyFrame(
          "Edit the ACS rights, if necessary, then continue ",
          panel, 520, 380);
        
        // creates button action listeners
        continueButton.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent evt) {
                  toFinalize_ = false;
                  try {
                    frame.setClosed(true);
                  } catch (PropertyVetoException p) {
                      throw new InternalError("in ActionNewACSyst.createFirstACSproperties()"+
                              " from continueButton "+p);
                  }
                  
                  // for 'derived from ' acs, updates factory_ with the
                  // reference ACS properties
                  if (refAcs != null) {
                      fillDerivedACSproperties(refAcs);
                  } // if
                  
                  // main work: displays a new dialog frame for updating
                  // the other ACS properties
                  if (factory_ != null) // true if fillDerivedACSproperties() calls endACScreation()
                      createUserDefinedACS(panel, tPane);

          } // actionPerformed()
        }); // checkButton.addActionListener 
        
            // for closing actions, through an inner class InternalFrameAdapter
        frame.addInternalFrameListener(new InternalFrameAdapter() {
              public void internalFrameClosing(InternalFrameEvent _e){
                    try {
                        // if not from the continueButton, finalize
                        if (toFinalize_) {
                            // deletes the ACSRun node and its ascrights in the explorer
                            removePreACSinGUI(factory_.getName());                            
                            factory_.finalizeForUser(); // may throw UpDateError
                            // unstores factory_ for ActionOpenBeamer
                            ActionUtilities.getInstance().setACSFactory(null);
                            endACScreation(null);
                        } // if
                    } catch (UpDateError u) {
                        throw new InternalError("in ActionNewACSyst."+
                                "createFirstACSproperties() frame"+
                                " closing: "+u);
                    }
 
                    toFinalize_ = true;
             } // public void internalFrameClosing
        }); // new InternalFrameAdapter()
           
    } // catch
    catch (ProcessError p) {
        DialogBox.showErrorMessage(p.toString() );
        beanInfoClass = null;
        rightClassName = null;
        return;
    } // catch
    catch (InstantiationException i) {
        DialogBox.showErrorMessage(i.toString() );
        beanInfoClass = null;
        rightClassName = null;
        return;
    } // catch 
    catch (IllegalAccessException l) {
        DialogBox.showErrorMessage(l.toString() );
        beanInfoClass = null;
        rightClassName = null;
        return;
    } // catch
    
                    // for testing and tracking
                            //System.out.println("in ActionNewACSyst.createFirstACSproperties(), "+
                            //  "before the call to "+
                            //  "ACSTree.createACSTree(acs), the resources list size is: "+
                            //  acs.getEorM_Resources().keySet().size());
  } // createFirstACSproperties()
   
  /**
   * Called by createFirstACSproperties() to create a specific ACS and, before,
   * to display its properties in a generic frame for allowing the user
   * to enter the main values.
   * Calls endACScreation(), removePreACSinGUI(),
   * ActionUtilities.getInstance().setACSFactory(),
   * UtilityImpl.selectBeanInfo().
   * Based on ACSFactoryImpl.
   * @param _panel created in createFirstACSproperties()
   * @param _tPane created in createFirstACSproperties(). It contains empty
   *        properties or the derived ones from a reference ACS
   * */
   public void createUserDefinedACS(JPanel _panel, JTabbedPane _tPane) {
    final JPanel pane = _panel;
    try {
        // finalizes the previous editors, but not the factory
        finalizePropertyEditors();
        _panel.removeAll();

        _tPane.removeAll();

        // sets all the other ACS properties from ACSFactoryImplBeanInfo.
        // updates m_currentPropertyEditors_.
        BeamerManager.setJTabbedPane(
          factory_,
          UtilityImpl.getFirstInstance().selectBeanInfo(factory_).
                getPropertyDescriptors(),
          _tPane,
          m_currentPropertyEditors_,
          null);

        _panel.add( _tPane, BorderLayout.CENTER );

        // adds a button to call ACSFactoryImpl.checkACSbeforeCreation()
        JPanel      panel2 = new JPanel();
        _panel.add( panel2, BorderLayout.SOUTH );
        JButton checkButton = new JButton("  ACS checking  ");
        checkButton.setPreferredSize(new Dimension( 130, 40));
        checkButton.setBorder(new BevelBorder(BevelBorder.RAISED));
        _panel.add( checkButton, BorderLayout.SOUTH );

        /*
        // adds an empty button
        JButton emptyButton = new JButton("    ");
        emptyButton.setPreferredSize(new Dimension( 50, 40));
        //emptyButton.setBorder(new BevelBorder(BevelBorder.RAISED));
        _panel.add( emptyButton, BorderLayout.SOUTH );
         * */
        
        // adds a button to create the ACS
        JButton okButton = new JButton("  Create the ACS  ");
        okButton.setPreferredSize(new Dimension( 130, 40));
        okButton.setBorder(new BevelBorder(BevelBorder.RAISED));
        _panel.add( okButton, BorderLayout.SOUTH );

        // the frame has the dimension (520, 760)
        final JInternalFrame  frame = InternalFrame.showEmptyFrame(
          "Edit the properties of "+factory_.getNickName(), _panel, 520, 740); // 760
        
        // creates button action listeners
        checkButton.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent evt) {

                String message = factory_.checkACSbeforeCreation();
                if (message != null) DialogBox.showErrorMessage(message);
                else DialogBox.showOkMessage("All the properties are consistent. "+
                        "You may create this ACS now, or you may continue to edit the properties.");
                message = null;
                return;
          } // actionPerformed()
        }); // checkButton.addActionListener            
        
        // creates button action listeners
        okButton.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent evt) {

                toFinalize_ = false;
                try {
                    frame.setClosed(true);
                } catch (PropertyVetoException p) {
                    throw new InternalError("in ActionNewACSyst."+
                            "createUserDefinedACS() from continueButton "+p);
                }

                try {
                    // does the main job
                    endACScreation(factory_.createACS()); // finalizes factory_
                } catch (BaseError b) {
                    endACScreation(null);
                    b.printStackTrace();
                    throw new InternalError("in ActionNewACSyst.createUserDefinedACS(), trace from"+
                            " factory_.createACS() after OK. ");
                }

          } // actionPerformed()
        }); // checkButton.addActionListener 
        
            // for closing actions, through an inner class InternalFrameAdapter
        frame.addInternalFrameListener(new InternalFrameAdapter() {
              public void internalFrameClosing(InternalFrameEvent _e) {
                    // delete the ACSRun node and its ascrights in the explorer
                    removePreACSinGUI(factory_.getName());           

                    try {
                        // if not from the okButton, finalize
                        if (toFinalize_) factory_.finalizeForUser(); // may throw UpDateError
                    } catch (UpDateError u) {
                        throw new InternalError("in ActionNewACSyst.createUserDefinedACS() frame"+
                                " closing - "+u);
                    }

                    // finalizes the operations
                    finalizePropertyEditors();
                    pane.removeAll();                     
                    // unstore factory_ for ActionOpenBeamer
                    ActionUtilities.getInstance().setACSFactory(null);

                    if (toFinalize_) endACScreation(null);
                    toFinalize_ = true;
              } // public void internalFrameClosing
        }); // new InternalFrameAdapter()
    
        panel2 = null;
        checkButton = null;
        okButton = null;
    } catch (ProcessError p) {
        System.out.println("ActionNewACSyst.createUserDefinedACS() "+p);
    }
  } // createUserDefinedACS()
   
  /**
   * Invoked by setCurrentAncestor(), handleCreationDialog() and createUserDefinedACS()
   * with a null argument, or by createACS() to display the ACS.
   * Reinitializes the instance properties.
   * For an AcsAddon, calls AcsFactoryForAcsAddon.configure() on the right class
   * from ACS.getM_AcsVocabulary().
   * Saves the ACS.
   * Calls ActionOpenACSyst.displayOneACS().
   * @param _acs after its creation. Null for a LAMP server or when the process is aborted.
   * */
   private void endACScreation(ACSImpl _acs) {

    if (_acs != null) {

                    // for testing and tracking
                    //System.out.println("in ActionNewACSyst.endACScreation(), for "+
                      //      _acs.getName());

        if (_acs.getClass().toString().indexOf("ARoad0.AcsAddon.") != -1) {

            String    editor = null;
            String    addon = null;
            Class     factory = null;
            Method    method = null;

            try {

                editor = _acs.getM_AcsVocabulary().get(
                        ACSFactoryUtilityImpl.ACSADDON_PACKAGE_EDITOR)[0];
                addon = _acs.getM_AcsVocabulary().get(
                        ACSFactoryUtilityImpl.ACSADDON_NAME)[0];

                factory = Class.forName(
                        "ARoad0.AcsAddon."+editor+"."+addon+"."+"gWork.AcsFactory"+addon);


                method = factory.getDeclaredMethod("getInstance", (Class[])null);

                _acs = ((AcsFactoryForAcsAddon)method.invoke(factory, (Object[])null)).
                            configure(_acs);

            }
            catch (ClassNotFoundException c) {
                throw new InternalError(" The AcsAddon class is not found: "+c.toString());
            }
            catch (IllegalAccessException k) {
                throw new InternalError(" The AcsAddon class is not instantiated: "+k.toString());
            }
            catch (NoSuchMethodException n) {
                throw new InternalError(" The AcsAddon class is not instantiated: "+n.toString());
            }
            catch (InvocationTargetException g) {
                throw new InternalError(" The AcsAddon class is not instantiated: "+g.toString());
            }

            factory = null;
            method = null;

        }
        else {
            fillWithAcsObjects(_acs);
        }

        // ACS saving and opening in the GUI
        try {
            BaseManagerImpl.getInstance().saveOpenACS(_acs);
        }
        catch(BaseError b) {
            b.printStackTrace();
            throw new InternalError("ActionNewACSyst.endACScreation(): "+b.toString());
        }
        
        ActionOpenACSyst.getInstance().displayOneACS(_acs);

    } // if (_acs != null)

    // like an initialization of this instance
    instance__ = new ActionNewACSyst();
    factory_ = null;
    isField_ = null;
    acsField_ = null;
    group_ = null; 
    name_ = null;
    iSname_ = null;
    parent_ = null;

    componentField_ = new JTextField();

    //initialization
    DefaultListModel componentsModel = new DefaultListModel();
    for (String type : ACSFactoryUtilityImpl.getL_LimitedValuesForComponentNodes(null, null)) {
        componentsModel.addElement( type );
    } // for ite

    componentList_ = new JList(componentsModel);
    componentField_.setText("                 ");

    nickName_ = null;
    currentAncestor_ = null;
    toFinalize_ = true;
  } // endACScreation()   

  /**
   * Invoked by createACS() to create a Role-Based Access Control application on a Linux,
   * and to display the new ACS in the explorer. An RBAC application uses some
   * functional roles to organize the user rights through ACLs.
   * Uses gWork.AcsFactory to create the ACS in the base.
   * @param _name is the name of the new application
   * @return new ACS or null
   * */
   private ACSImpl createRBAC(Name _name) {
    if (_name == null ) return null;
       
       // variables
    ACSImpl                 acs1 = null;
    ACSImpl                 acs2 = null;
    JOptionPane             pane = null;
    Set                     list = null;
    JList                   jlist = null;
    JDialog                 dialog = null;
    Map.Entry                entry = null;

    if (ACS_Facade.getOpenACS().isEmpty() ) {
      // display a message
      DialogBox.showErrorMessage("No Linux ACS to select as parent ACS");
      return null;
    }

    // creates the list of open Linux ACS
    list = new HashSet(50);
    for (Object obj : ACS_Facade.getOpenACS().entrySet()) {
        entry = (Map.Entry) obj;
        if ( ((ACS)entry.getValue()).getACSType().indexOf("Linux") != -1 )
            list.add(entry.getKey());
    } // for obj

    // shows the dialog box
    jlist = new JList(list.toArray() );
    dialog = DialogBox.showListDialog(
        "Select the parent Linux ACS", jlist);

    // here, dialog_ waits an enter from the user,
    // then the next instructions are processed.

    pane = (JOptionPane)dialog.getContentPane().getComponent(0);
    // no action if CLOSED, and close the dialog box
    if (pane.getValue() == null) return null;
    // else, it is an Integer
    int value = ((Integer)pane.getValue()).intValue();

    // if not OK, no action
    if (value != JOptionPane.OK_OPTION) {
        pane = null;
        dialog = null;
        list = null;
        jlist = null;
        entry = null;
        return null;
    } // if
    
    try {       
      // gets a Linux Ubuntu 8.04 Desktop
      acs1 = ACS_Facade.openOrGetACS((Name)jlist.getSelectedValue());
      
      if (!_name.context().equals(acs1.getName()) ) {
        DialogBox.showErrorMessage("For an RBAC application on Linux, "+
                acs1.getName().lastComponent()+
                " must be the direct parent of "+_name.lastComponent());  
        pane = null;
        dialog = null;
        list = null;
        jlist = null;
        entry = null;
        return null;          
      } // if
      
      // creates an RBAC application server on the Linux
      acs2 = AcsFactory.createRBAC(_name, "RBAC application", acs1);
      
      if (acs2 == null)
          DialogBox.showErrorMessage("The creation of a RBAC application is cancelled.");
      
    } catch (BaseError b) {
        DialogBox.showErrorMessage((String) b.getLastException() );
        acs1 = null;
        acs2 = null;
        dialog = null;
        list = null;
        jlist = null;
        entry = null;
        return null;
    } // catch
    
    acs1 = null;
    dialog = null;
    list = null;
    jlist = null;
    entry = null;
    return acs2;
  } // createRBAC
   
  /**
   * Invoked by createUserDefinedACS() to select a generic type from ACSImpl.getL_AcsTypes(),
   * or to create a new one.
   * @return String which is never null, and "no choice" if there is
   *   no user selection.
   * */
   private String selectNoNX_ACSType() {
        // variables
        List l_AcsTypes = ACSImpl.getL_AcsTypes();
        l_AcsTypes.remove(ACSImpl.LINUX);
        String[] l_values = new String[l_AcsTypes.size() + 1];
        int index = 0;
        String result = null;
        int choice = -1;
        
        for (Iterator ite = l_AcsTypes.iterator(); ite.hasNext();) {
           l_values[index] = (String)ite.next();
           index++;
        }
        
        l_values[index] = "Another type"; 

        try {
          choice = DialogBox.showOneSelectionOneDialog(
            "type for the new ACS", l_values);
        } catch (SmallError s) {
        }
        
                // for testing and tracking
                //System.out.println("in ActionNewACSyst.selectACSType()");
        
        if (choice != -1 && choice != index) result = l_values[choice];
        else if (choice == index) {
            // "Another type" is selected
            result = DialogBox.showSingleEditor("enter a type for the new ACS");
            if (result == null) result = "no choice";        
        } // else
        else result = "no choice";
        l_AcsTypes = null;
        l_values = null;
                
                // for testing and tracking
                //System.out.println("in ActionNewACSyst.selectNoNX_ACSType(), result: "+result);
        
        return result;
   } // selectNoNX_ACSType

  /**
   * Invoked by createFirstACSproperties() to allow the displaying
   * of the ACS rights in the explorer.
   * @param _acs is an uncomplet ACSFactoryImpl
   * */
   private void addPreACSinGUI(ACSRun _acs) throws BaseError {
    JTree                   eTree = null;
    DefaultMutableTreeNode  acsNode = null;
    DefaultTreeModel        treeModel = null;
    int                     index;
       
                // for testing and tracking
                //System.out.println("call to ActionNewACSyst.addPreACSinGUI()");
    
    // updates the explorer
    eTree = InternalFrameModel.getTreeInExplorer();
    treeModel = (DefaultTreeModel) eTree.getModel();
        // index is the number of currently dislayed ACS in Explorer
    index = treeModel.getChildCount(((DefaultMutableTreeNode)treeModel.
        getRoot()).getChildAt(0) );
      // creates the ACS node with its instance as user object
    acsNode = new DefaultMutableTreeNode(_acs );
    treeModel.insertNodeInto(acsNode,
        (DefaultMutableTreeNode)(((DefaultMutableTreeNode)treeModel.
        getRoot()).getChildAt(0)) , index);

    try {
      ACSTree.createACSExplorer(_acs, eTree, acsNode);
    } catch (ProcessError p) {
      throw new InternalError(p.toString());
    }
    
    // sets the ACS visible
    TreePath path = new TreePath(acsNode.getPath());
    eTree.scrollPathToVisible(path);
    
    eTree = null;
    acsNode = null;
    treeModel = null;
    path = null;
   } // addPreACSinGUI
 
  /**
  * Remove the node associated to an ACSRun in the explorer.
  * Called by createFirstACSproperties() and createUserDefinedACS().
  * @param _objectName of the ACS
  */
  public static final void removePreACSinGUI(ImmutableName _objectName) {
    // variables
    DefaultTreeModel        model;
    DefaultMutableTreeNode  root;
    DefaultMutableTreeNode  aCSnode;

    // do not call ACSTreeUtilities.removeACSorViewNode() which relies on ACS_Facade.knowsACS()
    // so do the job entirely
    model = (DefaultTreeModel)InternalFrameModel.
            getTreeInExplorer().getModel();
    root = (DefaultMutableTreeNode)model.getRoot();
    
    DefaultMutableTreeNode  entityNode = null; 
    DefaultMutableTreeNode  searchedNode = null; 

    // get the node "open ACS" in the explorer
    aCSnode = (DefaultMutableTreeNode)model.getChild(root, 0);
    
    // iterate on 'Open Entities' node children
    // no iteration if entityNode is empty
    for (int i = 0 ; i < model.getChildCount(aCSnode) ; i++) {

        entityNode = (DefaultMutableTreeNode)model.getChild(aCSnode, i);
        if ( ((ACSRun)entityNode.getUserObject()).getName().equals(_objectName) ) {
          searchedNode = entityNode;
          break;
        } // if
    } // for i

    if (searchedNode != null)
      model.removeNodeFromParent(searchedNode);
    else 
      System.out.println("in ActionNewACSyst.removePreACSNode(), "+
          "the object is not found.");      

     // remove all the TreeBaseListeners associated to the ACS
    try {
      TreeManager.removeAllTreeBaseListenerFor(_objectName);
    } catch (SmallError s) {
        DialogBox.showOkMessage("On-going entity closing,"+
          " but the treeBaseListeners are not all removed."+
          " An exception has been thrown: "+
          s.toString() );
    }    
    
    
    model = null;
    root = null;
    aCSnode = null;
    entityNode = null;
    searchedNode = null;
    return;
  } // removePreACSinGUI()   

  /**
  * Called by BaseObjectHandler for defining the allowed upper rights to which
  * the new acsright may be connected. If the metaright is classed,
  * the selected superior rights are classed, but not only with the same class.
  * If the metaright is negative, the selected superior rights are negative.
  * @param _newRightKey the internal key of the acsright to create
  * @param _meta the selected metaright for the new acsright. May be null.
  * @return array of all the rights which have an internal key greater than
   *    the argument. May be null.
  */
  public StringRight[] getRestrictedUpperRightsForRight(Integer _newRightKey, StringRight _meta) {

    StringRight[]   l_result;
    ACSFactoryImpl  factory = ActionUtilities.getInstance().getACSFactory();
    List<StringRight>            list = new ArrayList( Arrays.asList(factory.getL_Rights()));
    StringRight     right = null;
    
    if (list.isEmpty()) return null;

            // for testing and tracking
            //System.out.println("in ActionNewACSyst.getRestrictedUpperRightsForRight(): ");
            //for (Iterator ite = list.iterator(); ite.hasNext(); ) {
              //  System.out.println(" --------- class is: "+ite.next().getClass());
            //} // for ite    
    
    for (Iterator ite = list.iterator(); ite.hasNext(); ) {
        right = (StringRight)ite.next();
        if (  right.getInternalKey() <= _newRightKey)
            ite.remove();
        else if ( _meta != null && right.isGrantingRight() && 
                !_meta.isGrantingRight())
            ite.remove();
        else if ( _meta != null && !right.isGrantingRight() && 
                _meta.isGrantingRight())
            ite.remove();
        else if ( _meta != null && right.isTargetClassed() && 
                !_meta.isTargetClassed())
            ite.remove();
        else if ( _meta != null && !right.isTargetClassed() && 
                _meta.isTargetClassed())
            ite.remove();
    } // for ite
    
    if (list.isEmpty()) l_result = null;            
    else l_result = (StringRight[])list.toArray(new StringRight[0]);
            
    list = null;
    factory = null;
    return l_result;
  } // getRestrictedUpperRightsForRight()  
  
  /**
  * Finalizes the current property editors.
  * Called by createUserDefinedACS().
  */
  private void finalizePropertyEditors() {

    if (!m_currentPropertyEditors_.isEmpty()) {
      for (Iterator ite = m_currentPropertyEditors_.values().iterator() ; ite.hasNext() ;) {
        ((CommonPropertyEditor)ite.next()).finalizeForProcess();
      }
    }
    
    m_currentPropertyEditors_.clear();
    return;
  } // finalizePropertyEditors()
 
  /**
   * Called by createACS() to fill the new acs factory with all the
   * ACS rights from the ACS to be copied.
   * Updates factory_.
   * @param _refAcs reference ACS from which the new ACS is build up when
   * it is a derived ACS
   * */
   private void fillAcsRightsForDerivedACS(ACS _refAcs) {
        StringRight[]   l_rights = ACS_Facade.getAcsRightArray(_refAcs);
        String          st = null;
        int             index = -1;
        StringRight[]   l_uprights = null;
        StringRightImpl[] l_newuprights = null;
        StringRight     right = null;
        
        try {
            for (int i = 0; i < l_rights.length; i++) {
                st = l_rights[i].getRight();
                if (l_rights[i].getTargetClassType() != null) {
                    index = st.indexOf(l_rights[i].getTargetClassType());
                    st = st.substring(0, index - 1);
                }
        
                        // for testing purpose
                        //System.out.println("ActionNewACSyst.fillAcsRightsForDerivedACS() for "+
                          //      factory_.getACSType());
                        //System.out.println("right to create: "+l_rights[i]+" and its metaright:"+
                          //      l_rights[i].getEorMetaRight());
                
                l_uprights = l_rights[i].getUpperRights();
                l_newuprights = new StringRightImpl[l_uprights.length];
                for (int j = 0; j < l_uprights.length; j++) {
                    l_newuprights[j] = (StringRightImpl)
                                factory_.getOneRight(l_uprights[j].getRight());
                } // for j
                        
                if (l_rights[i].getUpperRights().length != 0)
                    right = new StringRightImpl(st, l_rights[i].getEorMetaRight(),
                        l_rights[i].getInternalKey(), l_newuprights,
                        l_rights[i].getTargetClassType(), factory_);
                else
                    right = new StringRightImpl(st, l_rights[i].getEorMetaRight(),
                        l_rights[i].getInternalKey(), null,
                        l_rights[i].getTargetClassType(), factory_);
                
                right.setDesactivated(l_rights[i].isDesactivated());
                // two following lines may throw BaseError
                right.setParent(l_rights[i].isParent());
                right.setSlave(l_rights[i].isSlave());

            } // for i
        } // try
        catch (CreateError e) {
            DialogBox.showErrorMessage(e.toString());
            e.printStackTrace();
            throw new InternalError("This trace from "+
                    "ActionNewACSyst.fillAcsRightsForDerivedACS() for "+name_); 
        } catch (BaseError b) {
            DialogBox.showErrorMessage(b.toString());
            b.printStackTrace();
            throw new InternalError("This trace from "+
                    "ActionNewACSyst.fillAcsRightsForDerivedACS() for right "+right);
        } // catch
        
        l_rights = null;
        st = null;
        l_uprights = null;
        l_newuprights = null;
   } // fillAcsRightsForDerivedACS()
  
  /**
   * This important method fills the ACSFactory with all
   * the ACS properties to be copied from the ACS reference.
   * The alternates, the new components and the composite links are not added there,
   * but the user may do it manually.
   * The current ACSObjects in the ACS reference are not copied from the ACS reference,
   * although the AcsFactoryForAcsAddon.configure() method is called to provide
   * the standard set of ACSObjects. This method adds the comment
   * 'derived by hand by the ACS creator' to the resulting ACS.
   * Based on ACSFactoryImpl. Called by createFirstACSproperties().
   * Caution: the roots actions adaptation does not work when the key is not the direct parent.
   * @param _refAcs is the reference acs from which the new acs is build up when
   *    it is a derived acs
   * */
   private void fillDerivedACSproperties(ACSImpl _refAcs) {
       ImmutableNameISTree     tree = null;
       StringRight[]    l_rights = null;
       Set<StringRight> l_newRights = new HashSet(30);
       StringRight      right = null;
       Map              map = null;
       Map.Entry        entry = null;
       List<String[]>   l_commands = null;
       Iterator         ita = null;
       String[]         l_val = null;
       String[]         l_val2 = null;
       Set<String>      l_keys = null;

                                   // for testing and tracking
                                   //System.out.println(
                                     //   "ActionNewACSyst.fillDerivedACSproperties() with factory_ "+
                                       // "from the reference "+_refAcs.getName());


       try {

           // Structure and Structure 2 tabs.
           // the booleans ACS rights and Meta rights are set by ACSFactoryImpl,
           // like the Granting rights and Denying rights
            factory_.setManageResources(_refAcs.getManageResources());
            factory_.setManageResourceTrees(_refAcs.getManageResourceTrees());
            factory_.setManageAccounts(_refAcs.getManageAccounts());
            factory_.setManageGroups(_refAcs.getManageGroups());
            factory_.setManageAGORights(_refAcs.getManageAGORights());
            factory_.setManageActors(_refAcs.getManageActors());
            factory_.setManageInternalAcls(_refAcs.getManageInternalAcls());
            factory_.setManageExternalAcls(_refAcs.getManageExternalAcls());
            factory_.setManageBridges(_refAcs.getManageBridges());
            factory_.setManageSecGroupsForActor(_refAcs.getManageSecGroupsForActor());
            factory_.setManageGroupTrees(_refAcs.getManageGroupTrees());

            // from 0.7.0
            factory_.setManageACLRightsInheritance(_refAcs.getManageACLRightsInheritance());
            factory_.setManageAGORightsInheritance(_refAcs.getManageAGORightsInheritance());
            factory_.setManageActorInEPVirtualFolders(_refAcs.getManageActorInEPVirtualFolders());
            factory_.setManageAuthorizationServerTypedPrivileges(_refAcs.getManageAuthorizationServerTypedPrivileges());
            factory_.setManageClassedSpecificTypedPrivileges(_refAcs.getManageClassedSpecificTypedPrivileges());
            factory_.setManageClassedTypedPrivileges(_refAcs.getManageClassedTypedPrivileges());
            factory_.setManageDenyingRightPrivileges(_refAcs.getManageDenyingRightPrivileges());
            factory_.setManageDirectoryEPs(_refAcs.getManageDirectoryEPs());
            factory_.setManageDirectoryEPinEPVirtualFolders(_refAcs.getManageDirectoryEPinEPVirtualFolders());
            factory_.setManageDirectoryInResourceVirtualFolders(_refAcs.getManageDirectoryInResourceVirtualFolders());
            factory_.setManageEPVirtualFolders(_refAcs.getManageEPVirtualFolders());
            factory_.setManageExternalForOneToOneLinkedPrivileges(_refAcs.getManageExternalForOneToOneLinkedPrivileges());
            factory_.setManageSoftAlias(_refAcs.getManageSoftAlias());
            factory_.setManageFlexibleConditionalAcls(_refAcs.getManageFlexibleConditionalAcls());
            factory_.setManageGrantingRightPrivileges(_refAcs.getManageGrantingRightPrivileges());
            factory_.setManageIfAllSourceLinkedPrivileges(_refAcs.getManageIfAllSourceLinkedPrivileges());
            factory_.setManageInternalForOneToOneLinkedPrivileges(_refAcs.getManageInternalForOneToOneLinkedPrivileges());
            factory_.setManageInternalHardAlias(_refAcs.getManageInternalHardAlias());
            factory_.setManageLPRIRightsInheritance(_refAcs.getManageLPRIRightsInheritance());
            factory_.setManageOneTimeInVirtualFolderTreeForMember(_refAcs.getManageOneTimeInVirtualFolderTreeForMember());
            factory_.setManagePrivilegeRights(_refAcs.getManagePrivilegeRights());
            factory_.setManagePrivilegesInResources(_refAcs.getManagePrivilegesInResources());
            factory_.setManagePrivilegesInVirtualFolders(_refAcs.getManagePrivilegesInVirtualFolders());
            factory_.setManageResourceVirtualFolders(_refAcs.getManageResourceVirtualFolders());
            factory_.setManageRightDefinedConditionalAcls(_refAcs.getManageRightDefinedConditionalAcls());
            factory_.setManageSimpleLinkedPrivileges(_refAcs.getManageSimpleLinkedPrivileges());
            factory_.setManageSimpleResourceVirtualFolders(_refAcs.getManageSimpleResourceVirtualFolders());
            factory_.setManageSimpleTypedPrivileges(_refAcs.getManageSimpleTypedPrivileges());
            factory_.setManageVirtualFolderRightsPropagationToMembers(_refAcs.getManageVirtualFolderRightsPropagationToMembers());
            factory_.setManageVirtualFolderTrees(_refAcs.getManageVirtualFolderTrees());
            factory_.setManageVirtualFolders(_refAcs.getManageVirtualFolders());

           // Main tab
           factory_.setComment("derived by hand by the ACS creator");
           factory_.setEorIcon(_refAcs.getEorIcon());
           factory_.setIncompleteBehaviorModeling(_refAcs.getIncompleteBehaviorModeling());
           factory_.setIncompleteStructureModeling(_refAcs.getIncompleteStructureModeling());

           if (iSname_ != null) {
             tree = (ImmutableNameISTree) BaseManagerImpl.getInstance().getISTree(iSname_);
             if (tree != null) {
                 factory_.setVisibleSpace(tree.getExtendedVisibleSpace(_refAcs.getName()));
                 factory_.setHubProperty(tree.isHubNode(_refAcs.getName()));
                 factory_.setTiedToParentACS(
                         Arrays.asList(tree.getLinks(_refAcs.getName())).contains(
                            ISFactoryUtilityImpl.IS_TIED_CHILD));
                 factory_.setIsAuthorizationServer(_refAcs.getIsAuthorizationServer());
             } // if
           } // if (iSname_

           // copies the two ACS type policy maps
           map = _refAcs.getM_TypesAndGUIPolicy();
           if (map != null) {
               // Types tab
               factory_.setL_ResourceTypes(
                       (String[])map.get("Resource.ResourceType"));
               factory_.setL_EPTypes(
                       (String[])map.get("EligibleParty.EPType"));
               factory_.setL_ResourceVirtualFolderTypes(
                       (String[])map.get("VirtualFolder.resourceVFType"));
               factory_.setL_EPVirtualFolderTypes(
                       (String[])map.get("VirtualFolder.EPVFType"));
               // Types 2 tab
               factory_.setL_PrivilegeForTypeTypes(
                       (String[])map.get("PrivilegeForType.SecondType"));
               factory_.setL_PrivilegeForLinksTypes(
                       (String[])map.get("PrivilegeForLinks.SecondType"));

               // l_keys depends on the resource/EP/VF/PRI types,
               // and contains keys into getM_TypesAndGUIPolicy() or getM_TypeOrientedPolicy()
               l_keys = factory_.getL_MapKeysForTypePolicies();

               // Constraints tab
               if (l_keys != null) {
                   for (String key : l_keys) {
                       //if (key.equals("Resource.ResourceType") ||
                         //      key.equals("EligibleParty.EPType"))
                           //continue;

                       if (map.get(key) != null)
                           factory_.addTypesAndExplorer(
                               key,
                               (String[])map.get(key));
                   } // for key
               } // if (l_keys != null)
           } // if (map

           map = _refAcs.getM_TypeOrientedPolicy();
           if (map != null && l_keys != null) {

                                   // for testing and tracking
                                   //System.out.println(
                                     //   "ActionNewACSyst.fillDerivedACSproperties() "+
                                       // "from the reference "+_refAcs.getNickName()+
                                       // ", with getL_MapKeysForTypePolicies() keys:");

               for (String key : l_keys) {

                                   // for testing and tracking
                                   //System.out.println("----- "+key);

                   if (map.get(key) != null)
                       factory_.addTypesAndExplorer(
                           key,
                           (String[])map.get(key));
               } // for key
           } // if map

            // Structure 3 tab
           map = _refAcs.getM_SubAndGroupPolicy();
           if (map != null) {
               for (Iterator ite = map.entrySet().iterator(); ite.hasNext(); ) {
                   entry = (Map.Entry)ite.next();
                   // ((String[])entry.getValue()).length may be 0
                   l_val = new String[((String[])entry.getValue()).length];
                   System.arraycopy((String[])entry.getValue(), 0, l_val, 0, l_val.length);

                   factory_.addSubAndGroupPolicy(
                       new String((String)entry.getKey()), l_val);
               } // for ite
           } // if

           // The alternates, the new components, the composite links are not added there.
           // The user should do it manually.

           for (Map.Entry<String, String[]> ent : _refAcs.getM_AcsVocabulary().entrySet()) {
               // ((String[])entry.getValue()).length may be 0
               l_val = new String[((String[])ent.getValue()).length];
               System.arraycopy((String[])ent.getValue(), 0, l_val, 0, l_val.length);

               factory_.addAcsVocabulary(
                        new String(ent.getKey()), l_val);
           } // for ent

           // Roots tab
           // roots with changed ACS parent
           // caution: does not work when the key is not the direct parent
           map = _refAcs.getM_RootCommandsForOpeningAndClosing();
           if (!map.isEmpty()) {
               for (Object ent : AssociateUtilityImpl.fillDerivedAssociations(
                       factory_, _refAcs, map).entrySet()) {

                   entry = (Map.Entry)ent;
                   l_commands = (List<String[]>) entry.getValue();
                   ita = l_commands.iterator();
                   while (ita.hasNext()) {
                       l_val2 = (String[])ita.next();
                       // l_val2.length may be 0
                       l_val = new String[l_val2.length];
                       System.arraycopy(l_val2, 0, l_val, 0, l_val.length);

                       factory_.addOpeningAndClosingRoots(
                               new String((String)entry.getKey()), l_val);
                   } // while
               } // for ent
           } // if

           // roots with changed ACS parent
           // caution: does not work when the key is not the direct parent
           map = _refAcs.getM_RootCommandsForOpeningNoClosing();
           if (!map.isEmpty()) {
               for (Object ent : AssociateUtilityImpl.fillDerivedAssociations(
                       factory_, _refAcs, map).entrySet()) {

                   entry = (Map.Entry)ent;
                   l_commands = (List<String[]>) entry.getValue();
                   ita = l_commands.iterator();
                   while (ita.hasNext()) {
                       l_val2 = (String[])ita.next();
                       // l_val2.length may be 0
                       l_val = new String[l_val2.length];
                       System.arraycopy(l_val2, 0, l_val, 0, l_val.length);

                       factory_.addOpeningNoClosingRoots(
                               new String((String)entry.getKey()), l_val);
                   } // while
               } // for ent
           } // if

                       // for testing and tracking
                       //System.out.println("ActionNewACSyst.fillDerivedACSproperties() with factory_ "+
                         //   "opening no-closing roots scripts:");
                       //String[] l_scripts = null;
                       //for (Object ent : factory_.getM_OpeningNoClosingRoots().entrySet()) {
                         //  l_scripts = (String[]) ((Map.Entry)ent).getValue();
                           //for (int i = 0; i < l_scripts.length; i++) {
                             //  System.out.println("--------- "+l_scripts[i]);
                           //} // for i
                       //} // for ent

           // roots with changed ACS parent
           // caution: does not work when the key is not the direct parent
           map = _refAcs.getM_RootCommandsForIntegrity();
           if (!map.isEmpty()) {
               for (Object ent : AssociateUtilityImpl.fillDerivedAssociations(
                       factory_, _refAcs, map).entrySet()) {

                   entry = (Map.Entry)ent;
                   l_commands = (List<String[]>) entry.getValue();
                   ita = l_commands.iterator();
                   while (ita.hasNext()) {
                       l_val2 = (String[])ita.next();
                       // l_val2.length may be 0
                       l_val = new String[l_val2.length];
                       System.arraycopy(l_val2, 0, l_val, 0, l_val.length);

                       factory_.addIntegrityRoots(
                               new String((String)entry.getKey()), l_val);
                   } // while
               } // for ent
           } // if

           // Constraints tab

           // the associations depend heavily on the _refAcs objects.
           // the user may delete them by hand before the ACS creation.
           map = _refAcs.getM_AssociationsOnNewObject();
           if (!map.isEmpty()) {
               for (Object ent : AssociateUtilityImpl.fillDerivedAssociations(
                       factory_, _refAcs, map).entrySet()) {

                   entry = (Map.Entry)ent;
                   l_commands = (List<String[]>) entry.getValue();
                   ita = l_commands.iterator();
                   while (ita.hasNext()) {
                       l_val2 = (String[])ita.next();
                       // l_val2.length may be 0
                       l_val = new String[l_val2.length];
                       System.arraycopy(l_val2, 0, l_val, 0, l_val.length);

                       factory_.addAssociationsToNewObject(
                               new String((String)entry.getKey()), l_val);
                   } // while
               } // for ent
           } // if

           // See Why tab
           map = _refAcs.getM_CommentsOnProperties();
           if (map != null) {
               for (Iterator ite = map.entrySet().iterator(); ite.hasNext(); ) {
                   entry = (Map.Entry)ite.next();

                   factory_.addCommentsOnProperties(
                       (String)entry.getKey(), (String[])entry.getValue());
               } // for ite
           } // if

           // Rights and Rights 2 tabs
           map = _refAcs.getM_StandardRights();
           if (map != null) {
               if (!_refAcs.getManageAcsRights()) {
                   
                   factory_.setL_AccountRightsSet((StringRight[])
                           map.get("Resource.OwnerRights"));
                   // 'Resource.OtherRights' returns the same array
                   factory_.setL_GroupRightsSet((StringRight[])
                           map.get("Resource.GroupRights"));
                   factory_.setL_AclRightsSet((StringRight[])
                           map.get("Resource.AclRights"));
                   factory_.setL_bridgeRightsSet((StringRight[])
                           map.get("Actor.BridgeRights"));

                   // for Privileges, adding in the privileges map
                   // after a convertion of the key
                   if (factory_.getManageSimpleTypedPrivileges() ||
                           factory_.getManageSimpleLinkedPrivileges()) {
                       for (Object obj : PrivilegeAbst.getL_PrivilegeTypes()) {
                           l_rights = (StringRight[])map.get(
                                   "PrivilegeRights.MainType."+(String)obj);
                           if (l_rights != null && l_rights.length != 0) {
                               factory_.addPrivilegeRights(
                                     ACSFactoryUtilityImpl.convertKeyOfPrivilegeRightsForNewACS(
                                        "PrivilegeRights.MainType."+(String)obj),
                                     l_rights);
                           } // if
                       } // for obj
                   } // if (factory_

               } // if (!_refAcs
               else { // adapts the ACS rights to the new ACS
                   
                   l_rights = (StringRight[]) map.get("Resource.OwnerRights");
                   if (l_rights != null && l_rights.length != 0) {
                       l_newRights.clear();
                       for (int i = 0; i < l_rights.length; i++) {
                           right = factory_.getOneRight(l_rights[i].getRight());
                           // null if deleted by the user for the derived acs
                           if (right != null) l_newRights.add(right);
                       } // for i
                       factory_.setL_AccountRightsSet(l_newRights.toArray(new StringRight[0]));
                   } // if (l_rights

                   l_rights = (StringRight[]) map.get("Resource.GroupRights");
                   if (l_rights != null && l_rights.length != 0) {
                       l_newRights.clear();
                       for (int i = 0; i < l_rights.length; i++) {
                           right = factory_.getOneRight(l_rights[i].getRight());
                           // null if deleted by the user for the derived acs
                           if (right != null) l_newRights.add(right);
                       } // for i
                       factory_.setL_GroupRightsSet(l_newRights.toArray(new StringRight[0]));
                   } // if (l_rights               

                   l_rights = (StringRight[]) map.get("Resource.AclRights");
                   if (l_rights != null && l_rights.length != 0) {
                       l_newRights.clear();
                       for (int i = 0; i < l_rights.length; i++) {
                           right = factory_.getOneRight(l_rights[i].getRight());
                           // null if deleted by the user for the derived acs
                           if (right != null) l_newRights.add(right);
                       } // for i
                       factory_.setL_AclRightsSet(l_newRights.toArray(new StringRight[0]));
                   } // if (l_rights  

                   l_rights = (StringRight[]) map.get("Actor.BridgeRights");
                   if (l_rights != null && l_rights.length != 0) {
                       l_newRights.clear();
                       for (int i = 0; i < l_rights.length; i++) {
                           right = factory_.getOneRight(l_rights[i].getRight());
                           // null if deleted by the user for the derived acs
                           if (right != null) l_newRights.add(right);
                       } // for i
                       factory_.setL_bridgeRightsSet(l_newRights.toArray(new StringRight[0]));
                   } // if (l_rights

                   // for Privileges, adding in the privileges map
                   // after a convertion of the key
                   if (factory_.getManageSimpleTypedPrivileges() ||
                           factory_.getManageSimpleLinkedPrivileges()) {
                       for (Object obj : PrivilegeAbst.getL_PrivilegeTypes()) {
                           l_rights = (StringRight[])map.get(
                                   "PrivilegeRights.MainType."+(String)obj);
                           if (l_rights != null && l_rights.length != 0) {
                               l_newRights.clear();
                               for (int i = 0; i < l_rights.length; i++) {
                                   right = factory_.getOneRight(l_rights[i].getRight());
                                   // null if deleted by the user for the derived acs
                                   if (right != null) l_newRights.add(right);
                               } // for i

                               factory_.addPrivilegeRights(
                                     ACSFactoryUtilityImpl.convertKeyOfPrivilegeRightsForNewACS(
                                        "PrivilegeRights.MainType."+(String)obj),
                                     l_newRights.toArray(new StringRight[0]));
                           } // if
                       } // for obj
                   } // if (factory_

               } // else // adapts the ACS rights to the new ACS
           } // if (map

                               // for testing and tracking
                               //System.out.println("ActionNewACSyst.fillDerivedACSproperties() "+
                                 //   "_refAcs.getM_SpecializedRights() keys:");

           // Rights 3 tab
           map = _refAcs.getM_SpecializedRights();
           if (map != null) {

               for (Iterator ite = map.entrySet().iterator(); ite.hasNext(); ) {
                   entry = (Map.Entry)ite.next();

                               // for testing and tracking
                               //System.out.println("------ "+entry.getKey());

                   if (!_refAcs.getManageAcsRights()) {
                       // if this method is not for Privileges
                       if (((String)entry.getKey()).indexOf("PrvRightsSet") == -1 )
                           factory_.addSpecializedRights(
                                (String)entry.getKey(), (StringRight[])entry.getValue());

                       else if (factory_.getManageSimpleTypedPrivileges() &&
                               ((String)entry.getKey()).startsWith("PFType")) {
                           factory_.addPrivilegeRights(
                                 ACSFactoryUtilityImpl.convertKeyOfPrivilegeRightsForNewACS(
                                    (String)entry.getKey()),
                                 (StringRight[])entry.getValue());
                       } // else if
                       else if (factory_.getManageSimpleLinkedPrivileges() &&
                               ((String)entry.getKey()).startsWith("PFLink")) {
                           factory_.addPrivilegeRights(
                                 ACSFactoryUtilityImpl.convertKeyOfPrivilegeRightsForNewACS(
                                    (String)entry.getKey()),
                                 (StringRight[])entry.getValue());
                       } // else if
                   } // if
                   else {
                       l_rights = (StringRight[])entry.getValue();
                       l_newRights.clear();
                       for (int i = 0; i < l_rights.length; i++) {
                           right = factory_.getOneRight(l_rights[i].getRight());
                           // null if deleted by the user for the derived acs
                           if (right != null) l_newRights.add(right);
                       } // for i

                       if (l_newRights.isEmpty()) continue;

                       // if this method is not for Privileges
                       if (((String)entry.getKey()).indexOf("PrvRightsSet") == -1 )
                           factory_.addSpecializedRights(
                                (String)entry.getKey(), l_newRights.toArray(new StringRight[0]));
                       
                       else if (factory_.getManageSimpleTypedPrivileges() &&
                               ((String)entry.getKey()).startsWith("PFType")) {
                           factory_.addPrivilegeRights(
                                 ACSFactoryUtilityImpl.convertKeyOfPrivilegeRightsForNewACS(
                                    (String)entry.getKey()),
                                 l_newRights.toArray(new StringRight[0]));
                       } // else if
                       else if (factory_.getManageSimpleLinkedPrivileges() &&
                               ((String)entry.getKey()).startsWith("PFLink")) {
                           factory_.addPrivilegeRights(
                                 ACSFactoryUtilityImpl.convertKeyOfPrivilegeRightsForNewACS(
                                    (String)entry.getKey()),
                                 l_newRights.toArray(new StringRight[0]));
                       } // else if
                   } // else
               } // for ite
               
           } // if (map != null)

       }
       catch (BaseError b) {
            // deletes the ACSRun node and its ACS rights in the explorer
            removePreACSinGUI(factory_.getName());
            // unstores factory_ for ActionOpenBeamer
            ActionUtilities.getInstance().setACSFactory(null);
            DialogBox.showErrorMessage(b.toString());
            System.out.println("ActionNewACSyst.fillDerivedACSproperties(): ");
            b.printStackTrace();
            endACScreation(null);
            return;
       }
       catch (UpDateError u) {
            // deletes the ACSRun node and its ACS rights in the explorer
            removePreACSinGUI(factory_.getName());
            // unstores factory_ for ActionOpenBeamer
            ActionUtilities.getInstance().setACSFactory(null);
            DialogBox.showErrorMessage(u.toString());
            System.out.println("ActionNewACSyst.fillDerivedACSproperties(): ");
            u.printStackTrace();
            endACScreation(null);
            return;
       } 
       
       tree = null;
       l_rights = null;
       l_newRights = null;
       map = null;
       entry = null;
       l_commands = null;
       ita = null;
       l_keys = null;
       l_val = null;
       l_val2 = null;
   } // fillDerivedACSproperties()

  /**
   * Adds the immutable objects under the subACS nodes or the groups tree roots.
   * This is the unique opportunity for the user to add these objects, since
   * the beamer does not allow to do it. There is no adding under a node if
   * the acs has already at most one occurence which is a node child, when the child
   * has been created by coding. <P>
   * On the contrary of a creation through coding, it is not
   * possible for the user to add a tree of Resources, since the resources which
   * may be created there, are all under a Resources subACS node.
   * Called by endACScreation() before the opening of the new ACS in the explorer.
   * @param _acs is a new ACS having some subACSs or groups tree roots
   * */
   private void fillWithAcsObjects(ACSImpl _acs) {
                    // for testing and tracking
                    //System.out.println("call to ActionNewACSyst.fillWithAcsObjects()");

       try {
           if (_acs.getACSType().startsWith("RBAC application")) {
                    AcsFactory.configureRBAC(
                            _acs,
                            ACS_Facade.openOrGetACS(_acs.getParentName()));
           }
           else if (_acs.getACSType().startsWith("An ACS for testing VirtualFolders and Privileges")) {
                    // configure
                   AcsFactory.configureTestForVFandPRI(_acs);
           } // else
           else if (_acs.getACSType().startsWith("An ACS for testing AG/ACL/PRI rights "+
                   "and AG inheritance")) {
                    // configure
                   AcsFactory.configureTestOfAll(_acs);
           } // else
       }
       catch (BaseError b) {
            b.printStackTrace();
            throw new InternalError("... from ActionNewACSyst.fillWithAcsObjects(): "+
                    b.toString());
       }

       if (_acs.getM_SubAndGroupPolicy().isEmpty()) return;

       // mainly for 'Open ACS to specify extensively' ACS, but also
       // to complement a native, non-derived application from the ACS objects set
       // by AcsFactory.configureXXX()
       Map                      m_SubAndGroups = _acs.getM_SubAndGroupPolicy();
       Map.Entry                entry = null;
       boolean                  hasResourcesSubACStoCreate = false;
       boolean                  hasGroupsSubACStoCreate = true;
       boolean                  hasGroupsTreeToCreate = true;
       String[]                 l_values = null;
       String                   name = null;
       ImmutableLeaf[]          l_leaves = null;
       Directory                directory = null;
       SortedMap                m_ep = null;
       ImmutableEligibleParty   member = null;

       for (Iterator ite = m_SubAndGroups.entrySet().iterator(); ite.hasNext(); ) {
            entry = (Map.Entry<String,String[]>) ite.next();
            l_values = (String[]) entry.getValue();
            name = (String) entry.getKey();

            hasResourcesSubACStoCreate = false;
            hasGroupsSubACStoCreate = true;
            hasGroupsTreeToCreate = true;

             for (int i = 0; i < l_values.length; i++) {
                 if (l_values[i].equals("<ResourceSubACS>")) {
                     directory = ((Directory)_acs.getEorM_OwnResources().get(
                             ((Name)_acs.getName()).extended("<"+name+"_subacs>")));

                     if (directory != null) {
                         l_leaves = directory.getEorL_Children();
                         if (l_leaves == null || l_leaves.length == 0)
                                    hasResourcesSubACStoCreate = true;
                     } // if
                     else
                         throw new InternalError("ActionNewACSyst."+
                                 "fillUserDefinedSubAcsAndGroupTreesObjects()"+
                                 " detects an error: no '<"+name+"_subacs>' directory.");
                 } // if
                 else if (l_values[i].equals("<GroupSubACS>")) {
                     m_ep = _acs.getEorM_OwnEligibleParties();

                     for (Object obj : m_ep.values()) {
                         member = (ImmutableEligibleParty) obj;
                         if (member instanceof ImmutableGroupID &&
                                 member.getEPType().indexOf("<byAcsCreator> "+name) != -1) {
                             hasGroupsSubACStoCreate = false;
                             break;
                         }
                     } // for obj
                 }
                 else if (l_values[i].equals("<GroupTree>")) {
                     m_ep = _acs.getEorM_OwnEligibleParties();

                     for (Object obj : m_ep.values()) {
                         member = (ImmutableEligibleParty) obj;
                         if (member instanceof ImmutableGroupID &&
                                 member.getEPType().indexOf("<byAcsCreator> "+name) != -1) {
                             hasGroupsTreeToCreate = false;
                             break;
                         }
                     } // for obj
                 }
             } // for i

             if (!Arrays.asList(l_values).contains("<GroupSubACS>"))
                 hasGroupsSubACStoCreate = false;
             if (!Arrays.asList(l_values).contains("<GroupTree>"))
                 hasGroupsTreeToCreate = false;

                        // for testing and tracking
                        //System.out.println("ActionNewACSyst."+
                          //       "fillUserDefinedSubAcsAndGroupTreesObjects() "+
                            //    "for name: "+name+", hasResourcesSubACStoCreate: "+
                             //   hasResourcesSubACStoCreate+", hasGroupsSubACStoCreate: "+
                               // hasGroupsSubACStoCreate+", hasGroupsTreeToCreate: "+
                                //hasGroupsTreeToCreate);

            if (hasResourcesSubACStoCreate) {
               DialogBox.showOkMessage("RESOURCES SubACS(s)"+
                       " to be defined. You should create the child(ren) of <"+name+"_subacs>");

               while (hasResourcesSubACStoCreate) {
                   setNewResource(_acs, directory, "<byAcsCreator> "+name);

                   hasResourcesSubACStoCreate = DialogBox.showConfirmDialog(
                           "Do you want to create another resource under "+
                           directory.getName().lastComponent(), null);
               } // while
            } // if
            if (hasGroupsSubACStoCreate) {
               DialogBox.showOkMessage("GROUPS SubACS(s) to be defined."+
                       " You should create the first group(s) with the type '<byAcsCreator> "+name+"'");

               while (hasGroupsSubACStoCreate) {
                   setNewGroup(_acs, "<byAcsCreator> "+name);

                   hasGroupsSubACStoCreate = DialogBox.showConfirmDialog(
                           "Do you want to create another group with the starting type "+
                           "'<byAcsCreator> "+name, null);
               } // while
            } // if
            if (hasGroupsTreeToCreate) {
               DialogBox.showOkMessage("GROUP TREE(S) to be defined."+
                       " You should create the first group(s) with the type '<byAcsCreator> "+name+"'");

               while (hasGroupsTreeToCreate) {
                   setNewGroup(_acs, "<byAcsCreator> "+name);

                   hasGroupsTreeToCreate = DialogBox.showConfirmDialog(
                           "Do you want to create another group with the starting type "+
                           "'<byAcsCreator> "+name, null);
               } // while
            } // if
            
            DialogBox.showOkMessage("End of the creation procedure for "+name+" Resources/Groups.");

       } // for entry

       DialogBox.showOkMessage("End of the creation for the ACS "+_acs.getName());
       
       m_SubAndGroups = null;
       l_values = null;
       entry = null;
       m_ep = null;
       member = null;
       l_leaves = null;
   } // fillWithAcsObjects()


  /**
  * Handles the definition by the user of the type, name and account of a new
  * resource, and creates it. The new resource creation updates the GUI.
  * Called by fillUserDefinedSubAcsAndGroupTreesObjects(). For creating a full
  * tree of directories and resources as a subACS, it is mandatory to code a specific
  * method, like for Linux.
  * @param _acs the new ACS
  * @param _parent the parent of the new directory to create
  * @param _typeStarting the starting of the type for the new directory
  */
  private void setNewResource(ACS _acs, Directory _parent, String _typeStarting) {

        // for testing purpose
        //System.out.println("setNewDirectory() called in ActionNewACSyst");

    UserID[]        l_users = ACS_Facade.getOwnUserArray(_acs);
    UserID          account = null;
    String          endOfName = null;
    String          type = null;
    ImmutableName   name = null;
    List<String>    l_values = new ArrayList(10);
    int             choice = -1;
    String[]        l_types = (String[])_acs.getM_TypesAndGUIPolicy().
                        get("Resource.ResourceType");


    endOfName = DialogBox.showSingleEditor("Enter the resource name for "+_typeStarting);
    if (endOfName == null || endOfName.trim().length() == 0) {
        DialogBox.showErrorMessage("No operation.");
        endOfName = null;
        l_values = null;
        l_types = null;
        return;
    }
    if (!ACS_Facade.validateFrameName(new NameImpl(endOfName.trim())) ) {
        DialogBox.showErrorMessage("This name contains forbidden character(s).");
        endOfName = null;
        l_values = null;
        l_types = null;
        return;
    }
    name = ((Name)_parent.getName()).extended(endOfName.trim());

    for (int i = 0; i < l_types.length; i++) {
        if (l_types[i].startsWith(_typeStarting))
            l_values.add(l_types[i]);
    }
    l_types = l_values.toArray(new String[0]);
    if (l_types.length == 0) {
        DialogBox.showErrorMessage("Wrong Resource types list. "+
                "There is no matching types for resources "+
                "starting with "+_typeStarting);
        endOfName = null;
        l_values = null;
        l_types = null;
        return;
    }
    else if (l_types.length == 1) type = l_types[0];
    else {
        try {
            choice = DialogBox.showOneSelectionOneDialog(
                    "type for the resource", l_types);
        } catch (SmallError s) {
            throw new InternalError("ActionNewACSyst.setNewGroup(), "+
                    "exception: "+s);
        }
        type = l_types[choice];
    } // else

    if (l_users.length != 0 && l_users[0] != null) {
        try {
            choice = DialogBox.showOneSelectionOneDialog(
                    "account for the resource, or nothing", l_users);
        } catch (SmallError s) {
            throw new InternalError("ActionNewACSyst.setNewDirectory(), "+
                    "exception: "+s);
        }
        if (choice != -1) account = l_users[choice];
    } // if

    try {
        new ResourceImpl(name, _acs, _parent, account, type);
    } catch (CreateError s) {
        throw new InternalError("ActionNewACSyst.setNewDirectory(), "+
                "exception: "+s);
    }

    endOfName = null;
    type = null;
    name = null;
    l_values = null;
    l_types = null;
    l_users = null;
    account = null;
  } // setNewResource()

  /**
  * Handles the definition by the user of the type and name of a new
  * group, and creates it. The new group creation updates the GUI.
  * Called by fillUserDefinedSubAcsAndGroupTreesObjects().
  * Calls ActionNewEligibleParty.handleNewGroup() to listen the group members for
  * the ACS trees in the GUI.
  * @param _acs the new ACS
  * @param _typeStarting the starting of the type for the new group
  */
  private void setNewGroup(ACS _acs, String _typeStarting) {

        // for testing purpose
        //System.out.println("setNewGroup() called in ActionNewACSyst");

    boolean         isAdmin = false;
    String          endOfName = null;
    String          type = null;
    ImmutableName   name = null;
    List<String>    l_values = new ArrayList(10);
    int             choice = -1;
    String[]        l_types = (String[])_acs.getM_TypesAndGUIPolicy().
                        get("EligibleParty.EPType");
    ImmutableGroupID result = null;

    
    endOfName = DialogBox.showSingleEditor("Enter the group name for "+_typeStarting);
    if (endOfName == null || endOfName.trim().length() == 0) {
        DialogBox.showErrorMessage("No operation.");
        endOfName = null;
        l_values = null;
        l_types = null;
        return;
    }
    if (!ACS_Facade.validateFrameName(new NameImpl(endOfName.trim())) ) {
        DialogBox.showErrorMessage("This name contains forbidden character(s).");
        endOfName = null;
        l_values = null;
        l_types = null;
        return;
    }
    name = ((Name)_acs.getName()).extended(endOfName.trim());

    for (int i = 0; i < l_types.length; i++) {
        if (l_types[i].startsWith(_typeStarting))
            l_values.add(l_types[i]);
    }
    l_types = l_values.toArray(new String[0]);
    if (l_types.length == 0) {
        DialogBox.showErrorMessage("Wrong EP types list. "+
                "There is no matching types for groups "+
                "starting with "+_typeStarting);
        endOfName = null;
        l_values = null;
        l_types = null;
        return;
    }
    else if (l_types.length == 1) type = l_types[0];
    else {
        try {
            choice = DialogBox.showOneSelectionOneDialog(
                    "type for the group", l_types);
        } catch (SmallError s) {
            throw new InternalError("ActionNewACSyst.setNewGroup(), "+
                    "exception: "+s);
        }
        type = l_types[choice];
    } // else

    try {
        choice = DialogBox.showOneSelectionOneDialog(
                "TRUE for an administrative group", new String[]{"TRUE", "FALSE"});
    } catch (SmallError s) {
        throw new InternalError("ActionNewACSyst.setNewGroup(), "+
                "exception: "+s);
    }
    if (choice == 0) isAdmin = true;

    try {
        result = new GroupIDImpl(name, _acs, type, isAdmin);
    } catch (CreateError s) {
        throw new InternalError("ActionNewACSyst.setNewGroup(), "+
                "exception: "+s);
    }

    ActionNewEligibleParty.handleNewGroup(result);

    endOfName = null;
    type = null;
    name = null;
    l_values = null;
    l_types = null;
    result = null;
  } // setNewGroup()

} // class ActionNewACSyst