//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.AbstractAction;
import java.awt.event.ActionEvent;
import java.util.Iterator;
import java.util.List;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.Gui2.*;
import ARoad0.gWork.ACS_Facade;
import ARoad0.ErrW.*;
import ARoad0.ErrB.*;
import ARoad0.gBaseInterface.EligibleParty;
import ARoad0.gBaseInterface.GroupID;
import ARoad0.gBaseInterface.GroupIDMember;
import ARoad0.gBaseInterface.ImmutableGroupID;
import ARoad0.gBaseInterface.UserID;
import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBaseInterface.ACS;
import ARoad0.gBaseInterface.ImmutableACS;
import ARoad0.gBase.AssociateUtilityImpl;
import ARoad0.gBase.ACSFactoryUtilityImpl;
import ARoad0.gBase.UtilityImpl;

/**
 * An action that creates a new eligible party
 * when the user requests it in the beamer. createEligibleParty() is the
 * main method, and it is invoked by the beamer PropertyEditors.
 * The name has the structure 'IS-name :: ACS-name :: Resource-name'.
 * This class manages all the dialog boxes required for the user input,
 * controls the inputs, creates the eligible party in its ACS,
 * then updates all the open frames in the GUI.
 */
public class ActionNewEligibleParty extends AbstractAction
implements Singleton, NewBaseObjectListener {

  /**
   * The value of the ACS which is the source.
   */
  protected BaseObject source_;    
    
  /**
   *  the only instance of the class, following singleton pattern
   */
  private static ActionNewEligibleParty    instance__ = new ActionNewEligibleParty();

  /**
  * with this private constructor, the compiler won't generate
  * a default public constructor
  */
  private ActionNewEligibleParty() {
    super("New Eligible Party");

    //initialization

  }

  /**
   * only public method to get the unique instance
   * @return ActionNewEligibleParty, the unique instance
   */
  public static synchronized ActionNewEligibleParty getInstance() {
    return instance__;
  } // getInstance()

  /**
   * Gets the value of the source.
   * @return the source of the property.
   */
  public BaseObject getSource() {
    return source_;
  }  // getSource()    
  
  /**
  * Manages the actions fired from the main menu,
  * and displays a dialog box. No use.
  * @param ev ActionEvent
  */
  public void actionPerformed(ActionEvent ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
    
    // test
    if (ACS_Facade.getOpenACS().isEmpty()) {
      DialogBox.showOkMessage("You must open an ACS, first.");
      return;
    }
    createEligibleParty(new Object[] {null, null, null});
  } // actionPerformed(ActionEvent)

  /**
   * Invoked by CollectionPropertyEditor.addElement(),
   * (from information of ACS/DirectoryImplBeanInfo classes in gBase) to create the EP.
   * Reduces the arguments array to the 3 first ones if the class to create
   * is GroupID ou UserID. Uses a BaseObjectHandler instance.
   * Calls ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames() and
   * getEffectiveClassName().
   * @param _l_fixedValue array of the arguments.
   *  They may be all Strings, all ImmutableNames or all BaseObjects,
   *  as arguments which has been chosen by the user before the call of this method.
   *  The value at the index 0 is the chosen value for the first argument, and so on.
   *  A value in the array is null if the argument for this index is not pre-defined.
   *  The size of the array may be inferior or superior to the number of arguments.
   **/
   public void createEligibleParty(Object[] _l_fixedValue) {
    if (_l_fixedValue == null || _l_fixedValue.length < 2 || 
            !(_l_fixedValue[1] instanceof ACS))
                throw new InternalError("in ActionNewEligibleParty.createEligibleParty(), "+
                  "the argument is not valid.");
       
       // initialization
    String[]    l_object = null;
    List<String>    l_class = null;
    source_ = (ACS)_l_fixedValue[1];
    ACS         acs =(ACS) source_;
    int         select;
    Object[]    l_fixedValue = _l_fixedValue;
    Class       effectiveClass = null;
    String      effectiveClassName = null;
    
    if (!acs.getManageAccounts() && !acs.getManageGroups() && !acs.getManageActors()) {
        DialogBox.showAlarmMessage("The ACS does not manage the EligibleParties!");
        l_fixedValue = null;
        acs = null;
        return;
    }

    if (acs.getManageAccounts() && !acs.getManageGroups() && !acs.getManageActors()) {

        l_class = ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.USER_CLASS);
        l_class.add(ACSFactoryUtilityImpl.USER_CLASS);

        l_object = l_class.toArray(new String[0]);
    } else if (acs.getManageAccounts() && acs.getManageGroups() && !acs.getManageActors()) {

        l_class = ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.USER_CLASS);
        l_class.add(ACSFactoryUtilityImpl.USER_CLASS);

        l_class.addAll(ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.GROUP_CLASS));
        l_class.add(ACSFactoryUtilityImpl.GROUP_CLASS);

        l_object = l_class.toArray(new String[0]);
    } else if (!acs.getManageAccounts() && !acs.getManageGroups() && acs.getManageActors()) {

        l_class = ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.ACTOR_CLASS);
        l_class.add(ACSFactoryUtilityImpl.ACTOR_CLASS);

        l_object = l_class.toArray(new String[0]);
    } else if (acs.getManageAccounts() && !acs.getManageGroups() && acs.getManageActors() ) {

        l_class = ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.USER_CLASS);
        l_class.add(ACSFactoryUtilityImpl.USER_CLASS);

        l_class.addAll(ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.ACTOR_CLASS));
        l_class.add(ACSFactoryUtilityImpl.ACTOR_CLASS);

        l_object = l_class.toArray(new String[0]);
    } else if (!acs.getManageAccounts() && acs.getManageGroups() && !acs.getManageActors() ) {

        l_class = ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.GROUP_CLASS);
        l_class.add(ACSFactoryUtilityImpl.GROUP_CLASS);

        l_object = l_class.toArray(new String[0]);
    } else if (!acs.getManageAccounts() && acs.getManageGroups() && acs.getManageActors() ) {

        l_class = ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.ACTOR_CLASS);
        l_class.add(ACSFactoryUtilityImpl.ACTOR_CLASS);

        l_class.addAll(ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.GROUP_CLASS));
        l_class.add(ACSFactoryUtilityImpl.GROUP_CLASS);

        l_object = l_class.toArray(new String[0]);
    } else if (acs.getManageAccounts() && acs.getManageGroups() && acs.getManageActors() ) {

        l_class = ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.USER_CLASS);
        l_class.add(ACSFactoryUtilityImpl.USER_CLASS);

        l_class.addAll(ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.ACTOR_CLASS));
        l_class.add(ACSFactoryUtilityImpl.ACTOR_CLASS);

        l_class.addAll(ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.GROUP_CLASS));
        l_class.add(ACSFactoryUtilityImpl.GROUP_CLASS);

        l_object = l_class.toArray(new String[0]);
    } // else

    try {
      if (l_object.length > 1)
          select = DialogBox.showOneSelectionOneDialog("eligible party type:", l_object);
      else select = 0;
    } catch (SmallError s) {
      throw new InternalError(s.toString());
    }

    if (select == -1) {
      effectiveClass = null;
      effectiveClassName = null;
      acs = null;
      l_fixedValue = null;
      return;
    }
    
    try {
        // get the generic class, the AcsAddon classic class, or the AcsAddon extension class
        effectiveClassName = UtilityImpl.getFirstInstance().getEffectiveClassName(
                acs.getM_AcsVocabulary(), l_object[select]);

        effectiveClass = Class.forName(effectiveClassName);

        // reduces the arguments array to the 3 first ones if the class to create
        // is GroupID ou UserID.
        // Actor has a constructor which needs 7 arguments.
        if (GroupID.class.isAssignableFrom(effectiveClass) ||
               UserID.class.isAssignableFrom(effectiveClass) ) {
            l_fixedValue = new Object[3];
            System.arraycopy(_l_fixedValue, 0, l_fixedValue, 0, 3);
        } // if

          // creates any new BaseObject, with a constructor method.
            // calls back the handleNewBaseObject() method
            // when the new object has been defined by the user
            // through the BaseObjectHandler frame
        BaseObjectHandler.valueOf(this, effectiveClass).
            getBaseObjectEditor(l_fixedValue);
    } catch (Exception e) {
        System.out.println("In ActionNewEligibleParty.createEligibleParty(),"+
            " call to BaseObjectHandler.valueOf(this, l_objectClass[select])."+
            "getBaseObjectEditor(l_fixedValue) with"+
            " the exception: "+e.toString());
    } // catch
        
    effectiveClass = null;
    effectiveClassName = null;
    l_fixedValue = null;
    acs = null;
   } // createEligibleParty

  /**
  * This method registers a GroupID in the current ACSTreeBaseListenerImpl as listeners for
  * the 'l_Members' bound property. Adds the associates, if any, to the EligibleParty.
  * Displays to the user a message on the recommanded groups the new GroupIDMember
  * should belong to. The number of recommanded groups is from 1 to 4, inclusive.
  * Called by BaseObjectHandler for createEligibleParty().
  * @param _obj is the eligible party instance which has been created
  */
  public void handleNewBaseObject(BaseObject _obj) {

                // for testing and tracking
                //System.out.println("Call to ActionNewEligibleParty. "+
                //"New eligible party: "+ _obj.getName());
      
    if (!(_obj instanceof ImmutableGroupID)) {
        // add the associates, if any
        try {
            AssociateUtilityImpl.addAssociatesAfterCreation((EligibleParty)_obj);
        } catch (BaseError b) {
            DialogBox.showAlarmMessage("After the creation of "+
                    ((EligibleParty)_obj).getName().lastComponent()+
                    ", one of its associates is not correctly created or linked to it: "+
                    b.toString());
            b.printStackTrace();
        }
    } // if (!(_obj instanceof ImmutableGroupID))
    else {
         // registers the ImmutableGroupID in ACSTreeBaseListenerImpls
        handleNewGroup((ImmutableGroupID)_obj);

        // adds the associates to the ImmutableGroupID, if any
        try {
            AssociateUtilityImpl.addAssociatesAfterCreation((EligibleParty)_obj);
        } catch (BaseError b) {
            DialogBox.showAlarmMessage("After the creation of "+
                    ((EligibleParty)_obj).getName().lastComponent()+
                    ", one of its associates is not correctly created or linked to it: "+
                    b.toString());
            b.printStackTrace();
        } 
    } // else
    
    // displays to the user a message on the recommanded groups the new GroupIDMember
    // should belong to. The number of groups is from 1 to 4.
    if (_obj instanceof GroupIDMember) {
        ImmutableACS acs = ((GroupIDMember)_obj).getEorACS();
        if (acs.getM_TypeOrientedPolicy() != null) {
              String[] l_types = (String[]) acs.getM_TypeOrientedPolicy().get(
                "GroupIDMember.RecommandedGroupsAsMember."+
                ((GroupIDMember)_obj).getEPType());
              if (l_types != null && l_types.length == 1) 
                  DialogBox.showOkMessage("The new object "+
                      _obj.getName().lastComponent()+" has to be the direct member of "+
                      "a group having the type "+l_types[0]);
              else if (l_types != null && l_types.length == 2) 
                  DialogBox.showOkMessage("The new object "+
                      _obj.getName().lastComponent()+" has to be the direct member of "+
                      "2 differents groups having the types listed hereafter: \n - "+
                      l_types[0]+",\n - "+l_types[1]);
              else if (l_types != null && l_types.length == 3) 
                  DialogBox.showOkMessage("The new object "+
                      _obj.getName().lastComponent()+" has to be the direct member of "+
                      "3 differents groups having the 3 types listed hereafter:\n - "+
                      l_types[0]+",\n - "+l_types[1]+
                      ",\n - "+l_types[2]);              
              else if (l_types != null && l_types.length == 4) 
                  DialogBox.showOkMessage("The new object "+
                      _obj.getName().lastComponent()+" has to be the direct member of "+
                      "4 differents groups having the 4 types listed hereafter:\n"+
                      l_types[0]+", \n - "+l_types[1]+
                      ", \n - "+l_types[2]+",\n - "+l_types[3]); 
              l_types = null;
        } // if (acs.getM_TypeOrientedPolicy() != null)
        acs = null;
    } // if (_obj instanceof GroupIDMember
  } // handleNewBaseObject()

  /**
  * This method has to be invoked just after the creation of a GroupIDImpl,
  * to register the current ACSTreeBaseListenerImpls as listeners of any change
  * among the GroupID members.
  * Called by BaseObjectHandler for handleNewBaseObject(), gWork.AcsFactory,
  * gBase.AssociateUtilityImpl.addAssociateActions(), ActionNewACSyst.setNewGroup(),
  * and from the AcsAddons. This method may be called several times for the same new group.
  * @param _obj group which is just created
  */
  public static void handleNewGroup(ImmutableGroupID _obj) {

    TreeBaseListener listener;
    for (Iterator ite = TreeManager.getL_TreeBaseListener().iterator(); ite.hasNext(); ) {
        listener = (TreeBaseListener) ite.next();
        if (!(listener instanceof ACSTreeBaseListenerImpl)) continue;
        if (listener.getEorMainEventSource().equals(((ImmutableGroupID)_obj).getAcsName())) {

                // for testing and tracking
                //System.out.println("in ActionNewEligibleParty.handleNewGroup() "+
                //"add the 'l_Members' listener "+listener+" to the GroupID "+ _obj.toString() );
            
            ((ImmutableGroupID)_obj).addPropertyChangeListener("l_Members", listener);
        }
    } // for  
  } // handleNewGroup()
  
} // class ActionNewEligibleParty