//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard ( )
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.AbstractAction;
import java.awt.event.ActionEvent;
import java.util.List;
import java.util.Map;

import ARoad0.gWork.ACS_Facade;
import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.ErrW.*;
import ARoad0.ErrB.*;

import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBaseInterface.ACS;
import ARoad0.gBaseInterface.Resource;
import ARoad0.gBase.AssociateUtilityImpl;
import ARoad0.gBase.ACSFactoryUtilityImpl;
import ARoad0.gBase.UtilityImpl;

/**
 * An action that creates a new resource
 * when the user requests it in the beamer. createResource() is the
 * main method, and it is invoked by the beamer PropertyEditors.
 * The name has the structure 'IS-name :: ACS-name :: Resource-name'.
 * This class manages all the dialog boxes required for the user input,
 * controls the inputs, creates the resource in its ACS,
 * then updates all the open frames in the GUI.
 */
public class ActionNewResource extends AbstractAction
implements Singleton, NewBaseObjectListener {

  /**
   * The value of the ACS which is the source.
   */
  protected BaseObject source_;
    
  /**
   *  the only instance of the class, following singleton pattern
   */
  private static ActionNewResource    instance__ = new ActionNewResource();

  /**
  * with this private constructor, the compiler won't generate
  * a default public constructor
  */
  private ActionNewResource() {
    super("New Resource");

    //initialization

  }

  /**
   * only public method to get the unique instance
   * @return ActionNewResource, the unique instance
   */
  public static synchronized ActionNewResource getInstance() {
    return instance__;
  } // getInstance()

  /**
   * Gets the value of the ACS.
   * @return the source of the property.
   */
  public BaseObject getSource() {
    return source_;
  }  // getSource()   
  
  /**
   * NO USE.
  * Manage the actions fired from the main menu,
  * and display a dialog box.
  * @param ev ActionEvent
  */
  public void actionPerformed(ActionEvent ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
    
    // test
    if (ACS_Facade.getOpenACS().isEmpty()) {
      DialogBox.showOkMessage("You must open an Access Control System, first.");
      return;
    }
    createResource(new Object[] {null});
    return;
  } // actionPerformed(ActionEvent)

  /**
   * Invoked by actionPerformed() or by CollectionPropertyEditor
   * (from data in ACS/DirectoryImplBeanInfo classes)
   * to create the resource. Uses a BaseObjectHandler instance.
   * Calls ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames() and
   * getEffectiveClassName().
   * @param _l_fixedValue array of the arguments.
   *  _l_fixedValue[1] is always the ACS.
   *  They may be all Strings, all ImmutableNames or all BaseObjects,
   *  as arguments which has been chosen by the user or which are set in the BeanInfo.
   *  The value at the index 0 is the chosen value for the first argument, and so on.
   *  A value in the array is null if the argument for this index is not pre-defined.
   *  The size of the array may be inferior or superior to the number of arguments.
   * */
  public void createResource(Object[] _l_fixedValue) {

            // for testing and tracking
            //System.out.println("Call to ActionNewResource.createResource() with the fixed values:" );
            //for (int i = 0; i < _l_fixedValue.length; i++) {
              //  System.out.println("---------- "+_l_fixedValue[i]);
            //} // for
       
    if (_l_fixedValue == null || _l_fixedValue.length < 2 || 
            !(_l_fixedValue[1] instanceof ACS))
                throw new InternalError("in ActionNewResource.createResource(), "+
                  "the argument is not valid.");
       
       // initialization
    String[]    l_object = null;
    List<String>    l_class = null;
    ACS         acs = (ACS)_l_fixedValue[1];
    Class       effectiveClass = null;
    String      effectiveClassName = null;
    int         select;

            // for testing and tracking
            //System.out.println("Call to ActionNewResource.createResource() with acs : "+
              //      acs.getClass());


    source_ = acs;
    
    if (!acs.getManageResources()) {
        DialogBox.showAlarmMessage("The ACS does not manage resources!");
        acs = null;
        return;
    }

    if (!acs.getManageResourceTrees() && !acs.getManageActors()) {

        l_class = ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.RESOURCE_CLASS);
        l_class.add(ACSFactoryUtilityImpl.RESOURCE_CLASS);

        l_object = l_class.toArray(new String[0]);
    } else if (!acs.getManageResourceTrees() && acs.getManageActors()) {

        l_class = ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.RESOURCE_CLASS);
        l_class.add(ACSFactoryUtilityImpl.RESOURCE_CLASS);

        l_class.addAll(ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.ACTOR_CLASS));
        l_class.add(ACSFactoryUtilityImpl.ACTOR_CLASS);

        l_object = l_class.toArray(new String[0]);
    } else if (acs.getManageResourceTrees() && !acs.getManageActors()) {

        l_class = ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.RESOURCE_CLASS);
        l_class.add(ACSFactoryUtilityImpl.RESOURCE_CLASS);

        l_class.addAll(ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.DIRECTORY_CLASS));
        l_class.add(ACSFactoryUtilityImpl.DIRECTORY_CLASS);

        l_object = l_class.toArray(new String[0]);
    } else if (acs.getManageResourceTrees() && acs.getManageActors()) {

        l_class = ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.RESOURCE_CLASS);
        l_class.add(ACSFactoryUtilityImpl.RESOURCE_CLASS);

        l_class.addAll(ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.DIRECTORY_CLASS));
        l_class.add(ACSFactoryUtilityImpl.DIRECTORY_CLASS);

        l_class.addAll(ACSFactoryUtilityImpl.getL_AcsAddonExtensionClassNames(
          acs.getM_AcsVocabulary(), ACSFactoryUtilityImpl.ACTOR_CLASS));
        l_class.add(ACSFactoryUtilityImpl.ACTOR_CLASS);

        l_object = l_class.toArray(new String[0]);
    } // else if
    
    try {
      if (l_object.length > 1)
                select = DialogBox.showOneSelectionOneDialog("resource type:", l_object);
      else select = 0;
    } catch (SmallError s) {
      throw new InternalError(s.toString());
    }

    if (select == -1) {
      l_object = null;
      //l_fixedValue = null;
      acs = null;
      effectiveClass = null;
      effectiveClassName = null;
      return;
    } // if
    
    try {
                        // for testing and tracking
                        //System.out.println("in ActionNewResource.createResource(), "+
                          //      "acs.getM_AcsVocabulary():");
                        //for (Object obj : acs.getM_AcsVocabulary().entrySet()) {
                          //  Map.Entry entry = (Map.Entry)obj;
                            //System.out.println("--- value at 0: "+((String[])entry.getValue())[0]);
                        //} // for

        // get the generic class, the AcsAddon classic class, or the AcsAddon extension class
        effectiveClassName = UtilityImpl.getFirstInstance().getEffectiveClassName(
                acs.getM_AcsVocabulary(), l_object[select]);

        effectiveClass = Class.forName(effectiveClassName);

                        // for testing and tracking
                        //System.out.println("Call to ActionNewResource.createResource() for the final class : "+
                          //      effectiveClassName);

          // creates any new BaseObject, with a constructor method.
            // calls back the handleNewBaseObject() method
            // when the new object has been defined by the user
            // through the BaseObjectHandler frame
        BaseObjectHandler.valueOf(this, effectiveClass).
            getBaseObjectEditor(_l_fixedValue);
    } catch (Exception e) {
        System.out.println("In ActionNewResource.createResource(),"+
            " call to BaseObjectHandler.valueOf(this, l_objectClass[select])."+
            "getBaseObjectEditor(l_fixedValue) with"+
            " the exception: "+e.toString());
        e.printStackTrace();
    }

    l_object = null;
    //l_fixedValue = null;
    acs = null;
    effectiveClass = null;
    effectiveClassName = null;
  } // createResource

  /**
  * Called by BaseObjectHandler for createResource().
  * Add the associates, if any.
  * @param _obj is the Resource instance which has been created
  */
  public void handleNewBaseObject(BaseObject _obj) {

                // for testing and tracking
                //System.out.println("Call to ActionNewResource. The new resource is: "+ _obj.toString() );
    
    // add the associates, if any
    try {
        AssociateUtilityImpl.addAssociatesAfterCreation((Resource)_obj);
    } catch (BaseError b) {
        DialogBox.showAlarmMessage("After the creation of "+
                ((Resource)_obj).getName().lastComponent()+
                ", one of its associates is not correctly created or linked to it: "+
                b.toString());
    }
  } // handleNewBaseObject()

} // class ActionNewResource