//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.Iterator;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.ErrW.*;
import ARoad0.ErrB.CreateError;
import ARoad0.ErrB.BaseError;
import ARoad0.gWork.ACS_Facade;
import ARoad0.gWork.ViewInBase_Facade;

import ARoad0.gBaseInterface.ViewInBase;
import ARoad0.gBaseInterface.EPRViewInBase;
//import ARoad0.gBaseInterface.NoThanViewInBase;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.Name;
import ARoad0.gBase.NameImpl;

/**
 * An action that creates a new EPR or NoMore-NoLess view
 * with the enter of its base objects by the user.
 * The view name has the structure "view-domain/view-name".
 * This class manages all the dialog boxes required for the user input,
 * controls the inputs, invokes Gui2.Diagram.createEPtoResourceView() that
 * creates the view through the call to gWork.ViewInBase_Facade and
 * gWork.Rights.findRights(),
 * then display the view in a new internal frame
 * @version 0.7.0
 */
public class ActionNewView extends AbstractAction
    implements Singleton, SelectionDialogListener {

  /**
   *  the only instance of the class, following singleton pattern
   */
  private static ActionNewView      instance__ = new ActionNewView();

  // for Set
  private static final int          INITIAL_CAPACITY = 5; //the default value: 101

  // variables for the user data
  private JTextField                groupField_;
  private JTextField                newNameField_;
  private JTextField                copiedNameField_;
  private Dimension                 dimension_ = new Dimension(150,25);
  private boolean                   forNoThanView_ = false;
      // for EPRView
  Class[]                           ePRClassArray_ = new Class[3];

  // environment
  private static String             lineSepar__ = System.getProperty("line.separator");

  /**
  * with this private constructor, the compiler won't generate
  * a default public constructor
  */
  private ActionNewView() {
    // used only by the menu, then reserved to new view creation
    super("New View");

    //initialization
      // for EPRView
    try {
      ePRClassArray_[0] = Class.forName("ARoad0.gBaseInterface.ImmutableEligibleParty");
      ePRClassArray_[1] = Class.forName("ARoad0.gBaseInterface.ImmutableResource");
      ePRClassArray_[2] = Class.forName("ARoad0.gBaseInterface.ImmutableVirtualFolder");
    } catch (ClassNotFoundException c) {
      throw new InternalError("In the ActionNewView constructor "+c.toString());
    }
  }

  /**
  * only public method to get the unique instance
  * @return ActionNewView, the unique instance
  */
  public static synchronized ActionNewView getInstance() {
    return instance__;
  } // getInstance()

  /**
  * Manages the actions fired from the main menu,
  * and display a dialog box.
  * A control validates the user inputs (eligible parties
  * and resources selection).
  * @param ev ActionEvent
  */
  public void actionPerformed(ActionEvent ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
    
    // test
    if (ACS_Facade.getOpenACS().isEmpty()) {
      DialogBox.showOkMessage("Open an Access Control System first.");
      return;
    }

    // test to avoid a second parallel call to showExplorerSelectionDialog()
    if (newNameField_ != null && newNameField_.getText().length() != 0) {
        DialogBox.showErrorMessage(
        "Please carry on first the current view selection dialog, or cancel it.");
        //System.out.println(" in actionPerformed(), newNameField_.getText(): "+newNameField_.getText() );
        return;
    }
    try {
        int choice = DialogBox.showOneSelectionOneDialog("view type",
                new String[] {"Full view", "NoMore-NoLess view"});
        if (choice == 1) {
                    forNoThanView_ = true;
                    
                  // for testing and tracking
                  //System.out.println(" in displayEPRView().actionPerformed(), forNoThanView_ set to true " );
                    
        } // if
        else if (choice == -1) return;
    } catch (SmallError s) {
        throw new InternalError("ActionNewView.actionPerformed(), error from DialogBox: "+
                s.toString());
    }

    // fired from the main menu, so
    // displays a dialog box for the name inputs
    if (showNameDialog("Enter the view name", null)) showSelectionDialog();
    else {
        forNoThanView_ = false;
        DialogBox.showErrorMessage("A name component is null.");
    }
    return;
  } // actionPerformed(ActionEvent)

  /**
   * Invoked by actionPerformed() to create
   * a selection dialog for entering the user selection of eligible parties, resources
   * or virtual folders.
   * If the name component is null or empty, contains 'ArchitectureOfAcs' or
   * is already known by the base, an error message is displayed.
   * Calls DialogBox.showExplorerSelectionDialog(), which calls back handleSelectionDialog().
   * Calls createEPRView().
   * There is no control of space character at the end of the text
   * entered by the user, nor control of the text size.
   * */
   private void showSelectionDialog() {
      // variables
      Set    objectsTypes = new HashSet(INITIAL_CAPACITY);
      Name   name = new NameImpl();
      boolean toreturn = false;

      // controls if there is at least one open ACS
      if (ACS_Facade.getOpenACS().isEmpty() ) {
        DialogBox.showErrorMessage(
          "Open at least one ACS, if you want to create a new view.");
        toreturn = true;
      }

      // controls an empty name
      if (!toreturn && (groupField_.getText() == null ||
        groupField_.getText().equals(new String())) ) {
        DialogBox.showErrorMessage(
          "Please enter a name for the view set.");
        toreturn = true;
      }
      if (!toreturn && (newNameField_.getText() == null ||
        newNameField_.getText().equals(new String())) ) {
        DialogBox.showErrorMessage(
          "Please enter a name for the view.");
        toreturn = true;
      }

      // controls a known name and ARCHI_VIEW_NAME
      name = new NameImpl(groupField_.getText() );
      name.extended(newNameField_.getText());

      if (!toreturn && (ViewInBase_Facade.knowsView( name ) || ACS_Facade.knowsACS(name))) {
          DialogBox.showErrorMessage(
            "This view name is already used by a view or an ACS."+
            lineSepar__+"Remove the known object before, if you want to create"+
            " a new view with this name.");
          toreturn = true;
      }
      else if (!toreturn && name.equals(ActionOpenArchitecture.ARCHI_VIEW_NAME) ) {
          DialogBox.showErrorMessage(
            "This name is reserved to the Architecture view.");
          toreturn = true;
      }
      else if (!toreturn && 
              (name.component(0).length() > 9 || name.component(1).length() > 9)) {
          DialogBox.showErrorMessage(
            "A name component has a length superior to 9.");
          toreturn = true;
      }
      
      if (toreturn) {
          groupField_.setText(null);
          newNameField_.setText(null);
          if (copiedNameField_ != null) copiedNameField_.setText(null);
          forNoThanView_ = false;
          return;
      }
      
      if (forNoThanView_)
          objectsTypes.add( ePRClassArray_[0] );
      else {
          objectsTypes.add( ePRClassArray_[2] );
          objectsTypes.add( ePRClassArray_[1] );
          objectsTypes.add( ePRClassArray_[0] ); // EP, to be the first one in the dialog
      }

      try {
          // calls handleSelectionDialog()
        DialogBox.getInstance().showExplorerSelectionDialog(
                this, "View first objects", objectsTypes, null, false);
      } catch (SmallError s) {
        throw new InternalError("in ActionNewView.showSelectionDialog(): "+s);
      }

      objectsTypes = null;
      name = null;
   } // showSelectionDialog()
  
/**
  * Verifies the user choices (selected eligible parties, resources or virtual folders).
  * Called from DialogBox.showExplorerSelectionDialog() AFTER a click on an active button,
  * even if the dialog is interrupted. Call createXXXView().
  * @param _validatedObjectsSetMap Map where the key is a class implementing 
  * the BaseObject interface, the value is a List (not a JList) of objects
  * that are instances of the key. Is null or empty if the dialog
  * has been interrupted by the user.
 */
  public void handleSelectionDialog(Map _validatedObjectsSetMap) {
      // variables
      boolean find = false;
      boolean toreturn = false;

              // for testing
              //System.out.println("ActionNewView.handleSelectionDialog(): start with forNoThanView_ = "+
                //      forNoThanView_);
      
      // stop if the argument value is null or empty
      if (_validatedObjectsSetMap == null || _validatedObjectsSetMap.isEmpty()) {
        groupField_.setText(null);
        newNameField_.setText(null);
        if (copiedNameField_ != null) copiedNameField_.setText(null);
        forNoThanView_ = false;
        return;
      } // if

      //if (!forNoThanView_ && _validatedObjectsSetMap.keySet().size() != 2) {
        //System.out.println("ActionNewView.handleSelectionDialog(): error on size");
        //return;
      //}
      for (Iterator ite = _validatedObjectsSetMap.keySet().iterator(); ite.hasNext() ; ) {
        Class type = (Class) ite.next();
        for (int i = 0 ; i < _validatedObjectsSetMap.keySet().size() ; i++) {
          if ( type.equals(ePRClassArray_[i]) ) {
            find = true;
            break;
          }
        } // for i
        if (!find) {
            toreturn = true;
            break;
        }
        find = false;
      } // for ite
      if (toreturn) {
        groupField_.setText(null);
        newNameField_.setText(null);
        if (copiedNameField_ != null) copiedNameField_.setText(null);
        forNoThanView_ = false;
        return;
      }
      
              // for testing
              //System.out.println("ActionNewView.handleSelectionDialog(): ready to create the view");
      
      // this is right data for an EPRView
      createView(_validatedObjectsSetMap);
  } // handleSelectionDialog()

  /**
  * Invoked by handleSelectionDialog(). Includes in the NoThanViewInBase name, the
  * ViewInBase_Facade.NOTHAN_VIEW_INDICATOR string value
  * as a component before the last component. Creates the EPRViewInBase or NoThanViewInBase,
  * and calls displayEPRView() to display its objects.
  * @param _validedObjectsSetMap Map where, if it is not a NoThanViewInBase,
  * the 2 keys are the classes ImmutableEligibleParty and ImmutableResource,
  * the value is a List of objects instances of the key.
  * */
   private void createView(Map _validedObjectsSetMap) {
    // variables
    ViewInBase          view = null;
    Name                name = new NameImpl(groupField_.getText());
    List                targetList = null;
    List                sourceList = null;
    
    if (forNoThanView_ ) name.extended(ViewInBase_Facade.NOTHAN_VIEW_INDICATOR);
    name.extended(newNameField_.getText());

    try {
      // creates the NoThanViewInBase
      if (forNoThanView_ ) {
        view = ViewInBase_Facade.openOrCreateNoThanView(name,
            (List)_validedObjectsSetMap.get(_validedObjectsSetMap.keySet().toArray()[0]));
      }
      else {
          // creates the view with the right 3 Lists as parameters
          if (_validedObjectsSetMap.keySet().toArray()[0].equals(ePRClassArray_[0]) ) {
              sourceList = (List)_validedObjectsSetMap.get(
                    _validedObjectsSetMap.keySet().toArray()[0]);

              if (_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[1]) != null)
                  targetList = (List)_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[1]);
              if (targetList != null && _validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[2]) != null)
                targetList.addAll((List)_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[2]));
              else if (_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[2]) != null)
                  targetList = (List)_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[2]);
          } // if
          if (_validedObjectsSetMap.keySet().toArray()[1].equals(ePRClassArray_[0]) ) {
              sourceList = (List)_validedObjectsSetMap.get(
                    _validedObjectsSetMap.keySet().toArray()[1]);

              if (_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[0]) != null)
                  targetList = (List)_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[0]);
              if (targetList != null && _validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[2]) != null)
                targetList.addAll((List)_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[2]));
              else if (_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[2]) != null)
                  targetList = (List)_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[2]);
          } // if
          else {
              sourceList = (List)_validedObjectsSetMap.get(
                    _validedObjectsSetMap.keySet().toArray()[2]);

              if (_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[0]) != null)
                  targetList = (List)_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[0]);
              if (targetList != null && _validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[1]) != null)
                targetList.addAll((List)_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[1]));
              else if (_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[1]) != null)
                  targetList = (List)_validedObjectsSetMap.get(
                      _validedObjectsSetMap.keySet().toArray()[1]);
          } // else

          view = ViewInBase_Facade.openOrCreateEPRView(name,
                targetList,
                sourceList );
      } // else
    } catch (DialogError d) {
      DialogBox.showErrorMessage("The view parameters are not correct.");
      view = null;
      name = null;
      targetList = null;
      sourceList = null;
      return;
    }

    displayView(view);

    view = null;
    name = null;
    targetList = null;
    sourceList = null;
   } // createView()

  /**
  * Called by createView() and by createExtendedEPRView().
  * @param _view to display
  * */
   private void displayView(ViewInBase _view) {

    ActionOpenGraphicView.openViewInGui((EPRViewInBase)_view, true, null);
       
    // null for the next view creation
    // until this step, actionPerformed() and createExtendedEPRView() are inactivated
    groupField_.setText(null);
    newNameField_.setText(null);
    if (copiedNameField_ != null) copiedNameField_.setText(null);
    forNoThanView_ = false;
          // for testing and tracking
          //System.out.println(" in displayEPRView(), newNameField_.getText(): "
          //+newNameField_.getText() );
   } // displayView()
   
  /**
  * Displays a dialog box to let the user enters the view name, when he has
  * clicked on the 'New extended view' in a view, and creates the new view
  * from the BaseObject list.
  * @param _currentViewName is the name of the current view from which an extended
  *   view is created
  * @param _l_BaseObject is the list of the BaseObjects in the view.
  */
  public void createExtentedEPRView(ImmutableName _currentViewName, Set _l_BaseObject) {
    // variables
    Name           name;
    EPRViewInBase  view;

    // tests to avoid a second parallel call to this instance
    if (newNameField_ != null && !newNameField_.getText().equals(new String() )) {
        DialogBox.showErrorMessage(
        "Please, carry on the current view creation, or cancel it.");
        //System.out.println(" in actionPerformed(), newNameField_.getText(): "+
        //  newNameField_.getText() );
        return;
    }

    /*
    copiedNameField_ = new JTextField();
    copiedNameField_.setActionCommand("copiedNameField_");
    copiedNameField_.setMaximumSize(dimension_);

    JPanel lastPanel = new JPanel();

    label = new JLabel("_future_ option:");
    label.setAlignmentX(label2.CENTER_ALIGNMENT);
    lastPanel.add(label );
    label = new JLabel("Name of a view to copy in the same group");
    label.setAlignmentX(label2.CENTER_ALIGNMENT);
    lastPanel.add(label );
    copiedNameField_.setAlignmentX(copiedNameField_.CENTER_ALIGNMENT);
    lastPanel.add(copiedNameField_);
    */

    if (!showNameDialog("Create from "+_currentViewName.toString(), null)) {
        groupField_.setText(null);
        newNameField_.setText(null);
        if (copiedNameField_ != null) copiedNameField_.setText(null);
        return;
    }

    name = new NameImpl(groupField_.getText());
    name.extended(newNameField_.getText());
    try {
      view = ViewInBase_Facade.createEPRView(name, _l_BaseObject);
    }
    catch (CreateError c) {
      c.printStackTrace();
      DialogBox.showErrorMessage(
        "This name view is already used."+
        lineSepar__+"Remove the view before, if you want to create"+
        " a new view with this name.");
      groupField_.setText(null);
      newNameField_.setText(null);
      if (copiedNameField_ != null) copiedNameField_.setText(null);
      return;
    }
    catch (BaseError c) {
      c.printStackTrace();
      DialogBox.showErrorMessage(
        "This name view is already used."+
        lineSepar__+"Remove the view before, if you want to create"+
        " a new view with this name.");
      groupField_.setText(null);
      newNameField_.setText(null);
      if (copiedNameField_ != null) copiedNameField_.setText(null);
      return;
    }

    displayView(view);

    name = null;
    view = null;
  } // createExtentedView()


  //
  // ---------------------------- Private methods ----------------------------
  //

  /**
  * Displays a dialog box to let the user enters the view type and the view name.
  * Used by actionPerformed() and createExtentedEPRView().
  * Calls ACS_Facade.validateFrameName().
  * Updates groupField_ and newNameField_.
  * @param _title of the dialog box
  * @param _lastPanel contains the name of the copied view. May be null.
  * @return true if the user has entered the data
  */
  private boolean showNameDialog(String _title, JComponent _lastPanel) {

    // variables for the dialog box
    JOptionPane               pane;
    JPanel                    panel = new JPanel();
    JLabel                    label = new JLabel("Use mouse or Tab to edit.");
    JDialog                   dialog;

      // initialization to each new dialog box
    groupField_ = new JTextField();
    newNameField_ = new JTextField();
    groupField_.setActionCommand("groupField_");
    newNameField_.setActionCommand("newNameField_");
    groupField_.setMaximumSize(dimension_);
    newNameField_.setMaximumSize(dimension_);

    label.setAlignmentX(label.CENTER_ALIGNMENT);
    panel.add( label );
    label = new JLabel(" "); // a space
    label.setAlignmentX(label.CENTER_ALIGNMENT);
    panel.add( label );

    label = new JLabel("Name of the view set (< 10 char.)");
    label.setAlignmentX(label.CENTER_ALIGNMENT);
    panel.add( label );
    groupField_.setAlignmentX(groupField_.CENTER_ALIGNMENT);
    panel.add(groupField_);
    label = new JLabel(" "); // a space
    label.setAlignmentX(label.CENTER_ALIGNMENT);
    panel.add( label);

    label = new JLabel("Name of the view (< 10 char.)");
    label.setAlignmentX(label.CENTER_ALIGNMENT);
    panel.add( label );
    newNameField_.setAlignmentX(newNameField_.CENTER_ALIGNMENT);
    panel.add(newNameField_);
    label = new JLabel(" "); // a space
    label.setAlignmentX(label.CENTER_ALIGNMENT);
    panel.add( label );

    if (_lastPanel != null) panel.add(_lastPanel);

    panel.setSize(200, 75);

    // show the dialog box
    dialog = DialogBox.showOkCancelDialog(_title, panel);

    // here, dialog waits an enter from the user,
    // then the next instructions are processed.
      // if the user enters a "return" in an editable field,
      // it is also processed here as OK_OPTION.

    pane = (JOptionPane)dialog.getContentPane().getComponent(0);

    // no action if CLOSED, and close the dialog box
    if ( pane.getValue() == null) {
      groupField_.setText(null);
      newNameField_.setText(null);
      if (copiedNameField_ != null) copiedNameField_.setText(null);
      return false;
    }
    // else, it is an Integer
    int value = ((Integer)pane.getValue()).intValue();

    pane = null;
    panel = null;
    label = null;
    dialog = null;

      // no action if CANCEL, and close the dialog box
    if (value == JOptionPane.CANCEL_OPTION) {
      groupField_.setText(null);
      newNameField_.setText(null);
      if (copiedNameField_ != null) copiedNameField_.setText(null);
      return false;
    }
      // if OK, action and close the dialog box
    else if (value == JOptionPane.OK_OPTION &&
            ACS_Facade.validateFrameName(new NameImpl(groupField_.getText())) &&
            ACS_Facade.validateFrameName(new NameImpl(newNameField_.getText())) ) {        
        return true;
    } // else

    else {
        groupField_.setText(null);
        newNameField_.setText(null);
        return false;
    }
  } // showNameDialog()


} // class ActionNewView