//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.*;
import java.beans.*;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.ArrayList;
import java.util.Iterator;
import java.net.MalformedURLException;
import java.lang.reflect.InvocationTargetException;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.Gui2.*;
import ARoad0.ErrW.*;
import ARoad0.ErrB.BaseError;
import ARoad0.gWork.ViewInBase_Facade;

import ARoad0.gBaseInterface.NoThanViewInBase;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBase.BaseManagerImpl;
import ARoad0.gBase.NameImpl;

/**
 * An action that opens all the open NoThan views in a frame with a tabbed pane,
 * where each view is displayed in a dedicated tab with its 'See why' text.
 * @see ActionOpenAllFullViews
 * @version 0.7.0
 */
public class ActionOpenAllNoThanViews extends AbstractAction implements Singleton, ChangeListener {

  /* the only instance of the class, following singleton pattern */
  private static ActionOpenAllNoThanViews instance__ = new ActionOpenAllNoThanViews();
  private static ImageIcon                icon__ = null;
  
  static {
      try {
            icon__ = new ImageIcon(
                    ARoad0.Gui1.Desktop.AR_URI.resolve("Gui1"+
                    ARoad0.Gui1.Desktop.URL_SEPAR+"images"+
                    ARoad0.Gui1.Desktop.URL_SEPAR+"view.gif").toURL());
      } catch (MalformedURLException m) {
            m.printStackTrace();
      }
  } // static
  
  /**
   * Current CommonPropertyEditors in the user dialog. The key is the display name.
   */
  private Map                       m_currentPropertyEditors_ = new HashMap(40);

  /**
   * with this private constructor, the compiler won't generate
   * a default public constructor
   */
  private ActionOpenAllNoThanViews() {
    super("All NoMore-NoLess Views", icon__);
  }

  /**
   * Only method to get an instance.
   * @return ActionOpenAllNoThanViews
   */
  public static synchronized ActionOpenAllNoThanViews getInstance(){
    return instance__;
  } // getInstance()

  /**
   * Close the NoThan view main frames and the'See why' frames, it they exist
   * @param _ev ActionEvent
   */
  public void actionPerformed(ActionEvent _ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
      
    Map map = ViewInBase_Facade.getOpenNoThanViews();
    
    Map m_GUIconfig;
    // set the frame view and the graphic view disposition
    try {
        m_GUIconfig = BaseManagerImpl.getInstance().getGUI();
    } catch (BaseError b) {
        throw new InternalError("in ActionOpenAllNoThanViews.actionPerformed(), error: "+b);
    }    
    
    if (map == null || map.isEmpty()) {
        DialogBox.showErrorMessage("No view to display");
        m_GUIconfig = null;
        return;
    } else if (InternalFrameModel.withAllNoThanViews()) {
        DialogBox.showErrorMessage("The 'All NoMore-NoLess views' window is already open");
        map = null;
        m_GUIconfig = null;
        return;    
    } else {
        // close the NoThan view main frames and the'See why' frames, it they exist
        JInternalFrame frame;
        ImmutableName name;
        ArrayList list = new ArrayList(InternalFrameModel.
                getMainFramesOfBaseObjects().values());
        try {
            for (Iterator ite = list.iterator(); ite.hasNext(); ) {
                frame = (JInternalFrame)ite.next();
                if (InternalFrameUtilities.isViewFrame(frame)) {
                    name = InternalFrameUtilities.extractViewNameFromFrameTitle(frame.getTitle());
                    // close the main and the 'See why' frames.
                    // the graphic views are still living, to be used by openAllFullViews(map, m_GUIconfig).
                    if (ViewInBase_Facade.isNoThanViewName(name))
                        ActionClose.closeMainFrameInGUI(name, frame, false);
                } // if
            } // for
        } catch (DialogError d) {
            throw new InternalError("in ActionOpenAllNoThanViews.actionPerformed(ActionEvent _ev), "+
                    "the extraction of the view name fails.");
        }
        frame = null;
        name = null;
        list = null;
        
        // do the main job
        openAllNoThanViews(map, m_GUIconfig);
    } // else
    
    map = null;
    m_GUIconfig = null;
    return;
  } // actionPerformed()

 /**
 * Invoked by actionPerformed() and Desktop to display the 'All NoMore-NoLess views'
 * frame including a tabbed pane. Run in the event dispatch thread.
 * Use a worker thread to create the graphic views and to go
 * in the event dispatch thread, for displaying an information message 'Please wait'
 * and for opening the 'All NoThan views' frame.
 * @param _viewMap of the views to open; the name is the key and the view is the value
 * @param _m_GUIconfig is the map from the GUI saving, to get the graphic view
 *          disposition if the view is closed when this method is invoked. May be null.
 */
 public synchronized void openAllNoThanViews(Map _viewMap, Map _m_GUIconfig) {
    // variables
    Map             m_Config = null;
    Map             m_previousConfig = null;

    if (_m_GUIconfig != null) {
        m_Config = (Map) _m_GUIconfig.get(new NameImpl("<Last GUI>"));
        m_previousConfig = (Map) _m_GUIconfig.get(new NameImpl("<Previous GUI>"));
    }

    // final values
    final Map viewmap = _viewMap;
    final Map m_config = m_Config;
    final Map m_previousconfig = m_previousConfig;
    final ChangeListener thislistener = this;

    ViewModel.getViewsThreadsExecutor().execute(new Runnable() {
        @Override
        public void run() {
            // call in a worker Thread

            GraphicView     gview = null;
            NoThanViewInBase view;
            Map             gview_config = null;
            
                                    //System.out.println("call to "+
                                    //"ActionOpenAllNoThanViews.openAllNoThanViews() : ");

            final JInternalFrame mesframe = new JInternalFrame(
                    "The views are opening...", false, true, false , false);

            try {
                SwingUtilities.invokeAndWait(new Runnable(){
                    public void run(){
                        InternalFrame.prepareInformationFrame(mesframe, 300, 50);
                        mesframe.setVisible(true);
                    } // run
                }); // invokeAndWait()

                // in the worker thread
                for (Iterator ite = viewmap.values().iterator(); ite.hasNext();) {

                    view = (NoThanViewInBase)ite.next();

                    // no processing if the graphic view exists, when its main frame was opened
                    // before the call of this method
                    if (ViewModel.getGraphicViewOfView(view.getName()) == null) {
                        if (m_config != null) {
                           gview_config = (Map)m_config.get(
                                   (new NameImpl("<GraphicView for>")).extended(view.getName()) );
                        } // if
                        if (gview_config == null && m_previousconfig != null) {
                            gview_config = (Map)m_previousconfig.
                                    get( (new NameImpl("<GraphicView for>")).extended(view.getName()) );
                        } // if

                        ViewModel.createViewRightsMediator(view);
                        // gview_config may be null
                        gview = new GraphicNoThanView(view, gview_config);
                        gview_config = null;
                        // create the view GraphicViewBaseListenerImpl,
                        // and register the GraphicView
                        ViewModel.createGraphicViewBaseListener(view, gview);
                    } // if

                    // for testing and tracking
                    //System.out.println("in ActionOpenAllNoThanViews.openAllNoThanViews(), "+
                      //"one of the open view is: "+view.getName());
                      //"\n and the graphic view is: "+
                      //((GraphicView)ViewModel.getGraphicViewOfView(view.getName())).toString());

                } // for ite

                // in the Event Dispatch Thread
                SwingUtilities.invokeLater(new Runnable(){
                    public void run(){

                                // for testing and tracking
                                //System.out.println("in ActionOpenAllNoThanViews.openAllNoThanViews(), "+
                                  //"start the work in the event dispatch thread");

                        JPanel panel = new JPanel();
                        panel.setName("NoMore-NoLess Views");
                        JInternalFrame viewframe;

                        // get the tabbed pane from the view model
                        JTabbedPane tPane = ViewModel.getAllNoThanViewsJTabbedPane();
                        //tPane.removeAll();

                        try {
                            // to display the views.
                            // creates m_currentPropertyEditors_.
                            BeamerManager.setJTabbedPane(
                              (BaseObject)viewmap.values().toArray()[0], // first ViewInBase - not used as argument.
                              getNoThanViewsPropertyDescriptors(viewmap.keySet()),
                              tPane,
                              m_currentPropertyEditors_,
                              ActionClose.getInstance() );

                            panel.add( tPane, BorderLayout.CENTER );

                            // register the frame in InternalFrameModel and set the disposition
                            // from the GUI saving
                            viewframe = InternalFrame.showAllNoThanViewsFrame(panel);
                            Rectangle bounds = null;
                            if (m_config != null ) {
                                bounds = (Rectangle)m_config.get(new NameImpl("<AllNoThanViews>"));
                            }
                            if (bounds == null && m_previousconfig != null ) {
                                bounds = (Rectangle)m_previousconfig.get(new NameImpl("<AllNoThanViews>"));
                            }

                            if (bounds != null) viewframe.setBounds(bounds);
                        } catch (ProcessError p) {
                            throw new InternalError("in ActionOpenAllNoThanViews.openAllNoThanViews(): "+p);
                        }

                        // select the tab 0 and its view in the explorer
                        tPane.addChangeListener(thislistener);
                        if (tPane.getTabCount() > 0)
                            tPane.setSelectedIndex(0);

                        // close the information message
                        try {
                            mesframe.setClosed(true);
                        } catch (PropertyVetoException p) {
                            System.out.println("in ActionOpenAllNoThanViews.openAllNoThanViews(),"+
                              " the frame closing fires a PropertyVetoException: "+
                              p.toString());
                        }

                                // for testing and tracking
                                //System.out.println("in ActionOpenAllNoThanViews.openAllNoThanViews(), "+
                                  //"end of the work in the event dispatch thread");

                        panel = null;
                        tPane = null;
                        viewframe = null;

                    } // run
                }); // invokeLater()

                        // for testing and tracking
                        //System.out.println("in ActionOpenAllNoThanViews.openAllNoThanViews(): "+
                          //      "end of run().");
            } catch (ProcessError p) {
              DialogBox.showErrorMessage(p.toString());
              return;
            } catch (InterruptedException i) {
                    // for testing and tracking
                    System.out.println("in ActionOpenAllNoThanViews.openAllNoThanViews(), "+
                      "one of the open view is not created: "+i.getMessage());
            } catch (InvocationTargetException n) {
                    // for testing and tracking
                    System.out.println("in ActionOpenAllNoThanViews.openAllNoThanViews() "+
                      ": "+n.getMessage());
            }
        } // run()

    }); // new Thread


    m_Config = null;
    m_previousConfig = null;
    return;
  } // openAllNoThanViews() 
  
 /**
  * Listen the changes in the selected tab of the 'AllNoThanViews' JTabbedPane,
  * but no action.
  * @param _evt
  */
 public void stateChanged(ChangeEvent _evt) {
     //JTabbedPane tPane = ViewModel.getAllNoThanViewsJTabbedPane();
     
     //if (InternalFrameModel.getAllNoThanViewsFrame() != InternalFrameModel.getActiveFrame()) return;
     //if (tPane.getSelectedIndex() == -1) return;
     
     //NoThanViewInBase view = (NoThanViewInBase)ViewInBase_Facade.getAllStandardOpenViews().get(
       //                 ViewModel.getViewNameFromTabTitle(tPane.getTitleAt(tPane.getSelectedIndex())));

            // for testing and tracking
            //System.out.println("ActionOpenAllNoThanViews.stateChanged() for "+view.getName()); 
     
     //TreeManager.selectNodeInExplorer(view);
     
     //tPane = null;
     //view = null;
 } // stateChanged()
 
 // ------------ Package methods --------------------------------------

  /**
  * Called by ActionOpenGraphicView.
  * @return the map of the GraphicViewPropertyEditors, where the key is 
  *   the display name of the view.
  */
  Map getAllViewsPropertyEditors() {
    return m_currentPropertyEditors_;
  } // getEPRviewsPropertyEditors() 


  /**
  * Finalizes the current property editors.
  * Called by ActionClose(). 
  */
  void finalizePropertyEditors() {
     synchronized (m_currentPropertyEditors_) {
        if (!m_currentPropertyEditors_.isEmpty()) {
          for (Iterator ite = m_currentPropertyEditors_.values().iterator() ; ite.hasNext() ;) {
            ((CommonPropertyEditor)ite.next()).finalizeForProcess();
          }
        }
        m_currentPropertyEditors_.clear();
     }
    return;
  } // finalizePropertyEditors() 
  
 // ------------ Private methods --------------------------------------
  
 /**
 * Constructs the property descriptors for the current open NoThan views in the GUI,
 * where the display name is the ViewInBase name. 
 * Call ActionOpenGraphicView.getViewPropertyDescriptor().
 * @param _list of the open ViewInBase names
 * @return the descriptors
 */
 private static PropertyDescriptor[] getNoThanViewsPropertyDescriptors(Set _list) {
    // variables
    PropertyDescriptor[] l_result = new PropertyDescriptor[_list.size()];
    ImmutableName name;
    int index = 0;

    for (Iterator ite = _list.iterator(); ite.hasNext(); ) {
      name = (ImmutableName)ite.next();
      l_result[index] = ActionOpenGraphicView.getViewPropertyDescriptor(name);
      index++;
    } // for ite

    return l_result;
  } // getNoThanViewsPropertyDescriptors()    
 
} // class ActionOpenAllNoThanViews

