//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.Map;
import java.net.MalformedURLException;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.Gui2.*;
import ARoad0.ErrW.*;
import ARoad0.ErrB.BaseError;
import ARoad0.ErrB.CreateError;
import ARoad0.gWork.ACS_Facade;
import ARoad0.gWork.ViewInBase_Facade;

import ARoad0.gBaseInterface.ViewInBase;
import ARoad0.gBaseInterface.Name;
import ARoad0.gBase.BaseManagerImpl;
import ARoad0.gBase.NameImpl;
import ARoad0.gBase.ViewInBaseImpl;

/**
 * UNUSED - An action that opens a GraphicView displaying all the open ACS
 * with their bridge links.
 * @version 0.7.0
 */
public class ActionOpenArchitecture extends AbstractAction implements Singleton {

  /* the only instance of the class, following singleton pattern */
  private static ActionOpenArchitecture instance__ = new ActionOpenArchitecture(); 

  public static final String            ARCHI_VIEW_STRING = ViewInBaseImpl.ARCHI_VIEW_STRING;
  public static final Name              ARCHI_VIEW_NAME = ViewInBaseImpl.ARCHI_VIEW_NAME;
  private static ImageIcon              icon__ = null;
  
  static {
      try {
          icon__ = new ImageIcon(
                    ARoad0.Gui1.Desktop.AR_URI.resolve("Gui1"+
                    ARoad0.Gui1.Desktop.URL_SEPAR+"images"+
                    ARoad0.Gui1.Desktop.URL_SEPAR+"view.gif").toURL());
      } catch (MalformedURLException m) {
        m.printStackTrace();
      }
  } // static
  
  /**
   * with this private constructor, the compiler won't generate
   * a default public constructor
   */
  private ActionOpenArchitecture() {
    super("Architecture", icon__);
  }

  /**
   * Only method to get an instance.
   * @return ActionOpenArchitecture
   */
  public static synchronized ActionOpenArchitecture getInstance(){
    return instance__;
  } // getInstance()

  /**
   * Manages opening commands from one source:
   * - from 'Display Architecture' in the main menu.
  * @param _ev ActionEvent produced by the click on "Open Architecture".
   */
  public void actionPerformed(ActionEvent _ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
      
    Map map = ACS_Facade.getOpenACS();
    Map m_GUIconfig;
    // set the frame view and the graphic view disposition
    try {
        m_GUIconfig = BaseManagerImpl.getInstance().getGUI();
    } catch (BaseError b) {
        throw new InternalError("in ActionOpenArchitecture.actionPerformed(), error: "+b);
    }    
    
    if (map == null || map.isEmpty()) {
        DialogBox.showErrorMessage("There is no open ACS to display in the architecture");
        m_GUIconfig = null;
        return;
    } else if (InternalFrameModel.withArchitecture()) {
        DialogBox.showErrorMessage("The 'Architecture' window is already open");
        map = null;
        m_GUIconfig = null;
        return;    
    } else {
        
        // do the main job
        openArchitecture(map, m_GUIconfig);
    } // else
    
    map = null;
    m_GUIconfig = null;
    return;
  } // actionPerformed()

 /**
 * Invoked by actionPerformed(), Desktop and ActionOpenMainFrame.actionPerformed()
 * to display the 'Architecture' frame. Noop if the architecture main frame is
 * already displayed in the GUI, but it may be opened in the explorer without main frame,
 * and then, this method creates the main frame. If it is not opened in the explorer,
 * the view is displayed in it and the main frame is created.
 * @param _acsMap of the ACS to display; the name is the key and the acs is the value
 * @param _m_GUIconfig is the map from the GUI saving, to get the frame disposition.
 *          May be null.
 * @return the 'Architecture' view in all cases
 */
 public static synchronized ViewInBase openArchitecture(Map _acsMap, Map _m_GUIconfig) {
    // variables
    Map             archi_config = null;
    ViewInBase      view = null;
    GraphicView     gview;
    JInternalFrame  frame;
    boolean         inExplorer = false;
    

                        // for testing and tracking
                    //System.out.println("start of ActionOpenArchitecture.openArchitecture()");
    
    if (InternalFrameModel.getMainFramesOfBaseObjects().get(ARCHI_VIEW_NAME) != null)
        return (ViewInBase)InternalFrameModel.getBaseObjectsInMainFrames().get(ARCHI_VIEW_NAME);
    
    try {
        
        if (ViewInBase_Facade.isOpenView(ARCHI_VIEW_NAME)) {
            view = (ViewInBase)ViewInBase_Facade.getArchitectureView();
            inExplorer = true;
        }
        else {        
            // get a new Architecture view from the open ACS
            if (ViewInBase_Facade.knowsView(ARCHI_VIEW_NAME)) {
                view = ViewInBase_Facade.openOrCreateView(ARCHI_VIEW_NAME, null);
                // delete the view since it may contain incorrect ACS
                ViewInBase_Facade.removeView(ARCHI_VIEW_NAME);
                // create the view from open ACS, and rewrite the view file
                view = ViewInBase_Facade.createView(ARCHI_VIEW_NAME, _acsMap);
            } // if
            // create a ViewInBase from the open ACS
            else
                view = ViewInBase_Facade.createView(ARCHI_VIEW_NAME, _acsMap);
        } // else
        
    } catch (DialogError d) { // never thrown for architecture
      DialogBox.showErrorMessage(d.toString());
      return null;
    } catch (BaseError b) {
      DialogBox.showErrorMessage(b.toString());
      return null;
    } catch (CreateError b) {
      DialogBox.showErrorMessage(b.toString());
      return null;
    }

                        // for testing and tracking
                    //System.out.println("middle of ActionOpenArchitecture.openArchitecture()");

    try {
        
                        // for testing and tracking
                    //Name key = null;
                    //System.out.println("in ActionOpenArchitecture.openArchitecture(), "+ 
                      //"the objects in the view are:");
                    //for (Iterator ite = view.getEorM_BaseObjects().keySet().iterator() ; ite.hasNext() ; ) {
                      // key = (Name) ite.next();
                      // System.out.println("------- "+key);
                    //} // for ite          
        
        if (_m_GUIconfig != null && _m_GUIconfig.get(new NameImpl("<Last GUI>")) != null) {
           archi_config = (Map)((Map)_m_GUIconfig.get(new NameImpl("<Last GUI>"))).get(
               (new NameImpl("<GraphicView for>")).extended(ARCHI_VIEW_NAME) );
        } // if
        if (_m_GUIconfig != null && archi_config == null &&
                _m_GUIconfig.get(new NameImpl("<Previous GUI>")) != null)
           archi_config = (Map)((Map)_m_GUIconfig.get(new NameImpl("<Previous GUI>"))).get(
               (new NameImpl("<GraphicView for>")).extended(ARCHI_VIEW_NAME) );

        ViewModel.createViewRightsMediator(view);
        // archi_config may be null
        gview = new GraphicView(view, archi_config);
        gview.setBackground(InternalFrame.NON_EDITABLECOLOR);

        // register the frame in InternalFrameModel and set the disposition
        // from the GUI saving
        frame = InternalFrame.showArchitectureFrame(gview, view);
        
    } catch (ProcessError p) {
      p.printStackTrace();
      return null;
    } 
    
    // for the frame disposition
    Rectangle bounds = null;
    if (_m_GUIconfig != null && _m_GUIconfig.get(new NameImpl("<Last GUI>")) != null) {
        bounds = (Rectangle)((Map)_m_GUIconfig.get(new NameImpl("<Last GUI>"))).
                get( ARCHI_VIEW_NAME );
    } // if
    if (_m_GUIconfig != null && bounds == null &&
            _m_GUIconfig.get(new NameImpl("<Previous GUI>")) != null) {
        bounds = (Rectangle)((Map)_m_GUIconfig.get(new NameImpl("<Previous GUI>"))).
                get( ARCHI_VIEW_NAME );
    } // if
    if (bounds != null) frame.setBounds(bounds);
    bounds = null;
    
    // create the view GraphicViewBaseListenerImpl,
    // and register the graphic view
    ViewModel.createGraphicViewBaseListener(view, gview);
    
     // create the ViewTreeBaseListenerImpl and update the explorer
    if (!inExplorer) {
    
                    //System.out.println("in ActionOpenArchitecture.openArchitecture(), "+ 
                      //"before the call to ViewTree.createViewExplorer() ");
                    
        ViewTree.createViewExplorer(view);
    }    

                        // for testing and tracking
                    //System.out.println("end of ActionOpenArchitecture.openArchitecture()");
    
    gview = null;

    return view;
  } // openArchitecture()   
 
} // class ActionOpenArchitecture

