//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.AbstractAction;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import java.awt.event.ActionEvent;

import ARoad0.Pattern.*;
import ARoad0.ErrW.ProcessError;
import ARoad0.Gui1.*;
import ARoad0.Gui2.TreeManager;
import ARoad0.gWork.ACS_Facade;
import ARoad0.gWork.ViewInBase_Facade;

import ARoad0.gBaseInterface.ACS;
import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.Name;
import ARoad0.gBase.BaseUtilityImpl;
import ARoad0.gBase.ACSFactoryImpl;
import ARoad0.ErrB.BaseError;

/**
 * An action that opens the beamer if necessary, listens the selection
 * of a base object in the explorer, opens a JInternalFrame to display
 * the base object properties, and edit them in the beamer.
 * Manages updates in the beamer from one source:<P>
 * - when in the explorer, a base object is selected
 * @see ARoad0.gDMak.ActionUtilities
 * @version 0.7.0
 * @since 0.5.1
 */
public class ActionOpenBeamer extends AbstractAction
  implements Singleton, TreeSelectionListener {

  /* the only instance of the class, following singleton pattern */
  private static ActionOpenBeamer   instance__ = new ActionOpenBeamer();

  /**
  * with this private constructor, the compiler won't generate
 * a default public constructor
 */
 private ActionOpenBeamer() {
    super("Beamer");
  }

  /**
   * constructor()
   * @return ActionOpenBeamer
   */
  public static synchronized ActionOpenBeamer getInstance(){
    return instance__;
  } // getInstance()

  /**
   * Manages opening commands from one source:
   * - from 'Open Beamer' in the main menu,
   *    when in the explorer tree, a node is selected.
  * @param _ev ActionEvent produced by the click on "Open Beamer".
  */
  public void actionPerformed(ActionEvent _ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
      
    if (InternalFrameModel.withBeamer()) {
      DialogBox.showOkMessage("The beamer is already opened.");
      return;
    }

    // initialization
    Object[]            l_analyzedEvent = new Object[3];

    l_analyzedEvent = ActionUtilities.getInstance().analyzeActionEvent(_ev);

          // for testing and tracking
    //System.out.println("in ActionOpenBeamer.actionPerformed()"+
    //  "l_analyzedEvent: ");
    //for (int i = 0 ; i < l_analyzedEvent.length ; i++) {
    //  System.out.println(l_analyzedEvent[i].toString());
    //}

    if (l_analyzedEvent[0] == null) {
      DialogBox.showOkMessage("No selection or no-beamer selection.");

      l_analyzedEvent = null;
      return;
    }

    openSelection(l_analyzedEvent);

  } // actionPerformed()

  /**
   * Manages the displaying of a base object in the beamer when,
   * in the explorer tree, a node is selected.
   * No action if the beamer is closed, or if a PropertyEditor
   * or BaseObjectHandler listens selections in the explorer.
   * Based on TreeManager.analyzeSelectedNodeInExplorer()
   * @param _ev TreeSelectionEvent
   */
   public void valueChanged(TreeSelectionEvent _ev) {
    // no action if the beamer is closed
    if (!InternalFrameModel.withBeamer()) return;
    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;

    // initialization
    Object[]               l_analyzedEvent = null;

    if (_ev.getNewLeadSelectionPath() != null)
      l_analyzedEvent = TreeManager.analyzeSelectedNodeInExplorer();
    else {
                  // for testing and tracking
            //System.out.println("in ActionOpenBeamer.valueChanged(), the selection path is null");
            
      return;
    }

    // for testing and tracking
    //System.out.println("source of the TreeSelectionEvent: "+
    //  ((javax.swing.JTree)_ev.getSource()).getName());

    if (l_analyzedEvent[0] == null || l_analyzedEvent[0].equals("")) {
        
                                  // for testing and tracking
                            //System.out.println("in ActionOpenBeamer.valueChanged(), "+
                              //      "the detected type is null");
        
      l_analyzedEvent = null;
      return;
    }

                                  // for testing and tracking
                             //System.out.println("in ActionOpenBeamer.valueChanged(), "+
                               // "2 first values in l_analyzedEvent: ");
                             //for (int i = 0 ; i < 2; i++) {
                               // System.out.println(l_analyzedEvent[i].toString());
                             //}
            
    // processes all the cases
    openSelection(l_analyzedEvent);
    l_analyzedEvent = null;
  } // valueChanged(TreeSelectionEvent)

  /**
  * Opens the selected base object in the beamer, and opens the beamer
  * before if it is closed. Called by valueChanged().
  * @param _l_analyzedEvent is an Object[] of three items:
  *   0- String type of the selected base object
  *     "acs", "resource", "actor", "ec_resource",
  *     "groupid", "userid", "aclentry", "view", "metaright", "acsright",
  *     "virtualfolder", "privilege" - may be null
  *   1- ImmutableName name of the base object - may be null
  *   2- JInternalFrame open main frame of the base object - may be null
  */
  static private void openSelection(Object[] _l_analyzedEvent) {

    BaseObject objectInBeamer = null;
    int        nameSize = -1;
    int        index = 0;
    ACS        acs = null;
    Name       name = null;

    // selects the ACS
    if (((String)_l_analyzedEvent[0]).equals("acs")) {
      objectInBeamer = (BaseObject) ACS_Facade.getOpenACS().get(_l_analyzedEvent[1]);
    } // if (((String)_l_analyzedEvent[0]).equals("acs"))

    // selects from the base
    else if (((String)_l_analyzedEvent[0]).equals("metaright") ) {
        try {
          objectInBeamer = ACS_Facade.getEorBaseObject(
            (ImmutableName)_l_analyzedEvent[1]);
        } catch (BaseError b) {
          throw new InternalError("In ActionOpenBeamer.openSelection()"+
            ", for "+((ImmutableName)_l_analyzedEvent[1]).toString()+
            ", there is the error:"+b.toString());
        }
    } // if (((String)_l_analyzedEvent[0]).equals("metaright"))

    // selects the resource through its ACS.
    // This is faster than ACS_Facade.getEorBaseObject() in large bases.
    else if (((String)_l_analyzedEvent[0]).equals("actor") ||
      ((String)_l_analyzedEvent[0]).equals("resource") ||
      ((String)_l_analyzedEvent[0]).equals("ec_resource") ) {

        nameSize = ((ImmutableName)_l_analyzedEvent[1]).noOfComponents();
        name = ((ImmutableName)_l_analyzedEvent[1]).subname(0, 1);

        while (name.noOfComponents() < nameSize) {
            index++;
            name.extended( ((ImmutableName)_l_analyzedEvent[1]).component(index) );

            if (ACS_Facade.isOpenACS(name)) {
                try {
                    acs = ACS_Facade.openOrGetACS(name);
                    objectInBeamer = (BaseObject) acs.getEorM_Resources().
                        get( (ImmutableName)_l_analyzedEvent[1]);
                } catch (BaseError b) {}

                // objectInBeamer may be null if the right ACS is a child of 'acs'
                // and then, the while loop has to continue
                if (objectInBeamer != null) break;
                //else {

                          // for testing and tracking
                       //System.out.println("ActionOpenBeamer.openSelection() on "+
                         //      _l_analyzedEvent[1]+
                           // ", the resource is unknown in the ACS "+name);
                //}
            } // if
            //else {

                  // for testing and tracking
               //System.out.println("ActionOpenBeamer.openSelection() on "+
                 //      _l_analyzedEvent[1]+
                   // ", but the resource is in an unknown or closed ACS: "+name);

            //} // else
        } // while

    } // if (((String)_l_analyzedEvent[0]).equals("actor"))

    // selects the eligible party through its ACS
    else if (((String)_l_analyzedEvent[0]).equals("groupid") ||
      ((String)_l_analyzedEvent[0]).equals("userid") ) {

        nameSize = ((ImmutableName)_l_analyzedEvent[1]).noOfComponents();
        name = ((ImmutableName)_l_analyzedEvent[1]).subname(0, 1);

        while (name.noOfComponents() < nameSize) {
            index++;
            name.extended( ((ImmutableName)_l_analyzedEvent[1]).component(index) );

            if (ACS_Facade.isOpenACS(name)) {
                try {
                    acs = ACS_Facade.openOrGetACS(name);
                    objectInBeamer = (BaseObject) acs.getEorM_EligibleParties().
                        get( (ImmutableName)_l_analyzedEvent[1]);
                } catch (BaseError b) {}
                // objectInBeamer may be null if the right ACS is a child of 'acs'
                if (objectInBeamer != null) break;
            } // if
        } // while
        
    } // if (((String)_l_analyzedEvent[0]).equals("groupid"))

    // selects the virtual folder through its ACS
    else if (((String)_l_analyzedEvent[0]).equals("virtualfolder")  ) {

        nameSize = ((ImmutableName)_l_analyzedEvent[1]).noOfComponents();
        name = ((ImmutableName)_l_analyzedEvent[1]).subname(0, 1);

        while (name.noOfComponents() < nameSize) {
            index++;
            name.extended( ((ImmutableName)_l_analyzedEvent[1]).component(index) );

            if (ACS_Facade.isOpenACS(name)) {
                try {
                    acs = ACS_Facade.openOrGetACS(name);
                    objectInBeamer = (BaseObject) acs.getOneVirtualFolder(
                        (ImmutableName)_l_analyzedEvent[1]);
                } catch (BaseError b) {}
                // objectInBeamer may be null if the right ACS is a child of 'acs'
                if (objectInBeamer != null) break;
            } // if
        } // while
        
    } // if (((String)_l_analyzedEvent[0]).equals("virtualfolder"))

    // selects the AclEntry through its ACS
    else if (((String)_l_analyzedEvent[0]).equals("aclentry") ) {

      try {
        objectInBeamer = ACS_Facade.getAclEntry((ImmutableName)_l_analyzedEvent[1]);
      } catch (BaseError b) {
          throw new InternalError("In ActionOpenBeamer.openSelection()"+
            ", for "+((ImmutableName)_l_analyzedEvent[1]).toString()+
            ", the error is:"+(String)b.getLastException());
      } // catch

                  // for testing and tracking
               //System.out.println("in ActionOpenBeamer.openSelection(), with 'aclentry' "+
                 //   "the ACL is: "+objectInBeamer+" from the _l_analyzedEvent[1]: "+
                   // _l_analyzedEvent[1]);

    } // if (((String)_l_analyzedEvent[0]).equals("aclentry"))

    // selects the Privilege through its ACS
    else if (((String)_l_analyzedEvent[0]).equals("privilege") ) {

      try {
        objectInBeamer = ACS_Facade.getPrivilege((ImmutableName)_l_analyzedEvent[1]);
      } catch (BaseError b) {
          throw new InternalError("In ActionOpenBeamer.openSelection()"+
            ", for "+((ImmutableName)_l_analyzedEvent[1]).toString()+
            ", there is the error:"+(String)b.getLastException());
      } // catch

    } // if (((String)_l_analyzedEvent[0]).equals("privilege"))

    // selects the view
    else if (((String)_l_analyzedEvent[0]).equals("view") ) {
      if (ActionOpenArchitecture.ARCHI_VIEW_NAME.equals(_l_analyzedEvent[1]))
          objectInBeamer = ViewInBase_Facade.getArchitectureView();
      else
          objectInBeamer = (BaseObject) ViewInBase_Facade.getAllStandardOpenViews().
                  get(_l_analyzedEvent[1]);
    } // if (((String)_l_analyzedEvent[0]).equals("view"))

    // selects from the base or from ActionNewACSyst
    else if (((String)_l_analyzedEvent[0]).equals("acsright")  ) {
        try {
 
                  // for testing and tracking
                //System.out.println("in ActionOpenBeamer.openSelection(), with 'acsright' "+
                  //  "in ActionNewACSyst the factory is: "+ActionUtilities.getInstance().getACSFactory());

           ACSFactoryImpl factory = ActionUtilities.getInstance().getACSFactory(); 
           // there, ACS_Facade cannont request to BaseManagerImpl to find the ACS
           // which owns this right, since the ACS does not already exist
           if (factory != null && factory.getName().equals(
                   BaseUtilityImpl.getAcsNameFromName((ImmutableName)_l_analyzedEvent[1])))
                        objectInBeamer = factory.
                                getOneRight((ImmutableName)_l_analyzedEvent[1]) ;
           else objectInBeamer = ACS_Facade.getEorBaseObject(
                        (ImmutableName)_l_analyzedEvent[1]);
           factory = null;
          
        } catch (BaseError b) {
          throw new InternalError("In ActionOpenBeamer.openSelection()"+
            ", for "+((ImmutableName)_l_analyzedEvent[1]).toString()+
            ", there is the error:"+b.toString());
        }

    } // if (((String)_l_analyzedEvent[0]).equals("acsright"))
    
    // opens or updates the beamer
    try {
                            //System.out.println("in ActionOpenBeamer.openSelection(), "+
                              //  "the object to open is: "+objectInBeamer);
        
        if (objectInBeamer != null)
              InternalFrame.openBeamer(
                BeamerManager.getInstance().updateBeamerPane(objectInBeamer));
    } catch (ProcessError p) {
      DialogBox.showAlarmMessage("The beamer can't detect"+
        " the object to display.");
    }

    objectInBeamer = null;
    acs = null;
    name = null;
  } // openSelection()


} // class ActionOpenBeamer