//Title:        Access Road
//Version:      0.7.0
//Copyright:    2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.event.*;
import java.awt.*;
import java.beans.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.net.MalformedURLException;
import java.lang.reflect.InvocationTargetException;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.Gui2.*;
import ARoad0.ErrW.*;
import ARoad0.ErrB.BaseError;
import ARoad0.ErrB.CreateError;
import ARoad0.gWork.ACS_Facade;

import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.Name;
import ARoad0.gBaseInterface.ImmutableNameISTree;

import ARoad0.gBase.BaseManagerImpl;
import ARoad0.gBase.NameImpl;
import ARoad0.gBase.BaseUtilityImpl;
import ARoad0.gBase.NameTreeImpl;
import ARoad0.gBase.ISNodeImpl;
import ARoad0.gBase.ISFactoryUtilityImpl;

/**
 * An action that opens the main frame of the IS structure for
 * all the information systems. The tree of information systems with their nodes
 * is displayed in a dedicated window where the properties of the selected node,
 * in the tree, are displayed in a tabbed pane on the top of the IS structure window.
 * This displaying works quite like the beamer, and this is why BeamerManager.setJTabbedPane()
 * delivers the proper tabbed pane for the Is structure window. Nonetheless,
 * the beamer displays BaseObjects and the IS structure displays NameISTrees.
 * A NameISTree has no BeanInfo class to set how it has to be displayed. This
 * is the responsability of the method getISnodePropertyDescriptors.
 * @version 0.7.0
 */
public class ActionOpenGlobalStructure extends AbstractAction 
        implements Singleton, TreeSelectionListener {
    
  public static final String    LOCATION_STRUCTURE_STRING = "<GlobalStructure>";
  public static final Name      LOCATION_STRUCTURE_NAME = new NameImpl(LOCATION_STRUCTURE_STRING);
  private static ImageIcon      icon__ = null;

  /**
   * Current CommonPropertyEditors in the user dialog. The key is the display name.
   */
  private Map                   m_currentPropertyEditors_ = new HashMap(40);
  /**
   * The tabbed pane of the upper panel where the properties of the node
   * are displayed.
   */
  private final JTabbedPane     tabbedPane__ = new JTabbedPane();
  
  static {
      try {
          icon__ = new ImageIcon(
                    ARoad0.Gui1.Desktop.AR_URI.resolve("Gui1"+
                    ARoad0.Gui1.Desktop.URL_SEPAR+"images"+
                    ARoad0.Gui1.Desktop.URL_SEPAR+"treeView.gif").toURL());
      } catch (MalformedURLException m) {
        m.printStackTrace();
      }
  } // static

  // the only instance of the class, following singleton pattern
  private static ActionOpenGlobalStructure   instance__ = new ActionOpenGlobalStructure();

  /**
 * With this private constructor, the compiler won't generate
 * a default public constructor.
 */
  private ActionOpenGlobalStructure() {
    super("IS Structure");
  }

  /**
  * To get an instance.
  * @return ActionOpenGlobalStructure. This is the unique mean to get
  * an instance.
  */
  public static synchronized ActionOpenGlobalStructure getInstance(){
    return instance__;
  } // getInstance()

  /**
   * Manages opening commands from one source:
   * - from 'IS Structure' in the main menu
   * Call openFrame().
  * @param _ev ActionEvent produced by the click in the menu.
  */
  public void actionPerformed(ActionEvent _ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;

      // initialization
    Map m_GUIconfig = null;

    if (ACS_Facade.getL_IS().isEmpty()) {
        DialogBox.showErrorMessage("No IS structure to display");
        return;
    } else if (InternalFrameModel.withStructure()) {
        DialogBox.showErrorMessage("The 'IS Structure' window is already open");
        return;
    }

      // set the frame and the graphic view disposition from the GUI saving
    try {
            m_GUIconfig = BaseManagerImpl.getInstance().getGUI();
    } catch (BaseError b) {
            m_GUIconfig = null;
            throw new InternalError("in ActionOpenGlobalStructure.actionPerformed(), error: "+b);
    } // catch

    openFrame(m_GUIconfig);

    m_GUIconfig = null;
  } // actionPerformed()

  /**
   * Manages the displaying of the properties of a node when,
   * in the IS structure tree, a new node is selected or the IS has been updated.
   * Based on TreeManager.analyzeSelectedNodeInStructure() and
   * getISnodePropertyDescriptors().
   * Called by the IS structure tree and by ISTreeBaseListenerImpl when
   * the selected node properties may have changed.
   * Calls TreeManager.analyzeSelectedNodeInStructure() and getTrueUserObjectNameInStructureTree(treeNode),
   * the ISNodeImpl constructor, BeamerManager.setJTabbedPane(), InternalFrameModel.getStructureFrame().
   * @param _ev the event. No use of its properties.
   */
   public void valueChanged(TreeSelectionEvent _ev) {
    // no action if the global structure is closed
    if (!InternalFrameModel.withStructure()) return;
    
    // action if the explorer is currently used for an object selection
    // or if there is an user dialog currently, so:
    //if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
    
    DefaultMutableTreeNode treeNode = TreeManager.analyzeSelectedNodeInStructure();

    if (treeNode == null) return;

    ImmutableName   name = TreeManager.getTrueUserObjectNameInStructureTree(treeNode);
    ISNodeImpl      node = null;
    int             size = name.toString().length();
    ImmutableName   iSname = null;

    finalizePropertyEditors();

    try {
        iSname = BaseManagerImpl.getInstance().getISofName(name);

        if (name != null && !name.equals(TreeManager.STRUCTURE_ROOT_NAME) &&
                iSname != null) {
            
            node = new ISNodeImpl(
                    BaseManagerImpl.getInstance().getISTree(iSname),
                    name);

            if (node == null)
                throw new InternalError("ActionOpenGlobalStructure.valueChanged(): node null");

            BeamerManager.setJTabbedPane(
              node,
              getISnodePropertyDescriptors(name),
              tabbedPane__,
              m_currentPropertyEditors_,
              null );
        } // if
        else if (name != null && name.equals(TreeManager.STRUCTURE_ROOT_NAME) ) {

            node = new ISNodeImpl(
                    null,
                    name);

            BeamerManager.setJTabbedPane(
              node,
              getISnodePropertyDescriptors(name),
              tabbedPane__,
              m_currentPropertyEditors_,
              null );
        } // if

    }
    catch (ProcessError p) {
        throw new InternalError("ActionOpenGlobalStructure.valueChanged(): "+
                p.toString());
    }    
    catch (CreateError c) {
        throw new InternalError("ActionOpenGlobalStructure.valueChanged(): "+
                c.toString());
    }

    tabbedPane__.repaint();

    if (size < 35)
        InternalFrameModel.getStructureFrame().setTitle(
                ISFactoryUtilityImpl.STRUCTURE_ROOT_STRING+": "+name.toString());
    else
        InternalFrameModel.getStructureFrame().setTitle(
                ISFactoryUtilityImpl.STRUCTURE_ROOT_STRING+": "+name.toString().substring(size - 34));
    
    name = null;
    node = null;
    iSname = null;
  } // valueChanged(TreeSelectionEvent)

  /**
  * Opens the IS structure frame. Called by actionPerformed().
  * Restores the frame location. The main work is to build up the structure tree
  * and the tabbed pane, and it is done in a worker thread.
  * An user message requests to wait for the end of the processing.
  * @param _m_GUIconfig is the GUI configuration map. May be null.
  */
  public synchronized void openFrame(Map _m_GUIconfig) {

    Map             m_Config = null;
    Map             m_previousConfig = null;

    if (_m_GUIconfig != null) {
        m_Config = (Map) _m_GUIconfig.get(new NameImpl("<Last GUI>"));
        m_previousConfig = (Map) _m_GUIconfig.get(new NameImpl("<Previous GUI>"));
    }

    // final values
    final Map           m_config = m_Config;
    final Map           m_previousconfig = m_previousConfig;
    final TreeSelectionListener thisInstance = this;

    // calls in a worker thread
    ViewModel.getViewsThreadsExecutor().execute(new Runnable() {
        @Override
        public void run() {

                                    //System.out.println("call to "+
                                    //"ActionOpenGlobalStructure.openFrame() : ");

            // BaseManagerImpl.getInstance() updates the internal variables
            Map                       m_iSTree = BaseManagerImpl.getInstance().getM_ISTrees();
            ImmutableNameISTree       iSTree = null;
            ISNodeImpl                node = null;

            try {
                if (!m_iSTree.isEmpty()) {
                    iSTree = (ImmutableNameISTree) m_iSTree.values().iterator().next();

                    // first IS in the structure
                    node = new ISNodeImpl(iSTree, iSTree.getRoot());
                } // if
                else {
                    m_iSTree = null;
                    return;
                }
            }
            catch (CreateError c) {
                throw new InternalError("ActionOpenGlobalStructure.openFrame(): "+c.toString());
            }

            final JPanel              panel = new JPanel();
            final JInternalFrame      mesframe = new JInternalFrame(
                    "The structure is opening...", false, true, false , false);

            try {
                SwingUtilities.invokeAndWait(new Runnable() {
                    public void run(){
                        InternalFrame.prepareInformationFrame(mesframe, 300, 50);
                        mesframe.setVisible(true);
                    } // run
                }); // invokeAndWait()

                // to create the IS structure tree in the worker thread if it is huge.
                // creates m_currentPropertyEditors_.
                BeamerManager.setJTabbedPane(
                  node,
                  getISnodePropertyDescriptors(iSTree.getRoot()),
                  tabbedPane__,
                  m_currentPropertyEditors_,
                  null);

                panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

                final JTree     tree = TreeManager.createStructureTree();

                JScrollPane     scrollPane;
                Dimension       dimension = new Dimension(370, 710);

                tree.addTreeSelectionListener(thisInstance);

                // adds the property editors at the top of the panel
                tabbedPane__.setPreferredSize(new Dimension(300, 200));
                //tabbedPane__.setMaximumSize(new Dimension(300, 300));

                panel.add( tabbedPane__ );

                // adds the IS nameTrees
                scrollPane = new JScrollPane(tree);

                // puts the scrollPane in result
                JPanel panel2 = new JPanel();
                panel2.setLayout(new BorderLayout());
                //panel.setSize(
                  //new Dimension(GraphicNode.VIEW_INTAB_SIZE));
                panel2.setMinimumSize(
                  new Dimension(GraphicNode.VIEW_INTAB_SIZE));
                panel2.setPreferredSize(
                  new Dimension(GraphicNode.VIEW_INTAB_SIZE));
                panel2.add(scrollPane);
                //scrollPane.getVerticalScrollBar().setValue(
                  //scrollPane.getVerticalScrollBar().getMinimum() );  // does not work
                panel.add(panel2);

                panel.setBackground(Color.white);
                panel.setPreferredSize(dimension);
                panel.setMaximumSize(dimension);
                panel.setMinimumSize(dimension);

                scrollPane = null;
                dimension = null;

                // in the Event Dispatch Thread
                SwingUtilities.invokeLater(new Runnable(){
                    public void run(){

                                // for testing and tracking
                                //System.out.println("in ActionOpenGlobalStructure.openFrame(), "+
                                  //"starts the work in the event dispatch thread");

                        try {
                            // registers the frame in InternalFrameModel and sets the disposition
                            // from the GUI saving
                            JInternalFrame frame = InternalFrame.showStructureFrame(panel, tree);

                            Rectangle bounds = null;
                            if (m_config != null ) {
                                bounds = (Rectangle)m_config.get(LOCATION_STRUCTURE_NAME);
                            }
                            if (bounds == null && m_previousconfig != null ) {
                                bounds = (Rectangle)m_previousconfig.get(LOCATION_STRUCTURE_NAME);
                            }

                            if (bounds != null) frame.setBounds(bounds);

                            // expands the last node - does not work
                            tree.expandPath(tree.getPathForRow(tree.getRowCount()- 1));
                            frame = null;
                        } catch (ProcessError p) {
                            throw new InternalError("in ActionOpenGlobalStructure.openFrame(): "+p);
                        }

                        // closes the information message
                        try {
                            mesframe.setClosed(true);
                        } catch (PropertyVetoException p) {
                            System.out.println("in ActionOpenGlobalStructure.openFrame(),"+
                              " the frame closing fires a PropertyVetoException: "+
                            p.toString());
                        }

                                // for testing and tracking
                                //System.out.println("in ActionOpenGlobalStructure.openFrame(), "+
                                  //"end of the work in the event dispatch thread");

                    } // run
                }); // invokeLater()

                        // for testing and tracking
                        //System.out.println("in ActionOpenGlobalStructure.openFrame(): "+
                          //      "end of run().");
            } catch (ProcessError p) {
              DialogBox.showErrorMessage(p.toString());
              return;
            } catch (InterruptedException i) {
                    // for testing and tracking
                    System.out.println("in ActionOpenGlobalStructures.openFrame(), "+
                      "one of the open view is not created: "+i.getMessage());
            } catch (InvocationTargetException n) {
                    // for testing and tracking
                    System.out.println("in ActionOpenGlobalStructure.openFrame()"+
                      ": "+n.getMessage());
            }
            
            m_iSTree = null;
            iSTree = null;
            node = null;
        } // run()

    }); // new Thread

    m_Config = null;
    m_previousConfig = null;
  } // openFrame()

  /**
  * Finalizes the current property editors and removes the children of tabbedPane__.
  * Called by ActionClose() and valueChanged().
  */
  void finalizePropertyEditors() {
     synchronized (m_currentPropertyEditors_) {
        if (!m_currentPropertyEditors_.isEmpty()) {
          for (Iterator ite = m_currentPropertyEditors_.values().iterator() ; ite.hasNext() ;) {
            ((CommonPropertyEditor)ite.next()).finalizeForProcess();
          }
        }
        m_currentPropertyEditors_.clear();
     }
     tabbedPane__.removeAll();
  } // finalizePropertyEditors()


 // ------------ Private methods --------------------------------------

 /**
 * Constructs the property descriptors for the currently selected node
 * in the IS structure. The beamer displays BaseObjects and the IS structure displays NameISTrees.
 * A NameISTree has no BeanInfo class to set how it has to be displayed. This
 * is the responsability of this method, that calls BaseUtilityImpl.getBeanInfoNameForISnode()
 * to get the name of the right BeanInfo class.
 * The root TreeManager.STRUCTURE_ROOT_NAME is used to display the list of ACS addons.
 * Called by valueChanged() and openFrame().
 * @param _name of a node in the IS nametrees. 
 * @return the descriptors
 */
 private static synchronized PropertyDescriptor[] getISnodePropertyDescriptors(
         ImmutableName _name) {
    // variables
     PropertyDescriptor[] result = null;
    BeanInfoPattern   info = null; // no-sens init
    Class             beanInfoClass;
    String            nodeType = null;

                // for testing and tracking
             //System.out.println("in ActionOpenGlobalStructure.getISnodePropertyDescriptors(), "+
              //"name is: "+_name);
             //System.out.println("in ActionOpenGlobalStructure.getISnodePropertyDescriptors(), "+
              //"BaseManagerImpl.getInstance().getISofName(_name): "+
              //BaseManagerImpl.getInstance().getISofName(_name));
             //System.out.println("in ActionOpenGlobalStructure.getISnodePropertyDescriptors(), "+
              //"name is: "+_name);

    try {
      if (!_name.equals(TreeManager.STRUCTURE_ROOT_NAME)) {
          nodeType = ((NameTreeImpl) BaseManagerImpl.getInstance().getM_ISTrees().get(
                BaseManagerImpl.getInstance().getISofName(_name) )).getType(_name);

          beanInfoClass = Class.forName(
                  BaseUtilityImpl.getBeanInfoNameForISnode(nodeType));
      }
      else {
          beanInfoClass = Class.forName(
                  BaseUtilityImpl.getBeanInfoNameForISnode(
                        ISFactoryUtilityImpl.STRUCTURE_ROOT_STRING));
      }
      
      info = (BeanInfoPattern) beanInfoClass.newInstance();

    } catch (BaseError b) {
      new ProcessError("from ActionOpenGlobalStructure.getISnodePropertyDescriptors: "+
              b.toString());
      throw new InternalError("from ActionOpenGlobalStructure.getISnodePropertyDescriptors(): "+
              b.toString());
    } catch (Exception l) {
      new ProcessError("from ActionOpenGlobalStructure.getISnodePropertyDescriptorso: "+
              l.toString());
      throw new InternalError("from ActionOpenGlobalStructure.getISnodePropertyDescriptors(): "+
              l.toString());
    } catch (Error e) {
      new ProcessError("from ActionOpenGlobalStructure.getISnodePropertyDescriptors: "+
              e.toString());
      throw new InternalError("from ActionOpenGlobalStructure.getISnodePropertyDescriptors(): "+
              e.toString());
    }

    beanInfoClass = null;
    nodeType = null;
    result = info.getPropertyDescriptors();
    info = null;

    return result;
  } // getISnodePropertyDescriptors()


} // class ActionOpenGlobalStructure



