//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.beans.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.net.MalformedURLException;
import java.lang.reflect.InvocationTargetException;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.Gui2.*;
import ARoad0.ErrB.BaseError;
import ARoad0.ErrW.*;
import ARoad0.gWork.ViewInBase_Facade;

import ARoad0.gBaseInterface.ViewInBase;
import ARoad0.gBaseInterface.EPRViewInBase;
import ARoad0.gBaseInterface.NoThanViewInBase;
import ARoad0.gBase.NameImpl;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBase.BaseManagerImpl;

/**
 * An action that opens a GraphicView in a frame.
 * @version 0.7.0
 * @since 0.5.0
 */
public class ActionOpenGraphicView extends AbstractAction implements Singleton {

  /* the only instance of the class, following singleton pattern */
  private static ActionOpenGraphicView instance__ = new ActionOpenGraphicView();
  private static ImageIcon              icon__ = null;
  
  static {
      try {
          icon__ = new ImageIcon(
                    ARoad0.Gui1.Desktop.AR_URI.resolve("Gui1"+
                    ARoad0.Gui1.Desktop.URL_SEPAR+"images"+
                    ARoad0.Gui1.Desktop.URL_SEPAR+"copy.gif").toURL());
      } catch (MalformedURLException m) {
        m.printStackTrace();
      }
  } // static
  
  // variables for the user data
  private JDialog                   dialog_ = new JDialog();
  private JList                     jlist_ = new JList();


  /**
   * with this private constructor, the compiler won't generate
   * a default public constructor
   */
  private ActionOpenGraphicView() {
  super("Open View", icon__);
  }

  /**
   * Only method to get an instance.
   * @return ActionOpenGraphicView
   */
  public static synchronized ActionOpenGraphicView getInstance(){
    return instance__;
  } // getInstance()

  /**
   * @param _ev ActionEvent
   */
  public void actionPerformed(ActionEvent _ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently going on
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
      
      // variables of the dialog box
    String          select = null;
    JOptionPane     pane = new JOptionPane();
    List            list = new ArrayList();
    JScrollPane     scrollPane;

    // create a dialog box from the list of closed ACS
    // valueChanged() in jlist is NOT overiden because user must click
    // on a button in the dialog box to valid its input
    list = ViewInBase_Facade.getClosedViews();
    jlist_ = new JList(list.toArray() );

    // no work, because selection is null when the user clicks on OK
    if (list.isEmpty() ) {
      // display a message
      DialogBox.showErrorMessage("No view to open");
      return;
    }

    // show the dialog box
    dialog_ = DialogBox.showListDialog(
        "Select all the views to open", jlist_);

    // here, dialog_ waits an enter from the user,
    // then the next instructions are processed.

    pane = (JOptionPane)dialog_.getContentPane().getComponent(0);
    // no action if CLOSED, and close the dialog box
    if (pane.getValue() == null) return;
    // else, it is an Integer
    int value = ((Integer)pane.getValue()).intValue();

    // no action if CANCEL, and close the dialog box
    if (value == JOptionPane.CANCEL_OPTION) {
      //Object nullObject = new Object();
      //nullObject = null;
      //jlist_.setSelectedValue(nullObject, false);
      return;
    }
    // if OK, action and close the dialog box
    if (value == JOptionPane.OK_OPTION) openView( jlist_.getSelectedValues() );

    pane = null;
  } // actionPerformed()

  /**
  * Called by actionPerformed(ActionEvent) to open the selected EPRviews,
   * to display their resources in the explorer and display the view in a frame.
   * Call openViewInGui().
   * @param _l_names is the list of view names to open
   */
  public void openView(Object[] _l_names) {
      // variables
      EPRViewInBase   view = null;
      Map m_GUIconfig = null;      

      if (_l_names == null) {
        // display a warning mesage if the selection is empty
        DialogBox.showErrorMessage(
          "Please, select at least one name to open a view.");
        return;
      }

      // get the frame view and the graphic view disposition from the GUI saving
      try {
            m_GUIconfig = BaseManagerImpl.getInstance().getGUI();
      } catch (BaseError b) {
            throw new InternalError("in ActionOpenGraphicView.openView(), error: "+b);
      }
      
      // for each selected view name
      for (int i = 0; i < _l_names.length ; i++) {
                // open the view
                // ViewTreeBaseListener not still created, then no fired event

                  // for testing and tracking
                  //System.out.println("End of ActionOpenACSyst.openView(), selection"+
                  //  i+": has the class "+selection[i].getClass()+
                  //  " and the value: "+(ImmutableName)selection[i]);

        try {
          if (!ViewInBase_Facade.isNoThanViewName((ImmutableName)_l_names[i]))
              view = ViewInBase_Facade.openOrCreateEPRView(
                    (ImmutableName)_l_names[i], null, null);
          else
              view = ViewInBase_Facade.openOrCreateNoThanView(
                      (ImmutableName)_l_names[i], null);
        } catch (DialogError d) {
              // the view contains an item from closed or deleted acs 
              d.printStackTrace();
              if (d.getMessage() != null)
                  DialogBox.showErrorMessage(d.getMessage());
              // there, the view has not been created in the base
              m_GUIconfig = null;
              return;
        }

        // display the view in a window and in the explorer
        openViewInGui(view, true, m_GUIconfig);

      } // for i

                  // for testing and tracking
                  //System.out.println("End of ActionOpenGraphicView.openView()");
      
    m_GUIconfig = null;
    view = null;
  } // openView()

 /**
 * Invoked by openView(), ActionNewView, ActionOpenMainFrame, Desktop, ActionCopy
 * to create a GraphicEPRView or a GraphicNoThanView for a EPRViewInBase or a NoThanViewInBase,
 * to display their resources in the explorer, and to display the view in a frame or
 * in one of the two 'all views' frames.
 * No check if the view is already open in a frame or a tab.
 * This method works in the event dispatch thread, and calls a worker thread to create
 * the graphic view. The worker thread calls back the event dispatch thread
 * to display the view in a tab or in a frame.
 * @param _view to open
 * @param _createInExplorer is false if the view is already open in the explorer
 * @param _m_GUIconfig is the GUI configuration map. May be null.
 */
 public static synchronized void openViewInGui(EPRViewInBase _view,
   boolean _createInExplorer, Map _m_GUIconfig) {
    // variables
    Map view_config = null;

    if (_m_GUIconfig != null && _m_GUIconfig.get(new NameImpl("<Last GUI>")) != null) {
       view_config = (Map)((Map)_m_GUIconfig.get(new NameImpl("<Last GUI>"))).get( 
               (new NameImpl("<GraphicView for>")).extended(_view.getName()) );
    } // if
    if (_m_GUIconfig != null && view_config == null &&
            _m_GUIconfig.get(new NameImpl("<Previous GUI>")) != null) {
        view_config = (Map)((Map)_m_GUIconfig.get(new NameImpl("<Previous GUI>"))).
                get( (new NameImpl("<GraphicView for>")).extended(_view.getName()) );
    } // if

                          // for testing and tracking
                          //if (view_config == null) {
                                //System.out.println("in ActionOpenGraphicView.openViewInGui(), "+
                                  //"view_config is null for "+_view.getName());


                                // for testing and tracking
                                //System.out.println("In ActionOpenGraphicView.openViewInGui(), m_previousConfig keys:");
                                //if ( _m_GUIconfig.get(new NameImpl("<Previous GUI>") ) != null) {
                                    //for (Iterator ite = ((Map)_m_GUIconfig.get(new NameImpl("<Previous GUI>"))).
                                      //      entrySet().iterator(); ite.hasNext(); ) {
                                        //Map.Entry entry = (Map.Entry) ite.next();
                                        //System.out.println("----------- "+entry.getKey());
                                    //} // for ite
                                //} // if
                                //else
                                    //System.out.println("----------- "+
                                      //      "((Map)_m_GUIconfig.get(new NameImpl('<Previous GUI>')) is null");


                                // for testing and tracking
                                //System.out.println("In ActionOpenGraphicView.openViewInGui(), _m_GUIconfig keys:");
                                //for (Iterator ite = _m_GUIconfig.entrySet().iterator(); ite.hasNext(); ) {
                                  //  Map.Entry entry = (Map.Entry) ite.next();
                                  //  System.out.println("----------- "+entry.getKey());
                                //} // for ite
                          //} // if

    final Map           config = view_config;
    final EPRViewInBase view = _view;
    final boolean       createInExplorer = _createInExplorer;
    final Map           m_GUIconfig = _m_GUIconfig;

    ViewModel.getViewsThreadsExecutor().execute(new Runnable() {
        @Override
        public void run() {
            // call in a worker thread
            GraphicView gview = null;

            final JInternalFrame mesframe = new JInternalFrame(
                    "The view "+view.getName()+" is opening...", false, true, false , false);

            try {
                // in the Event Dispatch Thread
              SwingUtilities.invokeAndWait(new Runnable() {
                    public void run(){
                        InternalFrame.prepareInformationFrame(mesframe, 500, 50);
                        mesframe.setVisible(true);
                    } // run
              }); // invokeAndWait()

              // in a worker thread
              ViewModel.createViewRightsMediator(view);

                // config may be null
              if (!ViewInBase_Facade.isNoThanViewName(view.getName()))
                  gview = new GraphicEPRView(view, config);
              else
                  gview = new GraphicNoThanView((NoThanViewInBase)view, config);

            } catch (ProcessError p) {
                // not in the Event Dispatch Thread
              DialogBox.showErrorMessage(p.toString());
              // create the ViewTreeBaseListenerImp and update the explorer
              if (createInExplorer) ViewTree.createViewExplorer(view);
            } catch (InterruptedException i) {
                // not in the Event Dispatch Thread
              DialogBox.showErrorMessage(i.toString());
              // create the ViewTreeBaseListenerImp and update the explorer
              if (createInExplorer) ViewTree.createViewExplorer(view);
            } catch (InvocationTargetException n) {
                    // for testing and tracking
                    System.out.println("in ActionOpenAllFullViews.openViewInGui() "+
                      ": "+n.getMessage());
            }

            final GraphicView gview2 = gview;

            // in the Event Dispatch Thread
            SwingUtilities.invokeLater(new Runnable(){
                public void run(){
                    JInternalFrame frame = null;
                    
                    // display the view in a tab or in a frame
                    try {
                        if (InternalFrameModel.withAllNoThanViews() &&
                                ViewInBase_Facade.isNoThanViewName(view.getName())) {

                                          // for testing and tracking
                                          //System.out.println("in ActionOpenGraphicView.openViewInGui(), "+
                                            //      "call for 'All NoThan views' for "+_view.getName());

                            // create the view GraphicEPRViewBaseListenerImpl,
                            // and register the graphic view
                            ViewModel.createGraphicViewBaseListener(view, gview2);
                            // register the view JPanel in InternalFrameModel
                            // work even if ActionOpenAllViews has not been called yet
                            BeamerManager.setJTabbedPane(
                                    view, // unused argument
                                    new PropertyDescriptor[] {getViewPropertyDescriptor(view.getName())},
                                    ViewModel.getAllNoThanViewsJTabbedPane(),
                                    ActionOpenAllNoThanViews.getInstance().getAllViewsPropertyEditors(),
                                    ActionClose.getInstance() );
                        }
                        else if (InternalFrameModel.withAllFullViews() &&
                                !ViewInBase_Facade.isNoThanViewName(view.getName())) {
                            // create the view GraphicEPRViewBaseListenerImpl,
                            // and register the graphic view
                            ViewModel.createGraphicViewBaseListener(view, gview2);
                            // register the view JPanel in InternalFrameModel
                            // work even if ActionOpenAllViews has not been called yet
                            BeamerManager.setJTabbedPane(
                                    view, // unused argument
                                    new PropertyDescriptor[] {getViewPropertyDescriptor(view.getName())},
                                    ViewModel.getAllFullViewsJTabbedPane(),
                                    ActionOpenAllFullViews.getInstance().getAllViewsPropertyEditors(),
                                    ActionClose.getInstance() );
                        } else {

                                          // for testing and tracking
                                          //System.out.println("in ActionOpenGraphicView.openViewInGui(), "+
                                            //      "call for creating a JInternalFrame for "+_view.getName()+
                                              //    " with the GraphicView "+gview);

                            frame = InternalFrame.showViewFrame(gview2, view);

                                            // for testing and tracking
                                            //System.out.println("in ActionOpenGraphicView.openViewInGui(), gview.getRightsRelatedBaseObjects():");
                                            //for (Iterator ite = gview.getRightsRelatedBaseObjects().iterator(); ite.hasNext(); ) {
                                              //  System.out.println("-------"+ite.next());
                                            //}


                            // create the view GraphicViewBaseListenerImpl,
                            // and register the graphic view
                            ViewModel.createGraphicViewBaseListener(view, gview2);
                            // open the 'See why' frame if necessary,
                            // set its bound, and set the bounds of 'frame' if it is required
                            restoreMainAndSeeWhyFrames(
                                    view.getName(), frame, m_GUIconfig);
                        } // else
                    } catch (ProcessError p) {
                        throw new InternalError("in ActionOpenGraphicView.openViewInGui(), "+
                                "error: "+p);
                    }
                    // close the information message
                    try {
                        mesframe.setClosed(true);
                    } catch (PropertyVetoException p) {
                        System.out.println("in ActionOpenGraphicView.openViewInGui(),"+
                          " the frame closing fires a PropertyVetoException: "+
                          p.toString());
                    }

                            // for testing and tracking
                            //System.out.println("in ActionOpenGraphicView.openViewInGu(), "+
                              //"end of the work in the event dispatch thread");

                    frame = null;
                } // run
            }); // invokeLater()

            gview = null;
        } // run()

    }); // new Thread

    // creates the ViewTreeBaseListenerImpl and updates the explorer
    if (_createInExplorer) ViewTree.createViewExplorer(_view);     

    view_config = null;
  } // openViewInGui()
  
 /**
 * Opens the 'See why' frame and set the bounds of the main and 'See why' frames,
 * if it is required in the GUI configuration. The graphic view must have been
 * registered in ViewModel. Noop if _m_GUIconfig does not contain the right data.
 * Does not restore the node locations in the GraphicView, since only the frames are handled.
 * Called by openViewInGui() in the event dispatch thread.
 * @param _name is the open ViewInBase name, after the creation of the relevant graphic view
 * @param _frame is the main frame of the view
 * @param _m_GUIconfig is the GUI configuration map. May be null. 
 */
 private static void restoreMainAndSeeWhyFrames(ImmutableName _name, 
         JInternalFrame _frame, Map _m_GUIconfig) {
    
    JInternalFrame  frame2 = null;
    Rectangle       bounds = null;
    Map             m_Config = null;
    Map             m_previousConfig = null;

    if (_m_GUIconfig != null) {
        m_Config = (Map) _m_GUIconfig.get(new NameImpl("<Last GUI>"));
        m_previousConfig = (Map) _m_GUIconfig.get(new NameImpl("<Previous GUI>"));
    }

    if (m_Config != null) {
       bounds = (Rectangle)m_Config.get(_name );
    } // if
    if (m_previousConfig != null && bounds == null) {
       bounds = (Rectangle)m_previousConfig.get( _name );
    } // if
    if (bounds != null) _frame.setBounds(bounds );

    // open and set the 'See why' frame disposition if it is in the GUI saving
    bounds = null;
    if (m_Config != null ) {
        bounds = (Rectangle)m_Config.get((new NameImpl("<See why for>")).extended(_name));
    }
    if (bounds == null && m_previousConfig != null ) {
        bounds = (Rectangle)m_previousConfig.get((new NameImpl("<See why for>")).extended(_name));
    }

    if (bounds != null) {
        frame2 = InternalFrame.showTextFrame(
                        ((GraphicView)ViewModel.getGraphicViewOfView(_name)).setWhyText(),
                        _name.toString()+ " 'See why' text",
                        ARoad0.Gui1.Desktop.AR_URI.resolve("Gui1"+
                            ARoad0.Gui1.Desktop.URL_SEPAR+"images"+
                            ARoad0.Gui1.Desktop.URL_SEPAR+"view.gif"));
        frame2.setBounds(bounds );
    } // if

    frame2 = null;
    bounds = null;
    m_Config = null;
    m_previousConfig = null;
  } // restoreMainAndSeeWhyFrames()    
 
  
 /**
 * Constructs the property descriptor for the open view in the 'Full views' or
 * 'NoMore-NoLess views' tabbed pane, where the display name is the ViewInBase name. 
 * Called by openViewInGui() and ActionOpenAllFull/NoThanViews.getAllViewsPropertyDescriptors().
 * @param _name is the open ViewInBase name
 * @return the descriptor
 */
 public static PropertyDescriptor getViewPropertyDescriptor(ImmutableName _name) {
    // variables
    PropertyDescriptor result;

    try {
          // false arguments since the true property is the GraphicView which
          // is not a BaseObject with a BeanInfo... These arguments are not
          // used by the target GraphicViewPropertEditor.
          result = new PropertyDescriptor("Name", ViewInBase.class, "getName", null);
          // allow the GraphicViewPropertEditor to get the view
          result.setDisplayName(_name.toString());
          result.setShortDescription("The view and its justification.");
          result.setBound(false);
          // to select a GraphicViewPropertEditor
          result.setValue("isView", Boolean.TRUE );

          // 'tabbedPaneName' is the name (String value) of the pane where this property
          // has to be displayed in the beamer: one tab per view, and the title 
          // is the nick name
          result.setValue("tabbedPaneName", ViewModel.getTabTitleFromViewName( _name) );
          
                  // for testing and tracking
                  //System.out.println("ActionOpenGraphicView.getViewPropertyDescriptor(): "+
                          //"for the view "+_name+" the tab title is "+
                          //ViewModel.getTabTitleFromViewName( _name));
          
    } catch (IntrospectionException e) {
      new ProcessError("In ActionOpenGraphicView.getViewPropertyDescriptor(), "+
        "IntrospectionException: "+e);
      throw new InternalError("In ActionOpenGraphicView.getViewPropertyDescriptor(), "+
        "IntrospectionException: "+e);
    } // catch
    return result;
  } // getViewPropertyDescriptor()  
 
} // class ActionOpenGraphicView


