//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.event.*;
import java.awt.*;
import java.util.Map;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.Gui2.*;
import ARoad0.ErrW.*;
import ARoad0.ErrB.BaseError;
import ARoad0.gWork.ACS_Facade;
import ARoad0.gWork.ViewInBase_Facade;

import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.ACS;
import ARoad0.gBaseInterface.EPRViewInBase;
import ARoad0.gBaseInterface.ViewInBase;
import ARoad0.gBase.BaseManagerImpl;
import ARoad0.gBase.NameImpl;

/**
 * An action that opens the main frame of an Access Control System
 * or a View selected in the main menu, if it is closed.
 * @version 0.7.0
 */
public class ActionOpenMainFrame extends AbstractAction implements Singleton {

  // the only instance of the class, following singleton pattern
  private static ActionOpenMainFrame   instance__ = new ActionOpenMainFrame();

  /**
 * with this private constructor, the compiler won't generate
 * a default public constructor.
 */
  private ActionOpenMainFrame() {
    super("View diagram or ACS tree");
  }

  /**
  * To get an instance.
  * @return ActionOpenMainFrame. This is the unique mean to get
  * an instance.
  */
  public static synchronized ActionOpenMainFrame getInstance(){
    return instance__;
  } // getInstance()

  /**
   * Manages opening commands from one source:
   * - from 'Open Main Frame' in the main menu when, in the explorer,
   *    an ACS or a view node is selected
   * Call openACSmainFrame(). Do not handle 'Architecture' view.
  * @param _ev ActionEvent produced by the click for "Open Main Frame".
  */
  public void actionPerformed(ActionEvent _ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
      
      // initialization
    Object[]            l_analyzedEvent = new Object[3];
    ViewInBase          view = null;
    JInternalFrame      frame;
    Map                 m_GUIconfig = null;    

    l_analyzedEvent = ActionUtilities.getInstance().analyzeActionEvent(_ev);

                  // for testing and tracking
            //System.out.println("in ActionOpenMainFrame.actionPerformed(),"+
              //  " _ev: "+_ev+
              //", l_analyzedEvent: ");
            //for (int i = 0 ; i < l_analyzedEvent.length ; i++) {
              //if (l_analyzedEvent[i] != null) System.out.println(l_analyzedEvent[i]);
            //}

    if (l_analyzedEvent[0] == null ) {
      DialogBox.showOkMessage("No selection or unworkable selection.");

      l_analyzedEvent = null;
      view = null;
      return;
    }
    else if (l_analyzedEvent[2] != null ) {
      DialogBox.showOkMessage("The selected node is not an ACS"+
        " or a view, or the main frame is already open.");

                    // for testing and tracking
                    //System.out.println("in ActionOpenMainFrame.actionPerformed(), "+
                      //      "the main frame is already open: "+l_analyzedEvent[2]);
      
      l_analyzedEvent = null;
      view = null;
      return;
    }

      // set the frame and the graphic view disposition from the GUI saving
    try {
            m_GUIconfig = BaseManagerImpl.getInstance().getGUI();
    } catch (BaseError b) {
            l_analyzedEvent = null;
            view = null;
            m_GUIconfig = null;        
            throw new InternalError("in ActionOpenMainFrame.actionPerformed(), error: "+b);
    } // catch
    
    // open the ACS main frame
    if (((String)l_analyzedEvent[0]).equals("acs")) {
        
      frame = openACSmainFrame((ImmutableName)l_analyzedEvent[1], m_GUIconfig);
      
    } // if (((String)l_analyzedEvent[0]).equals("acs"))
    else if (((String)l_analyzedEvent[0]).equals("view")) {

      if (ActionOpenArchitecture.ARCHI_VIEW_NAME.equals(l_analyzedEvent[1]))
          view = (ViewInBase) ViewInBase_Facade.getArchitectureView();
      else
          view = (ViewInBase) ViewInBase_Facade.getAllStandardOpenViews().
                    get((ImmutableName)l_analyzedEvent[1]);
      
      if (ViewModel.getViewRightsMediator(view) != null) {
            l_analyzedEvent = null;
            view = null;
            m_GUIconfig = null;
            DialogBox.showErrorMessage("This view window is already open.");
            return;          
      } // if  
      
      if ( ((ImmutableName)l_analyzedEvent[1]).equals(ActionOpenArchitecture.ARCHI_VIEW_NAME) ) {
            l_analyzedEvent = null;
            view = null;
            m_GUIconfig = null;
            
            Map map = ACS_Facade.getOpenACS();  
            ActionOpenArchitecture.openArchitecture(map, m_GUIconfig);
            map = null;
            return;          
      } // if 

                  // for testing and tracking
            //System.out.println("in ActionOpenMainFrame.actionPerformed(), "+
              //      "call to ActionOpenGraphicView.openViewInGui() for "+view.getName());
      
      ActionOpenGraphicView.openViewInGui((EPRViewInBase)view, false, m_GUIconfig);

    } // if (((String)l_analyzedEvent[0]).equals("view"))

    else  DialogBox.showOkMessage("The selected node is not an ACS"+
        " nor a view.");


    l_analyzedEvent = null;
    view = null;
    m_GUIconfig = null;
    frame = null;
    return;
  } // actionPerformed()


  /**
  * To open the ACS main frame. Called by actionPerformed() and Desktop.restoreLastGUI().
  * Restore the main frame location.
  * @param _name is the acs name
  * @param _m_GUIconfig is the GUI configuration map. May be null.
  * @return the acs main frame
  */
  public static synchronized JInternalFrame openACSmainFrame(ImmutableName _name,
          Map _m_GUIconfig) {
      JTree               tree = null;
      JInternalFrame      frame = null;
      ACS                 acs = (ACS)ACS_Facade.getOpenACS().get(_name);

      try {
        tree = ACSTree.createACSTree(acs);
        frame = InternalFrame.showACSTreeFrame(_name, tree, acs);
        tree.getSelectionModel().setSelectionMode(
          TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
      } catch (ProcessError p) {
          // for testing and tracking
          System.out.println("in ActionOpenMainFrame.actionPerformed(): "+p.toString());
      }

      Map               m_Config = null;
      Map               m_previousConfig = null;

      if (_m_GUIconfig != null) {
          m_Config = (Map) _m_GUIconfig.get(new NameImpl("<Last GUI>"));
          m_previousConfig = (Map) _m_GUIconfig.get(new NameImpl("<Previous GUI>"));
      }

      if (m_Config != null && m_Config.get(_name) != null)
          frame.setBounds( (Rectangle)m_Config.get(_name) );
      else if (m_previousConfig != null && m_previousConfig.get(_name) != null)
          frame.setBounds( (Rectangle)m_previousConfig.get(_name) );

      tree = null;
      acs = null;
      m_Config = null;
      m_previousConfig = null;

      return frame;
  } // openACSmainFrame()
} // class ActionOpenMainFrame


