//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import java.awt.event.ActionEvent;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.net.MalformedURLException;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.Gui2.*;
import ARoad0.ErrB.CreateError;
import ARoad0.ErrB.BaseError;
import ARoad0.ErrW.*;
import ARoad0.gWork.ACS_Facade;
import ARoad0.gWork.ViewInBase_Facade;
import ARoad0.gWork.RightsMediatorImpl;

import ARoad0.gBaseInterface.ViewInBase;
import ARoad0.gBaseInterface.SketchViewInBase;
import ARoad0.gBaseInterface.Name;
import ARoad0.gBaseInterface.BaseObject;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBase.NameImpl;

/**
 * An action that opens a GraphicSketchView for any currently-selected object in the explorer.
 * @see ARoad0.Gui2.GraphicSketchView
 */
public class ActionOpenSketcher extends AbstractAction implements 
        Singleton, TreeSelectionListener {

  /* the only instance of the class, following singleton pattern */
  private static ActionOpenSketcher instance__ = new ActionOpenSketcher(); 

  public static final String            SKETCH_VIEW_STRING = "<Sketcher>";  
  public static final Name              SKETCH_VIEW_NAME = new NameImpl(SKETCH_VIEW_STRING);
  private static ImageIcon              icon__ = null;
  
  static {
      try {
          icon__ = new ImageIcon(
                    ARoad0.Gui1.Desktop.AR_URI.resolve("Gui1"+
                    ARoad0.Gui1.Desktop.URL_SEPAR+"images"+
                    ARoad0.Gui1.Desktop.URL_SEPAR+"treeView.gif").toURL());
      } catch (MalformedURLException m) {
        m.printStackTrace();
      }
  } // static
  
  /**
   * with this private constructor, the compiler won't generate
   * a default public constructor
   */
  private ActionOpenSketcher() {
    super("Sketcher", icon__);
  }

  /**
   * Only method to get an instance.
   * @return ActionOpenSketcher
   */
  public static synchronized ActionOpenSketcher getInstance() {
    return instance__;
  } // getInstance()

  /**
   * Manages opening commands from one source:
   * - from 'Open Sketcher' in the main menu,
   *     when in the explorer tree, a node is selected.
  * @param _ev ActionEvent produced by the click on "Open Sketcher".
  */
  public void actionPerformed(ActionEvent _ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;
      
    if (InternalFrameModel.withSketcher()) {
      DialogBox.showOkMessage("The sketcher is already open.");
      return;
    }

    // initialization
    Object[] l_analyzedEvent = ActionUtilities.getInstance().analyzeActionEvent(_ev);

          // for testing and tracking
    //System.out.println("in ActionOpenBeamer.actionPerformed()"+
    //  "l_analyzedEvent: ");
    //for (int i = 0 ; i < l_analyzedEvent.length ; i++) {
    //  System.out.println(l_analyzedEvent[i].toString());
    //}

    if (l_analyzedEvent[0] == null) {
      DialogBox.showOkMessage("No selection or no sketcher-compatible selection.");

      l_analyzedEvent = null;
      return;
    }   
    
    // do the main job
    openSketcher(l_analyzedEvent);

    return;

  } // actionPerformed()

  /**
   * Manages the displaying of a base object in the sketcher when,
   * in the explorer tree, a node is selected.
   * No action if the sketcher is closed, or if a PropertyEditor
   * or BaseObjectHandler listens a selection in the explorer.
   * Based on TreeManager.analyzeSelectedNodeInExplorer()
   * @param _ev is from the explorer tree
   */
   public void valueChanged(TreeSelectionEvent _ev) {
    // no action if the sketcher is closed
    if (!InternalFrameModel.withSketcher()) return;
    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) return;

    // initialization
    Object[]               l_analyzedEvent = new Object[3];

    if (_ev.getNewLeadSelectionPath() != null)
      l_analyzedEvent = TreeManager.analyzeSelectedNodeInExplorer();
    else {
      l_analyzedEvent = null;
      return;
    }

    // for testing and tracking
    //System.out.println("source of the TreeSelectionEvent: "+
    //  ((javax.swing.JTree)_ev.getSource()).getName());

    if (l_analyzedEvent[0] == null || l_analyzedEvent[0].equals("")) {
      l_analyzedEvent = null;
      return;
    }

                              // for testing and tracking
                        //System.out.println("in ActionOpenSketcher.valueChanged(), "+
                          // "l_analyzedEvent: ");
                         //for (int i = 0 ; i < 2; i++) {
                           //System.out.println(l_analyzedEvent[i].toString());
                         //}

    openSelection(l_analyzedEvent);
    l_analyzedEvent = null;
    return;
  } // valueChanged(TreeSelectionEvent)
  
 /**
 * Invoked by actionPerformed() and Desktop to display the 'Sketcher' window.
  * @param _l_analyzedEvent is an Object[] of three items:
  *   0- String type of the selected base object
  *     from TreeManager.analyzeSelectedNodeInExplorer() - may be null
  *   1- ImmutableName name of the base object - may be null
  *   2- JInternalFrame open main frame of the base object - may be null
 * @return the 'Sketcher' SketchViewInBase or null
 */
 public static synchronized ViewInBase openSketcher(Object[] _l_analyzedEvent) {
    // variables
    SketchViewInBase      view = null;
    GraphicSketchView     gview;
    
    try {        
        // create an empty SketchViewInBase, not registered in the base and not saved,
        // and creates ViewInBase_Facade.SKETCH_MEDIATOR as a static property
        // to use by the static methods in gWork
        view = ViewInBase_Facade.createSketchView(SKETCH_VIEW_NAME);
        ViewModel.createViewRightsMediator(view);
        ViewInBase_Facade.SKETCH_MEDIATOR = (RightsMediatorImpl)ViewModel.
                                                getViewRightsMediator(view);
        
        // create an empty graphic view, 
        // which does not operate until the call to GraphicSketchView.resetGraphicView()
        gview = new GraphicSketchView(view);

        // register the frame in InternalFrameModel and set the disposition
        // from the GUI saving
        InternalFrame.openSketcher(gview);
        
    } catch (BaseError b) {
      DialogBox.showErrorMessage(b.toString());
      gview = null;
      view = null;
      return null;
    } catch (CreateError b) {
      DialogBox.showErrorMessage(b.toString());
      gview = null;
      view = null;
      return null;
    }
    
    // do not create the view listeners (GraphicViewBaseListenerImpl,
    // RightsMediatorImpl) in the right order,
    // nor register the graphic view 
  
    openSelection(_l_analyzedEvent); 
    
    gview = null;

    return view;
  } // openSketcher()   

  /**
  * Opens the selected base object in the sketcher, which has to be open.
  * Called by openSketcher() and valueChanged().
  * Process the following ACSObjects: actor, resource, ec_resource,
  * groupid, userid. Do not process metarights, acsrights, AclEntries and privileges.
  * No operation if the selected object in the explorer is an acsrun, a metaright, 
  * an acsright, an aclentry, a privilege, a view or an acs, and
  * this is the main difference with the beamer.
  * @param _l_analyzedEvent is an Object[] of three items:
  *   0- String type of the selected base object
  *     ("acs", "acsrun", "resource", "actor", "ec_resource",
  *     "groupid", "userid", "aclentry", "virtualfolder", "privilege" or "view") - may be null
  *   1- ImmutableName name of the base object - may be null
  *   2- JInternalFrame open main frame of the base object - may be null
  */
  static private void openSelection(Object[] _l_analyzedEvent) {

    BaseObject          center = null;
    GraphicSketchView   gview = InternalFrameModel.getSketcherPane();
    SketchViewInBase    view = (SketchViewInBase)gview.getEorDataModel();
    String              type = (String)_l_analyzedEvent[0];
    Map                 m_l_DisplayableLinks;
    List                l_empty = new ArrayList(5);
    int                 result = 0;

    // selects the center
    if (type.equals("actor") ||
      type.equals("resource") ||
      type.equals("ec_resource") ||
      type.equals("groupid") ||
      type.equals("userid") ||
      type.equals("virtualfolder") //||
      //type.equals("acsright") 
      ) {

                        // for testing and tracking
                    //Name key = null;
                    //System.out.println("in ActionOpenSketcher.openSelection(), "+
                      //"the objects in the view are:");
                    //for (Iterator ite = view.getEorM_BaseObjects().keySet().iterator() ; ite.hasNext() ; ) {
                      // key = (Name) ite.next();
                       //System.out.println("------- "+key);
                    //} // for ite

                    
        try {
          center = ACS_Facade.getEorBaseObject( (ImmutableName)_l_analyzedEvent[1]);
        } catch (BaseError b) {
          throw new InternalError("In ActionOpenSketcher.openSelection()"+
            ", for "+((ImmutableName)_l_analyzedEvent[1]).toString()+
            ", there is the error:"+b.toString());
        }

                        // for testing and tracking
                        //System.out.println("in ActionOpenSketcher.openSelection(), center: "+
                          //      ((BaseObject)center).getName());
        
        // clear the view   
        try {    
            view.setEorL_BaseObjectsForSketcher(l_empty);
            //view.setEorL_EligiblePartiesForSketcher(l_empty);
        } catch (BaseError b) {
              throw new InternalError("In ActionOpenSketcher.openSelection(): "+
                b.toString());
        }        
        
        // sketch view creation from the center.
        // updates the view objects.
        // m_l_DisplayableLinks is null if the size is 22 or more
        m_l_DisplayableLinks = ViewInBase_Facade.populateSketchView(view, center, type);

        if (m_l_DisplayableLinks != null) {
            gview.setBackground(InternalFrame.BACKCOLOR_8);
            // reset the graphic view
            try {  
                        // for testing and tracking
                        //System.out.println("in ActionOpenSketcher.openSelection(), call to "+
                          //      "GraphicSketchView.resetGraphicView()");

                result = gview.resetGraphicView(center, m_l_DisplayableLinks);
            } catch (ProcessError p) {

                                // for testing and tracking
                                System.out.println("Error in ActionOpenSketcher.openSelection(), "+
                                        "GraphicSketchView.resetGraphicView() "+
                                        "throws: "+p.toString());

                result = -1;
            } // catch            
        } // if
        
        if (m_l_DisplayableLinks != null && result != -1) { //  
            view = null;
            gview = null;
            center = null;
            type = null;
            m_l_DisplayableLinks = null;
            l_empty = null;
            return;             
        } // else

    } // if type.equals("actor") || ...)

    // else, set the graphic view as empty   
    try {    
                        // for testing and tracking
                        //System.out.println("in ActionOpenSketcher.openSelection(), SECOND call to "+
                          //      "GraphicSketchView.resetGraphicView() to reset it");

        view.setEorL_BaseObjectsForSketcher(l_empty);
        gview.resetGraphicView(null, null);
        gview.setBackground(InternalFrame.BACKCOLOR_6);
    } catch (ProcessError p) {
          //throw new InternalError("In ActionOpenSketcher.openSelection(): "+
            //p.toString());
    } catch (BaseError b) {
          //throw new InternalError("In ActionOpenSketcher.openSelection(): "+
            //b.toString());
    }
    view = null;
    gview = null;
    l_empty = null;
    type = null;       
  } // openSelection() 
 
} // class ActionOpenSketcher

