//Title:        Access Road
//Version:      0.6.5
//Copyright:    2002, 2004, 2007, 2008, 2009 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import ARoad0.gWorkInterface.RightsNetworkObserver;
import javax.swing.*;
import java.awt.event.*;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.Gui2.*;
import ARoad0.gWork.*;
import ARoad0.ErrW.*;
import ARoad0.ErrB.BaseError;
import ARoad0.gBase.NameImpl;
import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.ViewInBase;
import ARoad0.gBase.BaseManagerImpl;

/**
 * An action that saves the object (ACS, view) associated to the active frame
 * in the GUI.
 * @version 0.6.5
 */
public class ActionSave extends AbstractAction implements Singleton {

  /* the only instance of the class, following singleton pattern */
  private static ActionSave     instance__ = new ActionSave();

  // environment
  private static String         currentDir__ = System.getProperty("user.dir");
  private static String         userDir__ = System.getProperty("user.home");
  private static String         fileSepar__ = System.getProperty("file.separator");
  private static String         lineSepar__ = System.getProperty("line.separator");

  /**
   * With this private constructor, the compiler won't generate
   * a default public constructor.
   */
  private ActionSave() {
    super("Save");
  }

  /**
   * The only mean to get the instance.
   * @return ActionSave
   */
  public static synchronized ActionSave getInstance(){
    return instance__;
  } // getInstance()

  /**
  * Save the selected frame object.
  * Manage the actions fired from two sources: <P>
   * - from 'Save' in the main menu when the main frame is activated
   *    in the desktop <P>
   * - from 'Save' in the main menu when, in the explorer, an ACS or view node
   *    is selected <P>
  * A control validates the selected frame or node by verifying that
  * it is associated to an ACS.
  * @param _ev ActionEvent
  */
  public void actionPerformed(ActionEvent _ev) {

    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) {
        // display a warning message
        DialogBox.showErrorMessage("No saving during a user dialog.");        
        return;
    }
      
      // initialization
    Object[]                l_analyzedEvent = new Object[3];
    Map                     m_GUIconfig = null;
    //boolean                 userChoice = false;


    l_analyzedEvent = ActionUtilities.getInstance().analyzeActionEvent(_ev);
    
    if (l_analyzedEvent[0] == null ) {
            // display a warning message
            DialogBox.showErrorMessage(
              "No selection or unworkable selection.");
            l_analyzedEvent = null;
            return;
    } // if
    
          // for testing and tracking
    //System.out.println("in ActionSave.actionPerformed()"+
    //  "l_analyzedEvent: ");
    //for (int i = 0 ; i < l_analyzedEvent.length ; i++) {
    //  System.out.println(l_analyzedEvent[i].toString());
    //}

    if (l_analyzedEvent[0] == null ||
      (!((String)l_analyzedEvent[0]).equals("acs") &&
      !((String)l_analyzedEvent[0]).equals("view") )
      ) {
        // display a warning message
        DialogBox.showErrorMessage(
          "No saving. Only a selected ACS or view can be saved.");
        return;
    } // if

    try {
        m_GUIconfig = BaseManagerImpl.getInstance().getGUI();
    } catch (BaseError b) {
        throw new InternalError("in ActionSaveAll.saveDefaultBase(), error: "+b);
    }

    save((String)l_analyzedEvent[0], (ImmutableName)l_analyzedEvent[1], m_GUIconfig, true);

    l_analyzedEvent = null;
    m_GUIconfig = null;
    return;
  } // actionPerformed(ActionEvent)

/**
 * Save an opened ACS or ViewInBase, which will be restored by Gui1.Desktop at
 * the next opening.
 * The Sketcher is the unique view to be never processed in this method,
 * since it is not in the explorer.
 * Add items in the GUI configuration map and save in a file, but never remove an item.
 * In the GUI configuration map, work only on the map which is the value
 * for the key 'new NameImpl("<Last GUI>")'.
 * Called by actionPerformed() and ActionSaveAll.
 * @param _type equals to 'acs' or 'view'. Noop otherwise.
 * @param _name of the object to save
 * @param _m_GUIconfig is the GUI configuration map. May be null. If not, it is
 *      updated and saved by this method.
 * @param _withGraphicViewListeningRestoration true to restore, after the saving
 *      of an ACS, the listening of events from the base by the GraphicViewBaseListeners.
 *      Set to false only when the acs or the program is closing. No use if _type
 *      is 'view'.
 * @see ARoad0.Gui1.Desktop
 * @since 0.6.5
*/
  public void save(String _type, ImmutableName _name, Map _m_GUIconfig,
          boolean _withGraphicViewListeningRestoration) {

    Map m_Config = null;
    if (_m_GUIconfig == null) _m_GUIconfig = new HashMap(10);
    else m_Config = (Map) _m_GUIconfig.get(new NameImpl("<Last GUI>"));

    if (m_Config == null) m_Config = new HashMap(10);

    // whatever the event source, requests to the user
    // if he wants the saving of the base object.
    // userChoice is true if the saving is requested.
    //userChoice = ActionUtilities.getInstance().showSavingDialog(
      //(ImmutableName)l_analyzedEvent[1], false);

    if (_type.equals("acs") ) { // && userChoice

      BaseListener          listener;
      RightsNetworkObserver observer;
      ViewInBase            view;
      //Resource              res;
      //AclEntry[]            l_acl;
      JInternalFrame        frame = (JInternalFrame)InternalFrameModel.
                                        getMainFramesOfBaseObjects().get(_name);
      
        // remove the view listeners to avoid to take account of the AclEntries switches
        // and other accesses to the base objects by the view resettings.
        // the sketch view has no listener.
        // no risk of an updating of ViewModel.getL_GraphicViewBaseListener() by another thread.
      for (Iterator ite = ViewModel.getL_GraphicViewBaseListener().iterator(); ite.hasNext();) {
        listener = (BaseListener)ite.next();

        view = (ViewInBase)ViewInBase_Facade.getAllStandardOpenViews().get(
                listener.getEorMainEventSource() );      

        // the Architecture view is implied - new in v065
        if (ActionOpenArchitecture.ARCHI_VIEW_NAME.equals(listener.getEorMainEventSource()))
            view = (ViewInBase)ViewInBase_Facade.getArchitectureView();
        else
            view = (ViewInBase)ViewInBase_Facade.getAllStandardOpenViews().get(
                listener.getEorMainEventSource() );

        observer = ViewModel.getViewRightsMediator(view);
        observer.removePropertyChangeListener(listener);

        // new in v065
        listener.setListenEvents(false);
      } // for (Iterator ite = ViewModel.getL_GraphicViewBaseListener()

      
      ACS_Facade.saveACS(_name );
      // tell the acs was opened in the explorer
      m_Config.put(_name.toString(), null);
      _m_GUIconfig.put(new NameImpl("<Last GUI>"), m_Config);
      
      if (frame != null) {
        try {
            // the acs main frame configuration is saved, and m_Config is updated
            // in _m_GUIconfig
            _m_GUIconfig = InternalFrameUtilities.saveFrameDispositionInGUI(
                   frame,
                   _m_GUIconfig); // the value type is Rectangle
        } catch (ProcessError p) {
            throw new InternalError("in ActionSave, for the view, error: "+p);
        } // catch
      } // if (frame
          
      if (_withGraphicViewListeningRestoration) { // new in v065
          for (Iterator ite = ViewModel.getL_GraphicViewBaseListener().iterator(); ite.hasNext();) {
            listener = (BaseListener)ite.next();

            view = (ViewInBase)ViewInBase_Facade.getAllStandardOpenViews().get(
                    listener.getEorMainEventSource() );

            // the Architecture view is implied - new in v065
            if (ActionOpenArchitecture.ARCHI_VIEW_NAME.equals(listener.getEorMainEventSource()))
                view = (ViewInBase)ViewInBase_Facade.getArchitectureView();
            else
                view = (ViewInBase)ViewInBase_Facade.getAllStandardOpenViews().get(
                    listener.getEorMainEventSource() );

            observer = ViewModel.getViewRightsMediator(view);
            observer.addPropertyChangeListener(listener);

            // new in v065
            listener.setListenEvents(true);
          } // for (Iterator ite = ViewModel.getL_GraphicViewBaseListener()
      } // if
    
                              // for testing and tracking
                        //System.out.println("End of ActionSave.save() for the acs: "+_name);
      
      listener = null;
      observer = null;
      view = null;
      //res = null;
      //l_acl = null;
      frame = null;
    } // if

    else if (_type.equals("view") ) { // && userChoice

                      // for testing and tracking
                      //System.out.println("Start of ActionSave.save() for the view "+_name);

        // process the Architecture view, any Full or NoThan view
        try {
            // tell the view was opened in the explorer
            // caution: the view may be opened in the explorer without diagram
            m_Config.put(_name.toString(), null);
            // save the GraphicView configuration in the GUI configuration,
            // if the view frame or tab is opened
            if (ViewModel.getGraphicViewOfView((ImmutableName)_name) != null)
                m_Config.put(
                    (new NameImpl("<GraphicView for>")).extended( (ImmutableName)_name ),
                    ViewModel.getGraphicViewOfView((ImmutableName)_name).
                        getIndexesOfNodeLocations()); // the value type is Map

            _m_GUIconfig.put(new NameImpl("<Last GUI>"), m_Config);

            // the view main frame may be null, and no change then
            // m_Config may be updated
            _m_GUIconfig = InternalFrameUtilities.saveFrameDispositionInGUI(
                   (JInternalFrame)InternalFrameModel.getMainFramesOfBaseObjects().get(
                       (ImmutableName)_name),
                   _m_GUIconfig); // the value type is Rectangle
            // the 'See why' frame may be null, and no change then
            // m_Config may be updated
            _m_GUIconfig = InternalFrameUtilities.saveFrameDispositionInGUI(
                   InternalFrameModel.getOpenedSeeWhyFrame((ImmutableName)_name),
                   _m_GUIconfig); // the value type is Rectangle
                       
        } catch (ProcessError p) {
            throw new InternalError("in ActionSave, for the view, error: "+p);
        }
    
                              // for testing and tracking
                        //System.out.println("End of ActionSave.save() for the view: "+_name);
                        //  "l_analyzedEvent: ");
                        //for (int i = 0 ; i < l_analyzedEvent.length ; i++) {
                        //  System.out.println(l_analyzedEvent[i].toString());
                        //}
        
        ViewInBase_Facade.saveView( (ImmutableName)_name );
    } // else if
    else {
        _m_GUIconfig = null;
        m_Config = null;
        return;
    } // else

    try {
        BaseManagerImpl.getInstance().saveGUI(_m_GUIconfig);
    } catch (BaseError b) {
        throw new InternalError("in ActionSave, error: "+b);
    } // catch

    m_Config = null;
  } // save()

} // class ActionSave