//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import java.awt.event.*;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

import ARoad0.ErrB.BaseError;
import ARoad0.ErrW.ProcessError;
import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.gWork.*;
import ARoad0.gBaseInterface.ImmutableName;

import ARoad0.gBase.BaseManagerImpl;
import ARoad0.gBase.NameImpl;

/**
 * An action that saves all the ACS and the views.
 * This is a typical sequence of calls, in the inverse order,
 * when the ACS of an external linked privilege is saved: <P>
 * ARoad0.Gui1.CollectionPropertyEditor.propertyChange() <P>
 * at java.beans.PropertyChangeSupport.firePropertyChange(PropertyChangeSupport.java:339) <P>
 * at java.beans.PropertyChangeSupport.firePropertyChange(PropertyChangeSupport.java:347) <P>
 * at java.beans.PropertyChangeSupport.firePropertyChange(PropertyChangeSupport.java:276) <P>
 * at ARoad0.gBase.ACSImpl.firePropertyChange(ACSImpl.java:5454) <P>
 * at ARoad0.gBase.ACSImpl.removeAcsController(ACSImpl.java:7981) <P>
 * at ARoad0.gBase.ACSImpl.removeEorResource(ACSImpl.java:4014) <P>
 * at ARoad0.gBase.PrivilegeForLinksImpl.removeEorPrivilegeTargetLink(PrivilegeForLinksImpl.java:1447) <P>
 * at ARoad0.gBase.CPrivilegeUserImpl.removeEorPrivilegeToTarget(CPrivilegeUserImpl.java:481) <P>
 * at ARoad0.gBase.ResourceImpl.removeEorPrivilegeToTarget(ResourceImpl.java:1999) <P>
 * at ARoad0.gBase.PrivilegeForLinksImpl.finalizeForBase(PrivilegeForLinksImpl.java:1829) <P>
 * at ARoad0.gBase.PrivilegeAbst.finalizeForUser(PrivilegeAbst.java:1281) <P>
 * at ARoad0.gBase.ACSImpl.closeExternalLinkedPrivileges(ACSImpl.java:6327) <P>
 * at ARoad0.gBase.BaseManagerImpl.cutACSExternalLinks(BaseManagerImpl.java:2915) <P>
 * at ARoad0.gBase.BaseManagerImpl.saveOpenACS(BaseManagerImpl.java:2393) <P>
 * at ARoad0.gWork.ACS_Facade.saveACS(ACS_Facade.java:188) <P>
 * at ARoad0.gDMak.ActionSave.save(ActionSave.java:210) <P>
 * at ARoad0.gDMak.ActionSaveAll.saveAll(ActionSaveAll.java:302) <P>
 * at ARoad0.gDMak.ActionSaveAll.actionPerformed(ActionSaveAll.java:87) <P>
 * @version 0.7.0
 * @since 0.5.0
 */
public class ActionSaveAll extends AbstractAction implements Singleton {

  /* the only instance of the class, following singleton pattern */
  private static ActionSaveAll      instance__ = new ActionSaveAll();

  // the desktop to work with
  // It is initialized only in actionPerformed() because Desktop calls
  // this class before the creation of its JDesktopPane
  //private static JDesktopPane       desk__;

  // environment
  private static String             lineSepar__ = System.getProperty("line.separator");

  /**
  * With this private constructor, the compiler won't generate
  * a default public constructor.
  */
  private ActionSaveAll() {
    super("Save All");
  }

  /**
  * The only mean to get the instance.
  * @return ActionSaveAll
  */
  public static synchronized ActionSaveAll getInstance(){
    return instance__;
  } // getInstance()

  /**
  * Manages the SaveAll action fired from the main menu.
  * @param ev ActionEvent
  */
  public void actionPerformed(ActionEvent ev) {
    // no action if the explorer is currently used for an object selection
    // or if there is an user dialog currently
    if (DialogBox.onGoingObjectSelectionOrUserDialog()) {
        // display a warning message
        DialogBox.showErrorMessage(
          "No saving during a user dialog.");        
        return;
    }

    saveAll(true);
  } // actionPerformed(ActionEvent)

  /**
  * Saves all the open ACS and views in the base, and the positions of
  * the other frames like the explorer if they are open.
 * The structure of the GUI configuration map is: <P>
 * - the 3 possible keys are the Names '<Desktop>', '<Last GUI>' and '<Previous GUI>' <P>
 * - the 2 possible values are a Rectangle for '<Desktop>' only, and a Map, used for the last
 *  general saving of the GUI configuration for '<Last GUI>', and for '<Previous GUI>', a Map
 *  of the GUI configuration for the objects which was not open at the last saving of the GUI,
 *  but has been saved before. <P>
 * The rectangle for '<Desktop>' is the size of the Access Road main window. <P>
 * <P>
 * The Map for '<Last GUI>' contains a description of all the open internal frames and views
 * at the last saving of the GUI. <P>
 * Its structure is: <P>
 * - the first possible entry is (String acs_name, null) when the acs is open <P>
 * - the second possible entry is (Name acs_name, Rectangle) where the value is
 *      the position of the acs main frame <P>
 * - the third possible entry is (Name name, Rectangle) where the name is equals to
 *      '<Sketcher>' and the rectangle is the position of the Sketcher view <P>
 * - the fourth possible entry is (String view_name, null) when the full view,
 *      the NoThan view or the Architecture view is open <P>
 * - the fifth possible entry is (Name '<AllFullViews>', Rectangle) where the value is
 *      the position of the <AllFullViews> frame <P>
 * - the sixth possible entry is (Name '<AllNoThanViews>', Rectangle) where the value is
 *      the position of the <AllNoThanViews> frame <P>
 * - the seventh possible entry is (Name view_name, Rectangle) where
 *      the rectangle is the position of the view main frame <P>
 * - the eighth possible entry is (NameImpl("<GraphicView for>")).extended(view_name),
 *      Map nodes_location) where nodes_location is an argument in the constructor
 *      of a GraphicView, to set the position of each node in the graphic <P>
 * - the ninth possible entry is (NameImpl("<See why for>")).extended(view_name), Rectangle)
 *      where the rectangle is the position of the 'See Why' frame for the view
 * - the tenth possible entry is (NameImpl("<GlobalStructure>"), Rectangle)
 *      where the rectangle is the position of the global structure frame
 * <P>
 * The Map for '<Previous GUI>' contains a description of all the open internal frames
 * and views which was not open at the last saving of the GUI, but has been saved before. <P>
 * Its structure is: <P>
 * - the first possible entry is (Name acs_name, Rectangle) where the value is
 *      the position of the acs main frame <P>
 * - the second possible entry is (Name name, Rectangle) where the name is equals to
 *      '<Sketcher>' and the rectangle is the position of the Sketcher view <P>
 * - the third possible entry is (Name '<AllFullViews>', Rectangle) where the value is
 *      the position of the <AllFullViews> frame <P>
 * - the fourth possible entry is (Name '<AllNoThanViews>', Rectangle) where the value is
 *      the position of the <AllNoThanViews> frame <P>
 * - the fiveth possible entry is (Name view_name, Rectangle) where
 *      the rectangle is the position of the view main frame <P>
 * - the sixth possible entry is (NameImpl("<GraphicView for>")).extended(view_name),
 *      Map nodes_location) where nodes_location is an argument in the constructor
 *      of a GraphicView, to set the position of each node in the graphic <P>
 * - the seventh possible entry is (NameImpl("<See why for>")).extended(view_name), Rectangle)
 *      where the rectangle is the position of the 'See Why' frame for the view
 * - the eighth possible entry is (NameImpl("<GlobalStructure>"), Rectangle)
 *      where the rectangle is the position of the global structure frame
 * <P>
 * Calls ActionSave.save() for the ACS and the views to save.
 * Called by actionPerformed() and ActionExit.
 * @param _withGraphicViewListeningRestoration true to restore, after the saving
 *      of an ACS, the listening of events from the base by the GraphicViewBaseListeners.
 *      To set to false only when the acs or the program is closing. No use for
 *      the view savings.
  */
  public void saveAll(boolean _withGraphicViewListeningRestoration) {


                  // for testing and tracking
                  //System.out.println("Start of ActionSaveAll.saveAll(). ");

    // save the base
    try {
      JInternalFrame[]  allframes;
      Map               m_Config = null;
      Map               m_GUIconfig = null;
      Map               m_previousConfig = null;
      ImmutableName     name = null;

      // gets the m_GUIconfig to compare its '<Last GUI>' map
      // and the currently open objects, copies the objects in '<Last GUI>'
      // which are closed in '<Previous GUI>', and deletes them in '<Last GUI>'
      try {
        m_GUIconfig = BaseManagerImpl.getInstance().getGUI();
      } catch (BaseError b) {
        throw new InternalError("in ActionSaveAll.saveDefaultBase(), error: "+b);
      }

      if (m_GUIconfig == null) m_GUIconfig = new HashMap(10);
      else {
          m_Config = (Map) m_GUIconfig.get(new NameImpl("<Last GUI>"));
          m_previousConfig = (Map) m_GUIconfig.get(new NameImpl("<Previous GUI>"));
      }

      if (m_Config == null) m_Config = new HashMap(10);
      if (m_previousConfig == null) m_previousConfig = new HashMap(10);


                                // for testing and tracking
                                //System.out.println("In ActionSaveAll.saveAll(), "+
                                  //      "before the removing of closed ACS and views, m_Config keys:");
                                //for (Iterator ite = m_Config.entrySet().iterator(); ite.hasNext(); ) {
                                  //  Map.Entry entry = (Map.Entry) ite.next();
                                  //  System.out.println("----------- "+entry.getKey());
                                //} // for ite

      for (Iterator ite = m_Config.entrySet().iterator(); ite.hasNext(); ) {
          Map.Entry entry = (Map.Entry) ite.next();
          
          if (entry.getKey() instanceof String) {
              name = NameImpl.getNameFromString((String)entry.getKey());
                // - the first possible entry is (String acs_name, null) when the acs is open <P>
              if ( ACS_Facade.knowsACS(name) && !ACS_Facade.isOpenACS(name)) {

                                    // for testing and tracking
                                    //System.out.println("In ActionSaveAll.saveAll(), "+
                                      //      "removing of the closed ACS: "+entry.getKey());

                            ite.remove();
                // - the fourth possible entry is (String view_name, null) when the full view,
                //      the NoThan view or the Architecture view is open <P>
              }
              else if ( ViewInBase_Facade.knowsView(name) && !ViewInBase_Facade.isOpenView(name)) {

                                    // for testing and tracking
                                    //System.out.println("In ActionSaveAll.saveAll(), "+
                                      //      "removing of the closed view: "+entry.getKey());

                            ite.remove();
              } // else if
              continue;
          } // if (entry.getKey() instanceof String)

          // otherwise, the key is a name
          name = (ImmutableName) entry.getKey();
          
            // - the second possible entry is (Name acs_name, Rectangle) where the value is
            //      the position of the acs main frame <P>
          if ( ACS_Facade.knowsACS(name) && !ACS_Facade.isOpenACS(name)) {
                        m_previousConfig.put(name, entry.getValue());

                                // for testing and tracking
                                //System.out.println("In ActionSaveAll.saveAll(), "+
                                  //      "removing of the closed ACS main frame: "+entry.getKey());

                        ite.remove();
            // - the third possible entry is (Name name, Rectangle) where the name is equals to
            //  '<Sketcher>' and the rectangle is the position of the Sketcher view <P>
          } else if ( name.equals(new NameImpl("<Sketcher>")) && !InternalFrameModel.withSketcher()) {
                        m_previousConfig.put(name, entry.getValue());
                        ite.remove();
            // - the fifth possible entry is (Name '<AllFullViews>', Rectangle) where the value is
            //      the position of the <AllFullViews> frame <P>
          } else if ( name.equals(new NameImpl("<AllFullViews>")) &&
                  !InternalFrameModel.withAllFullViews()) {
                        m_previousConfig.put(name, entry.getValue());
                        ite.remove();
            // - the sixth possible entry is (Name '<AllNoThanViews>', Rectangle) where the value is
            //      the position of the <AllNoThanViews> frame <P>
          } else if ( name.equals(new NameImpl("<AllNoThanViews>")) &&
                  !InternalFrameModel.withAllNoThanViews()) {
                        m_previousConfig.put(name, entry.getValue());
                        ite.remove();
            // - the seventh possible entry is (Name view_name, Rectangle) where
            //      the rectangle is the position of the view main frame <P>
          } else if ( ViewInBase_Facade.knowsView(name) && !ViewInBase_Facade.isOpenView(name)) {
                        m_previousConfig.put(name, entry.getValue());
                        ite.remove();
            // - the eighth possible entry is (NameImpl("<GraphicView for>")).extended(view_name),
            //      Map nodes_location) where nodes_location is an argument in the constructor
            //      of a GraphicView, to set the position of each node in the graphic <P>
          } else if ( name.component(0).equals("<GraphicView for>") &&
                   !ViewInBase_Facade.isOpenView( name.subname(1, name.noOfComponents()) ) ) {
                        m_previousConfig.put(name, entry.getValue());

                                // for testing and tracking
                                //System.out.println("In ActionSaveAll.saveAll(), "+
                                  //      "removing of the closed view main frame: "+entry.getKey());

                        ite.remove();
            // - the ninth possible entry is (NameImpl("<See why for>")).extended(view_name), Rectangle)
            //      where the rectangle is the position of the 'See Why' frame for the view
          } else if ( name.component(0).equals("<See why for>") &&
                   !ViewInBase_Facade.isOpenView( name.subname(1, name.noOfComponents()) ) ) {
                        m_previousConfig.put(name, entry.getValue());
                        ite.remove();
            // - the tenth possible entry is (ActionOpenGlobalStructure.LOCATION_STRUCTURE_NAME,
            //      Rectangle)
            //      where the rectangle is the position of the 'Full Strcutre' frame
          } else if ( name.equals(ActionOpenGlobalStructure.LOCATION_STRUCTURE_NAME) ) {
                        m_previousConfig.put(name, entry.getValue());
                        ite.remove();
          }
      } // for ite

      m_GUIconfig.put(new NameImpl("<Previous GUI>"), m_previousConfig);

                                // for testing and tracking
                                //System.out.println("In ActionSaveAll.saveAll(), "+
                                  //      "before the saving of ACS and views, m_previousConfig keys:");
                                //for (Iterator ite = m_previousConfig.entrySet().iterator(); ite.hasNext(); ) {
                                  //  Map.Entry entry = (Map.Entry) ite.next();
                                  //  System.out.println("----------- "+entry.getKey());
                                //} // for ite
                                // for testing and tracking
                                //System.out.println("In ActionSaveAll.saveAll(), "+
                                  //      "before the saving of ACS and views, m_Config keys:");
                                //for (Iterator ite = m_Config.entrySet().iterator(); ite.hasNext(); ) {
                                  //  Map.Entry entry = (Map.Entry) ite.next();
                                  //  System.out.println("----------- "+entry.getKey());
                                //} // for ite

      // save the open ACS, and update the file of m_GUIconfig
      for (Iterator ite = ACS_Facade.getOpenACS().keySet().iterator(); ite.hasNext(); ) {
          ActionSave.getInstance().save("acs", (ImmutableName)ite.next(), m_GUIconfig,
                _withGraphicViewListeningRestoration);
      } // for
      // save the open Full and NoThan views, then the Architecture view
      for (Iterator ite = ViewInBase_Facade.getAllStandardOpenViews().keySet().iterator(); ite.hasNext(); ) {
          ActionSave.getInstance().save("view", (ImmutableName)ite.next(), m_GUIconfig, false);
      } // for
      if (ViewInBase_Facade.getArchitectureView() != null)
          ActionSave.getInstance().save("view",
              ActionOpenArchitecture.ARCHI_VIEW_NAME, m_GUIconfig, false);

      // use the newly updated m_GUIconfig to add the saving of the configurations
      // for the Access Road main window, the explorer, the sketcher and the beamer

                                // for testing and tracking
                                //System.out.println("In ActionSaveAll.saveAll(), "+
                                  //      "after the saving of ACS and views, m_Config keys:");
                                //for (Iterator ite = m_Config.entrySet().iterator(); ite.hasNext(); ) {
                                  //  Map.Entry entry = (Map.Entry) ite.next();
                                  //  System.out.println("----------- "+entry.getKey());
                                //} // for ite

      // save the frames configuration in the GUI, if they are open.
      // note: acs and view frames has been already saved in ActionSave.save(), but
      // not these frames.
      allframes = ARoad0.Gui1.Desktop.getCurrentFrames();
      for (int i = 0; i < allframes.length; i++) {

          if (allframes[i] == InternalFrameModel.getExplorer()) {
              m_GUIconfig = InternalFrameUtilities.saveFrameDispositionInGUI(
                      allframes[i], m_GUIconfig);
          } else if (allframes[i] == InternalFrameModel.getBeamer()) {
              m_GUIconfig = InternalFrameUtilities.saveFrameDispositionInGUI(
                      allframes[i], m_GUIconfig);
          } else if (allframes[i] == InternalFrameModel.getAllFullViewsFrame()) {
              m_GUIconfig = InternalFrameUtilities.saveFrameDispositionInGUI(
                      allframes[i], m_GUIconfig);
          } else if (allframes[i] == InternalFrameModel.getAllNoThanViewsFrame()) {
              m_GUIconfig = InternalFrameUtilities.saveFrameDispositionInGUI(
                      allframes[i], m_GUIconfig);
          } else if (allframes[i] == InternalFrameModel.getSketcher()) {
              m_GUIconfig = InternalFrameUtilities.saveFrameDispositionInGUI(
                      allframes[i], m_GUIconfig);
          } else if (allframes[i] == InternalFrameModel.getStructureFrame()) {
              m_GUIconfig = InternalFrameUtilities.saveFrameDispositionInGUI(
                      allframes[i], m_GUIconfig);
          }

      } // for

                  // for testing and tracking
                  //System.out.println("End of ActionSaveAll.saveAll(), before the saving of m_GUIconfig. ");

      // reset and save the GUI configuration map
      m_GUIconfig.put(
                new NameImpl("<Desktop>"),
                ARoad0.Gui1.Desktop.getDSize()); // the value type is Dimension

      BaseManagerImpl.getInstance().saveGUI(m_GUIconfig);

      m_GUIconfig = null;
      m_Config = null;
      m_previousConfig = null;
      allframes = null;
      name = null;
    } catch (BaseError b) {
          // display a warning message if there is an exception, and stop
      DialogBox.showAlarmMessage(
        "This is a serious problem. The saving of the base has aborted."+
        lineSepar__+"Your Access Road files are normally NOT corrupted,"+
        lineSepar__+"except perhaps if the following exception message"+
        " indicates that there is an IOException. In that case, you should exit Access Road,"+
        "identify and restore the corrupted files from their backup."+
        "The exception message is: "+ b.toString() );
      return; // no save
    } catch (ProcessError p) {
          throw new InternalError("in ActionSaveAll, error: "+p);
    }    

  }  // saveAll()

} // class ActionSaveAll