//Title:        Access Road
//Version:      0.6.0
//Copyright:    2002, 2004, 2007, 2008, 2009 EURL ACCBEE
//Author:       Patrick Thazard ( )
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gDMak;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.event.ActionEvent;
import java.util.Iterator;

import ARoad0.Pattern.*;
import ARoad0.Gui1.*;
import ARoad0.Gui2.*;

import ARoad0.ErrW.*;
import ARoad0.ErrB.BaseError;

import ARoad0.gBase.ACSFactoryImpl;
import ARoad0.gBase.NameImpl;

import ARoad0.gBaseInterface.ImmutableName;
import ARoad0.gBaseInterface.ImmutableGroupID;


/**
 * An utility class that closes the main frame of a base object,
 * analyzes the ActionEvent fired to an AbstractAction,
 * and manages the dialog to choose a closing and a saving of
 * the base object. Used by ActionCloseXXX and SimplePropertyEditor classes.
 * @version 0.6.0
 * @since 0.5.1
 */
public class ActionUtilities implements Singleton {

  // the only instance of the class, following singleton pattern
  private static ActionUtilities instance__ = new ActionUtilities();

  private Class                     internalFrameClass_;
  
  private ACSFactoryImpl            factory_ = null;
  
  /**
  * with this private constructor, the compiler won't generate
 * a default public constructor
 */
 private ActionUtilities() {
    // initialization
    try {
      internalFrameClass_ = Class.forName("javax.swing.JInternalFrame");
    } catch (ClassNotFoundException c) {}
 }



  //----------------------- PUBLIC METHODS ------------------------------------

  /**
   * constructor()
   * @return ActionUtilities
   */
  public static synchronized ActionUtilities getInstance(){
    return instance__;
  } // getInstance()

  /**
  * This method registers all the relevant ACSTreeBaseListeners
  * to listen the 'l_Members' events.
  * Called by ActionCopy.handleNewBaseObject().
  * @param _grp is the group which may fire 'l_Members' events
  */
  public void registerACSTreeListenerInGroup(ImmutableGroupID _grp) {
    TreeBaseListener listener;
    for (Iterator ite = TreeManager.getL_TreeBaseListener().iterator(); ite.hasNext(); ) {
       listener = (TreeBaseListener) ite.next();
       if (!(listener instanceof ACSTreeBaseListenerImpl)) continue;
       if (listener.getEorMainEventSource().equals(_grp.getAcsName())) {

                    // for testing and tracking
                    //System.out.println("in ActionCopy.registerACSTreeListenerInGroup() "+
                    //"add the 'l_Members' listener "+listener+" to the GroupID "+ _grp.toString() );

          ((ImmutableGroupID)_grp).addPropertyChangeListener("l_Members", listener);
       }
    } // for
    listener = null;     
    return;
  } // registerACSTreeListenerInGroup()
  
  /**
   * ActionNewACSyst calls this method to store the current ACSFactory.
   * @param _factory to store. May be null.
   */
  public void setACSFactory(ACSFactoryImpl _factory) {
    factory_ = _factory;
    return;
  } // setACSFactory()
  
  /**
   * Called by ActionOpenBeamer
   * @return _factory is currently used for an ACS creation. May be null.
   */
  public ACSFactoryImpl getACSFactory() {
    return factory_;
  } // getACSFactory()
  
  //
  //----------------------- PACKAGE METHODS -----------------------------------
  //

  /**
  * Analyzes an ActionEvent fired to actionPerformed() in an AbstractAction class,
  * to extract the main information. Called by ActionClose, ActionSave,
  * ActionRemove, ActionOpenBeamer, ActionExportRTF, ActionExportPNG.
  * Analyzes the ActionEvent fired from the main menu,
  * and also coming from an ACS, the beamer, the structure, a view main or
  * a 'See why' JInternalFrame.
  * The returned base object cannot be closed.
  * Call TreeManager.analyzeSelectedNodeInExplorer() if it is a node 
  * in the explorer, and analyzeSelectedNodeInStructure() for the structure.
  * @param _ev ActionEvent
  * @return Object[] of three objects:
  *   0- String type of the base object associated to the ActionEvent
  *     ("acs", "acsrun", "resource", "actor", "groupid", "userid",
  *      "aclentry", "view", "beamer", "metaright", "acsright", "structure" or ""
   *     if it is an another selected node) - may be null
  *   1- ImmutableName of the base object - may be null
  *   2- JComponent open main component (JInternalFrame or JPanel) of 
  *        the base object - may be null
  *   Return {null,null,null} if no frame is selected, or a no-workable frame,
  *   or if the selected node in the explorer is not workable.
  */
  Object[] analyzeActionEvent(ActionEvent _ev) {
    // initialization
    String                  type = "";
    ImmutableName           name = new NameImpl();
    JInternalFrame          activeFrame = InternalFrameModel.getActiveFrame();
    JComponent              mainComponent = activeFrame;
    Object[]                result = null;

                              // for testing and tracking
                      //System.out.println("in ActionUtilities.analyzeActionEvent(_ev)."+
                        //"_ev.getSource().getClass() is: "+_ev.getSource().getClass());
                      //System.out.println("--------- activeFrame: "+activeFrame.getTitle());
                      //System.out.println("--------- is beamer frame: "+
                        //      InternalFrameUtilities.isBeamerFrame(activeFrame));
                      //System.out.println("--------- beamerFrame: "+
                        //      InternalFrameModel.getBeamer().getTitle());
                      
    //
    // if event comes from a JMenuItem, extracts the selected node name if
    // the explorer is selected, or extract the base object name if a main
    // frame or the beamer is selected
    //
    if (_ev.getSource().getClass().equals(javax.swing.JMenuItem.class)) {

      // extracts the ACS name if the active frame is an ACS main frame
      if (InternalFrameUtilities.isACSFrame(activeFrame)) {
          try {

            name = InternalFrameUtilities.extractACSName(activeFrame.getTitle());
            type = "acs";
          } catch (DialogError d) {
            // impossible
          }
      // extracts the view name if the active frame is a view main frame (not 'All views')
      } else if (InternalFrameUtilities.isViewFrame(activeFrame)) {
          try {

            name = InternalFrameUtilities.extractViewNameFromFrameTitle(activeFrame.getTitle());
            type = "view";
          } catch (DialogError d) {
            // impossible
          }
      // extracts the view name of the selected tab if the active frame
          // is the 'All full views' frame
      } else if (InternalFrameUtilities.isAllFullViewsFrame(activeFrame)) {
          
            type = "view";
            mainComponent = (JPanel)ViewModel.getAllFullViewsJTabbedPane().getSelectedComponent();
            // the view name is from the selected tab component
            int index = ViewModel.getAllFullViewsJTabbedPane().getSelectedIndex();
            if (index >= 0) { // false if there is no open tab
                name = ViewModel.getViewNameFromTabTitle(
                        ViewModel.getAllFullViewsJTabbedPane().getTabComponentAt(index).getName());
                if (name == null) {
                    activeFrame = null;
                    mainComponent = null;

                        // for testing and tracking
                        //System.out.println("in ActionUtilites.analyzeActionEvent()"+
                          //  "analyzing of the tab in the 'AllFullViews' frame gets null for the"+
                            //" selected component: "+
                            //ViewModel.getAllFullViewsJTabbedPane().getSelectedComponent());

                    return new Object[] {null, null, null};
                } // else
            } // if (index
                        
                    // for testing and tracking
                    //System.out.println("in ActionUtilites.analyzeActionEvent()"+
                      //  "analyzing of the tab in the 'AllFullViews' frame gets the"+
                        //" selected component: "+name); 
            
      // extracts the view name of the selected tab if the active frame
            // is the 'All NoMore-NoLess views' frame
      } else if (InternalFrameUtilities.isAllNoThanViewsFrame(activeFrame)) {
          
            type = "view";
            mainComponent = (JPanel)ViewModel.getAllNoThanViewsJTabbedPane().getSelectedComponent();
            // the view name is from the selected tab component, if open
            int index = ViewModel.getAllNoThanViewsJTabbedPane().getSelectedIndex();
            if (index >= 0) { // false if there is no open tab
                name = ViewModel.getViewNameFromTabTitle(
                        ViewModel.getAllNoThanViewsJTabbedPane().getTabComponentAt(index).getName());
                if (name == null) {
                    activeFrame = null;
                    mainComponent = null;

                        // for testing and tracking
                        //System.out.println("in ActionUtilites.analyzeActionEvent()"+
                          //  "analyzing of the tab in the 'AllNoThanViews' frame gets null for the"+
                            //" selected component: "+
                            //ViewModel.getAllNoThanViewsJTabbedPane().getSelectedComponent());

                    return new Object[] {null, null, null};
                } // else
            } // if (index
                        
                    // for testing and tracking
                    //System.out.println("in ActionUtilites.analyzeActionEvent()"+
                      //  "analyzing of the tab in the 'AllNoThanViews' frame gets the"+
                        //" selected component: "+name); 
                        
      // extracts the Architecture name if the active frame is that
      } else if (activeFrame.getTitle().equals("Architecture ")) {
            name = ActionOpenArchitecture.ARCHI_VIEW_NAME;
            type = "view";
      // extracts the result as the current selection in the beamer
      } else if (InternalFrameUtilities.isBeamerFrame(activeFrame)) {
          try {
            // the beamer is the activeFrame, name may be empty
            name = InternalFrameUtilities.extractBeamerName(activeFrame.getTitle());
            type = "beamer";
          } catch (DialogError d) {
            // impossible
          }
      // extracts the result as the current selection in the IS structure
      } else if (InternalFrameUtilities.isISstructureFrame(activeFrame)) {

            // the structure frame is the activeFrame, name may be empty
            DefaultMutableTreeNode treeNode = TreeManager.analyzeSelectedNodeInStructure();
            name = TreeManager.getTrueUserObjectNameInStructureTree(treeNode);
            type = "structure";
            treeNode = null;
            
      // extracts the result as the current selection in the explorer
      } else {
          name = null;
          activeFrame = null;
          mainComponent = null;
          result = TreeManager.analyzeSelectedNodeInExplorer(); // return {null or "", null, null} if the node is not known
            
            // for testing and tracking
            //System.out.println("in ActionUtilites.analyzeActionEvent()"+
              //  " analyzing of the node in the explorer gets: ");
          
              //if (result[0] == null || result[0].equals("")) {
                //System.out.println("---------- null");
                  //return new Object[] {null, null, null};
              //} else {
                //    for (int i = 0; i < result.length; i++) {
                  //     System.out.println("--------- "+result[i]);    
                    //}
                    //return result;
              //} // else
            
      } // else
      
      if (result != null) {

                    // for testing and tracking
                    //System.out.println("in ActionUtilites.analyzeActionEvent()"+
                       // " analyzing from the menu, with a not-null result: ");          
                    //for (int i = 0; i < result.length; i++) {
                      // if (result[i] != null)
                        //   System.out.println("--------- "+result[i]);    
                    //}
                    
          return result;
      } 
      else {

                    // for testing and tracking
                    //System.out.println("in ActionUtilites.analyzeActionEvent()"+
                      //  " analyzing from the menu, with a null result gets: ");
                    //System.out.println("---- type: "+type); 
                    //System.out.println("---- name: "+name);
          
          return new Object[] {type, name, mainComponent};
      }
    } // if event comes from a JMenu


    //
    // if the event comes from an ACS, or from a view or a 'See why' JInternalFrame.
    // does not process the 'All views' frame and process the 'Architecture' frame.
    //
    else if (_ev.getSource().getClass().equals(internalFrameClass_ )) {

      try {
        if (InternalFrameUtilities.isACSFrame((JInternalFrame)_ev.getSource() )) {
            name = InternalFrameUtilities.extractACSName(_ev.getActionCommand() );
            type = "acs";
            mainComponent = (JInternalFrame) _ev.getSource();
        } else if (InternalFrameUtilities.isViewFrame((JInternalFrame)_ev.getSource() )) {
            name = InternalFrameUtilities.extractViewNameFromFrameTitle(_ev.getActionCommand() );
            type = "view";
            mainComponent = (JInternalFrame) _ev.getSource();
        } else if (((JInternalFrame)_ev.getSource() ).getTitle().equals("Architecture ")) {
            name = ActionOpenArchitecture.ARCHI_VIEW_NAME;
            type = "view";
            mainComponent = (JInternalFrame) _ev.getSource();
        } else if (((JInternalFrame)_ev.getSource() ).getTitle().indexOf("Sketcher ") != -1) {
            name = ActionOpenSketcher.SKETCH_VIEW_NAME;
            type = "view";
            mainComponent = (JInternalFrame) _ev.getSource();
        } else if (((JInternalFrame)_ev.getSource() ).getTitle().indexOf("See why") != -1) {
              try {
                  name = NameImpl.getNameFromString(
                          ((JInternalFrame)_ev.getSource() ).getTitle().substring(11) );
              } catch (BaseError b) {
                  throw new DialogError();
              } // catch
              type = "view";
              mainComponent = (JInternalFrame) InternalFrameModel.
                      getMainFramesOfBaseObjects().get(name);
        } else {
            // it is not an ACS or a view (noop for the 'All views' frame)
            // to avoid a return in a try bloc, throw a DialogError
            throw new DialogError();

            /*
            name = null;
            activeFrame = null;
            mainFrame = null;
                      // for testing and tracking
              //System.out.println("in ActionUtilites.analyzeActionEvent()"+
              //  "the JInternalFrame is not an main ACS nor a main view frame.");
            return new Object[] {null, null, null};
            */
        } // else
      } catch (DialogError d) {
            // it is not an ACS or a view frame
            name = null;
            activeFrame = null;
            mainComponent = null;
            
                      // for testing and tracking
                    //System.out.println("in ActionUtilites.analyzeActionEvent()"+
                      //  "a DialogError is fired: "+d);
            
            return new Object[] {null, null, null};
      } // catch
      
    } // if event comes from a JInternalFrame
    
                // for testing and tracking
              //System.out.println("ActionUtilites.analyzeActionEvent(), result is:");
                //  System.out.println("----- "+type);
                //  if (name != null) System.out.println("----- "+name.toString());
                //  if (mainComponent != null) System.out.println("----- "+mainComponent.getName());
    
    return new Object[] {type, name, mainComponent};
  } // analyzeActionEvent()


} // class ActionUtilities