//Title:        Access Road
//Version:      0.7.1
//Copyright:    2002, 2004, 2007, 2008, 2009, 2010, 2012 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gWork;

/*
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
*/

import java.util.Map;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.Collections;
import java.util.Iterator;
import java.util.Arrays;

import ARoad0.Pattern.StaticClass;
import ARoad0.ErrB.*;
import ARoad0.ErrW.DialogError;
import ARoad0.gBase.*;
import ARoad0.gBaseInterface.*;
import ARoad0.Gui1.DialogBox;

/**
 * This class is the "facade" (GoF pattern) for some operations
 * about ACS that are called by Gui1, Gui2 and gDMak objects.
 * It contains only static methods.
 * @version 0.7.1
 */
public class ACS_Facade implements StaticClass {

  // the only instance of the class, following singleton pattern
//  private static ACS_Facade             instance__ = new ACS_Facade();

//  private static PropertyChangeSupport  propChangeSupport__;

  /**
  * only one private constructor
  */
  private ACS_Facade() {
    // propChangeSupport__ = new PropertyChangeSupport(this);
  }

  /**
  * Only method to get an instance.
  */
/*  public static synchronized ACS_Facade getInstance(){
    return instance__;
  } // getInstance()
*/
  /**
  * Return the names of the registered IS, which are also the IS nametree names.
  * An IS may contains opened and closed ACS.
  * @return cloned set of IS names. Never null.
  */
  public static Set getL_IS() {
      return BaseManagerImpl.getInstance().getL_IS();
  }

  /**
  * @param _name of an ACS
  * @return true if _name is known as an ACS name. The ACS may be
  * opened or closed.
  */
  public static boolean knowsACS(ImmutableName _name){
    return BaseManagerImpl.getInstance().knowsACS(_name);
  }

  /**
  * @param _name of an ACS
  * @return boolean true if _name is known as an ACS name and the ACS is open.
  */
  public static boolean isOpenACS(ImmutableName _name){
    return BaseManagerImpl.getInstance().isOpenACS(_name);
  }

  /**
  * Called by ActionOpenACS. 
  * @return unmodifiable List of the closed ACS names.
  */
  public static List getClosedACSnames() {
    BaseManagerImpl base = BaseManagerImpl.getInstance();
    List result = base.getL_ACSNames();
    if (result.isEmpty()) {
        
                            // for testing and tracking        
                    //System.out.println("From ACS_Facade.getClosedACSnames(),"+
                      //" result is empty");
        
        return result;
    }
    Set l_openACSnames = BaseManagerImpl.getInstance().getEorM_OpenACS().keySet();
    ImmutableName name = null;

                            // for testing and tracking
                    //System.out.println("From ACS_Facade.getClosedACSnames(),"+
                      //" open ACS:");
                    //for (Object obj : result) {
                      //  System.out.println("--- "+obj);
                    //}


    // removes the open ACS names in result
    for(Iterator ite = result.iterator() ;ite.hasNext() ; ) {
      name = (ImmutableName)ite.next();
      if (l_openACSnames.contains(name)) ite.remove();
    }

                            // for testing and tracking
                    //System.out.println("From ACS_Facade.getClosedACSnames(),"+
                      //" result:");
                    //for (Object obj : result) {
                      //  System.out.println("--- "+obj);
                    //}


    name = null;
    l_openACSnames = null;
    return Collections.unmodifiableList(result);
  } // getClosedACSnames()

  /**
  * @return unmodifiable Map of the non-null open ACS instances,
  *   where the ACS name is the key. Never null.
  */
  public static Map getOpenACS() {
    return BaseManagerImpl.getInstance().getEorM_OpenACS();
  }

  /**
  *  Save the base with the ACS and view lists, even if it is empty.
  *  All open ACS and views, but not the closed ones,
  *  are saved in some files (one per opened ACS and view) in the default directory.
  *  Caution: the references between the ACS are broken then restored. So,
  *  all aclEntries must be restored in gWork and the GUI.
  * @exception BaseError if an acs is not open, ACSname.lastComponent() is null,
  *   FileNotFoundException or IOException.
  */
  public static void saveDefaultBase() throws BaseError {
    BaseManagerImpl.getInstance().saveDefaultBase();
  }

  /**
  * Close the base without savings, with all the open ACS and all the open views first.
  * @exception BaseError if an acs is not opened.
  */
  public static void closeDefaultBase() throws BaseError {
    BaseManagerImpl.getInstance().closeDefaultBase();
  }

  /**
  *  Save and close the base with the ACS and view lists, even if it is empty.
  *  All open ACS and views, but not the closed ones,
  *  are saved then closed in some files (one per opened ACS and view) in the default directory.
  * @exception BaseError if an acs is not open, ACSname.lastComponent() is null,
  *   FileNotFoundException or IOException.
  */
  public static void saveAndCloseDefaultBase() throws BaseError {
    BaseManagerImpl.getInstance().saveAndCloseDefaultBase();
  }

  /**
  * Save the ACS on a file..
  * @param _name ImmutableName of the ACS to save
  */
  public static void saveACS(ImmutableName _name) {
    // get the acs
    ACSImpl acs = (ACSImpl) BaseManagerImpl.getInstance().getEorM_OpenACS().get(_name);
    try {
      BaseManagerImpl.getInstance().saveOpenACS(acs);
    } catch (BaseError b) {
      throw new InternalError("in gWork.ACS_Facade.saveACS(), there is a problem: " +
              b.toString());
    }
  } // saveACS()

  /**
   * Get the open ACS, and before open it if it is closed. 
   * Does not open the tied child ACS, and this has to be done by the calling method
   * if it is an opening. Called by ActionOpenACSyst.openACS() for the ACS opening.
   * @param _name of the ACS to get or open.
   * No test of null nor empty. Used only to open an ACS.
   * @exception BaseError if the ACS is not known, the ACS is open, the open ACS has not
   * the same name, or ClassNotFoundException, IOException with
   * invalidClassException if the code of ACS has changed.
   * @return the requested ACS or an empty acs
   */
  public static synchronized ACSImpl openOrGetACS(ImmutableName _name) throws BaseError {
    // variables
    ACSImpl acs = ACSImpl.EMPTY_INSTANCE;

    // open the ACS
    if (BaseManagerImpl.getInstance().knowsACS(_name) ) {
      // opens the ACS or gets it
      if (BaseManagerImpl.getInstance().isOpenACS(_name) ) {
        acs = (ACSImpl) BaseManagerImpl.getInstance().getEorM_OpenACS().get(_name);
      } else acs = BaseManagerImpl.getInstance().openACS(_name);
    } // else

    return acs;
  } // openOrGetACS()

  

  /**
   * Closes any open ACS and saves it before.
   * @param _acs is an ACS to close
   */
  public static synchronized void saveAndCloseOpenACS(ACSImpl _acs) {
    try {
      BaseManagerImpl.getInstance().closeOpenACS(_acs, true);
    } catch (BaseError b) {
      throw new InternalError("In ACS_Facade.saveAndCloseOpenACS(), the exception"+
        " from BaseManagerImpl is: "+ b.toString());
    }
  } // saveAndCloseOpenACS()


  /**
   * Close any open ACS without save it. The open views that contains some objects
   * from this ACS are also closed without user message.
   * @param _acs to close
   * @exception BaseError if _acs is not open, ACSname.lastComponent() is null,
   *  or FileNotFoundException, IOException.
   */
  public static synchronized void closeOpenACS(ACSImpl _acs)
    throws BaseError {
      BaseManagerImpl.getInstance().closeOpenACS(_acs, false);
  } // closeOpenACS()


  /**
   * Removes any ACS, close and save it before if necessary. Removes its node and
   * all the children nodes in the IS nametree. Removes the IS if it is the last ACS in it.
   * Removes the children ACS.
   * @param _name of the ACS to remove from the base
   * @exception BaseError if the ACS is unknown, ACSname.lastComponent() is null,
   *  or FileNotFoundException, IOException.
   */
  public static synchronized void removeACS(ImmutableName _name)
    throws BaseError {
      BaseManagerImpl.getInstance().removeACS(_name);
  }  // removeACS(ImmutableName _name)

  /**
  * Builds up an ordered array of references to the ACS own groupIDs.
  * @param _acs is an ACS
  * @return array of GroupID in _acs.
  *   it is "new GroupID[1]" if the array is empty.
  */
  public static GroupID[] getOwnGroupArray(ACS _acs){
    if (_acs.getEorM_OwnEligibleParties().values().isEmpty()) return new GroupID[1];
    // variables
    Object[] list = _acs.getEorM_OwnEligibleParties().values().toArray();
    short shiftingBack = 0;
      // return value
    GroupID[] groupSet;
    GroupID[] newArray = new GroupID[list.length];

    // select the GroupID in _acs
    for (int  i=0 ; i < list.length ; i++) {
      if (list[i] == null || !(list[i] instanceof GroupID) ) shiftingBack++;
      else newArray[i - shiftingBack] = (GroupID)list[i];
    } // for

    // build up and sort groupSet
    if (list.length != shiftingBack) {
      groupSet = new GroupID[list.length - shiftingBack];
      System.arraycopy(newArray, 0, groupSet, 0, newArray.length - shiftingBack);
      try {
        Arrays.sort(groupSet);
      } catch (ClassCastException c) {throw new InternalError(
          "in ACS_Facade.getGroupArray(), ClassCastException in Arrays.sort()");}
    }
    else groupSet = new GroupID[1]; // one null member
    return groupSet;
  } // getOwnGroupArray()

  /**
  * Builds up an ordered array of references to the ACS own userIDs.
  * @param _acs is an ACS
  * @return array of UserID in _acs.
  *   it is "new UserID[1]" if the array is empty.
  */
  public static UserID[] getOwnUserArray(ACS _acs){
    // variables
    Object[] list = _acs.getEorM_OwnEligibleParties().values().toArray();
    short shiftingBack = 0;
      // return value
    UserID[] userSet;
    UserID[] newArray = new UserID[list.length];

    // select the userID in _acs
    for (int  i=0 ; i < list.length ; i++) {
      if (list[i] == null || !(list[i] instanceof UserID) ) shiftingBack++;
      else newArray[i - shiftingBack] = (UserID)list[i];
    } // for

    // build up and sort userSet
    if (list.length != shiftingBack) {
      userSet = new UserID[list.length - shiftingBack];
      System.arraycopy(newArray, 0, userSet, 0, newArray.length - shiftingBack);
      try {
        Arrays.sort(userSet);
      } catch (ClassCastException c) {throw new InternalError(
          "in ACS_Facade.getUserArray(), ClassCastException in Arrays.sort()");}
    }
    else userSet = new UserID[1]; // one null member
    return userSet;
  } // getOwnUserArray()


  /**
  * Builds up an array of references to the ACS own actors.
  * @param _acs is an ACS
  * @return array in _acs.
  *   it is "new Actor[1]" if the array is empty.
  */
  public static Actor[] getOwnActorArray(ACS _acs){
    if (_acs.getEorM_OwnActors().values().isEmpty()) return new Actor[1];
    Actor[] result = new Actor[_acs.getEorM_OwnActors().size()];
    int i = 0;
    for (Iterator ite = _acs.getEorM_OwnActors().values().iterator() ; ite.hasNext() ; ) {
      result[i] = (Actor) ite.next();
      i++;
    }
    return result;
  } // getOwnActorArray()


  /**
  * Returns up an ordered array of references to the ACS own AclEntries.
  * The order criterion are, first, the EligibleParty class (UserID, then GroupID,
  * then Actor), and, second, the EligibleParty name in its natural order.
  * An AclEntry may include references to other-ACS objects.
  * The access-target object may be an 'external controlled resource' (including actor)
  * owned by another ACS. The right user may be an 'external eligible party'
  * (an actor, user or group) owned by another ACS.
  * The external AclEntries are added by ACSTreeBaseListenerImpl.changeOneAclEntry().
  * Called by Gui1.ACSTree.createOwnAclGraphicalTree().
  * @param _acs ACS
  * @return ImmutableAclEntry[] array in _acs. Never null. May be empty.
  */
  public static ImmutableAclEntry[] getOwnAclEntryArray(ACS _acs){

    ImmutableAclEntry[] l_acl = _acs.getEorL_AclEntries();

    if (l_acl == null) return new ImmutableAclEntry[0]; // one null member
    if (l_acl.length == 1
        && l_acl[0].getEorEP() == null )  return new ImmutableAclEntry[0];

                // for testing and tracking
                //System.out.println("ACS_Facade.getOwnAclEntryArray() result for "+_acs.getName());
                //for (int i = 0 ; i < l_acl.length ; i++ ) {
                  //System.out.println("--- at "+i+": "+l_acl[i] );
                //}

    return l_acl;
  }

  /**
  * Builds up an array of references to the ACS external controlled actors
  * (not eligible ones) for which an aclEntry in this ACS apply some rights,
  * seeing the actor as a resource belonged by an another ACS.
  * @param _acs is an ACS
  * @return array in _acs.
  *   it is "new Actor[1]" if the array is empty.
  */
  public static Actor[] getExtControlledActorArray(ImmutableACS _acs){
    // variables
    Map actorsMap = _acs.getEorM_Actors();
    if (actorsMap.values().isEmpty()) return new Actor[1];
    Map ownActorsMap = _acs.getEorM_OwnActors();
    Actor[] actArray = new Actor[actorsMap.size()];
    Actor[] result;
    Actor actor = new ActorImpl();
    int i = 0;
    int count = 0;

    for (Iterator ite = actorsMap.values().iterator() ; ite.hasNext() ; ) {
      actor = (Actor) ite.next();
      if (ownActorsMap.containsValue(actor) ) {
        count++;
        i++;
        continue;
      }
      actArray[i - count] = actor;
      i++;
    } // for

    if (actArray.length == count) {
      ownActorsMap = null;
      actArray = null;
      actor = null;
      return new Actor[1]; // one null member
    }
    result = new Actor[actArray.length - count];
    System.arraycopy(actArray, 0, result, 0, actArray.length - count);

    actorsMap = null;
    ownActorsMap = null;
    actArray = null;
    actor = null;

        // for testing and tracking
          //System.out.println("in gWork.ACS_Facade.getExtControlledActorArray(), the result is: ");
          //for (int j = 0; j < result.length; j++) {
          //  System.out.println("   at "+j+": "+result[j]);
          //}

    return result;
  } // getExtControlledActorArray()

  /**
  * Builds up an array of references to the ACS external actors
  * (not controlled!) for which an external AclEntry or Linked Privilege
  * in this ACS delivers a specific right.
  * The array contains no duplicate elements.
  * Called by ACSTree.getExtEligibleActorTree().
  * @param _acs is an ACS
  * @return array in _acs. May be null.
  */
  public static Actor[] getExtEligibleActorArray(ImmutableACS _acs){
    // never null
    Map<ImmutableName, ImmutableEligibleParty> m_ext = _acs.getEorM_ExternalEligibleParties();
    List<GroupID> list = new ArrayList(m_ext.values());
    ImmutableEligibleParty ep = null;
    Actor[] result;

    for (Iterator ite = list.iterator(); ite.hasNext(); ) {
        ep = (ImmutableEligibleParty)ite.next();
        if (!(ep instanceof Actor)) ite.remove();
    }

    if (list.isEmpty()) {
        m_ext = null;
        list = null;
        ep = null;
        return null;
    }

    result = list.toArray(new Actor[0]);

    m_ext = null;
    list = null;
    ep = null;
    return result;
  } // getExtEligibleActorArray()

  /**
  * Builds up an ordered array of references to the ACS external
  * eligible GroupIDs, without duplicates.
  * @param _acs is an ACS
  * @return array of external GroupID in _acs. May be null.
  */
  public static GroupID[] getExtEligibleGroupArray(ImmutableACS _acs){
    // never null
    Map<ImmutableName, ImmutableEligibleParty> m_ext = _acs.getEorM_ExternalEligibleParties();
    List<GroupID> list = new ArrayList(m_ext.values());
    ImmutableEligibleParty ep = null;
    GroupID[] result;

    for (Iterator ite = list.iterator(); ite.hasNext(); ) {
        ep = (ImmutableEligibleParty)ite.next();
        if (!(ep instanceof GroupID)) ite.remove();
    }

    if (list.isEmpty()) {
        m_ext = null;
        list = null;
        ep = null;
        return null;
    }

    result = list.toArray(new GroupID[0]);

    m_ext = null;
    list = null;
    ep = null;
    return result;
  } // getExtEligibleGroupArray()
  
  /**
  * Build up an ordered array of references to the ACS external
  * eligible UserIDs, without duplicates.
  * @param _acs is never null
  * @return array of external UserID in _acs. May be null.
  */
  public static UserID[] getExtEligibleUserArray(ImmutableACS _acs){
    // never null
    Map<ImmutableName, ImmutableEligibleParty> m_ext = _acs.getEorM_ExternalEligibleParties();
    List<UserID> list = new ArrayList(m_ext.values());
    ImmutableEligibleParty ep = null;
    UserID[] result;

    for (Iterator ite = list.iterator(); ite.hasNext(); ) {
        ep = (ImmutableEligibleParty)ite.next();
        if (!(ep instanceof UserID)) ite.remove();
    }

    if (list.isEmpty()) {
                // for testing and tracking
              //System.out.println("in ACS_Facade.getExtEligibleUserArray(),"+
                //" no external UserIDs in "+_acs.getNickName());

        m_ext = null;
        list = null;
        ep = null;
        return null;
    }

    result = list.toArray(new UserID[0]);

    m_ext = null;
    list = null;
    ep = null;
    return result;
  } // getExtEligibleUserArray()

  /**
  * Build up an array of references to the ACS external controlled resources
  * that are NOT actors and for which an external-oriented aclEntry,
  * in this ACS, applies some rights.
  * Each external resource is belonged by an another ACS.
  * @param _acs ACS
  * @return Resource[] array in _acs.
  *   it is "new Resource[1]" if the array is empty.
  */
  public static Resource[] getExtControlledNonActorResourceArray(ImmutableACS _acs){
    if (_acs.getEorM_ExternalResources().values().isEmpty())
        return new Resource[1]; // one null member
    // variables
    Resource[] resArray = new Resource[
        _acs.getEorM_ExternalResources().values().size()];
    int count = 0;
    Resource resource = new ResourceImpl();
    for (Iterator ite = _acs.getEorM_ExternalResources().values().iterator() ;
        ite.hasNext() ; ) {
      resource = (Resource) ite.next();
      if (!(resource instanceof ImmutableActor)) {
        resArray[count] = resource;
        count++;
      }
    } // for ite
    // match the array to the right size
    if (count == 0) return new Resource[1]; // one null member
    Resource[] result = new Resource[count];
    System.arraycopy(resArray, 0, result, 0, count);
    // sort the result
    try {
      Arrays.sort(result);
    } catch (ClassCastException c) {throw new InternalError(
          "in ACS_Facade.getExtControlledNonActorResourceArray(), ClassCastException in Arrays.sort()");}
    resource = null;
    resArray = null;

      //System.out.println("in gWork.ACS_Facade.getExtControlledNonActorResourceArray(), the result is: ");
      //for (int j = 0; j < result.length; j++) {
        //System.out.println("   at "+j+": "+result[j]);
      //}

    return result;
  } // getExtControlledNonActorResourceArray()

  /**
  * Build up an ordered array of decreasing references to the acsrights, after a filtering
  * of the metarights.
  * Called by ACSTree.createAcsRightGraphicalTree() and so, the acsrights
  * list in the explorer does not include the metarights used by the ACS.
  * Called by ActionNewACSyst.selectACS().
  * @param _acs is an ACS or an ACSFactoryImpl
  * @return array of acsrights in _acs. May be null.
  */
  public static StringRight[] getAcsRightArray(ACSRun _acs){
    // variables
    StringRight[] list = _acs.getL_Rights();
      // return value
    StringRight[] l_result = null;

    // select the acsrights
    for (int  i = 0 ; i < list.length ; i++) {
      if (list[i] != null && !(list[i] instanceof StringMetaRight) )
        l_result = BaseUtilityImpl.addRightInDecreasingSort(l_result, list[i]);
    } // for

    list = null;
    return l_result;
  } // getAcsRightArray()  

  /**
  * Gets the BaseObject object. It may be an open ACS, an open ViewInBase,
  * a metaright, or an ACSObject in an open ACS.
  * @param _name of the object
  * @return the object
  * @exception BaseError if there is no object with this name
  */
  public static BaseObject getEorBaseObject(ImmutableName _name) throws BaseError {
    return BaseUtilityImpl.getEorBaseObject(_name);
  } //getEorBaseObject()

  /**
  * Gets the ACSObject from its detailled name. It must be an AclEntry or a Privilege.
  * Calls the relevant ACS methods, first to try to get an AclEntry then to get a Privilege.
  * Called by TreeManager.getTrueUserObjectInTreeExplorer().
  * @param _acs of the object
  * @param _detailledName of the object
  * @return the object
  * @exception BaseError if there is no object with this name
  */
  public static ACSObject getACSObjectFromDetailledName(ACS _acs, String _detailledName)
    throws BaseError {

      ACSObject obj = _acs.getOneAclEntryFromDetailledName(_detailledName);
      if (obj != null) return obj;
      obj = _acs.getOnePrivilegeFromDetailledName(_detailledName);
      if (obj != null) return obj;
      else throw new BaseError("Unknown ACS object in the ACS "+_acs.getNickName());
  } // getACSObjectFromDetailledName()

  /**
   * Get the name from the string returned by toString().
   * @param _text the name, as get through toString()
   * @return the name
   * @exception BaseError if the string argument is null or empty,
   *  or if the string is not ended by ':: '.
   */
  public static Name getNameFromString(String _text) throws BaseError {
      return NameImpl.getNameFromString(_text);
  } // getNameFromString()   
    
  /**
  * Gets the AclEntry from its base object name, with a format like
  * ACS:: acsName::  ELIGIBLEPARTY:: epName::  >> RESOURCE:: resName || positiveRight
  * Called by ActionOpenBeamer.openSelection().
  * @param _name name of the AclEntry
  * @return the AclEntry
  * @exception BaseError if the name has a wrong format, or
   * if it is not a known AclEntry
   */
  public static ImmutableAclEntry getAclEntry(ImmutableName _name) throws BaseError {

    if (_name == null || _name.noOfComponents() < 9)
        throw new BaseError("In ACS_Facade.getAclEntry(), "+
            "the name is null or too short to have the right format.");

      // initialization
    ACS             acs = null;
    ImmutableName   acsName = null;
    int             epIndex = 0;
    Object[]        l_components = _name.immutableComponents().toArray();
    String          detailledName = null;
    ImmutableAclEntry result = null;
    
    for (int i = 0 ; i < l_components.length ; i++) {
      if ( l_components[i].equals("ELIGIBLEPARTY") ) {
          epIndex = i ;
      }
    } // for
    if (epIndex == 0) throw new BaseError("In ACS_Facade.getAclEntry(), "+
      "the name does not include the component 'ELIGIBLEPARTY'.");

    acsName = _name.subname(1, epIndex);
    
    // controls the ACS
    acs = (ACS) BaseManagerImpl.getInstance().getEorM_OpenACS().get(acsName);
    if (acs == null)
        throw new BaseError("In ACS_Facade.getAclEntry(), "+
            "the AclEntry belongs to a closed ACS or is unknown.");

    detailledName = AclEntryImpl.EMPTY_INSTANCE.getDetailledNameFromName(_name);

    result = acs.getOneAclEntryFromDetailledName(detailledName);

                // for testing and tracking
                //System.out.println("ACS_Facade.getAclEntry() name as argument: "+
                  //  _name);
                //System.out.println("with the acs: "+
                  //  acs.getName());
                //System.out.println("with the detailled name: "+
                  //  detailledName);
                //System.out.println("and the resulting AclEntry: "+
                  //  result);

    acs = null;
    acsName = null;
    l_components = null;
    detailledName = null;
    return result;
  } // getAclEntry()
    
  /**
  * Gets the Privilege from its base object name.
  * The Privilege name is composed of: <P>
  * ACS:: ACS name <P>
  * ::  || PRIV_TYPE::  type <P>
  * ::  || SHORT_NAME::  shortPrivilege <P>
  * ::  || FIRST_RIGHT::  right name <P>
  * ::  || XXXX::  xxxx value <P>
  * ::  >> YYYY::  yyyy value <P>
  * where: <P>
  *   - ':: ' is added by gBase.NameImpl.toString(), <P>
  *   - the type is one of the types defined in the PrivilegeAbst class, <P>
  *   - XXXX and YYYY are implementation-dependant values (see the relevant subclasses
  *     of PrivilegeAbst). <P>
  * Called by ActionOpenBeamer.openSelection().
  * @param _name the name of the Privilege
  * @return the Privilege
  * @exception BaseError if the privilege name has a wrong format, or
   * if it is an unknown privilege
   */
  public static ImmutablePrivilege getPrivilege(ImmutableName _name) throws BaseError {

    ImmutablePrivilege  result = null;
    String              detailledName = null;
    ImmutableACS        acs = null;

    acs = PrivilegeAbst.EMPTY_INSTANCE.getAcsFromName(_name);
    
    detailledName = PrivilegeAbst.EMPTY_INSTANCE.getDetailledNameFromName(_name);

    if (detailledName == null)
        throw new BaseError("In ACS_Facade.getPrivilege(), "+
            "the Privilege name has not the right format.");

    result = acs.getOnePrivilegeFromDetailledName(detailledName);

    detailledName = null;
    acs = null;
    return result;
  } // getPrivilege()

  /**
  * Validate a name, mainly as a title of an internal frame,
  * as without ' ::' nor ' >>', nor '<>' or ViewInBase_Facade.NOTHAN_VIEW_INDICATOR, 
  * nor non-null and not empty.
  * Display a warning message to user if _name contains a forbidden sequence.
  * @param _name to validate
  * @return boolean true if the name is good
  */
  public static boolean validateFrameName(ImmutableName _name) {
    if (_name == null || _name.isEmpty()) return false;
    
    for (int i = 0 ; i < _name.noOfComponents() ; i++) {
        if (_name.component(i).indexOf(" ::") != -1 ) {
              DialogBox.showOkMessage(
                "The sequence ' ::' is not allowed in a name.");
              return false;
        }
        if (_name.component(i).indexOf(" >>") != -1 ) {
              DialogBox.showOkMessage(
                "The sequence ' >>' is not allowed in a name.");
              return false;
        }
        if (_name.component(i).indexOf("<") != -1 || _name.component(i).indexOf(">") != -1) {
              DialogBox.showOkMessage(
                "'<' and '>' are not allowed in a name ");
              return false;
        }       
        if (_name.component(i).indexOf(ViewInBase_Facade.NOTHAN_VIEW_INDICATOR) != -1) {
              DialogBox.showOkMessage(
                ViewInBase_Facade.NOTHAN_VIEW_INDICATOR+" is not allowed in a name ");
              return false;
        }  
    } // for

    return true;
  } // validateName()

   /**
    * Imports an ACS, opens and saves it.
    * Called by ActionImportACS. This is a wrapper to call BaseManagerImpl.importACS().
    * @see ARoad0.gBase.BaseManagerImpl#importACS(ARoad0.gBaseInterface.ImmutableName) 
    * @param _name ImmutableName of the ACS to import. Its file name is built
    *   with the two last components of the name, or with the single component
    * @exception BaseError if the ACS name is known or null argument, or SecurityException
    * @exception DialogError if the open ACS has not the same name,
    *   or ClassNotFoundException, IOException with
    *   invalidClassException, or wrong saving
   * @return the map of the closed external ACSObjects which have a Bridge,
   * AclEntries or linked Privileges in this imported ACS. May be empty but not null.
   */
  public static synchronized Map<String, List<String>> getImportedACS(
          ImmutableName _name) throws BaseError, DialogError {
    return BaseManagerImpl.getInstance().importACS(_name);
  } // getImportedACS()

  /**
  * Builds up an ordered array of references to the ACS own Resource VirtualFolders.
  * @param _acs is an ACS
  * @return list of VirtualFolders in the ACS. May be empty but not null.
  */
  public static List<VirtualFolder> getOwnResourceVirtualFolderList(ACS _acs) {
    if (!_acs.getManageResourceVirtualFolders()) return Collections.EMPTY_LIST;
    else return _acs.getEorL_ResourceVirtualFolders();
  } // getOwnResourceVirtualFolderList()

  /**
  * Builds up an ordered array of references to the ACS own Eligible Party VirtualFolders.
  * @param _acs is an ACS
  * @return list of VirtualFolders in the ACS. May be empty but not null.
  */
  public static List<VirtualFolder> getOwnEPVirtualFolderList(ACS _acs) {
    if (!_acs.getManageEPVirtualFolders()) return Collections.EMPTY_LIST;
    else return _acs.getEorL_EPVirtualFolders();
  } // getOwnEPVirtualFolderList()

  /**
  * Builds up an ordered array of references to the ACS own Privileges.
  * The typed privileges are listed first, then the linked privileges.
  * The external AclEntries are added by ACSTreeBaseListenerImpl.changeOnePrivilege().
  * Called by Gui1.ACSTree.createOwnPrivilegeGraphicalTree().
  * @param _acs is an ACS
  * @return list of Privileges in the ACS. May be empty but not null.
  */
  public static List<Privilege> getOwnPrivilegeList(ACS _acs) {
    if (!_acs.getManagePrivilegeRights())
        return Collections.EMPTY_LIST;
    else {
        List<Privilege> l_result = _acs.getEorL_PrivilegeForTypes();
        l_result.addAll(_acs.getEorL_PrivilegeForLinks());
        return l_result;
    }
  } // getOwnPrivilegeList()
  
  //
  // --------------------- NON-STATIC METHODS -----------------------------------
  //


} // ACS_Facade
