//Title:        Access Road
//Version:      0.7.0
//Copyright:    2002, 2004, 2007-2011 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gWork;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.util.Collection;

import ARoad0.CNot.*;
import ARoad0.ErrB.*;
import ARoad0.gWorkInterface.*;

import ARoad0.gBase.BaseManagerImpl;
import ARoad0.gBase.UtilityImpl;
import ARoad0.gBaseInterface.*;
import ARoad0.ErrW.*;

/**
 * This class provides core algorithms about DisplayableLinks.
 * This class is stateless, and RightsMediatorImpl at its creation
 * setups an instance for its ViewInBase.
 * There is no direct call to the ACS methods.
 * It is used by the gWork and Gui2 classes.
 * @see ARoad0.CNot.DisplayableLinkImpl
 * @see ARoad0.gWorkInterface.CoreAlgorithm
 * @version 0.7.0
 */
public class DisplayableLinkUtilities implements CoreAlgorithm {
  private AlgorithmInterpreter      interpreter_ = null;
  private UtilityImpl               baseUtil_ = null;
  /**
   * To speed up mergeLinks()
   */
  private final ArrayList           resultVector_ = new ArrayList(101);

  /**
  * only one protected constructor
  * @param _interpreter algorithm interpreter
  */
  protected DisplayableLinkUtilities(AlgorithmInterpreter _interpreter) {
      interpreter_ = _interpreter;
      baseUtil_ = (UtilityImpl)interpreter_.getCoreAlgorithm("BaseUtil");

                    //for testing and tracking
                    //System.out.println("in DisplayableLinkUtilities constructor for "+
                      //  _interpreter.getSource().getName()+
                        //", resultVector_: "+resultVector_);
                    //for (Iterator ite = l_result.iterator() ; ite.hasNext() ; ) {
                      //System.out.println("-------"+ ((DisplayableLinkImpl)ite.next()).toString());
                    //}
  }

  /**
   * Gets all the first or all the second ends. Called by getDLinksForLoopsOnTarget().
   * Called by ThreeRightsFactoryImpl.
  * @param _linksCollection of DisplayableLinkImpls
  * @param _forFirstEnd false to get the second ends, true for the first ends
  * @exception InternalError if an element in _linksCollection is not a DisplayableLinkImpl.
  * @return Set of the BaseObjects which are all the first or all the second ends.
  **/
  public final Set getAllEndsInDisplayableLinks(
    Collection _linksCollection, boolean _forFirstEnd) {
      // variables
      Set                 l_result = new HashSet(INITIAL_CAPACITY / 3);
      DisplayableLinkImpl link;
      Object              obj;

      for (Iterator ita = _linksCollection.iterator(); ita.hasNext() ; ) {

          obj = ita.next();
          if (!(obj instanceof DisplayableLinkImpl)) throw new InternalError(
            "in RightsFactoryUtilities.getAllEndsInDisplayableLinks(), "+
            "an element in _linksCollection is not a DisplayableLinkImpl.");

          link = (DisplayableLinkImpl)obj;
          if (_forFirstEnd) l_result.add( link.getEorFirstEnd() );
          else l_result.add( link.getEorSecondEnd() );

      }
      link = null;
      obj = null;
      return l_result;
  } // getAllEndsInDisplayableLinks()

  /**
   * Get all the nodes,  including the intermediate ones, the first and the second ones.
  * @param _linksCollection of DisplayableLinkImpls
  * @exception InternalError if an element in _linksCollection is not a DisplayableLinkImpl.
  * @return Set of the relevant BaseObjects as path nodes
  **/
  public final Set getAllNodesInDisplayableLinks(
    Collection _linksCollection) {
      // variables
      Set                 l_result = new HashSet(INITIAL_CAPACITY);
      Object              obj;
      BaseObject[]        l_nodes;

      for (Iterator ita = _linksCollection.iterator(); ita.hasNext() ; ) {
          obj = ita.next();
          if (!(obj instanceof DisplayableLinkImpl)) throw new InternalError(
            "in RightsFactoryUtilities.getAllNodesInDisplayableLinks(), "+
            "an element in _linksCollection is not a DisplayableLinkImpl.");
          l_nodes = ((DisplayableLinkImpl)obj).getEorAllNodes();
          if (l_nodes == null) continue;
          for (int i = 0; i < l_nodes.length; i++) {
            l_result.add( l_nodes[i] );
          }
      } // for ita
      l_nodes = null;
      obj = null;
      return l_result;
  } // getAllNodesInDisplayableLinks()

  /**
  * Gets all the links which contain all the path, in the same order.
  * @param _linksCollection of DisplayableLinkImpls
  * @param _l_path to detect in each _linksCollection element. May be null.
  * @exception InternalError if an element in _linksCollection is not
  *   a DisplayableLinkImpl.
  * @return Set of the relevant DisplayableLinks. May be empty,
  *   but not null.
  **/
  public final Set<DisplayableLinkImpl> selectNodesPathInDisplayableLinks(
    Set<DisplayableLinkImpl> _linksCollection, BaseObject[] _l_path) {

      // variables
      Set<DisplayableLinkImpl>  l_result = new HashSet(INITIAL_CAPACITY);
      BaseObject[]        l_nodes;
      boolean             sameLink = true;

      if (_l_path == null || _l_path.length == 0) return l_result;

      for (DisplayableLinkImpl link: _linksCollection) {
          if (!(link instanceof DisplayableLinkImpl)) throw new InternalError(
            "in RightsFactoryUtilities.selectNodesPathInDisplayableLinks(), "+
            "an element in _linksCollection is not a DisplayableLinkImpl.");
          l_nodes = ((DisplayableLinkImpl)link).getEorAllNodes();
          if (l_nodes == null) continue;
          if (l_nodes.length < _l_path.length) continue;

          for (int j = 0 ; j < l_nodes.length - _l_path.length + 1; j++) {
            // compare the nodes one to one
            for (int i = 0 ; i < _l_path.length; i++) {
                if (l_nodes[i+j] != _l_path[i]) {
                  sameLink = false;
                  break;
                }
            } // for (int i
            // if obj is the good one
            if (sameLink) {
              l_result.add(link);
              break;
            } // if
            else sameLink = true;
          } // for j
      } // for link

                    //for testing and tracking
                    //System.out.println("in RightsFactoryUtilities."+
                      //"selectNodesPathInDisplayableLinks(), l_result:");
                    //for (Iterator ite = l_result.iterator() ; ite.hasNext() ; ) {
                      //System.out.println("-------"+ ((DisplayableLinkImpl)ite.next()).toString());
                    //}

      l_nodes = null;
      return l_result;
  } // selectNodesPathInDisplayableLinks()

  /**
  * Returns true if there are DisplayableLinkImpls for a couple of BaseObjects.
  * @param _ep is the first end of the DisplayableLinkImpl
  * @param _obj is the second end of the DisplayableLinkImpl. It is a resource or
  *   a GroupID.
  * @param _m_l_DisplayableLinks Map of DisplayableLinks lists (one per pair)
  * (_ep, _obj).
  * @return true if this couple has at least one DisplayableLinkImpl
  **/
  public final boolean containsDisplayableLinks(
     BaseObject _ep, BaseObject _obj, Map _m_l_DisplayableLinks) {
        // variables
        boolean     result = false;
        List        l_link;
        Name        pairKey;

        // constructs the _m_l_DisplayableLinks key
        // with the names of _ep and _res
        pairKey = ((Name)_ep.getName()).extended(_obj.getName());

        if (!_m_l_DisplayableLinks.containsKey(pairKey) ) {
          result = false;
        }
        else {
          l_link = (List) _m_l_DisplayableLinks.get(pairKey);
          if (l_link.isEmpty())
              result = false;
          else
              result = true;
        } // else

        l_link = null;
        pairKey = null;

        return result;
  } // containsDisplayableLinks()

  /**
  * Gets a DisplayableLinkImpl if it exists.
  * Compares the identity of the nodes, in the same order, but does not compare
  * the AccessControlLinkImpls,
  * Note: derived from the equals method in DisplayableLinkImpl; it declares
  * that 2 instances are equal if and only if the sequences of nodes are identical,
  * and if the nodes are identical, and not simply equal.
  * @param _ep is the first end of the DisplayableLinkImpl
  * @param _obj is the second end of the DisplayableLinkImpl. It is a resource or
  *   a GroupID.
  * @param _path BaseObject[] of the intermediate nodes in a DisplayableLink
  * to create or update, that is associated to the pair (_ep, _obj).
  * Is null when there is no intermediate nodes.
  * @param _m_l_DisplayableLinks Map of DisplayableLinks lists (one per pair)
  * (_ep, _obj).
  * @return a DisplayableLinkImpl in _m_l_DisplayableLinks, or null.
  **/
  public final DisplayableLinkImpl getDisplayableLink(
     BaseObject _ep, BaseObject _obj,
     BaseObject[] _path, Map _m_l_DisplayableLinks) {
        // variables
        DisplayableLinkImpl     result = null;
        List                    l_link;
        Name                    pairKey;
        BaseObject[]            l_nodes;
        boolean                 sameLink = false;

        // constructs the _m_l_DisplayableLinks key
        // with the names of _ep and _res
        pairKey = ((Name)_ep.getName()).extended(_obj.getName());

        if (!_m_l_DisplayableLinks.containsKey(pairKey) ) {
          return result;
        }
        else {
          // searching for the right DisplayableLinkImpl in the list
          // if it exists with the right nodes chain
          // in the same order
          l_link = (List) _m_l_DisplayableLinks.get(pairKey);
          for (Iterator ita = l_link.iterator() ; ita.hasNext() ; ) {
            result = (DisplayableLinkImpl)ita.next();
            l_nodes = result.getEorAllNodes();
            if (l_nodes == null) continue;
            if (l_nodes.length == 2 && _path == null) {
              sameLink = true;
              break;
            }
            if (_path == null || l_nodes.length != _path.length + 2) continue;

            sameLink = true;
            // compares the nodes one to one, without the l_nodes ends
            for (int i = 1 ; i < l_nodes.length - 1; i++) {
                if (l_nodes[i] != _path[i - 1]) {
                  sameLink = false;
                  break;
                }
            } // for (int i

            // tests if result is the good one
            if (sameLink) {
              break;
            }
            // else, continue the iteration
          } // for (Iterator ita

        } // else

        l_link = null;
        pairKey = null;
        l_nodes = null;

        if (sameLink) return result;
        else return null;
  } // getDisplayableLink()
  
  /**
  * Selects the DisplayableLinks which contain _l_path at the beginning of their
  * node chains. If _l_path is found as a complet chain, returns only this link.
  * Called by AgoRightsFactoryImpl.selectHiddenButNoDirectGroupLinks().
  * @param _l_path chain of nodes.
  * @param _m_l_DisplayableLinks Map of DisplayableLinks lists to analyze
  * @return Set of DisplayableLinkImpl in _m_l_DisplayableLinks, or null if _l_path
  *   is not in any DisplayableLink in the map.
  **/
  public final Set<DisplayableLinkImpl> selectDisplayableLinkWithStartPath(
     ACSObject[] _l_path, Map _m_l_DisplayableLinks) {

        if (_l_path == null || _l_path.length == 0) return null;

        // variables
        DisplayableLinkImpl     dLink;
        Set<DisplayableLinkImpl> result = new HashSet(INITIAL_CAPACITY / 5);
        List                    l_link;
        BaseObject[]            l_nodes;
        boolean                 sameLink = false;
        boolean                 found = false;

        for (Iterator ite = _m_l_DisplayableLinks.values().iterator() ; ite.hasNext() ; ) {
          l_link = (List) ite.next();

          for (Iterator ita = l_link.iterator() ; ita.hasNext() ; ) {
            dLink = (DisplayableLinkImpl)ita.next();
            l_nodes = dLink.getEorAllNodes();
            if (l_nodes == null) continue;
            if (l_nodes.length < _l_path.length) continue;

            sameLink = true;
            // compare the nodes one to one
            for (int i = 0 ; i < _l_path.length; i++) {
                if (l_nodes[i] != _l_path[i]) {
                  sameLink = false;
                  break;
                }
            } // for (int i

            // test if result is the good one
            if (sameLink) {
              result.add(dLink);
              found = true;
            }
            // else, continue the iteration
          } // for (Iterator ita


        } // for ite

                      //for testing and tracking
                      //System.out.println("in RightsFactoryUtilities."+
                      //  "selectDisplayableLinkWithStartPath(), is the result found? "+found);
                      //for (Iterator ite = result.iterator() ; ite.hasNext() ; ) {
                      //  System.out.println("-------"+ ((DisplayableLinkImpl)ite.next()).toString());
                      //}

        l_link = null;
        l_nodes = null;
        dLink = null;

        if (!found) return null;
        else return result;
  } // selectDisplayableLinkWithStartPath()

  /**
  * Selects all the DisplayableLinks for which a BaseObject is a second end
  * in a view.
  * @param _obj BaseObject
  * @param _m_l_DisplayableLinks Map of DisplayableLinks lists (one per pair).
  * @return Set of DisplayableLinks that contains _obj as a second end.
  * May be null but not empty.
  **/
  public final Set<DisplayableLinkImpl> selectDisplayableLinksForSecondEnd(
    BaseObject _obj, Map _m_l_DisplayableLinks) {
      // variables
      List l_dLinks;
      DisplayableLinkImpl link;
      Set<DisplayableLinkImpl> l_result = null;

      for (Iterator ite = _m_l_DisplayableLinks.values().iterator() ;
        ite.hasNext() ; ) {
          l_dLinks = (List) ite.next();
          if (((DisplayableLinkImpl)l_dLinks.get(0)).
            getEorSecondEnd() != _obj) continue;
          for (Iterator itu = l_dLinks.iterator(); itu.hasNext(); ) {
              link = (DisplayableLinkImpl)itu.next();
              if (link != null) {
                  if (l_result == null)
                      l_result = new HashSet(INITIAL_CAPACITY);
                  l_result.add(link);
              }
          } // for itu
      } // for (Iterator ite

      l_dLinks = null;
      link = null;
      return l_result;

  } // selectDisplayableLinksForSecondEnd()

  /**
  * Select all the DisplayableLinks for which a BaseObject is a first end
  * in a view.
  * @param _obj BaseObject
  * @param _l_DisplayableLinks is a Collection of DisplayableLinks.
  * @return Set of DisplayableLinks that contains _obj as a first end.
  * May be null but not empty.
  **/
  public final Set<DisplayableLinkImpl> selectDisplayableLinksForFirstEnd(
    BaseObject _obj, Collection _l_DisplayableLinks) {
      // variables
      DisplayableLinkImpl  dLink;
      Set<DisplayableLinkImpl> l_result = null;

      for (Iterator ite = _l_DisplayableLinks.iterator() ; ite.hasNext() ; ) {
          dLink = (DisplayableLinkImpl) ite.next();
          if (dLink.getEorFirstEnd() != _obj) continue;
          if (l_result == null)
              l_result = new HashSet(INITIAL_CAPACITY);
          l_result.add(dLink);
      } // for (Iterator ite

      dLink = null;
      return l_result;
  } // selectDisplayableLinksForFirstEnd()

  /**
  * Select all the DisplayableLinks for which a BaseObject is a second end
  * in a view.
  * @param _obj BaseObject
  * @param _l_DisplayableLinks is a Collection of DisplayableLinks.
  * @return Set of DisplayableLinks that contains _obj as a second end.
  * May be null but not empty.
  **/
  public final Set<DisplayableLinkImpl> selectDisplayableLinksForSecondEnd(
    BaseObject _obj, Collection _l_DisplayableLinks) {
      // variables
      DisplayableLinkImpl  dLink;
      Set<DisplayableLinkImpl> l_result = null;

      for (Iterator ite = _l_DisplayableLinks.iterator() ; ite.hasNext() ; ) {
          dLink = (DisplayableLinkImpl) ite.next();
          if (dLink.getEorSecondEnd() != _obj) continue;
          if (l_result == null)
              l_result = new HashSet(INITIAL_CAPACITY);
          l_result.add(dLink);
      } // for (Iterator ite

      dLink = null;
      return l_result;
  } // selectDisplayableLinksForSecondEnd()

  /**
  * Select all the DisplayableLinks for which a triplet of BaseObjects are nodes
  * in a continous sequence without other internmediate nodes.
  * Some of these nodes may be ends or not.
  * Called by removeTooNumerous3NodesSequence().
  * Call DisplayableLinkImpl.contains3NodesSequence(BaseObject[] _l_sequence).
  * @param _obj1 is the first BaseObject to test. Never null.
  * @param _obj2 is the second BaseObject to test has beeing just after _obj1. Never null.
  * @param _obj3 is the third BaseObject to test has beeing just after _obj2. Never null.
  * @param _l_DisplayableLinks is a Collection of DisplayableLinks. Never null.
  * @return Set of DisplayableLinks that contains the sequence _obj1/_obj2/_obj3.
  * May be empty. Never null.
  **/
  public final Set<DisplayableLinkImpl> selectDisplayableLinksWith3NodesSequence(
    BaseObject _obj1, BaseObject _obj2, BaseObject _obj3, Collection _l_DisplayableLinks) {
      
      // variables
      DisplayableLinkImpl  dLink;
      Set<DisplayableLinkImpl> l_result = new HashSet(INITIAL_CAPACITY);
      
      for (Iterator ite = _l_DisplayableLinks.iterator() ; ite.hasNext() ; ) {
          dLink = (DisplayableLinkImpl) ite.next();
          if (dLink.noOfNodes() < 3 || !dLink.containsBaseObject(_obj1) 
                  || !dLink.containsBaseObject(_obj2) || !dLink.containsBaseObject(_obj3)) 
                            continue;
          if (dLink.contains3NodesSequence(new BaseObject[] {_obj1, _obj2, _obj3}))
            l_result.add(dLink);
      } // for (Iterator ite

                  // for testing and tracking
                  //System.out.println("in RightsFactoryUtilities."+
                  //  "selectDisplayableLinksWith3NodesSequence(), result:");
                  //for (Iterator itetest = l_result.iterator();
                  //  itetest.hasNext(); ) {
                  //    System.out.println("---- "+itetest.next());
                  //} // for itetest

      dLink = null;
      return l_result;
  } // selectDisplayableLinksWith3NodesSequence()

  /**
  * Select all the DisplayableLinks for which a couple of BaseObjects are nodes
  * in a continous sequence without other internmediate nodes, and for a given first end.
  * Some of these nodes may be ends or not. 
  * Calls DisplayableLinkImpl.contains3NodesSequence(BaseObject[] _l_sequence).
  * @param _obj1 is the first BaseObject to test. Never null.
  * @param _obj2 is the second BaseObject to test has beeing just after _obj1. Never null.
  * @param _firstEnd is the first end to test. Never null.
  * @param _l_DisplayableLinks is a Collection of DisplayableLinks. Never null.
  * @return Set of DisplayableLinks that contains the sequence _obj1/_obj2, and _firstEnd
  *     as first end. May be empty. Never null.
  **/
  public final Set<DisplayableLinkImpl> selectDisplayableLinksWith2NodesSequenceAndFirstEnd(
    BaseObject _obj1, BaseObject _obj2, BaseObject _firstEnd, Collection _l_DisplayableLinks) {
      
      // variables
      DisplayableLinkImpl  dLink;
      Set<DisplayableLinkImpl> l_result = new HashSet(INITIAL_CAPACITY);
      
      for (Iterator ite = _l_DisplayableLinks.iterator() ; ite.hasNext() ; ) {
          dLink = (DisplayableLinkImpl) ite.next();
          if (dLink.noOfNodes() < 3 || !dLink.containsBaseObject(_obj1) 
                  || !dLink.containsBaseObject(_obj2) || !dLink.containsBaseObject(_firstEnd)) 
                            continue;
          if (dLink.contains2NodesSequenceAndFirstEnd(new BaseObject[] {_obj1, _obj2}, _firstEnd))
            l_result.add(dLink);
      } // for (Iterator ite

                  // for testing and tracking
                  //System.out.println("in RightsFactoryUtilities."+
                    //"selectDisplayableLinksWith2NodesSequenceAndFirstEnd(), result:");
                  //for (Iterator itetest = l_result.iterator();
                    //itetest.hasNext(); ) {
                      //System.out.println("---- "+itetest.next());
                  //} // for itetest

      dLink = null;
      return l_result;
  } // selectDisplayableLinksWith2NodesSequenceAndFirstEnd()

  /**
  * Select all the DisplayableLinks for which a BaseObject is a node,
  * and for a given first end.
  * The node may be an end or not. 
  * @param _obj1 is the first BaseObject to test. Never null.
  * @param _firstEnd is the first end to test. Never null.
  * @param _l_DisplayableLinks is a Collection of DisplayableLinks. Never null.
  * @return Set of DisplayableLinks that contains _obj1 and _firstEnd
  *     as first end. May be empty. Never null.
  **/
  public final Set<DisplayableLinkImpl> selectDisplayableLinksWithOneNodeAndFirstEnd(
    BaseObject _obj1, BaseObject _firstEnd, Collection _l_DisplayableLinks) {
      
      // variables
      DisplayableLinkImpl  dLink;
      Set<DisplayableLinkImpl> l_result = new HashSet(INITIAL_CAPACITY);
      
      for (Iterator ite = _l_DisplayableLinks.iterator() ; ite.hasNext() ; ) {
          dLink = (DisplayableLinkImpl) ite.next();
          if (dLink.noOfNodes() < 2 || !dLink.containsBaseObject(_obj1) 
                  || dLink.getEorFirstEnd() != _firstEnd)
                            continue;
          l_result.add(dLink);
      } // for (Iterator ite

                  // for testing and tracking
                  //System.out.println("in RightsFactoryUtilities."+
                    //"selectDisplayableLinksWithOneNodeAndFirstEnd(), result:");
                  //for (Iterator itetest = l_result.iterator();
                    //itetest.hasNext(); ) {
                      //System.out.println("---- "+itetest.next());
                  //} // for itetest

      dLink = null;
      return l_result;
  } // selectDisplayableLinksWithOneNodeAndFirstEnd()

  /**
  * Select all the DisplayableLinks for which a BaseObject is a node,
  * and for a given second end.
  * The node may be an end or not. 
  * @param _obj1 is the first BaseObject to test. Never null.
  * @param _secondEnd is the second end to test. Never null.
  * @param _l_DisplayableLinks is a Collection of DisplayableLinks. Never null.
  * @return Set of DisplayableLinks that contains _obj1 and _secondEnd
  * as a second end. May be empty. Never null.
  **/
  public final Set<DisplayableLinkImpl> selectDisplayableLinksWithOneNodeAndSecondEnd(
    BaseObject _obj1, BaseObject _secondEnd, Collection _l_DisplayableLinks) {
      
      // variables
      DisplayableLinkImpl  dLink;
      Set<DisplayableLinkImpl> l_result = new HashSet(INITIAL_CAPACITY);
      
      for (Iterator ite = _l_DisplayableLinks.iterator() ; ite.hasNext() ; ) {
          dLink = (DisplayableLinkImpl) ite.next();
          if (dLink.noOfNodes() < 2 || !dLink.containsBaseObject(_obj1) 
                  || dLink.getEorSecondEnd() != _secondEnd)
                            continue;
          l_result.add(dLink);
      } // for ite

                  // for testing and tracking
                  //System.out.println("in DisplayableLinkUtilities."+
                    //"selectDisplayableLinksWithOneNodeAndSecondEnd(), result:");
                  //for (Iterator itetest = l_result.iterator();
                    //itetest.hasNext(); ) {
                      //System.out.println("---- "+itetest.next());
                  //} // for itetest

      dLink = null;
      return l_result;
  } // selectDisplayableLinksWithOneNodeAndSecondEnd()

  /**
  * Select all the DisplayableLinks for which a BaseObject is a node,
  * and for a given couple of ends.
  * The node may be an end or not. 
  * Called by CompoundRightsFactoryimpl.detectHiddenCompoundEpRights().
  * @param _obj1 is the first BaseObject to test. May be an end. Never null.
  * @param _firstEnd is the first end to test. Never null.
  * @param _secondEnd is the first end to test. Never null.
  * @param _l_DisplayableLinks is a Collection of DisplayableLinks. Never null.
  * @return Set of DisplayableLinks that contains _obj1 and _firstEnd
  *     as first end. May be empty. Never null.
  **/
  public final Set<DisplayableLinkImpl> selectDisplayableLinksWithOneNodeAndTwoEnds(
    BaseObject _obj1, BaseObject _firstEnd, BaseObject _secondEnd, 
    Collection _l_DisplayableLinks) {
      
      // variables
      DisplayableLinkImpl  dLink;
      Set<DisplayableLinkImpl> l_result = new HashSet(INITIAL_CAPACITY);
      
      for (Iterator ite = _l_DisplayableLinks.iterator() ; ite.hasNext() ; ) {
          dLink = (DisplayableLinkImpl) ite.next();
          if (dLink.noOfNodes() < 3 || !dLink.containsBaseObject(_obj1) ||
                  dLink.getEorFirstEnd() != _firstEnd ||
                  dLink.getEorSecondEnd() != _secondEnd)
                            continue;
          l_result.add(dLink);
      } // for (Iterator ite

                  // for testing and tracking
                  //System.out.println("in RightsFactoryUtilities."+
                    //"selectDisplayableLinksWithOneNodeAndFirstEnd(), result:");
                  //for (Iterator itetest = l_result.iterator();
                    //itetest.hasNext(); ) {
                      //System.out.println("---- "+itetest.next());
                  //} // for itetest

      dLink = null;
      return l_result;
  } // selectDisplayableLinksWithOneNodeAndTwoEnds()
  
  /**
  * Merge each starting DisplayableLinkImpl with each ending DisplayableLinkImpl
  * to build up a new DisplayableLink which is put on the returned set.
  * So, if there are 2 starting DisplayableLinkImpls and 3 ending DisplayableLinkImpls,
  * the size of the returned set is 6.
  * All starting DisplayableLinkImpls must have the same second end, that
  * must be the first end of all the ending DisplayableLinkImpls. This BaseObject
  * is called the central object.
  * No new link if there is a loop in the new link. All the AccessControlLinkImpls
  * are cloned in the returned DisplayableLinkImpls.
  * Called by detectHiddenCompoundEpRights().
  * Call the mergeLinks() method.
  * @param _l_startingDisplayableLinks is a collection of DisplayableLinks
  *   for which the second node is associated to the central object.
  * @param _l_endingDisplayableLinks is a collection of DisplayableLinks
  *   for which the first node is associated to the central object.
  * @exception InternalError if a parameter is null, empty, if it is not
  *   a DisplayableLinkImpl collection, or if the central objet rule is not satisfyed.
  * @return Set of merged DisplayableLinks. Never empty.
  **/
  public final Set mergeTwoLinksCollection(
    Collection _l_startingDisplayableLinks, Collection _l_endingDisplayableLinks) {

      if (_l_startingDisplayableLinks == null || _l_endingDisplayableLinks == null )
            throw new InternalError("In RightsFactoryUtilities.mergeTwoLinksCollections(), "+
             "one parameter is null");
      if (_l_startingDisplayableLinks.isEmpty() || _l_endingDisplayableLinks.isEmpty() )
            throw new InternalError("In RightsFactoryUtilities.mergeTwoLinksCollections(), "+
             "one parameter is empty");
      if (!(_l_startingDisplayableLinks.toArray()[0] instanceof DisplayableLinkImpl) ||
        !(_l_endingDisplayableLinks.toArray()[0] instanceof DisplayableLinkImpl))
            throw new InternalError("In RightsFactoryUtilities.mergeTwoLinksCollections(), "+
             "the first parameter is not a DisplayableLinkImpl collection");      
      
      // variables
      DisplayableLinkImpl  dLink;
      DisplayableLinkImpl  dLink_2;
      Set                  l_result = new HashSet(INITIAL_CAPACITY*3); // size: 45

      for (Iterator ite = _l_startingDisplayableLinks.iterator() ; ite.hasNext() ; ) {
          dLink = (DisplayableLinkImpl) ite.next();

          for (Iterator itu = _l_endingDisplayableLinks.iterator() ;
            itu.hasNext() ; ) {
              dLink_2 = mergeLinks(dLink, (DisplayableLinkImpl) itu.next());

                        // for testing and tracking
                        //System.out.println("in RightsFactoryUtilities."+
                        //  "mergeTwoLinksCollection(), add dLink_2: "+dLink_2);

              if (dLink_2 != null) l_result.add(dLink_2);
          } // for itu

      } // for (Iterator ite

      dLink = null;
      dLink_2 = null;
      return l_result;
  } // mergeTwoLinksCollections()

  /**
  * Puts the ending DisplayableLinkImpl after the starting DisplayableLinkImpl
  * to build up a new DisplayableLink where the AccessControlLinkImpls are cloned.
  * The starting link second end must be the ending link first end. This BaseObject
  * is called the central object. The properties of _startingDisplayableLink
  * are copied in the result, while the properties of _endingDisplayableLink
  * are not copied. No operation if there is a loop in the new link.
  * Called by mergeTwoLinksCollections() and getDLinksForLoopsOnTarget().
  * @param _startingDisplayableLink is a link
  *   for which the second node is associated to the central object.
  * @param _endingDisplayableLink is a link
  *   for which the first node is associated to the central object.
  * @exception InternalError if a parameter is null, empty, or if
  *   the central objet rule is not satisfied.
  * @return a new DisplayableLinkImpl, or null if there is a loop in the new link.
  **/
  public final DisplayableLinkImpl mergeLinks(DisplayableLinkImpl
    _startingDisplayableLink, DisplayableLinkImpl _endingDisplayableLink) {

      if (_startingDisplayableLink == null || _endingDisplayableLink == null ||
        _startingDisplayableLink.isEmpty() || _endingDisplayableLink.isEmpty() )
            throw new InternalError("In RightsFactoryUtilities.mergeLinks(), "+
             "one parameter is null or empty");

      if (_startingDisplayableLink.getEorSecondEnd() !=
        _endingDisplayableLink.getEorFirstEnd() )
            throw new InternalError("In RightsFactoryUtilities.mergeLinks(), "+
             "there is no central object");

                    //for testing and tracking
                    //System.out.println("in DisplayableLinkUtilities.mergeLinks() for "+
                      //  interpreter_.getSource().getName()+
                       // " for the thread "+Thread.currentThread().getName()+
                        //", resultVector_: "+resultVector_);


      // variables where the AccessControlLinkImpl are cloned
      resultVector_.clear();
      resultVector_.addAll(_startingDisplayableLink.immutableComponents());
      Object[] l_endingLink = _endingDisplayableLink.immutableComponents().toArray();
      boolean doubleNode = false;

      resultVector_.remove(1);
      if (!resultVector_.contains(_endingDisplayableLink.getEorSecondEnd()))
        resultVector_.add(1, _endingDisplayableLink.getEorSecondEnd());
      else {
        l_endingLink = null;
        return null;
      }
      resultVector_.add(l_endingLink[0]);
      for (int i = 2; i < l_endingLink.length; i++ ) {
          if (!resultVector_.contains(l_endingLink[i]))
            resultVector_.add(l_endingLink[i]);
          else {
            doubleNode = true;
            break;
          }
      } // for ite

      if (doubleNode) {
          l_endingLink = null;
          return null;
      }

      // resultVector_ is copied
      DisplayableLinkImpl   result = new DisplayableLinkImpl(resultVector_);
      Map.Entry             entry = null;
      Map<String, Object>   m_properties = _startingDisplayableLink.getProperties();

      if (m_properties != null) {
          for (Object ob : m_properties.entrySet()) {
              entry = (Map.Entry)ob;
              if (entry.getValue() instanceof ImmutableName)
                  result.addProperty(
                          (String)entry.getKey(),
                          ((ImmutableName)entry.getValue()).clone());
              if (entry.getValue() instanceof Integer)
                  result.addProperty(
                          (String)entry.getKey(),
                          new Integer(((Integer)entry.getValue()).intValue()));
              if (entry.getValue() instanceof HashSet)
                  result.addProperty(
                          (String)entry.getKey(),
                          ((HashSet)entry.getValue()).clone());
              if (entry.getValue() instanceof ArrayList)
                  result.addProperty(
                          (String)entry.getKey(),
                          ((ArrayList)entry.getValue()).clone());
              else
                  result.addProperty(
                          (String)entry.getKey(),
                          entry.getValue());
          } // for
      } // if (m_properties != null)

      m_properties = null;
      entry = null;
      return result;

  } // mergeLinks()

  /**
  * Select the lists of intermediate nodes in the DisplayableLinks associated
  * to a BaseObject pair.
  * Called by NodeRightsImpl.selectHiddenButNoDirectGroupLinks().
  * @param _pairKey is  a key of _m_l_DisplayableLinks - for instance
  *   an eligible party name extended by a resource name.
  * @param _m_l_DisplayableLinks Map of DisplayableLinks lists (one per pair).
  * @return Set of BaseObject[] where the intermediate nodes are from the first
  *   intermediate node to the last one, without the first end but with the second end.
  *   May be empty.
  **/
  public final Set getIntermediateNodesListsForPair(
    ImmutableName _pairKey, Map _m_l_DisplayableLinks) {
      // variables
      List l_dLinks = (List)_m_l_DisplayableLinks.get(_pairKey);
      Set l_result = new HashSet(INITIAL_CAPACITY / 3);

      if (l_dLinks == null) return l_result;

      BaseObject[] l_nodes;
      BaseObject[] list;

      for (Iterator ite = l_dLinks.iterator() ; ite.hasNext() ; ) {
          l_nodes = ((DisplayableLinkImpl) ite.next()).getEorAllNodes();
          if (l_nodes.length < 3) continue;
          list = new BaseObject[l_nodes.length - 1];
          System.arraycopy(l_nodes, 1, list, 0, list.length);
          l_result.add(list);

                      //for testing and tracking
                      //System.out.println("in RightsFactoryUtilities."+
                      //  "getIntermediateNodesListsForPair(), for "+
                      //  _pairKey+", add a list:");
                      //for (int i = 0; i < list.length; i++ ) {
                      //  System.out.println("-------"+ list[i]);
                      //}

      } // for (Iterator ite

      l_dLinks = null;
      list = null;
      l_nodes = null;
      return l_result;

  } // getIntermediateNodesListsForPair()  
  
  
  /**
   * NO USE.
   * Copy the comments in the AccessControlLink before a node, into the
   * AccessControlLink after the node, in a DisplayableLinkImpl.
   * Copy all the XID comments (ACL, OWNER, CONTAIN, IS_MEMBER, IS_INDIRECT_MEMBER)
   * or copy all the comments, to the same comments type, or to the GLOBAL comments.
   * addComment() is used, so the old comments are not deleted
   * and the link types are not used.
   * @param _dLink contains the node to process; the AccessControlLink
   *   after the node is updated
   * @param _nodeIndex is the index of a _dLink node to process
   * @param _forXIDonly is true if only ACL, OWNER, CONTAIN, IS_MEMBER, IS_INDIRECT_MEMBER,
   *   and false  if all link types are to be copied
   * @param _toGLOBAL is true if the comments are copied to the GLOBAL link type,
   *   and false if each link type is copied to the same type
   *   (for example, ACL in ACL)
  * */
  public final void copyUpNodeComments(DisplayableLinkImpl _dLink,
    int _nodeIndex, boolean _forXIDonly, boolean _toGLOBAL) {

      BaseObject            node = _dLink.getEorAllNodes()[_nodeIndex];
      AccessControlLinkImpl[]  l_acLinks = _dLink.getEorConnectedLinks(node);

      if (l_acLinks[0].isEmpty()) return;
      if (_toGLOBAL) {
        l_acLinks[1].addComment(
          AccessControlLink.GLOBAL,
          l_acLinks[0].getLinkTypeComments(AccessControlLink.ACL));
        l_acLinks[1].addComment(
          AccessControlLink.GLOBAL,
          l_acLinks[0].getLinkTypeComments(AccessControlLink.OWNER));
        l_acLinks[1].addComment(
          AccessControlLink.GLOBAL,
          l_acLinks[0].getLinkTypeComments(AccessControlLink.CONTAIN));
        l_acLinks[1].addComment(
          AccessControlLink.GLOBAL,
          l_acLinks[0].getLinkTypeComments(AccessControlLink.IS_MEMBER));
        l_acLinks[1].addComment(
          AccessControlLink.GLOBAL,
          l_acLinks[0].getLinkTypeComments(AccessControlLink.IS_INDIRECT_MEMBER));
        
                      //for testing and tracking
                      //System.out.println("\n in RightsFactoryUtilities.copyUpNodeComments(), "+
                        //"if (_toGLOBAL) is true, \n --- node: "+node.getName()+
                        //"\n --- _dLink; "+_dLink+
                        //"\n All comments on l_acLinks[1] after the copy: "); 
                      //for (Iterator ite = l_acLinks[1].getAllComments().iterator(); ite.hasNext(); ) {
                          //System.out.println((String)ite.next());
                      //}
        
        if (!_forXIDonly) {
          l_acLinks[1].addComment(
            AccessControlLink.GLOBAL,
            l_acLinks[0].getLinkTypeComments(AccessControlLink.NEG_ACL));
          l_acLinks[1].addComment(
            AccessControlLink.GLOBAL,
            l_acLinks[0].getLinkTypeComments(AccessControlLink.SPREAD));
          l_acLinks[1].addComment(
            AccessControlLink.GLOBAL,
            l_acLinks[0].getLinkTypeComments(AccessControlLink.GLOBAL));
        } // if (!_forXIDonly) {
      }
      else {
        if (!_forXIDonly) l_acLinks[1].mergeCommentsWithoutFiltering(l_acLinks[0]);
        else {
          l_acLinks[1].addComment(
            AccessControlLink.ACL,
            l_acLinks[0].getLinkTypeComments(AccessControlLink.ACL));
          l_acLinks[1].addComment(
            AccessControlLink.OWNER,
            l_acLinks[0].getLinkTypeComments(AccessControlLink.OWNER));
          l_acLinks[1].addComment(
            AccessControlLink.CONTAIN,
            l_acLinks[0].getLinkTypeComments(AccessControlLink.CONTAIN));
          l_acLinks[1].addComment(
            AccessControlLink.IS_MEMBER,
            l_acLinks[0].getLinkTypeComments(AccessControlLink.IS_MEMBER));
          l_acLinks[1].addComment(
            AccessControlLink.IS_MEMBER,
            l_acLinks[0].getLinkTypeComments(AccessControlLink.IS_INDIRECT_MEMBER));
        } // else
      } // else

      l_acLinks = null;
      node = null;
  } // copyUpNodeComments()  

  /**
  * Removes in the argument the DisplayableLink which fulfills with this criterion:<P>
  * - contains a sequence of 3 nodes A-B-C.
  * - A-B-C is a sequence which is in 5 other DisplayableLinks in the argument.
  * It is recommanded to have an argument larger than 6 to avoid useless processing.
  * Called by RightsFactoryUtilities.getEPRWhyText().
  * @param _l_dLinksToUpdate to analyze, all with the same two ends. Not null. This
  *            argument is updated.
  * @return set of BaseObject[] having a length of 3. May be null.
  **/
  public final Set<BaseObject[]> removeTooNumerous3NodesSequence(
          List _l_dLinksToUpdate) {
    
                  // for testing and tracking
                  //System.out.println("DisplayableLinkUtilities.removeTooNumerous3NodesSequence()"+
                    //      " dLinks in the argument:");
                  //for (Iterator ite = _l_dLinksToUpdate.iterator(); ite.hasNext(); ) {
                    //  System.out.println("----- "+ite.next());
                  //}

      // variables
    List                    l_dLinks2 = new ArrayList(_l_dLinksToUpdate);
    DisplayableLinkImpl     dLink = null;
    DisplayableLinkImpl     dLink2 = null;
    BaseObject[]            l_obj = null;
    Set                     l_moreThanFive = new HashSet(INITIAL_CAPACITY);
    Set                     l_tested = new HashSet(INITIAL_CAPACITY);
    Set<BaseObject[]>       l_result = new HashSet(INITIAL_CAPACITY);
    Set                     l_toRemove = new HashSet(INITIAL_CAPACITY);
    int                     count = 0;
    Set                     l_dLinksWithSequence;

    // initialize l_moreThanFive which contains all the nodes that are
    // more than 5 times in the nodes of _l_dLinksToUpdate
    for (Iterator ite = _l_dLinksToUpdate.iterator(); ite.hasNext(); ) {
        dLink = (DisplayableLinkImpl)ite.next();
        l_dLinks2.remove(dLink);
        l_obj = dLink.getEorAllNodes();
        
        for (int i = 0; i < l_obj.length; i++) {
            count = 0;
            
            if (l_tested.contains(l_obj[i])) continue;
            else l_tested.add(l_obj[i]);
            
            for (Iterator itu = l_dLinks2.iterator(); itu.hasNext(); ) {
                dLink2 = (DisplayableLinkImpl)itu.next();
                
                if (dLink2.containsBaseObject(l_obj[i])) {
                    count++;
                    if (count > 4) {
                        l_moreThanFive.add(l_obj[i]);
                        break;
                    }
                }
            } // for itu
        } // for i
    } // for ite
    
    if (l_moreThanFive.isEmpty()) return null;
    
    // for each sequence A-B-C of nodes in a DisplayableLink
    // where each of 3 nodes is in l_moreThanFive, test if the sequence
    // is more than 5 times in _l_dLinksToUpdate
    l_tested.clear();
    count = 0;
    for (Iterator ite = _l_dLinksToUpdate.iterator(); ite.hasNext(); ) {
        dLink = (DisplayableLinkImpl)ite.next();
        if (l_toRemove.contains(dLink)) continue;
        l_obj = dLink.getEorAllNodes();
        if (l_obj.length < 3) continue;
        
        for (int i = 0; i < l_obj.length - 2; i++) { 
            if (!l_moreThanFive.contains(l_obj[i]) ||
                    !l_moreThanFive.contains(l_obj[i+1]) ||
                    !l_moreThanFive.contains(l_obj[i+2])) 
                            continue;
            if (l_tested.contains(new DisplayableLinkImpl(l_obj[i], l_obj[i+1],
                    new BaseObject[] {l_obj[i+2]}))) continue;
            
            l_dLinksWithSequence = selectDisplayableLinksWith3NodesSequence(
                l_obj[i], l_obj[i+1], l_obj[i+2], _l_dLinksToUpdate);
            
            // if there are 8 dLinks in l_dLinksWithSequence,
            // the 3 first ones are removed, even if there are simplest than the other ones.
            if (l_dLinksWithSequence.size() > 5) {
                while (l_dLinksWithSequence.size() > 5) {
                    Iterator ita = l_dLinksWithSequence.iterator();
                    l_toRemove.add(ita.next());
                    ita.remove();
                }
                l_result.add(new BaseObject[] {l_obj[i], l_obj[i+1], l_obj[i+2]});
                l_tested.add(new DisplayableLinkImpl(l_obj[i], l_obj[i+1],
                    new BaseObject[] {l_obj[i+2]}));
                break;
            } else l_tested.add(new DisplayableLinkImpl(l_obj[i], l_obj[i+1],
                    new BaseObject[] {l_obj[i+2]}));
        } // for i
    } // for ite

    for (Iterator ite = l_toRemove.iterator(); ite.hasNext(); ) {
        _l_dLinksToUpdate.remove(ite.next());
    }      
    
    l_dLinks2 = null;
    dLink = null;
    dLink2 = null;
    l_obj = null;
    l_moreThanFive = null;
    l_tested = null;
    l_toRemove = null;
    l_dLinksWithSequence = null;
    
                      // for testing and tracking
                      //System.out.println("DisplayableLinkUtilities.removeTooNumerous3NodesSequence()"+
                        //      " triplets in the result:");
                      //for (BaseObject[] array : l_result) {
                        //  System.out.println("-- one triplet: ");
                          //for (int i = 0; i < array.length; i++) {
                          //System.out.println("------- "+array[i]);
                          //} // for i
                      //} // for array
    return l_result;

  } // removeTooNumerous3NodesSequence()

  /**
  * This central method gets the current effective rights in a DisplayableLinkImpl,
  * and the rights for comments, with the list of all the implied rights.
  * This is the single method to get the true rights from a DisplayableLinkImpl.
  * A DisplayableLinkImpl may be effective even if it has no effective rights.
  * The result is never null to be well used by getFirstShapeEffectiveRights().
  * Called by RightsFactory_Facade.getEffectiveRightsForPair() and
  * getFirstShapeEffectiveRights().
  * Calls AccessControlLinkImpl.getFilteredRights(),
  * BaseUtilityImpl.subtractNegativeNotRemovingInOneACS(),
  * UtilityImpl.selectHeaders().
  * @param _dLink to analyze. Not null.
  * @param _withComment true to return the second set of detailled rights for comments
  * @return an array of two sets, where the first one is the effective rights
  *     (non-null set, may be empty), and the second one is the detailled rights
  *     for the 'See why' text. Never Null. May be empty.
  **/
  protected final Set<StringRight>[] getEffectiveRightsAndCommentOnRights(
          DisplayableLinkImpl _dLink, boolean _withComment) {

    // variables
    Set<StringRight>          l_rights  = new HashSet(INITIAL_CAPACITY);
    Set<StringRight>          l_forCommentRights = new HashSet(INITIAL_CAPACITY);
    Set<StringRight>[]        l_result = null;
    Set<StringRight>[]        l_l_rights = null;
    Set<StringRight>          l_tempRights  = null; 
    ImmutableACS              acs = null;
    boolean                   forNode = false;
    int                       immutableRange = 1;      

          
    if (_dLink.getEorSecondEnd() instanceof ACSObject)
        acs = (ImmutableACS) BaseManagerImpl.getInstance().getEorM_OpenACS().get(
              ((ACSObject)_dLink.getEorSecondEnd()).getAcsName());
    else acs = null;

    if (_dLink.getEorSecondEnd() instanceof Node) // also for VirtualFolder
        forNode = true;
    
    // implicit ACS rules are:
    // 'AGO right: A rights overlay G rights'
    // 'right type rule: root, A in AGO granting/denying rights are not changed by other rights'     
    if (_dLink.getEorLastLink().getLinkTypeValue(AccessControlLink.OWNER))
      l_l_rights = _dLink.getEorLastLink().getFilteredRights(immutableRange, true, true);
    else if (_dLink.getEorLastLink().getLinkTypeValue(AccessControlLink.CONTAIN)) // _withoutGroupInAGOrights false
      l_l_rights = _dLink.getEorLastLink().getFilteredRights(immutableRange, false, true);

    else { // _withoutOtherInAGOrights true for the 'run_under' and 'has_member' types
        if (_dLink.getEorLastLink().isTrueForOneStrengthenLinkType())
            l_l_rights = _dLink.getEorLastLink().getFilteredRights(immutableRange, false, true);
        else l_l_rights = _dLink.getEorLastLink().getFilteredRights(immutableRange, false, false);
    } // else

                      // for testing and tracking
                      //System.out.println("DisplayableLinkUtilities.getEffectiveRightsAndComment(), "+
                        //"for dLink: "+_dLink);
                      //System.out.println("----- positive immutable rights: ");
                      //for (Iterator ita = l_l_rights[0].iterator(); ita.hasNext(); ) {
                        //  System.out.println("------------ "+ita.next());
                      //} // for ite
                      //System.out.println("----- negative immutable rights: ");
                      //for (Iterator ita = l_l_rights[1].iterator(); ita.hasNext(); ) {
                        //  System.out.println("------------ "+ita.next());
                      //} // for ite   
                      //System.out.println("----- positive mutable rights: ");
                      //for (Iterator ita = l_l_rights[2].iterator(); ita.hasNext(); ) {
                        //  System.out.println("------------ "+ita.next());
                      //} // for ite
                      //System.out.println("----- negative mutable rights: ");
                      //for (Iterator ita = l_l_rights[3].iterator(); ita.hasNext(); ) {
                        //  System.out.println("------------ "+ita.next());
                      //} // for ite   


    if (l_l_rights[0].isEmpty() && l_l_rights[1].isEmpty() && l_l_rights[3].isEmpty()) {
        l_rights.addAll( l_l_rights[2] );
        if (_withComment) {
            
                        // for testing and tracking
                        //System.out.println("----- add to l_forCommentRights: ");
                          //for (Iterator ita = l_l_rights[2].iterator(); ita.hasNext(); ) {
                            //  System.out.println("------------ "+ita.next());
                          //} // for ite

            l_forCommentRights.addAll( l_l_rights[2] );
        } // if
    }
    else {
        // process the filtered positive immutable rights first
        if (!l_l_rights[0].isEmpty() && !l_l_rights[1].isEmpty()) {
            l_tempRights = baseUtil_.subtractNegativeNotRemovingInOneACS(
                l_l_rights[0], l_l_rights[1], acs, forNode); // acs may be null
            if (!l_tempRights.isEmpty())
                l_rights.addAll( l_tempRights ); 
            else if (l_l_rights[1].size() < 2)
                l_rights.addAll( l_l_rights[1] );
            else {
                try {
                    baseUtil_.selectHeaders(l_l_rights[1]);


                                      // for testing and tracking
                                      //System.out.println("l_headNegRights after selectHeaders():");
                                      //for (Iterator ite = l_headNegRights.iterator(); ite.hasNext(); ) {
                                              // for testing and tracking
                                        //      System.out.println(" ------------------------ "+ite.next());
                                      //} // for         
                }
                catch (BaseError b) {
                    throw new InternalError("in DisplayableLinkUtilities."+
                            "getEffectiveRightsAndComment(), "+
                        "from StringRightUtilityImpl.selectHeaders(): "+b);
                }
                l_rights.addAll( l_l_rights[1] );
            } // else

            if (_withComment) {

                        // for testing and tracking
                        //System.out.println("----- add to l_forCommentRights: ");
                          //for (Iterator ita = l_l_rights[0].iterator(); ita.hasNext(); ) {
                            //  System.out.println("------------ "+ita.next());
                          //} // for ite
                        // for testing and tracking
                        //System.out.println("----- add to l_forCommentRights: ");
                          //for (Iterator ita = l_l_rights[1].iterator(); ita.hasNext(); ) {
                            //  System.out.println("------------ "+ita.next());
                          //} // for ite

                l_forCommentRights.addAll( l_l_rights[0] );
                l_forCommentRights.addAll( l_l_rights[1] );
            } // if
        }
        else if (!l_l_rights[0].isEmpty() ) {
            l_rights.addAll(  l_l_rights[0] );
            
            if (_withComment) {

                        // for testing and tracking
                        //System.out.println("----- add to l_forCommentRights: ");
                          //for (Iterator ita = l_l_rights[0].iterator(); ita.hasNext(); ) {
                            //  System.out.println("------------ "+ita.next());
                          //} // for ite

                l_forCommentRights.addAll( l_l_rights[0] );
            } // if
        }

        // add all the other filtered rights
        if (!l_l_rights[2].isEmpty() && !l_l_rights[3].isEmpty()) {
            l_tempRights = baseUtil_.subtractNegativeNotRemovingInOneACS(
                l_l_rights[2], l_l_rights[3], acs, forNode); // acs may be null
            if (!l_tempRights.isEmpty())
                l_rights.addAll( l_tempRights ); 
            else if (l_l_rights[3].size() < 2)
                l_rights.addAll( l_l_rights[3] );
            else {
                try {
                    baseUtil_.selectHeaders(l_l_rights[3]);


                                      // for testing and tracking
                                      //System.out.println("l_headNegRights after selectHeaders():");
                                      //for (Iterator ite = l_headNegRights.iterator(); ite.hasNext(); ) {
                                              // for testing and tracking
                                        //      System.out.println(" ------------------------ "+ite.next());
                                      //} // for         
                }
                catch (BaseError b) {
                    throw new InternalError("in DisplayableLinkUtilities."+
                            "getEffectiveRightsAndComment(), "+
                        "from StringRightUtilityImpl.selectHeaders(): "+b);
                }
                l_rights.addAll( l_l_rights[3] );
            } // else

            if (_withComment) {

                        // for testing and tracking
                        //System.out.println("----- add to l_forCommentRights: ");
                          //for (Iterator ita = l_l_rights[2].iterator(); ita.hasNext(); ) {
                            //  System.out.println("------------ "+ita.next());
                          //} // for ite
                        // for testing and tracking
                        //System.out.println("----- add to l_forCommentRights: ");
                          //for (Iterator ita = l_l_rights[3].iterator(); ita.hasNext(); ) {
                            //  System.out.println("------------ "+ita.next());
                          //} // for ite

                l_forCommentRights.addAll( l_l_rights[2] );
                l_forCommentRights.addAll( l_l_rights[3] );
            } // if
        } // if
        else if (!l_l_rights[2].isEmpty() ) {
            l_rights.addAll( l_l_rights[2] ); 
            if (_withComment) {

                        // for testing and tracking
                        //System.out.println("----- add to l_forCommentRights: ");
                          //for (Iterator ita = l_l_rights[2].iterator(); ita.hasNext(); ) {
                            //  System.out.println("------------ "+ita.next());
                          //} // for ite

                l_forCommentRights.addAll( l_l_rights[2] );
            } // if
        }
        else if (!l_l_rights[3].isEmpty() ) {
            l_rights.addAll( l_l_rights[3] ); // add negative rights

            if (_withComment) {

                        // for testing and tracking
                        //System.out.println("----- add to l_forCommentRights: ");
                          //for (Iterator ita = l_l_rights[3].iterator(); ita.hasNext(); ) {
                           //   System.out.println("------------ "+ita.next());
                          //} // for ite

                l_forCommentRights.addAll( l_l_rights[3] );
            } // if
        }
    } // else   
    
    l_result = new Set[] {l_rights, l_forCommentRights};

                      // for testing and tracking
                      //System.out.println("DisplayableLinkUtilities.getEffectiveRightsAndComment(), "+
                        //"for dLink, at the end: "+_dLink);
                      //System.out.println("----- not-empty l_rights: "+!l_result[0].isEmpty());
                      //for (Iterator ita = l_result[0].iterator(); ita.hasNext(); ) {
                        //  System.out.println("------------ "+ita.next());
                      //} // for ite
                      //if (l_result[1] != null) {
                        //  System.out.println("----- final l_forCommentRights: ");
                          //for (Iterator ita = l_result[1].iterator(); ita.hasNext(); ) {
                            //  System.out.println("------------ "+ita.next());
                          //} // for ite
                      //} // if
    
    
    l_rights  = null;
    l_forCommentRights = null;
    l_l_rights = null;
    acs = null;
    l_tempRights  = null;
    
    return l_result;
  } // getEffectiveRightsAndComment()
  
  /**
   * UNUSED -
  * Analyzes a network of nodes and oriented links to detect the paths
  * which end to _target and may participate to some loops.
  * In the returned list, there are no crossroads where two DisplayableLinkImpls
  * have a commun intermediate node, but the aim is there to return those having
  * a common first end, plus _target as common second end, 
  * since this is the definition of a path loop.
  * Does not read the link types and the rights in each DisplayableLink.
  * Creates for each matched path a DisplayableLink without rights nor comments.
  * Intermediate nodes and AccessControlLinkImpls in _l_paths are not processed.
  * Performance: the number of search cycles is limited to 200.
  * Calls mergeLinks(), getAllEndsInDisplayableLinks().
  * Throw an InternalError if the search is in more than 200 cycles.
  * @param _target a target name present in _l_paths
  * @param _l_paths set of simple DisplayableLinkImpls, which define
  *         oriented links betwen the two end nodes, and where the link gives,
  *         at AccessControlLinkImpl.GLOBAL, the effective rights between the two ends. 
  * @return new simple or not-simple DisplayableLinks where _target is the name
  *      of the second end, node names are end names in _l_paths, and which may
  *     participate to a loop or not.
  **/
  public final List<DisplayableLinkImpl> getDLinksForLoopsOnTarget(
          ImmutableName _target, Collection<DisplayableLinkImpl> _l_paths) {
      // variables
    List<DisplayableLinkImpl>   l_result = new ArrayList(INITIAL_CAPACITY);
    List<DisplayableLinkImpl>   l_sourcesDLinks;
    List<DisplayableLinkImpl>   l_intermediateDLinks = new ArrayList(INITIAL_CAPACITY);
    Map<ImmutableName, List<DisplayableLinkImpl>>   m_dLinksFromSources = new HashMap(INITIAL_CAPACITY);
    DisplayableLinkImpl         dLink = null;
    DisplayableLinkImpl         dLink2 = null;
    boolean                     toContinue = false;
    ImmutableName               name = null;
    Map.Entry                   entry = null;
    int                         count = 0;
    
                      // for testing and tracking
                      //System.out.println("DisplayableLinkUtilities.getDLinksForLoopsOnTarget() starting for "+
                        //      _target);
    
    for (Iterator ita = _l_paths.iterator(); ita.hasNext(); ) {
        dLink2 = (DisplayableLinkImpl)ita.next();
        // if dLink is not simple, it should contain nodes which are not end
        // in any DisplayableLinkImpl of _l_paths. To avoid to check that,
        // only simple DisplayableLinkImpl are handled.
        if (dLink2.getEorSecondEnd().getName().equals(_target) && dLink2.isSimple()) {
            l_result.add(dLink2);
            l_sourcesDLinks = new ArrayList(INITIAL_CAPACITY);
            l_sourcesDLinks.add(dLink2);
            m_dLinksFromSources.put(dLink2.getEorFirstEnd().getName(), l_sourcesDLinks);
            toContinue = true;
        } // if
    } // for ita
    
    while (toContinue) {
        toContinue = false;
        count++;
        if (count > 200) {
            throw new InternalError("DisplayableLinkUtilities.getDLinksForTargetFromNameNetwork()"+
                    " in a search for more than 200 cycles");
        }
        
        for (Iterator ite = m_dLinksFromSources.entrySet().iterator(); ite.hasNext(); ) {
            entry = (Map.Entry) ite.next();
            name = (ImmutableName)entry.getKey();
            for (Iterator ita = _l_paths.iterator(); ita.hasNext(); ) {
                dLink = (DisplayableLinkImpl)ita.next();
                
                if (dLink.getEorSecondEnd().getName().equals(name) ) {
                    for (Iterator itu = ((List)entry.getValue()).iterator(); itu.hasNext(); ) {
                        dLink2 = (DisplayableLinkImpl)itu.next();
                        
                        dLink2 = mergeLinks(dLink, dLink2); // null if contain loops
                        if (dLink2 != null) { 
                            // if the first end is already known as the first end
                            // of a dLink in l_result, no adding for further processing
                            // of this first end as second end
                            if (!getAllEndsInDisplayableLinks(l_result, true).
                                    contains(dLink2.getEorFirstEnd()) )
                                        l_intermediateDLinks.add(dLink2);
                            l_result.add(dLink2);
                            toContinue = true; 
                        } // if
                       
                    } // for itu
                } // if
            } // for ita   
        } // for ite
        
        m_dLinksFromSources.clear();
        // initialize m_dLinksFromSources
        if (toContinue && !l_intermediateDLinks.isEmpty()) {
            for (Iterator ita = l_intermediateDLinks.iterator(); ita.hasNext(); ) {
                dLink = (DisplayableLinkImpl)ita.next();
                
                if (m_dLinksFromSources.get(dLink.getEorFirstEnd().getName()) == null) {
                    l_sourcesDLinks = new ArrayList(INITIAL_CAPACITY);
                    l_sourcesDLinks.add(dLink);
                    m_dLinksFromSources.put(dLink.getEorFirstEnd().getName(), l_sourcesDLinks);                            
                } else {
                    ((List)m_dLinksFromSources.get(dLink.getEorFirstEnd().getName())).add(dLink);
                } // else
            } // for ita
            l_intermediateDLinks.clear();
        } // if
    } // while
    
    l_sourcesDLinks = null;
    l_intermediateDLinks = null;
    m_dLinksFromSources = null;
    dLink = null;
    dLink2 = null;
    name = null;
    entry = null;
    
                      // for testing and tracking
                      //System.out.println("DisplayableLinkUtilities.getDLinksForLoopsOnTarget() result for "+
                        //      _target);
                      //for (Iterator ite = l_result.iterator(); ite.hasNext(); ) {
                        //  System.out.println("----- "+ite.next());
                      //}
                      
    return l_result;
  } // getDLinksForLoopsOnTarget()  
  
  /**
  * This important method returns the current Account/Groups context of
  * the DisplayableLinkImpl second end, and it may update the argument.
  * The AG context is a set of GroupIDMembers that may contain, first,
  * an optional couple (UserID, GroupID), and, second, a list of optional GroupIDs.
  * If the property for the DisplayableLinkImpl.AG_CONTEXT_NODE_POSITION is
  * the position of the second end, this method returns the current AG context, and
  * it does not change the argument. Otherwise, the argument is updated.
  * This method split the call to LinkRights.updateAGrunningContext() for each ACS
  * the argument uses.
  * Called by CompoundRightsFactoryImpl.detectHiddenCompoundEpRights() and the AcsAddons.
  * Calls LinkRights.updateAGrunningContext(), that the AcsAddon may overrides.
  * It may also override LinkRights.setNextNodeAGrunningContext() and isTransferingAGcontext()
  * the method updateAGrunningContext() uses.
  * @param _upd_dLink is never null and it is updated for the properties
  * DisplayableLinkImpl.AG_CONTEXT_NODE_POSITION and DisplayableLinkImpl.AG_CONTEXT_SET.
  * _upd_dLink.getProperties() is never null after the call to this method.
  * @return set of one UserID and some GroupIDs, as the AG context of the second end
  * of the argument. May be null but never empty.
  * */
  public final Set<ImmutableGroupIDMember> getSecondEndAGcontext(
          DisplayableLinkImpl _upd_dLink) {
      // variables
    LinkRights      linkRights = null;
    BaseObject[]    l_nodes = _upd_dLink.getEorAllNodes();
    int             nextIndex = 1;
    Set<ImmutableGroupIDMember> l_result = null;

                                        // for testing and tracking
                                        //if (_upd_dLink.getEorSecondEnd().getNickName().equals("Amounts view")) {
                                          //  System.out.println("start of DisplayableLinkUtilities."+
                                            //        "getSecondEndAGcontext() for the link "+_upd_dLink);
                                        //}

    if (_upd_dLink.getProperties() != null &&
            _upd_dLink.getProperties().get(
                DisplayableLinkImpl.AG_CONTEXT_NODE_POSITION) != null) {
        
        nextIndex = ((Integer)_upd_dLink.getProperties().get(
                DisplayableLinkImpl.AG_CONTEXT_NODE_POSITION)).
                intValue();
        
        if (nextIndex < l_nodes.length - 1) {
            nextIndex++;
        }
        else {
            // no change in _upd_dLink
            l_nodes = null;
            l_result = (Set<ImmutableGroupIDMember>)
                    _upd_dLink.getProperties().get(DisplayableLinkImpl.AG_CONTEXT_SET);
            

                                        // for testing and tracking
                                        //if (_upd_dLink.getEorSecondEnd().getNickName().equals("Amounts view")) {
                                            //System.out.println(
                                              //"\n In DisplayableLinkUtilities.getSecondEndAGcontext(), "+
                                              //"for the _upd_dLink "+_upd_dLink);
                                            //System.out.println("-- AT THE FIRST STEP no change - the results are: ");
                                            //if (l_result != null) {
                                              //  for (Iterator ita = l_result.iterator() ; ita.hasNext() ; ) {
                                                //    System.out.println("------------"+
                                                  //    ((GroupIDMember)ita.next()).toString() );
                                                //} // for ita
                                            //} // if
                                            //else System.out.println("------ null");
                                        //} // if
                        

            return l_result;
        } // else

    } // if

                    // for testing and tracking
                    //if (iterationsCount == 4) {
                        //System.out.println("in DisplayableLinkUtilities.getSecondEndAGcontext(), "+
                        //"starting nextIndex = "+nextIndex+" for dLink "+_upd_dLink);
                    //} // if

    try {
        // call updateAGrunningContext() from the node at nextIndex, and up to the second end
        for (int i = 1; i < l_nodes.length; i++) {

                                // for testing and tracking
                                //System.out.println(
                                  //"---- nextIndex = "+nextIndex+
                                  //"; i is processed if it is nextIndex - i = "+i);

            if (i != nextIndex) continue;

            linkRights = interpreter_.getLinkRightsInstances().get(
                        l_nodes[i].getName());
            if (linkRights == null) {
                    interpreter_.addInstancesForNode(l_nodes[i]);
                    linkRights = interpreter_.getLinkRightsInstances().get(
                        l_nodes[i].getName());
            } // if

            // stops the updating when the next node is from a different ACS
            l_result = linkRights.updateAGrunningContext(_upd_dLink);

            if (((Integer)_upd_dLink.getProperties().get(
                    DisplayableLinkImpl.AG_CONTEXT_NODE_POSITION)).
                    intValue() == _upd_dLink.noOfNodes() - 1) {

                // l_result is set for the dLink second end
                break;
            }
            else {

                                        // for testing and tracking
                                        //if (_upd_dLink.getEorSecondEnd().getNickName().equals("Amounts view")) {
                                          //  System.out.println("in DisplayableLinkUtilities.getSecondEndAGcontext(), "+
                                            //"-- increment, from the current AG position, nextIndex = "+
                                              //      nextIndex+"; at i = "+i);
                                        //} // if

                // if i == 1, for exemple we have
                // nextIndex == 4 if 2 successive nodes from the ACS
                // has been processed in linkRights.updateAGrunningContext(dLink)
                nextIndex = 1 + ((Integer)_upd_dLink.getProperties().get(
                    DisplayableLinkImpl.AG_CONTEXT_NODE_POSITION)).
                    intValue();
            } // else
        } // for i
    }
    catch (ProcessError p) {
        throw new InternalError("DisplayableLinkUtilities.getSecondEndAGcontext(): "+
                p.toString());
    }

                                        // for testing and tracking
                                        //if (_upd_dLink.getEorSecondEnd().getNickName().equals("Amounts view")) {
                                            //System.out.println(
                                              //"In DisplayableLinkUtilities.getSecondEndAGcontext(), "+
                                              //"for the _upd_dLink "+_upd_dLink);
                                            //System.out.println("--- the SECOND STEP node position is: "+
                                              //  _upd_dLink.getProperties().get(
                                                //    DisplayableLinkImpl.AG_CONTEXT_NODE_POSITION));
                                            //System.out.println("--- the SECOND STEP AG context is: ");
                                            //if (l_result != null) {
                                              //  for (Iterator ita = l_result.iterator() ; ita.hasNext() ; ) {
                                                //    System.out.println("------------"+
                                                  //    ((GroupIDMember)ita.next()).toString() );
                                                //} // for ita
                                            //} // if
                                            //else System.out.println("------ null");
                                        //} // if

    linkRights = null;
    l_nodes = null;
    return l_result;
  } // getSecondEndAGcontext()

  /**
  * Tests the link for denying, if the result is true, the adding of a new Resource node
  * to the argument, if this Resource is not a VirtualFolder member.
  * Implements the rule 'virtual folder: an indirect access to a Resource that is not
  * a virtual member, is always through a virtual member'.
  * Called by CompoundRightsFactoryImpl.detectHiddenCompoundEpRights() and
  * ThreeNodesRightsFactoryImpl.
  * Calls DisplayableLinkImpl.getEorLastVirtualFolderInPath().
  * @param _dLink is never null
  * @return true if the argument contains a VirtualFolder node without any virtual
   * member of the VirtualFolder as next node, after the VirtualFolder position in the link
  * */
  public final boolean getContainVirtualFolderWithoutMember(DisplayableLinkImpl _dLink) {
      // variables
    BaseObject[]    l_nodes = _dLink.getEorAllNodes();
    boolean         toForbid = false;
    ImmutableVirtualFolder lastVF = _dLink.getEorLastVirtualFolderInPath();

    if (lastVF == null) return toForbid;

    int             index = _dLink.indexOfBaseObject(lastVF);
    
    toForbid = true;

    if (index == l_nodes.length - 1) return toForbid;

    for (int i = index+1; i < l_nodes.length; i++) {

                    // for testing and tracking
                    //if (iterationsCount == 4) {
                      //  System.out.println("in DisplayableLinkUtilities.getContainVirtualFolderWithoutMember(), "+
                        //  "in the central loop, the second ite, third step, in l_nodes loop:");
                        //System.out.println("-- nextIndex = "+nextIndex+"; i = "+i);
                    //} // if

         if (!(l_nodes[i] instanceof ACSObject)) break;
         if (!((ACSObject)l_nodes[i]).getAcsName().equals(lastVF.getAcsName())) break;
         if (lastVF.containsBaseObject(l_nodes[i])) {
             toForbid = false;
             break;
         }
    } // for i

                    // for testing and tracking
                    //System.out.println("\n In DisplayableLinkUtilities.getContainVirtualFolderWithoutMember(), "+
                      //"the result is: "+toForbid+" for dLink "+_dLink);

    lastVF = null;;
    l_nodes = null;
    return toForbid;
  } // getContainVirtualFolderWithoutMember()

  /**
  * Gets the simple DisplayableLinksImpls for which there is at least one indirect path
  * through the other view nodes. For instance, in a view containing the nodes A, B, C and D,
  * all the DisplayableLinksImpls in _l_dLinks from A to B are returned, if there are,
  * in the arguments _m_rights, some rights from A to C, some rights from C to D,
  * and some rights from D to B. The value of _detectPositiveIndirectPaths enforces
  * the matched indirect path to have only positive or only negative rights,
  * in _m_rights, for each couple of nodes in the indirect path.
  * To detect the structural paths, a positive metaright may be used in _m_rights.
  * The current rights of each _l_dLinks argument are not considered.
  * Called by CompoundRightsFactoryImpl.addPathFromNegativeRightsInOneACS(), with
  * a _l_dLinks argument containing only simple links.
  * Calls getCurrentEndKeys().
  * @param _l_dLinks to analyze. Not null.
  * @param _m_rights where the key is a _m_l_DisplayableLinks key where rights are activated,
  *  and the value is a set of StringRights for this couple of nodes
   * (it may be an empty set, but never null).
  * @param _detectPositiveIndirectPaths true to detect the indirect positive paths only,
  * and false to detect the indirect negative paths only
  * @return simple DisplayableLinks which are already in _l_dLinks.
  * May be null but not empty.
  **/
  public final List<DisplayableLinkImpl> containsIndirectPaths(
          List<DisplayableLinkImpl> _l_dLinks, Map<ImmutableName, Set<StringRight>> _m_rights,
          boolean _detectPositiveIndirectPaths) throws InterruptedException {
    // variables
    List<DisplayableLinkImpl>   l_result = new ArrayList(INITIAL_CAPACITY);
    Set<ImmutableName>          l_currentFirstEnds = new HashSet(INITIAL_CAPACITY);
    Set<ImmutableName>          l_currentSecondEnds = new HashSet(INITIAL_CAPACITY);
    Set<ImmutableName>          l_testedFirstEnds = new HashSet(INITIAL_CAPACITY);
    Set<ImmutableName>          l_testedSecondEnds = new HashSet(INITIAL_CAPACITY);
    Set<ImmutableName>          l_testedKeys = new HashSet(INITIAL_CAPACITY);
    Set<ImmutableName>          l_nextFirstEnds = new HashSet(INITIAL_CAPACITY);
    Set<ImmutableName>          l_nextSecondEnds = new HashSet(INITIAL_CAPACITY);
    List<Name>[]                l_CurrentKeys = null;
    ImmutableName               currentLinkKey = null;
    Name                        name2 = null;
    boolean                     found = false;

                        // for testing and tracking
                        //System.out.println("");
                        //System.out.println("DisplayableLinkUtilities.containsIndirectPaths() starting.");

    for (DisplayableLinkImpl link : _l_dLinks) {

        l_nextFirstEnds.add(link.getEorFirstEnd().getName());
        l_nextSecondEnds.add(link.getEorSecondEnd().getName());
        found = false;

        while (!l_nextFirstEnds.isEmpty() || !l_nextFirstEnds.isEmpty()) {
            l_currentFirstEnds.addAll(l_nextFirstEnds);
            l_currentSecondEnds.addAll(l_nextSecondEnds);
            l_nextFirstEnds.clear();
            l_nextSecondEnds.clear();

            for (ImmutableName firstEndName : l_currentFirstEnds) {
                for (ImmutableName secondEndName : l_currentSecondEnds) {

                    currentLinkKey = ((Name)firstEndName.clone()).extended(secondEndName);
                    if (l_testedKeys.contains(currentLinkKey))
                        continue;

                    l_testedFirstEnds.add(firstEndName);
                    l_testedSecondEnds.add(secondEndName);
                    l_testedKeys.add(currentLinkKey);

                    l_CurrentKeys = getCurrentEndKeys(
                                            firstEndName, secondEndName, _m_rights,
                                            _detectPositiveIndirectPaths);

                    // if a name in l_currentFirstEndKeys ends with the name
                    // which starts a name in l_currentSecondEndKeys,
                    // there is an indirect 3-nodes path for 'link'
                    for (Name na : l_CurrentKeys[0]) {
                        for (Name na2 : l_CurrentKeys[1]) {
                            if (((Name)na.clone()).without((Name)firstEndName).equals(
                                    ((Name)na2.clone()).without((Name)secondEndName))) {
                                        found = true;
                                        l_result.add(link);
                                        break;
                            } // break
                        } // for na2
                        if (found) break;
                    } // for na

                    if (found) break;

                    for (ImmutableName name : l_CurrentKeys[0]) {
                        // from name, name2 is the complement after firstEndName
                        name2 = name.subname(
                                firstEndName.noOfComponents(),
                                name.noOfComponents());
                        if (l_testedFirstEnds.contains(name2))
                            continue;

                        l_nextFirstEnds.add(name2);
                    } // for name
                    for (ImmutableName name : l_CurrentKeys[1]) {

                        // for testing and tracking
                        //System.out.println("DisplayableLinkUtilities.containsIndirectPaths() "+
                          //      "l_CurrentKeys[1] name: "+name);
                        //System.out.println("DisplayableLinkUtilities.containsIndirectPaths() "+
                          //      "secondEndName: "+secondEndName);

                        // from name, name2 is the complement before secondEndName
                        name2 = name.subname(
                                0,
                                name.noOfComponents() - secondEndName.noOfComponents());
                        if (l_testedSecondEnds.contains(name2))
                            continue;

                        l_nextSecondEnds.add(name2);
                    } // for name
                } // for secondEndName
                if (found) break;
            } // for firstEndName

            if (found) break;
        } // while (!l_nextFirstEnds.isEmpty() ||

        if (Thread.interrupted()) {
            throw new InterruptedException();
        }

    } // for ite

                      // for testing and tracking
                      //System.out.println("DisplayableLinkUtilities.containsIndirectPaths(), "+
                        // "l_result:");
                      //for (Iterator ite = l_result.iterator(); ite.hasNext(); ) {
                        //  System.out.println("----- "+ite.next());
                      //}

    l_currentFirstEnds.clear();
    l_currentSecondEnds.clear();
    l_testedFirstEnds.clear();
    l_testedSecondEnds.clear();
    l_testedKeys.clear();
    l_nextFirstEnds.clear();
    l_nextSecondEnds.clear();
    l_CurrentKeys = null;
    currentLinkKey = null;
    name2 = null;

    return l_result;
  } // containsIndirectPaths()

  /**
  * Gets a first list containing the _m_rights keys starting with _firstEndName,
  * and a second list containing the _m_rights keys ending with _secondEndName,
  * if the _m_rights key has the sens of _detectPositiveRights.
  * This method detects also a structural link if the argument _m_rights contains
  * a positive metaright for a key.
  * Called by containsIndirectPaths().
  * @param _firstEndName is never null.
  * @param _secondEndName is never null.
  * @param _m_rights where the key is a _m_l_DisplayableLinks key where rights are activated,
  *  and the value is a set of StringRights for this couple of nodes
  * (it may be an empty set, but never null).
  * @param _detectPositiveRights true to detect the indirect positive paths only,
  * and false to detect the indirect negative paths only
  * @return array of two Name lists, where the first list contains the _m_rights keys
  * starting with _firstEndName, and the second list contains the _m_rights keys ending
  * with _secondEndName. A list may be empty.
  **/
  private final List<Name>[] getCurrentEndKeys(
          ImmutableName _firstEndName, ImmutableName _secondEndName, 
          Map<ImmutableName, Set<StringRight>> _m_rights,
          boolean _detectPositiveRights) throws InterruptedException {
    // variables
    List<Name>[]                l_result = new List[] {new ArrayList(INITIAL_CAPACITY),
                                                new ArrayList(INITIAL_CAPACITY)};
    ImmutableName               currentLinkKey = ((Name)_firstEndName.clone()).extended(
                                                _secondEndName);
    Name                        name2 = null;

                        // for testing and tracking
                        //System.out.println("");
                        //System.out.println("DisplayableLinkUtilities.getCurrentEndKeys() start.");


    for (ImmutableName name : _m_rights.keySet()) {
        if (name.equals(currentLinkKey))
            continue;

                                    // for testing and tracking
                                    //if (_m_rights.get(name) == null) {
                                      //  System.out.println(
                                        //        "DisplayableLinkUtilities.getCurrentEndKeys()"+
                                          //      " with NULL value in _m_rights for the name "+
                                            //    name);
                                    //} // if
                                    //else {
                                      //  System.out.println(
                                        //        "DisplayableLinkUtilities.getCurrentEndKeys()"+
                                          //      " with value in _m_rights for the name "+
                                            //    name);
                                    //}

        if (_m_rights.get(name).isEmpty())
            continue;
        if (!_detectPositiveRights &&
                _m_rights.get(name).iterator().next().isGrantingRight())
            continue;
        if (_detectPositiveRights &&
                !_m_rights.get(name).iterator().next().isGrantingRight())
            continue;

        // initialize l_currentFirstEndKeys and l_currentSecondEndKeys
        if (name.noOfComponents() > _firstEndName.noOfComponents()) {
            name2 = name.subname(0, _firstEndName.noOfComponents());
            if (name2 != null && _firstEndName.equals(name2)) {
                l_result[0].add((Name)((Name)name).clone());
            } // if (name2 != null
        } // if (name.noOfComponents() > currentFirstEndName.noOfComponents())
        if (name.noOfComponents() > _secondEndName.noOfComponents()) {
            name2 = name.subname(
                    name.noOfComponents() - _secondEndName.noOfComponents(),
                    name.noOfComponents());
            if (name2 != null && _secondEndName.equals(name2)) {
                l_result[1].add((Name)((Name)name).clone());
            } // if (name2 != null
        } // if (name.noOfComponents() > currentFirstEndName.noOfComponents())

    } // for name


                      // for testing and tracking
                      //System.out.println("DisplayableLinkUtilities.getCurrentEndKeys(), "+
                        // "l_result[0]:");
                      //for (Iterator ite = l_result[0].iterator(); ite.hasNext(); ) {
                        //  System.out.println("----- "+ite.next());
                      //}
                      //System.out.println("DisplayableLinkUtilities.getCurrentEndKeys(), "+
                        // "l_result[1]:");
                      //for (Iterator ite = l_result[1].iterator(); ite.hasNext(); ) {
                        //  System.out.println("----- "+ite.next());
                      //}

    currentLinkKey = null;
    name2 = null;
    return l_result;
  } // getCurrentEndKeys()

  /**
   * Finalizes the instance.
   * Called by RightsMediatorImpl.finalizeForProcess().
   */
  public void finalizeForProcess() {
    interpreter_ = null;
    baseUtil_ = null;
    resultVector_.clear();


                    //for testing and tracking
                    //System.out.println("in DisplayableLinkUtilities.finalizeForProcess(), "+
                      //"resultVector_ is null ");
                    //for (Iterator ite = l_result.iterator() ; ite.hasNext() ; ) {
                      //System.out.println("-------"+ ((DisplayableLinkImpl)ite.next()).toString());
                    //}
  } // finalizeForProcess()
  
} // DisplayableLinkUtilities
