//Title:        Access Road
//Version:      0.6.0
//Copyright:    2002, 2004, 2007-2009 EURL ACCBEE
//Author:       Patrick Thazard
//Description:  a simulator, designer and controller for IT system access controls

/* This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 3 of the License,
or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package ARoad0.gWorkInterface;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import java.util.Set;

import ARoad0.gBaseInterface.BaseObject;
import ARoad0.Pattern.MVC_Controller;

/**
 * This interface is responsible to follow the property change events
 * from all the BaseObjects associated to a view,
 * to inform the view GraphicViewBaselistenerImpl that the rights may have changed.
 * It is independant of the graphic view, which may be deleted and created several times
 * by GraphicViewBaselistenerImpl.
 * The BaseListener which manages the view instance must register itself
 * as a PropertyChangeListener of this RightsNetworkObserver instance.
 * @see ARoad0.gWork.RightsMediatorImpl
 */
public interface RightsNetworkObserver extends MVC_Controller, PropertyChangeListener {

  static final int    INITIAL_CAPACITY = 10;

  /**
   *
   * @return the view or another base object which defines the rights network
   */
  public BaseObject getSource();

  /**
   * Register a listener for the PropertyChange event
   * The class will fire a PropertyChange value whenever the value is updated.
   * @param _listener  An object to be invoked when a PropertyChange
   *		event is fired.
   */
  public void addPropertyChangeListener(
    PropertyChangeListener _listener);

  /**
   * Remove a listener for the PropertyChange event
   * @param _listener to be removed.
   */
  public void removePropertyChangeListener(
    PropertyChangeListener _listener);

  /**
   * Manage the events fired by the base objects of the rights network.
   * @param _evt to process
   */
  public void propertyChange(PropertyChangeEvent _evt);

  /**
   * For a new view or when the view nodes have changed, this method
   * registers this instance as a listener of the bound property changes in
   * the network of ACSObjects and properties which are implied in the view.<P>
   * This includes the listening of the intermediate nodes for each access path,
   * and also for the non-intermediates which should become intermediates when
   * a property change. This includes the AclEntries and the PrivilegeForLinks,
   * respectively for their rights and their sources/targets. This includes the
   * listening of the specific events from any AcsAddon ACS or any of its objects,
   * for which BaseUtilityImpl.add/removeAddonInner/ParentPropertyChangeListener()
   * are called.
   * @param _l_baseObjects the BaseObjects which are intermediate nodes in
   *          the detected access paths of the view, or an intermediate-node AclEntry
   */
  public void setRightsNetwork(Set _l_baseObjects);

  /**
   * Get the values of the view
   * @return view.toString() with the name of the class
   */
  public String toString();

  /**
   * Finalize the instance; the call is reserved to ViewTreeBaseListenerImpl.
   */
  public void finalizeForProcess();

} // RightsNetworkObserver
